/*
 * Copyright (C) 2004-2015 L2J Server
 * 
 * This file is part of L2J Server.
 * 
 * L2J Server is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * L2J Server is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.l2jserver.gameserver.network;

import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.l2jserver.Config;
import com.l2jserver.gameserver.model.clientstrings.Builder;
import com.l2jserver.gameserver.network.serverpackets.ExShowScreenMessage;

/**
 * NpcStringId implementation
 * @author UnAfraid
 */
public final class NpcStringId
{
	private static final Logger _log = Logger.getLogger(NpcStringId.class.getName());
	private static final NSLocalisation[] EMPTY_NSL_ARRAY = new NSLocalisation[0];
	public static final NpcStringId[] EMPTY_ARRAY = new NpcStringId[0];
	
	/**
	 * Map containing all NpcStringId<br>
	 */
	private static Map<Integer, NpcStringId> VALUES = new HashMap<>();
	
	@ClientString(id = 1, message = "Hello! I am $s1. You are $s2, right? Hehehe")
	public static NpcStringId HELLO_I_AM_S1_YOU_ARE_S2_RIGHT_HEHEHE;
	
	@ClientString(id = 2, message = "$s1--$s2--$s3--$s4//$s5 Hehehe")
	public static NpcStringId S1_S2_S3_S4_S5_HEHEHE;
	
	@ClientString(id = 3, message = "none")
	public static NpcStringId NONE;
	
	@ClientString(id = 4, message = "none")
	public static NpcStringId NONE2;
	
	@ClientString(id = 5, message = "Withdraw the fee for the next time at $s1/$s2 $s3")
	public static NpcStringId WITHDRAW_THE_FEE_FOR_THE_NEXT_TIME_AT_S1_S2_S3;
	
	@ClientString(id = 6, message = "((font color='FFAABB')$s1(/font) Adena/$s2 Day(s))")
	public static NpcStringId FONT_COLOR_FFAABB_S1_FONT_ADENA_S2_DAY_S;
	
	@ClientString(id = 7, message = "$s1(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-270andreply=$s2000')Deactivate(/a)$s3")
	public static NpcStringId S1_BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_270ANDREPLY_S2000_DEACTIVATE_A_S3;
	
	@ClientString(id = 8, message = "Stage")
	public static NpcStringId STAGE;
	
	@ClientString(id = 9, message = "Stage $s1")
	public static NpcStringId STAGE_S1;
	
	@ClientString(id = 10, message = "$s1%%")
	public static NpcStringId S1;
	
	@ClientString(id = 101, message = "Letters of Love")
	public static NpcStringId LETTERS_OF_LOVE;
	
	@ClientString(id = 102, message = "Letters of Love (In Progress)")
	public static NpcStringId LETTERS_OF_LOVE_IN_PROGRESS;
	
	@ClientString(id = 103, message = "Letters of Love (Done)")
	public static NpcStringId LETTERS_OF_LOVE_DONE;
	
	@ClientString(id = 201, message = "What Women Want")
	public static NpcStringId WHAT_WOMEN_WANT;
	
	@ClientString(id = 202, message = "What Women Want (In Progress)")
	public static NpcStringId WHAT_WOMEN_WANT_IN_PROGRESS;
	
	@ClientString(id = 203, message = "What Women Want (Done)")
	public static NpcStringId WHAT_WOMEN_WANT_DONE;
	
	@ClientString(id = 301, message = "Will the Seal be Broken?")
	public static NpcStringId WILL_THE_SEAL_BE_BROKEN;
	
	@ClientString(id = 302, message = "Will the Seal be Broken? (In Progress)")
	public static NpcStringId WILL_THE_SEAL_BE_BROKEN_IN_PROGRESS;
	
	@ClientString(id = 303, message = "Will the Seal be Broken? (Done)")
	public static NpcStringId WILL_THE_SEAL_BE_BROKEN_DONE;
	
	@ClientString(id = 401, message = "Long Live the Pa'agrio Lord")
	public static NpcStringId LONG_LIVE_THE_PA_AGRIO_LORD;
	
	@ClientString(id = 402, message = "Long Live the Pa'agrio Lord (In Progress)")
	public static NpcStringId LONG_LIVE_THE_PA_AGRIO_LORD_IN_PROGRESS;
	
	@ClientString(id = 403, message = "Long Live the Pa'agrio Lord (Done)")
	public static NpcStringId LONG_LIVE_THE_PA_AGRIO_LORD_DONE;
	
	@ClientString(id = 501, message = "Miner's Favor")
	public static NpcStringId MINER_S_FAVOR;
	
	@ClientString(id = 502, message = "Miner's Favor (In Progress)")
	public static NpcStringId MINER_S_FAVOR_IN_PROGRESS;
	
	@ClientString(id = 503, message = "Miner's Favor (Done)")
	public static NpcStringId MINER_S_FAVOR_DONE;
	
	@ClientString(id = 601, message = "Step Into the Future")
	public static NpcStringId STEP_INTO_THE_FUTURE;
	
	@ClientString(id = 602, message = "Step Into the Future (In Progress)")
	public static NpcStringId STEP_INTO_THE_FUTURE_IN_PROGRESS;
	
	@ClientString(id = 603, message = "Step Into the Future (Done)")
	public static NpcStringId STEP_INTO_THE_FUTURE_DONE;
	
	@ClientString(id = 701, message = "A Trip Begins")
	public static NpcStringId A_TRIP_BEGINS;
	
	@ClientString(id = 702, message = "A Trip Begins (In Progress)")
	public static NpcStringId A_TRIP_BEGINS_IN_PROGRESS;
	
	@ClientString(id = 703, message = "A Trip Begins (Done)")
	public static NpcStringId A_TRIP_BEGINS_DONE;
	
	@ClientString(id = 801, message = "An Adventure Begins")
	public static NpcStringId AN_ADVENTURE_BEGINS;
	
	@ClientString(id = 802, message = "An Adventure Begins (In Progress)")
	public static NpcStringId AN_ADVENTURE_BEGINS_IN_PROGRESS;
	
	@ClientString(id = 803, message = "An Adventure Begins (Done)")
	public static NpcStringId AN_ADVENTURE_BEGINS_DONE;
	
	@ClientString(id = 901, message = "Into the City of Humans")
	public static NpcStringId INTO_THE_CITY_OF_HUMANS;
	
	@ClientString(id = 902, message = "Into the City of Humans (In Progress)")
	public static NpcStringId INTO_THE_CITY_OF_HUMANS_IN_PROGRESS;
	
	@ClientString(id = 903, message = "Into the City of Humans (Done)")
	public static NpcStringId INTO_THE_CITY_OF_HUMANS_DONE;
	
	@ClientString(id = 1001, message = "Into the World")
	public static NpcStringId INTO_THE_WORLD;
	
	@ClientString(id = 1002, message = "Into the World (In Progress)")
	public static NpcStringId INTO_THE_WORLD_IN_PROGRESS;
	
	@ClientString(id = 1003, message = "Into the World (Done)")
	public static NpcStringId INTO_THE_WORLD_DONE;
	
	@ClientString(id = 1101, message = "Secret Meeting with Ketra Orcs")
	public static NpcStringId SECRET_MEETING_WITH_KETRA_ORCS;
	
	@ClientString(id = 1102, message = "Secret Meeting with Ketra Orcs (In Progress)")
	public static NpcStringId SECRET_MEETING_WITH_KETRA_ORCS_IN_PROGRESS;
	
	@ClientString(id = 1103, message = "Secret Meeting with Ketra Orcs (Done)")
	public static NpcStringId SECRET_MEETING_WITH_KETRA_ORCS_DONE;
	
	@ClientString(id = 1104, message = "Secret Meeting with Ketra Orcs")
	public static NpcStringId SECRET_MEETING_WITH_KETRA_ORCS2;
	
	@ClientString(id = 1201, message = "Secret Meeting with Varka Silenos")
	public static NpcStringId SECRET_MEETING_WITH_VARKA_SILENOS;
	
	@ClientString(id = 1202, message = "Secret Meeting with Varka Silenos (In Progress)")
	public static NpcStringId SECRET_MEETING_WITH_VARKA_SILENOS_IN_PROGRESS;
	
	@ClientString(id = 1203, message = "Secret Meeting with Varka Silenos (Done)")
	public static NpcStringId SECRET_MEETING_WITH_VARKA_SILENOS_DONE;
	
	@ClientString(id = 1204, message = "Secret Meeting with Varka Silenos")
	public static NpcStringId SECRET_MEETING_WITH_VARKA_SILENOS2;
	
	@ClientString(id = 1301, message = "Parcel Delivery (Lv. 74-80)")
	public static NpcStringId PARCEL_DELIVERY_LV_74_80;
	
	@ClientString(id = 1302, message = "Parcel Delivery (Lv. 74-80) (In Progress)")
	public static NpcStringId PARCEL_DELIVERY_LV_74_80_IN_PROGRESS;
	
	@ClientString(id = 1303, message = "Parcel Delivery (Lv. 74-80) (Done)")
	public static NpcStringId PARCEL_DELIVERY_LV_74_80_DONE;
	
	@ClientString(id = 1304, message = "Parcel Delivery (Lv. 74-80)")
	public static NpcStringId PARCEL_DELIVERY_LV_74_802;
	
	@ClientString(id = 1401, message = "Whereabouts of the Archaeologist")
	public static NpcStringId WHEREABOUTS_OF_THE_ARCHAEOLOGIST;
	
	@ClientString(id = 1402, message = "Whereabouts of the Archaeologist (In Progress)")
	public static NpcStringId WHEREABOUTS_OF_THE_ARCHAEOLOGIST_IN_PROGRESS;
	
	@ClientString(id = 1403, message = "Whereabouts of the Archaeologist (Done)")
	public static NpcStringId WHEREABOUTS_OF_THE_ARCHAEOLOGIST_DONE;
	
	@ClientString(id = 1404, message = "Whereabouts of the Archaeologist")
	public static NpcStringId WHEREABOUTS_OF_THE_ARCHAEOLOGIST2;
	
	@ClientString(id = 1501, message = "Sweet Whispers (Lv. 60-99)")
	public static NpcStringId SWEET_WHISPERS_LV_60_99;
	
	@ClientString(id = 1502, message = "Sweet Whispers (Lv. 60-99) (In Progress)")
	public static NpcStringId SWEET_WHISPERS_LV_60_99_IN_PROGRESS;
	
	@ClientString(id = 1503, message = "Sweet Whispers (Lv. 60-99) (Done)")
	public static NpcStringId SWEET_WHISPERS_LV_60_99_DONE;
	
	@ClientString(id = 1504, message = "Sweet Whispers (Lv. 60-99)")
	public static NpcStringId SWEET_WHISPERS_LV_60_992;
	
	@ClientString(id = 1601, message = "The Coming Darkness (Lv. 62-99)")
	public static NpcStringId THE_COMING_DARKNESS_LV_62_99;
	
	@ClientString(id = 1602, message = "The Coming Darkness (Lv. 62-99) (In Progress)")
	public static NpcStringId THE_COMING_DARKNESS_LV_62_99_IN_PROGRESS;
	
	@ClientString(id = 1603, message = "The Coming Darkness (Lv. 62-99) (Done)")
	public static NpcStringId THE_COMING_DARKNESS_LV_62_99_DONE;
	
	@ClientString(id = 1604, message = "The Coming Darkness (Lv. 62-99)")
	public static NpcStringId THE_COMING_DARKNESS_LV_62_992;
	
	@ClientString(id = 1701, message = "Light and Darkness (Lv. 61-99)")
	public static NpcStringId LIGHT_AND_DARKNESS_LV_61_99;
	
	@ClientString(id = 1702, message = "Light and Darkness (Lv. 61-99) (In Progress)")
	public static NpcStringId LIGHT_AND_DARKNESS_LV_61_99_IN_PROGRESS;
	
	@ClientString(id = 1703, message = "Light and Darkness (Lv. 61-99) (Done)")
	public static NpcStringId LIGHT_AND_DARKNESS_LV_61_99_DONE;
	
	@ClientString(id = 1704, message = "Light and Darkness (Lv. 61-99)")
	public static NpcStringId LIGHT_AND_DARKNESS_LV_61_992;
	
	@ClientString(id = 1801, message = "Meeting with the Golden Ram")
	public static NpcStringId MEETING_WITH_THE_GOLDEN_RAM;
	
	@ClientString(id = 1802, message = "Meeting with the Golden Ram (In Progress)")
	public static NpcStringId MEETING_WITH_THE_GOLDEN_RAM_IN_PROGRESS;
	
	@ClientString(id = 1803, message = "Meeting with the Golden Ram (Done)")
	public static NpcStringId MEETING_WITH_THE_GOLDEN_RAM_DONE;
	
	@ClientString(id = 1804, message = "Meeting with the Golden Ram")
	public static NpcStringId MEETING_WITH_THE_GOLDEN_RAM2;
	
	@ClientString(id = 1901, message = "Go to the Pastureland (Lv. 82-99)")
	public static NpcStringId GO_TO_THE_PASTURELAND_LV_82_99;
	
	@ClientString(id = 1902, message = "Go to the Pastureland (Lv. 82-99) (In Progress)")
	public static NpcStringId GO_TO_THE_PASTURELAND_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 1903, message = "Go to the Pastureland (Lv. 82-99) (Done)")
	public static NpcStringId GO_TO_THE_PASTURELAND_LV_82_99_DONE;
	
	@ClientString(id = 1904, message = "Go to the Pastureland (Lv. 82-99)")
	public static NpcStringId GO_TO_THE_PASTURELAND_LV_82_992;
	
	@ClientString(id = 2001, message = "Bring Up With Love (Lv. 82-99)")
	public static NpcStringId BRING_UP_WITH_LOVE_LV_82_99;
	
	@ClientString(id = 2002, message = "Bring Up With Love (Lv. 82-99) (In Progress)")
	public static NpcStringId BRING_UP_WITH_LOVE_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 2003, message = "Bring Up With Love (Lv. 82-99) (Done)")
	public static NpcStringId BRING_UP_WITH_LOVE_LV_82_99_DONE;
	
	@ClientString(id = 2004, message = "Bring Up with Love (Lv. 82-99)")
	public static NpcStringId BRING_UP_WITH_LOVE_LV_82_992;
	
	@ClientString(id = 2005, message = "Are you trying to tame me? Don't do that!")
	public static NpcStringId ARE_YOU_TRYING_TO_TAME_ME_DON_T_DO_THAT;
	
	@ClientString(id = 2006, message = "Don't give such a thing.  You can endanger yourself!")
	public static NpcStringId DON_T_GIVE_SUCH_A_THING_YOU_CAN_ENDANGER_YOURSELF;
	
	@ClientString(id = 2007, message = "Yuck!  What is this?  It tastes terrible!")
	public static NpcStringId YUCK_WHAT_IS_THIS_IT_TASTES_TERRIBLE;
	
	@ClientString(id = 2008, message = "I'm hungry.  Give me a little more, please.")
	public static NpcStringId I_M_HUNGRY_GIVE_ME_A_LITTLE_MORE_PLEASE;
	
	@ClientString(id = 2009, message = "What is this?  Is this edible?")
	public static NpcStringId WHAT_IS_THIS_IS_THIS_EDIBLE;
	
	@ClientString(id = 2010, message = "Don't worry about me.")
	public static NpcStringId DON_T_WORRY_ABOUT_ME;
	
	@ClientString(id = 2011, message = "Thank you.  That was delicious!")
	public static NpcStringId THANK_YOU_THAT_WAS_DELICIOUS;
	
	@ClientString(id = 2012, message = "I think I am starting to like you!")
	public static NpcStringId I_THINK_I_AM_STARTING_TO_LIKE_YOU;
	
	@ClientString(id = 2013, message = "Eeeeek!  Eeeeek!")
	public static NpcStringId EEEEEK_EEEEEK;
	
	@ClientString(id = 2014, message = "Don't keep trying to tame me.  I don't want to be tamed.")
	public static NpcStringId DON_T_KEEP_TRYING_TO_TAME_ME_I_DON_T_WANT_TO_BE_TAMED;
	
	@ClientString(id = 2015, message = "It is just food to me.  Although it may also be your hand.")
	public static NpcStringId IT_IS_JUST_FOOD_TO_ME_ALTHOUGH_IT_MAY_ALSO_BE_YOUR_HAND;
	
	@ClientString(id = 2016, message = "If I keep eating like this, won't I become fat?  *chomp chomp*")
	public static NpcStringId IF_I_KEEP_EATING_LIKE_THIS_WON_T_I_BECOME_FAT_CHOMP_CHOMP;
	
	@ClientString(id = 2017, message = "Why do you keep feeding me?")
	public static NpcStringId WHY_DO_YOU_KEEP_FEEDING_ME;
	
	@ClientString(id = 2018, message = "Don't trust me.  I'm afraid I may betray you later.")
	public static NpcStringId DON_T_TRUST_ME_I_M_AFRAID_I_MAY_BETRAY_YOU_LATER;
	
	@ClientString(id = 2019, message = "Grrrrr....")
	public static NpcStringId GRRRRR;
	
	@ClientString(id = 2020, message = "You brought this upon yourself...!")
	public static NpcStringId YOU_BROUGHT_THIS_UPON_YOURSELF;
	
	@ClientString(id = 2021, message = "I feel strange...!  I keep having these evil thoughts...!")
	public static NpcStringId I_FEEL_STRANGE_I_KEEP_HAVING_THESE_EVIL_THOUGHTS;
	
	@ClientString(id = 2022, message = "Alas... so this is how it all ends...")
	public static NpcStringId ALAS_SO_THIS_IS_HOW_IT_ALL_ENDS;
	
	@ClientString(id = 2023, message = "I don't feel so good... Oh, my mind is very troubled...!")
	public static NpcStringId I_DON_T_FEEL_SO_GOOD_OH_MY_MIND_IS_VERY_TROUBLED;
	
	@ClientString(id = 2024, message = "$s1, so what do you think it is like to be tamed?")
	public static NpcStringId S1_SO_WHAT_DO_YOU_THINK_IT_IS_LIKE_TO_BE_TAMED;
	
	@ClientString(id = 2025, message = "$s1, whenever I see spice, I think I will miss your hand that used to feed it to me.")
	public static NpcStringId S1_WHENEVER_I_SEE_SPICE_I_THINK_I_WILL_MISS_YOUR_HAND_THAT_USED_TO_FEED_IT_TO_ME;
	
	@ClientString(id = 2026, message = "$s1, don't go to the village.  I don't have the strength to follow you.")
	public static NpcStringId S1_DON_T_GO_TO_THE_VILLAGE_I_DON_T_HAVE_THE_STRENGTH_TO_FOLLOW_YOU;
	
	@ClientString(id = 2027, message = "Thank you for trusting me, $s1. I hope I will be helpful to you.")
	public static NpcStringId THANK_YOU_FOR_TRUSTING_ME_S1_I_HOPE_I_WILL_BE_HELPFUL_TO_YOU;
	
	@ClientString(id = 2028, message = "$s1, will I be able to help you?")
	public static NpcStringId S1_WILL_I_BE_ABLE_TO_HELP_YOU;
	
	@ClientString(id = 2029, message = "I guess it's just my animal magnetism.")
	public static NpcStringId I_GUESS_IT_S_JUST_MY_ANIMAL_MAGNETISM;
	
	@ClientString(id = 2030, message = "Too much spicy food makes me sweat like a beast.")
	public static NpcStringId TOO_MUCH_SPICY_FOOD_MAKES_ME_SWEAT_LIKE_A_BEAST;
	
	@ClientString(id = 2031, message = "Animals need love too.")
	public static NpcStringId ANIMALS_NEED_LOVE_TOO;
	
	@ClientString(id = 2032, message = "What'd I miss? What'd I miss?")
	public static NpcStringId WHAT_D_I_MISS_WHAT_D_I_MISS;
	
	@ClientString(id = 2033, message = "I just know before this is over, I'm gonna need a lot of serious therapy.")
	public static NpcStringId I_JUST_KNOW_BEFORE_THIS_IS_OVER_I_M_GONNA_NEED_A_LOT_OF_SERIOUS_THERAPY;
	
	@ClientString(id = 2034, message = "I sense great wisdom in you... I'm an animal, and I got instincts.")
	public static NpcStringId I_SENSE_GREAT_WISDOM_IN_YOU_I_M_AN_ANIMAL_AND_I_GOT_INSTINCTS;
	
	@ClientString(id = 2035, message = "Remember, I'm here to help.")
	public static NpcStringId REMEMBER_I_M_HERE_TO_HELP;
	
	@ClientString(id = 2036, message = "Are we there yet?")
	public static NpcStringId ARE_WE_THERE_YET;
	
	@ClientString(id = 2037, message = "That really made me feel good to see that.")
	public static NpcStringId THAT_REALLY_MADE_ME_FEEL_GOOD_TO_SEE_THAT;
	
	@ClientString(id = 2038, message = "Oh, no, no, no, Nooooo!")
	public static NpcStringId OH_NO_NO_NO_NOOOOO;
	
	@ClientString(id = 2101, message = "Hidden Truth")
	public static NpcStringId HIDDEN_TRUTH;
	
	@ClientString(id = 2102, message = "Hidden Truth (In Progress)")
	public static NpcStringId HIDDEN_TRUTH_IN_PROGRESS;
	
	@ClientString(id = 2103, message = "Hidden Truth (Done)")
	public static NpcStringId HIDDEN_TRUTH_DONE;
	
	@ClientString(id = 2104, message = "Hidden Truth")
	public static NpcStringId HIDDEN_TRUTH2;
	
	@ClientString(id = 2114, message = "What did you just do to me?")
	public static NpcStringId WHAT_DID_YOU_JUST_DO_TO_ME;
	
	@ClientString(id = 2150, message = "Who awoke me?")
	public static NpcStringId WHO_AWOKE_ME;
	
	@ClientString(id = 2151, message = "My master has instructed me to be your guide, $s1.")
	public static NpcStringId MY_MASTER_HAS_INSTRUCTED_ME_TO_BE_YOUR_GUIDE_S1;
	
	@ClientString(id = 2152, message = "Please check this bookcase, $s1.")
	public static NpcStringId PLEASE_CHECK_THIS_BOOKCASE_S1;
	
	@ClientString(id = 2201, message = "Tragedy in Von Hellmann Forest")
	public static NpcStringId TRAGEDY_IN_VON_HELLMANN_FOREST;
	
	@ClientString(id = 2202, message = "Tragedy in Von Hellmann Forest (In Progress)")
	public static NpcStringId TRAGEDY_IN_VON_HELLMANN_FOREST_IN_PROGRESS;
	
	@ClientString(id = 2203, message = "Tragedy in Von Hellmann Forest (Done)")
	public static NpcStringId TRAGEDY_IN_VON_HELLMANN_FOREST_DONE;
	
	@ClientString(id = 2204, message = "Tragedy in Von Hellmann Forest")
	public static NpcStringId TRAGEDY_IN_VON_HELLMANN_FOREST2;
	
	@ClientString(id = 2250, message = "Did you call me, $s1?")
	public static NpcStringId DID_YOU_CALL_ME_S1;
	
	@ClientString(id = 2251, message = "I'm confused! Maybe it's time to go back.")
	public static NpcStringId I_M_CONFUSED_MAYBE_IT_S_TIME_TO_GO_BACK;
	
	@ClientString(id = 2301, message = "Lidia's Heart")
	public static NpcStringId LIDIA_S_HEART;
	
	@ClientString(id = 2302, message = "Lidia's Heart (In Progress)")
	public static NpcStringId LIDIA_S_HEART_IN_PROGRESS;
	
	@ClientString(id = 2303, message = "Lidia's Heart (Done)")
	public static NpcStringId LIDIA_S_HEART_DONE;
	
	@ClientString(id = 2304, message = "Lidia's Heart")
	public static NpcStringId LIDIA_S_HEART2;
	
	@ClientString(id = 2401, message = "Inhabitants of the Forest of the Dead")
	public static NpcStringId INHABITANTS_OF_THE_FOREST_OF_THE_DEAD;
	
	@ClientString(id = 2402, message = "Inhabitants of the Forest of the Dead (In Progress)")
	public static NpcStringId INHABITANTS_OF_THE_FOREST_OF_THE_DEAD_IN_PROGRESS;
	
	@ClientString(id = 2403, message = "Inhabitants of the Forest of the Dead (Done)")
	public static NpcStringId INHABITANTS_OF_THE_FOREST_OF_THE_DEAD_DONE;
	
	@ClientString(id = 2404, message = "Inhabitants of the Forest of the Dead")
	public static NpcStringId INHABITANTS_OF_THE_FOREST_OF_THE_DEAD2;
	
	@ClientString(id = 2450, message = "That sign!")
	public static NpcStringId THAT_SIGN;
	
	@ClientString(id = 2501, message = "Hiding Behind the Truth")
	public static NpcStringId HIDING_BEHIND_THE_TRUTH;
	
	@ClientString(id = 2502, message = "Hiding Behind the Truth (In Progress)")
	public static NpcStringId HIDING_BEHIND_THE_TRUTH_IN_PROGRESS;
	
	@ClientString(id = 2503, message = "Hiding Behind the Truth (Done)")
	public static NpcStringId HIDING_BEHIND_THE_TRUTH_DONE;
	
	@ClientString(id = 2504, message = "Hiding Behind the Truth")
	public static NpcStringId HIDING_BEHIND_THE_TRUTH2;
	
	@ClientString(id = 2550, message = "That box was sealed by my master, $s1! Don't touch it!")
	public static NpcStringId THAT_BOX_WAS_SEALED_BY_MY_MASTER_S1_DON_T_TOUCH_IT;
	
	@ClientString(id = 2551, message = "You've ended my immortal life! You're protected by the feudal lord, aren't you?")
	public static NpcStringId YOU_VE_ENDED_MY_IMMORTAL_LIFE_YOU_RE_PROTECTED_BY_THE_FEUDAL_LORD_AREN_T_YOU;
	
	@ClientString(id = 2601, message = "Tired of Waiting (Lv. 80-99)")
	public static NpcStringId TIRED_OF_WAITING_LV_80_99;
	
	@ClientString(id = 2602, message = "Tired of Waiting (Lv. 80-99) (In Progress)")
	public static NpcStringId TIRED_OF_WAITING_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 2603, message = "Tired of Waiting (Lv. 80-99) (Done)")
	public static NpcStringId TIRED_OF_WAITING_LV_80_99_DONE;
	
	@ClientString(id = 2701, message = "Chest Caught with a Wind Bait")
	public static NpcStringId CHEST_CAUGHT_WITH_A_WIND_BAIT;
	
	@ClientString(id = 2702, message = "Chest Caught with a Wind Bait (In Progress)")
	public static NpcStringId CHEST_CAUGHT_WITH_A_WIND_BAIT_IN_PROGRESS;
	
	@ClientString(id = 2703, message = "Chest Caught with a Wind Bait (Done)")
	public static NpcStringId CHEST_CAUGHT_WITH_A_WIND_BAIT_DONE;
	
	@ClientString(id = 2704, message = "Chest Caught with a Wind Bait")
	public static NpcStringId CHEST_CAUGHT_WITH_A_WIND_BAIT2;
	
	@ClientString(id = 2801, message = "Chest Caught with an Icy Air Bait")
	public static NpcStringId CHEST_CAUGHT_WITH_AN_ICY_AIR_BAIT;
	
	@ClientString(id = 2802, message = "Chest Caught with an Icy Air Bait (In Progress)")
	public static NpcStringId CHEST_CAUGHT_WITH_AN_ICY_AIR_BAIT_IN_PROGRESS;
	
	@ClientString(id = 2803, message = "Chest Caught with an Icy Air Bait (Done)")
	public static NpcStringId CHEST_CAUGHT_WITH_AN_ICY_AIR_BAIT_DONE;
	
	@ClientString(id = 2804, message = "Chest Caught with an Icy Air Bait")
	public static NpcStringId CHEST_CAUGHT_WITH_AN_ICY_AIR_BAIT2;
	
	@ClientString(id = 2901, message = "Chest Caught with an Earth Bait")
	public static NpcStringId CHEST_CAUGHT_WITH_AN_EARTH_BAIT;
	
	@ClientString(id = 2902, message = "Chest Caught with an Earth Bait (In Progress)")
	public static NpcStringId CHEST_CAUGHT_WITH_AN_EARTH_BAIT_IN_PROGRESS;
	
	@ClientString(id = 2903, message = "Chest Caught with an Earth Bait (Done)")
	public static NpcStringId CHEST_CAUGHT_WITH_AN_EARTH_BAIT_DONE;
	
	@ClientString(id = 2904, message = "Chest Caught with an Earth Bait")
	public static NpcStringId CHEST_CAUGHT_WITH_AN_EARTH_BAIT2;
	
	@ClientString(id = 3001, message = "Chest Caught with a Bait of Flame")
	public static NpcStringId CHEST_CAUGHT_WITH_A_BAIT_OF_FLAME;
	
	@ClientString(id = 3002, message = "Chest Caught with a Bait of Flame (In Progress)")
	public static NpcStringId CHEST_CAUGHT_WITH_A_BAIT_OF_FLAME_IN_PROGRESS;
	
	@ClientString(id = 3003, message = "Chest Caught with a Bait of Flame (Done)")
	public static NpcStringId CHEST_CAUGHT_WITH_A_BAIT_OF_FLAME_DONE;
	
	@ClientString(id = 3004, message = "Chest Caught with a Bait of Fire")
	public static NpcStringId CHEST_CAUGHT_WITH_A_BAIT_OF_FIRE;
	
	@ClientString(id = 3101, message = "Secret Buried in the Swamp (Lv. 66-76)")
	public static NpcStringId SECRET_BURIED_IN_THE_SWAMP_LV_66_76;
	
	@ClientString(id = 3102, message = "Secret Buried in the Swamp (Lv. 66-76) (In Progress)")
	public static NpcStringId SECRET_BURIED_IN_THE_SWAMP_LV_66_76_IN_PROGRESS;
	
	@ClientString(id = 3103, message = "Secret Buried in the Swamp (Lv. 66-76) (Done)")
	public static NpcStringId SECRET_BURIED_IN_THE_SWAMP_LV_66_76_DONE;
	
	@ClientString(id = 3104, message = "Secret Buried in the Swamp (Lv. 66-76)")
	public static NpcStringId SECRET_BURIED_IN_THE_SWAMP_LV_66_762;
	
	@ClientString(id = 3201, message = "An Obvious Lie (Lv. 45-99)")
	public static NpcStringId AN_OBVIOUS_LIE_LV_45_99;
	
	@ClientString(id = 3202, message = "An Obvious Lie (Lv. 45-99) (In Progress)")
	public static NpcStringId AN_OBVIOUS_LIE_LV_45_99_IN_PROGRESS;
	
	@ClientString(id = 3203, message = "An Obvious Lie (Lv. 45-99) (Done)")
	public static NpcStringId AN_OBVIOUS_LIE_LV_45_99_DONE;
	
	@ClientString(id = 3204, message = "An Obvious Lie (Lv. 45-99)")
	public static NpcStringId AN_OBVIOUS_LIE_LV_45_992;
	
	@ClientString(id = 3301, message = "Make a Pair of Dress Shoes (Lv. 60-99)")
	public static NpcStringId MAKE_A_PAIR_OF_DRESS_SHOES_LV_60_99;
	
	@ClientString(id = 3302, message = "Make a Pair of Dress Shoes (Lv. 60-99) (In Progress)")
	public static NpcStringId MAKE_A_PAIR_OF_DRESS_SHOES_LV_60_99_IN_PROGRESS;
	
	@ClientString(id = 3303, message = "Make a Pair of Dress Shoes (Lv. 60-99) (Done)")
	public static NpcStringId MAKE_A_PAIR_OF_DRESS_SHOES_LV_60_99_DONE;
	
	@ClientString(id = 3304, message = "Make a Pair of Dress Shoes (Lv. 60-99)")
	public static NpcStringId MAKE_A_PAIR_OF_DRESS_SHOES_LV_60_992;
	
	@ClientString(id = 3401, message = "In Search of Cloth (Lv. 60-99)")
	public static NpcStringId IN_SEARCH_OF_CLOTH_LV_60_99;
	
	@ClientString(id = 3402, message = "In Search of Cloth (Lv. 60-99) (In Progress)")
	public static NpcStringId IN_SEARCH_OF_CLOTH_LV_60_99_IN_PROGRESS;
	
	@ClientString(id = 3403, message = "In Search of Cloth (Lv. 60-99) (Done)")
	public static NpcStringId IN_SEARCH_OF_CLOTH_LV_60_99_DONE;
	
	@ClientString(id = 3404, message = "In Search of Cloth (Lv. 60-99)")
	public static NpcStringId IN_SEARCH_OF_CLOTH_LV_60_992;
	
	@ClientString(id = 3501, message = "Find Glittering Jewelry (Lv. 60-99)")
	public static NpcStringId FIND_GLITTERING_JEWELRY_LV_60_99;
	
	@ClientString(id = 3502, message = "Find Glittering Jewelry (Lv. 60-99) (In Progress)")
	public static NpcStringId FIND_GLITTERING_JEWELRY_LV_60_99_IN_PROGRESS;
	
	@ClientString(id = 3503, message = "Find Glittering Jewelry (Lv. 60-99) (In Progress)")
	public static NpcStringId FIND_GLITTERING_JEWELRY_LV_60_99_IN_PROGRESS2;
	
	@ClientString(id = 3504, message = "Find Glittering Jewelry (Lv. 60-99)")
	public static NpcStringId FIND_GLITTERING_JEWELRY_LV_60_992;
	
	@ClientString(id = 3601, message = "Make a Sewing Kit (Lv. 60-99)")
	public static NpcStringId MAKE_A_SEWING_KIT_LV_60_99;
	
	@ClientString(id = 3602, message = "Make a Sewing Kit (Lv. 60-99) (In Progress)")
	public static NpcStringId MAKE_A_SEWING_KIT_LV_60_99_IN_PROGRESS;
	
	@ClientString(id = 3603, message = "Make a Sewing Kit (Lv. 60-99) (Done)")
	public static NpcStringId MAKE_A_SEWING_KIT_LV_60_99_DONE;
	
	@ClientString(id = 3604, message = "Make a Sewing Kit (Lv. 60-99)")
	public static NpcStringId MAKE_A_SEWING_KIT_LV_60_992;
	
	@ClientString(id = 3701, message = "Make Formal Wear (Lv. 60-99)")
	public static NpcStringId MAKE_FORMAL_WEAR_LV_60_99;
	
	@ClientString(id = 3702, message = "Make Formal Wear (Lv. 60-99) (In Progress)")
	public static NpcStringId MAKE_FORMAL_WEAR_LV_60_99_IN_PROGRESS;
	
	@ClientString(id = 3703, message = "Make Formal Wear (Lv. 60-99) (Done)")
	public static NpcStringId MAKE_FORMAL_WEAR_LV_60_99_DONE;
	
	@ClientString(id = 3704, message = "Make Formal Wear (Lv. 60-99)")
	public static NpcStringId MAKE_FORMAL_WEAR_LV_60_992;
	
	@ClientString(id = 3801, message = "Dragon Fangs (Lv. 19-29)")
	public static NpcStringId DRAGON_FANGS_LV_19_29;
	
	@ClientString(id = 3802, message = "Dragon Fangs (Lv. 19-29) (In Progress)")
	public static NpcStringId DRAGON_FANGS_LV_19_29_IN_PROGRESS;
	
	@ClientString(id = 3803, message = "Dragon Fangs (Lv. 19-29) (Done)")
	public static NpcStringId DRAGON_FANGS_LV_19_29_DONE;
	
	@ClientString(id = 3804, message = "Dragon Fangs (Lv. 19-29)")
	public static NpcStringId DRAGON_FANGS_LV_19_292;
	
	@ClientString(id = 3901, message = "Red-eyed Invader (Lv. 20-28)")
	public static NpcStringId RED_EYED_INVADER_LV_20_28;
	
	@ClientString(id = 3902, message = "Red-eyed Invader (Lv. 20-28) (In Progress)")
	public static NpcStringId RED_EYED_INVADER_LV_20_28_IN_PROGRESS;
	
	@ClientString(id = 3903, message = "Red-eyed Invader (Lv. 20-28) (Done)")
	public static NpcStringId RED_EYED_INVADER_LV_20_28_DONE;
	
	@ClientString(id = 3904, message = "Red-eyed Invader (Lv. 20-28)")
	public static NpcStringId RED_EYED_INVADER_LV_20_282;
	
	@ClientString(id = 4001, message = "A Special Order (Lv. 40-99)")
	public static NpcStringId A_SPECIAL_ORDER_LV_40_99;
	
	@ClientString(id = 4002, message = "A Special Order (Lv. 40-99) (In Progress)")
	public static NpcStringId A_SPECIAL_ORDER_LV_40_99_IN_PROGRESS;
	
	@ClientString(id = 4003, message = "A Special Order (Lv. 40-99) (Done)")
	public static NpcStringId A_SPECIAL_ORDER_LV_40_99_DONE;
	
	@ClientString(id = 4004, message = "A Special Order (Lv. 40-99)")
	public static NpcStringId A_SPECIAL_ORDER_LV_40_992;
	
	@ClientString(id = 4101, message = "The Adventurer's Challenge")
	public static NpcStringId THE_ADVENTURER_S_CHALLENGE;
	
	@ClientString(id = 4102, message = "The Adventurer's Challenge (In Progress)")
	public static NpcStringId THE_ADVENTURER_S_CHALLENGE_IN_PROGRESS;
	
	@ClientString(id = 4103, message = "The Adventurer's Challenge (Done)")
	public static NpcStringId THE_ADVENTURER_S_CHALLENGE_DONE;
	
	@ClientString(id = 4151, message = "Delivery duty complete. n Go find the Newbie Helper.")
	public static NpcStringId DELIVERY_DUTY_COMPLETE_N_GO_FIND_THE_NEWBIE_HELPER;
	
	@ClientString(id = 4152, message = "Acquisition of Soulshot for beginners complete. n Go find the Newbie Helper.")
	public static NpcStringId ACQUISITION_OF_SOULSHOT_FOR_BEGINNERS_COMPLETE_N_GO_FIND_THE_NEWBIE_HELPER;
	
	@ClientString(id = 4153, message = "Acquisition of Weapon Coupon for beginners complete. n Go speak with the Newbie Helper.")
	public static NpcStringId ACQUISITION_OF_WEAPON_COUPON_FOR_BEGINNERS_COMPLETE_N_GO_SPEAK_WITH_THE_NEWBIE_HELPER;
	
	@ClientString(id = 4154, message = "Acquisition of race-specific weapon complete. n Go find the Newbie Helper.")
	public static NpcStringId ACQUISITION_OF_RACE_SPECIFIC_WEAPON_COMPLETE_N_GO_FIND_THE_NEWBIE_HELPER;
	
	@ClientString(id = 4155, message = "Last duty complete. n Go find the Newbie Helper.")
	public static NpcStringId LAST_DUTY_COMPLETE_N_GO_FIND_THE_NEWBIE_HELPER;
	
	@ClientString(id = 4201, message = "Help the Uncle! (Lv. 25-99)")
	public static NpcStringId HELP_THE_UNCLE_LV_25_99;
	
	@ClientString(id = 4202, message = "Help the Uncle! (Lv. 25-99) (In Progress)")
	public static NpcStringId HELP_THE_UNCLE_LV_25_99_IN_PROGRESS;
	
	@ClientString(id = 4203, message = "Help the Uncle! (Lv. 25-99) (Done)")
	public static NpcStringId HELP_THE_UNCLE_LV_25_99_DONE;
	
	@ClientString(id = 4204, message = "Help the Uncle! (Lv. 25-99)")
	public static NpcStringId HELP_THE_UNCLE_LV_25_992;
	
	@ClientString(id = 4301, message = "Help the Sister! (Lv. 26-99)")
	public static NpcStringId HELP_THE_SISTER_LV_26_99;
	
	@ClientString(id = 4302, message = "Help the Sister! (Lv. 26-99) (In Progress)")
	public static NpcStringId HELP_THE_SISTER_LV_26_99_IN_PROGRESS;
	
	@ClientString(id = 4303, message = "Help the Sister! (Lv. 26-99) (Done)")
	public static NpcStringId HELP_THE_SISTER_LV_26_99_DONE;
	
	@ClientString(id = 4304, message = "Help the Sister! (Lv. 26-99)")
	public static NpcStringId HELP_THE_SISTER_LV_26_992;
	
	@ClientString(id = 4401, message = "Help the Son! (Lv. 24-99)")
	public static NpcStringId HELP_THE_SON_LV_24_99;
	
	@ClientString(id = 4402, message = "Help the Son! (Lv. 24-99) (In Progress)")
	public static NpcStringId HELP_THE_SON_LV_24_99_IN_PROGRESS;
	
	@ClientString(id = 4403, message = "Help the Son! (Lv. 24-99) (Done)")
	public static NpcStringId HELP_THE_SON_LV_24_99_DONE;
	
	@ClientString(id = 4404, message = "Help the Son! (Lv. 24-99)")
	public static NpcStringId HELP_THE_SON_LV_24_992;
	
	@ClientString(id = 4501, message = "To Talking Island")
	public static NpcStringId TO_TALKING_ISLAND;
	
	@ClientString(id = 4502, message = "To Talking Island (In Progress)")
	public static NpcStringId TO_TALKING_ISLAND_IN_PROGRESS;
	
	@ClientString(id = 4503, message = "To Talking Island (Done)")
	public static NpcStringId TO_TALKING_ISLAND_DONE;
	
	@ClientString(id = 4601, message = "Once More In the Arms of the Mother Tree")
	public static NpcStringId ONCE_MORE_IN_THE_ARMS_OF_THE_MOTHER_TREE;
	
	@ClientString(id = 4602, message = "Once More In the Arms of the Mother Tree (In Progress)")
	public static NpcStringId ONCE_MORE_IN_THE_ARMS_OF_THE_MOTHER_TREE_IN_PROGRESS;
	
	@ClientString(id = 4603, message = "Once More In the Arms of the Mother Tree (Done)")
	public static NpcStringId ONCE_MORE_IN_THE_ARMS_OF_THE_MOTHER_TREE_DONE;
	
	@ClientString(id = 4701, message = "Into the Dark Elven Forest")
	public static NpcStringId INTO_THE_DARK_ELVEN_FOREST;
	
	@ClientString(id = 4702, message = "Into the Dark Elven Forest (In Progress)")
	public static NpcStringId INTO_THE_DARK_ELVEN_FOREST_IN_PROGRESS;
	
	@ClientString(id = 4703, message = "Into the Dark Elven Forest (Done)")
	public static NpcStringId INTO_THE_DARK_ELVEN_FOREST_DONE;
	
	@ClientString(id = 4801, message = "To the Immortal Plateau")
	public static NpcStringId TO_THE_IMMORTAL_PLATEAU;
	
	@ClientString(id = 4802, message = "To the Immortal Plateau (In Progress)")
	public static NpcStringId TO_THE_IMMORTAL_PLATEAU_IN_PROGRESS;
	
	@ClientString(id = 4803, message = "To the Immortal Plateau (Done)")
	public static NpcStringId TO_THE_IMMORTAL_PLATEAU_DONE;
	
	@ClientString(id = 4901, message = "The Road Home")
	public static NpcStringId THE_ROAD_HOME;
	
	@ClientString(id = 4902, message = "The Road Home (In Progress)")
	public static NpcStringId THE_ROAD_HOME_IN_PROGRESS;
	
	@ClientString(id = 4903, message = "The Road Home (Done)")
	public static NpcStringId THE_ROAD_HOME_DONE;
	
	@ClientString(id = 5001, message = "Lanosco's Special Bait")
	public static NpcStringId LANOSCO_S_SPECIAL_BAIT;
	
	@ClientString(id = 5002, message = "Lanosco's Special Bait (In Progress)")
	public static NpcStringId LANOSCO_S_SPECIAL_BAIT_IN_PROGRESS;
	
	@ClientString(id = 5003, message = "Lanosco's Special Bait (Done)")
	public static NpcStringId LANOSCO_S_SPECIAL_BAIT_DONE;
	
	@ClientString(id = 5004, message = "Lasosco's Special Bait")
	public static NpcStringId LASOSCO_S_SPECIAL_BAIT;
	
	@ClientString(id = 5101, message = "O'Fulle's Special Bait")
	public static NpcStringId O_FULLE_S_SPECIAL_BAIT;
	
	@ClientString(id = 5102, message = "O'Fulle's Special Bait (In Progress)")
	public static NpcStringId O_FULLE_S_SPECIAL_BAIT_IN_PROGRESS;
	
	@ClientString(id = 5103, message = "O'Fulle's Special Bait (Done)")
	public static NpcStringId O_FULLE_S_SPECIAL_BAIT_DONE;
	
	@ClientString(id = 5104, message = "O'Fulle's Special Bait")
	public static NpcStringId O_FULLE_S_SPECIAL_BAIT2;
	
	@ClientString(id = 5201, message = "Willie's Special Bait")
	public static NpcStringId WILLIE_S_SPECIAL_BAIT;
	
	@ClientString(id = 5202, message = "Willie's Special Bait (In Progress)")
	public static NpcStringId WILLIE_S_SPECIAL_BAIT_IN_PROGRESS;
	
	@ClientString(id = 5203, message = "Willie's Special Bait (Done)")
	public static NpcStringId WILLIE_S_SPECIAL_BAIT_DONE;
	
	@ClientString(id = 5204, message = "Willie's Special Bait")
	public static NpcStringId WILLIE_S_SPECIAL_BAIT2;
	
	@ClientString(id = 5301, message = "Linnaeus' Special Bait")
	public static NpcStringId LINNAEUS_SPECIAL_BAIT;
	
	@ClientString(id = 5302, message = "Linnaeus' Special Bait (In Progress)")
	public static NpcStringId LINNAEUS_SPECIAL_BAIT_IN_PROGRESS;
	
	@ClientString(id = 5303, message = "Linnaeus' Special Bait (Done)")
	public static NpcStringId LINNAEUS_SPECIAL_BAIT_DONE;
	
	@ClientString(id = 5304, message = "Linnaeus' Special Bait")
	public static NpcStringId LINNAEUS_SPECIAL_BAIT2;
	
	@ClientString(id = 6001, message = "Good Work's Reward")
	public static NpcStringId GOOD_WORK_S_REWARD;
	
	@ClientString(id = 6002, message = "Good Work's Reward (In Progress)")
	public static NpcStringId GOOD_WORK_S_REWARD_IN_PROGRESS;
	
	@ClientString(id = 6003, message = "Good Work's Reward (Done)")
	public static NpcStringId GOOD_WORK_S_REWARD_DONE;
	
	@ClientString(id = 6051, message = "$s1! I must kill you. Blame your own curiosity.")
	public static NpcStringId S1_I_MUST_KILL_YOU_BLAME_YOUR_OWN_CURIOSITY;
	
	@ClientString(id = 6052, message = "You have good luck. I shall return.")
	public static NpcStringId YOU_HAVE_GOOD_LUCK_I_SHALL_RETURN;
	
	@ClientString(id = 6053, message = "You are strong. This was a mistake.")
	public static NpcStringId YOU_ARE_STRONG_THIS_WAS_A_MISTAKE;
	
	@ClientString(id = 6054, message = "Who are you to join in the battle? How upsetting.")
	public static NpcStringId WHO_ARE_YOU_TO_JOIN_IN_THE_BATTLE_HOW_UPSETTING;
	
	@ClientString(id = 6101, message = "Law Enforcement (Lv. 76-99)")
	public static NpcStringId LAW_ENFORCEMENT_LV_76_99;
	
	@ClientString(id = 6102, message = "Law Enforcement (Lv. 76-99) (In Progress)")
	public static NpcStringId LAW_ENFORCEMENT_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 6103, message = "Law Enforcement (Lv. 76-99) (Done)")
	public static NpcStringId LAW_ENFORCEMENT_LV_76_99_DONE;
	
	@ClientString(id = 6104, message = "Law Enforcement (Lv. 76-99)")
	public static NpcStringId LAW_ENFORCEMENT_LV_76_992;
	
	@ClientString(id = 6201, message = "Path of the Trooper")
	public static NpcStringId PATH_OF_THE_TROOPER;
	
	@ClientString(id = 6202, message = "Path of the Trooper (In Progress)")
	public static NpcStringId PATH_OF_THE_TROOPER_IN_PROGRESS;
	
	@ClientString(id = 6203, message = "Path of the Trooper (Done)")
	public static NpcStringId PATH_OF_THE_TROOPER_DONE;
	
	@ClientString(id = 6301, message = "Path of the Warder")
	public static NpcStringId PATH_OF_THE_WARDER;
	
	@ClientString(id = 6302, message = "Path of the Warder (In Progress)")
	public static NpcStringId PATH_OF_THE_WARDER_IN_PROGRESS;
	
	@ClientString(id = 6303, message = "Path of the Warder (Done)")
	public static NpcStringId PATH_OF_THE_WARDER_DONE;
	
	@ClientString(id = 6401, message = "Certified Berserker")
	public static NpcStringId CERTIFIED_BERSERKER;
	
	@ClientString(id = 6402, message = "Certified Berserker (In Progress)")
	public static NpcStringId CERTIFIED_BERSERKER_IN_PROGRESS;
	
	@ClientString(id = 6403, message = "Certified Berserker (Done)")
	public static NpcStringId CERTIFIED_BERSERKER_DONE;
	
	@ClientString(id = 6451, message = "$s1, did you come to help me?")
	public static NpcStringId S1_DID_YOU_COME_TO_HELP_ME;
	
	@ClientString(id = 6501, message = "Certified Soul Breaker")
	public static NpcStringId CERTIFIED_SOUL_BREAKER;
	
	@ClientString(id = 6502, message = "Certified Soul Breaker (In Progress)")
	public static NpcStringId CERTIFIED_SOUL_BREAKER_IN_PROGRESS;
	
	@ClientString(id = 6503, message = "Certified Soul Breaker (Done)")
	public static NpcStringId CERTIFIED_SOUL_BREAKER_DONE;
	
	@ClientString(id = 6551, message = "Drats! How could I be so wrong??")
	public static NpcStringId DRATS_HOW_COULD_I_BE_SO_WRONG;
	
	@ClientString(id = 6552, message = "$s1! Step back from the confounded box! I will take it myself!")
	public static NpcStringId S1_STEP_BACK_FROM_THE_CONFOUNDED_BOX_I_WILL_TAKE_IT_MYSELF;
	
	@ClientString(id = 6553, message = "$s1! I will be back soon. Stay there and don't you dare wander off!")
	public static NpcStringId S1_I_WILL_BE_BACK_SOON_STAY_THERE_AND_DON_T_YOU_DARE_WANDER_OFF;
	
	@ClientString(id = 6554, message = "Grr. I've been hit...")
	public static NpcStringId GRR_I_VE_BEEN_HIT;
	
	@ClientString(id = 6555, message = "Grr! Who are you and why have you stopped me?")
	public static NpcStringId GRR_WHO_ARE_YOU_AND_WHY_HAVE_YOU_STOPPED_ME;
	
	@ClientString(id = 6556, message = "I am late!")
	public static NpcStringId I_AM_LATE;
	
	@ClientString(id = 6557, message = "Good luck!")
	public static NpcStringId GOOD_LUCK;
	
	@ClientString(id = 6601, message = "Certified Arbalester")
	public static NpcStringId CERTIFIED_ARBALESTER;
	
	@ClientString(id = 6602, message = "Certified Arbalester (In Progress)")
	public static NpcStringId CERTIFIED_ARBALESTER_IN_PROGRESS;
	
	@ClientString(id = 6603, message = "Certified Arbalester (Done)")
	public static NpcStringId CERTIFIED_ARBALESTER_DONE;
	
	@ClientString(id = 6701, message = "Saga of the Doombringer")
	public static NpcStringId SAGA_OF_THE_DOOMBRINGER;
	
	@ClientString(id = 6702, message = "Saga of the Doombringer (In Progress)")
	public static NpcStringId SAGA_OF_THE_DOOMBRINGER_IN_PROGRESS;
	
	@ClientString(id = 6703, message = "Saga of the Doombringer (Done)")
	public static NpcStringId SAGA_OF_THE_DOOMBRINGER_DONE;
	
	@ClientString(id = 6750, message = "$s1! You seek the forbidden knowledge and I cannot let you have it!")
	public static NpcStringId S1_YOU_SEEK_THE_FORBIDDEN_KNOWLEDGE_AND_I_CANNOT_LET_YOU_HAVE_IT;
	
	@ClientString(id = 6751, message = "Is this all I am allowed to have?...")
	public static NpcStringId IS_THIS_ALL_I_AM_ALLOWED_TO_HAVE;
	
	@ClientString(id = 6752, message = "You defeated me, but our doom approaches...")
	public static NpcStringId YOU_DEFEATED_ME_BUT_OUR_DOOM_APPROACHES;
	
	@ClientString(id = 6753, message = "$s1! Who are you? Why are you bothering my minions?")
	public static NpcStringId S1_WHO_ARE_YOU_WHY_ARE_YOU_BOTHERING_MY_MINIONS;
	
	@ClientString(id = 6754, message = "Beefcake!!")
	public static NpcStringId BEEFCAKE;
	
	@ClientString(id = 6755, message = "Grr! Why are you sticking your nose in our business?")
	public static NpcStringId GRR_WHY_ARE_YOU_STICKING_YOUR_NOSE_IN_OUR_BUSINESS;
	
	@ClientString(id = 6756, message = "Farewell and watch your back!")
	public static NpcStringId FAREWELL_AND_WATCH_YOUR_BACK;
	
	@ClientString(id = 6757, message = "Kamael! Good to see you. I have something to ask you...")
	public static NpcStringId KAMAEL_GOOD_TO_SEE_YOU_I_HAVE_SOMETHING_TO_ASK_YOU;
	
	@ClientString(id = 6758, message = "$s1! Go get him!!")
	public static NpcStringId S1_GO_GET_HIM;
	
	@ClientString(id = 6759, message = "$s1! What are you doing? Attack him!")
	public static NpcStringId S1_WHAT_ARE_YOU_DOING_ATTACK_HIM;
	
	@ClientString(id = 6760, message = "$s1! Is ? your full potential?")
	public static NpcStringId S1_IS_YOUR_FULL_POTENTIAL;
	
	@ClientString(id = 6761, message = "Thanks! I must go and hunt down those that oppose me.")
	public static NpcStringId THANKS_I_MUST_GO_AND_HUNT_DOWN_THOSE_THAT_OPPOSE_ME;
	
	@ClientString(id = 6762, message = "You are so stubborn... I must follow him now...")
	public static NpcStringId YOU_ARE_SO_STUBBORN_I_MUST_FOLLOW_HIM_NOW;
	
	@ClientString(id = 6763, message = "Seek enlightenment from the Tablet.")
	public static NpcStringId SEEK_ENLIGHTENMENT_FROM_THE_TABLET;
	
	@ClientString(id = 6764, message = "Arrogant beings! You are all doomed!")
	public static NpcStringId ARROGANT_BEINGS_YOU_ARE_ALL_DOOMED;
	
	@ClientString(id = 6765, message = "My  time in your world has come to an end. Consider yourselves lucky...")
	public static NpcStringId MY_TIME_IN_YOUR_WORLD_HAS_COME_TO_AN_END_CONSIDER_YOURSELVES_LUCKY;
	
	@ClientString(id = 6766, message = "$s1! How dare you!!!")
	public static NpcStringId S1_HOW_DARE_YOU;
	
	@ClientString(id = 6767, message = "$s1! Ahhaa! Your god forsakes you!")
	public static NpcStringId S1_AHHAA_YOUR_GOD_FORSAKES_YOU;
	
	@ClientString(id = 6801, message = "Saga of the Soul Hound")
	public static NpcStringId SAGA_OF_THE_SOUL_HOUND;
	
	@ClientString(id = 6802, message = "Saga of the Soul Hound (In Progress)")
	public static NpcStringId SAGA_OF_THE_SOUL_HOUND_IN_PROGRESS;
	
	@ClientString(id = 6803, message = "Saga of the Soul Hound (Done)")
	public static NpcStringId SAGA_OF_THE_SOUL_HOUND_DONE;
	
	@ClientString(id = 6851, message = "$s1! Your time is up. Prepare to die a horrible death.")
	public static NpcStringId S1_YOUR_TIME_IS_UP_PREPARE_TO_DIE_A_HORRIBLE_DEATH;
	
	@ClientString(id = 6852, message = "Consider yourself lucky. The next time we meet, you will die - PERMANENTLY!")
	public static NpcStringId CONSIDER_YOURSELF_LUCKY_THE_NEXT_TIME_WE_MEET_YOU_WILL_DIE_PERMANENTLY;
	
	@ClientString(id = 6853, message = "Fare thee well! We shall meet again.")
	public static NpcStringId FARE_THEE_WELL_WE_SHALL_MEET_AGAIN;
	
	@ClientString(id = 6854, message = "$s1! Who are you? And better yet, why are you bothering my minions?")
	public static NpcStringId S1_WHO_ARE_YOU_AND_BETTER_YET_WHY_ARE_YOU_BOTHERING_MY_MINIONS;
	
	@ClientString(id = 6855, message = "Strength beyond strength!!")
	public static NpcStringId STRENGTH_BEYOND_STRENGTH;
	
	@ClientString(id = 6856, message = "Grr! Why are you sticking your nose where it doesn't belong?")
	public static NpcStringId GRR_WHY_ARE_YOU_STICKING_YOUR_NOSE_WHERE_IT_DOESN_T_BELONG;
	
	@ClientString(id = 6857, message = "You've won for now, but we will meet again!")
	public static NpcStringId YOU_VE_WON_FOR_NOW_BUT_WE_WILL_MEET_AGAIN;
	
	@ClientString(id = 6858, message = "Are they tired of following me?")
	public static NpcStringId ARE_THEY_TIRED_OF_FOLLOWING_ME;
	
	@ClientString(id = 6859, message = "$s1! Can you help me?")
	public static NpcStringId S1_CAN_YOU_HELP_ME;
	
	@ClientString(id = 6860, message = "Is that all you got, little $s1?")
	public static NpcStringId IS_THAT_ALL_YOU_GOT_LITTLE_S1;
	
	@ClientString(id = 6861, message = "$s1! Wake up fool! Don't let him get away!")
	public static NpcStringId S1_WAKE_UP_FOOL_DON_T_LET_HIM_GET_AWAY;
	
	@ClientString(id = 6862, message = "The path is clear, but be careful!")
	public static NpcStringId THE_PATH_IS_CLEAR_BUT_BE_CAREFUL;
	
	@ClientString(id = 6863, message = "I must follow someone now. See you around!")
	public static NpcStringId I_MUST_FOLLOW_SOMEONE_NOW_SEE_YOU_AROUND;
	
	@ClientString(id = 6864, message = "May we meet again.")
	public static NpcStringId MAY_WE_MEET_AGAIN;
	
	@ClientString(id = 6865, message = "Curses on those who blaspheme the Gods!")
	public static NpcStringId CURSES_ON_THOSE_WHO_BLASPHEME_THE_GODS;
	
	@ClientString(id = 6866, message = "Einhasad is calling me! You are lucky and I shall continue my punishment later!")
	public static NpcStringId EINHASAD_IS_CALLING_ME_YOU_ARE_LUCKY_AND_I_SHALL_CONTINUE_MY_PUNISHMENT_LATER;
	
	@ClientString(id = 6867, message = "By the power vested in me by the gods, you $s1, shall die!")
	public static NpcStringId BY_THE_POWER_VESTED_IN_ME_BY_THE_GODS_YOU_S1_SHALL_DIE;
	
	@ClientString(id = 6868, message = "$s1! I will not forget you!")
	public static NpcStringId S1_I_WILL_NOT_FORGET_YOU;
	
	@ClientString(id = 6901, message = "Saga of the Trickster")
	public static NpcStringId SAGA_OF_THE_TRICKSTER;
	
	@ClientString(id = 6902, message = "Saga of the Trickster (In Progress)")
	public static NpcStringId SAGA_OF_THE_TRICKSTER_IN_PROGRESS;
	
	@ClientString(id = 6903, message = "Saga of the Trickster (Done)")
	public static NpcStringId SAGA_OF_THE_TRICKSTER_DONE;
	
	@ClientString(id = 6950, message = "$s1! How dare you interfere! You shall pay for this!")
	public static NpcStringId S1_HOW_DARE_YOU_INTERFERE_YOU_SHALL_PAY_FOR_THIS;
	
	@ClientString(id = 6951, message = "Beleth is calling me. You are lucky but still a fool...")
	public static NpcStringId BELETH_IS_CALLING_ME_YOU_ARE_LUCKY_BUT_STILL_A_FOOL;
	
	@ClientString(id = 6952, message = "I shall take my leave, but will never surrender!")
	public static NpcStringId I_SHALL_TAKE_MY_LEAVE_BUT_WILL_NEVER_SURRENDER;
	
	@ClientString(id = 6953, message = "$s1! Who are you? Why are you bothering my minions?")
	public static NpcStringId S1_WHO_ARE_YOU_WHY_ARE_YOU_BOTHERING_MY_MINIONS2;
	
	@ClientString(id = 6954, message = "Cower before my awesome and mighty power!!")
	public static NpcStringId COWER_BEFORE_MY_AWESOME_AND_MIGHTY_POWER;
	
	@ClientString(id = 6955, message = "Grr! Can't you find something better to do with your time?")
	public static NpcStringId GRR_CAN_T_YOU_FIND_SOMETHING_BETTER_TO_DO_WITH_YOUR_TIME;
	
	@ClientString(id = 6956, message = "I shall take my leave, but your head will be mine some day.. Oh yes..yes it will!")
	public static NpcStringId I_SHALL_TAKE_MY_LEAVE_BUT_YOUR_HEAD_WILL_BE_MINE_SOME_DAY_OH_YES_YES_IT_WILL;
	
	@ClientString(id = 6957, message = "My children are stronger!")
	public static NpcStringId MY_CHILDREN_ARE_STRONGER;
	
	@ClientString(id = 6958, message = "$s1! Let's kill them all.")
	public static NpcStringId S1_LET_S_KILL_THEM_ALL;
	
	@ClientString(id = 6959, message = "$s1! Come my children...")
	public static NpcStringId S1_COME_MY_CHILDREN;
	
	@ClientString(id = 6960, message = "$s1! Muster your strength... Pick them off like chickens.")
	public static NpcStringId S1_MUSTER_YOUR_STRENGTH_PICK_THEM_OFF_LIKE_CHICKENS;
	
	@ClientString(id = 6961, message = "Thank you my children... Someday, we will meet again.")
	public static NpcStringId THANK_YOU_MY_CHILDREN_SOMEDAY_WE_WILL_MEET_AGAIN;
	
	@ClientString(id = 6962, message = "My children... Seek my enemies.")
	public static NpcStringId MY_CHILDREN_SEEK_MY_ENEMIES;
	
	@ClientString(id = 6963, message = "My children... I grant you my blessings...")
	public static NpcStringId MY_CHILDREN_I_GRANT_YOU_MY_BLESSINGS;
	
	@ClientString(id = 6964, message = "You worthless beings!")
	public static NpcStringId YOU_WORTHLESS_BEINGS;
	
	@ClientString(id = 6965, message = "My time in the Material Realm has ended. You are lucky...")
	public static NpcStringId MY_TIME_IN_THE_MATERIAL_REALM_HAS_ENDED_YOU_ARE_LUCKY;
	
	@ClientString(id = 6966, message = "$s1! Worthless beings! Begone!")
	public static NpcStringId S1_WORTHLESS_BEINGS_BEGONE;
	
	@ClientString(id = 6967, message = "$s1! You are the meaning of the word 'danger'...")
	public static NpcStringId S1_YOU_ARE_THE_MEANING_OF_THE_WORD_DANGER;
	
	@ClientString(id = 7001, message = "Saga of the Phoenix Knight")
	public static NpcStringId SAGA_OF_THE_PHOENIX_KNIGHT;
	
	@ClientString(id = 7002, message = "Saga of the Phoenix Knight (In Progress)")
	public static NpcStringId SAGA_OF_THE_PHOENIX_KNIGHT_IN_PROGRESS;
	
	@ClientString(id = 7003, message = "Saga of the Phoenix Knight (Done)")
	public static NpcStringId SAGA_OF_THE_PHOENIX_KNIGHT_DONE;
	
	@ClientString(id = 7050, message = "You made it here, $s1. I'll show my strength. Die!")
	public static NpcStringId YOU_MADE_IT_HERE_S1_I_LL_SHOW_MY_STRENGTH_DIE;
	
	@ClientString(id = 7051, message = "Ha! You failed! Are you ready to quit?")
	public static NpcStringId HA_YOU_FAILED_ARE_YOU_READY_TO_QUIT;
	
	@ClientString(id = 7052, message = "I'm the strongest! I lost everything to win!")
	public static NpcStringId I_M_THE_STRONGEST_I_LOST_EVERYTHING_TO_WIN;
	
	@ClientString(id = 7053, message = "$s1! Are you doing this because they're Halisha's minions?")
	public static NpcStringId S1_ARE_YOU_DOING_THIS_BECAUSE_THEY_RE_HALISHA_S_MINIONS;
	
	@ClientString(id = 7054, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA;
	
	@ClientString(id = 7055, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS;
	
	@ClientString(id = 7056, message = "This fight is a waste of time. Goodbye!")
	public static NpcStringId THIS_FIGHT_IS_A_WASTE_OF_TIME_GOODBYE;
	
	@ClientString(id = 7057, message = "Every cloud has a silver lining, don't you think?")
	public static NpcStringId EVERY_CLOUD_HAS_A_SILVER_LINING_DON_T_YOU_THINK;
	
	@ClientString(id = 7058, message = "$s1! Don't listen to this demon.")
	public static NpcStringId S1_DON_T_LISTEN_TO_THIS_DEMON;
	
	@ClientString(id = 7059, message = "$s1! Hesitation betrays your heart. Fight!")
	public static NpcStringId S1_HESITATION_BETRAYS_YOUR_HEART_FIGHT;
	
	@ClientString(id = 7060, message = "$s1! Isn't protecting somebody worthy? Isn't that justice?")
	public static NpcStringId S1_ISN_T_PROTECTING_SOMEBODY_WORTHY_ISN_T_THAT_JUSTICE;
	
	@ClientString(id = 7061, message = "I have urgent business! I gotta go.")
	public static NpcStringId I_HAVE_URGENT_BUSINESS_I_GOTTA_GO;
	
	@ClientString(id = 7062, message = "Are my efforts in vain? Is this the end?")
	public static NpcStringId ARE_MY_EFFORTS_IN_VAIN_IS_THIS_THE_END;
	
	@ClientString(id = 7063, message = "Goodbye, friend. I hope to see you again.")
	public static NpcStringId GOODBYE_FRIEND_I_HOPE_TO_SEE_YOU_AGAIN;
	
	@ClientString(id = 7064, message = "Knights are always foolish!")
	public static NpcStringId KNIGHTS_ARE_ALWAYS_FOOLISH;
	
	@ClientString(id = 7065, message = "I'll show mercy on you for now.")
	public static NpcStringId I_LL_SHOW_MERCY_ON_YOU_FOR_NOW;
	
	@ClientString(id = 7066, message = "$s1! Your justice is just hypocrisy if you give up on what you've promised to protect.")
	public static NpcStringId S1_YOUR_JUSTICE_IS_JUST_HYPOCRISY_IF_YOU_GIVE_UP_ON_WHAT_YOU_VE_PROMISED_TO_PROTECT;
	
	@ClientString(id = 7067, message = "$s1...Don't think you've won! Your dark shadow will always follow you...hypocrite!")
	public static NpcStringId S1_DON_T_THINK_YOU_VE_WON_YOUR_DARK_SHADOW_WILL_ALWAYS_FOLLOW_YOU_HYPOCRITE;
	
	@ClientString(id = 7101, message = "Saga of Eva's Templar")
	public static NpcStringId SAGA_OF_EVA_S_TEMPLAR;
	
	@ClientString(id = 7102, message = "Saga of Eva's Templar (In Progress)")
	public static NpcStringId SAGA_OF_EVA_S_TEMPLAR_IN_PROGRESS;
	
	@ClientString(id = 7103, message = "Saga of Eva's Templar (Done)")
	public static NpcStringId SAGA_OF_EVA_S_TEMPLAR_DONE;
	
	@ClientString(id = 7150, message = "A temple knight guards the Mother Tree! $s1! Has Human contact made you forget that?")
	public static NpcStringId A_TEMPLE_KNIGHT_GUARDS_THE_MOTHER_TREE_S1_HAS_HUMAN_CONTACT_MADE_YOU_FORGET_THAT;
	
	@ClientString(id = 7151, message = "I must stop. Remember, the ones you're protecting will someday defeat the Elves.")
	public static NpcStringId I_MUST_STOP_REMEMBER_THE_ONES_YOU_RE_PROTECTING_WILL_SOMEDAY_DEFEAT_THE_ELVES;
	
	@ClientString(id = 7152, message = "What! That will just strengthen the enemy!")
	public static NpcStringId WHAT_THAT_WILL_JUST_STRENGTHEN_THE_ENEMY;
	
	@ClientString(id = 7153, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S1;
	
	@ClientString(id = 7154, message = "My spirit is releasing from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASING_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA;
	
	@ClientString(id = 7155, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS2;
	
	@ClientString(id = 7156, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE;
	
	@ClientString(id = 7157, message = "I'm not like my brother Panacea. Ghost of the past, begone!")
	public static NpcStringId I_M_NOT_LIKE_MY_BROTHER_PANACEA_GHOST_OF_THE_PAST_BEGONE;
	
	@ClientString(id = 7158, message = "The Elves no longer rule! Help me, $s1!")
	public static NpcStringId THE_ELVES_NO_LONGER_RULE_HELP_ME_S1;
	
	@ClientString(id = 7159, message = "Don't give up, $s1! He' a demon from the past!")
	public static NpcStringId DON_T_GIVE_UP_S1_HE_A_DEMON_FROM_THE_PAST;
	
	@ClientString(id = 7160, message = "Be proud, $s1. We protect this world together.")
	public static NpcStringId BE_PROUD_S1_WE_PROTECT_THIS_WORLD_TOGETHER;
	
	@ClientString(id = 7161, message = "I have to go. I've got some business to take care of.")
	public static NpcStringId I_HAVE_TO_GO_I_VE_GOT_SOME_BUSINESS_TO_TAKE_CARE_OF;
	
	@ClientString(id = 7162, message = "Ugh! Don't let him get away!")
	public static NpcStringId UGH_DON_T_LET_HIM_GET_AWAY;
	
	@ClientString(id = 7163, message = "Don't forget your pride. You're protecting the world!")
	public static NpcStringId DON_T_FORGET_YOUR_PRIDE_YOU_RE_PROTECTING_THE_WORLD;
	
	@ClientString(id = 7164, message = "Ha, ha, ha!...")
	public static NpcStringId HA_HA_HA;
	
	@ClientString(id = 7165, message = "Kuh, huh...")
	public static NpcStringId KUH_HUH;
	
	@ClientString(id = 7166, message = "Aah! Kuh...$s1!...Kuh, huh...")
	public static NpcStringId AAH_KUH_S1_KUH_HUH;
	
	@ClientString(id = 7167, message = "$s1!...Re... mem... Ugh...Uh...")
	public static NpcStringId S1_RE_MEM_UGH_UH;
	
	@ClientString(id = 7201, message = "Saga of the Sword Muse")
	public static NpcStringId SAGA_OF_THE_SWORD_MUSE;
	
	@ClientString(id = 7202, message = "Saga of the Sword Muse (In Progress)")
	public static NpcStringId SAGA_OF_THE_SWORD_MUSE_IN_PROGRESS;
	
	@ClientString(id = 7203, message = "Saga of the Sword Muse (Done)")
	public static NpcStringId SAGA_OF_THE_SWORD_MUSE_DONE;
	
	@ClientString(id = 7250, message = "$s1, You'd better listen.")
	public static NpcStringId S1_YOU_D_BETTER_LISTEN;
	
	@ClientString(id = 7251, message = "Huh? It's curtain time! I won't get any money.")
	public static NpcStringId HUH_IT_S_CURTAIN_TIME_I_WON_T_GET_ANY_MONEY;
	
	@ClientString(id = 7252, message = "Ugh...It can't be...!")
	public static NpcStringId UGH_IT_CAN_T_BE;
	
	@ClientString(id = 7253, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S12;
	
	@ClientString(id = 7254, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA2;
	
	@ClientString(id = 7255, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS3;
	
	@ClientString(id = 7256, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE2;
	
	@ClientString(id = 7257, message = "You won't get away this time, Narcissus!")
	public static NpcStringId YOU_WON_T_GET_AWAY_THIS_TIME_NARCISSUS;
	
	@ClientString(id = 7258, message = "$s1! Help me!")
	public static NpcStringId S1_HELP_ME;
	
	@ClientString(id = 7259, message = "You must be aware of your audience when singing, %s! =) Join us $s1! A song that nobody listens to is empty.")
	public static NpcStringId YOU_MUST_BE_AWARE_OF_YOUR_AUDIENCE_WHEN_SINGING_S_JOIN_US_S1_A_SONG_THAT_NOBODY_LISTENS_TO_IS_EMPTY;
	
	@ClientString(id = 7260, message = "You must work harder to be victorious, $s1.")
	public static NpcStringId YOU_MUST_WORK_HARDER_TO_BE_VICTORIOUS_S1;
	
	@ClientString(id = 7261, message = "My song is over, I must go. Goodbye!")
	public static NpcStringId MY_SONG_IS_OVER_I_MUST_GO_GOODBYE;
	
	@ClientString(id = 7262, message = "How could I miss!")
	public static NpcStringId HOW_COULD_I_MISS;
	
	@ClientString(id = 7263, message = "Don't forget. Song comes with harmony.")
	public static NpcStringId DON_T_FORGET_SONG_COMES_WITH_HARMONY;
	
	@ClientString(id = 7264, message = "Sing. Everyone will listen.")
	public static NpcStringId SING_EVERYONE_WILL_LISTEN;
	
	@ClientString(id = 7265, message = "You don't deserve my blessing.")
	public static NpcStringId YOU_DON_T_DESERVE_MY_BLESSING;
	
	@ClientString(id = 7266, message = "Do you reject my blessing, $s1?")
	public static NpcStringId DO_YOU_REJECT_MY_BLESSING_S1;
	
	@ClientString(id = 7267, message = "But why, $s1. Everyone would praise you!")
	public static NpcStringId BUT_WHY_S1_EVERYONE_WOULD_PRAISE_YOU;
	
	@ClientString(id = 7301, message = "Saga of the Duelist")
	public static NpcStringId SAGA_OF_THE_DUELIST;
	
	@ClientString(id = 7302, message = "Saga of the Duelist (In Progress)")
	public static NpcStringId SAGA_OF_THE_DUELIST_IN_PROGRESS;
	
	@ClientString(id = 7303, message = "Saga of the Duelist (Done)")
	public static NpcStringId SAGA_OF_THE_DUELIST_DONE;
	
	@ClientString(id = 7350, message = "$s1! Attack me? I'm immortal! I'm unrivaled!")
	public static NpcStringId S1_ATTACK_ME_I_M_IMMORTAL_I_M_UNRIVALED;
	
	@ClientString(id = 7351, message = "Ha! I'm immortal. This scar will soon heal. You'll die next time.")
	public static NpcStringId HA_I_M_IMMORTAL_THIS_SCAR_WILL_SOON_HEAL_YOU_LL_DIE_NEXT_TIME;
	
	@ClientString(id = 7352, message = "Metellus! You promised me an immortal life! How could I, Swordmaster Iron, lose?")
	public static NpcStringId METELLUS_YOU_PROMISED_ME_AN_IMMORTAL_LIFE_HOW_COULD_I_SWORDMASTER_IRON_LOSE;
	
	@ClientString(id = 7353, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S13;
	
	@ClientString(id = 7354, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA3;
	
	@ClientString(id = 7355, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS4;
	
	@ClientString(id = 7356, message = "This is a waste of time... Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE3;
	
	@ClientString(id = 7357, message = "Fallen Angel? It's worth trying.")
	public static NpcStringId FALLEN_ANGEL_IT_S_WORTH_TRYING;
	
	@ClientString(id = 7358, message = "Hey $s1! Why don't you join? It's your best shot!")
	public static NpcStringId HEY_S1_WHY_DON_T_YOU_JOIN_IT_S_YOUR_BEST_SHOT;
	
	@ClientString(id = 7359, message = "Are you interested in immortal life, $s1? It's too boring for me.")
	public static NpcStringId ARE_YOU_INTERESTED_IN_IMMORTAL_LIFE_S1_IT_S_TOO_BORING_FOR_ME;
	
	@ClientString(id = 7360, message = "Excellent, $s1! Show me what you learned when your life was on the line!")
	public static NpcStringId EXCELLENT_S1_SHOW_ME_WHAT_YOU_LEARNED_WHEN_YOUR_LIFE_WAS_ON_THE_LINE;
	
	@ClientString(id = 7361, message = "I have to go take a break.")
	public static NpcStringId I_HAVE_TO_GO_TAKE_A_BREAK;
	
	@ClientString(id = 7362, message = "You missed?! What a fool you are!")
	public static NpcStringId YOU_MISSED_WHAT_A_FOOL_YOU_ARE;
	
	@ClientString(id = 7363, message = "Fighting without risk, training without danger and growing without hardship will only lead to an inflated ego. Don't forget.")
	public static NpcStringId FIGHTING_WITHOUT_RISK_TRAINING_WITHOUT_DANGER_AND_GROWING_WITHOUT_HARDSHIP_WILL_ONLY_LEAD_TO_AN_INFLATED_EGO_DON_T_FORGET;
	
	@ClientString(id = 7364, message = "Do you want an immortal life?")
	public static NpcStringId DO_YOU_WANT_AN_IMMORTAL_LIFE;
	
	@ClientString(id = 7365, message = "Think it over. An immortal life and assured victory...")
	public static NpcStringId THINK_IT_OVER_AN_IMMORTAL_LIFE_AND_ASSURED_VICTORY;
	
	@ClientString(id = 7366, message = "That's good, $s1! Do you want my blessing to improve your skills?")
	public static NpcStringId THAT_S_GOOD_S1_DO_YOU_WANT_MY_BLESSING_TO_IMPROVE_YOUR_SKILLS;
	
	@ClientString(id = 7367, message = "$s1! Why do you reject guaranteed victory?")
	public static NpcStringId S1_WHY_DO_YOU_REJECT_GUARANTEED_VICTORY;
	
	@ClientString(id = 7401, message = "Saga of the Dreadnought")
	public static NpcStringId SAGA_OF_THE_DREADNOUGHT;
	
	@ClientString(id = 7402, message = "Saga of the Dreadnought (In Progress)")
	public static NpcStringId SAGA_OF_THE_DREADNOUGHT_IN_PROGRESS;
	
	@ClientString(id = 7403, message = "Saga of the Dreadnought (Done)")
	public static NpcStringId SAGA_OF_THE_DREADNOUGHT_DONE;
	
	@ClientString(id = 7450, message = "In the name of gods, I punish you, $s1! You can't rival us all, no matter how strong you think you are!")
	public static NpcStringId IN_THE_NAME_OF_GODS_I_PUNISH_YOU_S1_YOU_CAN_T_RIVAL_US_ALL_NO_MATTER_HOW_STRONG_YOU_THINK_YOU_ARE;
	
	@ClientString(id = 7451, message = "I have to stop for now, but I'll defeat the power of the dragon yet!")
	public static NpcStringId I_HAVE_TO_STOP_FOR_NOW_BUT_I_LL_DEFEAT_THE_POWER_OF_THE_DRAGON_YET;
	
	@ClientString(id = 7452, message = "It is...The power that shouldn't live!")
	public static NpcStringId IT_IS_THE_POWER_THAT_SHOULDN_T_LIVE;
	
	@ClientString(id = 7453, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S14;
	
	@ClientString(id = 7454, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA4;
	
	@ClientString(id = 7455, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS5;
	
	@ClientString(id = 7456, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE4;
	
	@ClientString(id = 7457, message = "Isn't it unwise for an angel to interfere in Human affairs?")
	public static NpcStringId ISN_T_IT_UNWISE_FOR_AN_ANGEL_TO_INTERFERE_IN_HUMAN_AFFAIRS;
	
	@ClientString(id = 7458, message = "This is tough! $s1, would you help me?")
	public static NpcStringId THIS_IS_TOUGH_S1_WOULD_YOU_HELP_ME;
	
	@ClientString(id = 7459, message = "$s1! He's keeping an eye on all those in succession to the blood of dragons, including you!")
	public static NpcStringId S1_HE_S_KEEPING_AN_EYE_ON_ALL_THOSE_IN_SUCCESSION_TO_THE_BLOOD_OF_DRAGONS_INCLUDING_YOU;
	
	@ClientString(id = 7460, message = "Attack the rear, $s1! If I'm killed, you're next!")
	public static NpcStringId ATTACK_THE_REAR_S1_IF_I_M_KILLED_YOU_RE_NEXT;
	
	@ClientString(id = 7461, message = "I can't stay any longer. There are too many eyes on us. Farewell!")
	public static NpcStringId I_CAN_T_STAY_ANY_LONGER_THERE_ARE_TOO_MANY_EYES_ON_US_FAREWELL;
	
	@ClientString(id = 7462, message = "Get away? You're still alive. But...")
	public static NpcStringId GET_AWAY_YOU_RE_STILL_ALIVE_BUT;
	
	@ClientString(id = 7463, message = "If we can't avoid this fight, we'll need great strength. It's the only way to peace.")
	public static NpcStringId IF_WE_CAN_T_AVOID_THIS_FIGHT_WE_LL_NEED_GREAT_STRENGTH_IT_S_THE_ONLY_WAY_TO_PEACE;
	
	@ClientString(id = 7464, message = "Warlord, you'll never learn the techniques of the dragon!")
	public static NpcStringId WARLORD_YOU_LL_NEVER_LEARN_THE_TECHNIQUES_OF_THE_DRAGON;
	
	@ClientString(id = 7465, message = "Hey, why bother with this. Isn't it your mother's business?")
	public static NpcStringId HEY_WHY_BOTHER_WITH_THIS_ISN_T_IT_YOUR_MOTHER_S_BUSINESS;
	
	@ClientString(id = 7466, message = "$s1! Are you against your mother's wishes? You're not worthy of the secrets of Shilen's children!")
	public static NpcStringId S1_ARE_YOU_AGAINST_YOUR_MOTHER_S_WISHES_YOU_RE_NOT_WORTHY_OF_THE_SECRETS_OF_SHILEN_S_CHILDREN;
	
	@ClientString(id = 7467, message = "Excellent technique, $s1. Unfortunately, you're the one destined for tragedy!")
	public static NpcStringId EXCELLENT_TECHNIQUE_S1_UNFORTUNATELY_YOU_RE_THE_ONE_DESTINED_FOR_TRAGEDY;
	
	@ClientString(id = 7501, message = "Saga of the Titan")
	public static NpcStringId SAGA_OF_THE_TITAN;
	
	@ClientString(id = 7502, message = "Saga of the Titan (In Progress)")
	public static NpcStringId SAGA_OF_THE_TITAN_IN_PROGRESS;
	
	@ClientString(id = 7503, message = "Saga of the Titan (Done)")
	public static NpcStringId SAGA_OF_THE_TITAN_DONE;
	
	@ClientString(id = 7550, message = "$s1! You may follow me, but an Orc is no match for my giant's strength!")
	public static NpcStringId S1_YOU_MAY_FOLLOW_ME_BUT_AN_ORC_IS_NO_MATCH_FOR_MY_GIANT_S_STRENGTH;
	
	@ClientString(id = 7551, message = "Kuh...My body fails..This is the end!")
	public static NpcStringId KUH_MY_BODY_FAILS_THIS_IS_THE_END;
	
	@ClientString(id = 7552, message = "How could I lose with the powers of a giant? Aargh!!!")
	public static NpcStringId HOW_COULD_I_LOSE_WITH_THE_POWERS_OF_A_GIANT_AARGH;
	
	@ClientString(id = 7553, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S15;
	
	@ClientString(id = 7554, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA5;
	
	@ClientString(id = 7555, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS6;
	
	@ClientString(id = 7556, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE5;
	
	@ClientString(id = 7557, message = "That's the enemy.")
	public static NpcStringId THAT_S_THE_ENEMY;
	
	@ClientString(id = 7558, message = "Hmm.. $s1! Will you just stand there doing nothing?")
	public static NpcStringId HMM_S1_WILL_YOU_JUST_STAND_THERE_DOING_NOTHING;
	
	@ClientString(id = 7559, message = "$s1. Nothing risked, nothing gained. Only those who fight enjoy the spoils of victory.")
	public static NpcStringId S1_NOTHING_RISKED_NOTHING_GAINED_ONLY_THOSE_WHO_FIGHT_ENJOY_THE_SPOILS_OF_VICTORY;
	
	@ClientString(id = 7560, message = "A sword isn't jewelry, $s1. Don't you agree?")
	public static NpcStringId A_SWORD_ISN_T_JEWELRY_S1_DON_T_YOU_AGREE;
	
	@ClientString(id = 7561, message = "With no fight, I have no reason to stay.")
	public static NpcStringId WITH_NO_FIGHT_I_HAVE_NO_REASON_TO_STAY;
	
	@ClientString(id = 7562, message = "Miss...")
	public static NpcStringId MISS;
	
	@ClientString(id = 7563, message = "Pick your battles wisely, or you'll regret it.")
	public static NpcStringId PICK_YOUR_BATTLES_WISELY_OR_YOU_LL_REGRET_IT;
	
	@ClientString(id = 7564, message = "What a fool to challenge the giant of the Oroka tribe!")
	public static NpcStringId WHAT_A_FOOL_TO_CHALLENGE_THE_GIANT_OF_THE_OROKA_TRIBE;
	
	@ClientString(id = 7565, message = "Running low on steam. I must withdraw.")
	public static NpcStringId RUNNING_LOW_ON_STEAM_I_MUST_WITHDRAW;
	
	@ClientString(id = 7566, message = "$s1. You're the one who defeated Guardian Muhark!")
	public static NpcStringId S1_YOU_RE_THE_ONE_WHO_DEFEATED_GUARDIAN_MUHARK;
	
	@ClientString(id = 7567, message = "$s1....! I must succeed...")
	public static NpcStringId S1_I_MUST_SUCCEED;
	
	@ClientString(id = 7601, message = "Saga of the Grand Khavatari")
	public static NpcStringId SAGA_OF_THE_GRAND_KHAVATARI;
	
	@ClientString(id = 7602, message = "Saga of the Grand Khavatari (In Progress)")
	public static NpcStringId SAGA_OF_THE_GRAND_KHAVATARI_IN_PROGRESS;
	
	@ClientString(id = 7603, message = "Saga of the Grand Khavatari (Done)")
	public static NpcStringId SAGA_OF_THE_GRAND_KHAVATARI_DONE;
	
	@ClientString(id = 7650, message = "$s1... Would you fight Uruz, who has reached the power of Azira?")
	public static NpcStringId S1_WOULD_YOU_FIGHT_URUZ_WHO_HAS_REACHED_THE_POWER_OF_AZIRA;
	
	@ClientString(id = 7651, message = "I can't handle the power of Azira yet. First...")
	public static NpcStringId I_CAN_T_HANDLE_THE_POWER_OF_AZIRA_YET_FIRST;
	
	@ClientString(id = 7652, message = "This can't be happening! I have the power of Azira! How could I fall so easily?")
	public static NpcStringId THIS_CAN_T_BE_HAPPENING_I_HAVE_THE_POWER_OF_AZIRA_HOW_COULD_I_FALL_SO_EASILY;
	
	@ClientString(id = 7653, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S16;
	
	@ClientString(id = 7654, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA6;
	
	@ClientString(id = 7655, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS7;
	
	@ClientString(id = 7656, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE6;
	
	@ClientString(id = 7657, message = "Azira, born from the Evil Flame, I'll kill you with my bare hands!")
	public static NpcStringId AZIRA_BORN_FROM_THE_EVIL_FLAME_I_LL_KILL_YOU_WITH_MY_BARE_HANDS;
	
	@ClientString(id = 7658, message = "$s1! In the name of Khavatari Hubai, strike this evil with your fists!")
	public static NpcStringId S1_IN_THE_NAME_OF_KHAVATARI_HUBAI_STRIKE_THIS_EVIL_WITH_YOUR_FISTS;
	
	@ClientString(id = 7659, message = "$s1! Attack from both sides! Hit hard!")
	public static NpcStringId S1_ATTACK_FROM_BOTH_SIDES_HIT_HARD;
	
	@ClientString(id = 7660, message = "$s1! Strike with all your heart. It must work.")
	public static NpcStringId S1_STRIKE_WITH_ALL_YOUR_HEART_IT_MUST_WORK;
	
	@ClientString(id = 7661, message = "Damn! It's time to go. Until next time.")
	public static NpcStringId DAMN_IT_S_TIME_TO_GO_UNTIL_NEXT_TIME;
	
	@ClientString(id = 7662, message = "Evil Flame Spirit! I won't give up!")
	public static NpcStringId EVIL_FLAME_SPIRIT_I_WON_T_GIVE_UP;
	
	@ClientString(id = 7663, message = "My fist works even on the evil spirit. Don't forget!")
	public static NpcStringId MY_FIST_WORKS_EVEN_ON_THE_EVIL_SPIRIT_DON_T_FORGET;
	
	@ClientString(id = 7664, message = "Foolish Khavatari...Do you think your power will work on me? I'm the source of your martial art!")
	public static NpcStringId FOOLISH_KHAVATARI_DO_YOU_THINK_YOUR_POWER_WILL_WORK_ON_ME_I_M_THE_SOURCE_OF_YOUR_MARTIAL_ART;
	
	@ClientString(id = 7665, message = "No more games...")
	public static NpcStringId NO_MORE_GAMES;
	
	@ClientString(id = 7666, message = "$s1...Are you next after Khavatari? Do you know who I am?")
	public static NpcStringId S1_ARE_YOU_NEXT_AFTER_KHAVATARI_DO_YOU_KNOW_WHO_I_AM;
	
	@ClientString(id = 7667, message = "$s1...Kashu. Not a bad attack. I can't hold on much longer.")
	public static NpcStringId S1_KASHU_NOT_A_BAD_ATTACK_I_CAN_T_HOLD_ON_MUCH_LONGER;
	
	@ClientString(id = 7701, message = "Saga of the Dominator")
	public static NpcStringId SAGA_OF_THE_DOMINATOR;
	
	@ClientString(id = 7702, message = "Saga of the Dominator (In Progress)")
	public static NpcStringId SAGA_OF_THE_DOMINATOR_IN_PROGRESS;
	
	@ClientString(id = 7703, message = "Saga of the Dominator (Done)")
	public static NpcStringId SAGA_OF_THE_DOMINATOR_DONE;
	
	@ClientString(id = 7750, message = "$s1, Akkan, you can't be my rival! I'll kill everything! I'll be the king!")
	public static NpcStringId S1_AKKAN_YOU_CAN_T_BE_MY_RIVAL_I_LL_KILL_EVERYTHING_I_LL_BE_THE_KING;
	
	@ClientString(id = 7751, message = "Ha! I'll show mercy on you this time. I know well of your technique!")
	public static NpcStringId HA_I_LL_SHOW_MERCY_ON_YOU_THIS_TIME_I_KNOW_WELL_OF_YOUR_TECHNIQUE;
	
	@ClientString(id = 7752, message = "I have in me the blood of a king! How could I lose?!")
	public static NpcStringId I_HAVE_IN_ME_THE_BLOOD_OF_A_KING_HOW_COULD_I_LOSE;
	
	@ClientString(id = 7753, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S17;
	
	@ClientString(id = 7754, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA7;
	
	@ClientString(id = 7755, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS8;
	
	@ClientString(id = 7756, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE7;
	
	@ClientString(id = 7757, message = "Are you....tyrant!")
	public static NpcStringId ARE_YOU_TYRANT;
	
	@ClientString(id = 7758, message = "You're not a king! You're just a tyrant! $s1, we must fight together!")
	public static NpcStringId YOU_RE_NOT_A_KING_YOU_RE_JUST_A_TYRANT_S1_WE_MUST_FIGHT_TOGETHER;
	
	@ClientString(id = 7759, message = "Such rule is ruining the country! $s1, I can't bear this tyranny any longer!")
	public static NpcStringId SUCH_RULE_IS_RUINING_THE_COUNTRY_S1_I_CAN_T_BEAR_THIS_TYRANNY_ANY_LONGER;
	
	@ClientString(id = 7760, message = "$s1, leaders must always resist tyranny!")
	public static NpcStringId S1_LEADERS_MUST_ALWAYS_RESIST_TYRANNY;
	
	@ClientString(id = 7761, message = "I stayed too long. I'll be punished for being away so long.")
	public static NpcStringId I_STAYED_TOO_LONG_I_LL_BE_PUNISHED_FOR_BEING_AWAY_SO_LONG;
	
	@ClientString(id = 7762, message = "He got away, Dammit! We must catch this dark soul!")
	public static NpcStringId HE_GOT_AWAY_DAMMIT_WE_MUST_CATCH_THIS_DARK_SOUL;
	
	@ClientString(id = 7763, message = "What is a king? What must one do to be a good king? Think it over.")
	public static NpcStringId WHAT_IS_A_KING_WHAT_MUST_ONE_DO_TO_BE_A_GOOD_KING_THINK_IT_OVER;
	
	@ClientString(id = 7764, message = "Kneel down before me! Foolish people!")
	public static NpcStringId KNEEL_DOWN_BEFORE_ME_FOOLISH_PEOPLE;
	
	@ClientString(id = 7765, message = "It's time for the king to leave! Bow your head and say goodbye!")
	public static NpcStringId IT_S_TIME_FOR_THE_KING_TO_LEAVE_BOW_YOUR_HEAD_AND_SAY_GOODBYE;
	
	@ClientString(id = 7766, message = "$s1! You dare to fight me? A king's power must be great to rule!")
	public static NpcStringId S1_YOU_DARE_TO_FIGHT_ME_A_KING_S_POWER_MUST_BE_GREAT_TO_RULE;
	
	@ClientString(id = 7767, message = "You would fight the king, $s1? Traitor!")
	public static NpcStringId YOU_WOULD_FIGHT_THE_KING_S1_TRAITOR;
	
	@ClientString(id = 7801, message = "Saga of the Doomcryer")
	public static NpcStringId SAGA_OF_THE_DOOMCRYER;
	
	@ClientString(id = 7802, message = "Saga of the Doomcryer (In Progress)")
	public static NpcStringId SAGA_OF_THE_DOOMCRYER_IN_PROGRESS;
	
	@ClientString(id = 7803, message = "Saga of the Doomcryer (Done)")
	public static NpcStringId SAGA_OF_THE_DOOMCRYER_DONE;
	
	@ClientString(id = 7850, message = "Tejakar Sharuhi! $s1, I'll show you the power of Sharuhi Mouth Mudaha!")
	public static NpcStringId TEJAKAR_SHARUHI_S1_I_LL_SHOW_YOU_THE_POWER_OF_SHARUHI_MOUTH_MUDAHA;
	
	@ClientString(id = 7851, message = "Aaargh! My soul won't keep quiet. Now I must take my leave.")
	public static NpcStringId AAARGH_MY_SOUL_WON_T_KEEP_QUIET_NOW_I_MUST_TAKE_MY_LEAVE;
	
	@ClientString(id = 7852, message = "No, Sharuhi. You're soul is mine!")
	public static NpcStringId NO_SHARUHI_YOU_RE_SOUL_IS_MINE;
	
	@ClientString(id = 7853, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S18;
	
	@ClientString(id = 7854, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA8;
	
	@ClientString(id = 7855, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS9;
	
	@ClientString(id = 7856, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE8;
	
	@ClientString(id = 7857, message = "Tejakar Oroca! Tejakar Duda-Mara!")
	public static NpcStringId TEJAKAR_OROCA_TEJAKAR_DUDA_MARA;
	
	@ClientString(id = 7858, message = "$s1, we must fight this soul together to prevent an everlasting winter!")
	public static NpcStringId S1_WE_MUST_FIGHT_THIS_SOUL_TOGETHER_TO_PREVENT_AN_EVERLASTING_WINTER;
	
	@ClientString(id = 7859, message = "$s1! The soul responds to you. May your attack quiet him!")
	public static NpcStringId S1_THE_SOUL_RESPONDS_TO_YOU_MAY_YOUR_ATTACK_QUIET_HIM;
	
	@ClientString(id = 7860, message = "$s1! Calm Sharuhi! He doesn't listen to me anymore.")
	public static NpcStringId S1_CALM_SHARUHI_HE_DOESN_T_LISTEN_TO_ME_ANYMORE;
	
	@ClientString(id = 7861, message = "It's time to go! May the eternal flame bless you!")
	public static NpcStringId IT_S_TIME_TO_GO_MAY_THE_ETERNAL_FLAME_BLESS_YOU;
	
	@ClientString(id = 7862, message = "He left...That's too bad..Too bad...")
	public static NpcStringId HE_LEFT_THAT_S_TOO_BAD_TOO_BAD;
	
	@ClientString(id = 7863, message = "Don't forget your strong will now!")
	public static NpcStringId DON_T_FORGET_YOUR_STRONG_WILL_NOW;
	
	@ClientString(id = 7864, message = "Ha! Nobody will rule over me anymore!")
	public static NpcStringId HA_NOBODY_WILL_RULE_OVER_ME_ANYMORE;
	
	@ClientString(id = 7865, message = "Freedom... freedom... freedom!")
	public static NpcStringId FREEDOM_FREEDOM_FREEDOM;
	
	@ClientString(id = 7866, message = "$s1, You released me, but you also want to catch me. Ha!")
	public static NpcStringId S1_YOU_RELEASED_ME_BUT_YOU_ALSO_WANT_TO_CATCH_ME_HA;
	
	@ClientString(id = 7867, message = "...$s1...Me?....All right...I'll help you.")
	public static NpcStringId S1_ME_ALL_RIGHT_I_LL_HELP_YOU;
	
	@ClientString(id = 7901, message = "Saga of the Adventurer")
	public static NpcStringId SAGA_OF_THE_ADVENTURER;
	
	@ClientString(id = 7902, message = "Saga of the Adventurer (In Progress)")
	public static NpcStringId SAGA_OF_THE_ADVENTURER_IN_PROGRESS;
	
	@ClientString(id = 7903, message = "Saga of the Adventurer (Done)")
	public static NpcStringId SAGA_OF_THE_ADVENTURER_DONE;
	
	@ClientString(id = 7950, message = "Get out of here! This place is forbidden by god.")
	public static NpcStringId GET_OUT_OF_HERE_THIS_PLACE_IS_FORBIDDEN_BY_GOD;
	
	@ClientString(id = 7951, message = "Einhasad is calling me.")
	public static NpcStringId EINHASAD_IS_CALLING_ME;
	
	@ClientString(id = 7952, message = "You killed me! Aren't you afraid of god's curse?")
	public static NpcStringId YOU_KILLED_ME_AREN_T_YOU_AFRAID_OF_GOD_S_CURSE;
	
	@ClientString(id = 7953, message = "You bother my minions, $s1. Do you want to die?")
	public static NpcStringId YOU_BOTHER_MY_MINIONS_S1_DO_YOU_WANT_TO_DIE;
	
	@ClientString(id = 7954, message = "What the hell... I lost.")
	public static NpcStringId WHAT_THE_HELL_I_LOST;
	
	@ClientString(id = 7955, message = "Who are you? Why are you interfering in our business?")
	public static NpcStringId WHO_ARE_YOU_WHY_ARE_YOU_INTERFERING_IN_OUR_BUSINESS;
	
	@ClientString(id = 7956, message = "You're strong. I'll get you next time!")
	public static NpcStringId YOU_RE_STRONG_I_LL_GET_YOU_NEXT_TIME;
	
	@ClientString(id = 7957, message = "We meet again. I'll have you this time!")
	public static NpcStringId WE_MEET_AGAIN_I_LL_HAVE_YOU_THIS_TIME;
	
	@ClientString(id = 7958, message = "A worthy opponent. $s1. Help me!")
	public static NpcStringId A_WORTHY_OPPONENT_S1_HELP_ME;
	
	@ClientString(id = 7959, message = "$s1! Hurry before he gets away!")
	public static NpcStringId S1_HURRY_BEFORE_HE_GETS_AWAY;
	
	@ClientString(id = 7960, message = "I'll kill you!")
	public static NpcStringId I_LL_KILL_YOU;
	
	@ClientString(id = 7961, message = "Why don't you fight me someday?")
	public static NpcStringId WHY_DON_T_YOU_FIGHT_ME_SOMEDAY;
	
	@ClientString(id = 7962, message = "I missed again. Dammit!")
	public static NpcStringId I_MISSED_AGAIN_DAMMIT;
	
	@ClientString(id = 7963, message = "I'm sure we'll meet again someday.")
	public static NpcStringId I_M_SURE_WE_LL_MEET_AGAIN_SOMEDAY;
	
	@ClientString(id = 7964, message = "Curse those who defy the gods!")
	public static NpcStringId CURSE_THOSE_WHO_DEFY_THE_GODS;
	
	@ClientString(id = 7965, message = "Einhasad is calling me.")
	public static NpcStringId EINHASAD_IS_CALLING_ME2;
	
	@ClientString(id = 7966, message = "You would fight me, a messenger of the gods?")
	public static NpcStringId YOU_WOULD_FIGHT_ME_A_MESSENGER_OF_THE_GODS;
	
	@ClientString(id = 7967, message = "$s1! I won't forget you.")
	public static NpcStringId S1_I_WON_T_FORGET_YOU;
	
	@ClientString(id = 8001, message = "Saga of the Wind Rider")
	public static NpcStringId SAGA_OF_THE_WIND_RIDER;
	
	@ClientString(id = 8002, message = "Saga of the Wind Rider (In Progress)")
	public static NpcStringId SAGA_OF_THE_WIND_RIDER_IN_PROGRESS;
	
	@ClientString(id = 8003, message = "Saga of the Wind Rider (Done)")
	public static NpcStringId SAGA_OF_THE_WIND_RIDER_DONE;
	
	@ClientString(id = 8050, message = "$s1! How could you desecrate a holy place?")
	public static NpcStringId S1_HOW_COULD_YOU_DESECRATE_A_HOLY_PLACE;
	
	@ClientString(id = 8051, message = "Leave before you are severely punished!")
	public static NpcStringId LEAVE_BEFORE_YOU_ARE_SEVERELY_PUNISHED;
	
	@ClientString(id = 8052, message = "Einhasad, don't give up on me!")
	public static NpcStringId EINHASAD_DON_T_GIVE_UP_ON_ME;
	
	@ClientString(id = 8053, message = "$s1, so you're the one who's looking for me?")
	public static NpcStringId S1_SO_YOU_RE_THE_ONE_WHO_S_LOOKING_FOR_ME;
	
	@ClientString(id = 8054, message = "A mere mortal has defeated me!")
	public static NpcStringId A_MERE_MORTAL_HAS_DEFEATED_ME;
	
	@ClientString(id = 8055, message = "How cowardly to intrude in other people's business.")
	public static NpcStringId HOW_COWARDLY_TO_INTRUDE_IN_OTHER_PEOPLE_S_BUSINESS;
	
	@ClientString(id = 8056, message = "Time is up.")
	public static NpcStringId TIME_IS_UP;
	
	@ClientString(id = 8057, message = "I'll kill you with my sword!")
	public static NpcStringId I_LL_KILL_YOU_WITH_MY_SWORD;
	
	@ClientString(id = 8058, message = "Help me!")
	public static NpcStringId HELP_ME;
	
	@ClientString(id = 8059, message = "Don't miss!")
	public static NpcStringId DON_T_MISS;
	
	@ClientString(id = 8060, message = "Keep pushing!")
	public static NpcStringId KEEP_PUSHING;
	
	@ClientString(id = 8061, message = "I'll get him. You'll have your revenge.")
	public static NpcStringId I_LL_GET_HIM_YOU_LL_HAVE_YOUR_REVENGE;
	
	@ClientString(id = 8062, message = "I missed him again. I'll kill you.")
	public static NpcStringId I_MISSED_HIM_AGAIN_I_LL_KILL_YOU;
	
	@ClientString(id = 8063, message = "I must follow him.")
	public static NpcStringId I_MUST_FOLLOW_HIM;
	
	@ClientString(id = 8064, message = "Curse those who defy the gods!")
	public static NpcStringId CURSE_THOSE_WHO_DEFY_THE_GODS2;
	
	@ClientString(id = 8065, message = "Einhasad is calling me.")
	public static NpcStringId EINHASAD_IS_CALLING_ME3;
	
	@ClientString(id = 8066, message = "You would fight me, a messenger of the gods?")
	public static NpcStringId YOU_WOULD_FIGHT_ME_A_MESSENGER_OF_THE_GODS2;
	
	@ClientString(id = 8067, message = "$s1! I won't forget you.")
	public static NpcStringId S1_I_WON_T_FORGET_YOU2;
	
	@ClientString(id = 8101, message = "Saga of the Ghost Hunter")
	public static NpcStringId SAGA_OF_THE_GHOST_HUNTER;
	
	@ClientString(id = 8102, message = "Saga of the Ghost Hunter (In Progress)")
	public static NpcStringId SAGA_OF_THE_GHOST_HUNTER_IN_PROGRESS;
	
	@ClientString(id = 8103, message = "Saga of the Ghost Hunter (Done)")
	public static NpcStringId SAGA_OF_THE_GHOST_HUNTER_DONE;
	
	@ClientString(id = 8150, message = "$s1, you should leave if you fear god's wrath!")
	public static NpcStringId S1_YOU_SHOULD_LEAVE_IF_YOU_FEAR_GOD_S_WRATH;
	
	@ClientString(id = 8151, message = "What's going on?")
	public static NpcStringId WHAT_S_GOING_ON;
	
	@ClientString(id = 8152, message = "I'll see you again!")
	public static NpcStringId I_LL_SEE_YOU_AGAIN;
	
	@ClientString(id = 8153, message = "Who are you? Why are you bothering my minions?")
	public static NpcStringId WHO_ARE_YOU_WHY_ARE_YOU_BOTHERING_MY_MINIONS;
	
	@ClientString(id = 8154, message = "No way!!!")
	public static NpcStringId NO_WAY;
	
	@ClientString(id = 8155, message = "Why are you sticking your nose in our business?")
	public static NpcStringId WHY_ARE_YOU_STICKING_YOUR_NOSE_IN_OUR_BUSINESS;
	
	@ClientString(id = 8156, message = "Who are you? How can a creature from the netherworld be so powerful?")
	public static NpcStringId WHO_ARE_YOU_HOW_CAN_A_CREATURE_FROM_THE_NETHERWORLD_BE_SO_POWERFUL;
	
	@ClientString(id = 8157, message = "Is this the end?")
	public static NpcStringId IS_THIS_THE_END;
	
	@ClientString(id = 8158, message = "Show me what you're made of. Kill him!")
	public static NpcStringId SHOW_ME_WHAT_YOU_RE_MADE_OF_KILL_HIM;
	
	@ClientString(id = 8159, message = "You think you can get him with that?")
	public static NpcStringId YOU_THINK_YOU_CAN_GET_HIM_WITH_THAT;
	
	@ClientString(id = 8160, message = "Pull yourself together! He's trying to get away.")
	public static NpcStringId PULL_YOURSELF_TOGETHER_HE_S_TRYING_TO_GET_AWAY;
	
	@ClientString(id = 8161, message = "Tell the Black Cat that I got his paid back.")
	public static NpcStringId TELL_THE_BLACK_CAT_THAT_I_GOT_HIS_PAID_BACK;
	
	@ClientString(id = 8162, message = "Black Cat, he'll blame me.")
	public static NpcStringId BLACK_CAT_HE_LL_BLAME_ME;
	
	@ClientString(id = 8163, message = "I gotta' go now.")
	public static NpcStringId I_GOTTA_GO_NOW;
	
	@ClientString(id = 8164, message = "Curse those who defy the gods!")
	public static NpcStringId CURSE_THOSE_WHO_DEFY_THE_GODS3;
	
	@ClientString(id = 8165, message = "Einhasad is calling me.")
	public static NpcStringId EINHASAD_IS_CALLING_ME4;
	
	@ClientString(id = 8166, message = "I'll kill you in the name of god.")
	public static NpcStringId I_LL_KILL_YOU_IN_THE_NAME_OF_GOD;
	
	@ClientString(id = 8167, message = "$s1! See you later.")
	public static NpcStringId S1_SEE_YOU_LATER;
	
	@ClientString(id = 8201, message = "Saga of the Sagittarius")
	public static NpcStringId SAGA_OF_THE_SAGITTARIUS;
	
	@ClientString(id = 8202, message = "Saga of the Sagittarius (In Progress)")
	public static NpcStringId SAGA_OF_THE_SAGITTARIUS_IN_PROGRESS;
	
	@ClientString(id = 8203, message = "Saga of the Sagittarius (Done)")
	public static NpcStringId SAGA_OF_THE_SAGITTARIUS_DONE;
	
	@ClientString(id = 8250, message = "$s1! How could you desecrate a holy place?")
	public static NpcStringId S1_HOW_COULD_YOU_DESECRATE_A_HOLY_PLACE2;
	
	@ClientString(id = 8251, message = "Get out before you're punished!")
	public static NpcStringId GET_OUT_BEFORE_YOU_RE_PUNISHED;
	
	@ClientString(id = 8252, message = "Einhasad, please don't give up on me!")
	public static NpcStringId EINHASAD_PLEASE_DON_T_GIVE_UP_ON_ME;
	
	@ClientString(id = 8253, message = "$s1, are you looking for me?")
	public static NpcStringId S1_ARE_YOU_LOOKING_FOR_ME;
	
	@ClientString(id = 8254, message = "A mere mortal is killing me!")
	public static NpcStringId A_MERE_MORTAL_IS_KILLING_ME;
	
	@ClientString(id = 8255, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS10;
	
	@ClientString(id = 8256, message = "Mortal, don't you recognize my greatness?")
	public static NpcStringId MORTAL_DON_T_YOU_RECOGNIZE_MY_GREATNESS;
	
	@ClientString(id = 8257, message = "I'll get you this time.")
	public static NpcStringId I_LL_GET_YOU_THIS_TIME;
	
	@ClientString(id = 8258, message = "I'll never forget the taste of his steel, $s1! Let's fight him together!")
	public static NpcStringId I_LL_NEVER_FORGET_THE_TASTE_OF_HIS_STEEL_S1_LET_S_FIGHT_HIM_TOGETHER;
	
	@ClientString(id = 8259, message = "$s1! Pull yourself together. We'll miss him!")
	public static NpcStringId S1_PULL_YOURSELF_TOGETHER_WE_LL_MISS_HIM;
	
	@ClientString(id = 8260, message = "$s1! He's trying to get away.")
	public static NpcStringId S1_HE_S_TRYING_TO_GET_AWAY;
	
	@ClientString(id = 8261, message = "I missed again! Next time...")
	public static NpcStringId I_MISSED_AGAIN_NEXT_TIME;
	
	@ClientString(id = 8262, message = "Dammit! Failed again!")
	public static NpcStringId DAMMIT_FAILED_AGAIN;
	
	@ClientString(id = 8263, message = "I gotta' go now.")
	public static NpcStringId I_GOTTA_GO_NOW2;
	
	@ClientString(id = 8264, message = "Curse those who defy the gods!")
	public static NpcStringId CURSE_THOSE_WHO_DEFY_THE_GODS4;
	
	@ClientString(id = 8265, message = "Einhasad is calling me.")
	public static NpcStringId EINHASAD_IS_CALLING_ME5;
	
	@ClientString(id = 8266, message = "You would fight me, a messenger of the gods?")
	public static NpcStringId YOU_WOULD_FIGHT_ME_A_MESSENGER_OF_THE_GODS3;
	
	@ClientString(id = 8267, message = "$s1! I won't forget you.")
	public static NpcStringId S1_I_WON_T_FORGET_YOU3;
	
	@ClientString(id = 8301, message = "Saga of the Moonlight Sentinel")
	public static NpcStringId SAGA_OF_THE_MOONLIGHT_SENTINEL;
	
	@ClientString(id = 8302, message = "Saga of the Moonlight Sentinel (In Progress)")
	public static NpcStringId SAGA_OF_THE_MOONLIGHT_SENTINEL_IN_PROGRESS;
	
	@ClientString(id = 8303, message = "Saga of the Moonlight Sentinel (Done)")
	public static NpcStringId SAGA_OF_THE_MOONLIGHT_SENTINEL_DONE;
	
	@ClientString(id = 8350, message = "$s1! How could you desecrate a holy place?")
	public static NpcStringId S1_HOW_COULD_YOU_DESECRATE_A_HOLY_PLACE3;
	
	@ClientString(id = 8351, message = "Get out before you're punished!")
	public static NpcStringId GET_OUT_BEFORE_YOU_RE_PUNISHED2;
	
	@ClientString(id = 8352, message = "Einhasad, don't give up on me!")
	public static NpcStringId EINHASAD_DON_T_GIVE_UP_ON_ME2;
	
	@ClientString(id = 8353, message = "You are the one who's looking for me, $s1?")
	public static NpcStringId YOU_ARE_THE_ONE_WHO_S_LOOKING_FOR_ME_S1;
	
	@ClientString(id = 8354, message = "A mere mortal has killed me!")
	public static NpcStringId A_MERE_MORTAL_HAS_KILLED_ME;
	
	@ClientString(id = 8355, message = "Who are you? This is none of your business!")
	public static NpcStringId WHO_ARE_YOU_THIS_IS_NONE_OF_YOUR_BUSINESS;
	
	@ClientString(id = 8356, message = "Mortal, don't you recognize my greatness?")
	public static NpcStringId MORTAL_DON_T_YOU_RECOGNIZE_MY_GREATNESS2;
	
	@ClientString(id = 8357, message = "I'll get you this time.")
	public static NpcStringId I_LL_GET_YOU_THIS_TIME2;
	
	@ClientString(id = 8358, message = "I'll never forget the taste of his steel, $s1! Let's fight him together!")
	public static NpcStringId I_LL_NEVER_FORGET_THE_TASTE_OF_HIS_STEEL_S1_LET_S_FIGHT_HIM_TOGETHER2;
	
	@ClientString(id = 8359, message = "$s1! Pull yourself together.")
	public static NpcStringId S1_PULL_YOURSELF_TOGETHER;
	
	@ClientString(id = 8360, message = "$s1! He'll get away!")
	public static NpcStringId S1_HE_LL_GET_AWAY;
	
	@ClientString(id = 8361, message = "I missed again! Next time...")
	public static NpcStringId I_MISSED_AGAIN_NEXT_TIME2;
	
	@ClientString(id = 8362, message = "Dammit! Failed again!")
	public static NpcStringId DAMMIT_FAILED_AGAIN2;
	
	@ClientString(id = 8363, message = "I gotta' go now.")
	public static NpcStringId I_GOTTA_GO_NOW3;
	
	@ClientString(id = 8364, message = "Curse those who defy the gods!")
	public static NpcStringId CURSE_THOSE_WHO_DEFY_THE_GODS5;
	
	@ClientString(id = 8365, message = "Einhasad is calling me.")
	public static NpcStringId EINHASAD_IS_CALLING_ME6;
	
	@ClientString(id = 8366, message = "You would fight me, a messenger of the gods?")
	public static NpcStringId YOU_WOULD_FIGHT_ME_A_MESSENGER_OF_THE_GODS4;
	
	@ClientString(id = 8367, message = "$s1! I won't forget you.")
	public static NpcStringId S1_I_WON_T_FORGET_YOU4;
	
	@ClientString(id = 8401, message = "Saga of the Ghost Sentinel")
	public static NpcStringId SAGA_OF_THE_GHOST_SENTINEL;
	
	@ClientString(id = 8402, message = "Saga of the Ghost Sentinel (In Progress)")
	public static NpcStringId SAGA_OF_THE_GHOST_SENTINEL_IN_PROGRESS;
	
	@ClientString(id = 8403, message = "Saga of the Ghost Sentinel (Done)")
	public static NpcStringId SAGA_OF_THE_GHOST_SENTINEL_DONE;
	
	@ClientString(id = 8450, message = "$s1! How could you desecrate a holy place?")
	public static NpcStringId S1_HOW_COULD_YOU_DESECRATE_A_HOLY_PLACE4;
	
	@ClientString(id = 8451, message = "Get out before you're punished!")
	public static NpcStringId GET_OUT_BEFORE_YOU_RE_PUNISHED3;
	
	@ClientString(id = 8452, message = "Einhasad, please don't forsake me!")
	public static NpcStringId EINHASAD_PLEASE_DON_T_FORSAKE_ME;
	
	@ClientString(id = 8453, message = "Looking for me, $s1?")
	public static NpcStringId LOOKING_FOR_ME_S1;
	
	@ClientString(id = 8454, message = "A mere mortal is killing me!")
	public static NpcStringId A_MERE_MORTAL_IS_KILLING_ME2;
	
	@ClientString(id = 8455, message = "Who are you? This is none of your business!")
	public static NpcStringId WHO_ARE_YOU_THIS_IS_NONE_OF_YOUR_BUSINESS2;
	
	@ClientString(id = 8456, message = "Mortal! Don't you recognize my greatness?")
	public static NpcStringId MORTAL_DON_T_YOU_RECOGNIZE_MY_GREATNESS3;
	
	@ClientString(id = 8457, message = "I'll get you this time.")
	public static NpcStringId I_LL_GET_YOU_THIS_TIME3;
	
	@ClientString(id = 8458, message = "I'll never forget the taste of his steel, $s1! Let's fight him together!")
	public static NpcStringId I_LL_NEVER_FORGET_THE_TASTE_OF_HIS_STEEL_S1_LET_S_FIGHT_HIM_TOGETHER3;
	
	@ClientString(id = 8459, message = "$s1! Pull yourself together!")
	public static NpcStringId S1_PULL_YOURSELF_TOGETHER2;
	
	@ClientString(id = 8460, message = "$s1! He'll get away!")
	public static NpcStringId S1_HE_LL_GET_AWAY2;
	
	@ClientString(id = 8461, message = "I missed again! Next time...")
	public static NpcStringId I_MISSED_AGAIN_NEXT_TIME3;
	
	@ClientString(id = 8462, message = "Dammit! Failed again!")
	public static NpcStringId DAMMIT_FAILED_AGAIN3;
	
	@ClientString(id = 8463, message = "I gotta' go now.")
	public static NpcStringId I_GOTTA_GO_NOW4;
	
	@ClientString(id = 8464, message = "Curse those who defy the gods!")
	public static NpcStringId CURSE_THOSE_WHO_DEFY_THE_GODS6;
	
	@ClientString(id = 8465, message = "Einhasad is calling me.")
	public static NpcStringId EINHASAD_IS_CALLING_ME7;
	
	@ClientString(id = 8466, message = "You would fight me, a messenger of the gods?")
	public static NpcStringId YOU_WOULD_FIGHT_ME_A_MESSENGER_OF_THE_GODS5;
	
	@ClientString(id = 8467, message = "$s1! I won't forget you.")
	public static NpcStringId S1_I_WON_T_FORGET_YOU5;
	
	@ClientString(id = 8501, message = "Saga of the Cardinal")
	public static NpcStringId SAGA_OF_THE_CARDINAL;
	
	@ClientString(id = 8502, message = "Saga of the Cardinal (In Progress)")
	public static NpcStringId SAGA_OF_THE_CARDINAL_IN_PROGRESS;
	
	@ClientString(id = 8503, message = "Saga of the Cardinal (Done)")
	public static NpcStringId SAGA_OF_THE_CARDINAL_DONE;
	
	@ClientString(id = 8550, message = "$s1! Bishop, how foolish to go against the will of god!")
	public static NpcStringId S1_BISHOP_HOW_FOOLISH_TO_GO_AGAINST_THE_WILL_OF_GOD;
	
	@ClientString(id = 8551, message = "Your faith is stronger than I thought. I'll pay you back next time.")
	public static NpcStringId YOUR_FAITH_IS_STRONGER_THAN_I_THOUGHT_I_LL_PAY_YOU_BACK_NEXT_TIME;
	
	@ClientString(id = 8552, message = "Tanakia! Forgive me. I couldn't fulfill your dream!")
	public static NpcStringId TANAKIA_FORGIVE_ME_I_COULDN_T_FULFILL_YOUR_DREAM;
	
	@ClientString(id = 8553, message = "$s1, you are the won who's been bothering my minions?")
	public static NpcStringId S1_YOU_ARE_THE_WON_WHO_S_BEEN_BOTHERING_MY_MINIONS;
	
	@ClientString(id = 8554, message = "Damn! You've beaten me.")
	public static NpcStringId DAMN_YOU_VE_BEATEN_ME;
	
	@ClientString(id = 8555, message = "Who are you? This isn't your business, coward.")
	public static NpcStringId WHO_ARE_YOU_THIS_ISN_T_YOUR_BUSINESS_COWARD;
	
	@ClientString(id = 8556, message = "How weak. I'll forgive you this time because you made me laugh.")
	public static NpcStringId HOW_WEAK_I_LL_FORGIVE_YOU_THIS_TIME_BECAUSE_YOU_MADE_ME_LAUGH;
	
	@ClientString(id = 8557, message = "You are stronger than I thought, but I'm no weakling!")
	public static NpcStringId YOU_ARE_STRONGER_THAN_I_THOUGHT_BUT_I_M_NO_WEAKLING;
	
	@ClientString(id = 8558, message = "He's got a tough shell. $s1! Let's fight together and crack his skull!")
	public static NpcStringId HE_S_GOT_A_TOUGH_SHELL_S1_LET_S_FIGHT_TOGETHER_AND_CRACK_HIS_SKULL;
	
	@ClientString(id = 8559, message = "$s1! Pull yourself together.")
	public static NpcStringId S1_PULL_YOURSELF_TOGETHER3;
	
	@ClientString(id = 8560, message = "$s1! We won't beat him unless we give it our all. Come on!")
	public static NpcStringId S1_WE_WON_T_BEAT_HIM_UNLESS_WE_GIVE_IT_OUR_ALL_COME_ON;
	
	@ClientString(id = 8561, message = "I'll follow him.")
	public static NpcStringId I_LL_FOLLOW_HIM;
	
	@ClientString(id = 8562, message = "I missed again! He's hard to follow.")
	public static NpcStringId I_MISSED_AGAIN_HE_S_HARD_TO_FOLLOW;
	
	@ClientString(id = 8563, message = "We'll see what the future brings.")
	public static NpcStringId WE_LL_SEE_WHAT_THE_FUTURE_BRINGS;
	
	@ClientString(id = 8564, message = "For Shilen!")
	public static NpcStringId FOR_SHILEN;
	
	@ClientString(id = 8565, message = "I'll be back. I'll deal with you then.")
	public static NpcStringId I_LL_BE_BACK_I_LL_DEAL_WITH_YOU_THEN;
	
	@ClientString(id = 8566, message = "$s1! Are you going to fight me?")
	public static NpcStringId S1_ARE_YOU_GOING_TO_FIGHT_ME;
	
	@ClientString(id = 8567, message = "$s1! I'll pay you back. I won't forget you.")
	public static NpcStringId S1_I_LL_PAY_YOU_BACK_I_WON_T_FORGET_YOU;
	
	@ClientString(id = 8601, message = "Saga of the Hierophant")
	public static NpcStringId SAGA_OF_THE_HIEROPHANT;
	
	@ClientString(id = 8602, message = "Saga of the Hierophant (In Progress)")
	public static NpcStringId SAGA_OF_THE_HIEROPHANT_IN_PROGRESS;
	
	@ClientString(id = 8603, message = "Saga of the Hierophant (Done)")
	public static NpcStringId SAGA_OF_THE_HIEROPHANT_DONE;
	
	@ClientString(id = 8650, message = "$s1! Prophet, how foolish to go against the will of god!")
	public static NpcStringId S1_PROPHET_HOW_FOOLISH_TO_GO_AGAINST_THE_WILL_OF_GOD;
	
	@ClientString(id = 8651, message = "Your faith is stronger than I thought. I'll deal with you next time.")
	public static NpcStringId YOUR_FAITH_IS_STRONGER_THAN_I_THOUGHT_I_LL_DEAL_WITH_YOU_NEXT_TIME;
	
	@ClientString(id = 8652, message = "Tanakia! Forgive me. I couldn't fulfill your dream!")
	public static NpcStringId TANAKIA_FORGIVE_ME_I_COULDN_T_FULFILL_YOUR_DREAM2;
	
	@ClientString(id = 8653, message = "Are you the one who's been bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_S_BEEN_BOTHERING_MY_MINIONS_S1;
	
	@ClientString(id = 8654, message = "Damn! I can't believe I've been beaten by you!")
	public static NpcStringId DAMN_I_CAN_T_BELIEVE_I_VE_BEEN_BEATEN_BY_YOU;
	
	@ClientString(id = 8655, message = "Who are you? This is none of your business, coward.")
	public static NpcStringId WHO_ARE_YOU_THIS_IS_NONE_OF_YOUR_BUSINESS_COWARD;
	
	@ClientString(id = 8656, message = "How weak. I'll forgive you this time because you made me laugh.")
	public static NpcStringId HOW_WEAK_I_LL_FORGIVE_YOU_THIS_TIME_BECAUSE_YOU_MADE_ME_LAUGH2;
	
	@ClientString(id = 8657, message = "I'll destroy the darkness surrounding the world with the power of light!")
	public static NpcStringId I_LL_DESTROY_THE_DARKNESS_SURROUNDING_THE_WORLD_WITH_THE_POWER_OF_LIGHT;
	
	@ClientString(id = 8658, message = "$s1! Fight the Fallen Angel with me. Show the true power of light!")
	public static NpcStringId S1_FIGHT_THE_FALLEN_ANGEL_WITH_ME_SHOW_THE_TRUE_POWER_OF_LIGHT;
	
	@ClientString(id = 8659, message = "$s1! Go! We must stop fighting here.")
	public static NpcStringId S1_GO_WE_MUST_STOP_FIGHTING_HERE;
	
	@ClientString(id = 8660, message = "We mustn't lose, $s1! Pull yourself together!")
	public static NpcStringId WE_MUSTN_T_LOSE_S1_PULL_YOURSELF_TOGETHER;
	
	@ClientString(id = 8661, message = "We'll meet again if fate wills it.")
	public static NpcStringId WE_LL_MEET_AGAIN_IF_FATE_WILLS_IT;
	
	@ClientString(id = 8662, message = "I'll follow the cowardly devil.")
	public static NpcStringId I_LL_FOLLOW_THE_COWARDLY_DEVIL;
	
	@ClientString(id = 8663, message = "We'll meet again if fate wills it.")
	public static NpcStringId WE_LL_MEET_AGAIN_IF_FATE_WILLS_IT2;
	
	@ClientString(id = 8664, message = "For Shilen!")
	public static NpcStringId FOR_SHILEN2;
	
	@ClientString(id = 8665, message = "I'll be back. I'll deal with you then.")
	public static NpcStringId I_LL_BE_BACK_I_LL_DEAL_WITH_YOU_THEN2;
	
	@ClientString(id = 8666, message = "$s1! Are you going to fight me?")
	public static NpcStringId S1_ARE_YOU_GOING_TO_FIGHT_ME2;
	
	@ClientString(id = 8667, message = "$s1! I'll pay you back. I won't forget you.")
	public static NpcStringId S1_I_LL_PAY_YOU_BACK_I_WON_T_FORGET_YOU2;
	
	@ClientString(id = 8701, message = "Saga of Eva's Saint")
	public static NpcStringId SAGA_OF_EVA_S_SAINT;
	
	@ClientString(id = 8702, message = "Saga of Eva's Saint (In Progress)")
	public static NpcStringId SAGA_OF_EVA_S_SAINT_IN_PROGRESS;
	
	@ClientString(id = 8703, message = "Saga of Eva's Saint (Done)")
	public static NpcStringId SAGA_OF_EVA_S_SAINT_DONE;
	
	@ClientString(id = 8750, message = "$s1! Elder, it's foolish of you to go against the will of the gods.")
	public static NpcStringId S1_ELDER_IT_S_FOOLISH_OF_YOU_TO_GO_AGAINST_THE_WILL_OF_THE_GODS;
	
	@ClientString(id = 8751, message = "Your faith is stronger than I thought. I'll pay you back next time.")
	public static NpcStringId YOUR_FAITH_IS_STRONGER_THAN_I_THOUGHT_I_LL_PAY_YOU_BACK_NEXT_TIME2;
	
	@ClientString(id = 8752, message = "Tanakia! Forgive me. I couldn't fulfill your dream!")
	public static NpcStringId TANAKIA_FORGIVE_ME_I_COULDN_T_FULFILL_YOUR_DREAM3;
	
	@ClientString(id = 8753, message = "Are you the one who's been bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_S_BEEN_BOTHERING_MY_MINIONS_S12;
	
	@ClientString(id = 8754, message = "Damn! I can't believe I've been beaten by you.")
	public static NpcStringId DAMN_I_CAN_T_BELIEVE_I_VE_BEEN_BEATEN_BY_YOU2;
	
	@ClientString(id = 8755, message = "Who are you? This is none of your business, coward.")
	public static NpcStringId WHO_ARE_YOU_THIS_IS_NONE_OF_YOUR_BUSINESS_COWARD2;
	
	@ClientString(id = 8756, message = "How weak. I'll forgive you this time because you made me laugh.")
	public static NpcStringId HOW_WEAK_I_LL_FORGIVE_YOU_THIS_TIME_BECAUSE_YOU_MADE_ME_LAUGH3;
	
	@ClientString(id = 8757, message = "You're stronger than I thought, but I'm no weakling either!")
	public static NpcStringId YOU_RE_STRONGER_THAN_I_THOUGHT_BUT_I_M_NO_WEAKLING_EITHER;
	
	@ClientString(id = 8758, message = "He's got a tough shell. $s1! Let's fight together and crack his skull!")
	public static NpcStringId HE_S_GOT_A_TOUGH_SHELL_S1_LET_S_FIGHT_TOGETHER_AND_CRACK_HIS_SKULL2;
	
	@ClientString(id = 8759, message = "$s1! Pull yourself together.")
	public static NpcStringId S1_PULL_YOURSELF_TOGETHER4;
	
	@ClientString(id = 8760, message = "$s1! We'll never win unless we give it our all. Come on!")
	public static NpcStringId S1_WE_LL_NEVER_WIN_UNLESS_WE_GIVE_IT_OUR_ALL_COME_ON;
	
	@ClientString(id = 8761, message = "I'll follow him.")
	public static NpcStringId I_LL_FOLLOW_HIM2;
	
	@ClientString(id = 8762, message = "I missed again! He's hard to follow.")
	public static NpcStringId I_MISSED_AGAIN_HE_S_HARD_TO_FOLLOW2;
	
	@ClientString(id = 8763, message = "We'll see what the future brings.")
	public static NpcStringId WE_LL_SEE_WHAT_THE_FUTURE_BRINGS2;
	
	@ClientString(id = 8764, message = "For Shilen!")
	public static NpcStringId FOR_SHILEN3;
	
	@ClientString(id = 8765, message = "I'll be back. I'll deal with you then.")
	public static NpcStringId I_LL_BE_BACK_I_LL_DEAL_WITH_YOU_THEN3;
	
	@ClientString(id = 8766, message = "$s1! Are you going to fight me?")
	public static NpcStringId S1_ARE_YOU_GOING_TO_FIGHT_ME3;
	
	@ClientString(id = 8767, message = "$s1! I'll pay you back. I won't forget you.")
	public static NpcStringId S1_I_LL_PAY_YOU_BACK_I_WON_T_FORGET_YOU3;
	
	@ClientString(id = 8801, message = "Saga of the Archmage")
	public static NpcStringId SAGA_OF_THE_ARCHMAGE;
	
	@ClientString(id = 8802, message = "Saga of the Archmage (In Progress)")
	public static NpcStringId SAGA_OF_THE_ARCHMAGE_IN_PROGRESS;
	
	@ClientString(id = 8803, message = "Saga of the Archmage (Done)")
	public static NpcStringId SAGA_OF_THE_ARCHMAGE_DONE;
	
	@ClientString(id = 8850, message = "Are you $s1? Oh! I have the Resonance Amulet!")
	public static NpcStringId ARE_YOU_S1_OH_I_HAVE_THE_RESONANCE_AMULET;
	
	@ClientString(id = 8851, message = "You're feistier than I thought! I'll quit here for today.")
	public static NpcStringId YOU_RE_FEISTIER_THAN_I_THOUGHT_I_LL_QUIT_HERE_FOR_TODAY;
	
	@ClientString(id = 8852, message = "Aaargh! I can't believe I lost...")
	public static NpcStringId AAARGH_I_CAN_T_BELIEVE_I_LOST;
	
	@ClientString(id = 8853, message = "Are you the one who's been bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_S_BEEN_BOTHERING_MY_MINIONS_S13;
	
	@ClientString(id = 8854, message = "Yikes! You're tough!")
	public static NpcStringId YIKES_YOU_RE_TOUGH;
	
	@ClientString(id = 8855, message = "Why do you interfere in other people's business...")
	public static NpcStringId WHY_DO_YOU_INTERFERE_IN_OTHER_PEOPLE_S_BUSINESS;
	
	@ClientString(id = 8856, message = "I'll stop here for today.")
	public static NpcStringId I_LL_STOP_HERE_FOR_TODAY;
	
	@ClientString(id = 8857, message = "I won't miss you this time!")
	public static NpcStringId I_WON_T_MISS_YOU_THIS_TIME;
	
	@ClientString(id = 8858, message = "Dammit! This is too hard by myself... $s1! Give me a hand!")
	public static NpcStringId DAMMIT_THIS_IS_TOO_HARD_BY_MYSELF_S1_GIVE_ME_A_HAND;
	
	@ClientString(id = 8859, message = "$s1! Hurry up, we'll miss him.")
	public static NpcStringId S1_HURRY_UP_WE_LL_MISS_HIM;
	
	@ClientString(id = 8860, message = "$s1! Come on. Hurry up!")
	public static NpcStringId S1_COME_ON_HURRY_UP;
	
	@ClientString(id = 8861, message = "I gotta' go follow him.")
	public static NpcStringId I_GOTTA_GO_FOLLOW_HIM;
	
	@ClientString(id = 8862, message = "Hey, quit running! Stop!")
	public static NpcStringId HEY_QUIT_RUNNING_STOP;
	
	@ClientString(id = 8863, message = "See you next time~")
	public static NpcStringId SEE_YOU_NEXT_TIME;
	
	@ClientString(id = 8864, message = "What? Think you can get in my way?")
	public static NpcStringId WHAT_THINK_YOU_CAN_GET_IN_MY_WAY;
	
	@ClientString(id = 8865, message = "You are so weak. I gotta' go now!")
	public static NpcStringId YOU_ARE_SO_WEAK_I_GOTTA_GO_NOW;
	
	@ClientString(id = 8866, message = "$s1! Good. I'll help you.")
	public static NpcStringId S1_GOOD_I_LL_HELP_YOU;
	
	@ClientString(id = 8867, message = "$s1, you're stronger than I thought. See you next time.")
	public static NpcStringId S1_YOU_RE_STRONGER_THAN_I_THOUGHT_SEE_YOU_NEXT_TIME;
	
	@ClientString(id = 8901, message = "Saga of the Mystic Muse")
	public static NpcStringId SAGA_OF_THE_MYSTIC_MUSE;
	
	@ClientString(id = 8902, message = "Saga of the Mystic Muse (In Progress)")
	public static NpcStringId SAGA_OF_THE_MYSTIC_MUSE_IN_PROGRESS;
	
	@ClientString(id = 8903, message = "Saga of the Mystic Muse (Done)")
	public static NpcStringId SAGA_OF_THE_MYSTIC_MUSE_DONE;
	
	@ClientString(id = 8950, message = "Are you $s1? Oh! I have the Resonance Amulet!")
	public static NpcStringId ARE_YOU_S1_OH_I_HAVE_THE_RESONANCE_AMULET2;
	
	@ClientString(id = 8951, message = "You're feistier than I thought! I'll stop here today.")
	public static NpcStringId YOU_RE_FEISTIER_THAN_I_THOUGHT_I_LL_STOP_HERE_TODAY;
	
	@ClientString(id = 8952, message = "Aargh! I can't believe I lost...")
	public static NpcStringId AARGH_I_CAN_T_BELIEVE_I_LOST;
	
	@ClientString(id = 8953, message = "Are you the one who's been bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_S_BEEN_BOTHERING_MY_MINIONS_S14;
	
	@ClientString(id = 8954, message = "Yikes! You're tough!")
	public static NpcStringId YIKES_YOU_RE_TOUGH2;
	
	@ClientString(id = 8955, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS11;
	
	@ClientString(id = 8956, message = "I'll stop here today.")
	public static NpcStringId I_LL_STOP_HERE_TODAY;
	
	@ClientString(id = 8957, message = "I'll get you this time!")
	public static NpcStringId I_LL_GET_YOU_THIS_TIME4;
	
	@ClientString(id = 8958, message = "Damn! It's too much by myself...$s1! Give me a hand!")
	public static NpcStringId DAMN_IT_S_TOO_MUCH_BY_MYSELF_S1_GIVE_ME_A_HAND;
	
	@ClientString(id = 8959, message = "$s1! Hurry, we'll miss him.")
	public static NpcStringId S1_HURRY_WE_LL_MISS_HIM;
	
	@ClientString(id = 8960, message = "$s1! Hurry, please!")
	public static NpcStringId S1_HURRY_PLEASE;
	
	@ClientString(id = 8961, message = "I gotta' go follow him now.")
	public static NpcStringId I_GOTTA_GO_FOLLOW_HIM_NOW;
	
	@ClientString(id = 8962, message = "Are you running away? Stop!")
	public static NpcStringId ARE_YOU_RUNNING_AWAY_STOP;
	
	@ClientString(id = 8963, message = "See you next time~")
	public static NpcStringId SEE_YOU_NEXT_TIME2;
	
	@ClientString(id = 8964, message = "Do you think you can stop me?")
	public static NpcStringId DO_YOU_THINK_YOU_CAN_STOP_ME;
	
	@ClientString(id = 8965, message = "You're so weak. I gotta' go now...")
	public static NpcStringId YOU_RE_SO_WEAK_I_GOTTA_GO_NOW;
	
	@ClientString(id = 8966, message = "You're $s1! Good. I'll help you.")
	public static NpcStringId YOU_RE_S1_GOOD_I_LL_HELP_YOU;
	
	@ClientString(id = 8967, message = "$s1! You're stronger than I thought. See you next time.")
	public static NpcStringId S1_YOU_RE_STRONGER_THAN_I_THOUGHT_SEE_YOU_NEXT_TIME2;
	
	@ClientString(id = 9001, message = "Saga of the Storm Screamer")
	public static NpcStringId SAGA_OF_THE_STORM_SCREAMER;
	
	@ClientString(id = 9002, message = "Saga of the Storm Screamer (In Progress)")
	public static NpcStringId SAGA_OF_THE_STORM_SCREAMER_IN_PROGRESS;
	
	@ClientString(id = 9003, message = "Saga of the Storm Screamer (Done)")
	public static NpcStringId SAGA_OF_THE_STORM_SCREAMER_DONE;
	
	@ClientString(id = 9050, message = "Are you $s1? Oh! I have a Resonance Amulet!")
	public static NpcStringId ARE_YOU_S1_OH_I_HAVE_A_RESONANCE_AMULET;
	
	@ClientString(id = 9051, message = "Hey, you're more tenacious than I thought! I'll stop here today.")
	public static NpcStringId HEY_YOU_RE_MORE_TENACIOUS_THAN_I_THOUGHT_I_LL_STOP_HERE_TODAY;
	
	@ClientString(id = 9052, message = "Aargh! I can't believe I lost...")
	public static NpcStringId AARGH_I_CAN_T_BELIEVE_I_LOST2;
	
	@ClientString(id = 9053, message = "Are you the one who's been bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_S_BEEN_BOTHERING_MY_MINIONS_S15;
	
	@ClientString(id = 9054, message = "Yikes! You're tough!")
	public static NpcStringId YIKES_YOU_RE_TOUGH3;
	
	@ClientString(id = 9055, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS12;
	
	@ClientString(id = 9056, message = "I'll stop here today.")
	public static NpcStringId I_LL_STOP_HERE_TODAY2;
	
	@ClientString(id = 9057, message = "I won't miss you this time!")
	public static NpcStringId I_WON_T_MISS_YOU_THIS_TIME2;
	
	@ClientString(id = 9058, message = "Dammit! I can't do this alone, $s1! Give me a hand!")
	public static NpcStringId DAMMIT_I_CAN_T_DO_THIS_ALONE_S1_GIVE_ME_A_HAND;
	
	@ClientString(id = 9059, message = "$s1! Hurry or we'll miss him.")
	public static NpcStringId S1_HURRY_OR_WE_LL_MISS_HIM;
	
	@ClientString(id = 9060, message = "$s1! Hurry up!")
	public static NpcStringId S1_HURRY_UP;
	
	@ClientString(id = 9061, message = "I gotta' follow him now.")
	public static NpcStringId I_GOTTA_FOLLOW_HIM_NOW;
	
	@ClientString(id = 9062, message = "Hey, are you running? Stop!")
	public static NpcStringId HEY_ARE_YOU_RUNNING_STOP;
	
	@ClientString(id = 9063, message = "See you next time~")
	public static NpcStringId SEE_YOU_NEXT_TIME3;
	
	@ClientString(id = 9064, message = "Do you think you can stop me?")
	public static NpcStringId DO_YOU_THINK_YOU_CAN_STOP_ME2;
	
	@ClientString(id = 9065, message = "You're so weak. I gotta' go now...")
	public static NpcStringId YOU_RE_SO_WEAK_I_GOTTA_GO_NOW2;
	
	@ClientString(id = 9066, message = "Oh! You're $s1! Good. I'll help you.")
	public static NpcStringId OH_YOU_RE_S1_GOOD_I_LL_HELP_YOU;
	
	@ClientString(id = 9067, message = "$s1. You're stronger than I thought. See you next time.")
	public static NpcStringId S1_YOU_RE_STRONGER_THAN_I_THOUGHT_SEE_YOU_NEXT_TIME3;
	
	@ClientString(id = 9101, message = "Saga of the Arcana Lord")
	public static NpcStringId SAGA_OF_THE_ARCANA_LORD;
	
	@ClientString(id = 9102, message = "Saga of the Arcana Lord (In Progress)")
	public static NpcStringId SAGA_OF_THE_ARCANA_LORD_IN_PROGRESS;
	
	@ClientString(id = 9103, message = "Saga of the Arcana Lord (Done)")
	public static NpcStringId SAGA_OF_THE_ARCANA_LORD_DONE;
	
	@ClientString(id = 9150, message = "You carouse with evil spirits, $s1! You're not worthy of the holy wisdom!")
	public static NpcStringId YOU_CAROUSE_WITH_EVIL_SPIRITS_S1_YOU_RE_NOT_WORTHY_OF_THE_HOLY_WISDOM;
	
	@ClientString(id = 9151, message = "You're so stubborn! I can't boss you around any more, can I?")
	public static NpcStringId YOU_RE_SO_STUBBORN_I_CAN_T_BOSS_YOU_AROUND_ANY_MORE_CAN_I;
	
	@ClientString(id = 9152, message = "How could it happen? Defeated by a Human!")
	public static NpcStringId HOW_COULD_IT_HAPPEN_DEFEATED_BY_A_HUMAN;
	
	@ClientString(id = 9153, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S19;
	
	@ClientString(id = 9154, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA9;
	
	@ClientString(id = 9155, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS13;
	
	@ClientString(id = 9156, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE9;
	
	@ClientString(id = 9157, message = "My master sent me here! I'll give you a hand.")
	public static NpcStringId MY_MASTER_SENT_ME_HERE_I_LL_GIVE_YOU_A_HAND;
	
	@ClientString(id = 9158, message = "Meow~! Master $s1, help me!")
	public static NpcStringId MEOW_MASTER_S1_HELP_ME;
	
	@ClientString(id = 9159, message = "Master $s1. Punish him so he can't bother Belinda!")
	public static NpcStringId MASTER_S1_PUNISH_HIM_SO_HE_CAN_T_BOTHER_BELINDA;
	
	@ClientString(id = 9160, message = "Master $s1, We'll miss him!")
	public static NpcStringId MASTER_S1_WE_LL_MISS_HIM;
	
	@ClientString(id = 9161, message = "Meow~! My master is calling. Meow! I gotta' go now~!")
	public static NpcStringId MEOW_MY_MASTER_IS_CALLING_MEOW_I_GOTTA_GO_NOW;
	
	@ClientString(id = 9162, message = "Meow~! I missed him. Meow!")
	public static NpcStringId MEOW_I_MISSED_HIM_MEOW;
	
	@ClientString(id = 9163, message = "Good luck! Meow~! I gotta' go now.")
	public static NpcStringId GOOD_LUCK_MEOW_I_GOTTA_GO_NOW;
	
	@ClientString(id = 9164, message = "Curiosity killed the cat? I'll show you!")
	public static NpcStringId CURIOSITY_KILLED_THE_CAT_I_LL_SHOW_YOU;
	
	@ClientString(id = 9165, message = "That's all for today...!")
	public static NpcStringId THAT_S_ALL_FOR_TODAY;
	
	@ClientString(id = 9166, message = "Are you trying to take Belinda from me, $s1? I'll show you!")
	public static NpcStringId ARE_YOU_TRYING_TO_TAKE_BELINDA_FROM_ME_S1_I_LL_SHOW_YOU;
	
	@ClientString(id = 9167, message = "Belinda! I love you! Yikes!!!")
	public static NpcStringId BELINDA_I_LOVE_YOU_YIKES;
	
	@ClientString(id = 9201, message = "Saga of the Elemental Master")
	public static NpcStringId SAGA_OF_THE_ELEMENTAL_MASTER;
	
	@ClientString(id = 9202, message = "Saga of the Elemental Master (In Progress)")
	public static NpcStringId SAGA_OF_THE_ELEMENTAL_MASTER_IN_PROGRESS;
	
	@ClientString(id = 9203, message = "Saga of the Elemental Master (Done)")
	public static NpcStringId SAGA_OF_THE_ELEMENTAL_MASTER_DONE;
	
	@ClientString(id = 9250, message = "You carouse with evil spirits, $s1! You're not worthy of the holy wisdom!")
	public static NpcStringId YOU_CAROUSE_WITH_EVIL_SPIRITS_S1_YOU_RE_NOT_WORTHY_OF_THE_HOLY_WISDOM2;
	
	@ClientString(id = 9251, message = "You're stubborn as a mule! Guess I can't boss you around any more!")
	public static NpcStringId YOU_RE_STUBBORN_AS_A_MULE_GUESS_I_CAN_T_BOSS_YOU_AROUND_ANY_MORE;
	
	@ClientString(id = 9252, message = "How could it be?...Defeated by an Elf!")
	public static NpcStringId HOW_COULD_IT_BE_DEFEATED_BY_AN_ELF;
	
	@ClientString(id = 9253, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S110;
	
	@ClientString(id = 9254, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA10;
	
	@ClientString(id = 9255, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS14;
	
	@ClientString(id = 9256, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE10;
	
	@ClientString(id = 9257, message = "I came to help you. It's the will of Radyss.")
	public static NpcStringId I_CAME_TO_HELP_YOU_IT_S_THE_WILL_OF_RADYSS;
	
	@ClientString(id = 9258, message = "$s1! Fight with me!")
	public static NpcStringId S1_FIGHT_WITH_ME;
	
	@ClientString(id = 9259, message = "$s1! We must defeat him!")
	public static NpcStringId S1_WE_MUST_DEFEAT_HIM;
	
	@ClientString(id = 9260, message = "$s1. There's no time. We must defeat him!")
	public static NpcStringId S1_THERE_S_NO_TIME_WE_MUST_DEFEAT_HIM;
	
	@ClientString(id = 9261, message = "Radyss is calling me. I gotta' go now.")
	public static NpcStringId RADYSS_IS_CALLING_ME_I_GOTTA_GO_NOW;
	
	@ClientString(id = 9262, message = "I was unable to avenge my brother.")
	public static NpcStringId I_WAS_UNABLE_TO_AVENGE_MY_BROTHER;
	
	@ClientString(id = 9263, message = "May you be blessed.")
	public static NpcStringId MAY_YOU_BE_BLESSED;
	
	@ClientString(id = 9264, message = "The proud, repent! The foolish, awaken! Sinners, die!")
	public static NpcStringId THE_PROUD_REPENT_THE_FOOLISH_AWAKEN_SINNERS_DIE;
	
	@ClientString(id = 9265, message = "Hell's master is calling. Atonement will have to wait!")
	public static NpcStringId HELL_S_MASTER_IS_CALLING_ATONEMENT_WILL_HAVE_TO_WAIT;
	
	@ClientString(id = 9266, message = "$s1, I'll remember your name, heathen.")
	public static NpcStringId S1_I_LL_REMEMBER_YOUR_NAME_HEATHEN;
	
	@ClientString(id = 9267, message = "I won't forget the name of one who doesn't obey holy judgment, $s1!")
	public static NpcStringId I_WON_T_FORGET_THE_NAME_OF_ONE_WHO_DOESN_T_OBEY_HOLY_JUDGMENT_S1;
	
	@ClientString(id = 9301, message = "Saga of the Spectral Master")
	public static NpcStringId SAGA_OF_THE_SPECTRAL_MASTER;
	
	@ClientString(id = 9302, message = "Saga of the Spectral Master (In Progress)")
	public static NpcStringId SAGA_OF_THE_SPECTRAL_MASTER_IN_PROGRESS;
	
	@ClientString(id = 9303, message = "Saga of the Spectral Master (Done)")
	public static NpcStringId SAGA_OF_THE_SPECTRAL_MASTER_DONE;
	
	@ClientString(id = 9350, message = "You carouse with evil spirits, $s1! You're not worthy of the holy wisdom!")
	public static NpcStringId YOU_CAROUSE_WITH_EVIL_SPIRITS_S1_YOU_RE_NOT_WORTHY_OF_THE_HOLY_WISDOM3;
	
	@ClientString(id = 9351, message = "You're stubborn as a mule! I guess I can't boss you around any more!")
	public static NpcStringId YOU_RE_STUBBORN_AS_A_MULE_I_GUESS_I_CAN_T_BOSS_YOU_AROUND_ANY_MORE;
	
	@ClientString(id = 9352, message = "Could it be...? Defeated by a Dark Elf!")
	public static NpcStringId COULD_IT_BE_DEFEATED_BY_A_DARK_ELF;
	
	@ClientString(id = 9353, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S111;
	
	@ClientString(id = 9354, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA11;
	
	@ClientString(id = 9355, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS15;
	
	@ClientString(id = 9356, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE11;
	
	@ClientString(id = 9357, message = "Shadow Summoner, I came here to help you.")
	public static NpcStringId SHADOW_SUMMONER_I_CAME_HERE_TO_HELP_YOU;
	
	@ClientString(id = 9358, message = "Shadow Summoner, $s1! Fight with me!")
	public static NpcStringId SHADOW_SUMMONER_S1_FIGHT_WITH_ME;
	
	@ClientString(id = 9359, message = "$s1, You'll die if you don't kill him!")
	public static NpcStringId S1_YOU_LL_DIE_IF_YOU_DON_T_KILL_HIM;
	
	@ClientString(id = 9360, message = "Hurry, $s1! Don't miss him!")
	public static NpcStringId HURRY_S1_DON_T_MISS_HIM;
	
	@ClientString(id = 9361, message = "I can't hold on any longer...")
	public static NpcStringId I_CAN_T_HOLD_ON_ANY_LONGER;
	
	@ClientString(id = 9362, message = "After all that...I missed him...")
	public static NpcStringId AFTER_ALL_THAT_I_MISSED_HIM;
	
	@ClientString(id = 9363, message = "Shadow summoner! May you be blessed!")
	public static NpcStringId SHADOW_SUMMONER_MAY_YOU_BE_BLESSED;
	
	@ClientString(id = 9364, message = "My master sent me here to kill you!")
	public static NpcStringId MY_MASTER_SENT_ME_HERE_TO_KILL_YOU;
	
	@ClientString(id = 9365, message = "The shadow is calling me...")
	public static NpcStringId THE_SHADOW_IS_CALLING_ME;
	
	@ClientString(id = 9366, message = "$s1, you want to die early? I'll send you to the darkness!")
	public static NpcStringId S1_YOU_WANT_TO_DIE_EARLY_I_LL_SEND_YOU_TO_THE_DARKNESS;
	
	@ClientString(id = 9367, message = "You deal in darkness, $s1! I'll pay you back.")
	public static NpcStringId YOU_DEAL_IN_DARKNESS_S1_I_LL_PAY_YOU_BACK;
	
	@ClientString(id = 9401, message = "Saga of the Soultaker")
	public static NpcStringId SAGA_OF_THE_SOULTAKER;
	
	@ClientString(id = 9402, message = "Saga of the Soultaker (In Progress)")
	public static NpcStringId SAGA_OF_THE_SOULTAKER_IN_PROGRESS;
	
	@ClientString(id = 9403, message = "Saga of the Soultaker (Done)")
	public static NpcStringId SAGA_OF_THE_SOULTAKER_DONE;
	
	@ClientString(id = 9450, message = "You're $s1? I won't be like Hindemith!")
	public static NpcStringId YOU_RE_S1_I_WON_T_BE_LIKE_HINDEMITH;
	
	@ClientString(id = 9451, message = "You're feistier than I thought! I'll stop here for today.")
	public static NpcStringId YOU_RE_FEISTIER_THAN_I_THOUGHT_I_LL_STOP_HERE_FOR_TODAY;
	
	@ClientString(id = 9452, message = "Aargh! I can't believe I lost...")
	public static NpcStringId AARGH_I_CAN_T_BELIEVE_I_LOST3;
	
	@ClientString(id = 9453, message = "Are you the one who is bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_IS_BOTHERING_MY_MINIONS_S1;
	
	@ClientString(id = 9454, message = "Yikes! You're tough!")
	public static NpcStringId YIKES_YOU_RE_TOUGH4;
	
	@ClientString(id = 9455, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS16;
	
	@ClientString(id = 9456, message = "I'll stop here for today.")
	public static NpcStringId I_LL_STOP_HERE_FOR_TODAY2;
	
	@ClientString(id = 9457, message = "I can't let you commune with Tablet of Vision! Give me the Resonance Amulet!")
	public static NpcStringId I_CAN_T_LET_YOU_COMMUNE_WITH_TABLET_OF_VISION_GIVE_ME_THE_RESONANCE_AMULET;
	
	@ClientString(id = 9458, message = "Dammit! I can't do this alone, $s1! Give me a hand!")
	public static NpcStringId DAMMIT_I_CAN_T_DO_THIS_ALONE_S1_GIVE_ME_A_HAND2;
	
	@ClientString(id = 9459, message = "$s1! Hurry or we'll miss him.")
	public static NpcStringId S1_HURRY_OR_WE_LL_MISS_HIM2;
	
	@ClientString(id = 9460, message = "$s1! Please hurry!")
	public static NpcStringId S1_PLEASE_HURRY;
	
	@ClientString(id = 9461, message = "I must follow him now.")
	public static NpcStringId I_MUST_FOLLOW_HIM_NOW;
	
	@ClientString(id = 9462, message = "Are you running? Stop!")
	public static NpcStringId ARE_YOU_RUNNING_STOP;
	
	@ClientString(id = 9463, message = "See you next time.")
	public static NpcStringId SEE_YOU_NEXT_TIME4;
	
	@ClientString(id = 9464, message = "Are you betraying me? I thought something was wrong...I'll stop here.")
	public static NpcStringId ARE_YOU_BETRAYING_ME_I_THOUGHT_SOMETHING_WAS_WRONG_I_LL_STOP_HERE;
	
	@ClientString(id = 9465, message = "I gotta' go now.")
	public static NpcStringId I_GOTTA_GO_NOW5;
	
	@ClientString(id = 9466, message = "You're $s1? Even two of you can't stop me!")
	public static NpcStringId YOU_RE_S1_EVEN_TWO_OF_YOU_CAN_T_STOP_ME;
	
	@ClientString(id = 9467, message = "Dammit! My Resonance Amulet...$s1, I'll never forget to pay you back.")
	public static NpcStringId DAMMIT_MY_RESONANCE_AMULET_S1_I_LL_NEVER_FORGET_TO_PAY_YOU_BACK;
	
	@ClientString(id = 9501, message = "Saga of the Hell Knight")
	public static NpcStringId SAGA_OF_THE_HELL_KNIGHT;
	
	@ClientString(id = 9502, message = "Saga of the Hell Knight (In Progress)")
	public static NpcStringId SAGA_OF_THE_HELL_KNIGHT_IN_PROGRESS;
	
	@ClientString(id = 9503, message = "Saga of the Hell Knight (Done)")
	public static NpcStringId SAGA_OF_THE_HELL_KNIGHT_DONE;
	
	@ClientString(id = 9550, message = "Are you... $s1? I won't be like Waldstein!")
	public static NpcStringId ARE_YOU_S1_I_WON_T_BE_LIKE_WALDSTEIN;
	
	@ClientString(id = 9551, message = "You're feistier than I thought! I'll stop here for today.")
	public static NpcStringId YOU_RE_FEISTIER_THAN_I_THOUGHT_I_LL_STOP_HERE_FOR_TODAY2;
	
	@ClientString(id = 9552, message = "Yikes! I can't believe I lost...")
	public static NpcStringId YIKES_I_CAN_T_BELIEVE_I_LOST;
	
	@ClientString(id = 9553, message = "Are you the one bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_BOTHERING_MY_MINIONS_S1;
	
	@ClientString(id = 9554, message = "Yikes! You're tough!")
	public static NpcStringId YIKES_YOU_RE_TOUGH5;
	
	@ClientString(id = 9555, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS17;
	
	@ClientString(id = 9556, message = "I'll stop here for today.")
	public static NpcStringId I_LL_STOP_HERE_FOR_TODAY3;
	
	@ClientString(id = 9557, message = "You can't commune with the Tablet of Vision! Give me the Resonance Amulet!")
	public static NpcStringId YOU_CAN_T_COMMUNE_WITH_THE_TABLET_OF_VISION_GIVE_ME_THE_RESONANCE_AMULET;
	
	@ClientString(id = 9558, message = "Dammit! I can't do this alone, $s1! Give me a hand!")
	public static NpcStringId DAMMIT_I_CAN_T_DO_THIS_ALONE_S1_GIVE_ME_A_HAND3;
	
	@ClientString(id = 9559, message = "$s1! Hurry or we'll miss him.")
	public static NpcStringId S1_HURRY_OR_WE_LL_MISS_HIM3;
	
	@ClientString(id = 9560, message = "$s1! Please hurry!")
	public static NpcStringId S1_PLEASE_HURRY2;
	
	@ClientString(id = 9561, message = "I gotta' go follow him.")
	public static NpcStringId I_GOTTA_GO_FOLLOW_HIM2;
	
	@ClientString(id = 9562, message = "Are you running? Stop!")
	public static NpcStringId ARE_YOU_RUNNING_STOP2;
	
	@ClientString(id = 9563, message = "See you next time.")
	public static NpcStringId SEE_YOU_NEXT_TIME5;
	
	@ClientString(id = 9564, message = "Are you betraying me? I thought something was wrong...I'll stop here.")
	public static NpcStringId ARE_YOU_BETRAYING_ME_I_THOUGHT_SOMETHING_WAS_WRONG_I_LL_STOP_HERE2;
	
	@ClientString(id = 9565, message = "I gotta go now...")
	public static NpcStringId I_GOTTA_GO_NOW6;
	
	@ClientString(id = 9566, message = "You're... $s1? Even two of you can't stop me!")
	public static NpcStringId YOU_RE_S1_EVEN_TWO_OF_YOU_CAN_T_STOP_ME2;
	
	@ClientString(id = 9567, message = "Dammit! My Resonance Amulet...$s1, I'll never forget this.")
	public static NpcStringId DAMMIT_MY_RESONANCE_AMULET_S1_I_LL_NEVER_FORGET_THIS;
	
	@ClientString(id = 9601, message = "Saga of the Spectral Dancer")
	public static NpcStringId SAGA_OF_THE_SPECTRAL_DANCER;
	
	@ClientString(id = 9602, message = "Saga of the Spectral Dancer (In Progress)")
	public static NpcStringId SAGA_OF_THE_SPECTRAL_DANCER_IN_PROGRESS;
	
	@ClientString(id = 9603, message = "Saga of the Spectral Dancer (Done)")
	public static NpcStringId SAGA_OF_THE_SPECTRAL_DANCER_DONE;
	
	@ClientString(id = 9650, message = "You're $s1? I'll kill you for Hallate!")
	public static NpcStringId YOU_RE_S1_I_LL_KILL_YOU_FOR_HALLATE;
	
	@ClientString(id = 9651, message = "You're tougher than I thought, but you still can't rival me!")
	public static NpcStringId YOU_RE_TOUGHER_THAN_I_THOUGHT_BUT_YOU_STILL_CAN_T_RIVAL_ME;
	
	@ClientString(id = 9652, message = "Hallate! Forgive me! I can't help you.")
	public static NpcStringId HALLATE_FORGIVE_ME_I_CAN_T_HELP_YOU;
	
	@ClientString(id = 9653, message = "Are you the one who's been bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_S_BEEN_BOTHERING_MY_MINIONS_S16;
	
	@ClientString(id = 9654, message = "Dammit! I can't believe you beat me!")
	public static NpcStringId DAMMIT_I_CAN_T_BELIEVE_YOU_BEAT_ME;
	
	@ClientString(id = 9655, message = "Who are you? Mind your own business, coward.")
	public static NpcStringId WHO_ARE_YOU_MIND_YOUR_OWN_BUSINESS_COWARD;
	
	@ClientString(id = 9656, message = "How weak. I'll forgive you this time because you made me laugh.")
	public static NpcStringId HOW_WEAK_I_LL_FORGIVE_YOU_THIS_TIME_BECAUSE_YOU_MADE_ME_LAUGH4;
	
	@ClientString(id = 9657, message = "Purgatory Lord, I won't fail this time.")
	public static NpcStringId PURGATORY_LORD_I_WON_T_FAIL_THIS_TIME;
	
	@ClientString(id = 9658, message = "$s1! Now's the time to put your training to the test!")
	public static NpcStringId S1_NOW_S_THE_TIME_TO_PUT_YOUR_TRAINING_TO_THE_TEST;
	
	@ClientString(id = 9659, message = "$s1! Your sword skills can't be that bad.")
	public static NpcStringId S1_YOUR_SWORD_SKILLS_CAN_T_BE_THAT_BAD;
	
	@ClientString(id = 9660, message = "$s1! Show your strength!")
	public static NpcStringId S1_SHOW_YOUR_STRENGTH;
	
	@ClientString(id = 9661, message = "I have some pressing business. I have to go.")
	public static NpcStringId I_HAVE_SOME_PRESSING_BUSINESS_I_HAVE_TO_GO;
	
	@ClientString(id = 9662, message = "I missed him! Dammit.")
	public static NpcStringId I_MISSED_HIM_DAMMIT;
	
	@ClientString(id = 9663, message = "Try again sometime.")
	public static NpcStringId TRY_AGAIN_SOMETIME;
	
	@ClientString(id = 9664, message = "I'll kill anyone who gets in my way!")
	public static NpcStringId I_LL_KILL_ANYONE_WHO_GETS_IN_MY_WAY;
	
	@ClientString(id = 9665, message = "This is pathetic! You make me laugh.")
	public static NpcStringId THIS_IS_PATHETIC_YOU_MAKE_ME_LAUGH;
	
	@ClientString(id = 9666, message = "$s1! Are you trying to get in my way?")
	public static NpcStringId S1_ARE_YOU_TRYING_TO_GET_IN_MY_WAY;
	
	@ClientString(id = 9667, message = "$s1! When I come back, I'll kill you.")
	public static NpcStringId S1_WHEN_I_COME_BACK_I_LL_KILL_YOU;
	
	@ClientString(id = 9701, message = "Saga of the Shillien Templar")
	public static NpcStringId SAGA_OF_THE_SHILLIEN_TEMPLAR;
	
	@ClientString(id = 9702, message = "Saga of the Shillien Templar (In Progress)")
	public static NpcStringId SAGA_OF_THE_SHILLIEN_TEMPLAR_IN_PROGRESS;
	
	@ClientString(id = 9703, message = "Saga of the Shillien Templar (Done)")
	public static NpcStringId SAGA_OF_THE_SHILLIEN_TEMPLAR_DONE;
	
	@ClientString(id = 9750, message = "$s1? Wake up! Time to die!")
	public static NpcStringId S1_WAKE_UP_TIME_TO_DIE;
	
	@ClientString(id = 9751, message = "You're tougher than I thought!  I'll be back!")
	public static NpcStringId YOU_RE_TOUGHER_THAN_I_THOUGHT_I_LL_BE_BACK;
	
	@ClientString(id = 9752, message = "I lost? It can't be!")
	public static NpcStringId I_LOST_IT_CAN_T_BE;
	
	@ClientString(id = 9753, message = "Are you the one who's been bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_S_BEEN_BOTHERING_MY_MINIONS_S17;
	
	@ClientString(id = 9754, message = "Dammit! I can't believe you beat me!")
	public static NpcStringId DAMMIT_I_CAN_T_BELIEVE_YOU_BEAT_ME2;
	
	@ClientString(id = 9755, message = "Who are you? Mind your own business, coward.")
	public static NpcStringId WHO_ARE_YOU_MIND_YOUR_OWN_BUSINESS_COWARD2;
	
	@ClientString(id = 9756, message = "How weak. I'll forgive you this time because you made me laugh.")
	public static NpcStringId HOW_WEAK_I_LL_FORGIVE_YOU_THIS_TIME_BECAUSE_YOU_MADE_ME_LAUGH5;
	
	@ClientString(id = 9757, message = "You're a cunning fiend. I won't fail again.")
	public static NpcStringId YOU_RE_A_CUNNING_FIEND_I_WON_T_FAIL_AGAIN;
	
	@ClientString(id = 9758, message = "$s1! It's after you! Fight!")
	public static NpcStringId S1_IT_S_AFTER_YOU_FIGHT;
	
	@ClientString(id = 9759, message = "$s1! You have to fight better than that if you expect to survive!")
	public static NpcStringId S1_YOU_HAVE_TO_FIGHT_BETTER_THAN_THAT_IF_YOU_EXPECT_TO_SURVIVE;
	
	@ClientString(id = 9760, message = "$s1! Pull yourself together. Fight!")
	public static NpcStringId S1_PULL_YOURSELF_TOGETHER_FIGHT;
	
	@ClientString(id = 9761, message = "I'll catch the cunning fiend.")
	public static NpcStringId I_LL_CATCH_THE_CUNNING_FIEND;
	
	@ClientString(id = 9762, message = "I missed him again! He's clever!")
	public static NpcStringId I_MISSED_HIM_AGAIN_HE_S_CLEVER;
	
	@ClientString(id = 9763, message = "Don't cower like a puppy next time!")
	public static NpcStringId DON_T_COWER_LIKE_A_PUPPY_NEXT_TIME;
	
	@ClientString(id = 9764, message = "I have only one goal. Get out of my way.")
	public static NpcStringId I_HAVE_ONLY_ONE_GOAL_GET_OUT_OF_MY_WAY;
	
	@ClientString(id = 9765, message = "Just wait. You'll get yours!")
	public static NpcStringId JUST_WAIT_YOU_LL_GET_YOURS;
	
	@ClientString(id = 9766, message = "$s1! You're a coward, aren't you!")
	public static NpcStringId S1_YOU_RE_A_COWARD_AREN_T_YOU;
	
	@ClientString(id = 9767, message = "$s1! I'll kill you next time.")
	public static NpcStringId S1_I_LL_KILL_YOU_NEXT_TIME;
	
	@ClientString(id = 9801, message = "Saga of the Shillien Saint")
	public static NpcStringId SAGA_OF_THE_SHILLIEN_SAINT;
	
	@ClientString(id = 9802, message = "Saga of the Shillien Saint (In Progress)")
	public static NpcStringId SAGA_OF_THE_SHILLIEN_SAINT_IN_PROGRESS;
	
	@ClientString(id = 9803, message = "Saga of the Shillien Saint (Done)")
	public static NpcStringId SAGA_OF_THE_SHILLIEN_SAINT_DONE;
	
	@ClientString(id = 9850, message = "$s1! How foolish to act against the will of god.")
	public static NpcStringId S1_HOW_FOOLISH_TO_ACT_AGAINST_THE_WILL_OF_GOD;
	
	@ClientString(id = 9851, message = "Your faith is stronger than I thought. I'll get you next time.")
	public static NpcStringId YOUR_FAITH_IS_STRONGER_THAN_I_THOUGHT_I_LL_GET_YOU_NEXT_TIME;
	
	@ClientString(id = 9852, message = "Tanakia, forgive me! I couldn't fulfill your dream!")
	public static NpcStringId TANAKIA_FORGIVE_ME_I_COULDN_T_FULFILL_YOUR_DREAM4;
	
	@ClientString(id = 9853, message = "Are you the one who's been bothering my minions, $s1?")
	public static NpcStringId ARE_YOU_THE_ONE_WHO_S_BEEN_BOTHERING_MY_MINIONS_S18;
	
	@ClientString(id = 9854, message = "Dammit! I can't believe you beat me!")
	public static NpcStringId DAMMIT_I_CAN_T_BELIEVE_YOU_BEAT_ME3;
	
	@ClientString(id = 9855, message = "Who are you? Mind your own business, you coward!")
	public static NpcStringId WHO_ARE_YOU_MIND_YOUR_OWN_BUSINESS_YOU_COWARD;
	
	@ClientString(id = 9856, message = "How weak. I'll forgive you this time because you made me laugh.")
	public static NpcStringId HOW_WEAK_I_LL_FORGIVE_YOU_THIS_TIME_BECAUSE_YOU_MADE_ME_LAUGH6;
	
	@ClientString(id = 9857, message = "Tanakia, your lie has already been exposed!")
	public static NpcStringId TANAKIA_YOUR_LIE_HAS_ALREADY_BEEN_EXPOSED;
	
	@ClientString(id = 9858, message = "$s1! Help me overcome this.")
	public static NpcStringId S1_HELP_ME_OVERCOME_THIS;
	
	@ClientString(id = 9859, message = "$s1! We can't defeat Tanakia this way.")
	public static NpcStringId S1_WE_CAN_T_DEFEAT_TANAKIA_THIS_WAY;
	
	@ClientString(id = 9860, message = "$s1! Here's our chance. Don't squander the opportunity!")
	public static NpcStringId S1_HERE_S_OUR_CHANCE_DON_T_SQUANDER_THE_OPPORTUNITY;
	
	@ClientString(id = 9861, message = "Goodbye. We'll meet again if fate allows.")
	public static NpcStringId GOODBYE_WE_LL_MEET_AGAIN_IF_FATE_ALLOWS;
	
	@ClientString(id = 9862, message = "I'll follow Tanakia to correct this falsehood.")
	public static NpcStringId I_LL_FOLLOW_TANAKIA_TO_CORRECT_THIS_FALSEHOOD;
	
	@ClientString(id = 9863, message = "I'll be back if fate allows.")
	public static NpcStringId I_LL_BE_BACK_IF_FATE_ALLOWS;
	
	@ClientString(id = 9864, message = "For Shilen!")
	public static NpcStringId FOR_SHILEN4;
	
	@ClientString(id = 9865, message = "I'll be back. You'll pay.")
	public static NpcStringId I_LL_BE_BACK_YOU_LL_PAY;
	
	@ClientString(id = 9866, message = "$s1! Are you trying to spoil my plan?")
	public static NpcStringId S1_ARE_YOU_TRYING_TO_SPOIL_MY_PLAN;
	
	@ClientString(id = 9867, message = "$s1! I won't forget you. You'll pay.")
	public static NpcStringId S1_I_WON_T_FORGET_YOU_YOU_LL_PAY;
	
	@ClientString(id = 9901, message = "Saga of the Fortune Seeker")
	public static NpcStringId SAGA_OF_THE_FORTUNE_SEEKER;
	
	@ClientString(id = 9902, message = "Saga of the Fortune Seeker (In Progress)")
	public static NpcStringId SAGA_OF_THE_FORTUNE_SEEKER_IN_PROGRESS;
	
	@ClientString(id = 9903, message = "Saga of the Fortune Seeker (Done)")
	public static NpcStringId SAGA_OF_THE_FORTUNE_SEEKER_DONE;
	
	@ClientString(id = 9950, message = "$s1, You have an affinity for dangerous ideas. Are you ready to die?")
	public static NpcStringId S1_YOU_HAVE_AN_AFFINITY_FOR_DANGEROUS_IDEAS_ARE_YOU_READY_TO_DIE;
	
	@ClientString(id = 9951, message = "My time is up...")
	public static NpcStringId MY_TIME_IS_UP;
	
	@ClientString(id = 9952, message = "I can't believe I must kneel to a Human!")
	public static NpcStringId I_CAN_T_BELIEVE_I_MUST_KNEEL_TO_A_HUMAN;
	
	@ClientString(id = 9953, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S112;
	
	@ClientString(id = 9954, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA12;
	
	@ClientString(id = 9955, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS18;
	
	@ClientString(id = 9956, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE12;
	
	@ClientString(id = 9957, message = "Minervia! What's the matter?")
	public static NpcStringId MINERVIA_WHAT_S_THE_MATTER;
	
	@ClientString(id = 9958, message = "The princess is in danger. Why are you staring?")
	public static NpcStringId THE_PRINCESS_IS_IN_DANGER_WHY_ARE_YOU_STARING;
	
	@ClientString(id = 9959, message = "Master $s1! Come on, Hurry up!")
	public static NpcStringId MASTER_S1_COME_ON_HURRY_UP;
	
	@ClientString(id = 9960, message = "We can't fail! Master $s1, Pull yourself together!")
	public static NpcStringId WE_CAN_T_FAIL_MASTER_S1_PULL_YOURSELF_TOGETHER;
	
	@ClientString(id = 9961, message = "What am I doing... I gotta' go! Goodbye.")
	public static NpcStringId WHAT_AM_I_DOING_I_GOTTA_GO_GOODBYE;
	
	@ClientString(id = 9962, message = "Dammit! I missed...!")
	public static NpcStringId DAMMIT_I_MISSED;
	
	@ClientString(id = 9963, message = "Sorry, but I must say goodbye again... Good luck to you!")
	public static NpcStringId SORRY_BUT_I_MUST_SAY_GOODBYE_AGAIN_GOOD_LUCK_TO_YOU;
	
	@ClientString(id = 9964, message = "I can't yield the secret of the tablet!")
	public static NpcStringId I_CAN_T_YIELD_THE_SECRET_OF_THE_TABLET;
	
	@ClientString(id = 9965, message = "I'll stop here for now...")
	public static NpcStringId I_LL_STOP_HERE_FOR_NOW;
	
	@ClientString(id = 9966, message = "$s1, you dared to leave scar on my face! I'll kill you!!!")
	public static NpcStringId S1_YOU_DARED_TO_LEAVE_SCAR_ON_MY_FACE_I_LL_KILL_YOU;
	
	@ClientString(id = 9967, message = "$s1, I won't forget your name...Ha!")
	public static NpcStringId S1_I_WON_T_FORGET_YOUR_NAME_HA;
	
	@ClientString(id = 10001, message = "Saga of the Maestro")
	public static NpcStringId SAGA_OF_THE_MAESTRO;
	
	@ClientString(id = 10002, message = "Saga of the Maestro (In Progress)")
	public static NpcStringId SAGA_OF_THE_MAESTRO_IN_PROGRESS;
	
	@ClientString(id = 10003, message = "Saga of the Maestro (Done)")
	public static NpcStringId SAGA_OF_THE_MAESTRO_DONE;
	
	@ClientString(id = 10050, message = "$s1? You have an affinity for bad ideas. Are you ready to die?")
	public static NpcStringId S1_YOU_HAVE_AN_AFFINITY_FOR_BAD_IDEAS_ARE_YOU_READY_TO_DIE;
	
	@ClientString(id = 10051, message = "My time is up...")
	public static NpcStringId MY_TIME_IS_UP2;
	
	@ClientString(id = 10052, message = "I can't believe I must kneel before a Human!")
	public static NpcStringId I_CAN_T_BELIEVE_I_MUST_KNEEL_BEFORE_A_HUMAN;
	
	@ClientString(id = 10053, message = "You dare to disturb the order of the shrine! Die, $s1!")
	public static NpcStringId YOU_DARE_TO_DISTURB_THE_ORDER_OF_THE_SHRINE_DIE_S113;
	
	@ClientString(id = 10054, message = "My spirit is released from this shell. I'm getting close to Halisha...")
	public static NpcStringId MY_SPIRIT_IS_RELEASED_FROM_THIS_SHELL_I_M_GETTING_CLOSE_TO_HALISHA13;
	
	@ClientString(id = 10055, message = "Mind your own business!")
	public static NpcStringId MIND_YOUR_OWN_BUSINESS19;
	
	@ClientString(id = 10056, message = "This is a waste of time. Goodbye!")
	public static NpcStringId THIS_IS_A_WASTE_OF_TIME_GOODBYE13;
	
	@ClientString(id = 10057, message = "You thief! Give me the Resonance Amulet!")
	public static NpcStringId YOU_THIEF_GIVE_ME_THE_RESONANCE_AMULET;
	
	@ClientString(id = 10058, message = "Ugh! $s1, Help me!")
	public static NpcStringId UGH_S1_HELP_ME;
	
	@ClientString(id = 10059, message = "$s1. Please, help me! Together we can beat him.")
	public static NpcStringId S1_PLEASE_HELP_ME_TOGETHER_WE_CAN_BEAT_HIM;
	
	@ClientString(id = 10060, message = "$s1! Are you going to let a guild member die?")
	public static NpcStringId S1_ARE_YOU_GOING_TO_LET_A_GUILD_MEMBER_DIE;
	
	@ClientString(id = 10061, message = "I'm sorry, but I gotta' go first!")
	public static NpcStringId I_M_SORRY_BUT_I_GOTTA_GO_FIRST;
	
	@ClientString(id = 10062, message = "Aaaah! I couldn't get the Resonance Amulet.")
	public static NpcStringId AAAAH_I_COULDN_T_GET_THE_RESONANCE_AMULET;
	
	@ClientString(id = 10063, message = "Take care! I gotta' go now~!")
	public static NpcStringId TAKE_CARE_I_GOTTA_GO_NOW;
	
	@ClientString(id = 10064, message = "I'm sorry, but it's my job to kill you now!")
	public static NpcStringId I_M_SORRY_BUT_IT_S_MY_JOB_TO_KILL_YOU_NOW;
	
	@ClientString(id = 10065, message = "What a waste of time!")
	public static NpcStringId WHAT_A_WASTE_OF_TIME;
	
	@ClientString(id = 10066, message = "$s1! How could you do this? I'll kill you!")
	public static NpcStringId S1_HOW_COULD_YOU_DO_THIS_I_LL_KILL_YOU;
	
	@ClientString(id = 10067, message = "$s1! I'll pay you back!")
	public static NpcStringId S1_I_LL_PAY_YOU_BACK;
	
	@ClientString(id = 10068, message = "Why don't you just die?!")
	public static NpcStringId WHY_DON_T_YOU_JUST_DIE;
	
	@ClientString(id = 10069, message = "Taste the sting of Level 5 Spoil!")
	public static NpcStringId TASTE_THE_STING_OF_LEVEL_5_SPOIL;
	
	@ClientString(id = 10070, message = "The item is already inside you...")
	public static NpcStringId THE_ITEM_IS_ALREADY_INSIDE_YOU;
	
	@ClientString(id = 10071, message = "This potion you're making me drink is worth its weight in gold!")
	public static NpcStringId THIS_POTION_YOU_RE_MAKING_ME_DRINK_IS_WORTH_ITS_WEIGHT_IN_GOLD;
	
	@ClientString(id = 10072, message = "This potion is prepared from the ground gall of a bear. Be careful - it packs quite a punch!")
	public static NpcStringId THIS_POTION_IS_PREPARED_FROM_THE_GROUND_GALL_OF_A_BEAR_BE_CAREFUL_IT_PACKS_QUITE_A_PUNCH;
	
	@ClientString(id = 10073, message = "How can you use a potion on a newbie...")
	public static NpcStringId HOW_CAN_YOU_USE_A_POTION_ON_A_NEWBIE;
	
	@ClientString(id = 10074, message = "Listen to me, $s1! Unless you have prior authorization, you can't carry a weapon here!")
	public static NpcStringId LISTEN_TO_ME_S1_UNLESS_YOU_HAVE_PRIOR_AUTHORIZATION_YOU_CAN_T_CARRY_A_WEAPON_HERE;
	
	@ClientString(id = 10075, message = "Dear $s1, may the blessings of Einhasad be with you always.")
	public static NpcStringId DEAR_S1_MAY_THE_BLESSINGS_OF_EINHASAD_BE_WITH_YOU_ALWAYS;
	
	@ClientString(id = 10076, message = "Dear brother $s1, follow the path of light with me...")
	public static NpcStringId DEAR_BROTHER_S1_FOLLOW_THE_PATH_OF_LIGHT_WITH_ME;
	
	@ClientString(id = 10077, message = "$s1, why would you choose the path of darkness?!")
	public static NpcStringId S1_WHY_WOULD_YOU_CHOOSE_THE_PATH_OF_DARKNESS;
	
	@ClientString(id = 10078, message = "$s1!  How dare you defy the will of Einhasad!")
	public static NpcStringId S1_HOW_DARE_YOU_DEFY_THE_WILL_OF_EINHASAD;
	
	@ClientString(id = 10079, message = "The door to the 3rd floor of the altar is now open.")
	public static NpcStringId THE_DOOR_TO_THE_3RD_FLOOR_OF_THE_ALTAR_IS_NOW_OPEN;
	
	@ClientString(id = 10101, message = "Sword of Solidarity")
	public static NpcStringId SWORD_OF_SOLIDARITY;
	
	@ClientString(id = 10102, message = "Sword of Solidarity (In Progress)")
	public static NpcStringId SWORD_OF_SOLIDARITY_IN_PROGRESS;
	
	@ClientString(id = 10103, message = "Sword of Solidarity (Done)")
	public static NpcStringId SWORD_OF_SOLIDARITY_DONE;
	
	@ClientString(id = 10201, message = "Sea of Spores Fever")
	public static NpcStringId SEA_OF_SPORES_FEVER;
	
	@ClientString(id = 10202, message = "Sea of Spores Fever (In Progress)")
	public static NpcStringId SEA_OF_SPORES_FEVER_IN_PROGRESS;
	
	@ClientString(id = 10203, message = "Sea of Spores Fever (Done)")
	public static NpcStringId SEA_OF_SPORES_FEVER_DONE;
	
	@ClientString(id = 10301, message = "Spirit of Craftsman")
	public static NpcStringId SPIRIT_OF_CRAFTSMAN;
	
	@ClientString(id = 10302, message = "Spirit of Craftsman (In Progress)")
	public static NpcStringId SPIRIT_OF_CRAFTSMAN_IN_PROGRESS;
	
	@ClientString(id = 10303, message = "Spirit of Craftsman (Done)")
	public static NpcStringId SPIRIT_OF_CRAFTSMAN_DONE;
	
	@ClientString(id = 10401, message = "Spirit of Mirrors")
	public static NpcStringId SPIRIT_OF_MIRRORS;
	
	@ClientString(id = 10402, message = "Spirit of Mirrors (In Progress)")
	public static NpcStringId SPIRIT_OF_MIRRORS_IN_PROGRESS;
	
	@ClientString(id = 10403, message = "Spirit of Mirrors (Done)")
	public static NpcStringId SPIRIT_OF_MIRRORS_DONE;
	
	@ClientString(id = 10501, message = "Skirmish with the Orcs")
	public static NpcStringId SKIRMISH_WITH_THE_ORCS;
	
	@ClientString(id = 10502, message = "Skirmish with the Orcs (In Progress)")
	public static NpcStringId SKIRMISH_WITH_THE_ORCS_IN_PROGRESS;
	
	@ClientString(id = 10503, message = "Skirmish with the Orcs (Done)")
	public static NpcStringId SKIRMISH_WITH_THE_ORCS_DONE;
	
	@ClientString(id = 10601, message = "Forgotten Truth")
	public static NpcStringId FORGOTTEN_TRUTH;
	
	@ClientString(id = 10602, message = "Forgotten Truth (In Progress)")
	public static NpcStringId FORGOTTEN_TRUTH_IN_PROGRESS;
	
	@ClientString(id = 10603, message = "Forgotten Truth (Done)")
	public static NpcStringId FORGOTTEN_TRUTH_DONE;
	
	@ClientString(id = 10701, message = "Merciless Punishment")
	public static NpcStringId MERCILESS_PUNISHMENT;
	
	@ClientString(id = 10702, message = "Merciless Punishment (In Progress)")
	public static NpcStringId MERCILESS_PUNISHMENT_IN_PROGRESS;
	
	@ClientString(id = 10703, message = "Merciless Punishment (Done)")
	public static NpcStringId MERCILESS_PUNISHMENT_DONE;
	
	@ClientString(id = 10801, message = "Jumble, Tumble, Diamond Fuss")
	public static NpcStringId JUMBLE_TUMBLE_DIAMOND_FUSS;
	
	@ClientString(id = 10802, message = "Jumble, Tumble, Diamond Fuss (In Progress)")
	public static NpcStringId JUMBLE_TUMBLE_DIAMOND_FUSS_IN_PROGRESS;
	
	@ClientString(id = 10803, message = "Jumble, Tumble, Diamond Fuss (Done)")
	public static NpcStringId JUMBLE_TUMBLE_DIAMOND_FUSS_DONE;
	
	@ClientString(id = 10901, message = "In Search of the Nest (Lv. 81-99)")
	public static NpcStringId IN_SEARCH_OF_THE_NEST_LV_81_99;
	
	@ClientString(id = 10902, message = "In Search of the Nest (Lv. 81-99) (In Progress)")
	public static NpcStringId IN_SEARCH_OF_THE_NEST_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 10903, message = "In Search of the Nest (Lv. 81-99) (Done)")
	public static NpcStringId IN_SEARCH_OF_THE_NEST_LV_81_99_DONE;
	
	@ClientString(id = 10904, message = "In Search of the Nest (Lv. 81-99)")
	public static NpcStringId IN_SEARCH_OF_THE_NEST_LV_81_992;
	
	@ClientString(id = 11001, message = "To the Primeval Isle (Lv. 75-99)")
	public static NpcStringId TO_THE_PRIMEVAL_ISLE_LV_75_99;
	
	@ClientString(id = 11002, message = "To the Primeval Isle (Lv. 75-99) (In Progress)")
	public static NpcStringId TO_THE_PRIMEVAL_ISLE_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 11003, message = "To the Primeval Isle (Lv. 75-99) (Done)")
	public static NpcStringId TO_THE_PRIMEVAL_ISLE_LV_75_99_DONE;
	
	@ClientString(id = 11004, message = "To the Primeval Isle (Lv. 75-99)")
	public static NpcStringId TO_THE_PRIMEVAL_ISLE_LV_75_992;
	
	@ClientString(id = 11101, message = "Elrokian Hunter's Proof (Lv. 75-99)")
	public static NpcStringId ELROKIAN_HUNTER_S_PROOF_LV_75_99;
	
	@ClientString(id = 11102, message = "Elrokian Hunter's Proof (Lv. 75-99) (In Progress)")
	public static NpcStringId ELROKIAN_HUNTER_S_PROOF_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 11103, message = "Elrokian Hunter's Proof (Lv. 75-99) (Done)")
	public static NpcStringId ELROKIAN_HUNTER_S_PROOF_LV_75_99_DONE;
	
	@ClientString(id = 11104, message = "Elrokian Hunter's Proof (Lv. 75-99)")
	public static NpcStringId ELROKIAN_HUNTER_S_PROOF_LV_75_992;
	
	@ClientString(id = 11201, message = "Walk of Fate")
	public static NpcStringId WALK_OF_FATE;
	
	@ClientString(id = 11202, message = "Walk of Fate (In Progress)")
	public static NpcStringId WALK_OF_FATE_IN_PROGRESS;
	
	@ClientString(id = 11203, message = "Walk of Fate (Done)")
	public static NpcStringId WALK_OF_FATE_DONE;
	
	@ClientString(id = 11301, message = "Status of the Beacon Tower (Lv. 80-99)")
	public static NpcStringId STATUS_OF_THE_BEACON_TOWER_LV_80_99;
	
	@ClientString(id = 11302, message = "Status of the Beacon Tower (Lv. 80-99) (In Progress)")
	public static NpcStringId STATUS_OF_THE_BEACON_TOWER_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 11303, message = "Status of the Beacon Tower (Lv. 80-99) (Done)")
	public static NpcStringId STATUS_OF_THE_BEACON_TOWER_LV_80_99_DONE;
	
	@ClientString(id = 11304, message = "Status of the Beacon Tower (Lv. 80-99)")
	public static NpcStringId STATUS_OF_THE_BEACON_TOWER_LV_80_992;
	
	@ClientString(id = 11401, message = "Resurrection of an Old Manager (Lv. 70-99)")
	public static NpcStringId RESURRECTION_OF_AN_OLD_MANAGER_LV_70_99;
	
	@ClientString(id = 11402, message = "Resurrection of an Old Manager (Lv. 70-99) (In Progress)")
	public static NpcStringId RESURRECTION_OF_AN_OLD_MANAGER_LV_70_99_IN_PROGRESS;
	
	@ClientString(id = 11403, message = "Resurrection of an Old Manager (Lv. 70-99) (Done)")
	public static NpcStringId RESURRECTION_OF_AN_OLD_MANAGER_LV_70_99_DONE;
	
	@ClientString(id = 11404, message = "Resurrection of an Old Manager (Lv. 70-99)")
	public static NpcStringId RESURRECTION_OF_AN_OLD_MANAGER_LV_70_992;
	
	@ClientString(id = 11450, message = "You, $s1, you attacked Wendy. Prepare to die!")
	public static NpcStringId YOU_S1_YOU_ATTACKED_WENDY_PREPARE_TO_DIE;
	
	@ClientString(id = 11451, message = "$s1, your enemy was driven out. I will now withdraw and await your next command.")
	public static NpcStringId S1_YOUR_ENEMY_WAS_DRIVEN_OUT_I_WILL_NOW_WITHDRAW_AND_AWAIT_YOUR_NEXT_COMMAND;
	
	@ClientString(id = 11452, message = "This enemy is far too powerful for me to fight. I must withdraw.")
	public static NpcStringId THIS_ENEMY_IS_FAR_TOO_POWERFUL_FOR_ME_TO_FIGHT_I_MUST_WITHDRAW;
	
	@ClientString(id = 11453, message = "The radio signal detector is responding. # A suspicious pile of stones catches your eye.")
	public static NpcStringId THE_RADIO_SIGNAL_DETECTOR_IS_RESPONDING_A_SUSPICIOUS_PILE_OF_STONES_CATCHES_YOUR_EYE;
	
	@ClientString(id = 11501, message = "The Other Side of Truth (Lv. 53-63)")
	public static NpcStringId THE_OTHER_SIDE_OF_TRUTH_LV_53_63;
	
	@ClientString(id = 11502, message = "The Other Side of Truth (Lv. 53-63) (In Progress)")
	public static NpcStringId THE_OTHER_SIDE_OF_TRUTH_LV_53_63_IN_PROGRESS;
	
	@ClientString(id = 11503, message = "The Other Side of Truth (Lv. 53-63) (Done)")
	public static NpcStringId THE_OTHER_SIDE_OF_TRUTH_LV_53_63_DONE;
	
	@ClientString(id = 11504, message = "The Other Side of Truth (Lv. 53-63)")
	public static NpcStringId THE_OTHER_SIDE_OF_TRUTH_LV_53_632;
	
	@ClientString(id = 11505, message = "$s1 has become the Hero of the Sigel Phoenix Knight class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_SIGEL_PHOENIX_KNIGHT_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11506, message = "$s1 has become the Hero of the Sigel Hell Knight class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_SIGEL_HELL_KNIGHT_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11507, message = "$s1 has become the Hero of the Sigel Eva's Templar class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_SIGEL_EVA_S_TEMPLAR_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11508, message = "$s1 has become the Hero of the Sigel Shillien Templar class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_SIGEL_SHILLIEN_TEMPLAR_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11509, message = "$s1 has become the Hero of the Tyrr Duelist class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_TYRR_DUELIST_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11510, message = "$s1 has become the Hero of the Tyrr Dreadnought class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_TYRR_DREADNOUGHT_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11511, message = "$s1 has become the Hero of the Tyrr Titan class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_TYRR_TITAN_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11512, message = "$s1 has become the Hero of the Tyrr Grand Khavatari class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_TYRR_GRAND_KHAVATARI_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11513, message = "$s1 has become the Hero of the Tyrr Maestro class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_TYRR_MAESTRO_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11514, message = "$s1 has become the Hero of the Tyrr DoomBringer class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_TYRR_DOOMBRINGER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11515, message = "$s1 has become the Hero of the Othell Adventurer class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_OTHELL_ADVENTURER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11516, message = "$s1 has become the Hero of the Othell Wind Rider class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_OTHELL_WIND_RIDER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11517, message = "$s1 has become the Hero of the Othell Ghost Hunter class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_OTHELL_GHOST_HUNTER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11518, message = "$s1 has become the Hero of the Othell Fortune Seeker class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_OTHELL_FORTUNE_SEEKER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11519, message = "$s1 has become the Hero of the Yul Sagittarius class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_YUL_SAGITTARIUS_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11520, message = "$s1 has become the Hero of the Yul Moonlight Sentinel class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_YUL_MOONLIGHT_SENTINEL_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11521, message = "$s1 has become the Hero of the Yul Ghost Sentinel class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_YUL_GHOST_SENTINEL_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11522, message = "$s1 has become the Hero of the Yul Trickster class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_YUL_TRICKSTER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11523, message = "$s1 has become the Hero of the Feoh Archmage class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_FEOH_ARCHMAGE_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11524, message = "$s1 has become the Hero of the Feoh Soultaker class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_FEOH_SOULTAKER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11525, message = "$s1 has become the Hero of the Feoh Mystic Muse class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_FEOH_MYSTIC_MUSE_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11526, message = "$s1 has become the Hero of the Feoh Storm Screamer class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_FEOH_STORM_SCREAMER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11527, message = "$s1 has become the Hero of the Feoh Soul Hound class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_FEOH_SOUL_HOUND_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11528, message = "$s1 has become the Hero of the Iss Hierophant class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_ISS_HIEROPHANT_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11529, message = "$s1 has become the Hero of the Iss Sword Muse class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_ISS_SWORD_MUSE_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11530, message = "$s1 has become the Hero of the Iss Spectral Dancer class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_ISS_SPECTRAL_DANCER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11531, message = "$s1 has become the Hero of the Iss Dominator class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_ISS_DOMINATOR_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11532, message = "$s1 has become the Hero of the Iss DoomCryer class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_ISS_DOOMCRYER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11533, message = "$s1 has become the Hero of the Wynn Arcana Lord class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_WYNN_ARCANA_LORD_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11534, message = "$s1 has become the Hero of the Wynn Elemental Master class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_WYNN_ELEMENTAL_MASTER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11535, message = "$s1 has become the Hero of the Wynn Spectral Master class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_WYNN_SPECTRAL_MASTER_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11536, message = "$s1 has become the Hero of the Aeore Cardinal class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_AEORE_CARDINAL_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11537, message = "$s1 has become the Hero of the Aeore Eva's Saint class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_AEORE_EVA_S_SAINT_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11538, message = "$s1 has become the Hero of the Aeore Shillien Saint class. Congratulations!")
	public static NpcStringId S1_HAS_BECOME_THE_HERO_OF_THE_AEORE_SHILLIEN_SAINT_CLASS_CONGRATULATIONS;
	
	@ClientString(id = 11539, message = "Congratulations! You have become a Hero of the Eviscerators.")
	public static NpcStringId CONGRATULATIONS_YOU_HAVE_BECOME_A_HERO_OF_THE_EVISCERATORS;
	
	@ClientString(id = 11540, message = "Congratulations! You have become a Hero of Sayha's Seers.")
	public static NpcStringId CONGRATULATIONS_YOU_HAVE_BECOME_A_HERO_OF_SAYHA_S_SEERS;
	
	@ClientString(id = 11550, message = "This looks like the right place...")
	public static NpcStringId THIS_LOOKS_LIKE_THE_RIGHT_PLACE;
	
	@ClientString(id = 11551, message = "I see someone. Is this fate?")
	public static NpcStringId I_SEE_SOMEONE_IS_THIS_FATE;
	
	@ClientString(id = 11552, message = "We meet again.")
	public static NpcStringId WE_MEET_AGAIN;
	
	@ClientString(id = 11553, message = "Don't bother trying to find out more about me. Follow your own destiny.")
	public static NpcStringId DON_T_BOTHER_TRYING_TO_FIND_OUT_MORE_ABOUT_ME_FOLLOW_YOUR_OWN_DESTINY;
	
	@ClientString(id = 11601, message = "Beyond the Hills of Winter")
	public static NpcStringId BEYOND_THE_HILLS_OF_WINTER;
	
	@ClientString(id = 11602, message = "Beyond the Hills of Winter (In Progress)")
	public static NpcStringId BEYOND_THE_HILLS_OF_WINTER_IN_PROGRESS;
	
	@ClientString(id = 11603, message = "Beyond the Hills of Winter (Done)")
	public static NpcStringId BEYOND_THE_HILLS_OF_WINTER_DONE;
	
	@ClientString(id = 11701, message = "The Ocean of Distant Stars (Lv. 39-99)")
	public static NpcStringId THE_OCEAN_OF_DISTANT_STARS_LV_39_99;
	
	@ClientString(id = 11702, message = "The Ocean of Distant Stars (Lv. 39-99) (In Progress)")
	public static NpcStringId THE_OCEAN_OF_DISTANT_STARS_LV_39_99_IN_PROGRESS;
	
	@ClientString(id = 11703, message = "The Ocean of Distant Stars (Lv. 39-99) (Done)")
	public static NpcStringId THE_OCEAN_OF_DISTANT_STARS_LV_39_99_DONE;
	
	@ClientString(id = 11704, message = "The Ocean of Distant Stars (Lv. 39-99)")
	public static NpcStringId THE_OCEAN_OF_DISTANT_STARS_LV_39_992;
	
	@ClientString(id = 11801, message = "To Lead and Be Led")
	public static NpcStringId TO_LEAD_AND_BE_LED;
	
	@ClientString(id = 11802, message = "To Lead and Be Led (In Progress)")
	public static NpcStringId TO_LEAD_AND_BE_LED_IN_PROGRESS;
	
	@ClientString(id = 11803, message = "To Lead and Be Led (Done)")
	public static NpcStringId TO_LEAD_AND_BE_LED_DONE;
	
	@ClientString(id = 11804, message = "To Lead and Be Led (Sponsor)")
	public static NpcStringId TO_LEAD_AND_BE_LED_SPONSOR;
	
	@ClientString(id = 11901, message = "Last Imperial Prince (Lv. 74-80)")
	public static NpcStringId LAST_IMPERIAL_PRINCE_LV_74_80;
	
	@ClientString(id = 11902, message = "Last Imperial Prince (Lv. 74-80) (In Progress)")
	public static NpcStringId LAST_IMPERIAL_PRINCE_LV_74_80_IN_PROGRESS;
	
	@ClientString(id = 11903, message = "Last Imperial Prince (Lv. 74-80) (Done)")
	public static NpcStringId LAST_IMPERIAL_PRINCE_LV_74_80_DONE;
	
	@ClientString(id = 11904, message = "Last Imperial Prince (Lv. 74-80)")
	public static NpcStringId LAST_IMPERIAL_PRINCE_LV_74_802;
	
	@ClientString(id = 12001, message = "Pavel's Last Research (Lv. 70-99)")
	public static NpcStringId PAVEL_S_LAST_RESEARCH_LV_70_99;
	
	@ClientString(id = 12002, message = "Pavel's Last Research (Lv. 70-99) (In Progress)")
	public static NpcStringId PAVEL_S_LAST_RESEARCH_LV_70_99_IN_PROGRESS;
	
	@ClientString(id = 12003, message = "Pavel's Last Research (Lv. 70-99) (Done)")
	public static NpcStringId PAVEL_S_LAST_RESEARCH_LV_70_99_DONE;
	
	@ClientString(id = 12004, message = "Pavel's Last Research (Lv. 70-99)")
	public static NpcStringId PAVEL_S_LAST_RESEARCH_LV_70_992;
	
	@ClientString(id = 12101, message = "Pavel the Giant (Lv. 70-99)")
	public static NpcStringId PAVEL_THE_GIANT_LV_70_99;
	
	@ClientString(id = 12102, message = "Pavel the Giant (Lv. 70-99) (In Progress)")
	public static NpcStringId PAVEL_THE_GIANT_LV_70_99_IN_PROGRESS;
	
	@ClientString(id = 12103, message = "Pavel the Giant (Lv. 70-99) (Done)")
	public static NpcStringId PAVEL_THE_GIANT_LV_70_99_DONE;
	
	@ClientString(id = 12104, message = "Pavel the Giant (Lv. 70-99)")
	public static NpcStringId PAVEL_THE_GIANT_LV_70_992;
	
	@ClientString(id = 12201, message = "Ominous News (Lv. 20-36)")
	public static NpcStringId OMINOUS_NEWS_LV_20_36;
	
	@ClientString(id = 12202, message = "Ominous News (Lv. 20-36) (In Progress)")
	public static NpcStringId OMINOUS_NEWS_LV_20_36_IN_PROGRESS;
	
	@ClientString(id = 12203, message = "Ominous News (Lv. 20-36) (Done)")
	public static NpcStringId OMINOUS_NEWS_LV_20_36_DONE;
	
	@ClientString(id = 12204, message = "Ominous News (Lv. 20-36)")
	public static NpcStringId OMINOUS_NEWS_LV_20_362;
	
	@ClientString(id = 12301, message = "The Leader and the Follower")
	public static NpcStringId THE_LEADER_AND_THE_FOLLOWER;
	
	@ClientString(id = 12302, message = "The Leader and the Follower (In Progress)")
	public static NpcStringId THE_LEADER_AND_THE_FOLLOWER_IN_PROGRESS;
	
	@ClientString(id = 12303, message = "The Leader and the Follower (Done)")
	public static NpcStringId THE_LEADER_AND_THE_FOLLOWER_DONE;
	
	@ClientString(id = 12304, message = "The Leader and the Follower (Sponsor)")
	public static NpcStringId THE_LEADER_AND_THE_FOLLOWER_SPONSOR;
	
	@ClientString(id = 12401, message = "Meeting the Elroki (Lv. 75-99)")
	public static NpcStringId MEETING_THE_ELROKI_LV_75_99;
	
	@ClientString(id = 12402, message = "Meeting the Elroki (Lv. 75-99) (In Progress)")
	public static NpcStringId MEETING_THE_ELROKI_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 12403, message = "Meeting the Elroki (Lv. 75-99) (Done)")
	public static NpcStringId MEETING_THE_ELROKI_LV_75_99_DONE;
	
	@ClientString(id = 12404, message = "Meeting the Elroki (Lv. 75-99)")
	public static NpcStringId MEETING_THE_ELROKI_LV_75_992;
	
	@ClientString(id = 12501, message = "The Name of Evil 1 (Lv. 76-99)")
	public static NpcStringId THE_NAME_OF_EVIL_1_LV_76_99;
	
	@ClientString(id = 12502, message = "The Name of Evil 1 (Lv. 76-99) (In Progress)")
	public static NpcStringId THE_NAME_OF_EVIL_1_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 12503, message = "The Name of Evil 1 (Lv. 76-99) (Done)")
	public static NpcStringId THE_NAME_OF_EVIL_1_LV_76_99_DONE;
	
	@ClientString(id = 12504, message = "The Name of Evil 1 (Lv. 76-99)")
	public static NpcStringId THE_NAME_OF_EVIL_1_LV_76_992;
	
	@ClientString(id = 12601, message = "The Name of Evil 2 (Lv. 77-99)")
	public static NpcStringId THE_NAME_OF_EVIL_2_LV_77_99;
	
	@ClientString(id = 12602, message = "The Name of Evil 2 (Lv. 77-99) (In Progress)")
	public static NpcStringId THE_NAME_OF_EVIL_2_LV_77_99_IN_PROGRESS;
	
	@ClientString(id = 12603, message = "The Name of Evil 2 (Lv. 77-99) (Done)")
	public static NpcStringId THE_NAME_OF_EVIL_2_LV_77_99_DONE;
	
	@ClientString(id = 12604, message = "The Name of Evil 2 (Lv. 77-99)")
	public static NpcStringId THE_NAME_OF_EVIL_2_LV_77_992;
	
	@ClientString(id = 12701, message = "Kamael: A Window to the Future")
	public static NpcStringId KAMAEL_A_WINDOW_TO_THE_FUTURE;
	
	@ClientString(id = 12702, message = "Kamael: A Window to the Future (In Progress)")
	public static NpcStringId KAMAEL_A_WINDOW_TO_THE_FUTURE_IN_PROGRESS;
	
	@ClientString(id = 12703, message = "Kamael: A Window to the Future (Done)")
	public static NpcStringId KAMAEL_A_WINDOW_TO_THE_FUTURE_DONE;
	
	@ClientString(id = 12801, message = "Pailaka - Song of Ice and Fire (Lv. 49-55)")
	public static NpcStringId PAILAKA_SONG_OF_ICE_AND_FIRE_LV_49_55;
	
	@ClientString(id = 12802, message = "Pailaka - Song of Ice and Fire (Lv. 49-55) (In Progress)")
	public static NpcStringId PAILAKA_SONG_OF_ICE_AND_FIRE_LV_49_55_IN_PROGRESS;
	
	@ClientString(id = 12803, message = "Pailaka - Song of Ice and Fire (Lv. 49-55) (done)")
	public static NpcStringId PAILAKA_SONG_OF_ICE_AND_FIRE_LV_49_55_DONE;
	
	@ClientString(id = 12804, message = "Pailaka - Song of Ice and Fire (Lv. 49-55)")
	public static NpcStringId PAILAKA_SONG_OF_ICE_AND_FIRE_LV_49_552;
	
	@ClientString(id = 12901, message = "Pailaka - Devil's Legacy (Lv. 61-67)")
	public static NpcStringId PAILAKA_DEVIL_S_LEGACY_LV_61_67;
	
	@ClientString(id = 12902, message = "Pailaka - Devil's Legacy (Lv. 61-67) (In Progress)")
	public static NpcStringId PAILAKA_DEVIL_S_LEGACY_LV_61_67_IN_PROGRESS;
	
	@ClientString(id = 12903, message = "Pailaka - Devil's Legacy (Lv. 61-67) (done)")
	public static NpcStringId PAILAKA_DEVIL_S_LEGACY_LV_61_67_DONE;
	
	@ClientString(id = 13001, message = "Path to Hellbound")
	public static NpcStringId PATH_TO_HELLBOUND;
	
	@ClientString(id = 13002, message = "Path to Hellbound (In Progress)")
	public static NpcStringId PATH_TO_HELLBOUND_IN_PROGRESS;
	
	@ClientString(id = 13003, message = "Path to Hellbound (Done)")
	public static NpcStringId PATH_TO_HELLBOUND_DONE;
	
	@ClientString(id = 13004, message = "Path to Hellbound")
	public static NpcStringId PATH_TO_HELLBOUND2;
	
	@ClientString(id = 13101, message = "Bird in a Cage")
	public static NpcStringId BIRD_IN_A_CAGE;
	
	@ClientString(id = 13102, message = "Bird in a Cage (In Progress)")
	public static NpcStringId BIRD_IN_A_CAGE_IN_PROGRESS;
	
	@ClientString(id = 13103, message = "Bird in a Cage (Done)")
	public static NpcStringId BIRD_IN_A_CAGE_DONE;
	
	@ClientString(id = 13104, message = "Bird in a Cage")
	public static NpcStringId BIRD_IN_A_CAGE2;
	
	@ClientString(id = 13201, message = "Curiosity of a Matras")
	public static NpcStringId CURIOSITY_OF_A_MATRAS;
	
	@ClientString(id = 13202, message = "Curiosity of a Matras (In Progress)")
	public static NpcStringId CURIOSITY_OF_A_MATRAS_IN_PROGRESS;
	
	@ClientString(id = 13203, message = "Curiosity of a Matras (Done)")
	public static NpcStringId CURIOSITY_OF_A_MATRAS_DONE;
	
	@ClientString(id = 13204, message = "Curiosity of a Matras")
	public static NpcStringId CURIOSITY_OF_A_MATRAS2;
	
	@ClientString(id = 13301, message = "That's Bloody Hot!")
	public static NpcStringId THAT_S_BLOODY_HOT;
	
	@ClientString(id = 13302, message = "That's Bloody Hot! (In Progress)")
	public static NpcStringId THAT_S_BLOODY_HOT_IN_PROGRESS;
	
	@ClientString(id = 13303, message = "That's Bloody Hot! (Done)")
	public static NpcStringId THAT_S_BLOODY_HOT_DONE;
	
	@ClientString(id = 13304, message = "That's Bloody Hot!")
	public static NpcStringId THAT_S_BLOODY_HOT2;
	
	@ClientString(id = 13401, message = "Temple Missionary (Lv. 35-99)")
	public static NpcStringId TEMPLE_MISSIONARY_LV_35_99;
	
	@ClientString(id = 13402, message = "Temple Missionary (Lv. 35-99) (In Progress)")
	public static NpcStringId TEMPLE_MISSIONARY_LV_35_99_IN_PROGRESS;
	
	@ClientString(id = 13403, message = "Temple Missionary (Lv. 35-99) (Done)")
	public static NpcStringId TEMPLE_MISSIONARY_LV_35_99_DONE;
	
	@ClientString(id = 13404, message = "Temple Missionary (Lv. 35-99)")
	public static NpcStringId TEMPLE_MISSIONARY_LV_35_992;
	
	@ClientString(id = 13501, message = "Temple Executor (Lv. 35-99)")
	public static NpcStringId TEMPLE_EXECUTOR_LV_35_99;
	
	@ClientString(id = 13502, message = "Temple Executor (Lv. 35-99) (In Progress)")
	public static NpcStringId TEMPLE_EXECUTOR_LV_35_99_IN_PROGRESS;
	
	@ClientString(id = 13503, message = "Temple Executor (Lv. 35-99) (Done)")
	public static NpcStringId TEMPLE_EXECUTOR_LV_35_99_DONE;
	
	@ClientString(id = 13504, message = "Temple Executor (Lv. 35-99)")
	public static NpcStringId TEMPLE_EXECUTOR_LV_35_992;
	
	@ClientString(id = 13601, message = "More Than Meets the Eye (Lv. 50-99)")
	public static NpcStringId MORE_THAN_MEETS_THE_EYE_LV_50_99;
	
	@ClientString(id = 13602, message = "More Than Meets the Eye (Lv. 50-99) (In Progress)")
	public static NpcStringId MORE_THAN_MEETS_THE_EYE_LV_50_99_IN_PROGRESS;
	
	@ClientString(id = 13603, message = "More Than Meets the Eye (Lv. 50-99) (Done)")
	public static NpcStringId MORE_THAN_MEETS_THE_EYE_LV_50_99_DONE;
	
	@ClientString(id = 13604, message = "More Than Meets the Eye (Lv. 50-99)")
	public static NpcStringId MORE_THAN_MEETS_THE_EYE_LV_50_992;
	
	@ClientString(id = 13701, message = "Temple Champion - 1 (Lv. 35-99)")
	public static NpcStringId TEMPLE_CHAMPION_1_LV_35_99;
	
	@ClientString(id = 13702, message = "Temple Champion - 1 (Lv. 35-99) (In Progress)")
	public static NpcStringId TEMPLE_CHAMPION_1_LV_35_99_IN_PROGRESS;
	
	@ClientString(id = 13703, message = "Temple Champion - 1 (Lv. 35-99) (Done)")
	public static NpcStringId TEMPLE_CHAMPION_1_LV_35_99_DONE;
	
	@ClientString(id = 13704, message = "Temple Champion - 1 (Lv. 35-99)")
	public static NpcStringId TEMPLE_CHAMPION_1_LV_35_992;
	
	@ClientString(id = 13801, message = "Temple Champion - 2 (Lv. 36-99)")
	public static NpcStringId TEMPLE_CHAMPION_2_LV_36_99;
	
	@ClientString(id = 13802, message = "Temple Champion - 2 (Lv. 36-99) (In Progress)")
	public static NpcStringId TEMPLE_CHAMPION_2_LV_36_99_IN_PROGRESS;
	
	@ClientString(id = 13803, message = "Temple Champion - 2 (Lv. 36-99) (Done)")
	public static NpcStringId TEMPLE_CHAMPION_2_LV_36_99_DONE;
	
	@ClientString(id = 13804, message = "Temple Champion - 2 (Lv. 36-99)")
	public static NpcStringId TEMPLE_CHAMPION_2_LV_36_992;
	
	@ClientString(id = 13901, message = "Shadow Fox - 1 (Lv. 37-99)")
	public static NpcStringId SHADOW_FOX_1_LV_37_99;
	
	@ClientString(id = 13902, message = "Shadow Fox - 1 (Lv. 37-99) (In Progress)")
	public static NpcStringId SHADOW_FOX_1_LV_37_99_IN_PROGRESS;
	
	@ClientString(id = 13903, message = "Shadow Fox - 1 (Lv. 37-99) (Done)")
	public static NpcStringId SHADOW_FOX_1_LV_37_99_DONE;
	
	@ClientString(id = 13904, message = "Shadow Fox - 1 (Lv. 37-99)")
	public static NpcStringId SHADOW_FOX_1_LV_37_992;
	
	@ClientString(id = 14001, message = "Shadow Fox - 2 (Lv. 37-99)")
	public static NpcStringId SHADOW_FOX_2_LV_37_99;
	
	@ClientString(id = 14002, message = "Shadow Fox - 2 (Lv. 37-99) (In Progress)")
	public static NpcStringId SHADOW_FOX_2_LV_37_99_IN_PROGRESS;
	
	@ClientString(id = 14003, message = "Shadow Fox - 2 (Lv. 37-99) (Done)")
	public static NpcStringId SHADOW_FOX_2_LV_37_99_DONE;
	
	@ClientString(id = 14004, message = "Shadow Fox - 2 (Lv. 37-99)")
	public static NpcStringId SHADOW_FOX_2_LV_37_992;
	
	@ClientString(id = 14101, message = "Shadow Fox - 3 (Lv. 37-99)")
	public static NpcStringId SHADOW_FOX_3_LV_37_99;
	
	@ClientString(id = 14102, message = "Shadow Fox - 3 (Lv. 37-99) (In Progress)")
	public static NpcStringId SHADOW_FOX_3_LV_37_99_IN_PROGRESS;
	
	@ClientString(id = 14103, message = "Shadow Fox - 3 (Lv. 37-99) (Done)")
	public static NpcStringId SHADOW_FOX_3_LV_37_99_DONE;
	
	@ClientString(id = 14104, message = "Shadow Fox - 3 (Lv. 37-99)")
	public static NpcStringId SHADOW_FOX_3_LV_37_992;
	
	@ClientString(id = 14201, message = "Fallen Angel - Request of Dawn (Lv. 38-99)")
	public static NpcStringId FALLEN_ANGEL_REQUEST_OF_DAWN_LV_38_99;
	
	@ClientString(id = 14202, message = "Fallen Angel - Request of Dawn (Lv. 38-99) (In Progress)")
	public static NpcStringId FALLEN_ANGEL_REQUEST_OF_DAWN_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 14203, message = "Fallen Angel - Request of Dawn (Lv. 38-99) (Done)")
	public static NpcStringId FALLEN_ANGEL_REQUEST_OF_DAWN_LV_38_99_DONE;
	
	@ClientString(id = 14204, message = "Fallen Angel - Request of Dawn (Lv. 38-99)")
	public static NpcStringId FALLEN_ANGEL_REQUEST_OF_DAWN_LV_38_992;
	
	@ClientString(id = 14205, message = "Fallen Angel - Select")
	public static NpcStringId FALLEN_ANGEL_SELECT;
	
	@ClientString(id = 14301, message = "Fallen Angel - Request of Dusk (Lv. 38-99)")
	public static NpcStringId FALLEN_ANGEL_REQUEST_OF_DUSK_LV_38_99;
	
	@ClientString(id = 14302, message = "Fallen Angel - Request of Dusk (Lv. 38-99) (In Progress)")
	public static NpcStringId FALLEN_ANGEL_REQUEST_OF_DUSK_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 14303, message = "Fallen Angel - Request of Dusk (Lv. 38-99) (Done)")
	public static NpcStringId FALLEN_ANGEL_REQUEST_OF_DUSK_LV_38_99_DONE;
	
	@ClientString(id = 14401, message = "Pailaka - Injured Dragon (Lv. 73-77)")
	public static NpcStringId PAILAKA_INJURED_DRAGON_LV_73_77;
	
	@ClientString(id = 14402, message = "Pailaka - Injured Dragon (Lv. 73-77) (In Progress)")
	public static NpcStringId PAILAKA_INJURED_DRAGON_LV_73_77_IN_PROGRESS;
	
	@ClientString(id = 14403, message = "Pailaka - Injured Dragon (Lv. 73-77) (done)")
	public static NpcStringId PAILAKA_INJURED_DRAGON_LV_73_77_DONE;
	
	@ClientString(id = 14701, message = "Path to Becoming an Elite Mercenary")
	public static NpcStringId PATH_TO_BECOMING_AN_ELITE_MERCENARY;
	
	@ClientString(id = 14702, message = "Path to Becoming an Elite Mercenary (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_AN_ELITE_MERCENARY_IN_PROGRESS;
	
	@ClientString(id = 14703, message = "Path to Becoming an Elite Mercenary (Done)")
	public static NpcStringId PATH_TO_BECOMING_AN_ELITE_MERCENARY_DONE;
	
	@ClientString(id = 14801, message = "Path to Becoming an Exalted Mercenary")
	public static NpcStringId PATH_TO_BECOMING_AN_EXALTED_MERCENARY;
	
	@ClientString(id = 14802, message = "Path to Becoming an Exalted Mercenary (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_AN_EXALTED_MERCENARY_IN_PROGRESS;
	
	@ClientString(id = 14803, message = "Path to Becoming an Exalted Mercenary (Done)")
	public static NpcStringId PATH_TO_BECOMING_AN_EXALTED_MERCENARY_DONE;
	
	@ClientString(id = 14901, message = "Primal Mother, Istina (Lv. 90-99)")
	public static NpcStringId PRIMAL_MOTHER_ISTINA_LV_90_99;
	
	@ClientString(id = 14902, message = "Primal Mother, Istina (Lv. 90-99) (In Progress)")
	public static NpcStringId PRIMAL_MOTHER_ISTINA_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 14903, message = "Primal Mother, Istina (Lv. 90-99) (Done)")
	public static NpcStringId PRIMAL_MOTHER_ISTINA_LV_90_99_DONE;
	
	@ClientString(id = 14904, message = "Primal Mother, Istina (Lv. 90-99)")
	public static NpcStringId PRIMAL_MOTHER_ISTINA_LV_90_992;
	
	@ClientString(id = 15001, message = "Extreme Challenge: Primal Mother Resurrected (Lv. 97-99)")
	public static NpcStringId EXTREME_CHALLENGE_PRIMAL_MOTHER_RESURRECTED_LV_97_99;
	
	@ClientString(id = 15002, message = "Extreme Challenge: Primal Mother Resurrected (Lv. 97-99) (In Progress)")
	public static NpcStringId EXTREME_CHALLENGE_PRIMAL_MOTHER_RESURRECTED_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 15003, message = "Extreme Challenge: Primal Mother Resurrected (Lv. 97-99) (Done)")
	public static NpcStringId EXTREME_CHALLENGE_PRIMAL_MOTHER_RESURRECTED_LV_97_99_DONE;
	
	@ClientString(id = 15004, message = "Extreme Challenge: Primal Mother Resurrected (Lv. 97-99)")
	public static NpcStringId EXTREME_CHALLENGE_PRIMAL_MOTHER_RESURRECTED_LV_97_992;
	
	@ClientString(id = 15101, message = "Cure for Fever")
	public static NpcStringId CURE_FOR_FEVER;
	
	@ClientString(id = 15102, message = "Cure for Fever (In Progress)")
	public static NpcStringId CURE_FOR_FEVER_IN_PROGRESS;
	
	@ClientString(id = 15103, message = "Cure for Fever (Done)")
	public static NpcStringId CURE_FOR_FEVER_DONE;
	
	@ClientString(id = 15201, message = "Shards of Golem")
	public static NpcStringId SHARDS_OF_GOLEM;
	
	@ClientString(id = 15202, message = "Shards of Golem (In Progress)")
	public static NpcStringId SHARDS_OF_GOLEM_IN_PROGRESS;
	
	@ClientString(id = 15203, message = "Shards of Golem (Done)")
	public static NpcStringId SHARDS_OF_GOLEM_DONE;
	
	@ClientString(id = 15301, message = "Deliver Goods")
	public static NpcStringId DELIVER_GOODS;
	
	@ClientString(id = 15302, message = "Deliver Goods (In Progress)")
	public static NpcStringId DELIVER_GOODS_IN_PROGRESS;
	
	@ClientString(id = 15303, message = "Deliver Goods (Done)")
	public static NpcStringId DELIVER_GOODS_DONE;
	
	@ClientString(id = 15401, message = "Sacrifice to the Sea")
	public static NpcStringId SACRIFICE_TO_THE_SEA;
	
	@ClientString(id = 15402, message = "Sacrifice to the Sea (In Progress)")
	public static NpcStringId SACRIFICE_TO_THE_SEA_IN_PROGRESS;
	
	@ClientString(id = 15403, message = "Sacrifice to the Sea (Done)")
	public static NpcStringId SACRIFICE_TO_THE_SEA_DONE;
	
	@ClientString(id = 15501, message = "Find Sir Windawood")
	public static NpcStringId FIND_SIR_WINDAWOOD;
	
	@ClientString(id = 15502, message = "Find Sir Windawood (In Progress)")
	public static NpcStringId FIND_SIR_WINDAWOOD_IN_PROGRESS;
	
	@ClientString(id = 15503, message = "Find Sir Windawood (Done)")
	public static NpcStringId FIND_SIR_WINDAWOOD_DONE;
	
	@ClientString(id = 15601, message = "Millennium Love")
	public static NpcStringId MILLENNIUM_LOVE;
	
	@ClientString(id = 15602, message = "Millennium Love (In Progress)")
	public static NpcStringId MILLENNIUM_LOVE_IN_PROGRESS;
	
	@ClientString(id = 15603, message = "Millennium Love (Done)")
	public static NpcStringId MILLENNIUM_LOVE_DONE;
	
	@ClientString(id = 15701, message = "Recover Smuggled Goods")
	public static NpcStringId RECOVER_SMUGGLED_GOODS;
	
	@ClientString(id = 15702, message = "Recover Smuggled Goods (In Progress)")
	public static NpcStringId RECOVER_SMUGGLED_GOODS_IN_PROGRESS;
	
	@ClientString(id = 15703, message = "Recover Smuggled Goods (Done)")
	public static NpcStringId RECOVER_SMUGGLED_GOODS_DONE;
	
	@ClientString(id = 15801, message = "Seed of Evil")
	public static NpcStringId SEED_OF_EVIL;
	
	@ClientString(id = 15802, message = "Seed of Evil (In Progress)")
	public static NpcStringId SEED_OF_EVIL_IN_PROGRESS;
	
	@ClientString(id = 15803, message = "Seed of Evil (Done)")
	public static NpcStringId SEED_OF_EVIL_DONE;
	
	@ClientString(id = 15804, message = "... How dare you challenge me!")
	public static NpcStringId HOW_DARE_YOU_CHALLENGE_ME;
	
	@ClientString(id = 15805, message = "The power of Lord Beleth rules the whole world...!")
	public static NpcStringId THE_POWER_OF_LORD_BELETH_RULES_THE_WHOLE_WORLD;
	
	@ClientString(id = 15901, message = "Protect the Water Source")
	public static NpcStringId PROTECT_THE_WATER_SOURCE;
	
	@ClientString(id = 15902, message = "Protect the Water Source (In Progress)")
	public static NpcStringId PROTECT_THE_WATER_SOURCE_IN_PROGRESS;
	
	@ClientString(id = 15903, message = "Protect the Water Source (Done)")
	public static NpcStringId PROTECT_THE_WATER_SOURCE_DONE;
	
	@ClientString(id = 16001, message = "Nerupa's Request")
	public static NpcStringId NERUPA_S_REQUEST;
	
	@ClientString(id = 16002, message = "Nerupa's Request (In Progress)")
	public static NpcStringId NERUPA_S_REQUEST_IN_PROGRESS;
	
	@ClientString(id = 16003, message = "Nerupa's Request (Done)")
	public static NpcStringId NERUPA_S_REQUEST_DONE;
	
	@ClientString(id = 16101, message = "Fruit of the Mother Tree")
	public static NpcStringId FRUIT_OF_THE_MOTHER_TREE;
	
	@ClientString(id = 16102, message = "Fruit of the Mother Tree (In Progress)")
	public static NpcStringId FRUIT_OF_THE_MOTHER_TREE_IN_PROGRESS;
	
	@ClientString(id = 16103, message = "Fruit of the Mother Tree (Done)")
	public static NpcStringId FRUIT_OF_THE_MOTHER_TREE_DONE;
	
	@ClientString(id = 16201, message = "Curse of the Fortress")
	public static NpcStringId CURSE_OF_THE_FORTRESS;
	
	@ClientString(id = 16202, message = "Curse of the Fortress (In Progress)")
	public static NpcStringId CURSE_OF_THE_FORTRESS_IN_PROGRESS;
	
	@ClientString(id = 16203, message = "Curse of the Fortress (Done)")
	public static NpcStringId CURSE_OF_THE_FORTRESS_DONE;
	
	@ClientString(id = 16301, message = "Legacy of the Poet")
	public static NpcStringId LEGACY_OF_THE_POET;
	
	@ClientString(id = 16302, message = "Legacy of the Poet (In Progress)")
	public static NpcStringId LEGACY_OF_THE_POET_IN_PROGRESS;
	
	@ClientString(id = 16303, message = "Legacy of the Poet (Done)")
	public static NpcStringId LEGACY_OF_THE_POET_DONE;
	
	@ClientString(id = 16401, message = "Blood Fiend")
	public static NpcStringId BLOOD_FIEND;
	
	@ClientString(id = 16402, message = "Blood Fiend (In Progress)")
	public static NpcStringId BLOOD_FIEND_IN_PROGRESS;
	
	@ClientString(id = 16403, message = "Blood Fiend (Done)")
	public static NpcStringId BLOOD_FIEND_DONE;
	
	@ClientString(id = 16404, message = "I will taste your blood!")
	public static NpcStringId I_WILL_TASTE_YOUR_BLOOD;
	
	@ClientString(id = 16405, message = "I have fulfilled my contract with Trader Creamees.")
	public static NpcStringId I_HAVE_FULFILLED_MY_CONTRACT_WITH_TRADER_CREAMEES;
	
	@ClientString(id = 16501, message = "Shilen's Hunt")
	public static NpcStringId SHILEN_S_HUNT;
	
	@ClientString(id = 16502, message = "Shilen's Hunt (In Progress)")
	public static NpcStringId SHILEN_S_HUNT_IN_PROGRESS;
	
	@ClientString(id = 16503, message = "Shilen's Hunt (Done)")
	public static NpcStringId SHILEN_S_HUNT_DONE;
	
	@ClientString(id = 16601, message = "Mass of Darkness")
	public static NpcStringId MASS_OF_DARKNESS;
	
	@ClientString(id = 16602, message = "Mass of Darkness (In Progress)")
	public static NpcStringId MASS_OF_DARKNESS_IN_PROGRESS;
	
	@ClientString(id = 16603, message = "Mass of Darkness (Done)")
	public static NpcStringId MASS_OF_DARKNESS_DONE;
	
	@ClientString(id = 16701, message = "Dwarven Kinship")
	public static NpcStringId DWARVEN_KINSHIP;
	
	@ClientString(id = 16702, message = "Dwarven Kinship (In Progress)")
	public static NpcStringId DWARVEN_KINSHIP_IN_PROGRESS;
	
	@ClientString(id = 16703, message = "Dwarven Kinship (Done)")
	public static NpcStringId DWARVEN_KINSHIP_DONE;
	
	@ClientString(id = 16801, message = "Deliver Supplies")
	public static NpcStringId DELIVER_SUPPLIES;
	
	@ClientString(id = 16802, message = "Deliver Supplies (In Progress)")
	public static NpcStringId DELIVER_SUPPLIES_IN_PROGRESS;
	
	@ClientString(id = 16803, message = "Deliver Supplies (Done)")
	public static NpcStringId DELIVER_SUPPLIES_DONE;
	
	@ClientString(id = 16901, message = "Offspring of Nightmares")
	public static NpcStringId OFFSPRING_OF_NIGHTMARES;
	
	@ClientString(id = 16902, message = "Offspring of Nightmares (In Progress)")
	public static NpcStringId OFFSPRING_OF_NIGHTMARES_IN_PROGRESS;
	
	@ClientString(id = 16903, message = "Offspring of Nightmares (Done)")
	public static NpcStringId OFFSPRING_OF_NIGHTMARES_DONE;
	
	@ClientString(id = 17001, message = "Dangerous Seduction")
	public static NpcStringId DANGEROUS_SEDUCTION;
	
	@ClientString(id = 17002, message = "Dangerous Seduction (In Progress)")
	public static NpcStringId DANGEROUS_SEDUCTION_IN_PROGRESS;
	
	@ClientString(id = 17003, message = "Dangerous Seduction (Done)")
	public static NpcStringId DANGEROUS_SEDUCTION_DONE;
	
	@ClientString(id = 17004, message = "I'll cast you into an eternal nightmare!")
	public static NpcStringId I_LL_CAST_YOU_INTO_AN_ETERNAL_NIGHTMARE;
	
	@ClientString(id = 17005, message = "Send my soul to Lich King Icarus...")
	public static NpcStringId SEND_MY_SOUL_TO_LICH_KING_ICARUS;
	
	@ClientString(id = 17101, message = "Acts of Evil")
	public static NpcStringId ACTS_OF_EVIL;
	
	@ClientString(id = 17102, message = "Acts of Evil (In Progress)")
	public static NpcStringId ACTS_OF_EVIL_IN_PROGRESS;
	
	@ClientString(id = 17103, message = "Acts of Evil (Done)")
	public static NpcStringId ACTS_OF_EVIL_DONE;
	
	@ClientString(id = 17104, message = "Acts of Evil")
	public static NpcStringId ACTS_OF_EVIL2;
	
	@ClientString(id = 17151, message = "You should consider going back...")
	public static NpcStringId YOU_SHOULD_CONSIDER_GOING_BACK;
	
	@ClientString(id = 17201, message = "New Horizons")
	public static NpcStringId NEW_HORIZONS;
	
	@ClientString(id = 17202, message = "New Horizons (In Progress)")
	public static NpcStringId NEW_HORIZONS_IN_PROGRESS;
	
	@ClientString(id = 17203, message = "New Horizons (Done)")
	public static NpcStringId NEW_HORIZONS_DONE;
	
	@ClientString(id = 17301, message = "To the Isle of Souls")
	public static NpcStringId TO_THE_ISLE_OF_SOULS;
	
	@ClientString(id = 17302, message = "To the Isle of Souls (In Progress)")
	public static NpcStringId TO_THE_ISLE_OF_SOULS_IN_PROGRESS;
	
	@ClientString(id = 17303, message = "To the Isle of Souls (Done)")
	public static NpcStringId TO_THE_ISLE_OF_SOULS_DONE;
	
	@ClientString(id = 17401, message = "Supply Check")
	public static NpcStringId SUPPLY_CHECK;
	
	@ClientString(id = 17402, message = "Supply Check (In Progress)")
	public static NpcStringId SUPPLY_CHECK_IN_PROGRESS;
	
	@ClientString(id = 17403, message = "Supply Check (Done)")
	public static NpcStringId SUPPLY_CHECK_DONE;
	
	@ClientString(id = 17501, message = "The Way of the Warrior")
	public static NpcStringId THE_WAY_OF_THE_WARRIOR;
	
	@ClientString(id = 17502, message = "The Way of the Warrior (In Progress)")
	public static NpcStringId THE_WAY_OF_THE_WARRIOR_IN_PROGRESS;
	
	@ClientString(id = 17503, message = "The Way of the Warrior (Done)")
	public static NpcStringId THE_WAY_OF_THE_WARRIOR_DONE;
	
	@ClientString(id = 17601, message = "Steps for Honor")
	public static NpcStringId STEPS_FOR_HONOR;
	
	@ClientString(id = 17602, message = "Steps for Honor (In Progress)")
	public static NpcStringId STEPS_FOR_HONOR_IN_PROGRESS;
	
	@ClientString(id = 17603, message = "Steps for Honor (Done)")
	public static NpcStringId STEPS_FOR_HONOR_DONE;
	
	@ClientString(id = 17604, message = "Steps for Honor")
	public static NpcStringId STEPS_FOR_HONOR2;
	
	@ClientString(id = 17701, message = "Split Destiny (Lv. 80-99)")
	public static NpcStringId SPLIT_DESTINY_LV_80_99;
	
	@ClientString(id = 17702, message = "Split Destiny (Lv. 80-99) (In Progress)")
	public static NpcStringId SPLIT_DESTINY_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 17703, message = "Split Destiny (Lv. 80-99) (Done)")
	public static NpcStringId SPLIT_DESTINY_LV_80_99_DONE;
	
	@ClientString(id = 17704, message = "Split Destiny (Lv. 80-99)")
	public static NpcStringId SPLIT_DESTINY_LV_80_992;
	
	@ClientString(id = 17801, message = "Iconic Trinity")
	public static NpcStringId ICONIC_TRINITY;
	
	@ClientString(id = 17802, message = "Iconic Trinity (In Progress)")
	public static NpcStringId ICONIC_TRINITY_IN_PROGRESS;
	
	@ClientString(id = 17803, message = "Iconic Trinity (Done)")
	public static NpcStringId ICONIC_TRINITY_DONE;
	
	@ClientString(id = 17901, message = "Into the Large Cavern")
	public static NpcStringId INTO_THE_LARGE_CAVERN;
	
	@ClientString(id = 17902, message = "Into the Large Cavern (In Progress)")
	public static NpcStringId INTO_THE_LARGE_CAVERN_IN_PROGRESS;
	
	@ClientString(id = 17903, message = "Into the Large Cavern (Done)")
	public static NpcStringId INTO_THE_LARGE_CAVERN_DONE;
	
	@ClientString(id = 17951, message = "The Veiled Creator...")
	public static NpcStringId THE_VEILED_CREATOR;
	
	@ClientString(id = 17952, message = "The Conspiracy of the Ancient Race")
	public static NpcStringId THE_CONSPIRACY_OF_THE_ANCIENT_RACE;
	
	@ClientString(id = 17953, message = "Chaos and Time...")
	public static NpcStringId CHAOS_AND_TIME;
	
	@ClientString(id = 18001, message = "Infernal Flames Burning in Crystal Prison (Lv. 97-99)")
	public static NpcStringId INFERNAL_FLAMES_BURNING_IN_CRYSTAL_PRISON_LV_97_99;
	
	@ClientString(id = 18002, message = "Infernal Flames Burning in Crystal Prison (Lv. 97-99) (In Progress)")
	public static NpcStringId INFERNAL_FLAMES_BURNING_IN_CRYSTAL_PRISON_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 18003, message = "Infernal Flames Burning in Crystal Prison (Lv. 97-99) (Done)")
	public static NpcStringId INFERNAL_FLAMES_BURNING_IN_CRYSTAL_PRISON_LV_97_99_DONE;
	
	@ClientString(id = 18004, message = "Infernal Flames Burning in Crystal Prison (Lv. 97-99)")
	public static NpcStringId INFERNAL_FLAMES_BURNING_IN_CRYSTAL_PRISON_LV_97_992;
	
	@ClientString(id = 18101, message = "Devils Strike Back, Advent of Balok (Lv. 97-99)")
	public static NpcStringId DEVILS_STRIKE_BACK_ADVENT_OF_BALOK_LV_97_99;
	
	@ClientString(id = 18102, message = "Devils Strike Back, Advent of Balok (Lv. 97-99) (In Progress)")
	public static NpcStringId DEVILS_STRIKE_BACK_ADVENT_OF_BALOK_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 18103, message = "Devils Strike Back, Advent of Balok (Lv. 97-99) (Done)")
	public static NpcStringId DEVILS_STRIKE_BACK_ADVENT_OF_BALOK_LV_97_99_DONE;
	
	@ClientString(id = 18104, message = "Devils Strike Back, Advent of Balok (Lv. 97-99)")
	public static NpcStringId DEVILS_STRIKE_BACK_ADVENT_OF_BALOK_LV_97_992;
	
	@ClientString(id = 18201, message = "New Recruits")
	public static NpcStringId NEW_RECRUITS;
	
	@ClientString(id = 18202, message = "New Recruits (In Progress)")
	public static NpcStringId NEW_RECRUITS_IN_PROGRESS;
	
	@ClientString(id = 18203, message = "New Recruits (Done)")
	public static NpcStringId NEW_RECRUITS_DONE;
	
	@ClientString(id = 18301, message = "Relics Exploration (Lv. 40-99)")
	public static NpcStringId RELICS_EXPLORATION_LV_40_99;
	
	@ClientString(id = 18302, message = "Relics Exploration (Lv. 40-99) (In Progress)")
	public static NpcStringId RELICS_EXPLORATION_LV_40_99_IN_PROGRESS;
	
	@ClientString(id = 18303, message = "Relics Exploration (Lv. 40-99) (Done)")
	public static NpcStringId RELICS_EXPLORATION_LV_40_99_DONE;
	
	@ClientString(id = 18304, message = "Relics Exploration (Lv. 40-99)")
	public static NpcStringId RELICS_EXPLORATION_LV_40_992;
	
	@ClientString(id = 18401, message = "Art of Persuasion (Lv. 40-99)")
	public static NpcStringId ART_OF_PERSUASION_LV_40_99;
	
	@ClientString(id = 18402, message = "Art of Persuasion (Lv. 40-99) (In Progress)")
	public static NpcStringId ART_OF_PERSUASION_LV_40_99_IN_PROGRESS;
	
	@ClientString(id = 18403, message = "Art of Persuasion (Lv. 40-99) (Done)")
	public static NpcStringId ART_OF_PERSUASION_LV_40_99_DONE;
	
	@ClientString(id = 18404, message = "Art of Persuasion (Lv. 40-99)")
	public static NpcStringId ART_OF_PERSUASION_LV_40_992;
	
	@ClientString(id = 18451, message = "Intruder Alert! The alarm will self-destruct in 2 minutes.")
	public static NpcStringId INTRUDER_ALERT_THE_ALARM_WILL_SELF_DESTRUCT_IN_2_MINUTES;
	
	@ClientString(id = 18452, message = "The alarm will self-destruct in 60 seconds. Enter passcode to override.")
	public static NpcStringId THE_ALARM_WILL_SELF_DESTRUCT_IN_60_SECONDS_ENTER_PASSCODE_TO_OVERRIDE;
	
	@ClientString(id = 18453, message = "The alarm will self-destruct in 30 seconds. Enter passcode to override.")
	public static NpcStringId THE_ALARM_WILL_SELF_DESTRUCT_IN_30_SECONDS_ENTER_PASSCODE_TO_OVERRIDE;
	
	@ClientString(id = 18454, message = "The alarm will self-destruct in 10 seconds. Enter passcode to override.")
	public static NpcStringId THE_ALARM_WILL_SELF_DESTRUCT_IN_10_SECONDS_ENTER_PASSCODE_TO_OVERRIDE;
	
	@ClientString(id = 18455, message = "Recorder crushed.")
	public static NpcStringId RECORDER_CRUSHED;
	
	@ClientString(id = 18501, message = "Nikola's Cooperation (Lv. 40-99)")
	public static NpcStringId NIKOLA_S_COOPERATION_LV_40_99;
	
	@ClientString(id = 18502, message = "Nikola's Cooperation (Lv. 40-99) (In Progress)")
	public static NpcStringId NIKOLA_S_COOPERATION_LV_40_99_IN_PROGRESS;
	
	@ClientString(id = 18503, message = "Nikola's Cooperation (Lv. 40-99) (Done)")
	public static NpcStringId NIKOLA_S_COOPERATION_LV_40_99_DONE;
	
	@ClientString(id = 18504, message = "Nikola's Cooperation (Lv. 40-99)")
	public static NpcStringId NIKOLA_S_COOPERATION_LV_40_992;
	
	@ClientString(id = 18551, message = "Intruder Alert! The alarm will self-destruct in 2 minutes.")
	public static NpcStringId INTRUDER_ALERT_THE_ALARM_WILL_SELF_DESTRUCT_IN_2_MINUTES2;
	
	@ClientString(id = 18552, message = "The alarm will self-destruct in 60 seconds. Please evacuate immediately!")
	public static NpcStringId THE_ALARM_WILL_SELF_DESTRUCT_IN_60_SECONDS_PLEASE_EVACUATE_IMMEDIATELY;
	
	@ClientString(id = 18553, message = "The alarm will self-destruct in 30 seconds. Please evacuate immediately!")
	public static NpcStringId THE_ALARM_WILL_SELF_DESTRUCT_IN_30_SECONDS_PLEASE_EVACUATE_IMMEDIATELY;
	
	@ClientString(id = 18554, message = "The alarm will self-destruct in 10 seconds. Please evacuate immediately!")
	public static NpcStringId THE_ALARM_WILL_SELF_DESTRUCT_IN_10_SECONDS_PLEASE_EVACUATE_IMMEDIATELY;
	
	@ClientString(id = 18555, message = "So bored… is there no one I can play with?")
	public static NpcStringId SO_BORED_IS_THERE_NO_ONE_I_CAN_PLAY_WITH;
	
	@ClientString(id = 18556, message = "Hey, I didn't get to put in one word! Sheesh!")
	public static NpcStringId HEY_I_DIDN_T_GET_TO_PUT_IN_ONE_WORD_SHEESH;
	
	@ClientString(id = 18557, message = "Hmm…")
	public static NpcStringId HMM;
	
	@ClientString(id = 18558, message = "Subquests have been renovated! Check them out and see if you're eligible for rewards!")
	public static NpcStringId SUBQUESTS_HAVE_BEEN_RENOVATED_CHECK_THEM_OUT_AND_SEE_IF_YOU_RE_ELIGIBLE_FOR_REWARDS;
	
	@ClientString(id = 18559, message = "Dr. Chaos invades! Isle of Souls is in danger!")
	public static NpcStringId DR_CHAOS_INVADES_ISLE_OF_SOULS_IS_IN_DANGER;
	
	@ClientString(id = 18560, message = "Spicula Zero emerges! What is happening in Nornil's Garden?")
	public static NpcStringId SPICULA_ZERO_EMERGES_WHAT_IS_HAPPENING_IN_NORNIL_S_GARDEN;
	
	@ClientString(id = 18561, message = "A sudden outpour of Spiculas! Keep an eye out for Nornil's Cave!")
	public static NpcStringId A_SUDDEN_OUTPOUR_OF_SPICULAS_KEEP_AN_EYE_OUT_FOR_NORNIL_S_CAVE;
	
	@ClientString(id = 18562, message = "Check out our latest updates!")
	public static NpcStringId CHECK_OUT_OUR_LATEST_UPDATES;
	
	@ClientString(id = 18563, message = "Brightest Light? How dare you desecrate the Altar of Shilen!")
	public static NpcStringId BRIGHTEST_LIGHT_HOW_DARE_YOU_DESECRATE_THE_ALTAR_OF_SHILEN;
	
	@ClientString(id = 18564, message = "You must defeat Shilen's Messenger.")
	public static NpcStringId YOU_MUST_DEFEAT_SHILEN_S_MESSENGER;
	
	@ClientString(id = 18565, message = "Did you see the Ertheia? They are such an enigma!")
	public static NpcStringId DID_YOU_SEE_THE_ERTHEIA_THEY_ARE_SUCH_AN_ENIGMA;
	
	@ClientString(id = 18566, message = "Meet the Ertheia! They are an otherwordly people that came through the dimensions!")
	public static NpcStringId MEET_THE_ERTHEIA_THEY_ARE_AN_OTHERWORDLY_PEOPLE_THAT_CAME_THROUGH_THE_DIMENSIONS;
	
	@ClientString(id = 18567, message = "Go on an adventure! Sing with the winds with the Ertheia, children of the wind!")
	public static NpcStringId GO_ON_AN_ADVENTURE_SING_WITH_THE_WINDS_WITH_THE_ERTHEIA_CHILDREN_OF_THE_WIND;
	
	@ClientString(id = 18601, message = "Contract Execution (Lv. 41-99)")
	public static NpcStringId CONTRACT_EXECUTION_LV_41_99;
	
	@ClientString(id = 18602, message = "Contract Execution (Lv. 41-99) (In Progress)")
	public static NpcStringId CONTRACT_EXECUTION_LV_41_99_IN_PROGRESS;
	
	@ClientString(id = 18603, message = "Contract Execution (Lv. 41-99) (Done)")
	public static NpcStringId CONTRACT_EXECUTION_LV_41_99_DONE;
	
	@ClientString(id = 18604, message = "Contract Execution (Lv. 41-99)")
	public static NpcStringId CONTRACT_EXECUTION_LV_41_992;
	
	@ClientString(id = 18701, message = "Nikola's Heart (Lv. 41-99)")
	public static NpcStringId NIKOLA_S_HEART_LV_41_99;
	
	@ClientString(id = 18702, message = "Nikola's Heart (Lv. 41-99) (In Progress)")
	public static NpcStringId NIKOLA_S_HEART_LV_41_99_IN_PROGRESS;
	
	@ClientString(id = 18703, message = "Nikola's Heart (Lv. 41-99) (Done)")
	public static NpcStringId NIKOLA_S_HEART_LV_41_99_DONE;
	
	@ClientString(id = 18704, message = "Nikola's Heart (Lv. 41-99)")
	public static NpcStringId NIKOLA_S_HEART_LV_41_992;
	
	@ClientString(id = 18801, message = "Seal Removal (Lv. 41-99)")
	public static NpcStringId SEAL_REMOVAL_LV_41_99;
	
	@ClientString(id = 18802, message = "Seal Removal (Lv. 41-99) (In Progress)")
	public static NpcStringId SEAL_REMOVAL_LV_41_99_IN_PROGRESS;
	
	@ClientString(id = 18803, message = "Seal Removal (Lv. 41-99) (Done)")
	public static NpcStringId SEAL_REMOVAL_LV_41_99_DONE;
	
	@ClientString(id = 18804, message = "Seal Removal (Lv. 41-99)")
	public static NpcStringId SEAL_REMOVAL_LV_41_992;
	
	@ClientString(id = 18901, message = "Contract Completion (Lv. 42-99)")
	public static NpcStringId CONTRACT_COMPLETION_LV_42_99;
	
	@ClientString(id = 18902, message = "Contract Completion (Lv. 42-99) (In Progress)")
	public static NpcStringId CONTRACT_COMPLETION_LV_42_99_IN_PROGRESS;
	
	@ClientString(id = 18903, message = "Contract Completion (Lv. 42-99) (Done)")
	public static NpcStringId CONTRACT_COMPLETION_LV_42_99_DONE;
	
	@ClientString(id = 18904, message = "Contract Completion (Lv. 42-99)")
	public static NpcStringId CONTRACT_COMPLETION_LV_42_992;
	
	@ClientString(id = 19001, message = "Lost Dream (Lv. 42-99)")
	public static NpcStringId LOST_DREAM_LV_42_99;
	
	@ClientString(id = 19002, message = "Lost Dream (Lv. 42-99) (In Progress)")
	public static NpcStringId LOST_DREAM_LV_42_99_IN_PROGRESS;
	
	@ClientString(id = 19003, message = "Lost Dream (Lv. 42-99) (Done)")
	public static NpcStringId LOST_DREAM_LV_42_99_DONE;
	
	@ClientString(id = 19004, message = "Lost Dream (Lv. 42-99)")
	public static NpcStringId LOST_DREAM_LV_42_992;
	
	@ClientString(id = 19101, message = "Vain Conclusion (Lv. 42-99)")
	public static NpcStringId VAIN_CONCLUSION_LV_42_99;
	
	@ClientString(id = 19102, message = "Vain Conclusion (Lv. 42-99) (In Progress)")
	public static NpcStringId VAIN_CONCLUSION_LV_42_99_IN_PROGRESS;
	
	@ClientString(id = 19103, message = "Vain Conclusion (Lv. 42-99) (Done)")
	public static NpcStringId VAIN_CONCLUSION_LV_42_99_DONE;
	
	@ClientString(id = 19104, message = "Vain Conclusion (Lv. 42-99)")
	public static NpcStringId VAIN_CONCLUSION_LV_42_992;
	
	@ClientString(id = 19201, message = "Seven Signs, Series of Doubt (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_SERIES_OF_DOUBT_LV_79_99;
	
	@ClientString(id = 19202, message = "Seven Signs, Series of Doubt (Lv. 79-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_SERIES_OF_DOUBT_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 19203, message = "Seven Signs, Series of Doubt (Lv. 79-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_SERIES_OF_DOUBT_LV_79_99_DONE;
	
	@ClientString(id = 19204, message = "Seven Signs, Series of Doubt (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_SERIES_OF_DOUBT_LV_79_992;
	
	@ClientString(id = 19301, message = "Seven Signs, Dying Message (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_DYING_MESSAGE_LV_79_99;
	
	@ClientString(id = 19302, message = "Seven Signs, Dying Message (Lv. 79-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_DYING_MESSAGE_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 19303, message = "Seven Signs, Dying Message (Lv. 79-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_DYING_MESSAGE_LV_79_99_DONE;
	
	@ClientString(id = 19304, message = "Seven Signs, Dying Message (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_DYING_MESSAGE_LV_79_992;
	
	@ClientString(id = 19305, message = "Next time, you will not escape!")
	public static NpcStringId NEXT_TIME_YOU_WILL_NOT_ESCAPE;
	
	@ClientString(id = 19306, message = "$s1! You may have won this time... But next time, I will surely capture you!")
	public static NpcStringId S1_YOU_MAY_HAVE_WON_THIS_TIME_BUT_NEXT_TIME_I_WILL_SURELY_CAPTURE_YOU;
	
	@ClientString(id = 19314, message = "$s1! You are not the owner of that item.")
	public static NpcStringId S1_YOU_ARE_NOT_THE_OWNER_OF_THAT_ITEM;
	
	@ClientString(id = 19401, message = "Seven Signs, Mammon's Contract (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_MAMMON_S_CONTRACT_LV_79_99;
	
	@ClientString(id = 19402, message = "Seven Signs, Mammon's Contract (Lv. 79-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_MAMMON_S_CONTRACT_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 19403, message = "Seven Signs, Mammon's Contract (Lv. 79-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_MAMMON_S_CONTRACT_LV_79_99_DONE;
	
	@ClientString(id = 19404, message = "Seven Signs, Mammon's Contract (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_MAMMON_S_CONTRACT_LV_79_992;
	
	@ClientString(id = 19501, message = "Seven Signs, Secret Ritual of the Priests (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_SECRET_RITUAL_OF_THE_PRIESTS_LV_79_99;
	
	@ClientString(id = 19502, message = "Seven Signs, Secret Ritual of the Priests (Lv. 79-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_SECRET_RITUAL_OF_THE_PRIESTS_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 19503, message = "Seven Signs, Secret Ritual of the Priests (Lv. 79-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_SECRET_RITUAL_OF_THE_PRIESTS_LV_79_99_DONE;
	
	@ClientString(id = 19504, message = "Seven Signs, Secret Ritual of the Priests (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_SECRET_RITUAL_OF_THE_PRIESTS_LV_79_992;
	
	@ClientString(id = 19505, message = "Who are you?! A new face like you can't approach this place!")
	public static NpcStringId WHO_ARE_YOU_A_NEW_FACE_LIKE_YOU_CAN_T_APPROACH_THIS_PLACE;
	
	@ClientString(id = 19506, message = "How dare you intrude with that transformation! Get lost!")
	public static NpcStringId HOW_DARE_YOU_INTRUDE_WITH_THAT_TRANSFORMATION_GET_LOST;
	
	@ClientString(id = 19514, message = "Intruder! Protect the Priests of Dawn!")
	public static NpcStringId INTRUDER_PROTECT_THE_PRIESTS_OF_DAWN;
	
	@ClientString(id = 19601, message = "Seven Signs, Seal of the Emperor (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_SEAL_OF_THE_EMPEROR_LV_79_99;
	
	@ClientString(id = 19602, message = "Seven Signs, Seal of the Emperor (Lv. 79-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_SEAL_OF_THE_EMPEROR_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 19603, message = "Seven Signs, Seal of the Emperor (Lv. 79-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_SEAL_OF_THE_EMPEROR_LV_79_99_DONE;
	
	@ClientString(id = 19604, message = "Seven Signs, Seal of the Emperor (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_SEAL_OF_THE_EMPEROR_LV_79_992;
	
	@ClientString(id = 19605, message = "The ancient promise to the Emperor has been fulfilled.")
	public static NpcStringId THE_ANCIENT_PROMISE_TO_THE_EMPEROR_HAS_BEEN_FULFILLED;
	
	@ClientString(id = 19606, message = "For the eternity of Einhasad!!!")
	public static NpcStringId FOR_THE_ETERNITY_OF_EINHASAD;
	
	@ClientString(id = 19607, message = "Dear Shillien's offsprings! You are not capable of confronting us!")
	public static NpcStringId DEAR_SHILLIEN_S_OFFSPRINGS_YOU_ARE_NOT_CAPABLE_OF_CONFRONTING_US;
	
	@ClientString(id = 19608, message = "I'll show you the real power of Einhasad!")
	public static NpcStringId I_LL_SHOW_YOU_THE_REAL_POWER_OF_EINHASAD;
	
	@ClientString(id = 19609, message = "Dear Military Force of Light! Go destroy the offsprings of Shillien!!!")
	public static NpcStringId DEAR_MILITARY_FORCE_OF_LIGHT_GO_DESTROY_THE_OFFSPRINGS_OF_SHILLIEN;
	
	@ClientString(id = 19610, message = "Everything is owing to $s1. Thank you.")
	public static NpcStringId EVERYTHING_IS_OWING_TO_S1_THANK_YOU;
	
	@ClientString(id = 19611, message = "My power's weakening.. Hurry and turn on the sealing device!!!")
	public static NpcStringId MY_POWER_S_WEAKENING_HURRY_AND_TURN_ON_THE_SEALING_DEVICE;
	
	@ClientString(id = 19612, message = "All 4 sealing devices must be turned on!!!")
	public static NpcStringId ALL_4_SEALING_DEVICES_MUST_BE_TURNED_ON;
	
	@ClientString(id = 19613, message = "Lilith' attack is getting stronger! Go ahead and turn it on!")
	public static NpcStringId LILITH_ATTACK_IS_GETTING_STRONGER_GO_AHEAD_AND_TURN_IT_ON;
	
	@ClientString(id = 19614, message = "Dear $s1, give me more strength.")
	public static NpcStringId DEAR_S1_GIVE_ME_MORE_STRENGTH;
	
	@ClientString(id = 19615, message = "You, such a fool! The victory over this war belongs to Shilen!!!")
	public static NpcStringId YOU_SUCH_A_FOOL_THE_VICTORY_OVER_THIS_WAR_BELONGS_TO_SHILEN;
	
	@ClientString(id = 19616, message = "How dare you try to contend against me in strength? Ridiculous.")
	public static NpcStringId HOW_DARE_YOU_TRY_TO_CONTEND_AGAINST_ME_IN_STRENGTH_RIDICULOUS;
	
	@ClientString(id = 19617, message = "Anakim! In the name of Great Shilen, I will cut your throat!")
	public static NpcStringId ANAKIM_IN_THE_NAME_OF_GREAT_SHILEN_I_WILL_CUT_YOUR_THROAT;
	
	@ClientString(id = 19618, message = "You cannot be the match of Lilith. I'll teach you a lesson!")
	public static NpcStringId YOU_CANNOT_BE_THE_MATCH_OF_LILITH_I_LL_TEACH_YOU_A_LESSON;
	
	@ClientString(id = 19619, message = "I must go back to Shilen just like this. Outrageous.")
	public static NpcStringId I_MUST_GO_BACK_TO_SHILEN_JUST_LIKE_THIS_OUTRAGEOUS;
	
	@ClientString(id = 19624, message = "Who dares summon the merchant of mammon…?")
	public static NpcStringId WHO_DARES_SUMMON_THE_MERCHANT_OF_MAMMON;
	
	@ClientString(id = 19701, message = "Seven Signs, the Sacred Book of Seal (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_THE_SACRED_BOOK_OF_SEAL_LV_79_99;
	
	@ClientString(id = 19702, message = "Seven Signs, the Sacred Book of Seal (Lv. 79-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_THE_SACRED_BOOK_OF_SEAL_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 19703, message = "Seven Signs, the Sacred Book of Seal (Lv. 79-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_THE_SACRED_BOOK_OF_SEAL_LV_79_99_DONE;
	
	@ClientString(id = 19704, message = "Seven Signs, the Sacred Book of Seal (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_THE_SACRED_BOOK_OF_SEAL_LV_79_992;
	
	@ClientString(id = 19801, message = "Seven Signs, Embryo (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_EMBRYO_LV_79_99;
	
	@ClientString(id = 19802, message = "Seven Signs, Embryo (Lv. 79-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_EMBRYO_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 19803, message = "Seven Signs, Embryo (Lv. 79-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_EMBRYO_LV_79_99_DONE;
	
	@ClientString(id = 19804, message = "Seven Signs, Embryo (Lv. 79-99)")
	public static NpcStringId SEVEN_SIGNS_EMBRYO_LV_79_992;
	
	@ClientString(id = 19805, message = "We will be with you always...")
	public static NpcStringId WE_WILL_BE_WITH_YOU_ALWAYS;
	
	@ClientString(id = 19806, message = "You are not the owner of that item.")
	public static NpcStringId YOU_ARE_NOT_THE_OWNER_OF_THAT_ITEM;
	
	@ClientString(id = 19807, message = "Embryo...")
	public static NpcStringId EMBRYO;
	
	@ClientString(id = 19814, message = "Death to the enemies of the Lords of Dawn!")
	public static NpcStringId DEATH_TO_THE_ENEMIES_OF_THE_LORDS_OF_DAWN;
	
	@ClientString(id = 20101, message = "Fighter's Tutorial")
	public static NpcStringId FIGHTER_S_TUTORIAL;
	
	@ClientString(id = 20102, message = "Fighter's Tutorial (In Progress)")
	public static NpcStringId FIGHTER_S_TUTORIAL_IN_PROGRESS;
	
	@ClientString(id = 20103, message = "Fighter's Tutorial (Done)")
	public static NpcStringId FIGHTER_S_TUTORIAL_DONE;
	
	@ClientString(id = 20201, message = "Mystic's Tutorial")
	public static NpcStringId MYSTIC_S_TUTORIAL;
	
	@ClientString(id = 20202, message = "Mystic's Tutorial (In Progress)")
	public static NpcStringId MYSTIC_S_TUTORIAL_IN_PROGRESS;
	
	@ClientString(id = 20203, message = "Mystic's Tutorial (Done)")
	public static NpcStringId MYSTIC_S_TUTORIAL_DONE;
	
	@ClientString(id = 20301, message = "Elf's Tutorial")
	public static NpcStringId ELF_S_TUTORIAL;
	
	@ClientString(id = 20302, message = "Elf's Tutorial (In Progress)")
	public static NpcStringId ELF_S_TUTORIAL_IN_PROGRESS;
	
	@ClientString(id = 20303, message = "Elf's Tutorial (Done)")
	public static NpcStringId ELF_S_TUTORIAL_DONE;
	
	@ClientString(id = 20401, message = "Dark Elf's Tutorial")
	public static NpcStringId DARK_ELF_S_TUTORIAL;
	
	@ClientString(id = 20402, message = "Dark Elf's Tutorial (In Progress)")
	public static NpcStringId DARK_ELF_S_TUTORIAL_IN_PROGRESS;
	
	@ClientString(id = 20403, message = "Dark Elf's Tutorial (Done)")
	public static NpcStringId DARK_ELF_S_TUTORIAL_DONE;
	
	@ClientString(id = 20501, message = "Orc's Tutorial")
	public static NpcStringId ORC_S_TUTORIAL;
	
	@ClientString(id = 20502, message = "Orc's Tutorial (In Progress)")
	public static NpcStringId ORC_S_TUTORIAL_IN_PROGRESS;
	
	@ClientString(id = 20503, message = "Orc's Tutorial (Done)")
	public static NpcStringId ORC_S_TUTORIAL_DONE;
	
	@ClientString(id = 20601, message = "Dwarf's Tutorial")
	public static NpcStringId DWARF_S_TUTORIAL;
	
	@ClientString(id = 20602, message = "Dwarf's Tutorial (In Progress)")
	public static NpcStringId DWARF_S_TUTORIAL_IN_PROGRESS;
	
	@ClientString(id = 20603, message = "Dwarf's Tutorial (Done)")
	public static NpcStringId DWARF_S_TUTORIAL_DONE;
	
	@ClientString(id = 20901, message = "Kamael Tutorial")
	public static NpcStringId KAMAEL_TUTORIAL;
	
	@ClientString(id = 21001, message = "Obtain a Wolf Pet (Lv. 15-99)")
	public static NpcStringId OBTAIN_A_WOLF_PET_LV_15_99;
	
	@ClientString(id = 21002, message = "Obtain a Wolf Pet (Lv. 15-99) (In Progress)")
	public static NpcStringId OBTAIN_A_WOLF_PET_LV_15_99_IN_PROGRESS;
	
	@ClientString(id = 21003, message = "Obtain a Wolf Pet (Lv. 15-99) (Complete)")
	public static NpcStringId OBTAIN_A_WOLF_PET_LV_15_99_COMPLETE;
	
	@ClientString(id = 21004, message = "Obtain a Wolf Pet (Lv. 15-99)")
	public static NpcStringId OBTAIN_A_WOLF_PET_LV_15_992;
	
	@ClientString(id = 21101, message = "Trial of the Challenger")
	public static NpcStringId TRIAL_OF_THE_CHALLENGER;
	
	@ClientString(id = 21102, message = "Trial of the Challenger (In Progress)")
	public static NpcStringId TRIAL_OF_THE_CHALLENGER_IN_PROGRESS;
	
	@ClientString(id = 21103, message = "Trial of the Challenger (Done)")
	public static NpcStringId TRIAL_OF_THE_CHALLENGER_DONE;
	
	@ClientString(id = 21201, message = "Trial of Duty")
	public static NpcStringId TRIAL_OF_DUTY;
	
	@ClientString(id = 21202, message = "Trial of Duty (In Progress)")
	public static NpcStringId TRIAL_OF_DUTY_IN_PROGRESS;
	
	@ClientString(id = 21203, message = "Trial of Duty (Done)")
	public static NpcStringId TRIAL_OF_DUTY_DONE;
	
	@ClientString(id = 21301, message = "Trial of the Seeker")
	public static NpcStringId TRIAL_OF_THE_SEEKER;
	
	@ClientString(id = 21302, message = "Trial of the Seeker (In Progress)")
	public static NpcStringId TRIAL_OF_THE_SEEKER_IN_PROGRESS;
	
	@ClientString(id = 21303, message = "Trial of the Seeker (Done)")
	public static NpcStringId TRIAL_OF_THE_SEEKER_DONE;
	
	@ClientString(id = 21401, message = "Trial of the Scholar")
	public static NpcStringId TRIAL_OF_THE_SCHOLAR;
	
	@ClientString(id = 21402, message = "Trial of the Scholar (In Progress)")
	public static NpcStringId TRIAL_OF_THE_SCHOLAR_IN_PROGRESS;
	
	@ClientString(id = 21403, message = "Trial of the Scholar (Done)")
	public static NpcStringId TRIAL_OF_THE_SCHOLAR_DONE;
	
	@ClientString(id = 21501, message = "Trial of the Pilgrim")
	public static NpcStringId TRIAL_OF_THE_PILGRIM;
	
	@ClientString(id = 21502, message = "Trial of the Pilgrim (In Progress)")
	public static NpcStringId TRIAL_OF_THE_PILGRIM_IN_PROGRESS;
	
	@ClientString(id = 21503, message = "Trial of the Pilgrim (Done)")
	public static NpcStringId TRIAL_OF_THE_PILGRIM_DONE;
	
	@ClientString(id = 21601, message = "Trial of the Guildsman")
	public static NpcStringId TRIAL_OF_THE_GUILDSMAN;
	
	@ClientString(id = 21602, message = "Trial of the Guildsman (In Progress)")
	public static NpcStringId TRIAL_OF_THE_GUILDSMAN_IN_PROGRESS;
	
	@ClientString(id = 21603, message = "Trial of the Guildsman (Done)")
	public static NpcStringId TRIAL_OF_THE_GUILDSMAN_DONE;
	
	@ClientString(id = 21701, message = "Testimony of Trust")
	public static NpcStringId TESTIMONY_OF_TRUST;
	
	@ClientString(id = 21702, message = "Testimony of Trust (In Progress)")
	public static NpcStringId TESTIMONY_OF_TRUST_IN_PROGRESS;
	
	@ClientString(id = 21703, message = "Testimony of Trust (Done)")
	public static NpcStringId TESTIMONY_OF_TRUST_DONE;
	
	@ClientString(id = 21801, message = "Testimony of Life")
	public static NpcStringId TESTIMONY_OF_LIFE;
	
	@ClientString(id = 21802, message = "Testimony of Life (In Progress)")
	public static NpcStringId TESTIMONY_OF_LIFE_IN_PROGRESS;
	
	@ClientString(id = 21803, message = "Testimony of Life (Done)")
	public static NpcStringId TESTIMONY_OF_LIFE_DONE;
	
	@ClientString(id = 21901, message = "Testimony of Fate")
	public static NpcStringId TESTIMONY_OF_FATE;
	
	@ClientString(id = 21902, message = "Testimony of Fate (In Progress)")
	public static NpcStringId TESTIMONY_OF_FATE_IN_PROGRESS;
	
	@ClientString(id = 21903, message = "Testimony of Fate (Done)")
	public static NpcStringId TESTIMONY_OF_FATE_DONE;
	
	@ClientString(id = 22001, message = "Testimony of Glory")
	public static NpcStringId TESTIMONY_OF_GLORY;
	
	@ClientString(id = 22002, message = "Testimony of Glory (In Progress)")
	public static NpcStringId TESTIMONY_OF_GLORY_IN_PROGRESS;
	
	@ClientString(id = 22003, message = "Testimony of Glory (Done)")
	public static NpcStringId TESTIMONY_OF_GLORY_DONE;
	
	@ClientString(id = 22051, message = "Is it a lackey of Kakai?!")
	public static NpcStringId IS_IT_A_LACKEY_OF_KAKAI;
	
	@ClientString(id = 22052, message = "Too late!")
	public static NpcStringId TOO_LATE;
	
	@ClientString(id = 22053, message = "Is it a lackey of Kakai?!")
	public static NpcStringId IS_IT_A_LACKEY_OF_KAKAI2;
	
	@ClientString(id = 22054, message = "Too late!")
	public static NpcStringId TOO_LATE2;
	
	@ClientString(id = 22055, message = "How regretful! Unjust dishonor!")
	public static NpcStringId HOW_REGRETFUL_UNJUST_DISHONOR;
	
	@ClientString(id = 22056, message = "I'll get revenge someday!!")
	public static NpcStringId I_LL_GET_REVENGE_SOMEDAY;
	
	@ClientString(id = 22057, message = "Indignant and unfair death!")
	public static NpcStringId INDIGNANT_AND_UNFAIR_DEATH;
	
	@ClientString(id = 22101, message = "Testimony of Prosperity")
	public static NpcStringId TESTIMONY_OF_PROSPERITY;
	
	@ClientString(id = 22102, message = "Testimony of Prosperity (In Progress)")
	public static NpcStringId TESTIMONY_OF_PROSPERITY_IN_PROGRESS;
	
	@ClientString(id = 22103, message = "Testimony of Prosperity (Done)")
	public static NpcStringId TESTIMONY_OF_PROSPERITY_DONE;
	
	@ClientString(id = 22201, message = "Test of the Duelist")
	public static NpcStringId TEST_OF_THE_DUELIST;
	
	@ClientString(id = 22202, message = "Test of the Duelist (In Progress)")
	public static NpcStringId TEST_OF_THE_DUELIST_IN_PROGRESS;
	
	@ClientString(id = 22203, message = "Test of the Duelist (Done)")
	public static NpcStringId TEST_OF_THE_DUELIST_DONE;
	
	@ClientString(id = 22301, message = "Test of the Champion")
	public static NpcStringId TEST_OF_THE_CHAMPION;
	
	@ClientString(id = 22302, message = "Test of the Champion (In Progress)")
	public static NpcStringId TEST_OF_THE_CHAMPION_IN_PROGRESS;
	
	@ClientString(id = 22303, message = "Test of the Champion (Done)")
	public static NpcStringId TEST_OF_THE_CHAMPION_DONE;
	
	@ClientString(id = 22401, message = "Test of Sagittarius")
	public static NpcStringId TEST_OF_SAGITTARIUS;
	
	@ClientString(id = 22402, message = "Test of Sagittarius (In Progress)")
	public static NpcStringId TEST_OF_SAGITTARIUS_IN_PROGRESS;
	
	@ClientString(id = 22403, message = "Test of Sagittarius (Done)")
	public static NpcStringId TEST_OF_SAGITTARIUS_DONE;
	
	@ClientString(id = 22501, message = "Test of the Searcher")
	public static NpcStringId TEST_OF_THE_SEARCHER;
	
	@ClientString(id = 22502, message = "Test of the Searcher (In Progress)")
	public static NpcStringId TEST_OF_THE_SEARCHER_IN_PROGRESS;
	
	@ClientString(id = 22503, message = "Test of the Searcher (Done)")
	public static NpcStringId TEST_OF_THE_SEARCHER_DONE;
	
	@ClientString(id = 22601, message = "Test of the Healer")
	public static NpcStringId TEST_OF_THE_HEALER;
	
	@ClientString(id = 22602, message = "Test of the Healer (In Progress)")
	public static NpcStringId TEST_OF_THE_HEALER_IN_PROGRESS;
	
	@ClientString(id = 22603, message = "Test of the Healer (Done)")
	public static NpcStringId TEST_OF_THE_HEALER_DONE;
	
	@ClientString(id = 22701, message = "Test of the Reformer")
	public static NpcStringId TEST_OF_THE_REFORMER;
	
	@ClientString(id = 22702, message = "Test of the Reformer (In Progress)")
	public static NpcStringId TEST_OF_THE_REFORMER_IN_PROGRESS;
	
	@ClientString(id = 22703, message = "Test of the Reformer (Done)")
	public static NpcStringId TEST_OF_THE_REFORMER_DONE;
	
	@ClientString(id = 22719, message = "The concealed truth will always be revealed...!")
	public static NpcStringId THE_CONCEALED_TRUTH_WILL_ALWAYS_BE_REVEALED;
	
	@ClientString(id = 22720, message = "Cowardly guy!")
	public static NpcStringId COWARDLY_GUY;
	
	@ClientString(id = 22801, message = "Test of Magus")
	public static NpcStringId TEST_OF_MAGUS;
	
	@ClientString(id = 22802, message = "Test of Magus (In Progress)")
	public static NpcStringId TEST_OF_MAGUS_IN_PROGRESS;
	
	@ClientString(id = 22803, message = "Test of Magus (Done)")
	public static NpcStringId TEST_OF_MAGUS_DONE;
	
	@ClientString(id = 22819, message = "I am a tree of nothing... a tree... that knows where to return...")
	public static NpcStringId I_AM_A_TREE_OF_NOTHING_A_TREE_THAT_KNOWS_WHERE_TO_RETURN;
	
	@ClientString(id = 22820, message = "I am a creature that shows the truth of the place deep in my heart...")
	public static NpcStringId I_AM_A_CREATURE_THAT_SHOWS_THE_TRUTH_OF_THE_PLACE_DEEP_IN_MY_HEART;
	
	@ClientString(id = 22821, message = "I am a mirror of darkness... an illusion of darkness...")
	public static NpcStringId I_AM_A_MIRROR_OF_DARKNESS_AN_ILLUSION_OF_DARKNESS;
	
	@ClientString(id = 22901, message = "Test of Witchcraft")
	public static NpcStringId TEST_OF_WITCHCRAFT;
	
	@ClientString(id = 22902, message = "Test of Witchcraft (In Progress)")
	public static NpcStringId TEST_OF_WITCHCRAFT_IN_PROGRESS;
	
	@ClientString(id = 22903, message = "Test of Witchcraft (Done)")
	public static NpcStringId TEST_OF_WITCHCRAFT_DONE;
	
	@ClientString(id = 22933, message = "I absolutely cannot give it to you! It is my precious jewel!")
	public static NpcStringId I_ABSOLUTELY_CANNOT_GIVE_IT_TO_YOU_IT_IS_MY_PRECIOUS_JEWEL;
	
	@ClientString(id = 22934, message = "I'll take your lives later!")
	public static NpcStringId I_LL_TAKE_YOUR_LIVES_LATER;
	
	@ClientString(id = 22935, message = "That sword is really...!")
	public static NpcStringId THAT_SWORD_IS_REALLY;
	
	@ClientString(id = 22936, message = "No! I haven't completely finished the command for destruction and slaughter yet!!!")
	public static NpcStringId NO_I_HAVEN_T_COMPLETELY_FINISHED_THE_COMMAND_FOR_DESTRUCTION_AND_SLAUGHTER_YET;
	
	@ClientString(id = 22937, message = "How dare you wake me!  Now you shall die!")
	public static NpcStringId HOW_DARE_YOU_WAKE_ME_NOW_YOU_SHALL_DIE;
	
	@ClientString(id = 23001, message = "Test of the Summoner")
	public static NpcStringId TEST_OF_THE_SUMMONER;
	
	@ClientString(id = 23002, message = "Test of the Summoner (In Progress)")
	public static NpcStringId TEST_OF_THE_SUMMONER_IN_PROGRESS;
	
	@ClientString(id = 23003, message = "Test of the Summoner (Done)")
	public static NpcStringId TEST_OF_THE_SUMMONER_DONE;
	
	@ClientString(id = 23060, message = "START DUEL")
	public static NpcStringId START_DUEL;
	
	@ClientString(id = 23061, message = "RULE VIOLATION")
	public static NpcStringId RULE_VIOLATION;
	
	@ClientString(id = 23062, message = "I LOSE")
	public static NpcStringId I_LOSE;
	
	@ClientString(id = 23063, message = "Whhiisshh!")
	public static NpcStringId WHHIISSHH;
	
	@ClientString(id = 23064, message = "Rule violation!")
	public static NpcStringId RULE_VIOLATION2;
	
	@ClientString(id = 23065, message = "I'm sorry, Lord!")
	public static NpcStringId I_M_SORRY_LORD;
	
	@ClientString(id = 23066, message = "Whish! Fight!")
	public static NpcStringId WHISH_FIGHT;
	
	@ClientString(id = 23067, message = "Rule violation!")
	public static NpcStringId RULE_VIOLATION3;
	
	@ClientString(id = 23068, message = "Lost! Sorry, Lord!")
	public static NpcStringId LOST_SORRY_LORD;
	
	@ClientString(id = 23069, message = "START DUEL")
	public static NpcStringId START_DUEL2;
	
	@ClientString(id = 23070, message = "RULE VIOLATION")
	public static NpcStringId RULE_VIOLATION4;
	
	@ClientString(id = 23071, message = "I LOSE")
	public static NpcStringId I_LOSE2;
	
	@ClientString(id = 23072, message = "So shall we start?!")
	public static NpcStringId SO_SHALL_WE_START;
	
	@ClientString(id = 23073, message = "Rule violation!!!")
	public static NpcStringId RULE_VIOLATION5;
	
	@ClientString(id = 23074, message = "Ugh! I lost...!")
	public static NpcStringId UGH_I_LOST;
	
	@ClientString(id = 23075, message = "I'll walk all over you!")
	public static NpcStringId I_LL_WALK_ALL_OVER_YOU;
	
	@ClientString(id = 23076, message = "Rule violation!!!")
	public static NpcStringId RULE_VIOLATION6;
	
	@ClientString(id = 23077, message = "Ugh! Can this be happening?!")
	public static NpcStringId UGH_CAN_THIS_BE_HAPPENING;
	
	@ClientString(id = 23078, message = "It's the natural result!")
	public static NpcStringId IT_S_THE_NATURAL_RESULT;
	
	@ClientString(id = 23079, message = "Ho, ho! I win!")
	public static NpcStringId HO_HO_I_WIN;
	
	@ClientString(id = 23080, message = "I WIN")
	public static NpcStringId I_WIN;
	
	@ClientString(id = 23081, message = "Whish! I won!")
	public static NpcStringId WHISH_I_WON;
	
	@ClientString(id = 23082, message = "Whhiisshh!")
	public static NpcStringId WHHIISSHH2;
	
	@ClientString(id = 23083, message = "I WIN")
	public static NpcStringId I_WIN2;
	
	@ClientString(id = 23101, message = "Test of the Maestro")
	public static NpcStringId TEST_OF_THE_MAESTRO;
	
	@ClientString(id = 23102, message = "Test of the Maestro (In Progress)")
	public static NpcStringId TEST_OF_THE_MAESTRO_IN_PROGRESS;
	
	@ClientString(id = 23103, message = "Test of the Maestro (Done)")
	public static NpcStringId TEST_OF_THE_MAESTRO_DONE;
	
	@ClientString(id = 23201, message = "Test of the Lord")
	public static NpcStringId TEST_OF_THE_LORD;
	
	@ClientString(id = 23202, message = "Test of the Lord (In Progress)")
	public static NpcStringId TEST_OF_THE_LORD_IN_PROGRESS;
	
	@ClientString(id = 23203, message = "Test of the Lord (Done)")
	public static NpcStringId TEST_OF_THE_LORD_DONE;
	
	@ClientString(id = 23301, message = "Test of the War Spirit")
	public static NpcStringId TEST_OF_THE_WAR_SPIRIT;
	
	@ClientString(id = 23302, message = "Test of the War Spirit (In Progress)")
	public static NpcStringId TEST_OF_THE_WAR_SPIRIT_IN_PROGRESS;
	
	@ClientString(id = 23303, message = "Test of the War Spirit (Done)")
	public static NpcStringId TEST_OF_THE_WAR_SPIRIT_DONE;
	
	@ClientString(id = 23401, message = "Fate's Whisper")
	public static NpcStringId FATE_S_WHISPER;
	
	@ClientString(id = 23402, message = "Fate's Whisper (In Progress)")
	public static NpcStringId FATE_S_WHISPER_IN_PROGRESS;
	
	@ClientString(id = 23403, message = "Fate's Whisper (Done)")
	public static NpcStringId FATE_S_WHISPER_DONE;
	
	@ClientString(id = 23404, message = "Fate's Whisper")
	public static NpcStringId FATE_S_WHISPER2;
	
	@ClientString(id = 23434, message = "Who dares to try and steal my noble blood?")
	public static NpcStringId WHO_DARES_TO_TRY_AND_STEAL_MY_NOBLE_BLOOD;
	
	@ClientString(id = 23501, message = "Mimir's Elixir")
	public static NpcStringId MIMIR_S_ELIXIR;
	
	@ClientString(id = 23502, message = "Mimir's Elixir (In Progress)")
	public static NpcStringId MIMIR_S_ELIXIR_IN_PROGRESS;
	
	@ClientString(id = 23503, message = "Mimir's Elixir (Done)")
	public static NpcStringId MIMIR_S_ELIXIR_DONE;
	
	@ClientString(id = 23504, message = "Mimir's Elixir")
	public static NpcStringId MIMIR_S_ELIXIR2;
	
	@ClientString(id = 23601, message = "Seeds of Chaos")
	public static NpcStringId SEEDS_OF_CHAOS;
	
	@ClientString(id = 23602, message = "Seeds of Chaos (In Progress)")
	public static NpcStringId SEEDS_OF_CHAOS_IN_PROGRESS;
	
	@ClientString(id = 23603, message = "Seeds of Chaos (Done)")
	public static NpcStringId SEEDS_OF_CHAOS_DONE;
	
	@ClientString(id = 23604, message = "Seeds of Chaos")
	public static NpcStringId SEEDS_OF_CHAOS2;
	
	@ClientString(id = 23651, message = "$s1! Finally, we meet!")
	public static NpcStringId S1_FINALLY_WE_MEET;
	
	@ClientString(id = 23652, message = "Hmm, where did my friend go?")
	public static NpcStringId HMM_WHERE_DID_MY_FRIEND_GO;
	
	@ClientString(id = 23653, message = "Best of luck with your future endeavors.")
	public static NpcStringId BEST_OF_LUCK_WITH_YOUR_FUTURE_ENDEAVORS;
	
	@ClientString(id = 23661, message = "$s1! Did you wait for long?")
	public static NpcStringId S1_DID_YOU_WAIT_FOR_LONG;
	
	@ClientString(id = 23662, message = "Hmm, where did my friend go?")
	public static NpcStringId HMM_WHERE_DID_MY_FRIEND_GO2;
	
	@ClientString(id = 23663, message = "Best of luck with your future endeavors.")
	public static NpcStringId BEST_OF_LUCK_WITH_YOUR_FUTURE_ENDEAVORS2;
	
	@ClientString(id = 23671, message = "Did you bring what I asked, $s1?")
	public static NpcStringId DID_YOU_BRING_WHAT_I_ASKED_S1;
	
	@ClientString(id = 23672, message = "Hmm, where did my friend go?")
	public static NpcStringId HMM_WHERE_DID_MY_FRIEND_GO3;
	
	@ClientString(id = 23673, message = "Best of luck with your future endeavors.")
	public static NpcStringId BEST_OF_LUCK_WITH_YOUR_FUTURE_ENDEAVORS3;
	
	@ClientString(id = 23681, message = "Hmm? Is someone approaching?")
	public static NpcStringId HMM_IS_SOMEONE_APPROACHING;
	
	@ClientString(id = 23682, message = "Graaah, we're being attacked!")
	public static NpcStringId GRAAAH_WE_RE_BEING_ATTACKED;
	
	@ClientString(id = 23683, message = "In that case, I wish you good luck.")
	public static NpcStringId IN_THAT_CASE_I_WISH_YOU_GOOD_LUCK;
	
	@ClientString(id = 23685, message = "$s1, has everything been found?")
	public static NpcStringId S1_HAS_EVERYTHING_BEEN_FOUND;
	
	@ClientString(id = 23686, message = "Graaah, we're being attacked!")
	public static NpcStringId GRAAAH_WE_RE_BEING_ATTACKED2;
	
	@ClientString(id = 23687, message = "Safe travels!")
	public static NpcStringId SAFE_TRAVELS;
	
	@ClientString(id = 23701, message = "Winds of Change (Lv. 82-99)")
	public static NpcStringId WINDS_OF_CHANGE_LV_82_99;
	
	@ClientString(id = 23702, message = "Winds of Change (Lv. 82-99) (In Progress)")
	public static NpcStringId WINDS_OF_CHANGE_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 23703, message = "Winds of Change (Lv. 82-99) (Done)")
	public static NpcStringId WINDS_OF_CHANGE_LV_82_99_DONE;
	
	@ClientString(id = 23801, message = "Success/Failure of Business (Lv. 82-99)")
	public static NpcStringId SUCCESS_FAILURE_OF_BUSINESS_LV_82_99;
	
	@ClientString(id = 23802, message = "Success/Failure of Business (Lv. 82-99) (In Progress)")
	public static NpcStringId SUCCESS_FAILURE_OF_BUSINESS_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 23803, message = "Success/Failure of Business (Lv. 82-99) (Done)")
	public static NpcStringId SUCCESS_FAILURE_OF_BUSINESS_LV_82_99_DONE;
	
	@ClientString(id = 23804, message = "Success/Failure of Business (Lv. 82-99)")
	public static NpcStringId SUCCESS_FAILURE_OF_BUSINESS_LV_82_992;
	
	@ClientString(id = 23901, message = "Won't You Join Us? (Lv. 82-99)")
	public static NpcStringId WON_T_YOU_JOIN_US_LV_82_99;
	
	@ClientString(id = 23902, message = "Won't You Join Us? (Lv. 82-99) (In Progress)")
	public static NpcStringId WON_T_YOU_JOIN_US_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 23903, message = "Won't You Join Us? (Lv. 82-99) (Done)")
	public static NpcStringId WON_T_YOU_JOIN_US_LV_82_99_DONE;
	
	@ClientString(id = 23904, message = "Won't You Join Us? (Lv. 82-99)")
	public static NpcStringId WON_T_YOU_JOIN_US_LV_82_992;
	
	@ClientString(id = 24001, message = "I'm the Only One You Can Trust (Lv. 81-99)")
	public static NpcStringId I_M_THE_ONLY_ONE_YOU_CAN_TRUST_LV_81_99;
	
	@ClientString(id = 24002, message = "I'm the Only One You Can Trust (Lv. 81-99) (In Progress)")
	public static NpcStringId I_M_THE_ONLY_ONE_YOU_CAN_TRUST_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 24003, message = "I'm the Only One You Can Trust (Lv. 81-99) (Done)")
	public static NpcStringId I_M_THE_ONLY_ONE_YOU_CAN_TRUST_LV_81_99_DONE;
	
	@ClientString(id = 24004, message = "I'm the Only One You Can Trust (Lv. 81-99)")
	public static NpcStringId I_M_THE_ONLY_ONE_YOU_CAN_TRUST_LV_81_992;
	
	@ClientString(id = 24101, message = "Path of the Noblesse, Precious Soul - 1")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_1;
	
	@ClientString(id = 24102, message = "Path of the Noblesse, Precious Soul - 1 (In Progress)")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_1_IN_PROGRESS;
	
	@ClientString(id = 24103, message = "Path of the Noblesse, Precious Soul - 1 (Done)")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_1_DONE;
	
	@ClientString(id = 24104, message = "Path of the Noblesse, Precious Soul - 1")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_12;
	
	@ClientString(id = 24201, message = "Path of the Noblesse, Precious Soul - 2")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_2;
	
	@ClientString(id = 24202, message = "Path of the Noblesse, Precious Soul - 2 (In Progress)")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_2_IN_PROGRESS;
	
	@ClientString(id = 24203, message = "Path of the Noblesse, Precious Soul - 2 (Done)")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_2_DONE;
	
	@ClientString(id = 24204, message = "Path of the Noblesse, Precious Soul - 2")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_22;
	
	@ClientString(id = 24401, message = "Follow Me")
	public static NpcStringId FOLLOW_ME;
	
	@ClientString(id = 24402, message = "Follow Me (In Progress)")
	public static NpcStringId FOLLOW_ME_IN_PROGRESS;
	
	@ClientString(id = 24403, message = "Follow Me (Done)")
	public static NpcStringId FOLLOW_ME_DONE;
	
	@ClientString(id = 24404, message = "Follow Me")
	public static NpcStringId FOLLOW_ME2;
	
	@ClientString(id = 24411, message = "Follow Me (Sponsor)")
	public static NpcStringId FOLLOW_ME_SPONSOR;
	
	@ClientString(id = 24501, message = "Come to Me (Lv. 70-75)")
	public static NpcStringId COME_TO_ME_LV_70_75;
	
	@ClientString(id = 24502, message = "Come to Me (Lv. 70-75) (In Progress)")
	public static NpcStringId COME_TO_ME_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 24503, message = "Come to Me (Lv. 70-75) (Done)")
	public static NpcStringId COME_TO_ME_LV_70_75_DONE;
	
	@ClientString(id = 24504, message = "Come to Me (Lv. 70-75)")
	public static NpcStringId COME_TO_ME_LV_70_752;
	
	@ClientString(id = 24511, message = "Come to Me (Sponsor)")
	public static NpcStringId COME_TO_ME_SPONSOR;
	
	@ClientString(id = 24601, message = "Path of the Noblesse, Precious Soul - 3")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_3;
	
	@ClientString(id = 24602, message = "Path of the Noblesse, Precious Soul - 3 (In Progress)")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_3_IN_PROGRESS;
	
	@ClientString(id = 24603, message = "Path of the Noblesse, Precious Soul - 3 (Done)")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_3_DONE;
	
	@ClientString(id = 24604, message = "Path of the Noblesse, Precious Soul - 3")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_32;
	
	@ClientString(id = 24701, message = "Path of the Noblesse, Precious Soul - 4")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_4;
	
	@ClientString(id = 24702, message = "Path of the Noblesse, Precious Soul - 4 (In Progress)")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_4_IN_PROGRESS;
	
	@ClientString(id = 24703, message = "Path of the Noblesse, Precious Soul - 4 (Done)")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_4_DONE;
	
	@ClientString(id = 24704, message = "Path of the Noblesse, Precious Soul - 4")
	public static NpcStringId PATH_OF_THE_NOBLESSE_PRECIOUS_SOUL_42;
	
	@ClientString(id = 24801, message = "Returner Tutorial")
	public static NpcStringId RETURNER_TUTORIAL;
	
	@ClientString(id = 24850, message = "You've gained $s2 XP and $s3 skill points  with $s1 Mark of Effort.")
	public static NpcStringId YOU_VE_GAINED_S2_XP_AND_S3_SKILL_POINTS_WITH_S1_MARK_OF_EFFORT;
	
	@ClientString(id = 24851, message = "You've received $s3 Marks of Effort from leveling up from $s1 to $s2.")
	public static NpcStringId YOU_VE_RECEIVED_S3_MARKS_OF_EFFORT_FROM_LEVELING_UP_FROM_S1_TO_S2;
	
	@ClientString(id = 24901, message = "Poisoned Plains of the Lizardmen")
	public static NpcStringId POISONED_PLAINS_OF_THE_LIZARDMEN;
	
	@ClientString(id = 24902, message = "Poisoned Plains of the Lizardmen (In Progress)")
	public static NpcStringId POISONED_PLAINS_OF_THE_LIZARDMEN_IN_PROGRESS;
	
	@ClientString(id = 24903, message = "Poisoned Plains of the Lizardmen (Done)")
	public static NpcStringId POISONED_PLAINS_OF_THE_LIZARDMEN_DONE;
	
	@ClientString(id = 24904, message = "Poisoned Plains of the Lizardmen")
	public static NpcStringId POISONED_PLAINS_OF_THE_LIZARDMEN2;
	
	@ClientString(id = 25001, message = "Watch What You Eat")
	public static NpcStringId WATCH_WHAT_YOU_EAT;
	
	@ClientString(id = 25002, message = "Watch What You Eat (In Progress)")
	public static NpcStringId WATCH_WHAT_YOU_EAT_IN_PROGRESS;
	
	@ClientString(id = 25003, message = "Watch What You Eat (Done)")
	public static NpcStringId WATCH_WHAT_YOU_EAT_DONE;
	
	@ClientString(id = 25004, message = "Watch What You Eat")
	public static NpcStringId WATCH_WHAT_YOU_EAT2;
	
	@ClientString(id = 25101, message = "No Secrets")
	public static NpcStringId NO_SECRETS;
	
	@ClientString(id = 25102, message = "No Secrets (In Progress)")
	public static NpcStringId NO_SECRETS_IN_PROGRESS;
	
	@ClientString(id = 25103, message = "No Secrets (Done)")
	public static NpcStringId NO_SECRETS_DONE;
	
	@ClientString(id = 25104, message = "No Secrets")
	public static NpcStringId NO_SECRETS2;
	
	@ClientString(id = 25201, message = "It Smells Delicious!")
	public static NpcStringId IT_SMELLS_DELICIOUS;
	
	@ClientString(id = 25202, message = "It Smells Delicious! (In Progress)")
	public static NpcStringId IT_SMELLS_DELICIOUS_IN_PROGRESS;
	
	@ClientString(id = 25203, message = "It Smells Delicious! (Done)")
	public static NpcStringId IT_SMELLS_DELICIOUS_DONE;
	
	@ClientString(id = 25204, message = "It Smells Delicious!")
	public static NpcStringId IT_SMELLS_DELICIOUS2;
	
	@ClientString(id = 25401, message = "Legendary Tales (Lv. 80-99)")
	public static NpcStringId LEGENDARY_TALES_LV_80_99;
	
	@ClientString(id = 25402, message = "Legendary Tales (Lv. 80-99) (In Progress)")
	public static NpcStringId LEGENDARY_TALES_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 25403, message = "Legendary Tales (Lv. 80-99) (Done)")
	public static NpcStringId LEGENDARY_TALES_LV_80_99_DONE;
	
	@ClientString(id = 25701, message = "The Guard is Busy")
	public static NpcStringId THE_GUARD_IS_BUSY;
	
	@ClientString(id = 25702, message = "The Guard is Busy (In Progress)")
	public static NpcStringId THE_GUARD_IS_BUSY_IN_PROGRESS;
	
	@ClientString(id = 25703, message = "The Guard is Busy (Done)")
	public static NpcStringId THE_GUARD_IS_BUSY_DONE;
	
	@ClientString(id = 25801, message = "Bring Wolf Pelts")
	public static NpcStringId BRING_WOLF_PELTS;
	
	@ClientString(id = 25802, message = "Bring Wolf Pelts (In Progress)")
	public static NpcStringId BRING_WOLF_PELTS_IN_PROGRESS;
	
	@ClientString(id = 25901, message = "Request from the Farm Owner")
	public static NpcStringId REQUEST_FROM_THE_FARM_OWNER;
	
	@ClientString(id = 25902, message = "Request from the Farm Owner (In Progress)")
	public static NpcStringId REQUEST_FROM_THE_FARM_OWNER_IN_PROGRESS;
	
	@ClientString(id = 25904, message = "Request from the Farm Owner")
	public static NpcStringId REQUEST_FROM_THE_FARM_OWNER2;
	
	@ClientString(id = 26001, message = "Orc Hunting")
	public static NpcStringId ORC_HUNTING;
	
	@ClientString(id = 26002, message = "Orc Hunting (In Progress)")
	public static NpcStringId ORC_HUNTING_IN_PROGRESS;
	
	@ClientString(id = 26101, message = "Collector's Dream")
	public static NpcStringId COLLECTOR_S_DREAM;
	
	@ClientString(id = 26102, message = "Collector's Dream (In Progress)")
	public static NpcStringId COLLECTOR_S_DREAM_IN_PROGRESS;
	
	@ClientString(id = 26201, message = "Trade with the Ivory Tower")
	public static NpcStringId TRADE_WITH_THE_IVORY_TOWER;
	
	@ClientString(id = 26202, message = "Trade with the Ivory Tower (In Progress)")
	public static NpcStringId TRADE_WITH_THE_IVORY_TOWER_IN_PROGRESS;
	
	@ClientString(id = 26301, message = "Orc Subjugation")
	public static NpcStringId ORC_SUBJUGATION;
	
	@ClientString(id = 26302, message = "Orc Subjugation (In Progress)")
	public static NpcStringId ORC_SUBJUGATION_IN_PROGRESS;
	
	@ClientString(id = 26401, message = "Keen Claws")
	public static NpcStringId KEEN_CLAWS;
	
	@ClientString(id = 26402, message = "Keen Claws (In Progress)")
	public static NpcStringId KEEN_CLAWS_IN_PROGRESS;
	
	@ClientString(id = 26501, message = "Bonds of Slavery")
	public static NpcStringId BONDS_OF_SLAVERY;
	
	@ClientString(id = 26502, message = "Bonds of Slavery (In Progress)")
	public static NpcStringId BONDS_OF_SLAVERY_IN_PROGRESS;
	
	@ClientString(id = 26601, message = "Pleas of Pixies")
	public static NpcStringId PLEAS_OF_PIXIES;
	
	@ClientString(id = 26602, message = "Pleas of Pixies (In Progress)")
	public static NpcStringId PLEAS_OF_PIXIES_IN_PROGRESS;
	
	@ClientString(id = 26701, message = "Wrath of Verdure")
	public static NpcStringId WRATH_OF_VERDURE;
	
	@ClientString(id = 26702, message = "Wrath of Verdure (In Progress)")
	public static NpcStringId WRATH_OF_VERDURE_IN_PROGRESS;
	
	@ClientString(id = 26801, message = "Traces of Evil")
	public static NpcStringId TRACES_OF_EVIL;
	
	@ClientString(id = 26802, message = "Traces of Evil (In Progress)")
	public static NpcStringId TRACES_OF_EVIL_IN_PROGRESS;
	
	@ClientString(id = 26901, message = "Invention Ambition")
	public static NpcStringId INVENTION_AMBITION;
	
	@ClientString(id = 26902, message = "Invention Ambition (In Progress)")
	public static NpcStringId INVENTION_AMBITION_IN_PROGRESS;
	
	@ClientString(id = 27001, message = "The One Who Ends Silence (Lv. 82-99)")
	public static NpcStringId THE_ONE_WHO_ENDS_SILENCE_LV_82_99;
	
	@ClientString(id = 27002, message = "The One Who Ends Silence (Lv. 82-99) (In Progress)")
	public static NpcStringId THE_ONE_WHO_ENDS_SILENCE_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 27004, message = "The One Who Ends Silence (Lv. 82-99)")
	public static NpcStringId THE_ONE_WHO_ENDS_SILENCE_LV_82_992;
	
	@ClientString(id = 27101, message = "Proof of Valor")
	public static NpcStringId PROOF_OF_VALOR;
	
	@ClientString(id = 27102, message = "Proof of Valor (In Progress)")
	public static NpcStringId PROOF_OF_VALOR_IN_PROGRESS;
	
	@ClientString(id = 27201, message = "Wrath of Ancestors")
	public static NpcStringId WRATH_OF_ANCESTORS;
	
	@ClientString(id = 27202, message = "Wrath of Ancestors (In Progress)")
	public static NpcStringId WRATH_OF_ANCESTORS_IN_PROGRESS;
	
	@ClientString(id = 27301, message = "Invaders of the Holy Land")
	public static NpcStringId INVADERS_OF_THE_HOLY_LAND;
	
	@ClientString(id = 27302, message = "Invaders of the Holy Land (In Progress)")
	public static NpcStringId INVADERS_OF_THE_HOLY_LAND_IN_PROGRESS;
	
	@ClientString(id = 27401, message = "Skirmish with the Werewolves")
	public static NpcStringId SKIRMISH_WITH_THE_WEREWOLVES;
	
	@ClientString(id = 27402, message = "Skirmish with the Werewolves (In Progress)")
	public static NpcStringId SKIRMISH_WITH_THE_WEREWOLVES_IN_PROGRESS;
	
	@ClientString(id = 27501, message = "Dark Winged Spies")
	public static NpcStringId DARK_WINGED_SPIES;
	
	@ClientString(id = 27502, message = "Dark Winged Spies (In Progress)")
	public static NpcStringId DARK_WINGED_SPIES_IN_PROGRESS;
	
	@ClientString(id = 27601, message = "Totem of the Hestui")
	public static NpcStringId TOTEM_OF_THE_HESTUI;
	
	@ClientString(id = 27602, message = "Totem of the Hestui (In Progress)")
	public static NpcStringId TOTEM_OF_THE_HESTUI_IN_PROGRESS;
	
	@ClientString(id = 27701, message = "Gatekeeper's Offering")
	public static NpcStringId GATEKEEPER_S_OFFERING;
	
	@ClientString(id = 27702, message = "Gatekeeper's Offering (In Progress)")
	public static NpcStringId GATEKEEPER_S_OFFERING_IN_PROGRESS;
	
	@ClientString(id = 27801, message = "Home Security (Lv. 82-99)")
	public static NpcStringId HOME_SECURITY_LV_82_99;
	
	@ClientString(id = 27802, message = "Home Security (Lv. 82-99) (In Progress)")
	public static NpcStringId HOME_SECURITY_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 27901, message = "Target of Opportunity (Lv. 82-99)")
	public static NpcStringId TARGET_OF_OPPORTUNITY_LV_82_99;
	
	@ClientString(id = 27902, message = "Target of Opportunity (Lv. 82-99) (In Progress)")
	public static NpcStringId TARGET_OF_OPPORTUNITY_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 27904, message = "Target of Opportunity (Lv. 82-99)")
	public static NpcStringId TARGET_OF_OPPORTUNITY_LV_82_992;
	
	@ClientString(id = 28001, message = "The Food Chain")
	public static NpcStringId THE_FOOD_CHAIN;
	
	@ClientString(id = 28002, message = "The Food Chain (In Progress)")
	public static NpcStringId THE_FOOD_CHAIN_IN_PROGRESS;
	
	@ClientString(id = 28101, message = "Head for the Hills!")
	public static NpcStringId HEAD_FOR_THE_HILLS;
	
	@ClientString(id = 28102, message = "Head for the Hills! (In Progress)")
	public static NpcStringId HEAD_FOR_THE_HILLS_IN_PROGRESS;
	
	@ClientString(id = 28301, message = "The Few, The Proud, The Brave")
	public static NpcStringId THE_FEW_THE_PROUD_THE_BRAVE;
	
	@ClientString(id = 28302, message = "The Few, The Proud, The Brave (In Progress)")
	public static NpcStringId THE_FEW_THE_PROUD_THE_BRAVE_IN_PROGRESS;
	
	@ClientString(id = 28401, message = "Muertos Hunting")
	public static NpcStringId MUERTOS_HUNTING;
	
	@ClientString(id = 28402, message = "Muertos Hunting (In Progress)")
	public static NpcStringId MUERTOS_HUNTING_IN_PROGRESS;
	
	@ClientString(id = 28601, message = "Fabulous Feathers")
	public static NpcStringId FABULOUS_FEATHERS;
	
	@ClientString(id = 28602, message = "Fabulous Feathers (In Progress)")
	public static NpcStringId FABULOUS_FEATHERS_IN_PROGRESS;
	
	@ClientString(id = 28701, message = "Figuring It Out!")
	public static NpcStringId FIGURING_IT_OUT;
	
	@ClientString(id = 28702, message = "Figuring It Out! (In Progress)")
	public static NpcStringId FIGURING_IT_OUT_IN_PROGRESS;
	
	@ClientString(id = 28704, message = "Figuring It Out!")
	public static NpcStringId FIGURING_IT_OUT2;
	
	@ClientString(id = 28801, message = "Handle With Care")
	public static NpcStringId HANDLE_WITH_CARE;
	
	@ClientString(id = 28802, message = "Handle With Care (In Progress)")
	public static NpcStringId HANDLE_WITH_CARE_IN_PROGRESS;
	
	@ClientString(id = 28804, message = "Handle With Care")
	public static NpcStringId HANDLE_WITH_CARE2;
	
	@ClientString(id = 28901, message = "No More Soup For You")
	public static NpcStringId NO_MORE_SOUP_FOR_YOU;
	
	@ClientString(id = 28902, message = "No More Soup For You (In Progress)")
	public static NpcStringId NO_MORE_SOUP_FOR_YOU_IN_PROGRESS;
	
	@ClientString(id = 28904, message = "No More Soup For You")
	public static NpcStringId NO_MORE_SOUP_FOR_YOU2;
	
	@ClientString(id = 29001, message = "Threat Removal")
	public static NpcStringId THREAT_REMOVAL;
	
	@ClientString(id = 29002, message = "Threat Removal (In Progress)")
	public static NpcStringId THREAT_REMOVAL_IN_PROGRESS;
	
	@ClientString(id = 29004, message = "Threat Removal")
	public static NpcStringId THREAT_REMOVAL2;
	
	@ClientString(id = 29101, message = "Revenge of the Redbonnet")
	public static NpcStringId REVENGE_OF_THE_REDBONNET;
	
	@ClientString(id = 29102, message = "Revenge of the Redbonnet (In Progress)")
	public static NpcStringId REVENGE_OF_THE_REDBONNET_IN_PROGRESS;
	
	@ClientString(id = 29201, message = "Brigands Sweep")
	public static NpcStringId BRIGANDS_SWEEP;
	
	@ClientString(id = 29202, message = "Brigands Sweep (In Progress)")
	public static NpcStringId BRIGANDS_SWEEP_IN_PROGRESS;
	
	@ClientString(id = 29301, message = "The Hidden Veins")
	public static NpcStringId THE_HIDDEN_VEINS;
	
	@ClientString(id = 29302, message = "The Hidden Veins (In Progress)")
	public static NpcStringId THE_HIDDEN_VEINS_IN_PROGRESS;
	
	@ClientString(id = 29401, message = "Covert Business")
	public static NpcStringId COVERT_BUSINESS;
	
	@ClientString(id = 29402, message = "Covert Business (In Progress)")
	public static NpcStringId COVERT_BUSINESS_IN_PROGRESS;
	
	@ClientString(id = 29501, message = "Dreaming of the Skies")
	public static NpcStringId DREAMING_OF_THE_SKIES;
	
	@ClientString(id = 29502, message = "Dreaming of the Skies (In Progress)")
	public static NpcStringId DREAMING_OF_THE_SKIES_IN_PROGRESS;
	
	@ClientString(id = 29601, message = "Tarantula's Spider Silk")
	public static NpcStringId TARANTULA_S_SPIDER_SILK;
	
	@ClientString(id = 29602, message = "Tarantula's Spider Silk (In Progress)")
	public static NpcStringId TARANTULA_S_SPIDER_SILK_IN_PROGRESS;
	
	@ClientString(id = 29701, message = "Gatekeeper's Favor")
	public static NpcStringId GATEKEEPER_S_FAVOR;
	
	@ClientString(id = 29702, message = "Gatekeeper's Favor (In Progress)")
	public static NpcStringId GATEKEEPER_S_FAVOR_IN_PROGRESS;
	
	@ClientString(id = 29801, message = "Lizardmen's Conspiracy (Lv. 25-34)")
	public static NpcStringId LIZARDMEN_S_CONSPIRACY_LV_25_34;
	
	@ClientString(id = 29802, message = "Lizardmen's Conspiracy (Lv. 25-34) (In Progress)")
	public static NpcStringId LIZARDMEN_S_CONSPIRACY_LV_25_34_IN_PROGRESS;
	
	@ClientString(id = 29804, message = "Lizardmen's Conspiracy (Lv. 25-34)")
	public static NpcStringId LIZARDMEN_S_CONSPIRACY_LV_25_342;
	
	@ClientString(id = 29901, message = "Gather Ingredients for Pie")
	public static NpcStringId GATHER_INGREDIENTS_FOR_PIE;
	
	@ClientString(id = 29902, message = "Gather Ingredients for Pie (In Progress)")
	public static NpcStringId GATHER_INGREDIENTS_FOR_PIE_IN_PROGRESS;
	
	@ClientString(id = 29904, message = "Gather Ingredients for Pie")
	public static NpcStringId GATHER_INGREDIENTS_FOR_PIE2;
	
	@ClientString(id = 30001, message = "Leto Lizardmen Hunting")
	public static NpcStringId LETO_LIZARDMEN_HUNTING;
	
	@ClientString(id = 30002, message = "Leto Lizardmen Hunting (In Progress)")
	public static NpcStringId LETO_LIZARDMEN_HUNTING_IN_PROGRESS;
	
	@ClientString(id = 30004, message = "Leto Lizardmen Hunting")
	public static NpcStringId LETO_LIZARDMEN_HUNTING2;
	
	@ClientString(id = 30301, message = "Collect Arrowheads")
	public static NpcStringId COLLECT_ARROWHEADS;
	
	@ClientString(id = 30302, message = "Collect Arrowheads (In Progress)")
	public static NpcStringId COLLECT_ARROWHEADS_IN_PROGRESS;
	
	@ClientString(id = 30601, message = "Crystals of Fire and Ice")
	public static NpcStringId CRYSTALS_OF_FIRE_AND_ICE;
	
	@ClientString(id = 30602, message = "Crystals of Fire and Ice (In Progress)")
	public static NpcStringId CRYSTALS_OF_FIRE_AND_ICE_IN_PROGRESS;
	
	@ClientString(id = 30701, message = "Control Device of the Giants (Lv. 79-99)")
	public static NpcStringId CONTROL_DEVICE_OF_THE_GIANTS_LV_79_99;
	
	@ClientString(id = 30702, message = "Control Device of the Giants (Lv. 79-99) (In Progress)")
	public static NpcStringId CONTROL_DEVICE_OF_THE_GIANTS_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 30704, message = "Control Device of the Giants (Lv. 79-99)")
	public static NpcStringId CONTROL_DEVICE_OF_THE_GIANTS_LV_79_992;
	
	@ClientString(id = 30801, message = "Maintaining the Field Business")
	public static NpcStringId MAINTAINING_THE_FIELD_BUSINESS;
	
	@ClientString(id = 30802, message = "Maintaining the Field Business (In Progress)")
	public static NpcStringId MAINTAINING_THE_FIELD_BUSINESS_IN_PROGRESS;
	
	@ClientString(id = 30804, message = "Maintaining the Field Business")
	public static NpcStringId MAINTAINING_THE_FIELD_BUSINESS2;
	
	@ClientString(id = 30901, message = "For a Good Cause")
	public static NpcStringId FOR_A_GOOD_CAUSE;
	
	@ClientString(id = 30902, message = "For a Good Cause (In Progress)")
	public static NpcStringId FOR_A_GOOD_CAUSE_IN_PROGRESS;
	
	@ClientString(id = 30904, message = "For a Good Cause")
	public static NpcStringId FOR_A_GOOD_CAUSE2;
	
	@ClientString(id = 31001, message = "Only What Remains (Lv. 81-99)")
	public static NpcStringId ONLY_WHAT_REMAINS_LV_81_99;
	
	@ClientString(id = 31002, message = "Only What Remains (Lv. 81-99) (In Progress)")
	public static NpcStringId ONLY_WHAT_REMAINS_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 31004, message = "Only What Remains (Lv. 81-99)")
	public static NpcStringId ONLY_WHAT_REMAINS_LV_81_992;
	
	@ClientString(id = 31101, message = "Expulsion of Evil Spirits")
	public static NpcStringId EXPULSION_OF_EVIL_SPIRITS;
	
	@ClientString(id = 31102, message = "Expulsion of Evil Spirits (In Progress)")
	public static NpcStringId EXPULSION_OF_EVIL_SPIRITS_IN_PROGRESS;
	
	@ClientString(id = 31104, message = "Expulsion of Evil Spirits")
	public static NpcStringId EXPULSION_OF_EVIL_SPIRITS2;
	
	@ClientString(id = 31201, message = "Take Advantage of the Crisis!")
	public static NpcStringId TAKE_ADVANTAGE_OF_THE_CRISIS;
	
	@ClientString(id = 31202, message = "Take Advantage of the Crisis! (In Progress)")
	public static NpcStringId TAKE_ADVANTAGE_OF_THE_CRISIS_IN_PROGRESS;
	
	@ClientString(id = 31204, message = "Take Advantage of the Crisis!")
	public static NpcStringId TAKE_ADVANTAGE_OF_THE_CRISIS2;
	
	@ClientString(id = 31301, message = "Collect Spores")
	public static NpcStringId COLLECT_SPORES;
	
	@ClientString(id = 31302, message = "Collect Spores (In Progress)")
	public static NpcStringId COLLECT_SPORES_IN_PROGRESS;
	
	@ClientString(id = 31601, message = "Destroy Plague Carriers")
	public static NpcStringId DESTROY_PLAGUE_CARRIERS;
	
	@ClientString(id = 31602, message = "Destroy Plague Carriers (In Progress)")
	public static NpcStringId DESTROY_PLAGUE_CARRIERS_IN_PROGRESS;
	
	@ClientString(id = 31603, message = "Why do you oppress us so?")
	public static NpcStringId WHY_DO_YOU_OPPRESS_US_SO;
	
	@ClientString(id = 31701, message = "Catch the Wind")
	public static NpcStringId CATCH_THE_WIND;
	
	@ClientString(id = 31702, message = "Catch the Wind (In Progress)")
	public static NpcStringId CATCH_THE_WIND_IN_PROGRESS;
	
	@ClientString(id = 31901, message = "Scent of Death")
	public static NpcStringId SCENT_OF_DEATH;
	
	@ClientString(id = 31902, message = "Scent of Death (In Progress)")
	public static NpcStringId SCENT_OF_DEATH_IN_PROGRESS;
	
	@ClientString(id = 32001, message = "Bones Tell the Future")
	public static NpcStringId BONES_TELL_THE_FUTURE;
	
	@ClientString(id = 32002, message = "Bones Tell the Future (In Progress)")
	public static NpcStringId BONES_TELL_THE_FUTURE_IN_PROGRESS;
	
	@ClientString(id = 32401, message = "Sweetest Venom")
	public static NpcStringId SWEETEST_VENOM;
	
	@ClientString(id = 32402, message = "Sweetest Venom (In Progress)")
	public static NpcStringId SWEETEST_VENOM_IN_PROGRESS;
	
	@ClientString(id = 32404, message = "Sweetest Venom")
	public static NpcStringId SWEETEST_VENOM2;
	
	@ClientString(id = 32501, message = "Grim Collector")
	public static NpcStringId GRIM_COLLECTOR;
	
	@ClientString(id = 32502, message = "Grim Collector (In Progress)")
	public static NpcStringId GRIM_COLLECTOR_IN_PROGRESS;
	
	@ClientString(id = 32504, message = "Grim Collector")
	public static NpcStringId GRIM_COLLECTOR2;
	
	@ClientString(id = 32601, message = "Vanquish Remnants")
	public static NpcStringId VANQUISH_REMNANTS;
	
	@ClientString(id = 32602, message = "Vanquish Remnants (In Progress)")
	public static NpcStringId VANQUISH_REMNANTS_IN_PROGRESS;
	
	@ClientString(id = 32701, message = "Recover the Farmland")
	public static NpcStringId RECOVER_THE_FARMLAND;
	
	@ClientString(id = 32702, message = "Recover the Farmland (In Progress)")
	public static NpcStringId RECOVER_THE_FARMLAND_IN_PROGRESS;
	
	@ClientString(id = 32801, message = "Sense for Business")
	public static NpcStringId SENSE_FOR_BUSINESS;
	
	@ClientString(id = 32802, message = "Sense for Business (In Progress)")
	public static NpcStringId SENSE_FOR_BUSINESS_IN_PROGRESS;
	
	@ClientString(id = 32804, message = "Sense for Business")
	public static NpcStringId SENSE_FOR_BUSINESS2;
	
	@ClientString(id = 32901, message = "Curiosity of a Dwarf")
	public static NpcStringId CURIOSITY_OF_A_DWARF;
	
	@ClientString(id = 32902, message = "Curiosity of a Dwarf (In Progress)")
	public static NpcStringId CURIOSITY_OF_A_DWARF_IN_PROGRESS;
	
	@ClientString(id = 32904, message = "Curiosity of a Dwarf")
	public static NpcStringId CURIOSITY_OF_A_DWARF2;
	
	@ClientString(id = 33001, message = "Adept of Taste")
	public static NpcStringId ADEPT_OF_TASTE;
	
	@ClientString(id = 33002, message = "Adept of Taste (In Progress)")
	public static NpcStringId ADEPT_OF_TASTE_IN_PROGRESS;
	
	@ClientString(id = 33004, message = "Adept of Taste")
	public static NpcStringId ADEPT_OF_TASTE2;
	
	@ClientString(id = 33101, message = "Arrow of Vengeance")
	public static NpcStringId ARROW_OF_VENGEANCE;
	
	@ClientString(id = 33102, message = "Arrow of Vengeance (In Progress)")
	public static NpcStringId ARROW_OF_VENGEANCE_IN_PROGRESS;
	
	@ClientString(id = 33104, message = "Arrow of Vengeance")
	public static NpcStringId ARROW_OF_VENGEANCE2;
	
	@ClientString(id = 33301, message = "Hunt of the Black Lion")
	public static NpcStringId HUNT_OF_THE_BLACK_LION;
	
	@ClientString(id = 33302, message = "Hunt of the Black Lion (In Progress)")
	public static NpcStringId HUNT_OF_THE_BLACK_LION_IN_PROGRESS;
	
	@ClientString(id = 33304, message = "Hunt of the Black Lion")
	public static NpcStringId HUNT_OF_THE_BLACK_LION2;
	
	@ClientString(id = 33401, message = "The Wishing Potion")
	public static NpcStringId THE_WISHING_POTION;
	
	@ClientString(id = 33402, message = "The Wishing Potion (In Progress)")
	public static NpcStringId THE_WISHING_POTION_IN_PROGRESS;
	
	@ClientString(id = 33403, message = "The Wishing Potion (Done)")
	public static NpcStringId THE_WISHING_POTION_DONE;
	
	@ClientString(id = 33404, message = "The Wishing Potion")
	public static NpcStringId THE_WISHING_POTION2;
	
	@ClientString(id = 33409, message = "Don't interrupt my rest again")
	public static NpcStringId DON_T_INTERRUPT_MY_REST_AGAIN;
	
	@ClientString(id = 33410, message = "You're a great devil now...")
	public static NpcStringId YOU_RE_A_GREAT_DEVIL_NOW;
	
	@ClientString(id = 33411, message = "Oh, it's not an opponent of mine. Ha, ha, ha!")
	public static NpcStringId OH_IT_S_NOT_AN_OPPONENT_OF_MINE_HA_HA_HA;
	
	@ClientString(id = 33412, message = "Oh... Great Demon King...")
	public static NpcStringId OH_GREAT_DEMON_KING;
	
	@ClientString(id = 33413, message = "Revenge is Overlord Ramsebalius of the evil world!")
	public static NpcStringId REVENGE_IS_OVERLORD_RAMSEBALIUS_OF_THE_EVIL_WORLD;
	
	@ClientString(id = 33414, message = "Bonaparterius, Abyss King, will punish you")
	public static NpcStringId BONAPARTERIUS_ABYSS_KING_WILL_PUNISH_YOU;
	
	@ClientString(id = 33415, message = "OK, everybody pray fervently!")
	public static NpcStringId OK_EVERYBODY_PRAY_FERVENTLY;
	
	@ClientString(id = 33416, message = "Both hands to heaven! Everybody yell together!")
	public static NpcStringId BOTH_HANDS_TO_HEAVEN_EVERYBODY_YELL_TOGETHER;
	
	@ClientString(id = 33417, message = "One! Two! May your dreams come true!")
	public static NpcStringId ONE_TWO_MAY_YOUR_DREAMS_COME_TRUE;
	
	@ClientString(id = 33418, message = "Who killed my underling devil?")
	public static NpcStringId WHO_KILLED_MY_UNDERLING_DEVIL;
	
	@ClientString(id = 33420, message = "I will make your love come true~ love, love, love~")
	public static NpcStringId I_WILL_MAKE_YOUR_LOVE_COME_TRUE_LOVE_LOVE_LOVE;
	
	@ClientString(id = 33421, message = "I have wisdom in me. I am the box of wisdom!")
	public static NpcStringId I_HAVE_WISDOM_IN_ME_I_AM_THE_BOX_OF_WISDOM;
	
	@ClientString(id = 33422, message = "Oh, oh, oh!")
	public static NpcStringId OH_OH_OH;
	
	@ClientString(id = 33423, message = "Do you want us to love you? Oh.")
	public static NpcStringId DO_YOU_WANT_US_TO_LOVE_YOU_OH;
	
	@ClientString(id = 33424, message = "Who is calling the Lord of Darkness!")
	public static NpcStringId WHO_IS_CALLING_THE_LORD_OF_DARKNESS;
	
	@ClientString(id = 33425, message = "I am a great empire, Bonaparterius!")
	public static NpcStringId I_AM_A_GREAT_EMPIRE_BONAPARTERIUS;
	
	@ClientString(id = 33426, message = "Let your head down before the Lord!")
	public static NpcStringId LET_YOUR_HEAD_DOWN_BEFORE_THE_LORD;
	
	@ClientString(id = 33501, message = "The Song of the Hunter")
	public static NpcStringId THE_SONG_OF_THE_HUNTER;
	
	@ClientString(id = 33502, message = "The Song of the Hunter  (In Progress)")
	public static NpcStringId THE_SONG_OF_THE_HUNTER_IN_PROGRESS;
	
	@ClientString(id = 33504, message = "The Song of the Hunter")
	public static NpcStringId THE_SONG_OF_THE_HUNTER2;
	
	@ClientString(id = 33511, message = "We'll take the property of the ancient empire!")
	public static NpcStringId WE_LL_TAKE_THE_PROPERTY_OF_THE_ANCIENT_EMPIRE;
	
	@ClientString(id = 33512, message = "Show me the pretty sparkling things! They're all mine!")
	public static NpcStringId SHOW_ME_THE_PRETTY_SPARKLING_THINGS_THEY_RE_ALL_MINE;
	
	@ClientString(id = 33513, message = "Pretty good!")
	public static NpcStringId PRETTY_GOOD;
	
	@ClientString(id = 33520, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=5')C: 40 Totems of Kadesh(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_5_C_40_TOTEMS_OF_KADESH_BUTTON;
	
	@ClientString(id = 33521, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=6')C: 50 Jade Necklaces of Timak(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_6_C_50_JADE_NECKLACES_OF_TIMAK_BUTTON;
	
	@ClientString(id = 33522, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=7')C: 50 Reinforced Golem Shards(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_7_C_50_REINFORCED_GOLEM_SHARDS_BUTTON;
	
	@ClientString(id = 33523, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=8')C: 30 Pieces Monster Eye Meat(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_8_C_30_PIECES_MONSTER_EYE_MEAT_BUTTON;
	
	@ClientString(id = 33524, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=9')C: 40 Eggs of Dire Wyrm(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_9_C_40_EGGS_OF_DIRE_WYRM_BUTTON;
	
	@ClientString(id = 33525, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=10')C: 100 Claws of Guardian Basilisk(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_10_C_100_CLAWS_OF_GUARDIAN_BASILISK_BUTTON;
	
	@ClientString(id = 33526, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=11')C: 50 Revenant Chains (/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_11_C_50_REVENANT_CHAINS_BUTTON;
	
	@ClientString(id = 33527, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=12')C: 30 Windsus Tusks(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_12_C_30_WINDSUS_TUSKS_BUTTON;
	
	@ClientString(id = 33528, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=13')C: 100 Skulls of Grandis(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_13_C_100_SKULLS_OF_GRANDIS_BUTTON;
	
	@ClientString(id = 33529, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=14')C: 50 Taik Obsidian Amulets(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_14_C_50_TAIK_OBSIDIAN_AMULETS_BUTTON;
	
	@ClientString(id = 33530, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=15')C: 30 Heads of Karul Bugbear(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_15_C_30_HEADS_OF_KARUL_BUGBEAR_BUTTON;
	
	@ClientString(id = 33531, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=16')C: 40 Ivory Charms of Tamlin(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_16_C_40_IVORY_CHARMS_OF_TAMLIN_BUTTON;
	
	@ClientString(id = 33532, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=17')B: Situation Preparation - Leto Chief(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_17_B_SITUATION_PREPARATION_LETO_CHIEF_BUTTON;
	
	@ClientString(id = 33533, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=18')B: 50 Reinforced Gargoyle Horns(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_18_B_50_REINFORCED_GARGOYLE_HORNS_BUTTON;
	
	@ClientString(id = 33534, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=19')B: 50 Coiled Serpent Totems(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_19_B_50_COILED_SERPENT_TOTEMS_BUTTON;
	
	@ClientString(id = 33535, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=20')B: Situation Preparation - Sorcerer Catch of Leto(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_20_B_SITUATION_PREPARATION_SORCERER_CATCH_OF_LETO_BUTTON;
	
	@ClientString(id = 33536, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=21')B: Situation Preparation - Timak Raider Kaikee(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_21_B_SITUATION_PREPARATION_TIMAK_RAIDER_KAIKEE_BUTTON;
	
	@ClientString(id = 33537, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=22')B: 30 Kronbe Venom Sacs(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_22_B_30_KRONBE_VENOM_SACS_BUTTON;
	
	@ClientString(id = 33538, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=24')A: Lesser Giant's Tablet(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_24_A_LESSER_GIANT_S_TABLET_BUTTON;
	
	@ClientString(id = 33539, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=25')A: Book of Shunaiman(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_25_A_BOOK_OF_SHUNAIMAN_BUTTON;
	
	@ClientString(id = 33540, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=27')C: 40 Trisalim Venom Sacs(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_27_C_40_TRISALIM_VENOM_SACS_BUTTON;
	
	@ClientString(id = 33541, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=28')C: 50 Totems of Taik Orc(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_28_C_50_TOTEMS_OF_TAIK_ORC_BUTTON;
	
	@ClientString(id = 33542, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=29')C: 40 Harit Barbed Necklaces(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_29_C_40_HARIT_BARBED_NECKLACES_BUTTON;
	
	@ClientString(id = 33543, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=30')C: 20 Coins of Ancient Empire(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_30_C_20_COINS_OF_ANCIENT_EMPIRE_BUTTON;
	
	@ClientString(id = 33544, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=31')C: 30 Skins of Farcran(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_31_C_30_SKINS_OF_FARCRAN_BUTTON;
	
	@ClientString(id = 33545, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=32')C: 40 Tempest Shards(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_32_C_40_TEMPEST_SHARDS_BUTTON;
	
	@ClientString(id = 33546, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=36')C: 40 Tsunami Shards(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_36_C_40_TSUNAMI_SHARDS_BUTTON;
	
	@ClientString(id = 33547, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=34')C: 40 Manes of Pan Ruem(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_34_C_40_MANES_OF_PAN_RUEM_BUTTON;
	
	@ClientString(id = 33548, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=36')C: 30 Manes of Vanor Silenos(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_36_C_30_MANES_OF_VANOR_SILENOS_BUTTON;
	
	@ClientString(id = 33549, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=37')C: 30 Totems of Tarlk Bugbears(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_37_C_30_TOTEMS_OF_TARLK_BUGBEARS_BUTTON;
	
	@ClientString(id = 33550, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=38')B: Situation Preparation - Overlord Okun of Timak(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_38_B_SITUATION_PREPARATION_OVERLORD_OKUN_OF_TIMAK_BUTTON;
	
	@ClientString(id = 33551, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=39')B: Situation Preparation - Overlord Kakran of Taik(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_39_B_SITUATION_PREPARATION_OVERLORD_KAKRAN_OF_TAIK_BUTTON;
	
	@ClientString(id = 33552, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=40')B: 40 Narcissus Soulstones(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_40_B_40_NARCISSUS_SOULSTONES_BUTTON;
	
	@ClientString(id = 33553, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=41')B: 20 Eyes of Deprived(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_41_B_20_EYES_OF_DEPRIVED_BUTTON;
	
	@ClientString(id = 33554, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=42')B: 20 Unicorn Horns(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_42_B_20_UNICORN_HORNS_BUTTON;
	
	@ClientString(id = 33555, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=43')B: Silenos Golden Mane(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_43_B_SILENOS_GOLDEN_MANE_BUTTON;
	
	@ClientString(id = 33556, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=44')A: 20 Skulls of the Executed(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_44_A_20_SKULLS_OF_THE_EXECUTED_BUTTON;
	
	@ClientString(id = 33557, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=45')A: Bust of Travis(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_45_A_BUST_OF_TRAVIS_BUTTON;
	
	@ClientString(id = 33558, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=335andreply=46')A: 10 Swords of Cadmus(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_335ANDREPLY_46_A_10_SWORDS_OF_CADMUS_BUTTON;
	
	@ClientString(id = 33601, message = "Coins of Magic (Lv. 40-60)")
	public static NpcStringId COINS_OF_MAGIC_LV_40_60;
	
	@ClientString(id = 33602, message = "Coins of Magic (Lv. 40-60) (In Progress)")
	public static NpcStringId COINS_OF_MAGIC_LV_40_60_IN_PROGRESS;
	
	@ClientString(id = 33604, message = "Coins of Magic (Lv. 40-60)")
	public static NpcStringId COINS_OF_MAGIC_LV_40_602;
	
	@ClientString(id = 33701, message = "Audience with the Land Dragon (Lv. 50-64)")
	public static NpcStringId AUDIENCE_WITH_THE_LAND_DRAGON_LV_50_64;
	
	@ClientString(id = 33702, message = "Audience with the Land Dragon (Lv. 50-64) (In Progress)")
	public static NpcStringId AUDIENCE_WITH_THE_LAND_DRAGON_LV_50_64_IN_PROGRESS;
	
	@ClientString(id = 33704, message = "Audience with the Land Dragon (Lv. 50-64)")
	public static NpcStringId AUDIENCE_WITH_THE_LAND_DRAGON_LV_50_642;
	
	@ClientString(id = 33801, message = "Alligator Hunter")
	public static NpcStringId ALLIGATOR_HUNTER;
	
	@ClientString(id = 33802, message = "Alligator Hunter (In Progress)")
	public static NpcStringId ALLIGATOR_HUNTER_IN_PROGRESS;
	
	@ClientString(id = 33804, message = "Alligator Hunter")
	public static NpcStringId ALLIGATOR_HUNTER2;
	
	@ClientString(id = 34001, message = "Subjugation of Lizardmen")
	public static NpcStringId SUBJUGATION_OF_LIZARDMEN;
	
	@ClientString(id = 34002, message = "Subjugation of Lizardmen (In Progress)")
	public static NpcStringId SUBJUGATION_OF_LIZARDMEN_IN_PROGRESS;
	
	@ClientString(id = 34004, message = "Subjugation of Lizardmen")
	public static NpcStringId SUBJUGATION_OF_LIZARDMEN2;
	
	@ClientString(id = 34101, message = "Hunting for Wild Beasts")
	public static NpcStringId HUNTING_FOR_WILD_BEASTS;
	
	@ClientString(id = 34102, message = "Hunting for Wild Beasts (In Progress)")
	public static NpcStringId HUNTING_FOR_WILD_BEASTS_IN_PROGRESS;
	
	@ClientString(id = 34104, message = "Hunting for Wild Beasts")
	public static NpcStringId HUNTING_FOR_WILD_BEASTS2;
	
	@ClientString(id = 34301, message = "Under the Shadow of the Ivory Tower")
	public static NpcStringId UNDER_THE_SHADOW_OF_THE_IVORY_TOWER;
	
	@ClientString(id = 34302, message = "Under the Shadow of the Ivory Tower (In Progress)")
	public static NpcStringId UNDER_THE_SHADOW_OF_THE_IVORY_TOWER_IN_PROGRESS;
	
	@ClientString(id = 34304, message = "Under the Shadow of the Ivory Tower")
	public static NpcStringId UNDER_THE_SHADOW_OF_THE_IVORY_TOWER2;
	
	@ClientString(id = 34401, message = "1000 Years, the End of Lamentation (Lv. 48-55)")
	public static NpcStringId YEARS_THE_END_OF_LAMENTATION_LV_48_55;
	
	@ClientString(id = 34402, message = "1000 Years, the End of Lamentation (Lv. 48-55) (In Progress)")
	public static NpcStringId YEARS_THE_END_OF_LAMENTATION_LV_48_55_IN_PROGRESS;
	
	@ClientString(id = 34404, message = "1000 years, the End of Lamentation (Lv. 48-55)")
	public static NpcStringId YEARS_THE_END_OF_LAMENTATION_LV_48_552;
	
	@ClientString(id = 34501, message = "Method to Raise the Dead")
	public static NpcStringId METHOD_TO_RAISE_THE_DEAD;
	
	@ClientString(id = 34502, message = "Method to Raise the Dead (In Progress)")
	public static NpcStringId METHOD_TO_RAISE_THE_DEAD_IN_PROGRESS;
	
	@ClientString(id = 34504, message = "Method to Raise the Dead")
	public static NpcStringId METHOD_TO_RAISE_THE_DEAD2;
	
	@ClientString(id = 34701, message = "Go Get the Calculator")
	public static NpcStringId GO_GET_THE_CALCULATOR;
	
	@ClientString(id = 34702, message = "Go Get the Calculator (In Progress)")
	public static NpcStringId GO_GET_THE_CALCULATOR_IN_PROGRESS;
	
	@ClientString(id = 34704, message = "Go Get the Calculator")
	public static NpcStringId GO_GET_THE_CALCULATOR2;
	
	@ClientString(id = 34801, message = "An Arrogant Search")
	public static NpcStringId AN_ARROGANT_SEARCH;
	
	@ClientString(id = 34802, message = "An Arrogant Search (In Progress)")
	public static NpcStringId AN_ARROGANT_SEARCH_IN_PROGRESS;
	
	@ClientString(id = 34804, message = "An Arrogant Search")
	public static NpcStringId AN_ARROGANT_SEARCH2;
	
	@ClientString(id = 34830, message = "Ha, that was fun! If you wish to find the key, search the corpse.")
	public static NpcStringId HA_THAT_WAS_FUN_IF_YOU_WISH_TO_FIND_THE_KEY_SEARCH_THE_CORPSE;
	
	@ClientString(id = 34831, message = "I have the key. Why don't you come and take it?")
	public static NpcStringId I_HAVE_THE_KEY_WHY_DON_T_YOU_COME_AND_TAKE_IT;
	
	@ClientString(id = 34832, message = "You fools will get what's coming to you!")
	public static NpcStringId YOU_FOOLS_WILL_GET_WHAT_S_COMING_TO_YOU;
	
	@ClientString(id = 34833, message = "Sorry about this, but I must kill you now.")
	public static NpcStringId SORRY_ABOUT_THIS_BUT_I_MUST_KILL_YOU_NOW;
	
	@ClientString(id = 34834, message = "I have the key. Why don't you come and take it?")
	public static NpcStringId I_HAVE_THE_KEY_WHY_DON_T_YOU_COME_AND_TAKE_IT2;
	
	@ClientString(id = 34835, message = "You guys wouldn't know... the seven seals are...  Arrrgh!")
	public static NpcStringId YOU_GUYS_WOULDN_T_KNOW_THE_SEVEN_SEALS_ARE_ARRRGH;
	
	@ClientString(id = 34836, message = "I shall drench this mountain with your blood!")
	public static NpcStringId I_SHALL_DRENCH_THIS_MOUNTAIN_WITH_YOUR_BLOOD;
	
	@ClientString(id = 34837, message = "That doesn't belong to you. Don't touch it!")
	public static NpcStringId THAT_DOESN_T_BELONG_TO_YOU_DON_T_TOUCH_IT;
	
	@ClientString(id = 34838, message = "Get out of my sight, you infidels!")
	public static NpcStringId GET_OUT_OF_MY_SIGHT_YOU_INFIDELS;
	
	@ClientString(id = 34839, message = "We don't have any further business to discuss... Have you searched the corpse for the key?")
	public static NpcStringId WE_DON_T_HAVE_ANY_FURTHER_BUSINESS_TO_DISCUSS_HAVE_YOU_SEARCHED_THE_CORPSE_FOR_THE_KEY;
	
	@ClientString(id = 35001, message = "Enhance Your Weapon (Lv. 40-80)")
	public static NpcStringId ENHANCE_YOUR_WEAPON_LV_40_80;
	
	@ClientString(id = 35002, message = "Enhance Your Weapon (Lv. 40-80) (In Progress)")
	public static NpcStringId ENHANCE_YOUR_WEAPON_LV_40_80_IN_PROGRESS;
	
	@ClientString(id = 35004, message = "Enhance Your Weapon (Lv. 40-80)")
	public static NpcStringId ENHANCE_YOUR_WEAPON_LV_40_802;
	
	@ClientString(id = 35051, message = "$s1 has earned a stage $s2 Blue Soul Crystal.")
	public static NpcStringId S1_HAS_EARNED_A_STAGE_S2_BLUE_SOUL_CRYSTAL;
	
	@ClientString(id = 35052, message = "$s1 has earned a stage $s2 Red Soul Crystal.")
	public static NpcStringId S1_HAS_EARNED_A_STAGE_S2_RED_SOUL_CRYSTAL;
	
	@ClientString(id = 35053, message = "$s1 has earned a stage $s2 Green Soul Crystal.")
	public static NpcStringId S1_HAS_EARNED_A_STAGE_S2_GREEN_SOUL_CRYSTAL;
	
	@ClientString(id = 35054, message = "$s1 has earned a Stage $s2 Blue Cursed Soul Crystal.")
	public static NpcStringId S1_HAS_EARNED_A_STAGE_S2_BLUE_CURSED_SOUL_CRYSTAL;
	
	@ClientString(id = 35055, message = "$s1 has earned a Stage $s2 Red Cursed Soul Crystal.")
	public static NpcStringId S1_HAS_EARNED_A_STAGE_S2_RED_CURSED_SOUL_CRYSTAL;
	
	@ClientString(id = 35056, message = "$s1 has earned a Stage $s2 Green Cursed Soul Crystal.")
	public static NpcStringId S1_HAS_EARNED_A_STAGE_S2_GREEN_CURSED_SOUL_CRYSTAL;
	
	@ClientString(id = 35101, message = "Black Swan")
	public static NpcStringId BLACK_SWAN;
	
	@ClientString(id = 35102, message = "Black Swan (In Progress)")
	public static NpcStringId BLACK_SWAN_IN_PROGRESS;
	
	@ClientString(id = 35104, message = "Black Swan")
	public static NpcStringId BLACK_SWAN2;
	
	@ClientString(id = 35201, message = "Help Rood Raise A New Pet!")
	public static NpcStringId HELP_ROOD_RAISE_A_NEW_PET;
	
	@ClientString(id = 35202, message = "Help Rood Raise A New Pet! (In Progress)")
	public static NpcStringId HELP_ROOD_RAISE_A_NEW_PET_IN_PROGRESS;
	
	@ClientString(id = 35204, message = "Help Rood Raise A New Pet!")
	public static NpcStringId HELP_ROOD_RAISE_A_NEW_PET2;
	
	@ClientString(id = 35301, message = "Power of Darkness (Lv. 55-60)")
	public static NpcStringId POWER_OF_DARKNESS_LV_55_60;
	
	@ClientString(id = 35302, message = "Power of Darkness (Lv. 55-60) (In Progress)")
	public static NpcStringId POWER_OF_DARKNESS_LV_55_60_IN_PROGRESS;
	
	@ClientString(id = 35401, message = "Conquest of Alligator Island")
	public static NpcStringId CONQUEST_OF_ALLIGATOR_ISLAND;
	
	@ClientString(id = 35402, message = "Conquest of Alligator Island (In Progress)")
	public static NpcStringId CONQUEST_OF_ALLIGATOR_ISLAND_IN_PROGRESS;
	
	@ClientString(id = 35404, message = "Conquest of Alligator Island")
	public static NpcStringId CONQUEST_OF_ALLIGATOR_ISLAND2;
	
	@ClientString(id = 35501, message = "Family Honor")
	public static NpcStringId FAMILY_HONOR;
	
	@ClientString(id = 35502, message = "Family Honor (In Progress)")
	public static NpcStringId FAMILY_HONOR_IN_PROGRESS;
	
	@ClientString(id = 35504, message = "Family Honor")
	public static NpcStringId FAMILY_HONOR2;
	
	@ClientString(id = 35601, message = "Dig Up the Sea of Spores!")
	public static NpcStringId DIG_UP_THE_SEA_OF_SPORES;
	
	@ClientString(id = 35602, message = "Dig Up the Sea of Spores! (In Progress)")
	public static NpcStringId DIG_UP_THE_SEA_OF_SPORES_IN_PROGRESS;
	
	@ClientString(id = 35701, message = "Warehouse Keeper's Ambition (Lv. 47-57)")
	public static NpcStringId WAREHOUSE_KEEPER_S_AMBITION_LV_47_57;
	
	@ClientString(id = 35702, message = "Warehouse Keeper's Ambition (Lv. 47-57) (In Progress)")
	public static NpcStringId WAREHOUSE_KEEPER_S_AMBITION_LV_47_57_IN_PROGRESS;
	
	@ClientString(id = 35704, message = "Warehouse Keeper's Ambition (Lv. 47-57)")
	public static NpcStringId WAREHOUSE_KEEPER_S_AMBITION_LV_47_572;
	
	@ClientString(id = 35801, message = "Illegitimate Child of a Goddess")
	public static NpcStringId ILLEGITIMATE_CHILD_OF_A_GODDESS;
	
	@ClientString(id = 35802, message = "Illegitimate Child of a Goddess (In Progress)")
	public static NpcStringId ILLEGITIMATE_CHILD_OF_A_GODDESS_IN_PROGRESS;
	
	@ClientString(id = 35804, message = "Illegitimate Child of a Goddess")
	public static NpcStringId ILLEGITIMATE_CHILD_OF_A_GODDESS2;
	
	@ClientString(id = 35901, message = "For a Sleepless Deadman (Lv. 60-67)")
	public static NpcStringId FOR_A_SLEEPLESS_DEADMAN_LV_60_67;
	
	@ClientString(id = 35902, message = "For a Sleepless Deadman (Lv. 60-67) (In Progress)")
	public static NpcStringId FOR_A_SLEEPLESS_DEADMAN_LV_60_67_IN_PROGRESS;
	
	@ClientString(id = 35904, message = "For a Sleepless Deadman (Lv. 60-67)")
	public static NpcStringId FOR_A_SLEEPLESS_DEADMAN_LV_60_672;
	
	@ClientString(id = 36001, message = "Plunder the Supplies")
	public static NpcStringId PLUNDER_THE_SUPPLIES;
	
	@ClientString(id = 36002, message = "Plunder the Supplies (In Progress)")
	public static NpcStringId PLUNDER_THE_SUPPLIES_IN_PROGRESS;
	
	@ClientString(id = 36004, message = "Plunder the Supplies")
	public static NpcStringId PLUNDER_THE_SUPPLIES2;
	
	@ClientString(id = 36201, message = "Bard's Mandolin")
	public static NpcStringId BARD_S_MANDOLIN;
	
	@ClientString(id = 36202, message = "Bard's Mandolin (In Progress)")
	public static NpcStringId BARD_S_MANDOLIN_IN_PROGRESS;
	
	@ClientString(id = 36204, message = "Bard's Mandolin")
	public static NpcStringId BARD_S_MANDOLIN2;
	
	@ClientString(id = 36301, message = "Sorrowful Sound of Flute")
	public static NpcStringId SORROWFUL_SOUND_OF_FLUTE;
	
	@ClientString(id = 36302, message = "Sorrowful Sound of Flute (In Progress)")
	public static NpcStringId SORROWFUL_SOUND_OF_FLUTE_IN_PROGRESS;
	
	@ClientString(id = 36304, message = "Sorrowful Sound of Flute")
	public static NpcStringId SORROWFUL_SOUND_OF_FLUTE2;
	
	@ClientString(id = 36401, message = "Jovial Accordion")
	public static NpcStringId JOVIAL_ACCORDION;
	
	@ClientString(id = 36402, message = "Jovial Accordion (In Progress)")
	public static NpcStringId JOVIAL_ACCORDION_IN_PROGRESS;
	
	@ClientString(id = 36404, message = "Jovial Accordion")
	public static NpcStringId JOVIAL_ACCORDION2;
	
	@ClientString(id = 36501, message = "Demon's Legacy")
	public static NpcStringId DEMON_S_LEGACY;
	
	@ClientString(id = 36502, message = "Demon's Legacy (In Progress)")
	public static NpcStringId DEMON_S_LEGACY_IN_PROGRESS;
	
	@ClientString(id = 36504, message = "Demon's Legacy")
	public static NpcStringId DEMON_S_LEGACY2;
	
	@ClientString(id = 36601, message = "Silver Haired Shaman (Lv. 48-58)")
	public static NpcStringId SILVER_HAIRED_SHAMAN_LV_48_58;
	
	@ClientString(id = 36602, message = "Silver Haired Shaman (Lv. 48-58) (In Progress)")
	public static NpcStringId SILVER_HAIRED_SHAMAN_LV_48_58_IN_PROGRESS;
	
	@ClientString(id = 36604, message = "Silver Haired Shaman (Lv. 48-58)")
	public static NpcStringId SILVER_HAIRED_SHAMAN_LV_48_582;
	
	@ClientString(id = 36701, message = "Electrifying Recharge!")
	public static NpcStringId ELECTRIFYING_RECHARGE;
	
	@ClientString(id = 36702, message = "Electrifying Recharge! (In Progress)")
	public static NpcStringId ELECTRIFYING_RECHARGE_IN_PROGRESS;
	
	@ClientString(id = 36704, message = "Electrifying Recharge!")
	public static NpcStringId ELECTRIFYING_RECHARGE2;
	
	@ClientString(id = 36801, message = "Trespassing into the Holy Ground")
	public static NpcStringId TRESPASSING_INTO_THE_HOLY_GROUND;
	
	@ClientString(id = 36802, message = "Trespassing into the Holy Ground (In Progress)")
	public static NpcStringId TRESPASSING_INTO_THE_HOLY_GROUND_IN_PROGRESS;
	
	@ClientString(id = 36804, message = "Trespassing into the Holy Ground")
	public static NpcStringId TRESPASSING_INTO_THE_HOLY_GROUND2;
	
	@ClientString(id = 36901, message = "Collector of Jewels")
	public static NpcStringId COLLECTOR_OF_JEWELS;
	
	@ClientString(id = 36902, message = "Collector of Jewels (In Progress)")
	public static NpcStringId COLLECTOR_OF_JEWELS_IN_PROGRESS;
	
	@ClientString(id = 36904, message = "Collector of Jewels")
	public static NpcStringId COLLECTOR_OF_JEWELS2;
	
	@ClientString(id = 37001, message = "An Elder Sows Seeds")
	public static NpcStringId AN_ELDER_SOWS_SEEDS;
	
	@ClientString(id = 37002, message = "An Elder Sows Seeds (In Progress)")
	public static NpcStringId AN_ELDER_SOWS_SEEDS_IN_PROGRESS;
	
	@ClientString(id = 37004, message = "An Elder Sows Seeds")
	public static NpcStringId AN_ELDER_SOWS_SEEDS2;
	
	@ClientString(id = 37101, message = "Shrieks of Ghosts (Lv. 59-71)")
	public static NpcStringId SHRIEKS_OF_GHOSTS_LV_59_71;
	
	@ClientString(id = 37102, message = "Shrieks of Ghosts (Lv. 59-71)")
	public static NpcStringId SHRIEKS_OF_GHOSTS_LV_59_712;
	
	@ClientString(id = 37104, message = "Shrieks of Ghosts (Lv. 59-71)")
	public static NpcStringId SHRIEKS_OF_GHOSTS_LV_59_713;
	
	@ClientString(id = 37201, message = "Legacy of Insolence (Lv. 59-75)")
	public static NpcStringId LEGACY_OF_INSOLENCE_LV_59_75;
	
	@ClientString(id = 37202, message = "Legacy of Insolence (Lv. 59-75) (In Progress)")
	public static NpcStringId LEGACY_OF_INSOLENCE_LV_59_75_IN_PROGRESS;
	
	@ClientString(id = 37204, message = "Legacy of Insolence (Lv. 59-75)")
	public static NpcStringId LEGACY_OF_INSOLENCE_LV_59_752;
	
	@ClientString(id = 37301, message = "Supplier of Reagents (Lv. 57-75)")
	public static NpcStringId SUPPLIER_OF_REAGENTS_LV_57_75;
	
	@ClientString(id = 37302, message = "Supplier of Reagents (Lv. 57-75) (In Progress)")
	public static NpcStringId SUPPLIER_OF_REAGENTS_LV_57_75_IN_PROGRESS;
	
	@ClientString(id = 37304, message = "Supplier of Reagents (Lv. 57-75)")
	public static NpcStringId SUPPLIER_OF_REAGENTS_LV_57_752;
	
	@ClientString(id = 37401, message = "Whisper of Dreams - Part 1 (Lv. 56-66)")
	public static NpcStringId WHISPER_OF_DREAMS_PART_1_LV_56_66;
	
	@ClientString(id = 37402, message = "Whisper of Dreams - Part 1 (Lv. 56-66) (In Progress)")
	public static NpcStringId WHISPER_OF_DREAMS_PART_1_LV_56_66_IN_PROGRESS;
	
	@ClientString(id = 37501, message = "Whisper of Dreams - Part 2 (Lv. 60-74)")
	public static NpcStringId WHISPER_OF_DREAMS_PART_2_LV_60_74;
	
	@ClientString(id = 37502, message = "Whisper of Dreams - Part 2 (Lv. 60-74) (In Progress)")
	public static NpcStringId WHISPER_OF_DREAMS_PART_2_LV_60_74_IN_PROGRESS;
	
	@ClientString(id = 37601, message = "Exploration of the Giants' Cave - Part 1 (Lv. 79-99)")
	public static NpcStringId EXPLORATION_OF_THE_GIANTS_CAVE_PART_1_LV_79_99;
	
	@ClientString(id = 37602, message = "Exploration of the Giants' Cave - Part 1 (Lv. 79-99) (In Progress)")
	public static NpcStringId EXPLORATION_OF_THE_GIANTS_CAVE_PART_1_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 37604, message = "Exploration of the Giants' Cave - Part 1 (Lv. 79-99)")
	public static NpcStringId EXPLORATION_OF_THE_GIANTS_CAVE_PART_1_LV_79_992;
	
	@ClientString(id = 37701, message = "Exploration of the Giants' Cave - Part 2 (Lv. 79-99)")
	public static NpcStringId EXPLORATION_OF_THE_GIANTS_CAVE_PART_2_LV_79_99;
	
	@ClientString(id = 37702, message = "Exploration of the Giants' Cave - Part 2 (Lv. 79-99) (In Progress)")
	public static NpcStringId EXPLORATION_OF_THE_GIANTS_CAVE_PART_2_LV_79_99_IN_PROGRESS;
	
	@ClientString(id = 37704, message = "Exploration of the Giants' Cave - Part 2 (Lv. 79-99)")
	public static NpcStringId EXPLORATION_OF_THE_GIANTS_CAVE_PART_2_LV_79_992;
	
	@ClientString(id = 37801, message = "Grand Feast")
	public static NpcStringId GRAND_FEAST;
	
	@ClientString(id = 37802, message = "Grand Feast (In Progress)")
	public static NpcStringId GRAND_FEAST_IN_PROGRESS;
	
	@ClientString(id = 37804, message = "Grand Feast")
	public static NpcStringId GRAND_FEAST2;
	
	@ClientString(id = 37901, message = "Fantasy Wine")
	public static NpcStringId FANTASY_WINE;
	
	@ClientString(id = 37902, message = "Fantasy Wine (In Progress)")
	public static NpcStringId FANTASY_WINE_IN_PROGRESS;
	
	@ClientString(id = 37904, message = "Fantasy Wine")
	public static NpcStringId FANTASY_WINE2;
	
	@ClientString(id = 38001, message = "Bring Out the Flavor of Ingredients!")
	public static NpcStringId BRING_OUT_THE_FLAVOR_OF_INGREDIENTS;
	
	@ClientString(id = 38002, message = "Bring Out the Flavor of Ingredients! (In Progress)")
	public static NpcStringId BRING_OUT_THE_FLAVOR_OF_INGREDIENTS_IN_PROGRESS;
	
	@ClientString(id = 38004, message = "Bring Out the Flavor of Ingredients!")
	public static NpcStringId BRING_OUT_THE_FLAVOR_OF_INGREDIENTS2;
	
	@ClientString(id = 38101, message = "Let's Become a Royal Member! (Lv. 55-65)")
	public static NpcStringId LET_S_BECOME_A_ROYAL_MEMBER_LV_55_65;
	
	@ClientString(id = 38102, message = "Let's Become a Royal Member! (Lv. 55-65) (In Progress)")
	public static NpcStringId LET_S_BECOME_A_ROYAL_MEMBER_LV_55_65_IN_PROGRESS;
	
	@ClientString(id = 38104, message = "Let's Become a Royal Member! (Lv. 55-65)")
	public static NpcStringId LET_S_BECOME_A_ROYAL_MEMBER_LV_55_652;
	
	@ClientString(id = 38201, message = "Kail's Magic Coin (Lv. 55-60)")
	public static NpcStringId KAIL_S_MAGIC_COIN_LV_55_60;
	
	@ClientString(id = 38202, message = "Kail's Magic Coin (Lv. 55-60) (In Progress)")
	public static NpcStringId KAIL_S_MAGIC_COIN_LV_55_60_IN_PROGRESS;
	
	@ClientString(id = 38204, message = "Kail's Magic Coin (Lv. 55-60)")
	public static NpcStringId KAIL_S_MAGIC_COIN_LV_55_602;
	
	@ClientString(id = 38301, message = "Treasure Hunt")
	public static NpcStringId TREASURE_HUNT;
	
	@ClientString(id = 38302, message = "Treasure Hunt (In Progress)")
	public static NpcStringId TREASURE_HUNT_IN_PROGRESS;
	
	@ClientString(id = 38304, message = "Treasure Hunt")
	public static NpcStringId TREASURE_HUNT2;
	
	@ClientString(id = 38401, message = "Warehouse Keeper's Pastime")
	public static NpcStringId WAREHOUSE_KEEPER_S_PASTIME;
	
	@ClientString(id = 38402, message = "Warehouse Keeper's Pastime (In Progress)")
	public static NpcStringId WAREHOUSE_KEEPER_S_PASTIME_IN_PROGRESS;
	
	@ClientString(id = 38404, message = "Warehouse Keeper's Pastime")
	public static NpcStringId WAREHOUSE_KEEPER_S_PASTIME2;
	
	@ClientString(id = 38451, message = "Slot $s1: $s2")
	public static NpcStringId SLOT_S1_S2;
	
	@ClientString(id = 38501, message = "Yoke of the Past")
	public static NpcStringId YOKE_OF_THE_PAST;
	
	@ClientString(id = 38502, message = "Yoke of the Past (In Progress)")
	public static NpcStringId YOKE_OF_THE_PAST_IN_PROGRESS;
	
	@ClientString(id = 38504, message = "Yoke of the Past")
	public static NpcStringId YOKE_OF_THE_PAST2;
	
	@ClientString(id = 38601, message = "Stolen Dignity (Lv. 58-75)")
	public static NpcStringId STOLEN_DIGNITY_LV_58_75;
	
	@ClientString(id = 38602, message = "Stolen Dignity (Lv. 58-75) (In Progress)")
	public static NpcStringId STOLEN_DIGNITY_LV_58_75_IN_PROGRESS;
	
	@ClientString(id = 38604, message = "Stolen Dignity (Lv. 58-75)")
	public static NpcStringId STOLEN_DIGNITY_LV_58_752;
	
	@ClientString(id = 40101, message = "Path of the Warrior")
	public static NpcStringId PATH_OF_THE_WARRIOR;
	
	@ClientString(id = 40102, message = "Path of the Warrior (In Progress)")
	public static NpcStringId PATH_OF_THE_WARRIOR_IN_PROGRESS;
	
	@ClientString(id = 40201, message = "Path of the Human Knight")
	public static NpcStringId PATH_OF_THE_HUMAN_KNIGHT;
	
	@ClientString(id = 40202, message = "Path of the Human Knight (In Progress)")
	public static NpcStringId PATH_OF_THE_HUMAN_KNIGHT_IN_PROGRESS;
	
	@ClientString(id = 40301, message = "Path of the Rogue")
	public static NpcStringId PATH_OF_THE_ROGUE;
	
	@ClientString(id = 40302, message = "Path of the Rogue (In Progress)")
	public static NpcStringId PATH_OF_THE_ROGUE_IN_PROGRESS;
	
	@ClientString(id = 40306, message = "You childish fool, do you think you can catch me?")
	public static NpcStringId YOU_CHILDISH_FOOL_DO_YOU_THINK_YOU_CAN_CATCH_ME;
	
	@ClientString(id = 40307, message = "I must do something about this shameful incident...")
	public static NpcStringId I_MUST_DO_SOMETHING_ABOUT_THIS_SHAMEFUL_INCIDENT;
	
	@ClientString(id = 40308, message = "What, do you dare to challenge me!")
	public static NpcStringId WHAT_DO_YOU_DARE_TO_CHALLENGE_ME;
	
	@ClientString(id = 40309, message = "The red-eyed thieves will revenge!")
	public static NpcStringId THE_RED_EYED_THIEVES_WILL_REVENGE;
	
	@ClientString(id = 40310, message = "Go ahead, you child!")
	public static NpcStringId GO_AHEAD_YOU_CHILD;
	
	@ClientString(id = 40311, message = "My friends are sure to revenge!")
	public static NpcStringId MY_FRIENDS_ARE_SURE_TO_REVENGE;
	
	@ClientString(id = 40312, message = "You ruthless fool, I will show you what real fighting is all about!")
	public static NpcStringId YOU_RUTHLESS_FOOL_I_WILL_SHOW_YOU_WHAT_REAL_FIGHTING_IS_ALL_ABOUT;
	
	@ClientString(id = 40313, message = "Ahh, how can it end like this... it is not fair!")
	public static NpcStringId AHH_HOW_CAN_IT_END_LIKE_THIS_IT_IS_NOT_FAIR;
	
	@ClientString(id = 40401, message = "Path of the Human Wizard")
	public static NpcStringId PATH_OF_THE_HUMAN_WIZARD;
	
	@ClientString(id = 40402, message = "Path of the Human Wizard (In Progress)")
	public static NpcStringId PATH_OF_THE_HUMAN_WIZARD_IN_PROGRESS;
	
	@ClientString(id = 40501, message = "Path of the Cleric")
	public static NpcStringId PATH_OF_THE_CLERIC;
	
	@ClientString(id = 40502, message = "Path of the Cleric (In Progress)")
	public static NpcStringId PATH_OF_THE_CLERIC_IN_PROGRESS;
	
	@ClientString(id = 40601, message = "Path of the Elven Knight")
	public static NpcStringId PATH_OF_THE_ELVEN_KNIGHT;
	
	@ClientString(id = 40602, message = "Path of the Elven Knight (In Progress)")
	public static NpcStringId PATH_OF_THE_ELVEN_KNIGHT_IN_PROGRESS;
	
	@ClientString(id = 40701, message = "Path of the Elven Scout")
	public static NpcStringId PATH_OF_THE_ELVEN_SCOUT;
	
	@ClientString(id = 40702, message = "Path of the Elven Scout (In Progress)")
	public static NpcStringId PATH_OF_THE_ELVEN_SCOUT_IN_PROGRESS;
	
	@ClientString(id = 40801, message = "Path of the Elven Wizard")
	public static NpcStringId PATH_OF_THE_ELVEN_WIZARD;
	
	@ClientString(id = 40802, message = "Path of the Elven Wizard (In Progress)")
	public static NpcStringId PATH_OF_THE_ELVEN_WIZARD_IN_PROGRESS;
	
	@ClientString(id = 40901, message = "Path of the Elven Oracle")
	public static NpcStringId PATH_OF_THE_ELVEN_ORACLE;
	
	@ClientString(id = 40902, message = "Path of the Elven Oracle (In Progress)")
	public static NpcStringId PATH_OF_THE_ELVEN_ORACLE_IN_PROGRESS;
	
	@ClientString(id = 40909, message = "The sacred flame is ours!")
	public static NpcStringId THE_SACRED_FLAME_IS_OURS;
	
	@ClientString(id = 40910, message = "Arrghh...we shall never.. surrender...")
	public static NpcStringId ARRGHH_WE_SHALL_NEVER_SURRENDER;
	
	@ClientString(id = 40911, message = "The sacred flame is ours")
	public static NpcStringId THE_SACRED_FLAME_IS_OURS2;
	
	@ClientString(id = 40912, message = "The sacred flame is ours")
	public static NpcStringId THE_SACRED_FLAME_IS_OURS3;
	
	@ClientString(id = 40913, message = "As you wish, master!")
	public static NpcStringId AS_YOU_WISH_MASTER;
	
	@ClientString(id = 41001, message = "Path of the Palus Knight")
	public static NpcStringId PATH_OF_THE_PALUS_KNIGHT;
	
	@ClientString(id = 41002, message = "Path of the Palus Knight (In Progress)")
	public static NpcStringId PATH_OF_THE_PALUS_KNIGHT_IN_PROGRESS;
	
	@ClientString(id = 41101, message = "Path of the Assassin")
	public static NpcStringId PATH_OF_THE_ASSASSIN;
	
	@ClientString(id = 41102, message = "Path of the Assassin (In Progress)")
	public static NpcStringId PATH_OF_THE_ASSASSIN_IN_PROGRESS;
	
	@ClientString(id = 41201, message = "Path of the Dark Wizard")
	public static NpcStringId PATH_OF_THE_DARK_WIZARD;
	
	@ClientString(id = 41202, message = "Path of the Dark Wizard (In Progress)")
	public static NpcStringId PATH_OF_THE_DARK_WIZARD_IN_PROGRESS;
	
	@ClientString(id = 41301, message = "Path of the Shillien Oracle")
	public static NpcStringId PATH_OF_THE_SHILLIEN_ORACLE;
	
	@ClientString(id = 41302, message = "Path of the Shillien Oracle (In Progress)")
	public static NpcStringId PATH_OF_THE_SHILLIEN_ORACLE_IN_PROGRESS;
	
	@ClientString(id = 41401, message = "Path of the Orc Raider")
	public static NpcStringId PATH_OF_THE_ORC_RAIDER;
	
	@ClientString(id = 41402, message = "Path of the Orc Raider (In Progress)")
	public static NpcStringId PATH_OF_THE_ORC_RAIDER_IN_PROGRESS;
	
	@ClientString(id = 41501, message = "Path of the Orc Monk")
	public static NpcStringId PATH_OF_THE_ORC_MONK;
	
	@ClientString(id = 41502, message = "Path of the Orc Monk (In Progress)")
	public static NpcStringId PATH_OF_THE_ORC_MONK_IN_PROGRESS;
	
	@ClientString(id = 41601, message = "Path of the Orc Shaman")
	public static NpcStringId PATH_OF_THE_ORC_SHAMAN;
	
	@ClientString(id = 41602, message = "Path of the Orc Shaman (In Progress)")
	public static NpcStringId PATH_OF_THE_ORC_SHAMAN_IN_PROGRESS;
	
	@ClientString(id = 41651, message = "My dear friend of $s1, who has gone on ahead of me!")
	public static NpcStringId MY_DEAR_FRIEND_OF_S1_WHO_HAS_GONE_ON_AHEAD_OF_ME;
	
	@ClientString(id = 41652, message = "Listen to Tejakar Gandi, young Oroka! The spirit of the slain leopard is calling you, $s1!")
	public static NpcStringId LISTEN_TO_TEJAKAR_GANDI_YOUNG_OROKA_THE_SPIRIT_OF_THE_SLAIN_LEOPARD_IS_CALLING_YOU_S1;
	
	@ClientString(id = 41701, message = "Path of the Scavenger")
	public static NpcStringId PATH_OF_THE_SCAVENGER;
	
	@ClientString(id = 41702, message = "Path of the Scavenger (In Progress)")
	public static NpcStringId PATH_OF_THE_SCAVENGER_IN_PROGRESS;
	
	@ClientString(id = 41801, message = "Path of the Artisan")
	public static NpcStringId PATH_OF_THE_ARTISAN;
	
	@ClientString(id = 41802, message = "Path of the Artisan (In Progress)")
	public static NpcStringId PATH_OF_THE_ARTISAN_IN_PROGRESS;
	
	@ClientString(id = 41901, message = "Get a Pet")
	public static NpcStringId GET_A_PET;
	
	@ClientString(id = 41902, message = "Get a Pet (In Progress)")
	public static NpcStringId GET_A_PET_IN_PROGRESS;
	
	@ClientString(id = 41904, message = "Get a pet")
	public static NpcStringId GET_A_PET2;
	
	@ClientString(id = 42001, message = "Little Wing (Lv. 35-99)")
	public static NpcStringId LITTLE_WING_LV_35_99;
	
	@ClientString(id = 42002, message = "Little Wing (Lv. 35-99) (In Progress)")
	public static NpcStringId LITTLE_WING_LV_35_99_IN_PROGRESS;
	
	@ClientString(id = 42004, message = "Little Wing (Lv. 35-99)")
	public static NpcStringId LITTLE_WING_LV_35_992;
	
	@ClientString(id = 42046, message = "Hey! Everybody watch the eggs!")
	public static NpcStringId HEY_EVERYBODY_WATCH_THE_EGGS;
	
	@ClientString(id = 42047, message = "I thought I'd caught one share... Whew!")
	public static NpcStringId I_THOUGHT_I_D_CAUGHT_ONE_SHARE_WHEW;
	
	@ClientString(id = 42048, message = "The stone... the Elven stone... broke...")
	public static NpcStringId THE_STONE_THE_ELVEN_STONE_BROKE;
	
	@ClientString(id = 42049, message = "If the eggs get taken, we're dead!")
	public static NpcStringId IF_THE_EGGS_GET_TAKEN_WE_RE_DEAD;
	
	@ClientString(id = 42101, message = "Little Wing's Big Adventure (Lv. 45-99)")
	public static NpcStringId LITTLE_WING_S_BIG_ADVENTURE_LV_45_99;
	
	@ClientString(id = 42102, message = "Little Wing's Big Adventure (Lv. 45-99) (In Progress)")
	public static NpcStringId LITTLE_WING_S_BIG_ADVENTURE_LV_45_99_IN_PROGRESS;
	
	@ClientString(id = 42104, message = "Little Wing's Big Adventure (Lv. 45-99)")
	public static NpcStringId LITTLE_WING_S_BIG_ADVENTURE_LV_45_992;
	
	@ClientString(id = 42111, message = "Give me a Fairy Leaf...!")
	public static NpcStringId GIVE_ME_A_FAIRY_LEAF;
	
	@ClientString(id = 42112, message = "Why do you bother me again?")
	public static NpcStringId WHY_DO_YOU_BOTHER_ME_AGAIN;
	
	@ClientString(id = 42113, message = "Hey, you've already drunk the essence of wind!")
	public static NpcStringId HEY_YOU_VE_ALREADY_DRUNK_THE_ESSENCE_OF_WIND;
	
	@ClientString(id = 42114, message = "Leave now, before you incur the wrath of the guardian ghost...")
	public static NpcStringId LEAVE_NOW_BEFORE_YOU_INCUR_THE_WRATH_OF_THE_GUARDIAN_GHOST;
	
	@ClientString(id = 42115, message = "Hey, you've already drunk the essence of a star!")
	public static NpcStringId HEY_YOU_VE_ALREADY_DRUNK_THE_ESSENCE_OF_A_STAR;
	
	@ClientString(id = 42116, message = "Hey, you've already drunk the essence of dusk!")
	public static NpcStringId HEY_YOU_VE_ALREADY_DRUNK_THE_ESSENCE_OF_DUSK;
	
	@ClientString(id = 42117, message = "Hey, you've already drunk the essence of the abyss!")
	public static NpcStringId HEY_YOU_VE_ALREADY_DRUNK_THE_ESSENCE_OF_THE_ABYSS;
	
	@ClientString(id = 42118, message = "We must protect the fairy tree!")
	public static NpcStringId WE_MUST_PROTECT_THE_FAIRY_TREE;
	
	@ClientString(id = 42119, message = "Get out of the sacred tree, you scoundrels!")
	public static NpcStringId GET_OUT_OF_THE_SACRED_TREE_YOU_SCOUNDRELS;
	
	@ClientString(id = 42120, message = "Death to the thieves of the pure water of the world!")
	public static NpcStringId DEATH_TO_THE_THIEVES_OF_THE_PURE_WATER_OF_THE_WORLD;
	
	@ClientString(id = 42201, message = "Repent Your Sins")
	public static NpcStringId REPENT_YOUR_SINS;
	
	@ClientString(id = 42202, message = "Repent Your Sins (In Progress)")
	public static NpcStringId REPENT_YOUR_SINS_IN_PROGRESS;
	
	@ClientString(id = 42231, message = "Hey, it seems like you need my help, doesn't it?")
	public static NpcStringId HEY_IT_SEEMS_LIKE_YOU_NEED_MY_HELP_DOESN_T_IT;
	
	@ClientString(id = 42232, message = "Almost got it... Ouch! Stop!  Damn these bloody manacles!")
	public static NpcStringId ALMOST_GOT_IT_OUCH_STOP_DAMN_THESE_BLOODY_MANACLES;
	
	@ClientString(id = 42233, message = "Oh, that smarts!")
	public static NpcStringId OH_THAT_SMARTS;
	
	@ClientString(id = 42234, message = "Hey, master! Pay attention! I'm dying over here!")
	public static NpcStringId HEY_MASTER_PAY_ATTENTION_I_M_DYING_OVER_HERE;
	
	@ClientString(id = 42235, message = "What have I done to deserve this?")
	public static NpcStringId WHAT_HAVE_I_DONE_TO_DESERVE_THIS;
	
	@ClientString(id = 42236, message = "Oh, this is just great! What are you going to do now?")
	public static NpcStringId OH_THIS_IS_JUST_GREAT_WHAT_ARE_YOU_GOING_TO_DO_NOW;
	
	@ClientString(id = 42237, message = "You inconsiderate moron! Can't you even take care of little old me?!")
	public static NpcStringId YOU_INCONSIDERATE_MORON_CAN_T_YOU_EVEN_TAKE_CARE_OF_LITTLE_OLD_ME;
	
	@ClientString(id = 42238, message = "Oh no! The man who eats one's sins has died! Penitence is further away~!")
	public static NpcStringId OH_NO_THE_MAN_WHO_EATS_ONE_S_SINS_HAS_DIED_PENITENCE_IS_FURTHER_AWAY;
	
	@ClientString(id = 42239, message = "Using a special skill here could trigger a bloodbath!")
	public static NpcStringId USING_A_SPECIAL_SKILL_HERE_COULD_TRIGGER_A_BLOODBATH;
	
	@ClientString(id = 42240, message = "Hey, what do you expect of me?")
	public static NpcStringId HEY_WHAT_DO_YOU_EXPECT_OF_ME;
	
	@ClientString(id = 42241, message = "Ugggggh! Push! It's not coming out!")
	public static NpcStringId UGGGGGH_PUSH_IT_S_NOT_COMING_OUT;
	
	@ClientString(id = 42242, message = "Ah, I missed the mark!")
	public static NpcStringId AH_I_MISSED_THE_MARK;
	
	@ClientString(id = 42243, message = "Yawwwwn! It's so boring here. We should go and find some action!")
	public static NpcStringId YAWWWWN_IT_S_SO_BORING_HERE_WE_SHOULD_GO_AND_FIND_SOME_ACTION;
	
	@ClientString(id = 42244, message = "Hey, if you continue to waste time you will never finish your penance!")
	public static NpcStringId HEY_IF_YOU_CONTINUE_TO_WASTE_TIME_YOU_WILL_NEVER_FINISH_YOUR_PENANCE;
	
	@ClientString(id = 42245, message = "I know you don't like me. The feeling is mutual!")
	public static NpcStringId I_KNOW_YOU_DON_T_LIKE_ME_THE_FEELING_IS_MUTUAL;
	
	@ClientString(id = 42246, message = "I need a drink.")
	public static NpcStringId I_NEED_A_DRINK;
	
	@ClientString(id = 42247, message = "Oh, this is dragging on too long...  At this rate I won't make it home before the seven seals are broken.")
	public static NpcStringId OH_THIS_IS_DRAGGING_ON_TOO_LONG_AT_THIS_RATE_I_WON_T_MAKE_IT_HOME_BEFORE_THE_SEVEN_SEALS_ARE_BROKEN;
	
	@ClientString(id = 42301, message = "Take Your Best Shot")
	public static NpcStringId TAKE_YOUR_BEST_SHOT;
	
	@ClientString(id = 42302, message = "Take Your Best Shot (In Progress)")
	public static NpcStringId TAKE_YOUR_BEST_SHOT_IN_PROGRESS;
	
	@ClientString(id = 42304, message = "Take Your Best Shot")
	public static NpcStringId TAKE_YOUR_BEST_SHOT2;
	
	@ClientString(id = 42601, message = "Quest for Fishing shot")
	public static NpcStringId QUEST_FOR_FISHING_SHOT;
	
	@ClientString(id = 42602, message = "Quest for Fishing shot (In Progress)")
	public static NpcStringId QUEST_FOR_FISHING_SHOT_IN_PROGRESS;
	
	@ClientString(id = 43101, message = "Wedding March (Lv. 38-43)")
	public static NpcStringId WEDDING_MARCH_LV_38_43;
	
	@ClientString(id = 43102, message = "Wedding March (Lv. 38-43) (In Progress)")
	public static NpcStringId WEDDING_MARCH_LV_38_43_IN_PROGRESS;
	
	@ClientString(id = 43104, message = "Wedding March (Lv. 38-43)")
	public static NpcStringId WEDDING_MARCH_LV_38_432;
	
	@ClientString(id = 43201, message = "Birthday Party Song (Lv. 31-36)")
	public static NpcStringId BIRTHDAY_PARTY_SONG_LV_31_36;
	
	@ClientString(id = 43202, message = "Birthday Party Song (Lv. 31-36) (In Progress)")
	public static NpcStringId BIRTHDAY_PARTY_SONG_LV_31_36_IN_PROGRESS;
	
	@ClientString(id = 43204, message = "Birthday Party Song (Lv. 31-36)")
	public static NpcStringId BIRTHDAY_PARTY_SONG_LV_31_362;
	
	@ClientString(id = 45001, message = "Grave Robber Rescue (Lv. 80-99)")
	public static NpcStringId GRAVE_ROBBER_RESCUE_LV_80_99;
	
	@ClientString(id = 45002, message = "Grave Robber Rescue (Lv. 80-99) (In Progress)")
	public static NpcStringId GRAVE_ROBBER_RESCUE_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 45003, message = "Grave Robber Rescue (Lv. 80-99) (Done)")
	public static NpcStringId GRAVE_ROBBER_RESCUE_LV_80_99_DONE;
	
	@ClientString(id = 45004, message = "Grave Robber Rescue (Lv. 80-99)")
	public static NpcStringId GRAVE_ROBBER_RESCUE_LV_80_992;
	
	@ClientString(id = 45101, message = "Lucien's Altar (Lv. 80-99)")
	public static NpcStringId LUCIEN_S_ALTAR_LV_80_99;
	
	@ClientString(id = 45102, message = "Lucien's Altar (Lv. 80-99) (In Progress)")
	public static NpcStringId LUCIEN_S_ALTAR_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 45103, message = "Lucien's Altar (Lv. 80-99) (Done)")
	public static NpcStringId LUCIEN_S_ALTAR_LV_80_99_DONE;
	
	@ClientString(id = 45104, message = "Lucien's Altar (Lv. 80-99)")
	public static NpcStringId LUCIEN_S_ALTAR_LV_80_992;
	
	@ClientString(id = 45201, message = "Finding the Lost Soldiers (Lv. 85-99)")
	public static NpcStringId FINDING_THE_LOST_SOLDIERS_LV_85_99;
	
	@ClientString(id = 45202, message = "Finding the Lost Soldiers (Lv. 85-99) (In Progress)")
	public static NpcStringId FINDING_THE_LOST_SOLDIERS_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 45203, message = "Finding the Lost Soldiers (Lv. 85-99) (Done)")
	public static NpcStringId FINDING_THE_LOST_SOLDIERS_LV_85_99_DONE;
	
	@ClientString(id = 45204, message = "Finding the Lost Soldiers (Lv. 85-99)")
	public static NpcStringId FINDING_THE_LOST_SOLDIERS_LV_85_992;
	
	@ClientString(id = 45301, message = "Not Strong Enough Alone (Lv. 85-99)")
	public static NpcStringId NOT_STRONG_ENOUGH_ALONE_LV_85_99;
	
	@ClientString(id = 45302, message = "Not Strong Enough Alone (Lv. 85-99) (In Progress)")
	public static NpcStringId NOT_STRONG_ENOUGH_ALONE_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 45303, message = "Not Strong Enough Alone (Lv. 85-99) (Done)")
	public static NpcStringId NOT_STRONG_ENOUGH_ALONE_LV_85_99_DONE;
	
	@ClientString(id = 45304, message = "Not Strong Enough Alone (Lv. 85-99)")
	public static NpcStringId NOT_STRONG_ENOUGH_ALONE_LV_85_992;
	
	@ClientString(id = 45401, message = "Completely Lost (Lv. 85-99)")
	public static NpcStringId COMPLETELY_LOST_LV_85_99;
	
	@ClientString(id = 45402, message = "Completely Lost (Lv. 85-99) (In Progress)")
	public static NpcStringId COMPLETELY_LOST_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 45403, message = "Completely Lost (Lv. 85-99) (Done)")
	public static NpcStringId COMPLETELY_LOST_LV_85_99_DONE;
	
	@ClientString(id = 45404, message = "Completely Lost (Lv. 85-99)")
	public static NpcStringId COMPLETELY_LOST_LV_85_992;
	
	@ClientString(id = 45501, message = "Wings of Sand (Lv. 80-99)")
	public static NpcStringId WINGS_OF_SAND_LV_80_99;
	
	@ClientString(id = 45502, message = "Wings of Sand (Lv. 80-99) (In Progress)")
	public static NpcStringId WINGS_OF_SAND_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 45503, message = "Wings of Sand (Lv. 80-99) (Done)")
	public static NpcStringId WINGS_OF_SAND_LV_80_99_DONE;
	
	@ClientString(id = 45601, message = "Don't Know, Don't Care (Lv. 80-99)")
	public static NpcStringId DON_T_KNOW_DON_T_CARE_LV_80_99;
	
	@ClientString(id = 45602, message = "Don't Know, Don't Care (Lv. 80-99) (In Progress)")
	public static NpcStringId DON_T_KNOW_DON_T_CARE_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 45603, message = "Don't Know, Don't Care (Lv. 80-99) (Done)")
	public static NpcStringId DON_T_KNOW_DON_T_CARE_LV_80_99_DONE;
	
	@ClientString(id = 45650, message = "$s1 received a $s2 item as a reward from the separated soul.")
	public static NpcStringId S1_RECEIVED_A_S2_ITEM_AS_A_REWARD_FROM_THE_SEPARATED_SOUL;
	
	@ClientString(id = 45651, message = "Sealed Vorpal Helmet")
	public static NpcStringId SEALED_VORPAL_HELMET;
	
	@ClientString(id = 45652, message = "Sealed Vorpal Leather Helmet")
	public static NpcStringId SEALED_VORPAL_LEATHER_HELMET;
	
	@ClientString(id = 45653, message = "Sealed Vorpal Circlet")
	public static NpcStringId SEALED_VORPAL_CIRCLET;
	
	@ClientString(id = 45654, message = "Sealed Vorpal Breastplate")
	public static NpcStringId SEALED_VORPAL_BREASTPLATE;
	
	@ClientString(id = 45655, message = "Sealed Vorpal Leather Breastplate")
	public static NpcStringId SEALED_VORPAL_LEATHER_BREASTPLATE;
	
	@ClientString(id = 45656, message = "Sealed Vorpal Tunic")
	public static NpcStringId SEALED_VORPAL_TUNIC;
	
	@ClientString(id = 45657, message = "Sealed Vorpal Gaiters")
	public static NpcStringId SEALED_VORPAL_GAITERS;
	
	@ClientString(id = 45658, message = "Sealed Vorpal Leather Leggings")
	public static NpcStringId SEALED_VORPAL_LEATHER_LEGGINGS;
	
	@ClientString(id = 45659, message = "Sealed Vorpal Stockings")
	public static NpcStringId SEALED_VORPAL_STOCKINGS;
	
	@ClientString(id = 45660, message = "Sealed Vorpal Gauntlets")
	public static NpcStringId SEALED_VORPAL_GAUNTLETS;
	
	@ClientString(id = 45661, message = "Sealed Vorpal Leather Gloves")
	public static NpcStringId SEALED_VORPAL_LEATHER_GLOVES;
	
	@ClientString(id = 45662, message = "Sealed Vorpal Gloves")
	public static NpcStringId SEALED_VORPAL_GLOVES;
	
	@ClientString(id = 45663, message = "Sealed Vorpal Boots")
	public static NpcStringId SEALED_VORPAL_BOOTS;
	
	@ClientString(id = 45664, message = "Sealed Vorpal Leather Boots")
	public static NpcStringId SEALED_VORPAL_LEATHER_BOOTS;
	
	@ClientString(id = 45665, message = "Sealed Vorpal Shoes")
	public static NpcStringId SEALED_VORPAL_SHOES;
	
	@ClientString(id = 45666, message = "Sealed Vorpal Shield")
	public static NpcStringId SEALED_VORPAL_SHIELD;
	
	@ClientString(id = 45667, message = "Sealed Vorpal Sigil")
	public static NpcStringId SEALED_VORPAL_SIGIL;
	
	@ClientString(id = 45668, message = "Sealed Vorpal Ring")
	public static NpcStringId SEALED_VORPAL_RING;
	
	@ClientString(id = 45669, message = "Sealed Vorpal Earring")
	public static NpcStringId SEALED_VORPAL_EARRING;
	
	@ClientString(id = 45670, message = "Sealed Vorpal Necklace")
	public static NpcStringId SEALED_VORPAL_NECKLACE;
	
	@ClientString(id = 45671, message = "Periel Sword")
	public static NpcStringId PERIEL_SWORD;
	
	@ClientString(id = 45672, message = "Skull Edge")
	public static NpcStringId SKULL_EDGE;
	
	@ClientString(id = 45673, message = "Vigwik Axe")
	public static NpcStringId VIGWIK_AXE;
	
	@ClientString(id = 45674, message = "Devilish Maul")
	public static NpcStringId DEVILISH_MAUL;
	
	@ClientString(id = 45675, message = "Feather Eye Blade")
	public static NpcStringId FEATHER_EYE_BLADE;
	
	@ClientString(id = 45676, message = "Octo Claw")
	public static NpcStringId OCTO_CLAW;
	
	@ClientString(id = 45677, message = "Doubletop Spear")
	public static NpcStringId DOUBLETOP_SPEAR;
	
	@ClientString(id = 45678, message = "Rising Star")
	public static NpcStringId RISING_STAR;
	
	@ClientString(id = 45679, message = "Black Visage")
	public static NpcStringId BLACK_VISAGE;
	
	@ClientString(id = 45680, message = "Veniplant Sword")
	public static NpcStringId VENIPLANT_SWORD;
	
	@ClientString(id = 45681, message = "Skull Carnium Bow")
	public static NpcStringId SKULL_CARNIUM_BOW;
	
	@ClientString(id = 45682, message = "Gemtail Rapier")
	public static NpcStringId GEMTAIL_RAPIER;
	
	@ClientString(id = 45683, message = "Finale Blade")
	public static NpcStringId FINALE_BLADE;
	
	@ClientString(id = 45684, message = "Dominion Crossbow")
	public static NpcStringId DOMINION_CROSSBOW;
	
	@ClientString(id = 45685, message = "Blessed Scroll: Enchant Weapon (S-grade)")
	public static NpcStringId BLESSED_SCROLL_ENCHANT_WEAPON_S_GRADE;
	
	@ClientString(id = 45686, message = "Blessed Scroll: Enchant Armor (S-grade)")
	public static NpcStringId BLESSED_SCROLL_ENCHANT_ARMOR_S_GRADE;
	
	@ClientString(id = 45687, message = "Fire Crystal")
	public static NpcStringId FIRE_CRYSTAL;
	
	@ClientString(id = 45688, message = "Water Crystal")
	public static NpcStringId WATER_CRYSTAL;
	
	@ClientString(id = 45689, message = "Earth Crystal")
	public static NpcStringId EARTH_CRYSTAL;
	
	@ClientString(id = 45690, message = "Wind Crystal")
	public static NpcStringId WIND_CRYSTAL;
	
	@ClientString(id = 45691, message = "Holy Crystal")
	public static NpcStringId HOLY_CRYSTAL;
	
	@ClientString(id = 45692, message = "Dark Crystal")
	public static NpcStringId DARK_CRYSTAL;
	
	@ClientString(id = 45693, message = "Scroll: Enchant Weapon (S-grade)")
	public static NpcStringId SCROLL_ENCHANT_WEAPON_S_GRADE;
	
	@ClientString(id = 45701, message = "Lost and Found (Lv. 82-99)")
	public static NpcStringId LOST_AND_FOUND_LV_82_99;
	
	@ClientString(id = 45702, message = "Lost and Found (Lv. 82-99) (In Progress)")
	public static NpcStringId LOST_AND_FOUND_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 45703, message = "Lost and Found (Lv. 82-99) (Done)")
	public static NpcStringId LOST_AND_FOUND_LV_82_99_DONE;
	
	@ClientString(id = 45704, message = "Lost and Found (Lv. 82-99)")
	public static NpcStringId LOST_AND_FOUND_LV_82_992;
	
	@ClientString(id = 45801, message = "Perfect Form (Lv. 82-99)")
	public static NpcStringId PERFECT_FORM_LV_82_99;
	
	@ClientString(id = 45802, message = "Perfect Form (Lv. 82-99) (In Progress)")
	public static NpcStringId PERFECT_FORM_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 45803, message = "Perfect Form (Lv. 82-99) (Done)")
	public static NpcStringId PERFECT_FORM_LV_82_99_DONE;
	
	@ClientString(id = 45804, message = "Perfect Form (Lv. 82-99)")
	public static NpcStringId PERFECT_FORM_LV_82_992;
	
	@ClientString(id = 45901, message = "Monstrosity in the Underground Mine, Teredor (Lv. 85-99)")
	public static NpcStringId MONSTROSITY_IN_THE_UNDERGROUND_MINE_TEREDOR_LV_85_99;
	
	@ClientString(id = 45902, message = "Monstrosity in the Underground Mine, Teredor  (Lv. 85-99)(In Progress)")
	public static NpcStringId MONSTROSITY_IN_THE_UNDERGROUND_MINE_TEREDOR_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 45903, message = "Monstrosity in the Underground Mine, Teredor (Lv. 85-99) (Done)")
	public static NpcStringId MONSTROSITY_IN_THE_UNDERGROUND_MINE_TEREDOR_LV_85_99_DONE;
	
	@ClientString(id = 45904, message = "Monstrosity in the Underground Mine, Teredor (Lv. 85-99)")
	public static NpcStringId MONSTROSITY_IN_THE_UNDERGROUND_MINE_TEREDOR_LV_85_992;
	
	@ClientString(id = 46001, message = "Precious Research Ingredient (Lv. 85-99)")
	public static NpcStringId PRECIOUS_RESEARCH_INGREDIENT_LV_85_99;
	
	@ClientString(id = 46002, message = "Precious Research Ingredient (Lv. 85-99) (In Progress)")
	public static NpcStringId PRECIOUS_RESEARCH_INGREDIENT_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 46003, message = "Precious Research Ingredient (Lv. 85-99) (Done)")
	public static NpcStringId PRECIOUS_RESEARCH_INGREDIENT_LV_85_99_DONE;
	
	@ClientString(id = 46004, message = "Precious Research Ingredient (Lv. 85-99)")
	public static NpcStringId PRECIOUS_RESEARCH_INGREDIENT_LV_85_992;
	
	@ClientString(id = 46101, message = "Rumble in the Base")
	public static NpcStringId RUMBLE_IN_THE_BASE;
	
	@ClientString(id = 46102, message = "Rumble in the Base (In Progress)")
	public static NpcStringId RUMBLE_IN_THE_BASE_IN_PROGRESS;
	
	@ClientString(id = 46103, message = "Rumble in the Base (Done)")
	public static NpcStringId RUMBLE_IN_THE_BASE_DONE;
	
	@ClientString(id = 46104, message = "Rumble in the Base")
	public static NpcStringId RUMBLE_IN_THE_BASE2;
	
	@ClientString(id = 46201, message = "Preserved Ancient Heroes (Lv. 95-99)")
	public static NpcStringId PRESERVED_ANCIENT_HEROES_LV_95_99;
	
	@ClientString(id = 46202, message = "Preserved Ancient Heroes (Lv. 95-99) (In Progress)")
	public static NpcStringId PRESERVED_ANCIENT_HEROES_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 46203, message = "Preserved Ancient Heroes (Lv. 95-99) (Done)")
	public static NpcStringId PRESERVED_ANCIENT_HEROES_LV_95_99_DONE;
	
	@ClientString(id = 46204, message = "Preserved Ancient Heroes (Lv. 95-99)")
	public static NpcStringId PRESERVED_ANCIENT_HEROES_LV_95_992;
	
	@ClientString(id = 46301, message = "I Must Be a Genius (Lv. 70-99)")
	public static NpcStringId I_MUST_BE_A_GENIUS_LV_70_99;
	
	@ClientString(id = 46302, message = "I Must Be a Genius (Lv. 70-99) (In Progress)")
	public static NpcStringId I_MUST_BE_A_GENIUS_LV_70_99_IN_PROGRESS;
	
	@ClientString(id = 46303, message = "I Must Be a Genius (Lv. 70-99) (Done)")
	public static NpcStringId I_MUST_BE_A_GENIUS_LV_70_99_DONE;
	
	@ClientString(id = 46304, message = "I Must Be a Genius (Lv. 70-99)")
	public static NpcStringId I_MUST_BE_A_GENIUS_LV_70_992;
	
	@ClientString(id = 46350, message = "Att... attack... $s1.. Ro... rogue... $s2..")
	public static NpcStringId ATT_ATTACK_S1_RO_ROGUE_S2;
	
	@ClientString(id = 46401, message = "Oath (Lv. 82-99)")
	public static NpcStringId OATH_LV_82_99;
	
	@ClientString(id = 46402, message = "Oath  (Lv. 82-99)(In Progress)")
	public static NpcStringId OATH_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 46403, message = "Oath  (Lv. 82-99)(Done)")
	public static NpcStringId OATH_LV_82_99_DONE;
	
	@ClientString(id = 46501, message = "We are Friends (Lv. 88-99)")
	public static NpcStringId WE_ARE_FRIENDS_LV_88_99;
	
	@ClientString(id = 46502, message = "We are Friends (Lv. 88-99) (In progress)")
	public static NpcStringId WE_ARE_FRIENDS_LV_88_99_IN_PROGRESS;
	
	@ClientString(id = 46503, message = "We are Friends (Lv. 88-99) (Done)")
	public static NpcStringId WE_ARE_FRIENDS_LV_88_99_DONE;
	
	@ClientString(id = 46504, message = "We are Friends (Lv. 88-99)")
	public static NpcStringId WE_ARE_FRIENDS_LV_88_992;
	
	@ClientString(id = 46601, message = "Placing My Small Power (Lv. 88-99)")
	public static NpcStringId PLACING_MY_SMALL_POWER_LV_88_99;
	
	@ClientString(id = 46602, message = "Placing My Small Power (Lv. 88-99) (In Progress)")
	public static NpcStringId PLACING_MY_SMALL_POWER_LV_88_99_IN_PROGRESS;
	
	@ClientString(id = 46603, message = "Placing My Small Power (Lv. 88-99) (Done)")
	public static NpcStringId PLACING_MY_SMALL_POWER_LV_88_99_DONE;
	
	@ClientString(id = 46604, message = "Placing My Small Power (Lv. 88-99)")
	public static NpcStringId PLACING_MY_SMALL_POWER_LV_88_992;
	
	@ClientString(id = 46701, message = "The Oppressor and The Oppressed (Lv. 60-64)")
	public static NpcStringId THE_OPPRESSOR_AND_THE_OPPRESSED_LV_60_64;
	
	@ClientString(id = 46702, message = "The Oppressor and The Oppressed (Lv. 60-64) (In Progress)")
	public static NpcStringId THE_OPPRESSOR_AND_THE_OPPRESSED_LV_60_64_IN_PROGRESS;
	
	@ClientString(id = 46703, message = "The Oppressor and The Oppressed (Lv. 60-64) (Done)")
	public static NpcStringId THE_OPPRESSOR_AND_THE_OPPRESSED_LV_60_64_DONE;
	
	@ClientString(id = 46704, message = "The Oppressor and The Oppressed (Lv. 60-64)")
	public static NpcStringId THE_OPPRESSOR_AND_THE_OPPRESSED_LV_60_642;
	
	@ClientString(id = 46801, message = "Be Lost in the Mysterious Scent (Lv. 90-99)")
	public static NpcStringId BE_LOST_IN_THE_MYSTERIOUS_SCENT_LV_90_99;
	
	@ClientString(id = 46802, message = "Be Lost in the Mysterious Scent (Lv. 90-99) (In Progress)")
	public static NpcStringId BE_LOST_IN_THE_MYSTERIOUS_SCENT_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 46803, message = "Be Lost in the Mysterious Scent (Lv. 90-99) (Done)")
	public static NpcStringId BE_LOST_IN_THE_MYSTERIOUS_SCENT_LV_90_99_DONE;
	
	@ClientString(id = 46804, message = "Be Lost in the Mysterious Scent (Lv. 90-99)")
	public static NpcStringId BE_LOST_IN_THE_MYSTERIOUS_SCENT_LV_90_992;
	
	@ClientString(id = 46901, message = "Suspicious Gardener (Lv. 90-99)")
	public static NpcStringId SUSPICIOUS_GARDENER_LV_90_99;
	
	@ClientString(id = 46902, message = "Suspicious Gardener (Lv. 90-99) (In Progress)")
	public static NpcStringId SUSPICIOUS_GARDENER_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 46903, message = "Suspicious Gardener (Lv. 90-99) (Done)")
	public static NpcStringId SUSPICIOUS_GARDENER_LV_90_99_DONE;
	
	@ClientString(id = 46904, message = "Suspicious Gardener (Lv. 90-99)")
	public static NpcStringId SUSPICIOUS_GARDENER_LV_90_992;
	
	@ClientString(id = 47001, message = "Divinity Protector (Lv. 60-64)")
	public static NpcStringId DIVINITY_PROTECTOR_LV_60_64;
	
	@ClientString(id = 47002, message = "Divinity Protector (Lv. 60-64) (In progress)")
	public static NpcStringId DIVINITY_PROTECTOR_LV_60_64_IN_PROGRESS;
	
	@ClientString(id = 47003, message = "Divinity Protector (Lv. 60-64) (Done)")
	public static NpcStringId DIVINITY_PROTECTOR_LV_60_64_DONE;
	
	@ClientString(id = 47004, message = "Divinity Protector (Lv. 60-64)")
	public static NpcStringId DIVINITY_PROTECTOR_LV_60_642;
	
	@ClientString(id = 47101, message = "Breaking through Emerald Square (Lv. 97-99)")
	public static NpcStringId BREAKING_THROUGH_EMERALD_SQUARE_LV_97_99;
	
	@ClientString(id = 47102, message = "Breaking through Emerald Square (Lv. 97-99) (In Progress)")
	public static NpcStringId BREAKING_THROUGH_EMERALD_SQUARE_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 47103, message = "Breaking through Emerald Square (Lv. 97-99) (Done)")
	public static NpcStringId BREAKING_THROUGH_EMERALD_SQUARE_LV_97_99_DONE;
	
	@ClientString(id = 47104, message = "Breaking through Emerald Square (Lv. 97-99)")
	public static NpcStringId BREAKING_THROUGH_EMERALD_SQUARE_LV_97_992;
	
	@ClientString(id = 47201, message = "Challenge, Steam Corridor (Lv. 97-99)")
	public static NpcStringId CHALLENGE_STEAM_CORRIDOR_LV_97_99;
	
	@ClientString(id = 47202, message = "Challenge, Steam Corridor (Lv. 97-99) (In Progress)")
	public static NpcStringId CHALLENGE_STEAM_CORRIDOR_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 47203, message = "Challenge, Steam Corridor (Lv. 97-99) (Done)")
	public static NpcStringId CHALLENGE_STEAM_CORRIDOR_LV_97_99_DONE;
	
	@ClientString(id = 47204, message = "Challenge, Steam Corridor (Lv. 97-99)")
	public static NpcStringId CHALLENGE_STEAM_CORRIDOR_LV_97_992;
	
	@ClientString(id = 47301, message = "In the Coral Garden (Lv. 97-99)")
	public static NpcStringId IN_THE_CORAL_GARDEN_LV_97_99;
	
	@ClientString(id = 47302, message = "In the Coral Garden (Lv. 97-99) (In Progress)")
	public static NpcStringId IN_THE_CORAL_GARDEN_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 47303, message = "In the Coral Garden (Lv. 97-99) (Done)")
	public static NpcStringId IN_THE_CORAL_GARDEN_LV_97_99_DONE;
	
	@ClientString(id = 47304, message = "In the Coral Garden (Lv. 97-99)")
	public static NpcStringId IN_THE_CORAL_GARDEN_LV_97_992;
	
	@ClientString(id = 47401, message = "Waiting for the Summer (Lv. 60-64)")
	public static NpcStringId WAITING_FOR_THE_SUMMER_LV_60_64;
	
	@ClientString(id = 47402, message = "Waiting for the Summer (Lv. 60-64) (In Progress)")
	public static NpcStringId WAITING_FOR_THE_SUMMER_LV_60_64_IN_PROGRESS;
	
	@ClientString(id = 47403, message = "Waiting for the Summer (Lv. 60-64) (Done)")
	public static NpcStringId WAITING_FOR_THE_SUMMER_LV_60_64_DONE;
	
	@ClientString(id = 47404, message = "Waiting for the Summer (Lv. 60-64)")
	public static NpcStringId WAITING_FOR_THE_SUMMER_LV_60_642;
	
	@ClientString(id = 47501, message = "For the Sacrificed (Lv. 65-69)")
	public static NpcStringId FOR_THE_SACRIFICED_LV_65_69;
	
	@ClientString(id = 47502, message = "For the Sacrificed (Lv. 65-69) (In Progress)")
	public static NpcStringId FOR_THE_SACRIFICED_LV_65_69_IN_PROGRESS;
	
	@ClientString(id = 47503, message = "For the Sacrificed (Lv. 65-69) (Done)")
	public static NpcStringId FOR_THE_SACRIFICED_LV_65_69_DONE;
	
	@ClientString(id = 47504, message = "For the Sacrificed (Lv. 65-69)")
	public static NpcStringId FOR_THE_SACRIFICED_LV_65_692;
	
	@ClientString(id = 47601, message = "Plain Mission (Lv. 65-69)")
	public static NpcStringId PLAIN_MISSION_LV_65_69;
	
	@ClientString(id = 47602, message = "Plain Mission (Lv. 65-69) (In Progress)")
	public static NpcStringId PLAIN_MISSION_LV_65_69_IN_PROGRESS;
	
	@ClientString(id = 47603, message = "Plain Mission (Lv. 65-69) (Done)")
	public static NpcStringId PLAIN_MISSION_LV_65_69_DONE;
	
	@ClientString(id = 47604, message = "Plain Mission (Lv. 65-69)")
	public static NpcStringId PLAIN_MISSION_LV_65_692;
	
	@ClientString(id = 47701, message = "Blood from the Wall (Lv. 70-74)")
	public static NpcStringId BLOOD_FROM_THE_WALL_LV_70_74;
	
	@ClientString(id = 47702, message = "Blood from the Wall (Lv. 70-74) (In Progress)")
	public static NpcStringId BLOOD_FROM_THE_WALL_LV_70_74_IN_PROGRESS;
	
	@ClientString(id = 47703, message = "Blood from the Wall (Lv. 70-74) (Done)")
	public static NpcStringId BLOOD_FROM_THE_WALL_LV_70_74_DONE;
	
	@ClientString(id = 47704, message = "Blood from the Wall (Lv. 70-74)")
	public static NpcStringId BLOOD_FROM_THE_WALL_LV_70_742;
	
	@ClientString(id = 47801, message = "Nightmares of Dwarves")
	public static NpcStringId NIGHTMARES_OF_DWARVES;
	
	@ClientString(id = 47802, message = "Nightmares of Dwarves (In Progress)")
	public static NpcStringId NIGHTMARES_OF_DWARVES_IN_PROGRESS;
	
	@ClientString(id = 47803, message = "Nightmares of Dwarves (Done)")
	public static NpcStringId NIGHTMARES_OF_DWARVES_DONE;
	
	@ClientString(id = 47804, message = "Nightmares of Dwarves")
	public static NpcStringId NIGHTMARES_OF_DWARVES2;
	
	@ClientString(id = 47850, message = "$s1 received a $s2 item as a reward from the Head Priest of the Earth Daichir.")
	public static NpcStringId S1_RECEIVED_A_S2_ITEM_AS_A_REWARD_FROM_THE_HEAD_PRIEST_OF_THE_EARTH_DAICHIR;
	
	@ClientString(id = 47851, message = "Blessed God's Shaper")
	public static NpcStringId BLESSED_GOD_S_SHAPER;
	
	@ClientString(id = 47852, message = "Blessed God's Cutter")
	public static NpcStringId BLESSED_GOD_S_CUTTER;
	
	@ClientString(id = 47853, message = "Blessed God's Slasher")
	public static NpcStringId BLESSED_GOD_S_SLASHER;
	
	@ClientString(id = 47854, message = "Blessed God's Avenger")
	public static NpcStringId BLESSED_GOD_S_AVENGER;
	
	@ClientString(id = 47855, message = "Blessed God's Fighter")
	public static NpcStringId BLESSED_GOD_S_FIGHTER;
	
	@ClientString(id = 47856, message = "Blessed God's Stormer")
	public static NpcStringId BLESSED_GOD_S_STORMER;
	
	@ClientString(id = 47857, message = "Blessed God's Thrower")
	public static NpcStringId BLESSED_GOD_S_THROWER;
	
	@ClientString(id = 47858, message = "Blessed God's Shooter")
	public static NpcStringId BLESSED_GOD_S_SHOOTER;
	
	@ClientString(id = 47859, message = "Blessed God's Buster")
	public static NpcStringId BLESSED_GOD_S_BUSTER;
	
	@ClientString(id = 47860, message = "Blessed God's Caster")
	public static NpcStringId BLESSED_GOD_S_CASTER;
	
	@ClientString(id = 47861, message = "Blessed God's Retributer")
	public static NpcStringId BLESSED_GOD_S_RETRIBUTER;
	
	@ClientString(id = 47862, message = "Earth Wyrm Heart Necklace")
	public static NpcStringId EARTH_WYRM_HEART_NECKLACE;
	
	@ClientString(id = 47863, message = "Yellow Soul Crystal - R99-grade")
	public static NpcStringId YELLOW_SOUL_CRYSTAL_R99_GRADE;
	
	@ClientString(id = 47864, message = "Teal Soul Crystal (R99-grade)")
	public static NpcStringId TEAL_SOUL_CRYSTAL_R99_GRADE;
	
	@ClientString(id = 47865, message = "Purple Soul Crystal (R99-grade)")
	public static NpcStringId PURPLE_SOUL_CRYSTAL_R99_GRADE;
	
	@ClientString(id = 47866, message = "Fire Crystal")
	public static NpcStringId FIRE_CRYSTAL2;
	
	@ClientString(id = 47867, message = "Water Crystal")
	public static NpcStringId WATER_CRYSTAL2;
	
	@ClientString(id = 47868, message = "Earth Crystal")
	public static NpcStringId EARTH_CRYSTAL2;
	
	@ClientString(id = 47869, message = "Wind Crystal")
	public static NpcStringId WIND_CRYSTAL2;
	
	@ClientString(id = 47870, message = "Dark Crystal")
	public static NpcStringId DARK_CRYSTAL2;
	
	@ClientString(id = 47871, message = "Holy Crystal")
	public static NpcStringId HOLY_CRYSTAL2;
	
	@ClientString(id = 47872, message = "Antharas' Symbol Dye")
	public static NpcStringId ANTHARAS_SYMBOL_DYE;
	
	@ClientString(id = 47901, message = "Destroying the Eggs of Trasken")
	public static NpcStringId DESTROYING_THE_EGGS_OF_TRASKEN;
	
	@ClientString(id = 47902, message = "Destroying the Eggs of Trasken (In Progress)")
	public static NpcStringId DESTROYING_THE_EGGS_OF_TRASKEN_IN_PROGRESS;
	
	@ClientString(id = 47903, message = "Destroying the Eggs of Trasken (Done)")
	public static NpcStringId DESTROYING_THE_EGGS_OF_TRASKEN_DONE;
	
	@ClientString(id = 47904, message = "Destroying the Eggs of Trasken")
	public static NpcStringId DESTROYING_THE_EGGS_OF_TRASKEN2;
	
	@ClientString(id = 48001, message = "Another Legacy of Cruma Tower (Lv. 38-99)")
	public static NpcStringId ANOTHER_LEGACY_OF_CRUMA_TOWER_LV_38_99;
	
	@ClientString(id = 48002, message = "Another Legacy of Cruma Tower (Lv. 38-99) (In Progress)")
	public static NpcStringId ANOTHER_LEGACY_OF_CRUMA_TOWER_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 48003, message = "Another Legacy of Cruma Tower (Lv. 38-99) (Done)")
	public static NpcStringId ANOTHER_LEGACY_OF_CRUMA_TOWER_LV_38_99_DONE;
	
	@ClientString(id = 48004, message = "Another Legacy of Cruma Tower (Lv. 38-99)")
	public static NpcStringId ANOTHER_LEGACY_OF_CRUMA_TOWER_LV_38_992;
	
	@ClientString(id = 48101, message = "Shadow Helper (Lv. 38-99)")
	public static NpcStringId SHADOW_HELPER_LV_38_99;
	
	@ClientString(id = 48102, message = "Shadow Helper (Lv. 38-99) (In Progress)")
	public static NpcStringId SHADOW_HELPER_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 48103, message = "Shadow Helper (Lv. 38-99) (Done)")
	public static NpcStringId SHADOW_HELPER_LV_38_99_DONE;
	
	@ClientString(id = 48104, message = "Shadow Helper (Lv. 38-99)")
	public static NpcStringId SHADOW_HELPER_LV_38_992;
	
	@ClientString(id = 48201, message = "Recertification of Value (Lv. 48-99)")
	public static NpcStringId RECERTIFICATION_OF_VALUE_LV_48_99;
	
	@ClientString(id = 48202, message = "Recertification of Value (Lv. 48-99) (In Progress)")
	public static NpcStringId RECERTIFICATION_OF_VALUE_LV_48_99_IN_PROGRESS;
	
	@ClientString(id = 48203, message = "Recertification of Value (Lv. 48-99) (Done)")
	public static NpcStringId RECERTIFICATION_OF_VALUE_LV_48_99_DONE;
	
	@ClientString(id = 48204, message = "Recertification of Value (Lv. 48-99)")
	public static NpcStringId RECERTIFICATION_OF_VALUE_LV_48_992;
	
	@ClientString(id = 48301, message = "Intended Tactic (Lv. 48-99)")
	public static NpcStringId INTENDED_TACTIC_LV_48_99;
	
	@ClientString(id = 48302, message = "Intended Tactic (Lv. 48-99) (In Progress)")
	public static NpcStringId INTENDED_TACTIC_LV_48_99_IN_PROGRESS;
	
	@ClientString(id = 48303, message = "Intended Tactic (Lv. 48-99) (Done)")
	public static NpcStringId INTENDED_TACTIC_LV_48_99_DONE;
	
	@ClientString(id = 48304, message = "Intended Tactic (Lv. 48-99)")
	public static NpcStringId INTENDED_TACTIC_LV_48_992;
	
	@ClientString(id = 48501, message = "Hot Spring Water (Lv. 70-74)")
	public static NpcStringId HOT_SPRING_WATER_LV_70_74;
	
	@ClientString(id = 48502, message = "Hot Spring Water (Lv. 70-74) (In Progress)")
	public static NpcStringId HOT_SPRING_WATER_LV_70_74_IN_PROGRESS;
	
	@ClientString(id = 48503, message = "Hot Spring Water (Lv. 70-74) (Done)")
	public static NpcStringId HOT_SPRING_WATER_LV_70_74_DONE;
	
	@ClientString(id = 48504, message = "Hot Spring Water (Lv. 70-74)")
	public static NpcStringId HOT_SPRING_WATER_LV_70_742;
	
	@ClientString(id = 48601, message = "Be Well (Lv. 70-74)")
	public static NpcStringId BE_WELL_LV_70_74;
	
	@ClientString(id = 48602, message = "Be Well (Lv. 70-74) (In Progress)")
	public static NpcStringId BE_WELL_LV_70_74_IN_PROGRESS;
	
	@ClientString(id = 48603, message = "Be Well (Lv. 70-74) (Done)")
	public static NpcStringId BE_WELL_LV_70_74_DONE;
	
	@ClientString(id = 48604, message = "Be Well (Lv. 70-74)")
	public static NpcStringId BE_WELL_LV_70_742;
	
	@ClientString(id = 48701, message = "Open Secret (Lv. 75-79)")
	public static NpcStringId OPEN_SECRET_LV_75_79;
	
	@ClientString(id = 48702, message = "Open Secret (Lv. 75-79) (In Progress)")
	public static NpcStringId OPEN_SECRET_LV_75_79_IN_PROGRESS;
	
	@ClientString(id = 48703, message = "Open Secret (Lv. 75-79) (Done)")
	public static NpcStringId OPEN_SECRET_LV_75_79_DONE;
	
	@ClientString(id = 48704, message = "Open Secret (Lv. 75-79)")
	public static NpcStringId OPEN_SECRET_LV_75_792;
	
	@ClientString(id = 48801, message = "Wonders of Caring (Lv. 75-79)")
	public static NpcStringId WONDERS_OF_CARING_LV_75_79;
	
	@ClientString(id = 48802, message = "Wonders of Caring (Lv. 75-79) (In Progress)")
	public static NpcStringId WONDERS_OF_CARING_LV_75_79_IN_PROGRESS;
	
	@ClientString(id = 48803, message = "Wonders of Caring (Lv. 75-79) (Done)")
	public static NpcStringId WONDERS_OF_CARING_LV_75_79_DONE;
	
	@ClientString(id = 48804, message = "Wonders of Caring (Lv. 75-79)")
	public static NpcStringId WONDERS_OF_CARING_LV_75_792;
	
	@ClientString(id = 48901, message = "In This Quiet Place (Lv. 75-79)")
	public static NpcStringId IN_THIS_QUIET_PLACE_LV_75_79;
	
	@ClientString(id = 48902, message = "In This Quiet Place (Lv. 75-79) (In Progress)")
	public static NpcStringId IN_THIS_QUIET_PLACE_LV_75_79_IN_PROGRESS;
	
	@ClientString(id = 48903, message = "In This Quiet Place (Lv. 75-79) (Done)")
	public static NpcStringId IN_THIS_QUIET_PLACE_LV_75_79_DONE;
	
	@ClientString(id = 48904, message = "In This Quiet Place (Lv. 75-79)")
	public static NpcStringId IN_THIS_QUIET_PLACE_LV_75_792;
	
	@ClientString(id = 49001, message = "Duty of the Survivor (Lv. 85-89)")
	public static NpcStringId DUTY_OF_THE_SURVIVOR_LV_85_89;
	
	@ClientString(id = 49002, message = "Duty of the Survivor (Lv. 85-89) (In Progress)")
	public static NpcStringId DUTY_OF_THE_SURVIVOR_LV_85_89_IN_PROGRESS;
	
	@ClientString(id = 49003, message = "Duty of the Survivor (Lv. 85-89) (Completed)")
	public static NpcStringId DUTY_OF_THE_SURVIVOR_LV_85_89_COMPLETED;
	
	@ClientString(id = 49004, message = "Duty of the Survivor (Lv. 85-89)")
	public static NpcStringId DUTY_OF_THE_SURVIVOR_LV_85_892;
	
	@ClientString(id = 49101, message = "In Nomine Patris (Lv. 76-81)")
	public static NpcStringId IN_NOMINE_PATRIS_LV_76_81;
	
	@ClientString(id = 49102, message = "In Nomine Patris (Lv. 76-81) (In Progress)")
	public static NpcStringId IN_NOMINE_PATRIS_LV_76_81_IN_PROGRESS;
	
	@ClientString(id = 49103, message = "In Nomine Patris (Lv. 76-81) (Done)")
	public static NpcStringId IN_NOMINE_PATRIS_LV_76_81_DONE;
	
	@ClientString(id = 49104, message = "In Nomine Patris (Lv. 76-81)")
	public static NpcStringId IN_NOMINE_PATRIS_LV_76_812;
	
	@ClientString(id = 49151, message = "Hurry and defeat those monsters.")
	public static NpcStringId HURRY_AND_DEFEAT_THOSE_MONSTERS;
	
	@ClientString(id = 49152, message = "Great! Thanks to you, I don't have to use my strength...")
	public static NpcStringId GREAT_THANKS_TO_YOU_I_DON_T_HAVE_TO_USE_MY_STRENGTH;
	
	@ClientString(id = 49153, message = "Hmm, thank you. So then, maybe I won't have to get involved...")
	public static NpcStringId HMM_THANK_YOU_SO_THEN_MAYBE_I_WON_T_HAVE_TO_GET_INVOLVED;
	
	@ClientString(id = 49154, message = "Thanks, but... It looks like I might have to get involved...")
	public static NpcStringId THANKS_BUT_IT_LOOKS_LIKE_I_MIGHT_HAVE_TO_GET_INVOLVED;
	
	@ClientString(id = 49155, message = "So then, I will... No, no. I don't think I need to get involved.")
	public static NpcStringId SO_THEN_I_WILL_NO_NO_I_DON_T_THINK_I_NEED_TO_GET_INVOLVED;
	
	@ClientString(id = 49201, message = "Tomb Raiders (Lv. 80-99)")
	public static NpcStringId TOMB_RAIDERS_LV_80_99;
	
	@ClientString(id = 49202, message = "Tomb Raiders (Lv. 80-99) (In Progress)")
	public static NpcStringId TOMB_RAIDERS_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 49203, message = "Tomb Raiders (Lv. 80-99) (Done)")
	public static NpcStringId TOMB_RAIDERS_LV_80_99_DONE;
	
	@ClientString(id = 49204, message = "Tomb Raiders (Lv. 80-99)")
	public static NpcStringId TOMB_RAIDERS_LV_80_992;
	
	@ClientString(id = 49250, message = "Stop the monsters from collecting the relics.")
	public static NpcStringId STOP_THE_MONSTERS_FROM_COLLECTING_THE_RELICS;
	
	@ClientString(id = 49251, message = "Thanks again for today's work. I will see you tomorrow.")
	public static NpcStringId THANKS_AGAIN_FOR_TODAY_S_WORK_I_WILL_SEE_YOU_TOMORROW;
	
	@ClientString(id = 49301, message = "Kicking Out Unwelcome Guests (Lv. 95-99)")
	public static NpcStringId KICKING_OUT_UNWELCOME_GUESTS_LV_95_99;
	
	@ClientString(id = 49302, message = "Kicking Out Unwelcome Guests (Lv. 95-99) (In Progress)")
	public static NpcStringId KICKING_OUT_UNWELCOME_GUESTS_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 49303, message = "Kicking Out Unwelcome Guests (Lv. 95-99) (Done)")
	public static NpcStringId KICKING_OUT_UNWELCOME_GUESTS_LV_95_99_DONE;
	
	@ClientString(id = 49304, message = "Kicking Out Unwelcome Guests (Lv. 95-99)")
	public static NpcStringId KICKING_OUT_UNWELCOME_GUESTS_LV_95_992;
	
	@ClientString(id = 49401, message = "Incarnation of Greed Zellaka (Lv. 85-89) (Group)")
	public static NpcStringId INCARNATION_OF_GREED_ZELLAKA_LV_85_89_GROUP;
	
	@ClientString(id = 49402, message = "Incarnation of Greed Zellaka (Lv. 85-89) (Group) (In Progress)")
	public static NpcStringId INCARNATION_OF_GREED_ZELLAKA_LV_85_89_GROUP_IN_PROGRESS;
	
	@ClientString(id = 49403, message = "Incarnation of Greed Zellaka (Lv. 85-89) (Group) (Done)")
	public static NpcStringId INCARNATION_OF_GREED_ZELLAKA_LV_85_89_GROUP_DONE;
	
	@ClientString(id = 49404, message = "Incarnation of Greed Zellaka (Lv. 85-89) (Group)")
	public static NpcStringId INCARNATION_OF_GREED_ZELLAKA_LV_85_89_GROUP2;
	
	@ClientString(id = 49501, message = "Incarnation of Jealousy Pelline (Lv. 90-94) (Group)")
	public static NpcStringId INCARNATION_OF_JEALOUSY_PELLINE_LV_90_94_GROUP;
	
	@ClientString(id = 49502, message = "Incarnation of Jealousy Pelline (Lv. 90-94) (Group) (In Progress)")
	public static NpcStringId INCARNATION_OF_JEALOUSY_PELLINE_LV_90_94_GROUP_IN_PROGRESS;
	
	@ClientString(id = 49503, message = "Incarnation of Jealousy Pelline (Lv. 90-94) (Group) (Done)")
	public static NpcStringId INCARNATION_OF_JEALOUSY_PELLINE_LV_90_94_GROUP_DONE;
	
	@ClientString(id = 49504, message = "Incarnation of Jealousy Pelline (Lv. 90-94) (Group)")
	public static NpcStringId INCARNATION_OF_JEALOUSY_PELLINE_LV_90_94_GROUP2;
	
	@ClientString(id = 49601, message = "Incarnation of Gluttony Kalios (Lv. 95-99) (Group)")
	public static NpcStringId INCARNATION_OF_GLUTTONY_KALIOS_LV_95_99_GROUP;
	
	@ClientString(id = 49602, message = "Incarnation of Gluttony Kalios (Lv. 95-99) (Group) (In Progress)")
	public static NpcStringId INCARNATION_OF_GLUTTONY_KALIOS_LV_95_99_GROUP_IN_PROGRESS;
	
	@ClientString(id = 49603, message = "Incarnation of Gluttony Kalios (Lv. 95-99) (Group) (Done)")
	public static NpcStringId INCARNATION_OF_GLUTTONY_KALIOS_LV_95_99_GROUP_DONE;
	
	@ClientString(id = 49604, message = "Incarnation of Gluttony Kalios (Lv. 95-99) (Group)")
	public static NpcStringId INCARNATION_OF_GLUTTONY_KALIOS_LV_95_99_GROUP2;
	
	@ClientString(id = 49701, message = "Incarnation of Greed Zellaka (Lv. 85-89) (Solo)")
	public static NpcStringId INCARNATION_OF_GREED_ZELLAKA_LV_85_89_SOLO;
	
	@ClientString(id = 49702, message = "Incarnation of Greed Zellaka (Lv. 85-89) (Solo) (In Progress)")
	public static NpcStringId INCARNATION_OF_GREED_ZELLAKA_LV_85_89_SOLO_IN_PROGRESS;
	
	@ClientString(id = 49703, message = "Incarnation of Greed Zellaka (Lv. 85-89) (Solo) (Done)")
	public static NpcStringId INCARNATION_OF_GREED_ZELLAKA_LV_85_89_SOLO_DONE;
	
	@ClientString(id = 49704, message = "Incarnation of Greed Zellaka (Lv. 85-89) (Solo)")
	public static NpcStringId INCARNATION_OF_GREED_ZELLAKA_LV_85_89_SOLO2;
	
	@ClientString(id = 49801, message = "Incarnation of Jealousy Pelline (Lv. 90-94) (Solo)")
	public static NpcStringId INCARNATION_OF_JEALOUSY_PELLINE_LV_90_94_SOLO;
	
	@ClientString(id = 49802, message = "Incarnation of Jealousy Pelline (Lv. 90-94) (Solo) (In Progress)")
	public static NpcStringId INCARNATION_OF_JEALOUSY_PELLINE_LV_90_94_SOLO_IN_PROGRESS;
	
	@ClientString(id = 49803, message = "Incarnation of Jealousy Pelline (Lv. 90-94) (Solo) (Done)")
	public static NpcStringId INCARNATION_OF_JEALOUSY_PELLINE_LV_90_94_SOLO_DONE;
	
	@ClientString(id = 49804, message = "Incarnation of Jealousy Pelline (Lv. 90-94) (Solo)")
	public static NpcStringId INCARNATION_OF_JEALOUSY_PELLINE_LV_90_94_SOLO2;
	
	@ClientString(id = 49901, message = "Incarnation of Gluttony Kalios (Lv. 95-99) (Solo)")
	public static NpcStringId INCARNATION_OF_GLUTTONY_KALIOS_LV_95_99_SOLO;
	
	@ClientString(id = 49902, message = "Incarnation of Gluttony Kalios (Lv. 95-99) (Solo) (In Progress)")
	public static NpcStringId INCARNATION_OF_GLUTTONY_KALIOS_LV_95_99_SOLO_IN_PROGRESS;
	
	@ClientString(id = 49903, message = "Incarnation of Gluttony Kalios (Lv. 95-99) (Solo) (Done)")
	public static NpcStringId INCARNATION_OF_GLUTTONY_KALIOS_LV_95_99_SOLO_DONE;
	
	@ClientString(id = 49904, message = "Incarnation of Gluttony Kalios (Lv. 95-99) (Solo)")
	public static NpcStringId INCARNATION_OF_GLUTTONY_KALIOS_LV_95_99_SOLO2;
	
	@ClientString(id = 50101, message = "Proof of Clan Alliance (Lv. 1-99)")
	public static NpcStringId PROOF_OF_CLAN_ALLIANCE_LV_1_99;
	
	@ClientString(id = 50102, message = "Proof of Clan Alliance (Lv. 1-99) (In Progress)")
	public static NpcStringId PROOF_OF_CLAN_ALLIANCE_LV_1_99_IN_PROGRESS;
	
	@ClientString(id = 50104, message = "Proof of Clan Alliance (Lv. 1-99)")
	public static NpcStringId PROOF_OF_CLAN_ALLIANCE_LV_1_992;
	
	@ClientString(id = 50110, message = "##########Bingo!##########")
	public static NpcStringId BINGO;
	
	@ClientString(id = 50301, message = "Pursuit of Clan Ambition! (Lv. 1-99)")
	public static NpcStringId PURSUIT_OF_CLAN_AMBITION_LV_1_99;
	
	@ClientString(id = 50302, message = "Pursuit of Clan Ambition! (Lv. 1-99) (In Progress)")
	public static NpcStringId PURSUIT_OF_CLAN_AMBITION_LV_1_99_IN_PROGRESS;
	
	@ClientString(id = 50304, message = "Pursuit of Clan Ambition! (Lv. 1-99)")
	public static NpcStringId PURSUIT_OF_CLAN_AMBITION_LV_1_992;
	
	@ClientString(id = 50338, message = "Blood and honor!")
	public static NpcStringId BLOOD_AND_HONOR;
	
	@ClientString(id = 50339, message = "Curse of the gods on the one that defiles the property of the empire!")
	public static NpcStringId CURSE_OF_THE_GODS_ON_THE_ONE_THAT_DEFILES_THE_PROPERTY_OF_THE_EMPIRE;
	
	@ClientString(id = 50340, message = "War and death!")
	public static NpcStringId WAR_AND_DEATH;
	
	@ClientString(id = 50341, message = "Ambition and power!")
	public static NpcStringId AMBITION_AND_POWER;
	
	@ClientString(id = 50401, message = "Competition for the Bandit Stronghold (Lv. 1-99)")
	public static NpcStringId COMPETITION_FOR_THE_BANDIT_STRONGHOLD_LV_1_99;
	
	@ClientString(id = 50402, message = "Competition for the Bandit Stronghold (Lv. 1-99) (In Progress)")
	public static NpcStringId COMPETITION_FOR_THE_BANDIT_STRONGHOLD_LV_1_99_IN_PROGRESS;
	
	@ClientString(id = 50404, message = "Competition for the Bandit Stronghold (Lv. 1-99)")
	public static NpcStringId COMPETITION_FOR_THE_BANDIT_STRONGHOLD_LV_1_992;
	
	@ClientString(id = 50501, message = "Blood Offering (Lv. 1-99)")
	public static NpcStringId BLOOD_OFFERING_LV_1_99;
	
	@ClientString(id = 50502, message = "Blood Offering (Lv. 1-99) (In Progress)")
	public static NpcStringId BLOOD_OFFERING_LV_1_99_IN_PROGRESS;
	
	@ClientString(id = 50503, message = "Blood Offering (Lv. 1-99) (Done)")
	public static NpcStringId BLOOD_OFFERING_LV_1_99_DONE;
	
	@ClientString(id = 50504, message = "Blood Offering (Lv. 1-99)")
	public static NpcStringId BLOOD_OFFERING_LV_1_992;
	
	@ClientString(id = 50701, message = "Into the Chaos")
	public static NpcStringId INTO_THE_CHAOS;
	
	@ClientString(id = 50702, message = "Into the Chaos  (In Progress)")
	public static NpcStringId INTO_THE_CHAOS_IN_PROGRESS;
	
	@ClientString(id = 50704, message = "Into the Chaos")
	public static NpcStringId INTO_THE_CHAOS2;
	
	@ClientString(id = 50801, message = "A Clan's Reputation (Lv. 1-99)")
	public static NpcStringId A_CLAN_S_REPUTATION_LV_1_99;
	
	@ClientString(id = 50802, message = "A Clan's Reputation (Lv. 1-99) (In Progress)")
	public static NpcStringId A_CLAN_S_REPUTATION_LV_1_99_IN_PROGRESS;
	
	@ClientString(id = 50804, message = "A Clan's Reputation (Lv. 1-99)")
	public static NpcStringId A_CLAN_S_REPUTATION_LV_1_992;
	
	@ClientString(id = 50851, message = "You have successfully completed a clan quest. $s1 points have been added to your clan's reputation score.")
	public static NpcStringId YOU_HAVE_SUCCESSFULLY_COMPLETED_A_CLAN_QUEST_S1_POINTS_HAVE_BEEN_ADDED_TO_YOUR_CLAN_S_REPUTATION_SCORE;
	
	@ClientString(id = 50853, message = "Gah...Shilen... Why must you make us suffer...")
	public static NpcStringId GAH_SHILEN_WHY_MUST_YOU_MAKE_US_SUFFER;
	
	@ClientString(id = 50854, message = "Shilen... abandoned us. It is our time... to die.")
	public static NpcStringId SHILEN_ABANDONED_US_IT_IS_OUR_TIME_TO_DIE;
	
	@ClientString(id = 50855, message = "With our sacrifice will we fulfill the prophecy?")
	public static NpcStringId WITH_OUR_SACRIFICE_WILL_WE_FULFILL_THE_PROPHECY;
	
	@ClientString(id = 50856, message = "Bloody rain, plague, death... she is near.")
	public static NpcStringId BLOODY_RAIN_PLAGUE_DEATH_SHE_IS_NEAR;
	
	@ClientString(id = 50857, message = "Arhhhh...")
	public static NpcStringId ARHHHH;
	
	@ClientString(id = 50858, message = "We offer our blood as a sacrifice! Shilen see us!")
	public static NpcStringId WE_OFFER_OUR_BLOOD_AS_A_SACRIFICE_SHILEN_SEE_US;
	
	@ClientString(id = 50859, message = "Will Dark Elves be forgotten after what we have done?")
	public static NpcStringId WILL_DARK_ELVES_BE_FORGOTTEN_AFTER_WHAT_WE_HAVE_DONE;
	
	@ClientString(id = 50860, message = "Unbelievers run... death will follow you.")
	public static NpcStringId UNBELIEVERS_RUN_DEATH_WILL_FOLLOW_YOU;
	
	@ClientString(id = 50861, message = "I curse our blood.. I despise what we are.. Shilen…")
	public static NpcStringId I_CURSE_OUR_BLOOD_I_DESPISE_WHAT_WE_ARE_SHILEN;
	
	@ClientString(id = 50901, message = "A Clan's Fame (Lv. 1-99)")
	public static NpcStringId A_CLAN_S_FAME_LV_1_99;
	
	@ClientString(id = 50902, message = "A Clan's Fame (Lv. 1-99) (In Progress)")
	public static NpcStringId A_CLAN_S_FAME_LV_1_99_IN_PROGRESS;
	
	@ClientString(id = 50904, message = "A Clan's Fame (Lv. 1-99)")
	public static NpcStringId A_CLAN_S_FAME_LV_1_992;
	
	@ClientString(id = 51001, message = "A Clan's Prestige (Lv. 1-99)")
	public static NpcStringId A_CLAN_S_PRESTIGE_LV_1_99;
	
	@ClientString(id = 51002, message = "A Clan's Prestige (Lv. 1-99) (In Progress)")
	public static NpcStringId A_CLAN_S_PRESTIGE_LV_1_99_IN_PROGRESS;
	
	@ClientString(id = 51004, message = "A Clan's Prestige (Lv. 1-99)")
	public static NpcStringId A_CLAN_S_PRESTIGE_LV_1_992;
	
	@ClientString(id = 51101, message = "Awl Under Foot (Lv. 85-99)")
	public static NpcStringId AWL_UNDER_FOOT_LV_85_99;
	
	@ClientString(id = 51102, message = "Awl Under Foot (Lv. 85-99) (In Progress)")
	public static NpcStringId AWL_UNDER_FOOT_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 51104, message = "Awl Under Foot (Lv. 85-99)")
	public static NpcStringId AWL_UNDER_FOOT_LV_85_992;
	
	@ClientString(id = 51201, message = "Hidden Blade (Lv. 90-99)")
	public static NpcStringId HIDDEN_BLADE_LV_90_99;
	
	@ClientString(id = 51202, message = "Hidden Blade (Lv. 90-99) (In Progress)")
	public static NpcStringId HIDDEN_BLADE_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 51204, message = "Hidden Blade (Lv. 90-99)")
	public static NpcStringId HIDDEN_BLADE_LV_90_992;
	
	@ClientString(id = 53901, message = "Put on the Analysis Hat and try Analysis on me.")
	public static NpcStringId PUT_ON_THE_ANALYSIS_HAT_AND_TRY_ANALYSIS_ON_ME;
	
	@ClientString(id = 53902, message = "That's how you do it!")
	public static NpcStringId THAT_S_HOW_YOU_DO_IT;
	
	@ClientString(id = 55101, message = "Olympiad Starter (Lv. 85-99)")
	public static NpcStringId OLYMPIAD_STARTER_LV_85_99;
	
	@ClientString(id = 55102, message = "Olympiad Starter  (Lv. 85-99)(In Progress)")
	public static NpcStringId OLYMPIAD_STARTER_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 55103, message = "Olympiad Starter  (Lv. 85-99)(Done)")
	public static NpcStringId OLYMPIAD_STARTER_LV_85_99_DONE;
	
	@ClientString(id = 55104, message = "Olympiad Starter (Lv. 85-99)")
	public static NpcStringId OLYMPIAD_STARTER_LV_85_992;
	
	@ClientString(id = 55201, message = "Olympiad Veteran")
	public static NpcStringId OLYMPIAD_VETERAN;
	
	@ClientString(id = 55202, message = "Olympiad Veteran (In Progress)")
	public static NpcStringId OLYMPIAD_VETERAN_IN_PROGRESS;
	
	@ClientString(id = 55203, message = "Olympiad Veteran (Done)")
	public static NpcStringId OLYMPIAD_VETERAN_DONE;
	
	@ClientString(id = 55301, message = "Olympiad Undefeated (Lv. 85-99)")
	public static NpcStringId OLYMPIAD_UNDEFEATED_LV_85_99;
	
	@ClientString(id = 55302, message = "Olympiad Undefeated  (Lv. 85-99)(In Progress)")
	public static NpcStringId OLYMPIAD_UNDEFEATED_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 55303, message = "Olympiad Undefeated (Lv. 85-99) (Done)")
	public static NpcStringId OLYMPIAD_UNDEFEATED_LV_85_99_DONE;
	
	@ClientString(id = 55304, message = "Olympiad Undefeated (Lv. 85-99)")
	public static NpcStringId OLYMPIAD_UNDEFEATED_LV_85_992;
	
	@ClientString(id = 60000, message = "I'll start the furnace mechanism. Watch for the pattern.")
	public static NpcStringId I_LL_START_THE_FURNACE_MECHANISM_WATCH_FOR_THE_PATTERN;
	
	@ClientString(id = 60001, message = "Only 1 minute left!")
	public static NpcStringId ONLY_1_MINUTE_LEFT;
	
	@ClientString(id = 60002, message = "Just 10 seconds left!")
	public static NpcStringId JUST_10_SECONDS_LEFT;
	
	@ClientString(id = 60003, message = "Now, light the furnaces in the correct order.")
	public static NpcStringId NOW_LIGHT_THE_FURNACES_IN_THE_CORRECT_ORDER;
	
	@ClientString(id = 60004, message = "Too late, the torch has run out. Maybe next time.")
	public static NpcStringId TOO_LATE_THE_TORCH_HAS_RUN_OUT_MAYBE_NEXT_TIME;
	
	@ClientString(id = 60005, message = "That's it, you've done it!")
	public static NpcStringId THAT_S_IT_YOU_VE_DONE_IT;
	
	@ClientString(id = 60006, message = "Too bad… I will not give up on this though.")
	public static NpcStringId TOO_BAD_I_WILL_NOT_GIVE_UP_ON_THIS_THOUGH;
	
	@ClientString(id = 60007, message = "I've failed. Any further attempts would be wasteful.")
	public static NpcStringId I_VE_FAILED_ANY_FURTHER_ATTEMPTS_WOULD_BE_WASTEFUL;
	
	@ClientString(id = 60008, message = "Furnace of Balance")
	public static NpcStringId FURNACE_OF_BALANCE;
	
	@ClientString(id = 60009, message = "Furnace of Protection")
	public static NpcStringId FURNACE_OF_PROTECTION;
	
	@ClientString(id = 60010, message = "Furnace of Will")
	public static NpcStringId FURNACE_OF_WILL;
	
	@ClientString(id = 60011, message = "Furnace of Magic")
	public static NpcStringId FURNACE_OF_MAGIC;
	
	@ClientString(id = 60012, message = "Divine energy is beginning to encircle.")
	public static NpcStringId DIVINE_ENERGY_IS_BEGINNING_TO_ENCIRCLE;
	
	@ClientString(id = 60013, message = "For the glory of Solina!")
	public static NpcStringId FOR_THE_GLORY_OF_SOLINA;
	
	@ClientString(id = 60014, message = "Punish all those who tread footsteps in this place.")
	public static NpcStringId PUNISH_ALL_THOSE_WHO_TREAD_FOOTSTEPS_IN_THIS_PLACE;
	
	@ClientString(id = 60015, message = "We are the sword of truth, the sword of Solina.")
	public static NpcStringId WE_ARE_THE_SWORD_OF_TRUTH_THE_SWORD_OF_SOLINA;
	
	@ClientString(id = 60016, message = "We raise our blades for the glory of Solina.")
	public static NpcStringId WE_RAISE_OUR_BLADES_FOR_THE_GLORY_OF_SOLINA;
	
	@ClientString(id = 60017, message = "For the glory of Solina!")
	public static NpcStringId FOR_THE_GLORY_OF_SOLINA2;
	
	@ClientString(id = 60018, message = "Hey, don't go so fast.")
	public static NpcStringId HEY_DON_T_GO_SO_FAST;
	
	@ClientString(id = 60019, message = "It's hard to follow.")
	public static NpcStringId IT_S_HARD_TO_FOLLOW;
	
	@ClientString(id = 60020, message = "Huff huff. You're too fast. I can't follow anymore.")
	public static NpcStringId HUFF_HUFF_YOU_RE_TOO_FAST_I_CAN_T_FOLLOW_ANYMORE;
	
	@ClientString(id = 60021, message = "Ah... I think I remember this place.")
	public static NpcStringId AH_I_THINK_I_REMEMBER_THIS_PLACE;
	
	@ClientString(id = 60022, message = "Ah! Fresh air!")
	public static NpcStringId AH_FRESH_AIR;
	
	@ClientString(id = 60023, message = "What were you doing here?")
	public static NpcStringId WHAT_WERE_YOU_DOING_HERE;
	
	@ClientString(id = 60024, message = "I guess you're the silent type. Then are you looking for treasure like me?")
	public static NpcStringId I_GUESS_YOU_RE_THE_SILENT_TYPE_THEN_ARE_YOU_LOOKING_FOR_TREASURE_LIKE_ME;
	
	@ClientString(id = 60101, message = "Eye of Watchman")
	public static NpcStringId EYE_OF_WATCHMAN;
	
	@ClientString(id = 60102, message = "Eye of Watchman (In Progress)")
	public static NpcStringId EYE_OF_WATCHMAN_IN_PROGRESS;
	
	@ClientString(id = 60104, message = "Eye of the Watchman")
	public static NpcStringId EYE_OF_THE_WATCHMAN;
	
	@ClientString(id = 60201, message = "Shadow of Light")
	public static NpcStringId SHADOW_OF_LIGHT;
	
	@ClientString(id = 60202, message = "Shadow of Light (In Progress)")
	public static NpcStringId SHADOW_OF_LIGHT_IN_PROGRESS;
	
	@ClientString(id = 60204, message = "Shadow of Light")
	public static NpcStringId SHADOW_OF_LIGHT2;
	
	@ClientString(id = 60301, message = "Daimon the White-eyed - Part 1")
	public static NpcStringId DAIMON_THE_WHITE_EYED_PART_1;
	
	@ClientString(id = 60302, message = "Daimon the White-eyed - Part 1 (In Progress)")
	public static NpcStringId DAIMON_THE_WHITE_EYED_PART_1_IN_PROGRESS;
	
	@ClientString(id = 60304, message = "Daimon the White-eyed - Part 1")
	public static NpcStringId DAIMON_THE_WHITE_EYED_PART_12;
	
	@ClientString(id = 60401, message = "Daimon the White-eyed - Part 2")
	public static NpcStringId DAIMON_THE_WHITE_EYED_PART_2;
	
	@ClientString(id = 60402, message = "Daimon the White-eyed - Part 2 (In Progress)")
	public static NpcStringId DAIMON_THE_WHITE_EYED_PART_2_IN_PROGRESS;
	
	@ClientString(id = 60403, message = "Who is calling me?")
	public static NpcStringId WHO_IS_CALLING_ME;
	
	@ClientString(id = 60404, message = "Daimon the White-eyed - Part 2")
	public static NpcStringId DAIMON_THE_WHITE_EYED_PART_22;
	
	@ClientString(id = 60414, message = "Can Light Exist Without Darkness?")
	public static NpcStringId CAN_LIGHT_EXIST_WITHOUT_DARKNESS;
	
	@ClientString(id = 60501, message = "Alliance with Ketra Orcs")
	public static NpcStringId ALLIANCE_WITH_KETRA_ORCS;
	
	@ClientString(id = 60502, message = "Alliance with Ketra Orcs (In Progress)")
	public static NpcStringId ALLIANCE_WITH_KETRA_ORCS_IN_PROGRESS;
	
	@ClientString(id = 60504, message = "Alliance with Ketra Orcs")
	public static NpcStringId ALLIANCE_WITH_KETRA_ORCS2;
	
	@ClientString(id = 60601, message = "Battle against Varka Silenos")
	public static NpcStringId BATTLE_AGAINST_VARKA_SILENOS;
	
	@ClientString(id = 60602, message = "Battle against Varka Silenos (In Progress)")
	public static NpcStringId BATTLE_AGAINST_VARKA_SILENOS_IN_PROGRESS;
	
	@ClientString(id = 60604, message = "Battle against Varka Silenos")
	public static NpcStringId BATTLE_AGAINST_VARKA_SILENOS2;
	
	@ClientString(id = 60701, message = "Prove Your Courage! (Ketra)")
	public static NpcStringId PROVE_YOUR_COURAGE_KETRA;
	
	@ClientString(id = 60702, message = "Prove Your Courage! (Ketra) (In Progress)")
	public static NpcStringId PROVE_YOUR_COURAGE_KETRA_IN_PROGRESS;
	
	@ClientString(id = 60704, message = "Prove Your Courage! (Ketra)")
	public static NpcStringId PROVE_YOUR_COURAGE_KETRA2;
	
	@ClientString(id = 60801, message = "Slay the Enemy Commander! (Ketra)")
	public static NpcStringId SLAY_THE_ENEMY_COMMANDER_KETRA;
	
	@ClientString(id = 60802, message = "Slay the Enemy Commander! (Ketra) (In Progress)")
	public static NpcStringId SLAY_THE_ENEMY_COMMANDER_KETRA_IN_PROGRESS;
	
	@ClientString(id = 60804, message = "Slay the Enemy Commander! (Ketra)")
	public static NpcStringId SLAY_THE_ENEMY_COMMANDER_KETRA2;
	
	@ClientString(id = 60901, message = "Magical Power of Water - Part 1")
	public static NpcStringId MAGICAL_POWER_OF_WATER_PART_1;
	
	@ClientString(id = 60902, message = "Magical Power of Water - Part 1 (In Progress)")
	public static NpcStringId MAGICAL_POWER_OF_WATER_PART_1_IN_PROGRESS;
	
	@ClientString(id = 60903, message = "You can't avoid the eyes of Udan!")
	public static NpcStringId YOU_CAN_T_AVOID_THE_EYES_OF_UDAN;
	
	@ClientString(id = 60904, message = "Magical Power of Water - Part 1")
	public static NpcStringId MAGICAL_POWER_OF_WATER_PART_12;
	
	@ClientString(id = 60914, message = "Udan has Already Seen Your Face!")
	public static NpcStringId UDAN_HAS_ALREADY_SEEN_YOUR_FACE;
	
	@ClientString(id = 61001, message = "Magical Power of Water - Part 2")
	public static NpcStringId MAGICAL_POWER_OF_WATER_PART_2;
	
	@ClientString(id = 61002, message = "Magical Power of Water - Part 2 (In Progress)")
	public static NpcStringId MAGICAL_POWER_OF_WATER_PART_2_IN_PROGRESS;
	
	@ClientString(id = 61004, message = "Magical Power of Water - Part 2")
	public static NpcStringId MAGICAL_POWER_OF_WATER_PART_22;
	
	@ClientString(id = 61050, message = "The magical power of water comes from the power of storm and hail!  If you dare to confront it, only death will await you!")
	public static NpcStringId THE_MAGICAL_POWER_OF_WATER_COMES_FROM_THE_POWER_OF_STORM_AND_HAIL_IF_YOU_DARE_TO_CONFRONT_IT_ONLY_DEATH_WILL_AWAIT_YOU;
	
	@ClientString(id = 61051, message = "The power of constraint is getting weaker.  Your ritual has failed!")
	public static NpcStringId THE_POWER_OF_CONSTRAINT_IS_GETTING_WEAKER_YOUR_RITUAL_HAS_FAILED;
	
	@ClientString(id = 61101, message = "Alliance with Varka Silenos")
	public static NpcStringId ALLIANCE_WITH_VARKA_SILENOS;
	
	@ClientString(id = 61102, message = "Alliance with Varka Silenos (In Progress)")
	public static NpcStringId ALLIANCE_WITH_VARKA_SILENOS_IN_PROGRESS;
	
	@ClientString(id = 61104, message = "Alliance with Varka Silenos")
	public static NpcStringId ALLIANCE_WITH_VARKA_SILENOS2;
	
	@ClientString(id = 61201, message = "Battle against Ketra Orcs")
	public static NpcStringId BATTLE_AGAINST_KETRA_ORCS;
	
	@ClientString(id = 61202, message = "Battle against Ketra Orcs  (In Progress)")
	public static NpcStringId BATTLE_AGAINST_KETRA_ORCS_IN_PROGRESS;
	
	@ClientString(id = 61204, message = "Battle against Ketra Orcs")
	public static NpcStringId BATTLE_AGAINST_KETRA_ORCS2;
	
	@ClientString(id = 61301, message = "Prove Your Courage! (Varka)")
	public static NpcStringId PROVE_YOUR_COURAGE_VARKA;
	
	@ClientString(id = 61302, message = "Prove Your Courage! (Varka) (In Progress)")
	public static NpcStringId PROVE_YOUR_COURAGE_VARKA_IN_PROGRESS;
	
	@ClientString(id = 61304, message = "Prove Your Courage! (Varka)")
	public static NpcStringId PROVE_YOUR_COURAGE_VARKA2;
	
	@ClientString(id = 61401, message = "Slay the Enemy Commander! (Varka)")
	public static NpcStringId SLAY_THE_ENEMY_COMMANDER_VARKA;
	
	@ClientString(id = 61402, message = "Slay the Enemy Commander! (Varka) (In Progress)")
	public static NpcStringId SLAY_THE_ENEMY_COMMANDER_VARKA_IN_PROGRESS;
	
	@ClientString(id = 61404, message = "Slay the Enemy Commander! (Varka)")
	public static NpcStringId SLAY_THE_ENEMY_COMMANDER_VARKA2;
	
	@ClientString(id = 61501, message = "Magical Power of Fire - Part 1")
	public static NpcStringId MAGICAL_POWER_OF_FIRE_PART_1;
	
	@ClientString(id = 61502, message = "Magical Power of Fire - Part 1 (In Progress)")
	public static NpcStringId MAGICAL_POWER_OF_FIRE_PART_1_IN_PROGRESS;
	
	@ClientString(id = 61503, message = "You can't avoid the eyes of Asefa!")
	public static NpcStringId YOU_CAN_T_AVOID_THE_EYES_OF_ASEFA;
	
	@ClientString(id = 61504, message = "Magical Power of Fire - Part 1")
	public static NpcStringId MAGICAL_POWER_OF_FIRE_PART_12;
	
	@ClientString(id = 61514, message = "Asefa has Already Seen Your Face!")
	public static NpcStringId ASEFA_HAS_ALREADY_SEEN_YOUR_FACE;
	
	@ClientString(id = 61601, message = "Magical Power of Fire - Part 2")
	public static NpcStringId MAGICAL_POWER_OF_FIRE_PART_2;
	
	@ClientString(id = 61602, message = "Magical Power of Fire - Part 2 (In Progress)")
	public static NpcStringId MAGICAL_POWER_OF_FIRE_PART_2_IN_PROGRESS;
	
	@ClientString(id = 61604, message = "Magical Power of Fire - Part 2")
	public static NpcStringId MAGICAL_POWER_OF_FIRE_PART_22;
	
	@ClientString(id = 61650, message = "The magical power of fire is also the power of flames and lava!  If you dare to confront it, only death will await you!")
	public static NpcStringId THE_MAGICAL_POWER_OF_FIRE_IS_ALSO_THE_POWER_OF_FLAMES_AND_LAVA_IF_YOU_DARE_TO_CONFRONT_IT_ONLY_DEATH_WILL_AWAIT_YOU;
	
	@ClientString(id = 61651, message = "The power of constraint is getting weaker.  Your ritual has failed!")
	public static NpcStringId THE_POWER_OF_CONSTRAINT_IS_GETTING_WEAKER_YOUR_RITUAL_HAS_FAILED2;
	
	@ClientString(id = 61701, message = "Gather the Flames (Lv. 74-80)")
	public static NpcStringId GATHER_THE_FLAMES_LV_74_80;
	
	@ClientString(id = 61702, message = "Gather the Flames (Lv. 74-80) (In Progress)")
	public static NpcStringId GATHER_THE_FLAMES_LV_74_80_IN_PROGRESS;
	
	@ClientString(id = 61704, message = "Gather the Flames (Lv. 74-80)")
	public static NpcStringId GATHER_THE_FLAMES_LV_74_802;
	
	@ClientString(id = 61801, message = "Into the Flames (Lv. 60-99)")
	public static NpcStringId INTO_THE_FLAMES_LV_60_99;
	
	@ClientString(id = 61802, message = "Into the Flames (Lv. 60-99) (In Progress)")
	public static NpcStringId INTO_THE_FLAMES_LV_60_99_IN_PROGRESS;
	
	@ClientString(id = 61804, message = "Into the Flames (Lv. 60-99)")
	public static NpcStringId INTO_THE_FLAMES_LV_60_992;
	
	@ClientString(id = 61901, message = "Relics of the Old Empire")
	public static NpcStringId RELICS_OF_THE_OLD_EMPIRE;
	
	@ClientString(id = 61902, message = "Relics of the Old Empire (In Progress)")
	public static NpcStringId RELICS_OF_THE_OLD_EMPIRE_IN_PROGRESS;
	
	@ClientString(id = 61904, message = "Relics of the Old Empire")
	public static NpcStringId RELICS_OF_THE_OLD_EMPIRE2;
	
	@ClientString(id = 62001, message = "Four Goblets (Lv. 74-80)")
	public static NpcStringId FOUR_GOBLETS_LV_74_80;
	
	@ClientString(id = 62002, message = "Four Goblets (Lv. 74-80) (In Progress)")
	public static NpcStringId FOUR_GOBLETS_LV_74_80_IN_PROGRESS;
	
	@ClientString(id = 62004, message = "Four Goblets (Lv. 74-80)")
	public static NpcStringId FOUR_GOBLETS_LV_74_802;
	
	@ClientString(id = 62101, message = "Egg Delivery (Lv. 68-73)")
	public static NpcStringId EGG_DELIVERY_LV_68_73;
	
	@ClientString(id = 62102, message = "Egg Delivery (Lv. 68-73) (In Progress)")
	public static NpcStringId EGG_DELIVERY_LV_68_73_IN_PROGRESS;
	
	@ClientString(id = 62104, message = "Egg Delivery (Lv. 68-73)")
	public static NpcStringId EGG_DELIVERY_LV_68_732;
	
	@ClientString(id = 62201, message = "Specialty Liquor Delivery (Lv. 68-73)")
	public static NpcStringId SPECIALTY_LIQUOR_DELIVERY_LV_68_73;
	
	@ClientString(id = 62202, message = "Specialty Liquor Delivery (Lv. 68-73) (In Progress)")
	public static NpcStringId SPECIALTY_LIQUOR_DELIVERY_LV_68_73_IN_PROGRESS;
	
	@ClientString(id = 62204, message = "Specialty Liquor Delivery (Lv. 68-73)")
	public static NpcStringId SPECIALTY_LIQUOR_DELIVERY_LV_68_732;
	
	@ClientString(id = 62301, message = "The Finest Food (Lv. 71-78)")
	public static NpcStringId THE_FINEST_FOOD_LV_71_78;
	
	@ClientString(id = 62302, message = "The Finest Food (Lv. 71-78) (In Progress)")
	public static NpcStringId THE_FINEST_FOOD_LV_71_78_IN_PROGRESS;
	
	@ClientString(id = 62304, message = "The Finest Food (Lv. 71-78)")
	public static NpcStringId THE_FINEST_FOOD_LV_71_782;
	
	@ClientString(id = 62401, message = "The Finest Ingredients - Part 1 (Lv. 73-80)")
	public static NpcStringId THE_FINEST_INGREDIENTS_PART_1_LV_73_80;
	
	@ClientString(id = 62402, message = "The Finest Ingredients - Part 1 (Lv. 73-80) (In Progress)")
	public static NpcStringId THE_FINEST_INGREDIENTS_PART_1_LV_73_80_IN_PROGRESS;
	
	@ClientString(id = 62404, message = "The Finest Ingredients - Part 1 (Lv. 73-80)")
	public static NpcStringId THE_FINEST_INGREDIENTS_PART_1_LV_73_802;
	
	@ClientString(id = 62501, message = "The Finest Ingredients - Part 2 (Lv. 73-80)")
	public static NpcStringId THE_FINEST_INGREDIENTS_PART_2_LV_73_80;
	
	@ClientString(id = 62502, message = "The Finest Ingredients - Part 2 (Lv. 73-80) (In Progress)")
	public static NpcStringId THE_FINEST_INGREDIENTS_PART_2_LV_73_80_IN_PROGRESS;
	
	@ClientString(id = 62503, message = "The Finest Ingredients - Part 2 (Lv. 73-80) (Done)")
	public static NpcStringId THE_FINEST_INGREDIENTS_PART_2_LV_73_80_DONE;
	
	@ClientString(id = 62504, message = "The Finest Ingredients - Part 2 (Lv. 73-80)")
	public static NpcStringId THE_FINEST_INGREDIENTS_PART_2_LV_73_802;
	
	@ClientString(id = 62514, message = "Oooh!")
	public static NpcStringId OOOH;
	
	@ClientString(id = 62601, message = "A Dark Twilight (Lv. 60-71)")
	public static NpcStringId A_DARK_TWILIGHT_LV_60_71;
	
	@ClientString(id = 62602, message = "A Dark Twilight (Lv. 60-71) (In Progress)")
	public static NpcStringId A_DARK_TWILIGHT_LV_60_71_IN_PROGRESS;
	
	@ClientString(id = 62604, message = "Dark Twilight (Lv. 60-71)")
	public static NpcStringId DARK_TWILIGHT_LV_60_71;
	
	@ClientString(id = 62701, message = "Heart in Search of Power (Lv. 60-71)")
	public static NpcStringId HEART_IN_SEARCH_OF_POWER_LV_60_71;
	
	@ClientString(id = 62702, message = "Heart in Search of Power (Lv. 60-71) (In Progress)")
	public static NpcStringId HEART_IN_SEARCH_OF_POWER_LV_60_71_IN_PROGRESS;
	
	@ClientString(id = 62704, message = "Heart in Search of Power (Lv. 60-71)")
	public static NpcStringId HEART_IN_SEARCH_OF_POWER_LV_60_712;
	
	@ClientString(id = 62801, message = "Hunt of the Golden Ram Mercenary Force")
	public static NpcStringId HUNT_OF_THE_GOLDEN_RAM_MERCENARY_FORCE;
	
	@ClientString(id = 62802, message = "Hunt of the Golden Ram Mercenary Force (In Progress)")
	public static NpcStringId HUNT_OF_THE_GOLDEN_RAM_MERCENARY_FORCE_IN_PROGRESS;
	
	@ClientString(id = 62804, message = "Hunt of the Golden Ram Mercenary Force")
	public static NpcStringId HUNT_OF_THE_GOLDEN_RAM_MERCENARY_FORCE2;
	
	@ClientString(id = 62901, message = "Clean up the Swamp of Screams")
	public static NpcStringId CLEAN_UP_THE_SWAMP_OF_SCREAMS;
	
	@ClientString(id = 62902, message = "Clean up the Swamp of Screams (In Progress)")
	public static NpcStringId CLEAN_UP_THE_SWAMP_OF_SCREAMS_IN_PROGRESS;
	
	@ClientString(id = 62904, message = "Clean Up the Swamp of Screams")
	public static NpcStringId CLEAN_UP_THE_SWAMP_OF_SCREAMS2;
	
	@ClientString(id = 63101, message = "Delicious Top Choice Meat (Lv. 82-99)")
	public static NpcStringId DELICIOUS_TOP_CHOICE_MEAT_LV_82_99;
	
	@ClientString(id = 63102, message = "Delicious Top Choice Meat (Lv. 82-99) (In Progress)")
	public static NpcStringId DELICIOUS_TOP_CHOICE_MEAT_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 63201, message = "Necromancer's Request")
	public static NpcStringId NECROMANCER_S_REQUEST;
	
	@ClientString(id = 63202, message = "Necromancer's Request (In Progress)")
	public static NpcStringId NECROMANCER_S_REQUEST_IN_PROGRESS;
	
	@ClientString(id = 63204, message = "Necromancer's Request")
	public static NpcStringId NECROMANCER_S_REQUEST2;
	
	@ClientString(id = 63301, message = "In the Forgotten Village")
	public static NpcStringId IN_THE_FORGOTTEN_VILLAGE;
	
	@ClientString(id = 63302, message = "In the Forgotten Village (In Progress)")
	public static NpcStringId IN_THE_FORGOTTEN_VILLAGE_IN_PROGRESS;
	
	@ClientString(id = 63304, message = "In the Forgotten Village")
	public static NpcStringId IN_THE_FORGOTTEN_VILLAGE2;
	
	@ClientString(id = 63401, message = "In Search of Fragments of Dimension")
	public static NpcStringId IN_SEARCH_OF_FRAGMENTS_OF_DIMENSION;
	
	@ClientString(id = 63402, message = "In Search of Fragments of Dimension (In Progress)")
	public static NpcStringId IN_SEARCH_OF_FRAGMENTS_OF_DIMENSION_IN_PROGRESS;
	
	@ClientString(id = 63501, message = "Into the Dimensional Rift")
	public static NpcStringId INTO_THE_DIMENSIONAL_RIFT;
	
	@ClientString(id = 63502, message = "Into the Dimensional Rift (In Progress)")
	public static NpcStringId INTO_THE_DIMENSIONAL_RIFT_IN_PROGRESS;
	
	@ClientString(id = 63601, message = "Truth Beyond the Gate")
	public static NpcStringId TRUTH_BEYOND_THE_GATE;
	
	@ClientString(id = 63602, message = "Truth Beyond the Gate (In Progress)")
	public static NpcStringId TRUTH_BEYOND_THE_GATE_IN_PROGRESS;
	
	@ClientString(id = 63604, message = "Truth Beyond the Gate")
	public static NpcStringId TRUTH_BEYOND_THE_GATE2;
	
	@ClientString(id = 63701, message = "Through the Gate Once More")
	public static NpcStringId THROUGH_THE_GATE_ONCE_MORE;
	
	@ClientString(id = 63702, message = "Through the Gate Once More (In Progress)")
	public static NpcStringId THROUGH_THE_GATE_ONCE_MORE_IN_PROGRESS;
	
	@ClientString(id = 63704, message = "Through the Gate Once More")
	public static NpcStringId THROUGH_THE_GATE_ONCE_MORE2;
	
	@ClientString(id = 63801, message = "Seekers of the Holy Grail")
	public static NpcStringId SEEKERS_OF_THE_HOLY_GRAIL;
	
	@ClientString(id = 63802, message = "Seekers of the Holy Grail (In Progress)")
	public static NpcStringId SEEKERS_OF_THE_HOLY_GRAIL_IN_PROGRESS;
	
	@ClientString(id = 63804, message = "Seekers of the Holy Grail")
	public static NpcStringId SEEKERS_OF_THE_HOLY_GRAIL2;
	
	@ClientString(id = 63901, message = "Guardians of the Holy Grail")
	public static NpcStringId GUARDIANS_OF_THE_HOLY_GRAIL;
	
	@ClientString(id = 63902, message = "Guardians of the Holy Grail (In Progress)")
	public static NpcStringId GUARDIANS_OF_THE_HOLY_GRAIL_IN_PROGRESS;
	
	@ClientString(id = 63904, message = "Guardians of the Holy Grail")
	public static NpcStringId GUARDIANS_OF_THE_HOLY_GRAIL2;
	
	@ClientString(id = 64001, message = "The Zero Hour (Lv. 81-99)")
	public static NpcStringId THE_ZERO_HOUR_LV_81_99;
	
	@ClientString(id = 64002, message = "The Zero Hour (Lv. 81-99) (In Progress)")
	public static NpcStringId THE_ZERO_HOUR_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 64003, message = "The Zero Hour (Lv. 81-99) (Done)")
	public static NpcStringId THE_ZERO_HOUR_LV_81_99_DONE;
	
	@ClientString(id = 64004, message = "The Zero Hour (Lv. 81-99)")
	public static NpcStringId THE_ZERO_HOUR_LV_81_992;
	
	@ClientString(id = 64101, message = "Attack Sailren! (Lv. 77-99)")
	public static NpcStringId ATTACK_SAILREN_LV_77_99;
	
	@ClientString(id = 64102, message = "Attack Sailren! (Lv. 77-99) (In Progress)")
	public static NpcStringId ATTACK_SAILREN_LV_77_99_IN_PROGRESS;
	
	@ClientString(id = 64104, message = "Attack Sailren! (Lv. 77-99)")
	public static NpcStringId ATTACK_SAILREN_LV_77_992;
	
	@ClientString(id = 64201, message = "A Powerful Primeval Creature (Lv. 75-99)")
	public static NpcStringId A_POWERFUL_PRIMEVAL_CREATURE_LV_75_99;
	
	@ClientString(id = 64202, message = "A Powerful Primeval Creature (Lv. 75-99) (In Progress)")
	public static NpcStringId A_POWERFUL_PRIMEVAL_CREATURE_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 64204, message = "A Powerful Primeval Creature (Lv. 75-99)")
	public static NpcStringId A_POWERFUL_PRIMEVAL_CREATURE_LV_75_992;
	
	@ClientString(id = 64301, message = "Rise and Fall of the Elroki Tribe (Lv. 75-99)")
	public static NpcStringId RISE_AND_FALL_OF_THE_ELROKI_TRIBE_LV_75_99;
	
	@ClientString(id = 64302, message = "Rise and Fall of the Elroki Tribe (Lv. 75-99) (In Progress)")
	public static NpcStringId RISE_AND_FALL_OF_THE_ELROKI_TRIBE_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 64304, message = "Rise and Fall of the Elroki Tribe (Lv. 75-99)")
	public static NpcStringId RISE_AND_FALL_OF_THE_ELROKI_TRIBE_LV_75_992;
	
	@ClientString(id = 64401, message = "Grave Robber Annihilation")
	public static NpcStringId GRAVE_ROBBER_ANNIHILATION;
	
	@ClientString(id = 64402, message = "Grave Robber Annihilation (In Progress)")
	public static NpcStringId GRAVE_ROBBER_ANNIHILATION_IN_PROGRESS;
	
	@ClientString(id = 64404, message = "Grave Robber Annihilation")
	public static NpcStringId GRAVE_ROBBER_ANNIHILATION2;
	
	@ClientString(id = 64501, message = "Ghosts of Batur (Lv. 80-99)")
	public static NpcStringId GHOSTS_OF_BATUR_LV_80_99;
	
	@ClientString(id = 64502, message = "Ghosts of Batur (Lv. 80-99) (In Progress)")
	public static NpcStringId GHOSTS_OF_BATUR_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 64504, message = "Ghosts of Batur (Lv. 80-99)")
	public static NpcStringId GHOSTS_OF_BATUR_LV_80_992;
	
	@ClientString(id = 64601, message = "Signs of Revolt")
	public static NpcStringId SIGNS_OF_REVOLT;
	
	@ClientString(id = 64602, message = "Signs of Revolt (In Progress)")
	public static NpcStringId SIGNS_OF_REVOLT_IN_PROGRESS;
	
	@ClientString(id = 64701, message = "Influx of Machines (Lv. 70-99)")
	public static NpcStringId INFLUX_OF_MACHINES_LV_70_99;
	
	@ClientString(id = 64702, message = "Influx of Machines (Lv. 70-99) (In Progress)")
	public static NpcStringId INFLUX_OF_MACHINES_LV_70_99_IN_PROGRESS;
	
	@ClientString(id = 64704, message = "Influx of Machines (Lv. 70-99)")
	public static NpcStringId INFLUX_OF_MACHINES_LV_70_992;
	
	@ClientString(id = 64801, message = "An Ice Merchant's Dream (Lv. 53-63)")
	public static NpcStringId AN_ICE_MERCHANT_S_DREAM_LV_53_63;
	
	@ClientString(id = 64802, message = "An Ice Merchant's Dream (Lv. 53-63) (In Progress)")
	public static NpcStringId AN_ICE_MERCHANT_S_DREAM_LV_53_63_IN_PROGRESS;
	
	@ClientString(id = 64804, message = "An Ice Merchant's Dream (Lv. 53-63)")
	public static NpcStringId AN_ICE_MERCHANT_S_DREAM_LV_53_632;
	
	@ClientString(id = 64901, message = "A Looter and a Railroad Man")
	public static NpcStringId A_LOOTER_AND_A_RAILROAD_MAN;
	
	@ClientString(id = 64902, message = "A Looter and a Railroad Man (In Progress)")
	public static NpcStringId A_LOOTER_AND_A_RAILROAD_MAN_IN_PROGRESS;
	
	@ClientString(id = 64904, message = "A Looter and a Railroad Man")
	public static NpcStringId A_LOOTER_AND_A_RAILROAD_MAN2;
	
	@ClientString(id = 65001, message = "A Broken Dream (Lv. 39-99)")
	public static NpcStringId A_BROKEN_DREAM_LV_39_99;
	
	@ClientString(id = 65002, message = "A Broken Dream (Lv. 39-99) (In Progress)")
	public static NpcStringId A_BROKEN_DREAM_LV_39_99_IN_PROGRESS;
	
	@ClientString(id = 65004, message = "A Broken Dream (Lv. 39-99)")
	public static NpcStringId A_BROKEN_DREAM_LV_39_992;
	
	@ClientString(id = 65101, message = "Runaway Youth")
	public static NpcStringId RUNAWAY_YOUTH;
	
	@ClientString(id = 65102, message = "Runaway Youth (In Progress)")
	public static NpcStringId RUNAWAY_YOUTH_IN_PROGRESS;
	
	@ClientString(id = 65104, message = "Runaway Youth")
	public static NpcStringId RUNAWAY_YOUTH2;
	
	@ClientString(id = 65201, message = "An Aged Ex-Adventurer (Lv. 46-99)")
	public static NpcStringId AN_AGED_EX_ADVENTURER_LV_46_99;
	
	@ClientString(id = 65202, message = "An Aged Ex-Adventurer (Lv. 46-99) (In Progress)")
	public static NpcStringId AN_AGED_EX_ADVENTURER_LV_46_99_IN_PROGRESS;
	
	@ClientString(id = 65204, message = "An Aged Ex-Adventurer (Lv. 46-99)")
	public static NpcStringId AN_AGED_EX_ADVENTURER_LV_46_992;
	
	@ClientString(id = 65301, message = "Wild Maiden")
	public static NpcStringId WILD_MAIDEN;
	
	@ClientString(id = 65302, message = "Wild Maiden (In Progress)")
	public static NpcStringId WILD_MAIDEN_IN_PROGRESS;
	
	@ClientString(id = 65304, message = "Wild Maiden")
	public static NpcStringId WILD_MAIDEN2;
	
	@ClientString(id = 65401, message = "Journey to Settlement")
	public static NpcStringId JOURNEY_TO_SETTLEMENT;
	
	@ClientString(id = 65402, message = "Journey to Settlement (In Progress)")
	public static NpcStringId JOURNEY_TO_SETTLEMENT_IN_PROGRESS;
	
	@ClientString(id = 65404, message = "Journey to Settlement")
	public static NpcStringId JOURNEY_TO_SETTLEMENT2;
	
	@ClientString(id = 65501, message = "A Grand Plan for Taming Wild Beasts (Lv. 1-99)")
	public static NpcStringId A_GRAND_PLAN_FOR_TAMING_WILD_BEASTS_LV_1_99;
	
	@ClientString(id = 65502, message = "A Grand Plan for Taming Wild Beasts (Lv. 1-99) (In Progress)")
	public static NpcStringId A_GRAND_PLAN_FOR_TAMING_WILD_BEASTS_LV_1_99_IN_PROGRESS;
	
	@ClientString(id = 65504, message = "A Grand Plan for Taming Wild Beasts (Lv. 1-99)")
	public static NpcStringId A_GRAND_PLAN_FOR_TAMING_WILD_BEASTS_LV_1_992;
	
	@ClientString(id = 65901, message = "I'd Rather Be Collecting Fairy Breath")
	public static NpcStringId I_D_RATHER_BE_COLLECTING_FAIRY_BREATH;
	
	@ClientString(id = 65902, message = "I'd Rather Be Collecting Fairy Breath (In Progress)")
	public static NpcStringId I_D_RATHER_BE_COLLECTING_FAIRY_BREATH_IN_PROGRESS;
	
	@ClientString(id = 65904, message = "I'd Rather Be Collecting Fairy Breath")
	public static NpcStringId I_D_RATHER_BE_COLLECTING_FAIRY_BREATH2;
	
	@ClientString(id = 66001, message = "Aiding the Floran Village.")
	public static NpcStringId AIDING_THE_FLORAN_VILLAGE;
	
	@ClientString(id = 66002, message = "Aiding the Floran Village (In Progress)")
	public static NpcStringId AIDING_THE_FLORAN_VILLAGE_IN_PROGRESS;
	
	@ClientString(id = 66004, message = "Aiding the Floran Village")
	public static NpcStringId AIDING_THE_FLORAN_VILLAGE2;
	
	@ClientString(id = 66101, message = "Making the Harvest Grounds Safe")
	public static NpcStringId MAKING_THE_HARVEST_GROUNDS_SAFE;
	
	@ClientString(id = 66102, message = "Making the Harvest Grounds Safe (In Progress)")
	public static NpcStringId MAKING_THE_HARVEST_GROUNDS_SAFE_IN_PROGRESS;
	
	@ClientString(id = 66104, message = "Making the Harvest Grounds Safe")
	public static NpcStringId MAKING_THE_HARVEST_GROUNDS_SAFE2;
	
	@ClientString(id = 66201, message = "A Game of Cards (Lv. 61-80)")
	public static NpcStringId A_GAME_OF_CARDS_LV_61_80;
	
	@ClientString(id = 66202, message = "A Game of Cards (Lv. 61-80) (In Progress)")
	public static NpcStringId A_GAME_OF_CARDS_LV_61_80_IN_PROGRESS;
	
	@ClientString(id = 66204, message = "A Game of Cards (Lv. 61-80)")
	public static NpcStringId A_GAME_OF_CARDS_LV_61_802;
	
	@ClientString(id = 66300, message = "No such card")
	public static NpcStringId NO_SUCH_CARD;
	
	@ClientString(id = 66301, message = "Seductive Whispers (Lv. 50-99)")
	public static NpcStringId SEDUCTIVE_WHISPERS_LV_50_99;
	
	@ClientString(id = 66302, message = "Seductive Whispers (Lv. 50-99) (In Progress)")
	public static NpcStringId SEDUCTIVE_WHISPERS_LV_50_99_IN_PROGRESS;
	
	@ClientString(id = 66304, message = "Seductive Whispers (Lv. 50-99)")
	public static NpcStringId SEDUCTIVE_WHISPERS_LV_50_992;
	
	@ClientString(id = 66311, message = "(font color='ff453d') Sun Card: 1 (/font)")
	public static NpcStringId FONT_COLOR_FF453D_SUN_CARD_1_FONT;
	
	@ClientString(id = 66312, message = "(font color='ff453d') Sun Card: 2 (/font)")
	public static NpcStringId FONT_COLOR_FF453D_SUN_CARD_2_FONT;
	
	@ClientString(id = 66313, message = "(font color='ff453d') Sun Card: 3 (/font)")
	public static NpcStringId FONT_COLOR_FF453D_SUN_CARD_3_FONT;
	
	@ClientString(id = 66314, message = "(font color='ff453d') Sun Card: 4 (/font)")
	public static NpcStringId FONT_COLOR_FF453D_SUN_CARD_4_FONT;
	
	@ClientString(id = 66315, message = "(font color='ff453d') Sun Card: 5 (/font)")
	public static NpcStringId FONT_COLOR_FF453D_SUN_CARD_5_FONT;
	
	@ClientString(id = 66321, message = "(font color='fff802') Moon Card: 1 (/font)")
	public static NpcStringId FONT_COLOR_FFF802_MOON_CARD_1_FONT;
	
	@ClientString(id = 66322, message = "(font color='fff802') Moon Card: 2 (/font)")
	public static NpcStringId FONT_COLOR_FFF802_MOON_CARD_2_FONT;
	
	@ClientString(id = 66323, message = "(font color='fff802') Moon Card: 3 (/font)")
	public static NpcStringId FONT_COLOR_FFF802_MOON_CARD_3_FONT;
	
	@ClientString(id = 66324, message = "(font color='fff802') Moon Card: 4 (/font)")
	public static NpcStringId FONT_COLOR_FFF802_MOON_CARD_4_FONT;
	
	@ClientString(id = 66325, message = "(font color='fff802') Moon Card: 5 (/font)")
	public static NpcStringId FONT_COLOR_FFF802_MOON_CARD_5_FONT;
	
	@ClientString(id = 68801, message = "Defeat the Elrokian Raiders! (Lv. 75-99)")
	public static NpcStringId DEFEAT_THE_ELROKIAN_RAIDERS_LV_75_99;
	
	@ClientString(id = 68802, message = "Defeat the Elrokian Raiders! (Lv. 75-99) (In Progress)")
	public static NpcStringId DEFEAT_THE_ELROKIAN_RAIDERS_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 68804, message = "Defeat the Elrokian Raiders! (Lv. 75-99)")
	public static NpcStringId DEFEAT_THE_ELROKIAN_RAIDERS_LV_75_992;
	
	@ClientString(id = 69001, message = "Jude's Request")
	public static NpcStringId JUDE_S_REQUEST;
	
	@ClientString(id = 69002, message = "Jude's Request (In Progress)")
	public static NpcStringId JUDE_S_REQUEST_IN_PROGRESS;
	
	@ClientString(id = 69004, message = "Jude's Request")
	public static NpcStringId JUDE_S_REQUEST2;
	
	@ClientString(id = 69101, message = "Matras' Suspicious Request")
	public static NpcStringId MATRAS_SUSPICIOUS_REQUEST;
	
	@ClientString(id = 69102, message = "Matras' Suspicious Request (In Progress)")
	public static NpcStringId MATRAS_SUSPICIOUS_REQUEST_IN_PROGRESS;
	
	@ClientString(id = 69104, message = "Matras' Suspicious Request")
	public static NpcStringId MATRAS_SUSPICIOUS_REQUEST2;
	
	@ClientString(id = 69201, message = "How to Oppose Evil")
	public static NpcStringId HOW_TO_OPPOSE_EVIL;
	
	@ClientString(id = 69202, message = "How to Oppose Evil (In Progress)")
	public static NpcStringId HOW_TO_OPPOSE_EVIL_IN_PROGRESS;
	
	@ClientString(id = 69204, message = "How to Oppose Evil")
	public static NpcStringId HOW_TO_OPPOSE_EVIL2;
	
	@ClientString(id = 69301, message = "Defeating Dragonkin Remnants")
	public static NpcStringId DEFEATING_DRAGONKIN_REMNANTS;
	
	@ClientString(id = 69302, message = "Defeating Dragonkin Remnants (In Progress)")
	public static NpcStringId DEFEATING_DRAGONKIN_REMNANTS_IN_PROGRESS;
	
	@ClientString(id = 69304, message = "Defeating Dragonkin Remnants")
	public static NpcStringId DEFEATING_DRAGONKIN_REMNANTS2;
	
	@ClientString(id = 69401, message = "Break Through the Hall of Suffering")
	public static NpcStringId BREAK_THROUGH_THE_HALL_OF_SUFFERING;
	
	@ClientString(id = 69402, message = "Break Through the Hall of Suffering (In Progress)")
	public static NpcStringId BREAK_THROUGH_THE_HALL_OF_SUFFERING_IN_PROGRESS;
	
	@ClientString(id = 69404, message = "Break through the Hall of Suffering")
	public static NpcStringId BREAK_THROUGH_THE_HALL_OF_SUFFERING2;
	
	@ClientString(id = 69501, message = "Defend the Hall of Suffering")
	public static NpcStringId DEFEND_THE_HALL_OF_SUFFERING;
	
	@ClientString(id = 69502, message = "Defend the Hall of Suffering (In Progress)")
	public static NpcStringId DEFEND_THE_HALL_OF_SUFFERING_IN_PROGRESS;
	
	@ClientString(id = 69504, message = "Defend the Hall of Suffering")
	public static NpcStringId DEFEND_THE_HALL_OF_SUFFERING2;
	
	@ClientString(id = 69601, message = "Conquer the Hall of Erosion")
	public static NpcStringId CONQUER_THE_HALL_OF_EROSION;
	
	@ClientString(id = 69602, message = "Conquer the Hall of Erosion (In Progress)")
	public static NpcStringId CONQUER_THE_HALL_OF_EROSION_IN_PROGRESS;
	
	@ClientString(id = 69604, message = "Conquer the Hall of Erosion")
	public static NpcStringId CONQUER_THE_HALL_OF_EROSION2;
	
	@ClientString(id = 69701, message = "Defend the Hall of Erosion")
	public static NpcStringId DEFEND_THE_HALL_OF_EROSION;
	
	@ClientString(id = 69702, message = "Defend the Hall of Erosion (In Progress)")
	public static NpcStringId DEFEND_THE_HALL_OF_EROSION_IN_PROGRESS;
	
	@ClientString(id = 69801, message = "Block the Lord's Escape")
	public static NpcStringId BLOCK_THE_LORD_S_ESCAPE;
	
	@ClientString(id = 69802, message = "Block the Lord's Escape (In Progress)")
	public static NpcStringId BLOCK_THE_LORD_S_ESCAPE_IN_PROGRESS;
	
	@ClientString(id = 69804, message = "Block the Lord's Escape")
	public static NpcStringId BLOCK_THE_LORD_S_ESCAPE2;
	
	@ClientString(id = 69901, message = "Guardian of the Skies (Lv. 75-99)")
	public static NpcStringId GUARDIAN_OF_THE_SKIES_LV_75_99;
	
	@ClientString(id = 69902, message = "Guardian of the Skies (Lv. 75-99) (In Progress)")
	public static NpcStringId GUARDIAN_OF_THE_SKIES_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 69904, message = "Guardian of the Skies (Lv. 75-99)")
	public static NpcStringId GUARDIAN_OF_THE_SKIES_LV_75_992;
	
	@ClientString(id = 70001, message = "Cursed Life (Lv. 75-99)")
	public static NpcStringId CURSED_LIFE_LV_75_99;
	
	@ClientString(id = 70002, message = "Cursed Life (Lv. 75-99) (In Progress)")
	public static NpcStringId CURSED_LIFE_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 70101, message = "Proof of Existence (Lv. 78-99)")
	public static NpcStringId PROOF_OF_EXISTENCE_LV_78_99;
	
	@ClientString(id = 70102, message = "Proof of Existence (Lv. 78-99) (In Progress)")
	public static NpcStringId PROOF_OF_EXISTENCE_LV_78_99_IN_PROGRESS;
	
	@ClientString(id = 70104, message = "Proof of Existence (Lv. 78-99)")
	public static NpcStringId PROOF_OF_EXISTENCE_LV_78_992;
	
	@ClientString(id = 70201, message = "A Trap for Revenge (Lv. 78-99)")
	public static NpcStringId A_TRAP_FOR_REVENGE_LV_78_99;
	
	@ClientString(id = 70202, message = "A Trap for Revenge (Lv. 78-99) (In Progress)")
	public static NpcStringId A_TRAP_FOR_REVENGE_LV_78_99_IN_PROGRESS;
	
	@ClientString(id = 70204, message = "A Trap for Revenge (Lv. 78-99)")
	public static NpcStringId A_TRAP_FOR_REVENGE_LV_78_992;
	
	@ClientString(id = 70801, message = "Path to Becoming a Lord - Gludio")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GLUDIO;
	
	@ClientString(id = 70802, message = "Path to Becoming a Lord - Gludio (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GLUDIO_IN_PROGRESS;
	
	@ClientString(id = 70804, message = "Path to Becoming a Lord - Gludio")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GLUDIO2;
	
	@ClientString(id = 70851, message = "Have you completed your preparations to become a lord?")
	public static NpcStringId HAVE_YOU_COMPLETED_YOUR_PREPARATIONS_TO_BECOME_A_LORD;
	
	@ClientString(id = 70852, message = "$s1. Now, depart!")
	public static NpcStringId S1_NOW_DEPART;
	
	@ClientString(id = 70853, message = "Go find Sayres!")
	public static NpcStringId GO_FIND_SAYRES;
	
	@ClientString(id = 70854, message = "Listen, you villagers. Our liege, who will soon become a lord, has defeated the Headless Knight. You can now rest easy!")
	public static NpcStringId LISTEN_YOU_VILLAGERS_OUR_LIEGE_WHO_WILL_SOON_BECOME_A_LORD_HAS_DEFEATED_THE_HEADLESS_KNIGHT_YOU_CAN_NOW_REST_EASY;
	
	@ClientString(id = 70855, message = "$s1! Do you dare defy my subordinates?")
	public static NpcStringId S1_DO_YOU_DARE_DEFY_MY_SUBORDINATES;
	
	@ClientString(id = 70856, message = "Does my mission to block the supplies end here?")
	public static NpcStringId DOES_MY_MISSION_TO_BLOCK_THE_SUPPLIES_END_HERE;
	
	@ClientString(id = 70859, message = "$s1 has become lord of the Town of Gludio. Long may he reign!")
	public static NpcStringId S1_HAS_BECOME_LORD_OF_THE_TOWN_OF_GLUDIO_LONG_MAY_HE_REIGN;
	
	@ClientString(id = 70901, message = "Path to Becoming a Lord - Dion")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_DION;
	
	@ClientString(id = 70902, message = "Path to Becoming a Lord - Dion (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_DION_IN_PROGRESS;
	
	@ClientString(id = 70904, message = "Path to Becoming a Lord - Dion")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_DION2;
	
	@ClientString(id = 70951, message = "Have you completed your preparations to become a lord?")
	public static NpcStringId HAVE_YOU_COMPLETED_YOUR_PREPARATIONS_TO_BECOME_A_LORD2;
	
	@ClientString(id = 70952, message = "$s1. Now, depart!")
	public static NpcStringId S1_NOW_DEPART2;
	
	@ClientString(id = 70955, message = "$s1! Do you dare defy my subordinates?")
	public static NpcStringId S1_DO_YOU_DARE_DEFY_MY_SUBORDINATES2;
	
	@ClientString(id = 70956, message = "Does my mission to block the supplies end here?")
	public static NpcStringId DOES_MY_MISSION_TO_BLOCK_THE_SUPPLIES_END_HERE2;
	
	@ClientString(id = 70957, message = "You'll see! I won't forgive you next time!")
	public static NpcStringId YOU_LL_SEE_I_WON_T_FORGIVE_YOU_NEXT_TIME;
	
	@ClientString(id = 70959, message = "$s1 has become lord of the Town of Gludio. Long may he reign!")
	public static NpcStringId S1_HAS_BECOME_LORD_OF_THE_TOWN_OF_GLUDIO_LONG_MAY_HE_REIGN2;
	
	@ClientString(id = 71001, message = "Path to Becoming a Lord - Giran")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GIRAN;
	
	@ClientString(id = 71002, message = "Path to Becoming a Lord - Giran (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GIRAN_IN_PROGRESS;
	
	@ClientString(id = 71004, message = "Path to Becoming a Lord - Giran")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GIRAN2;
	
	@ClientString(id = 71051, message = "Have you completed your preparations to become a lord?")
	public static NpcStringId HAVE_YOU_COMPLETED_YOUR_PREPARATIONS_TO_BECOME_A_LORD3;
	
	@ClientString(id = 71052, message = "It's the enemy! No mercy!")
	public static NpcStringId IT_S_THE_ENEMY_NO_MERCY;
	
	@ClientString(id = 71053, message = "What are you doing? We are still superior!")
	public static NpcStringId WHAT_ARE_YOU_DOING_WE_ARE_STILL_SUPERIOR;
	
	@ClientString(id = 71054, message = "How infuriating! This enemy...")
	public static NpcStringId HOW_INFURIATING_THIS_ENEMY;
	
	@ClientString(id = 71059, message = "$s1 has become the lord of the Town of Giran. May there be glory in the territory of Giran!")
	public static NpcStringId S1_HAS_BECOME_THE_LORD_OF_THE_TOWN_OF_GIRAN_MAY_THERE_BE_GLORY_IN_THE_TERRITORY_OF_GIRAN;
	
	@ClientString(id = 71101, message = "Path to Becoming a Lord - Innadril")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_INNADRIL;
	
	@ClientString(id = 71102, message = "Path to Becoming a Lord - Innadril (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_INNADRIL_IN_PROGRESS;
	
	@ClientString(id = 71104, message = "Path to Becoming a Lord - Innadril")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_INNADRIL2;
	
	@ClientString(id = 71151, message = "My liege! Where are you?")
	public static NpcStringId MY_LIEGE_WHERE_ARE_YOU;
	
	@ClientString(id = 71152, message = "$s1. Now, depart!")
	public static NpcStringId S1_NOW_DEPART3;
	
	@ClientString(id = 71159, message = "$s1 has become the lord of the Town of Innadril. May there be glory in the territory of Innadril!")
	public static NpcStringId S1_HAS_BECOME_THE_LORD_OF_THE_TOWN_OF_INNADRIL_MAY_THERE_BE_GLORY_IN_THE_TERRITORY_OF_INNADRIL;
	
	@ClientString(id = 71201, message = "Path to Becoming a Lord - Oren")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_OREN;
	
	@ClientString(id = 71202, message = "Path to Becoming a Lord - Oren (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_OREN_IN_PROGRESS;
	
	@ClientString(id = 71204, message = "Path to Becoming a Lord - Oren")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_OREN2;
	
	@ClientString(id = 71251, message = "Have you completed your preparations to become a lord?")
	public static NpcStringId HAVE_YOU_COMPLETED_YOUR_PREPARATIONS_TO_BECOME_A_LORD4;
	
	@ClientString(id = 71252, message = "You have found all the Nebulite Orbs!")
	public static NpcStringId YOU_HAVE_FOUND_ALL_THE_NEBULITE_ORBS;
	
	@ClientString(id = 71259, message = "$s1 has become the lord of the Town of Oren. May there be glory in the territory of Oren!")
	public static NpcStringId S1_HAS_BECOME_THE_LORD_OF_THE_TOWN_OF_OREN_MAY_THERE_BE_GLORY_IN_THE_TERRITORY_OF_OREN;
	
	@ClientString(id = 71301, message = "Path to Becoming a Lord - Aden")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_ADEN;
	
	@ClientString(id = 71302, message = "Path to Becoming a Lord - Aden (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_ADEN_IN_PROGRESS;
	
	@ClientString(id = 71304, message = "Path to Becoming a Lord - Aden")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_ADEN2;
	
	@ClientString(id = 71351, message = "$s1 has become the lord of the Town of Aden. May there be glory in the territory of Aden!")
	public static NpcStringId S1_HAS_BECOME_THE_LORD_OF_THE_TOWN_OF_ADEN_MAY_THERE_BE_GLORY_IN_THE_TERRITORY_OF_ADEN;
	
	@ClientString(id = 71401, message = "Path to Becoming a Lord - Schuttgart")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_SCHUTTGART;
	
	@ClientString(id = 71402, message = "Path to Becoming a Lord - Schuttgart (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_SCHUTTGART_IN_PROGRESS;
	
	@ClientString(id = 71404, message = "Path to Becoming a Lord - Schuttgart")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_SCHUTTGART2;
	
	@ClientString(id = 71451, message = "Have you completed your preparations to become a lord?")
	public static NpcStringId HAVE_YOU_COMPLETED_YOUR_PREPARATIONS_TO_BECOME_A_LORD5;
	
	@ClientString(id = 71459, message = "$s1 has become the lord of the Town of Schuttgart. May there be glory in the territory of Schuttgart!")
	public static NpcStringId S1_HAS_BECOME_THE_LORD_OF_THE_TOWN_OF_SCHUTTGART_MAY_THERE_BE_GLORY_IN_THE_TERRITORY_OF_SCHUTTGART;
	
	@ClientString(id = 71501, message = "Path to Becoming a Lord - Goddard")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GODDARD;
	
	@ClientString(id = 71502, message = "Path to Becoming a Lord - Goddard (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GODDARD_IN_PROGRESS;
	
	@ClientString(id = 71504, message = "Path to Becoming a Lord - Goddard")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_GODDARD2;
	
	@ClientString(id = 71551, message = "$s1, I will remember you.")
	public static NpcStringId S1_I_WILL_REMEMBER_YOU;
	
	@ClientString(id = 71601, message = "Path to Becoming a Lord - Rune")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_RUNE;
	
	@ClientString(id = 71602, message = "Path to Becoming a Lord - Rune (In Progress)")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_RUNE_IN_PROGRESS;
	
	@ClientString(id = 71604, message = "Path to Becoming a Lord - Rune")
	public static NpcStringId PATH_TO_BECOMING_A_LORD_RUNE2;
	
	@ClientString(id = 71652, message = "$s1. Now, depart!")
	public static NpcStringId S1_NOW_DEPART4;
	
	@ClientString(id = 71653, message = "Frederick is looking for you, my liege.")
	public static NpcStringId FREDERICK_IS_LOOKING_FOR_YOU_MY_LIEGE;
	
	@ClientString(id = 71654, message = "Ho ho! Did you think you could really stop trading with us?")
	public static NpcStringId HO_HO_DID_YOU_THINK_YOU_COULD_REALLY_STOP_TRADING_WITH_US;
	
	@ClientString(id = 71655, message = "You have charged into the temple.")
	public static NpcStringId YOU_HAVE_CHARGED_INTO_THE_TEMPLE;
	
	@ClientString(id = 71656, message = "You are in the midst of dealing with the heretic of Heretic Temple.")
	public static NpcStringId YOU_ARE_IN_THE_MIDST_OF_DEALING_WITH_THE_HERETIC_OF_HERETIC_TEMPLE;
	
	@ClientString(id = 71657, message = "The Heretic Temple is descending into chaos.")
	public static NpcStringId THE_HERETIC_TEMPLE_IS_DESCENDING_INTO_CHAOS;
	
	@ClientString(id = 71659, message = "$s1 has become the lord of the Town of Rune. May there be glory in the territory of Rune!")
	public static NpcStringId S1_HAS_BECOME_THE_LORD_OF_THE_TOWN_OF_RUNE_MAY_THERE_BE_GLORY_IN_THE_TERRITORY_OF_RUNE;
	
	@ClientString(id = 71701, message = "For the Sake of the Territory - Gludio")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_GLUDIO;
	
	@ClientString(id = 71702, message = "For the Sake of the Territory - Gludio (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_GLUDIO_IN_PROGRESS;
	
	@ClientString(id = 71751, message = "$s1! Raise your weapons for the sake of the territory!")
	public static NpcStringId S1_RAISE_YOUR_WEAPONS_FOR_THE_SAKE_OF_THE_TERRITORY;
	
	@ClientString(id = 71752, message = "$s1! The war is over. Lower your sword for the sake of the future.")
	public static NpcStringId S1_THE_WAR_IS_OVER_LOWER_YOUR_SWORD_FOR_THE_SAKE_OF_THE_FUTURE;
	
	@ClientString(id = 71753, message = "$s1 Territory Badge(s) and $s2 Adena")
	public static NpcStringId S1_TERRITORY_BADGE_S_AND_S2_ADENA;
	
	@ClientString(id = 71754, message = "90 Territory Badges, $s1 score(s) in Fame and $s2 Adena")
	public static NpcStringId TERRITORY_BADGES_S1_SCORE_S_IN_FAME_AND_S2_ADENA;
	
	@ClientString(id = 71755, message = "90 Territory Badges, 450 scores in Fame and $s2 Adena")
	public static NpcStringId TERRITORY_BADGES_450_SCORES_IN_FAME_AND_S2_ADENA;
	
	@ClientString(id = 71801, message = "For the Sake of the Territory - Dion")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_DION;
	
	@ClientString(id = 71802, message = "For the Sake of the Territory - Dion (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_DION_IN_PROGRESS;
	
	@ClientString(id = 71901, message = "For the Sake of the Territory - Giran")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_GIRAN;
	
	@ClientString(id = 71902, message = "For the Sake of the Territory - Giran (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_GIRAN_IN_PROGRESS;
	
	@ClientString(id = 72001, message = "For the Sake of the Territory - Innadril")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_INNADRIL;
	
	@ClientString(id = 72002, message = "For the Sake of the Territory - Innadril (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_INNADRIL_IN_PROGRESS;
	
	@ClientString(id = 72101, message = "For the Sake of the Territory - Oren")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_OREN;
	
	@ClientString(id = 72102, message = "For the Sake of the Territory - Oren (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_OREN_IN_PROGRESS;
	
	@ClientString(id = 72201, message = "For the Sake of the Territory - Aden")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_ADEN;
	
	@ClientString(id = 72202, message = "For the Sake of the Territory - Aden (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_ADEN_IN_PROGRESS;
	
	@ClientString(id = 72301, message = "For the Sake of the Territory - Schuttgart")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_SCHUTTGART;
	
	@ClientString(id = 72302, message = "For the Sake of the Territory - Schuttgart (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_SCHUTTGART_IN_PROGRESS;
	
	@ClientString(id = 72401, message = "For the Sake of the Territory - Goddard")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_GODDARD;
	
	@ClientString(id = 72402, message = "For the Sake of the Territory - Goddard (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_GODDARD_IN_PROGRESS;
	
	@ClientString(id = 72501, message = "For the Sake of the Territory - Rune")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_RUNE;
	
	@ClientString(id = 72502, message = "For the Sake of the Territory - Rune (In Progress)")
	public static NpcStringId FOR_THE_SAKE_OF_THE_TERRITORY_RUNE_IN_PROGRESS;
	
	@ClientString(id = 72601, message = "Light within the Darkness (Lv. 85-99)")
	public static NpcStringId LIGHT_WITHIN_THE_DARKNESS_LV_85_99;
	
	@ClientString(id = 72602, message = "Light within the Darkness (Lv. 85-99) (In Progress)")
	public static NpcStringId LIGHT_WITHIN_THE_DARKNESS_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 72604, message = "Light within the Darkness (Lv. 85-99)")
	public static NpcStringId LIGHT_WITHIN_THE_DARKNESS_LV_85_992;
	
	@ClientString(id = 72701, message = "Hope within the Darkness (Lv. 90-99)")
	public static NpcStringId HOPE_WITHIN_THE_DARKNESS_LV_90_99;
	
	@ClientString(id = 72702, message = "Hope within the Darkness (Lv. 90-99) (In Progress)")
	public static NpcStringId HOPE_WITHIN_THE_DARKNESS_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 72704, message = "Hope within the Darkness (Lv. 90-99)")
	public static NpcStringId HOPE_WITHIN_THE_DARKNESS_LV_90_992;
	
	@ClientString(id = 72901, message = "Protect the Territory Catapult!")
	public static NpcStringId PROTECT_THE_TERRITORY_CATAPULT;
	
	@ClientString(id = 72902, message = "Protect the Territory Catapult! (In Progress)")
	public static NpcStringId PROTECT_THE_TERRITORY_CATAPULT_IN_PROGRESS;
	
	@ClientString(id = 72903, message = "The mercenary quest number is $s1; memostate1 is $s2; and memostate2 is $s3.")
	public static NpcStringId THE_MERCENARY_QUEST_NUMBER_IS_S1_MEMOSTATE1_IS_S2_AND_MEMOSTATE2_IS_S3;
	
	@ClientString(id = 72904, message = "user_connected event occurrence success. Siege Id is $s1, Number 728 memo2 is $s2. 729/memo2 is $s3, and 255/memo1 is $s4.")
	public static NpcStringId USER_CONNECTED_EVENT_OCCURRENCE_SUCCESS_SIEGE_ID_IS_S1_NUMBER_728_MEMO2_IS_S2_729_MEMO2_IS_S3_AND_255_MEMO1_IS_S4;
	
	@ClientString(id = 72905, message = "Territory Catapult dying event catapult's territory ID $s1, party status $s2.")
	public static NpcStringId TERRITORY_CATAPULT_DYING_EVENT_CATAPULT_S_TERRITORY_ID_S1_PARTY_STATUS_S2;
	
	@ClientString(id = 72951, message = "Protect the catapult of Gludio!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_GLUDIO;
	
	@ClientString(id = 72952, message = "Protect the catapult of Dion!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_DION;
	
	@ClientString(id = 72953, message = "Protect the catapult of Giran!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_GIRAN;
	
	@ClientString(id = 72954, message = "Protect the catapult of Oren!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_OREN;
	
	@ClientString(id = 72955, message = "Protect the catapult of Aden!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_ADEN;
	
	@ClientString(id = 72956, message = "Protect the catapult of Innadril!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_INNADRIL;
	
	@ClientString(id = 72957, message = "Protect the catapult of Goddard!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_GODDARD;
	
	@ClientString(id = 72958, message = "Protect the catapult of Rune!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_RUNE;
	
	@ClientString(id = 72959, message = "Protect the catapult of Schuttgart!")
	public static NpcStringId PROTECT_THE_CATAPULT_OF_SCHUTTGART;
	
	@ClientString(id = 72961, message = "The catapult of Gludio has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_GLUDIO_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72962, message = "The catapult of Dion has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_DION_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72963, message = "The catapult of Giran has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_GIRAN_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72964, message = "The catapult of Oren has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_OREN_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72965, message = "The catapult of Aden has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_ADEN_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72966, message = "The catapult of Innadril has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_INNADRIL_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72967, message = "The catapult of Goddard has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_GODDARD_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72968, message = "The catapult of Rune has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_RUNE_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72969, message = "The catapult of Schuttgart has been destroyed!")
	public static NpcStringId THE_CATAPULT_OF_SCHUTTGART_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 72981, message = "Gludio")
	public static NpcStringId GLUDIO;
	
	@ClientString(id = 72982, message = "Dion")
	public static NpcStringId DION;
	
	@ClientString(id = 72983, message = "Giran")
	public static NpcStringId GIRAN;
	
	@ClientString(id = 72984, message = "Oren")
	public static NpcStringId OREN;
	
	@ClientString(id = 72985, message = "Aden")
	public static NpcStringId ADEN;
	
	@ClientString(id = 72986, message = "Innadril")
	public static NpcStringId INNADRIL;
	
	@ClientString(id = 72987, message = "Goddard")
	public static NpcStringId GODDARD;
	
	@ClientString(id = 72988, message = "Rune")
	public static NpcStringId RUNE;
	
	@ClientString(id = 72989, message = "Schuttgart")
	public static NpcStringId SCHUTTGART;
	
	@ClientString(id = 73051, message = "Protect the supplies safe of Gludio!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_GLUDIO;
	
	@ClientString(id = 73052, message = "Protect the supplies safe of Dion!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_DION;
	
	@ClientString(id = 73053, message = "Protect the supplies safe of Giran!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_GIRAN;
	
	@ClientString(id = 73054, message = "Protect the supplies safe of Oren!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_OREN;
	
	@ClientString(id = 73055, message = "Protect the supplies safe of Aden!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_ADEN;
	
	@ClientString(id = 73056, message = "Protect the supplies safe of Innadril!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_INNADRIL;
	
	@ClientString(id = 73057, message = "Protect the supplies safe of Goddard!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_GODDARD;
	
	@ClientString(id = 73058, message = "Protect the supplies safe of Rune!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_RUNE;
	
	@ClientString(id = 73059, message = "Protect the supplies safe of Schuttgart!")
	public static NpcStringId PROTECT_THE_SUPPLIES_SAFE_OF_SCHUTTGART;
	
	@ClientString(id = 73061, message = "The supplies safe of Gludio has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_GLUDIO_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73062, message = "The supplies safe of Dion has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_DION_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73063, message = "The supplies safe of Giran has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_GIRAN_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73064, message = "The supplies safe of Oren has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_OREN_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73065, message = "The supplies safe of Aden has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_ADEN_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73066, message = "The supplies safe of Innadril has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_INNADRIL_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73067, message = "The supplies safe of Goddard has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_GODDARD_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73068, message = "The supplies safe of Rune has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_RUNE_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73069, message = "The supplies safe of Schuttgart has been destroyed!")
	public static NpcStringId THE_SUPPLIES_SAFE_OF_SCHUTTGART_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 73101, message = "Protect the Military Association Leader")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER;
	
	@ClientString(id = 73102, message = "Protect the Military Association Leader (In Progress)")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_IN_PROGRESS;
	
	@ClientString(id = 73151, message = "Protect the Military Association Leader of Gludio!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_GLUDIO;
	
	@ClientString(id = 73152, message = "Protect the Military Association Leader of Dion!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_DION;
	
	@ClientString(id = 73153, message = "Protect the Military Association Leader of Giran!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_GIRAN;
	
	@ClientString(id = 73154, message = "Protect the Military Association Leader of Oren!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_OREN;
	
	@ClientString(id = 73155, message = "Protect the Military Association Leader of Aden!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_ADEN;
	
	@ClientString(id = 73156, message = "Protect the Military Association Leader of Innadril!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_INNADRIL;
	
	@ClientString(id = 73157, message = "Protect the Military Association Leader of Goddard!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_GODDARD;
	
	@ClientString(id = 73158, message = "Protect the Military Association Leader of Rune!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_RUNE;
	
	@ClientString(id = 73159, message = "Protect the Military Association Leader of Schuttgart!")
	public static NpcStringId PROTECT_THE_MILITARY_ASSOCIATION_LEADER_OF_SCHUTTGART;
	
	@ClientString(id = 73161, message = "The Military Association Leader of Gludio is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_GLUDIO_IS_DEAD;
	
	@ClientString(id = 73162, message = "The Military Association Leader of Dion is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_DION_IS_DEAD;
	
	@ClientString(id = 73163, message = "The Military Association Leader of Giran is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_GIRAN_IS_DEAD;
	
	@ClientString(id = 73164, message = "The Military Association Leader of Oren is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_OREN_IS_DEAD;
	
	@ClientString(id = 73165, message = "The Military Association Leader of Aden is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_ADEN_IS_DEAD;
	
	@ClientString(id = 73166, message = "The Military Association Leader of Innadril is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_INNADRIL_IS_DEAD;
	
	@ClientString(id = 73167, message = "The Military Association Leader of Goddard is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_GODDARD_IS_DEAD;
	
	@ClientString(id = 73168, message = "The Military Association Leader of Rune is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_RUNE_IS_DEAD;
	
	@ClientString(id = 73169, message = "The Military Association Leader of Schuttgart is dead!")
	public static NpcStringId THE_MILITARY_ASSOCIATION_LEADER_OF_SCHUTTGART_IS_DEAD;
	
	@ClientString(id = 73201, message = "Protect the Religious Association Leader")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER;
	
	@ClientString(id = 73202, message = "Protect the Religious Association Leader (In Progress)")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_IN_PROGRESS;
	
	@ClientString(id = 73251, message = "Protect the Religious Association Leader of Gludio!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_GLUDIO;
	
	@ClientString(id = 73252, message = "Protect the Religious Association Leader of Dion!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_DION;
	
	@ClientString(id = 73253, message = "Protect the Religious Association Leader of Giran!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_GIRAN;
	
	@ClientString(id = 73254, message = "Protect the Religious Association Leader of Oren!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_OREN;
	
	@ClientString(id = 73255, message = "Protect the Religious Association Leader of Aden!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_ADEN;
	
	@ClientString(id = 73256, message = "Protect the Religious Association Leader of Innadril!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_INNADRIL;
	
	@ClientString(id = 73257, message = "Protect the Religious Association Leader of Goddard!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_GODDARD;
	
	@ClientString(id = 73258, message = "Protect the Religious Association Leader of Rune!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_RUNE;
	
	@ClientString(id = 73259, message = "Protect the Religious Association Leader of Schuttgart!")
	public static NpcStringId PROTECT_THE_RELIGIOUS_ASSOCIATION_LEADER_OF_SCHUTTGART;
	
	@ClientString(id = 73261, message = "The Religious Association Leader of Gludio is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_GLUDIO_IS_DEAD;
	
	@ClientString(id = 73262, message = "The Religious Association Leader of Dion is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_DION_IS_DEAD;
	
	@ClientString(id = 73263, message = "The Religious Association Leader of Giran is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_GIRAN_IS_DEAD;
	
	@ClientString(id = 73264, message = "The Religious Association Leader of Oren is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_OREN_IS_DEAD;
	
	@ClientString(id = 73265, message = "The Religious Association Leader of Aden is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_ADEN_IS_DEAD;
	
	@ClientString(id = 73266, message = "The Religious Association Leader of Innadril is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_INNADRIL_IS_DEAD;
	
	@ClientString(id = 73267, message = "The Religious Association Leader of Goddard is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_GODDARD_IS_DEAD;
	
	@ClientString(id = 73268, message = "The Religious Association Leader of Rune is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_RUNE_IS_DEAD;
	
	@ClientString(id = 73269, message = "The Religious Association Leader of Schuttgart is dead!")
	public static NpcStringId THE_RELIGIOUS_ASSOCIATION_LEADER_OF_SCHUTTGART_IS_DEAD;
	
	@ClientString(id = 73301, message = "Protect the Economic Association Leader")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER;
	
	@ClientString(id = 73302, message = "Protect the Economic Association Leader (In Progress)")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_IN_PROGRESS;
	
	@ClientString(id = 73351, message = "Protect the Economic Association Leader of Gludio!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_GLUDIO;
	
	@ClientString(id = 73352, message = "Protect the Economic Association Leader of Dion!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_DION;
	
	@ClientString(id = 73353, message = "Protect the Economic Association Leader of Giran!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_GIRAN;
	
	@ClientString(id = 73354, message = "Protect the Economic Association Leader of Oren!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_OREN;
	
	@ClientString(id = 73355, message = "Protect the Economic Association Leader of Aden!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_ADEN;
	
	@ClientString(id = 73356, message = "Protect the Economic Association Leader of Innadril!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_INNADRIL;
	
	@ClientString(id = 73357, message = "Protect the Economic Association Leader of Goddard!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_GODDARD;
	
	@ClientString(id = 73358, message = "Protect the Economic Association Leader of Rune!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_RUNE;
	
	@ClientString(id = 73359, message = "Protect the Economic Association Leader of Schuttgart!")
	public static NpcStringId PROTECT_THE_ECONOMIC_ASSOCIATION_LEADER_OF_SCHUTTGART;
	
	@ClientString(id = 73361, message = "The Economic Association Leader of Gludio is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_GLUDIO_IS_DEAD;
	
	@ClientString(id = 73362, message = "The Economic Association Leader of Dion is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_DION_IS_DEAD;
	
	@ClientString(id = 73363, message = "The Economic Association Leader of Giran is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_GIRAN_IS_DEAD;
	
	@ClientString(id = 73364, message = "The Economic Association Leader of Oren is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_OREN_IS_DEAD;
	
	@ClientString(id = 73365, message = "The Economic Association Leader of Aden is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_ADEN_IS_DEAD;
	
	@ClientString(id = 73366, message = "The Economic Association Leader of Innadril is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_INNADRIL_IS_DEAD;
	
	@ClientString(id = 73367, message = "The Economic Association Leader of Goddard is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_GODDARD_IS_DEAD;
	
	@ClientString(id = 73368, message = "The Economic Association Leader of Rune is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_RUNE_IS_DEAD;
	
	@ClientString(id = 73369, message = "The Economic Association Leader of Schuttgart is dead!")
	public static NpcStringId THE_ECONOMIC_ASSOCIATION_LEADER_OF_SCHUTTGART_IS_DEAD;
	
	@ClientString(id = 73451, message = "Defeat $s1 enemy knights!")
	public static NpcStringId DEFEAT_S1_ENEMY_KNIGHTS;
	
	@ClientString(id = 73461, message = "You have defeated $s2 of $s1 knights.")
	public static NpcStringId YOU_HAVE_DEFEATED_S2_OF_S1_KNIGHTS;
	
	@ClientString(id = 73462, message = "You weakened the enemy's defense!")
	public static NpcStringId YOU_WEAKENED_THE_ENEMY_S_DEFENSE;
	
	@ClientString(id = 73551, message = "Defeat $s1 warriors and rogues!")
	public static NpcStringId DEFEAT_S1_WARRIORS_AND_ROGUES;
	
	@ClientString(id = 73561, message = "You have defeated $s2 of $s1 warriors and rogues.")
	public static NpcStringId YOU_HAVE_DEFEATED_S2_OF_S1_WARRIORS_AND_ROGUES;
	
	@ClientString(id = 73562, message = "You weakened the enemy's attack!")
	public static NpcStringId YOU_WEAKENED_THE_ENEMY_S_ATTACK;
	
	@ClientString(id = 73651, message = "Defeat $s1 wizards and summoners!")
	public static NpcStringId DEFEAT_S1_WIZARDS_AND_SUMMONERS;
	
	@ClientString(id = 73661, message = "You have defeated $s2 of $s1 enemies.")
	public static NpcStringId YOU_HAVE_DEFEATED_S2_OF_S1_ENEMIES;
	
	@ClientString(id = 73662, message = "You weakened the enemy's magic!")
	public static NpcStringId YOU_WEAKENED_THE_ENEMY_S_MAGIC;
	
	@ClientString(id = 73751, message = "Defeat $s1 healers and buffers.")
	public static NpcStringId DEFEAT_S1_HEALERS_AND_BUFFERS;
	
	@ClientString(id = 73761, message = "You have defeated $s2 of $s1 healers and buffers.")
	public static NpcStringId YOU_HAVE_DEFEATED_S2_OF_S1_HEALERS_AND_BUFFERS;
	
	@ClientString(id = 73762, message = "You have weakened the enemy's support!")
	public static NpcStringId YOU_HAVE_WEAKENED_THE_ENEMY_S_SUPPORT;
	
	@ClientString(id = 73851, message = "Defeat $s1 warsmiths and overlords.")
	public static NpcStringId DEFEAT_S1_WARSMITHS_AND_OVERLORDS;
	
	@ClientString(id = 73861, message = "You have defeated $s2 of $s1 warsmiths and overlords.")
	public static NpcStringId YOU_HAVE_DEFEATED_S2_OF_S1_WARSMITHS_AND_OVERLORDS;
	
	@ClientString(id = 73862, message = "You destroyed the enemy's professionals!")
	public static NpcStringId YOU_DESTROYED_THE_ENEMY_S_PROFESSIONALS;
	
	@ClientString(id = 74101, message = "Castle of Darkness: Guardian of Light")
	public static NpcStringId CASTLE_OF_DARKNESS_GUARDIAN_OF_LIGHT;
	
	@ClientString(id = 74102, message = "Castle of Darkness: Guardian of Light (In Progress)")
	public static NpcStringId CASTLE_OF_DARKNESS_GUARDIAN_OF_LIGHT_IN_PROGRESS;
	
	@ClientString(id = 74104, message = "Castle of Darkness: Guardian of Light")
	public static NpcStringId CASTLE_OF_DARKNESS_GUARDIAN_OF_LIGHT2;
	
	@ClientString(id = 74151, message = "Please save the darkness-filled castle!")
	public static NpcStringId PLEASE_SAVE_THE_DARKNESS_FILLED_CASTLE;
	
	@ClientString(id = 74152, message = "You and your fellow clan members have completed a great feat!")
	public static NpcStringId YOU_AND_YOUR_FELLOW_CLAN_MEMBERS_HAVE_COMPLETED_A_GREAT_FEAT;
	
	@ClientString(id = 74201, message = "Castle of Darkness: Liberation")
	public static NpcStringId CASTLE_OF_DARKNESS_LIBERATION;
	
	@ClientString(id = 74202, message = "Castle of Darkness: Liberation (In Progress)")
	public static NpcStringId CASTLE_OF_DARKNESS_LIBERATION_IN_PROGRESS;
	
	@ClientString(id = 74204, message = "Castle of Darkness: Liberation")
	public static NpcStringId CASTLE_OF_DARKNESS_LIBERATION2;
	
	@ClientString(id = 74251, message = "You liberated the darkness-filled castle!")
	public static NpcStringId YOU_LIBERATED_THE_DARKNESS_FILLED_CASTLE;
	
	@ClientString(id = 74252, message = "You and your comrades have completed a great feat!")
	public static NpcStringId YOU_AND_YOUR_COMRADES_HAVE_COMPLETED_A_GREAT_FEAT;
	
	@ClientString(id = 75001, message = "Seven Flowers (Lv. 95-99)")
	public static NpcStringId SEVEN_FLOWERS_LV_95_99;
	
	@ClientString(id = 75002, message = "Seven Flowers (Lv. 95-99) (In Progress)")
	public static NpcStringId SEVEN_FLOWERS_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 75003, message = "Seven Flowers (Lv. 95-99) (Done)")
	public static NpcStringId SEVEN_FLOWERS_LV_95_99_DONE;
	
	@ClientString(id = 75004, message = "Seven Flowers (Lv. 95-99)")
	public static NpcStringId SEVEN_FLOWERS_LV_95_992;
	
	@ClientString(id = 75011, message = "$s1's Fresh Flower")
	public static NpcStringId S1_S_FRESH_FLOWER;
	
	@ClientString(id = 75012, message = "$s1's Flower")
	public static NpcStringId S1_S_FLOWER;
	
	@ClientString(id = 75013, message = "$s1's Wilted Flower")
	public static NpcStringId S1_S_WILTED_FLOWER;
	
	@ClientString(id = 75014, message = "$s1 has received one White Chrysanthemum from $s2.")
	public static NpcStringId S1_HAS_RECEIVED_ONE_WHITE_CHRYSANTHEMUM_FROM_S2;
	
	@ClientString(id = 75015, message = "$s1 has received one Daylily from $s2.")
	public static NpcStringId S1_HAS_RECEIVED_ONE_DAYLILY_FROM_S2;
	
	@ClientString(id = 75016, message = "$s1 has received one Peony from $s2.")
	public static NpcStringId S1_HAS_RECEIVED_ONE_PEONY_FROM_S2;
	
	@ClientString(id = 75017, message = "$s1 has received one Petunia from $s2.")
	public static NpcStringId S1_HAS_RECEIVED_ONE_PETUNIA_FROM_S2;
	
	@ClientString(id = 75018, message = "$s1 has received one Pansy from $s2.")
	public static NpcStringId S1_HAS_RECEIVED_ONE_PANSY_FROM_S2;
	
	@ClientString(id = 75019, message = "$s1 has received one Iris from $s2.")
	public static NpcStringId S1_HAS_RECEIVED_ONE_IRIS_FROM_S2;
	
	@ClientString(id = 75020, message = "$s1 has received one Kuon from $s2.")
	public static NpcStringId S1_HAS_RECEIVED_ONE_KUON_FROM_S2;
	
	@ClientString(id = 75021, message = "Mysterious Seed")
	public static NpcStringId MYSTERIOUS_SEED;
	
	@ClientString(id = 75022, message = "Expand Flower Types")
	public static NpcStringId EXPAND_FLOWER_TYPES;
	
	@ClientString(id = 75051, message = "Mandragoras are located throughout the Guillotine Fortress.")
	public static NpcStringId MANDRAGORAS_ARE_LOCATED_THROUGHOUT_THE_GUILLOTINE_FORTRESS;
	
	@ClientString(id = 75052, message = "I have lingering feelings, but you have given me comfort.")
	public static NpcStringId I_HAVE_LINGERING_FEELINGS_BUT_YOU_HAVE_GIVEN_ME_COMFORT;
	
	@ClientString(id = 75053, message = "You have given me comfort.")
	public static NpcStringId YOU_HAVE_GIVEN_ME_COMFORT;
	
	@ClientString(id = 75054, message = "Thank you. I really loved these flowers when I was alive.")
	public static NpcStringId THANK_YOU_I_REALLY_LOVED_THESE_FLOWERS_WHEN_I_WAS_ALIVE;
	
	@ClientString(id = 75055, message = "I think I feel better. I was comforted.")
	public static NpcStringId I_THINK_I_FEEL_BETTER_I_WAS_COMFORTED;
	
	@ClientString(id = 75056, message = "I think I feel better. I have no longer lingering feelings.")
	public static NpcStringId I_THINK_I_FEEL_BETTER_I_HAVE_NO_LONGER_LINGERING_FEELINGS;
	
	@ClientString(id = 75057, message = "I have found the courage to go to the place where I should go. Thank you.")
	public static NpcStringId I_HAVE_FOUND_THE_COURAGE_TO_GO_TO_THE_PLACE_WHERE_I_SHOULD_GO_THANK_YOU;
	
	@ClientString(id = 75058, message = "My suffering is slowly washing away...")
	public static NpcStringId MY_SUFFERING_IS_SLOWLY_WASHING_AWAY;
	
	@ClientString(id = 75059, message = "My suffering is washing away... Thank you.")
	public static NpcStringId MY_SUFFERING_IS_WASHING_AWAY_THANK_YOU;
	
	@ClientString(id = 75060, message = "Your comforting feels genuine.")
	public static NpcStringId YOUR_COMFORTING_FEELS_GENUINE;
	
	@ClientString(id = 75101, message = "Wraith' Liberation (Lv. 95-99)")
	public static NpcStringId WRAITH_LIBERATION_LV_95_99;
	
	@ClientString(id = 75102, message = "Wraith' Liberation (Lv. 95-99) (In Progress)")
	public static NpcStringId WRAITH_LIBERATION_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 75103, message = "Wraith' Liberation (Lv. 95-99) (Done)")
	public static NpcStringId WRAITH_LIBERATION_LV_95_99_DONE;
	
	@ClientString(id = 75104, message = "Wraith' Liberation (Lv. 95-99)")
	public static NpcStringId WRAITH_LIBERATION_LV_95_992;
	
	@ClientString(id = 75151, message = "You aren't planning on going by yourself, are you?")
	public static NpcStringId YOU_AREN_T_PLANNING_ON_GOING_BY_YOURSELF_ARE_YOU;
	
	@ClientString(id = 75152, message = "The mission is not yet complete.")
	public static NpcStringId THE_MISSION_IS_NOT_YET_COMPLETE;
	
	@ClientString(id = 75153, message = "Well done! Please report to Roderik about the completed mission.")
	public static NpcStringId WELL_DONE_PLEASE_REPORT_TO_RODERIK_ABOUT_THE_COMPLETED_MISSION;
	
	@ClientString(id = 75201, message = "Uncover the Secret (Lv. 93-99)")
	public static NpcStringId UNCOVER_THE_SECRET_LV_93_99;
	
	@ClientString(id = 75202, message = "Uncover the Secret (Lv. 93-99) (In Progress)")
	public static NpcStringId UNCOVER_THE_SECRET_LV_93_99_IN_PROGRESS;
	
	@ClientString(id = 75203, message = "Uncover the Secret (Lv. 93-99) (Complete)")
	public static NpcStringId UNCOVER_THE_SECRET_LV_93_99_COMPLETE;
	
	@ClientString(id = 75204, message = "Uncover the Secret (Lv. 93-99)")
	public static NpcStringId UNCOVER_THE_SECRET_LV_93_992;
	
	@ClientString(id = 75301, message = "Reacting to a Crisis (Lv. 93-99)")
	public static NpcStringId REACTING_TO_A_CRISIS_LV_93_99;
	
	@ClientString(id = 75302, message = "Reacting to a Crisis (Lv. 93-99) (In Progress)")
	public static NpcStringId REACTING_TO_A_CRISIS_LV_93_99_IN_PROGRESS;
	
	@ClientString(id = 75303, message = "Reacting to a Crisis (Lv. 93-99) (Complete)")
	public static NpcStringId REACTING_TO_A_CRISIS_LV_93_99_COMPLETE;
	
	@ClientString(id = 75304, message = "Reacting to a Crisis (Lv. 93-99)")
	public static NpcStringId REACTING_TO_A_CRISIS_LV_93_992;
	
	@ClientString(id = 75401, message = "Resistance Support (Lv. 97-99)")
	public static NpcStringId RESISTANCE_SUPPORT_LV_97_99;
	
	@ClientString(id = 75402, message = "Resistance Support (Lv. 97-99) (In Progress)")
	public static NpcStringId RESISTANCE_SUPPORT_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 75403, message = "Resistance Support (Lv. 97-99) (Done)")
	public static NpcStringId RESISTANCE_SUPPORT_LV_97_99_DONE;
	
	@ClientString(id = 75404, message = "Resistance Support (Lv. 97-99)")
	public static NpcStringId RESISTANCE_SUPPORT_LV_97_992;
	
	@ClientString(id = 75450, message = "You can find the Kundas on the 2nd floor of the Seed of Hellfire.")
	public static NpcStringId YOU_CAN_FIND_THE_KUNDAS_ON_THE_2ND_FLOOR_OF_THE_SEED_OF_HELLFIRE;
	
	@ClientString(id = 75451, message = "Well done. I hope you will do well tomorrow as well.")
	public static NpcStringId WELL_DONE_I_HOPE_YOU_WILL_DO_WELL_TOMORROW_AS_WELL;
	
	@ClientString(id = 75452, message = "Receive the mission from Sizrak again tomorrow.")
	public static NpcStringId RECEIVE_THE_MISSION_FROM_SIZRAK_AGAIN_TOMORROW;
	
	@ClientString(id = 75501, message = "The Need for Petra (Lv. 97-99)")
	public static NpcStringId THE_NEED_FOR_PETRA_LV_97_99;
	
	@ClientString(id = 75502, message = "The Need for Petra (Lv. 97-99) (In Progress)")
	public static NpcStringId THE_NEED_FOR_PETRA_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 75503, message = "The Need for Petra (Lv. 97-99) (Done)")
	public static NpcStringId THE_NEED_FOR_PETRA_LV_97_99_DONE;
	
	@ClientString(id = 75504, message = "The Need for Petra (Lv. 97-99)")
	public static NpcStringId THE_NEED_FOR_PETRA_LV_97_992;
	
	@ClientString(id = 75550, message = "You can obtain Petras by defeating Zofans.")
	public static NpcStringId YOU_CAN_OBTAIN_PETRAS_BY_DEFEATING_ZOFANS;
	
	@ClientString(id = 75551, message = "Well done. Then, I will see you tomorrow. Huhuhu.")
	public static NpcStringId WELL_DONE_THEN_I_WILL_SEE_YOU_TOMORROW_HUHUHU;
	
	@ClientString(id = 75552, message = "Find Lord Aku tomorrow and receive another mission.")
	public static NpcStringId FIND_LORD_AKU_TOMORROW_AND_RECEIVE_ANOTHER_MISSION;
	
	@ClientString(id = 75601, message = "Top Quality Petra (Lv. 97-99)")
	public static NpcStringId TOP_QUALITY_PETRA_LV_97_99;
	
	@ClientString(id = 75602, message = "Top Quality Petra (Lv. 97-99) (In Progress)")
	public static NpcStringId TOP_QUALITY_PETRA_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 75603, message = "Top Quality Petra (Lv. 97-99) (Done)")
	public static NpcStringId TOP_QUALITY_PETRA_LV_97_99_DONE;
	
	@ClientString(id = 75604, message = "Top Quality Petra (Lv. 97-99)")
	public static NpcStringId TOP_QUALITY_PETRA_LV_97_992;
	
	@ClientString(id = 75650, message = "If you find top quality Petra, bring it to me again. OK?")
	public static NpcStringId IF_YOU_FIND_TOP_QUALITY_PETRA_BRING_IT_TO_ME_AGAIN_OK;
	
	@ClientString(id = 75701, message = "Triol's Movement (Lv. 97-99)")
	public static NpcStringId TRIOL_S_MOVEMENT_LV_97_99;
	
	@ClientString(id = 75702, message = "Triol's Movement (Lv. 97-99) (In Progress)")
	public static NpcStringId TRIOL_S_MOVEMENT_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 75703, message = "Triol's Movement (Lv. 97-99) (Complete)")
	public static NpcStringId TRIOL_S_MOVEMENT_LV_97_99_COMPLETE;
	
	@ClientString(id = 75704, message = "Triol's Movement (Lv. 97-99)")
	public static NpcStringId TRIOL_S_MOVEMENT_LV_97_992;
	
	@ClientString(id = 75801, message = "The Fallen King's Men (Lv. 97-99)")
	public static NpcStringId THE_FALLEN_KING_S_MEN_LV_97_99;
	
	@ClientString(id = 75802, message = "The Fallen King's Men (Lv. 97-99) (In Progress)")
	public static NpcStringId THE_FALLEN_KING_S_MEN_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 75803, message = "The Fallen King's Men (Lv. 97-99) (Complete)")
	public static NpcStringId THE_FALLEN_KING_S_MEN_LV_97_99_COMPLETE;
	
	@ClientString(id = 75804, message = "The Fallen King's Men (Lv. 97-99)")
	public static NpcStringId THE_FALLEN_KING_S_MEN_LV_97_992;
	
	@ClientString(id = 75901, message = "The Dwarven Nightmare Continues (Lv. 98-99)")
	public static NpcStringId THE_DWARVEN_NIGHTMARE_CONTINUES_LV_98_99;
	
	@ClientString(id = 75902, message = "The Dwarven Nightmare Continues (Lv. 98-99) (In Progress)")
	public static NpcStringId THE_DWARVEN_NIGHTMARE_CONTINUES_LV_98_99_IN_PROGRESS;
	
	@ClientString(id = 75903, message = "The Dwarven Nightmare Continues (Lv. 98-99) (Complete)")
	public static NpcStringId THE_DWARVEN_NIGHTMARE_CONTINUES_LV_98_99_COMPLETE;
	
	@ClientString(id = 75904, message = "The Dwarven Nightmare Continues (Lv. 98-99)")
	public static NpcStringId THE_DWARVEN_NIGHTMARE_CONTINUES_LV_98_992;
	
	@ClientString(id = 75950, message = "$s1 has received $s2 as a reward from Head Priest of the Earth Daichir.")
	public static NpcStringId S1_HAS_RECEIVED_S2_AS_A_REWARD_FROM_HEAD_PRIEST_OF_THE_EARTH_DAICHIR;
	
	@ClientString(id = 75951, message = "Amaranthine Shaper Fragment")
	public static NpcStringId AMARANTHINE_SHAPER_FRAGMENT;
	
	@ClientString(id = 75952, message = "Amaranthine Cutter Fragment")
	public static NpcStringId AMARANTHINE_CUTTER_FRAGMENT;
	
	@ClientString(id = 75953, message = "Amaranthine Slasher Fragment")
	public static NpcStringId AMARANTHINE_SLASHER_FRAGMENT;
	
	@ClientString(id = 75954, message = "Amaranthine Avenger Fragment")
	public static NpcStringId AMARANTHINE_AVENGER_FRAGMENT;
	
	@ClientString(id = 75955, message = "Amaranthine Fighter Fragment")
	public static NpcStringId AMARANTHINE_FIGHTER_FRAGMENT;
	
	@ClientString(id = 75956, message = "Amaranthine Stormer Fragment")
	public static NpcStringId AMARANTHINE_STORMER_FRAGMENT;
	
	@ClientString(id = 75957, message = "Amaranthine Thrower Fragment")
	public static NpcStringId AMARANTHINE_THROWER_FRAGMENT;
	
	@ClientString(id = 75958, message = "Amaranthine Shooter Fragment")
	public static NpcStringId AMARANTHINE_SHOOTER_FRAGMENT;
	
	@ClientString(id = 75959, message = "Amaranthine Buster Fragment")
	public static NpcStringId AMARANTHINE_BUSTER_FRAGMENT;
	
	@ClientString(id = 75960, message = "Amaranthine Caster Fragment")
	public static NpcStringId AMARANTHINE_CASTER_FRAGMENT;
	
	@ClientString(id = 75961, message = "Amaranthine Retributer Fragment")
	public static NpcStringId AMARANTHINE_RETRIBUTER_FRAGMENT;
	
	@ClientString(id = 75962, message = "Earth Wyrm Heart Necklace")
	public static NpcStringId EARTH_WYRM_HEART_NECKLACE2;
	
	@ClientString(id = 75963, message = "Yellow Soul Crystal (R99-grade)")
	public static NpcStringId YELLOW_SOUL_CRYSTAL_R99_GRADE2;
	
	@ClientString(id = 75964, message = "Teal Soul Crystal (R99-grade)")
	public static NpcStringId TEAL_SOUL_CRYSTAL_R99_GRADE2;
	
	@ClientString(id = 75965, message = "Purple Soul Crystal (R99-grade)")
	public static NpcStringId PURPLE_SOUL_CRYSTAL_R99_GRADE2;
	
	@ClientString(id = 75966, message = "Fire Crystal")
	public static NpcStringId FIRE_CRYSTAL3;
	
	@ClientString(id = 75967, message = "Water Crystal")
	public static NpcStringId WATER_CRYSTAL3;
	
	@ClientString(id = 75968, message = "Earth Crystal")
	public static NpcStringId EARTH_CRYSTAL3;
	
	@ClientString(id = 75969, message = "Wind Crystal")
	public static NpcStringId WIND_CRYSTAL3;
	
	@ClientString(id = 75970, message = "Dark Crystal")
	public static NpcStringId DARK_CRYSTAL3;
	
	@ClientString(id = 75971, message = "Holy Crystal")
	public static NpcStringId HOLY_CRYSTAL3;
	
	@ClientString(id = 75972, message = "Antharas' Symbol Dye")
	public static NpcStringId ANTHARAS_SYMBOL_DYE2;
	
	@ClientString(id = 76101, message = "Assisting the Golden Ram Army (Lv. 65-70)")
	public static NpcStringId ASSISTING_THE_GOLDEN_RAM_ARMY_LV_65_70;
	
	@ClientString(id = 76102, message = "Assisting the Golden Ram Army (Lv. 65-70) (In Progress)")
	public static NpcStringId ASSISTING_THE_GOLDEN_RAM_ARMY_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 76103, message = "Assisting the Golden Ram Army (Lv. 65-70) (Complete)")
	public static NpcStringId ASSISTING_THE_GOLDEN_RAM_ARMY_LV_65_70_COMPLETE;
	
	@ClientString(id = 76104, message = "Assisting the Golden Ram Army (Lv. 65-70)")
	public static NpcStringId ASSISTING_THE_GOLDEN_RAM_ARMY_LV_65_702;
	
	@ClientString(id = 76201, message = "An Ominous Request (Lv. 65-70)")
	public static NpcStringId AN_OMINOUS_REQUEST_LV_65_70;
	
	@ClientString(id = 76202, message = "An Ominous Request (Lv. 65-70) (In Progress)")
	public static NpcStringId AN_OMINOUS_REQUEST_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 76203, message = "An Ominous Request (Lv. 65-70) (Complete)")
	public static NpcStringId AN_OMINOUS_REQUEST_LV_65_70_COMPLETE;
	
	@ClientString(id = 76204, message = "An Ominous Request (Lv. 65-70)")
	public static NpcStringId AN_OMINOUS_REQUEST_LV_65_702;
	
	@ClientString(id = 76301, message = "A Daunting Task (Lv. 70-75)")
	public static NpcStringId A_DAUNTING_TASK_LV_70_75;
	
	@ClientString(id = 76302, message = "A Daunting Task (Lv. 70-75) (In Progress)")
	public static NpcStringId A_DAUNTING_TASK_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 76303, message = "A Daunting Task (Lv. 70-75) (Complete)")
	public static NpcStringId A_DAUNTING_TASK_LV_70_75_COMPLETE;
	
	@ClientString(id = 76304, message = "A Daunting Task (Lv. 70-75)")
	public static NpcStringId A_DAUNTING_TASK_LV_70_752;
	
	@ClientString(id = 76401, message = "Weakening the Varka Silenos Forces (Lv. 76-80)")
	public static NpcStringId WEAKENING_THE_VARKA_SILENOS_FORCES_LV_76_80;
	
	@ClientString(id = 76402, message = "Weakening the Varka Silenos Forces (Lv. 76-80) (In Progress)")
	public static NpcStringId WEAKENING_THE_VARKA_SILENOS_FORCES_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 76403, message = "Weakening the Varka Silenos Forces (Lv. 76-80) (Complete)")
	public static NpcStringId WEAKENING_THE_VARKA_SILENOS_FORCES_LV_76_80_COMPLETE;
	
	@ClientString(id = 76404, message = "Weakening the Varka Silenos Forces (Lv. 76-80)")
	public static NpcStringId WEAKENING_THE_VARKA_SILENOS_FORCES_LV_76_802;
	
	@ClientString(id = 76501, message = "Weakening the Ketra Orc Forces (Lv. 76-80)")
	public static NpcStringId WEAKENING_THE_KETRA_ORC_FORCES_LV_76_80;
	
	@ClientString(id = 76502, message = "Weakening the Ketra Orc Forces (Lv. 76-80) (In Progress)")
	public static NpcStringId WEAKENING_THE_KETRA_ORC_FORCES_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 76503, message = "Weakening the Ketra Orc Forces (Lv. 76-80) (Complete)")
	public static NpcStringId WEAKENING_THE_KETRA_ORC_FORCES_LV_76_80_COMPLETE;
	
	@ClientString(id = 76504, message = "Weakening the Ketra Orc Forces (Lv. 76-80)")
	public static NpcStringId WEAKENING_THE_KETRA_ORC_FORCES_LV_76_802;
	
	@ClientString(id = 76701, message = "Banishing Evil (Lv. 81-84)")
	public static NpcStringId BANISHING_EVIL_LV_81_84;
	
	@ClientString(id = 76702, message = "Banishing Evil (Lv. 81-84) (In Progress)")
	public static NpcStringId BANISHING_EVIL_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 76703, message = "Banishing Evil (Lv. 81-84) (Complete)")
	public static NpcStringId BANISHING_EVIL_LV_81_84_COMPLETE;
	
	@ClientString(id = 76704, message = "Banishing Evil (Lv. 81-84)")
	public static NpcStringId BANISHING_EVIL_LV_81_842;
	
	@ClientString(id = 76801, message = "Terror of Town (Lv. 81-84)")
	public static NpcStringId TERROR_OF_TOWN_LV_81_84;
	
	@ClientString(id = 76802, message = "Terror of Town (Lv. 81-84) (In Progress)")
	public static NpcStringId TERROR_OF_TOWN_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 76803, message = "Terror of Town (Lv. 81-84) (Complete)")
	public static NpcStringId TERROR_OF_TOWN_LV_81_84_COMPLETE;
	
	@ClientString(id = 76804, message = "Terror of Town (Lv. 81-84)")
	public static NpcStringId TERROR_OF_TOWN_LV_81_842;
	
	@ClientString(id = 76901, message = "The Truth Concealed in Beauty (Lv. 81-84)")
	public static NpcStringId THE_TRUTH_CONCEALED_IN_BEAUTY_LV_81_84;
	
	@ClientString(id = 76902, message = "The Truth Concealed in Beauty (Lv. 81-84) (In Progress)")
	public static NpcStringId THE_TRUTH_CONCEALED_IN_BEAUTY_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 76903, message = "The Truth Concealed in Beauty (Lv. 81-84) (Complete)")
	public static NpcStringId THE_TRUTH_CONCEALED_IN_BEAUTY_LV_81_84_COMPLETE;
	
	@ClientString(id = 76904, message = "The Truth Concealed in Beauty (Lv. 81-84)")
	public static NpcStringId THE_TRUTH_CONCEALED_IN_BEAUTY_LV_81_842;
	
	@ClientString(id = 77001, message = "Maintaining the Field Business (Lv. 81-84)")
	public static NpcStringId MAINTAINING_THE_FIELD_BUSINESS_LV_81_84;
	
	@ClientString(id = 77002, message = "Maintaining the Field Business (Lv. 81-84) (In Progress)")
	public static NpcStringId MAINTAINING_THE_FIELD_BUSINESS_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 77003, message = "Maintaining the Field Business (Lv. 81-84) (Complete)")
	public static NpcStringId MAINTAINING_THE_FIELD_BUSINESS_LV_81_84_COMPLETE;
	
	@ClientString(id = 77004, message = "Maintaining the Field Business (Lv. 81-84)")
	public static NpcStringId MAINTAINING_THE_FIELD_BUSINESS_LV_81_842;
	
	@ClientString(id = 77101, message = "Partaking in the Purification Campaign (Lv. 81-84)")
	public static NpcStringId PARTAKING_IN_THE_PURIFICATION_CAMPAIGN_LV_81_84;
	
	@ClientString(id = 77102, message = "Partaking in the Purification Campaign (Lv. 81-84) (In Progress)")
	public static NpcStringId PARTAKING_IN_THE_PURIFICATION_CAMPAIGN_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 77103, message = "Partaking in the Purification Campaign (Lv. 81-84) (Complete)")
	public static NpcStringId PARTAKING_IN_THE_PURIFICATION_CAMPAIGN_LV_81_84_COMPLETE;
	
	@ClientString(id = 77104, message = "Partaking in the Purification Campaign (Lv. 81-84)")
	public static NpcStringId PARTAKING_IN_THE_PURIFICATION_CAMPAIGN_LV_81_842;
	
	@ClientString(id = 77201, message = "Purifying Souls (Lv. 99-99)")
	public static NpcStringId PURIFYING_SOULS_LV_99_99;
	
	@ClientString(id = 77202, message = "Purifying Souls (Lv. 99-99) (In Progress)")
	public static NpcStringId PURIFYING_SOULS_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 77203, message = "Purifying Souls (Lv. 99-99) (Complete)")
	public static NpcStringId PURIFYING_SOULS_LV_99_99_COMPLETE;
	
	@ClientString(id = 77204, message = "Purifying Souls (Lv. 99-99)")
	public static NpcStringId PURIFYING_SOULS_LV_99_992;
	
	@ClientString(id = 77501, message = "Retrieving the Chaos Fragment (Lv. 99-99)")
	public static NpcStringId RETRIEVING_THE_CHAOS_FRAGMENT_LV_99_99;
	
	@ClientString(id = 77502, message = "Retrieving the Chaos Fragment (Lv. 99-99) (In Progress)")
	public static NpcStringId RETRIEVING_THE_CHAOS_FRAGMENT_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 77503, message = "Retrieving the Chaos Fragment (Lv. 99-99) (Complete)")
	public static NpcStringId RETRIEVING_THE_CHAOS_FRAGMENT_LV_99_99_COMPLETE;
	
	@ClientString(id = 77504, message = "Retrieving the Chaos Fragment (Lv. 99-99)")
	public static NpcStringId RETRIEVING_THE_CHAOS_FRAGMENT_LV_99_992;
	
	@ClientString(id = 77521, message = "You can obtain Chaos Fragments throughout all of Hellbound.")
	public static NpcStringId YOU_CAN_OBTAIN_CHAOS_FRAGMENTS_THROUGHOUT_ALL_OF_HELLBOUND;
	
	@ClientString(id = 77522, message = "Well done! Your Chaos Fragments will be used to attract Beleth.")
	public static NpcStringId WELL_DONE_YOUR_CHAOS_FRAGMENTS_WILL_BE_USED_TO_ATTRACT_BELETH;
	
	@ClientString(id = 77601, message = "Slay Dark Lord Ekimus (Lv. 95-99)")
	public static NpcStringId SLAY_DARK_LORD_EKIMUS_LV_95_99;
	
	@ClientString(id = 77602, message = "Slay Dark Lord Ekimus (Lv. 95-99) (In Progress)")
	public static NpcStringId SLAY_DARK_LORD_EKIMUS_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 77603, message = "Slay Dark Lord Ekimus (Lv. 95-99) (Complete)")
	public static NpcStringId SLAY_DARK_LORD_EKIMUS_LV_95_99_COMPLETE;
	
	@ClientString(id = 77604, message = "Slay Dark Lord Ekimus (Lv. 95-99)")
	public static NpcStringId SLAY_DARK_LORD_EKIMUS_LV_95_992;
	
	@ClientString(id = 77611, message = "Infiltrate the Hall of Suffering")
	public static NpcStringId INFILTRATE_THE_HALL_OF_SUFFERING;
	
	@ClientString(id = 77612, message = "Infiltrate the Heart of Infinity")
	public static NpcStringId INFILTRATE_THE_HEART_OF_INFINITY;
	
	@ClientString(id = 77701, message = "Slay Dark Lord Tiat (Lv. 93-97)")
	public static NpcStringId SLAY_DARK_LORD_TIAT_LV_93_97;
	
	@ClientString(id = 77702, message = "Slay Dark Lord Tiat (Lv. 93-97) (In Progress)")
	public static NpcStringId SLAY_DARK_LORD_TIAT_LV_93_97_IN_PROGRESS;
	
	@ClientString(id = 77703, message = "Slay Dark Lord Tiat (Lv. 93-97) (Complete)")
	public static NpcStringId SLAY_DARK_LORD_TIAT_LV_93_97_COMPLETE;
	
	@ClientString(id = 77704, message = "Slay Dark Lord Tiat (Lv. 93-97)")
	public static NpcStringId SLAY_DARK_LORD_TIAT_LV_93_972;
	
	@ClientString(id = 77711, message = "Infiltrate the Seed of Destruction")
	public static NpcStringId INFILTRATE_THE_SEED_OF_DESTRUCTION;
	
	@ClientString(id = 77801, message = "Operation Roaring Flame (Lv. 99-99)")
	public static NpcStringId OPERATION_ROARING_FLAME_LV_99_99;
	
	@ClientString(id = 77802, message = "Operation Roaring Flame (Lv. 99-99) (In Progress)")
	public static NpcStringId OPERATION_ROARING_FLAME_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 77803, message = "Operation Roaring Flame (Lv. 99-99) (Complete)")
	public static NpcStringId OPERATION_ROARING_FLAME_LV_99_99_COMPLETE;
	
	@ClientString(id = 77804, message = "Operation Roaring Flame (Lv. 99-99)")
	public static NpcStringId OPERATION_ROARING_FLAME_LV_99_992;
	
	@ClientString(id = 77901, message = "Utilize the Darkness - Seed of Destruction (Lv. 93-97)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_DESTRUCTION_LV_93_97;
	
	@ClientString(id = 77902, message = "Utilize the Darkness - Seed of Destruction (Lv. 93-97) (In Progress)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_DESTRUCTION_LV_93_97_IN_PROGRESS;
	
	@ClientString(id = 77903, message = "Utilize the Darkness - Seed of Destruction (Lv. 93-97) (Complete)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_DESTRUCTION_LV_93_97_COMPLETE;
	
	@ClientString(id = 77904, message = "Utilize the Darkness - Seed of Destruction (Lv. 93-97)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_DESTRUCTION_LV_93_972;
	
	@ClientString(id = 78001, message = "Utilize the Darkness - Seed of Infinity (Lv. 95-99)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_INFINITY_LV_95_99;
	
	@ClientString(id = 78002, message = "Utilize the Darkness - Seed of Infinity (Lv. 95-99) (In Progress)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_INFINITY_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 78003, message = "Utilize the Darkness - Seed of Infinity (Lv. 95-99) (Complete)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_INFINITY_LV_95_99_COMPLETE;
	
	@ClientString(id = 78004, message = "Utilize the Darkness - Seed of Infinity (Lv. 95-99)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_INFINITY_LV_95_992;
	
	@ClientString(id = 78101, message = "Utilize the Darkness - Seed of Annihilation (Lv. 85-99)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_ANNIHILATION_LV_85_99;
	
	@ClientString(id = 78102, message = "Utilize the Darkness - Seed of Annihilation (Lv. 85-99) (In Progress)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_ANNIHILATION_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 78103, message = "Utilize the Darkness - Seed of Annihilation (Lv. 85-99) (Complete)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_ANNIHILATION_LV_85_99_COMPLETE;
	
	@ClientString(id = 78104, message = "Utilize the Darkness - Seed of Annihilation (Lv. 85-99)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_ANNIHILATION_LV_85_992;
	
	@ClientString(id = 78201, message = "Utilize the Darkness - Seed of Hellfire (Lv. 97-99)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_HELLFIRE_LV_97_99;
	
	@ClientString(id = 78202, message = "Utilize the Darkness - Seed of Hellfire (Lv. 97-99) (In Progress)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_HELLFIRE_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 78203, message = "Utilize the Darkness - Seed of Hellfire (Lv. 97-99) (Complete)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_HELLFIRE_LV_97_99_COMPLETE;
	
	@ClientString(id = 78204, message = "Utilize the Darkness - Seed of Hellfire (Lv. 97-99)")
	public static NpcStringId UTILIZE_THE_DARKNESS_SEED_OF_HELLFIRE_LV_97_992;
	
	@ClientString(id = 78401, message = "The Quiet Killer (Lv. 65-70)")
	public static NpcStringId THE_QUIET_KILLER_LV_65_70;
	
	@ClientString(id = 78402, message = "The Quiet Killer (Lv. 65-70) (In Progress)")
	public static NpcStringId THE_QUIET_KILLER_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 78403, message = "The Quiet Killer (Lv. 65-70) (Done)")
	public static NpcStringId THE_QUIET_KILLER_LV_65_70_DONE;
	
	@ClientString(id = 78404, message = "The Quiet Killer (Lv. 65-70)")
	public static NpcStringId THE_QUIET_KILLER_LV_65_702;
	
	@ClientString(id = 78501, message = "A Suspicious Ingredient (Lv. 65-70)")
	public static NpcStringId A_SUSPICIOUS_INGREDIENT_LV_65_70;
	
	@ClientString(id = 78502, message = "A Suspicious Ingredient (Lv. 65-70) (In Progress)")
	public static NpcStringId A_SUSPICIOUS_INGREDIENT_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 78503, message = "A Suspicious Ingredient (Lv. 65-70) (Done)")
	public static NpcStringId A_SUSPICIOUS_INGREDIENT_LV_65_70_DONE;
	
	@ClientString(id = 78504, message = "A Suspicious Ingredient (Lv. 65-70)")
	public static NpcStringId A_SUSPICIOUS_INGREDIENT_LV_65_702;
	
	@ClientString(id = 78601, message = "Awaiting the Voice of the Gods (Lv. 70-75)")
	public static NpcStringId AWAITING_THE_VOICE_OF_THE_GODS_LV_70_75;
	
	@ClientString(id = 78602, message = "Awaiting the Voice of the Gods (Lv. 70-75) (In Progress)")
	public static NpcStringId AWAITING_THE_VOICE_OF_THE_GODS_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 78603, message = "Awaiting the Voice of the Gods (Lv. 70-75) (Done)")
	public static NpcStringId AWAITING_THE_VOICE_OF_THE_GODS_LV_70_75_DONE;
	
	@ClientString(id = 78604, message = "Awaiting the Voice of the Gods (Lv. 70-75)")
	public static NpcStringId AWAITING_THE_VOICE_OF_THE_GODS_LV_70_752;
	
	@ClientString(id = 78701, message = "The Role of a Watcher (Lv. 76-85)")
	public static NpcStringId THE_ROLE_OF_A_WATCHER_LV_76_85;
	
	@ClientString(id = 78702, message = "The Role of a Watcher (Lv. 76-85) (In Progress)")
	public static NpcStringId THE_ROLE_OF_A_WATCHER_LV_76_85_IN_PROGRESS;
	
	@ClientString(id = 78703, message = "The Role of a Watcher (Lv. 76-85) (Done)")
	public static NpcStringId THE_ROLE_OF_A_WATCHER_LV_76_85_DONE;
	
	@ClientString(id = 78704, message = "The Role of a Watcher (Lv. 76-85)")
	public static NpcStringId THE_ROLE_OF_A_WATCHER_LV_76_852;
	
	@ClientString(id = 78901, message = "Waiting for Pa'agrio (Lv. 97-99)")
	public static NpcStringId WAITING_FOR_PA_AGRIO_LV_97_99;
	
	@ClientString(id = 78902, message = "Waiting for Pa'agrio (Lv. 97-99) (In Progress)")
	public static NpcStringId WAITING_FOR_PA_AGRIO_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 78903, message = "Waiting for Pa'agrio (Lv. 97-99) (Done)")
	public static NpcStringId WAITING_FOR_PA_AGRIO_LV_97_99_DONE;
	
	@ClientString(id = 78904, message = "Waiting for Pa'agrio (Lv. 97-99)")
	public static NpcStringId WAITING_FOR_PA_AGRIO_LV_97_992;
	
	@ClientString(id = 85001, message = "The Times Call for a New Hero - Aden")
	public static NpcStringId THE_TIMES_CALL_FOR_A_NEW_HERO_ADEN;
	
	@ClientString(id = 85002, message = "The Times Call for a New Hero - Aden (In Progress)")
	public static NpcStringId THE_TIMES_CALL_FOR_A_NEW_HERO_ADEN_IN_PROGRESS;
	
	@ClientString(id = 85003, message = "The Times Call for a New Hero - Aden (Done)")
	public static NpcStringId THE_TIMES_CALL_FOR_A_NEW_HERO_ADEN_DONE;
	
	@ClientString(id = 85004, message = "The Times Call for a New Hero - Aden")
	public static NpcStringId THE_TIMES_CALL_FOR_A_NEW_HERO_ADEN2;
	
	@ClientString(id = 85010, message = "The Outer Gates of the Castle of Aden have been weakened due to the Corrosive Chemical.")
	public static NpcStringId THE_OUTER_GATES_OF_THE_CASTLE_OF_ADEN_HAVE_BEEN_WEAKENED_DUE_TO_THE_CORROSIVE_CHEMICAL;
	
	@ClientString(id = 85011, message = "No changes to the Outer Castle Gates. Try using the Corrosive Chemical again.")
	public static NpcStringId NO_CHANGES_TO_THE_OUTER_CASTLE_GATES_TRY_USING_THE_CORROSIVE_CHEMICAL_AGAIN;
	
	@ClientString(id = 85012, message = "No more need for use.")
	public static NpcStringId NO_MORE_NEED_FOR_USE;
	
	@ClientString(id = 85013, message = "No changes felt.")
	public static NpcStringId NO_CHANGES_FELT;
	
	@ClientString(id = 85014, message = "1 Ballista has been repaired.")
	public static NpcStringId BALLISTA_HAS_BEEN_REPAIRED;
	
	@ClientString(id = 85015, message = "2 Ballistas have been repaired.")
	public static NpcStringId BALLISTAS_HAVE_BEEN_REPAIRED;
	
	@ClientString(id = 85016, message = "3 Ballistas have been repaired.")
	public static NpcStringId BALLISTAS_HAVE_BEEN_REPAIRED2;
	
	@ClientString(id = 85017, message = "4 Ballistas have been repaired.")
	public static NpcStringId BALLISTAS_HAVE_BEEN_REPAIRED3;
	
	@ClientString(id = 85018, message = "All Ballistas have been repaired. Please check your next mission.")
	public static NpcStringId ALL_BALLISTAS_HAVE_BEEN_REPAIRED_PLEASE_CHECK_YOUR_NEXT_MISSION;
	
	@ClientString(id = 85019, message = "The Engravement has begun on Aden's Holy Artifact.")
	public static NpcStringId THE_ENGRAVEMENT_HAS_BEGUN_ON_ADEN_S_HOLY_ARTIFACT;
	
	@ClientString(id = 85020, message = "The Engravement has been rejected by the power of the holy artifact. Please try again.")
	public static NpcStringId THE_ENGRAVEMENT_HAS_BEEN_REJECTED_BY_THE_POWER_OF_THE_HOLY_ARTIFACT_PLEASE_TRY_AGAIN;
	
	@ClientString(id = 85021, message = "This is bad... the castle siege equipment is broken, but the war has already started...")
	public static NpcStringId THIS_IS_BAD_THE_CASTLE_SIEGE_EQUIPMENT_IS_BROKEN_BUT_THE_WAR_HAS_ALREADY_STARTED;
	
	@ClientString(id = 85022, message = "We can't go on fighting with equipment like this! We need assistance.")
	public static NpcStringId WE_CAN_T_GO_ON_FIGHTING_WITH_EQUIPMENT_LIKE_THIS_WE_NEED_ASSISTANCE;
	
	@ClientString(id = 85023, message = "It looks like we could repair this ballista with the repair tool you have there!")
	public static NpcStringId IT_LOOKS_LIKE_WE_COULD_REPAIR_THIS_BALLISTA_WITH_THE_REPAIR_TOOL_YOU_HAVE_THERE;
	
	@ClientString(id = 85024, message = "I think the ballista can move now! Let's start a test run!")
	public static NpcStringId I_THINK_THE_BALLISTA_CAN_MOVE_NOW_LET_S_START_A_TEST_RUN;
	
	@ClientString(id = 85025, message = "This is moving properly, right? Are you sure?")
	public static NpcStringId THIS_IS_MOVING_PROPERLY_RIGHT_ARE_YOU_SURE;
	
	@ClientString(id = 85026, message = "Ballista Repair has failed, ending in destruction.")
	public static NpcStringId BALLISTA_REPAIR_HAS_FAILED_ENDING_IN_DESTRUCTION;
	
	@ClientString(id = 85101, message = "The Times Call for a New Hero - Rune")
	public static NpcStringId THE_TIMES_CALL_FOR_A_NEW_HERO_RUNE;
	
	@ClientString(id = 85102, message = "The Times Call for a New Hero - Rune (In Progress)")
	public static NpcStringId THE_TIMES_CALL_FOR_A_NEW_HERO_RUNE_IN_PROGRESS;
	
	@ClientString(id = 85103, message = "The Times Call for a New Hero - Rune (Done)")
	public static NpcStringId THE_TIMES_CALL_FOR_A_NEW_HERO_RUNE_DONE;
	
	@ClientString(id = 85104, message = "The Times Call for a New Hero - Rune")
	public static NpcStringId THE_TIMES_CALL_FOR_A_NEW_HERO_RUNE2;
	
	@ClientString(id = 85110, message = "The Outer Castle Gates of the Rune Castle have been weakened due to the Corrosive Chemical.")
	public static NpcStringId THE_OUTER_CASTLE_GATES_OF_THE_RUNE_CASTLE_HAVE_BEEN_WEAKENED_DUE_TO_THE_CORROSIVE_CHEMICAL;
	
	@ClientString(id = 85111, message = "No changes to the Outer Castle Gates. Try using the Corrosive Chemical again.")
	public static NpcStringId NO_CHANGES_TO_THE_OUTER_CASTLE_GATES_TRY_USING_THE_CORROSIVE_CHEMICAL_AGAIN2;
	
	@ClientString(id = 85119, message = "The Engravement has begun on Rune's Altar of the Gods.")
	public static NpcStringId THE_ENGRAVEMENT_HAS_BEGUN_ON_RUNE_S_ALTAR_OF_THE_GODS;
	
	@ClientString(id = 85201, message = "Guidance of Light - Go after the Leading Monsters")
	public static NpcStringId GUIDANCE_OF_LIGHT_GO_AFTER_THE_LEADING_MONSTERS;
	
	@ClientString(id = 85202, message = "Guidance of Light - Go after the Leading Monsters (In Progress)")
	public static NpcStringId GUIDANCE_OF_LIGHT_GO_AFTER_THE_LEADING_MONSTERS_IN_PROGRESS;
	
	@ClientString(id = 85203, message = "Guidance of Light - Go after the Leading Monsters (Complete)")
	public static NpcStringId GUIDANCE_OF_LIGHT_GO_AFTER_THE_LEADING_MONSTERS_COMPLETE;
	
	@ClientString(id = 85204, message = "Guidance of Light - Go after the Leading Monsters")
	public static NpcStringId GUIDANCE_OF_LIGHT_GO_AFTER_THE_LEADING_MONSTERS2;
	
	@ClientString(id = 85301, message = "Revelation of Light - Eliminate the Elite Monsters")
	public static NpcStringId REVELATION_OF_LIGHT_ELIMINATE_THE_ELITE_MONSTERS;
	
	@ClientString(id = 85302, message = "Revelation of Light - Eliminate the Elite Monsters (In Progress)")
	public static NpcStringId REVELATION_OF_LIGHT_ELIMINATE_THE_ELITE_MONSTERS_IN_PROGRESS;
	
	@ClientString(id = 85303, message = "Revelation of Light - Eliminate the Elite Monsters (Complete)")
	public static NpcStringId REVELATION_OF_LIGHT_ELIMINATE_THE_ELITE_MONSTERS_COMPLETE;
	
	@ClientString(id = 85304, message = "Revelation of Light - Eliminate the Elite Monsters")
	public static NpcStringId REVELATION_OF_LIGHT_ELIMINATE_THE_ELITE_MONSTERS2;
	
	@ClientString(id = 85401, message = "Wrath of Light - Clear out the Elite Monsters' Lair")
	public static NpcStringId WRATH_OF_LIGHT_CLEAR_OUT_THE_ELITE_MONSTERS_LAIR;
	
	@ClientString(id = 85402, message = "Wrath of Light - Clear out the Elite Monsters' Lair (In Progress)")
	public static NpcStringId WRATH_OF_LIGHT_CLEAR_OUT_THE_ELITE_MONSTERS_LAIR_IN_PROGRESS;
	
	@ClientString(id = 85403, message = "Wrath of Light - Clear out the Elite Monsters' Lair (Complete)")
	public static NpcStringId WRATH_OF_LIGHT_CLEAR_OUT_THE_ELITE_MONSTERS_LAIR_COMPLETE;
	
	@ClientString(id = 85404, message = "Wrath of Light - Clear out the Elite Monsters' Lair")
	public static NpcStringId WRATH_OF_LIGHT_CLEAR_OUT_THE_ELITE_MONSTERS_LAIR2;
	
	@ClientString(id = 85501, message = "Judgment of Light - Triumph over the Elite Leaders")
	public static NpcStringId JUDGMENT_OF_LIGHT_TRIUMPH_OVER_THE_ELITE_LEADERS;
	
	@ClientString(id = 85502, message = "Judgment of Light - Triumph over the Elite Leaders (In Progress)")
	public static NpcStringId JUDGMENT_OF_LIGHT_TRIUMPH_OVER_THE_ELITE_LEADERS_IN_PROGRESS;
	
	@ClientString(id = 85503, message = "Judgment of Light - Triumph over the Elite Leaders (Complete)")
	public static NpcStringId JUDGMENT_OF_LIGHT_TRIUMPH_OVER_THE_ELITE_LEADERS_COMPLETE;
	
	@ClientString(id = 85504, message = "Judgment of Light - Triumph over the Elite Leaders")
	public static NpcStringId JUDGMENT_OF_LIGHT_TRIUMPH_OVER_THE_ELITE_LEADERS2;
	
	@ClientString(id = 85601, message = "Judgment of Light - Corner the Leaders")
	public static NpcStringId JUDGMENT_OF_LIGHT_CORNER_THE_LEADERS;
	
	@ClientString(id = 85602, message = "Judgment of Light - Corner the Leaders (In Progress)")
	public static NpcStringId JUDGMENT_OF_LIGHT_CORNER_THE_LEADERS_IN_PROGRESS;
	
	@ClientString(id = 85603, message = "Judgment of Light - Corner the Leaders (Complete)")
	public static NpcStringId JUDGMENT_OF_LIGHT_CORNER_THE_LEADERS_COMPLETE;
	
	@ClientString(id = 85604, message = "Judgment of Light - Corner the Leaders")
	public static NpcStringId JUDGMENT_OF_LIGHT_CORNER_THE_LEADERS2;
	
	@ClientString(id = 85701, message = "A Memory Shoot in Fantasy Isle (Event)")
	public static NpcStringId A_MEMORY_SHOOT_IN_FANTASY_ISLE_EVENT;
	
	@ClientString(id = 85702, message = "A Memory Shoot in Fantasy Isle (Event) (In Progress)")
	public static NpcStringId A_MEMORY_SHOOT_IN_FANTASY_ISLE_EVENT_IN_PROGRESS;
	
	@ClientString(id = 85703, message = "A Memory Shoot in Fantasy Isle (Event) (Complete)")
	public static NpcStringId A_MEMORY_SHOOT_IN_FANTASY_ISLE_EVENT_COMPLETE;
	
	@ClientString(id = 85704, message = "A Memory Shoot in Fantasy Isle (Event)")
	public static NpcStringId A_MEMORY_SHOOT_IN_FANTASY_ISLE_EVENT2;
	
	@ClientString(id = 85705, message = "Fantasy Isle is hosting a promotional festival! Come and see!")
	public static NpcStringId FANTASY_ISLE_IS_HOSTING_A_PROMOTIONAL_FESTIVAL_COME_AND_SEE;
	
	@ClientString(id = 85706, message = "Come to Fantasy Isle, where exciting new attractions await you!")
	public static NpcStringId COME_TO_FANTASY_ISLE_WHERE_EXCITING_NEW_ATTRACTIONS_AWAIT_YOU;
	
	@ClientString(id = 85707, message = "There's even more you can do now with Fantasy Isle Coins! Come take a look!")
	public static NpcStringId THERE_S_EVEN_MORE_YOU_CAN_DO_NOW_WITH_FANTASY_ISLE_COINS_COME_TAKE_A_LOOK;
	
	@ClientString(id = 85708, message = "Fantasy Isle is hosting a fashion show for Andron, the greatest fashion designer in Aden!")
	public static NpcStringId FANTASY_ISLE_IS_HOSTING_A_FASHION_SHOW_FOR_ANDRON_THE_GREATEST_FASHION_DESIGNER_IN_ADEN;
	
	@ClientString(id = 85709, message = "Fantasy Isle now has a monster battle system! Come check it out.")
	public static NpcStringId FANTASY_ISLE_NOW_HAS_A_MONSTER_BATTLE_SYSTEM_COME_CHECK_IT_OUT;
	
	@ClientString(id = 85710, message = "Want the inside scoop on the Fantasy Isle Festival? Find Small Bard at the town center and he'll tell you! He's the festival manager. And my brother too - hey, are you listening?")
	public static NpcStringId WANT_THE_INSIDE_SCOOP_ON_THE_FANTASY_ISLE_FESTIVAL_FIND_SMALL_BARD_AT_THE_TOWN_CENTER_AND_HE_LL_TELL_YOU_HE_S_THE_FESTIVAL_MANAGER_AND_MY_BROTHER_TOO_HEY_ARE_YOU_LISTENING;
	
	@ClientString(id = 85711, message = "Wait up, you clowns! You're gonna lose me in the crowd with all that airheaded blathering!")
	public static NpcStringId WAIT_UP_YOU_CLOWNS_YOU_RE_GONNA_LOSE_ME_IN_THE_CROWD_WITH_ALL_THAT_AIRHEADED_BLATHERING;
	
	@ClientString(id = 85712, message = "Ooh, pictures! Ok, I'm ready. Cheese!")
	public static NpcStringId OOH_PICTURES_OK_I_M_READY_CHEESE;
	
	@ClientString(id = 85713, message = "Whoa, wait a minute you. Who do you think you are, shoving a camera in my face without permission? I'm going to report you!")
	public static NpcStringId WHOA_WAIT_A_MINUTE_YOU_WHO_DO_YOU_THINK_YOU_ARE_SHOVING_A_CAMERA_IN_MY_FACE_WITHOUT_PERMISSION_I_M_GOING_TO_REPORT_YOU;
	
	@ClientString(id = 85714, message = "No no, no pictures! I look horrible today!")
	public static NpcStringId NO_NO_NO_PICTURES_I_LOOK_HORRIBLE_TODAY;
	
	@ClientString(id = 85715, message = "You scared me! Geez. Oh hey, if anyone asks…you didn't see me taking a break from the fashion show, ok?")
	public static NpcStringId YOU_SCARED_ME_GEEZ_OH_HEY_IF_ANYONE_ASKS_YOU_DIDN_T_SEE_ME_TAKING_A_BREAK_FROM_THE_FASHION_SHOW_OK;
	
	@ClientString(id = 85716, message = "Ta-da! What do you think? Like, totally trendy, right?")
	public static NpcStringId TA_DA_WHAT_DO_YOU_THINK_LIKE_TOTALLY_TRENDY_RIGHT;
	
	@ClientString(id = 85717, message = "I see…a future lying ahead of you. Don't speak! I see…I see…yes, a trendsetter. You shall be the trendiest trendsetter! Muahaha!")
	public static NpcStringId I_SEE_A_FUTURE_LYING_AHEAD_OF_YOU_DON_T_SPEAK_I_SEE_I_SEE_YES_A_TRENDSETTER_YOU_SHALL_BE_THE_TRENDIEST_TRENDSETTER_MUAHAHA;
	
	@ClientString(id = 85718, message = "You are cordially invited to this year's special fashion show featuring Sir Adenia Andron.")
	public static NpcStringId YOU_ARE_CORDIALLY_INVITED_TO_THIS_YEAR_S_SPECIAL_FASHION_SHOW_FEATURING_SIR_ADENIA_ANDRON;
	
	@ClientString(id = 85719, message = "Whew…walking practices all day, like a newbie! It's humiliating!")
	public static NpcStringId WHEW_WALKING_PRACTICES_ALL_DAY_LIKE_A_NEWBIE_IT_S_HUMILIATING;
	
	@ClientString(id = 85720, message = "Be sure to watch the fashion show! Take this little gift from me.")
	public static NpcStringId BE_SURE_TO_WATCH_THE_FASHION_SHOW_TAKE_THIS_LITTLE_GIFT_FROM_ME;
	
	@ClientString(id = 85721, message = "You want my autograph? Then come to the fashion show! Here, you can take this.")
	public static NpcStringId YOU_WANT_MY_AUTOGRAPH_THEN_COME_TO_THE_FASHION_SHOW_HERE_YOU_CAN_TAKE_THIS;
	
	@ClientString(id = 85722, message = "Eye-catching fashion is a must in a festival! Try on Sir Adenia Andron's high-end styles!")
	public static NpcStringId EYE_CATCHING_FASHION_IS_A_MUST_IN_A_FESTIVAL_TRY_ON_SIR_ADENIA_ANDRON_S_HIGH_END_STYLES;
	
	@ClientString(id = 85723, message = "Haha, ha… um, I'll be going now. See you at the fashion show!")
	public static NpcStringId HAHA_HA_UM_I_LL_BE_GOING_NOW_SEE_YOU_AT_THE_FASHION_SHOW;
	
	@ClientString(id = 85724, message = "Here is a Contender's Mark! Please be sure to attend the Fantasy Isle Festival!")
	public static NpcStringId HERE_IS_A_CONTENDER_S_MARK_PLEASE_BE_SURE_TO_ATTEND_THE_FANTASY_ISLE_FESTIVAL;
	
	@ClientString(id = 85725, message = "So long, adventurer! See you again at the Fantasy Isle Festival!")
	public static NpcStringId SO_LONG_ADVENTURER_SEE_YOU_AGAIN_AT_THE_FANTASY_ISLE_FESTIVAL;
	
	@ClientString(id = 85726, message = "So apparently they make monsters duel and reward the winner! Sounds interesting, doesn't it?")
	public static NpcStringId SO_APPARENTLY_THEY_MAKE_MONSTERS_DUEL_AND_REWARD_THE_WINNER_SOUNDS_INTERESTING_DOESN_T_IT;
	
	@ClientString(id = 85727, message = "These contenders are bloodthirsty nothings. No wonder I can't win!")
	public static NpcStringId THESE_CONTENDERS_ARE_BLOODTHIRSTY_NOTHINGS_NO_WONDER_I_CAN_T_WIN;
	
	@ClientString(id = 85728, message = "Hey, this match is a lot like a… wait, no. It can't be…it can't be.")
	public static NpcStringId HEY_THIS_MATCH_IS_A_LOT_LIKE_A_WAIT_NO_IT_CAN_T_BE_IT_CAN_T_BE;
	
	@ClientString(id = 85729, message = "So you can use the Monster Arena if you have a Contender's Mark, huh?")
	public static NpcStringId SO_YOU_CAN_USE_THE_MONSTER_ARENA_IF_YOU_HAVE_A_CONTENDER_S_MARK_HUH;
	
	@ClientString(id = 85730, message = "Aren't there any cute monsters to look at? These are really not aesthetically appealing…")
	public static NpcStringId AREN_T_THERE_ANY_CUTE_MONSTERS_TO_LOOK_AT_THESE_ARE_REALLY_NOT_AESTHETICALLY_APPEALING;
	
	@ClientString(id = 85780, message = "Ahh, how long since I last visited Fantasy Isle again?")
	public static NpcStringId AHH_HOW_LONG_SINCE_I_LAST_VISITED_FANTASY_ISLE_AGAIN;
	
	@ClientString(id = 85781, message = "Mm, yes. The crowd, the noise, the junk food. The markers of a true festival!")
	public static NpcStringId MM_YES_THE_CROWD_THE_NOISE_THE_JUNK_FOOD_THE_MARKERS_OF_A_TRUE_FESTIVAL;
	
	@ClientString(id = 85782, message = "Oh hey, new attractions! I can't leave these things untested now, can I?")
	public static NpcStringId OH_HEY_NEW_ATTRACTIONS_I_CAN_T_LEAVE_THESE_THINGS_UNTESTED_NOW_CAN_I;
	
	@ClientString(id = 85783, message = "New facilities are great and all, but nothing beats the good ol' Block Checkers or Kratei's Cube.")
	public static NpcStringId NEW_FACILITIES_ARE_GREAT_AND_ALL_BUT_NOTHING_BEATS_THE_GOOD_OL_BLOCK_CHECKERS_OR_KRATEI_S_CUBE;
	
	@ClientString(id = 85784, message = "Let's see…where should I start?")
	public static NpcStringId LET_S_SEE_WHERE_SHOULD_I_START;
	
	@ClientString(id = 85785, message = "Pst. Hey, rumor going around. Is it true that Fantasy Isle is falling on hard times?")
	public static NpcStringId PST_HEY_RUMOR_GOING_AROUND_IS_IT_TRUE_THAT_FANTASY_ISLE_IS_FALLING_ON_HARD_TIMES;
	
	@ClientString(id = 85790, message = "Fantasy Isle Tourist")
	public static NpcStringId FANTASY_ISLE_TOURIST;
	
	@ClientString(id = 85791, message = "Handy's Block Checker")
	public static NpcStringId HANDY_S_BLOCK_CHECKER;
	
	@ClientString(id = 85792, message = "Kratei's Cube")
	public static NpcStringId KRATEI_S_CUBE;
	
	@ClientString(id = 85793, message = "Coliseum")
	public static NpcStringId COLISEUM;
	
	@ClientString(id = 85794, message = "Duel Guide Mate")
	public static NpcStringId DUEL_GUIDE_MATE;
	
	@ClientString(id = 85795, message = "Fashion Show Catwalk")
	public static NpcStringId FASHION_SHOW_CATWALK;
	
	@ClientString(id = 85796, message = "Fantasy Isle Festival Promoter")
	public static NpcStringId FANTASY_ISLE_FESTIVAL_PROMOTER;
	
	@ClientString(id = 85797, message = "Fashion Show Model")
	public static NpcStringId FASHION_SHOW_MODEL;
	
	@ClientString(id = 90101, message = "How a Lavasaurus is Made (Lv. 76-99)")
	public static NpcStringId HOW_A_LAVASAURUS_IS_MADE_LV_76_99;
	
	@ClientString(id = 90102, message = "How a Lavasaurus is Made (Lv. 76-99) (In Progress)")
	public static NpcStringId HOW_A_LAVASAURUS_IS_MADE_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 90103, message = "How a Lavasaurus is Made (Lv. 76-99) (Done)")
	public static NpcStringId HOW_A_LAVASAURUS_IS_MADE_LV_76_99_DONE;
	
	@ClientString(id = 90104, message = "How a Lavasaurus is Made (Lv. 76-99)")
	public static NpcStringId HOW_A_LAVASAURUS_IS_MADE_LV_76_992;
	
	@ClientString(id = 90201, message = "Reclaim Our Era (Lv. 80-99)")
	public static NpcStringId RECLAIM_OUR_ERA_LV_80_99;
	
	@ClientString(id = 90202, message = "Reclaim Our Era (Lv. 80-99) (In Progress)")
	public static NpcStringId RECLAIM_OUR_ERA_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 90203, message = "Reclaim Our Era (Lv. 80-99) (Done)")
	public static NpcStringId RECLAIM_OUR_ERA_LV_80_99_DONE;
	
	@ClientString(id = 90204, message = "Reclaim Our Era (Lv. 80-99)")
	public static NpcStringId RECLAIM_OUR_ERA_LV_80_992;
	
	@ClientString(id = 90301, message = "The Call of Antharas (Lv. 83-99)")
	public static NpcStringId THE_CALL_OF_ANTHARAS_LV_83_99;
	
	@ClientString(id = 90302, message = "The Call of Antharas (Lv. 83-99) (In Progress)")
	public static NpcStringId THE_CALL_OF_ANTHARAS_LV_83_99_IN_PROGRESS;
	
	@ClientString(id = 90303, message = "The Call of Antharas (Lv. 83-99) (Done)")
	public static NpcStringId THE_CALL_OF_ANTHARAS_LV_83_99_DONE;
	
	@ClientString(id = 90304, message = "The Call of Antharas (Lv. 83-99)")
	public static NpcStringId THE_CALL_OF_ANTHARAS_LV_83_992;
	
	@ClientString(id = 90401, message = "Dragon Trophy - Antharas (Lv. 84-99)")
	public static NpcStringId DRAGON_TROPHY_ANTHARAS_LV_84_99;
	
	@ClientString(id = 90402, message = "Dragon Trophy - Antharas (Lv. 84-99) (In Progress)")
	public static NpcStringId DRAGON_TROPHY_ANTHARAS_LV_84_99_IN_PROGRESS;
	
	@ClientString(id = 90403, message = "Dragon Trophy - Antharas (Lv. 84-99) (Done)")
	public static NpcStringId DRAGON_TROPHY_ANTHARAS_LV_84_99_DONE;
	
	@ClientString(id = 90404, message = "Dragon Trophy - Antharas (Lv. 84-99)")
	public static NpcStringId DRAGON_TROPHY_ANTHARAS_LV_84_992;
	
	@ClientString(id = 90501, message = "Refined Dragon Blood (Lv. 83-99)")
	public static NpcStringId REFINED_DRAGON_BLOOD_LV_83_99;
	
	@ClientString(id = 90502, message = "Refined Dragon Blood (Lv. 83-99) (In Progress)")
	public static NpcStringId REFINED_DRAGON_BLOOD_LV_83_99_IN_PROGRESS;
	
	@ClientString(id = 90503, message = "Refined Dragon Blood (Lv. 83-99) (Done)")
	public static NpcStringId REFINED_DRAGON_BLOOD_LV_83_99_DONE;
	
	@ClientString(id = 90504, message = "Refined Dragon Blood (Lv. 83-99)")
	public static NpcStringId REFINED_DRAGON_BLOOD_LV_83_992;
	
	@ClientString(id = 90601, message = "The Call of Valakas (Lv. 83-99)")
	public static NpcStringId THE_CALL_OF_VALAKAS_LV_83_99;
	
	@ClientString(id = 90602, message = "The Call of Valakas (Lv. 83-99) (In Progress)")
	public static NpcStringId THE_CALL_OF_VALAKAS_LV_83_99_IN_PROGRESS;
	
	@ClientString(id = 90603, message = "The Call of Valakas (Lv. 83-99) (Done)")
	public static NpcStringId THE_CALL_OF_VALAKAS_LV_83_99_DONE;
	
	@ClientString(id = 90604, message = "The Call of Valakas (Lv. 83-99)")
	public static NpcStringId THE_CALL_OF_VALAKAS_LV_83_992;
	
	@ClientString(id = 90701, message = "Dragon Trophy - Valakas (Lv. 84-99)")
	public static NpcStringId DRAGON_TROPHY_VALAKAS_LV_84_99;
	
	@ClientString(id = 90702, message = "Dragon Trophy - Valakas (Lv. 84-99) (In Progress)")
	public static NpcStringId DRAGON_TROPHY_VALAKAS_LV_84_99_IN_PROGRESS;
	
	@ClientString(id = 90703, message = "Dragon Trophy - Valakas (Lv. 84-99) (Done)")
	public static NpcStringId DRAGON_TROPHY_VALAKAS_LV_84_99_DONE;
	
	@ClientString(id = 90704, message = "Dragon Trophy - Valakas (Lv. 84-99)")
	public static NpcStringId DRAGON_TROPHY_VALAKAS_LV_84_992;
	
	@ClientString(id = 94301, message = "Filling the Energy of Destruction (Lv. 90-99)")
	public static NpcStringId FILLING_THE_ENERGY_OF_DESTRUCTION_LV_90_99;
	
	@ClientString(id = 94302, message = "Filling the Energy of Destruction (Lv. 90-99) (In Progress)")
	public static NpcStringId FILLING_THE_ENERGY_OF_DESTRUCTION_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 94303, message = "Filling the Energy of Destruction (Lv. 90-99) (Done)")
	public static NpcStringId FILLING_THE_ENERGY_OF_DESTRUCTION_LV_90_99_DONE;
	
	@ClientString(id = 94304, message = "Filling the Energy of Destruction (Lv. 90-99)")
	public static NpcStringId FILLING_THE_ENERGY_OF_DESTRUCTION_LV_90_992;
	
	@ClientString(id = 98701, message = "Ilvenis' Test")
	public static NpcStringId ILVENIS_TEST;
	
	@ClientString(id = 98702, message = "Ilvenis' Test (In Progress)")
	public static NpcStringId ILVENIS_TEST_IN_PROGRESS;
	
	@ClientString(id = 98703, message = "Empty Cannon")
	public static NpcStringId EMPTY_CANNON;
	
	@ClientString(id = 98704, message = "Cannon is loading")
	public static NpcStringId CANNON_IS_LOADING;
	
	@ClientString(id = 98711, message = "Hit Ilvenis")
	public static NpcStringId HIT_ILVENIS;
	
	@ClientString(id = 99601, message = "Tra la la... Today, I'm going to make another fun-filled trip.  I wonder what I should look for this time...")
	public static NpcStringId TRA_LA_LA_TODAY_I_M_GOING_TO_MAKE_ANOTHER_FUN_FILLED_TRIP_I_WONDER_WHAT_I_SHOULD_LOOK_FOR_THIS_TIME;
	
	@ClientString(id = 99700, message = "What's this? Why am I being disturbed?")
	public static NpcStringId WHAT_S_THIS_WHY_AM_I_BEING_DISTURBED;
	
	@ClientString(id = 99701, message = "Ta-da! Here I am!")
	public static NpcStringId TA_DA_HERE_I_AM;
	
	@ClientString(id = 99702, message = "What are you looking at?")
	public static NpcStringId WHAT_ARE_YOU_LOOKING_AT;
	
	@ClientString(id = 99703, message = "If you give me nectar, this little Wintermelon will grow up quickly!")
	public static NpcStringId IF_YOU_GIVE_ME_NECTAR_THIS_LITTLE_WINTERMELON_WILL_GROW_UP_QUICKLY;
	
	@ClientString(id = 99704, message = "Are you my mommy?")
	public static NpcStringId ARE_YOU_MY_MOMMY;
	
	@ClientString(id = 99705, message = "Fancy meeting you here!")
	public static NpcStringId FANCY_MEETING_YOU_HERE;
	
	@ClientString(id = 99706, message = "Are you afraid of the big-bad Wintermlon?")
	public static NpcStringId ARE_YOU_AFRAID_OF_THE_BIG_BAD_WINTERMLON;
	
	@ClientString(id = 99707, message = "Impressive, aren't I?")
	public static NpcStringId IMPRESSIVE_AREN_T_I;
	
	@ClientString(id = 99708, message = "Obey me!!")
	public static NpcStringId OBEY_ME;
	
	@ClientString(id = 99709, message = "Raise me well and you'll be rewarded. Neglect me and suffer the consequences!")
	public static NpcStringId RAISE_ME_WELL_AND_YOU_LL_BE_REWARDED_NEGLECT_ME_AND_SUFFER_THE_CONSEQUENCES;
	
	@ClientString(id = 99710, message = "Transform!")
	public static NpcStringId TRANSFORM;
	
	@ClientString(id = 99711, message = "I feel different?")
	public static NpcStringId I_FEEL_DIFFERENT;
	
	@ClientString(id = 99712, message = "I'm bigger now! Bring it on!")
	public static NpcStringId I_M_BIGGER_NOW_BRING_IT_ON;
	
	@ClientString(id = 99713, message = "I'm not a kid anymore!")
	public static NpcStringId I_M_NOT_A_KID_ANYMORE;
	
	@ClientString(id = 99714, message = "Big time!")
	public static NpcStringId BIG_TIME;
	
	@ClientString(id = 99715, message = "Good luck!")
	public static NpcStringId GOOD_LUCK2;
	
	@ClientString(id = 99716, message = "I'm all grown up now!")
	public static NpcStringId I_M_ALL_GROWN_UP_NOW;
	
	@ClientString(id = 99717, message = "If you let me go, I'll be your best friend.")
	public static NpcStringId IF_YOU_LET_ME_GO_I_LL_BE_YOUR_BEST_FRIEND;
	
	@ClientString(id = 99718, message = "I'm chuck full of goodness!")
	public static NpcStringId I_M_CHUCK_FULL_OF_GOODNESS;
	
	@ClientString(id = 99719, message = "Good job! Now what are you going to do?")
	public static NpcStringId GOOD_JOB_NOW_WHAT_ARE_YOU_GOING_TO_DO;
	
	@ClientString(id = 99720, message = "Keep it coming!")
	public static NpcStringId KEEP_IT_COMING;
	
	@ClientString(id = 99721, message = "That's what I'm talking about!")
	public static NpcStringId THAT_S_WHAT_I_M_TALKING_ABOUT;
	
	@ClientString(id = 99722, message = "May I have some more?")
	public static NpcStringId MAY_I_HAVE_SOME_MORE;
	
	@ClientString(id = 99723, message = "That hit the spot!")
	public static NpcStringId THAT_HIT_THE_SPOT;
	
	@ClientString(id = 99724, message = "I feel special!")
	public static NpcStringId I_FEEL_SPECIAL;
	
	@ClientString(id = 99725, message = "I think it's working!")
	public static NpcStringId I_THINK_IT_S_WORKING;
	
	@ClientString(id = 99726, message = "You DO understand!")
	public static NpcStringId YOU_DO_UNDERSTAND;
	
	@ClientString(id = 99727, message = "Yuck! What is this? Ha Ha just kidding!")
	public static NpcStringId YUCK_WHAT_IS_THIS_HA_HA_JUST_KIDDING;
	
	@ClientString(id = 99728, message = "A total of five and I'll be twice as alive!")
	public static NpcStringId A_TOTAL_OF_FIVE_AND_I_LL_BE_TWICE_AS_ALIVE;
	
	@ClientString(id = 99729, message = "Nectar is sublime!")
	public static NpcStringId NECTAR_IS_SUBLIME;
	
	@ClientString(id = 99730, message = "You call that a hit?")
	public static NpcStringId YOU_CALL_THAT_A_HIT;
	
	@ClientString(id = 99731, message = "Why are you hitting me? Ouch, stop it! Give me nectar!")
	public static NpcStringId WHY_ARE_YOU_HITTING_ME_OUCH_STOP_IT_GIVE_ME_NECTAR;
	
	@ClientString(id = 99732, message = "Stop or I'll wilt!")
	public static NpcStringId STOP_OR_I_LL_WILT;
	
	@ClientString(id = 99733, message = "I'm not fully grown yet! Oh well, do what you will. I'll fade away without nectar anyway!")
	public static NpcStringId I_M_NOT_FULLY_GROWN_YET_OH_WELL_DO_WHAT_YOU_WILL_I_LL_FADE_AWAY_WITHOUT_NECTAR_ANYWAY;
	
	@ClientString(id = 99734, message = "Go ahead and hit me again. It won't do you any good!")
	public static NpcStringId GO_AHEAD_AND_HIT_ME_AGAIN_IT_WON_T_DO_YOU_ANY_GOOD;
	
	@ClientString(id = 99735, message = "Woe is me! I'm wilting!")
	public static NpcStringId WOE_IS_ME_I_M_WILTING;
	
	@ClientString(id = 99736, message = "I'm not fully grown yet! How about some nectar to ease my pain?")
	public static NpcStringId I_M_NOT_FULLY_GROWN_YET_HOW_ABOUT_SOME_NECTAR_TO_EASE_MY_PAIN;
	
	@ClientString(id = 99737, message = "The end is near!")
	public static NpcStringId THE_END_IS_NEAR;
	
	@ClientString(id = 99738, message = "Pretty please... with sugar on top, give me some nectar!")
	public static NpcStringId PRETTY_PLEASE_WITH_SUGAR_ON_TOP_GIVE_ME_SOME_NECTAR;
	
	@ClientString(id = 99739, message = "If I die without nectar, you'll get nothing.")
	public static NpcStringId IF_I_DIE_WITHOUT_NECTAR_YOU_LL_GET_NOTHING;
	
	@ClientString(id = 99740, message = "I'm feeling better! Another thirty seconds and I'll be out of here!")
	public static NpcStringId I_M_FEELING_BETTER_ANOTHER_THIRTY_SECONDS_AND_I_LL_BE_OUT_OF_HERE;
	
	@ClientString(id = 99741, message = "Twenty seconds and it's ciao, baby!")
	public static NpcStringId TWENTY_SECONDS_AND_IT_S_CIAO_BABY;
	
	@ClientString(id = 99742, message = "Woohoo, just ten seconds left! nine... eight... seven!")
	public static NpcStringId WOOHOO_JUST_TEN_SECONDS_LEFT_NINE_EIGHT_SEVEN;
	
	@ClientString(id = 99743, message = "Give me nectar or I'll be gone in two minutes!")
	public static NpcStringId GIVE_ME_NECTAR_OR_I_LL_BE_GONE_IN_TWO_MINUTES;
	
	@ClientString(id = 99744, message = "Give me nectar or I'll be gone in one minute!")
	public static NpcStringId GIVE_ME_NECTAR_OR_I_LL_BE_GONE_IN_ONE_MINUTE;
	
	@ClientString(id = 99745, message = "So long, losers!")
	public static NpcStringId SO_LONG_LOSERS;
	
	@ClientString(id = 99746, message = "I'm out of here!")
	public static NpcStringId I_M_OUT_OF_HERE;
	
	@ClientString(id = 99747, message = "I must be going! Have fun everybody!")
	public static NpcStringId I_MUST_BE_GOING_HAVE_FUN_EVERYBODY;
	
	@ClientString(id = 99748, message = "Time is up! Put your weapons down!")
	public static NpcStringId TIME_IS_UP_PUT_YOUR_WEAPONS_DOWN;
	
	@ClientString(id = 99749, message = "Good for me, bad for you!")
	public static NpcStringId GOOD_FOR_ME_BAD_FOR_YOU;
	
	@ClientString(id = 99750, message = "Soundtastic!")
	public static NpcStringId SOUNDTASTIC;
	
	@ClientString(id = 99751, message = "I can sing along if you like?")
	public static NpcStringId I_CAN_SING_ALONG_IF_YOU_LIKE;
	
	@ClientString(id = 99752, message = "I think you need some backup!")
	public static NpcStringId I_THINK_YOU_NEED_SOME_BACKUP;
	
	@ClientString(id = 99753, message = "Keep up that rhythm and you'll be a star!")
	public static NpcStringId KEEP_UP_THAT_RHYTHM_AND_YOU_LL_BE_A_STAR;
	
	@ClientString(id = 99754, message = "My heart yearns for more music.")
	public static NpcStringId MY_HEART_YEARNS_FOR_MORE_MUSIC;
	
	@ClientString(id = 99755, message = "You're out of tune again!")
	public static NpcStringId YOU_RE_OUT_OF_TUNE_AGAIN;
	
	@ClientString(id = 99756, message = "This is awful!")
	public static NpcStringId THIS_IS_AWFUL;
	
	@ClientString(id = 99757, message = "I think I broke something!")
	public static NpcStringId I_THINK_I_BROKE_SOMETHING;
	
	@ClientString(id = 99758, message = "What a lovely melody! Play it again!")
	public static NpcStringId WHAT_A_LOVELY_MELODY_PLAY_IT_AGAIN;
	
	@ClientString(id = 99759, message = "Music to my, uh... ears!")
	public static NpcStringId MUSIC_TO_MY_UH_EARS;
	
	@ClientString(id = 99760, message = "You need music lessons!")
	public static NpcStringId YOU_NEED_MUSIC_LESSONS;
	
	@ClientString(id = 99761, message = "I can't hear you!")
	public static NpcStringId I_CAN_T_HEAR_YOU;
	
	@ClientString(id = 99762, message = "You can't hurt me like that!")
	public static NpcStringId YOU_CAN_T_HURT_ME_LIKE_THAT;
	
	@ClientString(id = 99763, message = "I'm stronger than you are!")
	public static NpcStringId I_M_STRONGER_THAN_YOU_ARE;
	
	@ClientString(id = 99764, message = "No music? I'm out of here!")
	public static NpcStringId NO_MUSIC_I_M_OUT_OF_HERE;
	
	@ClientString(id = 99765, message = "That racket is getting on my nerves! Tone it down a bit!")
	public static NpcStringId THAT_RACKET_IS_GETTING_ON_MY_NERVES_TONE_IT_DOWN_A_BIT;
	
	@ClientString(id = 99766, message = "You can only hurt me through music!")
	public static NpcStringId YOU_CAN_ONLY_HURT_ME_THROUGH_MUSIC;
	
	@ClientString(id = 99767, message = "Only musical instruments can hurt me! Nothing else!")
	public static NpcStringId ONLY_MUSICAL_INSTRUMENTS_CAN_HURT_ME_NOTHING_ELSE;
	
	@ClientString(id = 99768, message = "Your skills are impressive, but sadly, useless...")
	public static NpcStringId YOUR_SKILLS_ARE_IMPRESSIVE_BUT_SADLY_USELESS;
	
	@ClientString(id = 99769, message = "Catch a Chrono for me please.")
	public static NpcStringId CATCH_A_CHRONO_FOR_ME_PLEASE;
	
	@ClientString(id = 99770, message = "You got me!")
	public static NpcStringId YOU_GOT_ME;
	
	@ClientString(id = 99771, message = "Now look at what you've done!")
	public static NpcStringId NOW_LOOK_AT_WHAT_YOU_VE_DONE;
	
	@ClientString(id = 99772, message = "You win!")
	public static NpcStringId YOU_WIN;
	
	@ClientString(id = 99773, message = "Fingers crossed!")
	public static NpcStringId FINGERS_CROSSED;
	
	@ClientString(id = 99774, message = "Don't tell anyone!")
	public static NpcStringId DON_T_TELL_ANYONE;
	
	@ClientString(id = 99775, message = "Gross! My guts are coming out!")
	public static NpcStringId GROSS_MY_GUTS_ARE_COMING_OUT;
	
	@ClientString(id = 99776, message = "Take it and go.")
	public static NpcStringId TAKE_IT_AND_GO;
	
	@ClientString(id = 99777, message = "I should've left when I could!")
	public static NpcStringId I_SHOULD_VE_LEFT_WHEN_I_COULD;
	
	@ClientString(id = 99778, message = "Now look what you have done!")
	public static NpcStringId NOW_LOOK_WHAT_YOU_HAVE_DONE;
	
	@ClientString(id = 99779, message = "I feel dirty!")
	public static NpcStringId I_FEEL_DIRTY;
	
	@ClientString(id = 99780, message = "Better luck next time!")
	public static NpcStringId BETTER_LUCK_NEXT_TIME;
	
	@ClientString(id = 99781, message = "Nice shot!")
	public static NpcStringId NICE_SHOT;
	
	@ClientString(id = 99782, message = "I'm not afraid of you!")
	public static NpcStringId I_M_NOT_AFRAID_OF_YOU;
	
	@ClientString(id = 99783, message = "If I knew this was going to happen, I would have stayed home!")
	public static NpcStringId IF_I_KNEW_THIS_WAS_GOING_TO_HAPPEN_I_WOULD_HAVE_STAYED_HOME;
	
	@ClientString(id = 99784, message = "Try harder or I'm out of here!")
	public static NpcStringId TRY_HARDER_OR_I_M_OUT_OF_HERE;
	
	@ClientString(id = 99785, message = "I'm tougher than I look!")
	public static NpcStringId I_M_TOUGHER_THAN_I_LOOK;
	
	@ClientString(id = 99786, message = "Good strike!")
	public static NpcStringId GOOD_STRIKE;
	
	@ClientString(id = 99787, message = "Oh my gourd!")
	public static NpcStringId OH_MY_GOURD;
	
	@ClientString(id = 99788, message = "That's all you've got?")
	public static NpcStringId THAT_S_ALL_YOU_VE_GOT;
	
	@ClientString(id = 99789, message = "Why me?")
	public static NpcStringId WHY_ME;
	
	@ClientString(id = 99790, message = "Bring me nectar!")
	public static NpcStringId BRING_ME_NECTAR;
	
	@ClientString(id = 99791, message = "I must have nectar to grow!")
	public static NpcStringId I_MUST_HAVE_NECTAR_TO_GROW;
	
	@ClientString(id = 99792, message = "Give me some nectar quickly or you'll get nothing!")
	public static NpcStringId GIVE_ME_SOME_NECTAR_QUICKLY_OR_YOU_LL_GET_NOTHING;
	
	@ClientString(id = 99793, message = "Please give me some nectar! I'm hungry!")
	public static NpcStringId PLEASE_GIVE_ME_SOME_NECTAR_I_M_HUNGRY;
	
	@ClientString(id = 99794, message = "Nectar please!")
	public static NpcStringId NECTAR_PLEASE;
	
	@ClientString(id = 99795, message = "Nectar will make me grow quickly!")
	public static NpcStringId NECTAR_WILL_MAKE_ME_GROW_QUICKLY;
	
	@ClientString(id = 99796, message = "Don't you want a bigger wintermelon? Give me some Nectar and I'll grow much larger!")
	public static NpcStringId DON_T_YOU_WANT_A_BIGGER_WINTERMELON_GIVE_ME_SOME_NECTAR_AND_I_LL_GROW_MUCH_LARGER;
	
	@ClientString(id = 99797, message = "If you raise me well, you'll get prizes! Or not...")
	public static NpcStringId IF_YOU_RAISE_ME_WELL_YOU_LL_GET_PRIZES_OR_NOT;
	
	@ClientString(id = 99798, message = "You are here for the stuff, eh? Well it's mine, all mine!")
	public static NpcStringId YOU_ARE_HERE_FOR_THE_STUFF_EH_WELL_IT_S_MINE_ALL_MINE;
	
	@ClientString(id = 99799, message = "Trust me, give me some Nectar and I'll become a giant Wintermelon!")
	public static NpcStringId TRUST_ME_GIVE_ME_SOME_NECTAR_AND_I_LL_BECOME_A_GIANT_WINTERMELON;
	
	@ClientString(id = 526701, message = "Journey to Gracia")
	public static NpcStringId JOURNEY_TO_GRACIA;
	
	@ClientString(id = 526702, message = "Journey to Gracia (In Progress)")
	public static NpcStringId JOURNEY_TO_GRACIA_IN_PROGRESS;
	
	@ClientString(id = 526703, message = "Journey to Gracia (Done)")
	public static NpcStringId JOURNEY_TO_GRACIA_DONE;
	
	@ClientString(id = 526704, message = "Journey to Gracia")
	public static NpcStringId JOURNEY_TO_GRACIA2;
	
	@ClientString(id = 526801, message = "To the Seed of Infinity")
	public static NpcStringId TO_THE_SEED_OF_INFINITY;
	
	@ClientString(id = 526802, message = "To the Seed of Infinity (In Progress)")
	public static NpcStringId TO_THE_SEED_OF_INFINITY_IN_PROGRESS;
	
	@ClientString(id = 526803, message = "To the Seed of Infinity (Done)")
	public static NpcStringId TO_THE_SEED_OF_INFINITY_DONE;
	
	@ClientString(id = 526804, message = "To the Seed of Infinity")
	public static NpcStringId TO_THE_SEED_OF_INFINITY2;
	
	@ClientString(id = 526901, message = "To the Seed of Destruction")
	public static NpcStringId TO_THE_SEED_OF_DESTRUCTION;
	
	@ClientString(id = 526902, message = "To the Seed of Destruction (In Progress)")
	public static NpcStringId TO_THE_SEED_OF_DESTRUCTION_IN_PROGRESS;
	
	@ClientString(id = 526903, message = "To the Seed of Destruction (Done)")
	public static NpcStringId TO_THE_SEED_OF_DESTRUCTION_DONE;
	
	@ClientString(id = 526904, message = "To the Seed of Destruction")
	public static NpcStringId TO_THE_SEED_OF_DESTRUCTION2;
	
	@ClientString(id = 527001, message = "Birth of the Seed")
	public static NpcStringId BIRTH_OF_THE_SEED;
	
	@ClientString(id = 527002, message = "Birth of the Seed (In Progress)")
	public static NpcStringId BIRTH_OF_THE_SEED_IN_PROGRESS;
	
	@ClientString(id = 527003, message = "Birth of the Seed (Done)")
	public static NpcStringId BIRTH_OF_THE_SEED_DONE;
	
	@ClientString(id = 527004, message = "Birth of the Seed")
	public static NpcStringId BIRTH_OF_THE_SEED2;
	
	@ClientString(id = 527101, message = "The Enveloping Darkness")
	public static NpcStringId THE_ENVELOPING_DARKNESS;
	
	@ClientString(id = 527102, message = "The Enveloping Darkness (In Progress)")
	public static NpcStringId THE_ENVELOPING_DARKNESS_IN_PROGRESS;
	
	@ClientString(id = 527103, message = "The Enveloping Darkness (Done)")
	public static NpcStringId THE_ENVELOPING_DARKNESS_DONE;
	
	@ClientString(id = 527104, message = "The Enveloping Darkness")
	public static NpcStringId THE_ENVELOPING_DARKNESS2;
	
	@ClientString(id = 527201, message = "Light Fragment")
	public static NpcStringId LIGHT_FRAGMENT;
	
	@ClientString(id = 527202, message = "Light Fragment (In Progress)")
	public static NpcStringId LIGHT_FRAGMENT_IN_PROGRESS;
	
	@ClientString(id = 527203, message = "Light Fragment (Done)")
	public static NpcStringId LIGHT_FRAGMENT_DONE;
	
	@ClientString(id = 527204, message = "Light Fragment")
	public static NpcStringId LIGHT_FRAGMENT2;
	
	@ClientString(id = 527301, message = "Good Day to Fly (Lv. 75-99)")
	public static NpcStringId GOOD_DAY_TO_FLY_LV_75_99;
	
	@ClientString(id = 527302, message = "Good Day to Fly (Lv. 75-99) (In Progress)")
	public static NpcStringId GOOD_DAY_TO_FLY_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 527303, message = "Good Day to Fly (Lv. 75-99) (Done)")
	public static NpcStringId GOOD_DAY_TO_FLY_LV_75_99_DONE;
	
	@ClientString(id = 527304, message = "Good Day to Fly (Lv. 75-99)")
	public static NpcStringId GOOD_DAY_TO_FLY_LV_75_992;
	
	@ClientString(id = 527401, message = "Collecting in the Air (Lv. 75-99)")
	public static NpcStringId COLLECTING_IN_THE_AIR_LV_75_99;
	
	@ClientString(id = 527402, message = "Collecting in the Air (Lv. 75-99) (In Progress)")
	public static NpcStringId COLLECTING_IN_THE_AIR_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 527403, message = "Collecting in the Air (Lv. 75-99) (Done)")
	public static NpcStringId COLLECTING_IN_THE_AIR_LV_75_99_DONE;
	
	@ClientString(id = 527404, message = "Collecting in the Air (Lv. 75-99)")
	public static NpcStringId COLLECTING_IN_THE_AIR_LV_75_992;
	
	@ClientString(id = 527501, message = "Containing the Attribute Power (Lv. 76-99)")
	public static NpcStringId CONTAINING_THE_ATTRIBUTE_POWER_LV_76_99;
	
	@ClientString(id = 527502, message = "Containing the Attribute Power (Lv. 76-99) (In Progress)")
	public static NpcStringId CONTAINING_THE_ATTRIBUTE_POWER_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 527503, message = "Containing the Attribute Power (Lv. 76-99) (Done)")
	public static NpcStringId CONTAINING_THE_ATTRIBUTE_POWER_LV_76_99_DONE;
	
	@ClientString(id = 527504, message = "Containing the Attribute Power (Lv. 76-99)")
	public static NpcStringId CONTAINING_THE_ATTRIBUTE_POWER_LV_76_992;
	
	@ClientString(id = 527601, message = "Mutated Kaneus - Gludio (Lv. 18-99)")
	public static NpcStringId MUTATED_KANEUS_GLUDIO_LV_18_99;
	
	@ClientString(id = 527602, message = "Mutated Kaneus - Gludio (Lv. 18-99) (In Progress)")
	public static NpcStringId MUTATED_KANEUS_GLUDIO_LV_18_99_IN_PROGRESS;
	
	@ClientString(id = 527603, message = "Mutated Kaneus - Gludio (Lv. 18-99) (Done)")
	public static NpcStringId MUTATED_KANEUS_GLUDIO_LV_18_99_DONE;
	
	@ClientString(id = 527604, message = "Mutated Kaneus - Gludio (Lv. 18-99)")
	public static NpcStringId MUTATED_KANEUS_GLUDIO_LV_18_992;
	
	@ClientString(id = 527701, message = "Mutated Kaneus - Dion (Lv. 28-99)")
	public static NpcStringId MUTATED_KANEUS_DION_LV_28_99;
	
	@ClientString(id = 527702, message = "Mutated Kaneus - Dion (Lv. 28-99) (In Progress)")
	public static NpcStringId MUTATED_KANEUS_DION_LV_28_99_IN_PROGRESS;
	
	@ClientString(id = 527703, message = "Mutated Kaneus - Dion (Lv. 28-99) (Done)")
	public static NpcStringId MUTATED_KANEUS_DION_LV_28_99_DONE;
	
	@ClientString(id = 527704, message = "Mutated Kaneus - Dion (Lv. 28-99)")
	public static NpcStringId MUTATED_KANEUS_DION_LV_28_992;
	
	@ClientString(id = 527801, message = "Mutated Kaneus - Heine (Lv. 38-99)")
	public static NpcStringId MUTATED_KANEUS_HEINE_LV_38_99;
	
	@ClientString(id = 527802, message = "Mutated Kaneus - Heine (Lv. 38-99) (In Progress)")
	public static NpcStringId MUTATED_KANEUS_HEINE_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 527803, message = "Mutated Kaneus - Heine (Lv. 38-99) (Done)")
	public static NpcStringId MUTATED_KANEUS_HEINE_LV_38_99_DONE;
	
	@ClientString(id = 527804, message = "Mutated Kaneus - Heine (Lv. 38-99)")
	public static NpcStringId MUTATED_KANEUS_HEINE_LV_38_992;
	
	@ClientString(id = 527901, message = "Mutated Kaneus - Oren (Lv. 48-99)")
	public static NpcStringId MUTATED_KANEUS_OREN_LV_48_99;
	
	@ClientString(id = 527902, message = "Mutated Kaneus - Oren (Lv. 48-99) (In Progress)")
	public static NpcStringId MUTATED_KANEUS_OREN_LV_48_99_IN_PROGRESS;
	
	@ClientString(id = 527903, message = "Mutated Kaneus - Oren (Lv. 48-99) (Done)")
	public static NpcStringId MUTATED_KANEUS_OREN_LV_48_99_DONE;
	
	@ClientString(id = 527904, message = "Mutated Kaneus - Oren (Lv. 48-99)")
	public static NpcStringId MUTATED_KANEUS_OREN_LV_48_992;
	
	@ClientString(id = 528001, message = "Mutated Kaneus - Schuttgart (Lv. 58-99)")
	public static NpcStringId MUTATED_KANEUS_SCHUTTGART_LV_58_99;
	
	@ClientString(id = 528002, message = "Mutated Kaneus - Schuttgart (Lv. 58-99) (In Progress)")
	public static NpcStringId MUTATED_KANEUS_SCHUTTGART_LV_58_99_IN_PROGRESS;
	
	@ClientString(id = 528003, message = "Mutated Kaneus - Schuttgart (Lv. 58-99) (Done)")
	public static NpcStringId MUTATED_KANEUS_SCHUTTGART_LV_58_99_DONE;
	
	@ClientString(id = 528004, message = "Mutated Kaneus - Schuttgart (Lv. 58-99)")
	public static NpcStringId MUTATED_KANEUS_SCHUTTGART_LV_58_992;
	
	@ClientString(id = 528101, message = "Mutated Kaneus - Rune (Lv. 68-99)")
	public static NpcStringId MUTATED_KANEUS_RUNE_LV_68_99;
	
	@ClientString(id = 528102, message = "Mutated Kaneus - Rune (Lv. 68-99) (In Progress)")
	public static NpcStringId MUTATED_KANEUS_RUNE_LV_68_99_IN_PROGRESS;
	
	@ClientString(id = 528103, message = "Mutated Kaneus - Rune (Lv. 68-99) (Done)")
	public static NpcStringId MUTATED_KANEUS_RUNE_LV_68_99_DONE;
	
	@ClientString(id = 528104, message = "Mutated Kaneus - Rune (Lv. 68-99)")
	public static NpcStringId MUTATED_KANEUS_RUNE_LV_68_992;
	
	@ClientString(id = 528201, message = "To the Seed of Annihilation (Lv. 85-99)")
	public static NpcStringId TO_THE_SEED_OF_ANNIHILATION_LV_85_99;
	
	@ClientString(id = 528202, message = "To the Seed of Annihilation (Lv. 85-99) (In Progress)")
	public static NpcStringId TO_THE_SEED_OF_ANNIHILATION_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 528203, message = "To the Seed of Annihilation (Lv. 85-99) (Done)")
	public static NpcStringId TO_THE_SEED_OF_ANNIHILATION_LV_85_99_DONE;
	
	@ClientString(id = 528301, message = "Request of Ice Merchant (Lv. 82-99)")
	public static NpcStringId REQUEST_OF_ICE_MERCHANT_LV_82_99;
	
	@ClientString(id = 528302, message = "Request of Ice Merchant  (Lv. 82-99)(In Progress)")
	public static NpcStringId REQUEST_OF_ICE_MERCHANT_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 528303, message = "Request of Ice Merchant  (Lv. 82-99)(Done)")
	public static NpcStringId REQUEST_OF_ICE_MERCHANT_LV_82_99_DONE;
	
	@ClientString(id = 528304, message = "Request of Ice Merchant (Lv. 82-99)")
	public static NpcStringId REQUEST_OF_ICE_MERCHANT_LV_82_992;
	
	@ClientString(id = 528401, message = "Acquisition of Divine Sword (Lv. 82-99)")
	public static NpcStringId ACQUISITION_OF_DIVINE_SWORD_LV_82_99;
	
	@ClientString(id = 528402, message = "Acquisition of Divine Sword  (Lv. 82-99)(In Progress)")
	public static NpcStringId ACQUISITION_OF_DIVINE_SWORD_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 528403, message = "Acquisition of Divine Sword  (Lv. 82-99)(Done)")
	public static NpcStringId ACQUISITION_OF_DIVINE_SWORD_LV_82_99_DONE;
	
	@ClientString(id = 528404, message = "Acquisition of Divine Sword (Lv. 82-99)")
	public static NpcStringId ACQUISITION_OF_DIVINE_SWORD_LV_82_992;
	
	@ClientString(id = 528501, message = "Meeting Sirra (Lv. 82-99)")
	public static NpcStringId MEETING_SIRRA_LV_82_99;
	
	@ClientString(id = 528502, message = "Meeting Sirra  (Lv. 82-99)(In Progress)")
	public static NpcStringId MEETING_SIRRA_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 528503, message = "Meeting Sirra  (Lv. 82-99)(Done)")
	public static NpcStringId MEETING_SIRRA_LV_82_99_DONE;
	
	@ClientString(id = 528504, message = "Meeting Sirra (Lv. 82-99)")
	public static NpcStringId MEETING_SIRRA_LV_82_992;
	
	@ClientString(id = 528551, message = "There's nothing you can't say. I can't listen to you anymore!")
	public static NpcStringId THERE_S_NOTHING_YOU_CAN_T_SAY_I_CAN_T_LISTEN_TO_YOU_ANYMORE;
	
	@ClientString(id = 528601, message = "Reunion with Sirra (Lv. 82-99)")
	public static NpcStringId REUNION_WITH_SIRRA_LV_82_99;
	
	@ClientString(id = 528602, message = "Reunion with Sirra  (Lv. 82-99)(In Progress)")
	public static NpcStringId REUNION_WITH_SIRRA_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 528603, message = "Reunion with Sirra  (Lv. 82-99)(Done)")
	public static NpcStringId REUNION_WITH_SIRRA_LV_82_99_DONE;
	
	@ClientString(id = 528604, message = "Reunion with Sirra (Lv. 82-99)")
	public static NpcStringId REUNION_WITH_SIRRA_LV_82_992;
	
	@ClientString(id = 528651, message = "You advanced bravely but got such a tiny result. Hohoho.")
	public static NpcStringId YOU_ADVANCED_BRAVELY_BUT_GOT_SUCH_A_TINY_RESULT_HOHOHO;
	
	@ClientString(id = 528701, message = "Story of Those Left (Lv. 82-99)")
	public static NpcStringId STORY_OF_THOSE_LEFT_LV_82_99;
	
	@ClientString(id = 528702, message = "Story of Those Left  (Lv. 82-99)(In Progress)")
	public static NpcStringId STORY_OF_THOSE_LEFT_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 528703, message = "Story of Those Left (Lv. 82-99) (Done)")
	public static NpcStringId STORY_OF_THOSE_LEFT_LV_82_99_DONE;
	
	@ClientString(id = 528704, message = "Story of Those Left (Lv. 82-99)")
	public static NpcStringId STORY_OF_THOSE_LEFT_LV_82_992;
	
	@ClientString(id = 528801, message = "Secret Mission (Lv. 82-99)")
	public static NpcStringId SECRET_MISSION_LV_82_99;
	
	@ClientString(id = 528802, message = "Secret Mission  (Lv. 82-99)(In Progress)")
	public static NpcStringId SECRET_MISSION_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 528803, message = "Secret Mission  (Lv. 82-99)(Done)")
	public static NpcStringId SECRET_MISSION_LV_82_99_DONE;
	
	@ClientString(id = 528804, message = "Secret Mission (Lv. 82-99)")
	public static NpcStringId SECRET_MISSION_LV_82_992;
	
	@ClientString(id = 528901, message = "Fade to Black (Lv. 82-99)")
	public static NpcStringId FADE_TO_BLACK_LV_82_99;
	
	@ClientString(id = 528902, message = "Fade to Black  (Lv. 82-99)(In Progress)")
	public static NpcStringId FADE_TO_BLACK_LV_82_99_IN_PROGRESS;
	
	@ClientString(id = 528903, message = "Fade to Black (Lv. 82-99) (Done)")
	public static NpcStringId FADE_TO_BLACK_LV_82_99_DONE;
	
	@ClientString(id = 528904, message = "Fade to Black (Lv. 82-99)")
	public static NpcStringId FADE_TO_BLACK_LV_82_992;
	
	@ClientString(id = 529001, message = "Land Dragon Conqueror (Lv. 83-99)")
	public static NpcStringId LAND_DRAGON_CONQUEROR_LV_83_99;
	
	@ClientString(id = 529002, message = "Land Dragon Conqueror (Lv. 83-99) (In Progress)")
	public static NpcStringId LAND_DRAGON_CONQUEROR_LV_83_99_IN_PROGRESS;
	
	@ClientString(id = 529003, message = "Land Dragon Conqueror (Lv. 83-99) (Done)")
	public static NpcStringId LAND_DRAGON_CONQUEROR_LV_83_99_DONE;
	
	@ClientString(id = 529004, message = "Land Dragon Conqueror (Lv. 83-99)")
	public static NpcStringId LAND_DRAGON_CONQUEROR_LV_83_992;
	
	@ClientString(id = 529101, message = "Fire Dragon Destroyer (Lv. 83-99)")
	public static NpcStringId FIRE_DRAGON_DESTROYER_LV_83_99;
	
	@ClientString(id = 529102, message = "Fire Dragon Destroyer (Lv. 83-99) (In Progress)")
	public static NpcStringId FIRE_DRAGON_DESTROYER_LV_83_99_IN_PROGRESS;
	
	@ClientString(id = 529103, message = "Fire Dragon Destroyer (Lv. 83-99) (Done)")
	public static NpcStringId FIRE_DRAGON_DESTROYER_LV_83_99_DONE;
	
	@ClientString(id = 529104, message = "Fire Dragon Destroyer (Lv. 83-99)")
	public static NpcStringId FIRE_DRAGON_DESTROYER_LV_83_992;
	
	@ClientString(id = 529201, message = "Seven Signs, Mysterious Girl (Lv. 81-99)")
	public static NpcStringId SEVEN_SIGNS_MYSTERIOUS_GIRL_LV_81_99;
	
	@ClientString(id = 529202, message = "Seven Signs, Mysterious Girl (Lv. 81-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_MYSTERIOUS_GIRL_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 529203, message = "Seven Signs, Mysterious Girl (Lv. 81-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_MYSTERIOUS_GIRL_LV_81_99_DONE;
	
	@ClientString(id = 529204, message = "Seven Signs, Mysterious Girl (Lv. 81-99)")
	public static NpcStringId SEVEN_SIGNS_MYSTERIOUS_GIRL_LV_81_992;
	
	@ClientString(id = 529301, message = "Seven Signs, Forbidden Book of the Elmoreden Kingdom (Lv. 81-99)")
	public static NpcStringId SEVEN_SIGNS_FORBIDDEN_BOOK_OF_THE_ELMOREDEN_KINGDOM_LV_81_99;
	
	@ClientString(id = 529302, message = "Seven Signs, Forbidden Book of the Elmoreden Kingdom (Lv. 81-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_FORBIDDEN_BOOK_OF_THE_ELMOREDEN_KINGDOM_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 529303, message = "Seven Signs, Forbidden Book of the Elmoreden Kingdom (Lv. 81-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_FORBIDDEN_BOOK_OF_THE_ELMOREDEN_KINGDOM_LV_81_99_DONE;
	
	@ClientString(id = 529401, message = "Seven Signs, To the Monastery of Silence (Lv. 81-99)")
	public static NpcStringId SEVEN_SIGNS_TO_THE_MONASTERY_OF_SILENCE_LV_81_99;
	
	@ClientString(id = 529402, message = "Seven Signs, To the Monastery of Silence (Lv. 81-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_TO_THE_MONASTERY_OF_SILENCE_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 529403, message = "Seven Signs, To the Monastery of Silence (Lv. 81-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_TO_THE_MONASTERY_OF_SILENCE_LV_81_99_DONE;
	
	@ClientString(id = 529501, message = "Seven Signs, Solina Tomb (Lv. 81-99)")
	public static NpcStringId SEVEN_SIGNS_SOLINA_TOMB_LV_81_99;
	
	@ClientString(id = 529502, message = "Seven Signs, Solina Tomb (Lv. 81-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_SOLINA_TOMB_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 529503, message = "Seven Signs, Solina Tomb (Lv. 81-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_SOLINA_TOMB_LV_81_99_DONE;
	
	@ClientString(id = 529601, message = "Seven Signs, One Who Seeks the Power of the Seal (Lv. 81-99)")
	public static NpcStringId SEVEN_SIGNS_ONE_WHO_SEEKS_THE_POWER_OF_THE_SEAL_LV_81_99;
	
	@ClientString(id = 529602, message = "Seven Signs, One Who Seeks the Power of the Seal (Lv. 81-99) (In Progress)")
	public static NpcStringId SEVEN_SIGNS_ONE_WHO_SEEKS_THE_POWER_OF_THE_SEAL_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 529603, message = "Seven Signs, One Who Seeks the Power of the Seal (Lv. 81-99) (Done)")
	public static NpcStringId SEVEN_SIGNS_ONE_WHO_SEEKS_THE_POWER_OF_THE_SEAL_LV_81_99_DONE;
	
	@ClientString(id = 529901, message = "How to Stand Up For Yourself")
	public static NpcStringId HOW_TO_STAND_UP_FOR_YOURSELF;
	
	@ClientString(id = 529902, message = "How to Stand Up For Yourself (In Progress)")
	public static NpcStringId HOW_TO_STAND_UP_FOR_YOURSELF_IN_PROGRESS;
	
	@ClientString(id = 529903, message = "How to Stand Up For Yourself (Done)")
	public static NpcStringId HOW_TO_STAND_UP_FOR_YOURSELF_DONE;
	
	@ClientString(id = 529904, message = "How to Stand Up For Yourself")
	public static NpcStringId HOW_TO_STAND_UP_FOR_YOURSELF2;
	
	@ClientString(id = 530001, message = "Berserk Outlaws (Lv. 50-54)")
	public static NpcStringId BERSERK_OUTLAWS_LV_50_54;
	
	@ClientString(id = 530002, message = "Berserk Outlaws (Lv. 50-54) (In Progress)")
	public static NpcStringId BERSERK_OUTLAWS_LV_50_54_IN_PROGRESS;
	
	@ClientString(id = 530003, message = "Berserk Outlaws (Lv. 50-54) (Done)")
	public static NpcStringId BERSERK_OUTLAWS_LV_50_54_DONE;
	
	@ClientString(id = 530004, message = "Berserk Outlaws (Lv. 50-54)")
	public static NpcStringId BERSERK_OUTLAWS_LV_50_542;
	
	@ClientString(id = 530101, message = "Shadow of Terror: Blackish Red Fog (Lv. 88-99)")
	public static NpcStringId SHADOW_OF_TERROR_BLACKISH_RED_FOG_LV_88_99;
	
	@ClientString(id = 530102, message = "Shadow of Terror: Blackish Red Fog (Lv. 88-99) (In Progress)")
	public static NpcStringId SHADOW_OF_TERROR_BLACKISH_RED_FOG_LV_88_99_IN_PROGRESS;
	
	@ClientString(id = 530103, message = "Shadow of Terror: Blackish Red Fog (Lv. 88-99) (Done)")
	public static NpcStringId SHADOW_OF_TERROR_BLACKISH_RED_FOG_LV_88_99_DONE;
	
	@ClientString(id = 530104, message = "Shadow of Terror: Blackish Red Fog (Lv. 88-99)")
	public static NpcStringId SHADOW_OF_TERROR_BLACKISH_RED_FOG_LV_88_992;
	
	@ClientString(id = 530201, message = "Unsettling Shadow and Rumors (Lv. 88-99)")
	public static NpcStringId UNSETTLING_SHADOW_AND_RUMORS_LV_88_99;
	
	@ClientString(id = 530202, message = "Unsettling Shadow and Rumors (Lv. 88-99) (In Progress)")
	public static NpcStringId UNSETTLING_SHADOW_AND_RUMORS_LV_88_99_IN_PROGRESS;
	
	@ClientString(id = 530203, message = "Unsettling Shadow and Rumors (Lv. 88-99) (Done)")
	public static NpcStringId UNSETTLING_SHADOW_AND_RUMORS_LV_88_99_DONE;
	
	@ClientString(id = 530204, message = "Unsettling Shadow and Rumors (Lv. 88-99)")
	public static NpcStringId UNSETTLING_SHADOW_AND_RUMORS_LV_88_992;
	
	@ClientString(id = 530301, message = "Verge of Light and Darkness (Lv. 90-99)")
	public static NpcStringId VERGE_OF_LIGHT_AND_DARKNESS_LV_90_99;
	
	@ClientString(id = 530302, message = "Verge of Light and Darkness (Lv. 90-99) (In Progress)")
	public static NpcStringId VERGE_OF_LIGHT_AND_DARKNESS_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 530303, message = "Verge of Light and Darkness (Lv. 90-99) (Done)")
	public static NpcStringId VERGE_OF_LIGHT_AND_DARKNESS_LV_90_99_DONE;
	
	@ClientString(id = 530304, message = "Verge of Light and Darkness (Lv. 90-99)")
	public static NpcStringId VERGE_OF_LIGHT_AND_DARKNESS_LV_90_992;
	
	@ClientString(id = 530400, message = "You can use the old roll of paper to begin the quest.")
	public static NpcStringId YOU_CAN_USE_THE_OLD_ROLL_OF_PAPER_TO_BEGIN_THE_QUEST;
	
	@ClientString(id = 530401, message = "For the Forgotten Heroes (Lv. 90-99)")
	public static NpcStringId FOR_THE_FORGOTTEN_HEROES_LV_90_99;
	
	@ClientString(id = 530402, message = "For the Forgotten Heroes (Lv. 90-99) (In Progress)")
	public static NpcStringId FOR_THE_FORGOTTEN_HEROES_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 530403, message = "For the Forgotten Heroes (Lv. 90-99) (Completed)")
	public static NpcStringId FOR_THE_FORGOTTEN_HEROES_LV_90_99_COMPLETED;
	
	@ClientString(id = 530404, message = "For the Forgotten Heroes (Lv. 90-99)")
	public static NpcStringId FOR_THE_FORGOTTEN_HEROES_LV_90_992;
	
	@ClientString(id = 530501, message = "Unstoppable Void Efforts (Lv. 88-99)")
	public static NpcStringId UNSTOPPABLE_VOID_EFFORTS_LV_88_99;
	
	@ClientString(id = 530502, message = "Unstoppable Void Efforts (Lv. 88-99) (In Progress)")
	public static NpcStringId UNSTOPPABLE_VOID_EFFORTS_LV_88_99_IN_PROGRESS;
	
	@ClientString(id = 530503, message = "Unstoppable Void Efforts (Lv. 88-99) (Done)")
	public static NpcStringId UNSTOPPABLE_VOID_EFFORTS_LV_88_99_DONE;
	
	@ClientString(id = 530504, message = "Unstoppable Void Efforts (Lv. 88-99)")
	public static NpcStringId UNSTOPPABLE_VOID_EFFORTS_LV_88_992;
	
	@ClientString(id = 530601, message = "The Corrupt Leader (Lv. 90-99)")
	public static NpcStringId THE_CORRUPT_LEADER_LV_90_99;
	
	@ClientString(id = 530602, message = "The Corrupt Leader (Lv. 90-99) (In Progress)")
	public static NpcStringId THE_CORRUPT_LEADER_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 530603, message = "The Corrupt Leader (Lv. 90-99) (Done)")
	public static NpcStringId THE_CORRUPT_LEADER_LV_90_99_DONE;
	
	@ClientString(id = 530604, message = "The Corrupt Leader (Lv. 90-99)")
	public static NpcStringId THE_CORRUPT_LEADER_LV_90_992;
	
	@ClientString(id = 530701, message = "The Corrupt Leader, His Truth (Lv. 90-99)")
	public static NpcStringId THE_CORRUPT_LEADER_HIS_TRUTH_LV_90_99;
	
	@ClientString(id = 530702, message = "The Corrupt Leader, His Truth (Lv. 90-99) (In Progress)")
	public static NpcStringId THE_CORRUPT_LEADER_HIS_TRUTH_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 530703, message = "The Corrupt Leader, His Truth (Lv. 90-99) (Done)")
	public static NpcStringId THE_CORRUPT_LEADER_HIS_TRUTH_LV_90_99_DONE;
	
	@ClientString(id = 530704, message = "The Corrupt Leader, His Truth (Lv. 90-99)")
	public static NpcStringId THE_CORRUPT_LEADER_HIS_TRUTH_LV_90_992;
	
	@ClientString(id = 530801, message = "To Not Be Forgotten (Lv. 55-59)")
	public static NpcStringId TO_NOT_BE_FORGOTTEN_LV_55_59;
	
	@ClientString(id = 530802, message = "To Not Be Forgotten (Lv. 55-59) (In Progress)")
	public static NpcStringId TO_NOT_BE_FORGOTTEN_LV_55_59_IN_PROGRESS;
	
	@ClientString(id = 530803, message = "To Not Be Forgotten (Lv. 55-59) (Done)")
	public static NpcStringId TO_NOT_BE_FORGOTTEN_LV_55_59_DONE;
	
	@ClientString(id = 530804, message = "To Not Be Forgotten (Lv. 55-59)")
	public static NpcStringId TO_NOT_BE_FORGOTTEN_LV_55_592;
	
	@ClientString(id = 530901, message = "Mysterious Letter (Lv. 65-69)")
	public static NpcStringId MYSTERIOUS_LETTER_LV_65_69;
	
	@ClientString(id = 530902, message = "Mysterious Letter (Lv. 65-69) (In Progress)")
	public static NpcStringId MYSTERIOUS_LETTER_LV_65_69_IN_PROGRESS;
	
	@ClientString(id = 530903, message = "Mysterious Letter (Lv. 65-69) (Done)")
	public static NpcStringId MYSTERIOUS_LETTER_LV_65_69_DONE;
	
	@ClientString(id = 530904, message = "Mysterious Letter (Lv. 65-69)")
	public static NpcStringId MYSTERIOUS_LETTER_LV_65_692;
	
	@ClientString(id = 531001, message = "Twisted Creation Tree (Lv. 90-99)")
	public static NpcStringId TWISTED_CREATION_TREE_LV_90_99;
	
	@ClientString(id = 531002, message = "Twisted Creation Tree (Lv. 90-99) (In Progress)")
	public static NpcStringId TWISTED_CREATION_TREE_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 531003, message = "Twisted Creation Tree (Lv. 90-99) (Done)")
	public static NpcStringId TWISTED_CREATION_TREE_LV_90_99_DONE;
	
	@ClientString(id = 531004, message = "Twisted Creation Tree (Lv. 90-99)")
	public static NpcStringId TWISTED_CREATION_TREE_LV_90_992;
	
	@ClientString(id = 531101, message = "Day of Rest Is Over (Lv. 90-99)")
	public static NpcStringId DAY_OF_REST_IS_OVER_LV_90_99;
	
	@ClientString(id = 531102, message = "Day of Rest Is Over (Lv. 90-99) (In Progress)")
	public static NpcStringId DAY_OF_REST_IS_OVER_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 531103, message = "Day of Rest Is Over (Lv. 90-99) (Done)")
	public static NpcStringId DAY_OF_REST_IS_OVER_LV_90_99_DONE;
	
	@ClientString(id = 531104, message = "Day of Rest Is Over (Lv. 90-99)")
	public static NpcStringId DAY_OF_REST_IS_OVER_LV_90_992;
	
	@ClientString(id = 531201, message = "Abandoned Creation of Gods (Lv. 90-99)")
	public static NpcStringId ABANDONED_CREATION_OF_GODS_LV_90_99;
	
	@ClientString(id = 531202, message = "Abandoned Creation of Gods (Lv. 90-99) (In Progress)")
	public static NpcStringId ABANDONED_CREATION_OF_GODS_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 531203, message = "Abandoned Creation of Gods (Lv. 90-99) (Done)")
	public static NpcStringId ABANDONED_CREATION_OF_GODS_LV_90_99_DONE;
	
	@ClientString(id = 531204, message = "Abandoned Creation of Gods (Lv. 90-99)")
	public static NpcStringId ABANDONED_CREATION_OF_GODS_LV_90_992;
	
	@ClientString(id = 531501, message = "To the Prison of Darkness (Lv. 90-99)")
	public static NpcStringId TO_THE_PRISON_OF_DARKNESS_LV_90_99;
	
	@ClientString(id = 531502, message = "To the Prison of Darkness (Lv. 90-99) (In Progress)")
	public static NpcStringId TO_THE_PRISON_OF_DARKNESS_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 531503, message = "To the Prison of Darkness (Lv. 90-99) (Done)")
	public static NpcStringId TO_THE_PRISON_OF_DARKNESS_LV_90_99_DONE;
	
	@ClientString(id = 531504, message = "To the Prison of Darkness (Lv. 90-99)")
	public static NpcStringId TO_THE_PRISON_OF_DARKNESS_LV_90_992;
	
	@ClientString(id = 531601, message = "Undecaying Memory of the Past (Lv. 90-99)")
	public static NpcStringId UNDECAYING_MEMORY_OF_THE_PAST_LV_90_99;
	
	@ClientString(id = 531602, message = "Undecaying Memory of the Past (Lv. 90-99) (In Progress)")
	public static NpcStringId UNDECAYING_MEMORY_OF_THE_PAST_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 531603, message = "Undecaying Memory of the Past (Lv. 90-99) (Done)")
	public static NpcStringId UNDECAYING_MEMORY_OF_THE_PAST_LV_90_99_DONE;
	
	@ClientString(id = 531604, message = "Undecaying Memory of the Past (Lv. 90-99)")
	public static NpcStringId UNDECAYING_MEMORY_OF_THE_PAST_LV_90_992;
	
	@ClientString(id = 531701, message = "Witch of Orbis (Lv. 95-99)")
	public static NpcStringId WITCH_OF_ORBIS_LV_95_99;
	
	@ClientString(id = 531702, message = "Witch of Orbis (Lv. 95-99) (In Progress)")
	public static NpcStringId WITCH_OF_ORBIS_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 531703, message = "Witch of Orbis (Lv. 95-99) (Done)")
	public static NpcStringId WITCH_OF_ORBIS_LV_95_99_DONE;
	
	@ClientString(id = 531704, message = "Witch of Orbis (Lv. 95-99)")
	public static NpcStringId WITCH_OF_ORBIS_LV_95_992;
	
	@ClientString(id = 531801, message = "Decaying Darkness (Lv. 95-99)")
	public static NpcStringId DECAYING_DARKNESS_LV_95_99;
	
	@ClientString(id = 531802, message = "Decaying Darkness (Lv. 95-99) (In progress)")
	public static NpcStringId DECAYING_DARKNESS_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 531803, message = "Decaying Darkness (Lv. 95-99) (Done)")
	public static NpcStringId DECAYING_DARKNESS_LV_95_99_DONE;
	
	@ClientString(id = 531804, message = "Decaying Darkness (Lv. 95-99)")
	public static NpcStringId DECAYING_DARKNESS_LV_95_992;
	
	@ClientString(id = 532001, message = "Let's Go to the Central Square (Lv. 1-20)")
	public static NpcStringId LET_S_GO_TO_THE_CENTRAL_SQUARE_LV_1_20;
	
	@ClientString(id = 532002, message = "Let's Go to the Central Square (Lv. 1-20) (In Progress)")
	public static NpcStringId LET_S_GO_TO_THE_CENTRAL_SQUARE_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532003, message = "Let's Go to the Central Square (Lv. 1-20) (Done)")
	public static NpcStringId LET_S_GO_TO_THE_CENTRAL_SQUARE_LV_1_20_DONE;
	
	@ClientString(id = 532004, message = "Let's Go to the Central Square (Lv. 1-20)")
	public static NpcStringId LET_S_GO_TO_THE_CENTRAL_SQUARE_LV_1_202;
	
	@ClientString(id = 532101, message = "Qualifications of the Seeker (Lv. 1-20)")
	public static NpcStringId QUALIFICATIONS_OF_THE_SEEKER_LV_1_20;
	
	@ClientString(id = 532102, message = "Qualifications of the Seeker (Lv. 1-20) (In Progress)")
	public static NpcStringId QUALIFICATIONS_OF_THE_SEEKER_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532103, message = "Qualifications of the Seeker (Lv. 1-20) (Done)")
	public static NpcStringId QUALIFICATIONS_OF_THE_SEEKER_LV_1_20_DONE;
	
	@ClientString(id = 532104, message = "Qualifications of the Seeker (Lv. 1-20)")
	public static NpcStringId QUALIFICATIONS_OF_THE_SEEKER_LV_1_202;
	
	@ClientString(id = 532201, message = "Searching for the Mysterious Power (Lv. 1-20)")
	public static NpcStringId SEARCHING_FOR_THE_MYSTERIOUS_POWER_LV_1_20;
	
	@ClientString(id = 532202, message = "Searching for the Mysterious Power (Lv. 1-20) (In Progress)")
	public static NpcStringId SEARCHING_FOR_THE_MYSTERIOUS_POWER_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532203, message = "Searching for the Mysterious Power (Lv. 1-20) (Done)")
	public static NpcStringId SEARCHING_FOR_THE_MYSTERIOUS_POWER_LV_1_20_DONE;
	
	@ClientString(id = 532204, message = "Searching for the Mysterious Power (Lv. 1-20)")
	public static NpcStringId SEARCHING_FOR_THE_MYSTERIOUS_POWER_LV_1_202;
	
	@ClientString(id = 532301, message = "Train Like It's Real (Lv. 1-20)")
	public static NpcStringId TRAIN_LIKE_IT_S_REAL_LV_1_20;
	
	@ClientString(id = 532302, message = "Train Like It's Real (Lv. 1-20) (In Progress)")
	public static NpcStringId TRAIN_LIKE_IT_S_REAL_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532303, message = "Train Like It's Real (Lv. 1-20) (Complete)")
	public static NpcStringId TRAIN_LIKE_IT_S_REAL_LV_1_20_COMPLETE;
	
	@ClientString(id = 532304, message = "Train Like It's Real (Lv. 1-20)")
	public static NpcStringId TRAIN_LIKE_IT_S_REAL_LV_1_202;
	
	@ClientString(id = 532311, message = "Eliminate the Training Golem")
	public static NpcStringId ELIMINATE_THE_TRAINING_GOLEM;
	
	@ClientString(id = 532312, message = "Eliminate the Training Golem")
	public static NpcStringId ELIMINATE_THE_TRAINING_GOLEM2;
	
	@ClientString(id = 532331, message = "Help the guards take down the monsters in the underground training facility.")
	public static NpcStringId HELP_THE_GUARDS_TAKE_DOWN_THE_MONSTERS_IN_THE_UNDERGROUND_TRAINING_FACILITY;
	
	@ClientString(id = 532332, message = "Talk to Aymen to exit the underground training facility.")
	public static NpcStringId TALK_TO_AYMEN_TO_EXIT_THE_UNDERGROUND_TRAINING_FACILITY;
	
	@ClientString(id = 532333, message = "Return to Shannon at Talking Island Village's basic training field.")
	public static NpcStringId RETURN_TO_SHANNON_AT_TALKING_ISLAND_VILLAGE_S_BASIC_TRAINING_FIELD;
	
	@ClientString(id = 532401, message = "Finding Magister Gallint (Lv. 1-20)")
	public static NpcStringId FINDING_MAGISTER_GALLINT_LV_1_20;
	
	@ClientString(id = 532402, message = "Finding Magister Gallint (Lv. 1-20) (In Progress)")
	public static NpcStringId FINDING_MAGISTER_GALLINT_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532403, message = "Finding Magister Gallint (Lv. 1-20) (Done)")
	public static NpcStringId FINDING_MAGISTER_GALLINT_LV_1_20_DONE;
	
	@ClientString(id = 532404, message = "Finding Magister Gallint (Lv. 1-20)")
	public static NpcStringId FINDING_MAGISTER_GALLINT_LV_1_202;
	
	@ClientString(id = 532501, message = "Searching for New Power (Lv. 1-20)")
	public static NpcStringId SEARCHING_FOR_NEW_POWER_LV_1_20;
	
	@ClientString(id = 532502, message = "Searching for New Power (Lv. 1-20) (In Progress)")
	public static NpcStringId SEARCHING_FOR_NEW_POWER_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532503, message = "Searching for New Power (Lv. 1-20) (Done)")
	public static NpcStringId SEARCHING_FOR_NEW_POWER_LV_1_20_DONE;
	
	@ClientString(id = 532504, message = "Searching for New Power (Lv. 1-20)")
	public static NpcStringId SEARCHING_FOR_NEW_POWER_LV_1_202;
	
	@ClientString(id = 532601, message = "Respect Your Elders! (Lv. 1-20)")
	public static NpcStringId RESPECT_YOUR_ELDERS_LV_1_20;
	
	@ClientString(id = 532602, message = "Respect Your Elders! (Lv. 1-20) (In Progress)")
	public static NpcStringId RESPECT_YOUR_ELDERS_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532603, message = "Respect Your Elders! (Lv. 1-20) (Done)")
	public static NpcStringId RESPECT_YOUR_ELDERS_LV_1_20_DONE;
	
	@ClientString(id = 532604, message = "Respect Your Elders! (Lv. 1-20)")
	public static NpcStringId RESPECT_YOUR_ELDERS_LV_1_202;
	
	@ClientString(id = 532701, message = "Intruder Who Wants the Book of Giants (Lv. 1-20)")
	public static NpcStringId INTRUDER_WHO_WANTS_THE_BOOK_OF_GIANTS_LV_1_20;
	
	@ClientString(id = 532702, message = "Intruder Who Wants the Book of Giants (Lv. 1-20) (In Progress)")
	public static NpcStringId INTRUDER_WHO_WANTS_THE_BOOK_OF_GIANTS_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532703, message = "Intruder Who Wants the Book of Giants (Lv. 1-20) (Done)")
	public static NpcStringId INTRUDER_WHO_WANTS_THE_BOOK_OF_GIANTS_LV_1_20_DONE;
	
	@ClientString(id = 532704, message = "Intruder Who Wants the Book of Giants (Lv. 1-20)")
	public static NpcStringId INTRUDER_WHO_WANTS_THE_BOOK_OF_GIANTS_LV_1_202;
	
	@ClientString(id = 532731, message = "Among the 4 bookshelves, find the one containing a volume called 'The War of Gods and Giants'.")
	public static NpcStringId AMONG_THE_4_BOOKSHELVES_FIND_THE_ONE_CONTAINING_A_VOLUME_CALLED_THE_WAR_OF_GODS_AND_GIANTS;
	
	@ClientString(id = 532732, message = "Talk to Toyron to return to the museum lobby.")
	public static NpcStringId TALK_TO_TOYRON_TO_RETURN_TO_THE_MUSEUM_LOBBY;
	
	@ClientString(id = 532801, message = "Request to Seal the Evil Fragment (Lv. 1-20)")
	public static NpcStringId REQUEST_TO_SEAL_THE_EVIL_FRAGMENT_LV_1_20;
	
	@ClientString(id = 532802, message = "Request to Seal the Evil Fragment (Lv. 1-20) (In Progress)")
	public static NpcStringId REQUEST_TO_SEAL_THE_EVIL_FRAGMENT_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532803, message = "Request to Seal the Evil Fragment (Lv. 1-20) (Done)")
	public static NpcStringId REQUEST_TO_SEAL_THE_EVIL_FRAGMENT_LV_1_20_DONE;
	
	@ClientString(id = 532804, message = "Request to Seal the Evil Fragment (Lv. 1-20)")
	public static NpcStringId REQUEST_TO_SEAL_THE_EVIL_FRAGMENT_LV_1_202;
	
	@ClientString(id = 532901, message = "Backup Seekers (Lv. 1-20)")
	public static NpcStringId BACKUP_SEEKERS_LV_1_20;
	
	@ClientString(id = 532902, message = "Backup Seekers (Lv. 1-20) (In Progress)")
	public static NpcStringId BACKUP_SEEKERS_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 532903, message = "Backup Seekers (Lv. 1-20) (Done)")
	public static NpcStringId BACKUP_SEEKERS_LV_1_20_DONE;
	
	@ClientString(id = 532904, message = "Backup Seekers (Lv. 1-20)")
	public static NpcStringId BACKUP_SEEKERS_LV_1_202;
	
	@ClientString(id = 533001, message = "To the Ruins of Ye Sagira (Lv. 8-20)")
	public static NpcStringId TO_THE_RUINS_OF_YE_SAGIRA_LV_8_20;
	
	@ClientString(id = 533002, message = "To the Ruins of Ye Sagira (Lv. 8-20) (In Progress)")
	public static NpcStringId TO_THE_RUINS_OF_YE_SAGIRA_LV_8_20_IN_PROGRESS;
	
	@ClientString(id = 533003, message = "To the Ruins of Ye Sagira (Lv. 8-20) (Done)")
	public static NpcStringId TO_THE_RUINS_OF_YE_SAGIRA_LV_8_20_DONE;
	
	@ClientString(id = 533004, message = "To the Ruins of Ye Sagira (Lv. 8-20)")
	public static NpcStringId TO_THE_RUINS_OF_YE_SAGIRA_LV_8_202;
	
	@ClientString(id = 533101, message = "Start of Fate (Lv. 18-99)")
	public static NpcStringId START_OF_FATE_LV_18_99;
	
	@ClientString(id = 533102, message = "Start of Fate (Lv. 18-99) (In Progress)")
	public static NpcStringId START_OF_FATE_LV_18_99_IN_PROGRESS;
	
	@ClientString(id = 533103, message = "Start of Fate (Lv. 18-99) (Done)")
	public static NpcStringId START_OF_FATE_LV_18_99_DONE;
	
	@ClientString(id = 533104, message = "Start of Fate (Lv. 18-99)")
	public static NpcStringId START_OF_FATE_LV_18_992;
	
	@ClientString(id = 533131, message = "Go to the entrance of the Ruins of Ye Sagira through Gatekeeper Milia in Talking Island Village.")
	public static NpcStringId GO_TO_THE_ENTRANCE_OF_THE_RUINS_OF_YE_SAGIRA_THROUGH_GATEKEEPER_MILIA_IN_TALKING_ISLAND_VILLAGE;
	
	@ClientString(id = 533201, message = "Tough Road (Lv. 20-40)")
	public static NpcStringId TOUGH_ROAD_LV_20_40;
	
	@ClientString(id = 533202, message = "Tough Road (Lv. 20-40) (In Progress)")
	public static NpcStringId TOUGH_ROAD_LV_20_40_IN_PROGRESS;
	
	@ClientString(id = 533203, message = "Tough Road (Lv. 20-40) (Done)")
	public static NpcStringId TOUGH_ROAD_LV_20_40_DONE;
	
	@ClientString(id = 533204, message = "Tough Road (Lv. 20-40)")
	public static NpcStringId TOUGH_ROAD_LV_20_402;
	
	@ClientString(id = 533301, message = "Disappeared Sakum (Lv. 18-40)")
	public static NpcStringId DISAPPEARED_SAKUM_LV_18_40;
	
	@ClientString(id = 533302, message = "Disappeared Sakum (Lv. 18-40) (In Progress)")
	public static NpcStringId DISAPPEARED_SAKUM_LV_18_40_IN_PROGRESS;
	
	@ClientString(id = 533303, message = "Disappeared Sakum (Lv. 18-40) (Done)")
	public static NpcStringId DISAPPEARED_SAKUM_LV_18_40_DONE;
	
	@ClientString(id = 533304, message = "Disappeared Sakum (Lv. 18-40)")
	public static NpcStringId DISAPPEARED_SAKUM_LV_18_402;
	
	@ClientString(id = 533401, message = "Reporting the Status of the Windmill Hill (Lv. 22-40)")
	public static NpcStringId REPORTING_THE_STATUS_OF_THE_WINDMILL_HILL_LV_22_40;
	
	@ClientString(id = 533402, message = "Reporting the Status of the Windmill Hill (Lv. 22-40) (In Progress)")
	public static NpcStringId REPORTING_THE_STATUS_OF_THE_WINDMILL_HILL_LV_22_40_IN_PROGRESS;
	
	@ClientString(id = 533403, message = "Reporting the Status of the Windmill Hill (Lv. 22-40) (Done)")
	public static NpcStringId REPORTING_THE_STATUS_OF_THE_WINDMILL_HILL_LV_22_40_DONE;
	
	@ClientString(id = 533404, message = "Reporting the Status of the Windmill Hill (Lv. 22-40)")
	public static NpcStringId REPORTING_THE_STATUS_OF_THE_WINDMILL_HILL_LV_22_402;
	
	@ClientString(id = 533501, message = "Request to Find Sakum (Lv. 23-40)")
	public static NpcStringId REQUEST_TO_FIND_SAKUM_LV_23_40;
	
	@ClientString(id = 533502, message = "Request to Find Sakum (Lv. 23-40) (In Progress)")
	public static NpcStringId REQUEST_TO_FIND_SAKUM_LV_23_40_IN_PROGRESS;
	
	@ClientString(id = 533503, message = "Request to Find Sakum (Lv. 23-40) (Done)")
	public static NpcStringId REQUEST_TO_FIND_SAKUM_LV_23_40_DONE;
	
	@ClientString(id = 533504, message = "Request to Find Sakum (Lv. 23-40)")
	public static NpcStringId REQUEST_TO_FIND_SAKUM_LV_23_402;
	
	@ClientString(id = 533601, message = "Divided Sakum, Kanilov (Lv. 27-40)")
	public static NpcStringId DIVIDED_SAKUM_KANILOV_LV_27_40;
	
	@ClientString(id = 533602, message = "Divided Sakum, Kanilov (Lv. 27-40) (In Progress)")
	public static NpcStringId DIVIDED_SAKUM_KANILOV_LV_27_40_IN_PROGRESS;
	
	@ClientString(id = 533603, message = "Divided Sakum, Kanilov (Lv. 27-40) (Done)")
	public static NpcStringId DIVIDED_SAKUM_KANILOV_LV_27_40_DONE;
	
	@ClientString(id = 533604, message = "Divided Sakum, Kanilov (Lv. 27-40)")
	public static NpcStringId DIVIDED_SAKUM_KANILOV_LV_27_402;
	
	@ClientString(id = 533701, message = "Sakum's Influence (Lv. 28-40)")
	public static NpcStringId SAKUM_S_INFLUENCE_LV_28_40;
	
	@ClientString(id = 533702, message = "Sakum's Influence (Lv. 28-40) (In Progress)")
	public static NpcStringId SAKUM_S_INFLUENCE_LV_28_40_IN_PROGRESS;
	
	@ClientString(id = 533703, message = "Sakum's Influence (Lv. 28-40) (Done)")
	public static NpcStringId SAKUM_S_INFLUENCE_LV_28_40_DONE;
	
	@ClientString(id = 533704, message = "Sakum's Influence (Lv. 28-40)")
	public static NpcStringId SAKUM_S_INFLUENCE_LV_28_402;
	
	@ClientString(id = 533801, message = "Seize Your Destiny (Lv. 85-99)")
	public static NpcStringId SEIZE_YOUR_DESTINY_LV_85_99;
	
	@ClientString(id = 533802, message = "Seize Your Destiny (Lv. 85-99) (In Progress)")
	public static NpcStringId SEIZE_YOUR_DESTINY_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 533803, message = "Seize Your Destiny (Lv. 85-99) (Done)")
	public static NpcStringId SEIZE_YOUR_DESTINY_LV_85_99_DONE;
	
	@ClientString(id = 533804, message = "Seize Your Destiny (Lv. 85-99)")
	public static NpcStringId SEIZE_YOUR_DESTINY_LV_85_992;
	
	@ClientString(id = 533901, message = "Fighting the Forgotten (Lv. 85-99)")
	public static NpcStringId FIGHTING_THE_FORGOTTEN_LV_85_99;
	
	@ClientString(id = 533902, message = "Fighting the Forgotten (Lv. 85-99) (In Progress)")
	public static NpcStringId FIGHTING_THE_FORGOTTEN_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 533903, message = "Fighting the Forgotten (Lv. 85-99) (Complete)")
	public static NpcStringId FIGHTING_THE_FORGOTTEN_LV_85_99_COMPLETE;
	
	@ClientString(id = 533904, message = "Fighting the Forgotten (Lv. 85-99)")
	public static NpcStringId FIGHTING_THE_FORGOTTEN_LV_85_992;
	
	@ClientString(id = 533911, message = "Conversation with Hadel")
	public static NpcStringId CONVERSATION_WITH_HADEL;
	
	@ClientString(id = 533912, message = "Eliminating the Ancient Ghosts")
	public static NpcStringId ELIMINATING_THE_ANCIENT_GHOSTS;
	
	@ClientString(id = 534001, message = "Revived Power of the Giant")
	public static NpcStringId REVIVED_POWER_OF_THE_GIANT;
	
	@ClientString(id = 534002, message = "Revived Power of the Giant (In Progress)")
	public static NpcStringId REVIVED_POWER_OF_THE_GIANT_IN_PROGRESS;
	
	@ClientString(id = 534003, message = "Revived Power of the Giant (Done)")
	public static NpcStringId REVIVED_POWER_OF_THE_GIANT_DONE;
	
	@ClientString(id = 534004, message = "Revived Power of the Giant")
	public static NpcStringId REVIVED_POWER_OF_THE_GIANT2;
	
	@ClientString(id = 534101, message = "Day of Fate - Human Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_HUMAN_FATE_LV_76_99;
	
	@ClientString(id = 534102, message = "Day of Fate - Human Fate (Lv. 76-99) (In Progress)")
	public static NpcStringId DAY_OF_FATE_HUMAN_FATE_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 534103, message = "Day of Fate - Human Fate (Lv. 76-99) (Done)")
	public static NpcStringId DAY_OF_FATE_HUMAN_FATE_LV_76_99_DONE;
	
	@ClientString(id = 534104, message = "Day of Fate - Human Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_HUMAN_FATE_LV_76_992;
	
	@ClientString(id = 534201, message = "Day of Fate - Elven Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_ELVEN_FATE_LV_76_99;
	
	@ClientString(id = 534202, message = "Day of Fate - Elven Fate (Lv. 76-99) (In Progress)")
	public static NpcStringId DAY_OF_FATE_ELVEN_FATE_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 534203, message = "Day of Fate - Elven Fate (Lv. 76-99) (Done)")
	public static NpcStringId DAY_OF_FATE_ELVEN_FATE_LV_76_99_DONE;
	
	@ClientString(id = 534204, message = "Day of Fate - Elven Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_ELVEN_FATE_LV_76_992;
	
	@ClientString(id = 534301, message = "Day of Fate - Dark Elven Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_DARK_ELVEN_FATE_LV_76_99;
	
	@ClientString(id = 534302, message = "Day of Fate - Dark Elven Fate (Lv. 76-99) (In Progress)")
	public static NpcStringId DAY_OF_FATE_DARK_ELVEN_FATE_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 534303, message = "Day of Fate - Dark Elven Fate (Lv. 76-99) (Done)")
	public static NpcStringId DAY_OF_FATE_DARK_ELVEN_FATE_LV_76_99_DONE;
	
	@ClientString(id = 534304, message = "Day of Fate - Dark Elven Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_DARK_ELVEN_FATE_LV_76_992;
	
	@ClientString(id = 534401, message = "Day of Fate - Orc's Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_ORC_S_FATE_LV_76_99;
	
	@ClientString(id = 534402, message = "Day of Fate - Orc's Fate (Lv. 76-99) (In Progress)")
	public static NpcStringId DAY_OF_FATE_ORC_S_FATE_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 534403, message = "Day of Fate - Orc's Fate (Lv. 76-99) (Done)")
	public static NpcStringId DAY_OF_FATE_ORC_S_FATE_LV_76_99_DONE;
	
	@ClientString(id = 534404, message = "Day of Fate - Orc's Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_ORC_S_FATE_LV_76_992;
	
	@ClientString(id = 534501, message = "Day of Fate - Dwarf's Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_DWARF_S_FATE_LV_76_99;
	
	@ClientString(id = 534502, message = "Day of Fate - Dwarf's Fate (Lv. 76-99) (In Progress)")
	public static NpcStringId DAY_OF_FATE_DWARF_S_FATE_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 534503, message = "Day of Fate - Dwarf's Fate (Lv. 76-99) (Done)")
	public static NpcStringId DAY_OF_FATE_DWARF_S_FATE_LV_76_99_DONE;
	
	@ClientString(id = 534504, message = "Day of Fate - Dwarf's Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_DWARF_S_FATE_LV_76_992;
	
	@ClientString(id = 534601, message = "Day of Fate - Kamael's Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_KAMAEL_S_FATE_LV_76_99;
	
	@ClientString(id = 534602, message = "Day of Fate - Kamael's Fate (Lv. 76-99) (In Progress)")
	public static NpcStringId DAY_OF_FATE_KAMAEL_S_FATE_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 534603, message = "Day of Fate - Kamael's Fate (Lv. 76-99) (Done)")
	public static NpcStringId DAY_OF_FATE_KAMAEL_S_FATE_LV_76_99_DONE;
	
	@ClientString(id = 534604, message = "Day of Fate - Kamael's Fate (Lv. 76-99)")
	public static NpcStringId DAY_OF_FATE_KAMAEL_S_FATE_LV_76_992;
	
	@ClientString(id = 534701, message = "Seven Sign, Destruction Came about like that")
	public static NpcStringId SEVEN_SIGN_DESTRUCTION_CAME_ABOUT_LIKE_THAT;
	
	@ClientString(id = 534702, message = "Seven Sign, Destruction Came about like that (In Progress)")
	public static NpcStringId SEVEN_SIGN_DESTRUCTION_CAME_ABOUT_LIKE_THAT_IN_PROGRESS;
	
	@ClientString(id = 534703, message = "Seven Sign, Destruction Came about like that (Done)")
	public static NpcStringId SEVEN_SIGN_DESTRUCTION_CAME_ABOUT_LIKE_THAT_DONE;
	
	@ClientString(id = 534704, message = "Seven Sign, Destruction Came about like that")
	public static NpcStringId SEVEN_SIGN_DESTRUCTION_CAME_ABOUT_LIKE_THAT2;
	
	@ClientString(id = 534801, message = "Seven Sign, Shadow that swallowed the moon")
	public static NpcStringId SEVEN_SIGN_SHADOW_THAT_SWALLOWED_THE_MOON;
	
	@ClientString(id = 534802, message = "Seven Sign, Shadow that swallowed the moon (In Progress)")
	public static NpcStringId SEVEN_SIGN_SHADOW_THAT_SWALLOWED_THE_MOON_IN_PROGRESS;
	
	@ClientString(id = 534803, message = "Seven Sign, Shadow that swallowed the moon (Done)")
	public static NpcStringId SEVEN_SIGN_SHADOW_THAT_SWALLOWED_THE_MOON_DONE;
	
	@ClientString(id = 534804, message = "Seven Sign, Shadow that swallowed the moon")
	public static NpcStringId SEVEN_SIGN_SHADOW_THAT_SWALLOWED_THE_MOON2;
	
	@ClientString(id = 534901, message = "Seven Signs: Dusky Dawn")
	public static NpcStringId SEVEN_SIGNS_DUSKY_DAWN;
	
	@ClientString(id = 534902, message = "Seven Signs: Dusky Dawn (In Progress)")
	public static NpcStringId SEVEN_SIGNS_DUSKY_DAWN_IN_PROGRESS;
	
	@ClientString(id = 534903, message = "Seven Signs: Dusky Dawn (Done)")
	public static NpcStringId SEVEN_SIGNS_DUSKY_DAWN_DONE;
	
	@ClientString(id = 534904, message = "Seven Signs: Dusky Dawn")
	public static NpcStringId SEVEN_SIGNS_DUSKY_DAWN2;
	
	@ClientString(id = 535001, message = "Mother of Monstrosities")
	public static NpcStringId MOTHER_OF_MONSTROSITIES;
	
	@ClientString(id = 535002, message = "Mother of Monstrosities (In Progress)")
	public static NpcStringId MOTHER_OF_MONSTROSITIES_IN_PROGRESS;
	
	@ClientString(id = 535003, message = "Mother of Monstrosities (Done)")
	public static NpcStringId MOTHER_OF_MONSTROSITIES_DONE;
	
	@ClientString(id = 535004, message = "Mother of Monstrosities")
	public static NpcStringId MOTHER_OF_MONSTROSITIES2;
	
	@ClientString(id = 535101, message = "Owner of Hall (Lv. 95-99)")
	public static NpcStringId OWNER_OF_HALL_LV_95_99;
	
	@ClientString(id = 535102, message = "Owner of Hall (Lv. 95-99) (In Progress)")
	public static NpcStringId OWNER_OF_HALL_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 535103, message = "Owner of Hall (Lv. 95-99) (Done)")
	public static NpcStringId OWNER_OF_HALL_LV_95_99_DONE;
	
	@ClientString(id = 535104, message = "Owner of Hall (Lv. 95-99)")
	public static NpcStringId OWNER_OF_HALL_LV_95_992;
	
	@ClientString(id = 535201, message = "Legacy of Cruma Tower (Lv. 38-99)")
	public static NpcStringId LEGACY_OF_CRUMA_TOWER_LV_38_99;
	
	@ClientString(id = 535202, message = "Legacy of Cruma Tower (Lv. 38-99) (In Progress)")
	public static NpcStringId LEGACY_OF_CRUMA_TOWER_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 535203, message = "Legacy of Cruma Tower (Lv. 38-99) (Done)")
	public static NpcStringId LEGACY_OF_CRUMA_TOWER_LV_38_99_DONE;
	
	@ClientString(id = 535204, message = "Legacy of Cruma Tower (Lv. 38-99)")
	public static NpcStringId LEGACY_OF_CRUMA_TOWER_LV_38_992;
	
	@ClientString(id = 535301, message = "Certification of Value (Lv. 48-99)")
	public static NpcStringId CERTIFICATION_OF_VALUE_LV_48_99;
	
	@ClientString(id = 535302, message = "Certification of Value (Lv. 48-99) (In Progress)")
	public static NpcStringId CERTIFICATION_OF_VALUE_LV_48_99_IN_PROGRESS;
	
	@ClientString(id = 535303, message = "Certification of Value (Lv. 48-99) (Done)")
	public static NpcStringId CERTIFICATION_OF_VALUE_LV_48_99_DONE;
	
	@ClientString(id = 535304, message = "Certification of Value (Lv. 48-99)")
	public static NpcStringId CERTIFICATION_OF_VALUE_LV_48_992;
	
	@ClientString(id = 535401, message = "Resurrected Owner of Hall (Lv. 95-99)")
	public static NpcStringId RESURRECTED_OWNER_OF_HALL_LV_95_99;
	
	@ClientString(id = 535402, message = "Resurrected Owner of Hall (Lv. 95-99) (In Progress)")
	public static NpcStringId RESURRECTED_OWNER_OF_HALL_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 535403, message = "Resurrected Owner of Hall (Lv. 95-99) (Done)")
	public static NpcStringId RESURRECTED_OWNER_OF_HALL_LV_95_99_DONE;
	
	@ClientString(id = 535404, message = "Resurrected Owner of Hall (Lv. 95-99)")
	public static NpcStringId RESURRECTED_OWNER_OF_HALL_LV_95_992;
	
	@ClientString(id = 535701, message = "Altar of Blood that Awakens Destruction (Lv. 95-99)")
	public static NpcStringId ALTAR_OF_BLOOD_THAT_AWAKENS_DESTRUCTION_LV_95_99;
	
	@ClientString(id = 535702, message = "Altar of Blood that Awakens Destruction (Lv. 95-99) (In Progress)")
	public static NpcStringId ALTAR_OF_BLOOD_THAT_AWAKENS_DESTRUCTION_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 535703, message = "Altar of Blood that Awakens Destruction (Lv. 95-99) (Done)")
	public static NpcStringId ALTAR_OF_BLOOD_THAT_AWAKENS_DESTRUCTION_LV_95_99_DONE;
	
	@ClientString(id = 535704, message = "Altar of Blood that Awakens Destruction (Lv. 95-99)")
	public static NpcStringId ALTAR_OF_BLOOD_THAT_AWAKENS_DESTRUCTION_LV_95_992;
	
	@ClientString(id = 535801, message = "Divided Sakum, Poslof (Lv. 33-40)")
	public static NpcStringId DIVIDED_SAKUM_POSLOF_LV_33_40;
	
	@ClientString(id = 535802, message = "Divided Sakum, Poslof (Lv. 33-40) (In Progress)")
	public static NpcStringId DIVIDED_SAKUM_POSLOF_LV_33_40_IN_PROGRESS;
	
	@ClientString(id = 535803, message = "Divided Sakum, Poslof (Lv. 33-40) (In Progress)")
	public static NpcStringId DIVIDED_SAKUM_POSLOF_LV_33_40_IN_PROGRESS2;
	
	@ClientString(id = 535804, message = "Divided Sakum, Poslof (Lv. 33-40)")
	public static NpcStringId DIVIDED_SAKUM_POSLOF_LV_33_402;
	
	@ClientString(id = 535901, message = "Traces of Evil (Lv. 34-40)")
	public static NpcStringId TRACES_OF_EVIL_LV_34_40;
	
	@ClientString(id = 535902, message = "Traces of Evil (Lv. 34-40) (In Progress)")
	public static NpcStringId TRACES_OF_EVIL_LV_34_40_IN_PROGRESS;
	
	@ClientString(id = 535903, message = "Traces of Evil (Lv. 34-40) (Done)")
	public static NpcStringId TRACES_OF_EVIL_LV_34_40_DONE;
	
	@ClientString(id = 535904, message = "Traces of Evil (Lv. 34-40)")
	public static NpcStringId TRACES_OF_EVIL_LV_34_402;
	
	@ClientString(id = 536001, message = "Certification of Fate (Lv. 38-99)")
	public static NpcStringId CERTIFICATION_OF_FATE_LV_38_99;
	
	@ClientString(id = 536002, message = "Certification of Fate (Lv. 38-99) (In Progress)")
	public static NpcStringId CERTIFICATION_OF_FATE_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 536003, message = "Certification of Fate (Lv. 38-99) (Done)")
	public static NpcStringId CERTIFICATION_OF_FATE_LV_38_99_DONE;
	
	@ClientString(id = 536004, message = "Certification of Fate (Lv. 38-99)")
	public static NpcStringId CERTIFICATION_OF_FATE_LV_38_992;
	
	@ClientString(id = 536101, message = "Roles of the Seeker (Lv. 10-20)")
	public static NpcStringId ROLES_OF_THE_SEEKER_LV_10_20;
	
	@ClientString(id = 536102, message = "Roles of the Seeker (Lv. 10-20) (In Progress)")
	public static NpcStringId ROLES_OF_THE_SEEKER_LV_10_20_IN_PROGRESS;
	
	@ClientString(id = 536103, message = "Roles of the Seeker (Lv. 10-20) (Done)")
	public static NpcStringId ROLES_OF_THE_SEEKER_LV_10_20_DONE;
	
	@ClientString(id = 536104, message = "Roles of the Seeker (Lv. 10-20)")
	public static NpcStringId ROLES_OF_THE_SEEKER_LV_10_202;
	
	@ClientString(id = 536131, message = "Enter the Ruins of Ye Sagira through the Ye Sagira Teleport Device.")
	public static NpcStringId ENTER_THE_RUINS_OF_YE_SAGIRA_THROUGH_THE_YE_SAGIRA_TELEPORT_DEVICE;
	
	@ClientString(id = 536132, message = "Use the Ye Sagira Teleport Device to go to Exploration Area 1.")
	public static NpcStringId USE_THE_YE_SAGIRA_TELEPORT_DEVICE_TO_GO_TO_EXPLORATION_AREA_1;
	
	@ClientString(id = 536201, message = "Certification of the Seeker (Lv. 10-20)")
	public static NpcStringId CERTIFICATION_OF_THE_SEEKER_LV_10_20;
	
	@ClientString(id = 536202, message = "Certification of the Seeker (Lv. 10-20) (In Progress)")
	public static NpcStringId CERTIFICATION_OF_THE_SEEKER_LV_10_20_IN_PROGRESS;
	
	@ClientString(id = 536203, message = "Certification of the Seeker (Lv. 10-20) (Done)")
	public static NpcStringId CERTIFICATION_OF_THE_SEEKER_LV_10_20_DONE;
	
	@ClientString(id = 536204, message = "Certification of the Seeker (Lv. 10-20)")
	public static NpcStringId CERTIFICATION_OF_THE_SEEKER_LV_10_202;
	
	@ClientString(id = 536233, message = "Use the Ye Sagira Teleport Device to go to Exploration Area 2.")
	public static NpcStringId USE_THE_YE_SAGIRA_TELEPORT_DEVICE_TO_GO_TO_EXPLORATION_AREA_2;
	
	@ClientString(id = 536301, message = "Request of the Seeker (Lv. 12-20)")
	public static NpcStringId REQUEST_OF_THE_SEEKER_LV_12_20;
	
	@ClientString(id = 536302, message = "Request of the Seeker (Lv. 12-20) (In Progress)")
	public static NpcStringId REQUEST_OF_THE_SEEKER_LV_12_20_IN_PROGRESS;
	
	@ClientString(id = 536303, message = "Request of the Seeker (Lv. 12-20) (Done)")
	public static NpcStringId REQUEST_OF_THE_SEEKER_LV_12_20_DONE;
	
	@ClientString(id = 536304, message = "Request of the Seeker (Lv. 12-20)")
	public static NpcStringId REQUEST_OF_THE_SEEKER_LV_12_202;
	
	@ClientString(id = 536341, message = "Use the Ye Sagira Teleport Device to go to Exploration Area 3.")
	public static NpcStringId USE_THE_YE_SAGIRA_TELEPORT_DEVICE_TO_GO_TO_EXPLORATION_AREA_3;
	
	@ClientString(id = 536401, message = "Obligations of the Seeker (Lv. 14-25)")
	public static NpcStringId OBLIGATIONS_OF_THE_SEEKER_LV_14_25;
	
	@ClientString(id = 536402, message = "Obligations of the Seeker (Lv. 14-25) (In Progress)")
	public static NpcStringId OBLIGATIONS_OF_THE_SEEKER_LV_14_25_IN_PROGRESS;
	
	@ClientString(id = 536403, message = "Obligations of the Seeker (Lv. 14-25) (Done)")
	public static NpcStringId OBLIGATIONS_OF_THE_SEEKER_LV_14_25_DONE;
	
	@ClientString(id = 536404, message = "Obligations of the Seeker (Lv. 14-25)")
	public static NpcStringId OBLIGATIONS_OF_THE_SEEKER_LV_14_252;
	
	@ClientString(id = 536442, message = "Use the Ye Sagira Teleport Device to go to Exploration Area 4.")
	public static NpcStringId USE_THE_YE_SAGIRA_TELEPORT_DEVICE_TO_GO_TO_EXPLORATION_AREA_4;
	
	@ClientString(id = 536501, message = "Seeker Escort (Lv. 16-25)")
	public static NpcStringId SEEKER_ESCORT_LV_16_25;
	
	@ClientString(id = 536502, message = "Seeker Escort (Lv. 16-25) (In Progress)")
	public static NpcStringId SEEKER_ESCORT_LV_16_25_IN_PROGRESS;
	
	@ClientString(id = 536503, message = "Seeker Escort (Lv. 16-25) (Done)")
	public static NpcStringId SEEKER_ESCORT_LV_16_25_DONE;
	
	@ClientString(id = 536504, message = "Seeker Escort (Lv. 16-25)")
	public static NpcStringId SEEKER_ESCORT_LV_16_252;
	
	@ClientString(id = 536601, message = "Ruins Status Update (Lv. 16-25)")
	public static NpcStringId RUINS_STATUS_UPDATE_LV_16_25;
	
	@ClientString(id = 536602, message = "Ruins Status Update (Lv. 16-25) (In Progress)")
	public static NpcStringId RUINS_STATUS_UPDATE_LV_16_25_IN_PROGRESS;
	
	@ClientString(id = 536603, message = "Ruins Status Update (Lv. 16-25) (Done)")
	public static NpcStringId RUINS_STATUS_UPDATE_LV_16_25_DONE;
	
	@ClientString(id = 536604, message = "Ruins Status Update (Lv. 16-25)")
	public static NpcStringId RUINS_STATUS_UPDATE_LV_16_252;
	
	@ClientString(id = 536631, message = "Use the escape scroll in your inventory to go to the Administrative Office in Talking Island.")
	public static NpcStringId USE_THE_ESCAPE_SCROLL_IN_YOUR_INVENTORY_TO_GO_TO_THE_ADMINISTRATIVE_OFFICE_IN_TALKING_ISLAND;
	
	@ClientString(id = 536701, message = "Start of Fate (Lv. 18-99)")
	public static NpcStringId START_OF_FATE_LV_18_993;
	
	@ClientString(id = 536702, message = "Start of Fate (Lv. 18-99) (In Progress)")
	public static NpcStringId START_OF_FATE_LV_18_99_IN_PROGRESS2;
	
	@ClientString(id = 536703, message = "Start of Fate (Lv. 18-99) (Done)")
	public static NpcStringId START_OF_FATE_LV_18_99_DONE2;
	
	@ClientString(id = 536704, message = "Start of Fate (Lv. 18-99)")
	public static NpcStringId START_OF_FATE_LV_18_994;
	
	@ClientString(id = 536801, message = "Monster Uprising (Lv. 34-40)")
	public static NpcStringId MONSTER_UPRISING_LV_34_40;
	
	@ClientString(id = 536802, message = "Monster Uprising (Lv. 34-40) (In Progress)")
	public static NpcStringId MONSTER_UPRISING_LV_34_40_IN_PROGRESS;
	
	@ClientString(id = 536803, message = "Monster Uprising (Lv. 34-40) (Done)")
	public static NpcStringId MONSTER_UPRISING_LV_34_40_DONE;
	
	@ClientString(id = 536804, message = "Monster Uprising (Lv. 34-40)")
	public static NpcStringId MONSTER_UPRISING_LV_34_402;
	
	@ClientString(id = 536901, message = "Noblesse, Soul Testing (Lv. 75-99)")
	public static NpcStringId NOBLESSE_SOUL_TESTING_LV_75_99;
	
	@ClientString(id = 536902, message = "Noblesse, Soul Testing (Lv. 75-99) (In Progress)")
	public static NpcStringId NOBLESSE_SOUL_TESTING_LV_75_99_IN_PROGRESS;
	
	@ClientString(id = 536903, message = "Noblesse, Soul Testing (Lv. 75-99) (Done)")
	public static NpcStringId NOBLESSE_SOUL_TESTING_LV_75_99_DONE;
	
	@ClientString(id = 536904, message = "Noblesse, Soul Testing (Lv. 75-99)")
	public static NpcStringId NOBLESSE_SOUL_TESTING_LV_75_992;
	
	@ClientString(id = 537001, message = "Menacing Times (Lv. 76-81)")
	public static NpcStringId MENACING_TIMES_LV_76_81;
	
	@ClientString(id = 537002, message = "Menacing Times (Lv. 76-81) (In Progress)")
	public static NpcStringId MENACING_TIMES_LV_76_81_IN_PROGRESS;
	
	@ClientString(id = 537003, message = "Menacing Times (Lv. 76-81) (Done)")
	public static NpcStringId MENACING_TIMES_LV_76_81_DONE;
	
	@ClientString(id = 537004, message = "Menacing Times (Lv. 76-81)")
	public static NpcStringId MENACING_TIMES_LV_76_812;
	
	@ClientString(id = 537050, message = "May the blessing of Einhasad be with you...")
	public static NpcStringId MAY_THE_BLESSING_OF_EINHASAD_BE_WITH_YOU;
	
	@ClientString(id = 537051, message = "Eva will protect you.")
	public static NpcStringId EVA_WILL_PROTECT_YOU;
	
	@ClientString(id = 537052, message = "She is always watching over.")
	public static NpcStringId SHE_IS_ALWAYS_WATCHING_OVER;
	
	@ClientString(id = 537053, message = "The Flames of Pa'agrio shall be with you.")
	public static NpcStringId THE_FLAMES_OF_PA_AGRIO_SHALL_BE_WITH_YOU;
	
	@ClientString(id = 537054, message = "Karna Maphr! May the goddess bless you!")
	public static NpcStringId KARNA_MAPHR_MAY_THE_GODDESS_BLESS_YOU;
	
	@ClientString(id = 537055, message = "Do not forget the importance of your mission!")
	public static NpcStringId DO_NOT_FORGET_THE_IMPORTANCE_OF_YOUR_MISSION;
	
	@ClientString(id = 537056, message = "This isn't over. Do not go too far away.")
	public static NpcStringId THIS_ISN_T_OVER_DO_NOT_GO_TOO_FAR_AWAY;
	
	@ClientString(id = 537101, message = "Grasp Thy Power (Lv. 76-81)")
	public static NpcStringId GRASP_THY_POWER_LV_76_81;
	
	@ClientString(id = 537102, message = "Grasp Thy Power (Lv. 76-81) (In Progress)")
	public static NpcStringId GRASP_THY_POWER_LV_76_81_IN_PROGRESS;
	
	@ClientString(id = 537103, message = "Grasp Thy Power (Lv. 76-81) (Done)")
	public static NpcStringId GRASP_THY_POWER_LV_76_81_DONE;
	
	@ClientString(id = 537104, message = "Grasp Thy Power (Lv. 76-81)")
	public static NpcStringId GRASP_THY_POWER_LV_76_812;
	
	@ClientString(id = 537150, message = "A mysterious power from the Bloody Succubus fills you with strength.")
	public static NpcStringId A_MYSTERIOUS_POWER_FROM_THE_BLOODY_SUCCUBUS_FILLS_YOU_WITH_STRENGTH;
	
	@ClientString(id = 537151, message = "Another mysterious power fills you with strength again.")
	public static NpcStringId ANOTHER_MYSTERIOUS_POWER_FILLS_YOU_WITH_STRENGTH_AGAIN;
	
	@ClientString(id = 537152, message = "You can feel another mysterious power.")
	public static NpcStringId YOU_CAN_FEEL_ANOTHER_MYSTERIOUS_POWER;
	
	@ClientString(id = 537153, message = "You have acquired more mysterious power.")
	public static NpcStringId YOU_HAVE_ACQUIRED_MORE_MYSTERIOUS_POWER;
	
	@ClientString(id = 537155, message = "Please check to see if the monsters are suspicious.")
	public static NpcStringId PLEASE_CHECK_TO_SEE_IF_THE_MONSTERS_ARE_SUSPICIOUS;
	
	@ClientString(id = 537156, message = "Take a break, then speak with me again.")
	public static NpcStringId TAKE_A_BREAK_THEN_SPEAK_WITH_ME_AGAIN;
	
	@ClientString(id = 537201, message = "Purgatory Volvere (Lv. 76-81)")
	public static NpcStringId PURGATORY_VOLVERE_LV_76_81;
	
	@ClientString(id = 537202, message = "Purgatory Volvere (Lv. 76-81) (In Progress)")
	public static NpcStringId PURGATORY_VOLVERE_LV_76_81_IN_PROGRESS;
	
	@ClientString(id = 537203, message = "Purgatory Volvere (Lv. 76-81) (Done)")
	public static NpcStringId PURGATORY_VOLVERE_LV_76_81_DONE;
	
	@ClientString(id = 537204, message = "Purgatory Volvere (Lv. 76-81)")
	public static NpcStringId PURGATORY_VOLVERE_LV_76_812;
	
	@ClientString(id = 537250, message = "Please defeat the monster called the Bloody Succubus.")
	public static NpcStringId PLEASE_DEFEAT_THE_MONSTER_CALLED_THE_BLOODY_SUCCUBUS;
	
	@ClientString(id = 537251, message = "If what you say is true, this must be very serious.")
	public static NpcStringId IF_WHAT_YOU_SAY_IS_TRUE_THIS_MUST_BE_VERY_SERIOUS;
	
	@ClientString(id = 537401, message = "That Place Succubus (Lv. 80-99)")
	public static NpcStringId THAT_PLACE_SUCCUBUS_LV_80_99;
	
	@ClientString(id = 537402, message = "That Place Succubus (Lv. 80-99) (In Progress)")
	public static NpcStringId THAT_PLACE_SUCCUBUS_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 537403, message = "That Place Succubus (Lv. 80-99) (Done)")
	public static NpcStringId THAT_PLACE_SUCCUBUS_LV_80_99_DONE;
	
	@ClientString(id = 537404, message = "That Place Succubus (Lv. 80-99)")
	public static NpcStringId THAT_PLACE_SUCCUBUS_LV_80_992;
	
	@ClientString(id = 537450, message = "Then, I will look forward to your actions.")
	public static NpcStringId THEN_I_WILL_LOOK_FORWARD_TO_YOUR_ACTIONS;
	
	@ClientString(id = 537451, message = "Thank you. With your help, we were able to survive a great crisis.")
	public static NpcStringId THANK_YOU_WITH_YOUR_HELP_WE_WERE_ABLE_TO_SURVIVE_A_GREAT_CRISIS;
	
	@ClientString(id = 537501, message = "Succubus Disciples (Lv. 80-99)")
	public static NpcStringId SUCCUBUS_DISCIPLES_LV_80_99;
	
	@ClientString(id = 537502, message = "Succubus Disciples (Lv. 80-99) (In Progress)")
	public static NpcStringId SUCCUBUS_DISCIPLES_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 537503, message = "Succubus Disciples (Lv. 80-99) (Done)")
	public static NpcStringId SUCCUBUS_DISCIPLES_LV_80_99_DONE;
	
	@ClientString(id = 537504, message = "Succubus Disciples (Lv. 80-99)")
	public static NpcStringId SUCCUBUS_DISCIPLES_LV_80_992;
	
	@ClientString(id = 537550, message = "I shall await your safe return.")
	public static NpcStringId I_SHALL_AWAIT_YOUR_SAFE_RETURN;
	
	@ClientString(id = 537551, message = "You are truly amazing for defeating the Succubus disciples.")
	public static NpcStringId YOU_ARE_TRULY_AMAZING_FOR_DEFEATING_THE_SUCCUBUS_DISCIPLES;
	
	@ClientString(id = 537601, message = "Bloody Good Time")
	public static NpcStringId BLOODY_GOOD_TIME;
	
	@ClientString(id = 537602, message = "Bloody Good Time (Lv. 80-99) (In Progress)")
	public static NpcStringId BLOODY_GOOD_TIME_LV_80_99_IN_PROGRESS;
	
	@ClientString(id = 537603, message = "Bloody Good Time (Lv. 80-99) (Done)")
	public static NpcStringId BLOODY_GOOD_TIME_LV_80_99_DONE;
	
	@ClientString(id = 537604, message = "Bloody Good Time (Lv. 80-99)")
	public static NpcStringId BLOODY_GOOD_TIME_LV_80_99;
	
	@ClientString(id = 537650, message = "Go and speak with Vice Hierarch Casca.")
	public static NpcStringId GO_AND_SPEAK_WITH_VICE_HIERARCH_CASCA;
	
	@ClientString(id = 537651, message = "Well done! I was right to entrust this to you.")
	public static NpcStringId WELL_DONE_I_WAS_RIGHT_TO_ENTRUST_THIS_TO_YOU;
	
	@ClientString(id = 537701, message = "The Invaded Execution Grounds (Lv. 95-99)")
	public static NpcStringId THE_INVADED_EXECUTION_GROUNDS_LV_95_99;
	
	@ClientString(id = 537702, message = "The Invaded Execution Grounds (Lv. 95-99) (In Progress)")
	public static NpcStringId THE_INVADED_EXECUTION_GROUNDS_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 537703, message = "The Invaded Execution Grounds (Lv. 95-99) (Done)")
	public static NpcStringId THE_INVADED_EXECUTION_GROUNDS_LV_95_99_DONE;
	
	@ClientString(id = 537704, message = "The Invaded Execution Grounds (Lv. 95-99)")
	public static NpcStringId THE_INVADED_EXECUTION_GROUNDS_LV_95_992;
	
	@ClientString(id = 537751, message = "Central Fortress Area Patrol")
	public static NpcStringId CENTRAL_FORTRESS_AREA_PATROL;
	
	@ClientString(id = 537752, message = "Western Fortress Area Patrol")
	public static NpcStringId WESTERN_FORTRESS_AREA_PATROL;
	
	@ClientString(id = 537753, message = "Northern Fortress Area Patrol")
	public static NpcStringId NORTHERN_FORTRESS_AREA_PATROL;
	
	@ClientString(id = 537754, message = "Southeastern Fortress Area Patrol")
	public static NpcStringId SOUTHEASTERN_FORTRESS_AREA_PATROL;
	
	@ClientString(id = 537755, message = "Oh, gods! Thank you for sending us an adventurer like $s1!")
	public static NpcStringId OH_GODS_THANK_YOU_FOR_SENDING_US_AN_ADVENTURER_LIKE_S1;
	
	@ClientString(id = 537756, message = "The mission is not yet complete.")
	public static NpcStringId THE_MISSION_IS_NOT_YET_COMPLETE2;
	
	@ClientString(id = 537801, message = "Weeding Work (Lv. 95-99)")
	public static NpcStringId WEEDING_WORK_LV_95_99;
	
	@ClientString(id = 537802, message = "Weeding Work (Lv. 95-99) (In Progress)")
	public static NpcStringId WEEDING_WORK_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 537803, message = "Weeding Work (Lv. 95-99) (Done)")
	public static NpcStringId WEEDING_WORK_LV_95_99_DONE;
	
	@ClientString(id = 537804, message = "Weeding Work (Lv. 95-99)")
	public static NpcStringId WEEDING_WORK_LV_95_992;
	
	@ClientString(id = 537851, message = "Please help us discover the cause of this chaos.")
	public static NpcStringId PLEASE_HELP_US_DISCOVER_THE_CAUSE_OF_THIS_CHAOS;
	
	@ClientString(id = 537852, message = "Thank you. It will certainly help the research.")
	public static NpcStringId THANK_YOU_IT_WILL_CERTAINLY_HELP_THE_RESEARCH;
	
	@ClientString(id = 537901, message = "An Uninvited Guest (Lv. 95-99)")
	public static NpcStringId AN_UNINVITED_GUEST_LV_95_99;
	
	@ClientString(id = 537902, message = "An Uninvited Guest (Lv. 95-99) (In Progress)")
	public static NpcStringId AN_UNINVITED_GUEST_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 537903, message = "An Uninvited Guest (Lv. 95-99) (Done)")
	public static NpcStringId AN_UNINVITED_GUEST_LV_95_99_DONE;
	
	@ClientString(id = 537904, message = "An Uninvited Guest (Lv. 95-99)")
	public static NpcStringId AN_UNINVITED_GUEST_LV_95_992;
	
	@ClientString(id = 537921, message = "Summon Scaldisect of Hellfire")
	public static NpcStringId SUMMON_SCALDISECT_OF_HELLFIRE;
	
	@ClientString(id = 537952, message = "Scaldisect of Hellfire! Be prepared for its attack.")
	public static NpcStringId SCALDISECT_OF_HELLFIRE_BE_PREPARED_FOR_ITS_ATTACK;
	
	@ClientString(id = 537953, message = "You obtained a really important result! I will definitely tell the rulers of Dion!")
	public static NpcStringId YOU_OBTAINED_A_REALLY_IMPORTANT_RESULT_I_WILL_DEFINITELY_TELL_THE_RULERS_OF_DION;
	
	@ClientString(id = 537954, message = "Well done! Watchman Endrigo of the Guillotine Fortress will give you a new mission, so be sure to speak with him!")
	public static NpcStringId WELL_DONE_WATCHMAN_ENDRIGO_OF_THE_GUILLOTINE_FORTRESS_WILL_GIVE_YOU_A_NEW_MISSION_SO_BE_SURE_TO_SPEAK_WITH_HIM;
	
	@ClientString(id = 538001, message = "The Executioner's Execution (Lv. 95-99)")
	public static NpcStringId THE_EXECUTIONER_S_EXECUTION_LV_95_99;
	
	@ClientString(id = 538002, message = "The Executioner's Execution (Lv. 95-99) (In Progress)")
	public static NpcStringId THE_EXECUTIONER_S_EXECUTION_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 538003, message = "The Executioner's Execution (Lv. 95-99) (Done)")
	public static NpcStringId THE_EXECUTIONER_S_EXECUTION_LV_95_99_DONE;
	
	@ClientString(id = 538004, message = "The Executioner's Execution (Lv. 95-99)")
	public static NpcStringId THE_EXECUTIONER_S_EXECUTION_LV_95_992;
	
	@ClientString(id = 538051, message = "It is time to put this to an end. Are you ready?")
	public static NpcStringId IT_IS_TIME_TO_PUT_THIS_TO_AN_END_ARE_YOU_READY;
	
	@ClientString(id = 538052, message = "You defeated the Guillotine of Death... I think that was truly amazing.")
	public static NpcStringId YOU_DEFEATED_THE_GUILLOTINE_OF_DEATH_I_THINK_THAT_WAS_TRULY_AMAZING;
	
	@ClientString(id = 538053, message = "To defeat the Guillotine of Death... How amazing.")
	public static NpcStringId TO_DEFEAT_THE_GUILLOTINE_OF_DEATH_HOW_AMAZING;
	
	@ClientString(id = 538101, message = "To the Seed of Hellfire (Lv. 97-99)")
	public static NpcStringId TO_THE_SEED_OF_HELLFIRE_LV_97_99;
	
	@ClientString(id = 538102, message = "To the Seed of Hellfire (Lv. 97-99) (In Progress)")
	public static NpcStringId TO_THE_SEED_OF_HELLFIRE_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 538103, message = "To the Seed of Hellfire (Lv. 97-99) (Done)")
	public static NpcStringId TO_THE_SEED_OF_HELLFIRE_LV_97_99_DONE;
	
	@ClientString(id = 538104, message = "To the Seed of Hellfire (Lv. 97-99)")
	public static NpcStringId TO_THE_SEED_OF_HELLFIRE_LV_97_992;
	
	@ClientString(id = 538150, message = "Hurry and go find Commander Kbaldir.")
	public static NpcStringId HURRY_AND_GO_FIND_COMMANDER_KBALDIR;
	
	@ClientString(id = 538151, message = "You can reach the Seed of Hellfire through the Seed Teleport Device.")
	public static NpcStringId YOU_CAN_REACH_THE_SEED_OF_HELLFIRE_THROUGH_THE_SEED_TELEPORT_DEVICE;
	
	@ClientString(id = 538152, message = "I will look forward to your activity.")
	public static NpcStringId I_WILL_LOOK_FORWARD_TO_YOUR_ACTIVITY;
	
	@ClientString(id = 538201, message = "Day of Liberation (Lv. 97-99)")
	public static NpcStringId DAY_OF_LIBERATION_LV_97_99;
	
	@ClientString(id = 538202, message = "Day of Liberation (Lv. 97-99) (In Progress)")
	public static NpcStringId DAY_OF_LIBERATION_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 538203, message = "Day of Liberation (Lv. 97-99) (Done)")
	public static NpcStringId DAY_OF_LIBERATION_LV_97_99_DONE;
	
	@ClientString(id = 538204, message = "Day of Liberation (Lv. 97-99)")
	public static NpcStringId DAY_OF_LIBERATION_LV_97_992;
	
	@ClientString(id = 538250, message = "Now! Drive out that Tauti and his followers!")
	public static NpcStringId NOW_DRIVE_OUT_THAT_TAUTI_AND_HIS_FOLLOWERS;
	
	@ClientString(id = 538251, message = "We are free! Feel the joy of victory?")
	public static NpcStringId WE_ARE_FREE_FEEL_THE_JOY_OF_VICTORY;
	
	@ClientString(id = 538301, message = "Fergason's Offer (Lv. 97-99)")
	public static NpcStringId FERGASON_S_OFFER_LV_97_99;
	
	@ClientString(id = 538302, message = "Fergason's Offer (Lv. 97-99) (In Progress)")
	public static NpcStringId FERGASON_S_OFFER_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 538303, message = "Fergason's Offer (Lv. 97-99) (Done)")
	public static NpcStringId FERGASON_S_OFFER_LV_97_99_DONE;
	
	@ClientString(id = 538304, message = "Fergason's Offer (Lv. 97-99)")
	public static NpcStringId FERGASON_S_OFFER_LV_97_992;
	
	@ClientString(id = 538350, message = "Fergason. He is a good Dwarf. Go ahead and meet him.")
	public static NpcStringId FERGASON_HE_IS_A_GOOD_DWARF_GO_AHEAD_AND_MEET_HIM;
	
	@ClientString(id = 538351, message = "Try to handle it discretely rather than drawing attention.")
	public static NpcStringId TRY_TO_HANDLE_IT_DISCRETELY_RATHER_THAN_DRAWING_ATTENTION;
	
	@ClientString(id = 538352, message = "Hoho. You look pretty useful.")
	public static NpcStringId HOHO_YOU_LOOK_PRETTY_USEFUL;
	
	@ClientString(id = 538401, message = "An Audience With Tauti (Lv. 97-99)")
	public static NpcStringId AN_AUDIENCE_WITH_TAUTI_LV_97_99;
	
	@ClientString(id = 538402, message = "An Audience With Tauti (Lv. 97-99) (In Progress)")
	public static NpcStringId AN_AUDIENCE_WITH_TAUTI_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 538403, message = "An Audience With Tauti (Lv. 97-99) (Done)")
	public static NpcStringId AN_AUDIENCE_WITH_TAUTI_LV_97_99_DONE;
	
	@ClientString(id = 538404, message = "An Audience With Tauti (Lv. 97-99)")
	public static NpcStringId AN_AUDIENCE_WITH_TAUTI_LV_97_992;
	
	@ClientString(id = 538450, message = "Then, I will entrust this to you. Hohoho.")
	public static NpcStringId THEN_I_WILL_ENTRUST_THIS_TO_YOU_HOHOHO;
	
	@ClientString(id = 538451, message = "Mind your manners when you see Lord Tauti, alright?")
	public static NpcStringId MIND_YOUR_MANNERS_WHEN_YOU_SEE_LORD_TAUTI_ALRIGHT;
	
	@ClientString(id = 538452, message = "Hahaha! Now, my dream can finally come true!")
	public static NpcStringId HAHAHA_NOW_MY_DREAM_CAN_FINALLY_COME_TRUE;
	
	@ClientString(id = 538501, message = "Red Thread of Fate (Lv. 85-99)")
	public static NpcStringId RED_THREAD_OF_FATE_LV_85_99;
	
	@ClientString(id = 538502, message = "Red Thread of Fate (Lv. 85-99) (In Progress)")
	public static NpcStringId RED_THREAD_OF_FATE_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 538503, message = "Red Thread of Fate (Lv. 85-99) (Complete)")
	public static NpcStringId RED_THREAD_OF_FATE_LV_85_99_COMPLETE;
	
	@ClientString(id = 538504, message = "Red Thread of Fate (Lv. 85-99)")
	public static NpcStringId RED_THREAD_OF_FATE_LV_85_992;
	
	@ClientString(id = 538601, message = "Mysterious Journey (Lv. 93-99)")
	public static NpcStringId MYSTERIOUS_JOURNEY_LV_93_99;
	
	@ClientString(id = 538602, message = "Mysterious Journey (Lv. 93-99) (In Progress)")
	public static NpcStringId MYSTERIOUS_JOURNEY_LV_93_99_IN_PROGRESS;
	
	@ClientString(id = 538603, message = "Mysterious Journey (Lv. 93-99) (Complete)")
	public static NpcStringId MYSTERIOUS_JOURNEY_LV_93_99_COMPLETE;
	
	@ClientString(id = 538604, message = "Mysterious Journey (Lv. 93-99)")
	public static NpcStringId MYSTERIOUS_JOURNEY_LV_93_992;
	
	@ClientString(id = 538701, message = "Soulless One (Lv. 93-99)")
	public static NpcStringId SOULLESS_ONE_LV_93_99;
	
	@ClientString(id = 538702, message = "Soulless One (Lv. 93-99) (In Progress)")
	public static NpcStringId SOULLESS_ONE_LV_93_99_IN_PROGRESS;
	
	@ClientString(id = 538703, message = "Soulless One (Lv. 93-99) (Complete)")
	public static NpcStringId SOULLESS_ONE_LV_93_99_COMPLETE;
	
	@ClientString(id = 538704, message = "Soulless One (Lv. 93-99)")
	public static NpcStringId SOULLESS_ONE_LV_93_992;
	
	@ClientString(id = 538801, message = "Conspiracy Behind Doors (Lv. 97-99)")
	public static NpcStringId CONSPIRACY_BEHIND_DOORS_LV_97_99;
	
	@ClientString(id = 538802, message = "Conspiracy Behind Doors (Lv. 97-99) (In Progress)")
	public static NpcStringId CONSPIRACY_BEHIND_DOORS_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 538803, message = "Conspiracy Behind Doors (Lv. 97-99) (Complete)")
	public static NpcStringId CONSPIRACY_BEHIND_DOORS_LV_97_99_COMPLETE;
	
	@ClientString(id = 538804, message = "Conspiracy Behind Doors (Lv. 97-99)")
	public static NpcStringId CONSPIRACY_BEHIND_DOORS_LV_97_992;
	
	@ClientString(id = 538901, message = "The Voice of Authority (Lv. 97-99)")
	public static NpcStringId THE_VOICE_OF_AUTHORITY_LV_97_99;
	
	@ClientString(id = 538902, message = "The Voice of Authority (Lv. 97-99) (In Progress)")
	public static NpcStringId THE_VOICE_OF_AUTHORITY_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 538903, message = "The Voice of Authority (Lv. 97-99) (Complete)")
	public static NpcStringId THE_VOICE_OF_AUTHORITY_LV_97_99_COMPLETE;
	
	@ClientString(id = 538904, message = "The Voice of Authority (Lv. 97-99)")
	public static NpcStringId THE_VOICE_OF_AUTHORITY_LV_97_992;
	
	@ClientString(id = 538951, message = "Eliminate the Pagans in the Anteroom")
	public static NpcStringId ELIMINATE_THE_PAGANS_IN_THE_ANTEROOM;
	
	@ClientString(id = 539001, message = "Kekropus' Letter (Lv. 40-45)")
	public static NpcStringId KEKROPUS_LETTER_LV_40_45;
	
	@ClientString(id = 539002, message = "Kekropus' Letter (Lv. 40-45) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_LV_40_45_IN_PROGRESS;
	
	@ClientString(id = 539003, message = "Kekropus' Letter (Lv. 40-45) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_LV_40_45_COMPLETE;
	
	@ClientString(id = 539004, message = "Kekropus' Letter (Lv. 40-45)")
	public static NpcStringId KEKROPUS_LETTER_LV_40_452;
	
	@ClientString(id = 539031, message = "Talk to Bathis")
	public static NpcStringId TALK_TO_BATHIS;
	
	@ClientString(id = 539032, message = "Talk to Bathis Again")
	public static NpcStringId TALK_TO_BATHIS_AGAIN;
	
	@ClientString(id = 539033, message = "Talk to Gosta")
	public static NpcStringId TALK_TO_GOSTA;
	
	@ClientString(id = 539034, message = "Talk to Eli")
	public static NpcStringId TALK_TO_ELI;
	
	@ClientString(id = 539051, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ;
	
	@ClientString(id = 539101, message = "A Suspicious Helper (Lv. 40-46)")
	public static NpcStringId A_SUSPICIOUS_HELPER_LV_40_46;
	
	@ClientString(id = 539102, message = "A Suspicious Helper (Lv. 40-46) (In Progress)")
	public static NpcStringId A_SUSPICIOUS_HELPER_LV_40_46_IN_PROGRESS;
	
	@ClientString(id = 539103, message = "A Suspicious Helper (Lv. 40-46) (Complete)")
	public static NpcStringId A_SUSPICIOUS_HELPER_LV_40_46_COMPLETE;
	
	@ClientString(id = 539104, message = "A Suspicious Helper (Lv. 40-46)")
	public static NpcStringId A_SUSPICIOUS_HELPER_LV_40_462;
	
	@ClientString(id = 539201, message = "Failure and its Consequences (Lv. 40-46)")
	public static NpcStringId FAILURE_AND_ITS_CONSEQUENCES_LV_40_46;
	
	@ClientString(id = 539202, message = "Failure and its Consequences (Lv. 40-46) (In Progress)")
	public static NpcStringId FAILURE_AND_ITS_CONSEQUENCES_LV_40_46_IN_PROGRESS;
	
	@ClientString(id = 539203, message = "Failure and its Consequences (Lv. 40-46) (Complete)")
	public static NpcStringId FAILURE_AND_ITS_CONSEQUENCES_LV_40_46_COMPLETE;
	
	@ClientString(id = 539204, message = "Failure and its Consequences (Lv. 40-46)")
	public static NpcStringId FAILURE_AND_ITS_CONSEQUENCES_LV_40_462;
	
	@ClientString(id = 539301, message = "Kekropus' Letter: A Clue Completed (Lv. 46-51)")
	public static NpcStringId KEKROPUS_LETTER_A_CLUE_COMPLETED_LV_46_51;
	
	@ClientString(id = 539302, message = "Kekropus' Letter: A Clue Completed (Lv. 46-51) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_A_CLUE_COMPLETED_LV_46_51_IN_PROGRESS;
	
	@ClientString(id = 539303, message = "Kekropus' Letter: A Clue Completed (Lv. 46-51) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_A_CLUE_COMPLETED_LV_46_51_COMPLETE;
	
	@ClientString(id = 539304, message = "Kekropus' Letter: A Clue Completed (Lv. 46-51)")
	public static NpcStringId KEKROPUS_LETTER_A_CLUE_COMPLETED_LV_46_512;
	
	@ClientString(id = 539331, message = "Talk to Flutter")
	public static NpcStringId TALK_TO_FLUTTER;
	
	@ClientString(id = 539332, message = "Talk to Kelios")
	public static NpcStringId TALK_TO_KELIOS;
	
	@ClientString(id = 539351, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ2;
	
	@ClientString(id = 539354, message = "Use Vaccine on Golem Generator")
	public static NpcStringId USE_VACCINE_ON_GOLEM_GENERATOR;
	
	@ClientString(id = 539401, message = "Mutual Benefit (Lv. 46-52)")
	public static NpcStringId MUTUAL_BENEFIT_LV_46_52;
	
	@ClientString(id = 539402, message = "Mutual Benefit (Lv. 46-52) (In Progress)")
	public static NpcStringId MUTUAL_BENEFIT_LV_46_52_IN_PROGRESS;
	
	@ClientString(id = 539403, message = "Mutual Benefit (Lv. 46-52) (Complete)")
	public static NpcStringId MUTUAL_BENEFIT_LV_46_52_COMPLETE;
	
	@ClientString(id = 539404, message = "Mutual Benefit (Lv. 46-52)")
	public static NpcStringId MUTUAL_BENEFIT_LV_46_522;
	
	@ClientString(id = 539501, message = "Not a Traitor (Lv. 46-52)")
	public static NpcStringId NOT_A_TRAITOR_LV_46_52;
	
	@ClientString(id = 539502, message = "Not a Traitor (Lv. 46-52) (In Progress)")
	public static NpcStringId NOT_A_TRAITOR_LV_46_52_IN_PROGRESS;
	
	@ClientString(id = 539503, message = "Not a Traitor (Lv. 46-52) (Complete)")
	public static NpcStringId NOT_A_TRAITOR_LV_46_52_COMPLETE;
	
	@ClientString(id = 539504, message = "Not a Traitor (Lv. 46-52)")
	public static NpcStringId NOT_A_TRAITOR_LV_46_522;
	
	@ClientString(id = 539511, message = "Eliminate the Oel Mahum Monsters")
	public static NpcStringId ELIMINATE_THE_OEL_MAHUM_MONSTERS;
	
	@ClientString(id = 539701, message = "Kekropus' Letter: A Suspicious Badge (Lv. 52-57)")
	public static NpcStringId KEKROPUS_LETTER_A_SUSPICIOUS_BADGE_LV_52_57;
	
	@ClientString(id = 539702, message = "Kekropus' Letter: A Suspicious Badge (Lv. 52-57) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_A_SUSPICIOUS_BADGE_LV_52_57_IN_PROGRESS;
	
	@ClientString(id = 539703, message = "Kekropus' Letter: A Suspicious Badge (Lv. 52-57) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_A_SUSPICIOUS_BADGE_LV_52_57_COMPLETE;
	
	@ClientString(id = 539704, message = "Kekropus' Letter: A Suspicious Badge (Lv. 52-57)")
	public static NpcStringId KEKROPUS_LETTER_A_SUSPICIOUS_BADGE_LV_52_572;
	
	@ClientString(id = 539731, message = "Talk to Mouen")
	public static NpcStringId TALK_TO_MOUEN;
	
	@ClientString(id = 539732, message = "Talk to Andy")
	public static NpcStringId TALK_TO_ANDY;
	
	@ClientString(id = 539751, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ3;
	
	@ClientString(id = 539801, message = "A Suspicious Badge (Lv. 52-58)")
	public static NpcStringId A_SUSPICIOUS_BADGE_LV_52_58;
	
	@ClientString(id = 539802, message = "A Suspicious Badge (Lv. 52-58) (In Progress)")
	public static NpcStringId A_SUSPICIOUS_BADGE_LV_52_58_IN_PROGRESS;
	
	@ClientString(id = 539803, message = "A Suspicious Badge (Lv. 52-58) (Complete)")
	public static NpcStringId A_SUSPICIOUS_BADGE_LV_52_58_COMPLETE;
	
	@ClientString(id = 539804, message = "A Suspicious Badge (Lv. 52-58)")
	public static NpcStringId A_SUSPICIOUS_BADGE_LV_52_582;
	
	@ClientString(id = 539901, message = "The Alphabet of the Giants (Lv. 52-58)")
	public static NpcStringId THE_ALPHABET_OF_THE_GIANTS_LV_52_58;
	
	@ClientString(id = 539902, message = "The Alphabet of the Giants (Lv. 52-58) (In Progress)")
	public static NpcStringId THE_ALPHABET_OF_THE_GIANTS_LV_52_58_IN_PROGRESS;
	
	@ClientString(id = 539903, message = "The Alphabet of the Giants (Lv. 52-58) (Complete)")
	public static NpcStringId THE_ALPHABET_OF_THE_GIANTS_LV_52_58_COMPLETE;
	
	@ClientString(id = 539904, message = "The Alphabet of the Giants (Lv. 52-58)")
	public static NpcStringId THE_ALPHABET_OF_THE_GIANTS_LV_52_582;
	
	@ClientString(id = 540101, message = "Kekropus' Letter: Decoding the Badge (Lv. 58-60)")
	public static NpcStringId KEKROPUS_LETTER_DECODING_THE_BADGE_LV_58_60;
	
	@ClientString(id = 540102, message = "Kekropus' Letter: Decoding the Badge (Lv. 58-60) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_DECODING_THE_BADGE_LV_58_60_IN_PROGRESS;
	
	@ClientString(id = 540103, message = "Kekropus' Letter: Decoding the Badge (Lv. 58-60) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_DECODING_THE_BADGE_LV_58_60_COMPLETE;
	
	@ClientString(id = 540104, message = "Kekropus' Letter: Decoding the Badge (Lv. 58-60)")
	public static NpcStringId KEKROPUS_LETTER_DECODING_THE_BADGE_LV_58_602;
	
	@ClientString(id = 540131, message = "Talk to Paterson")
	public static NpcStringId TALK_TO_PATERSON;
	
	@ClientString(id = 540132, message = "Talk to Eblune")
	public static NpcStringId TALK_TO_EBLUNE;
	
	@ClientString(id = 540151, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ4;
	
	@ClientString(id = 540201, message = "Nowhere to Turn (Lv. 58-61)")
	public static NpcStringId NOWHERE_TO_TURN_LV_58_61;
	
	@ClientString(id = 540202, message = "Nowhere to Turn (Lv. 58-61) (In Progress)")
	public static NpcStringId NOWHERE_TO_TURN_LV_58_61_IN_PROGRESS;
	
	@ClientString(id = 540203, message = "Nowhere to Turn (Lv. 58-61) (Complete)")
	public static NpcStringId NOWHERE_TO_TURN_LV_58_61_COMPLETE;
	
	@ClientString(id = 540204, message = "Nowhere to Turn (Lv. 58-61)")
	public static NpcStringId NOWHERE_TO_TURN_LV_58_612;
	
	@ClientString(id = 540211, message = "Eliminate Monsters in the Forsaken Plains")
	public static NpcStringId ELIMINATE_MONSTERS_IN_THE_FORSAKEN_PLAINS;
	
	@ClientString(id = 540301, message = "The Guardian Giant (Lv. 58-61)")
	public static NpcStringId THE_GUARDIAN_GIANT_LV_58_61;
	
	@ClientString(id = 540302, message = "The Guardian Giant (Lv. 58-61) (In Progress)")
	public static NpcStringId THE_GUARDIAN_GIANT_LV_58_61_IN_PROGRESS;
	
	@ClientString(id = 540303, message = "The Guardian Giant (Lv. 58-61) (Complete)")
	public static NpcStringId THE_GUARDIAN_GIANT_LV_58_61_COMPLETE;
	
	@ClientString(id = 540304, message = "The Guardian Giant (Lv. 58-61)")
	public static NpcStringId THE_GUARDIAN_GIANT_LV_58_612;
	
	@ClientString(id = 540351, message = "Who is it…that threatens us…? You, with the power of the gods… why do you covet our powers?")
	public static NpcStringId WHO_IS_IT_THAT_THREATENS_US_YOU_WITH_THE_POWER_OF_THE_GODS_WHY_DO_YOU_COVET_OUR_POWERS;
	
	@ClientString(id = 540352, message = "You…with the power of the gods…cease your masquerading as our masters. Or else…")
	public static NpcStringId YOU_WITH_THE_POWER_OF_THE_GODS_CEASE_YOUR_MASQUERADING_AS_OUR_MASTERS_OR_ELSE;
	
	@ClientString(id = 540401, message = "Kekropus' Letter: A Hidden Meaning (Lv. 61-64)")
	public static NpcStringId KEKROPUS_LETTER_A_HIDDEN_MEANING_LV_61_64;
	
	@ClientString(id = 540402, message = "Kekropus' Letter: A Hidden Meaning (Lv. 61-64) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_A_HIDDEN_MEANING_LV_61_64_IN_PROGRESS;
	
	@ClientString(id = 540403, message = "Kekropus' Letter: A Hidden Meaning (Lv. 61-64) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_A_HIDDEN_MEANING_LV_61_64_COMPLETE;
	
	@ClientString(id = 540404, message = "Kekropus' Letter: A Hidden Meaning (Lv. 61-64)")
	public static NpcStringId KEKROPUS_LETTER_A_HIDDEN_MEANING_LV_61_642;
	
	@ClientString(id = 540431, message = "Talk to Paterson")
	public static NpcStringId TALK_TO_PATERSON2;
	
	@ClientString(id = 540432, message = "Talk to Shuvann")
	public static NpcStringId TALK_TO_SHUVANN;
	
	@ClientString(id = 540451, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ5;
	
	@ClientString(id = 540501, message = "Kartia's Seed (Lv. 61-65)")
	public static NpcStringId KARTIA_S_SEED_LV_61_65;
	
	@ClientString(id = 540502, message = "Kartia's Seed (Lv. 61-65) (In Progress)")
	public static NpcStringId KARTIA_S_SEED_LV_61_65_IN_PROGRESS;
	
	@ClientString(id = 540503, message = "Kartia's Seed (Lv. 61-65) (Complete)")
	public static NpcStringId KARTIA_S_SEED_LV_61_65_COMPLETE;
	
	@ClientString(id = 540504, message = "Kartia's Seed (Lv. 61-65)")
	public static NpcStringId KARTIA_S_SEED_LV_61_652;
	
	@ClientString(id = 540601, message = "Before Darkness Bears Fruit (Lv. 61-65)")
	public static NpcStringId BEFORE_DARKNESS_BEARS_FRUIT_LV_61_65;
	
	@ClientString(id = 540602, message = "Before Darkness Bears Fruit (Lv. 61-65) (In Progress)")
	public static NpcStringId BEFORE_DARKNESS_BEARS_FRUIT_LV_61_65_IN_PROGRESS;
	
	@ClientString(id = 540603, message = "Before Darkness Bears Fruit (Lv. 61-65) (Complete)")
	public static NpcStringId BEFORE_DARKNESS_BEARS_FRUIT_LV_61_65_COMPLETE;
	
	@ClientString(id = 540604, message = "Before Darkness Bears Fruit (Lv. 61-65)")
	public static NpcStringId BEFORE_DARKNESS_BEARS_FRUIT_LV_61_652;
	
	@ClientString(id = 540651, message = "There is only death for intruders!")
	public static NpcStringId THERE_IS_ONLY_DEATH_FOR_INTRUDERS;
	
	@ClientString(id = 540652, message = "You dig your own grave, coming here!")
	public static NpcStringId YOU_DIG_YOUR_OWN_GRAVE_COMING_HERE;
	
	@ClientString(id = 540653, message = "Die!")
	public static NpcStringId DIE;
	
	@ClientString(id = 540654, message = "Do not touch that flower!")
	public static NpcStringId DO_NOT_TOUCH_THAT_FLOWER;
	
	@ClientString(id = 540655, message = "Hah! You believe that is enough to stand in the path of darkness?")
	public static NpcStringId HAH_YOU_BELIEVE_THAT_IS_ENOUGH_TO_STAND_IN_THE_PATH_OF_DARKNESS;
	
	@ClientString(id = 540801, message = "Kekropus' Letter: The Swamp of Screams (Lv. 65-69)")
	public static NpcStringId KEKROPUS_LETTER_THE_SWAMP_OF_SCREAMS_LV_65_69;
	
	@ClientString(id = 540802, message = "Kekropus' Letter: The Swamp of Screams (Lv. 65-69) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_THE_SWAMP_OF_SCREAMS_LV_65_69_IN_PROGRESS;
	
	@ClientString(id = 540803, message = "Kekropus' Letter: The Swamp of Screams (Lv. 65-69) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_THE_SWAMP_OF_SCREAMS_LV_65_69_COMPLETE;
	
	@ClientString(id = 540804, message = "Kekropus' Letter: The Swamp of Screams (Lv. 65-69)")
	public static NpcStringId KEKROPUS_LETTER_THE_SWAMP_OF_SCREAMS_LV_65_692;
	
	@ClientString(id = 540831, message = "Talk to Mathias")
	public static NpcStringId TALK_TO_MATHIAS;
	
	@ClientString(id = 540832, message = "Talk to Dokara")
	public static NpcStringId TALK_TO_DOKARA;
	
	@ClientString(id = 540851, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ6;
	
	@ClientString(id = 540901, message = "A Suspicious Vagabond in the Swamp (Lv. 65-70)")
	public static NpcStringId A_SUSPICIOUS_VAGABOND_IN_THE_SWAMP_LV_65_70;
	
	@ClientString(id = 540902, message = "A Suspicious Vagabond in the Swamp (Lv. 65-70) (In Progress)")
	public static NpcStringId A_SUSPICIOUS_VAGABOND_IN_THE_SWAMP_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 540903, message = "A Suspicious Vagabond in the Swamp (Lv. 65-70) (Complete)")
	public static NpcStringId A_SUSPICIOUS_VAGABOND_IN_THE_SWAMP_LV_65_70_COMPLETE;
	
	@ClientString(id = 540904, message = "A Suspicious Vagabond in the Swamp (Lv. 65-70)")
	public static NpcStringId A_SUSPICIOUS_VAGABOND_IN_THE_SWAMP_LV_65_702;
	
	@ClientString(id = 541001, message = "Embryo in the Swamp of Screams (Lv. 65-70)")
	public static NpcStringId EMBRYO_IN_THE_SWAMP_OF_SCREAMS_LV_65_70;
	
	@ClientString(id = 541002, message = "Embryo in the Swamp of Screams (Lv. 65-70) (In Progress)")
	public static NpcStringId EMBRYO_IN_THE_SWAMP_OF_SCREAMS_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 541003, message = "Embryo in the Swamp of Screams (Lv. 65-70) (Complete)")
	public static NpcStringId EMBRYO_IN_THE_SWAMP_OF_SCREAMS_LV_65_70_COMPLETE;
	
	@ClientString(id = 541004, message = "Embryo in the Swamp of Screams (Lv. 65-70)")
	public static NpcStringId EMBRYO_IN_THE_SWAMP_OF_SCREAMS_LV_65_702;
	
	@ClientString(id = 541101, message = "Kekropus' Letter: The Forest of the Dead (Lv. 65-69)")
	public static NpcStringId KEKROPUS_LETTER_THE_FOREST_OF_THE_DEAD_LV_65_69;
	
	@ClientString(id = 541102, message = "Kekropus' Letter: The Forest of the Dead (Lv. 65-69) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_THE_FOREST_OF_THE_DEAD_LV_65_69_IN_PROGRESS;
	
	@ClientString(id = 541103, message = "Kekropus' Letter: The Forest of the Dead (Lv. 65-69) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_THE_FOREST_OF_THE_DEAD_LV_65_69_COMPLETE;
	
	@ClientString(id = 541104, message = "Kekropus' Letter: The Forest of the Dead (Lv. 65-69)")
	public static NpcStringId KEKROPUS_LETTER_THE_FOREST_OF_THE_DEAD_LV_65_692;
	
	@ClientString(id = 541131, message = "Talk to Mathias")
	public static NpcStringId TALK_TO_MATHIAS2;
	
	@ClientString(id = 541132, message = "Talk to Hatuba")
	public static NpcStringId TALK_TO_HATUBA;
	
	@ClientString(id = 541151, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ7;
	
	@ClientString(id = 541201, message = "A Suspicious Vagabond in the Forest (Lv. 65-70)")
	public static NpcStringId A_SUSPICIOUS_VAGABOND_IN_THE_FOREST_LV_65_70;
	
	@ClientString(id = 541202, message = "A Suspicious Vagabond in the Forest (Lv. 65-70) (In Progress)")
	public static NpcStringId A_SUSPICIOUS_VAGABOND_IN_THE_FOREST_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 541203, message = "A Suspicious Vagabond in the Forest (Lv. 65-70) (Complete)")
	public static NpcStringId A_SUSPICIOUS_VAGABOND_IN_THE_FOREST_LV_65_70_COMPLETE;
	
	@ClientString(id = 541204, message = "A Suspicious Vagabond in the Forest (Lv. 65-70)")
	public static NpcStringId A_SUSPICIOUS_VAGABOND_IN_THE_FOREST_LV_65_702;
	
	@ClientString(id = 541301, message = "Embryo in the Forest of the Dead (Lv. 65-70)")
	public static NpcStringId EMBRYO_IN_THE_FOREST_OF_THE_DEAD_LV_65_70;
	
	@ClientString(id = 541302, message = "Embryo in the Forest of the Dead (Lv. 65-70) (In Progress)")
	public static NpcStringId EMBRYO_IN_THE_FOREST_OF_THE_DEAD_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 541303, message = "Embryo in the Forest of the Dead (Lv. 65-70) (Complete)")
	public static NpcStringId EMBRYO_IN_THE_FOREST_OF_THE_DEAD_LV_65_70_COMPLETE;
	
	@ClientString(id = 541304, message = "Embryo in the Forest of the Dead (Lv. 65-70)")
	public static NpcStringId EMBRYO_IN_THE_FOREST_OF_THE_DEAD_LV_65_702;
	
	@ClientString(id = 541401, message = "Kekropus' Letter: With Courage (Lv. 70-75)")
	public static NpcStringId KEKROPUS_LETTER_WITH_COURAGE_LV_70_75;
	
	@ClientString(id = 541402, message = "Kekropus' Letter: With Courage (Lv. 70-75) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_WITH_COURAGE_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 541403, message = "Kekropus' Letter: With Courage (Lv. 70-75) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_WITH_COURAGE_LV_70_75_COMPLETE;
	
	@ClientString(id = 541404, message = "Kekropus' Letter: With Courage (Lv. 70-75)")
	public static NpcStringId KEKROPUS_LETTER_WITH_COURAGE_LV_70_752;
	
	@ClientString(id = 541431, message = "Talk to Andrei")
	public static NpcStringId TALK_TO_ANDREI;
	
	@ClientString(id = 541432, message = "Talk to Janitt")
	public static NpcStringId TALK_TO_JANITT;
	
	@ClientString(id = 541451, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ8;
	
	@ClientString(id = 541501, message = "Kekropus' Letter: With Wisdom (Lv. 70-75)")
	public static NpcStringId KEKROPUS_LETTER_WITH_WISDOM_LV_70_75;
	
	@ClientString(id = 541502, message = "Kekropus' Letter: With Wisdom (Lv. 70-75) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_WITH_WISDOM_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 541503, message = "Kekropus' Letter: With Wisdom (Lv. 70-75) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_WITH_WISDOM_LV_70_75_COMPLETE;
	
	@ClientString(id = 541504, message = "Kekropus' Letter: With Wisdom (Lv. 70-75)")
	public static NpcStringId KEKROPUS_LETTER_WITH_WISDOM_LV_70_752;
	
	@ClientString(id = 541531, message = "Talk to Andrei")
	public static NpcStringId TALK_TO_ANDREI2;
	
	@ClientString(id = 541532, message = "Talk to Janitt")
	public static NpcStringId TALK_TO_JANITT2;
	
	@ClientString(id = 541551, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ9;
	
	@ClientString(id = 541601, message = "In Search of the Eye of Argos (Lv. 70-75)")
	public static NpcStringId IN_SEARCH_OF_THE_EYE_OF_ARGOS_LV_70_75;
	
	@ClientString(id = 541602, message = "In Search of the Eye of Argos (Lv. 70-75) (In Progress)")
	public static NpcStringId IN_SEARCH_OF_THE_EYE_OF_ARGOS_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 541603, message = "In Search of the Eye of Argos (Lv. 70-75) (Complete)")
	public static NpcStringId IN_SEARCH_OF_THE_EYE_OF_ARGOS_LV_70_75_COMPLETE;
	
	@ClientString(id = 541604, message = "In Search of the Eye of Argos (Lv. 70-75)")
	public static NpcStringId IN_SEARCH_OF_THE_EYE_OF_ARGOS_LV_70_752;
	
	@ClientString(id = 541701, message = "Daimon the White-eyed (Lv. 70-75)")
	public static NpcStringId DAIMON_THE_WHITE_EYED_LV_70_75;
	
	@ClientString(id = 541702, message = "Daimon the White-eyed (Lv. 70-75) (In Progress)")
	public static NpcStringId DAIMON_THE_WHITE_EYED_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 541703, message = "Daimon the White-eyed (Lv. 70-75) (Complete)")
	public static NpcStringId DAIMON_THE_WHITE_EYED_LV_70_75_COMPLETE;
	
	@ClientString(id = 541704, message = "Daimon the White-eyed (Lv. 70-75)")
	public static NpcStringId DAIMON_THE_WHITE_EYED_LV_70_752;
	
	@ClientString(id = 541901, message = "Kekropus' Letter: Kampf's Whereabouts (Lv. 76-80)")
	public static NpcStringId KEKROPUS_LETTER_KAMPF_S_WHEREABOUTS_LV_76_80;
	
	@ClientString(id = 541902, message = "Kekropus' Letter: Kampf's Whereabouts (Lv. 76-80) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_KAMPF_S_WHEREABOUTS_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 541903, message = "Kekropus' Letter: Kampf's Whereabouts (Lv. 76-80) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_KAMPF_S_WHEREABOUTS_LV_76_80_COMPLETE;
	
	@ClientString(id = 541904, message = "Kekropus' Letter: Kampf's Whereabouts (Lv. 76-80)")
	public static NpcStringId KEKROPUS_LETTER_KAMPF_S_WHEREABOUTS_LV_76_802;
	
	@ClientString(id = 541931, message = "Talk to Andrei")
	public static NpcStringId TALK_TO_ANDREI3;
	
	@ClientString(id = 541932, message = "Talk to Hansen")
	public static NpcStringId TALK_TO_HANSEN;
	
	@ClientString(id = 541951, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ10;
	
	@ClientString(id = 542001, message = "The Varka Silenos Supporters (Lv. 76-80)")
	public static NpcStringId THE_VARKA_SILENOS_SUPPORTERS_LV_76_80;
	
	@ClientString(id = 542002, message = "The Varka Silenos Supporters (Lv. 76-80) (In Progress)")
	public static NpcStringId THE_VARKA_SILENOS_SUPPORTERS_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 542003, message = "The Varka Silenos Supporters (Lv. 76-80) (Complete)")
	public static NpcStringId THE_VARKA_SILENOS_SUPPORTERS_LV_76_80_COMPLETE;
	
	@ClientString(id = 542004, message = "The Varka Silenos Supporters (Lv. 76-80)")
	public static NpcStringId THE_VARKA_SILENOS_SUPPORTERS_LV_76_802;
	
	@ClientString(id = 542101, message = "Assassination of the Varka Silenos Commander (Lv. 76-80)")
	public static NpcStringId ASSASSINATION_OF_THE_VARKA_SILENOS_COMMANDER_LV_76_80;
	
	@ClientString(id = 542102, message = "Assassination of the Varka Silenos Commander (Lv. 76-80) (In Progress)")
	public static NpcStringId ASSASSINATION_OF_THE_VARKA_SILENOS_COMMANDER_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 542103, message = "Assassination of the Varka Silenos Commander (Lv. 76-80) (Complete)")
	public static NpcStringId ASSASSINATION_OF_THE_VARKA_SILENOS_COMMANDER_LV_76_80_COMPLETE;
	
	@ClientString(id = 542104, message = "Assassination of the Varka Silenos Commander (Lv. 76-80)")
	public static NpcStringId ASSASSINATION_OF_THE_VARKA_SILENOS_COMMANDER_LV_76_802;
	
	@ClientString(id = 542201, message = "Assassination of the Varka Silenos Chief (Lv. 76-80)")
	public static NpcStringId ASSASSINATION_OF_THE_VARKA_SILENOS_CHIEF_LV_76_80;
	
	@ClientString(id = 542202, message = "Assassination of the Varka Silenos Chief (Lv. 76-80) (In Progress)")
	public static NpcStringId ASSASSINATION_OF_THE_VARKA_SILENOS_CHIEF_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 542203, message = "Assassination of the Varka Silenos Chief (Lv. 76-80) (Complete)")
	public static NpcStringId ASSASSINATION_OF_THE_VARKA_SILENOS_CHIEF_LV_76_80_COMPLETE;
	
	@ClientString(id = 542204, message = "Assassination of the Varka Silenos Chief (Lv. 76-80)")
	public static NpcStringId ASSASSINATION_OF_THE_VARKA_SILENOS_CHIEF_LV_76_802;
	
	@ClientString(id = 542401, message = "Kekropus' Letter: Belos' Whereabouts (Lv. 76-80)")
	public static NpcStringId KEKROPUS_LETTER_BELOS_WHEREABOUTS_LV_76_80;
	
	@ClientString(id = 542402, message = "Kekropus' Letter: Belos' Whereabouts (Lv. 76-80) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_BELOS_WHEREABOUTS_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 542403, message = "Kekropus' Letter: Belos' Whereabouts (Lv. 76-80) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_BELOS_WHEREABOUTS_LV_76_80_COMPLETE;
	
	@ClientString(id = 542404, message = "Kekropus' Letter: Belos' Whereabouts (Lv. 76-80)")
	public static NpcStringId KEKROPUS_LETTER_BELOS_WHEREABOUTS_LV_76_802;
	
	@ClientString(id = 542431, message = "Talk to Andrei")
	public static NpcStringId TALK_TO_ANDREI4;
	
	@ClientString(id = 542432, message = "Talk to Lugones")
	public static NpcStringId TALK_TO_LUGONES;
	
	@ClientString(id = 542451, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ11;
	
	@ClientString(id = 542501, message = "The Ketra Orc Supporters (Lv. 76-80)")
	public static NpcStringId THE_KETRA_ORC_SUPPORTERS_LV_76_80;
	
	@ClientString(id = 542502, message = "The Ketra Orc Supporters (Lv. 76-80) (In Progress)")
	public static NpcStringId THE_KETRA_ORC_SUPPORTERS_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 542503, message = "The Ketra Orc Supporters (Lv. 76-80) (Complete)")
	public static NpcStringId THE_KETRA_ORC_SUPPORTERS_LV_76_80_COMPLETE;
	
	@ClientString(id = 542504, message = "The Ketra Orc Supporters (Lv. 76-80)")
	public static NpcStringId THE_KETRA_ORC_SUPPORTERS_LV_76_802;
	
	@ClientString(id = 542601, message = "Assassination of the Ketra Orc Commander (Lv. 76-80)")
	public static NpcStringId ASSASSINATION_OF_THE_KETRA_ORC_COMMANDER_LV_76_80;
	
	@ClientString(id = 542602, message = "Assassination of the Ketra Orc Commander (Lv. 76-80) (In Progress)")
	public static NpcStringId ASSASSINATION_OF_THE_KETRA_ORC_COMMANDER_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 542603, message = "Assassination of the Ketra Orc Commander (Lv. 76-80) (Complete)")
	public static NpcStringId ASSASSINATION_OF_THE_KETRA_ORC_COMMANDER_LV_76_80_COMPLETE;
	
	@ClientString(id = 542604, message = "Assassination of the Ketra Orc Commander (Lv. 76-80)")
	public static NpcStringId ASSASSINATION_OF_THE_KETRA_ORC_COMMANDER_LV_76_802;
	
	@ClientString(id = 542701, message = "Assassination of the Ketra Orc Chief (Lv. 76-80)")
	public static NpcStringId ASSASSINATION_OF_THE_KETRA_ORC_CHIEF_LV_76_80;
	
	@ClientString(id = 542702, message = "Assassination of the Ketra Orc Chief (Lv. 76-80) (In Progress)")
	public static NpcStringId ASSASSINATION_OF_THE_KETRA_ORC_CHIEF_LV_76_80_IN_PROGRESS;
	
	@ClientString(id = 542703, message = "Assassination of the Ketra Orc Chief (Lv. 76-80) (Complete)")
	public static NpcStringId ASSASSINATION_OF_THE_KETRA_ORC_CHIEF_LV_76_80_COMPLETE;
	
	@ClientString(id = 542704, message = "Assassination of the Ketra Orc Chief (Lv. 76-80)")
	public static NpcStringId ASSASSINATION_OF_THE_KETRA_ORC_CHIEF_LV_76_802;
	
	@ClientString(id = 543001, message = "Kekropus' Letter: Tracking the Evil (Lv. 81-84)")
	public static NpcStringId KEKROPUS_LETTER_TRACKING_THE_EVIL_LV_81_84;
	
	@ClientString(id = 543002, message = "Kekropus' Letter: Tracking the Evil (Lv. 81-84) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_TRACKING_THE_EVIL_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543003, message = "Kekropus' Letter: Tracking the Evil (Lv. 81-84) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_TRACKING_THE_EVIL_LV_81_84_COMPLETE;
	
	@ClientString(id = 543004, message = "Kekropus' Letter: Tracking the Evil (Lv. 81-84)")
	public static NpcStringId KEKROPUS_LETTER_TRACKING_THE_EVIL_LV_81_842;
	
	@ClientString(id = 543031, message = "Talk to Vishotsky")
	public static NpcStringId TALK_TO_VISHOTSKY;
	
	@ClientString(id = 543032, message = "Talk to Jokel")
	public static NpcStringId TALK_TO_JOKEL;
	
	@ClientString(id = 543051, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ12;
	
	@ClientString(id = 543101, message = "The Seal of Punishment: Den of Evil (Lv. 81-84)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_DEN_OF_EVIL_LV_81_84;
	
	@ClientString(id = 543102, message = "The Seal of Punishment: Den of Evil (Lv. 81-84) (In Progress)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_DEN_OF_EVIL_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543103, message = "The Seal of Punishment: Den of Evil (Lv. 81-84) (Complete)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_DEN_OF_EVIL_LV_81_84_COMPLETE;
	
	@ClientString(id = 543104, message = "The Seal of Punishment: Den of Evil (Lv. 81-84)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_DEN_OF_EVIL_LV_81_842;
	
	@ClientString(id = 543201, message = "Chasing Varangka (Lv. 81-84)")
	public static NpcStringId CHASING_VARANGKA_LV_81_84;
	
	@ClientString(id = 543202, message = "Chasing Varangka (Lv. 81-84) (In Progress)")
	public static NpcStringId CHASING_VARANGKA_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543203, message = "Chasing Varangka (Lv. 81-84) (Complete)")
	public static NpcStringId CHASING_VARANGKA_LV_81_84_COMPLETE;
	
	@ClientString(id = 543204, message = "Chasing Varangka (Lv. 81-84)")
	public static NpcStringId CHASING_VARANGKA_LV_81_842;
	
	@ClientString(id = 543301, message = "Kekropus' Letter: Regarding a Seal (Lv. 81-84)")
	public static NpcStringId KEKROPUS_LETTER_REGARDING_A_SEAL_LV_81_84;
	
	@ClientString(id = 543302, message = "Kekropus' Letter: Regarding a Seal (Lv. 81-84) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_REGARDING_A_SEAL_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543303, message = "Kekropus' Letter: Regarding a Seal (Lv. 81-84) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_REGARDING_A_SEAL_LV_81_84_COMPLETE;
	
	@ClientString(id = 543304, message = "Kekropus' Letter: Regarding a Seal (Lv. 81-84)")
	public static NpcStringId KEKROPUS_LETTER_REGARDING_A_SEAL_LV_81_842;
	
	@ClientString(id = 543331, message = "Talk to Mouen")
	public static NpcStringId TALK_TO_MOUEN2;
	
	@ClientString(id = 543332, message = "Talk to Rua")
	public static NpcStringId TALK_TO_RUA;
	
	@ClientString(id = 543351, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ13;
	
	@ClientString(id = 543401, message = "The Seal of Punishment: Sel Mahum Training Grounds (Lv. 81-84)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_SEL_MAHUM_TRAINING_GROUNDS_LV_81_84;
	
	@ClientString(id = 543402, message = "The Seal of Punishment: Sel Mahum Training Grounds (Lv. 81-84) (In Progress)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_SEL_MAHUM_TRAINING_GROUNDS_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543403, message = "The Seal of Punishment: Sel Mahum Training Grounds (Lv. 81-84) (Complete)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_SEL_MAHUM_TRAINING_GROUNDS_LV_81_84_COMPLETE;
	
	@ClientString(id = 543404, message = "The Seal of Punishment: Sel Mahum Training Grounds (Lv. 81-84)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_SEL_MAHUM_TRAINING_GROUNDS_LV_81_842;
	
	@ClientString(id = 543501, message = "Chasing Keltron (Lv. 81-84)")
	public static NpcStringId CHASING_KELTRON_LV_81_84;
	
	@ClientString(id = 543502, message = "Chasing Keltron (Lv. 81-84) (In Progress)")
	public static NpcStringId CHASING_KELTRON_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543503, message = "Chasing Keltron (Lv. 81-84) (Complete)")
	public static NpcStringId CHASING_KELTRON_LV_81_84_COMPLETE;
	
	@ClientString(id = 543504, message = "Chasing Keltron (Lv. 81-84)")
	public static NpcStringId CHASING_KELTRON_LV_81_842;
	
	@ClientString(id = 543601, message = "Kekropus' Letter: The Seal of Punishment (Lv. 81-84)")
	public static NpcStringId KEKROPUS_LETTER_THE_SEAL_OF_PUNISHMENT_LV_81_84;
	
	@ClientString(id = 543602, message = "Kekropus' Letter: The Seal of Punishment (Lv. 81-84) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_THE_SEAL_OF_PUNISHMENT_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543603, message = "Kekropus' Letter: The Seal of Punishment (Lv. 81-84) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_THE_SEAL_OF_PUNISHMENT_LV_81_84_COMPLETE;
	
	@ClientString(id = 543604, message = "Kekropus' Letter: The Seal of Punishment (Lv. 81-84)")
	public static NpcStringId KEKROPUS_LETTER_THE_SEAL_OF_PUNISHMENT_LV_81_842;
	
	@ClientString(id = 543631, message = "Talk to Mouen")
	public static NpcStringId TALK_TO_MOUEN3;
	
	@ClientString(id = 543632, message = "Talk to Laki")
	public static NpcStringId TALK_TO_LAKI;
	
	@ClientString(id = 543651, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ14;
	
	@ClientString(id = 543701, message = "The Seal of Punishment: Plains of the Lizardmen (Lv. 81-84)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_PLAINS_OF_THE_LIZARDMEN_LV_81_84;
	
	@ClientString(id = 543702, message = "The Seal of Punishment: Plains of the Lizardmen (Lv. 81-84) (In Progress)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_PLAINS_OF_THE_LIZARDMEN_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543703, message = "The Seal of Punishment: Plains of the Lizardmen (Lv. 81-84) (Complete)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_PLAINS_OF_THE_LIZARDMEN_LV_81_84_COMPLETE;
	
	@ClientString(id = 543704, message = "The Seal of Punishment: Plains of the Lizardmen (Lv. 81-84)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_PLAINS_OF_THE_LIZARDMEN_LV_81_842;
	
	@ClientString(id = 543801, message = "Chasing Loygen (Lv. 81-84)")
	public static NpcStringId CHASING_LOYGEN_LV_81_84;
	
	@ClientString(id = 543802, message = "Chasing Loygen (Lv. 81-84) (In Progress)")
	public static NpcStringId CHASING_LOYGEN_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543803, message = "Chasing Loygen (Lv. 81-84) (Complete)")
	public static NpcStringId CHASING_LOYGEN_LV_81_84_COMPLETE;
	
	@ClientString(id = 543804, message = "Chasing Loygen (Lv. 81-84)")
	public static NpcStringId CHASING_LOYGEN_LV_81_842;
	
	@ClientString(id = 543901, message = "Kekropus' Letter: The Origins of a Rumor (Lv. 81-84)")
	public static NpcStringId KEKROPUS_LETTER_THE_ORIGINS_OF_A_RUMOR_LV_81_84;
	
	@ClientString(id = 543902, message = "Kekropus' Letter: The Origins of a Rumor (Lv. 81-84) (In Progress)")
	public static NpcStringId KEKROPUS_LETTER_THE_ORIGINS_OF_A_RUMOR_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 543903, message = "Kekropus' Letter: The Origins of a Rumor (Lv. 81-84) (Complete)")
	public static NpcStringId KEKROPUS_LETTER_THE_ORIGINS_OF_A_RUMOR_LV_81_84_COMPLETE;
	
	@ClientString(id = 543904, message = "Kekropus' Letter: The Origins of a Rumor (Lv. 81-84)")
	public static NpcStringId KEKROPUS_LETTER_THE_ORIGINS_OF_A_RUMOR_LV_81_842;
	
	@ClientString(id = 543931, message = "Talk to Gosta")
	public static NpcStringId TALK_TO_GOSTA2;
	
	@ClientString(id = 543932, message = "Talk to Dr. Helvetica")
	public static NpcStringId TALK_TO_DR_HELVETICA;
	
	@ClientString(id = 543933, message = "Talk to Athenia")
	public static NpcStringId TALK_TO_ATHENIA;
	
	@ClientString(id = 543951, message = "Kekropus' Letter has arrived.nClick the question-mark icon to read.")
	public static NpcStringId KEKROPUS_LETTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ15;
	
	@ClientString(id = 544001, message = "The Seal of Punishment: The Fields (Lv. 81-84)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_THE_FIELDS_LV_81_84;
	
	@ClientString(id = 544002, message = "The Seal of Punishment: The Fields (Lv. 81-84) (In Progress)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_THE_FIELDS_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 544003, message = "The Seal of Punishment: The Fields (Lv. 81-84) (Complete)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_THE_FIELDS_LV_81_84_COMPLETE;
	
	@ClientString(id = 544004, message = "The Seal of Punishment: The Fields (Lv. 81-84)")
	public static NpcStringId THE_SEAL_OF_PUNISHMENT_THE_FIELDS_LV_81_842;
	
	@ClientString(id = 544051, message = "Eliminating the Mucrokians")
	public static NpcStringId ELIMINATING_THE_MUCROKIANS;
	
	@ClientString(id = 544101, message = "Chasing Meccadan (Lv. 81-84)")
	public static NpcStringId CHASING_MECCADAN_LV_81_84;
	
	@ClientString(id = 544102, message = "Chasing Meccadan (Lv. 81-84) (In Progress)")
	public static NpcStringId CHASING_MECCADAN_LV_81_84_IN_PROGRESS;
	
	@ClientString(id = 544103, message = "Chasing Meccadan (Lv. 81-84) (Complete)")
	public static NpcStringId CHASING_MECCADAN_LV_81_84_COMPLETE;
	
	@ClientString(id = 544104, message = "Chasing Meccadan (Lv. 81-84)")
	public static NpcStringId CHASING_MECCADAN_LV_81_842;
	
	@ClientString(id = 544151, message = "Coveting the power of the Seal, huh? Such courage should be rewarded with…death!")
	public static NpcStringId COVETING_THE_POWER_OF_THE_SEAL_HUH_SUCH_COURAGE_SHOULD_BE_REWARDED_WITH_DEATH;
	
	@ClientString(id = 544152, message = "Fools…you have…also…been used…by... Shilen…argh…even if I die, the Seal of Punishment will…arghhh…")
	public static NpcStringId FOOLS_YOU_HAVE_ALSO_BEEN_USED_BY_SHILEN_ARGH_EVEN_IF_I_DIE_THE_SEAL_OF_PUNISHMENT_WILL_ARGHHH;
	
	@ClientString(id = 544201, message = "The Annihilated Plains - 1 (Lv. 99-99)")
	public static NpcStringId THE_ANNIHILATED_PLAINS_1_LV_99_99;
	
	@ClientString(id = 544202, message = "The Annihilated Plains - 1 (Lv. 99-99) (In Progress)")
	public static NpcStringId THE_ANNIHILATED_PLAINS_1_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 544203, message = "The Annihilated Plains - 1 (Lv. 99-99) (Complete)")
	public static NpcStringId THE_ANNIHILATED_PLAINS_1_LV_99_99_COMPLETE;
	
	@ClientString(id = 544204, message = "The Annihilated Plains - 1 (Lv. 99-99)")
	public static NpcStringId THE_ANNIHILATED_PLAINS_1_LV_99_992;
	
	@ClientString(id = 544301, message = "The Annihilated Plains - 2 (Lv. 99-99)")
	public static NpcStringId THE_ANNIHILATED_PLAINS_2_LV_99_99;
	
	@ClientString(id = 544302, message = "The Annihilated Plains - 2 (Lv. 99-99) (In Progress)")
	public static NpcStringId THE_ANNIHILATED_PLAINS_2_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 544303, message = "The Annihilated Plains - 2 (Lv. 99-99) (Complete)")
	public static NpcStringId THE_ANNIHILATED_PLAINS_2_LV_99_99_COMPLETE;
	
	@ClientString(id = 544304, message = "The Annihilated Plains - 2 (Lv. 99-99)")
	public static NpcStringId THE_ANNIHILATED_PLAINS_2_LV_99_992;
	
	@ClientString(id = 544401, message = "The Origin of Monsters (Lv. 99-99)")
	public static NpcStringId THE_ORIGIN_OF_MONSTERS_LV_99_99;
	
	@ClientString(id = 544402, message = "The Origin of Monsters (Lv. 99-99) (In Progress)")
	public static NpcStringId THE_ORIGIN_OF_MONSTERS_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 544403, message = "The Origin of Monsters (Lv. 99-99) (Complete)")
	public static NpcStringId THE_ORIGIN_OF_MONSTERS_LV_99_99_COMPLETE;
	
	@ClientString(id = 544404, message = "The Origin of Monsters (Lv. 99-99)")
	public static NpcStringId THE_ORIGIN_OF_MONSTERS_LV_99_992;
	
	@ClientString(id = 544501, message = "An Impending Threat (Lv. 99-99)")
	public static NpcStringId AN_IMPENDING_THREAT_LV_99_99;
	
	@ClientString(id = 544502, message = "An Impending Threat (Lv. 99-99) (In Progress)")
	public static NpcStringId AN_IMPENDING_THREAT_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 544503, message = "An Impending Threat (Lv. 99-99) (Complete)")
	public static NpcStringId AN_IMPENDING_THREAT_LV_99_99_COMPLETE;
	
	@ClientString(id = 544504, message = "An Impending Threat (Lv. 99-99)")
	public static NpcStringId AN_IMPENDING_THREAT_LV_99_992;
	
	@ClientString(id = 544511, message = "Go to Tuska")
	public static NpcStringId GO_TO_TUSKA;
	
	@ClientString(id = 544512, message = "Go to Bruener")
	public static NpcStringId GO_TO_BRUENER;
	
	@ClientString(id = 544601, message = "Hit and Run (Lv. 99-99)")
	public static NpcStringId HIT_AND_RUN_LV_99_99;
	
	@ClientString(id = 544602, message = "Hit and Run (Lv. 99-99) (In Progress)")
	public static NpcStringId HIT_AND_RUN_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 544603, message = "Hit and Run (Lv. 99-99) (Complete)")
	public static NpcStringId HIT_AND_RUN_LV_99_99_COMPLETE;
	
	@ClientString(id = 544604, message = "Hit and Run (Lv. 99-99)")
	public static NpcStringId HIT_AND_RUN_LV_99_992;
	
	@ClientString(id = 544701, message = "Timing is Everything (Lv. 99-99)")
	public static NpcStringId TIMING_IS_EVERYTHING_LV_99_99;
	
	@ClientString(id = 544702, message = "Timing is Everything (Lv. 99-99) (In Progress)")
	public static NpcStringId TIMING_IS_EVERYTHING_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 544703, message = "Timing is Everything (Lv. 99-99) (Complete)")
	public static NpcStringId TIMING_IS_EVERYTHING_LV_99_99_COMPLETE;
	
	@ClientString(id = 544704, message = "Timing is Everything (Lv. 99-99)")
	public static NpcStringId TIMING_IS_EVERYTHING_LV_99_992;
	
	@ClientString(id = 544711, message = "Use Nerva's Temporary Prison Key")
	public static NpcStringId USE_NERVA_S_TEMPORARY_PRISON_KEY;
	
	@ClientString(id = 545001, message = "A Dark Ambition (Lv. 99-99)")
	public static NpcStringId A_DARK_AMBITION_LV_99_99;
	
	@ClientString(id = 545002, message = "A Dark Ambition (Lv. 99-99) (In Progress)")
	public static NpcStringId A_DARK_AMBITION_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 545003, message = "A Dark Ambition (Lv. 99-99) (Complete)")
	public static NpcStringId A_DARK_AMBITION_LV_99_99_COMPLETE;
	
	@ClientString(id = 545004, message = "A Dark Ambition (Lv. 99-99)")
	public static NpcStringId A_DARK_AMBITION_LV_99_992;
	
	@ClientString(id = 545201, message = "Is it Edible? (Lv. 81-99)")
	public static NpcStringId IS_IT_EDIBLE_LV_81_99;
	
	@ClientString(id = 545202, message = "Is it Edible? (Lv. 81-99) (In Progress)")
	public static NpcStringId IS_IT_EDIBLE_LV_81_99_IN_PROGRESS;
	
	@ClientString(id = 545203, message = "Is it Edible? (Lv. 81-99) (Complete)")
	public static NpcStringId IS_IT_EDIBLE_LV_81_99_COMPLETE;
	
	@ClientString(id = 545204, message = "Is it Edible? (Lv. 81-99)")
	public static NpcStringId IS_IT_EDIBLE_LV_81_992;
	
	@ClientString(id = 545301, message = "Stopping the Wind Dragon (Lv. 95-99)")
	public static NpcStringId STOPPING_THE_WIND_DRAGON_LV_95_99;
	
	@ClientString(id = 545302, message = "Stopping the Wind Dragon (Lv. 95-99) (In Progress)")
	public static NpcStringId STOPPING_THE_WIND_DRAGON_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 545303, message = "Stopping the Wind Dragon (Lv. 95-99) (Complete)")
	public static NpcStringId STOPPING_THE_WIND_DRAGON_LV_95_99_COMPLETE;
	
	@ClientString(id = 545304, message = "Stopping the Wind Dragon (Lv. 95-99)")
	public static NpcStringId STOPPING_THE_WIND_DRAGON_LV_95_992;
	
	@ClientString(id = 545501, message = "Elikia's Letter (Lv. 99-99)")
	public static NpcStringId ELIKIA_S_LETTER_LV_99_99;
	
	@ClientString(id = 545502, message = "Elikia's Letter (Lv. 99-99) (In Progress)")
	public static NpcStringId ELIKIA_S_LETTER_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 545503, message = "Elikia's Letter (Lv. 99-99) (Complete)")
	public static NpcStringId ELIKIA_S_LETTER_LV_99_99_COMPLETE;
	
	@ClientString(id = 545504, message = "Elikia's Letter (Lv. 99-99)")
	public static NpcStringId ELIKIA_S_LETTER_LV_99_992;
	
	@ClientString(id = 545521, message = "You must activate the Warp Gate behind me in order to teleport to Hellbound.")
	public static NpcStringId YOU_MUST_ACTIVATE_THE_WARP_GATE_BEHIND_ME_IN_ORDER_TO_TELEPORT_TO_HELLBOUND;
	
	@ClientString(id = 545522, message = "Have you made preparations for the mission? There isn't much time.")
	public static NpcStringId HAVE_YOU_MADE_PREPARATIONS_FOR_THE_MISSION_THERE_ISN_T_MUCH_TIME;
	
	@ClientString(id = 545601, message = "Operation: Rescue (Lv. 99-99)")
	public static NpcStringId OPERATION_RESCUE_LV_99_99;
	
	@ClientString(id = 545602, message = "Operation: Rescue (Lv. 99-99) (In Progress)")
	public static NpcStringId OPERATION_RESCUE_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 545603, message = "Operation: Rescue (Lv. 99-99) (Complete)")
	public static NpcStringId OPERATION_RESCUE_LV_99_99_COMPLETE;
	
	@ClientString(id = 545604, message = "Operation: Rescue (Lv. 99-99)")
	public static NpcStringId OPERATION_RESCUE_LV_99_992;
	
	@ClientString(id = 545611, message = "Free the Captives")
	public static NpcStringId FREE_THE_CAPTIVES;
	
	@ClientString(id = 545621, message = "I hope they are safe...")
	public static NpcStringId I_HOPE_THEY_ARE_SAFE;
	
	@ClientString(id = 545622, message = "You should be proud of what you have done!")
	public static NpcStringId YOU_SHOULD_BE_PROUD_OF_WHAT_YOU_HAVE_DONE;
	
	@ClientString(id = 545701, message = "Kefensis' Hallucination (Lv. 99-99)")
	public static NpcStringId KEFENSIS_HALLUCINATION_LV_99_99;
	
	@ClientString(id = 545702, message = "Kefensis' Hallucination (Lv. 99-99) (In Progress)")
	public static NpcStringId KEFENSIS_HALLUCINATION_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 545703, message = "Kefensis' Hallucination (Lv. 99-99) (Complete)")
	public static NpcStringId KEFENSIS_HALLUCINATION_LV_99_99_COMPLETE;
	
	@ClientString(id = 545704, message = "Kefensis' Hallucination (Lv. 99-99)")
	public static NpcStringId KEFENSIS_HALLUCINATION_LV_99_992;
	
	@ClientString(id = 545711, message = "Test of the Desert Thirst")
	public static NpcStringId TEST_OF_THE_DESERT_THIRST;
	
	@ClientString(id = 545712, message = "Meeting with the Phantom Monster")
	public static NpcStringId MEETING_WITH_THE_PHANTOM_MONSTER;
	
	@ClientString(id = 545713, message = "Defeat the Phantom Monster")
	public static NpcStringId DEFEAT_THE_PHANTOM_MONSTER;
	
	@ClientString(id = 545721, message = "You should go with trustworthy comrades!")
	public static NpcStringId YOU_SHOULD_GO_WITH_TRUSTWORTHY_COMRADES;
	
	@ClientString(id = 545722, message = "An illusion? This place is truly worthy of being called the Mirage Hills.")
	public static NpcStringId AN_ILLUSION_THIS_PLACE_IS_TRULY_WORTHY_OF_BEING_CALLED_THE_MIRAGE_HILLS;
	
	@ClientString(id = 545801, message = "To the Desert Quarry (Lv. 99-99)")
	public static NpcStringId TO_THE_DESERT_QUARRY_LV_99_99;
	
	@ClientString(id = 545802, message = "To the Desert Quarry (Lv. 99-99) (In Progress)")
	public static NpcStringId TO_THE_DESERT_QUARRY_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 545803, message = "To the Desert Quarry (Lv. 99-99) (Complete)")
	public static NpcStringId TO_THE_DESERT_QUARRY_LV_99_99_COMPLETE;
	
	@ClientString(id = 545804, message = "To the Desert Quarry (Lv. 99-99)")
	public static NpcStringId TO_THE_DESERT_QUARRY_LV_99_992;
	
	@ClientString(id = 545811, message = "Defeat the Evolved Monster")
	public static NpcStringId DEFEAT_THE_EVOLVED_MONSTER;
	
	@ClientString(id = 545821, message = "Can you do this much? Well? Get moving!")
	public static NpcStringId CAN_YOU_DO_THIS_MUCH_WELL_GET_MOVING;
	
	@ClientString(id = 545822, message = "Should I get started?")
	public static NpcStringId SHOULD_I_GET_STARTED;
	
	@ClientString(id = 545901, message = "A Sick Ambition (Lv. 99-99)")
	public static NpcStringId A_SICK_AMBITION_LV_99_99;
	
	@ClientString(id = 545902, message = "A Sick Ambition (Lv. 99-99) (In Progress)")
	public static NpcStringId A_SICK_AMBITION_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 545903, message = "A Sick Ambition (Lv. 99-99) (Complete)")
	public static NpcStringId A_SICK_AMBITION_LV_99_99_COMPLETE;
	
	@ClientString(id = 545904, message = "A Sick Ambition (Lv. 99-99)")
	public static NpcStringId A_SICK_AMBITION_LV_99_992;
	
	@ClientString(id = 545921, message = "Please defeat Beleth. We must save Hellbound.")
	public static NpcStringId PLEASE_DEFEAT_BELETH_WE_MUST_SAVE_HELLBOUND;
	
	@ClientString(id = 546001, message = "Return of the Alligator Hunter (Lv. 40-46)")
	public static NpcStringId RETURN_OF_THE_ALLIGATOR_HUNTER_LV_40_46;
	
	@ClientString(id = 546002, message = "Return of the Alligator Hunter (Lv. 40-46) (In Progress)")
	public static NpcStringId RETURN_OF_THE_ALLIGATOR_HUNTER_LV_40_46_IN_PROGRESS;
	
	@ClientString(id = 546003, message = "Return of the Alligator Hunter (Lv. 40-46) (Complete)")
	public static NpcStringId RETURN_OF_THE_ALLIGATOR_HUNTER_LV_40_46_COMPLETE;
	
	@ClientString(id = 546004, message = "Return of the Alligator Hunter (Lv. 40-46)")
	public static NpcStringId RETURN_OF_THE_ALLIGATOR_HUNTER_LV_40_462;
	
	@ClientString(id = 546101, message = "Tapping the Power Within (Lv. 99-99)")
	public static NpcStringId TAPPING_THE_POWER_WITHIN_LV_99_99;
	
	@ClientString(id = 546102, message = "Tapping the Power Within (Lv. 99-99) (In Progress)")
	public static NpcStringId TAPPING_THE_POWER_WITHIN_LV_99_99_IN_PROGRESS;
	
	@ClientString(id = 546103, message = "Tapping the Power Within (Lv. 99-99) (Complete)")
	public static NpcStringId TAPPING_THE_POWER_WITHIN_LV_99_99_COMPLETE;
	
	@ClientString(id = 546104, message = "Tapping the Power Within (Lv. 99-99)")
	public static NpcStringId TAPPING_THE_POWER_WITHIN_LV_99_992;
	
	@ClientString(id = 546121, message = "Does your fate not weigh upon you?")
	public static NpcStringId DOES_YOUR_FATE_NOT_WEIGH_UPON_YOU;
	
	@ClientString(id = 546122, message = "This is just the beginning.")
	public static NpcStringId THIS_IS_JUST_THE_BEGINNING;
	
	@ClientString(id = 546123, message = "Open the ability screen in the character information screen.nPress 'Adjust Points' to adjust the acquired SP and Ability points.")
	public static NpcStringId OPEN_THE_ABILITY_SCREEN_IN_THE_CHARACTER_INFORMATION_SCREEN_NPRESS_ADJUST_POINTS_TO_ADJUST_THE_ACQUIRED_SP_AND_ABILITY_POINTS;
	
	@ClientString(id = 546151, message = "The letter from Lionel Hunter has arrived.nClick the question mark icon to read the letter's contents.")
	public static NpcStringId THE_LETTER_FROM_LIONEL_HUNTER_HAS_ARRIVED_NCLICK_THE_QUESTION_MARK_ICON_TO_READ_THE_LETTER_S_CONTENTS;
	
	@ClientString(id = 546201, message = "Temper a Rusting Blade (Lv. 46-52)")
	public static NpcStringId TEMPER_A_RUSTING_BLADE_LV_46_52;
	
	@ClientString(id = 546202, message = "Temper a Rusting Blade (Lv. 46-52) (In Progress)")
	public static NpcStringId TEMPER_A_RUSTING_BLADE_LV_46_52_IN_PROGRESS;
	
	@ClientString(id = 546203, message = "Temper a Rusting Blade (Lv. 46-52) (Complete)")
	public static NpcStringId TEMPER_A_RUSTING_BLADE_LV_46_52_COMPLETE;
	
	@ClientString(id = 546204, message = "Temper a Rusting Blade (Lv. 46-52)")
	public static NpcStringId TEMPER_A_RUSTING_BLADE_LV_46_522;
	
	@ClientString(id = 546301, message = "The Soul of a Sword (Lv. 52-58)")
	public static NpcStringId THE_SOUL_OF_A_SWORD_LV_52_58;
	
	@ClientString(id = 546302, message = "The Soul of a Sword (Lv. 52-58) (In Progress)")
	public static NpcStringId THE_SOUL_OF_A_SWORD_LV_52_58_IN_PROGRESS;
	
	@ClientString(id = 546303, message = "The Soul of a Sword (Lv. 52-58) (Complete)")
	public static NpcStringId THE_SOUL_OF_A_SWORD_LV_52_58_COMPLETE;
	
	@ClientString(id = 546304, message = "The Soul of a Sword (Lv. 52-58)")
	public static NpcStringId THE_SOUL_OF_A_SWORD_LV_52_582;
	
	@ClientString(id = 546401, message = "Be Prepared for Anything (Lv. 58-65)")
	public static NpcStringId BE_PREPARED_FOR_ANYTHING_LV_58_65;
	
	@ClientString(id = 546402, message = "Be Prepared for Anything (Lv. 58-65) (In Progress)")
	public static NpcStringId BE_PREPARED_FOR_ANYTHING_LV_58_65_IN_PROGRESS;
	
	@ClientString(id = 546403, message = "Be Prepared for Anything (Lv. 58-65) (Complete)")
	public static NpcStringId BE_PREPARED_FOR_ANYTHING_LV_58_65_COMPLETE;
	
	@ClientString(id = 546404, message = "Be Prepared for Anything (Lv. 58-65)")
	public static NpcStringId BE_PREPARED_FOR_ANYTHING_LV_58_652;
	
	@ClientString(id = 547201, message = "Winds of Fate: Encroaching Shadows (Lv. 85-99)")
	public static NpcStringId WINDS_OF_FATE_ENCROACHING_SHADOWS_LV_85_99;
	
	@ClientString(id = 547202, message = "Winds of Fate: Encroaching Shadows (Lv. 85-99) (In Progress)")
	public static NpcStringId WINDS_OF_FATE_ENCROACHING_SHADOWS_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 547203, message = "Winds of Fate: Encroaching Shadows (Lv. 85-99) (Done)")
	public static NpcStringId WINDS_OF_FATE_ENCROACHING_SHADOWS_LV_85_99_DONE;
	
	@ClientString(id = 547204, message = "Winds of Fate: Encroaching Shadows (Lv. 85-99)")
	public static NpcStringId WINDS_OF_FATE_ENCROACHING_SHADOWS_LV_85_992;
	
	@ClientString(id = 547211, message = "Talk to Zephyra")
	public static NpcStringId TALK_TO_ZEPHYRA;
	
	@ClientString(id = 547212, message = "Talk to Momet")
	public static NpcStringId TALK_TO_MOMET;
	
	@ClientString(id = 547213, message = "Talk to the Black Marketeer of Mammon")
	public static NpcStringId TALK_TO_THE_BLACK_MARKETEER_OF_MAMMON;
	
	@ClientString(id = 547214, message = "Talk to the Blacksmith of Mammon")
	public static NpcStringId TALK_TO_THE_BLACKSMITH_OF_MAMMON;
	
	@ClientString(id = 547215, message = "Track Nidrah")
	public static NpcStringId TRACK_NIDRAH;
	
	@ClientString(id = 547216, message = "Wynn Summoner: Master Summoner")
	public static NpcStringId WYNN_SUMMONER_MASTER_SUMMONER;
	
	@ClientString(id = 547217, message = "Feoh Wizard: Master Wizard")
	public static NpcStringId FEOH_WIZARD_MASTER_WIZARD;
	
	@ClientString(id = 547218, message = "Tyrr Warrior: Weapon/Force Master")
	public static NpcStringId TYRR_WARRIOR_WEAPON_FORCE_MASTER;
	
	@ClientString(id = 547219, message = "Othell Rogue: Dagger Master")
	public static NpcStringId OTHELL_ROGUE_DAGGER_MASTER;
	
	@ClientString(id = 547220, message = "Iss Enchanter: Buff Master")
	public static NpcStringId ISS_ENCHANTER_BUFF_MASTER;
	
	@ClientString(id = 547221, message = "Yul Archer: Bow/Crossbow Master")
	public static NpcStringId YUL_ARCHER_BOW_CROSSBOW_MASTER;
	
	@ClientString(id = 547222, message = "Sigel Knight: Master Defender")
	public static NpcStringId SIGEL_KNIGHT_MASTER_DEFENDER;
	
	@ClientString(id = 547223, message = "Aeore Healer: Master Healer")
	public static NpcStringId AEORE_HEALER_MASTER_HEALER;
	
	@ClientString(id = 547224, message = "Talk to Hardin")
	public static NpcStringId TALK_TO_HARDIN;
	
	@ClientString(id = 547225, message = "Talk to Raina")
	public static NpcStringId TALK_TO_RAINA;
	
	@ClientString(id = 550101, message = "Embroidered on a Cloak, Part 1: Zaken's Soul")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_1_ZAKEN_S_SOUL;
	
	@ClientString(id = 550102, message = "Embroidered on a Cloak, Part 1: Zaken's Soul (In Progress)")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_1_ZAKEN_S_SOUL_IN_PROGRESS;
	
	@ClientString(id = 550103, message = "Embroidered on a Cloak, Part 1: Zaken's Soul (Done)")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_1_ZAKEN_S_SOUL_DONE;
	
	@ClientString(id = 550104, message = "Embroidered on a Cloak, Part 1: Zaken's Soul")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_1_ZAKEN_S_SOUL2;
	
	@ClientString(id = 550201, message = "Embroidered on a Cloak, Part 2: Freya's Soul")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_2_FREYA_S_SOUL;
	
	@ClientString(id = 550202, message = "Embroidered on a Cloak, Part 2: Freya's Soul (In Progress)")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_2_FREYA_S_SOUL_IN_PROGRESS;
	
	@ClientString(id = 550203, message = "Embroidered on a Cloak, Part 2: Freya's Soul (Done)")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_2_FREYA_S_SOUL_DONE;
	
	@ClientString(id = 550204, message = "Embroidered on a Cloak, Part 2: Freya's Soul")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_2_FREYA_S_SOUL2;
	
	@ClientString(id = 550301, message = "Embroidered on a Cloak, Part 3: Frintezza's Soul")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_3_FRINTEZZA_S_SOUL;
	
	@ClientString(id = 550302, message = "Embroidered on a Cloak, Part 3: Frintezza's Soul (In Progress)")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_3_FRINTEZZA_S_SOUL_IN_PROGRESS;
	
	@ClientString(id = 550303, message = "Embroidered on a Cloak, Part 3: Frintezza's Soul (Done)")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_3_FRINTEZZA_S_SOUL_DONE;
	
	@ClientString(id = 550304, message = "Embroidered on a Cloak, Part 3: Frintezza's Soul")
	public static NpcStringId EMBROIDERED_ON_A_CLOAK_PART_3_FRINTEZZA_S_SOUL2;
	
	@ClientString(id = 550401, message = "Jewel of Antharas (Lv. 84-99)")
	public static NpcStringId JEWEL_OF_ANTHARAS_LV_84_99;
	
	@ClientString(id = 550402, message = "Jewel of Antharas (Lv. 84-99) (In Progress)")
	public static NpcStringId JEWEL_OF_ANTHARAS_LV_84_99_IN_PROGRESS;
	
	@ClientString(id = 550403, message = "Jewel of Antharas (Lv. 84-99) (Done)")
	public static NpcStringId JEWEL_OF_ANTHARAS_LV_84_99_DONE;
	
	@ClientString(id = 550404, message = "Jewel of Antharas (Lv. 84-99)")
	public static NpcStringId JEWEL_OF_ANTHARAS_LV_84_992;
	
	@ClientString(id = 550501, message = "Jewel of Valakas (Lv. 84-99)")
	public static NpcStringId JEWEL_OF_VALAKAS_LV_84_99;
	
	@ClientString(id = 550502, message = "Jewel of Valakas (Lv. 84-99) (In Progress)")
	public static NpcStringId JEWEL_OF_VALAKAS_LV_84_99_IN_PROGRESS;
	
	@ClientString(id = 550503, message = "Jewel of Valakas (Lv. 84-99) (Done)")
	public static NpcStringId JEWEL_OF_VALAKAS_LV_84_99_DONE;
	
	@ClientString(id = 550504, message = "Jewel of Valakas (Lv. 84-99)")
	public static NpcStringId JEWEL_OF_VALAKAS_LV_84_992;
	
	@ClientString(id = 550601, message = "Paddy's Request")
	public static NpcStringId PADDY_S_REQUEST;
	
	@ClientString(id = 550602, message = "Paddy's Request (In Progress)")
	public static NpcStringId PADDY_S_REQUEST_IN_PROGRESS;
	
	@ClientString(id = 550603, message = "Paddy's Request (Done)")
	public static NpcStringId PADDY_S_REQUEST_DONE;
	
	@ClientString(id = 550604, message = "Paddy's Request")
	public static NpcStringId PADDY_S_REQUEST2;
	
	@ClientString(id = 550701, message = "Paddy's Request")
	public static NpcStringId PADDY_S_REQUEST3;
	
	@ClientString(id = 550702, message = "Paddy's Request (In Progress)")
	public static NpcStringId PADDY_S_REQUEST_IN_PROGRESS2;
	
	@ClientString(id = 550703, message = "Paddy's Request (Done)")
	public static NpcStringId PADDY_S_REQUEST_DONE2;
	
	@ClientString(id = 550704, message = "Paddy's Request")
	public static NpcStringId PADDY_S_REQUEST4;
	
	@ClientString(id = 550801, message = "Paddy's Request")
	public static NpcStringId PADDY_S_REQUEST5;
	
	@ClientString(id = 550802, message = "Paddy's Request (In Progress)")
	public static NpcStringId PADDY_S_REQUEST_IN_PROGRESS3;
	
	@ClientString(id = 550803, message = "Paddy's Request (Done)")
	public static NpcStringId PADDY_S_REQUEST_DONE3;
	
	@ClientString(id = 550804, message = "Paddy's Request")
	public static NpcStringId PADDY_S_REQUEST6;
	
	@ClientString(id = 570101, message = "The Road to Destruction (Lv. 93-99)")
	public static NpcStringId THE_ROAD_TO_DESTRUCTION_LV_93_99;
	
	@ClientString(id = 570102, message = "The Road to Destruction (Lv. 93-99) (In Progress)")
	public static NpcStringId THE_ROAD_TO_DESTRUCTION_LV_93_99_IN_PROGRESS;
	
	@ClientString(id = 570103, message = "The Road to Destruction (Lv. 93-99) (Complete)")
	public static NpcStringId THE_ROAD_TO_DESTRUCTION_LV_93_99_COMPLETE;
	
	@ClientString(id = 570104, message = "The Road to Destruction (Lv. 93-99)")
	public static NpcStringId THE_ROAD_TO_DESTRUCTION_LV_93_992;
	
	@ClientString(id = 570201, message = "The Road to Infinity (Lv. 95-99)")
	public static NpcStringId THE_ROAD_TO_INFINITY_LV_95_99;
	
	@ClientString(id = 570202, message = "The Road to Infinity (Lv. 95-99) (In Progress)")
	public static NpcStringId THE_ROAD_TO_INFINITY_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 570203, message = "The Road to Infinity (Lv. 95-99) (Complete)")
	public static NpcStringId THE_ROAD_TO_INFINITY_LV_95_99_COMPLETE;
	
	@ClientString(id = 570204, message = "The Road to Infinity (Lv. 95-99)")
	public static NpcStringId THE_ROAD_TO_INFINITY_LV_95_992;
	
	@ClientString(id = 570301, message = "Bottle of Istina's Soul (Lv. 97-99)")
	public static NpcStringId BOTTLE_OF_ISTINA_S_SOUL_LV_97_99;
	
	@ClientString(id = 570302, message = "Bottle of Istina's Soul (Lv. 97-99) (In Progress)")
	public static NpcStringId BOTTLE_OF_ISTINA_S_SOUL_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 570303, message = "Bottle of Istina's Soul (Lv. 97-99) (Complete)")
	public static NpcStringId BOTTLE_OF_ISTINA_S_SOUL_LV_97_99_COMPLETE;
	
	@ClientString(id = 570304, message = "Bottle of Istina's Soul (Lv. 97-99)")
	public static NpcStringId BOTTLE_OF_ISTINA_S_SOUL_LV_97_992;
	
	@ClientString(id = 570401, message = "Bottle of Octavis's Soul (Lv. 95-99)")
	public static NpcStringId BOTTLE_OF_OCTAVIS_S_SOUL_LV_95_99;
	
	@ClientString(id = 570402, message = "Bottle of Octavis's Soul (Lv. 95-99) (In Progress)")
	public static NpcStringId BOTTLE_OF_OCTAVIS_S_SOUL_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 570403, message = "Bottle of Octavis's Soul (Lv. 95-99) (Complete)")
	public static NpcStringId BOTTLE_OF_OCTAVIS_S_SOUL_LV_95_99_COMPLETE;
	
	@ClientString(id = 570404, message = "Bottle of Octavis's Soul (Lv. 95-99)")
	public static NpcStringId BOTTLE_OF_OCTAVIS_S_SOUL_LV_95_992;
	
	@ClientString(id = 570501, message = "Bottle of Tauti's Soul (Lv. 97-99)")
	public static NpcStringId BOTTLE_OF_TAUTI_S_SOUL_LV_97_99;
	
	@ClientString(id = 570502, message = "Bottle of Tauti's Soul (Lv. 97-99) (In Progress)")
	public static NpcStringId BOTTLE_OF_TAUTI_S_SOUL_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 570503, message = "Bottle of Tauti's Soul (Lv. 97-99) (Complete)")
	public static NpcStringId BOTTLE_OF_TAUTI_S_SOUL_LV_97_99_COMPLETE;
	
	@ClientString(id = 570504, message = "Bottle of Tauti's Soul (Lv. 97-99)")
	public static NpcStringId BOTTLE_OF_TAUTI_S_SOUL_LV_97_992;
	
	@ClientString(id = 570701, message = "Flames of Sorrow (Lv. 46-51)")
	public static NpcStringId FLAMES_OF_SORROW_LV_46_51;
	
	@ClientString(id = 570702, message = "Flames of Sorrow (Lv. 46-51) (In Progress)")
	public static NpcStringId FLAMES_OF_SORROW_LV_46_51_IN_PROGRESS;
	
	@ClientString(id = 570703, message = "Flames of Sorrow (Lv. 46-51) (Done)")
	public static NpcStringId FLAMES_OF_SORROW_LV_46_51_DONE;
	
	@ClientString(id = 570704, message = "Flames of Sorrow (Lv. 46-51)")
	public static NpcStringId FLAMES_OF_SORROW_LV_46_512;
	
	@ClientString(id = 570801, message = "Strengthen the Barrier (Lv. 52-57)")
	public static NpcStringId STRENGTHEN_THE_BARRIER_LV_52_57;
	
	@ClientString(id = 570802, message = "Strengthen the Barrier (Lv. 52-57) (In Progress)")
	public static NpcStringId STRENGTHEN_THE_BARRIER_LV_52_57_IN_PROGRESS;
	
	@ClientString(id = 570803, message = "Strengthen the Barrier (Lv. 52-57) (Done)")
	public static NpcStringId STRENGTHEN_THE_BARRIER_LV_52_57_DONE;
	
	@ClientString(id = 570804, message = "Strengthen the Barrier (Lv. 52-57)")
	public static NpcStringId STRENGTHEN_THE_BARRIER_LV_52_572;
	
	@ClientString(id = 570811, message = "Activate the Barrier Enforcer")
	public static NpcStringId ACTIVATE_THE_BARRIER_ENFORCER;
	
	@ClientString(id = 570901, message = "The Stolen Seed (Lv. 58-61)")
	public static NpcStringId THE_STOLEN_SEED_LV_58_61;
	
	@ClientString(id = 570902, message = "The Stolen Seed (Lv. 58-61) (In Progress)")
	public static NpcStringId THE_STOLEN_SEED_LV_58_61_IN_PROGRESS;
	
	@ClientString(id = 570903, message = "The Stolen Seed (Lv. 58-61) (Complete)")
	public static NpcStringId THE_STOLEN_SEED_LV_58_61_COMPLETE;
	
	@ClientString(id = 570904, message = "The Stolen Seed (Lv. 58-61)")
	public static NpcStringId THE_STOLEN_SEED_LV_58_612;
	
	@ClientString(id = 570911, message = "Activate the Magic Circle Control Device")
	public static NpcStringId ACTIVATE_THE_MAGIC_CIRCLE_CONTROL_DEVICE;
	
	@ClientString(id = 571001, message = "Life Energy Repository (Lv. 61-65)")
	public static NpcStringId LIFE_ENERGY_REPOSITORY_LV_61_65;
	
	@ClientString(id = 571002, message = "Life Energy Repository (Lv. 61-65) (In Progress)")
	public static NpcStringId LIFE_ENERGY_REPOSITORY_LV_61_65_IN_PROGRESS;
	
	@ClientString(id = 571003, message = "Life Energy Repository (Lv. 61-65) (Complete)")
	public static NpcStringId LIFE_ENERGY_REPOSITORY_LV_61_65_COMPLETE;
	
	@ClientString(id = 571004, message = "Life Energy Repository (Lv. 61-65)")
	public static NpcStringId LIFE_ENERGY_REPOSITORY_LV_61_652;
	
	@ClientString(id = 571011, message = "Find the Life Energy Repository")
	public static NpcStringId FIND_THE_LIFE_ENERGY_REPOSITORY;
	
	@ClientString(id = 571200, message = "A minstrel has sent an invitation.nClick the question-mark icon to read.")
	public static NpcStringId A_MINSTREL_HAS_SENT_AN_INVITATION_NCLICK_THE_QUESTION_MARK_ICON_TO_READ;
	
	@ClientString(id = 571201, message = "The Minstrel's Song, Part 1 (Lv. 85-87)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_1_LV_85_87;
	
	@ClientString(id = 571202, message = "The Minstrel's Song, Part 1 (Lv. 85-87) (In Progress)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_1_LV_85_87_IN_PROGRESS;
	
	@ClientString(id = 571203, message = "The Minstrel's Song, Part 1 (Lv. 85-87) (Done)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_1_LV_85_87_DONE;
	
	@ClientString(id = 571204, message = "The Minstrel's Song, Part 1 (Lv. 85-87)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_1_LV_85_872;
	
	@ClientString(id = 571211, message = "Talk to Liberatto")
	public static NpcStringId TALK_TO_LIBERATTO;
	
	@ClientString(id = 571301, message = "The Hero's Journey: Bloody Swampland (Lv. 85-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_BLOODY_SWAMPLAND_LV_85_99;
	
	@ClientString(id = 571302, message = "The Hero's Journey: Bloody Swampland (Lv. 85-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_BLOODY_SWAMPLAND_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 571303, message = "The Hero's Journey: Bloody Swampland (Lv. 85-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_BLOODY_SWAMPLAND_LV_85_99_DONE;
	
	@ClientString(id = 571304, message = "The Hero's Journey: Bloody Swampland (Lv. 85-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_BLOODY_SWAMPLAND_LV_85_992;
	
	@ClientString(id = 571311, message = "Talk to Liberatto")
	public static NpcStringId TALK_TO_LIBERATTO2;
	
	@ClientString(id = 571312, message = "Talk to Vollodos")
	public static NpcStringId TALK_TO_VOLLODOS;
	
	@ClientString(id = 571313, message = "Talk to Vollodos")
	public static NpcStringId TALK_TO_VOLLODOS2;
	
	@ClientString(id = 571401, message = "The Hero's Journey: Seed of Annihilation (Lv. 85-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_SEED_OF_ANNIHILATION_LV_85_99;
	
	@ClientString(id = 571402, message = "The Hero's Journey: Seed of Annihilation (Lv. 85-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_SEED_OF_ANNIHILATION_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 571403, message = "The Hero's Journey: Seed of Annihilation (Lv. 85-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_SEED_OF_ANNIHILATION_LV_85_99_DONE;
	
	@ClientString(id = 571404, message = "The Hero's Journey: Seed of Annihilation (Lv. 85-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_SEED_OF_ANNIHILATION_LV_85_992;
	
	@ClientString(id = 571411, message = "Talk to Liberatto")
	public static NpcStringId TALK_TO_LIBERATTO3;
	
	@ClientString(id = 571412, message = "Talk to Klemis")
	public static NpcStringId TALK_TO_KLEMIS;
	
	@ClientString(id = 571413, message = "Talk to Klemis")
	public static NpcStringId TALK_TO_KLEMIS2;
	
	@ClientString(id = 571501, message = "The Hero's Journey: Harnak Underground Ruins (Lv. 85-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_HARNAK_UNDERGROUND_RUINS_LV_85_99;
	
	@ClientString(id = 571502, message = "The Hero's Journey: Harnak Underground Ruins (Lv. 85-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_HARNAK_UNDERGROUND_RUINS_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 571503, message = "The Hero's Journey: Harnak Underground Ruins (Lv. 85-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_HARNAK_UNDERGROUND_RUINS_LV_85_99_DONE;
	
	@ClientString(id = 571504, message = "The Hero's Journey: Harnak Underground Ruins (Lv. 85-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_HARNAK_UNDERGROUND_RUINS_LV_85_992;
	
	@ClientString(id = 571511, message = "Talk to Liberatto")
	public static NpcStringId TALK_TO_LIBERATTO4;
	
	@ClientString(id = 571512, message = "Talk to Hadel")
	public static NpcStringId TALK_TO_HADEL;
	
	@ClientString(id = 571513, message = "Talk to Hadel")
	public static NpcStringId TALK_TO_HADEL2;
	
	@ClientString(id = 571601, message = "The Hero's Journey: Kartia's Labyrinth (Lv. 85-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_KARTIA_S_LABYRINTH_LV_85_99;
	
	@ClientString(id = 571602, message = "The Hero's Journey: Kartia's Labyrinth (Lv. 85-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_KARTIA_S_LABYRINTH_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 571603, message = "The Hero's Journey: Kartia's Labyrinth (Lv. 85-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_KARTIA_S_LABYRINTH_LV_85_99_DONE;
	
	@ClientString(id = 571604, message = "The Hero's Journey: Kartia's Labyrinth (Lv. 85-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_KARTIA_S_LABYRINTH_LV_85_992;
	
	@ClientString(id = 571611, message = "Talk to Liberatto")
	public static NpcStringId TALK_TO_LIBERATTO5;
	
	@ClientString(id = 571612, message = "Talk to Kartia Researcher")
	public static NpcStringId TALK_TO_KARTIA_RESEARCHER;
	
	@ClientString(id = 571613, message = "Talk to Kartia Researcher")
	public static NpcStringId TALK_TO_KARTIA_RESEARCHER2;
	
	@ClientString(id = 571701, message = "The Minstrel's Song, Part 2 (Lv. 88-89)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_2_LV_88_89;
	
	@ClientString(id = 571702, message = "The Minstrel's Song, Part 2 (Lv. 88-89) (In Progress)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_2_LV_88_89_IN_PROGRESS;
	
	@ClientString(id = 571703, message = "The Minstrel's Song, Part 2 (Lv. 88-89) (Done)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_2_LV_88_89_DONE;
	
	@ClientString(id = 571704, message = "The Minstrel's Song, Part 2 (Lv. 88-89)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_2_LV_88_892;
	
	@ClientString(id = 571711, message = "Talk to Miso")
	public static NpcStringId TALK_TO_MISO;
	
	@ClientString(id = 571801, message = "The Hero's Journey: Fairy Settlement (Lv. 88-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_FAIRY_SETTLEMENT_LV_88_99;
	
	@ClientString(id = 571802, message = "The Hero's Journey: Fairy Settlement (Lv. 88-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_FAIRY_SETTLEMENT_LV_88_99_IN_PROGRESS;
	
	@ClientString(id = 571803, message = "The Hero's Journey: Fairy Settlement (Lv. 88-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_FAIRY_SETTLEMENT_LV_88_99_DONE;
	
	@ClientString(id = 571804, message = "The Hero's Journey: Fairy Settlement (Lv. 88-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_FAIRY_SETTLEMENT_LV_88_992;
	
	@ClientString(id = 571811, message = "Talk to Miso")
	public static NpcStringId TALK_TO_MISO2;
	
	@ClientString(id = 571812, message = "Talk to Fairy Refugee")
	public static NpcStringId TALK_TO_FAIRY_REFUGEE;
	
	@ClientString(id = 571813, message = "Talk to Fairy Refugee")
	public static NpcStringId TALK_TO_FAIRY_REFUGEE2;
	
	@ClientString(id = 572001, message = "The Minstrel's Song, Part 3 (Lv. 90-91)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_3_LV_90_91;
	
	@ClientString(id = 572002, message = "The Minstrel's Song, Part 3 (Lv. 90-91) (In Progress)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_3_LV_90_91_IN_PROGRESS;
	
	@ClientString(id = 572003, message = "The Minstrel's Song, Part 3 (Lv. 90-91) (Done)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_3_LV_90_91_DONE;
	
	@ClientString(id = 572004, message = "The Minstrel's Song, Part 3 (Lv. 90-91)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_3_LV_90_912;
	
	@ClientString(id = 572011, message = "Talk to Miso")
	public static NpcStringId TALK_TO_MISO3;
	
	@ClientString(id = 572101, message = "The Hero's Journey: Prison of Darkness (Lv. 90-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_PRISON_OF_DARKNESS_LV_90_99;
	
	@ClientString(id = 572102, message = "The Hero's Journey: Prison of Darkness (Lv. 90-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_PRISON_OF_DARKNESS_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 572103, message = "The Hero's Journey: Prison of Darkness (Lv. 90-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_PRISON_OF_DARKNESS_LV_90_99_DONE;
	
	@ClientString(id = 572104, message = "The Hero's Journey: Prison of Darkness (Lv. 90-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_PRISON_OF_DARKNESS_LV_90_992;
	
	@ClientString(id = 572111, message = "Talk to Miso")
	public static NpcStringId TALK_TO_MISO4;
	
	@ClientString(id = 572112, message = "Talk to Opera")
	public static NpcStringId TALK_TO_OPERA;
	
	@ClientString(id = 572113, message = "Talk to Opera")
	public static NpcStringId TALK_TO_OPERA2;
	
	@ClientString(id = 572201, message = "The Hero's Journey: Fortuna (Lv. 90-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_FORTUNA_LV_90_99;
	
	@ClientString(id = 572202, message = "The Hero's Journey: Fortuna (Lv. 90-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_FORTUNA_LV_90_99_IN_PROGRESS;
	
	@ClientString(id = 572203, message = "The Hero's Journey: Fortuna (Lv. 90-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_FORTUNA_LV_90_99_DONE;
	
	@ClientString(id = 572204, message = "The Hero's Journey: Fortuna (Lv. 90-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_FORTUNA_LV_90_992;
	
	@ClientString(id = 572211, message = "Talk to Miso")
	public static NpcStringId TALK_TO_MISO5;
	
	@ClientString(id = 572212, message = "Talk to Izael")
	public static NpcStringId TALK_TO_IZAEL;
	
	@ClientString(id = 572213, message = "Talk to Izael")
	public static NpcStringId TALK_TO_IZAEL2;
	
	@ClientString(id = 572301, message = "The Minstrel's Song, Part 4 (Lv. 92-94)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_4_LV_92_94;
	
	@ClientString(id = 572302, message = "The Minstrel's Song, Part 4 (Lv. 92-94) (In Progress)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_4_LV_92_94_IN_PROGRESS;
	
	@ClientString(id = 572303, message = "The Minstrel's Song, Part 4 (Lv. 92-94) (Done)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_4_LV_92_94_DONE;
	
	@ClientString(id = 572304, message = "The Minstrel's Song, Part 4 (Lv. 92-94)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_4_LV_92_942;
	
	@ClientString(id = 572311, message = "Talk to Marmie")
	public static NpcStringId TALK_TO_MARMIE;
	
	@ClientString(id = 572401, message = "The Hero's Journey: Isle of Souls (Lv. 92-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_ISLE_OF_SOULS_LV_92_99;
	
	@ClientString(id = 572402, message = "The Hero's Journey: Isle of Souls (Lv. 92-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_ISLE_OF_SOULS_LV_92_99_IN_PROGRESS;
	
	@ClientString(id = 572403, message = "The Hero's Journey: Isle of Souls (Lv. 92-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_ISLE_OF_SOULS_LV_92_99_DONE;
	
	@ClientString(id = 572404, message = "The Hero's Journey: Isle of Souls (Lv. 92-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_ISLE_OF_SOULS_LV_92_992;
	
	@ClientString(id = 572411, message = "Talk to Marmie")
	public static NpcStringId TALK_TO_MARMIE2;
	
	@ClientString(id = 572412, message = "Talk to Tauresia")
	public static NpcStringId TALK_TO_TAURESIA;
	
	@ClientString(id = 572413, message = "Talk to Tauresia")
	public static NpcStringId TALK_TO_TAURESIA2;
	
	@ClientString(id = 572501, message = "The Hero's Journey: Nornil's Cave (Lv. 92-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_NORNIL_S_CAVE_LV_92_99;
	
	@ClientString(id = 572502, message = "The Hero's Journey: Nornil's Cave (Lv. 92-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_NORNIL_S_CAVE_LV_92_99_IN_PROGRESS;
	
	@ClientString(id = 572503, message = "The Hero's Journey: Nornil's Cave (Lv. 92-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_NORNIL_S_CAVE_LV_92_99_DONE;
	
	@ClientString(id = 572504, message = "The Hero's Journey: Nornil's Cave (Lv. 92-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_NORNIL_S_CAVE_LV_92_992;
	
	@ClientString(id = 572511, message = "Talk to Marmie")
	public static NpcStringId TALK_TO_MARMIE3;
	
	@ClientString(id = 572512, message = "Talk to Verna")
	public static NpcStringId TALK_TO_VERNA;
	
	@ClientString(id = 572513, message = "Talk to Verna")
	public static NpcStringId TALK_TO_VERNA2;
	
	@ClientString(id = 572601, message = "The Minstrel's Song, Part 5 (Lv. 95-96)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_5_LV_95_96;
	
	@ClientString(id = 572602, message = "The Minstrel's Song, Part 5 (Lv. 95-96) (In Progress)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_5_LV_95_96_IN_PROGRESS;
	
	@ClientString(id = 572603, message = "The Minstrel's Song, Part 5 (Lv. 95-96) (Done)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_5_LV_95_96_DONE;
	
	@ClientString(id = 572604, message = "The Minstrel's Song, Part 5 (Lv. 95-96)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_5_LV_95_962;
	
	@ClientString(id = 572611, message = "Talk to Tulesir")
	public static NpcStringId TALK_TO_TULESIR;
	
	@ClientString(id = 572701, message = "The Hero's Journey: Seal of Shilen (Lv. 95-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_SEAL_OF_SHILEN_LV_95_99;
	
	@ClientString(id = 572702, message = "The Hero's Journey: Seal of Shilen (Lv. 95-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_SEAL_OF_SHILEN_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 572703, message = "The Hero's Journey: Seal of Shilen (Lv. 95-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_SEAL_OF_SHILEN_LV_95_99_DONE;
	
	@ClientString(id = 572704, message = "The Hero's Journey: Seal of Shilen (Lv. 95-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_SEAL_OF_SHILEN_LV_95_992;
	
	@ClientString(id = 572711, message = "Talk to Tulesir")
	public static NpcStringId TALK_TO_TULESIR2;
	
	@ClientString(id = 572712, message = "Talk to Aden Vanguard Member")
	public static NpcStringId TALK_TO_ADEN_VANGUARD_MEMBER;
	
	@ClientString(id = 572713, message = "Talk to Aden Vanguard Member")
	public static NpcStringId TALK_TO_ADEN_VANGUARD_MEMBER2;
	
	@ClientString(id = 572801, message = "The Hero's Journey: Cemetary (Lv. 97-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_CEMETARY_LV_97_99;
	
	@ClientString(id = 572802, message = "The Hero's Journey: Cemetary (Lv. 97-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_CEMETARY_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 572803, message = "The Hero's Journey: Cemetary (Lv. 97-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_CEMETARY_LV_97_99_DONE;
	
	@ClientString(id = 572804, message = "The Hero's Journey: Cemetary (Lv. 97-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_CEMETARY_LV_97_992;
	
	@ClientString(id = 572811, message = "Talk to Tulesir")
	public static NpcStringId TALK_TO_TULESIR3;
	
	@ClientString(id = 572812, message = "Talk to Aden Vanguard Member")
	public static NpcStringId TALK_TO_ADEN_VANGUARD_MEMBER3;
	
	@ClientString(id = 572813, message = "Talk to Aden Vanguard Member")
	public static NpcStringId TALK_TO_ADEN_VANGUARD_MEMBER4;
	
	@ClientString(id = 572901, message = "The Hero's Journey: Guillotine Fortress (Lv. 95-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_GUILLOTINE_FORTRESS_LV_95_99;
	
	@ClientString(id = 572902, message = "The Hero's Journey: Guillotine Fortress (Lv. 95-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_GUILLOTINE_FORTRESS_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 572903, message = "The Hero's Journey: Guillotine Fortress (Lv. 95-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_GUILLOTINE_FORTRESS_LV_95_99_DONE;
	
	@ClientString(id = 572904, message = "The Hero's Journey: Guillotine Fortress (Lv. 95-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_GUILLOTINE_FORTRESS_LV_95_992;
	
	@ClientString(id = 572911, message = "Talk to Tulesir")
	public static NpcStringId TALK_TO_TULESIR4;
	
	@ClientString(id = 572912, message = "Talk to Mortania")
	public static NpcStringId TALK_TO_MORTANIA;
	
	@ClientString(id = 572913, message = "Talk to Mortania")
	public static NpcStringId TALK_TO_MORTANIA2;
	
	@ClientString(id = 573001, message = "The Hero's Journey: Orbis Temple (Lv. 95-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_ORBIS_TEMPLE_LV_95_99;
	
	@ClientString(id = 573002, message = "The Hero's Journey: Orbis Temple (Lv. 95-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_ORBIS_TEMPLE_LV_95_99_IN_PROGRESS;
	
	@ClientString(id = 573003, message = "The Hero's Journey: Orbis Temple (Lv. 95-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_ORBIS_TEMPLE_LV_95_99_DONE;
	
	@ClientString(id = 573004, message = "The Hero's Journey: Orbis Temple (Lv. 95-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_ORBIS_TEMPLE_LV_95_992;
	
	@ClientString(id = 573011, message = "Talk to Tulesir")
	public static NpcStringId TALK_TO_TULESIR5;
	
	@ClientString(id = 573012, message = "Talk to Severin")
	public static NpcStringId TALK_TO_SEVERIN;
	
	@ClientString(id = 573013, message = "Talk to Severin")
	public static NpcStringId TALK_TO_SEVERIN2;
	
	@ClientString(id = 573101, message = "The Minstrel's Song, Part 6 (Lv. 97-99)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_6_LV_97_99;
	
	@ClientString(id = 573102, message = "The Minstrel's Song, Part 6 (Lv. 97-99) (In Progress)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_6_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 573103, message = "The Minstrel's Song, Part 6 (Lv. 97-99) (Done)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_6_LV_97_99_DONE;
	
	@ClientString(id = 573104, message = "The Minstrel's Song, Part 6 (Lv. 97-99)")
	public static NpcStringId THE_MINSTREL_S_SONG_PART_6_LV_97_992;
	
	@ClientString(id = 573111, message = "Talk to Tulesir")
	public static NpcStringId TALK_TO_TULESIR6;
	
	@ClientString(id = 573201, message = "A Foreign Land (Lv. 1-20)")
	public static NpcStringId A_FOREIGN_LAND_LV_1_20;
	
	@ClientString(id = 573202, message = "A Foreign Land (Lv. 1-20) (In Progress)")
	public static NpcStringId A_FOREIGN_LAND_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 573203, message = "A Foreign Land (Lv. 1-20) (Done)")
	public static NpcStringId A_FOREIGN_LAND_LV_1_20_DONE;
	
	@ClientString(id = 573204, message = "A Foreign Land (Lv. 1-20)")
	public static NpcStringId A_FOREIGN_LAND_LV_1_202;
	
	@ClientString(id = 573205, message = "Talk to Gereth")
	public static NpcStringId TALK_TO_GERETH;
	
	@ClientString(id = 573301, message = "The Test for Survival (Lv. 1-20)")
	public static NpcStringId THE_TEST_FOR_SURVIVAL_LV_1_20;
	
	@ClientString(id = 573302, message = "The Test for Survival (Lv. 1-20) (In Progress)")
	public static NpcStringId THE_TEST_FOR_SURVIVAL_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 573303, message = "The Test for Survival (Lv. 1-20) (Done)")
	public static NpcStringId THE_TEST_FOR_SURVIVAL_LV_1_20_DONE;
	
	@ClientString(id = 573304, message = "The Test for Survival (Lv. 1-20)")
	public static NpcStringId THE_TEST_FOR_SURVIVAL_LV_1_202;
	
	@ClientString(id = 573305, message = "Talk to Katalin")
	public static NpcStringId TALK_TO_KATALIN;
	
	@ClientString(id = 573306, message = "Talk to Ayanthe")
	public static NpcStringId TALK_TO_AYANTHE;
	
	@ClientString(id = 573307, message = "Talk to Dia")
	public static NpcStringId TALK_TO_DIA;
	
	@ClientString(id = 573401, message = "Do or Die (Lv. 1-20)")
	public static NpcStringId DO_OR_DIE_LV_1_20;
	
	@ClientString(id = 573402, message = "Do or Die (Lv. 1-20) (In Progress)")
	public static NpcStringId DO_OR_DIE_LV_1_20_IN_PROGRESS;
	
	@ClientString(id = 573403, message = "Do or Die (Lv. 1-20) (Done)")
	public static NpcStringId DO_OR_DIE_LV_1_20_DONE;
	
	@ClientString(id = 573404, message = "Do or Die (Lv. 1-20)")
	public static NpcStringId DO_OR_DIE_LV_1_202;
	
	@ClientString(id = 573405, message = "Talk to Adventurer's Guide")
	public static NpcStringId TALK_TO_ADVENTURER_S_GUIDE;
	
	@ClientString(id = 573501, message = "A Special Power (Lv. 4-20)")
	public static NpcStringId A_SPECIAL_POWER_LV_4_20;
	
	@ClientString(id = 573502, message = "A Special Power (Lv. 4-20) (In Progress)")
	public static NpcStringId A_SPECIAL_POWER_LV_4_20_IN_PROGRESS;
	
	@ClientString(id = 573503, message = "A Special Power (Lv. 4-20) (Done)")
	public static NpcStringId A_SPECIAL_POWER_LV_4_20_DONE;
	
	@ClientString(id = 573504, message = "A Special Power (Lv. 4-20)")
	public static NpcStringId A_SPECIAL_POWER_LV_4_202;
	
	@ClientString(id = 573601, message = "A Special Power (Lv. 4-20)")
	public static NpcStringId A_SPECIAL_POWER_LV_4_203;
	
	@ClientString(id = 573602, message = "A Special Power (Lv. 4-20) (In Progress)")
	public static NpcStringId A_SPECIAL_POWER_LV_4_20_IN_PROGRESS2;
	
	@ClientString(id = 573603, message = "A Special Power (Lv. 4-20) (Done)")
	public static NpcStringId A_SPECIAL_POWER_LV_4_20_DONE2;
	
	@ClientString(id = 573604, message = "A Special Power (Lv. 4-20)")
	public static NpcStringId A_SPECIAL_POWER_LV_4_204;
	
	@ClientString(id = 573701, message = "Grakon's Warehouse (Lv. 5-20)")
	public static NpcStringId GRAKON_S_WAREHOUSE_LV_5_20;
	
	@ClientString(id = 573702, message = "Grakon's Warehouse (Lv. 5-20) (In Progress)")
	public static NpcStringId GRAKON_S_WAREHOUSE_LV_5_20_IN_PROGRESS;
	
	@ClientString(id = 573703, message = "Grakon's Warehouse (Lv. 5-20) (Done)")
	public static NpcStringId GRAKON_S_WAREHOUSE_LV_5_20_DONE;
	
	@ClientString(id = 573704, message = "Grakon's Warehouse (Lv. 5-20)")
	public static NpcStringId GRAKON_S_WAREHOUSE_LV_5_202;
	
	@ClientString(id = 573705, message = "Talk to Grakon")
	public static NpcStringId TALK_TO_GRAKON;
	
	@ClientString(id = 573801, message = "An Inner Beauty (Lv. 5-20)")
	public static NpcStringId AN_INNER_BEAUTY_LV_5_20;
	
	@ClientString(id = 573802, message = "An Inner Beauty (Lv. 5-20) (In Progress)")
	public static NpcStringId AN_INNER_BEAUTY_LV_5_20_IN_PROGRESS;
	
	@ClientString(id = 573803, message = "An Inner Beauty (Lv. 5-20) (Done)")
	public static NpcStringId AN_INNER_BEAUTY_LV_5_20_DONE;
	
	@ClientString(id = 573804, message = "An Inner Beauty (Lv. 5-20)")
	public static NpcStringId AN_INNER_BEAUTY_LV_5_202;
	
	@ClientString(id = 573805, message = "Talk to Evna")
	public static NpcStringId TALK_TO_EVNA;
	
	@ClientString(id = 573901, message = "Supply and Demand (Lv. 6-20)")
	public static NpcStringId SUPPLY_AND_DEMAND_LV_6_20;
	
	@ClientString(id = 573902, message = "Supply and Demand (Lv. 6-20) (In Progress)")
	public static NpcStringId SUPPLY_AND_DEMAND_LV_6_20_IN_PROGRESS;
	
	@ClientString(id = 573903, message = "Supply and Demand (Lv. 6-20) (Done)")
	public static NpcStringId SUPPLY_AND_DEMAND_LV_6_20_DONE;
	
	@ClientString(id = 573904, message = "Supply and Demand (Lv. 6-20)")
	public static NpcStringId SUPPLY_AND_DEMAND_LV_6_202;
	
	@ClientString(id = 573905, message = "Talk to Sivanthe")
	public static NpcStringId TALK_TO_SIVANTHE;
	
	@ClientString(id = 574001, message = "Never Forget (Lv. 8-20)")
	public static NpcStringId NEVER_FORGET_LV_8_20;
	
	@ClientString(id = 574002, message = "Never Forget (Lv. 8-20) (In Progress)")
	public static NpcStringId NEVER_FORGET_LV_8_20_IN_PROGRESS;
	
	@ClientString(id = 574003, message = "Never Forget (Lv. 8-20) (Done)")
	public static NpcStringId NEVER_FORGET_LV_8_20_DONE;
	
	@ClientString(id = 574004, message = "Never Forget (Lv. 8-20)")
	public static NpcStringId NEVER_FORGET_LV_8_202;
	
	@ClientString(id = 574005, message = "Go to the Remembrance Tower")
	public static NpcStringId GO_TO_THE_REMEMBRANCE_TOWER;
	
	@ClientString(id = 574101, message = "A Draught for the Cold (Lv. 10-20)")
	public static NpcStringId A_DRAUGHT_FOR_THE_COLD_LV_10_20;
	
	@ClientString(id = 574102, message = "A Draught for the Cold (Lv. 10-20) (In Progress)")
	public static NpcStringId A_DRAUGHT_FOR_THE_COLD_LV_10_20_IN_PROGRESS;
	
	@ClientString(id = 574103, message = "A Draught for the Cold (Lv. 10-20) (Done)")
	public static NpcStringId A_DRAUGHT_FOR_THE_COLD_LV_10_20_DONE;
	
	@ClientString(id = 574104, message = "A Draught for the Cold (Lv. 10-20)")
	public static NpcStringId A_DRAUGHT_FOR_THE_COLD_LV_10_202;
	
	@ClientString(id = 574105, message = "Talk to Leira")
	public static NpcStringId TALK_TO_LEIRA;
	
	@ClientString(id = 574201, message = "A Furry Friend (Lv. 11-20)")
	public static NpcStringId A_FURRY_FRIEND_LV_11_20;
	
	@ClientString(id = 574202, message = "A Furry Friend (Lv. 11-20) (In Progress)")
	public static NpcStringId A_FURRY_FRIEND_LV_11_20_IN_PROGRESS;
	
	@ClientString(id = 574203, message = "A Furry Friend (Lv. 11-20) (Done)")
	public static NpcStringId A_FURRY_FRIEND_LV_11_20_DONE;
	
	@ClientString(id = 574204, message = "A Furry Friend (Lv. 11-20)")
	public static NpcStringId A_FURRY_FRIEND_LV_11_202;
	
	@ClientString(id = 574205, message = "You are far from Ricky.")
	public static NpcStringId YOU_ARE_FAR_FROM_RICKY;
	
	@ClientString(id = 574206, message = "Ricky is entering Kiku's cave!")
	public static NpcStringId RICKY_IS_ENTERING_KIKU_S_CAVE;
	
	@ClientString(id = 574207, message = "Take Ricky to Leira in under 2 minutes.")
	public static NpcStringId TAKE_RICKY_TO_LEIRA_IN_UNDER_2_MINUTES;
	
	@ClientString(id = 574209, message = "Bring Back Ricky")
	public static NpcStringId BRING_BACK_RICKY;
	
	@ClientString(id = 574301, message = "Strange Fungus (Lv. 13-20)")
	public static NpcStringId STRANGE_FUNGUS_LV_13_20;
	
	@ClientString(id = 574302, message = "Strange Fungus (Lv. 13-20) (In Progress)")
	public static NpcStringId STRANGE_FUNGUS_LV_13_20_IN_PROGRESS;
	
	@ClientString(id = 574303, message = "Strange Fungus (Lv. 13-20) (Done)")
	public static NpcStringId STRANGE_FUNGUS_LV_13_20_DONE;
	
	@ClientString(id = 574304, message = "Strange Fungus (Lv. 13-20)")
	public static NpcStringId STRANGE_FUNGUS_LV_13_202;
	
	@ClientString(id = 574305, message = "Talk to Milone")
	public static NpcStringId TALK_TO_MILONE;
	
	@ClientString(id = 574401, message = "Stronger Than Steel (Lv. 15-20)")
	public static NpcStringId STRONGER_THAN_STEEL_LV_15_20;
	
	@ClientString(id = 574402, message = "Stronger Than Steel (Lv. 15-20) (In Progress)")
	public static NpcStringId STRONGER_THAN_STEEL_LV_15_20_IN_PROGRESS;
	
	@ClientString(id = 574403, message = "Stronger Than Steel (Lv. 15-20) (Done)")
	public static NpcStringId STRONGER_THAN_STEEL_LV_15_20_DONE;
	
	@ClientString(id = 574404, message = "Stronger Than Steel (Lv. 15-20)")
	public static NpcStringId STRONGER_THAN_STEEL_LV_15_202;
	
	@ClientString(id = 574405, message = "Talk to Dolkin")
	public static NpcStringId TALK_TO_DOLKIN;
	
	@ClientString(id = 574501, message = "The Secret Ingredient (Lv. 17-25)")
	public static NpcStringId THE_SECRET_INGREDIENT_LV_17_25;
	
	@ClientString(id = 574502, message = "The Secret Ingredient (Lv. 17-25) (In Progress)")
	public static NpcStringId THE_SECRET_INGREDIENT_LV_17_25_IN_PROGRESS;
	
	@ClientString(id = 574503, message = "The Secret Ingredient (Lv. 17-25) (Done)")
	public static NpcStringId THE_SECRET_INGREDIENT_LV_17_25_DONE;
	
	@ClientString(id = 574504, message = "The Secret Ingredient (Lv. 17-25)")
	public static NpcStringId THE_SECRET_INGREDIENT_LV_17_252;
	
	@ClientString(id = 574601, message = "See the World (Lv. 19-25)")
	public static NpcStringId SEE_THE_WORLD_LV_19_25;
	
	@ClientString(id = 574602, message = "See the World (Lv. 19-25) (In Progress)")
	public static NpcStringId SEE_THE_WORLD_LV_19_25_IN_PROGRESS;
	
	@ClientString(id = 574603, message = "See the World (Lv. 19-25) (Done)")
	public static NpcStringId SEE_THE_WORLD_LV_19_25_DONE;
	
	@ClientString(id = 574604, message = "See the World (Lv. 19-25)")
	public static NpcStringId SEE_THE_WORLD_LV_19_252;
	
	@ClientString(id = 574605, message = "Talk to Astiel")
	public static NpcStringId TALK_TO_ASTIEL;
	
	@ClientString(id = 574606, message = "Talk to Levian")
	public static NpcStringId TALK_TO_LEVIAN;
	
	@ClientString(id = 574701, message = "The Hero's Journey: Pagan Temple (Lv. 97-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_PAGAN_TEMPLE_LV_97_99;
	
	@ClientString(id = 574702, message = "The Hero's Journey: Pagan Temple (Lv. 97-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_PAGAN_TEMPLE_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 574703, message = "The Hero's Journey: Pagan Temple (Lv. 97-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_PAGAN_TEMPLE_LV_97_99_DONE;
	
	@ClientString(id = 574704, message = "The Hero's Journey: Pagan Temple (Lv. 97-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_PAGAN_TEMPLE_LV_97_992;
	
	@ClientString(id = 574711, message = "Talk to Tulesir")
	public static NpcStringId TALK_TO_TULESIR7;
	
	@ClientString(id = 574712, message = "Talk to Kargos")
	public static NpcStringId TALK_TO_KARGOS;
	
	@ClientString(id = 574713, message = "Talk to Kargos")
	public static NpcStringId TALK_TO_KARGOS2;
	
	@ClientString(id = 574801, message = "Mysterious Suggestion - 1")
	public static NpcStringId MYSTERIOUS_SUGGESTION_1;
	
	@ClientString(id = 574802, message = "Mysterious Suggestion - 1 (In Progress)")
	public static NpcStringId MYSTERIOUS_SUGGESTION_1_IN_PROGRESS;
	
	@ClientString(id = 574803, message = "Mysterious Suggestion - 1 (Done)")
	public static NpcStringId MYSTERIOUS_SUGGESTION_1_DONE;
	
	@ClientString(id = 574804, message = "Mysterious Suggestion - 1")
	public static NpcStringId MYSTERIOUS_SUGGESTION_12;
	
	@ClientString(id = 574901, message = "Mysterious Suggestion - 2")
	public static NpcStringId MYSTERIOUS_SUGGESTION_2;
	
	@ClientString(id = 574902, message = "Mysterious Suggestion - 2 (In Progress)")
	public static NpcStringId MYSTERIOUS_SUGGESTION_2_IN_PROGRESS;
	
	@ClientString(id = 574903, message = "Mysterious Suggestion - 2 (Done)")
	public static NpcStringId MYSTERIOUS_SUGGESTION_2_DONE;
	
	@ClientString(id = 574904, message = "Mysterious Suggestion - 2")
	public static NpcStringId MYSTERIOUS_SUGGESTION_22;
	
	@ClientString(id = 574905, message = "Open the item screen and double-click the empty water bottle!")
	public static NpcStringId OPEN_THE_ITEM_SCREEN_AND_DOUBLE_CLICK_THE_EMPTY_WATER_BOTTLE;
	
	@ClientString(id = 574906, message = "If you double-click the empty bottle, it will become full of water.")
	public static NpcStringId IF_YOU_DOUBLE_CLICK_THE_EMPTY_BOTTLE_IT_WILL_BECOME_FULL_OF_WATER;
	
	@ClientString(id = 574907, message = "Click the flame flower, then double click the trowel.")
	public static NpcStringId CLICK_THE_FLAME_FLOWER_THEN_DOUBLE_CLICK_THE_TROWEL;
	
	@ClientString(id = 574908, message = "If you double-click the Helping Seed item, a Tree of Helping will be planted.")
	public static NpcStringId IF_YOU_DOUBLE_CLICK_THE_HELPING_SEED_ITEM_A_TREE_OF_HELPING_WILL_BE_PLANTED;
	
	@ClientString(id = 575101, message = "Winds of Fate: Encounters (Lv. 38-99)")
	public static NpcStringId WINDS_OF_FATE_ENCOUNTERS_LV_38_99;
	
	@ClientString(id = 575102, message = "Winds of Fate: Encounters (Lv. 38-99) (In Progress)")
	public static NpcStringId WINDS_OF_FATE_ENCOUNTERS_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 575103, message = "Winds of Fate: Encounters (Lv. 38-99) (Done)")
	public static NpcStringId WINDS_OF_FATE_ENCOUNTERS_LV_38_99_DONE;
	
	@ClientString(id = 575104, message = "Winds of Fate: Encounters (Lv. 76-99)")
	public static NpcStringId WINDS_OF_FATE_ENCOUNTERS_LV_76_99;
	
	@ClientString(id = 575110, message = "Talk to Katalin")
	public static NpcStringId TALK_TO_KATALIN2;
	
	@ClientString(id = 575111, message = "Talk to Ayanthe")
	public static NpcStringId TALK_TO_AYANTHE2;
	
	@ClientString(id = 575112, message = "Talk to Raymond")
	public static NpcStringId TALK_TO_RAYMOND;
	
	@ClientString(id = 575113, message = "Kill Skeletons")
	public static NpcStringId KILL_SKELETONS;
	
	@ClientString(id = 575201, message = "Winds of Fate: A Promise (Lv. 76-99)")
	public static NpcStringId WINDS_OF_FATE_A_PROMISE_LV_76_99;
	
	@ClientString(id = 575202, message = "Winds of Fate: A Promise (Lv. 76-99) (In Progress)")
	public static NpcStringId WINDS_OF_FATE_A_PROMISE_LV_76_99_IN_PROGRESS;
	
	@ClientString(id = 575203, message = "Winds of Fate: A Promise (Lv. 76-99) (Done)")
	public static NpcStringId WINDS_OF_FATE_A_PROMISE_LV_76_99_DONE;
	
	@ClientString(id = 575204, message = "Winds of Fate: A Promise (Lv. 76-99)")
	public static NpcStringId WINDS_OF_FATE_A_PROMISE_LV_76_992;
	
	@ClientString(id = 575211, message = "Talk to Karla")
	public static NpcStringId TALK_TO_KARLA;
	
	@ClientString(id = 575212, message = "Talk to Siegmund")
	public static NpcStringId TALK_TO_SIEGMUND;
	
	@ClientString(id = 575213, message = "Talk to Lombert")
	public static NpcStringId TALK_TO_LOMBERT;
	
	@ClientString(id = 575214, message = "Talk to the Mysterious Wizard")
	public static NpcStringId TALK_TO_THE_MYSTERIOUS_WIZARD;
	
	@ClientString(id = 575215, message = "Check the Tombstone")
	public static NpcStringId CHECK_THE_TOMBSTONE;
	
	@ClientString(id = 575216, message = "Talk to the Ghost")
	public static NpcStringId TALK_TO_THE_GHOST;
	
	@ClientString(id = 575217, message = "A Place of Vengeance")
	public static NpcStringId A_PLACE_OF_VENGEANCE;
	
	@ClientString(id = 575218, message = "Giselle Von Hellmann")
	public static NpcStringId GISELLE_VON_HELLMANN;
	
	@ClientString(id = 575301, message = "Winds of Fate: Choices (Lv. 85-99)")
	public static NpcStringId WINDS_OF_FATE_CHOICES_LV_85_99;
	
	@ClientString(id = 575302, message = "Winds of Fate: Choices (Lv. 85-99) (In Progress)")
	public static NpcStringId WINDS_OF_FATE_CHOICES_LV_85_99_IN_PROGRESS;
	
	@ClientString(id = 575303, message = "Winds of Fate: Choices (Lv. 85-99) (Done)")
	public static NpcStringId WINDS_OF_FATE_CHOICES_LV_85_99_DONE;
	
	@ClientString(id = 575304, message = "Winds of Fate: Choices (Lv. 85-99)")
	public static NpcStringId WINDS_OF_FATE_CHOICES_LV_85_992;
	
	@ClientString(id = 575310, message = "Talk to Arkenias")
	public static NpcStringId TALK_TO_ARKENIAS;
	
	@ClientString(id = 575311, message = "Nebulite Eye: Crystal Eye")
	public static NpcStringId NEBULITE_EYE_CRYSTAL_EYE;
	
	@ClientString(id = 575312, message = "Nebulite Watch: Broken Stone of Purity")
	public static NpcStringId NEBULITE_WATCH_BROKEN_STONE_OF_PURITY;
	
	@ClientString(id = 575313, message = "Nebulite Golem: Miracle Drug of Enchantment")
	public static NpcStringId NEBULITE_GOLEM_MIRACLE_DRUG_OF_ENCHANTMENT;
	
	@ClientString(id = 575314, message = "Combine the Ingredients")
	public static NpcStringId COMBINE_THE_INGREDIENTS;
	
	@ClientString(id = 575315, message = "Talk to Hardin")
	public static NpcStringId TALK_TO_HARDIN2;
	
	@ClientString(id = 575316, message = "Talk to Icarus")
	public static NpcStringId TALK_TO_ICARUS;
	
	@ClientString(id = 575317, message = "Talk to Athrea")
	public static NpcStringId TALK_TO_ATHREA;
	
	@ClientString(id = 575318, message = "Check Athrea's Box")
	public static NpcStringId CHECK_ATHREA_S_BOX;
	
	@ClientString(id = 575319, message = "Talk to Gereth")
	public static NpcStringId TALK_TO_GERETH2;
	
	@ClientString(id = 575320, message = "Takl to Navari")
	public static NpcStringId TAKL_TO_NAVARI;
	
	@ClientString(id = 575500, message = "Queen Navari has sent a letter.nClick the question-mark icon to read.")
	public static NpcStringId QUEEN_NAVARI_HAS_SENT_A_LETTER_NCLICK_THE_QUESTION_MARK_ICON_TO_READ;
	
	@ClientString(id = 575501, message = "Letters from the Queen: Windy Hill (Lv. 20-29)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_WINDY_HILL_LV_20_29;
	
	@ClientString(id = 575502, message = "Letters from the Queen: Windy Hill (Lv. 20-29) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_WINDY_HILL_LV_20_29_IN_PROGRESS;
	
	@ClientString(id = 575503, message = "Letters from the Queen: Windy Hill (Lv. 20-29) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_WINDY_HILL_LV_20_29_DONE;
	
	@ClientString(id = 575504, message = "Letters from the Queen: Windy Hill (Lv. 20-29)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_WINDY_HILL_LV_20_292;
	
	@ClientString(id = 575511, message = "Talk to Levian")
	public static NpcStringId TALK_TO_LEVIAN2;
	
	@ClientString(id = 575512, message = "Talk to Pio")
	public static NpcStringId TALK_TO_PIO;
	
	@ClientString(id = 575601, message = "An Interdimensional Draft (Lv. 20-99)")
	public static NpcStringId AN_INTERDIMENSIONAL_DRAFT_LV_20_99;
	
	@ClientString(id = 575602, message = "An Interdimensional Draft (Lv. 20-99) (In Progress)")
	public static NpcStringId AN_INTERDIMENSIONAL_DRAFT_LV_20_99_IN_PROGRESS;
	
	@ClientString(id = 575603, message = "An Interdimensional Draft (Lv. 20-99) (Complete)")
	public static NpcStringId AN_INTERDIMENSIONAL_DRAFT_LV_20_99_COMPLETE;
	
	@ClientString(id = 575604, message = "An Interdimensional Draft (Lv. 20-99)")
	public static NpcStringId AN_INTERDIMENSIONAL_DRAFT_LV_20_992;
	
	@ClientString(id = 575701, message = "Quieting the Storm (Lv. 24-99)")
	public static NpcStringId QUIETING_THE_STORM_LV_24_99;
	
	@ClientString(id = 575702, message = "Quieting the Storm (Lv. 24-99) (In Progress)")
	public static NpcStringId QUIETING_THE_STORM_LV_24_99_IN_PROGRESS;
	
	@ClientString(id = 575703, message = "Quieting the Storm (Lv. 24-99) (Complete)")
	public static NpcStringId QUIETING_THE_STORM_LV_24_99_COMPLETE;
	
	@ClientString(id = 575704, message = "Quieting the Storm (Lv. 24-99)")
	public static NpcStringId QUIETING_THE_STORM_LV_24_992;
	
	@ClientString(id = 575711, message = "Immense Windima or Giant Windima")
	public static NpcStringId IMMENSE_WINDIMA_OR_GIANT_WINDIMA;
	
	@ClientString(id = 575801, message = "The Oath of the Wind (Lv. 28-99)")
	public static NpcStringId THE_OATH_OF_THE_WIND_LV_28_99;
	
	@ClientString(id = 575802, message = "The Oath of the Wind (Lv. 28-99) (In Progress)")
	public static NpcStringId THE_OATH_OF_THE_WIND_LV_28_99_IN_PROGRESS;
	
	@ClientString(id = 575803, message = "The Oath of the Wind (Lv. 28-99) (Complete)")
	public static NpcStringId THE_OATH_OF_THE_WIND_LV_28_99_COMPLETE;
	
	@ClientString(id = 575804, message = "The Oath of the Wind (Lv. 28-99)")
	public static NpcStringId THE_OATH_OF_THE_WIND_LV_28_992;
	
	@ClientString(id = 575811, message = "Ask Pio for a Summon")
	public static NpcStringId ASK_PIO_FOR_A_SUMMON;
	
	@ClientString(id = 576001, message = "Letters from the Queen: Orc Barracks (Lv. 30-39)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_ORC_BARRACKS_LV_30_39;
	
	@ClientString(id = 576002, message = "Letters from the Queen: Orc Barracks (Lv. 30-39) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_ORC_BARRACKS_LV_30_39_IN_PROGRESS;
	
	@ClientString(id = 576003, message = "Letters from the Queen: Orc Barracks (Lv. 30-39) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_ORC_BARRACKS_LV_30_39_DONE;
	
	@ClientString(id = 576004, message = "Letters from the Queen: Orc Barracks (Lv. 30-39)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_ORC_BARRACKS_LV_30_392;
	
	@ClientString(id = 576011, message = "Talk to Levian")
	public static NpcStringId TALK_TO_LEVIAN3;
	
	@ClientString(id = 576012, message = "Talk to Piotur")
	public static NpcStringId TALK_TO_PIOTUR;
	
	@ClientString(id = 576101, message = "An Orc in Love (Lv. 30-99)")
	public static NpcStringId AN_ORC_IN_LOVE_LV_30_99;
	
	@ClientString(id = 576102, message = "An Orc in Love (Lv. 30-99) (In Progress)")
	public static NpcStringId AN_ORC_IN_LOVE_LV_30_99_IN_PROGRESS;
	
	@ClientString(id = 576103, message = "An Orc in Love (Lv. 30-99) (Complete)")
	public static NpcStringId AN_ORC_IN_LOVE_LV_30_99_COMPLETE;
	
	@ClientString(id = 576104, message = "An Orc in Love (Lv. 30-99)")
	public static NpcStringId AN_ORC_IN_LOVE_LV_30_992;
	
	@ClientString(id = 576111, message = "Kill Turek Orcs")
	public static NpcStringId KILL_TUREK_ORCS;
	
	@ClientString(id = 576201, message = "Marionette Spirit (Lv. 34-99)")
	public static NpcStringId MARIONETTE_SPIRIT_LV_34_99;
	
	@ClientString(id = 576202, message = "Marionette Spirit (Lv. 34-99) (In Progress)")
	public static NpcStringId MARIONETTE_SPIRIT_LV_34_99_IN_PROGRESS;
	
	@ClientString(id = 576203, message = "Marionette Spirit (Lv. 34-99) (Complete)")
	public static NpcStringId MARIONETTE_SPIRIT_LV_34_99_COMPLETE;
	
	@ClientString(id = 576204, message = "Marionette Spirit (Lv. 34-99)")
	public static NpcStringId MARIONETTE_SPIRIT_LV_34_992;
	
	@ClientString(id = 576301, message = "Terrifying Chertuba (Lv. 34-99)")
	public static NpcStringId TERRIFYING_CHERTUBA_LV_34_99;
	
	@ClientString(id = 576302, message = "Terrifying Chertuba (Lv. 34-99) (In Progress)")
	public static NpcStringId TERRIFYING_CHERTUBA_LV_34_99_IN_PROGRESS;
	
	@ClientString(id = 576303, message = "Terrifying Chertuba (Lv. 34-99) (Complete)")
	public static NpcStringId TERRIFYING_CHERTUBA_LV_34_99_COMPLETE;
	
	@ClientString(id = 576304, message = "Terrifying Chertuba (Lv. 34-99)")
	public static NpcStringId TERRIFYING_CHERTUBA_LV_34_992;
	
	@ClientString(id = 576311, message = "Kill Elite Turek Orcs")
	public static NpcStringId KILL_ELITE_TUREK_ORCS;
	
	@ClientString(id = 576312, message = "Chertuba's Illusion or Chertuba's Mirage")
	public static NpcStringId CHERTUBA_S_ILLUSION_OR_CHERTUBA_S_MIRAGE;
	
	@ClientString(id = 576401, message = "Free Spirit (Lv. 38-99)")
	public static NpcStringId FREE_SPIRIT_LV_38_99;
	
	@ClientString(id = 576402, message = "Free Spirit (Lv. 38-99) (In Progress)")
	public static NpcStringId FREE_SPIRIT_LV_38_99_IN_PROGRESS;
	
	@ClientString(id = 576403, message = "Free Spirit (Lv. 38-99) (Done)")
	public static NpcStringId FREE_SPIRIT_LV_38_99_DONE;
	
	@ClientString(id = 576404, message = "Free Spirit (Lv. 38-99)")
	public static NpcStringId FREE_SPIRIT_LV_38_992;
	
	@ClientString(id = 576411, message = "Free the Imprisoned Spirits")
	public static NpcStringId FREE_THE_IMPRISONED_SPIRITS;
	
	@ClientString(id = 576601, message = "A New Craft (Lv. 40-99)")
	public static NpcStringId A_NEW_CRAFT_LV_40_99;
	
	@ClientString(id = 576602, message = "A New Craft (Lv. 40-99) (In Progress)")
	public static NpcStringId A_NEW_CRAFT_LV_40_99_IN_PROGRESS;
	
	@ClientString(id = 576603, message = "A New Craft (Lv. 40-99) (Done)")
	public static NpcStringId A_NEW_CRAFT_LV_40_99_DONE;
	
	@ClientString(id = 576604, message = "A New Craft (Lv. 40-99)")
	public static NpcStringId A_NEW_CRAFT_LV_40_992;
	
	@ClientString(id = 576701, message = "A Whole New Level of Alchemy (Lv. 97-99)")
	public static NpcStringId A_WHOLE_NEW_LEVEL_OF_ALCHEMY_LV_97_99;
	
	@ClientString(id = 576702, message = "A Whole New Level of Alchemy (Lv. 97-99) (In Progress)")
	public static NpcStringId A_WHOLE_NEW_LEVEL_OF_ALCHEMY_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 576703, message = "A Whole New Level of Alchemy (Lv. 97-99) (Done)")
	public static NpcStringId A_WHOLE_NEW_LEVEL_OF_ALCHEMY_LV_97_99_DONE;
	
	@ClientString(id = 576704, message = "A Whole New Level of Alchemy (Lv. 97-99)")
	public static NpcStringId A_WHOLE_NEW_LEVEL_OF_ALCHEMY_LV_97_992;
	
	@ClientString(id = 576901, message = "Letters from the Queen: Cruma Tower, Part 1 (Lv. 40-45)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_CRUMA_TOWER_PART_1_LV_40_45;
	
	@ClientString(id = 576902, message = "Letters from the Queen: Cruma Tower, Part 1 (Lv. 40-45) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_CRUMA_TOWER_PART_1_LV_40_45_IN_PROGRESS;
	
	@ClientString(id = 576903, message = "Letters from the Queen: Cruma Tower, Part 1 (Lv. 40-45) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_CRUMA_TOWER_PART_1_LV_40_45_DONE;
	
	@ClientString(id = 576904, message = "Letters from the Queen: Cruma Tower, Part 1 (Lv. 40-45)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_CRUMA_TOWER_PART_1_LV_40_452;
	
	@ClientString(id = 576911, message = "Talk to Sylvain")
	public static NpcStringId TALK_TO_SYLVAIN;
	
	@ClientString(id = 576912, message = "Talk to Lorain")
	public static NpcStringId TALK_TO_LORAIN;
	
	@ClientString(id = 577001, message = "In Search of the Grail (Lv. 40-99)")
	public static NpcStringId IN_SEARCH_OF_THE_GRAIL_LV_40_99;
	
	@ClientString(id = 577002, message = "In Search of the Grail (Lv. 40-99) (In Progress)")
	public static NpcStringId IN_SEARCH_OF_THE_GRAIL_LV_40_99_IN_PROGRESS;
	
	@ClientString(id = 577003, message = "In Search of the Grail (Lv. 40-99) (Done)")
	public static NpcStringId IN_SEARCH_OF_THE_GRAIL_LV_40_99_DONE;
	
	@ClientString(id = 577004, message = "In Search of the Grail (Lv. 40-99)")
	public static NpcStringId IN_SEARCH_OF_THE_GRAIL_LV_40_992;
	
	@ClientString(id = 577101, message = "Volatile Power (Lv. 44-99)")
	public static NpcStringId VOLATILE_POWER_LV_44_99;
	
	@ClientString(id = 577102, message = "Volatile Power (Lv. 44-99) (In Progress)")
	public static NpcStringId VOLATILE_POWER_LV_44_99_IN_PROGRESS;
	
	@ClientString(id = 577103, message = "Volatile Power (Lv. 44-99) (Done)")
	public static NpcStringId VOLATILE_POWER_LV_44_99_DONE;
	
	@ClientString(id = 577104, message = "Volatile Power (Lv. 44-99)")
	public static NpcStringId VOLATILE_POWER_LV_44_992;
	
	@ClientString(id = 577111, message = "Find the Hidden Crusher")
	public static NpcStringId FIND_THE_HIDDEN_CRUSHER;
	
	@ClientString(id = 577201, message = "Reports from Cruma Tower, Part 1 (Lv. 45-99)")
	public static NpcStringId REPORTS_FROM_CRUMA_TOWER_PART_1_LV_45_99;
	
	@ClientString(id = 577202, message = "Reports from Cruma Tower, Part 1 (Lv. 45-99) (In Progress)")
	public static NpcStringId REPORTS_FROM_CRUMA_TOWER_PART_1_LV_45_99_IN_PROGRESS;
	
	@ClientString(id = 577203, message = "Reports from Cruma Tower, Part 1 (Lv. 45-99) (Done)")
	public static NpcStringId REPORTS_FROM_CRUMA_TOWER_PART_1_LV_45_99_DONE;
	
	@ClientString(id = 577204, message = "Reports from Cruma Tower, Part 1 (Lv. 45-99)")
	public static NpcStringId REPORTS_FROM_CRUMA_TOWER_PART_1_LV_45_992;
	
	@ClientString(id = 577211, message = "Ask Janssen to Summon an Owl")
	public static NpcStringId ASK_JANSSEN_TO_SUMMON_AN_OWL;
	
	@ClientString(id = 577212, message = "Talk to the Owl")
	public static NpcStringId TALK_TO_THE_OWL;
	
	@ClientString(id = 577401, message = "Letters from the Queen: Cruma Tower, Part 2 (Lv. 46-50)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_CRUMA_TOWER_PART_2_LV_46_50;
	
	@ClientString(id = 577402, message = "Letters from the Queen: Cruma Tower, Part 2 (Lv. 46-50) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_CRUMA_TOWER_PART_2_LV_46_50_IN_PROGRESS;
	
	@ClientString(id = 577403, message = "Letters from the Queen: Cruma Tower, Part 2 (Lv. 46-50) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_CRUMA_TOWER_PART_2_LV_46_50_DONE;
	
	@ClientString(id = 577404, message = "Letters from the Queen: Cruma Tower, Part 2 (Lv. 46-50)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_CRUMA_TOWER_PART_2_LV_46_502;
	
	@ClientString(id = 577411, message = "Talk to Sylvain")
	public static NpcStringId TALK_TO_SYLVAIN2;
	
	@ClientString(id = 577412, message = "Talk to Rombel")
	public static NpcStringId TALK_TO_ROMBEL;
	
	@ClientString(id = 577501, message = "In Search of an Ancient Giant (Lv. 46-99)")
	public static NpcStringId IN_SEARCH_OF_AN_ANCIENT_GIANT_LV_46_99;
	
	@ClientString(id = 577502, message = "In Search of an Ancient Giant (Lv. 46-99) (In Progress)")
	public static NpcStringId IN_SEARCH_OF_AN_ANCIENT_GIANT_LV_46_99_IN_PROGRESS;
	
	@ClientString(id = 577503, message = "In Search of an Ancient Giant (Lv. 46-99) (Done)")
	public static NpcStringId IN_SEARCH_OF_AN_ANCIENT_GIANT_LV_46_99_DONE;
	
	@ClientString(id = 577504, message = "In Search of an Ancient Giant (Lv. 46-99)")
	public static NpcStringId IN_SEARCH_OF_AN_ANCIENT_GIANT_LV_46_992;
	
	@ClientString(id = 577511, message = "Go to Underground Lv. 2")
	public static NpcStringId GO_TO_UNDERGROUND_LV_2;
	
	@ClientString(id = 577601, message = "The Wrath of the Giants (Lv. 48-99)")
	public static NpcStringId THE_WRATH_OF_THE_GIANTS_LV_48_99;
	
	@ClientString(id = 577602, message = "The Wrath of the Giants (Lv. 48-99) (In Progress)")
	public static NpcStringId THE_WRATH_OF_THE_GIANTS_LV_48_99_IN_PROGRESS;
	
	@ClientString(id = 577603, message = "The Wrath of the Giants (Lv. 48-99) (Done)")
	public static NpcStringId THE_WRATH_OF_THE_GIANTS_LV_48_99_DONE;
	
	@ClientString(id = 577604, message = "The Wrath of the Giants (Lv. 48-99)")
	public static NpcStringId THE_WRATH_OF_THE_GIANTS_LV_48_992;
	
	@ClientString(id = 577611, message = "Find the Broken Device")
	public static NpcStringId FIND_THE_BROKEN_DEVICE;
	
	@ClientString(id = 577612, message = "Talk to the Ancient Giant")
	public static NpcStringId TALK_TO_THE_ANCIENT_GIANT;
	
	@ClientString(id = 577701, message = "Reports from Cruma Tower, Part 2 (Lv. 49-99)")
	public static NpcStringId REPORTS_FROM_CRUMA_TOWER_PART_2_LV_49_99;
	
	@ClientString(id = 577702, message = "Reports from Cruma Tower, Part 2 (Lv. 49-99) (In Progress)")
	public static NpcStringId REPORTS_FROM_CRUMA_TOWER_PART_2_LV_49_99_IN_PROGRESS;
	
	@ClientString(id = 577703, message = "Reports from Cruma Tower, Part 2 (Lv. 49-99) (Done)")
	public static NpcStringId REPORTS_FROM_CRUMA_TOWER_PART_2_LV_49_99_DONE;
	
	@ClientString(id = 577704, message = "Reports from Cruma Tower, Part 2 (Lv. 49-99)")
	public static NpcStringId REPORTS_FROM_CRUMA_TOWER_PART_2_LV_49_992;
	
	@ClientString(id = 577711, message = "Ask Belkadhi to Summon an Owl")
	public static NpcStringId ASK_BELKADHI_TO_SUMMON_AN_OWL;
	
	@ClientString(id = 577712, message = "Talk to the Owl")
	public static NpcStringId TALK_TO_THE_OWL2;
	
	@ClientString(id = 577901, message = "Letters from the Queen: Sea of Spores (Lv. 52-57)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_SEA_OF_SPORES_LV_52_57;
	
	@ClientString(id = 577902, message = "Letters from the Queen: Sea of Spores (Lv. 52-57) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_SEA_OF_SPORES_LV_52_57_IN_PROGRESS;
	
	@ClientString(id = 577903, message = "Letters from the Queen: Sea of Spores (Lv. 52-57) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_SEA_OF_SPORES_LV_52_57_DONE;
	
	@ClientString(id = 577904, message = "Letters from the Queen: Sea of Spores (Lv. 52-57)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_SEA_OF_SPORES_LV_52_572;
	
	@ClientString(id = 577911, message = "Talk to Hollint")
	public static NpcStringId TALK_TO_HOLLINT;
	
	@ClientString(id = 577912, message = "Talk to Andy")
	public static NpcStringId TALK_TO_ANDY2;
	
	@ClientString(id = 578001, message = "A Weakened Barrier (Lv. 52-58)")
	public static NpcStringId A_WEAKENED_BARRIER_LV_52_58;
	
	@ClientString(id = 578002, message = "A Weakened Barrier (Lv. 52-58) (In Progress)")
	public static NpcStringId A_WEAKENED_BARRIER_LV_52_58_IN_PROGRESS;
	
	@ClientString(id = 578003, message = "A Weakened Barrier (Lv. 52-58) (Done)")
	public static NpcStringId A_WEAKENED_BARRIER_LV_52_58_DONE;
	
	@ClientString(id = 578004, message = "A Weakened Barrier (Lv. 52-58)")
	public static NpcStringId A_WEAKENED_BARRIER_LV_52_582;
	
	@ClientString(id = 578011, message = "Kill Monsters near the Sea of Spores")
	public static NpcStringId KILL_MONSTERS_NEAR_THE_SEA_OF_SPORES;
	
	@ClientString(id = 578101, message = "Ingredients to Enforcements (Lv. 52-58)")
	public static NpcStringId INGREDIENTS_TO_ENFORCEMENTS_LV_52_58;
	
	@ClientString(id = 578102, message = "Ingredients to Enforcements (Lv. 52-58) (In Progress)")
	public static NpcStringId INGREDIENTS_TO_ENFORCEMENTS_LV_52_58_IN_PROGRESS;
	
	@ClientString(id = 578103, message = "Ingredients to Enforcements (Lv. 52-58) (Done)")
	public static NpcStringId INGREDIENTS_TO_ENFORCEMENTS_LV_52_58_DONE;
	
	@ClientString(id = 578104, message = "Ingredients to Enforcements (Lv. 52-58)")
	public static NpcStringId INGREDIENTS_TO_ENFORCEMENTS_LV_52_582;
	
	@ClientString(id = 578201, message = "Letters from the Queen: Forsaken Plains (Lv. 58-60)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FORSAKEN_PLAINS_LV_58_60;
	
	@ClientString(id = 578202, message = "Letters from the Queen: Forsaken Plains (Lv. 58-60) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FORSAKEN_PLAINS_LV_58_60_IN_PROGRESS;
	
	@ClientString(id = 578203, message = "Letters from the Queen: Forsaken Plains (Lv. 58-60) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FORSAKEN_PLAINS_LV_58_60_DONE;
	
	@ClientString(id = 578204, message = "Letters from the Queen: Forsaken Plains (Lv. 58-60)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FORSAKEN_PLAINS_LV_58_602;
	
	@ClientString(id = 578211, message = "Talk to Orven")
	public static NpcStringId TALK_TO_ORVEN;
	
	@ClientString(id = 578212, message = "Talk to Novain")
	public static NpcStringId TALK_TO_NOVAIN;
	
	@ClientString(id = 578301, message = "Traces of an Ambush (Lv. 58-61)")
	public static NpcStringId TRACES_OF_AN_AMBUSH_LV_58_61;
	
	@ClientString(id = 578302, message = "Traces of an Ambush (Lv. 58-61) (In Progress)")
	public static NpcStringId TRACES_OF_AN_AMBUSH_LV_58_61_IN_PROGRESS;
	
	@ClientString(id = 578303, message = "Traces of an Ambush (Lv. 58-61) (Done)")
	public static NpcStringId TRACES_OF_AN_AMBUSH_LV_58_61_DONE;
	
	@ClientString(id = 578304, message = "Traces of an Ambush (Lv. 58-61)")
	public static NpcStringId TRACES_OF_AN_AMBUSH_LV_58_612;
	
	@ClientString(id = 578351, message = "I will give you death!")
	public static NpcStringId I_WILL_GIVE_YOU_DEATH;
	
	@ClientString(id = 578352, message = "Back for more, huh?")
	public static NpcStringId BACK_FOR_MORE_HUH;
	
	@ClientString(id = 578353, message = "You little punk! Take that!")
	public static NpcStringId YOU_LITTLE_PUNK_TAKE_THAT;
	
	@ClientString(id = 578401, message = "The Broken Device (Lv. 58-61)")
	public static NpcStringId THE_BROKEN_DEVICE_LV_58_61;
	
	@ClientString(id = 578402, message = "The Broken Device (Lv. 58-61) (In Progress)")
	public static NpcStringId THE_BROKEN_DEVICE_LV_58_61_IN_PROGRESS;
	
	@ClientString(id = 578403, message = "The Broken Device (Lv. 58-61) (Done)")
	public static NpcStringId THE_BROKEN_DEVICE_LV_58_61_DONE;
	
	@ClientString(id = 578404, message = "The Broken Device (Lv. 58-61)")
	public static NpcStringId THE_BROKEN_DEVICE_LV_58_612;
	
	@ClientString(id = 578501, message = "Letters from the Queen: Fields of Massacre (Lv. 61-64)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FIELDS_OF_MASSACRE_LV_61_64;
	
	@ClientString(id = 578502, message = "Letters from the Queen: Fields of Massacre (Lv. 61-64) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FIELDS_OF_MASSACRE_LV_61_64_IN_PROGRESS;
	
	@ClientString(id = 578503, message = "Letters from the Queen: Fields of Massacre (Lv. 61-64) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FIELDS_OF_MASSACRE_LV_61_64_DONE;
	
	@ClientString(id = 578504, message = "Letters from the Queen: Fields of Massacre (Lv. 61-64)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FIELDS_OF_MASSACRE_LV_61_642;
	
	@ClientString(id = 578511, message = "Talk to Orven")
	public static NpcStringId TALK_TO_ORVEN2;
	
	@ClientString(id = 578512, message = "Talk to Shuvann")
	public static NpcStringId TALK_TO_SHUVANN2;
	
	@ClientString(id = 578601, message = "Resident Problem Solver (Lv. 61-65)")
	public static NpcStringId RESIDENT_PROBLEM_SOLVER_LV_61_65;
	
	@ClientString(id = 578602, message = "Resident Problem Solver (Lv. 61-65) (In Progress)")
	public static NpcStringId RESIDENT_PROBLEM_SOLVER_LV_61_65_IN_PROGRESS;
	
	@ClientString(id = 578603, message = "Resident Problem Solver (Lv. 61-65) (Done)")
	public static NpcStringId RESIDENT_PROBLEM_SOLVER_LV_61_65_DONE;
	
	@ClientString(id = 578604, message = "Resident Problem Solver (Lv. 61-65)")
	public static NpcStringId RESIDENT_PROBLEM_SOLVER_LV_61_652;
	
	@ClientString(id = 578611, message = "Kill Monsters in the Fields of Massacre")
	public static NpcStringId KILL_MONSTERS_IN_THE_FIELDS_OF_MASSACRE;
	
	@ClientString(id = 578701, message = "A Spy Mission (Lv. 61-65)")
	public static NpcStringId A_SPY_MISSION_LV_61_65;
	
	@ClientString(id = 578702, message = "A Spy Mission (Lv. 61-65) (In Progress)")
	public static NpcStringId A_SPY_MISSION_LV_61_65_IN_PROGRESS;
	
	@ClientString(id = 578703, message = "A Spy Mission (Lv. 61-65) (Done)")
	public static NpcStringId A_SPY_MISSION_LV_61_65_DONE;
	
	@ClientString(id = 578704, message = "A Spy Mission (Lv. 61-65)")
	public static NpcStringId A_SPY_MISSION_LV_61_652;
	
	@ClientString(id = 578901, message = "Letters from the Queen: Swamp of Screams (Lv. 65-69)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_SWAMP_OF_SCREAMS_LV_65_69;
	
	@ClientString(id = 578902, message = "Letters from the Queen: Swamp of Screams (Lv. 65-69) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_SWAMP_OF_SCREAMS_LV_65_69_IN_PROGRESS;
	
	@ClientString(id = 578903, message = "Letters from the Queen: Swamp of Screams (Lv. 65-69) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_SWAMP_OF_SCREAMS_LV_65_69_DONE;
	
	@ClientString(id = 578904, message = "Letters from the Queen: Swamp of Screams (Lv. 65-69)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_SWAMP_OF_SCREAMS_LV_65_692;
	
	@ClientString(id = 578911, message = "Talk to Innocentin")
	public static NpcStringId TALK_TO_INNOCENTIN;
	
	@ClientString(id = 578912, message = "Talk to Dokara")
	public static NpcStringId TALK_TO_DOKARA2;
	
	@ClientString(id = 579001, message = "A Mercenary Helper (Lv. 65-70)")
	public static NpcStringId A_MERCENARY_HELPER_LV_65_70;
	
	@ClientString(id = 579002, message = "A Mercenary Helper (Lv. 65-70) (In Progress)")
	public static NpcStringId A_MERCENARY_HELPER_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 579003, message = "A Mercenary Helper (Lv. 65-70) (Done)")
	public static NpcStringId A_MERCENARY_HELPER_LV_65_70_DONE;
	
	@ClientString(id = 579004, message = "A Mercenary Helper (Lv. 65-70)")
	public static NpcStringId A_MERCENARY_HELPER_LV_65_702;
	
	@ClientString(id = 579011, message = "Kill Stakatos")
	public static NpcStringId KILL_STAKATOS;
	
	@ClientString(id = 579101, message = "The Man of Mystery (Lv. 65-70)")
	public static NpcStringId THE_MAN_OF_MYSTERY_LV_65_70;
	
	@ClientString(id = 579102, message = "The Man of Mystery (Lv. 65-70) (In Progress)")
	public static NpcStringId THE_MAN_OF_MYSTERY_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 579103, message = "The Man of Mystery (Lv. 65-70) (Done)")
	public static NpcStringId THE_MAN_OF_MYSTERY_LV_65_70_DONE;
	
	@ClientString(id = 579104, message = "The Man of Mystery (Lv. 65-70)")
	public static NpcStringId THE_MAN_OF_MYSTERY_LV_65_702;
	
	@ClientString(id = 579201, message = "Letters from the Queen: Forest of the Dead (Lv. 65-69)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FOREST_OF_THE_DEAD_LV_65_69;
	
	@ClientString(id = 579202, message = "Letters from the Queen: Forest of the Dead (Lv. 65-69) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FOREST_OF_THE_DEAD_LV_65_69_IN_PROGRESS;
	
	@ClientString(id = 579203, message = "Letters from the Queen: Forest of the Dead (Lv. 65-69) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FOREST_OF_THE_DEAD_LV_65_69_DONE;
	
	@ClientString(id = 579204, message = "Letters from the Queen: Forest of the Dead (Lv. 65-69)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_FOREST_OF_THE_DEAD_LV_65_692;
	
	@ClientString(id = 579211, message = "Talk to Innocentin")
	public static NpcStringId TALK_TO_INNOCENTIN2;
	
	@ClientString(id = 579212, message = "Talk to Hatuba")
	public static NpcStringId TALK_TO_HATUBA2;
	
	@ClientString(id = 579301, message = "Save the Souls (Lv. 65-70)")
	public static NpcStringId SAVE_THE_SOULS_LV_65_70;
	
	@ClientString(id = 579302, message = "Save the Souls (Lv. 65-70) (In Progress)")
	public static NpcStringId SAVE_THE_SOULS_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 579303, message = "Save the Souls (Lv. 65-70) (Done)")
	public static NpcStringId SAVE_THE_SOULS_LV_65_70_DONE;
	
	@ClientString(id = 579304, message = "Save the Souls (Lv. 65-70)")
	public static NpcStringId SAVE_THE_SOULS_LV_65_702;
	
	@ClientString(id = 579311, message = "Kill the Undead")
	public static NpcStringId KILL_THE_UNDEAD;
	
	@ClientString(id = 579401, message = "Investigate the Forest (Lv. 65-70)")
	public static NpcStringId INVESTIGATE_THE_FOREST_LV_65_70;
	
	@ClientString(id = 579402, message = "Investigate the Forest (Lv. 65-70) (In Progress)")
	public static NpcStringId INVESTIGATE_THE_FOREST_LV_65_70_IN_PROGRESS;
	
	@ClientString(id = 579403, message = "Investigate the Forest (Lv. 65-70) (Done)")
	public static NpcStringId INVESTIGATE_THE_FOREST_LV_65_70_DONE;
	
	@ClientString(id = 579404, message = "Investigate the Forest (Lv. 65-70)")
	public static NpcStringId INVESTIGATE_THE_FOREST_LV_65_702;
	
	@ClientString(id = 579411, message = "Kill the Undead near the Cursed Village")
	public static NpcStringId KILL_THE_UNDEAD_NEAR_THE_CURSED_VILLAGE;
	
	@ClientString(id = 579412, message = "Investigate the Tombstone")
	public static NpcStringId INVESTIGATE_THE_TOMBSTONE;
	
	@ClientString(id = 579501, message = "Letters from the Queen: Wall of Argos (Lv. 70-75)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_WALL_OF_ARGOS_LV_70_75;
	
	@ClientString(id = 579502, message = "Letters from the Queen: Wall of Argos (Lv. 70-75) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_WALL_OF_ARGOS_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 579503, message = "Letters from the Queen: Wall of Argos (Lv. 70-75) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_WALL_OF_ARGOS_LV_70_75_DONE;
	
	@ClientString(id = 579504, message = "Letters from the Queen: Wall of Argos (Lv. 70-75)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_WALL_OF_ARGOS_LV_70_752;
	
	@ClientString(id = 579511, message = "Talk to Gregory")
	public static NpcStringId TALK_TO_GREGORY;
	
	@ClientString(id = 579512, message = "Talk to the Hermit")
	public static NpcStringId TALK_TO_THE_HERMIT;
	
	@ClientString(id = 579601, message = "The Eye that Defied the Gods (Lv. 70-75)")
	public static NpcStringId THE_EYE_THAT_DEFIED_THE_GODS_LV_70_75;
	
	@ClientString(id = 579602, message = "The Eye that Defied the Gods (Lv. 70-75) (In Progress)")
	public static NpcStringId THE_EYE_THAT_DEFIED_THE_GODS_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 579603, message = "The Eye that Defied the Gods (Lv. 70-75) (Done)")
	public static NpcStringId THE_EYE_THAT_DEFIED_THE_GODS_LV_70_75_DONE;
	
	@ClientString(id = 579604, message = "The Eye that Defied the Gods (Lv. 70-75)")
	public static NpcStringId THE_EYE_THAT_DEFIED_THE_GODS_LV_70_752;
	
	@ClientString(id = 579701, message = "Crossing Fate (Lv. 70-75)")
	public static NpcStringId CROSSING_FATE_LV_70_75;
	
	@ClientString(id = 579702, message = "Crossing Fate (Lv. 70-75) (In Progress)")
	public static NpcStringId CROSSING_FATE_LV_70_75_IN_PROGRESS;
	
	@ClientString(id = 579703, message = "Crossing Fate (Lv. 70-75) (Done)")
	public static NpcStringId CROSSING_FATE_LV_70_75_DONE;
	
	@ClientString(id = 579704, message = "Crossing Fate (Lv. 70-75)")
	public static NpcStringId CROSSING_FATE_LV_70_752;
	
	@ClientString(id = 579801, message = "Letters from the Queen: Dragon Valley (Lv. 76-84)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_DRAGON_VALLEY_LV_76_84;
	
	@ClientString(id = 579802, message = "Letters from the Queen: Dragon Valley (Lv. 76-84) (In Progress)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_DRAGON_VALLEY_LV_76_84_IN_PROGRESS;
	
	@ClientString(id = 579803, message = "Letters from the Queen: Dragon Valley (Lv. 76-84) (Done)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_DRAGON_VALLEY_LV_76_84_DONE;
	
	@ClientString(id = 579804, message = "Letters from the Queen: Dragon Valley (Lv. 76-84)")
	public static NpcStringId LETTERS_FROM_THE_QUEEN_DRAGON_VALLEY_LV_76_842;
	
	@ClientString(id = 579811, message = "Talk to Maximilian")
	public static NpcStringId TALK_TO_MAXIMILIAN;
	
	@ClientString(id = 579812, message = "Talk to Namo")
	public static NpcStringId TALK_TO_NAMO;
	
	@ClientString(id = 579901, message = "Strange Things Afoot in the Valley (Lv. 76-85)")
	public static NpcStringId STRANGE_THINGS_AFOOT_IN_THE_VALLEY_LV_76_85;
	
	@ClientString(id = 579902, message = "Strange Things Afoot in the Valley (Lv. 76-85) (In Progress)")
	public static NpcStringId STRANGE_THINGS_AFOOT_IN_THE_VALLEY_LV_76_85_IN_PROGRESS;
	
	@ClientString(id = 579903, message = "Strange Things Afoot in the Valley (Lv. 76-85) (Done)")
	public static NpcStringId STRANGE_THINGS_AFOOT_IN_THE_VALLEY_LV_76_85_DONE;
	
	@ClientString(id = 579904, message = "Strange Things Afoot in the Valley (Lv. 76-85)")
	public static NpcStringId STRANGE_THINGS_AFOOT_IN_THE_VALLEY_LV_76_852;
	
	@ClientString(id = 579911, message = "Kill Monsters in the Dragon Valley")
	public static NpcStringId KILL_MONSTERS_IN_THE_DRAGON_VALLEY;
	
	@ClientString(id = 580001, message = "Reconnaissance at Dragon Valley (Lv. 76-85)")
	public static NpcStringId RECONNAISSANCE_AT_DRAGON_VALLEY_LV_76_85;
	
	@ClientString(id = 580002, message = "Reconnaissance at Dragon Valley (Lv. 76-85) (In Progress)")
	public static NpcStringId RECONNAISSANCE_AT_DRAGON_VALLEY_LV_76_85_IN_PROGRESS;
	
	@ClientString(id = 580003, message = "Reconnaissance at Dragon Valley (Lv. 76-85) (Done)")
	public static NpcStringId RECONNAISSANCE_AT_DRAGON_VALLEY_LV_76_85_DONE;
	
	@ClientString(id = 580004, message = "Reconnaissance at Dragon Valley (Lv. 76-85)")
	public static NpcStringId RECONNAISSANCE_AT_DRAGON_VALLEY_LV_76_852;
	
	@ClientString(id = 580011, message = "Kill Monsters in Dragon Valley")
	public static NpcStringId KILL_MONSTERS_IN_DRAGON_VALLEY;
	
	@ClientString(id = 580101, message = "The Dimensional Warp, Part 1")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_1;
	
	@ClientString(id = 580102, message = "The Dimensional Warp, Part 1 (In Progress)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_1_IN_PROGRESS;
	
	@ClientString(id = 580103, message = "The Dimensional Warp, Part 1 (Done)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_1_DONE;
	
	@ClientString(id = 580104, message = "The Dimensional Warp, Part 1")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_12;
	
	@ClientString(id = 580201, message = "The Dimensional Warp, Part 2")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_2;
	
	@ClientString(id = 580202, message = "The Dimensional Warp, Part 2 (In Progress)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_2_IN_PROGRESS;
	
	@ClientString(id = 580203, message = "The Dimensional Warp, Part 2 (Done)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_2_DONE;
	
	@ClientString(id = 580204, message = "The Dimensional Warp, Part 2")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_22;
	
	@ClientString(id = 580301, message = "The Dimensional Warp, Part 3")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_3;
	
	@ClientString(id = 580302, message = "The Dimensional Warp, Part 3 (In Progress)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_3_IN_PROGRESS;
	
	@ClientString(id = 580303, message = "The Dimensional Warp, Part 3 (Done)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_3_DONE;
	
	@ClientString(id = 580304, message = "The Dimensional Warp, Part 3")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_32;
	
	@ClientString(id = 580401, message = "The Dimensional Warp, Part 4")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_4;
	
	@ClientString(id = 580402, message = "The Dimensional Warp, Part 4 (In Progress)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_4_IN_PROGRESS;
	
	@ClientString(id = 580403, message = "The Dimensional Warp, Part 4 (Done)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_4_DONE;
	
	@ClientString(id = 580404, message = "The Dimensional Warp, Part 4")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_42;
	
	@ClientString(id = 580501, message = "The Dimensional Warp, Part 5")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_5;
	
	@ClientString(id = 580502, message = "The Dimensional Warp, Part 5 (In Progress)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_5_IN_PROGRESS;
	
	@ClientString(id = 580503, message = "The Dimensional Warp, Part 5 (Done)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_5_DONE;
	
	@ClientString(id = 580504, message = "The Dimensional Warp, Part 5")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_52;
	
	@ClientString(id = 580601, message = "The Dimensional Warp, Part 6")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_6;
	
	@ClientString(id = 580602, message = "The Dimensional Warp, Part 6 (In Progress)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_6_IN_PROGRESS;
	
	@ClientString(id = 580603, message = "The Dimensional Warp, Part 6 (Done)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_6_DONE;
	
	@ClientString(id = 580604, message = "The Dimensional Warp, Part 6")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_62;
	
	@ClientString(id = 580701, message = "The Dimensional Warp, Part 7")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_7;
	
	@ClientString(id = 580702, message = "The Dimensional Warp, Part 7 (In Progress)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_7_IN_PROGRESS;
	
	@ClientString(id = 580703, message = "The Dimensional Warp, Part 7 (Done)")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_7_DONE;
	
	@ClientString(id = 580704, message = "The Dimensional Warp, Part 7")
	public static NpcStringId THE_DIMENSIONAL_WARP_PART_72;
	
	@ClientString(id = 580901, message = "The Hero's Journey: Blazing Swamp (Lv. 97-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_BLAZING_SWAMP_LV_97_99;
	
	@ClientString(id = 580902, message = "The Hero's Journey: Blazing Swamp (Lv. 97-99) (In Progress)")
	public static NpcStringId THE_HERO_S_JOURNEY_BLAZING_SWAMP_LV_97_99_IN_PROGRESS;
	
	@ClientString(id = 580903, message = "The Hero's Journey: Blazing Swamp (Lv. 97-99) (Done)")
	public static NpcStringId THE_HERO_S_JOURNEY_BLAZING_SWAMP_LV_97_99_DONE;
	
	@ClientString(id = 580904, message = "The Hero's Journey: Blazing Swamp (Lv. 97-99)")
	public static NpcStringId THE_HERO_S_JOURNEY_BLAZING_SWAMP_LV_97_992;
	
	@ClientString(id = 580905, message = "Talk to Harp Zu Hestui")
	public static NpcStringId TALK_TO_HARP_ZU_HESTUI;
	
	@ClientString(id = 1000001, message = "A non-permitted target has been discovered.")
	public static NpcStringId A_NON_PERMITTED_TARGET_HAS_BEEN_DISCOVERED;
	
	@ClientString(id = 1000002, message = "Intruder removal system initiated.")
	public static NpcStringId INTRUDER_REMOVAL_SYSTEM_INITIATED;
	
	@ClientString(id = 1000003, message = "Removing intruders.")
	public static NpcStringId REMOVING_INTRUDERS;
	
	@ClientString(id = 1000004, message = "A fatal error has occurred.")
	public static NpcStringId A_FATAL_ERROR_HAS_OCCURRED;
	
	@ClientString(id = 1000005, message = "System is being shut down...")
	public static NpcStringId SYSTEM_IS_BEING_SHUT_DOWN;
	
	@ClientString(id = 1000006, message = "......")
	public static NpcStringId EMPTY;
	
	@ClientString(id = 1000007, message = "We shall see about that!")
	public static NpcStringId WE_SHALL_SEE_ABOUT_THAT;
	
	@ClientString(id = 1000008, message = "I will definitely repay this humiliation!")
	public static NpcStringId I_WILL_DEFINITELY_REPAY_THIS_HUMILIATION;
	
	@ClientString(id = 1000009, message = "Retreat!")
	public static NpcStringId RETREAT;
	
	@ClientString(id = 1000010, message = "Tactical retreat!")
	public static NpcStringId TACTICAL_RETREAT;
	
	@ClientString(id = 1000011, message = "Mass fleeing!")
	public static NpcStringId MASS_FLEEING;
	
	@ClientString(id = 1000012, message = "It's stronger than expected!")
	public static NpcStringId IT_S_STRONGER_THAN_EXPECTED;
	
	@ClientString(id = 1000013, message = "I'll kill you next time!")
	public static NpcStringId I_LL_KILL_YOU_NEXT_TIME;
	
	@ClientString(id = 1000014, message = "I'll definitely kill you next time!")
	public static NpcStringId I_LL_DEFINITELY_KILL_YOU_NEXT_TIME;
	
	@ClientString(id = 1000015, message = "Oh! How strong!")
	public static NpcStringId OH_HOW_STRONG;
	
	@ClientString(id = 1000016, message = "Invader!")
	public static NpcStringId INVADER;
	
	@ClientString(id = 1000017, message = "There is no reason for you to kill me! I have nothing you need!")
	public static NpcStringId THERE_IS_NO_REASON_FOR_YOU_TO_KILL_ME_I_HAVE_NOTHING_YOU_NEED;
	
	@ClientString(id = 1000018, message = "Someday you will pay!")
	public static NpcStringId SOMEDAY_YOU_WILL_PAY;
	
	@ClientString(id = 1000019, message = "I won't just stand still while you hit me.")
	public static NpcStringId I_WON_T_JUST_STAND_STILL_WHILE_YOU_HIT_ME;
	
	@ClientString(id = 1000020, message = "Stop hitting!")
	public static NpcStringId STOP_HITTING;
	
	@ClientString(id = 1000021, message = "It hurts to the bone!")
	public static NpcStringId IT_HURTS_TO_THE_BONE;
	
	@ClientString(id = 1000022, message = "Am I the neighborhood drum for beating!")
	public static NpcStringId AM_I_THE_NEIGHBORHOOD_DRUM_FOR_BEATING;
	
	@ClientString(id = 1000023, message = "Follow me if you want!")
	public static NpcStringId FOLLOW_ME_IF_YOU_WANT;
	
	@ClientString(id = 1000024, message = "Surrender!")
	public static NpcStringId SURRENDER;
	
	@ClientString(id = 1000025, message = "Oh, I'm dead!")
	public static NpcStringId OH_I_M_DEAD;
	
	@ClientString(id = 1000026, message = "I'll be back!")
	public static NpcStringId I_LL_BE_BACK;
	
	@ClientString(id = 1000027, message = "I'll give you ten million arena if you let me live!")
	public static NpcStringId I_LL_GIVE_YOU_TEN_MILLION_ARENA_IF_YOU_LET_ME_LIVE;
	
	@ClientString(id = 1000028, message = "$s1. Stop kidding yourself about your own powerlessness!")
	public static NpcStringId S1_STOP_KIDDING_YOURSELF_ABOUT_YOUR_OWN_POWERLESSNESS;
	
	@ClientString(id = 1000029, message = "$s1. I'll make you feel what true fear is!")
	public static NpcStringId S1_I_LL_MAKE_YOU_FEEL_WHAT_TRUE_FEAR_IS;
	
	@ClientString(id = 1000030, message = "You're really stupid to have challenged me. $s1! Get ready!")
	public static NpcStringId YOU_RE_REALLY_STUPID_TO_HAVE_CHALLENGED_ME_S1_GET_READY;
	
	@ClientString(id = 1000031, message = "$s1. Do you think that's going to work?!")
	public static NpcStringId S1_DO_YOU_THINK_THAT_S_GOING_TO_WORK;
	
	@ClientString(id = 1000032, message = "I will definitely reclaim my honor which has been tarnished!")
	public static NpcStringId I_WILL_DEFINITELY_RECLAIM_MY_HONOR_WHICH_HAS_BEEN_TARNISHED;
	
	@ClientString(id = 1000033, message = "Show me the wrath of the knight whose honor has been downtrodden!")
	public static NpcStringId SHOW_ME_THE_WRATH_OF_THE_KNIGHT_WHOSE_HONOR_HAS_BEEN_DOWNTRODDEN;
	
	@ClientString(id = 1000034, message = "Death to the hypocrite!")
	public static NpcStringId DEATH_TO_THE_HYPOCRITE;
	
	@ClientString(id = 1000035, message = "I'll never sleep until I've shed my dishonor!")
	public static NpcStringId I_LL_NEVER_SLEEP_UNTIL_I_VE_SHED_MY_DISHONOR;
	
	@ClientString(id = 1000036, message = "I'm here for the ones that are cursing the world!")
	public static NpcStringId I_M_HERE_FOR_THE_ONES_THAT_ARE_CURSING_THE_WORLD;
	
	@ClientString(id = 1000037, message = "I'll turn you into a malignant spirit!")
	public static NpcStringId I_LL_TURN_YOU_INTO_A_MALIGNANT_SPIRIT;
	
	@ClientString(id = 1000038, message = "I'll curse you with the power of revenge and hate!")
	public static NpcStringId I_LL_CURSE_YOU_WITH_THE_POWER_OF_REVENGE_AND_HATE;
	
	@ClientString(id = 1000039, message = "For the glory of Gracia!")
	public static NpcStringId FOR_THE_GLORY_OF_GRACIA;
	
	@ClientString(id = 1000040, message = "Do you dare pit your power against me?")
	public static NpcStringId DO_YOU_DARE_PIT_YOUR_POWER_AGAINST_ME;
	
	@ClientString(id = 1000041, message = "I... I am defeated!!!")
	public static NpcStringId I_I_AM_DEFEATED;
	
	@ClientString(id = 1000042, message = "I am conveying the will of Nurka! Everybody get out of my way!")
	public static NpcStringId I_AM_CONVEYING_THE_WILL_OF_NURKA_EVERYBODY_GET_OUT_OF_MY_WAY;
	
	@ClientString(id = 1000043, message = "Those who stand against me shall die horribly!")
	public static NpcStringId THOSE_WHO_STAND_AGAINST_ME_SHALL_DIE_HORRIBLY;
	
	@ClientString(id = 1000044, message = "Do you dare to block my way?!")
	public static NpcStringId DO_YOU_DARE_TO_BLOCK_MY_WAY;
	
	@ClientString(id = 1000045, message = "My comrades will get revenge!")
	public static NpcStringId MY_COMRADES_WILL_GET_REVENGE;
	
	@ClientString(id = 1000046, message = "You heathen blasphemers of this holy place will be punished!")
	public static NpcStringId YOU_HEATHEN_BLASPHEMERS_OF_THIS_HOLY_PLACE_WILL_BE_PUNISHED;
	
	@ClientString(id = 1000047, message = "Step forward, you worthless creatures who challenge my authority!")
	public static NpcStringId STEP_FORWARD_YOU_WORTHLESS_CREATURES_WHO_CHALLENGE_MY_AUTHORITY;
	
	@ClientString(id = 1000048, message = "My creator... The unchanging faithfulness to my master...")
	public static NpcStringId MY_CREATOR_THE_UNCHANGING_FAITHFULNESS_TO_MY_MASTER;
	
	@ClientString(id = 1000049, message = "Master of the tower... My master... master... Where is he?")
	public static NpcStringId MASTER_OF_THE_TOWER_MY_MASTER_MASTER_WHERE_IS_HE;
	
	@ClientString(id = 1000050, message = "I AM THE ONE CARRYING OUT THE WILL OF CORE.")
	public static NpcStringId I_AM_THE_ONE_CARRYING_OUT_THE_WILL_OF_CORE;
	
	@ClientString(id = 1000051, message = "DESTROY THE INVADER.")
	public static NpcStringId DESTROY_THE_INVADER;
	
	@ClientString(id = 1000052, message = "STRANGE CONDITION - DOESN'T WORK")
	public static NpcStringId STRANGE_CONDITION_DOESN_T_WORK;
	
	@ClientString(id = 1000053, message = "According to the command of Beleth... I'm going to observe you guys!")
	public static NpcStringId ACCORDING_TO_THE_COMMAND_OF_BELETH_I_M_GOING_TO_OBSERVE_YOU_GUYS;
	
	@ClientString(id = 1000054, message = "You people make me sick! No sense of loyalty whatsoever!")
	public static NpcStringId YOU_PEOPLE_MAKE_ME_SICK_NO_SENSE_OF_LOYALTY_WHATSOEVER;
	
	@ClientString(id = 1000055, message = "A challenge against me is the same as a challenge against Beleth...")
	public static NpcStringId A_CHALLENGE_AGAINST_ME_IS_THE_SAME_AS_A_CHALLENGE_AGAINST_BELETH;
	
	@ClientString(id = 1000056, message = "Beleth is always watching over you guys!")
	public static NpcStringId BELETH_IS_ALWAYS_WATCHING_OVER_YOU_GUYS;
	
	@ClientString(id = 1000057, message = "That was really close! Antharas opened its eyes!")
	public static NpcStringId THAT_WAS_REALLY_CLOSE_ANTHARAS_OPENED_ITS_EYES;
	
	@ClientString(id = 1000058, message = "You who disobey the will of Antharas! Die!")
	public static NpcStringId YOU_WHO_DISOBEY_THE_WILL_OF_ANTHARAS_DIE;
	
	@ClientString(id = 1000059, message = "Antharas has taken my life!")
	public static NpcStringId ANTHARAS_HAS_TAKEN_MY_LIFE;
	
	@ClientString(id = 1000060, message = "I crossed back over the marshlands of death to reclaim the treasure!")
	public static NpcStringId I_CROSSED_BACK_OVER_THE_MARSHLANDS_OF_DEATH_TO_RECLAIM_THE_TREASURE;
	
	@ClientString(id = 1000061, message = "Bring over and surrender your precious gold treasure to me!")
	public static NpcStringId BRING_OVER_AND_SURRENDER_YOUR_PRECIOUS_GOLD_TREASURE_TO_ME;
	
	@ClientString(id = 1000062, message = "I'll kill you in an instant!")
	public static NpcStringId I_LL_KILL_YOU_IN_AN_INSTANT;
	
	@ClientString(id = 1000063, message = "No! The treasure is still..!")
	public static NpcStringId NO_THE_TREASURE_IS_STILL;
	
	@ClientString(id = 1000064, message = "Invaders of Dragon Valley will never live to return!")
	public static NpcStringId INVADERS_OF_DRAGON_VALLEY_WILL_NEVER_LIVE_TO_RETURN;
	
	@ClientString(id = 1000065, message = "I am the guardian that honors the command of Antharas to watch over this place!")
	public static NpcStringId I_AM_THE_GUARDIAN_THAT_HONORS_THE_COMMAND_OF_ANTHARAS_TO_WATCH_OVER_THIS_PLACE;
	
	@ClientString(id = 1000066, message = "You've set foot in Dragon Valley without permission! The penalty is death!")
	public static NpcStringId YOU_VE_SET_FOOT_IN_DRAGON_VALLEY_WITHOUT_PERMISSION_THE_PENALTY_IS_DEATH;
	
	@ClientString(id = 1000067, message = "Antharas has taken my life!")
	public static NpcStringId ANTHARAS_HAS_TAKEN_MY_LIFE2;
	
	@ClientString(id = 1000068, message = "The joy of killing! The ecstasy of looting! Hey guys, let's have a go at it again!")
	public static NpcStringId THE_JOY_OF_KILLING_THE_ECSTASY_OF_LOOTING_HEY_GUYS_LET_S_HAVE_A_GO_AT_IT_AGAIN;
	
	@ClientString(id = 1000069, message = "There really are still lots of folks in the world without fear! I'll teach you a lesson!")
	public static NpcStringId THERE_REALLY_ARE_STILL_LOTS_OF_FOLKS_IN_THE_WORLD_WITHOUT_FEAR_I_LL_TEACH_YOU_A_LESSON;
	
	@ClientString(id = 1000070, message = "If you hand over everything you've got, I'll at least spare your life!")
	public static NpcStringId IF_YOU_HAND_OVER_EVERYTHING_YOU_VE_GOT_I_LL_AT_LEAST_SPARE_YOUR_LIFE;
	
	@ClientString(id = 1000071, message = "Kneel down before one such as this!")
	public static NpcStringId KNEEL_DOWN_BEFORE_ONE_SUCH_AS_THIS;
	
	@ClientString(id = 1000072, message = "Honor the master's wishes and punish all the invaders!")
	public static NpcStringId HONOR_THE_MASTER_S_WISHES_AND_PUNISH_ALL_THE_INVADERS;
	
	@ClientString(id = 1000073, message = "Follow the master's wishes and punish the invaders!")
	public static NpcStringId FOLLOW_THE_MASTER_S_WISHES_AND_PUNISH_THE_INVADERS;
	
	@ClientString(id = 1000074, message = "Death is nothing more than a momentary rest...")
	public static NpcStringId DEATH_IS_NOTHING_MORE_THAN_A_MOMENTARY_REST;
	
	@ClientString(id = 1000075, message = "Listen! This is the end of the human era! Antharas has awakened!")
	public static NpcStringId LISTEN_THIS_IS_THE_END_OF_THE_HUMAN_ERA_ANTHARAS_HAS_AWAKENED;
	
	@ClientString(id = 1000076, message = "Present the lives of four people to Antharas!")
	public static NpcStringId PRESENT_THE_LIVES_OF_FOUR_PEOPLE_TO_ANTHARAS;
	
	@ClientString(id = 1000077, message = "This is unbelievable! How could I have lost to one so inferior to myself?")
	public static NpcStringId THIS_IS_UNBELIEVABLE_HOW_COULD_I_HAVE_LOST_TO_ONE_SO_INFERIOR_TO_MYSELF;
	
	@ClientString(id = 1000078, message = "I carry the power of darkness and have returned from the abyss.")
	public static NpcStringId I_CARRY_THE_POWER_OF_DARKNESS_AND_HAVE_RETURNED_FROM_THE_ABYSS;
	
	@ClientString(id = 1000079, message = "It's detestable.")
	public static NpcStringId IT_S_DETESTABLE;
	
	@ClientString(id = 1000080, message = "I finally find rest...")
	public static NpcStringId I_FINALLY_FIND_REST;
	
	@ClientString(id = 1000081, message = "Glory to Orfen!")
	public static NpcStringId GLORY_TO_ORFEN;
	
	@ClientString(id = 1000082, message = "In the name of Orfen, I can never forgive you who are invading this place!")
	public static NpcStringId IN_THE_NAME_OF_ORFEN_I_CAN_NEVER_FORGIVE_YOU_WHO_ARE_INVADING_THIS_PLACE;
	
	@ClientString(id = 1000083, message = "I'll make you pay the price for fearlessly entering Orfen's land!")
	public static NpcStringId I_LL_MAKE_YOU_PAY_THE_PRICE_FOR_FEARLESSLY_ENTERING_ORFEN_S_LAND;
	
	@ClientString(id = 1000084, message = "Even if you disappear into nothingness, you will still face the life-long suffering of the curse that I have given you.")
	public static NpcStringId EVEN_IF_YOU_DISAPPEAR_INTO_NOTHINGNESS_YOU_WILL_STILL_FACE_THE_LIFE_LONG_SUFFERING_OF_THE_CURSE_THAT_I_HAVE_GIVEN_YOU;
	
	@ClientString(id = 1000085, message = "I'll stand against anyone that makes light of the sacred place of the Elves!")
	public static NpcStringId I_LL_STAND_AGAINST_ANYONE_THAT_MAKES_LIGHT_OF_THE_SACRED_PLACE_OF_THE_ELVES;
	
	@ClientString(id = 1000086, message = "I will kill with my own hands anyone that defiles our home!")
	public static NpcStringId I_WILL_KILL_WITH_MY_OWN_HANDS_ANYONE_THAT_DEFILES_OUR_HOME;
	
	@ClientString(id = 1000087, message = "My brothers will never rest until we push you and your gang out of this valley!")
	public static NpcStringId MY_BROTHERS_WILL_NEVER_REST_UNTIL_WE_PUSH_YOU_AND_YOUR_GANG_OUT_OF_THIS_VALLEY;
	
	@ClientString(id = 1000088, message = "Until the day of destruction of Hestui!")
	public static NpcStringId UNTIL_THE_DAY_OF_DESTRUCTION_OF_HESTUI;
	
	@ClientString(id = 1000089, message = "If any intrepid Orcs remain, attack them!")
	public static NpcStringId IF_ANY_INTREPID_ORCS_REMAIN_ATTACK_THEM;
	
	@ClientString(id = 1000090, message = "I'll break your windpipe!")
	public static NpcStringId I_LL_BREAK_YOUR_WINDPIPE;
	
	@ClientString(id = 1000091, message = "Is revenge a failure?!")
	public static NpcStringId IS_REVENGE_A_FAILURE;
	
	@ClientString(id = 1000092, message = "The sparkling mithril of the dwarves and their pretty treasures! I'll get them all!")
	public static NpcStringId THE_SPARKLING_MITHRIL_OF_THE_DWARVES_AND_THEIR_PRETTY_TREASURES_I_LL_GET_THEM_ALL;
	
	@ClientString(id = 1000093, message = "Where are all the dreadful dwarves and their sparkling things?")
	public static NpcStringId WHERE_ARE_ALL_THE_DREADFUL_DWARVES_AND_THEIR_SPARKLING_THINGS;
	
	@ClientString(id = 1000094, message = "Hand over your pretty treasures!")
	public static NpcStringId HAND_OVER_YOUR_PRETTY_TREASURES;
	
	@ClientString(id = 1000095, message = "Hey! You should have run away!")
	public static NpcStringId HEY_YOU_SHOULD_HAVE_RUN_AWAY;
	
	@ClientString(id = 1000096, message = "DESTRUCTION - EXTINCTION - SLAUGHTER - COLLAPSE! DESTRUCTION - EXTINCTION - SLAUGHTER - COLLAPSE!")
	public static NpcStringId DESTRUCTION_EXTINCTION_SLAUGHTER_COLLAPSE_DESTRUCTION_EXTINCTION_SLAUGHTER_COLLAPSE;
	
	@ClientString(id = 1000097, message = "Destruction! Destruction! Destruction! Destruction!")
	public static NpcStringId DESTRUCTION_DESTRUCTION_DESTRUCTION_DESTRUCTION;
	
	@ClientString(id = 1000098, message = "Destruction! Destruction! Destruction. . .")
	public static NpcStringId DESTRUCTION_DESTRUCTION_DESTRUCTION;
	
	@ClientString(id = 1000099, message = "Ta-da! Uthanka has returned!")
	public static NpcStringId TA_DA_UTHANKA_HAS_RETURNED;
	
	@ClientString(id = 1000100, message = "Wah, ha, ha, ha! Uthanka has taken over this island today!")
	public static NpcStringId WAH_HA_HA_HA_UTHANKA_HAS_TAKEN_OVER_THIS_ISLAND_TODAY;
	
	@ClientString(id = 1000101, message = "Whew! He's quite a guy!")
	public static NpcStringId WHEW_HE_S_QUITE_A_GUY;
	
	@ClientString(id = 1000102, message = "How exasperating and unfair to have things happen in such a meaningless way like this...")
	public static NpcStringId HOW_EXASPERATING_AND_UNFAIR_TO_HAVE_THINGS_HAPPEN_IN_SUCH_A_MEANINGLESS_WAY_LIKE_THIS;
	
	@ClientString(id = 1000103, message = "This world should be filled with fear and sadness...")
	public static NpcStringId THIS_WORLD_SHOULD_BE_FILLED_WITH_FEAR_AND_SADNESS;
	
	@ClientString(id = 1000104, message = "I won't forgive the world that cursed me!")
	public static NpcStringId I_WON_T_FORGIVE_THE_WORLD_THAT_CURSED_ME;
	
	@ClientString(id = 1000105, message = "I'll make everyone feel the same suffering as me!")
	public static NpcStringId I_LL_MAKE_EVERYONE_FEEL_THE_SAME_SUFFERING_AS_ME;
	
	@ClientString(id = 1000106, message = "I'll give you a curse that you'll never be able to remove forever!")
	public static NpcStringId I_LL_GIVE_YOU_A_CURSE_THAT_YOU_LL_NEVER_BE_ABLE_TO_REMOVE_FOREVER;
	
	@ClientString(id = 1000107, message = "I'll get revenge on you who slaughtered my compatriots!")
	public static NpcStringId I_LL_GET_REVENGE_ON_YOU_WHO_SLAUGHTERED_MY_COMPATRIOTS;
	
	@ClientString(id = 1000108, message = "Those who are afraid should get away and those who are brave should fight!")
	public static NpcStringId THOSE_WHO_ARE_AFRAID_SHOULD_GET_AWAY_AND_THOSE_WHO_ARE_BRAVE_SHOULD_FIGHT;
	
	@ClientString(id = 1000109, message = "I've got power from Beleth so do you think I'll be easily defeated?!")
	public static NpcStringId I_VE_GOT_POWER_FROM_BELETH_SO_DO_YOU_THINK_I_LL_BE_EASILY_DEFEATED;
	
	@ClientString(id = 1000110, message = "I am leaving now, but soon someone will come who will teach you all a lesson!")
	public static NpcStringId I_AM_LEAVING_NOW_BUT_SOON_SOMEONE_WILL_COME_WHO_WILL_TEACH_YOU_ALL_A_LESSON;
	
	@ClientString(id = 1000111, message = "Hey guys, let's make a round of our territory!")
	public static NpcStringId HEY_GUYS_LET_S_MAKE_A_ROUND_OF_OUR_TERRITORY;
	
	@ClientString(id = 1000112, message = "The rumor is that there are wild, uncivilized ruffians who have recently arrived in my territory.")
	public static NpcStringId THE_RUMOR_IS_THAT_THERE_ARE_WILD_UNCIVILIZED_RUFFIANS_WHO_HAVE_RECENTLY_ARRIVED_IN_MY_TERRITORY;
	
	@ClientString(id = 1000113, message = "Do you know who I am?! I am Sirocco! Everyone, attack!")
	public static NpcStringId DO_YOU_KNOW_WHO_I_AM_I_AM_SIROCCO_EVERYONE_ATTACK;
	
	@ClientString(id = 1000114, message = "What's just happened?! The invincible Sirocco was defeated by someone like you?!")
	public static NpcStringId WHAT_S_JUST_HAPPENED_THE_INVINCIBLE_SIROCCO_WAS_DEFEATED_BY_SOMEONE_LIKE_YOU;
	
	@ClientString(id = 1000115, message = "Oh, I'm really hungry...")
	public static NpcStringId OH_I_M_REALLY_HUNGRY;
	
	@ClientString(id = 1000116, message = "I smell food. Ooh...")
	public static NpcStringId I_SMELL_FOOD_OOH;
	
	@ClientString(id = 1000117, message = "Ooh...")
	public static NpcStringId OOH;
	
	@ClientString(id = 1000118, message = "What does honey of this place taste like?!")
	public static NpcStringId WHAT_DOES_HONEY_OF_THIS_PLACE_TASTE_LIKE;
	
	@ClientString(id = 1000119, message = "Give me some sweet, delicious golden honey!")
	public static NpcStringId GIVE_ME_SOME_SWEET_DELICIOUS_GOLDEN_HONEY;
	
	@ClientString(id = 1000120, message = "If you give me some honey, I'll at least spare your life...")
	public static NpcStringId IF_YOU_GIVE_ME_SOME_HONEY_I_LL_AT_LEAST_SPARE_YOUR_LIFE;
	
	@ClientString(id = 1000121, message = "Only for lack of honey did I lose to the likes of you.")
	public static NpcStringId ONLY_FOR_LACK_OF_HONEY_DID_I_LOSE_TO_THE_LIKES_OF_YOU;
	
	@ClientString(id = 1000122, message = "Where is the traitor Kuroboros!?")
	public static NpcStringId WHERE_IS_THE_TRAITOR_KUROBOROS;
	
	@ClientString(id = 1000123, message = "Look in every nook and cranny around here!")
	public static NpcStringId LOOK_IN_EVERY_NOOK_AND_CRANNY_AROUND_HERE;
	
	@ClientString(id = 1000124, message = "Are you Lackey of Kuroboros?! I'll knock you out in one shot!")
	public static NpcStringId ARE_YOU_LACKEY_OF_KUROBOROS_I_LL_KNOCK_YOU_OUT_IN_ONE_SHOT;
	
	@ClientString(id = 1000125, message = "He just closed his eyes without disposing of the traitor... How unfair!")
	public static NpcStringId HE_JUST_CLOSED_HIS_EYES_WITHOUT_DISPOSING_OF_THE_TRAITOR_HOW_UNFAIR;
	
	@ClientString(id = 1000126, message = "Hell for unbelievers in Kuroboros!")
	public static NpcStringId HELL_FOR_UNBELIEVERS_IN_KUROBOROS;
	
	@ClientString(id = 1000127, message = "The person that does not believe in Kuroboros, his life will soon become hell!")
	public static NpcStringId THE_PERSON_THAT_DOES_NOT_BELIEVE_IN_KUROBOROS_HIS_LIFE_WILL_SOON_BECOME_HELL;
	
	@ClientString(id = 1000128, message = "The lackey of that demented devil, the servant of a false god! I'll send that fool straight to hell!")
	public static NpcStringId THE_LACKEY_OF_THAT_DEMENTED_DEVIL_THE_SERVANT_OF_A_FALSE_GOD_I_LL_SEND_THAT_FOOL_STRAIGHT_TO_HELL;
	
	@ClientString(id = 1000129, message = "Uh... I'm not dying; I'm just disappearing for a moment... I'll resurrect again!")
	public static NpcStringId UH_I_M_NOT_DYING_I_M_JUST_DISAPPEARING_FOR_A_MOMENT_I_LL_RESURRECT_AGAIN;
	
	@ClientString(id = 1000130, message = "Hail to Kuroboros, the founder of our religion!")
	public static NpcStringId HAIL_TO_KUROBOROS_THE_FOUNDER_OF_OUR_RELIGION;
	
	@ClientString(id = 1000131, message = "Only those who believe in Patriarch Kuroboros shall receive salvation!")
	public static NpcStringId ONLY_THOSE_WHO_BELIEVE_IN_PATRIARCH_KUROBOROS_SHALL_RECEIVE_SALVATION;
	
	@ClientString(id = 1000132, message = "Are you the ones that Sharuk has incited?! You also should trust in Kuroboros and be saved!")
	public static NpcStringId ARE_YOU_THE_ONES_THAT_SHARUK_HAS_INCITED_YOU_ALSO_SHOULD_TRUST_IN_KUROBOROS_AND_BE_SAVED;
	
	@ClientString(id = 1000133, message = "Kuroboros will punish you.")
	public static NpcStringId KUROBOROS_WILL_PUNISH_YOU;
	
	@ClientString(id = 1000134, message = "You who have beautiful spirits that shine brightly! I have returned!")
	public static NpcStringId YOU_WHO_HAVE_BEAUTIFUL_SPIRITS_THAT_SHINE_BRIGHTLY_I_HAVE_RETURNED;
	
	@ClientString(id = 1000135, message = "You that are weary and exhausted... Entrust your souls to me.")
	public static NpcStringId YOU_THAT_ARE_WEARY_AND_EXHAUSTED_ENTRUST_YOUR_SOULS_TO_ME;
	
	@ClientString(id = 1000136, message = "The color of your soul is very attractive.")
	public static NpcStringId THE_COLOR_OF_YOUR_SOUL_IS_VERY_ATTRACTIVE;
	
	@ClientString(id = 1000137, message = "Those of you who live! Do you know how beautiful your souls are?!")
	public static NpcStringId THOSE_OF_YOU_WHO_LIVE_DO_YOU_KNOW_HOW_BEAUTIFUL_YOUR_SOULS_ARE;
	
	@ClientString(id = 1000138, message = "It... will... kill... everyone...")
	public static NpcStringId IT_WILL_KILL_EVERYONE;
	
	@ClientString(id = 1000139, message = "I'm... so... lonely...")
	public static NpcStringId I_M_SO_LONELY;
	
	@ClientString(id = 1000140, message = "My... enemy...!")
	public static NpcStringId MY_ENEMY;
	
	@ClientString(id = 1000141, message = "... Now... I'm not so lonely!")
	public static NpcStringId NOW_I_M_NOT_SO_LONELY;
	
	@ClientString(id = 1000142, message = "I will never forgive the Pixy Murika... that is trying to... kill us!")
	public static NpcStringId I_WILL_NEVER_FORGIVE_THE_PIXY_MURIKA_THAT_IS_TRYING_TO_KILL_US;
	
	@ClientString(id = 1000143, message = "Attack all the dull and stupid followers of Murika!")
	public static NpcStringId ATTACK_ALL_THE_DULL_AND_STUPID_FOLLOWERS_OF_MURIKA;
	
	@ClientString(id = 1000144, message = "I didn't have any idea about such ambitions!")
	public static NpcStringId I_DIDN_T_HAVE_ANY_IDEA_ABOUT_SUCH_AMBITIONS;
	
	@ClientString(id = 1000145, message = "This is not the end... It's just the beginning.")
	public static NpcStringId THIS_IS_NOT_THE_END_IT_S_JUST_THE_BEGINNING;
	
	@ClientString(id = 1000146, message = "Hey... Shall we have some fun for the first time in a long while?...")
	public static NpcStringId HEY_SHALL_WE_HAVE_SOME_FUN_FOR_THE_FIRST_TIME_IN_A_LONG_WHILE;
	
	@ClientString(id = 1000147, message = "There've been some things going around like crazy here recently...")
	public static NpcStringId THERE_VE_BEEN_SOME_THINGS_GOING_AROUND_LIKE_CRAZY_HERE_RECENTLY;
	
	@ClientString(id = 1000148, message = "Hey! Do you know who I am? I am Malex, Herald of Dagoniel! Attack!")
	public static NpcStringId HEY_DO_YOU_KNOW_WHO_I_AM_I_AM_MALEX_HERALD_OF_DAGONIEL_ATTACK;
	
	@ClientString(id = 1000149, message = "What's just happened?! The invincible Malex just lost to the likes of you?!")
	public static NpcStringId WHAT_S_JUST_HAPPENED_THE_INVINCIBLE_MALEX_JUST_LOST_TO_THE_LIKES_OF_YOU;
	
	@ClientString(id = 1000150, message = "It's something repeated in a vain life...")
	public static NpcStringId IT_S_SOMETHING_REPEATED_IN_A_VAIN_LIFE;
	
	@ClientString(id = 1000151, message = "Shake in fear, all you who value your lives!")
	public static NpcStringId SHAKE_IN_FEAR_ALL_YOU_WHO_VALUE_YOUR_LIVES;
	
	@ClientString(id = 1000152, message = "I'll make you feel suffering like a flame that is never extinguished!")
	public static NpcStringId I_LL_MAKE_YOU_FEEL_SUFFERING_LIKE_A_FLAME_THAT_IS_NEVER_EXTINGUISHED;
	
	@ClientString(id = 1000153, message = "Back to the dirt...")
	public static NpcStringId BACK_TO_THE_DIRT;
	
	@ClientString(id = 1000154, message = "Hail Varika!!")
	public static NpcStringId HAIL_VARIKA;
	
	@ClientString(id = 1000155, message = "Nobody can stop us!")
	public static NpcStringId NOBODY_CAN_STOP_US;
	
	@ClientString(id = 1000156, message = "You move slowly!")
	public static NpcStringId YOU_MOVE_SLOWLY;
	
	@ClientString(id = 1000157, message = "Varika! Go first!")
	public static NpcStringId VARIKA_GO_FIRST;
	
	@ClientString(id = 1000158, message = "Where am I? Who am I?")
	public static NpcStringId WHERE_AM_I_WHO_AM_I;
	
	@ClientString(id = 1000159, message = "Uh... My head hurts like it's going to burst! Who am I?")
	public static NpcStringId UH_MY_HEAD_HURTS_LIKE_IT_S_GOING_TO_BURST_WHO_AM_I;
	
	@ClientString(id = 1000160, message = "You jerk. You're a devil! You're a devil to have made me like this!")
	public static NpcStringId YOU_JERK_YOU_RE_A_DEVIL_YOU_RE_A_DEVIL_TO_HAVE_MADE_ME_LIKE_THIS;
	
	@ClientString(id = 1000161, message = "Where am I? What happened? Thank you!")
	public static NpcStringId WHERE_AM_I_WHAT_HAPPENED_THANK_YOU;
	
	@ClientString(id = 1000162, message = "Ukru Master!")
	public static NpcStringId UKRU_MASTER;
	
	@ClientString(id = 1000163, message = "Are you Matu?")
	public static NpcStringId ARE_YOU_MATU;
	
	@ClientString(id = 1000164, message = "Marak! Tubarin! Sabaracha!")
	public static NpcStringId MARAK_TUBARIN_SABARACHA;
	
	@ClientString(id = 1000165, message = "Pa'agrio Tama!")
	public static NpcStringId PA_AGRIO_TAMA;
	
	@ClientString(id = 1000166, message = "Accept the will of Icarus!")
	public static NpcStringId ACCEPT_THE_WILL_OF_ICARUS;
	
	@ClientString(id = 1000167, message = "The people who are blocking my way will not be forgiven...")
	public static NpcStringId THE_PEOPLE_WHO_ARE_BLOCKING_MY_WAY_WILL_NOT_BE_FORGIVEN;
	
	@ClientString(id = 1000168, message = "You are scum.")
	public static NpcStringId YOU_ARE_SCUM;
	
	@ClientString(id = 1000169, message = "You lack power.")
	public static NpcStringId YOU_LACK_POWER;
	
	@ClientString(id = 1000170, message = "Return")
	public static NpcStringId RETURN;
	
	@ClientString(id = 1000171, message = "Adena has been transferred.")
	public static NpcStringId ADENA_HAS_BEEN_TRANSFERRED;
	
	@ClientString(id = 1000172, message = "Event Number")
	public static NpcStringId EVENT_NUMBER;
	
	@ClientString(id = 1000173, message = "First Prize")
	public static NpcStringId FIRST_PRIZE;
	
	@ClientString(id = 1000174, message = "Second Prize")
	public static NpcStringId SECOND_PRIZE;
	
	@ClientString(id = 1000175, message = "Third Prize")
	public static NpcStringId THIRD_PRIZE;
	
	@ClientString(id = 1000176, message = "Fourth Prize")
	public static NpcStringId FOURTH_PRIZE;
	
	@ClientString(id = 1000177, message = "There has been no winning lottery ticket.")
	public static NpcStringId THERE_HAS_BEEN_NO_WINNING_LOTTERY_TICKET;
	
	@ClientString(id = 1000178, message = "The most recent winning lottery numbers")
	public static NpcStringId THE_MOST_RECENT_WINNING_LOTTERY_NUMBERS;
	
	@ClientString(id = 1000179, message = "Your lucky numbers have been selected above.")
	public static NpcStringId YOUR_LUCKY_NUMBERS_HAVE_BEEN_SELECTED_ABOVE;
	
	@ClientString(id = 1000180, message = "I wonder who it is that is lurking about..")
	public static NpcStringId I_WONDER_WHO_IT_IS_THAT_IS_LURKING_ABOUT;
	
	@ClientString(id = 1000181, message = "Sacred magical research is conducted here.")
	public static NpcStringId SACRED_MAGICAL_RESEARCH_IS_CONDUCTED_HERE;
	
	@ClientString(id = 1000182, message = "Behold the awesome power of magic!")
	public static NpcStringId BEHOLD_THE_AWESOME_POWER_OF_MAGIC;
	
	@ClientString(id = 1000183, message = "Your powers are impressive but you must not annoy our high level sorcerer.")
	public static NpcStringId YOUR_POWERS_ARE_IMPRESSIVE_BUT_YOU_MUST_NOT_ANNOY_OUR_HIGH_LEVEL_SORCERER;
	
	@ClientString(id = 1000184, message = "I am Barda, master of the Bandit Stronghold!")
	public static NpcStringId I_AM_BARDA_MASTER_OF_THE_BANDIT_STRONGHOLD;
	
	@ClientString(id = 1000185, message = "I, Master Barda, once owned that stronghold,")
	public static NpcStringId I_MASTER_BARDA_ONCE_OWNED_THAT_STRONGHOLD;
	
	@ClientString(id = 1000186, message = "Ah, very interesting!")
	public static NpcStringId AH_VERY_INTERESTING;
	
	@ClientString(id = 1000187, message = "You are more powerful than you appear. We'll meet again!")
	public static NpcStringId YOU_ARE_MORE_POWERFUL_THAN_YOU_APPEAR_WE_LL_MEET_AGAIN;
	
	@ClientString(id = 1000188, message = "You filthy sorcerers disgust me!")
	public static NpcStringId YOU_FILTHY_SORCERERS_DISGUST_ME;
	
	@ClientString(id = 1000189, message = "Why would you build a tower in our territory?")
	public static NpcStringId WHY_WOULD_YOU_BUILD_A_TOWER_IN_OUR_TERRITORY;
	
	@ClientString(id = 1000190, message = "Are you part of that evil gang of sorcerers?")
	public static NpcStringId ARE_YOU_PART_OF_THAT_EVIL_GANG_OF_SORCERERS;
	
	@ClientString(id = 1000191, message = "That is why I don't bother with anyone below the level of sorcerer.")
	public static NpcStringId THAT_IS_WHY_I_DON_T_BOTHER_WITH_ANYONE_BELOW_THE_LEVEL_OF_SORCERER;
	
	@ClientString(id = 1000192, message = "Ah, another beautiful day!")
	public static NpcStringId AH_ANOTHER_BEAUTIFUL_DAY;
	
	@ClientString(id = 1000193, message = "Our specialties are arson and looting.")
	public static NpcStringId OUR_SPECIALTIES_ARE_ARSON_AND_LOOTING;
	
	@ClientString(id = 1000194, message = "You will leave empty-handed!")
	public static NpcStringId YOU_WILL_LEAVE_EMPTY_HANDED;
	
	@ClientString(id = 1000195, message = "Ah, so you admire my treasure, do you? Try finding it! Ha!")
	public static NpcStringId AH_SO_YOU_ADMIRE_MY_TREASURE_DO_YOU_TRY_FINDING_IT_HA;
	
	@ClientString(id = 1000196, message = "Is everybody listening? Sirion has come back. Everyone chant and bow...")
	public static NpcStringId IS_EVERYBODY_LISTENING_SIRION_HAS_COME_BACK_EVERYONE_CHANT_AND_BOW;
	
	@ClientString(id = 1000197, message = "Bow down, you worthless humans!")
	public static NpcStringId BOW_DOWN_YOU_WORTHLESS_HUMANS;
	
	@ClientString(id = 1000198, message = "Very tacky!")
	public static NpcStringId VERY_TACKY;
	
	@ClientString(id = 1000199, message = "Don't think that you are invincible just because you defeated me!")
	public static NpcStringId DON_T_THINK_THAT_YOU_ARE_INVINCIBLE_JUST_BECAUSE_YOU_DEFEATED_ME;
	
	@ClientString(id = 1000200, message = "The material desires of mortals are ultimately meaningless.")
	public static NpcStringId THE_MATERIAL_DESIRES_OF_MORTALS_ARE_ULTIMATELY_MEANINGLESS;
	
	@ClientString(id = 1000201, message = "Do not forget the reason the Tower of Insolence collapsed.")
	public static NpcStringId DO_NOT_FORGET_THE_REASON_THE_TOWER_OF_INSOLENCE_COLLAPSED;
	
	@ClientString(id = 1000202, message = "You humans are all alike, full of greed and avarice!")
	public static NpcStringId YOU_HUMANS_ARE_ALL_ALIKE_FULL_OF_GREED_AND_AVARICE;
	
	@ClientString(id = 1000203, message = "All for nothing,")
	public static NpcStringId ALL_FOR_NOTHING;
	
	@ClientString(id = 1000204, message = "What are all these people doing here?")
	public static NpcStringId WHAT_ARE_ALL_THESE_PEOPLE_DOING_HERE;
	
	@ClientString(id = 1000205, message = "I must find the secret of eternal life, here among these rotten angels!")
	public static NpcStringId I_MUST_FIND_THE_SECRET_OF_ETERNAL_LIFE_HERE_AMONG_THESE_ROTTEN_ANGELS;
	
	@ClientString(id = 1000206, message = "Do you also seek the secret of immortality?")
	public static NpcStringId DO_YOU_ALSO_SEEK_THE_SECRET_OF_IMMORTALITY;
	
	@ClientString(id = 1000207, message = "I shall never reveal my secrets!")
	public static NpcStringId I_SHALL_NEVER_REVEAL_MY_SECRETS;
	
	@ClientString(id = 1000208, message = "Who dares enter this place?")
	public static NpcStringId WHO_DARES_ENTER_THIS_PLACE;
	
	@ClientString(id = 1000209, message = "This is no place for humans! You must leave immediately.")
	public static NpcStringId THIS_IS_NO_PLACE_FOR_HUMANS_YOU_MUST_LEAVE_IMMEDIATELY;
	
	@ClientString(id = 1000210, message = "You poor creatures! Too stupid to realize your own ignorance!")
	public static NpcStringId YOU_POOR_CREATURES_TOO_STUPID_TO_REALIZE_YOUR_OWN_IGNORANCE;
	
	@ClientString(id = 1000211, message = "You mustn't go there!")
	public static NpcStringId YOU_MUSTN_T_GO_THERE;
	
	@ClientString(id = 1000212, message = "Who dares disturb this marsh?")
	public static NpcStringId WHO_DARES_DISTURB_THIS_MARSH;
	
	@ClientString(id = 1000213, message = "The humans must not be allowed to destroy the marshland for their greedy purposes.")
	public static NpcStringId THE_HUMANS_MUST_NOT_BE_ALLOWED_TO_DESTROY_THE_MARSHLAND_FOR_THEIR_GREEDY_PURPOSES;
	
	@ClientString(id = 1000214, message = "You are a brave man...")
	public static NpcStringId YOU_ARE_A_BRAVE_MAN;
	
	@ClientString(id = 1000215, message = "You idiots! Some day you shall also be gone!")
	public static NpcStringId YOU_IDIOTS_SOME_DAY_YOU_SHALL_ALSO_BE_GONE;
	
	@ClientString(id = 1000216, message = "Someone has entered the forest...")
	public static NpcStringId SOMEONE_HAS_ENTERED_THE_FOREST;
	
	@ClientString(id = 1000217, message = "The forest is very quiet and peaceful.")
	public static NpcStringId THE_FOREST_IS_VERY_QUIET_AND_PEACEFUL;
	
	@ClientString(id = 1000218, message = "Stay here in this wonderful forest!")
	public static NpcStringId STAY_HERE_IN_THIS_WONDERFUL_FOREST;
	
	@ClientString(id = 1000219, message = "My... my souls...")
	public static NpcStringId MY_MY_SOULS;
	
	@ClientString(id = 1000220, message = "This forest is a dangerous place.")
	public static NpcStringId THIS_FOREST_IS_A_DANGEROUS_PLACE;
	
	@ClientString(id = 1000221, message = "Unless you leave this forest immediately you are bound to run into serious trouble.")
	public static NpcStringId UNLESS_YOU_LEAVE_THIS_FOREST_IMMEDIATELY_YOU_ARE_BOUND_TO_RUN_INTO_SERIOUS_TROUBLE;
	
	@ClientString(id = 1000222, message = "Leave now!")
	public static NpcStringId LEAVE_NOW;
	
	@ClientString(id = 1000223, message = "Why do you ignore my warning?")
	public static NpcStringId WHY_DO_YOU_IGNORE_MY_WARNING;
	
	@ClientString(id = 1000224, message = "Harits of the world... I bring you peace!")
	public static NpcStringId HARITS_OF_THE_WORLD_I_BRING_YOU_PEACE;
	
	@ClientString(id = 1000225, message = "Harits! Be courageous!")
	public static NpcStringId HARITS_BE_COURAGEOUS;
	
	@ClientString(id = 1000226, message = "I shall eat your still-beating heart!.")
	public static NpcStringId I_SHALL_EAT_YOUR_STILL_BEATING_HEART;
	
	@ClientString(id = 1000227, message = "Harits! Keep faith until the day I return... Never lose hope!")
	public static NpcStringId HARITS_KEEP_FAITH_UNTIL_THE_DAY_I_RETURN_NEVER_LOSE_HOPE;
	
	@ClientString(id = 1000228, message = "Even the giants are gone! There's nothing left to be afraid of now!")
	public static NpcStringId EVEN_THE_GIANTS_ARE_GONE_THERE_S_NOTHING_LEFT_TO_BE_AFRAID_OF_NOW;
	
	@ClientString(id = 1000229, message = "Have you heard of the giants? Their downfall was inevitable!")
	public static NpcStringId HAVE_YOU_HEARD_OF_THE_GIANTS_THEIR_DOWNFALL_WAS_INEVITABLE;
	
	@ClientString(id = 1000230, message = "What nerve! Do you dare to challenge me?")
	public static NpcStringId WHAT_NERVE_DO_YOU_DARE_TO_CHALLENGE_ME;
	
	@ClientString(id = 1000231, message = "You are as evil as the giants...")
	public static NpcStringId YOU_ARE_AS_EVIL_AS_THE_GIANTS;
	
	@ClientString(id = 1000232, message = "This dungeon is still in good condition!")
	public static NpcStringId THIS_DUNGEON_IS_STILL_IN_GOOD_CONDITION;
	
	@ClientString(id = 1000233, message = "This place is spectacular, wouldn't you say?")
	public static NpcStringId THIS_PLACE_IS_SPECTACULAR_WOULDN_T_YOU_SAY;
	
	@ClientString(id = 1000234, message = "You are very brave warriors!")
	public static NpcStringId YOU_ARE_VERY_BRAVE_WARRIORS;
	
	@ClientString(id = 1000235, message = "Are the giants truly gone for good?")
	public static NpcStringId ARE_THE_GIANTS_TRULY_GONE_FOR_GOOD;
	
	@ClientString(id = 1000236, message = "These graves are good.")
	public static NpcStringId THESE_GRAVES_ARE_GOOD;
	
	@ClientString(id = 1000237, message = "Gold and silver are meaningless to a dead man!")
	public static NpcStringId GOLD_AND_SILVER_ARE_MEANINGLESS_TO_A_DEAD_MAN;
	
	@ClientString(id = 1000238, message = "Why would those corrupt aristocrats bury such useful things?")
	public static NpcStringId WHY_WOULD_THOSE_CORRUPT_ARISTOCRATS_BURY_SUCH_USEFUL_THINGS;
	
	@ClientString(id = 1000239, message = "You filthy pig! Eat and be merry now that you have shirked your responsibilities!")
	public static NpcStringId YOU_FILTHY_PIG_EAT_AND_BE_MERRY_NOW_THAT_YOU_HAVE_SHIRKED_YOUR_RESPONSIBILITIES;
	
	@ClientString(id = 1000240, message = "Those thugs! It would be too merciful to rip them apart and chew them up one at a time!")
	public static NpcStringId THOSE_THUGS_IT_WOULD_BE_TOO_MERCIFUL_TO_RIP_THEM_APART_AND_CHEW_THEM_UP_ONE_AT_A_TIME;
	
	@ClientString(id = 1000241, message = "You accursed scoundrels!")
	public static NpcStringId YOU_ACCURSED_SCOUNDRELS;
	
	@ClientString(id = 1000242, message = "Hmm, could this be the assassin sent by those idiots from Aden?")
	public static NpcStringId HMM_COULD_THIS_BE_THE_ASSASSIN_SENT_BY_THOSE_IDIOTS_FROM_ADEN;
	
	@ClientString(id = 1000243, message = "I shall curse your name with my last breath!")
	public static NpcStringId I_SHALL_CURSE_YOUR_NAME_WITH_MY_LAST_BREATH;
	
	@ClientString(id = 1000244, message = "My beloved Lord Shilen.")
	public static NpcStringId MY_BELOVED_LORD_SHILEN;
	
	@ClientString(id = 1000245, message = "I must break the seal and release Lord Shilen as soon as possible...")
	public static NpcStringId I_MUST_BREAK_THE_SEAL_AND_RELEASE_LORD_SHILEN_AS_SOON_AS_POSSIBLE;
	
	@ClientString(id = 1000246, message = "You shall taste the vengeance of Lord Shilen!")
	public static NpcStringId YOU_SHALL_TASTE_THE_VENGEANCE_OF_LORD_SHILEN;
	
	@ClientString(id = 1000247, message = "Lord Shilen... some day... you will accomplish... this mission...")
	public static NpcStringId LORD_SHILEN_SOME_DAY_YOU_WILL_ACCOMPLISH_THIS_MISSION;
	
	@ClientString(id = 1000248, message = "Towards immortality...")
	public static NpcStringId TOWARDS_IMMORTALITY;
	
	@ClientString(id = 1000249, message = "He who desires immortality... Come unto me.")
	public static NpcStringId HE_WHO_DESIRES_IMMORTALITY_COME_UNTO_ME;
	
	@ClientString(id = 1000250, message = "You shall be sacrificed to gain my immortality!")
	public static NpcStringId YOU_SHALL_BE_SACRIFICED_TO_GAIN_MY_IMMORTALITY;
	
	@ClientString(id = 1000251, message = "Eternal life in front of my eyes... I have collapsed in such a worthless way like this...")
	public static NpcStringId ETERNAL_LIFE_IN_FRONT_OF_MY_EYES_I_HAVE_COLLAPSED_IN_SUCH_A_WORTHLESS_WAY_LIKE_THIS;
	
	@ClientString(id = 1000252, message = "Zaken, you are a cowardly cur!")
	public static NpcStringId ZAKEN_YOU_ARE_A_COWARDLY_CUR;
	
	@ClientString(id = 1000253, message = "You are immortal, aren't you, Zaken?")
	public static NpcStringId YOU_ARE_IMMORTAL_AREN_T_YOU_ZAKEN;
	
	@ClientString(id = 1000254, message = "Please return my body to me.")
	public static NpcStringId PLEASE_RETURN_MY_BODY_TO_ME;
	
	@ClientString(id = 1000255, message = "Finally... will I be able to rest?")
	public static NpcStringId FINALLY_WILL_I_BE_ABLE_TO_REST;
	
	@ClientString(id = 1000256, message = "What is all that racket?")
	public static NpcStringId WHAT_IS_ALL_THAT_RACKET;
	
	@ClientString(id = 1000257, message = "Master Gildor does not like to be disturbed.")
	public static NpcStringId MASTER_GILDOR_DOES_NOT_LIKE_TO_BE_DISTURBED;
	
	@ClientString(id = 1000258, message = "Please, just hold it down...")
	public static NpcStringId PLEASE_JUST_HOLD_IT_DOWN;
	
	@ClientString(id = 1000259, message = "If you disturb Master Gildor I won't be able to help you.")
	public static NpcStringId IF_YOU_DISTURB_MASTER_GILDOR_I_WON_T_BE_ABLE_TO_HELP_YOU;
	
	@ClientString(id = 1000260, message = "Who dares approach?")
	public static NpcStringId WHO_DARES_APPROACH;
	
	@ClientString(id = 1000261, message = "These reeds are my territory...")
	public static NpcStringId THESE_REEDS_ARE_MY_TERRITORY;
	
	@ClientString(id = 1000262, message = "You fools! Today you shall learn a lesson!")
	public static NpcStringId YOU_FOOLS_TODAY_YOU_SHALL_LEARN_A_LESSON;
	
	@ClientString(id = 1000263, message = "The past goes by... Is a new era beginning?...")
	public static NpcStringId THE_PAST_GOES_BY_IS_A_NEW_ERA_BEGINNING;
	
	@ClientString(id = 1000264, message = "This is the garden of Eva.")
	public static NpcStringId THIS_IS_THE_GARDEN_OF_EVA;
	
	@ClientString(id = 1000265, message = "The garden of Eva is a sacred place.")
	public static NpcStringId THE_GARDEN_OF_EVA_IS_A_SACRED_PLACE;
	
	@ClientString(id = 1000266, message = "Do you mean to insult Eva?")
	public static NpcStringId DO_YOU_MEAN_TO_INSULT_EVA;
	
	@ClientString(id = 1000267, message = "How rude!  Eva's love is available to all, even to an ill-mannered lout like yourself!")
	public static NpcStringId HOW_RUDE_EVA_S_LOVE_IS_AVAILABLE_TO_ALL_EVEN_TO_AN_ILL_MANNERED_LOUT_LIKE_YOURSELF;
	
	@ClientString(id = 1000268, message = "This place once belonged to Lord Shilen.")
	public static NpcStringId THIS_PLACE_ONCE_BELONGED_TO_LORD_SHILEN;
	
	@ClientString(id = 1000269, message = "Leave this palace to us, spirits of Eva.")
	public static NpcStringId LEAVE_THIS_PALACE_TO_US_SPIRITS_OF_EVA;
	
	@ClientString(id = 1000270, message = "Why are you getting in our way?")
	public static NpcStringId WHY_ARE_YOU_GETTING_IN_OUR_WAY;
	
	@ClientString(id = 1000271, message = "Shilen... our Shilen!")
	public static NpcStringId SHILEN_OUR_SHILEN;
	
	@ClientString(id = 1000272, message = "All who fear of Fafurion... Leave this place at once!")
	public static NpcStringId ALL_WHO_FEAR_OF_FAFURION_LEAVE_THIS_PLACE_AT_ONCE;
	
	@ClientString(id = 1000273, message = "You are being punished in the name of Fafurion!")
	public static NpcStringId YOU_ARE_BEING_PUNISHED_IN_THE_NAME_OF_FAFURION;
	
	@ClientString(id = 1000274, message = "Oh, master... please forgive your humble servant...")
	public static NpcStringId OH_MASTER_PLEASE_FORGIVE_YOUR_HUMBLE_SERVANT;
	
	@ClientString(id = 1000275, message = "Prepare to die, foreign invaders! I am Gustav, the eternal ruler of this fortress and I have taken up my sword to repel thee!")
	public static NpcStringId PREPARE_TO_DIE_FOREIGN_INVADERS_I_AM_GUSTAV_THE_ETERNAL_RULER_OF_THIS_FORTRESS_AND_I_HAVE_TAKEN_UP_MY_SWORD_TO_REPEL_THEE;
	
	@ClientString(id = 1000276, message = "Glory to Aden, the Kingdom of the Lion! Glory to Sir Gustav, our immortal lord!")
	public static NpcStringId GLORY_TO_ADEN_THE_KINGDOM_OF_THE_LION_GLORY_TO_SIR_GUSTAV_OUR_IMMORTAL_LORD;
	
	@ClientString(id = 1000277, message = "Soldiers of Gustav, go forth and destroy the invaders!")
	public static NpcStringId SOLDIERS_OF_GUSTAV_GO_FORTH_AND_DESTROY_THE_INVADERS;
	
	@ClientString(id = 1000278, message = "This is unbelievable! Have I really been defeated? I shall return and take your head!")
	public static NpcStringId THIS_IS_UNBELIEVABLE_HAVE_I_REALLY_BEEN_DEFEATED_I_SHALL_RETURN_AND_TAKE_YOUR_HEAD;
	
	@ClientString(id = 1000279, message = "Could it be that I have reached my end? I cannot die without honor, without the permission of Sir Gustav!")
	public static NpcStringId COULD_IT_BE_THAT_I_HAVE_REACHED_MY_END_I_CANNOT_DIE_WITHOUT_HONOR_WITHOUT_THE_PERMISSION_OF_SIR_GUSTAV;
	
	@ClientString(id = 1000280, message = "Ah, the bitter taste of defeat... I fear my torments are not over...")
	public static NpcStringId AH_THE_BITTER_TASTE_OF_DEFEAT_I_FEAR_MY_TORMENTS_ARE_NOT_OVER;
	
	@ClientString(id = 1000281, message = "I follow the will of Fafurion.")
	public static NpcStringId I_FOLLOW_THE_WILL_OF_FAFURION;
	
	@ClientString(id = 1000282, message = "Tickets for the Lucky Lottery are now on sale!")
	public static NpcStringId TICKETS_FOR_THE_LUCKY_LOTTERY_ARE_NOW_ON_SALE;
	
	@ClientString(id = 1000283, message = "The Lucky Lottery drawing is about to begin!")
	public static NpcStringId THE_LUCKY_LOTTERY_DRAWING_IS_ABOUT_TO_BEGIN;
	
	@ClientString(id = 1000284, message = "The winning numbers for Lucky Lottery $s1 are $s2. Congratulations to the winners!")
	public static NpcStringId THE_WINNING_NUMBERS_FOR_LUCKY_LOTTERY_S1_ARE_S2_CONGRATULATIONS_TO_THE_WINNERS;
	
	@ClientString(id = 1000285, message = "You're too young to play Lucky Lottery!")
	public static NpcStringId YOU_RE_TOO_YOUNG_TO_PLAY_LUCKY_LOTTERY;
	
	@ClientString(id = 1000286, message = "$s1! Watch your back!!!")
	public static NpcStringId S1_WATCH_YOUR_BACK;
	
	@ClientString(id = 1000287, message = "$s1! Come on, I'll take you on!")
	public static NpcStringId S1_COME_ON_I_LL_TAKE_YOU_ON;
	
	@ClientString(id = 1000288, message = "$s1! How dare you interrupt our fight! Hey guys, help!")
	public static NpcStringId S1_HOW_DARE_YOU_INTERRUPT_OUR_FIGHT_HEY_GUYS_HELP;
	
	@ClientString(id = 1000289, message = "I'll help you! I'm no coward!")
	public static NpcStringId I_LL_HELP_YOU_I_M_NO_COWARD;
	
	@ClientString(id = 1000290, message = "Dear ultimate power!!!")
	public static NpcStringId DEAR_ULTIMATE_POWER;
	
	@ClientString(id = 1000291, message = "Everybody! Attack $s1!")
	public static NpcStringId EVERYBODY_ATTACK_S1;
	
	@ClientString(id = 1000292, message = "I will follow your order.")
	public static NpcStringId I_WILL_FOLLOW_YOUR_ORDER;
	
	@ClientString(id = 1000293, message = "Bet you didn't expect this!")
	public static NpcStringId BET_YOU_DIDN_T_EXPECT_THIS;
	
	@ClientString(id = 1000294, message = "Come out, you children of darkness!")
	public static NpcStringId COME_OUT_YOU_CHILDREN_OF_DARKNESS;
	
	@ClientString(id = 1000295, message = "Summon party members!")
	public static NpcStringId SUMMON_PARTY_MEMBERS;
	
	@ClientString(id = 1000296, message = "Master! Did you call me?")
	public static NpcStringId MASTER_DID_YOU_CALL_ME;
	
	@ClientString(id = 1000297, message = "You idiot!")
	public static NpcStringId YOU_IDIOT;
	
	@ClientString(id = 1000298, message = "What about this?")
	public static NpcStringId WHAT_ABOUT_THIS;
	
	@ClientString(id = 1000299, message = "Very impressive $s1! This is the last!")
	public static NpcStringId VERY_IMPRESSIVE_S1_THIS_IS_THE_LAST;
	
	@ClientString(id = 1000300, message = "Dawn")
	public static NpcStringId DAWN;
	
	@ClientString(id = 1000301, message = "Dusk")
	public static NpcStringId DUSK;
	
	@ClientString(id = 1000302, message = "Nothingness")
	public static NpcStringId NOTHINGNESS;
	
	@ClientString(id = 1000303, message = "This world will soon be annihilated!")
	public static NpcStringId THIS_WORLD_WILL_SOON_BE_ANNIHILATED;
	
	@ClientString(id = 1000304, message = "A curse upon you!")
	public static NpcStringId A_CURSE_UPON_YOU;
	
	@ClientString(id = 1000305, message = "The day of judgment is near!")
	public static NpcStringId THE_DAY_OF_JUDGMENT_IS_NEAR;
	
	@ClientString(id = 1000306, message = "I bestow upon you a blessing!")
	public static NpcStringId I_BESTOW_UPON_YOU_A_BLESSING;
	
	@ClientString(id = 1000307, message = "The first rule of fighting is to start by killing the weak ones!")
	public static NpcStringId THE_FIRST_RULE_OF_FIGHTING_IS_TO_START_BY_KILLING_THE_WEAK_ONES;
	
	@ClientString(id = 1000308, message = "Adena")
	public static NpcStringId ADENA;
	
	@ClientString(id = 1000309, message = "Ancient Adena")
	public static NpcStringId ANCIENT_ADENA;
	
	@ClientString(id = 1000310, message = "Dusk")
	public static NpcStringId DUSK2;
	
	@ClientString(id = 1000311, message = "Dawn")
	public static NpcStringId DAWN2;
	
	@ClientString(id = 1000312, message = "Level 31 or lower")
	public static NpcStringId LEVEL_31_OR_LOWER;
	
	@ClientString(id = 1000313, message = "Level 42 or lower")
	public static NpcStringId LEVEL_42_OR_LOWER;
	
	@ClientString(id = 1000314, message = "Level 53 or lower")
	public static NpcStringId LEVEL_53_OR_LOWER;
	
	@ClientString(id = 1000315, message = "Level 64 or lower")
	public static NpcStringId LEVEL_64_OR_LOWER;
	
	@ClientString(id = 1000316, message = "No Level Limit")
	public static NpcStringId NO_LEVEL_LIMIT;
	
	@ClientString(id = 1000317, message = "The main event will start in 2 minutes. Please register now.")
	public static NpcStringId THE_MAIN_EVENT_WILL_START_IN_2_MINUTES_PLEASE_REGISTER_NOW;
	
	@ClientString(id = 1000318, message = "The main event is now starting.")
	public static NpcStringId THE_MAIN_EVENT_IS_NOW_STARTING;
	
	@ClientString(id = 1000319, message = "The main event will close in 5 minutes.")
	public static NpcStringId THE_MAIN_EVENT_WILL_CLOSE_IN_5_MINUTES;
	
	@ClientString(id = 1000320, message = "The main event will finish in 2 minutes. Please prepare for the next game.")
	public static NpcStringId THE_MAIN_EVENT_WILL_FINISH_IN_2_MINUTES_PLEASE_PREPARE_FOR_THE_NEXT_GAME;
	
	@ClientString(id = 1000321, message = "The amount of SSQ contribution has increased by $s1.")
	public static NpcStringId THE_AMOUNT_OF_SSQ_CONTRIBUTION_HAS_INCREASED_BY_S1;
	
	@ClientString(id = 1000322, message = "No record exists")
	public static NpcStringId NO_RECORD_EXISTS;
	
	@ClientString(id = 1000324, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=324')Gladiator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_324_GLADIATOR_BUTTON;
	
	@ClientString(id = 1000325, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=325')Warlord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_325_WARLORD_BUTTON;
	
	@ClientString(id = 1000327, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=327')Paladin(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_327_PALADIN_BUTTON;
	
	@ClientString(id = 1000328, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=328')Dark Avenger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_328_DARK_AVENGER_BUTTON;
	
	@ClientString(id = 1000330, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=330')Treasure Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_330_TREASURE_HUNTER_BUTTON;
	
	@ClientString(id = 1000331, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=331')Hawkeye(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_331_HAWKEYE_BUTTON;
	
	@ClientString(id = 1000334, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=334')Sorcerer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_334_SORCERER_BUTTON;
	
	@ClientString(id = 1000335, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=335')Necromancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_335_NECROMANCER_BUTTON;
	
	@ClientString(id = 1000336, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=336')Warlock(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_336_WARLOCK_BUTTON;
	
	@ClientString(id = 1000338, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=338')Bishop(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_338_BISHOP_BUTTON;
	
	@ClientString(id = 1000339, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=339')Prophet(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_339_PROPHET_BUTTON;
	
	@ClientString(id = 1000342, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=342')Temple Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_342_TEMPLE_KNIGHT_BUTTON;
	
	@ClientString(id = 1000343, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=343')Swordsinger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_343_SWORDSINGER_BUTTON;
	
	@ClientString(id = 1000345, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=345')Plains Walker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_345_PLAINS_WALKER_BUTTON;
	
	@ClientString(id = 1000346, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=346')Silver Ranger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_346_SILVER_RANGER_BUTTON;
	
	@ClientString(id = 1000349, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=349')Spellsinger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_349_SPELLSINGER_BUTTON;
	
	@ClientString(id = 1000350, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=350')Elemental Summoner(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_350_ELEMENTAL_SUMMONER_BUTTON;
	
	@ClientString(id = 1000352, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=352')Elven Elder(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_352_ELVEN_ELDER_BUTTON;
	
	@ClientString(id = 1000355, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=355')Shillien Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_355_SHILLIEN_KNIGHT_BUTTON;
	
	@ClientString(id = 1000356, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=356')Bladedancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_356_BLADEDANCER_BUTTON;
	
	@ClientString(id = 1000358, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=358')Abyss Walker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_358_ABYSS_WALKER_BUTTON;
	
	@ClientString(id = 1000359, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=359')Phantom Ranger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_359_PHANTOM_RANGER_BUTTON;
	
	@ClientString(id = 1000362, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=362')Spellhowler(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_362_SPELLHOWLER_BUTTON;
	
	@ClientString(id = 1000363, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=363')Phantom Summoner(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_363_PHANTOM_SUMMONER_BUTTON;
	
	@ClientString(id = 1000365, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=365')Shillien Elder(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_365_SHILLIEN_ELDER_BUTTON;
	
	@ClientString(id = 1000368, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=368')Destroyer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_368_DESTROYER_BUTTON;
	
	@ClientString(id = 1000370, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=370')Tyrant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_370_TYRANT_BUTTON;
	
	@ClientString(id = 1000373, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=373')Overlord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_373_OVERLORD_BUTTON;
	
	@ClientString(id = 1000374, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=374')Warcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_374_WARCRYER_BUTTON;
	
	@ClientString(id = 1000377, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=377')Bounty Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_377_BOUNTY_HUNTER_BUTTON;
	
	@ClientString(id = 1000379, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=379')Warsmith(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_379_WARSMITH_BUTTON;
	
	@ClientString(id = 1000380, message = "That will do! I'll move you to the outside soon.")
	public static NpcStringId THAT_WILL_DO_I_LL_MOVE_YOU_TO_THE_OUTSIDE_SOON;
	
	@ClientString(id = 1000381, message = "$s1! Watch your back!")
	public static NpcStringId S1_WATCH_YOUR_BACK2;
	
	@ClientString(id = 1000382, message = "Your rear is practically unguarded!")
	public static NpcStringId YOUR_REAR_IS_PRACTICALLY_UNGUARDED;
	
	@ClientString(id = 1000383, message = "How dare you turn your back on me!")
	public static NpcStringId HOW_DARE_YOU_TURN_YOUR_BACK_ON_ME;
	
	@ClientString(id = 1000384, message = "$s1! I'll deal with you myself!")
	public static NpcStringId S1_I_LL_DEAL_WITH_YOU_MYSELF;
	
	@ClientString(id = 1000385, message = "$s1! This is personal!")
	public static NpcStringId S1_THIS_IS_PERSONAL;
	
	@ClientString(id = 1000386, message = "$s1! Leave us alone! This is between us!")
	public static NpcStringId S1_LEAVE_US_ALONE_THIS_IS_BETWEEN_US;
	
	@ClientString(id = 1000387, message = "$s1! Killing you will be a pleasure!")
	public static NpcStringId S1_KILLING_YOU_WILL_BE_A_PLEASURE;
	
	@ClientString(id = 1000388, message = "$s1! Hey! We're having a duel here!")
	public static NpcStringId S1_HEY_WE_RE_HAVING_A_DUEL_HERE;
	
	@ClientString(id = 1000389, message = "The duel is over! Attack!")
	public static NpcStringId THE_DUEL_IS_OVER_ATTACK;
	
	@ClientString(id = 1000390, message = "Foul! Kill the coward!")
	public static NpcStringId FOUL_KILL_THE_COWARD;
	
	@ClientString(id = 1000391, message = "How dare you interrupt a sacred duel! You must be taught a lesson!")
	public static NpcStringId HOW_DARE_YOU_INTERRUPT_A_SACRED_DUEL_YOU_MUST_BE_TAUGHT_A_LESSON;
	
	@ClientString(id = 1000392, message = "Die, you coward!")
	public static NpcStringId DIE_YOU_COWARD;
	
	@ClientString(id = 1000393, message = "What are you looking at?")
	public static NpcStringId WHAT_ARE_YOU_LOOKING_AT2;
	
	@ClientString(id = 1000394, message = "Kill the coward!")
	public static NpcStringId KILL_THE_COWARD;
	
	@ClientString(id = 1000395, message = "I never thought I'd use this against a novice!")
	public static NpcStringId I_NEVER_THOUGHT_I_D_USE_THIS_AGAINST_A_NOVICE;
	
	@ClientString(id = 1000396, message = "You won't take me down easily.")
	public static NpcStringId YOU_WON_T_TAKE_ME_DOWN_EASILY;
	
	@ClientString(id = 1000397, message = "The battle has just begun!")
	public static NpcStringId THE_BATTLE_HAS_JUST_BEGUN;
	
	@ClientString(id = 1000398, message = "Kill $s1 first!")
	public static NpcStringId KILL_S1_FIRST;
	
	@ClientString(id = 1000399, message = "Attack $s1!")
	public static NpcStringId ATTACK_S1;
	
	@ClientString(id = 1000400, message = "Attack! Attack!")
	public static NpcStringId ATTACK_ATTACK;
	
	@ClientString(id = 1000401, message = "Over here!")
	public static NpcStringId OVER_HERE;
	
	@ClientString(id = 1000402, message = "Roger!")
	public static NpcStringId ROGER;
	
	@ClientString(id = 1000403, message = "Show yourselves!")
	public static NpcStringId SHOW_YOURSELVES;
	
	@ClientString(id = 1000404, message = "Forces of darkness! Follow me!")
	public static NpcStringId FORCES_OF_DARKNESS_FOLLOW_ME;
	
	@ClientString(id = 1000405, message = "Destroy the enemy, my brothers!")
	public static NpcStringId DESTROY_THE_ENEMY_MY_BROTHERS;
	
	@ClientString(id = 1000406, message = "Now the fun starts!")
	public static NpcStringId NOW_THE_FUN_STARTS;
	
	@ClientString(id = 1000407, message = "Enough fooling around. Get ready to die!")
	public static NpcStringId ENOUGH_FOOLING_AROUND_GET_READY_TO_DIE;
	
	@ClientString(id = 1000408, message = "You idiot! I've just been toying with you!")
	public static NpcStringId YOU_IDIOT_I_VE_JUST_BEEN_TOYING_WITH_YOU;
	
	@ClientString(id = 1000409, message = "Witness my true power!")
	public static NpcStringId WITNESS_MY_TRUE_POWER;
	
	@ClientString(id = 1000410, message = "Now the battle begins!")
	public static NpcStringId NOW_THE_BATTLE_BEGINS;
	
	@ClientString(id = 1000411, message = "I must admit, no one makes my blood boil quite like you do!")
	public static NpcStringId I_MUST_ADMIT_NO_ONE_MAKES_MY_BLOOD_BOIL_QUITE_LIKE_YOU_DO;
	
	@ClientString(id = 1000412, message = "You have more skill than I thought!")
	public static NpcStringId YOU_HAVE_MORE_SKILL_THAN_I_THOUGHT;
	
	@ClientString(id = 1000413, message = "I'll double my strength!")
	public static NpcStringId I_LL_DOUBLE_MY_STRENGTH;
	
	@ClientString(id = 1000414, message = "Prepare to die!")
	public static NpcStringId PREPARE_TO_DIE;
	
	@ClientString(id = 1000415, message = "All is lost! Prepare to meet the goddess of death!")
	public static NpcStringId ALL_IS_LOST_PREPARE_TO_MEET_THE_GODDESS_OF_DEATH;
	
	@ClientString(id = 1000416, message = "All is lost! The prophecy of destruction has been fulfilled!")
	public static NpcStringId ALL_IS_LOST_THE_PROPHECY_OF_DESTRUCTION_HAS_BEEN_FULFILLED;
	
	@ClientString(id = 1000417, message = "The end of time has come! The prophecy of destruction has been fulfilled!")
	public static NpcStringId THE_END_OF_TIME_HAS_COME_THE_PROPHECY_OF_DESTRUCTION_HAS_BEEN_FULFILLED;
	
	@ClientString(id = 1000418, message = "$s1! You bring an ill wind!")
	public static NpcStringId S1_YOU_BRING_AN_ILL_WIND;
	
	@ClientString(id = 1000419, message = "$s1! You might as well give up!")
	public static NpcStringId S1_YOU_MIGHT_AS_WELL_GIVE_UP;
	
	@ClientString(id = 1000420, message = "You don't have any hope! Your end has come!")
	public static NpcStringId YOU_DON_T_HAVE_ANY_HOPE_YOUR_END_HAS_COME;
	
	@ClientString(id = 1000421, message = "The prophecy of darkness has been fulfilled!")
	public static NpcStringId THE_PROPHECY_OF_DARKNESS_HAS_BEEN_FULFILLED;
	
	@ClientString(id = 1000422, message = "As foretold in the prophecy of darkness, the era of chaos has begun!")
	public static NpcStringId AS_FORETOLD_IN_THE_PROPHECY_OF_DARKNESS_THE_ERA_OF_CHAOS_HAS_BEGUN;
	
	@ClientString(id = 1000423, message = "The prophecy of darkness has come to pass!")
	public static NpcStringId THE_PROPHECY_OF_DARKNESS_HAS_COME_TO_PASS;
	
	@ClientString(id = 1000424, message = "$s1! I give you the Blessing of Revelations!")
	public static NpcStringId S1_I_GIVE_YOU_THE_BLESSING_OF_REVELATIONS;
	
	@ClientString(id = 1000425, message = "$s1! I bestow upon you the authority of the abyss!")
	public static NpcStringId S1_I_BESTOW_UPON_YOU_THE_AUTHORITY_OF_THE_ABYSS;
	
	@ClientString(id = 1000426, message = "Herald of the new era, open your eyes!")
	public static NpcStringId HERALD_OF_THE_NEW_ERA_OPEN_YOUR_EYES;
	
	@ClientString(id = 1000427, message = "Remember, kill the weaklings first!")
	public static NpcStringId REMEMBER_KILL_THE_WEAKLINGS_FIRST;
	
	@ClientString(id = 1000428, message = "Prepare to die, maggot!")
	public static NpcStringId PREPARE_TO_DIE_MAGGOT;
	
	@ClientString(id = 1000429, message = "That will do. Prepare to be released!")
	public static NpcStringId THAT_WILL_DO_PREPARE_TO_BE_RELEASED;
	
	@ClientString(id = 1000430, message = "Draw")
	public static NpcStringId DRAW;
	
	@ClientString(id = 1000431, message = "I bring you wondrous gifts!")
	public static NpcStringId I_BRING_YOU_WONDROUS_GIFTS;
	
	@ClientString(id = 1000432, message = "I have some excellent weapons to show you!")
	public static NpcStringId I_HAVE_SOME_EXCELLENT_WEAPONS_TO_SHOW_YOU;
	
	@ClientString(id = 1000433, message = "I've been so busy lately, in addition to planning my trip!")
	public static NpcStringId I_VE_BEEN_SO_BUSY_LATELY_IN_ADDITION_TO_PLANNING_MY_TRIP;
	
	@ClientString(id = 1000434, message = "Your treatment of weaklings is unforgivable!")
	public static NpcStringId YOUR_TREATMENT_OF_WEAKLINGS_IS_UNFORGIVABLE;
	
	@ClientString(id = 1000435, message = "I'm here to help you! Hi yah!")
	public static NpcStringId I_M_HERE_TO_HELP_YOU_HI_YAH;
	
	@ClientString(id = 1000436, message = "Justice will be served!")
	public static NpcStringId JUSTICE_WILL_BE_SERVED;
	
	@ClientString(id = 1000437, message = "On to immortal glory!")
	public static NpcStringId ON_TO_IMMORTAL_GLORY;
	
	@ClientString(id = 1000438, message = "Justice always avenges the powerless!")
	public static NpcStringId JUSTICE_ALWAYS_AVENGES_THE_POWERLESS;
	
	@ClientString(id = 1000439, message = "Are you hurt? Hang in there, we've almost got them!")
	public static NpcStringId ARE_YOU_HURT_HANG_IN_THERE_WE_VE_ALMOST_GOT_THEM;
	
	@ClientString(id = 1000440, message = "Why should I tell you my name, you creep!?")
	public static NpcStringId WHY_SHOULD_I_TELL_YOU_MY_NAME_YOU_CREEP;
	
	@ClientString(id = 1000441, message = "0 minute")
	public static NpcStringId MINUTE;
	
	@ClientString(id = 1000442, message = "$s1 Minute(s)")
	public static NpcStringId S1_MINUTE_S;
	
	@ClientString(id = 1000443, message = "The defenders of $s1 castle will be teleported to the inner castle.")
	public static NpcStringId THE_DEFENDERS_OF_S1_CASTLE_WILL_BE_TELEPORTED_TO_THE_INNER_CASTLE;
	
	@ClientString(id = 1000444, message = "Sunday")
	public static NpcStringId SUNDAY;
	
	@ClientString(id = 1000445, message = "Monday")
	public static NpcStringId MONDAY;
	
	@ClientString(id = 1000446, message = "Tuesday")
	public static NpcStringId TUESDAY;
	
	@ClientString(id = 1000447, message = "Wednesday")
	public static NpcStringId WEDNESDAY;
	
	@ClientString(id = 1000448, message = "Thursday")
	public static NpcStringId THURSDAY;
	
	@ClientString(id = 1000449, message = "Friday")
	public static NpcStringId FRIDAY;
	
	@ClientString(id = 1000450, message = "Saturday")
	public static NpcStringId SATURDAY;
	
	@ClientString(id = 1000451, message = "Hour")
	public static NpcStringId HOUR;
	
	@ClientString(id = 1000452, message = "It's a good day to die! Welcome to hell, maggot!")
	public static NpcStringId IT_S_A_GOOD_DAY_TO_DIE_WELCOME_TO_HELL_MAGGOT;
	
	@ClientString(id = 1000453, message = "The Festival of Darkness will end in two minutes.")
	public static NpcStringId THE_FESTIVAL_OF_DARKNESS_WILL_END_IN_TWO_MINUTES;
	
	@ClientString(id = 1000454, message = "Noblesse Gate Pass")
	public static NpcStringId NOBLESSE_GATE_PASS;
	
	@ClientString(id = 1000455, message = "Current")
	public static NpcStringId CURRENT;
	
	@ClientString(id = 1000456, message = "minute(s) have passed.")
	public static NpcStringId MINUTE_S_HAVE_PASSED;
	
	@ClientString(id = 1000457, message = "Game over. The teleport will appear momentarily.")
	public static NpcStringId GAME_OVER_THE_TELEPORT_WILL_APPEAR_MOMENTARILY;
	
	@ClientString(id = 1000458, message = "You, who are like the slugs crawling on the ground. The generosity and greatness of me, Daimon the White-eyed is endless! Ha Ha Ha!")
	public static NpcStringId YOU_WHO_ARE_LIKE_THE_SLUGS_CRAWLING_ON_THE_GROUND_THE_GENEROSITY_AND_GREATNESS_OF_ME_DAIMON_THE_WHITE_EYED_IS_ENDLESS_HA_HA_HA;
	
	@ClientString(id = 1000459, message = "If you want to be the opponent of me, Daimon the White-eyed, you should at least have the basic skills~!")
	public static NpcStringId IF_YOU_WANT_TO_BE_THE_OPPONENT_OF_ME_DAIMON_THE_WHITE_EYED_YOU_SHOULD_AT_LEAST_HAVE_THE_BASIC_SKILLS;
	
	@ClientString(id = 1000460, message = "You stupid creatures that are bound to the earth. You are suffering too much while dragging your fat, heavy bodies. I, Daimon, will lighten your burden.")
	public static NpcStringId YOU_STUPID_CREATURES_THAT_ARE_BOUND_TO_THE_EARTH_YOU_ARE_SUFFERING_TOO_MUCH_WHILE_DRAGGING_YOUR_FAT_HEAVY_BODIES_I_DAIMON_WILL_LIGHTEN_YOUR_BURDEN;
	
	@ClientString(id = 1000461, message = "A weak and stupid tribe like yours doesn't deserve to be my enemy! Bwa ha ha ha!")
	public static NpcStringId A_WEAK_AND_STUPID_TRIBE_LIKE_YOURS_DOESN_T_DESERVE_TO_BE_MY_ENEMY_BWA_HA_HA_HA;
	
	@ClientString(id = 1000462, message = "You dare to invade the territory of Daimon, the White-eyed!  Now, you will pay the price for your action!")
	public static NpcStringId YOU_DARE_TO_INVADE_THE_TERRITORY_OF_DAIMON_THE_WHITE_EYED_NOW_YOU_WILL_PAY_THE_PRICE_FOR_YOUR_ACTION;
	
	@ClientString(id = 1000463, message = "This is the grace of Daimon the White-eyed, the great Monster Eye Lord! Ha Ha Ha!")
	public static NpcStringId THIS_IS_THE_GRACE_OF_DAIMON_THE_WHITE_EYED_THE_GREAT_MONSTER_EYE_LORD_HA_HA_HA;
	
	@ClientString(id = 1000464, message = "$s1!  You have become a Hero of Sigel Knights. Congratulations!")
	public static NpcStringId S1_YOU_HAVE_BECOME_A_HERO_OF_SIGEL_KNIGHTS_CONGRATULATIONS;
	
	@ClientString(id = 1000465, message = "$s1!  You have become a Hero of Tyrr Warriors. Congratulations!")
	public static NpcStringId S1_YOU_HAVE_BECOME_A_HERO_OF_TYRR_WARRIORS_CONGRATULATIONS;
	
	@ClientString(id = 1000466, message = "$s1!  You have become a Hero of Othell Rogue. Congratulations!")
	public static NpcStringId S1_YOU_HAVE_BECOME_A_HERO_OF_OTHELL_ROGUE_CONGRATULATIONS;
	
	@ClientString(id = 1000467, message = "$s1!  You have become a Hero of Yul Archer. Congratulations!")
	public static NpcStringId S1_YOU_HAVE_BECOME_A_HERO_OF_YUL_ARCHER_CONGRATULATIONS;
	
	@ClientString(id = 1000468, message = "$s1!  You have become a Hero of Feoh Wizard. Congratulations!")
	public static NpcStringId S1_YOU_HAVE_BECOME_A_HERO_OF_FEOH_WIZARD_CONGRATULATIONS;
	
	@ClientString(id = 1000469, message = "$s1!  You have become a Hero of Iss Enchanter. Congratulations!")
	public static NpcStringId S1_YOU_HAVE_BECOME_A_HERO_OF_ISS_ENCHANTER_CONGRATULATIONS;
	
	@ClientString(id = 1000470, message = "$s1!  You have become a Hero of Wynn Summoner. Congratulations!")
	public static NpcStringId S1_YOU_HAVE_BECOME_A_HERO_OF_WYNN_SUMMONER_CONGRATULATIONS;
	
	@ClientString(id = 1000471, message = "$s1! You have become a Hero of Aeore Healer. Congratulations!")
	public static NpcStringId S1_YOU_HAVE_BECOME_A_HERO_OF_AEORE_HEALER_CONGRATULATIONS;
	
	@ClientString(id = 1000472, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED;
	
	@ClientString(id = 1000473, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED2;
	
	@ClientString(id = 1000474, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED3;
	
	@ClientString(id = 1000475, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED4;
	
	@ClientString(id = 1000476, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED5;
	
	@ClientString(id = 1000477, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED6;
	
	@ClientString(id = 1000478, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED7;
	
	@ClientString(id = 1000479, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED8;
	
	@ClientString(id = 1000480, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED9;
	
	@ClientString(id = 1000481, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED10;
	
	@ClientString(id = 1000482, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED11;
	
	@ClientString(id = 1000483, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED12;
	
	@ClientString(id = 1000484, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED13;
	
	@ClientString(id = 1000485, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED14;
	
	@ClientString(id = 1000486, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED15;
	
	@ClientString(id = 1000487, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED16;
	
	@ClientString(id = 1000488, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED17;
	
	@ClientString(id = 1000489, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED18;
	
	@ClientString(id = 1000490, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED19;
	
	@ClientString(id = 1000491, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED20;
	
	@ClientString(id = 1000492, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED21;
	
	@ClientString(id = 1000493, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED22;
	
	@ClientString(id = 1000494, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED23;
	
	@ClientString(id = 1000495, message = "**unregistered**")
	public static NpcStringId UNREGISTERED24;
	
	@ClientString(id = 1000496, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED25;
	
	@ClientString(id = 1000497, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED26;
	
	@ClientString(id = 1000498, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED27;
	
	@ClientString(id = 1000499, message = "**Unregistered**")
	public static NpcStringId UNREGISTERED28;
	
	@ClientString(id = 1000500, message = "You may now enter the Sepulcher.")
	public static NpcStringId YOU_MAY_NOW_ENTER_THE_SEPULCHER;
	
	@ClientString(id = 1000501, message = "If you place your hand on the stone statue in front of each sepulcher, you will be able to enter.")
	public static NpcStringId IF_YOU_PLACE_YOUR_HAND_ON_THE_STONE_STATUE_IN_FRONT_OF_EACH_SEPULCHER_YOU_WILL_BE_ABLE_TO_ENTER;
	
	@ClientString(id = 1000502, message = "The monsters have spawned!")
	public static NpcStringId THE_MONSTERS_HAVE_SPAWNED;
	
	@ClientString(id = 1000503, message = "Thank you for saving me.")
	public static NpcStringId THANK_YOU_FOR_SAVING_ME;
	
	@ClientString(id = 1000504, message = "Fewer than $s1")
	public static NpcStringId FEWER_THAN_S1;
	
	@ClientString(id = 1000505, message = "More than $s1")
	public static NpcStringId MORE_THAN_S1;
	
	@ClientString(id = 1000506, message = "Point")
	public static NpcStringId POINT;
	
	@ClientString(id = 1000507, message = "Competition")
	public static NpcStringId COMPETITION;
	
	@ClientString(id = 1000508, message = "Seal Validation")
	public static NpcStringId SEAL_VALIDATION;
	
	@ClientString(id = 1000509, message = "Preparation")
	public static NpcStringId PREPARATION;
	
	@ClientString(id = 1000510, message = "Dusk")
	public static NpcStringId DUSK3;
	
	@ClientString(id = 1000511, message = "Dawn")
	public static NpcStringId DAWN3;
	
	@ClientString(id = 1000512, message = "No Owner")
	public static NpcStringId NO_OWNER;
	
	@ClientString(id = 1000513, message = "This place is dangerous, $s1. Please turn back.")
	public static NpcStringId THIS_PLACE_IS_DANGEROUS_S1_PLEASE_TURN_BACK;
	
	@ClientString(id = 1000514, message = "Who disturbs my sacred sleep?")
	public static NpcStringId WHO_DISTURBS_MY_SACRED_SLEEP;
	
	@ClientString(id = 1000515, message = "Begone, thief! Let our bones rest in peace.")
	public static NpcStringId BEGONE_THIEF_LET_OUR_BONES_REST_IN_PEACE;
	
	@ClientString(id = 1000516, message = "Leave us be, Hestui scum!")
	public static NpcStringId LEAVE_US_BE_HESTUI_SCUM;
	
	@ClientString(id = 1000517, message = "Thieving Kakai, may bloodbugs gnaw you in your sleep!")
	public static NpcStringId THIEVING_KAKAI_MAY_BLOODBUGS_GNAW_YOU_IN_YOUR_SLEEP;
	
	@ClientString(id = 1000518, message = "Newbie Travel Token")
	public static NpcStringId NEWBIE_TRAVEL_TOKEN;
	
	@ClientString(id = 1000519, message = "Arrogant fool! You dare to challenge me, the Ruler of Flames? Here is your reward!")
	public static NpcStringId ARROGANT_FOOL_YOU_DARE_TO_CHALLENGE_ME_THE_RULER_OF_FLAMES_HERE_IS_YOUR_REWARD;
	
	@ClientString(id = 1000520, message = "$s1!!!! You cannot hope to defeat me with your meager strength.")
	public static NpcStringId S1_YOU_CANNOT_HOPE_TO_DEFEAT_ME_WITH_YOUR_MEAGER_STRENGTH;
	
	@ClientString(id = 1000521, message = "Not even the gods themselves could touch me. But you, $s1, you dare challenge me?! Ignorant mortal!")
	public static NpcStringId NOT_EVEN_THE_GODS_THEMSELVES_COULD_TOUCH_ME_BUT_YOU_S1_YOU_DARE_CHALLENGE_ME_IGNORANT_MORTAL;
	
	@ClientString(id = 1000522, message = "Requiem of Hatred")
	public static NpcStringId REQUIEM_OF_HATRED;
	
	@ClientString(id = 1000523, message = "Fugue of Jubilation")
	public static NpcStringId FUGUE_OF_JUBILATION;
	
	@ClientString(id = 1000524, message = "Frenetic Toccata")
	public static NpcStringId FRENETIC_TOCCATA;
	
	@ClientString(id = 1000525, message = "Hypnotic Mazurka")
	public static NpcStringId HYPNOTIC_MAZURKA;
	
	@ClientString(id = 1000526, message = "Mournful Chorale Prelude")
	public static NpcStringId MOURNFUL_CHORALE_PRELUDE;
	
	@ClientString(id = 1000527, message = "Rondo of Solitude")
	public static NpcStringId RONDO_OF_SOLITUDE;
	
	@ClientString(id = 1000528, message = "Olympiad Token")
	public static NpcStringId OLYMPIAD_TOKEN;
	
	@ClientString(id = 1001000, message = "The Kingdom of Aden")
	public static NpcStringId THE_KINGDOM_OF_ADEN;
	
	@ClientString(id = 1001001, message = "Gludio")
	public static NpcStringId GLUDIO2;
	
	@ClientString(id = 1001002, message = "Dion")
	public static NpcStringId DION2;
	
	@ClientString(id = 1001003, message = "Giran")
	public static NpcStringId GIRAN2;
	
	@ClientString(id = 1001004, message = "Oren")
	public static NpcStringId OREN2;
	
	@ClientString(id = 1001005, message = "Aden")
	public static NpcStringId ADEN2;
	
	@ClientString(id = 1001006, message = "Innadril")
	public static NpcStringId INNADRIL2;
	
	@ClientString(id = 1001007, message = "Goddard")
	public static NpcStringId GODDARD2;
	
	@ClientString(id = 1001008, message = "Rune")
	public static NpcStringId RUNE2;
	
	@ClientString(id = 1001009, message = "Schuttgart")
	public static NpcStringId SCHUTTGART2;
	
	@ClientString(id = 1001100, message = "The Kingdom of Elmore")
	public static NpcStringId THE_KINGDOM_OF_ELMORE;
	
	@ClientString(id = 1010001, message = "Talking Island Village")
	public static NpcStringId TALKING_ISLAND_VILLAGE;
	
	@ClientString(id = 1010002, message = "Elven Village")
	public static NpcStringId ELVEN_VILLAGE;
	
	@ClientString(id = 1010003, message = "The Dark Elf Village")
	public static NpcStringId THE_DARK_ELF_VILLAGE;
	
	@ClientString(id = 1010004, message = "The Village of Gludin")
	public static NpcStringId THE_VILLAGE_OF_GLUDIN;
	
	@ClientString(id = 1010005, message = "The Town of Gludio")
	public static NpcStringId THE_TOWN_OF_GLUDIO;
	
	@ClientString(id = 1010006, message = "The Town of Dion")
	public static NpcStringId THE_TOWN_OF_DION;
	
	@ClientString(id = 1010007, message = "The Town of Giran")
	public static NpcStringId THE_TOWN_OF_GIRAN;
	
	@ClientString(id = 1010008, message = "Orc Village")
	public static NpcStringId ORC_VILLAGE;
	
	@ClientString(id = 1010009, message = "Dwarven Village")
	public static NpcStringId DWARVEN_VILLAGE;
	
	@ClientString(id = 1010010, message = "The Southern Part of the Dark Forest")
	public static NpcStringId THE_SOUTHERN_PART_OF_THE_DARK_FOREST;
	
	@ClientString(id = 1010011, message = "The Northeast Coast")
	public static NpcStringId THE_NORTHEAST_COAST;
	
	@ClientString(id = 1010012, message = "The Southern Entrance to the Wastelandss")
	public static NpcStringId THE_SOUTHERN_ENTRANCE_TO_THE_WASTELANDSS;
	
	@ClientString(id = 1010013, message = "Town of Oren")
	public static NpcStringId TOWN_OF_OREN;
	
	@ClientString(id = 1010014, message = "Ivory Tower")
	public static NpcStringId IVORY_TOWER;
	
	@ClientString(id = 1010015, message = "1st Floor Lobby")
	public static NpcStringId ST_FLOOR_LOBBY;
	
	@ClientString(id = 1010016, message = "Underground Shopping Area")
	public static NpcStringId UNDERGROUND_SHOPPING_AREA;
	
	@ClientString(id = 1010017, message = "2nd Floor Human Mystic Guild")
	public static NpcStringId ND_FLOOR_HUMAN_MYSTIC_GUILD;
	
	@ClientString(id = 1010018, message = "3rd Floor Elven Mystic Guild")
	public static NpcStringId RD_FLOOR_ELVEN_MYSTIC_GUILD;
	
	@ClientString(id = 1010019, message = "4th Floor Dark Mystic Guild")
	public static NpcStringId TH_FLOOR_DARK_MYSTIC_GUILD;
	
	@ClientString(id = 1010020, message = "Hunter's Village")
	public static NpcStringId HUNTER_S_VILLAGE;
	
	@ClientString(id = 1010021, message = "Giran Harbor")
	public static NpcStringId GIRAN_HARBOR;
	
	@ClientString(id = 1010022, message = "Hardin's Private Academy")
	public static NpcStringId HARDIN_S_PRIVATE_ACADEMY;
	
	@ClientString(id = 1010023, message = "Town of Aden")
	public static NpcStringId TOWN_OF_ADEN;
	
	@ClientString(id = 1010024, message = "Village Square")
	public static NpcStringId VILLAGE_SQUARE;
	
	@ClientString(id = 1010025, message = "North Gate Entrance")
	public static NpcStringId NORTH_GATE_ENTRANCE;
	
	@ClientString(id = 1010026, message = "East Gate Entrance")
	public static NpcStringId EAST_GATE_ENTRANCE;
	
	@ClientString(id = 1010027, message = "West Gate Entrance")
	public static NpcStringId WEST_GATE_ENTRANCE;
	
	@ClientString(id = 1010028, message = "South Gate Entrance")
	public static NpcStringId SOUTH_GATE_ENTRANCE;
	
	@ClientString(id = 1010029, message = "Entrance to Turek Orc Camp")
	public static NpcStringId ENTRANCE_TO_TUREK_ORC_CAMP;
	
	@ClientString(id = 1010030, message = "Entrance to Forgotten Temple")
	public static NpcStringId ENTRANCE_TO_FORGOTTEN_TEMPLE;
	
	@ClientString(id = 1010031, message = "Entrance to the Wastelands")
	public static NpcStringId ENTRANCE_TO_THE_WASTELANDS;
	
	@ClientString(id = 1010032, message = "Entrance to Abandoned Camp")
	public static NpcStringId ENTRANCE_TO_ABANDONED_CAMP;
	
	@ClientString(id = 1010033, message = "Entrance to Cruma Marshlands")
	public static NpcStringId ENTRANCE_TO_CRUMA_MARSHLANDS;
	
	@ClientString(id = 1010034, message = "Entrance to Execution Grounds")
	public static NpcStringId ENTRANCE_TO_EXECUTION_GROUNDS;
	
	@ClientString(id = 1010035, message = "Entrance to the Fortress of Resistance")
	public static NpcStringId ENTRANCE_TO_THE_FORTRESS_OF_RESISTANCE;
	
	@ClientString(id = 1010036, message = "Entrance to Floran Village")
	public static NpcStringId ENTRANCE_TO_FLORAN_VILLAGE;
	
	@ClientString(id = 1010037, message = "Neutral Zone")
	public static NpcStringId NEUTRAL_ZONE;
	
	@ClientString(id = 1010038, message = "Western Road of Giran")
	public static NpcStringId WESTERN_ROAD_OF_GIRAN;
	
	@ClientString(id = 1010039, message = "Eastern Road of Gludin Village")
	public static NpcStringId EASTERN_ROAD_OF_GLUDIN_VILLAGE;
	
	@ClientString(id = 1010040, message = "Entrance to the Fortress of Resistance")
	public static NpcStringId ENTRANCE_TO_THE_FORTRESS_OF_RESISTANCE2;
	
	@ClientString(id = 1010041, message = "Entrance to Cruma Tower")
	public static NpcStringId ENTRANCE_TO_CRUMA_TOWER;
	
	@ClientString(id = 1010042, message = "Death Pass (Lv. 35)")
	public static NpcStringId DEATH_PASS_LV_35;
	
	@ClientString(id = 1010043, message = "Northern part of the Marshlands")
	public static NpcStringId NORTHERN_PART_OF_THE_MARSHLANDS;
	
	@ClientString(id = 1010044, message = "Northeast of the Neutral Zone")
	public static NpcStringId NORTHEAST_OF_THE_NEUTRAL_ZONE;
	
	@ClientString(id = 1010045, message = "Immortal Plateau, Central Region")
	public static NpcStringId IMMORTAL_PLATEAU_CENTRAL_REGION;
	
	@ClientString(id = 1010046, message = "Immortal Plateau, Southern Region (Lv. 18)")
	public static NpcStringId IMMORTAL_PLATEAU_SOUTHERN_REGION_LV_18;
	
	@ClientString(id = 1010047, message = "Immortal Plateau, Southeast Region")
	public static NpcStringId IMMORTAL_PLATEAU_SOUTHEAST_REGION;
	
	@ClientString(id = 1010048, message = "Frozen Waterfalls (Lv. 15)")
	public static NpcStringId FROZEN_WATERFALLS_LV_15;
	
	@ClientString(id = 1010049, message = "Heine")
	public static NpcStringId HEINE;
	
	@ClientString(id = 1010050, message = "Tower of Insolence - 1st Floor")
	public static NpcStringId TOWER_OF_INSOLENCE_1ST_FLOOR;
	
	@ClientString(id = 1010051, message = "Tower of Insolence - 5th Floor")
	public static NpcStringId TOWER_OF_INSOLENCE_5TH_FLOOR;
	
	@ClientString(id = 1010052, message = "Tower of Insolence - 10th Floor")
	public static NpcStringId TOWER_OF_INSOLENCE_10TH_FLOOR;
	
	@ClientString(id = 1010053, message = "Coliseum")
	public static NpcStringId COLISEUM2;
	
	@ClientString(id = 1010054, message = "Monster Derby")
	public static NpcStringId MONSTER_DERBY;
	
	@ClientString(id = 1010055, message = "Near the frontier post")
	public static NpcStringId NEAR_THE_FRONTIER_POST;
	
	@ClientString(id = 1010056, message = "Entrance to the Sea of Spores")
	public static NpcStringId ENTRANCE_TO_THE_SEA_OF_SPORES;
	
	@ClientString(id = 1010057, message = "Ancient Battleground (Lv. 64)")
	public static NpcStringId ANCIENT_BATTLEGROUND_LV_64;
	
	@ClientString(id = 1010058, message = "Entrance to Enchanted Valley")
	public static NpcStringId ENTRANCE_TO_ENCHANTED_VALLEY;
	
	@ClientString(id = 1010059, message = "Entrance to the Tower of Insolence")
	public static NpcStringId ENTRANCE_TO_THE_TOWER_OF_INSOLENCE;
	
	@ClientString(id = 1010060, message = "Blazing Swamp (Lv. 97)")
	public static NpcStringId BLAZING_SWAMP_LV_97;
	
	@ClientString(id = 1010061, message = "Entrance to the Cemetary (Lv. 97)")
	public static NpcStringId ENTRANCE_TO_THE_CEMETARY_LV_97;
	
	@ClientString(id = 1010062, message = "Entrance to the Giant's Cave")
	public static NpcStringId ENTRANCE_TO_THE_GIANT_S_CAVE;
	
	@ClientString(id = 1010063, message = "Entrance to the Forest of Mirrors")
	public static NpcStringId ENTRANCE_TO_THE_FOREST_OF_MIRRORS;
	
	@ClientString(id = 1010064, message = "Seal of Shilen")
	public static NpcStringId SEAL_OF_SHILEN;
	
	@ClientString(id = 1010065, message = "Entrance to the Tower of Insolence")
	public static NpcStringId ENTRANCE_TO_THE_TOWER_OF_INSOLENCE2;
	
	@ClientString(id = 1010066, message = "Entrance to the Tanor Silenos Barracks")
	public static NpcStringId ENTRANCE_TO_THE_TANOR_SILENOS_BARRACKS;
	
	@ClientString(id = 1010067, message = "Dragon Valley (Lv. 76)")
	public static NpcStringId DRAGON_VALLEY_LV_76;
	
	@ClientString(id = 1010068, message = "Oracle of Dawn")
	public static NpcStringId ORACLE_OF_DAWN;
	
	@ClientString(id = 1010069, message = "Oracle of Dusk")
	public static NpcStringId ORACLE_OF_DUSK;
	
	@ClientString(id = 1010070, message = "Necropolis of Sacrifice (Lv. 21)")
	public static NpcStringId NECROPOLIS_OF_SACRIFICE_LV_21;
	
	@ClientString(id = 1010071, message = "The Pilgrim's Necropolis (Lv. 32)")
	public static NpcStringId THE_PILGRIM_S_NECROPOLIS_LV_32;
	
	@ClientString(id = 1010072, message = "Necropolis of Worship (Lv. 42)")
	public static NpcStringId NECROPOLIS_OF_WORSHIP_LV_42;
	
	@ClientString(id = 1010073, message = "The Patriot's Necropolis (Lv. 52)")
	public static NpcStringId THE_PATRIOT_S_NECROPOLIS_LV_52;
	
	@ClientString(id = 1010074, message = "Necropolis of Devotion (Lv. 60)")
	public static NpcStringId NECROPOLIS_OF_DEVOTION_LV_60;
	
	@ClientString(id = 1010075, message = "Necropolis of Martyrdom (Lv. 65)")
	public static NpcStringId NECROPOLIS_OF_MARTYRDOM_LV_65;
	
	@ClientString(id = 1010076, message = "The Disciple's Necropolis (Lv. 90)")
	public static NpcStringId THE_DISCIPLE_S_NECROPOLIS_LV_90;
	
	@ClientString(id = 1010077, message = "The Saint's Necropolis (Lv. 70)")
	public static NpcStringId THE_SAINT_S_NECROPOLIS_LV_70;
	
	@ClientString(id = 1010078, message = "The Catacomb of the Heretic (Lv. 30)")
	public static NpcStringId THE_CATACOMB_OF_THE_HERETIC_LV_30;
	
	@ClientString(id = 1010079, message = "Catacomb of the Branded")
	public static NpcStringId CATACOMB_OF_THE_BRANDED;
	
	@ClientString(id = 1010080, message = "Catacomb of the Apostate (Lv. 51)")
	public static NpcStringId CATACOMB_OF_THE_APOSTATE_LV_51;
	
	@ClientString(id = 1010081, message = "Catacomb of the Witch (Lv. 60)")
	public static NpcStringId CATACOMB_OF_THE_WITCH_LV_60;
	
	@ClientString(id = 1010082, message = "Catacomb of Dark Omens (Lv. 85)")
	public static NpcStringId CATACOMB_OF_DARK_OMENS_LV_85;
	
	@ClientString(id = 1010083, message = "Catacomb of the Forbidden Path (Lv. 72)")
	public static NpcStringId CATACOMB_OF_THE_FORBIDDEN_PATH_LV_72;
	
	@ClientString(id = 1010084, message = "Entrance to the Ruins of Despair")
	public static NpcStringId ENTRANCE_TO_THE_RUINS_OF_DESPAIR;
	
	@ClientString(id = 1010085, message = "Entrance to the Ruins of Despair")
	public static NpcStringId ENTRANCE_TO_THE_RUINS_OF_DESPAIR2;
	
	@ClientString(id = 1010086, message = "Entrance to the Ant Nest")
	public static NpcStringId ENTRANCE_TO_THE_ANT_NEST;
	
	@ClientString(id = 1010087, message = "Southern Dion")
	public static NpcStringId SOUTHERN_DION;
	
	@ClientString(id = 1010088, message = "Entrance to Dragon Valley")
	public static NpcStringId ENTRANCE_TO_DRAGON_VALLEY;
	
	@ClientString(id = 1010089, message = "Field of Silence (Lv. 81)")
	public static NpcStringId FIELD_OF_SILENCE_LV_81;
	
	@ClientString(id = 1010090, message = "Field of Whispers (Lv. 81)")
	public static NpcStringId FIELD_OF_WHISPERS_LV_81;
	
	@ClientString(id = 1010091, message = "Entrance to Alligator Island")
	public static NpcStringId ENTRANCE_TO_ALLIGATOR_ISLAND;
	
	@ClientString(id = 1010092, message = "Southern Plain of Oren")
	public static NpcStringId SOUTHERN_PLAIN_OF_OREN;
	
	@ClientString(id = 1010093, message = "Entrance to the Bandit Stronghold")
	public static NpcStringId ENTRANCE_TO_THE_BANDIT_STRONGHOLD;
	
	@ClientString(id = 1010094, message = "Windy Hill (Lv. 26)")
	public static NpcStringId WINDY_HILL_LV_26;
	
	@ClientString(id = 1010095, message = "Orc Barracks (Lv. 35)")
	public static NpcStringId ORC_BARRACKS_LV_35;
	
	@ClientString(id = 1010096, message = "Fellmere Harvesting Grounds (Lv. 17)")
	public static NpcStringId FELLMERE_HARVESTING_GROUNDS_LV_17;
	
	@ClientString(id = 1010097, message = "Ruins of Agony (Lv. 25)")
	public static NpcStringId RUINS_OF_AGONY_LV_25;
	
	@ClientString(id = 1010098, message = "Abandoned Camp (Lv. 21)")
	public static NpcStringId ABANDONED_CAMP_LV_21;
	
	@ClientString(id = 1010099, message = "Red Rock Ridge")
	public static NpcStringId RED_ROCK_RIDGE;
	
	@ClientString(id = 1010100, message = "Langk Lizardmen Dwellings (Lv. 16)")
	public static NpcStringId LANGK_LIZARDMEN_DWELLINGS_LV_16;
	
	@ClientString(id = 1010101, message = "Ruins of Despair (Lv. 30)")
	public static NpcStringId RUINS_OF_DESPAIR_LV_30;
	
	@ClientString(id = 1010102, message = "Windawood Manor (Lv. 22)")
	public static NpcStringId WINDAWOOD_MANOR_LV_22;
	
	@ClientString(id = 1010103, message = "Northern Pathway to the Wastelands")
	public static NpcStringId NORTHERN_PATHWAY_TO_THE_WASTELANDS;
	
	@ClientString(id = 1010104, message = "Western Pathway to the Wastelands")
	public static NpcStringId WESTERN_PATHWAY_TO_THE_WASTELANDS;
	
	@ClientString(id = 1010105, message = "Southern Pathway to the Wastelands")
	public static NpcStringId SOUTHERN_PATHWAY_TO_THE_WASTELANDS;
	
	@ClientString(id = 1010106, message = "Forgotten Temple (Lv. 27)")
	public static NpcStringId FORGOTTEN_TEMPLE_LV_27;
	
	@ClientString(id = 1010107, message = "South Entrance of Ant Nest")
	public static NpcStringId SOUTH_ENTRANCE_OF_ANT_NEST;
	
	@ClientString(id = 1010108, message = "East Entrance of Ant Nest")
	public static NpcStringId EAST_ENTRANCE_OF_ANT_NEST;
	
	@ClientString(id = 1010109, message = "West Entrance of Ant Nest")
	public static NpcStringId WEST_ENTRANCE_OF_ANT_NEST;
	
	@ClientString(id = 1010110, message = "Cruma Marshland")
	public static NpcStringId CRUMA_MARSHLAND;
	
	@ClientString(id = 1010111, message = "Plains of Dion (Lv. 23)")
	public static NpcStringId PLAINS_OF_DION_LV_23;
	
	@ClientString(id = 1010112, message = "Bee Hive")
	public static NpcStringId BEE_HIVE;
	
	@ClientString(id = 1010113, message = "Fortress of Resistance (Lv. 23)")
	public static NpcStringId FORTRESS_OF_RESISTANCE_LV_23;
	
	@ClientString(id = 1010114, message = "Guilloutine Fortress (Lv. 95)")
	public static NpcStringId GUILLOUTINE_FORTRESS_LV_95;
	
	@ClientString(id = 1010115, message = "Tanor Canyon (Lv. 46)")
	public static NpcStringId TANOR_CANYON_LV_46;
	
	@ClientString(id = 1010116, message = "Cruma Tower (Lv. 40)")
	public static NpcStringId CRUMA_TOWER_LV_40;
	
	@ClientString(id = 1010117, message = "Three-way crossroads at Dragon Valley")
	public static NpcStringId THREE_WAY_CROSSROADS_AT_DRAGON_VALLEY;
	
	@ClientString(id = 1010118, message = "Breka's Stronghold (Lv. 30)")
	public static NpcStringId BREKA_S_STRONGHOLD_LV_30;
	
	@ClientString(id = 1010119, message = "Gorgon Flower Garden (Lv. 31)")
	public static NpcStringId GORGON_FLOWER_GARDEN_LV_31;
	
	@ClientString(id = 1010120, message = "Antharas' Lair (Lv. 85)")
	public static NpcStringId ANTHARAS_LAIR_LV_85;
	
	@ClientString(id = 1010121, message = "Sea of Spores (Lv. 52)")
	public static NpcStringId SEA_OF_SPORES_LV_52;
	
	@ClientString(id = 1010122, message = "Outlaw Forest (Lv. 46)")
	public static NpcStringId OUTLAW_FOREST_LV_46;
	
	@ClientString(id = 1010123, message = "Forest of Evil and the Ivory Tower")
	public static NpcStringId FOREST_OF_EVIL_AND_THE_IVORY_TOWER;
	
	@ClientString(id = 1010124, message = "Timak Outpost")
	public static NpcStringId TIMAK_OUTPOST;
	
	@ClientString(id = 1010125, message = "Great Plains of Oren")
	public static NpcStringId GREAT_PLAINS_OF_OREN;
	
	@ClientString(id = 1010126, message = "Alchemist's Hut")
	public static NpcStringId ALCHEMIST_S_HUT;
	
	@ClientString(id = 1010127, message = "Ancient Battleground")
	public static NpcStringId ANCIENT_BATTLEGROUND;
	
	@ClientString(id = 1010128, message = "Northern Pathway of the Enchanted Valley")
	public static NpcStringId NORTHERN_PATHWAY_OF_THE_ENCHANTED_VALLEY;
	
	@ClientString(id = 1010129, message = "Southern Pathway of the Enchanted Valley")
	public static NpcStringId SOUTHERN_PATHWAY_OF_THE_ENCHANTED_VALLEY;
	
	@ClientString(id = 1010130, message = "Hunters Valley (Lv. 40)")
	public static NpcStringId HUNTERS_VALLEY_LV_40;
	
	@ClientString(id = 1010131, message = "Western Entrance of Blazing Swamp")
	public static NpcStringId WESTERN_ENTRANCE_OF_BLAZING_SWAMP;
	
	@ClientString(id = 1010132, message = "Eastern Entrance of Blazing Swamp")
	public static NpcStringId EASTERN_ENTRANCE_OF_BLAZING_SWAMP;
	
	@ClientString(id = 1010133, message = "Plains of Glory (Lv. 45)")
	public static NpcStringId PLAINS_OF_GLORY_LV_45;
	
	@ClientString(id = 1010134, message = "War-Torn Plains (Lv. 45)")
	public static NpcStringId WAR_TORN_PLAINS_LV_45;
	
	@ClientString(id = 1010135, message = "Northwestern Passage to the Forest of Mirrors")
	public static NpcStringId NORTHWESTERN_PASSAGE_TO_THE_FOREST_OF_MIRRORS;
	
	@ClientString(id = 1010136, message = "The Front of Anghel Waterfall")
	public static NpcStringId THE_FRONT_OF_ANGHEL_WATERFALL;
	
	@ClientString(id = 1010137, message = "South Entrance of Devastated Castle")
	public static NpcStringId SOUTH_ENTRANCE_OF_DEVASTATED_CASTLE;
	
	@ClientString(id = 1010138, message = "North Entrance of Devastated Castle")
	public static NpcStringId NORTH_ENTRANCE_OF_DEVASTATED_CASTLE;
	
	@ClientString(id = 1010139, message = "North Entrance of the Cemetery")
	public static NpcStringId NORTH_ENTRANCE_OF_THE_CEMETERY;
	
	@ClientString(id = 1010140, message = "South Entrance of the Cemetery")
	public static NpcStringId SOUTH_ENTRANCE_OF_THE_CEMETERY;
	
	@ClientString(id = 1010141, message = "West Entrance of the Cemetery")
	public static NpcStringId WEST_ENTRANCE_OF_THE_CEMETERY;
	
	@ClientString(id = 1010142, message = "Entrance of the Seal of Shilen")
	public static NpcStringId ENTRANCE_OF_THE_SEAL_OF_SHILEN;
	
	@ClientString(id = 1010143, message = "Forsaken Plains (Lv. 58)")
	public static NpcStringId FORSAKEN_PLAINS_LV_58;
	
	@ClientString(id = 1010144, message = "Tower of Insolence (Lv. 70)")
	public static NpcStringId TOWER_OF_INSOLENCE_LV_70;
	
	@ClientString(id = 1010145, message = "Giant's Cave")
	public static NpcStringId GIANT_S_CAVE;
	
	@ClientString(id = 1010146, message = "Northern Part of the Field of Silence")
	public static NpcStringId NORTHERN_PART_OF_THE_FIELD_OF_SILENCE;
	
	@ClientString(id = 1010147, message = "Western Part of the Field of Silence")
	public static NpcStringId WESTERN_PART_OF_THE_FIELD_OF_SILENCE;
	
	@ClientString(id = 1010148, message = "Eastern Part of the Field of Silence")
	public static NpcStringId EASTERN_PART_OF_THE_FIELD_OF_SILENCE;
	
	@ClientString(id = 1010149, message = "Western Part of the Field of Whispers")
	public static NpcStringId WESTERN_PART_OF_THE_FIELD_OF_WHISPERS;
	
	@ClientString(id = 1010150, message = "Alligator Island (Lv. 40)")
	public static NpcStringId ALLIGATOR_ISLAND_LV_40;
	
	@ClientString(id = 1010151, message = "Alligator Beach (Lv. 38)")
	public static NpcStringId ALLIGATOR_BEACH_LV_38;
	
	@ClientString(id = 1010152, message = "Devil's Isle")
	public static NpcStringId DEVIL_S_ISLE;
	
	@ClientString(id = 1010153, message = "Garden of Eva (Lv. 40)")
	public static NpcStringId GARDEN_OF_EVA_LV_40;
	
	@ClientString(id = 1010154, message = "Talking Island")
	public static NpcStringId TALKING_ISLAND;
	
	@ClientString(id = 1010155, message = "Elven Village")
	public static NpcStringId ELVEN_VILLAGE2;
	
	@ClientString(id = 1010156, message = "Dark Elf Village")
	public static NpcStringId DARK_ELF_VILLAGE;
	
	@ClientString(id = 1010157, message = "Orc Village")
	public static NpcStringId ORC_VILLAGE2;
	
	@ClientString(id = 1010158, message = "Dwarven Village")
	public static NpcStringId DWARVEN_VILLAGE2;
	
	@ClientString(id = 1010159, message = "Scenic Deck of Iris Lake")
	public static NpcStringId SCENIC_DECK_OF_IRIS_LAKE;
	
	@ClientString(id = 1010160, message = "Altar of Rites")
	public static NpcStringId ALTAR_OF_RITES;
	
	@ClientString(id = 1010161, message = "Dark Forest, Waterfall")
	public static NpcStringId DARK_FOREST_WATERFALL;
	
	@ClientString(id = 1010162, message = "Three-way Crossroads of the Neutral Zone")
	public static NpcStringId THREE_WAY_CROSSROADS_OF_THE_NEUTRAL_ZONE;
	
	@ClientString(id = 1010163, message = "Dark Forest")
	public static NpcStringId DARK_FOREST;
	
	@ClientString(id = 1010164, message = "Bloody Swampland (Lv. 85)")
	public static NpcStringId BLOODY_SWAMPLAND_LV_85;
	
	@ClientString(id = 1010165, message = "Black Rock Hill (Lv. 13)")
	public static NpcStringId BLACK_ROCK_HILL_LV_13;
	
	@ClientString(id = 1010166, message = "Spider Nest (Lv. 16)")
	public static NpcStringId SPIDER_NEST_LV_16;
	
	@ClientString(id = 1010167, message = "Elven Forest (Lv. 8)")
	public static NpcStringId ELVEN_FOREST_LV_8;
	
	@ClientString(id = 1010168, message = "Obelisk of Victory")
	public static NpcStringId OBELISK_OF_VICTORY;
	
	@ClientString(id = 1010169, message = "Northern Territory of Talking Island")
	public static NpcStringId NORTHERN_TERRITORY_OF_TALKING_ISLAND;
	
	@ClientString(id = 1010170, message = "Southern Territory of Talking Island")
	public static NpcStringId SOUTHERN_TERRITORY_OF_TALKING_ISLAND;
	
	@ClientString(id = 1010171, message = "Evil Hunting Grounds (Lv. 14)")
	public static NpcStringId EVIL_HUNTING_GROUNDS_LV_14;
	
	@ClientString(id = 1010172, message = "Maille Lizardmen Barracks")
	public static NpcStringId MAILLE_LIZARDMEN_BARRACKS;
	
	@ClientString(id = 1010173, message = "Ruins of Agony Bend (Lv. 19)")
	public static NpcStringId RUINS_OF_AGONY_BEND_LV_19;
	
	@ClientString(id = 1010174, message = "The Entrance to the Ruins of Despair (Lv. 19)")
	public static NpcStringId THE_ENTRANCE_TO_THE_RUINS_OF_DESPAIR_LV_19;
	
	@ClientString(id = 1010175, message = "Windmill Hill (Lv. 20)")
	public static NpcStringId WINDMILL_HILL_LV_20;
	
	@ClientString(id = 1010176, message = "Red Rock Ridge (Lv. 30)")
	public static NpcStringId RED_ROCK_RIDGE_LV_30;
	
	@ClientString(id = 1010177, message = "Floran Agricultural Area (Lv. 30)")
	public static NpcStringId FLORAN_AGRICULTURAL_AREA_LV_30;
	
	@ClientString(id = 1010178, message = "Western Tanor Canyon")
	public static NpcStringId WESTERN_TANOR_CANYON;
	
	@ClientString(id = 1010179, message = "Plains of the Lizardmen (Lv. 81)")
	public static NpcStringId PLAINS_OF_THE_LIZARDMEN_LV_81;
	
	@ClientString(id = 1010180, message = "Forest of Evil (Lv. 45)")
	public static NpcStringId FOREST_OF_EVIL_LV_45;
	
	@ClientString(id = 1010181, message = "Fields of Massacre (Lv. 61)")
	public static NpcStringId FIELDS_OF_MASSACRE_LV_61;
	
	@ClientString(id = 1010182, message = "Silent Valley (Lv. 75)")
	public static NpcStringId SILENT_VALLEY_LV_75;
	
	@ClientString(id = 1010183, message = "Northern Area of the Immortal Plateau, Northern Region")
	public static NpcStringId NORTHERN_AREA_OF_THE_IMMORTAL_PLATEAU_NORTHERN_REGION;
	
	@ClientString(id = 1010184, message = "Southern Area of the Immortal Plateau, Northern Region")
	public static NpcStringId SOUTHERN_AREA_OF_THE_IMMORTAL_PLATEAU_NORTHERN_REGION;
	
	@ClientString(id = 1010185, message = "Northern Area of the Immortal Plateau, Southern Region")
	public static NpcStringId NORTHERN_AREA_OF_THE_IMMORTAL_PLATEAU_SOUTHERN_REGION;
	
	@ClientString(id = 1010186, message = "Southern Area of the Immortal Plateau, Southern Region")
	public static NpcStringId SOUTHERN_AREA_OF_THE_IMMORTAL_PLATEAU_SOUTHERN_REGION;
	
	@ClientString(id = 1010187, message = "Western Mining Zone")
	public static NpcStringId WESTERN_MINING_ZONE;
	
	@ClientString(id = 1010188, message = "Western Mining Zone (Central Shore) (Lv. 8)")
	public static NpcStringId WESTERN_MINING_ZONE_CENTRAL_SHORE_LV_8;
	
	@ClientString(id = 1010189, message = "Eastern Mining Zone (Northeastern Shore) (Lv. 18)")
	public static NpcStringId EASTERN_MINING_ZONE_NORTHEASTERN_SHORE_LV_18;
	
	@ClientString(id = 1010190, message = "Entrance to the Abandoned Coal Mines")
	public static NpcStringId ENTRANCE_TO_THE_ABANDONED_COAL_MINES;
	
	@ClientString(id = 1010191, message = "Entrance to the Mithril Mines")
	public static NpcStringId ENTRANCE_TO_THE_MITHRIL_MINES;
	
	@ClientString(id = 1010192, message = "West Area of the Devastated Castle")
	public static NpcStringId WEST_AREA_OF_THE_DEVASTATED_CASTLE;
	
	@ClientString(id = 1010193, message = "Tower of Insolence, 3rd Floor")
	public static NpcStringId TOWER_OF_INSOLENCE_3RD_FLOOR;
	
	@ClientString(id = 1010194, message = "Tower of Insolence, 5th Floor")
	public static NpcStringId TOWER_OF_INSOLENCE_5TH_FLOOR2;
	
	@ClientString(id = 1010195, message = "Tower of Insolence, 7th Floor")
	public static NpcStringId TOWER_OF_INSOLENCE_7TH_FLOOR;
	
	@ClientString(id = 1010196, message = "Tower of Insolence, 10th Floor")
	public static NpcStringId TOWER_OF_INSOLENCE_10TH_FLOOR2;
	
	@ClientString(id = 1010197, message = "Tower of Insolence, 13th Floor")
	public static NpcStringId TOWER_OF_INSOLENCE_13TH_FLOOR;
	
	@ClientString(id = 1010198, message = "Southern Shore of Innadril (Garden of Eva)")
	public static NpcStringId SOUTHERN_SHORE_OF_INNADRIL_GARDEN_OF_EVA;
	
	@ClientString(id = 1010199, message = "Town of Goddard")
	public static NpcStringId TOWN_OF_GODDARD;
	
	@ClientString(id = 1010200, message = "Town of Rune")
	public static NpcStringId TOWN_OF_RUNE;
	
	@ClientString(id = 1010201, message = "A delivery for Mr. Lector? Very good!")
	public static NpcStringId A_DELIVERY_FOR_MR_LECTOR_VERY_GOOD;
	
	@ClientString(id = 1010202, message = "I need a break!")
	public static NpcStringId I_NEED_A_BREAK;
	
	@ClientString(id = 1010203, message = "Hello, Mr. Lector! Long time no see, Mr. Jackson!")
	public static NpcStringId HELLO_MR_LECTOR_LONG_TIME_NO_SEE_MR_JACKSON;
	
	@ClientString(id = 1010204, message = "Lulu!")
	public static NpcStringId LULU;
	
	@ClientString(id = 1010205, message = "Where has he gone?")
	public static NpcStringId WHERE_HAS_HE_GONE;
	
	@ClientString(id = 1010206, message = "Have you seen Windawood?")
	public static NpcStringId HAVE_YOU_SEEN_WINDAWOOD;
	
	@ClientString(id = 1010207, message = "Where did he go?")
	public static NpcStringId WHERE_DID_HE_GO;
	
	@ClientString(id = 1010208, message = "The Mother Tree is slowly dying.")
	public static NpcStringId THE_MOTHER_TREE_IS_SLOWLY_DYING;
	
	@ClientString(id = 1010209, message = "How can we save the Mother Tree?")
	public static NpcStringId HOW_CAN_WE_SAVE_THE_MOTHER_TREE;
	
	@ClientString(id = 1010210, message = "The Mother Tree is always so gorgeous!")
	public static NpcStringId THE_MOTHER_TREE_IS_ALWAYS_SO_GORGEOUS;
	
	@ClientString(id = 1010211, message = "Lady Mirabel, may the peace of the lake be with you!")
	public static NpcStringId LADY_MIRABEL_MAY_THE_PEACE_OF_THE_LAKE_BE_WITH_YOU;
	
	@ClientString(id = 1010212, message = "You're a hard worker, Rayla!")
	public static NpcStringId YOU_RE_A_HARD_WORKER_RAYLA;
	
	@ClientString(id = 1010213, message = "You're a hard worker!")
	public static NpcStringId YOU_RE_A_HARD_WORKER;
	
	@ClientString(id = 1010214, message = "The mass of darkness will start in a couple of days. Pay more attention to the guard!")
	public static NpcStringId THE_MASS_OF_DARKNESS_WILL_START_IN_A_COUPLE_OF_DAYS_PAY_MORE_ATTENTION_TO_THE_GUARD;
	
	@ClientString(id = 1010215, message = "Have you seen Torocco today?")
	public static NpcStringId HAVE_YOU_SEEN_TOROCCO_TODAY;
	
	@ClientString(id = 1010216, message = "Have you seen Torocco?")
	public static NpcStringId HAVE_YOU_SEEN_TOROCCO;
	
	@ClientString(id = 1010217, message = "Where is that fool hiding?")
	public static NpcStringId WHERE_IS_THAT_FOOL_HIDING;
	
	@ClientString(id = 1010218, message = "Care to go a round?")
	public static NpcStringId CARE_TO_GO_A_ROUND;
	
	@ClientString(id = 1010219, message = "Have a nice day, Mr. Garita and Mion!")
	public static NpcStringId HAVE_A_NICE_DAY_MR_GARITA_AND_MION;
	
	@ClientString(id = 1010220, message = "Mr. Lid, Murdoc, and Airy! How are you doing?")
	public static NpcStringId MR_LID_MURDOC_AND_AIRY_HOW_ARE_YOU_DOING;
	
	@ClientString(id = 1010221, message = "A black moon... Now do you understand that he has opened his eyes?")
	public static NpcStringId A_BLACK_MOON_NOW_DO_YOU_UNDERSTAND_THAT_HE_HAS_OPENED_HIS_EYES;
	
	@ClientString(id = 1010222, message = "Clouds of blood are gathering. Soon, it will start to rain. The rain of crimson blood...")
	public static NpcStringId CLOUDS_OF_BLOOD_ARE_GATHERING_SOON_IT_WILL_START_TO_RAIN_THE_RAIN_OF_CRIMSON_BLOOD;
	
	@ClientString(id = 1010223, message = "Darkness awakens while light remains in slumber...")
	public static NpcStringId DARKNESS_AWAKENS_WHILE_LIGHT_REMAINS_IN_SLUMBER;
	
	@ClientString(id = 1010224, message = "It is the deepest darkness. With its arrival, the world will soon die.")
	public static NpcStringId IT_IS_THE_DEEPEST_DARKNESS_WITH_ITS_ARRIVAL_THE_WORLD_WILL_SOON_DIE;
	
	@ClientString(id = 1010225, message = "Death is just a new beginning. Huhu... Fear not.")
	public static NpcStringId DEATH_IS_JUST_A_NEW_BEGINNING_HUHU_FEAR_NOT;
	
	@ClientString(id = 1010226, message = "Ahh! Beautiful goddess of death! Cover over the filth of this world with your darkness!")
	public static NpcStringId AHH_BEAUTIFUL_GODDESS_OF_DEATH_COVER_OVER_THE_FILTH_OF_THIS_WORLD_WITH_YOUR_DARKNESS;
	
	@ClientString(id = 1010227, message = "The goddess' resurrection has already begun. Huhu... Insignificant creatures like you can do nothing!")
	public static NpcStringId THE_GODDESS_RESURRECTION_HAS_ALREADY_BEGUN_HUHU_INSIGNIFICANT_CREATURES_LIKE_YOU_CAN_DO_NOTHING;
	
	@ClientString(id = 1010330, message = "You have obtained 500 Clan Reputation.")
	public static NpcStringId YOU_HAVE_OBTAINED_500_CLAN_REPUTATION;
	
	@ClientString(id = 1010400, message = "Croak, Croak! Food like $s1 in this place?!")
	public static NpcStringId CROAK_CROAK_FOOD_LIKE_S1_IN_THIS_PLACE;
	
	@ClientString(id = 1010401, message = "$s1, How lucky I am!")
	public static NpcStringId S1_HOW_LUCKY_I_AM;
	
	@ClientString(id = 1010402, message = "Pray that you caught a wrong fish $s1!")
	public static NpcStringId PRAY_THAT_YOU_CAUGHT_A_WRONG_FISH_S1;
	
	@ClientString(id = 1010403, message = "Do you know what a frog tastes like?")
	public static NpcStringId DO_YOU_KNOW_WHAT_A_FROG_TASTES_LIKE;
	
	@ClientString(id = 1010404, message = "I will show you the power of a frog!")
	public static NpcStringId I_WILL_SHOW_YOU_THE_POWER_OF_A_FROG;
	
	@ClientString(id = 1010405, message = "I will swallow at a mouthful!")
	public static NpcStringId I_WILL_SWALLOW_AT_A_MOUTHFUL;
	
	@ClientString(id = 1010406, message = "Ugh, no chance.  How could this elder pass away like this!")
	public static NpcStringId UGH_NO_CHANCE_HOW_COULD_THIS_ELDER_PASS_AWAY_LIKE_THIS;
	
	@ClientString(id = 1010407, message = "Croak! Croak! A frog is dying!")
	public static NpcStringId CROAK_CROAK_A_FROG_IS_DYING;
	
	@ClientString(id = 1010408, message = "A frog tastes bad!  Yuck!")
	public static NpcStringId A_FROG_TASTES_BAD_YUCK;
	
	@ClientString(id = 1010409, message = "Kaak! $s1, What are you doing now?")
	public static NpcStringId KAAK_S1_WHAT_ARE_YOU_DOING_NOW;
	
	@ClientString(id = 1010410, message = "Huh, $s1 You pierced into the body of the Spirit with a needle. Are you ready?")
	public static NpcStringId HUH_S1_YOU_PIERCED_INTO_THE_BODY_OF_THE_SPIRIT_WITH_A_NEEDLE_ARE_YOU_READY;
	
	@ClientString(id = 1010411, message = "Ooh $s1 That's you. But, no lady is pleased with this savage invitation!")
	public static NpcStringId OOH_S1_THAT_S_YOU_BUT_NO_LADY_IS_PLEASED_WITH_THIS_SAVAGE_INVITATION;
	
	@ClientString(id = 1010412, message = "You made me angry!")
	public static NpcStringId YOU_MADE_ME_ANGRY;
	
	@ClientString(id = 1010413, message = "It is but a scratch!  Is that all you can do?!")
	public static NpcStringId IT_IS_BUT_A_SCRATCH_IS_THAT_ALL_YOU_CAN_DO;
	
	@ClientString(id = 1010414, message = "Feel my pain!")
	public static NpcStringId FEEL_MY_PAIN;
	
	@ClientString(id = 1010415, message = "I'll get you for this!")
	public static NpcStringId I_LL_GET_YOU_FOR_THIS;
	
	@ClientString(id = 1010416, message = "I will tell fish not to take your bait!")
	public static NpcStringId I_WILL_TELL_FISH_NOT_TO_TAKE_YOUR_BAIT;
	
	@ClientString(id = 1010417, message = "You bothered such a weak spirit...Huh, Huh")
	public static NpcStringId YOU_BOTHERED_SUCH_A_WEAK_SPIRIT_HUH_HUH;
	
	@ClientString(id = 1010418, message = "Ke, ke, ke..$s1...I'm eating..Ke..")
	public static NpcStringId KE_KE_KE_S1_I_M_EATING_KE;
	
	@ClientString(id = 1010419, message = "Kuh..Ooh..$s1..Enmity...Fish....")
	public static NpcStringId KUH_OOH_S1_ENMITY_FISH;
	
	@ClientString(id = 1010420, message = "$s1...? Huh... Huh..huh...")
	public static NpcStringId S1_HUH_HUH_HUH;
	
	@ClientString(id = 1010421, message = "Ke, ke, ke, Rakul! Spin! Eh, eh, eh!")
	public static NpcStringId KE_KE_KE_RAKUL_SPIN_EH_EH_EH;
	
	@ClientString(id = 1010422, message = "Ah! Fafurion! Ah! Ah!")
	public static NpcStringId AH_FAFURION_AH_AH;
	
	@ClientString(id = 1010423, message = "Rakul! Rakul! Ra-kul!")
	public static NpcStringId RAKUL_RAKUL_RA_KUL;
	
	@ClientString(id = 1010424, message = "Eh..Enmity...Fish...")
	public static NpcStringId EH_ENMITY_FISH;
	
	@ClientString(id = 1010425, message = "I won't be eaten up...Kah, ah, ah")
	public static NpcStringId I_WON_T_BE_EATEN_UP_KAH_AH_AH;
	
	@ClientString(id = 1010426, message = "Cough! Rakul! Cough, Cough! Keh...")
	public static NpcStringId COUGH_RAKUL_COUGH_COUGH_KEH;
	
	@ClientString(id = 1010427, message = "Glory to Fafurion! Death to $s1!")
	public static NpcStringId GLORY_TO_FAFURION_DEATH_TO_S1;
	
	@ClientString(id = 1010428, message = "$s1! You are the one who bothered my poor fish!")
	public static NpcStringId S1_YOU_ARE_THE_ONE_WHO_BOTHERED_MY_POOR_FISH;
	
	@ClientString(id = 1010429, message = "Fafurion! A curse upon $s1!")
	public static NpcStringId FAFURION_A_CURSE_UPON_S1;
	
	@ClientString(id = 1010430, message = "Giant Special Attack!")
	public static NpcStringId GIANT_SPECIAL_ATTACK;
	
	@ClientString(id = 1010431, message = "Know the enmity of fish!")
	public static NpcStringId KNOW_THE_ENMITY_OF_FISH;
	
	@ClientString(id = 1010432, message = "I will show you the power of a spear!")
	public static NpcStringId I_WILL_SHOW_YOU_THE_POWER_OF_A_SPEAR;
	
	@ClientString(id = 1010433, message = "Glory to Fafurion!")
	public static NpcStringId GLORY_TO_FAFURION;
	
	@ClientString(id = 1010434, message = "Yipes!")
	public static NpcStringId YIPES;
	
	@ClientString(id = 1010435, message = "An old soldier does not die..! but just disappear...!")
	public static NpcStringId AN_OLD_SOLDIER_DOES_NOT_DIE_BUT_JUST_DISAPPEAR;
	
	@ClientString(id = 1010436, message = "$s1, Take my challenge, the knight of water!")
	public static NpcStringId S1_TAKE_MY_CHALLENGE_THE_KNIGHT_OF_WATER;
	
	@ClientString(id = 1010437, message = "Discover $s1 in the treasure chest of fish!")
	public static NpcStringId DISCOVER_S1_IN_THE_TREASURE_CHEST_OF_FISH;
	
	@ClientString(id = 1010438, message = "$s1, I took your bait!")
	public static NpcStringId S1_I_TOOK_YOUR_BAIT;
	
	@ClientString(id = 1010439, message = "I will show you spearmanship used in Dragon King's Palace!")
	public static NpcStringId I_WILL_SHOW_YOU_SPEARMANSHIP_USED_IN_DRAGON_KING_S_PALACE;
	
	@ClientString(id = 1010440, message = "This is the last gift I give you!")
	public static NpcStringId THIS_IS_THE_LAST_GIFT_I_GIVE_YOU;
	
	@ClientString(id = 1010441, message = "Your bait was too delicious! Now, I will kill you!")
	public static NpcStringId YOUR_BAIT_WAS_TOO_DELICIOUS_NOW_I_WILL_KILL_YOU;
	
	@ClientString(id = 1010442, message = "What a regret! The enmity of my brethren!")
	public static NpcStringId WHAT_A_REGRET_THE_ENMITY_OF_MY_BRETHREN;
	
	@ClientString(id = 1010443, message = "I'll pay you back! Somebody will have my revenge!")
	public static NpcStringId I_LL_PAY_YOU_BACK_SOMEBODY_WILL_HAVE_MY_REVENGE;
	
	@ClientString(id = 1010444, message = "Cough... But, I won't be eaten up by you...!")
	public static NpcStringId COUGH_BUT_I_WON_T_BE_EATEN_UP_BY_YOU;
	
	@ClientString(id = 1010445, message = "....? $s1... I will kill you...")
	public static NpcStringId S1_I_WILL_KILL_YOU;
	
	@ClientString(id = 1010446, message = "$s1... How could you catch me from the deep sea...")
	public static NpcStringId S1_HOW_COULD_YOU_CATCH_ME_FROM_THE_DEEP_SEA;
	
	@ClientString(id = 1010447, message = "$s1... Do you think I am a fish?")
	public static NpcStringId S1_DO_YOU_THINK_I_AM_A_FISH;
	
	@ClientString(id = 1010448, message = "Ebibibi~")
	public static NpcStringId EBIBIBI;
	
	@ClientString(id = 1010449, message = "He, he, he. Do you want me to roast you well?")
	public static NpcStringId HE_HE_HE_DO_YOU_WANT_ME_TO_ROAST_YOU_WELL;
	
	@ClientString(id = 1010450, message = "You didn't keep your eyes on me because I come from the sea?")
	public static NpcStringId YOU_DIDN_T_KEEP_YOUR_EYES_ON_ME_BECAUSE_I_COME_FROM_THE_SEA;
	
	@ClientString(id = 1010451, message = "Eeek... I feel sick...yow...!")
	public static NpcStringId EEEK_I_FEEL_SICK_YOW;
	
	@ClientString(id = 1010452, message = "I have failed...")
	public static NpcStringId I_HAVE_FAILED;
	
	@ClientString(id = 1010453, message = "Activity of life... Is stopped... Chizifc...")
	public static NpcStringId ACTIVITY_OF_LIFE_IS_STOPPED_CHIZIFC;
	
	@ClientString(id = 1010454, message = "Growling! $s1 -  Growling!")
	public static NpcStringId GROWLING_S1_GROWLING;
	
	@ClientString(id = 1010455, message = "I can smell $s1..!")
	public static NpcStringId I_CAN_SMELL_S1;
	
	@ClientString(id = 1010456, message = "Looks delicious, $s1!")
	public static NpcStringId LOOKS_DELICIOUS_S1;
	
	@ClientString(id = 1010457, message = "I will catch you!")
	public static NpcStringId I_WILL_CATCH_YOU;
	
	@ClientString(id = 1010458, message = "Uah, ah, ah, I couldn't eat anything for a long time!")
	public static NpcStringId UAH_AH_AH_I_COULDN_T_EAT_ANYTHING_FOR_A_LONG_TIME;
	
	@ClientString(id = 1010459, message = "I can swallow you at a mouthful!")
	public static NpcStringId I_CAN_SWALLOW_YOU_AT_A_MOUTHFUL;
	
	@ClientString(id = 1010460, message = "What?!  I am defeated by the prey!")
	public static NpcStringId WHAT_I_AM_DEFEATED_BY_THE_PREY;
	
	@ClientString(id = 1010461, message = "You are my food! I have to kill you!")
	public static NpcStringId YOU_ARE_MY_FOOD_I_HAVE_TO_KILL_YOU;
	
	@ClientString(id = 1010462, message = "I can't believe I am eaten up by my prey...  Gah!")
	public static NpcStringId I_CAN_T_BELIEVE_I_AM_EATEN_UP_BY_MY_PREY_GAH;
	
	@ClientString(id = 1010463, message = "....You caught me, $s1...?")
	public static NpcStringId YOU_CAUGHT_ME_S1;
	
	@ClientString(id = 1010464, message = "You're lucky to have even seen me, $s1.")
	public static NpcStringId YOU_RE_LUCKY_TO_HAVE_EVEN_SEEN_ME_S1;
	
	@ClientString(id = 1010465, message = "$s1, you can't leave here alive. Give up.")
	public static NpcStringId S1_YOU_CAN_T_LEAVE_HERE_ALIVE_GIVE_UP;
	
	@ClientString(id = 1010466, message = "I will show you the power of the deep sea!")
	public static NpcStringId I_WILL_SHOW_YOU_THE_POWER_OF_THE_DEEP_SEA;
	
	@ClientString(id = 1010467, message = "I will break the fishing pole!")
	public static NpcStringId I_WILL_BREAK_THE_FISHING_POLE;
	
	@ClientString(id = 1010468, message = "Your corpse will be good food for me!")
	public static NpcStringId YOUR_CORPSE_WILL_BE_GOOD_FOOD_FOR_ME;
	
	@ClientString(id = 1010469, message = "You are a good fisherman.")
	public static NpcStringId YOU_ARE_A_GOOD_FISHERMAN;
	
	@ClientString(id = 1010470, message = "Aren't you afraid of Fafurion?!")
	public static NpcStringId AREN_T_YOU_AFRAID_OF_FAFURION;
	
	@ClientString(id = 1010471, message = "You are excellent....")
	public static NpcStringId YOU_ARE_EXCELLENT;
	
	@ClientString(id = 1010472, message = "The Poison device has been activated.")
	public static NpcStringId THE_POISON_DEVICE_HAS_BEEN_ACTIVATED;
	
	@ClientString(id = 1010473, message = "The P. Atk. reduction device will be activated in 1 minute.")
	public static NpcStringId THE_P_ATK_REDUCTION_DEVICE_WILL_BE_ACTIVATED_IN_1_MINUTE;
	
	@ClientString(id = 1010474, message = "The Defense reduction device will be activated in 2 minutes.")
	public static NpcStringId THE_DEFENSE_REDUCTION_DEVICE_WILL_BE_ACTIVATED_IN_2_MINUTES;
	
	@ClientString(id = 1010475, message = "The HP regeneration reduction device will be activated in 3 minutes.")
	public static NpcStringId THE_HP_REGENERATION_REDUCTION_DEVICE_WILL_BE_ACTIVATED_IN_3_MINUTES;
	
	@ClientString(id = 1010476, message = "The P. Atk. reduction device has been activated.")
	public static NpcStringId THE_P_ATK_REDUCTION_DEVICE_HAS_BEEN_ACTIVATED;
	
	@ClientString(id = 1010477, message = "The Defense reduction device has been activated.")
	public static NpcStringId THE_DEFENSE_REDUCTION_DEVICE_HAS_BEEN_ACTIVATED;
	
	@ClientString(id = 1010478, message = "The HP regeneration reduction device has been activated.")
	public static NpcStringId THE_HP_REGENERATION_REDUCTION_DEVICE_HAS_BEEN_ACTIVATED;
	
	@ClientString(id = 1010479, message = "The poison device has now been destroyed.")
	public static NpcStringId THE_POISON_DEVICE_HAS_NOW_BEEN_DESTROYED;
	
	@ClientString(id = 1010480, message = "The P. Atk. reduction device has now been destroyed.")
	public static NpcStringId THE_P_ATK_REDUCTION_DEVICE_HAS_NOW_BEEN_DESTROYED;
	
	@ClientString(id = 1010481, message = "The Defense reduction device has been destroyed.")
	public static NpcStringId THE_DEFENSE_REDUCTION_DEVICE_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 1010482, message = "The HP regeneration reduction device will be activated in 3 minutes.")
	public static NpcStringId THE_HP_REGENERATION_REDUCTION_DEVICE_WILL_BE_ACTIVATED_IN_3_MINUTES2;
	
	@ClientString(id = 1010483, message = "$s1! Help me!!")
	public static NpcStringId S1_HELP_ME2;
	
	@ClientString(id = 1010484, message = "Help me!!")
	public static NpcStringId HELP_ME2;
	
	@ClientString(id = 1010485, message = "Entrance to the Cave of Trials")
	public static NpcStringId ENTRANCE_TO_THE_CAVE_OF_TRIALS;
	
	@ClientString(id = 1010486, message = "Inside the Elven Ruins")
	public static NpcStringId INSIDE_THE_ELVEN_RUINS;
	
	@ClientString(id = 1010487, message = "Entrance to the Elven Ruins")
	public static NpcStringId ENTRANCE_TO_THE_ELVEN_RUINS;
	
	@ClientString(id = 1010488, message = "Entrance to the School of Dark Arts")
	public static NpcStringId ENTRANCE_TO_THE_SCHOOL_OF_DARK_ARTS;
	
	@ClientString(id = 1010489, message = "Center of the School of Dark Arts")
	public static NpcStringId CENTER_OF_THE_SCHOOL_OF_DARK_ARTS;
	
	@ClientString(id = 1010490, message = "Entrance to the Elven Fortress")
	public static NpcStringId ENTRANCE_TO_THE_ELVEN_FORTRESS;
	
	@ClientString(id = 1010491, message = "Varka Silenos Barracks (Lv. 76)")
	public static NpcStringId VARKA_SILENOS_BARRACKS_LV_76;
	
	@ClientString(id = 1010492, message = "Ketra Orc Outpost (Lv. 76)")
	public static NpcStringId KETRA_ORC_OUTPOST_LV_76;
	
	@ClientString(id = 1010493, message = "Town of Rune Guild")
	public static NpcStringId TOWN_OF_RUNE_GUILD;
	
	@ClientString(id = 1010494, message = "Town of Rune Temple")
	public static NpcStringId TOWN_OF_RUNE_TEMPLE;
	
	@ClientString(id = 1010495, message = "Town of Rune Store")
	public static NpcStringId TOWN_OF_RUNE_STORE;
	
	@ClientString(id = 1010496, message = "Entrance to the Forest of the Dead")
	public static NpcStringId ENTRANCE_TO_THE_FOREST_OF_THE_DEAD;
	
	@ClientString(id = 1010497, message = "Western Entrance to the Swamp of Screams")
	public static NpcStringId WESTERN_ENTRANCE_TO_THE_SWAMP_OF_SCREAMS;
	
	@ClientString(id = 1010498, message = "Entrance to the Forgotten Temple")
	public static NpcStringId ENTRANCE_TO_THE_FORGOTTEN_TEMPLE;
	
	@ClientString(id = 1010499, message = "Center of the Forgotten Temple")
	public static NpcStringId CENTER_OF_THE_FORGOTTEN_TEMPLE;
	
	@ClientString(id = 1010500, message = "Entrance to the Cruma Tower")
	public static NpcStringId ENTRANCE_TO_THE_CRUMA_TOWER;
	
	@ClientString(id = 1010501, message = "Cruma Tower - First Floor")
	public static NpcStringId CRUMA_TOWER_FIRST_FLOOR;
	
	@ClientString(id = 1010502, message = "Cruma Tower - Second Floor")
	public static NpcStringId CRUMA_TOWER_SECOND_FLOOR;
	
	@ClientString(id = 1010503, message = "Cruma Tower - Third Floor")
	public static NpcStringId CRUMA_TOWER_THIRD_FLOOR;
	
	@ClientString(id = 1010504, message = "Entrance to Devil's Isle")
	public static NpcStringId ENTRANCE_TO_DEVIL_S_ISLE;
	
	@ClientString(id = 1010505, message = "Sel Mahum Training Grounds (West Gate) (Lv. 81)")
	public static NpcStringId SEL_MAHUM_TRAINING_GROUNDS_WEST_GATE_LV_81;
	
	@ClientString(id = 1010506, message = "Gludin Arena")
	public static NpcStringId GLUDIN_ARENA;
	
	@ClientString(id = 1010507, message = "Giran Arena")
	public static NpcStringId GIRAN_ARENA;
	
	@ClientString(id = 1010508, message = "Entrance to Antharas' Lair")
	public static NpcStringId ENTRANCE_TO_ANTHARAS_LAIR;
	
	@ClientString(id = 1010509, message = "Antharas' Lair - 1st Level")
	public static NpcStringId ANTHARAS_LAIR_1ST_LEVEL;
	
	@ClientString(id = 1010510, message = "Antharas' Lair - 2nd Level")
	public static NpcStringId ANTHARAS_LAIR_2ND_LEVEL;
	
	@ClientString(id = 1010511, message = "Antharas' Lair -  Magic Force Field Bridge")
	public static NpcStringId ANTHARAS_LAIR_MAGIC_FORCE_FIELD_BRIDGE;
	
	@ClientString(id = 1010512, message = "The Heart of Antharas' Lair")
	public static NpcStringId THE_HEART_OF_ANTHARAS_LAIR;
	
	@ClientString(id = 1010513, message = "East of the Field of Silence")
	public static NpcStringId EAST_OF_THE_FIELD_OF_SILENCE;
	
	@ClientString(id = 1010514, message = "West of the Field of Silence")
	public static NpcStringId WEST_OF_THE_FIELD_OF_SILENCE;
	
	@ClientString(id = 1010515, message = "East of the Field of Whispers")
	public static NpcStringId EAST_OF_THE_FIELD_OF_WHISPERS;
	
	@ClientString(id = 1010516, message = "West of the Field of Whispers")
	public static NpcStringId WEST_OF_THE_FIELD_OF_WHISPERS;
	
	@ClientString(id = 1010517, message = "Entrance to the Garden of Eva")
	public static NpcStringId ENTRANCE_TO_THE_GARDEN_OF_EVA;
	
	@ClientString(id = 1010518, message = "Entrance to Alligator Island")
	public static NpcStringId ENTRANCE_TO_ALLIGATOR_ISLAND2;
	
	@ClientString(id = 1010519, message = "Alligator Beach")
	public static NpcStringId ALLIGATOR_BEACH;
	
	@ClientString(id = 1010520, message = "Northern part of Alligator Island")
	public static NpcStringId NORTHERN_PART_OF_ALLIGATOR_ISLAND;
	
	@ClientString(id = 1010521, message = "Central part of Alligator Island")
	public static NpcStringId CENTRAL_PART_OF_ALLIGATOR_ISLAND;
	
	@ClientString(id = 1010522, message = "Garden of Eva - 2nd Level")
	public static NpcStringId GARDEN_OF_EVA_2ND_LEVEL;
	
	@ClientString(id = 1010523, message = "Garden of Eva - 3rd Level")
	public static NpcStringId GARDEN_OF_EVA_3RD_LEVEL;
	
	@ClientString(id = 1010524, message = "Garden of Eva - 4th Level")
	public static NpcStringId GARDEN_OF_EVA_4TH_LEVEL;
	
	@ClientString(id = 1010525, message = "Garden of Eva - 5th Level")
	public static NpcStringId GARDEN_OF_EVA_5TH_LEVEL;
	
	@ClientString(id = 1010526, message = "Inside the Garden of Eva")
	public static NpcStringId INSIDE_THE_GARDEN_OF_EVA;
	
	@ClientString(id = 1010527, message = "Four Sepulchers (Lv. 83)")
	public static NpcStringId FOUR_SEPULCHERS_LV_83;
	
	@ClientString(id = 1010528, message = "Imperial Tomb (Lv. 80)")
	public static NpcStringId IMPERIAL_TOMB_LV_80;
	
	@ClientString(id = 1010529, message = "Shrine of Loyalty (Lv. 76)")
	public static NpcStringId SHRINE_OF_LOYALTY_LV_76;
	
	@ClientString(id = 1010530, message = "Entrance to the Forge of the Gods (Lv. 78)")
	public static NpcStringId ENTRANCE_TO_THE_FORGE_OF_THE_GODS_LV_78;
	
	@ClientString(id = 1010531, message = "Forge of the Gods - Top Level")
	public static NpcStringId FORGE_OF_THE_GODS_TOP_LEVEL;
	
	@ClientString(id = 1010532, message = "Forge of the Gods - Lower Level")
	public static NpcStringId FORGE_OF_THE_GODS_LOWER_LEVEL;
	
	@ClientString(id = 1010533, message = "Entrance to the Wall of Argos")
	public static NpcStringId ENTRANCE_TO_THE_WALL_OF_ARGOS;
	
	@ClientString(id = 1010534, message = "Varka Silenos Village")
	public static NpcStringId VARKA_SILENOS_VILLAGE;
	
	@ClientString(id = 1010535, message = "Ketra Orc Village")
	public static NpcStringId KETRA_ORC_VILLAGE;
	
	@ClientString(id = 1010536, message = "Entrance to the Hot Springs Region")
	public static NpcStringId ENTRANCE_TO_THE_HOT_SPRINGS_REGION;
	
	@ClientString(id = 1010537, message = "Wild Beast Pastures (Lv. 83)")
	public static NpcStringId WILD_BEAST_PASTURES_LV_83;
	
	@ClientString(id = 1010538, message = "Entrance to the Valley of Saints")
	public static NpcStringId ENTRANCE_TO_THE_VALLEY_OF_SAINTS;
	
	@ClientString(id = 1010539, message = "Cursed Village")
	public static NpcStringId CURSED_VILLAGE;
	
	@ClientString(id = 1010540, message = "Southern Entrance of the Wild Beast Pastures")
	public static NpcStringId SOUTHERN_ENTRANCE_OF_THE_WILD_BEAST_PASTURES;
	
	@ClientString(id = 1010541, message = "Eastern Part of the Wild Beast Pastures")
	public static NpcStringId EASTERN_PART_OF_THE_WILD_BEAST_PASTURES;
	
	@ClientString(id = 1010542, message = "Western Part of the Wild Beast Pastures")
	public static NpcStringId WESTERN_PART_OF_THE_WILD_BEAST_PASTURES;
	
	@ClientString(id = 1010543, message = "Eastern Part of the Swamp of Screams")
	public static NpcStringId EASTERN_PART_OF_THE_SWAMP_OF_SCREAMS;
	
	@ClientString(id = 1010544, message = "Western Part of the Swamp of Screams")
	public static NpcStringId WESTERN_PART_OF_THE_SWAMP_OF_SCREAMS;
	
	@ClientString(id = 1010545, message = "Center of the Swamp of Screams")
	public static NpcStringId CENTER_OF_THE_SWAMP_OF_SCREAMS;
	
	@ClientString(id = 1010546, message = "Entrance to the Valley of Saints")
	public static NpcStringId ENTRANCE_TO_THE_VALLEY_OF_SAINTS2;
	
	@ClientString(id = 1010547, message = "Aden Frontier Gateway")
	public static NpcStringId ADEN_FRONTIER_GATEWAY;
	
	@ClientString(id = 1010548, message = "Oren Frontier Gateway")
	public static NpcStringId OREN_FRONTIER_GATEWAY;
	
	@ClientString(id = 1010549, message = "Garden of Wild Beasts (Lv. 67)")
	public static NpcStringId GARDEN_OF_WILD_BEASTS_LV_67;
	
	@ClientString(id = 1010550, message = "Devil's Pass (Lv. 60)")
	public static NpcStringId DEVIL_S_PASS_LV_60;
	
	@ClientString(id = 1010551, message = "The bullets are being loaded.")
	public static NpcStringId THE_BULLETS_ARE_BEING_LOADED;
	
	@ClientString(id = 1010552, message = "You can start at the scheduled time.")
	public static NpcStringId YOU_CAN_START_AT_THE_SCHEDULED_TIME;
	
	@ClientString(id = 1010553, message = "Entrance to the Giant's Cave (Lv. 80)")
	public static NpcStringId ENTRANCE_TO_THE_GIANT_S_CAVE_LV_80;
	
	@ClientString(id = 1010554, message = "Upper Level of the Giant's Cave")
	public static NpcStringId UPPER_LEVEL_OF_THE_GIANT_S_CAVE;
	
	@ClientString(id = 1010555, message = "Lower Level of the Giant's Cave")
	public static NpcStringId LOWER_LEVEL_OF_THE_GIANT_S_CAVE;
	
	@ClientString(id = 1010556, message = "Immortal Plateau, Northern Region")
	public static NpcStringId IMMORTAL_PLATEAU_NORTHERN_REGION;
	
	@ClientString(id = 1010557, message = "Elven Ruins")
	public static NpcStringId ELVEN_RUINS;
	
	@ClientString(id = 1010558, message = "Singing Waterfall")
	public static NpcStringId SINGING_WATERFALL;
	
	@ClientString(id = 1010559, message = "Talking Island, Northern Territory")
	public static NpcStringId TALKING_ISLAND_NORTHERN_TERRITORY;
	
	@ClientString(id = 1010560, message = "Elven Fortress (Lv. 10)")
	public static NpcStringId ELVEN_FORTRESS_LV_10;
	
	@ClientString(id = 1010561, message = "Pilgrim's Temple")
	public static NpcStringId PILGRIM_S_TEMPLE;
	
	@ClientString(id = 1010562, message = "Gludin Harbor")
	public static NpcStringId GLUDIN_HARBOR;
	
	@ClientString(id = 1010563, message = "Shilen's Garden")
	public static NpcStringId SHILEN_S_GARDEN;
	
	@ClientString(id = 1010564, message = "School of Dark Arts (Lv. 10)")
	public static NpcStringId SCHOOL_OF_DARK_ARTS_LV_10;
	
	@ClientString(id = 1010565, message = "Swamp of Screams (Lv. 66)")
	public static NpcStringId SWAMP_OF_SCREAMS_LV_66;
	
	@ClientString(id = 1010566, message = "Ant Nest (Lv. 29)")
	public static NpcStringId ANT_NEST_LV_29;
	
	@ClientString(id = 1010567, message = "Devil's Isle (Lv. 92)")
	public static NpcStringId DEVIL_S_ISLE_LV_92;
	
	@ClientString(id = 1010568, message = "Wall of Argos (Lv. 71)")
	public static NpcStringId WALL_OF_ARGOS_LV_71;
	
	@ClientString(id = 1010569, message = "Den of Evil (Lv. 81)")
	public static NpcStringId DEN_OF_EVIL_LV_81;
	
	@ClientString(id = 1010570, message = "Ice Merchant Cabin")
	public static NpcStringId ICE_MERCHANT_CABIN;
	
	@ClientString(id = 1010571, message = "Crypts of Disgrace (Lv. 80)")
	public static NpcStringId CRYPTS_OF_DISGRACE_LV_80;
	
	@ClientString(id = 1010572, message = "Plunderous Plains (Lv. 30)")
	public static NpcStringId PLUNDEROUS_PLAINS_LV_30;
	
	@ClientString(id = 1010573, message = "Pavel Ruins (Lv. 73)")
	public static NpcStringId PAVEL_RUINS_LV_73;
	
	@ClientString(id = 1010574, message = "Town of Schuttgart")
	public static NpcStringId TOWN_OF_SCHUTTGART;
	
	@ClientString(id = 1010575, message = "Monastery of Silence (Lv. 80)")
	public static NpcStringId MONASTERY_OF_SILENCE_LV_80;
	
	@ClientString(id = 1010576, message = "Monastery of Silence: Rear Gate")
	public static NpcStringId MONASTERY_OF_SILENCE_REAR_GATE;
	
	@ClientString(id = 1010577, message = "Stakato Nest (Lv. 82)")
	public static NpcStringId STAKATO_NEST_LV_82;
	
	@ClientString(id = 1010578, message = "How dare you trespass into my territory! Have you no fear?")
	public static NpcStringId HOW_DARE_YOU_TRESPASS_INTO_MY_TERRITORY_HAVE_YOU_NO_FEAR;
	
	@ClientString(id = 1010579, message = "Fools! Why haven't you fled yet? Prepare to learn a lesson!")
	public static NpcStringId FOOLS_WHY_HAVEN_T_YOU_FLED_YET_PREPARE_TO_LEARN_A_LESSON;
	
	@ClientString(id = 1010580, message = "Bwah-ha-ha! Your doom is at hand! Behold the Ultra Secret Super Weapon!")
	public static NpcStringId BWAH_HA_HA_YOUR_DOOM_IS_AT_HAND_BEHOLD_THE_ULTRA_SECRET_SUPER_WEAPON;
	
	@ClientString(id = 1010581, message = "Foolish, insignificant creatures! How dare you challenge me!")
	public static NpcStringId FOOLISH_INSIGNIFICANT_CREATURES_HOW_DARE_YOU_CHALLENGE_ME;
	
	@ClientString(id = 1010582, message = "I see that none will challenge me now!")
	public static NpcStringId I_SEE_THAT_NONE_WILL_CHALLENGE_ME_NOW;
	
	@ClientString(id = 1010583, message = "Urggh! You will pay dearly for this insult.")
	public static NpcStringId URGGH_YOU_WILL_PAY_DEARLY_FOR_THIS_INSULT;
	
	@ClientString(id = 1010584, message = "What? You haven't even two pennies to rub together? Harumph!")
	public static NpcStringId WHAT_YOU_HAVEN_T_EVEN_TWO_PENNIES_TO_RUB_TOGETHER_HARUMPH;
	
	@ClientString(id = 1010585, message = "Forest of Mirrors (Lv. 40)")
	public static NpcStringId FOREST_OF_MIRRORS_LV_40;
	
	@ClientString(id = 1010586, message = "The Center of the Forest of Mirrors")
	public static NpcStringId THE_CENTER_OF_THE_FOREST_OF_MIRRORS;
	
	@ClientString(id = 1010587, message = "Field of Silence (Western Section)")
	public static NpcStringId FIELD_OF_SILENCE_WESTERN_SECTION;
	
	@ClientString(id = 1010588, message = "Sky Wagon Relic (Lv. 39)")
	public static NpcStringId SKY_WAGON_RELIC_LV_39;
	
	@ClientString(id = 1010589, message = "Dark Forest")
	public static NpcStringId DARK_FOREST2;
	
	@ClientString(id = 1010590, message = "The Center of the Dark Forest")
	public static NpcStringId THE_CENTER_OF_THE_DARK_FOREST;
	
	@ClientString(id = 1010591, message = "Grave Robber Hideout")
	public static NpcStringId GRAVE_ROBBER_HIDEOUT;
	
	@ClientString(id = 1010592, message = "Forest of the Dead (Lv. 66)")
	public static NpcStringId FOREST_OF_THE_DEAD_LV_66;
	
	@ClientString(id = 1010593, message = "The Center of the Forest of the Dead")
	public static NpcStringId THE_CENTER_OF_THE_FOREST_OF_THE_DEAD;
	
	@ClientString(id = 1010594, message = "Mithril Mines (Lv. 81)")
	public static NpcStringId MITHRIL_MINES_LV_81;
	
	@ClientString(id = 1010595, message = "The Center of the Mithril Mines")
	public static NpcStringId THE_CENTER_OF_THE_MITHRIL_MINES;
	
	@ClientString(id = 1010596, message = "Abandoned Coal Mines (Lv. 10)")
	public static NpcStringId ABANDONED_COAL_MINES_LV_10;
	
	@ClientString(id = 1010597, message = "The Center of the Abandoned Coal Mines")
	public static NpcStringId THE_CENTER_OF_THE_ABANDONED_COAL_MINES;
	
	@ClientString(id = 1010598, message = "Immortal Plateau, Western Region")
	public static NpcStringId IMMORTAL_PLATEAU_WESTERN_REGION;
	
	@ClientString(id = 1010599, message = "Bee Hive (Lv. 34)")
	public static NpcStringId BEE_HIVE_LV_34;
	
	@ClientString(id = 1010600, message = "Valley of Saints (Lv. 60)")
	public static NpcStringId VALLEY_OF_SAINTS_LV_60;
	
	@ClientString(id = 1010601, message = "The Center of the Valley of Saints")
	public static NpcStringId THE_CENTER_OF_THE_VALLEY_OF_SAINTS;
	
	@ClientString(id = 1010602, message = "Field of Whispers (Eastern Section)")
	public static NpcStringId FIELD_OF_WHISPERS_EASTERN_SECTION;
	
	@ClientString(id = 1010603, message = "Cave of Trials (Lv. 10)")
	public static NpcStringId CAVE_OF_TRIALS_LV_10;
	
	@ClientString(id = 1010604, message = "Seal of Shilen (Lv. 95)")
	public static NpcStringId SEAL_OF_SHILEN_LV_95;
	
	@ClientString(id = 1010605, message = "The Center of the Wall of Argos")
	public static NpcStringId THE_CENTER_OF_THE_WALL_OF_ARGOS;
	
	@ClientString(id = 1010606, message = "The Center of Alligator Island")
	public static NpcStringId THE_CENTER_OF_ALLIGATOR_ISLAND;
	
	@ClientString(id = 1010607, message = "Anghel Waterfall")
	public static NpcStringId ANGHEL_WATERFALL;
	
	@ClientString(id = 1010608, message = "Center of the Elven Ruins")
	public static NpcStringId CENTER_OF_THE_ELVEN_RUINS;
	
	@ClientString(id = 1010609, message = "Hot Springs (Lv. 73)")
	public static NpcStringId HOT_SPRINGS_LV_73;
	
	@ClientString(id = 1010610, message = "The Center of the Hot Springs")
	public static NpcStringId THE_CENTER_OF_THE_HOT_SPRINGS;
	
	@ClientString(id = 1010611, message = "The Center of Dragon Valley")
	public static NpcStringId THE_CENTER_OF_DRAGON_VALLEY;
	
	@ClientString(id = 1010612, message = "Neutral Zone (Lv. 15)")
	public static NpcStringId NEUTRAL_ZONE_LV_15;
	
	@ClientString(id = 1010613, message = "The Center of the Neutral Zone")
	public static NpcStringId THE_CENTER_OF_THE_NEUTRAL_ZONE;
	
	@ClientString(id = 1010614, message = "Cruma Marshlands (Lv. 25)")
	public static NpcStringId CRUMA_MARSHLANDS_LV_25;
	
	@ClientString(id = 1010615, message = "The Center of the Cruma Marshlands")
	public static NpcStringId THE_CENTER_OF_THE_CRUMA_MARSHLANDS;
	
	@ClientString(id = 1010616, message = "Timak Outpost (Lv. 40)")
	public static NpcStringId TIMAK_OUTPOST_LV_40;
	
	@ClientString(id = 1010617, message = "The Center of the Enchanted Valley")
	public static NpcStringId THE_CENTER_OF_THE_ENCHANTED_VALLEY;
	
	@ClientString(id = 1010618, message = "Enchanted Valley, Southern Region (Lv. 45)")
	public static NpcStringId ENCHANTED_VALLEY_SOUTHERN_REGION_LV_45;
	
	@ClientString(id = 1010619, message = "Enchanted Valley, Northern Region (Lv. 45)")
	public static NpcStringId ENCHANTED_VALLEY_NORTHERN_REGION_LV_45;
	
	@ClientString(id = 1010620, message = "Frost Lake")
	public static NpcStringId FROST_LAKE;
	
	@ClientString(id = 1010621, message = "Wastelands (Lv. 35)")
	public static NpcStringId WASTELANDS_LV_35;
	
	@ClientString(id = 1010622, message = "Wastelands, Western Region")
	public static NpcStringId WASTELANDS_WESTERN_REGION;
	
	@ClientString(id = 1010623, message = "Who dares to covet the throne of our castle!  Leave immediately or you will pay the price of your audacity with your very own blood!")
	public static NpcStringId WHO_DARES_TO_COVET_THE_THRONE_OF_OUR_CASTLE_LEAVE_IMMEDIATELY_OR_YOU_WILL_PAY_THE_PRICE_OF_YOUR_AUDACITY_WITH_YOUR_VERY_OWN_BLOOD;
	
	@ClientString(id = 1010624, message = "Hmm, those who are not of the bloodline are coming this way to take over the castle?!  Humph!  The bitter grudges of the dead.  You must not make light of their power!")
	public static NpcStringId HMM_THOSE_WHO_ARE_NOT_OF_THE_BLOODLINE_ARE_COMING_THIS_WAY_TO_TAKE_OVER_THE_CASTLE_HUMPH_THE_BITTER_GRUDGES_OF_THE_DEAD_YOU_MUST_NOT_MAKE_LIGHT_OF_THEIR_POWER;
	
	@ClientString(id = 1010625, message = "Aargh...!  If I die, then the magic force field of blood will...!")
	public static NpcStringId AARGH_IF_I_DIE_THEN_THE_MAGIC_FORCE_FIELD_OF_BLOOD_WILL;
	
	@ClientString(id = 1010626, message = "It's not over yet...  It won't be... over... like this... Never...")
	public static NpcStringId IT_S_NOT_OVER_YET_IT_WON_T_BE_OVER_LIKE_THIS_NEVER;
	
	@ClientString(id = 1010627, message = "Oooh! Who poured nectar on my head while I was sleeping?")
	public static NpcStringId OOOH_WHO_POURED_NECTAR_ON_MY_HEAD_WHILE_I_WAS_SLEEPING;
	
	@ClientString(id = 1010628, message = "Please wait a moment.")
	public static NpcStringId PLEASE_WAIT_A_MOMENT;
	
	@ClientString(id = 1010629, message = "The word you need this time is $s1.")
	public static NpcStringId THE_WORD_YOU_NEED_THIS_TIME_IS_S1;
	
	@ClientString(id = 1010630, message = "Intruders! Sound the alarm!")
	public static NpcStringId INTRUDERS_SOUND_THE_ALARM;
	
	@ClientString(id = 1010631, message = "De-activate the alarm.")
	public static NpcStringId DE_ACTIVATE_THE_ALARM;
	
	@ClientString(id = 1010632, message = "Oh no! The defenses have failed. It is too dangerous to remain inside the castle. Flee! Every man for himself!")
	public static NpcStringId OH_NO_THE_DEFENSES_HAVE_FAILED_IT_IS_TOO_DANGEROUS_TO_REMAIN_INSIDE_THE_CASTLE_FLEE_EVERY_MAN_FOR_HIMSELF;
	
	@ClientString(id = 1010633, message = "The game has begun. Participants, prepare to learn an important word.")
	public static NpcStringId THE_GAME_HAS_BEGUN_PARTICIPANTS_PREPARE_TO_LEARN_AN_IMPORTANT_WORD;
	
	@ClientString(id = 1010634, message = "$s1 team's jackpot has $s2 percent of its HP remaining.")
	public static NpcStringId S1_TEAM_S_JACKPOT_HAS_S2_PERCENT_OF_ITS_HP_REMAINING;
	
	@ClientString(id = 1010635, message = "Undecided")
	public static NpcStringId UNDECIDED;
	
	@ClientString(id = 1010636, message = "Heh Heh... I see that the feast has begun! Be wary! The curse of the Hellmann family has poisoned this land!")
	public static NpcStringId HEH_HEH_I_SEE_THAT_THE_FEAST_HAS_BEGUN_BE_WARY_THE_CURSE_OF_THE_HELLMANN_FAMILY_HAS_POISONED_THIS_LAND;
	
	@ClientString(id = 1010637, message = "Arise, my faithful servants! You, my people who have inherited the blood.  It is the calling of my daughter.  The feast of blood will now begin!")
	public static NpcStringId ARISE_MY_FAITHFUL_SERVANTS_YOU_MY_PEOPLE_WHO_HAVE_INHERITED_THE_BLOOD_IT_IS_THE_CALLING_OF_MY_DAUGHTER_THE_FEAST_OF_BLOOD_WILL_NOW_BEGIN;
	
	@ClientString(id = 1010638, message = "You may have lordship over the forest for now…but remember! The true lord will return… Enjoy your power while it lasts, if you can manage to sleep at night! Hahaha!")
	public static NpcStringId YOU_MAY_HAVE_LORDSHIP_OVER_THE_FOREST_FOR_NOW_BUT_REMEMBER_THE_TRUE_LORD_WILL_RETURN_ENJOY_YOUR_POWER_WHILE_IT_LASTS_IF_YOU_CAN_MANAGE_TO_SLEEP_AT_NIGHT_HAHAHA;
	
	@ClientString(id = 1010639, message = "Grarr! For the next 2 minutes or so, the game arena are will be cleaned. Throw any items you don't need to the floor now.")
	public static NpcStringId GRARR_FOR_THE_NEXT_2_MINUTES_OR_SO_THE_GAME_ARENA_ARE_WILL_BE_CLEANED_THROW_ANY_ITEMS_YOU_DON_T_NEED_TO_THE_FLOOR_NOW;
	
	@ClientString(id = 1010640, message = "Grarr! $s1 team is using the Hot Spring Brimstone on the opponent's camp.")
	public static NpcStringId GRARR_S1_TEAM_IS_USING_THE_HOT_SPRING_BRIMSTONE_ON_THE_OPPONENT_S_CAMP;
	
	@ClientString(id = 1010641, message = "Grarr! $s1 team is attempting to steal the jackpot.")
	public static NpcStringId GRARR_S1_TEAM_IS_ATTEMPTING_TO_STEAL_THE_JACKPOT;
	
	@ClientString(id = 1010642, message = "** Vacant Seat **")
	public static NpcStringId VACANT_SEAT;
	
	@ClientString(id = 1010643, message = "$s1 minute(s) are remaining.")
	public static NpcStringId S1_MINUTE_S_ARE_REMAINING;
	
	@ClientString(id = 1010644, message = "How dare you ruin the performance of the Dark Choir... Unforgivable!")
	public static NpcStringId HOW_DARE_YOU_RUIN_THE_PERFORMANCE_OF_THE_DARK_CHOIR_UNFORGIVABLE;
	
	@ClientString(id = 1010645, message = "Get rid of the invaders who interrupt the performance of the Dark Choir!")
	public static NpcStringId GET_RID_OF_THE_INVADERS_WHO_INTERRUPT_THE_PERFORMANCE_OF_THE_DARK_CHOIR;
	
	@ClientString(id = 1010646, message = "Don't you hear the music of death? Reveal the horror of the Dark Choir!")
	public static NpcStringId DON_T_YOU_HEAR_THE_MUSIC_OF_DEATH_REVEAL_THE_HORROR_OF_THE_DARK_CHOIR;
	
	@ClientString(id = 1010647, message = "Immortal Plateau (Lv. 8)")
	public static NpcStringId IMMORTAL_PLATEAU_LV_8;
	
	@ClientString(id = 1010648, message = "Kamael Village")
	public static NpcStringId KAMAEL_VILLAGE;
	
	@ClientString(id = 1010649, message = "Isle of Souls Base")
	public static NpcStringId ISLE_OF_SOULS_BASE;
	
	@ClientString(id = 1010650, message = "Golden Hills Base")
	public static NpcStringId GOLDEN_HILLS_BASE;
	
	@ClientString(id = 1010651, message = "Mimir's Forest Base")
	public static NpcStringId MIMIR_S_FOREST_BASE;
	
	@ClientString(id = 1010652, message = "Isle of Souls Harbor(Lv. 93)")
	public static NpcStringId ISLE_OF_SOULS_HARBOR_LV_93;
	
	@ClientString(id = 1010653, message = "Stronghold I (Lv. 1)")
	public static NpcStringId STRONGHOLD_I_LV_1;
	
	@ClientString(id = 1010654, message = "Stronghold 2 (Lv. 93)")
	public static NpcStringId STRONGHOLD_2_LV_93;
	
	@ClientString(id = 1010655, message = "Stronghold 3 (Lv. 93)")
	public static NpcStringId STRONGHOLD_3_LV_93;
	
	@ClientString(id = 1010656, message = "Fortress West Gate")
	public static NpcStringId FORTRESS_WEST_GATE;
	
	@ClientString(id = 1010657, message = "Fortress East Gate")
	public static NpcStringId FORTRESS_EAST_GATE;
	
	@ClientString(id = 1010658, message = "Fortress North Gate")
	public static NpcStringId FORTRESS_NORTH_GATE;
	
	@ClientString(id = 1010659, message = "Fortress South Gate")
	public static NpcStringId FORTRESS_SOUTH_GATE;
	
	@ClientString(id = 1010660, message = "Front of the Valley Fortress")
	public static NpcStringId FRONT_OF_THE_VALLEY_FORTRESS;
	
	@ClientString(id = 1010661, message = "Goddard Town Square")
	public static NpcStringId GODDARD_TOWN_SQUARE;
	
	@ClientString(id = 1010662, message = "Front of the Goddard Castle Gate")
	public static NpcStringId FRONT_OF_THE_GODDARD_CASTLE_GATE;
	
	@ClientString(id = 1010663, message = "Gludio Town Square")
	public static NpcStringId GLUDIO_TOWN_SQUARE;
	
	@ClientString(id = 1010664, message = "Front of the Gludio Castle Gate")
	public static NpcStringId FRONT_OF_THE_GLUDIO_CASTLE_GATE;
	
	@ClientString(id = 1010665, message = "Giran Town Square")
	public static NpcStringId GIRAN_TOWN_SQUARE;
	
	@ClientString(id = 1010666, message = "Front of the Giran Castle Gate")
	public static NpcStringId FRONT_OF_THE_GIRAN_CASTLE_GATE;
	
	@ClientString(id = 1010667, message = "Front of the Southern Fortress")
	public static NpcStringId FRONT_OF_THE_SOUTHERN_FORTRESS;
	
	@ClientString(id = 1010668, message = "Front of the Swamp Fortress")
	public static NpcStringId FRONT_OF_THE_SWAMP_FORTRESS;
	
	@ClientString(id = 1010669, message = "Dion Town Square")
	public static NpcStringId DION_TOWN_SQUARE;
	
	@ClientString(id = 1010670, message = "Front of the Dion Castle Gate")
	public static NpcStringId FRONT_OF_THE_DION_CASTLE_GATE;
	
	@ClientString(id = 1010671, message = "Rune Town Square")
	public static NpcStringId RUNE_TOWN_SQUARE;
	
	@ClientString(id = 1010672, message = "Front of the Rune Castle Gate")
	public static NpcStringId FRONT_OF_THE_RUNE_CASTLE_GATE;
	
	@ClientString(id = 1010673, message = "Front of the White Sand Fortress")
	public static NpcStringId FRONT_OF_THE_WHITE_SAND_FORTRESS;
	
	@ClientString(id = 1010674, message = "Front of the Basin Fortress")
	public static NpcStringId FRONT_OF_THE_BASIN_FORTRESS;
	
	@ClientString(id = 1010675, message = "Front of the Ivory Fortress")
	public static NpcStringId FRONT_OF_THE_IVORY_FORTRESS;
	
	@ClientString(id = 1010676, message = "Schuttgart Town Square")
	public static NpcStringId SCHUTTGART_TOWN_SQUARE;
	
	@ClientString(id = 1010677, message = "Front of the Schuttgart Castle Gate")
	public static NpcStringId FRONT_OF_THE_SCHUTTGART_CASTLE_GATE;
	
	@ClientString(id = 1010678, message = "Aden Town Square")
	public static NpcStringId ADEN_TOWN_SQUARE;
	
	@ClientString(id = 1010679, message = "Front of the Aden Castle Gate")
	public static NpcStringId FRONT_OF_THE_ADEN_CASTLE_GATE;
	
	@ClientString(id = 1010680, message = "Front of the Camp Fortress")
	public static NpcStringId FRONT_OF_THE_CAMP_FORTRESS;
	
	@ClientString(id = 1010681, message = "Oren Town Square")
	public static NpcStringId OREN_TOWN_SQUARE;
	
	@ClientString(id = 1010682, message = "Front of the Oren Castle Gate")
	public static NpcStringId FRONT_OF_THE_OREN_CASTLE_GATE;
	
	@ClientString(id = 1010683, message = "Front of the Archaic Fortress")
	public static NpcStringId FRONT_OF_THE_ARCHAIC_FORTRESS;
	
	@ClientString(id = 1010684, message = "Front of the Innadril Castle Gate")
	public static NpcStringId FRONT_OF_THE_INNADRIL_CASTLE_GATE;
	
	@ClientString(id = 1010685, message = "Front of the Border Fortress")
	public static NpcStringId FRONT_OF_THE_BORDER_FORTRESS;
	
	@ClientString(id = 1010686, message = "Heine Town Square")
	public static NpcStringId HEINE_TOWN_SQUARE;
	
	@ClientString(id = 1010687, message = "Front of the Hive Fortress")
	public static NpcStringId FRONT_OF_THE_HIVE_FORTRESS;
	
	@ClientString(id = 1010688, message = "Front of the Narsell Fortress")
	public static NpcStringId FRONT_OF_THE_NARSELL_FORTRESS;
	
	@ClientString(id = 1010689, message = "Front of the Gludio Castle")
	public static NpcStringId FRONT_OF_THE_GLUDIO_CASTLE;
	
	@ClientString(id = 1010690, message = "Front of the Dion Castle")
	public static NpcStringId FRONT_OF_THE_DION_CASTLE;
	
	@ClientString(id = 1010691, message = "Front of the Giran Castle")
	public static NpcStringId FRONT_OF_THE_GIRAN_CASTLE;
	
	@ClientString(id = 1010692, message = "Front of the Oren Castle")
	public static NpcStringId FRONT_OF_THE_OREN_CASTLE;
	
	@ClientString(id = 1010693, message = "Front of the Aden Castle")
	public static NpcStringId FRONT_OF_THE_ADEN_CASTLE;
	
	@ClientString(id = 1010694, message = "Front of the Innadril Castle")
	public static NpcStringId FRONT_OF_THE_INNADRIL_CASTLE;
	
	@ClientString(id = 1010695, message = "Front of the Goddard Castle")
	public static NpcStringId FRONT_OF_THE_GODDARD_CASTLE;
	
	@ClientString(id = 1010696, message = "Front of the Rune Castle")
	public static NpcStringId FRONT_OF_THE_RUNE_CASTLE;
	
	@ClientString(id = 1010697, message = "Front of the Schuttgart Castle")
	public static NpcStringId FRONT_OF_THE_SCHUTTGART_CASTLE;
	
	@ClientString(id = 1010698, message = "Primeval Isle Wharf")
	public static NpcStringId PRIMEVAL_ISLE_WHARF;
	
	@ClientString(id = 1010699, message = "Isle of Prayer (Lv. 78)")
	public static NpcStringId ISLE_OF_PRAYER_LV_78;
	
	@ClientString(id = 1010700, message = "Mithril Mines Western Entrance (Lv. 81)")
	public static NpcStringId MITHRIL_MINES_WESTERN_ENTRANCE_LV_81;
	
	@ClientString(id = 1010701, message = "Mithril Mines Eastern Entrance (Lv. 81)")
	public static NpcStringId MITHRIL_MINES_EASTERN_ENTRANCE_LV_81;
	
	@ClientString(id = 1010702, message = "Giant's Cave Upper Layer")
	public static NpcStringId GIANT_S_CAVE_UPPER_LAYER;
	
	@ClientString(id = 1010703, message = "Giant's Cave Lower Layer")
	public static NpcStringId GIANT_S_CAVE_LOWER_LAYER;
	
	@ClientString(id = 1010704, message = "Field of Silence Center")
	public static NpcStringId FIELD_OF_SILENCE_CENTER;
	
	@ClientString(id = 1010705, message = "Field of Whispers Center")
	public static NpcStringId FIELD_OF_WHISPERS_CENTER;
	
	@ClientString(id = 1010706, message = "Shyeed's Cavern")
	public static NpcStringId SHYEED_S_CAVERN;
	
	@ClientString(id = 1010707, message = "Sel Mahum Training Grounds (South Gate)  (Lv. 81)")
	public static NpcStringId SEL_MAHUM_TRAINING_GROUNDS_SOUTH_GATE_LV_81;
	
	@ClientString(id = 1010708, message = "Sel Mahum Training Grounds (Lv. 81)")
	public static NpcStringId SEL_MAHUM_TRAINING_GROUNDS_LV_81;
	
	@ClientString(id = 1010709, message = "Seed of Infinity Dock")
	public static NpcStringId SEED_OF_INFINITY_DOCK;
	
	@ClientString(id = 1010710, message = "Seed of Destruction Dock")
	public static NpcStringId SEED_OF_DESTRUCTION_DOCK;
	
	@ClientString(id = 1010711, message = "Seed of Annihilation Dock")
	public static NpcStringId SEED_OF_ANNIHILATION_DOCK;
	
	@ClientString(id = 1010712, message = "Town of Aden Einhasad Temple Priest Wood")
	public static NpcStringId TOWN_OF_ADEN_EINHASAD_TEMPLE_PRIEST_WOOD;
	
	@ClientString(id = 1010713, message = "Hunter's Village Separated Soul Front")
	public static NpcStringId HUNTER_S_VILLAGE_SEPARATED_SOUL_FRONT;
	
	@ClientString(id = 1010714, message = "Lachik 1")
	public static NpcStringId LACHIK_1;
	
	@ClientString(id = 1010715, message = "Lachik 2")
	public static NpcStringId LACHIK_2;
	
	@ClientString(id = 1010716, message = "Lachik 3")
	public static NpcStringId LACHIK_3;
	
	@ClientString(id = 1010717, message = "Lachik 4")
	public static NpcStringId LACHIK_4;
	
	@ClientString(id = 1010718, message = "Parnassus")
	public static NpcStringId PARNASSUS;
	
	@ClientString(id = 1010719, message = "Frozen Labyrinth (Lv. 53)")
	public static NpcStringId FROZEN_LABYRINTH_LV_53;
	
	@ClientString(id = 1010720, message = "Town of Schuttgart")
	public static NpcStringId TOWN_OF_SCHUTTGART2;
	
	@ClientString(id = 1010721, message = "Seed of Annihilation (Lv. 85)")
	public static NpcStringId SEED_OF_ANNIHILATION_LV_85;
	
	@ClientString(id = 1010722, message = "Bloody Swampland (Lv. 85)")
	public static NpcStringId BLOODY_SWAMPLAND_LV_852;
	
	@ClientString(id = 1010723, message = "Ruins of Ye Sagira (Lv. 85)")
	public static NpcStringId RUINS_OF_YE_SAGIRA_LV_85;
	
	@ClientString(id = 1010724, message = "Ancient City Arcan")
	public static NpcStringId ANCIENT_CITY_ARCAN;
	
	@ClientString(id = 1010725, message = "Garden of Genesis (Lv. 90)")
	public static NpcStringId GARDEN_OF_GENESIS_LV_90;
	
	@ClientString(id = 1010726, message = "Fairy Settlement (Lv. 90)")
	public static NpcStringId FAIRY_SETTLEMENT_LV_90;
	
	@ClientString(id = 1010727, message = "Seal of Shilen (Lv. 95)")
	public static NpcStringId SEAL_OF_SHILEN_LV_952;
	
	@ClientString(id = 1010728, message = "Orbis Temple Entrance (Lv. 95)")
	public static NpcStringId ORBIS_TEMPLE_ENTRANCE_LV_95;
	
	@ClientString(id = 1010729, message = "Parnassus (Lv. 97)")
	public static NpcStringId PARNASSUS_LV_97;
	
	@ClientString(id = 1010730, message = "Gludio Wharf")
	public static NpcStringId GLUDIO_WHARF;
	
	@ClientString(id = 1010731, message = "Keucereus Alliance Base")
	public static NpcStringId KEUCEREUS_ALLIANCE_BASE;
	
	@ClientString(id = 1010732, message = "Isle of Souls Harbor (Lv. 93)")
	public static NpcStringId ISLE_OF_SOULS_HARBOR_LV_932;
	
	@ClientString(id = 1010733, message = "North of Alligator Island (Lv. 40)")
	public static NpcStringId NORTH_OF_ALLIGATOR_ISLAND_LV_40;
	
	@ClientString(id = 1010734, message = "East of the Swamp of Screams (Lv. 66)")
	public static NpcStringId EAST_OF_THE_SWAMP_OF_SCREAMS_LV_66;
	
	@ClientString(id = 1010735, message = "Center of the Wall of Argos (Lv. 71)")
	public static NpcStringId CENTER_OF_THE_WALL_OF_ARGOS_LV_71;
	
	@ClientString(id = 1010736, message = "North of the Den of Evil (Lv. 81)")
	public static NpcStringId NORTH_OF_THE_DEN_OF_EVIL_LV_81;
	
	@ClientString(id = 1010737, message = "Gainak Village")
	public static NpcStringId GAINAK_VILLAGE;
	
	@ClientString(id = 1010738, message = "In front of the Heine Warp Gate")
	public static NpcStringId IN_FRONT_OF_THE_HEINE_WARP_GATE;
	
	@ClientString(id = 1010739, message = "Ruins of Ye Sagira")
	public static NpcStringId RUINS_OF_YE_SAGIRA;
	
	@ClientString(id = 1010740, message = "Reliquary of the Giants")
	public static NpcStringId RELIQUARY_OF_THE_GIANTS;
	
	@ClientString(id = 1010741, message = "Faeron Village")
	public static NpcStringId FAERON_VILLAGE;
	
	@ClientString(id = 1010742, message = "Whispering Woods")
	public static NpcStringId WHISPERING_WOODS;
	
	@ClientString(id = 1029350, message = "What took so long? I waited for ever.")
	public static NpcStringId WHAT_TOOK_SO_LONG_I_WAITED_FOR_EVER;
	
	@ClientString(id = 1029351, message = "I must ask Librarian Sophia about the book.")
	public static NpcStringId I_MUST_ASK_LIBRARIAN_SOPHIA_ABOUT_THE_BOOK;
	
	@ClientString(id = 1029352, message = "This library... It's huge but there aren't many useful books, right?")
	public static NpcStringId THIS_LIBRARY_IT_S_HUGE_BUT_THERE_AREN_T_MANY_USEFUL_BOOKS_RIGHT;
	
	@ClientString(id = 1029353, message = "An underground library... I hate damp and smelly places...")
	public static NpcStringId AN_UNDERGROUND_LIBRARY_I_HATE_DAMP_AND_SMELLY_PLACES;
	
	@ClientString(id = 1029354, message = "The book that we seek is certainly here. Search inch by inch.")
	public static NpcStringId THE_BOOK_THAT_WE_SEEK_IS_CERTAINLY_HERE_SEARCH_INCH_BY_INCH;
	
	@ClientString(id = 1029450, message = "We must search high and low in every room for the reading desk that contains the book we seek.")
	public static NpcStringId WE_MUST_SEARCH_HIGH_AND_LOW_IN_EVERY_ROOM_FOR_THE_READING_DESK_THAT_CONTAINS_THE_BOOK_WE_SEEK;
	
	@ClientString(id = 1029451, message = "Remember the content of the books that you found. You can't take them out with you.")
	public static NpcStringId REMEMBER_THE_CONTENT_OF_THE_BOOKS_THAT_YOU_FOUND_YOU_CAN_T_TAKE_THEM_OUT_WITH_YOU;
	
	@ClientString(id = 1029452, message = "It seems that you cannot remember to the room of the watcher who found the book.")
	public static NpcStringId IT_SEEMS_THAT_YOU_CANNOT_REMEMBER_TO_THE_ROOM_OF_THE_WATCHER_WHO_FOUND_THE_BOOK;
	
	@ClientString(id = 1029453, message = "Your work here is done, so return to the central guardian.")
	public static NpcStringId YOUR_WORK_HERE_IS_DONE_SO_RETURN_TO_THE_CENTRAL_GUARDIAN;
	
	@ClientString(id = 1029460, message = "You foolish invaders who disturb the rest of Solina, be gone from this place.")
	public static NpcStringId YOU_FOOLISH_INVADERS_WHO_DISTURB_THE_REST_OF_SOLINA_BE_GONE_FROM_THIS_PLACE;
	
	@ClientString(id = 1029461, message = "I know not what you seek, but this truth cannot be handled by mere humans.")
	public static NpcStringId I_KNOW_NOT_WHAT_YOU_SEEK_BUT_THIS_TRUTH_CANNOT_BE_HANDLED_BY_MERE_HUMANS;
	
	@ClientString(id = 1029462, message = "I will not stand by and watch your foolish actions. I warn you, leave this place at once.")
	public static NpcStringId I_WILL_NOT_STAND_BY_AND_WATCH_YOUR_FOOLISH_ACTIONS_I_WARN_YOU_LEAVE_THIS_PLACE_AT_ONCE;
	
	@ClientString(id = 1029550, message = "The guardian of the seal doesn't seem to get injured at all until the barrier is destroyed.")
	public static NpcStringId THE_GUARDIAN_OF_THE_SEAL_DOESN_T_SEEM_TO_GET_INJURED_AT_ALL_UNTIL_THE_BARRIER_IS_DESTROYED;
	
	@ClientString(id = 1029551, message = "The device located in the room in front of the guardian of the seal is definitely the barrier that controls the guardian's power.")
	public static NpcStringId THE_DEVICE_LOCATED_IN_THE_ROOM_IN_FRONT_OF_THE_GUARDIAN_OF_THE_SEAL_IS_DEFINITELY_THE_BARRIER_THAT_CONTROLS_THE_GUARDIAN_S_POWER;
	
	@ClientString(id = 1029552, message = "To remove the barrier, you must find the relics that fit the barrier and activate the device.")
	public static NpcStringId TO_REMOVE_THE_BARRIER_YOU_MUST_FIND_THE_RELICS_THAT_FIT_THE_BARRIER_AND_ACTIVATE_THE_DEVICE;
	
	@ClientString(id = 1029553, message = "All the guardians were defeated, and the seal was removed. Teleport to the center.")
	public static NpcStringId ALL_THE_GUARDIANS_WERE_DEFEATED_AND_THE_SEAL_WAS_REMOVED_TELEPORT_TO_THE_CENTER;
	
	@ClientString(id = 1030101, message = "You've captured a Wisp successfully.")
	public static NpcStringId YOU_VE_CAPTURED_A_WISP_SUCCESSFULLY;
	
	@ClientString(id = 1032003, message = "Wait...wait a minute! I still have time!")
	public static NpcStringId WAIT_WAIT_A_MINUTE_I_STILL_HAVE_TIME;
	
	@ClientString(id = 1032103, message = "Hm? Don't just go. I still have tons to teach you.")
	public static NpcStringId HM_DON_T_JUST_GO_I_STILL_HAVE_TONS_TO_TEACH_YOU;
	
	@ClientString(id = 1032201, message = "Weapons have been added to your Inventory.")
	public static NpcStringId WEAPONS_HAVE_BEEN_ADDED_TO_YOUR_INVENTORY;
	
	@ClientString(id = 1032203, message = "There's the next training step.")
	public static NpcStringId THERE_S_THE_NEXT_TRAINING_STEP;
	
	@ClientString(id = 1032301, message = "$s1! Come with me! I will lead you to Ibane.")
	public static NpcStringId S1_COME_WITH_ME_I_WILL_LEAD_YOU_TO_IBANE;
	
	@ClientString(id = 1032302, message = "$s1! Come follow me!")
	public static NpcStringId S1_COME_FOLLOW_ME;
	
	@ClientString(id = 1032303, message = "$s1! That man in front is Ibane.")
	public static NpcStringId S1_THAT_MAN_IN_FRONT_IS_IBANE;
	
	@ClientString(id = 1032304, message = "$s1! Come with me! I will lead you to Holden.")
	public static NpcStringId S1_COME_WITH_ME_I_WILL_LEAD_YOU_TO_HOLDEN;
	
	@ClientString(id = 1032305, message = "$s1! Come follow me!")
	public static NpcStringId S1_COME_FOLLOW_ME2;
	
	@ClientString(id = 1032306, message = "$s1! That man in front is Holden.")
	public static NpcStringId S1_THAT_MAN_IN_FRONT_IS_HOLDEN;
	
	@ClientString(id = 1032307, message = "Come on~!! Creek~")
	public static NpcStringId COME_ON_CREEK;
	
	@ClientString(id = 1032308, message = "Come on~!! Creek~")
	public static NpcStringId COME_ON_CREEK2;
	
	@ClientString(id = 1032309, message = "Go Go Go~!! Creek~")
	public static NpcStringId GO_GO_GO_CREEK;
	
	@ClientString(id = 1032310, message = "Ah, I forgot to give you soulshot. Try this!")
	public static NpcStringId AH_I_FORGOT_TO_GIVE_YOU_SOULSHOT_TRY_THIS;
	
	@ClientString(id = 1032311, message = "You can use a much more powerful attack!")
	public static NpcStringId YOU_CAN_USE_A_MUCH_MORE_POWERFUL_ATTACK;
	
	@ClientString(id = 1032312, message = "Using the soulshot, defeat them faster!")
	public static NpcStringId USING_THE_SOULSHOT_DEFEAT_THEM_FASTER;
	
	@ClientString(id = 1032313, message = "It's quite a walk to Ruins of Ye Sagira…")
	public static NpcStringId IT_S_QUITE_A_WALK_TO_RUINS_OF_YE_SAGIRA;
	
	@ClientString(id = 1032314, message = "Isn't there an easier way to get there?")
	public static NpcStringId ISN_T_THERE_AN_EASIER_WAY_TO_GET_THERE;
	
	@ClientString(id = 1032315, message = "It's dangerous out there if you don't know what you're doing.")
	public static NpcStringId IT_S_DANGEROUS_OUT_THERE_IF_YOU_DON_T_KNOW_WHAT_YOU_RE_DOING;
	
	@ClientString(id = 1032316, message = "You will find Pantheon in the Museum.")
	public static NpcStringId YOU_WILL_FIND_PANTHEON_IN_THE_MUSEUM;
	
	@ClientString(id = 1032317, message = "Come browse our inventory!")
	public static NpcStringId COME_BROWSE_OUR_INVENTORY;
	
	@ClientString(id = 1032318, message = "Incredible weapons for sale!")
	public static NpcStringId INCREDIBLE_WEAPONS_FOR_SALE;
	
	@ClientString(id = 1032319, message = "View our wide variety of accessories.")
	public static NpcStringId VIEW_OUR_WIDE_VARIETY_OF_ACCESSORIES;
	
	@ClientString(id = 1032320, message = "The best weapon doesn't make you the best.")
	public static NpcStringId THE_BEST_WEAPON_DOESN_T_MAKE_YOU_THE_BEST;
	
	@ClientString(id = 1032321, message = "We buy and sell. Come take a look.")
	public static NpcStringId WE_BUY_AND_SELL_COME_TAKE_A_LOOK;
	
	@ClientString(id = 1032322, message = "Watch out! You are being attacked!")
	public static NpcStringId WATCH_OUT_YOU_ARE_BEING_ATTACKED;
	
	@ClientString(id = 1032323, message = "You'll never leave with that book!.")
	public static NpcStringId YOU_LL_NEVER_LEAVE_WITH_THAT_BOOK;
	
	@ClientString(id = 1032324, message = "Finally... I thought I was going to die waiting.")
	public static NpcStringId FINALLY_I_THOUGHT_I_WAS_GOING_TO_DIE_WAITING;
	
	@ClientString(id = 1032325, message = "Your normal attacks aren't working!")
	public static NpcStringId YOUR_NORMAL_ATTACKS_AREN_T_WORKING;
	
	@ClientString(id = 1032326, message = "Looks like only skill based attacks damage them!")
	public static NpcStringId LOOKS_LIKE_ONLY_SKILL_BASED_ATTACKS_DAMAGE_THEM;
	
	@ClientString(id = 1032327, message = "Use your skill attacks against them!")
	public static NpcStringId USE_YOUR_SKILL_ATTACKS_AGAINST_THEM;
	
	@ClientString(id = 1032328, message = "Enough of this… come at me!")
	public static NpcStringId ENOUGH_OF_THIS_COME_AT_ME;
	
	@ClientString(id = 1032329, message = "Hunting at the beach is a bad idea.")
	public static NpcStringId HUNTING_AT_THE_BEACH_IS_A_BAD_IDEA;
	
	@ClientString(id = 1032330, message = "Only the strong survive at Ruins of Ye Sagira")
	public static NpcStringId ONLY_THE_STRONG_SURVIVE_AT_RUINS_OF_YE_SAGIRA;
	
	@ClientString(id = 1032331, message = "Don't go hunting without Soulshot.")
	public static NpcStringId DON_T_GO_HUNTING_WITHOUT_SOULSHOT;
	
	@ClientString(id = 1032332, message = "Below level 75? Be sure to receive newbie buffs.")
	public static NpcStringId BELOW_LEVEL_75_BE_SURE_TO_RECEIVE_NEWBIE_BUFFS;
	
	@ClientString(id = 1032333, message = "Carry out your quests faithfully in Talking Island, and you'll get to the 1st Class Transfer in no time!")
	public static NpcStringId CARRY_OUT_YOUR_QUESTS_FAITHFULLY_IN_TALKING_ISLAND_AND_YOU_LL_GET_TO_THE_1ST_CLASS_TRANSFER_IN_NO_TIME;
	
	@ClientString(id = 1032334, message = "I see that adventurers are returning to Talking Island for the Awakening.")
	public static NpcStringId I_SEE_THAT_ADVENTURERS_ARE_RETURNING_TO_TALKING_ISLAND_FOR_THE_AWAKENING;
	
	@ClientString(id = 1032335, message = "You can see various statistics in the Museum Stats in the main menu.")
	public static NpcStringId YOU_CAN_SEE_VARIOUS_STATISTICS_IN_THE_MUSEUM_STATS_IN_THE_MAIN_MENU;
	
	@ClientString(id = 1032336, message = "When you go to the Museum, speak to Pantheon.")
	public static NpcStringId WHEN_YOU_GO_TO_THE_MUSEUM_SPEAK_TO_PANTHEON;
	
	@ClientString(id = 1032337, message = "Some folks don't know what they are doing.")
	public static NpcStringId SOME_FOLKS_DON_T_KNOW_WHAT_THEY_ARE_DOING;
	
	@ClientString(id = 1032338, message = "Don't know what to do? Look at the map.")
	public static NpcStringId DON_T_KNOW_WHAT_TO_DO_LOOK_AT_THE_MAP;
	
	@ClientString(id = 1032339, message = "Do you see a scroll icon? Go that location.")
	public static NpcStringId DO_YOU_SEE_A_SCROLL_ICON_GO_THAT_LOCATION;
	
	@ClientString(id = 1032340, message = "To your right the Administrative district and to the left is the Museum.")
	public static NpcStringId TO_YOUR_RIGHT_THE_ADMINISTRATIVE_DISTRICT_AND_TO_THE_LEFT_IS_THE_MUSEUM;
	
	@ClientString(id = 1032341, message = "When you use the teleporter, you can go to the Ruins of Ye Sagira.")
	public static NpcStringId WHEN_YOU_USE_THE_TELEPORTER_YOU_CAN_GO_TO_THE_RUINS_OF_YE_SAGIRA;
	
	@ClientString(id = 1032342, message = "Have you been to Ruins of Ye Sagira?ﾠ You have to go at least once.")
	public static NpcStringId HAVE_YOU_BEEN_TO_RUINS_OF_YE_SAGIRA_YOU_HAVE_TO_GO_AT_LEAST_ONCE;
	
	@ClientString(id = 1032343, message = "Training Ground is located straight ahead.")
	public static NpcStringId TRAINING_GROUND_IS_LOCATED_STRAIGHT_AHEAD;
	
	@ClientString(id = 1032344, message = "While training in the Training Grounds, it becomes progressively difficult.")
	public static NpcStringId WHILE_TRAINING_IN_THE_TRAINING_GROUNDS_IT_BECOMES_PROGRESSIVELY_DIFFICULT;
	
	@ClientString(id = 1032345, message = "Training Grounds access? You need to speak with Pantheon in the Museum.")
	public static NpcStringId TRAINING_GROUNDS_ACCESS_YOU_NEED_TO_SPEAK_WITH_PANTHEON_IN_THE_MUSEUM;
	
	@ClientString(id = 1032346, message = "Hey! Did you speak with Pantheon?")
	public static NpcStringId HEY_DID_YOU_SPEAK_WITH_PANTHEON;
	
	@ClientString(id = 1032347, message = "Everyone needs to meet Pantheon first before hunting.")
	public static NpcStringId EVERYONE_NEEDS_TO_MEET_PANTHEON_FIRST_BEFORE_HUNTING;
	
	@ClientString(id = 1032348, message = "When did they get in here?")
	public static NpcStringId WHEN_DID_THEY_GET_IN_HERE;
	
	@ClientString(id = 1032349, message = "Soulshot have been added to your Inventory.")
	public static NpcStringId SOULSHOT_HAVE_BEEN_ADDED_TO_YOUR_INVENTORY;
	
	@ClientString(id = 1032350, message = "Automate Soulshot as shown in the Tutorial.")
	public static NpcStringId AUTOMATE_SOULSHOT_AS_SHOWN_IN_THE_TUTORIAL;
	
	@ClientString(id = 1032351, message = "Spiritshot have been added to your Inventory.")
	public static NpcStringId SPIRITSHOT_HAVE_BEEN_ADDED_TO_YOUR_INVENTORY;
	
	@ClientString(id = 1032352, message = "Automate Spiritshot as shown in the Tutorial.")
	public static NpcStringId AUTOMATE_SPIRITSHOT_AS_SHOWN_IN_THE_TUTORIAL;
	
	@ClientString(id = 1032353, message = "Visit the Administration Office to complete your Class Transfers.")
	public static NpcStringId VISIT_THE_ADMINISTRATION_OFFICE_TO_COMPLETE_YOUR_CLASS_TRANSFERS;
	
	@ClientString(id = 1032354, message = "Improve yourself faster at the Training Grounds.")
	public static NpcStringId IMPROVE_YOURSELF_FASTER_AT_THE_TRAINING_GROUNDS;
	
	@ClientString(id = 1032355, message = "Information is collected in the Museum where can learn about the heroes.")
	public static NpcStringId INFORMATION_IS_COLLECTED_IN_THE_MUSEUM_WHERE_CAN_LEARN_ABOUT_THE_HEROES;
	
	@ClientString(id = 1032356, message = "Visit the Administration Office to form a clan.")
	public static NpcStringId VISIT_THE_ADMINISTRATION_OFFICE_TO_FORM_A_CLAN;
	
	@ClientString(id = 1032357, message = "Kakai manages the Administration Office.")
	public static NpcStringId KAKAI_MANAGES_THE_ADMINISTRATION_OFFICE;
	
	@ClientString(id = 1032358, message = "Training Grounds has underground facilities as well.")
	public static NpcStringId TRAINING_GROUNDS_HAS_UNDERGROUND_FACILITIES_AS_WELL;
	
	@ClientString(id = 1032359, message = "If you wish to use the Training Grounds, meet Shannon.")
	public static NpcStringId IF_YOU_WISH_TO_USE_THE_TRAINING_GROUNDS_MEET_SHANNON;
	
	@ClientString(id = 1032360, message = "This is Talking Island Village, right?")
	public static NpcStringId THIS_IS_TALKING_ISLAND_VILLAGE_RIGHT;
	
	@ClientString(id = 1033804, message = "Ah! Regenerator Kanilov appeared again!")
	public static NpcStringId AH_REGENERATOR_KANILOV_APPEARED_AGAIN;
	
	@ClientString(id = 1033814, message = "Ah! Regenerator Poslof appeared again!")
	public static NpcStringId AH_REGENERATOR_POSLOF_APPEARED_AGAIN;
	
	@ClientString(id = 1034101, message = "I hit things… they fall dead.")
	public static NpcStringId I_HIT_THINGS_THEY_FALL_DEAD;
	
	@ClientString(id = 1034102, message = "I can heal you during combat.")
	public static NpcStringId I_CAN_HEAL_YOU_DURING_COMBAT;
	
	@ClientString(id = 1034103, message = "My summons are not afraid of Shilen's monsters.")
	public static NpcStringId MY_SUMMONS_ARE_NOT_AFRAID_OF_SHILEN_S_MONSTERS;
	
	@ClientString(id = 1034104, message = "What do I feel when I kill Shilen's monsters? Recoil.")
	public static NpcStringId WHAT_DO_I_FEEL_WHEN_I_KILL_SHILEN_S_MONSTERS_RECOIL;
	
	@ClientString(id = 1034105, message = "Creatures Resurrected - Defend Yourself")
	public static NpcStringId CREATURES_RESURRECTED_DEFEND_YOURSELF;
	
	@ClientString(id = 1034106, message = "Creatures Resurrected - ﾠFirst Wave")
	public static NpcStringId CREATURES_RESURRECTED_FIRST_WAVE;
	
	@ClientString(id = 1034107, message = "Creatures Resurrected - Second Wave")
	public static NpcStringId CREATURES_RESURRECTED_SECOND_WAVE;
	
	@ClientString(id = 1034108, message = "Creatures Resurrected - Third Wave")
	public static NpcStringId CREATURES_RESURRECTED_THIRD_WAVE;
	
	@ClientString(id = 1034109, message = "Creatures Resurrected - Fourth Wave")
	public static NpcStringId CREATURES_RESURRECTED_FOURTH_WAVE;
	
	@ClientString(id = 1034110, message = "Creatures Resurrected - Fifth Wave")
	public static NpcStringId CREATURES_RESURRECTED_FIFTH_WAVE;
	
	@ClientString(id = 1034111, message = "Creatures Resurrected - Defend Yourself")
	public static NpcStringId CREATURES_RESURRECTED_DEFEND_YOURSELF2;
	
	@ClientString(id = 1034112, message = "Creatures Resurrected - Death Wound has been summoned")
	public static NpcStringId CREATURES_RESURRECTED_DEATH_WOUND_HAS_BEEN_SUMMONED;
	
	@ClientString(id = 1034113, message = "Creatures have stopped their attack. Rest and then speak with Adolph.")
	public static NpcStringId CREATURES_HAVE_STOPPED_THEIR_ATTACK_REST_AND_THEN_SPEAK_WITH_ADOLPH;
	
	@ClientString(id = 1034114, message = "Creatures Resurrected - Defend Yourself")
	public static NpcStringId CREATURES_RESURRECTED_DEFEND_YOURSELF3;
	
	@ClientString(id = 1034116, message = "I, Death Wound... Champion of Shilen, shall end your world.")
	public static NpcStringId I_DEATH_WOUND_CHAMPION_OF_SHILEN_SHALL_END_YOUR_WORLD;
	
	@ClientString(id = 1034117, message = "Agh... humans... ha... it does not matter, your world will end anyways.")
	public static NpcStringId AGH_HUMANS_HA_IT_DOES_NOT_MATTER_YOUR_WORLD_WILL_END_ANYWAYS;
	
	@ClientString(id = 1034118, message = "Get behind me! Get behind me!")
	public static NpcStringId GET_BEHIND_ME_GET_BEHIND_ME;
	
	@ClientString(id = 1034119, message = "The only good Shilen Creature, is a dead one.")
	public static NpcStringId THE_ONLY_GOOD_SHILEN_CREATURE_IS_A_DEAD_ONE;
	
	@ClientString(id = 1034120, message = "Did someone cry medic? Here, be healed!")
	public static NpcStringId DID_SOMEONE_CRY_MEDIC_HERE_BE_HEALED;
	
	@ClientString(id = 1034121, message = "I'm on fire! No wait, that would be you...")
	public static NpcStringId I_M_ON_FIRE_NO_WAIT_THAT_WOULD_BE_YOU;
	
	@ClientString(id = 1034122, message = "Boom! Headshot!")
	public static NpcStringId BOOM_HEADSHOT;
	
	@ClientString(id = 1034123, message = "Creatures have stopped attacking. Use this time to rest and recover!")
	public static NpcStringId CREATURES_HAVE_STOPPED_ATTACKING_USE_THIS_TIME_TO_REST_AND_RECOVER;
	
	@ClientString(id = 1034124, message = "The Cry of Destiny pendant will be helpful to you. Please equip it and bring out the power of the pendant to prepare for the next fight.")
	public static NpcStringId THE_CRY_OF_DESTINY_PENDANT_WILL_BE_HELPFUL_TO_YOU_PLEASE_EQUIP_IT_AND_BRING_OUT_THE_POWER_OF_THE_PENDANT_TO_PREPARE_FOR_THE_NEXT_FIGHT;
	
	@ClientString(id = 1034710, message = "Sacrifice begins to get absorbed into the Altar of Blood.")
	public static NpcStringId SACRIFICE_BEGINS_TO_GET_ABSORBED_INTO_THE_ALTAR_OF_BLOOD;
	
	@ClientString(id = 1034711, message = "First sacrifice got absorbed into the Altar of Blood.")
	public static NpcStringId FIRST_SACRIFICE_GOT_ABSORBED_INTO_THE_ALTAR_OF_BLOOD;
	
	@ClientString(id = 1034712, message = "Second sacrifice got absorbed into the Altar of Blood.")
	public static NpcStringId SECOND_SACRIFICE_GOT_ABSORBED_INTO_THE_ALTAR_OF_BLOOD;
	
	@ClientString(id = 1034713, message = "Third sacrifice got absorbed into the Altar of Blood.")
	public static NpcStringId THIRD_SACRIFICE_GOT_ABSORBED_INTO_THE_ALTAR_OF_BLOOD;
	
	@ClientString(id = 1034714, message = "Fourth sacrifice got absorbed into the Altar of Blood.")
	public static NpcStringId FOURTH_SACRIFICE_GOT_ABSORBED_INTO_THE_ALTAR_OF_BLOOD;
	
	@ClientString(id = 1034715, message = "Last sacrifice got absorbed into the Altar of Blood.")
	public static NpcStringId LAST_SACRIFICE_GOT_ABSORBED_INTO_THE_ALTAR_OF_BLOOD;
	
	@ClientString(id = 1034716, message = "Altar of Blood became active after absorbing all the sacrifices.")
	public static NpcStringId ALTAR_OF_BLOOD_BECAME_ACTIVE_AFTER_ABSORBING_ALL_THE_SACRIFICES;
	
	@ClientString(id = 1034717, message = "Failed to approach! Powerful energy emits from the Altar of Blood and chases the intruder away.")
	public static NpcStringId FAILED_TO_APPROACH_POWERFUL_ENERGY_EMITS_FROM_THE_ALTAR_OF_BLOOD_AND_CHASES_THE_INTRUDER_AWAY;
	
	@ClientString(id = 1036301, message = "You've shown your condolences to one corpse.")
	public static NpcStringId YOU_VE_SHOWN_YOUR_CONDOLENCES_TO_ONE_CORPSE;
	
	@ClientString(id = 1036302, message = "You've shown your condolences to a second corpse.")
	public static NpcStringId YOU_VE_SHOWN_YOUR_CONDOLENCES_TO_A_SECOND_CORPSE;
	
	@ClientString(id = 1036303, message = "You've shown your condolences to a third corpse.")
	public static NpcStringId YOU_VE_SHOWN_YOUR_CONDOLENCES_TO_A_THIRD_CORPSE;
	
	@ClientString(id = 1036304, message = "You've shown your condolences to a fourth corpse.")
	public static NpcStringId YOU_VE_SHOWN_YOUR_CONDOLENCES_TO_A_FOURTH_CORPSE;
	
	@ClientString(id = 1036305, message = "You've shown your condolences to a fifth corpse.")
	public static NpcStringId YOU_VE_SHOWN_YOUR_CONDOLENCES_TO_A_FIFTH_CORPSE;
	
	@ClientString(id = 1036344, message = "Don't toy with the dead!")
	public static NpcStringId DON_T_TOY_WITH_THE_DEAD;
	
	@ClientString(id = 1110001, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Can be used for item transportation.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_CAN_BE_USED_FOR_ITEM_TRANSPORTATION_BUTTON;
	
	@ClientString(id = 1110002, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Can help during hunting by assisting in attacks. (/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_CAN_HELP_DURING_HUNTING_BY_ASSISTING_IN_ATTACKS_BUTTON;
	
	@ClientString(id = 1110003, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')Can be sent to the village to buy items.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_CAN_BE_SENT_TO_THE_VILLAGE_TO_BUY_ITEMS_BUTTON;
	
	@ClientString(id = 1110004, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Can be traded or sold to a new owner for Adena.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_CAN_BE_TRADED_OR_SOLD_TO_A_NEW_OWNER_FOR_ADENA_BUTTON;
	
	@ClientString(id = 1110005, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON;
	
	@ClientString(id = 1110006, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')When taking down a monster, always have a pet's company.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_WHEN_TAKING_DOWN_A_MONSTER_ALWAYS_HAVE_A_PET_S_COMPANY_BUTTON;
	
	@ClientString(id = 1110007, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Tell your pet to pick up items.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_TELL_YOUR_PET_TO_PICK_UP_ITEMS_BUTTON;
	
	@ClientString(id = 1110008, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Tell your pet to attack monsters first.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_TELL_YOUR_PET_TO_ATTACK_MONSTERS_FIRST_BUTTON;
	
	@ClientString(id = 1110009, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Let your pet do what it wants.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_LET_YOUR_PET_DO_WHAT_IT_WANTS_BUTTON;
	
	@ClientString(id = 1110010, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON2;
	
	@ClientString(id = 1110011, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')10 hours(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_10_HOURS_BUTTON;
	
	@ClientString(id = 1110012, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')15 hours(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_15_HOURS_BUTTON;
	
	@ClientString(id = 1110013, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')It won't disappear.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_IT_WON_T_DISAPPEAR_BUTTON;
	
	@ClientString(id = 1110014, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')24 hours(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_24_HOURS_BUTTON;
	
	@ClientString(id = 1110015, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON3;
	
	@ClientString(id = 1110016, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')Dire Wolf(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_DIRE_WOLF_BUTTON;
	
	@ClientString(id = 1110017, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Air Wolf(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_AIR_WOLF_BUTTON;
	
	@ClientString(id = 1110018, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Turek Wolf(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_TUREK_WOLF_BUTTON;
	
	@ClientString(id = 1110019, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Kasha Wolf(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_KASHA_WOLF_BUTTON;
	
	@ClientString(id = 1110020, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON4;
	
	@ClientString(id = 1110021, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')It's tail is always pointing straight down.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_IT_S_TAIL_IS_ALWAYS_POINTING_STRAIGHT_DOWN_BUTTON;
	
	@ClientString(id = 1110022, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')It's tail is always curled up.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_IT_S_TAIL_IS_ALWAYS_CURLED_UP_BUTTON;
	
	@ClientString(id = 1110023, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')It's tail is always wagging back and forth.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_IT_S_TAIL_IS_ALWAYS_WAGGING_BACK_AND_FORTH_BUTTON;
	
	@ClientString(id = 1110024, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')What are you talking about?! A wolf doesn't have a tail.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_WHAT_ARE_YOU_TALKING_ABOUT_A_WOLF_DOESN_T_HAVE_A_TAIL_BUTTON;
	
	@ClientString(id = 1110025, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON5;
	
	@ClientString(id = 1110026, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Raccoon(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_RACCOON_BUTTON;
	
	@ClientString(id = 1110027, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Jackal(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_JACKAL_BUTTON;
	
	@ClientString(id = 1110028, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Fox(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_FOX_BUTTON;
	
	@ClientString(id = 1110029, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Shepherd Dog(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_SHEPHERD_DOG_BUTTON;
	
	@ClientString(id = 1110030, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_NONE_OF_THE_ABOVE_BUTTON;
	
	@ClientString(id = 1110031, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')1.4 km(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_1_4_KM_BUTTON;
	
	@ClientString(id = 1110032, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')2.4 km(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_2_4_KM_BUTTON;
	
	@ClientString(id = 1110033, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')3.4 km(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_3_4_KM_BUTTON;
	
	@ClientString(id = 1110034, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')4.4 km(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_4_4_KM_BUTTON;
	
	@ClientString(id = 1110035, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON6;
	
	@ClientString(id = 1110036, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')Male(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_MALE_BUTTON;
	
	@ClientString(id = 1110037, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Female(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_FEMALE_BUTTON;
	
	@ClientString(id = 1110038, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')A baby that was born last year(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_A_BABY_THAT_WAS_BORN_LAST_YEAR_BUTTON;
	
	@ClientString(id = 1110039, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')A baby that was born two years ago(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_A_BABY_THAT_WAS_BORN_TWO_YEARS_AGO_BUTTON;
	
	@ClientString(id = 1110040, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON7;
	
	@ClientString(id = 1110041, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Goat(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_GOAT_BUTTON;
	
	@ClientString(id = 1110042, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Meat of a dead animal(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_MEAT_OF_A_DEAD_ANIMAL_BUTTON;
	
	@ClientString(id = 1110043, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Berries(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_BERRIES_BUTTON;
	
	@ClientString(id = 1110044, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Wild Bird(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_WILD_BIRD_BUTTON;
	
	@ClientString(id = 1110045, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_NONE_OF_THE_ABOVE_BUTTON2;
	
	@ClientString(id = 1110046, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Breeding season is January-February.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_BREEDING_SEASON_IS_JANUARY_FEBRUARY_BUTTON;
	
	@ClientString(id = 1110047, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')Pregnancy is nine months.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_PREGNANCY_IS_NINE_MONTHS_BUTTON;
	
	@ClientString(id = 1110048, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Babies are born in April-June.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_BABIES_ARE_BORN_IN_APRIL_JUNE_BUTTON;
	
	@ClientString(id = 1110049, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Has up to ten offspring at one time.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_HAS_UP_TO_TEN_OFFSPRING_AT_ONE_TIME_BUTTON;
	
	@ClientString(id = 1110050, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON8;
	
	@ClientString(id = 1110051, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')3-6 years(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_3_6_YEARS_BUTTON;
	
	@ClientString(id = 1110052, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')6-9 years(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_6_9_YEARS_BUTTON;
	
	@ClientString(id = 1110053, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')9-12 years(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_9_12_YEARS_BUTTON;
	
	@ClientString(id = 1110054, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')12-15 years(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_12_15_YEARS_BUTTON;
	
	@ClientString(id = 1110055, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON9;
	
	@ClientString(id = 1110056, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Wolves gather and move in groups of 7-13 animals.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_WOLVES_GATHER_AND_MOVE_IN_GROUPS_OF_7_13_ANIMALS_BUTTON;
	
	@ClientString(id = 1110057, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Wolves can eat a whole calf in one sitting.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_WOLVES_CAN_EAT_A_WHOLE_CALF_IN_ONE_SITTING_BUTTON;
	
	@ClientString(id = 1110058, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')If they have water, wolves can live for 5-6 days without eating anything.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_IF_THEY_HAVE_WATER_WOLVES_CAN_LIVE_FOR_5_6_DAYS_WITHOUT_EATING_ANYTHING_BUTTON;
	
	@ClientString(id = 1110059, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')A pregnant wolf makes its home in a wide open place to have its babies.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_A_PREGNANT_WOLF_MAKES_ITS_HOME_IN_A_WIDE_OPEN_PLACE_TO_HAVE_ITS_BABIES_BUTTON;
	
	@ClientString(id = 1110060, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON10;
	
	@ClientString(id = 1110061, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')A grown wolf is still not as heavy as a fully-grown male adult human.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_A_GROWN_WOLF_IS_STILL_NOT_AS_HEAVY_AS_A_FULLY_GROWN_MALE_ADULT_HUMAN_BUTTON;
	
	@ClientString(id = 1110062, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')A wolf changes into a werewolf during a full-moon.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_A_WOLF_CHANGES_INTO_A_WEREWOLF_DURING_A_FULL_MOON_BUTTON;
	
	@ClientString(id = 1110063, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')The color of a wolf's fur is the same as the place where it lives.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_THE_COLOR_OF_A_WOLF_S_FUR_IS_THE_SAME_AS_THE_PLACE_WHERE_IT_LIVES_BUTTON;
	
	@ClientString(id = 1110064, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')A wolf enjoys eating Dwarves.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_A_WOLF_ENJOYS_EATING_DWARVES_BUTTON;
	
	@ClientString(id = 1110065, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON11;
	
	@ClientString(id = 1110066, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Talking Island - Wolf(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_TALKING_ISLAND_WOLF_BUTTON;
	
	@ClientString(id = 1110067, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Dark Forest - Ashen Wolf(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_DARK_FOREST_ASHEN_WOLF_BUTTON;
	
	@ClientString(id = 1110068, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')Elven Forest - Gray Wolf(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_ELVEN_FOREST_GRAY_WOLF_BUTTON;
	
	@ClientString(id = 1110069, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=1')Orc - Black Wolf(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_1_ORC_BLACK_WOLF_BUTTON;
	
	@ClientString(id = 1110070, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=419andreply=0')None of the above.(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_419ANDREPLY_0_NONE_OF_THE_ABOVE_BUTTON12;
	
	@ClientString(id = 1110071, message = "... is the process of standing up.")
	public static NpcStringId IS_THE_PROCESS_OF_STANDING_UP;
	
	@ClientString(id = 1110072, message = "... is the process of sitting down.")
	public static NpcStringId IS_THE_PROCESS_OF_SITTING_DOWN;
	
	@ClientString(id = 1110073, message = "It is possible to use a skill while sitting down.")
	public static NpcStringId IT_IS_POSSIBLE_TO_USE_A_SKILL_WHILE_SITTING_DOWN;
	
	@ClientString(id = 1110074, message = "...is out of range.")
	public static NpcStringId IS_OUT_OF_RANGE;
	
	@ClientString(id = 1110075, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=255andreply=3' msg='811;Monster Derby')Teleport to Monster Derby (Free)(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_255ANDREPLY_3_MSG_811_MONSTER_DERBY_TELEPORT_TO_MONSTER_DERBY_FREE_BUTTON;
	
	@ClientString(id = 1111002, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Gladiator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_GLADIATOR_BUTTON;
	
	@ClientString(id = 1111003, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Warlord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_WARLORD_BUTTON;
	
	@ClientString(id = 1111005, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Paladin(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_PALADIN_BUTTON;
	
	@ClientString(id = 1111006, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Dark Avenger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_DARK_AVENGER_BUTTON;
	
	@ClientString(id = 1111008, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Treasure Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_TREASURE_HUNTER_BUTTON;
	
	@ClientString(id = 1111009, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Hawkeye(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_HAWKEYE_BUTTON;
	
	@ClientString(id = 1111012, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Sorcerer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SORCERER_BUTTON;
	
	@ClientString(id = 1111013, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Necromancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_NECROMANCER_BUTTON;
	
	@ClientString(id = 1111014, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Warlock(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_WARLOCK_BUTTON;
	
	@ClientString(id = 1111016, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Bishop(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_BISHOP_BUTTON;
	
	@ClientString(id = 1111017, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Prophet(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_PROPHET_BUTTON;
	
	@ClientString(id = 1111020, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Temple Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_TEMPLE_KNIGHT_BUTTON;
	
	@ClientString(id = 1111021, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Swordsinger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SWORDSINGER_BUTTON;
	
	@ClientString(id = 1111023, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Plains Walker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_PLAINS_WALKER_BUTTON;
	
	@ClientString(id = 1111024, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Silver Ranger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SILVER_RANGER_BUTTON;
	
	@ClientString(id = 1111027, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Spellsinger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SPELLSINGER_BUTTON;
	
	@ClientString(id = 1111028, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Elemental Summoner(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_ELEMENTAL_SUMMONER_BUTTON;
	
	@ClientString(id = 1111030, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Elven Elder(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_ELVEN_ELDER_BUTTON;
	
	@ClientString(id = 1111033, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Shillien Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SHILLIEN_KNIGHT_BUTTON;
	
	@ClientString(id = 1111034, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Bladedancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_BLADEDANCER_BUTTON;
	
	@ClientString(id = 1111036, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Abyss Walker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_ABYSS_WALKER_BUTTON;
	
	@ClientString(id = 1111037, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Phantom Ranger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_PHANTOM_RANGER_BUTTON;
	
	@ClientString(id = 1111040, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Spellhowler(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SPELLHOWLER_BUTTON;
	
	@ClientString(id = 1111041, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Phantom Summoner(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_PHANTOM_SUMMONER_BUTTON;
	
	@ClientString(id = 1111043, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Shillien Elder(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SHILLIEN_ELDER_BUTTON;
	
	@ClientString(id = 1111046, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Destroyer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_DESTROYER_BUTTON;
	
	@ClientString(id = 1111048, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Tyrant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_TYRANT_BUTTON;
	
	@ClientString(id = 1111051, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Overlord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_OVERLORD_BUTTON;
	
	@ClientString(id = 1111052, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Warcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_WARCRYER_BUTTON;
	
	@ClientString(id = 1111055, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Bounty Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_BOUNTY_HUNTER_BUTTON;
	
	@ClientString(id = 1111057, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Warsmith(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_WARSMITH_BUTTON;
	
	@ClientString(id = 1111058, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Duelist(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_DUELIST_BUTTON;
	
	@ClientString(id = 1111059, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Dreadnought(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_DREADNOUGHT_BUTTON;
	
	@ClientString(id = 1111060, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Phoenix Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_PHOENIX_KNIGHT_BUTTON;
	
	@ClientString(id = 1111061, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Hell Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_HELL_KNIGHT_BUTTON;
	
	@ClientString(id = 1111062, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Sagittarius(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SAGITTARIUS_BUTTON;
	
	@ClientString(id = 1111063, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Adventurer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_ADVENTURER_BUTTON;
	
	@ClientString(id = 1111064, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Archmage(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_ARCHMAGE_BUTTON;
	
	@ClientString(id = 1111065, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Soultaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SOULTAKER_BUTTON;
	
	@ClientString(id = 1111066, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Arcana Lord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_ARCANA_LORD_BUTTON;
	
	@ClientString(id = 1111067, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Cardinal(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_CARDINAL_BUTTON;
	
	@ClientString(id = 1111068, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Hierophant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_HIEROPHANT_BUTTON;
	
	@ClientString(id = 1111069, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Eva's Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_EVA_S_TEMPLAR_BUTTON;
	
	@ClientString(id = 1111070, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Sword Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SWORD_MUSE_BUTTON;
	
	@ClientString(id = 1111071, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Wind Rider(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_WIND_RIDER_BUTTON;
	
	@ClientString(id = 1111072, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Moonlight Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_MOONLIGHT_SENTINEL_BUTTON;
	
	@ClientString(id = 1111073, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Mystic Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_MYSTIC_MUSE_BUTTON;
	
	@ClientString(id = 1111074, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Elemental Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_ELEMENTAL_MASTER_BUTTON;
	
	@ClientString(id = 1111075, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Eva's Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_EVA_S_SAINT_BUTTON;
	
	@ClientString(id = 1111076, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Shillien Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SHILLIEN_TEMPLAR_BUTTON;
	
	@ClientString(id = 1111077, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Spectral Dancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SPECTRAL_DANCER_BUTTON;
	
	@ClientString(id = 1111078, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Ghost Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_GHOST_HUNTER_BUTTON;
	
	@ClientString(id = 1111079, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Ghost Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_GHOST_SENTINEL_BUTTON;
	
	@ClientString(id = 1111080, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Storm Screamer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_STORM_SCREAMER_BUTTON;
	
	@ClientString(id = 1111081, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Spectral Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SPECTRAL_MASTER_BUTTON;
	
	@ClientString(id = 1111082, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Shillien Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SHILLIEN_SAINT_BUTTON;
	
	@ClientString(id = 1111083, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Titan(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_TITAN_BUTTON;
	
	@ClientString(id = 1111084, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Grand Khavatari(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_GRAND_KHAVATARI_BUTTON;
	
	@ClientString(id = 1111085, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Dominator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_DOMINATOR_BUTTON;
	
	@ClientString(id = 1111086, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Doomcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_DOOMCRYER_BUTTON;
	
	@ClientString(id = 1111087, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Fortune Seeker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_FORTUNE_SEEKER_BUTTON;
	
	@ClientString(id = 1111088, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Maestro(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_MAESTRO_BUTTON;
	
	@ClientString(id = 1112002, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Gladiator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_GLADIATOR_BUTTON;
	
	@ClientString(id = 1112003, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Warlord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_WARLORD_BUTTON;
	
	@ClientString(id = 1112005, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Paladin(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_PALADIN_BUTTON;
	
	@ClientString(id = 1112006, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Dark Avenger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_DARK_AVENGER_BUTTON;
	
	@ClientString(id = 1112008, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Treasure Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_TREASURE_HUNTER_BUTTON;
	
	@ClientString(id = 1112009, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Hawkeye(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_HAWKEYE_BUTTON;
	
	@ClientString(id = 1112012, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Sorcerer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SORCERER_BUTTON;
	
	@ClientString(id = 1112013, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Necromancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_NECROMANCER_BUTTON;
	
	@ClientString(id = 1112014, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Warlock(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_WARLOCK_BUTTON;
	
	@ClientString(id = 1112016, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Bishop(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_BISHOP_BUTTON;
	
	@ClientString(id = 1112017, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Prophet(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_PROPHET_BUTTON;
	
	@ClientString(id = 1112020, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Temple Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_TEMPLE_KNIGHT_BUTTON;
	
	@ClientString(id = 1112021, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Swordsinger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SWORDSINGER_BUTTON;
	
	@ClientString(id = 1112023, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Plains Walker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_PLAINS_WALKER_BUTTON;
	
	@ClientString(id = 1112024, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Silver Ranger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SILVER_RANGER_BUTTON;
	
	@ClientString(id = 1112027, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Spellsinger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SPELLSINGER_BUTTON;
	
	@ClientString(id = 1112028, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Elemental Summoner(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_ELEMENTAL_SUMMONER_BUTTON;
	
	@ClientString(id = 1112030, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Elven Elder(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_ELVEN_ELDER_BUTTON;
	
	@ClientString(id = 1112033, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Shillien Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SHILLIEN_KNIGHT_BUTTON;
	
	@ClientString(id = 1112034, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Bladedancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_BLADEDANCER_BUTTON;
	
	@ClientString(id = 1112036, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Abyss Walker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_ABYSS_WALKER_BUTTON;
	
	@ClientString(id = 1112037, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Phantom Ranger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_PHANTOM_RANGER_BUTTON;
	
	@ClientString(id = 1112040, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Spellhowler(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SPELLHOWLER_BUTTON;
	
	@ClientString(id = 1112041, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Phantom Summoner(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_PHANTOM_SUMMONER_BUTTON;
	
	@ClientString(id = 1112043, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Shillien Elder(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SHILLIEN_ELDER_BUTTON;
	
	@ClientString(id = 1112046, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Destroyer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_DESTROYER_BUTTON;
	
	@ClientString(id = 1112048, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Tyrant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_TYRANT_BUTTON;
	
	@ClientString(id = 1112051, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Overlord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_OVERLORD_BUTTON;
	
	@ClientString(id = 1112052, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Warcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_WARCRYER_BUTTON;
	
	@ClientString(id = 1112055, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Bounty Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_BOUNTY_HUNTER_BUTTON;
	
	@ClientString(id = 1112057, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Warsmith(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_WARSMITH_BUTTON;
	
	@ClientString(id = 1112058, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Duelist(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_DUELIST_BUTTON;
	
	@ClientString(id = 1112059, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Dreadnought(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_DREADNOUGHT_BUTTON;
	
	@ClientString(id = 1112060, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Phoenix Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_PHOENIX_KNIGHT_BUTTON;
	
	@ClientString(id = 1112061, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Hell Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_HELL_KNIGHT_BUTTON;
	
	@ClientString(id = 1112062, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Sagittarius(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SAGITTARIUS_BUTTON;
	
	@ClientString(id = 1112063, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Adventurer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_ADVENTURER_BUTTON;
	
	@ClientString(id = 1112064, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Archmage(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_ARCHMAGE_BUTTON;
	
	@ClientString(id = 1112065, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Soultaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SOULTAKER_BUTTON;
	
	@ClientString(id = 1112066, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Arcana Lord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_ARCANA_LORD_BUTTON;
	
	@ClientString(id = 1112067, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Cardinal(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_CARDINAL_BUTTON;
	
	@ClientString(id = 1112068, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Hierophant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_HIEROPHANT_BUTTON;
	
	@ClientString(id = 1112069, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Eva's Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_EVA_S_TEMPLAR_BUTTON;
	
	@ClientString(id = 1112070, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Sword Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SWORD_MUSE_BUTTON;
	
	@ClientString(id = 1112071, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Wind Rider(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_WIND_RIDER_BUTTON;
	
	@ClientString(id = 1112072, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Moonlight Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_MOONLIGHT_SENTINEL_BUTTON;
	
	@ClientString(id = 1112073, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Mystic Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_MYSTIC_MUSE_BUTTON;
	
	@ClientString(id = 1112074, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Elemental Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_ELEMENTAL_MASTER_BUTTON;
	
	@ClientString(id = 1112075, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Eva's Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_EVA_S_SAINT_BUTTON;
	
	@ClientString(id = 1112076, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Shillien Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SHILLIEN_TEMPLAR_BUTTON;
	
	@ClientString(id = 1112077, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Spectral Dancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SPECTRAL_DANCER_BUTTON;
	
	@ClientString(id = 1112078, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Ghost Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_GHOST_HUNTER_BUTTON;
	
	@ClientString(id = 1112079, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Ghost Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_GHOST_SENTINEL_BUTTON;
	
	@ClientString(id = 1112080, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Storm Screamer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_STORM_SCREAMER_BUTTON;
	
	@ClientString(id = 1112081, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Spectral Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SPECTRAL_MASTER_BUTTON;
	
	@ClientString(id = 1112082, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Shillien Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_SHILLIEN_SAINT_BUTTON;
	
	@ClientString(id = 1112083, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Titan(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_TITAN_BUTTON;
	
	@ClientString(id = 1112084, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Grand Khavatari(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_GRAND_KHAVATARI_BUTTON;
	
	@ClientString(id = 1112085, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Dominator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_DOMINATOR_BUTTON;
	
	@ClientString(id = 1112086, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Doomcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_DOOMCRYER_BUTTON;
	
	@ClientString(id = 1112087, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Fortune Seeker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_FORTUNE_SEEKER_BUTTON;
	
	@ClientString(id = 1112088, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=2')Maestro(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_2_MAESTRO_BUTTON;
	
	@ClientString(id = 1113002, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Gladiator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_GLADIATOR_BUTTON;
	
	@ClientString(id = 1113003, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Warlord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_WARLORD_BUTTON;
	
	@ClientString(id = 1113005, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Paladin(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_PALADIN_BUTTON;
	
	@ClientString(id = 1113006, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Dark Avenger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_DARK_AVENGER_BUTTON;
	
	@ClientString(id = 1113008, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Treasure Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_TREASURE_HUNTER_BUTTON;
	
	@ClientString(id = 1113009, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Hawkeye(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_HAWKEYE_BUTTON;
	
	@ClientString(id = 1113012, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Sorcerer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SORCERER_BUTTON;
	
	@ClientString(id = 1113013, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Necromancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_NECROMANCER_BUTTON;
	
	@ClientString(id = 1113014, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Warlock(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_WARLOCK_BUTTON;
	
	@ClientString(id = 1113016, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Bishop(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_BISHOP_BUTTON;
	
	@ClientString(id = 1113017, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Prophet(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_PROPHET_BUTTON;
	
	@ClientString(id = 1113020, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Temple Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_TEMPLE_KNIGHT_BUTTON;
	
	@ClientString(id = 1113021, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Swordsinger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SWORDSINGER_BUTTON;
	
	@ClientString(id = 1113023, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Plains Walker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_PLAINS_WALKER_BUTTON;
	
	@ClientString(id = 1113024, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Silver Ranger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SILVER_RANGER_BUTTON;
	
	@ClientString(id = 1113027, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Spellsinger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SPELLSINGER_BUTTON;
	
	@ClientString(id = 1113028, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Elemental Summoner(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_ELEMENTAL_SUMMONER_BUTTON;
	
	@ClientString(id = 1113030, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Elven Elder(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_ELVEN_ELDER_BUTTON;
	
	@ClientString(id = 1113033, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Shillien Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SHILLIEN_KNIGHT_BUTTON;
	
	@ClientString(id = 1113034, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Bladedancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_BLADEDANCER_BUTTON;
	
	@ClientString(id = 1113036, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Abyss Walker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_ABYSS_WALKER_BUTTON;
	
	@ClientString(id = 1113037, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Phantom Ranger(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_PHANTOM_RANGER_BUTTON;
	
	@ClientString(id = 1113040, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Spellhowler(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SPELLHOWLER_BUTTON;
	
	@ClientString(id = 1113041, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Phantom Summoner(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_PHANTOM_SUMMONER_BUTTON;
	
	@ClientString(id = 1113043, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Shillien Elder(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SHILLIEN_ELDER_BUTTON;
	
	@ClientString(id = 1113046, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Destroyer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_DESTROYER_BUTTON;
	
	@ClientString(id = 1113048, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Tyrant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_TYRANT_BUTTON;
	
	@ClientString(id = 1113051, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Overlord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_OVERLORD_BUTTON;
	
	@ClientString(id = 1113052, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Warcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_WARCRYER_BUTTON;
	
	@ClientString(id = 1113055, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Bounty Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_BOUNTY_HUNTER_BUTTON;
	
	@ClientString(id = 1113057, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Warsmith(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_WARSMITH_BUTTON;
	
	@ClientString(id = 1113058, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Duelist(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_DUELIST_BUTTON;
	
	@ClientString(id = 1113059, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Dreadnought(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_DREADNOUGHT_BUTTON;
	
	@ClientString(id = 1113060, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Phoenix Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_PHOENIX_KNIGHT_BUTTON;
	
	@ClientString(id = 1113061, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Hell Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_HELL_KNIGHT_BUTTON;
	
	@ClientString(id = 1113062, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Sagittarius(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SAGITTARIUS_BUTTON;
	
	@ClientString(id = 1113063, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Adventurer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_ADVENTURER_BUTTON;
	
	@ClientString(id = 1113064, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Archmage(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_ARCHMAGE_BUTTON;
	
	@ClientString(id = 1113065, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Soultaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SOULTAKER_BUTTON;
	
	@ClientString(id = 1113066, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Arcana Lord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_ARCANA_LORD_BUTTON;
	
	@ClientString(id = 1113067, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Cardinal(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_CARDINAL_BUTTON;
	
	@ClientString(id = 1113068, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Hierophant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_HIEROPHANT_BUTTON;
	
	@ClientString(id = 1113069, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Eva's Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_EVA_S_TEMPLAR_BUTTON;
	
	@ClientString(id = 1113070, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Sword Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SWORD_MUSE_BUTTON;
	
	@ClientString(id = 1113071, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Wind Rider(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_WIND_RIDER_BUTTON;
	
	@ClientString(id = 1113072, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Moonlight Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_MOONLIGHT_SENTINEL_BUTTON;
	
	@ClientString(id = 1113073, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Mystic Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_MYSTIC_MUSE_BUTTON;
	
	@ClientString(id = 1113074, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Elemental Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_ELEMENTAL_MASTER_BUTTON;
	
	@ClientString(id = 1113075, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Eva's Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_EVA_S_SAINT_BUTTON;
	
	@ClientString(id = 1113076, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Shillien Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SHILLIEN_TEMPLAR_BUTTON;
	
	@ClientString(id = 1113077, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Spectral Dancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SPECTRAL_DANCER_BUTTON;
	
	@ClientString(id = 1113078, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Ghost Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_GHOST_HUNTER_BUTTON;
	
	@ClientString(id = 1113079, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Ghost Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_GHOST_SENTINEL_BUTTON;
	
	@ClientString(id = 1113080, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Storm Screamer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_STORM_SCREAMER_BUTTON;
	
	@ClientString(id = 1113081, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Spectral Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SPECTRAL_MASTER_BUTTON;
	
	@ClientString(id = 1113082, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Shillien Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_SHILLIEN_SAINT_BUTTON;
	
	@ClientString(id = 1113083, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Titan(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_TITAN_BUTTON;
	
	@ClientString(id = 1113084, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Grand Khavatari(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_GRAND_KHAVATARI_BUTTON;
	
	@ClientString(id = 1113085, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Dominator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_DOMINATOR_BUTTON;
	
	@ClientString(id = 1113086, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Doomcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_DOOMCRYER_BUTTON;
	
	@ClientString(id = 1113087, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Fortune Seeker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_FORTUNE_SEEKER_BUTTON;
	
	@ClientString(id = 1113088, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=3')Maestro(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_3_MAESTRO_BUTTON;
	
	@ClientString(id = 1114088, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=380')Duelist(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_380_DUELIST_BUTTON;
	
	@ClientString(id = 1114089, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=381')Dreadnought(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_381_DREADNOUGHT_BUTTON;
	
	@ClientString(id = 1114090, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=382')Phoenix Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_382_PHOENIX_KNIGHT_BUTTON;
	
	@ClientString(id = 1114091, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=383')Hell Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_383_HELL_KNIGHT_BUTTON;
	
	@ClientString(id = 1114092, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=384')Sagittarius(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_384_SAGITTARIUS_BUTTON;
	
	@ClientString(id = 1114093, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=385')Adventurer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_385_ADVENTURER_BUTTON;
	
	@ClientString(id = 1114094, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=386')Archmage(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_386_ARCHMAGE_BUTTON;
	
	@ClientString(id = 1114095, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=387')Soultaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_387_SOULTAKER_BUTTON;
	
	@ClientString(id = 1114096, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=388')Arcana Lord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_388_ARCANA_LORD_BUTTON;
	
	@ClientString(id = 1114097, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=389')Cardinal(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_389_CARDINAL_BUTTON;
	
	@ClientString(id = 1114098, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=390')Hierophant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_390_HIEROPHANT_BUTTON;
	
	@ClientString(id = 1114099, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=391')Eva's Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_391_EVA_S_TEMPLAR_BUTTON;
	
	@ClientString(id = 1114100, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=392')Sword Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_392_SWORD_MUSE_BUTTON;
	
	@ClientString(id = 1114101, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=393')Wind Rider(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_393_WIND_RIDER_BUTTON;
	
	@ClientString(id = 1114102, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=394')Moonlight Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_394_MOONLIGHT_SENTINEL_BUTTON;
	
	@ClientString(id = 1114103, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=395')Mystic Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_395_MYSTIC_MUSE_BUTTON;
	
	@ClientString(id = 1114104, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=396')Elemental Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_396_ELEMENTAL_MASTER_BUTTON;
	
	@ClientString(id = 1114105, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=397')Eva's Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_397_EVA_S_SAINT_BUTTON;
	
	@ClientString(id = 1114106, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=398')Shillien Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_398_SHILLIEN_TEMPLAR_BUTTON;
	
	@ClientString(id = 1114107, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=399')Spectral Dancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_399_SPECTRAL_DANCER_BUTTON;
	
	@ClientString(id = 1114108, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=400')Ghost Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_400_GHOST_HUNTER_BUTTON;
	
	@ClientString(id = 1114109, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=401')Ghost Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_401_GHOST_SENTINEL_BUTTON;
	
	@ClientString(id = 1114110, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=402')Storm Screamer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_402_STORM_SCREAMER_BUTTON;
	
	@ClientString(id = 1114111, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=403')Spectral Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_403_SPECTRAL_MASTER_BUTTON;
	
	@ClientString(id = 1114112, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=404')Shillien Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_404_SHILLIEN_SAINT_BUTTON;
	
	@ClientString(id = 1114113, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=405')Titan(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_405_TITAN_BUTTON;
	
	@ClientString(id = 1114114, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=406')Grand Khavatari(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_406_GRAND_KHAVATARI_BUTTON;
	
	@ClientString(id = 1114115, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=407')Dominator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_407_DOMINATOR_BUTTON;
	
	@ClientString(id = 1114116, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=408')Doomcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_408_DOOMCRYER_BUTTON;
	
	@ClientString(id = 1114117, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=409')Fortune Seeker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_409_FORTUNE_SEEKER_BUTTON;
	
	@ClientString(id = 1114118, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=410')Maestro(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_410_MAESTRO_BUTTON;
	
	@ClientString(id = 1115127, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Berserker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_BERSERKER_BUTTON;
	
	@ClientString(id = 1115128, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Soul Breaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SOUL_BREAKER_BUTTON;
	
	@ClientString(id = 1115129, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Soul Breaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SOUL_BREAKER_BUTTON2;
	
	@ClientString(id = 1115130, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Arbalester(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_ARBALESTER_BUTTON;
	
	@ClientString(id = 1115131, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Doombringer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_DOOMBRINGER_BUTTON;
	
	@ClientString(id = 1115132, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Soul Hound(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SOUL_HOUND_BUTTON;
	
	@ClientString(id = 1115133, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Soul Hound(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_SOUL_HOUND_BUTTON2;
	
	@ClientString(id = 1115134, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=41andstate=1')Trickster(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_41ANDSTATE_1_TRICKSTER_BUTTON;
	
	@ClientString(id = 1116127, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=411')Berserker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_411_BERSERKER_BUTTON;
	
	@ClientString(id = 1116128, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=412')Soul Breaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_412_SOUL_BREAKER_BUTTON;
	
	@ClientString(id = 1116129, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=413')Soul Breaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_413_SOUL_BREAKER_BUTTON;
	
	@ClientString(id = 1116130, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=414')Arbalester(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_414_ARBALESTER_BUTTON;
	
	@ClientString(id = 1116131, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=415')Doombringer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_415_DOOMBRINGER_BUTTON;
	
	@ClientString(id = 1116132, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=416')Soul Hound(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_416_SOUL_HOUND_BUTTON;
	
	@ClientString(id = 1116133, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=417')Soul Hound(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_417_SOUL_HOUND_BUTTON;
	
	@ClientString(id = 1116134, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=418')Trickster(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_418_TRICKSTER_BUTTON;
	
	@ClientString(id = 1116135, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=419')Inspector(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_419_INSPECTOR_BUTTON;
	
	@ClientString(id = 1116136, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=-2andreply=420')Judicator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_2ANDREPLY_420_JUDICATOR_BUTTON;
	
	@ClientString(id = 1120001, message = "Greyclaw Kutus (lv23)")
	public static NpcStringId GREYCLAW_KUTUS_LV23;
	
	@ClientString(id = 1120002, message = "Turek Mercenary Captain (lv30)")
	public static NpcStringId TUREK_MERCENARY_CAPTAIN_LV30;
	
	@ClientString(id = 1120003, message = "Retreat Spider Cletu (lv42)")
	public static NpcStringId RETREAT_SPIDER_CLETU_LV42;
	
	@ClientString(id = 1120004, message = "Furious Thieles (lv55)")
	public static NpcStringId FURIOUS_THIELES_LV55;
	
	@ClientString(id = 1120005, message = "Spiteful Soul of Peasant Leader (lv50)")
	public static NpcStringId SPITEFUL_SOUL_OF_PEASANT_LEADER_LV50;
	
	@ClientString(id = 1120006, message = "The 3rd Underwater Guardian (lv60)")
	public static NpcStringId THE_3RD_UNDERWATER_GUARDIAN_LV60;
	
	@ClientString(id = 1120007, message = "Pan Dryad (lv25)")
	public static NpcStringId PAN_DRYAD_LV25;
	
	@ClientString(id = 1120008, message = "Breka Warlock Pastu (lv34)")
	public static NpcStringId BREKA_WARLOCK_PASTU_LV34;
	
	@ClientString(id = 1120009, message = "Stakato Queen Zyrnna (lv34)")
	public static NpcStringId STAKATO_QUEEN_ZYRNNA_LV34;
	
	@ClientString(id = 1120010, message = "Ketra's Commander Atis (lv49)")
	public static NpcStringId KETRA_S_COMMANDER_ATIS_LV49;
	
	@ClientString(id = 1120011, message = "Atraiban (lv53)")
	public static NpcStringId ATRAIBAN_LV53;
	
	@ClientString(id = 1120012, message = "Eva's Guardian Millenu (lv58)")
	public static NpcStringId EVA_S_GUARDIAN_MILLENU_LV58;
	
	@ClientString(id = 1120013, message = "Shilen's Messenger Cabrio (lv70)")
	public static NpcStringId SHILEN_S_MESSENGER_CABRIO_LV70;
	
	@ClientString(id = 1120014, message = "Tirak (lv28)")
	public static NpcStringId TIRAK_LV28;
	
	@ClientString(id = 1120015, message = "Remmel (lv35)")
	public static NpcStringId REMMEL_LV35;
	
	@ClientString(id = 1120016, message = "Barion (lv47)")
	public static NpcStringId BARION_LV47;
	
	@ClientString(id = 1120017, message = "Karte (lv49)")
	public static NpcStringId KARTE_LV49;
	
	@ClientString(id = 1120018, message = "Verfa (lv51)")
	public static NpcStringId VERFA_LV51;
	
	@ClientString(id = 1120019, message = "Rahha (lv65)")
	public static NpcStringId RAHHA_LV65;
	
	@ClientString(id = 1120020, message = "Kernon (lv75)")
	public static NpcStringId KERNON_LV75;
	
	@ClientString(id = 1120021, message = "Beacon of Blue Sky (lv45)")
	public static NpcStringId BEACON_OF_BLUE_SKY_LV45;
	
	@ClientString(id = 1120022, message = "Unrequited Kael (lv24)")
	public static NpcStringId UNREQUITED_KAEL_LV24;
	
	@ClientString(id = 1120023, message = "Chertuba of Great Soul (lv35)")
	public static NpcStringId CHERTUBA_OF_GREAT_SOUL_LV35;
	
	@ClientString(id = 1120024, message = "Mystic of Storm Teruk (lv40)")
	public static NpcStringId MYSTIC_OF_STORM_TERUK_LV40;
	
	@ClientString(id = 1120025, message = "Shaka, Captain of the Red Flag (lv52)")
	public static NpcStringId SHAKA_CAPTAIN_OF_THE_RED_FLAG_LV52;
	
	@ClientString(id = 1120026, message = "Enchanted Forest Watcher Ruell (lv55)")
	public static NpcStringId ENCHANTED_FOREST_WATCHER_RUELL_LV55;
	
	@ClientString(id = 1120027, message = "Bloody Priest Rudelto (lv69)")
	public static NpcStringId BLOODY_PRIEST_RUDELTO_LV69;
	
	@ClientString(id = 1120028, message = "Princess Molrang (lv25)")
	public static NpcStringId PRINCESS_MOLRANG_LV25;
	
	@ClientString(id = 1120029, message = "Cat's Eye (lv30)")
	public static NpcStringId CAT_S_EYE_LV30;
	
	@ClientString(id = 1120030, message = "Leader of Cat Gang(lv39)")
	public static NpcStringId LEADER_OF_CAT_GANG_LV39;
	
	@ClientString(id = 1120031, message = "Timak Orc Chief Ranger (lv44)")
	public static NpcStringId TIMAK_ORC_CHIEF_RANGER_LV44;
	
	@ClientString(id = 1120032, message = "Crazy Mechanic Golem (lv43)")
	public static NpcStringId CRAZY_MECHANIC_GOLEM_LV43;
	
	@ClientString(id = 1120033, message = "Soulless Wild Boar (lv59)")
	public static NpcStringId SOULLESS_WILD_BOAR_LV59;
	
	@ClientString(id = 1120034, message = "Korim (lv70)")
	public static NpcStringId KORIM_LV70;
	
	@ClientString(id = 1120035, message = "Elf Renoa (lv29)")
	public static NpcStringId ELF_RENOA_LV29;
	
	@ClientString(id = 1120036, message = "Sejarr's Servitor (lv35)")
	public static NpcStringId SEJARR_S_SERVITOR_LV35;
	
	@ClientString(id = 1120037, message = "Rotten Tree Repiro (lv44)")
	public static NpcStringId ROTTEN_TREE_REPIRO_LV44;
	
	@ClientString(id = 1120038, message = "Shacram (lv45)")
	public static NpcStringId SHACRAM_LV45;
	
	@ClientString(id = 1120039, message = "Wizard Isirr (lv55)")
	public static NpcStringId WIZARD_ISIRR_LV55;
	
	@ClientString(id = 1120040, message = "Lidia, Ghost of the Well (lv60)")
	public static NpcStringId LIDIA_GHOST_OF_THE_WELL_LV60;
	
	@ClientString(id = 1120041, message = "Cloe, Priest of Antharas (lv74)")
	public static NpcStringId CLOE_PRIEST_OF_ANTHARAS_LV74;
	
	@ClientString(id = 1120042, message = "Meana, Agent of Beres (lv30)")
	public static NpcStringId MEANA_AGENT_OF_BERES_LV30;
	
	@ClientString(id = 1120043, message = "Icarus Sample 1 (lv40)")
	public static NpcStringId ICARUS_SAMPLE_1_LV40;
	
	@ClientString(id = 1120044, message = "Guillotine, Warden of the Execution Grounds (lv35)")
	public static NpcStringId GUILLOTINE_WARDEN_OF_THE_EXECUTION_GROUNDS_LV35;
	
	@ClientString(id = 1120045, message = "Berun, Messenger of the Fairy Queen (lv50)")
	public static NpcStringId BERUN_MESSENGER_OF_THE_FAIRY_QUEEN_LV50;
	
	@ClientString(id = 1120046, message = "Hopeful Refugee Leo (lv56)")
	public static NpcStringId HOPEFUL_REFUGEE_LEO_LV56;
	
	@ClientString(id = 1120047, message = "Fierce Tiger King Angel (lv65)")
	public static NpcStringId FIERCE_TIGER_KING_ANGEL_LV65;
	
	@ClientString(id = 1120048, message = "Longhorn Golkonda (lv79)")
	public static NpcStringId LONGHORN_GOLKONDA_LV79;
	
	@ClientString(id = 1120049, message = "Langk Matriarch Rashkos (lv24)")
	public static NpcStringId LANGK_MATRIARCH_RASHKOS_LV24;
	
	@ClientString(id = 1120050, message = "Vuku Grand Priest Gharmash (lv33)")
	public static NpcStringId VUKU_GRAND_PRIEST_GHARMASH_LV33;
	
	@ClientString(id = 1120051, message = "Slaughter Lord Gato (lv50)")
	public static NpcStringId SLAUGHTER_LORD_GATO_LV50;
	
	@ClientString(id = 1120052, message = "Leto Chief Talkin (lv40)")
	public static NpcStringId LETO_CHIEF_TALKIN_LV40;
	
	@ClientString(id = 1120053, message = "Sephia, Seer of Bereth (lv55)")
	public static NpcStringId SEPHIA_SEER_OF_BERETH_LV55;
	
	@ClientString(id = 1120054, message = "Hekaton Prime (lv65)")
	public static NpcStringId HEKATON_PRIME_LV65;
	
	@ClientString(id = 1120055, message = "Shuriel, Fire of Wrath (lv78)")
	public static NpcStringId SHURIEL_FIRE_OF_WRATH_LV78;
	
	@ClientString(id = 1120056, message = "Evil Spirit Bifrons (lv21)")
	public static NpcStringId EVIL_SPIRIT_BIFRONS_LV21;
	
	@ClientString(id = 1120057, message = "Zombie Lord Crowl (lv25)")
	public static NpcStringId ZOMBIE_LORD_CROWL_LV25;
	
	@ClientString(id = 1120058, message = "Flame Lord Shadar (lv35)")
	public static NpcStringId FLAME_LORD_SHADAR_LV35;
	
	@ClientString(id = 1120059, message = "Shaman King Selu (lv40)")
	public static NpcStringId SHAMAN_KING_SELU_LV40;
	
	@ClientString(id = 1120060, message = "King Tarlk (lv48)")
	public static NpcStringId KING_TARLK_LV48;
	
	@ClientString(id = 1120061, message = "Unicorn Paniel (lv54)")
	public static NpcStringId UNICORN_PANIEL_LV54;
	
	@ClientString(id = 1120062, message = "Giant Marpanak (lv60)")
	public static NpcStringId GIANT_MARPANAK_LV60;
	
	@ClientString(id = 1120063, message = "Roaring Skylancer (lv70)")
	public static NpcStringId ROARING_SKYLANCER_LV70;
	
	@ClientString(id = 1120064, message = "Ikuntai (lv25)")
	public static NpcStringId IKUNTAI_LV25;
	
	@ClientString(id = 1120065, message = "Ragraman (lv30)")
	public static NpcStringId RAGRAMAN_LV30;
	
	@ClientString(id = 1120066, message = "Lizardmen Leader Hellion (lv38)")
	public static NpcStringId LIZARDMEN_LEADER_HELLION_LV38;
	
	@ClientString(id = 1120067, message = "Tiger King Karuta (lv45)")
	public static NpcStringId TIGER_KING_KARUTA_LV45;
	
	@ClientString(id = 1120068, message = "Black Lilly (lv55)")
	public static NpcStringId BLACK_LILLY_LV55;
	
	@ClientString(id = 1120069, message = "Karum, Guardian of the Statue of the Giant (lv60)")
	public static NpcStringId KARUM_GUARDIAN_OF_THE_STATUE_OF_THE_GIANT_LV60;
	
	@ClientString(id = 1120070, message = "Demon Kuri (lv59)")
	public static NpcStringId DEMON_KURI_LV59;
	
	@ClientString(id = 1120071, message = "Tasaba Patriarch Hellena (lv35)")
	public static NpcStringId TASABA_PATRIARCH_HELLENA_LV35;
	
	@ClientString(id = 1120072, message = "Apepi (lv30)")
	public static NpcStringId APEPI_LV30;
	
	@ClientString(id = 1120073, message = "Cronos' Servitor Mumu (lv34)")
	public static NpcStringId CRONOS_SERVITOR_MUMU_LV34;
	
	@ClientString(id = 1120074, message = "Earth Protector Panathen (lv43)")
	public static NpcStringId EARTH_PROTECTOR_PANATHEN_LV43;
	
	@ClientString(id = 1120075, message = "Fafurion's Messenger Loch Ness (lv70)")
	public static NpcStringId FAFURION_S_MESSENGER_LOCH_NESS_LV70;
	
	@ClientString(id = 1120076, message = "Fafurion's Seer Sheshark (lv72)")
	public static NpcStringId FAFURION_S_SEER_SHESHARK_LV72;
	
	@ClientString(id = 1120077, message = "Crokian Padisha Sobekk (lv74)")
	public static NpcStringId CROKIAN_PADISHA_SOBEKK_LV74;
	
	@ClientString(id = 1120078, message = "Ocean's Flame Ashakiel (lv76)")
	public static NpcStringId OCEAN_S_FLAME_ASHAKIEL_LV76;
	
	@ClientString(id = 1120079, message = "Water Couatl Ateka (lv40)")
	public static NpcStringId WATER_COUATL_ATEKA_LV40;
	
	@ClientString(id = 1120080, message = "Sebek (lv36)")
	public static NpcStringId SEBEK_LV36;
	
	@ClientString(id = 1120081, message = "Fafurion's Page Sika (lv40)")
	public static NpcStringId FAFURION_S_PAGE_SIKA_LV40;
	
	@ClientString(id = 1120082, message = "Cursed Clara (lv50)")
	public static NpcStringId CURSED_CLARA_LV50;
	
	@ClientString(id = 1120083, message = "Death Lord Hallate (lv73)")
	public static NpcStringId DEATH_LORD_HALLATE_LV73;
	
	@ClientString(id = 1120084, message = "Soul Collector Acheron (lv35)")
	public static NpcStringId SOUL_COLLECTOR_ACHERON_LV35;
	
	@ClientString(id = 1120085, message = "Roaring Lord Kastor (lv62)")
	public static NpcStringId ROARING_LORD_KASTOR_LV62;
	
	@ClientString(id = 1120086, message = "Storm Winged Naga (lv75)")
	public static NpcStringId STORM_WINGED_NAGA_LV75;
	
	@ClientString(id = 1120087, message = "Ragoth, Seer of Timak (lv57)")
	public static NpcStringId RAGOTH_SEER_OF_TIMAK_LV57;
	
	@ClientString(id = 1120088, message = "Spiteful Soul of Andras the Betrayer (lv69)")
	public static NpcStringId SPITEFUL_SOUL_OF_ANDRAS_THE_BETRAYER_LV69;
	
	@ClientString(id = 1120089, message = "Ancient Drake (lv60)")
	public static NpcStringId ANCIENT_DRAKE_LV60;
	
	@ClientString(id = 1120090, message = "Vanor Chief Kandra (lv72)")
	public static NpcStringId VANOR_CHIEF_KANDRA_LV72;
	
	@ClientString(id = 1120091, message = "Abyss Brukunt (lv59)")
	public static NpcStringId ABYSS_BRUKUNT_LV59;
	
	@ClientString(id = 1120092, message = "Harit Hero Tamash (lv55)")
	public static NpcStringId HARIT_HERO_TAMASH_LV55;
	
	@ClientString(id = 1120093, message = "Last Lesser Giant Olkuth (lv75)")
	public static NpcStringId LAST_LESSER_GIANT_OLKUTH_LV75;
	
	@ClientString(id = 1120094, message = "Last Lesser Giant Glaki (lv78)")
	public static NpcStringId LAST_LESSER_GIANT_GLAKI_LV78;
	
	@ClientString(id = 1120095, message = "Doom Blade Thanatos (lv72)")
	public static NpcStringId DOOM_BLADE_THANATOS_LV72;
	
	@ClientString(id = 1120096, message = "Palatanos of the Fearsome Power (lv75)")
	public static NpcStringId PALATANOS_OF_THE_FEARSOME_POWER_LV75;
	
	@ClientString(id = 1120097, message = "Palibati Queen Themis (lv70)")
	public static NpcStringId PALIBATI_QUEEN_THEMIS_LV70;
	
	@ClientString(id = 1120098, message = "Gargoyle Lord Tiphon (lv65)")
	public static NpcStringId GARGOYLE_LORD_TIPHON_LV65;
	
	@ClientString(id = 1120099, message = "Taik High Prefect Arak (lv60)")
	public static NpcStringId TAIK_HIGH_PREFECT_ARAK_LV60;
	
	@ClientString(id = 1120100, message = "Zaken's Butcher Krantz (lv55)")
	public static NpcStringId ZAKEN_S_BUTCHER_KRANTZ_LV55;
	
	@ClientString(id = 1120101, message = "Iron Giant Totem (lv45)")
	public static NpcStringId IRON_GIANT_TOTEM_LV45;
	
	@ClientString(id = 1120102, message = "Kernon's Faithful Servant Kelone (lv67)")
	public static NpcStringId KERNON_S_FAITHFUL_SERVANT_KELONE_LV67;
	
	@ClientString(id = 1120103, message = "Bloody Empress Decarbia (lv75)")
	public static NpcStringId BLOODY_EMPRESS_DECARBIA_LV75;
	
	@ClientString(id = 1120104, message = "Beast Lord Behemoth (lv70)")
	public static NpcStringId BEAST_LORD_BEHEMOTH_LV70;
	
	@ClientString(id = 1120105, message = "Partisan Leader Talakin (lv28)")
	public static NpcStringId PARTISAN_LEADER_TALAKIN_LV28;
	
	@ClientString(id = 1120106, message = "Carnamakos (lv50)")
	public static NpcStringId CARNAMAKOS_LV50;
	
	@ClientString(id = 1120107, message = "Death Lord Ipos (lv75)")
	public static NpcStringId DEATH_LORD_IPOS_LV75;
	
	@ClientString(id = 1120108, message = "Lilith' Witch Marilion (lv50)")
	public static NpcStringId LILITH_WITCH_MARILION_LV50;
	
	@ClientString(id = 1120109, message = "Pagan Watcher Cerberon (lv55)")
	public static NpcStringId PAGAN_WATCHER_CERBERON_LV55;
	
	@ClientString(id = 1120110, message = "Anakim's Nemesis Zakaron(lv70)")
	public static NpcStringId ANAKIM_S_NEMESIS_ZAKARON_LV70;
	
	@ClientString(id = 1120111, message = "Death Lord Shax (lv75)")
	public static NpcStringId DEATH_LORD_SHAX_LV75;
	
	@ClientString(id = 1120112, message = "Hestia, Guardian Deity of the Hot Springs (lv78)")
	public static NpcStringId HESTIA_GUARDIAN_DEITY_OF_THE_HOT_SPRINGS_LV78;
	
	@ClientString(id = 1120113, message = "Ketra's Hero Hekaton (lv80)")
	public static NpcStringId KETRA_S_HERO_HEKATON_LV80;
	
	@ClientString(id = 1120114, message = "Ketra's Commander Tayr (lv80)")
	public static NpcStringId KETRA_S_COMMANDER_TAYR_LV80;
	
	@ClientString(id = 1120115, message = "Ketra's Chief Braki (lv80)")
	public static NpcStringId KETRA_S_CHIEF_BRAKI_LV80;
	
	@ClientString(id = 1120116, message = "Varka's Hero Shadith (lv80)")
	public static NpcStringId VARKA_S_HERO_SHADITH_LV80;
	
	@ClientString(id = 1120117, message = "Varka's Commander Mos (lv80)")
	public static NpcStringId VARKA_S_COMMANDER_MOS_LV80;
	
	@ClientString(id = 1120118, message = "Varka's Chief Horus (lv80)")
	public static NpcStringId VARKA_S_CHIEF_HORUS_LV80;
	
	@ClientString(id = 1120119, message = "Ember (lv80)")
	public static NpcStringId EMBER_LV80;
	
	@ClientString(id = 1120120, message = "Demon's Agent Falston (lv66)")
	public static NpcStringId DEMON_S_AGENT_FALSTON_LV66;
	
	@ClientString(id = 1120121, message = "Barakiel, the Flame of Splendor (lv70)")
	public static NpcStringId BARAKIEL_THE_FLAME_OF_SPLENDOR_LV70;
	
	@ClientString(id = 1120122, message = "Eilhalder Von Hellmann (lv71)")
	public static NpcStringId EILHALDER_VON_HELLMANN_LV71;
	
	@ClientString(id = 1120123, message = "Giant Wastelands Basilisk (lv30)")
	public static NpcStringId GIANT_WASTELANDS_BASILISK_LV30;
	
	@ClientString(id = 1120124, message = "Gargoyle Lord Sirocco (lv35)")
	public static NpcStringId GARGOYLE_LORD_SIROCCO_LV35;
	
	@ClientString(id = 1120125, message = "Sukar Wererat Chief (lv21)")
	public static NpcStringId SUKAR_WERERAT_CHIEF_LV21;
	
	@ClientString(id = 1120126, message = "Tiger Hornet (lv26)")
	public static NpcStringId TIGER_HORNET_LV26;
	
	@ClientString(id = 1120127, message = "Tracker Captain Sharuk (lv23)")
	public static NpcStringId TRACKER_CAPTAIN_SHARUK_LV23;
	
	@ClientString(id = 1120128, message = "Patriarch Kuroboros (lv26)")
	public static NpcStringId PATRIARCH_KUROBOROS_LV26;
	
	@ClientString(id = 1120129, message = "Kuroboros' Priest (lv23)")
	public static NpcStringId KUROBOROS_PRIEST_LV23;
	
	@ClientString(id = 1120130, message = "Soul Scavenger (lv25)")
	public static NpcStringId SOUL_SCAVENGER_LV25;
	
	@ClientString(id = 1120131, message = "Discarded Guardian (lv20)")
	public static NpcStringId DISCARDED_GUARDIAN_LV20;
	
	@ClientString(id = 1120132, message = "Malex, Herald of Dagoniel  (lv21)")
	public static NpcStringId MALEX_HERALD_OF_DAGONIEL_LV21;
	
	@ClientString(id = 1120133, message = "Zombie Lord Ferkel (lv20)")
	public static NpcStringId ZOMBIE_LORD_FERKEL_LV20;
	
	@ClientString(id = 1120134, message = "Madness Beast (lv20)")
	public static NpcStringId MADNESS_BEAST_LV20;
	
	@ClientString(id = 1120135, message = "Kaysha, Herald of Icarus (lv21)")
	public static NpcStringId KAYSHA_HERALD_OF_ICARUS_LV21;
	
	@ClientString(id = 1120136, message = "Revenant of Sir Calibus (lv34)")
	public static NpcStringId REVENANT_OF_SIR_CALIBUS_LV34;
	
	@ClientString(id = 1120137, message = "Evil Spirit Tempest (lv36)")
	public static NpcStringId EVIL_SPIRIT_TEMPEST_LV36;
	
	@ClientString(id = 1120138, message = "Red Eye Captain Trakia (lv35)")
	public static NpcStringId RED_EYE_CAPTAIN_TRAKIA_LV35;
	
	@ClientString(id = 1120139, message = "Nurka's Messenger (lv33)")
	public static NpcStringId NURKA_S_MESSENGER_LV33;
	
	@ClientString(id = 1120140, message = "Captain of the Queen's Royal Guard (lv32)")
	public static NpcStringId CAPTAIN_OF_THE_QUEEN_S_ROYAL_GUARD_LV32;
	
	@ClientString(id = 1120141, message = "Premo Prime (lv38)")
	public static NpcStringId PREMO_PRIME_LV38;
	
	@ClientString(id = 1120142, message = "Archon Susceptor (lv45)")
	public static NpcStringId ARCHON_SUSCEPTOR_LV45;
	
	@ClientString(id = 1120143, message = "Eye of Beleth (lv35)")
	public static NpcStringId EYE_OF_BELETH_LV35;
	
	@ClientString(id = 1120144, message = "Skyla (lv32)")
	public static NpcStringId SKYLA_LV32;
	
	@ClientString(id = 1120145, message = "Corsair Captain Kylon (lv33)")
	public static NpcStringId CORSAIR_CAPTAIN_KYLON_LV33;
	
	@ClientString(id = 1120146, message = "Lord Ishka  (lv60)")
	public static NpcStringId LORD_ISHKA_LV60;
	
	@ClientString(id = 1120147, message = "Road Scavenger Leader (lv40)")
	public static NpcStringId ROAD_SCAVENGER_LEADER_LV40;
	
	@ClientString(id = 1120148, message = "Necrosentinel Royal Guard (lv47)")
	public static NpcStringId NECROSENTINEL_ROYAL_GUARD_LV47;
	
	@ClientString(id = 1120149, message = "Nakondas (lv40)")
	public static NpcStringId NAKONDAS_LV40;
	
	@ClientString(id = 1120150, message = "Dread Avenger Kraven (lv44)")
	public static NpcStringId DREAD_AVENGER_KRAVEN_LV44;
	
	@ClientString(id = 1120151, message = "Orfen's Handmaiden (lv48)")
	public static NpcStringId ORFEN_S_HANDMAIDEN_LV48;
	
	@ClientString(id = 1120152, message = "Fairy Queen Timiniel (lv61)")
	public static NpcStringId FAIRY_QUEEN_TIMINIEL_LV61;
	
	@ClientString(id = 1120153, message = "Freki, Betrayer of Urutu (lv25)")
	public static NpcStringId FREKI_BETRAYER_OF_URUTU_LV25;
	
	@ClientString(id = 1120154, message = "Mammon's Collector Talos (lv25)")
	public static NpcStringId MAMMON_S_COLLECTOR_TALOS_LV25;
	
	@ClientString(id = 1120155, message = "Flame Stone Golem (lv44)")
	public static NpcStringId FLAME_STONE_GOLEM_LV44;
	
	@ClientString(id = 1120156, message = "Bandit Leader Barda (lv55)")
	public static NpcStringId BANDIT_LEADER_BARDA_LV55;
	
	@ClientString(id = 1120157, message = "Timak Orc Gosmos (lv45)")
	public static NpcStringId TIMAK_ORC_GOSMOS_LV45;
	
	@ClientString(id = 1120158, message = "Thief Kelbar (lv44)")
	public static NpcStringId THIEF_KELBAR_LV44;
	
	@ClientString(id = 1120159, message = "Evil Spirit Cyrion (lv45)")
	public static NpcStringId EVIL_SPIRIT_CYRION_LV45;
	
	@ClientString(id = 1120160, message = "Enmity Ghost Ramdal (lv65)")
	public static NpcStringId ENMITY_GHOST_RAMDAL_LV65;
	
	@ClientString(id = 1120161, message = "Immortal Savior Mardil (lv71)")
	public static NpcStringId IMMORTAL_SAVIOR_MARDIL_LV71;
	
	@ClientString(id = 1120162, message = "Cherub Galaxia (lv79)")
	public static NpcStringId CHERUB_GALAXIA_LV79;
	
	@ClientString(id = 1120163, message = "Minas Anor (lv70)")
	public static NpcStringId MINAS_ANOR_LV70;
	
	@ClientString(id = 1120164, message = "Mirror of Oblivion (lv49)")
	public static NpcStringId MIRROR_OF_OBLIVION_LV49;
	
	@ClientString(id = 1120165, message = "Deadman Ereve (lv51)")
	public static NpcStringId DEADMAN_EREVE_LV51;
	
	@ClientString(id = 1120166, message = "Harit Guardian Garangky (lv56)")
	public static NpcStringId HARIT_GUARDIAN_GARANGKY_LV56;
	
	@ClientString(id = 1120167, message = "Gorgolos (lv64)")
	public static NpcStringId GORGOLOS_LV64;
	
	@ClientString(id = 1120168, message = "Utenus, the Last Lesser Giant (lv66)")
	public static NpcStringId UTENUS_THE_LAST_LESSER_GIANT_LV66;
	
	@ClientString(id = 1120169, message = "Grave Robber Kim (lv52)")
	public static NpcStringId GRAVE_ROBBER_KIM_LV52;
	
	@ClientString(id = 1120170, message = "Ghost Knight Kabed (lv55)")
	public static NpcStringId GHOST_KNIGHT_KABED_LV55;
	
	@ClientString(id = 1120171, message = "Hisilrome, Shilen's Priest (lv65)")
	public static NpcStringId HISILROME_SHILEN_S_PRIEST_LV65;
	
	@ClientString(id = 1120172, message = "Magus Kenishee (lv53)")
	public static NpcStringId MAGUS_KENISHEE_LV53;
	
	@ClientString(id = 1120173, message = "Zaken's Mate Tillion (lv50)")
	public static NpcStringId ZAKEN_S_MATE_TILLION_LV50;
	
	@ClientString(id = 1120174, message = "Water Spirit Lian (lv40)")
	public static NpcStringId WATER_SPIRIT_LIAN_LV40;
	
	@ClientString(id = 1120175, message = "Gwindorr (lv40)")
	public static NpcStringId GWINDORR_LV40;
	
	@ClientString(id = 1120176, message = "Eva's Spirit Niniel (lv55)")
	public static NpcStringId EVA_S_SPIRIT_NINIEL_LV55;
	
	@ClientString(id = 1120177, message = "Fafurion's Envoy Pingolpin (lv52)")
	public static NpcStringId FAFURION_S_ENVOY_PINGOLPIN_LV52;
	
	@ClientString(id = 1120178, message = "Fafurion's Henchman Istary (lv45)")
	public static NpcStringId FAFURION_S_HENCHMAN_ISTARY_LV45;
	
	@ClientString(id = 1120179, message = "Grave Robber Boss Akata (lv30)")
	public static NpcStringId GRAVE_ROBBER_BOSS_AKATA_LV30;
	
	@ClientString(id = 1120180, message = "Nellis' Vengeful Spirit (lv39)")
	public static NpcStringId NELLIS_VENGEFUL_SPIRIT_LV39;
	
	@ClientString(id = 1120181, message = "Rayito the Looter (lv37)")
	public static NpcStringId RAYITO_THE_LOOTER_LV37;
	
	@ClientString(id = 1120182, message = "Dark Shaman Varangka (lv53)")
	public static NpcStringId DARK_SHAMAN_VARANGKA_LV53;
	
	@ClientString(id = 1120183, message = "Gigantic Chaos Golem (lv52)")
	public static NpcStringId GIGANTIC_CHAOS_GOLEM_LV52;
	
	@ClientString(id = 1120184, message = "Captain of the Ice Queen's Royal Guard (lv59)")
	public static NpcStringId CAPTAIN_OF_THE_ICE_QUEEN_S_ROYAL_GUARD_LV59;
	
	@ClientString(id = 1120185, message = "Spiked Stakato Queen Shyeed (lv80)")
	public static NpcStringId SPIKED_STAKATO_QUEEN_SHYEED_LV80;
	
	@ClientString(id = 1120186, message = "Master Anays (lv80)")
	public static NpcStringId MASTER_ANAYS_LV80;
	
	@ClientString(id = 1120187, message = "High Priest Andreas Van Halter (lv80)")
	public static NpcStringId HIGH_PRIEST_ANDREAS_VAN_HALTER_LV80;
	
	@ClientString(id = 1120188, message = "Plague Golem (lv73)")
	public static NpcStringId PLAGUE_GOLEM_LV73;
	
	@ClientString(id = 1120189, message = "Flamestone Giant (lv76)")
	public static NpcStringId FLAMESTONE_GIANT_LV76;
	
	@ClientString(id = 1120190, message = "Lilith (Lv. 89)")
	public static NpcStringId LILITH_LV_89;
	
	@ClientString(id = 1120191, message = "Anakim (Lv. 94)")
	public static NpcStringId ANAKIM_LV_94;
	
	@ClientString(id = 1120192, message = "Ice Fairy Sirra (lv60)")
	public static NpcStringId ICE_FAIRY_SIRRA_LV60;
	
	@ClientString(id = 1120193, message = "Icicle Emperor Bumbalump (lv74)")
	public static NpcStringId ICICLE_EMPEROR_BUMBALUMP_LV74;
	
	@ClientString(id = 1120194, message = "Daimon the White-eyed (lv78)")
	public static NpcStringId DAIMON_THE_WHITE_EYED_LV78;
	
	@ClientString(id = 1120195, message = "Fire Spirit Nastron (lv80)")
	public static NpcStringId FIRE_SPIRIT_NASTRON_LV80;
	
	@ClientString(id = 1120196, message = "Water Spirit Ashutar (lv80)")
	public static NpcStringId WATER_SPIRIT_ASHUTAR_LV80;
	
	@ClientString(id = 1120197, message = "Uruka (lv80)")
	public static NpcStringId URUKA_LV80;
	
	@ClientString(id = 1120198, message = "Sailren (lv80)")
	public static NpcStringId SAILREN_LV80;
	
	@ClientString(id = 1120199, message = "Typhoon (lv83)")
	public static NpcStringId TYPHOON_LV83;
	
	@ClientString(id = 1120200, message = "Valdstone (lv80)")
	public static NpcStringId VALDSTONE_LV80;
	
	@ClientString(id = 1120201, message = "Rok (lv83)")
	public static NpcStringId ROK_LV83;
	
	@ClientString(id = 1120202, message = "Enira (lv85)")
	public static NpcStringId ENIRA_LV85;
	
	@ClientString(id = 1120203, message = "Dius (lv85)")
	public static NpcStringId DIUS_LV85;
	
	@ClientString(id = 1120204, message = "Yehan Klodekus (lv81)")
	public static NpcStringId YEHAN_KLODEKUS_LV81;
	
	@ClientString(id = 1120205, message = "Yehan Klanikus (lv81)")
	public static NpcStringId YEHAN_KLANIKUS_LV81;
	
	@ClientString(id = 1120206, message = "Cannibalistic Stakato Captain (lv84)")
	public static NpcStringId CANNIBALISTIC_STAKATO_CAPTAIN_LV84;
	
	@ClientString(id = 1120207, message = "Spike Stakato Queen Shyeed (lv84)")
	public static NpcStringId SPIKE_STAKATO_QUEEN_SHYEED_LV84;
	
	@ClientString(id = 1120208, message = "Gwindorr (lv83)")
	public static NpcStringId GWINDORR_LV83;
	
	@ClientString(id = 1120209, message = "Water Spirit (lv84)")
	public static NpcStringId WATER_SPIRIT_LV84;
	
	@ClientString(id = 1120210, message = "Giant Marpanak (lv82)")
	public static NpcStringId GIANT_MARPANAK_LV82;
	
	@ClientString(id = 1120211, message = "Gorgolos (lv82)")
	public static NpcStringId GORGOLOS_LV82;
	
	@ClientString(id = 1120212, message = "Utenus, the Last Lesser Giant (lv83)")
	public static NpcStringId UTENUS_THE_LAST_LESSER_GIANT_LV83;
	
	@ClientString(id = 1120213, message = "Hekaton Prime (lv83)")
	public static NpcStringId HEKATON_PRIME_LV83;
	
	@ClientString(id = 1120214, message = "Aenkinel (Lv. 81)")
	public static NpcStringId AENKINEL_LV_81;
	
	@ClientString(id = 1120215, message = "Aenkinel (Lv. 81)")
	public static NpcStringId AENKINEL_LV_812;
	
	@ClientString(id = 1120216, message = "Aenkinel (Lv. 81)")
	public static NpcStringId AENKINEL_LV_813;
	
	@ClientString(id = 1120217, message = "Aenkinel (Lv. 81)")
	public static NpcStringId AENKINEL_LV_814;
	
	@ClientString(id = 1120218, message = "Aenkinel (Lv. 82)")
	public static NpcStringId AENKINEL_LV_82;
	
	@ClientString(id = 1120219, message = "Aenkinel (Lv. 84)")
	public static NpcStringId AENKINEL_LV_84;
	
	@ClientString(id = 1120220, message = "Tacrakahn (Lv. 89)")
	public static NpcStringId TACRAKAHN_LV_89;
	
	@ClientString(id = 1120221, message = "Torumba (Lv. 89)")
	public static NpcStringId TORUMBA_LV_89;
	
	@ClientString(id = 1120222, message = "Dopagen (Lv. 89)")
	public static NpcStringId DOPAGEN_LV_89;
	
	@ClientString(id = 1120223, message = "Glakias (Lv. 85)")
	public static NpcStringId GLAKIAS_LV_85;
	
	@ClientString(id = 1120224, message = "Glakias (Lv. 88)")
	public static NpcStringId GLAKIAS_LV_88;
	
	@ClientString(id = 1120225, message = "Anais (Lv. 84)")
	public static NpcStringId ANAIS_LV_84;
	
	@ClientString(id = 1120226, message = "Gigantic Golem (Lv. 76)")
	public static NpcStringId GIGANTIC_GOLEM_LV_76;
	
	@ClientString(id = 1120227, message = "Lost Warden (Lv. 83)")
	public static NpcStringId LOST_WARDEN_LV_83;
	
	@ClientString(id = 1120228, message = "Lost Guard Captain (Lv. 83)")
	public static NpcStringId LOST_GUARD_CAPTAIN_LV_83;
	
	@ClientString(id = 1120229, message = "Darnel (Lv. 84)")
	public static NpcStringId DARNEL_LV_84;
	
	@ClientString(id = 1120230, message = "Kechi (Lv. 84)")
	public static NpcStringId KECHI_LV_84;
	
	@ClientString(id = 1120231, message = "Tears (Lv. 84)")
	public static NpcStringId TEARS_LV_84;
	
	@ClientString(id = 1120232, message = "Emerald Horn (Lv. 84)")
	public static NpcStringId EMERALD_HORN_LV_84;
	
	@ClientString(id = 1120233, message = "Dust Rider (Lv. 84)")
	public static NpcStringId DUST_RIDER_LV_84;
	
	@ClientString(id = 1120234, message = "Bleeding Fly (Lv. 84)")
	public static NpcStringId BLEEDING_FLY_LV_84;
	
	@ClientString(id = 1120235, message = "Blackdagger Wing (Lv. 84)")
	public static NpcStringId BLACKDAGGER_WING_LV_84;
	
	@ClientString(id = 1120236, message = "Shadow Summoner (Lv. 84)")
	public static NpcStringId SHADOW_SUMMONER_LV_84;
	
	@ClientString(id = 1120237, message = "Spike Slasher (Lv. 84)")
	public static NpcStringId SPIKE_SLASHER_LV_84;
	
	@ClientString(id = 1120238, message = "Muscle Bomber (Lv. 84)")
	public static NpcStringId MUSCLE_BOMBER_LV_84;
	
	@ClientString(id = 1120239, message = "Drake Lord (Lv. 85)")
	public static NpcStringId DRAKE_LORD_LV_85;
	
	@ClientString(id = 1120240, message = "Behemoth Leader (Lv. 85)")
	public static NpcStringId BEHEMOTH_LEADER_LV_85;
	
	@ClientString(id = 1120241, message = "Dragon Beast (Lv. 85)")
	public static NpcStringId DRAGON_BEAST_LV_85;
	
	@ClientString(id = 1120242, message = "Dragon Beast (Lv. 85)")
	public static NpcStringId DRAGON_BEAST_LV_852;
	
	@ClientString(id = 1120243, message = "Kimerian (Lv. 92)")
	public static NpcStringId KIMERIAN_LV_92;
	
	@ClientString(id = 1120244, message = "Kimerian (Lv. 90)")
	public static NpcStringId KIMERIAN_LV_90;
	
	@ClientString(id = 1120245, message = "Kimerian (Lv. 95)")
	public static NpcStringId KIMERIAN_LV_95;
	
	@ClientString(id = 1120246, message = "Turanclass (Lv. 98)")
	public static NpcStringId TURANCLASS_LV_98;
	
	@ClientString(id = 1120247, message = "Georgios (Lv. 97)")
	public static NpcStringId GEORGIOS_LV_97;
	
	@ClientString(id = 1120248, message = "Angelos (Lv. 97)")
	public static NpcStringId ANGELOS_LV_97;
	
	@ClientString(id = 1120249, message = "Theofanis (Lv. 97)")
	public static NpcStringId THEOFANIS_LV_97;
	
	@ClientString(id = 1120250, message = "Steregos (Lv. 97)")
	public static NpcStringId STEREGOS_LV_97;
	
	@ClientString(id = 1120251, message = "Talicrome (Lv. 98)")
	public static NpcStringId TALICROME_LV_98;
	
	@ClientString(id = 1120252, message = "Meikaliya (Lv. 97)")
	public static NpcStringId MEIKALIYA_LV_97;
	
	@ClientString(id = 1120253, message = "Evangelos (Lv. 97)")
	public static NpcStringId EVANGELOS_LV_97;
	
	@ClientString(id = 1120254, message = "Sotiris (Lv. 97)")
	public static NpcStringId SOTIRIS_LV_97;
	
	@ClientString(id = 1120255, message = "Lazaros (Lv. 97)")
	public static NpcStringId LAZAROS_LV_97;
	
	@ClientString(id = 1120256, message = "Harnak's Wraith (Lv. 87)")
	public static NpcStringId HARNAK_S_WRAITH_LV_87;
	
	@ClientString(id = 1120257, message = "Harnak's Wraith (Lv. 88)")
	public static NpcStringId HARNAK_S_WRAITH_LV_88;
	
	@ClientString(id = 1120258, message = "Apherus (Lv. 94)")
	public static NpcStringId APHERUS_LV_94;
	
	@ClientString(id = 1120259, message = "Spezion (Lv. 94)")
	public static NpcStringId SPEZION_LV_94;
	
	@ClientString(id = 1120260, message = "Eyesaroch (Lv. 20)")
	public static NpcStringId EYESAROCH_LV_20;
	
	@ClientString(id = 1120261, message = "Overseer (Lv. 10)")
	public static NpcStringId OVERSEER_LV_10;
	
	@ClientString(id = 1120262, message = "Teredor (Lv. 88)")
	public static NpcStringId TEREDOR_LV_88;
	
	@ClientString(id = 1120263, message = "Lachikios (Lv. 87)")
	public static NpcStringId LACHIKIOS_LV_87;
	
	@ClientString(id = 1120264, message = "Lachikios (Lv. 87)")
	public static NpcStringId LACHIKIOS_LV_872;
	
	@ClientString(id = 1120265, message = "Lachikios (Lv. 89)")
	public static NpcStringId LACHIKIOS_LV_89;
	
	@ClientString(id = 1120266, message = "Lachikios (Lv. 89)")
	public static NpcStringId LACHIKIOS_LV_892;
	
	@ClientString(id = 1120267, message = "Rachicues (Lv. 88)")
	public static NpcStringId RACHICUES_LV_88;
	
	@ClientString(id = 1120268, message = "Rachicues (Lv. 88)")
	public static NpcStringId RACHICUES_LV_882;
	
	@ClientString(id = 1120269, message = "Rachicues (Lv. 90)")
	public static NpcStringId RACHICUES_LV_90;
	
	@ClientString(id = 1120270, message = "Rachicues (Lv. 90)")
	public static NpcStringId RACHICUES_LV_902;
	
	@ClientString(id = 1120271, message = "Veridan (lv98)")
	public static NpcStringId VERIDAN_LV98;
	
	@ClientString(id = 1120272, message = "Kechi (lv98)")
	public static NpcStringId KECHI_LV98;
	
	@ClientString(id = 1120273, message = "Michaela (lv98)")
	public static NpcStringId MICHAELA_LV98;
	
	@ClientString(id = 1120274, message = "Pan Direm (lv89)")
	public static NpcStringId PAN_DIREM_LV89;
	
	@ClientString(id = 1120275, message = "Vladimir (lv59)")
	public static NpcStringId VLADIMIR_LV59;
	
	@ClientString(id = 1120276, message = "Lazearth (lv62)")
	public static NpcStringId LAZEARTH_LV62;
	
	@ClientString(id = 1120277, message = "Baron (lv66)")
	public static NpcStringId BARON_LV66;
	
	@ClientString(id = 1120278, message = "Ken (lv70)")
	public static NpcStringId KEN_LV70;
	
	@ClientString(id = 1120279, message = "Heaven's Palace Sniper Captain (lv72)")
	public static NpcStringId HEAVEN_S_PALACE_SNIPER_CAPTAIN_LV72;
	
	@ClientString(id = 1120280, message = "Emperor of Heaven's Palace (lv75)")
	public static NpcStringId EMPEROR_OF_HEAVEN_S_PALACE_LV75;
	
	@ClientString(id = 1120281, message = "Heaven's Palace Goddess (lv75)")
	public static NpcStringId HEAVEN_S_PALACE_GODDESS_LV75;
	
	@ClientString(id = 1120282, message = "Ron (lv97)")
	public static NpcStringId RON_LV97;
	
	@ClientString(id = 1120283, message = "Ron (lv97)")
	public static NpcStringId RON_LV972;
	
	@ClientString(id = 1120284, message = "Accencio (lv45)")
	public static NpcStringId ACCENCIO_LV45;
	
	@ClientString(id = 1120285, message = "Malicio (lv45)")
	public static NpcStringId MALICIO_LV45;
	
	@ClientString(id = 1120286, message = "Mertes (lv45)")
	public static NpcStringId MERTES_LV45;
	
	@ClientString(id = 1120287, message = "Eyesaroch (lv10)")
	public static NpcStringId EYESAROCH_LV10;
	
	@ClientString(id = 1120288, message = "Overseer (lv20)")
	public static NpcStringId OVERSEER_LV20;
	
	@ClientString(id = 1120289, message = "Weeping Yui (lv92)")
	public static NpcStringId WEEPING_YUI_LV92;
	
	@ClientString(id = 1120290, message = "Mukshu the Coward (lv93)")
	public static NpcStringId MUKSHU_THE_COWARD_LV93;
	
	@ClientString(id = 1120291, message = "Blind Hornafi (lv96)")
	public static NpcStringId BLIND_HORNAFI_LV96;
	
	@ClientString(id = 1120292, message = "Enraged Master Kinen (lv94)")
	public static NpcStringId ENRAGED_MASTER_KINEN_LV94;
	
	@ClientString(id = 1120293, message = "Sir Lesyinda of the Black Shadow (lv95)")
	public static NpcStringId SIR_LESYINDA_OF_THE_BLACK_SHADOW_LV95;
	
	@ClientString(id = 1120294, message = "Magical Warrior Konyar (lv96)")
	public static NpcStringId MAGICAL_WARRIOR_KONYAR_LV96;
	
	@ClientString(id = 1120295, message = "Magical Warrior Konyar (lv96)")
	public static NpcStringId MAGICAL_WARRIOR_KONYAR_LV962;
	
	@ClientString(id = 1120296, message = "Magical Warrior Konyar (lv96)")
	public static NpcStringId MAGICAL_WARRIOR_KONYAR_LV963;
	
	@ClientString(id = 1120297, message = "Yoentumak the Waiter (lv96)")
	public static NpcStringId YOENTUMAK_THE_WAITER_LV96;
	
	@ClientString(id = 1120298, message = "Flynt (lv97)")
	public static NpcStringId FLYNT_LV97;
	
	@ClientString(id = 1120299, message = "Harp (lv97)")
	public static NpcStringId HARP_LV97;
	
	@ClientString(id = 1120300, message = "Thank you... My book... Child...")
	public static NpcStringId THANK_YOU_MY_BOOK_CHILD;
	
	@ClientString(id = 1120301, message = "Killed by $s1")
	public static NpcStringId KILLED_BY_S1;
	
	@ClientString(id = 1120302, message = "Apherus (lv94)")
	public static NpcStringId APHERUS_LV94;
	
	@ClientString(id = 1120303, message = "Spezion (lv98)")
	public static NpcStringId SPEZION_LV98;
	
	@ClientString(id = 1120304, message = "Spezion (lv98)")
	public static NpcStringId SPEZION_LV982;
	
	@ClientString(id = 1120305, message = "Berserk Spezion (lv98)")
	public static NpcStringId BERSERK_SPEZION_LV98;
	
	@ClientString(id = 1120306, message = "Iris (lv43)")
	public static NpcStringId IRIS_LV43;
	
	@ClientString(id = 1120307, message = "Thorn (lv43)")
	public static NpcStringId THORN_LV43;
	
	@ClientString(id = 1120308, message = "Queen of Darkness (lv88)")
	public static NpcStringId QUEEN_OF_DARKNESS_LV88;
	
	@ClientString(id = 1120309, message = "Maliss (lv96)")
	public static NpcStringId MALISS_LV96;
	
	@ClientString(id = 1120310, message = "Isadora (lv96)")
	public static NpcStringId ISADORA_LV96;
	
	@ClientString(id = 1120311, message = "Georgios (lv97)")
	public static NpcStringId GEORGIOS_LV97;
	
	@ClientString(id = 1120312, message = "Veridan (lv98)")
	public static NpcStringId VERIDAN_LV982;
	
	@ClientString(id = 1120313, message = "Zellaka (lv88)")
	public static NpcStringId ZELLAKA_LV88;
	
	@ClientString(id = 1120314, message = "Pelline (lv93)")
	public static NpcStringId PELLINE_LV93;
	
	@ClientString(id = 1120315, message = "Kalios (lv98)")
	public static NpcStringId KALIOS_LV98;
	
	@ClientString(id = 1120316, message = "Guillotine of Death (lv97)")
	public static NpcStringId GUILLOTINE_OF_DEATH_LV97;
	
	@ClientString(id = 1120317, message = "Houpon the Warden Overseer (lv97)")
	public static NpcStringId HOUPON_THE_WARDEN_OVERSEER_LV97;
	
	@ClientString(id = 1120318, message = "Crook the Mad (lv97)")
	public static NpcStringId CROOK_THE_MAD_LV97;
	
	@ClientString(id = 1120319, message = "Execution Grounds Watchman Guillotine (lv97)")
	public static NpcStringId EXECUTION_GROUNDS_WATCHMAN_GUILLOTINE_LV97;
	
	@ClientString(id = 1120320, message = "Guillotine of Death (lv97)")
	public static NpcStringId GUILLOTINE_OF_DEATH_LV972;
	
	@ClientString(id = 1120321, message = "Spicula Zero (lv96)")
	public static NpcStringId SPICULA_ZERO_LV96;
	
	@ClientString(id = 1120322, message = "Gigantic Golem (lv96)")
	public static NpcStringId GIGANTIC_GOLEM_LV96;
	
	@ClientString(id = 1120323, message = "Queen Ant (lv92)")
	public static NpcStringId QUEEN_ANT_LV92;
	
	@ClientString(id = 1120324, message = "Orfen (lv92)")
	public static NpcStringId ORFEN_LV92;
	
	@ClientString(id = 1120325, message = "Anakim (lv92)")
	public static NpcStringId ANAKIM_LV92;
	
	@ClientString(id = 1120326, message = "Lilith (lv92)")
	public static NpcStringId LILITH_LV92;
	
	@ClientString(id = 1120327, message = "Nerva Chief Turakan (lv99)")
	public static NpcStringId NERVA_CHIEF_TURAKAN_LV99;
	
	@ClientString(id = 1120328, message = "Krogel (lv99)")
	public static NpcStringId KROGEL_LV99;
	
	@ClientString(id = 1120329, message = "Tebot (lv99)")
	public static NpcStringId TEBOT_LV99;
	
	@ClientString(id = 1120330, message = "Tegaffe (lv99)")
	public static NpcStringId TEGAFFE_LV99;
	
	@ClientString(id = 1120331, message = "Thesakar (lv99)")
	public static NpcStringId THESAKAR_LV99;
	
	@ClientString(id = 1120332, message = "Theor (lv99)")
	public static NpcStringId THEOR_LV99;
	
	@ClientString(id = 1120333, message = "Transformed: Dartanion (lv86)")
	public static NpcStringId TRANSFORMED_DARTANION_LV86;
	
	@ClientString(id = 1120334, message = "Garden Patrol Captain (lv93)")
	public static NpcStringId GARDEN_PATROL_CAPTAIN_LV93;
	
	@ClientString(id = 1120335, message = "Spicula Negative (lv95)")
	public static NpcStringId SPICULA_NEGATIVE_LV95;
	
	@ClientString(id = 1120336, message = "Beleth (lv99)")
	public static NpcStringId BELETH_LV99;
	
	@ClientString(id = 1120337, message = "Darion (lv99)")
	public static NpcStringId DARION_LV99;
	
	@ClientString(id = 1121000, message = "Steward: Please wait a moment.")
	public static NpcStringId STEWARD_PLEASE_WAIT_A_MOMENT;
	
	@ClientString(id = 1121001, message = "Steward: Please restore the Queen's former appearance!")
	public static NpcStringId STEWARD_PLEASE_RESTORE_THE_QUEEN_S_FORMER_APPEARANCE;
	
	@ClientString(id = 1121002, message = "Steward: Waste no time!  Please hurry!")
	public static NpcStringId STEWARD_WASTE_NO_TIME_PLEASE_HURRY;
	
	@ClientString(id = 1121003, message = "Steward: Was it indeed too much to ask...")
	public static NpcStringId STEWARD_WAS_IT_INDEED_TOO_MUCH_TO_ASK;
	
	@ClientString(id = 1121004, message = "Freya: Heathens! Feel my chill!")
	public static NpcStringId FREYA_HEATHENS_FEEL_MY_CHILL;
	
	@ClientString(id = 1121005, message = "Attention please! The gates will be closing shortly. All visitors to the Queen's Castle should leave immediately.")
	public static NpcStringId ATTENTION_PLEASE_THE_GATES_WILL_BE_CLOSING_SHORTLY_ALL_VISITORS_TO_THE_QUEEN_S_CASTLE_SHOULD_LEAVE_IMMEDIATELY;
	
	@ClientString(id = 1121006, message = "You cannot carry a weapon without authorization!")
	public static NpcStringId YOU_CANNOT_CARRY_A_WEAPON_WITHOUT_AUTHORIZATION;
	
	@ClientString(id = 1121007, message = "Are you trying to deceive me?  I'm disappointed.")
	public static NpcStringId ARE_YOU_TRYING_TO_DECEIVE_ME_I_M_DISAPPOINTED;
	
	@ClientString(id = 1121008, message = "30 minutes remain.")
	public static NpcStringId MINUTES_REMAIN;
	
	@ClientString(id = 1121009, message = "20 minutes remain.")
	public static NpcStringId MINUTES_REMAIN2;
	
	@ClientString(id = 1200001, message = "Chilly Coda")
	public static NpcStringId CHILLY_CODA;
	
	@ClientString(id = 1200002, message = "Burning Coda")
	public static NpcStringId BURNING_CODA;
	
	@ClientString(id = 1200003, message = "Blue Coda")
	public static NpcStringId BLUE_CODA;
	
	@ClientString(id = 1200004, message = "Red Coda")
	public static NpcStringId RED_CODA;
	
	@ClientString(id = 1200005, message = "Golden Coda")
	public static NpcStringId GOLDEN_CODA;
	
	@ClientString(id = 1200006, message = "Desert Coda")
	public static NpcStringId DESERT_CODA;
	
	@ClientString(id = 1200007, message = "Lute Coda")
	public static NpcStringId LUTE_CODA;
	
	@ClientString(id = 1200008, message = "Twin Coda")
	public static NpcStringId TWIN_CODA;
	
	@ClientString(id = 1200009, message = "Dark Coda")
	public static NpcStringId DARK_CODA;
	
	@ClientString(id = 1200010, message = "Coda of Light")
	public static NpcStringId CODA_OF_LIGHT;
	
	@ClientString(id = 1200011, message = "Chilly Cobol")
	public static NpcStringId CHILLY_COBOL;
	
	@ClientString(id = 1200012, message = "Burning Cobol")
	public static NpcStringId BURNING_COBOL;
	
	@ClientString(id = 1200013, message = "Blue Cobol")
	public static NpcStringId BLUE_COBOL;
	
	@ClientString(id = 1200014, message = "Red Cobol")
	public static NpcStringId RED_COBOL;
	
	@ClientString(id = 1200015, message = "Golden Cobol")
	public static NpcStringId GOLDEN_COBOL;
	
	@ClientString(id = 1200016, message = "Desert Cobol")
	public static NpcStringId DESERT_COBOL;
	
	@ClientString(id = 1200017, message = "Sea Cobol")
	public static NpcStringId SEA_COBOL;
	
	@ClientString(id = 1200018, message = "Thorn Cobol")
	public static NpcStringId THORN_COBOL;
	
	@ClientString(id = 1200019, message = "Dapple Cobol")
	public static NpcStringId DAPPLE_COBOL;
	
	@ClientString(id = 1200020, message = "Great Cobol")
	public static NpcStringId GREAT_COBOL;
	
	@ClientString(id = 1200021, message = "Chilly Codran")
	public static NpcStringId CHILLY_CODRAN;
	
	@ClientString(id = 1200022, message = "Burning Codran")
	public static NpcStringId BURNING_CODRAN;
	
	@ClientString(id = 1200023, message = "Blue Codran")
	public static NpcStringId BLUE_CODRAN;
	
	@ClientString(id = 1200024, message = "Red Codran")
	public static NpcStringId RED_CODRAN;
	
	@ClientString(id = 1200025, message = "Dapple Codran")
	public static NpcStringId DAPPLE_CODRAN;
	
	@ClientString(id = 1200026, message = "Desert Codran")
	public static NpcStringId DESERT_CODRAN;
	
	@ClientString(id = 1200027, message = "Sea Codran")
	public static NpcStringId SEA_CODRAN;
	
	@ClientString(id = 1200028, message = "Twin Codran")
	public static NpcStringId TWIN_CODRAN;
	
	@ClientString(id = 1200029, message = "Thorn Codran")
	public static NpcStringId THORN_CODRAN;
	
	@ClientString(id = 1200030, message = "Great Codran")
	public static NpcStringId GREAT_CODRAN;
	
	@ClientString(id = 1200031, message = "Alternative Dark Coda")
	public static NpcStringId ALTERNATIVE_DARK_CODA;
	
	@ClientString(id = 1200032, message = "Alternative Red Coda")
	public static NpcStringId ALTERNATIVE_RED_CODA;
	
	@ClientString(id = 1200033, message = "Alternative Chilly Coda")
	public static NpcStringId ALTERNATIVE_CHILLY_CODA;
	
	@ClientString(id = 1200034, message = "Alternative Blue Coda")
	public static NpcStringId ALTERNATIVE_BLUE_CODA;
	
	@ClientString(id = 1200035, message = "Alternative Golden Coda")
	public static NpcStringId ALTERNATIVE_GOLDEN_CODA;
	
	@ClientString(id = 1200036, message = "Alternative Lute Coda")
	public static NpcStringId ALTERNATIVE_LUTE_CODA;
	
	@ClientString(id = 1200037, message = "Alternative Desert Coda")
	public static NpcStringId ALTERNATIVE_DESERT_CODA;
	
	@ClientString(id = 1200038, message = "Alternative Red Cobol")
	public static NpcStringId ALTERNATIVE_RED_COBOL;
	
	@ClientString(id = 1200039, message = "Alternative Chilly Cobol")
	public static NpcStringId ALTERNATIVE_CHILLY_COBOL;
	
	@ClientString(id = 1200040, message = "Alternative Blue Cobol")
	public static NpcStringId ALTERNATIVE_BLUE_COBOL;
	
	@ClientString(id = 1200041, message = "Alternative Thorn Cobol")
	public static NpcStringId ALTERNATIVE_THORN_COBOL;
	
	@ClientString(id = 1200042, message = "Alternative Golden Cobol")
	public static NpcStringId ALTERNATIVE_GOLDEN_COBOL;
	
	@ClientString(id = 1200043, message = "Alternative Great Cobol")
	public static NpcStringId ALTERNATIVE_GREAT_COBOL;
	
	@ClientString(id = 1200044, message = "Alternative Red Codran")
	public static NpcStringId ALTERNATIVE_RED_CODRAN;
	
	@ClientString(id = 1200045, message = "Alternative Sea Codran")
	public static NpcStringId ALTERNATIVE_SEA_CODRAN;
	
	@ClientString(id = 1200046, message = "Alternative Chilly Codran")
	public static NpcStringId ALTERNATIVE_CHILLY_CODRAN;
	
	@ClientString(id = 1200047, message = "Alternative Blue Codran")
	public static NpcStringId ALTERNATIVE_BLUE_CODRAN;
	
	@ClientString(id = 1200048, message = "Alternative Twin Codran")
	public static NpcStringId ALTERNATIVE_TWIN_CODRAN;
	
	@ClientString(id = 1200049, message = "Alternative Great Codran")
	public static NpcStringId ALTERNATIVE_GREAT_CODRAN;
	
	@ClientString(id = 1200050, message = "Alternative Desert Codran")
	public static NpcStringId ALTERNATIVE_DESERT_CODRAN;
	
	@ClientString(id = 1300001, message = "We have broken through the gate! Destroy the encampment and move to the Command Post!")
	public static NpcStringId WE_HAVE_BROKEN_THROUGH_THE_GATE_DESTROY_THE_ENCAMPMENT_AND_MOVE_TO_THE_COMMAND_POST;
	
	@ClientString(id = 1300002, message = "The command gate has opened! Capture the flag quickly and raise it high to proclaim our victory!")
	public static NpcStringId THE_COMMAND_GATE_HAS_OPENED_CAPTURE_THE_FLAG_QUICKLY_AND_RAISE_IT_HIGH_TO_PROCLAIM_OUR_VICTORY;
	
	@ClientString(id = 1300003, message = "The gods have forsaken us... Retreat!!")
	public static NpcStringId THE_GODS_HAVE_FORSAKEN_US_RETREAT;
	
	@ClientString(id = 1300004, message = "You may have broken our arrows, but you will never break our will! Archers, retreat!")
	public static NpcStringId YOU_MAY_HAVE_BROKEN_OUR_ARROWS_BUT_YOU_WILL_NEVER_BREAK_OUR_WILL_ARCHERS_RETREAT;
	
	@ClientString(id = 1300005, message = "At last! The Magic Circle that protects the fortress has weakened! Volunteers, stand back!")
	public static NpcStringId AT_LAST_THE_MAGIC_CIRCLE_THAT_PROTECTS_THE_FORTRESS_HAS_WEAKENED_VOLUNTEERS_STAND_BACK;
	
	@ClientString(id = 1300006, message = "Aiieeee! Command Center! This is guard unit! We need backup right away!")
	public static NpcStringId AIIEEEE_COMMAND_CENTER_THIS_IS_GUARD_UNIT_WE_NEED_BACKUP_RIGHT_AWAY;
	
	@ClientString(id = 1300007, message = "Fortress power disabled.")
	public static NpcStringId FORTRESS_POWER_DISABLED;
	
	@ClientString(id = 1300008, message = "Oh my, what has become of me???  My fame.. my friends.. lost.. all lost...")
	public static NpcStringId OH_MY_WHAT_HAS_BECOME_OF_ME_MY_FAME_MY_FRIENDS_LOST_ALL_LOST;
	
	@ClientString(id = 1300009, message = "Machine No. 1 - Power Off!")
	public static NpcStringId MACHINE_NO_1_POWER_OFF;
	
	@ClientString(id = 1300010, message = "Machine No. 2  - Power Off!")
	public static NpcStringId MACHINE_NO_2_POWER_OFF;
	
	@ClientString(id = 1300011, message = "Machine No. 3  - Power Off!")
	public static NpcStringId MACHINE_NO_3_POWER_OFF;
	
	@ClientString(id = 1300012, message = "Everyone, concentrate your attacks on $s1! Show the enemy your resolve!")
	public static NpcStringId EVERYONE_CONCENTRATE_YOUR_ATTACKS_ON_S1_SHOW_THE_ENEMY_YOUR_RESOLVE;
	
	@ClientString(id = 1300013, message = "Attacking the enemy's reinforcements is necessary. Time to die!")
	public static NpcStringId ATTACKING_THE_ENEMY_S_REINFORCEMENTS_IS_NECESSARY_TIME_TO_DIE;
	
	@ClientString(id = 1300014, message = "Fire Spirit, unleash your power! Burn the enemy!!")
	public static NpcStringId FIRE_SPIRIT_UNLEASH_YOUR_POWER_BURN_THE_ENEMY;
	
	@ClientString(id = 1300015, message = "Hey, these foes are tougher than they look. I'm going to need some help here.")
	public static NpcStringId HEY_THESE_FOES_ARE_TOUGHER_THAN_THEY_LOOK_I_M_GOING_TO_NEED_SOME_HELP_HERE;
	
	@ClientString(id = 1300016, message = "Do you need my power? You seem to be struggling.")
	public static NpcStringId DO_YOU_NEED_MY_POWER_YOU_SEEM_TO_BE_STRUGGLING;
	
	@ClientString(id = 1300017, message = "I'm rather busy here as well.")
	public static NpcStringId I_M_RATHER_BUSY_HERE_AS_WELL;
	
	@ClientString(id = 1300018, message = "Don't think that it's gonna end like this. Your ambition will soon be destroyed as well.")
	public static NpcStringId DON_T_THINK_THAT_IT_S_GONNA_END_LIKE_THIS_YOUR_AMBITION_WILL_SOON_BE_DESTROYED_AS_WELL;
	
	@ClientString(id = 1300019, message = "You must have been prepared to die!")
	public static NpcStringId YOU_MUST_HAVE_BEEN_PREPARED_TO_DIE;
	
	@ClientString(id = 1300020, message = "I feel so much grief that I can't even take care of myself. There isn't any reason for me to stay here any longer.")
	public static NpcStringId I_FEEL_SO_MUCH_GRIEF_THAT_I_CAN_T_EVEN_TAKE_CARE_OF_MYSELF_THERE_ISN_T_ANY_REASON_FOR_ME_TO_STAY_HERE_ANY_LONGER;
	
	@ClientString(id = 1300101, message = "Camp Fortress")
	public static NpcStringId CAMP_FORTRESS;
	
	@ClientString(id = 1300102, message = "Southern Fortress")
	public static NpcStringId SOUTHERN_FORTRESS;
	
	@ClientString(id = 1300103, message = "Hive Fortress")
	public static NpcStringId HIVE_FORTRESS;
	
	@ClientString(id = 1300104, message = "Valley Fortress")
	public static NpcStringId VALLEY_FORTRESS;
	
	@ClientString(id = 1300105, message = "Ivory Fortress")
	public static NpcStringId IVORY_FORTRESS;
	
	@ClientString(id = 1300106, message = "Narsell Fortress")
	public static NpcStringId NARSELL_FORTRESS;
	
	@ClientString(id = 1300107, message = "Basin Fortress")
	public static NpcStringId BASIN_FORTRESS;
	
	@ClientString(id = 1300108, message = "White Sands Fortress")
	public static NpcStringId WHITE_SANDS_FORTRESS;
	
	@ClientString(id = 1300109, message = "Borderland Fortress")
	public static NpcStringId BORDERLAND_FORTRESS;
	
	@ClientString(id = 1300110, message = "Swamp Fortress")
	public static NpcStringId SWAMP_FORTRESS;
	
	@ClientString(id = 1300111, message = "Archaic Fortress")
	public static NpcStringId ARCHAIC_FORTRESS;
	
	@ClientString(id = 1300112, message = "Floran Fortress")
	public static NpcStringId FLORAN_FORTRESS;
	
	@ClientString(id = 1300113, message = "Cloud Mountain Fortress")
	public static NpcStringId CLOUD_MOUNTAIN_FORTRESS;
	
	@ClientString(id = 1300114, message = "Tanor Fortress")
	public static NpcStringId TANOR_FORTRESS;
	
	@ClientString(id = 1300115, message = "Dragonspine Fortress")
	public static NpcStringId DRAGONSPINE_FORTRESS;
	
	@ClientString(id = 1300116, message = "Antharas' Fortress")
	public static NpcStringId ANTHARAS_FORTRESS;
	
	@ClientString(id = 1300117, message = "Western Fortress")
	public static NpcStringId WESTERN_FORTRESS;
	
	@ClientString(id = 1300118, message = "Hunter's Fortress")
	public static NpcStringId HUNTER_S_FORTRESS;
	
	@ClientString(id = 1300119, message = "Aaru Fortress")
	public static NpcStringId AARU_FORTRESS;
	
	@ClientString(id = 1300120, message = "Demon Fortress")
	public static NpcStringId DEMON_FORTRESS;
	
	@ClientString(id = 1300121, message = "Monastic Fortress")
	public static NpcStringId MONASTIC_FORTRESS;
	
	@ClientString(id = 1300122, message = "Independent State")
	public static NpcStringId INDEPENDENT_STATE;
	
	@ClientString(id = 1300123, message = "Nonpartisan")
	public static NpcStringId NONPARTISAN;
	
	@ClientString(id = 1300124, message = "Contract State")
	public static NpcStringId CONTRACT_STATE;
	
	@ClientString(id = 1300125, message = "First password has been entered.")
	public static NpcStringId FIRST_PASSWORD_HAS_BEEN_ENTERED;
	
	@ClientString(id = 1300126, message = "Second password has been entered.")
	public static NpcStringId SECOND_PASSWORD_HAS_BEEN_ENTERED;
	
	@ClientString(id = 1300127, message = "Password has not been entered.")
	public static NpcStringId PASSWORD_HAS_NOT_BEEN_ENTERED;
	
	@ClientString(id = 1300128, message = "Attempt $s1 / 3 is in progress. =) This is the third attempt on $s1.")
	public static NpcStringId ATTEMPT_S1_3_IS_IN_PROGRESS_THIS_IS_THE_THIRD_ATTEMPT_ON_S1;
	
	@ClientString(id = 1300129, message = "The 1st Mark is correct.")
	public static NpcStringId THE_1ST_MARK_IS_CORRECT;
	
	@ClientString(id = 1300130, message = "The 2nd Mark is correct.")
	public static NpcStringId THE_2ND_MARK_IS_CORRECT;
	
	@ClientString(id = 1300131, message = "The Marks have not been assembled.")
	public static NpcStringId THE_MARKS_HAVE_NOT_BEEN_ASSEMBLED;
	
	@ClientString(id = 1300132, message = "Olympiad all-class battle is going to begin in Arena $s1 in a moment.")
	public static NpcStringId OLYMPIAD_ALL_CLASS_BATTLE_IS_GOING_TO_BEGIN_IN_ARENA_S1_IN_A_MOMENT;
	
	@ClientString(id = 1300133, message = "Domain Fortress")
	public static NpcStringId DOMAIN_FORTRESS;
	
	@ClientString(id = 1300134, message = "Boundary Fortress")
	public static NpcStringId BOUNDARY_FORTRESS;
	
	@ClientString(id = 1300135, message = "$s1hour $s2minute")
	public static NpcStringId S1HOUR_S2MINUTE;
	
	@ClientString(id = 1300136, message = "Not designated")
	public static NpcStringId NOT_DESIGNATED;
	
	@ClientString(id = 1300137, message = "Warriors, have you come to help those who are imprisoned here?")
	public static NpcStringId WARRIORS_HAVE_YOU_COME_TO_HELP_THOSE_WHO_ARE_IMPRISONED_HERE;
	
	@ClientString(id = 1300138, message = "Take that, you weakling!")
	public static NpcStringId TAKE_THAT_YOU_WEAKLING;
	
	@ClientString(id = 1300139, message = "Behold my might!")
	public static NpcStringId BEHOLD_MY_MIGHT;
	
	@ClientString(id = 1300140, message = "Your mind is going blank...")
	public static NpcStringId YOUR_MIND_IS_GOING_BLANK;
	
	@ClientString(id = 1300141, message = "Ugh, it hurts down to the bones...")
	public static NpcStringId UGH_IT_HURTS_DOWN_TO_THE_BONES;
	
	@ClientString(id = 1300142, message = "I can't stand it anymore. Aah...")
	public static NpcStringId I_CAN_T_STAND_IT_ANYMORE_AAH;
	
	@ClientString(id = 1300143, message = "Kyaaak!")
	public static NpcStringId KYAAAK;
	
	@ClientString(id = 1300144, message = "Gasp! How can this be?")
	public static NpcStringId GASP_HOW_CAN_THIS_BE;
	
	@ClientString(id = 1300145, message = "I'll rip the flesh from your bones!")
	public static NpcStringId I_LL_RIP_THE_FLESH_FROM_YOUR_BONES;
	
	@ClientString(id = 1300146, message = "You'll flounder in delusion for the rest of your life!")
	public static NpcStringId YOU_LL_FLOUNDER_IN_DELUSION_FOR_THE_REST_OF_YOUR_LIFE;
	
	@ClientString(id = 1300147, message = "There is no escape from this place!")
	public static NpcStringId THERE_IS_NO_ESCAPE_FROM_THIS_PLACE;
	
	@ClientString(id = 1300148, message = "How dare you!")
	public static NpcStringId HOW_DARE_YOU;
	
	@ClientString(id = 1300149, message = "I shall defeat you.")
	public static NpcStringId I_SHALL_DEFEAT_YOU;
	
	@ClientString(id = 1300150, message = "Begin stage 1!")
	public static NpcStringId BEGIN_STAGE_1;
	
	@ClientString(id = 1300151, message = "Begin stage 2!")
	public static NpcStringId BEGIN_STAGE_2;
	
	@ClientString(id = 1300152, message = "Begin stage 3!")
	public static NpcStringId BEGIN_STAGE_3;
	
	@ClientString(id = 1300153, message = "You've done it! We believed in you, warrior. We want to show our sincerity, though it is small. Please give me some of your time.")
	public static NpcStringId YOU_VE_DONE_IT_WE_BELIEVED_IN_YOU_WARRIOR_WE_WANT_TO_SHOW_OUR_SINCERITY_THOUGH_IT_IS_SMALL_PLEASE_GIVE_ME_SOME_OF_YOUR_TIME;
	
	@ClientString(id = 1300154, message = "The Central Stronghold's compressor is working.")
	public static NpcStringId THE_CENTRAL_STRONGHOLD_S_COMPRESSOR_IS_WORKING;
	
	@ClientString(id = 1300155, message = "Stronghold I's compressor is working.")
	public static NpcStringId STRONGHOLD_I_S_COMPRESSOR_IS_WORKING;
	
	@ClientString(id = 1300156, message = "Stronghold II's compressor is working.")
	public static NpcStringId STRONGHOLD_II_S_COMPRESSOR_IS_WORKING;
	
	@ClientString(id = 1300157, message = "Stronghold III's compressor is working.")
	public static NpcStringId STRONGHOLD_III_S_COMPRESSOR_IS_WORKING;
	
	@ClientString(id = 1300158, message = "The Central Stronghold's compressor has been destroyed.")
	public static NpcStringId THE_CENTRAL_STRONGHOLD_S_COMPRESSOR_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 1300159, message = "Stronghold I's compressor has been destroyed.")
	public static NpcStringId STRONGHOLD_I_S_COMPRESSOR_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 1300160, message = "Stronghold II's compressor has been destroyed.")
	public static NpcStringId STRONGHOLD_II_S_COMPRESSOR_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 1300161, message = "Stronghold III's compressor has been destroyed.")
	public static NpcStringId STRONGHOLD_III_S_COMPRESSOR_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 1300162, message = "What a predicament... my attempts were unsuccessful.")
	public static NpcStringId WHAT_A_PREDICAMENT_MY_ATTEMPTS_WERE_UNSUCCESSFUL;
	
	@ClientString(id = 1300163, message = "Courage! Ambition! Passion! Mercenaries who want to realize their dream of fighting in the territory war, come to me! Fortune and glory are waiting for you!")
	public static NpcStringId COURAGE_AMBITION_PASSION_MERCENARIES_WHO_WANT_TO_REALIZE_THEIR_DREAM_OF_FIGHTING_IN_THE_TERRITORY_WAR_COME_TO_ME_FORTUNE_AND_GLORY_ARE_WAITING_FOR_YOU;
	
	@ClientString(id = 1300164, message = "Do you wish to fight? Are you afraid? No matter how hard you try, you have nowhere to run. But if you face it head on, our mercenary troop will help you out!")
	public static NpcStringId DO_YOU_WISH_TO_FIGHT_ARE_YOU_AFRAID_NO_MATTER_HOW_HARD_YOU_TRY_YOU_HAVE_NOWHERE_TO_RUN_BUT_IF_YOU_FACE_IT_HEAD_ON_OUR_MERCENARY_TROOP_WILL_HELP_YOU_OUT;
	
	@ClientString(id = 1300165, message = "Charge! Charge! Charge!")
	public static NpcStringId CHARGE_CHARGE_CHARGE;
	
	@ClientString(id = 1300166, message = "Olympiad class-free individual match is going to begin in Arena $s1 in a moment.")
	public static NpcStringId OLYMPIAD_CLASS_FREE_INDIVIDUAL_MATCH_IS_GOING_TO_BEGIN_IN_ARENA_S1_IN_A_MOMENT;
	
	@ClientString(id = 1300167, message = "Olympiad class individual match is going to begin in Arena $s1 in a moment.")
	public static NpcStringId OLYMPIAD_CLASS_INDIVIDUAL_MATCH_IS_GOING_TO_BEGIN_IN_ARENA_S1_IN_A_MOMENT;
	
	@ClientString(id = 1300168, message = "Let us drive out the power of darkness.")
	public static NpcStringId LET_US_DRIVE_OUT_THE_POWER_OF_DARKNESS;
	
	@ClientString(id = 1300169, message = "Drive out the power of darkness along side our revolutionary forces.")
	public static NpcStringId DRIVE_OUT_THE_POWER_OF_DARKNESS_ALONG_SIDE_OUR_REVOLUTIONARY_FORCES;
	
	@ClientString(id = 1300170, message = "I feel that the powers of darkness are growing stronger.")
	public static NpcStringId I_FEEL_THAT_THE_POWERS_OF_DARKNESS_ARE_GROWING_STRONGER;
	
	@ClientString(id = 1300171, message = "We will execute our plan on Sunday night. If you join us, I will give you a substantial reward.")
	public static NpcStringId WE_WILL_EXECUTE_OUR_PLAN_ON_SUNDAY_NIGHT_IF_YOU_JOIN_US_I_WILL_GIVE_YOU_A_SUBSTANTIAL_REWARD;
	
	@ClientString(id = 1300172, message = "When the world plunges into chaos, we will need your help. We hope you join us when the time comes.")
	public static NpcStringId WHEN_THE_WORLD_PLUNGES_INTO_CHAOS_WE_WILL_NEED_YOUR_HELP_WE_HOPE_YOU_JOIN_US_WHEN_THE_TIME_COMES;
	
	@ClientString(id = 1300173, message = "The transportation unit has appeared. You can begin attacking it if you accept a campaign quest.")
	public static NpcStringId THE_TRANSPORTATION_UNIT_HAS_APPEARED_YOU_CAN_BEGIN_ATTACKING_IT_IF_YOU_ACCEPT_A_CAMPAIGN_QUEST;
	
	@ClientString(id = 1300174, message = "The remaining troops of the transportation unit have moved to the Aden Castle area.")
	public static NpcStringId THE_REMAINING_TROOPS_OF_THE_TRANSPORTATION_UNIT_HAVE_MOVED_TO_THE_ADEN_CASTLE_AREA;
	
	@ClientString(id = 1300175, message = "The remaining troops from the transportation unit will appear in front of Rune Castle in 5 minutes.")
	public static NpcStringId THE_REMAINING_TROOPS_FROM_THE_TRANSPORTATION_UNIT_WILL_APPEAR_IN_FRONT_OF_RUNE_CASTLE_IN_5_MINUTES;
	
	@ClientString(id = 1300176, message = "The Aden transportation unit has appeared at the east and west entrances to the Town of Aden.")
	public static NpcStringId THE_ADEN_TRANSPORTATION_UNIT_HAS_APPEARED_AT_THE_EAST_AND_WEST_ENTRANCES_TO_THE_TOWN_OF_ADEN;
	
	@ClientString(id = 1300177, message = "The Rune transportation unit has appeared on the battlefield of Rune.")
	public static NpcStringId THE_RUNE_TRANSPORTATION_UNIT_HAS_APPEARED_ON_THE_BATTLEFIELD_OF_RUNE;
	
	@ClientString(id = 1300178, message = "The Gludio transportation unit has appeared at the west entrance to the Town of Aden.")
	public static NpcStringId THE_GLUDIO_TRANSPORTATION_UNIT_HAS_APPEARED_AT_THE_WEST_ENTRANCE_TO_THE_TOWN_OF_ADEN;
	
	@ClientString(id = 1300179, message = "The Dion transportation unit has entered the battlefield.")
	public static NpcStringId THE_DION_TRANSPORTATION_UNIT_HAS_ENTERED_THE_BATTLEFIELD;
	
	@ClientString(id = 1300180, message = "The Giran transportation unit has appeared at the south entrance to the Town of Aden.")
	public static NpcStringId THE_GIRAN_TRANSPORTATION_UNIT_HAS_APPEARED_AT_THE_SOUTH_ENTRANCE_TO_THE_TOWN_OF_ADEN;
	
	@ClientString(id = 1300181, message = "The Oren transportation unit has appeared at the west entrance to the Town of Aden.")
	public static NpcStringId THE_OREN_TRANSPORTATION_UNIT_HAS_APPEARED_AT_THE_WEST_ENTRANCE_TO_THE_TOWN_OF_ADEN;
	
	@ClientString(id = 1300182, message = "The Innadril transportation unit has entered the battlefield.")
	public static NpcStringId THE_INNADRIL_TRANSPORTATION_UNIT_HAS_ENTERED_THE_BATTLEFIELD;
	
	@ClientString(id = 1300183, message = "The Goddard transportation unit has appeared in the battlefield area around Rune Castle.")
	public static NpcStringId THE_GODDARD_TRANSPORTATION_UNIT_HAS_APPEARED_IN_THE_BATTLEFIELD_AREA_AROUND_RUNE_CASTLE;
	
	@ClientString(id = 1300184, message = "The Schuttgart transportation unit has entered the battlefield.")
	public static NpcStringId THE_SCHUTTGART_TRANSPORTATION_UNIT_HAS_ENTERED_THE_BATTLEFIELD;
	
	@ClientString(id = 1300185, message = "The Aden Transportation Unit has appeared on the battlefield.")
	public static NpcStringId THE_ADEN_TRANSPORTATION_UNIT_HAS_APPEARED_ON_THE_BATTLEFIELD;
	
	@ClientString(id = 1300186, message = "The Rune Transportation Unit has appeared on the battlefield.")
	public static NpcStringId THE_RUNE_TRANSPORTATION_UNIT_HAS_APPEARED_ON_THE_BATTLEFIELD;
	
	@ClientString(id = 1300187, message = "Aah... Here...")
	public static NpcStringId AAH_HERE;
	
	@ClientString(id = 1300188, message = "The Dark Guards shall punish you.")
	public static NpcStringId THE_DARK_GUARDS_SHALL_PUNISH_YOU;
	
	@ClientString(id = 1300189, message = "Congratulations, my lord. Allow me to offer you the Dimensional Castle Lord's Support Box.")
	public static NpcStringId CONGRATULATIONS_MY_LORD_ALLOW_ME_TO_OFFER_YOU_THE_DIMENSIONAL_CASTLE_LORD_S_SUPPORT_BOX;
	
	@ClientString(id = 1300190, message = "Apologies, my lord. It seems that they do not view you as a friend.")
	public static NpcStringId APOLOGIES_MY_LORD_IT_SEEMS_THAT_THEY_DO_NOT_VIEW_YOU_AS_A_FRIEND;
	
	@ClientString(id = 1300191, message = "Apologies, my lord. It would seem wise that you keep y our distance from them.")
	public static NpcStringId APOLOGIES_MY_LORD_IT_WOULD_SEEM_WISE_THAT_YOU_KEEP_Y_OUR_DISTANCE_FROM_THEM;
	
	@ClientString(id = 1300192, message = "The lord of Aden Castle will be happy to receive your goodwill.")
	public static NpcStringId THE_LORD_OF_ADEN_CASTLE_WILL_BE_HAPPY_TO_RECEIVE_YOUR_GOODWILL;
	
	@ClientString(id = 1300193, message = "I must say I am disappointed. You turn down our goodwill and assist Rune Castle? I see how little you think of us. Very well, I will let your will be known to the lord of Aden Castle.")
	public static NpcStringId I_MUST_SAY_I_AM_DISAPPOINTED_YOU_TURN_DOWN_OUR_GOODWILL_AND_ASSIST_RUNE_CASTLE_I_SEE_HOW_LITTLE_YOU_THINK_OF_US_VERY_WELL_I_WILL_LET_YOUR_WILL_BE_KNOWN_TO_THE_LORD_OF_ADEN_CASTLE;
	
	@ClientString(id = 1300194, message = "The lord of Rune Castle will be happy to receive your goodwill.")
	public static NpcStringId THE_LORD_OF_RUNE_CASTLE_WILL_BE_HAPPY_TO_RECEIVE_YOUR_GOODWILL;
	
	@ClientString(id = 1300195, message = "I must say I am disappointed. You turn down our goodwill and assist Aden Castle? I see how little you think of us. Very well, I will let your will be known to the lord of Rune Castle.")
	public static NpcStringId I_MUST_SAY_I_AM_DISAPPOINTED_YOU_TURN_DOWN_OUR_GOODWILL_AND_ASSIST_ADEN_CASTLE_I_SEE_HOW_LITTLE_YOU_THINK_OF_US_VERY_WELL_I_WILL_LET_YOUR_WILL_BE_KNOWN_TO_THE_LORD_OF_RUNE_CASTLE;
	
	@ClientString(id = 1300196, message = "Congratulations! You are the lord of the Dimensional Castle. You will find the Dimensional Manager by the throne.")
	public static NpcStringId CONGRATULATIONS_YOU_ARE_THE_LORD_OF_THE_DIMENSIONAL_CASTLE_YOU_WILL_FIND_THE_DIMENSIONAL_MANAGER_BY_THE_THRONE;
	
	@ClientString(id = 1300197, message = "You have obtained the Dimensional Castle…! Well then, my lord, I have something for you.")
	public static NpcStringId YOU_HAVE_OBTAINED_THE_DIMENSIONAL_CASTLE_WELL_THEN_MY_LORD_I_HAVE_SOMETHING_FOR_YOU;
	
	@ClientString(id = 1300198, message = "I wish you luck on your next endeavors, heroes of the dimensions.")
	public static NpcStringId I_WISH_YOU_LUCK_ON_YOUR_NEXT_ENDEAVORS_HEROES_OF_THE_DIMENSIONS;
	
	@ClientString(id = 1300199, message = "Aden's heroes have seized victory in the Dimensional Space.")
	public static NpcStringId ADEN_S_HEROES_HAVE_SEIZED_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300200, message = "Rune's heroes have seized victory in the Dimensional Space.")
	public static NpcStringId RUNE_S_HEROES_HAVE_SEIZED_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300201, message = "With the power of darkness, Aden's heroes have seized victory in the Dimensional Space.")
	public static NpcStringId WITH_THE_POWER_OF_DARKNESS_ADEN_S_HEROES_HAVE_SEIZED_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300202, message = "With the power of darkness, Rune's heroes have seized victory in the Dimensional Space.")
	public static NpcStringId WITH_THE_POWER_OF_DARKNESS_RUNE_S_HEROES_HAVE_SEIZED_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300203, message = "You can feel the Dimensional Space Effect.")
	public static NpcStringId YOU_CAN_FEEL_THE_DIMENSIONAL_SPACE_EFFECT;
	
	@ClientString(id = 1300204, message = "The heroes of Gludio Castle have won a great victory in the Dimensional Space.")
	public static NpcStringId THE_HEROES_OF_GLUDIO_CASTLE_HAVE_WON_A_GREAT_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300205, message = "The heroes of Dion Castle have won a great victory in the Dimensional Space.")
	public static NpcStringId THE_HEROES_OF_DION_CASTLE_HAVE_WON_A_GREAT_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300206, message = "The heroes of Giran Castle have won a great victory in the Dimensional Space.")
	public static NpcStringId THE_HEROES_OF_GIRAN_CASTLE_HAVE_WON_A_GREAT_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300207, message = "The heroes of Oren Castle have won a great victory in the Dimensional Space.")
	public static NpcStringId THE_HEROES_OF_OREN_CASTLE_HAVE_WON_A_GREAT_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300208, message = "The heroes of Innadril Castle have won a great victory in the Dimensional Space.")
	public static NpcStringId THE_HEROES_OF_INNADRIL_CASTLE_HAVE_WON_A_GREAT_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300209, message = "The heroes of Goddard Castle have won a great victory in the Dimensional Space.")
	public static NpcStringId THE_HEROES_OF_GODDARD_CASTLE_HAVE_WON_A_GREAT_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300210, message = "The heroes of Schuttgart Castle have won a great victory in the Dimensional Space.")
	public static NpcStringId THE_HEROES_OF_SCHUTTGART_CASTLE_HAVE_WON_A_GREAT_VICTORY_IN_THE_DIMENSIONAL_SPACE;
	
	@ClientString(id = 1300300, message = "Try entering an Academy and gain special benefits! Come find me near the Gatekeeper of the Town of Gludio for a gift. For more details, visit the Library page of our website.")
	public static NpcStringId TRY_ENTERING_AN_ACADEMY_AND_GAIN_SPECIAL_BENEFITS_COME_FIND_ME_NEAR_THE_GATEKEEPER_OF_THE_TOWN_OF_GLUDIO_FOR_A_GIFT_FOR_MORE_DETAILS_VISIT_THE_LIBRARY_PAGE_OF_OUR_WEBSITE;
	
	@ClientString(id = 1300301, message = "Try entering an Academy and gain extra XP! Come find me near the Gatekeeper of the Town of Gludio for a gift. For more details, visit the Library page of our website.")
	public static NpcStringId TRY_ENTERING_AN_ACADEMY_AND_GAIN_EXTRA_XP_COME_FIND_ME_NEAR_THE_GATEKEEPER_OF_THE_TOWN_OF_GLUDIO_FOR_A_GIFT_FOR_MORE_DETAILS_VISIT_THE_LIBRARY_PAGE_OF_OUR_WEBSITE;
	
	@ClientString(id = 1600001, message = "Another player is currently being buffed. Please try again in a moment.")
	public static NpcStringId ANOTHER_PLAYER_IS_CURRENTLY_BEING_BUFFED_PLEASE_TRY_AGAIN_IN_A_MOMENT;
	
	@ClientString(id = 1600002, message = "You cannot mount while you are polymorphed.")
	public static NpcStringId YOU_CANNOT_MOUNT_WHILE_YOU_ARE_POLYMORPHED;
	
	@ClientString(id = 1600003, message = "You cannot mount a Wyvern while in battle mode or while mounted on a Strider.")
	public static NpcStringId YOU_CANNOT_MOUNT_A_WYVERN_WHILE_IN_BATTLE_MODE_OR_WHILE_MOUNTED_ON_A_STRIDER;
	
	@ClientString(id = 1600004, message = "Boo-hoo... I hate...")
	public static NpcStringId BOO_HOO_I_HATE;
	
	@ClientString(id = 1600005, message = "See you later.")
	public static NpcStringId SEE_YOU_LATER;
	
	@ClientString(id = 1600006, message = "You've made a great choice.")
	public static NpcStringId YOU_VE_MADE_A_GREAT_CHOICE;
	
	@ClientString(id = 1600007, message = "Thanks! I feel more relaxed…")
	public static NpcStringId THANKS_I_FEEL_MORE_RELAXED;
	
	@ClientString(id = 1600008, message = "Did you see that Firecracker explode?")
	public static NpcStringId DID_YOU_SEE_THAT_FIRECRACKER_EXPLODE;
	
	@ClientString(id = 1600009, message = "I am nothing.")
	public static NpcStringId I_AM_NOTHING;
	
	@ClientString(id = 1600010, message = "I am telling the truth.")
	public static NpcStringId I_AM_TELLING_THE_TRUTH;
	
	@ClientString(id = 1600011, message = "You cannot mount a Wyvern while in battle mode or while mounted on a Strider.")
	public static NpcStringId YOU_CANNOT_MOUNT_A_WYVERN_WHILE_IN_BATTLE_MODE_OR_WHILE_MOUNTED_ON_A_STRIDER2;
	
	@ClientString(id = 1600012, message = "It's free to go back to the village you teleported from.")
	public static NpcStringId IT_S_FREE_TO_GO_BACK_TO_THE_VILLAGE_YOU_TELEPORTED_FROM;
	
	@ClientString(id = 1600013, message = "If you collect 50 individual Treasure Sack Pieces, you can exchange them for a Treasure Sack.")
	public static NpcStringId IF_YOU_COLLECT_50_INDIVIDUAL_TREASURE_SACK_PIECES_YOU_CAN_EXCHANGE_THEM_FOR_A_TREASURE_SACK;
	
	@ClientString(id = 1600014, message = "You must be transformed into a Treasure Hunter to find a chest.")
	public static NpcStringId YOU_MUST_BE_TRANSFORMED_INTO_A_TREASURE_HUNTER_TO_FIND_A_CHEST;
	
	@ClientString(id = 1600015, message = "You'd better use the Transformation spell at the right moment since it doesn't last long.")
	public static NpcStringId YOU_D_BETTER_USE_THE_TRANSFORMATION_SPELL_AT_THE_RIGHT_MOMENT_SINCE_IT_DOESN_T_LAST_LONG;
	
	@ClientString(id = 1600016, message = "All of Fantasy Isle is a Peace Zone.")
	public static NpcStringId ALL_OF_FANTASY_ISLE_IS_A_PEACE_ZONE;
	
	@ClientString(id = 1600017, message = "If you need to go to Fantasy Isle, come see me.")
	public static NpcStringId IF_YOU_NEED_TO_GO_TO_FANTASY_ISLE_COME_SEE_ME;
	
	@ClientString(id = 1600018, message = "You can only purchase a Treasure Hunter Transformation Scroll once every 12 hours.")
	public static NpcStringId YOU_CAN_ONLY_PURCHASE_A_TREASURE_HUNTER_TRANSFORMATION_SCROLL_ONCE_EVERY_12_HOURS;
	
	@ClientString(id = 1600019, message = "If your means of arrival was a bit unconventional, then I'll be sending you back to the Town of Rune, which is the nearest town.")
	public static NpcStringId IF_YOUR_MEANS_OF_ARRIVAL_WAS_A_BIT_UNCONVENTIONAL_THEN_I_LL_BE_SENDING_YOU_BACK_TO_THE_TOWN_OF_RUNE_WHICH_IS_THE_NEAREST_TOWN;
	
	@ClientString(id = 1600020, message = "*Rattle*")
	public static NpcStringId RATTLE;
	
	@ClientString(id = 1600021, message = "*Bump*")
	public static NpcStringId BUMP;
	
	@ClientString(id = 1600022, message = "You will regret this.")
	public static NpcStringId YOU_WILL_REGRET_THIS;
	
	@ClientString(id = 1600023, message = "Care to challenge fate and test your luck?")
	public static NpcStringId CARE_TO_CHALLENGE_FATE_AND_TEST_YOUR_LUCK;
	
	@ClientString(id = 1600024, message = "Don't pass up the chance to win an S80 weapon.")
	public static NpcStringId DON_T_PASS_UP_THE_CHANCE_TO_WIN_AN_S80_WEAPON;
	
	@ClientString(id = 1600025, message = "The voltage is flowing. Run away.")
	public static NpcStringId THE_VOLTAGE_IS_FLOWING_RUN_AWAY;
	
	@ClientString(id = 1600026, message = "Begin Tutorial Quests")
	public static NpcStringId BEGIN_TUTORIAL_QUESTS;
	
	@ClientString(id = 1600027, message = "Newbie Helper has casted buffs on $s1.")
	public static NpcStringId NEWBIE_HELPER_HAS_CASTED_BUFFS_ON_S1;
	
	@ClientString(id = 1600028, message = "You win if you get to Ruins of Ye Sagira first.")
	public static NpcStringId YOU_WIN_IF_YOU_GET_TO_RUINS_OF_YE_SAGIRA_FIRST;
	
	@ClientString(id = 1600029, message = "Ah man, you beat me!")
	public static NpcStringId AH_MAN_YOU_BEAT_ME;
	
	@ClientString(id = 1600030, message = "Too bad. Try again.")
	public static NpcStringId TOO_BAD_TRY_AGAIN;
	
	@ClientString(id = 1600031, message = "$s1 has raised the temperature by $s2 degree(s).")
	public static NpcStringId S1_HAS_RAISED_THE_TEMPERATURE_BY_S2_DEGREE_S;
	
	@ClientString(id = 1600032, message = "$s1 has raised the temperature by $s2.$s3 degree(s).")
	public static NpcStringId S1_HAS_RAISED_THE_TEMPERATURE_BY_S2_S3_DEGREE_S;
	
	@ClientString(id = 1600033, message = "The Draco Egg cracked because the temperature was too high.")
	public static NpcStringId THE_DRACO_EGG_CRACKED_BECAUSE_THE_TEMPERATURE_WAS_TOO_HIGH;
	
	@ClientString(id = 1600034, message = "The Draco Egg was successfully incubated. Congratulations.")
	public static NpcStringId THE_DRACO_EGG_WAS_SUCCESSFULLY_INCUBATED_CONGRATULATIONS;
	
	@ClientString(id = 1600035, message = "The Draco Egg was successfully incubated at the optimal temperature. Congratulations.")
	public static NpcStringId THE_DRACO_EGG_WAS_SUCCESSFULLY_INCUBATED_AT_THE_OPTIMAL_TEMPERATURE_CONGRATULATIONS;
	
	@ClientString(id = 1600036, message = "=====(Temperature Raising Rank)=====")
	public static NpcStringId TEMPERATURE_RAISING_RANK;
	
	@ClientString(id = 1600037, message = "Rank $s1: $s2 ($s3.$s4 degree(s))")
	public static NpcStringId RANK_S1_S2_S3_S4_DEGREE_S;
	
	@ClientString(id = 1600038, message = "========================")
	public static NpcStringId EMPTY2;
	
	@ClientString(id = 1600039, message = "A gift is being given to $s1 for raising the temperature the most.")
	public static NpcStringId A_GIFT_IS_BEING_GIVEN_TO_S1_FOR_RAISING_THE_TEMPERATURE_THE_MOST;
	
	@ClientString(id = 1600040, message = "Incubation Possible")
	public static NpcStringId INCUBATION_POSSIBLE;
	
	@ClientString(id = 1600041, message = "Optimum Temperature")
	public static NpcStringId OPTIMUM_TEMPERATURE;
	
	@ClientString(id = 1600042, message = "Incubation Failure")
	public static NpcStringId INCUBATION_FAILURE;
	
	@ClientString(id = 1600043, message = "Your clan has acquired $s1 point(s) to its Clan Reputation.")
	public static NpcStringId YOUR_CLAN_HAS_ACQUIRED_S1_POINT_S_TO_ITS_CLAN_REPUTATION;
	
	@ClientString(id = 1600044, message = "Incubation Success")
	public static NpcStringId INCUBATION_SUCCESS;
	
	@ClientString(id = 1600045, message = "Incubation Failure")
	public static NpcStringId INCUBATION_FAILURE2;
	
	@ClientString(id = 1600046, message = "The leather machine and fur collection are working properly.")
	public static NpcStringId THE_LEATHER_MACHINE_AND_FUR_COLLECTION_ARE_WORKING_PROPERLY;
	
	@ClientString(id = 1600047, message = "The Energy Supplier is broken and the leather machine has stopped. Please repair the Energy Supplier.")
	public static NpcStringId THE_ENERGY_SUPPLIER_IS_BROKEN_AND_THE_LEATHER_MACHINE_HAS_STOPPED_PLEASE_REPAIR_THE_ENERGY_SUPPLIER;
	
	@ClientString(id = 1600048, message = "The leather machine is temporarily down. You can no longer collect fur from buffalo.")
	public static NpcStringId THE_LEATHER_MACHINE_IS_TEMPORARILY_DOWN_YOU_CAN_NO_LONGER_COLLECT_FUR_FROM_BUFFALO;
	
	@ClientString(id = 1600049, message = "Please bring $s1 Fruit(s) Imbued with Energy of Water, $s2 Fruit(s) Imbued with Energy of Fire, and $s3 Fruit(s) Imbued with Energy of Wind.")
	public static NpcStringId PLEASE_BRING_S1_FRUIT_S_IMBUED_WITH_ENERGY_OF_WATER_S2_FRUIT_S_IMBUED_WITH_ENERGY_OF_FIRE_AND_S3_FRUIT_S_IMBUED_WITH_ENERGY_OF_WIND;
	
	@ClientString(id = 1600050, message = "The number of fruits needed by the chef has changed. You now need to bring $s1 Fruit(s) Imbued with Energy of Water, $s2 Fruit(s) Imbued with Energy of Fire, and $s3 Fruit(s) Imbued with Energy of Wind.")
	public static NpcStringId THE_NUMBER_OF_FRUITS_NEEDED_BY_THE_CHEF_HAS_CHANGED_YOU_NOW_NEED_TO_BRING_S1_FRUIT_S_IMBUED_WITH_ENERGY_OF_WATER_S2_FRUIT_S_IMBUED_WITH_ENERGY_OF_FIRE_AND_S3_FRUIT_S_IMBUED_WITH_ENERGY_OF_WIND;
	
	@ClientString(id = 1600051, message = "The 4 Gathering Devices are activated. The Dark Summon Circle will soon disappear!")
	public static NpcStringId THE_4_GATHERING_DEVICES_ARE_ACTIVATED_THE_DARK_SUMMON_CIRCLE_WILL_SOON_DISAPPEAR;
	
	@ClientString(id = 1600052, message = "3 seconds until arrival. 3 seconds until arrival.")
	public static NpcStringId SECONDS_UNTIL_ARRIVAL_3_SECONDS_UNTIL_ARRIVAL;
	
	@ClientString(id = 1600053, message = "2 seconds until arrival. 2 seconds until arrival.")
	public static NpcStringId SECONDS_UNTIL_ARRIVAL_2_SECONDS_UNTIL_ARRIVAL;
	
	@ClientString(id = 1600054, message = "1 second until arrival. 1 second until arrival.")
	public static NpcStringId SECOND_UNTIL_ARRIVAL_1_SECOND_UNTIL_ARRIVAL;
	
	@ClientString(id = 1600055, message = "Reinforcements arrived. Reinforcements arrived.")
	public static NpcStringId REINFORCEMENTS_ARRIVED_REINFORCEMENTS_ARRIVED;
	
	@ClientString(id = 1600056, message = "Shut down. Shut down.")
	public static NpcStringId SHUT_DOWN_SHUT_DOWN;
	
	@ClientString(id = 1600057, message = "Boz' Energy")
	public static NpcStringId BOZ_ENERGY;
	
	@ClientString(id = 1600058, message = "The Beauty Shop is open! Come on in!")
	public static NpcStringId THE_BEAUTY_SHOP_IS_OPEN_COME_ON_IN;
	
	@ClientString(id = 1600059, message = "You can look good too, buddy! Come on, come on.")
	public static NpcStringId YOU_CAN_LOOK_GOOD_TOO_BUDDY_COME_ON_COME_ON;
	
	@ClientString(id = 1600060, message = "Now, don't leave me hanging! Let me see your beautiful self!")
	public static NpcStringId NOW_DON_T_LEAVE_ME_HANGING_LET_ME_SEE_YOUR_BEAUTIFUL_SELF;
	
	@ClientString(id = 1600061, message = "Come on, you beauty lovers! I'll show you the way!")
	public static NpcStringId COME_ON_YOU_BEAUTY_LOVERS_I_LL_SHOW_YOU_THE_WAY;
	
	@ClientString(id = 1600062, message = "Everyone come on, let's go Gangnam Style~")
	public static NpcStringId EVERYONE_COME_ON_LET_S_GO_GANGNAM_STYLE;
	
	@ClientString(id = 1600063, message = "Gainak in War")
	public static NpcStringId GAINAK_IN_WAR;
	
	@ClientString(id = 1600064, message = "Monitor the damage for 30 sec.")
	public static NpcStringId MONITOR_THE_DAMAGE_FOR_30_SEC;
	
	@ClientString(id = 1600065, message = "25 seconds left!")
	public static NpcStringId SECONDS_LEFT;
	
	@ClientString(id = 1600066, message = "Gainak in Peace")
	public static NpcStringId GAINAK_IN_PEACE;
	
	@ClientString(id = 1600067, message = "20 seconds left!")
	public static NpcStringId SECONDS_LEFT2;
	
	@ClientString(id = 1600068, message = "15 seconds left!")
	public static NpcStringId SECONDS_LEFT3;
	
	@ClientString(id = 1600069, message = "10 seconds left!")
	public static NpcStringId SECONDS_LEFT4;
	
	@ClientString(id = 1600070, message = "5 seconds left!")
	public static NpcStringId SECONDS_LEFT5;
	
	@ClientString(id = 1600071, message = "Demonic System will activate.")
	public static NpcStringId DEMONIC_SYSTEM_WILL_ACTIVATE;
	
	@ClientString(id = 1600072, message = "Monitor the damage for 50 sec.")
	public static NpcStringId MONITOR_THE_DAMAGE_FOR_50_SEC;
	
	@ClientString(id = 1600073, message = "40 seconds left!")
	public static NpcStringId SECONDS_LEFT6;
	
	@ClientString(id = 1600074, message = "35 seconds left!")
	public static NpcStringId SECONDS_LEFT7;
	
	@ClientString(id = 1600075, message = "30 seconds left!")
	public static NpcStringId SECONDS_LEFT8;
	
	@ClientString(id = 1600076, message = "45 seconds left!")
	public static NpcStringId SECONDS_LEFT9;
	
	@ClientString(id = 1620000, message = "Entered inside the Earth Wyrm.")
	public static NpcStringId ENTERED_INSIDE_THE_EARTH_WYRM;
	
	@ClientString(id = 1620001, message = "Acid is secreting from the Stomach Glands.")
	public static NpcStringId ACID_IS_SECRETING_FROM_THE_STOMACH_GLANDS;
	
	@ClientString(id = 1620002, message = "All Stomach Glands have been destroyed.")
	public static NpcStringId ALL_STOMACH_GLANDS_HAVE_BEEN_DESTROYED;
	
	@ClientString(id = 1620003, message = "The heart is being protected by foes.")
	public static NpcStringId THE_HEART_IS_BEING_PROTECTED_BY_FOES;
	
	@ClientString(id = 1620004, message = "You were expelled from the Earth Wyrm.")
	public static NpcStringId YOU_WERE_EXPELLED_FROM_THE_EARTH_WYRM;
	
	@ClientString(id = 1620005, message = "You were ingested by the Earth Wyrm.")
	public static NpcStringId YOU_WERE_INGESTED_BY_THE_EARTH_WYRM;
	
	@ClientString(id = 1620006, message = "Eliminate those who protect the heart of the Earth Wyrm.")
	public static NpcStringId ELIMINATE_THOSE_WHO_PROTECT_THE_HEART_OF_THE_EARTH_WYRM;
	
	@ClientString(id = 1620007, message = "That was a crazy trip on the way out.")
	public static NpcStringId THAT_WAS_A_CRAZY_TRIP_ON_THE_WAY_OUT;
	
	@ClientString(id = 1620008, message = "The field appears to have surpassed its limits.")
	public static NpcStringId THE_FIELD_APPEARS_TO_HAVE_SURPASSED_ITS_LIMITS;
	
	@ClientString(id = 1620009, message = "Earth Wyrm is gathering energy.")
	public static NpcStringId EARTH_WYRM_IS_GATHERING_ENERGY;
	
	@ClientString(id = 1620010, message = "Find the Earth Wyrm's weakness to defeat it.")
	public static NpcStringId FIND_THE_EARTH_WYRM_S_WEAKNESS_TO_DEFEAT_IT;
	
	@ClientString(id = 1620011, message = "Heart of Earth Wyrm has been destroyed.")
	public static NpcStringId HEART_OF_EARTH_WYRM_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 1620012, message = "You probably want to leave from a dreadful scene, but you have a responsibility to watch.")
	public static NpcStringId YOU_PROBABLY_WANT_TO_LEAVE_FROM_A_DREADFUL_SCENE_BUT_YOU_HAVE_A_RESPONSIBILITY_TO_WATCH;
	
	@ClientString(id = 1620013, message = "You've exceeded the maximum number of personnel.")
	public static NpcStringId YOU_VE_EXCEEDED_THE_MAXIMUM_NUMBER_OF_PERSONNEL;
	
	@ClientString(id = 1620014, message = "Yes!!! Sir!!!")
	public static NpcStringId YES_SIR;
	
	@ClientString(id = 1620015, message = "$s1 have entered currently")
	public static NpcStringId S1_HAVE_ENTERED_CURRENTLY;
	
	@ClientString(id = 1620016, message = "Violent")
	public static NpcStringId VIOLENT;
	
	@ClientString(id = 1620017, message = "Infiltrated")
	public static NpcStringId INFILTRATED;
	
	@ClientString(id = 1620018, message = "Activate Detector")
	public static NpcStringId ACTIVATE_DETECTOR;
	
	@ClientString(id = 1620019, message = "Detecting... Detecting...")
	public static NpcStringId DETECTING_DETECTING;
	
	@ClientString(id = 1620020, message = "Detecting")
	public static NpcStringId DETECTING;
	
	@ClientString(id = 1620021, message = "It has been detected from this direction")
	public static NpcStringId IT_HAS_BEEN_DETECTED_FROM_THIS_DIRECTION;
	
	@ClientString(id = 1620022, message = "Detected")
	public static NpcStringId DETECTED;
	
	@ClientString(id = 1620023, message = "It has been searched, but the distance is too far. Can't figure out the direction.")
	public static NpcStringId IT_HAS_BEEN_SEARCHED_BUT_THE_DISTANCE_IS_TOO_FAR_CAN_T_FIGURE_OUT_THE_DIRECTION;
	
	@ClientString(id = 1620024, message = "It disappears from being far away from the magnetic field of the tower.")
	public static NpcStringId IT_DISAPPEARS_FROM_BEING_FAR_AWAY_FROM_THE_MAGNETIC_FIELD_OF_THE_TOWER;
	
	@ClientString(id = 1620025, message = "Disappears after the power has depleted.")
	public static NpcStringId DISAPPEARS_AFTER_THE_POWER_HAS_DEPLETED;
	
	@ClientString(id = 1620026, message = "Could not detect.")
	public static NpcStringId COULD_NOT_DETECT;
	
	@ClientString(id = 1620027, message = "Detected Intruder")
	public static NpcStringId DETECTED_INTRUDER;
	
	@ClientString(id = 1620028, message = "Movement Evasion to the next location")
	public static NpcStringId MOVEMENT_EVASION_TO_THE_NEXT_LOCATION;
	
	@ClientString(id = 1620029, message = "Teleportation charged")
	public static NpcStringId TELEPORTATION_CHARGED;
	
	@ClientString(id = 1620030, message = "$s1 'til self destruction")
	public static NpcStringId S1_TIL_SELF_DESTRUCTION;
	
	@ClientString(id = 1620031, message = "Automatic explosion")
	public static NpcStringId AUTOMATIC_EXPLOSION;
	
	@ClientString(id = 1620032, message = "Evasion distance secured. Move to next location.")
	public static NpcStringId EVASION_DISTANCE_SECURED_MOVE_TO_NEXT_LOCATION;
	
	@ClientString(id = 1620033, message = "Evasion distance not secured. Entering combat mode.")
	public static NpcStringId EVASION_DISTANCE_NOT_SECURED_ENTERING_COMBAT_MODE;
	
	@ClientString(id = 1620034, message = "Enter the code after clicking on it.")
	public static NpcStringId ENTER_THE_CODE_AFTER_CLICKING_ON_IT;
	
	@ClientString(id = 1620035, message = "Greetings.")
	public static NpcStringId GREETINGS;
	
	@ClientString(id = 1620036, message = "Victory.")
	public static NpcStringId VICTORY;
	
	@ClientString(id = 1620037, message = "Charge.")
	public static NpcStringId CHARGE;
	
	@ClientString(id = 1620038, message = "Optimistic.")
	public static NpcStringId OPTIMISTIC;
	
	@ClientString(id = 1620039, message = "Pessimistic.")
	public static NpcStringId PESSIMISTIC;
	
	@ClientString(id = 1620040, message = "$s1 code recognition")
	public static NpcStringId S1_CODE_RECOGNITION;
	
	@ClientString(id = 1620041, message = "$s1 code error")
	public static NpcStringId S1_CODE_ERROR;
	
	@ClientString(id = 1620042, message = "Party error")
	public static NpcStringId PARTY_ERROR;
	
	@ClientString(id = 1620043, message = "Code recognition completed")
	public static NpcStringId CODE_RECOGNITION_COMPLETED;
	
	@ClientString(id = 1620044, message = "Frequency overload code error")
	public static NpcStringId FREQUENCY_OVERLOAD_CODE_ERROR;
	
	@ClientString(id = 1620045, message = "Frequency overload code error")
	public static NpcStringId FREQUENCY_OVERLOAD_CODE_ERROR2;
	
	@ClientString(id = 1620046, message = "*Gasp*")
	public static NpcStringId GASP;
	
	@ClientString(id = 1620047, message = "Help me!!")
	public static NpcStringId HELP_ME3;
	
	@ClientString(id = 1620048, message = "I've got a monster on my tail!")
	public static NpcStringId I_VE_GOT_A_MONSTER_ON_MY_TAIL;
	
	@ClientString(id = 1620049, message = "No way!!!")
	public static NpcStringId NO_WAY2;
	
	@ClientString(id = 1620050, message = "My god!!")
	public static NpcStringId MY_GOD;
	
	@ClientString(id = 1620051, message = "Mm.. I see")
	public static NpcStringId MM_I_SEE;
	
	@ClientString(id = 1620052, message = "Thank you for the report Rogin.")
	public static NpcStringId THANK_YOU_FOR_THE_REPORT_ROGIN;
	
	@ClientString(id = 1620053, message = "Soldiers, we're fighting a battle that can't be won.")
	public static NpcStringId SOLDIERS_WE_RE_FIGHTING_A_BATTLE_THAT_CAN_T_BE_WON;
	
	@ClientString(id = 1620054, message = "But we have to defend our village, so we're fighting.")
	public static NpcStringId BUT_WE_HAVE_TO_DEFEND_OUR_VILLAGE_SO_WE_RE_FIGHTING;
	
	@ClientString(id = 1620055, message = "For the fine wines and treasures of Aden!")
	public static NpcStringId FOR_THE_FINE_WINES_AND_TREASURES_OF_ADEN;
	
	@ClientString(id = 1620056, message = "I'm proud of every one of---")
	public static NpcStringId I_M_PROUD_OF_EVERY_ONE_OF;
	
	@ClientString(id = 1620057, message = "Ugh!! If I see you in the spirit world… first round is on me.")
	public static NpcStringId UGH_IF_I_SEE_YOU_IN_THE_SPIRIT_WORLD_FIRST_ROUND_IS_ON_ME;
	
	@ClientString(id = 1620058, message = "Rogin! I'm here!")
	public static NpcStringId ROGIN_I_M_HERE;
	
	@ClientString(id = 1620059, message = "Bronk!!!")
	public static NpcStringId BRONK;
	
	@ClientString(id = 1620060, message = "Chief!!!!")
	public static NpcStringId CHIEF;
	
	@ClientString(id = 1620061, message = "Bronk!!!")
	public static NpcStringId BRONK2;
	
	@ClientString(id = 1620062, message = "No way!!!")
	public static NpcStringId NO_WAY3;
	
	@ClientString(id = 1620063, message = "Silvera!!!!")
	public static NpcStringId SILVERA;
	
	@ClientString(id = 1620064, message = "We can't take any more losses!!!!")
	public static NpcStringId WE_CAN_T_TAKE_ANY_MORE_LOSSES;
	
	@ClientString(id = 1620065, message = "Too late..")
	public static NpcStringId TOO_LATE3;
	
	@ClientString(id = 1620066, message = "No way!!")
	public static NpcStringId NO_WAY4;
	
	@ClientString(id = 1620067, message = "Another one over there!")
	public static NpcStringId ANOTHER_ONE_OVER_THERE;
	
	@ClientString(id = 1620068, message = "For Bronk!!!")
	public static NpcStringId FOR_BRONK;
	
	@ClientString(id = 1620069, message = "Dwarves forever!!!")
	public static NpcStringId DWARVES_FOREVER;
	
	@ClientString(id = 1620070, message = "Save the Dwarven Village!!")
	public static NpcStringId SAVE_THE_DWARVEN_VILLAGE;
	
	@ClientString(id = 1620071, message = "Whoaaaaaa!!!!")
	public static NpcStringId WHOAAAAAA;
	
	@ClientString(id = 1620072, message = "Fight!!")
	public static NpcStringId FIGHT;
	
	@ClientString(id = 1620073, message = "Chief, reporting in.")
	public static NpcStringId CHIEF_REPORTING_IN;
	
	@ClientString(id = 1620074, message = "Enemies are approaching form the South!")
	public static NpcStringId ENEMIES_ARE_APPROACHING_FORM_THE_SOUTH;
	
	@ClientString(id = 1620075, message = "The Elders haven't been moved to safety.")
	public static NpcStringId THE_ELDERS_HAVEN_T_BEEN_MOVED_TO_SAFETY;
	
	@ClientString(id = 1620076, message = "Many residents still haven't left their homes.")
	public static NpcStringId MANY_RESIDENTS_STILL_HAVEN_T_LEFT_THEIR_HOMES;
	
	@ClientString(id = 1620077, message = "Chief!!!!")
	public static NpcStringId CHIEF2;
	
	@ClientString(id = 1620078, message = "Heyheyhey... Don't hit me, but love me~")
	public static NpcStringId HEYHEYHEY_DON_T_HIT_ME_BUT_LOVE_ME;
	
	@ClientString(id = 1620079, message = "Hello. Master~!")
	public static NpcStringId HELLO_MASTER;
	
	@ClientString(id = 1620080, message = "You're always so nice ^^")
	public static NpcStringId YOU_RE_ALWAYS_SO_NICE;
	
	@ClientString(id = 1620081, message = "Nice to see you too~")
	public static NpcStringId NICE_TO_SEE_YOU_TOO;
	
	@ClientString(id = 1620082, message = "Hello~ You don't have to be polite with me^^")
	public static NpcStringId HELLO_YOU_DON_T_HAVE_TO_BE_POLITE_WITH_ME;
	
	@ClientString(id = 1620083, message = "May the one strand of cool wind be with you master...")
	public static NpcStringId MAY_THE_ONE_STRAND_OF_COOL_WIND_BE_WITH_YOU_MASTER;
	
	@ClientString(id = 1620084, message = "The world revolves around you master~")
	public static NpcStringId THE_WORLD_REVOLVES_AROUND_YOU_MASTER;
	
	@ClientString(id = 1620085, message = "Master did something sad happen?")
	public static NpcStringId MASTER_DID_SOMETHING_SAD_HAPPEN;
	
	@ClientString(id = 1620086, message = "The sky becomes clear after a rain~. Let us both cheer up^^")
	public static NpcStringId THE_SKY_BECOMES_CLEAR_AFTER_A_RAIN_LET_US_BOTH_CHEER_UP;
	
	@ClientString(id = 1620087, message = "I'm beside you~")
	public static NpcStringId I_M_BESIDE_YOU;
	
	@ClientString(id = 1620088, message = "If you become the king of Aden... I will think about it..")
	public static NpcStringId IF_YOU_BECOME_THE_KING_OF_ADEN_I_WILL_THINK_ABOUT_IT;
	
	@ClientString(id = 1620089, message = "No you cannot. Master~!")
	public static NpcStringId NO_YOU_CANNOT_MASTER;
	
	@ClientString(id = 1620090, message = "Master~! -_-")
	public static NpcStringId MASTER;
	
	@ClientString(id = 1620091, message = "......")
	public static NpcStringId EMPTY3;
	
	@ClientString(id = 1620092, message = "This is a hideout made with 'Shine Stone' used in Giant's Dimensional technique.  Supposedly the capital of the Giants that have not been found is still in the dimensional gap somewhere...")
	public static NpcStringId THIS_IS_A_HIDEOUT_MADE_WITH_SHINE_STONE_USED_IN_GIANT_S_DIMENSIONAL_TECHNIQUE_SUPPOSEDLY_THE_CAPITAL_OF_THE_GIANTS_THAT_HAVE_NOT_BEEN_FOUND_IS_STILL_IN_THE_DIMENSIONAL_GAP_SOMEWHERE;
	
	@ClientString(id = 1620093, message = "The Earth Wyrm has lost consciousness.")
	public static NpcStringId THE_EARTH_WYRM_HAS_LOST_CONSCIOUSNESS;
	
	@ClientString(id = 1620094, message = "The Earth Wyrm stopped gathering energy.")
	public static NpcStringId THE_EARTH_WYRM_STOPPED_GATHERING_ENERGY;
	
	@ClientString(id = 1620095, message = "The Earth Wyrm completed gathering energy.")
	public static NpcStringId THE_EARTH_WYRM_COMPLETED_GATHERING_ENERGY;
	
	@ClientString(id = 1620096, message = "Watch the Dwarven Village Last Stand")
	public static NpcStringId WATCH_THE_DWARVEN_VILLAGE_LAST_STAND;
	
	@ClientString(id = 1620097, message = "The specter of light is shining brighter.")
	public static NpcStringId THE_SPECTER_OF_LIGHT_IS_SHINING_BRIGHTER;
	
	@ClientString(id = 1620098, message = "The specter of light is subsiding.")
	public static NpcStringId THE_SPECTER_OF_LIGHT_IS_SUBSIDING;
	
	@ClientString(id = 1620099, message = "Teredor is entering the Earth Wyrm.")
	public static NpcStringId TEREDOR_IS_ENTERING_THE_EARTH_WYRM;
	
	@ClientString(id = 1620100, message = "Teredor has entered.")
	public static NpcStringId TEREDOR_HAS_ENTERED;
	
	@ClientString(id = 1620101, message = "It is the selected time!")
	public static NpcStringId IT_IS_THE_SELECTED_TIME;
	
	@ClientString(id = 1622001, message = "There's a noble king who did not fall before the hammer of old gods")
	public static NpcStringId THERE_S_A_NOBLE_KING_WHO_DID_NOT_FALL_BEFORE_THE_HAMMER_OF_OLD_GODS;
	
	@ClientString(id = 1622002, message = "All the giants trusted their lives on that courage.")
	public static NpcStringId ALL_THE_GIANTS_TRUSTED_THEIR_LIVES_ON_THAT_COURAGE;
	
	@ClientString(id = 1622003, message = "Saving the lives with beautiful lady Belis, Slashing the Gods")
	public static NpcStringId SAVING_THE_LIVES_WITH_BEAUTIFUL_LADY_BELIS_SLASHING_THE_GODS;
	
	@ClientString(id = 1622004, message = "Even the five dragons of darkness swallowed the flames in reverence")
	public static NpcStringId EVEN_THE_FIVE_DRAGONS_OF_DARKNESS_SWALLOWED_THE_FLAMES_IN_REVERENCE;
	
	@ClientString(id = 1622005, message = "Sword rusts and magic is forgotten")
	public static NpcStringId SWORD_RUSTS_AND_MAGIC_IS_FORGOTTEN;
	
	@ClientString(id = 1622006, message = "Lady goes under the veil and the king falls on the dust")
	public static NpcStringId LADY_GOES_UNDER_THE_VEIL_AND_THE_KING_FALLS_ON_THE_DUST;
	
	@ClientString(id = 1622007, message = "Who remembers the halls of Harnak and powers of platinum")
	public static NpcStringId WHO_REMEMBERS_THE_HALLS_OF_HARNAK_AND_POWERS_OF_PLATINUM;
	
	@ClientString(id = 1622008, message = "Only the souls left to weep in the arms of eight knights")
	public static NpcStringId ONLY_THE_SOULS_LEFT_TO_WEEP_IN_THE_ARMS_OF_EIGHT_KNIGHTS;
	
	@ClientString(id = 1622009, message = "Stop enemies healing!")
	public static NpcStringId STOP_ENEMIES_HEALING;
	
	@ClientString(id = 1622010, message = "Attack weakened enemy!")
	public static NpcStringId ATTACK_WEAKENED_ENEMY;
	
	@ClientString(id = 1622011, message = "Come at me!")
	public static NpcStringId COME_AT_ME;
	
	@ClientString(id = 1622012, message = "Focus fire according to my orchestration!")
	public static NpcStringId FOCUS_FIRE_ACCORDING_TO_MY_ORCHESTRATION;
	
	@ClientString(id = 1622013, message = "I need help!")
	public static NpcStringId I_NEED_HELP;
	
	@ClientString(id = 1622014, message = "I need heal!")
	public static NpcStringId I_NEED_HEAL;
	
	@ClientString(id = 1622015, message = "I'm going to back off for a bit!")
	public static NpcStringId I_M_GOING_TO_BACK_OFF_FOR_A_BIT;
	
	@ClientString(id = 1622016, message = "Only death awaits for the weak!")
	public static NpcStringId ONLY_DEATH_AWAITS_FOR_THE_WEAK;
	
	@ClientString(id = 1622017, message = "I will attack the enemy's healer!")
	public static NpcStringId I_WILL_ATTACK_THE_ENEMY_S_HEALER;
	
	@ClientString(id = 1622018, message = "Blink attack!")
	public static NpcStringId BLINK_ATTACK;
	
	@ClientString(id = 1622019, message = "Switch to fast shooting mode")
	public static NpcStringId SWITCH_TO_FAST_SHOOTING_MODE;
	
	@ClientString(id = 1622020, message = "Switch to fast casting mode")
	public static NpcStringId SWITCH_TO_FAST_CASTING_MODE;
	
	@ClientString(id = 1622021, message = "When the song of giant is finished, Ghost of Harnak appeared.")
	public static NpcStringId WHEN_THE_SONG_OF_GIANT_IS_FINISHED_GHOST_OF_HARNAK_APPEARED;
	
	@ClientString(id = 1622022, message = "When the Knights of Seal fall, Harnak gets stronger.")
	public static NpcStringId WHEN_THE_KNIGHTS_OF_SEAL_FALL_HARNAK_GETS_STRONGER;
	
	@ClientString(id = 1622023, message = "Knight of seal falls. Harnak, who absorbs the powers and skills of the knight, becomes stronger.")
	public static NpcStringId KNIGHT_OF_SEAL_FALLS_HARNAK_WHO_ABSORBS_THE_POWERS_AND_SKILLS_OF_THE_KNIGHT_BECOMES_STRONGER;
	
	@ClientString(id = 1622024, message = "Ghost of Harnak has appeared.")
	public static NpcStringId GHOST_OF_HARNAK_HAS_APPEARED;
	
	@ClientString(id = 1622201, message = "All seals have been broken. Ghost of Harnak will appear soon")
	public static NpcStringId ALL_SEALS_HAVE_BEEN_BROKEN_GHOST_OF_HARNAK_WILL_APPEAR_SOON;
	
	@ClientString(id = 1622202, message = "All jails are open. Ghost of Harnak will appear soon")
	public static NpcStringId ALL_JAILS_ARE_OPEN_GHOST_OF_HARNAK_WILL_APPEAR_SOON;
	
	@ClientString(id = 1622203, message = "Ghost of Harnak can only be hit near Soul Circle.")
	public static NpcStringId GHOST_OF_HARNAK_CAN_ONLY_BE_HIT_NEAR_SOUL_CIRCLE;
	
	@ClientString(id = 1630001, message = "The Dimensional Door has opened near $s1!")
	public static NpcStringId THE_DIMENSIONAL_DOOR_HAS_OPENED_NEAR_S1;
	
	@ClientString(id = 1630002, message = "$s1 is what has been in my possession!")
	public static NpcStringId S1_IS_WHAT_HAS_BEEN_IN_MY_POSSESSION;
	
	@ClientString(id = 1717800, message = "We cannot resonate with voice at the current state.")
	public static NpcStringId WE_CANNOT_RESONATE_WITH_VOICE_AT_THE_CURRENT_STATE;
	
	@ClientString(id = 1717801, message = "$s1 exceeded 37.0 degrees. Incubation has failed.")
	public static NpcStringId S1_EXCEEDED_37_0_DEGREES_INCUBATION_HAS_FAILED;
	
	@ClientString(id = 1717802, message = "$s1 got to exactly 37.0 degrees.")
	public static NpcStringId S1_GOT_TO_EXACTLY_37_0_DEGREES;
	
	@ClientString(id = 1717803, message = "$s1 has succeeded at incubation.")
	public static NpcStringId S1_HAS_SUCCEEDED_AT_INCUBATION;
	
	@ClientString(id = 1717804, message = "$s1 has failed at incubation.")
	public static NpcStringId S1_HAS_FAILED_AT_INCUBATION;
	
	@ClientString(id = 1717805, message = "The time limit was exceeded. Incubation has failed.")
	public static NpcStringId THE_TIME_LIMIT_WAS_EXCEEDED_INCUBATION_HAS_FAILED;
	
	@ClientString(id = 1717806, message = "The winner of the Ceremony of Chaos has been born.")
	public static NpcStringId THE_WINNER_OF_THE_CEREMONY_OF_CHAOS_HAS_BEEN_BORN;
	
	@ClientString(id = 1717807, message = "What, new sacrificial offerings for Lord Triol?")
	public static NpcStringId WHAT_NEW_SACRIFICIAL_OFFERINGS_FOR_LORD_TRIOL;
	
	@ClientString(id = 1717808, message = "You'll be brainwashed just by the sound of Lord Triol's voice!")
	public static NpcStringId YOU_LL_BE_BRAINWASHED_JUST_BY_THE_SOUND_OF_LORD_TRIOL_S_VOICE;
	
	@ClientString(id = 1717809, message = "You're nothing but sacrifices to be offered up to Lord Triol! I'll chew you bone to bone! Except for your earrings.")
	public static NpcStringId YOU_RE_NOTHING_BUT_SACRIFICES_TO_BE_OFFERED_UP_TO_LORD_TRIOL_I_LL_CHEW_YOU_BONE_TO_BONE_EXCEPT_FOR_YOUR_EARRINGS;
	
	@ClientString(id = 1717810, message = "Lord Triol, please let us hear your voice again!")
	public static NpcStringId LORD_TRIOL_PLEASE_LET_US_HEAR_YOUR_VOICE_AGAIN;
	
	@ClientString(id = 1717811, message = "We must find creatures to sacrifice to Lord Triol.")
	public static NpcStringId WE_MUST_FIND_CREATURES_TO_SACRIFICE_TO_LORD_TRIOL;
	
	@ClientString(id = 1717812, message = "Is this where I can find creatures to sacrifice for Lord Triol?")
	public static NpcStringId IS_THIS_WHERE_I_CAN_FIND_CREATURES_TO_SACRIFICE_FOR_LORD_TRIOL;
	
	@ClientString(id = 1717813, message = "Found you! Consider it an honor to be offered up to Lord Triol.")
	public static NpcStringId FOUND_YOU_CONSIDER_IT_AN_HONOR_TO_BE_OFFERED_UP_TO_LORD_TRIOL;
	
	@ClientString(id = 1717814, message = "So weak-looking… must be easy to snatch away.")
	public static NpcStringId SO_WEAK_LOOKING_MUST_BE_EASY_TO_SNATCH_AWAY;
	
	@ClientString(id = 1717815, message = "Where did they go? Hmm… we'll withdraw for now. Can't find anything to serve as offerings here.")
	public static NpcStringId WHERE_DID_THEY_GO_HMM_WE_LL_WITHDRAW_FOR_NOW_CAN_T_FIND_ANYTHING_TO_SERVE_AS_OFFERINGS_HERE;
	
	@ClientString(id = 1717816, message = "I heard this is where I could find them… I guess I'll have to look elsewhere.")
	public static NpcStringId I_HEARD_THIS_IS_WHERE_I_COULD_FIND_THEM_I_GUESS_I_LL_HAVE_TO_LOOK_ELSEWHERE;
	
	@ClientString(id = 1717817, message = "Come out, Triol's Troopers! Let's bring this creature to Lord Triol!")
	public static NpcStringId COME_OUT_TRIOL_S_TROOPERS_LET_S_BRING_THIS_CREATURE_TO_LORD_TRIOL;
	
	@ClientString(id = 1800001, message = "(Queen Ant)  # $s1's Command Channel has looting rights.")
	public static NpcStringId QUEEN_ANT_S1_S_COMMAND_CHANNEL_HAS_LOOTING_RIGHTS;
	
	@ClientString(id = 1800002, message = "(Core)  # $s1's Command Channel has looting rights.")
	public static NpcStringId CORE_S1_S_COMMAND_CHANNEL_HAS_LOOTING_RIGHTS;
	
	@ClientString(id = 1800003, message = "(Orphen)  # $s1's Command Channel has looting rights.")
	public static NpcStringId ORPHEN_S1_S_COMMAND_CHANNEL_HAS_LOOTING_RIGHTS;
	
	@ClientString(id = 1800004, message = "(Zaken)  # $s1's Command Channel has looting rights.")
	public static NpcStringId ZAKEN_S1_S_COMMAND_CHANNEL_HAS_LOOTING_RIGHTS;
	
	@ClientString(id = 1800005, message = "(Queen Ant) Looting rules are no longer active.")
	public static NpcStringId QUEEN_ANT_LOOTING_RULES_ARE_NO_LONGER_ACTIVE;
	
	@ClientString(id = 1800006, message = "(Core) Looting rules are no longer active.")
	public static NpcStringId CORE_LOOTING_RULES_ARE_NO_LONGER_ACTIVE;
	
	@ClientString(id = 1800007, message = "(Orphen) Looting rules are no longer active.")
	public static NpcStringId ORPHEN_LOOTING_RULES_ARE_NO_LONGER_ACTIVE;
	
	@ClientString(id = 1800008, message = "(Zaken) Looting rules are no longer active.")
	public static NpcStringId ZAKEN_LOOTING_RULES_ARE_NO_LONGER_ACTIVE;
	
	@ClientString(id = 1800009, message = "# $s1's Command Channel has looting rights.")
	public static NpcStringId S1_S_COMMAND_CHANNEL_HAS_LOOTING_RIGHTS;
	
	@ClientString(id = 1800010, message = "Looting rules are no longer active.")
	public static NpcStringId LOOTING_RULES_ARE_NO_LONGER_ACTIVE;
	
	@ClientString(id = 1800011, message = "Our master now comes to claim our vengeance. Soon you will all be nothing more than dirt...")
	public static NpcStringId OUR_MASTER_NOW_COMES_TO_CLAIM_OUR_VENGEANCE_SOON_YOU_WILL_ALL_BE_NOTHING_MORE_THAN_DIRT;
	
	@ClientString(id = 1800012, message = "Death to those who challenge the Lords of Dawn!")
	public static NpcStringId DEATH_TO_THOSE_WHO_CHALLENGE_THE_LORDS_OF_DAWN;
	
	@ClientString(id = 1800013, message = "Death to those who challenge the Lord!")
	public static NpcStringId DEATH_TO_THOSE_WHO_CHALLENGE_THE_LORD;
	
	@ClientString(id = 1800014, message = "Oink oink! Pigs can do it too! Antharas-Surpassing Super Powered Pig Stun! How do like them apples?")
	public static NpcStringId OINK_OINK_PIGS_CAN_DO_IT_TOO_ANTHARAS_SURPASSING_SUPER_POWERED_PIG_STUN_HOW_DO_LIKE_THEM_APPLES;
	
	@ClientString(id = 1800015, message = "Oink oink! Take that! Valakas-Terrorizing Ultra Pig Fear! Ha ha!")
	public static NpcStringId OINK_OINK_TAKE_THAT_VALAKAS_TERRORIZING_ULTRA_PIG_FEAR_HA_HA;
	
	@ClientString(id = 1800016, message = "Oink oink! Go away! Stop bothering me!")
	public static NpcStringId OINK_OINK_GO_AWAY_STOP_BOTHERING_ME;
	
	@ClientString(id = 1800017, message = "Oink oink! Pigs of the world unite! Let's show them our strength!")
	public static NpcStringId OINK_OINK_PIGS_OF_THE_WORLD_UNITE_LET_S_SHOW_THEM_OUR_STRENGTH;
	
	@ClientString(id = 1800018, message = "You healed me. Thanks a lot! Oink oink!")
	public static NpcStringId YOU_HEALED_ME_THANKS_A_LOT_OINK_OINK;
	
	@ClientString(id = 1800019, message = "Oink oink! This treatment hurts too much! Are you sure that you're trying to heal me?")
	public static NpcStringId OINK_OINK_THIS_TREATMENT_HURTS_TOO_MUCH_ARE_YOU_SURE_THAT_YOU_RE_TRYING_TO_HEAL_ME;
	
	@ClientString(id = 1800020, message = "Oink oink! Transform with Moon Crystal Prism Power!")
	public static NpcStringId OINK_OINK_TRANSFORM_WITH_MOON_CRYSTAL_PRISM_POWER;
	
	@ClientString(id = 1800021, message = "Oink oink! Nooo! I don't want to go back to normal!")
	public static NpcStringId OINK_OINK_NOOO_I_DON_T_WANT_TO_GO_BACK_TO_NORMAL;
	
	@ClientString(id = 1800022, message = "Oink oink! I'm rich, so I've got plenty to share! Thanks!")
	public static NpcStringId OINK_OINK_I_M_RICH_SO_I_VE_GOT_PLENTY_TO_SHARE_THANKS;
	
	@ClientString(id = 1800023, message = "It's a weapon surrounded by an ominous aura. I'll discard it because it may be dangerous. Miss...!")
	public static NpcStringId IT_S_A_WEAPON_SURROUNDED_BY_AN_OMINOUS_AURA_I_LL_DISCARD_IT_BECAUSE_IT_MAY_BE_DANGEROUS_MISS;
	
	@ClientString(id = 1800024, message = "Thank you for saving me from the clutches of evil!")
	public static NpcStringId THANK_YOU_FOR_SAVING_ME_FROM_THE_CLUTCHES_OF_EVIL;
	
	@ClientString(id = 1800025, message = "That is it for today...let's retreat. Everyone pull back!")
	public static NpcStringId THAT_IS_IT_FOR_TODAY_LET_S_RETREAT_EVERYONE_PULL_BACK;
	
	@ClientString(id = 1800026, message = "Thank you for the rescue. It's a small gift.")
	public static NpcStringId THANK_YOU_FOR_THE_RESCUE_IT_S_A_SMALL_GIFT;
	
	@ClientString(id = 1800027, message = "$s1.. You don't have a Red Crystal...")
	public static NpcStringId S1_YOU_DON_T_HAVE_A_RED_CRYSTAL;
	
	@ClientString(id = 1800028, message = "$s1.. You don't have a Blue Crystal...")
	public static NpcStringId S1_YOU_DON_T_HAVE_A_BLUE_CRYSTAL;
	
	@ClientString(id = 1800029, message = "$s1.. You don't have a Clear Crystal...")
	public static NpcStringId S1_YOU_DON_T_HAVE_A_CLEAR_CRYSTAL;
	
	@ClientString(id = 1800030, message = "$s1.. If you are too far away from me...I can't let you go...")
	public static NpcStringId S1_IF_YOU_ARE_TOO_FAR_AWAY_FROM_ME_I_CAN_T_LET_YOU_GO;
	
	@ClientString(id = 1800031, message = "An alarm has been set off! Everybody will be in danger if they are not taken care of immediately!")
	public static NpcStringId AN_ALARM_HAS_BEEN_SET_OFF_EVERYBODY_WILL_BE_IN_DANGER_IF_THEY_ARE_NOT_TAKEN_CARE_OF_IMMEDIATELY;
	
	@ClientString(id = 1800032, message = "It will not be that easy to kill me!")
	public static NpcStringId IT_WILL_NOT_BE_THAT_EASY_TO_KILL_ME;
	
	@ClientString(id = 1800033, message = "No...You knew my weakness...")
	public static NpcStringId NO_YOU_KNEW_MY_WEAKNESS;
	
	@ClientString(id = 1800034, message = "Hello? Is anyone there?")
	public static NpcStringId HELLO_IS_ANYONE_THERE;
	
	@ClientString(id = 1800035, message = "Is no one there? How long have I been hiding? I have been starving for days and cannot hold out anymore!")
	public static NpcStringId IS_NO_ONE_THERE_HOW_LONG_HAVE_I_BEEN_HIDING_I_HAVE_BEEN_STARVING_FOR_DAYS_AND_CANNOT_HOLD_OUT_ANYMORE;
	
	@ClientString(id = 1800036, message = "If someone would give me some of those tasty Crystal Fragments, I would gladly tell them where Tears is hiding! Yummy, yummy!")
	public static NpcStringId IF_SOMEONE_WOULD_GIVE_ME_SOME_OF_THOSE_TASTY_CRYSTAL_FRAGMENTS_I_WOULD_GLADLY_TELL_THEM_WHERE_TEARS_IS_HIDING_YUMMY_YUMMY;
	
	@ClientString(id = 1800037, message = "Hey! You from above the ground! Let's share some Crystal Fragments, if you have any.")
	public static NpcStringId HEY_YOU_FROM_ABOVE_THE_GROUND_LET_S_SHARE_SOME_CRYSTAL_FRAGMENTS_IF_YOU_HAVE_ANY;
	
	@ClientString(id = 1800038, message = "Crispy and cold feeling! Teehee! Delicious!!")
	public static NpcStringId CRISPY_AND_COLD_FEELING_TEEHEE_DELICIOUS;
	
	@ClientString(id = 1800039, message = "Yummy! This is so tasty.")
	public static NpcStringId YUMMY_THIS_IS_SO_TASTY;
	
	@ClientString(id = 1800040, message = "Sniff, sniff! Give me more Crystal Fragments!")
	public static NpcStringId SNIFF_SNIFF_GIVE_ME_MORE_CRYSTAL_FRAGMENTS;
	
	@ClientString(id = 1800041, message = "How insensitive! It's not nice to give me just a piece! Can't you give me more?")
	public static NpcStringId HOW_INSENSITIVE_IT_S_NOT_NICE_TO_GIVE_ME_JUST_A_PIECE_CAN_T_YOU_GIVE_ME_MORE;
	
	@ClientString(id = 1800042, message = "Ah - I'm hungry!")
	public static NpcStringId AH_I_M_HUNGRY;
	
	@ClientString(id = 1800043, message = "I'm the real one!")
	public static NpcStringId I_M_THE_REAL_ONE;
	
	@ClientString(id = 1800044, message = "Pick me!")
	public static NpcStringId PICK_ME;
	
	@ClientString(id = 1800045, message = "Trust me!")
	public static NpcStringId TRUST_ME;
	
	@ClientString(id = 1800046, message = "Not that dude, I'm the real one!")
	public static NpcStringId NOT_THAT_DUDE_I_M_THE_REAL_ONE;
	
	@ClientString(id = 1800047, message = "Don't be fooled! Don't be fooled! I'm the real one!!")
	public static NpcStringId DON_T_BE_FOOLED_DON_T_BE_FOOLED_I_M_THE_REAL_ONE;
	
	@ClientString(id = 1800048, message = "Just act like the real one! Oops!")
	public static NpcStringId JUST_ACT_LIKE_THE_REAL_ONE_OOPS;
	
	@ClientString(id = 1800049, message = "You've been fooled!")
	public static NpcStringId YOU_VE_BEEN_FOOLED;
	
	@ClientString(id = 1800050, message = "Sorry, but...I'm the fake one.")
	public static NpcStringId SORRY_BUT_I_M_THE_FAKE_ONE;
	
	@ClientString(id = 1800051, message = "I'm the real one! Phew!!")
	public static NpcStringId I_M_THE_REAL_ONE_PHEW;
	
	@ClientString(id = 1800052, message = "Can't you even find out?")
	public static NpcStringId CAN_T_YOU_EVEN_FIND_OUT;
	
	@ClientString(id = 1800053, message = "Find me!")
	public static NpcStringId FIND_ME;
	
	@ClientString(id = 1800054, message = "Huh?! How did you know it was me?")
	public static NpcStringId HUH_HOW_DID_YOU_KNOW_IT_WAS_ME;
	
	@ClientString(id = 1800055, message = "Excellent choice! Teehee!")
	public static NpcStringId EXCELLENT_CHOICE_TEEHEE;
	
	@ClientString(id = 1800056, message = "You've done well!")
	public static NpcStringId YOU_VE_DONE_WELL;
	
	@ClientString(id = 1800057, message = "Oh... very sensible?")
	public static NpcStringId OH_VERY_SENSIBLE;
	
	@ClientString(id = 1800058, message = "Behold the mighty power of Baylor, foolish mortal!")
	public static NpcStringId BEHOLD_THE_MIGHTY_POWER_OF_BAYLOR_FOOLISH_MORTAL;
	
	@ClientString(id = 1800059, message = "No one is going to survive!")
	public static NpcStringId NO_ONE_IS_GOING_TO_SURVIVE;
	
	@ClientString(id = 1800060, message = "You'll see what hell is like...")
	public static NpcStringId YOU_LL_SEE_WHAT_HELL_IS_LIKE;
	
	@ClientString(id = 1800061, message = "You will be put in jail!")
	public static NpcStringId YOU_WILL_BE_PUT_IN_JAIL;
	
	@ClientString(id = 1800062, message = "Worthless creature... Go to hell!")
	public static NpcStringId WORTHLESS_CREATURE_GO_TO_HELL;
	
	@ClientString(id = 1800063, message = "I'll give you something that you'll never forget!")
	public static NpcStringId I_LL_GIVE_YOU_SOMETHING_THAT_YOU_LL_NEVER_FORGET;
	
	@ClientString(id = 1800064, message = "Why did you trust to choose me? Hahahaha....")
	public static NpcStringId WHY_DID_YOU_TRUST_TO_CHOOSE_ME_HAHAHAHA;
	
	@ClientString(id = 1800065, message = "I'll make you regret that you ever chose me...")
	public static NpcStringId I_LL_MAKE_YOU_REGRET_THAT_YOU_EVER_CHOSE_ME;
	
	@ClientString(id = 1800066, message = "Don't expect to get out alive..")
	public static NpcStringId DON_T_EXPECT_TO_GET_OUT_ALIVE;
	
	@ClientString(id = 1800067, message = "Demon King Beleth, give me the power! Aaahh!!!")
	public static NpcStringId DEMON_KING_BELETH_GIVE_ME_THE_POWER_AAAHH;
	
	@ClientString(id = 1800068, message = "No....... I feel the power of Fafurion.......")
	public static NpcStringId NO_I_FEEL_THE_POWER_OF_FAFURION;
	
	@ClientString(id = 1800069, message = "Fafurion! Please give power to this helpless witch!!")
	public static NpcStringId FAFURION_PLEASE_GIVE_POWER_TO_THIS_HELPLESS_WITCH;
	
	@ClientString(id = 1800070, message = "I can't help you much, but I can weaken the power of Baylor since I'm locked up here.")
	public static NpcStringId I_CAN_T_HELP_YOU_MUCH_BUT_I_CAN_WEAKEN_THE_POWER_OF_BAYLOR_SINCE_I_M_LOCKED_UP_HERE;
	
	@ClientString(id = 1800071, message = "Your skill is impressive. I'll admit that you are good enough to pass. Take the key and leave this place.")
	public static NpcStringId YOUR_SKILL_IS_IMPRESSIVE_I_LL_ADMIT_THAT_YOU_ARE_GOOD_ENOUGH_TO_PASS_TAKE_THE_KEY_AND_LEAVE_THIS_PLACE;
	
	@ClientString(id = 1800072, message = "Give me all you have! It's the only way I'll let you live!!")
	public static NpcStringId GIVE_ME_ALL_YOU_HAVE_IT_S_THE_ONLY_WAY_I_LL_LET_YOU_LIVE;
	
	@ClientString(id = 1800073, message = "Hun.. hungry")
	public static NpcStringId HUN_HUNGRY;
	
	@ClientString(id = 1800074, message = "Don't be lazy! You bastards!")
	public static NpcStringId DON_T_BE_LAZY_YOU_BASTARDS;
	
	@ClientString(id = 1800075, message = "They are just henchmen of the Iron Castle... Why did we hide?")
	public static NpcStringId THEY_ARE_JUST_HENCHMEN_OF_THE_IRON_CASTLE_WHY_DID_WE_HIDE;
	
	@ClientString(id = 1800076, message = "Guys, show them our power!!!!")
	public static NpcStringId GUYS_SHOW_THEM_OUR_POWER;
	
	@ClientString(id = 1800077, message = "You have finally come here. But you will not be able to find the secret room!")
	public static NpcStringId YOU_HAVE_FINALLY_COME_HERE_BUT_YOU_WILL_NOT_BE_ABLE_TO_FIND_THE_SECRET_ROOM;
	
	@ClientString(id = 1800078, message = "You have done well in finding me, but I cannot just hand you the key!")
	public static NpcStringId YOU_HAVE_DONE_WELL_IN_FINDING_ME_BUT_I_CANNOT_JUST_HAND_YOU_THE_KEY;
	
	@ClientString(id = 1800079, message = "$s1 second(s) remaining.")
	public static NpcStringId S1_SECOND_S_REMAINING;
	
	@ClientString(id = 1800080, message = "Match begins in $s1 minute(s). Please gather around the administrator.")
	public static NpcStringId MATCH_BEGINS_IN_S1_MINUTE_S_PLEASE_GATHER_AROUND_THE_ADMINISTRATOR;
	
	@ClientString(id = 1800081, message = "The match is automatically canceled because you are too far from the admission manager.")
	public static NpcStringId THE_MATCH_IS_AUTOMATICALLY_CANCELED_BECAUSE_YOU_ARE_TOO_FAR_FROM_THE_ADMISSION_MANAGER;
	
	@ClientString(id = 1800082, message = "May I have your attention please...")
	public static NpcStringId MAY_I_HAVE_YOUR_ATTENTION_PLEASE;
	
	@ClientString(id = 1800083, message = "Thank you all for coming here tonight.")
	public static NpcStringId THANK_YOU_ALL_FOR_COMING_HERE_TONIGHT;
	
	@ClientString(id = 1800084, message = "It is an honor to have you all in attendance this evening.")
	public static NpcStringId IT_IS_AN_HONOR_TO_HAVE_YOU_ALL_IN_ATTENDANCE_THIS_EVENING;
	
	@ClientString(id = 1800085, message = "Fantasy Isle takes great pride in providing the very best in entertainment.")
	public static NpcStringId FANTASY_ISLE_TAKES_GREAT_PRIDE_IN_PROVIDING_THE_VERY_BEST_IN_ENTERTAINMENT;
	
	@ClientString(id = 1800086, message = "Now I'd like to introduce the most beautiful singer in Aden. Please welcome... Leyla Mira!")
	public static NpcStringId NOW_I_D_LIKE_TO_INTRODUCE_THE_MOST_BEAUTIFUL_SINGER_IN_ADEN_PLEASE_WELCOME_LEYLA_MIRA;
	
	@ClientString(id = 1800087, message = "Performing her latest hit!")
	public static NpcStringId PERFORMING_HER_LATEST_HIT;
	
	@ClientString(id = 1800088, message = "Thank you very much, Leyla!")
	public static NpcStringId THANK_YOU_VERY_MUCH_LEYLA;
	
	@ClientString(id = 1800089, message = "Now we're in for a real treat.")
	public static NpcStringId NOW_WE_RE_IN_FOR_A_REAL_TREAT;
	
	@ClientString(id = 1800090, message = "Just back from their world tour... put your hands together for the Fantasy Isle Circus!")
	public static NpcStringId JUST_BACK_FROM_THEIR_WORLD_TOUR_PUT_YOUR_HANDS_TOGETHER_FOR_THE_FANTASY_ISLE_CIRCUS;
	
	@ClientString(id = 1800091, message = "Everyone give them a hand!")
	public static NpcStringId EVERYONE_GIVE_THEM_A_HAND;
	
	@ClientString(id = 1800092, message = "Impressive, no? A very talented group.")
	public static NpcStringId IMPRESSIVE_NO_A_VERY_TALENTED_GROUP;
	
	@ClientString(id = 1800093, message = "Now we have a special request from some folks who wish to perform.")
	public static NpcStringId NOW_WE_HAVE_A_SPECIAL_REQUEST_FROM_SOME_FOLKS_WHO_WISH_TO_PERFORM;
	
	@ClientString(id = 1800094, message = "Let's welcome our first volunteer.")
	public static NpcStringId LET_S_WELCOME_OUR_FIRST_VOLUNTEER;
	
	@ClientString(id = 1800095, message = "Ah...")
	public static NpcStringId AH;
	
	@ClientString(id = 1800096, message = "Moving on. Let's welcome our next volunteer.")
	public static NpcStringId MOVING_ON_LET_S_WELCOME_OUR_NEXT_VOLUNTEER;
	
	@ClientString(id = 1800097, message = "I'm sure they have something much more exciting.")
	public static NpcStringId I_M_SURE_THEY_HAVE_SOMETHING_MUCH_MORE_EXCITING;
	
	@ClientString(id = 1800098, message = "Really...")
	public static NpcStringId REALLY;
	
	@ClientString(id = 1800099, message = "That was just... super. And now for our last volunteer.")
	public static NpcStringId THAT_WAS_JUST_SUPER_AND_NOW_FOR_OUR_LAST_VOLUNTEER;
	
	@ClientString(id = 1800100, message = "... ... ... sigh. And that's our show folks!")
	public static NpcStringId SIGH_AND_THAT_S_OUR_SHOW_FOLKS;
	
	@ClientString(id = 1800101, message = "I hope everyone enjoyed the show!")
	public static NpcStringId I_HOPE_EVERYONE_ENJOYED_THE_SHOW;
	
	@ClientString(id = 1800102, message = "Please remember that Fantasy Isle is always planning a lot of great shows for you.")
	public static NpcStringId PLEASE_REMEMBER_THAT_FANTASY_ISLE_IS_ALWAYS_PLANNING_A_LOT_OF_GREAT_SHOWS_FOR_YOU;
	
	@ClientString(id = 1800103, message = "Well, we wish we could keep this party going but we must take a break. Thank you.")
	public static NpcStringId WELL_WE_WISH_WE_COULD_KEEP_THIS_PARTY_GOING_BUT_WE_MUST_TAKE_A_BREAK_THANK_YOU;
	
	@ClientString(id = 1800104, message = "We love you all!")
	public static NpcStringId WE_LOVE_YOU_ALL;
	
	@ClientString(id = 1800105, message = "Ladies and Gentlemen... the show is about to begin!")
	public static NpcStringId LADIES_AND_GENTLEMEN_THE_SHOW_IS_ABOUT_TO_BEGIN;
	
	@ClientString(id = 1800106, message = "The opponent team canceled the match.")
	public static NpcStringId THE_OPPONENT_TEAM_CANCELED_THE_MATCH;
	
	@ClientString(id = 1800107, message = "It's not easy to obtain.")
	public static NpcStringId IT_S_NOT_EASY_TO_OBTAIN;
	
	@ClientString(id = 1800108, message = "You're out of your mind coming here...")
	public static NpcStringId YOU_RE_OUT_OF_YOUR_MIND_COMING_HERE;
	
	@ClientString(id = 1800109, message = "Enemy invasion! Hurry up!")
	public static NpcStringId ENEMY_INVASION_HURRY_UP;
	
	@ClientString(id = 1800110, message = "Process... shouldn't... be delayed... because of me...")
	public static NpcStringId PROCESS_SHOULDN_T_BE_DELAYED_BECAUSE_OF_ME;
	
	@ClientString(id = 1800111, message = "Alright, now Leodas is yours!")
	public static NpcStringId ALRIGHT_NOW_LEODAS_IS_YOURS;
	
	@ClientString(id = 1800112, message = "We might need new slaves... I'll be back soon, so wait!")
	public static NpcStringId WE_MIGHT_NEED_NEW_SLAVES_I_LL_BE_BACK_SOON_SO_WAIT;
	
	@ClientString(id = 1800113, message = "Time rift device activation successful!")
	public static NpcStringId TIME_RIFT_DEVICE_ACTIVATION_SUCCESSFUL;
	
	@ClientString(id = 1800114, message = "Freedom or death!!!")
	public static NpcStringId FREEDOM_OR_DEATH;
	
	@ClientString(id = 1800115, message = "This is the will of true warriors!!!")
	public static NpcStringId THIS_IS_THE_WILL_OF_TRUE_WARRIORS;
	
	@ClientString(id = 1800116, message = "We'll have dinner in hell!!!")
	public static NpcStringId WE_LL_HAVE_DINNER_IN_HELL;
	
	@ClientString(id = 1800117, message = "Detonator initialization- time set for $s1 minute(s) from now-")
	public static NpcStringId DETONATOR_INITIALIZATION_TIME_SET_FOR_S1_MINUTE_S_FROM_NOW;
	
	@ClientString(id = 1800118, message = "Zzzz- city interference error - forward effect created-")
	public static NpcStringId ZZZZ_CITY_INTERFERENCE_ERROR_FORWARD_EFFECT_CREATED;
	
	@ClientString(id = 1800119, message = "Zzzz- city interference error - recurrence effect created-")
	public static NpcStringId ZZZZ_CITY_INTERFERENCE_ERROR_RECURRENCE_EFFECT_CREATED;
	
	@ClientString(id = 1800120, message = "Guards are coming, run!")
	public static NpcStringId GUARDS_ARE_COMING_RUN;
	
	@ClientString(id = 1800121, message = "Now I can escape on my own!")
	public static NpcStringId NOW_I_CAN_ESCAPE_ON_MY_OWN;
	
	@ClientString(id = 1800122, message = "Thanks for your help!")
	public static NpcStringId THANKS_FOR_YOUR_HELP;
	
	@ClientString(id = 1800123, message = "Match cancelled. Opponent did not meet the stadium admission requirements.")
	public static NpcStringId MATCH_CANCELLED_OPPONENT_DID_NOT_MEET_THE_STADIUM_ADMISSION_REQUIREMENTS;
	
	@ClientString(id = 1800124, message = "Ha-ha yes, die slowly writhing in pain and agony!")
	public static NpcStringId HA_HA_YES_DIE_SLOWLY_WRITHING_IN_PAIN_AND_AGONY;
	
	@ClientString(id = 1800125, message = "More... need more... severe pain...")
	public static NpcStringId MORE_NEED_MORE_SEVERE_PAIN;
	
	@ClientString(id = 1800126, message = "Ahh! My life is being drained out!")
	public static NpcStringId AHH_MY_LIFE_IS_BEING_DRAINED_OUT;
	
	@ClientString(id = 1800127, message = "Something is burning inside my body!")
	public static NpcStringId SOMETHING_IS_BURNING_INSIDE_MY_BODY;
	
	@ClientString(id = 1800128, message = "$s1. Not adequate for the stadium level.")
	public static NpcStringId S1_NOT_ADEQUATE_FOR_THE_STADIUM_LEVEL;
	
	@ClientString(id = 1800129, message = "$s1, thank you... for giving me your life...")
	public static NpcStringId S1_THANK_YOU_FOR_GIVING_ME_YOUR_LIFE;
	
	@ClientString(id = 1800130, message = "I failed... Please forgive me, Darion...")
	public static NpcStringId I_FAILED_PLEASE_FORGIVE_ME_DARION;
	
	@ClientString(id = 1800131, message = "$s1, I'll be back... don't get comfortable...")
	public static NpcStringId S1_I_LL_BE_BACK_DON_T_GET_COMFORTABLE;
	
	@ClientString(id = 1800132, message = "If you think I'm giving up like this.. You're wrong!!")
	public static NpcStringId IF_YOU_THINK_I_M_GIVING_UP_LIKE_THIS_YOU_RE_WRONG;
	
	@ClientString(id = 1800133, message = "So you're just going to watch me suffer?!")
	public static NpcStringId SO_YOU_RE_JUST_GOING_TO_WATCH_ME_SUFFER;
	
	@ClientString(id = 1800134, message = "It's not over yet!!")
	public static NpcStringId IT_S_NOT_OVER_YET;
	
	@ClientString(id = 1800135, message = "HA-HA! You were so afraid of death... let me see... If you find me in time... maybe you can... find a way ...")
	public static NpcStringId HA_HA_YOU_WERE_SO_AFRAID_OF_DEATH_LET_ME_SEE_IF_YOU_FIND_ME_IN_TIME_MAYBE_YOU_CAN_FIND_A_WAY;
	
	@ClientString(id = 1800136, message = "Don't kill me please.. Something's strangling me...")
	public static NpcStringId DON_T_KILL_ME_PLEASE_SOMETHING_S_STRANGLING_ME;
	
	@ClientString(id = 1800137, message = "Who will be the lucky one tonight? Ha-ha! Curious, very curious!")
	public static NpcStringId WHO_WILL_BE_THE_LUCKY_ONE_TONIGHT_HA_HA_CURIOUS_VERY_CURIOUS;
	
	@ClientString(id = 1800138, message = "Squeak! This will be stronger than the stun the pig used last time!")
	public static NpcStringId SQUEAK_THIS_WILL_BE_STRONGER_THAN_THE_STUN_THE_PIG_USED_LAST_TIME;
	
	@ClientString(id = 1800139, message = "Squeak! Here it goes! Extremely scary, even to Valakas!")
	public static NpcStringId SQUEAK_HERE_IT_GOES_EXTREMELY_SCARY_EVEN_TO_VALAKAS;
	
	@ClientString(id = 1800140, message = "Squeak! Go away! Leave us alone!")
	public static NpcStringId SQUEAK_GO_AWAY_LEAVE_US_ALONE;
	
	@ClientString(id = 1800141, message = "Squeak! Guys, gather up! Let's show our power!")
	public static NpcStringId SQUEAK_GUYS_GATHER_UP_LET_S_SHOW_OUR_POWER;
	
	@ClientString(id = 1800142, message = "It's not like I'm giving this because I'm grateful~ Squeak!")
	public static NpcStringId IT_S_NOT_LIKE_I_M_GIVING_THIS_BECAUSE_I_M_GRATEFUL_SQUEAK;
	
	@ClientString(id = 1800143, message = "Squeak! Even if it is treatment, my bottom hurts so much!")
	public static NpcStringId SQUEAK_EVEN_IF_IT_IS_TREATMENT_MY_BOTTOM_HURTS_SO_MUCH;
	
	@ClientString(id = 1800144, message = "Squeak! Transform to Moon Crystal Prism Power!")
	public static NpcStringId SQUEAK_TRANSFORM_TO_MOON_CRYSTAL_PRISM_POWER;
	
	@ClientString(id = 1800145, message = "Squeak! Oh, no! I don't want to turn back again...")
	public static NpcStringId SQUEAK_OH_NO_I_DON_T_WANT_TO_TURN_BACK_AGAIN;
	
	@ClientString(id = 1800146, message = "Squeak! I'm specially giving you a lot since I'm rich. Thank you")
	public static NpcStringId SQUEAK_I_M_SPECIALLY_GIVING_YOU_A_LOT_SINCE_I_M_RICH_THANK_YOU;
	
	@ClientString(id = 1800147, message = "Oink-oink! Rage is boiling up inside of me! Power! Infinite power!!")
	public static NpcStringId OINK_OINK_RAGE_IS_BOILING_UP_INSIDE_OF_ME_POWER_INFINITE_POWER;
	
	@ClientString(id = 1800148, message = "Oink-oink! I'm really furious right now!")
	public static NpcStringId OINK_OINK_I_M_REALLY_FURIOUS_RIGHT_NOW;
	
	@ClientString(id = 1800149, message = "Squeak! Rage is boiling up inside of me! Power! Infinite power!!")
	public static NpcStringId SQUEAK_RAGE_IS_BOILING_UP_INSIDE_OF_ME_POWER_INFINITE_POWER;
	
	@ClientString(id = 1800150, message = "Squeak! I'm really furious right now!!")
	public static NpcStringId SQUEAK_I_M_REALLY_FURIOUS_RIGHT_NOW;
	
	@ClientString(id = 1800151, message = "Hall of Nightmares (Lv. 20-30)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_20_30;
	
	@ClientString(id = 1800152, message = "Hall of Nightmares (Lv. 25-35)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_25_35;
	
	@ClientString(id = 1800153, message = "Hall of Nightmares (Lv. 30-40)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_30_40;
	
	@ClientString(id = 1800154, message = "Hall of Nightmares (Lv. 35-45)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_35_45;
	
	@ClientString(id = 1800155, message = "Hall of Nightmares (Lv. 40-50)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_40_50;
	
	@ClientString(id = 1800156, message = "Hall of Nightmares (Lv. 45-55)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_45_55;
	
	@ClientString(id = 1800157, message = "Hall of Nightmares (Lv. 50-60)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_50_60;
	
	@ClientString(id = 1800158, message = "Hall of Nightmares (Lv. 55-65)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_55_65;
	
	@ClientString(id = 1800159, message = "Hall of Nightmares (Lv. 60-70)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_60_70;
	
	@ClientString(id = 1800160, message = "Hall of Nightmares (Lv. 65-75)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_65_75;
	
	@ClientString(id = 1800161, message = "Hall of Nightmares (Lv. 70-80)")
	public static NpcStringId HALL_OF_NIGHTMARES_LV_70_80;
	
	@ClientString(id = 1800162, message = "G Rank")
	public static NpcStringId G_RANK;
	
	@ClientString(id = 1800163, message = "Huh... No one would have guessed that a doomed creature would be so powerful...")
	public static NpcStringId HUH_NO_ONE_WOULD_HAVE_GUESSED_THAT_A_DOOMED_CREATURE_WOULD_BE_SO_POWERFUL;
	
	@ClientString(id = 1800164, message = "S-grade")
	public static NpcStringId S_GRADE;
	
	@ClientString(id = 1800165, message = "A-grade")
	public static NpcStringId A_GRADE;
	
	@ClientString(id = 1800166, message = "B-grade")
	public static NpcStringId B_GRADE;
	
	@ClientString(id = 1800167, message = "C-grade")
	public static NpcStringId C_GRADE;
	
	@ClientString(id = 1800168, message = "D-grade")
	public static NpcStringId D_GRADE;
	
	@ClientString(id = 1800169, message = "F-grade")
	public static NpcStringId F_GRADE;
	
	@ClientString(id = 1800170, message = "This is... This is a great achievement that is worthy of the true heroes of legend!")
	public static NpcStringId THIS_IS_THIS_IS_A_GREAT_ACHIEVEMENT_THAT_IS_WORTHY_OF_THE_TRUE_HEROES_OF_LEGEND;
	
	@ClientString(id = 1800171, message = "Admirable. You greatly decreased the speed of invasion through Kamaloka.")
	public static NpcStringId ADMIRABLE_YOU_GREATLY_DECREASED_THE_SPEED_OF_INVASION_THROUGH_KAMALOKA;
	
	@ClientString(id = 1800172, message = "Very good. Your skill makes you a model for other adventurers to follow.")
	public static NpcStringId VERY_GOOD_YOUR_SKILL_MAKES_YOU_A_MODEL_FOR_OTHER_ADVENTURERS_TO_FOLLOW;
	
	@ClientString(id = 1800173, message = "Good work. If all adventurers produce results like you, we will slowly start to see the glimmer of hope.")
	public static NpcStringId GOOD_WORK_IF_ALL_ADVENTURERS_PRODUCE_RESULTS_LIKE_YOU_WE_WILL_SLOWLY_START_TO_SEE_THE_GLIMMER_OF_HOPE;
	
	@ClientString(id = 1800174, message = "Unfortunately, it seems that Rim Kamaloka cannot be easily approached by everyone.")
	public static NpcStringId UNFORTUNATELY_IT_SEEMS_THAT_RIM_KAMALOKA_CANNOT_BE_EASILY_APPROACHED_BY_EVERYONE;
	
	@ClientString(id = 1800175, message = "How disappointing. It looks like I made a mistake in sending you inside Rim Kamaloka.")
	public static NpcStringId HOW_DISAPPOINTING_IT_LOOKS_LIKE_I_MADE_A_MISTAKE_IN_SENDING_YOU_INSIDE_RIM_KAMALOKA;
	
	@ClientString(id = 1800176, message = "Intruder alert. Intruder alert.")
	public static NpcStringId INTRUDER_ALERT_INTRUDER_ALERT;
	
	@ClientString(id = 1800177, message = "What are you doing? Hurry up and help me!")
	public static NpcStringId WHAT_ARE_YOU_DOING_HURRY_UP_AND_HELP_ME;
	
	@ClientString(id = 1800178, message = "I've had it up to here with you! I'll take care of you!")
	public static NpcStringId I_VE_HAD_IT_UP_TO_HERE_WITH_YOU_I_LL_TAKE_CARE_OF_YOU;
	
	@ClientString(id = 1800179, message = "Ah... My mind is a wreck.")
	public static NpcStringId AH_MY_MIND_IS_A_WRECK;
	
	@ClientString(id = 1800180, message = "If you thought that my subordinates would be so few, you are mistaken!")
	public static NpcStringId IF_YOU_THOUGHT_THAT_MY_SUBORDINATES_WOULD_BE_SO_FEW_YOU_ARE_MISTAKEN;
	
	@ClientString(id = 1800181, message = "There's not much I can do, but I want to help you.")
	public static NpcStringId THERE_S_NOT_MUCH_I_CAN_DO_BUT_I_WANT_TO_HELP_YOU;
	
	@ClientString(id = 1800182, message = "You $s1! Attack them!")
	public static NpcStringId YOU_S1_ATTACK_THEM;
	
	@ClientString(id = 1800183, message = "Come out! My subordinate! I summon you to drive them out!")
	public static NpcStringId COME_OUT_MY_SUBORDINATE_I_SUMMON_YOU_TO_DRIVE_THEM_OUT;
	
	@ClientString(id = 1800184, message = "There's not much I can do, but I will risk my life to help you!")
	public static NpcStringId THERE_S_NOT_MUCH_I_CAN_DO_BUT_I_WILL_RISK_MY_LIFE_TO_HELP_YOU;
	
	@ClientString(id = 1800185, message = "Arg! The pain is more than I can stand!")
	public static NpcStringId ARG_THE_PAIN_IS_MORE_THAN_I_CAN_STAND;
	
	@ClientString(id = 1800186, message = "Ahh! How did he find my weakness?")
	public static NpcStringId AHH_HOW_DID_HE_FIND_MY_WEAKNESS;
	
	@ClientString(id = 1800187, message = "We were able to successfully collect the Essence of Kamaloka from the Kanabions that you defeated. Here they are.")
	public static NpcStringId WE_WERE_ABLE_TO_SUCCESSFULLY_COLLECT_THE_ESSENCE_OF_KAMALOKA_FROM_THE_KANABIONS_THAT_YOU_DEFEATED_HERE_THEY_ARE;
	
	@ClientString(id = 1800188, message = "But we were able to collect somehow the Essence of Kamaloka from the Kanabions that you defeated. Here they are.")
	public static NpcStringId BUT_WE_WERE_ABLE_TO_COLLECT_SOMEHOW_THE_ESSENCE_OF_KAMALOKA_FROM_THE_KANABIONS_THAT_YOU_DEFEATED_HERE_THEY_ARE;
	
	@ClientString(id = 1800189, message = "I'm sorry, but we were unable to collect the Essence of Kamaloka from the Kanabions that you defeated because their dark energy was too weak.")
	public static NpcStringId I_M_SORRY_BUT_WE_WERE_UNABLE_TO_COLLECT_THE_ESSENCE_OF_KAMALOKA_FROM_THE_KANABIONS_THAT_YOU_DEFEATED_BECAUSE_THEIR_DARK_ENERGY_WAS_TOO_WEAK;
	
	@ClientString(id = 1800190, message = "Rather than simply defeating the enemies, you seem to understand our goal and purpose as well.")
	public static NpcStringId RATHER_THAN_SIMPLY_DEFEATING_THE_ENEMIES_YOU_SEEM_TO_UNDERSTAND_OUR_GOAL_AND_PURPOSE_AS_WELL;
	
	@ClientString(id = 1800191, message = "Dopplers and Voids possess an enhanced amount of the Kanabions' dark energy, so it is important to concentrate on defeating them when blocking the Kamalokians attack.")
	public static NpcStringId DOPPLERS_AND_VOIDS_POSSESS_AN_ENHANCED_AMOUNT_OF_THE_KANABIONS_DARK_ENERGY_SO_IT_IS_IMPORTANT_TO_CONCENTRATE_ON_DEFEATING_THEM_WHEN_BLOCKING_THE_KAMALOKIANS_ATTACK;
	
	@ClientString(id = 1800192, message = "Have you seen Kanabions being remade as new Kanabions sometimes? You can see it occur more often by inflicting great damage during an attack or at the moment you defeat them.")
	public static NpcStringId HAVE_YOU_SEEN_KANABIONS_BEING_REMADE_AS_NEW_KANABIONS_SOMETIMES_YOU_CAN_SEE_IT_OCCUR_MORE_OFTEN_BY_INFLICTING_GREAT_DAMAGE_DURING_AN_ATTACK_OR_AT_THE_MOMENT_YOU_DEFEAT_THEM;
	
	@ClientString(id = 1800193, message = "As in any other battle, it is critical to protect yourself while you are inside Rim Kamaloka. We do not want to attack recklessly.")
	public static NpcStringId AS_IN_ANY_OTHER_BATTLE_IT_IS_CRITICAL_TO_PROTECT_YOURSELF_WHILE_YOU_ARE_INSIDE_RIM_KAMALOKA_WE_DO_NOT_WANT_TO_ATTACK_RECKLESSLY;
	
	@ClientString(id = 1800194, message = "We value developing an individual's overall power rather than a one-time victory. If you relied on another person's support this time, why don't you try to rely on your own strength next time?")
	public static NpcStringId WE_VALUE_DEVELOPING_AN_INDIVIDUAL_S_OVERALL_POWER_RATHER_THAN_A_ONE_TIME_VICTORY_IF_YOU_RELIED_ON_ANOTHER_PERSON_S_SUPPORT_THIS_TIME_WHY_DON_T_YOU_TRY_TO_RELY_ON_YOUR_OWN_STRENGTH_NEXT_TIME;
	
	@ClientString(id = 1800195, message = "Are you sure that the battle just now was at the appropriate level? Bothering lower Kanabions, as if for mere entertainment, is considered to be a wasted battle for us.")
	public static NpcStringId ARE_YOU_SURE_THAT_THE_BATTLE_JUST_NOW_WAS_AT_THE_APPROPRIATE_LEVEL_BOTHERING_LOWER_KANABIONS_AS_IF_FOR_MERE_ENTERTAINMENT_IS_CONSIDERED_TO_BE_A_WASTED_BATTLE_FOR_US;
	
	@ClientString(id = 1800196, message = "The greatest victory involves using all available resources, eliminating all of the enemy's opportunities, and bringing it to the fastest possible end. Don't you think so?")
	public static NpcStringId THE_GREATEST_VICTORY_INVOLVES_USING_ALL_AVAILABLE_RESOURCES_ELIMINATING_ALL_OF_THE_ENEMY_S_OPPORTUNITIES_AND_BRINGING_IT_TO_THE_FASTEST_POSSIBLE_END_DON_T_YOU_THINK_SO;
	
	@ClientString(id = 1800197, message = "Emergency! Emergency! The outer wall is weakening rapidly!")
	public static NpcStringId EMERGENCY_EMERGENCY_THE_OUTER_WALL_IS_WEAKENING_RAPIDLY;
	
	@ClientString(id = 1800198, message = "The remaining strength of the outer wall is $s1!")
	public static NpcStringId THE_REMAINING_STRENGTH_OF_THE_OUTER_WALL_IS_S1;
	
	@ClientString(id = 1800199, message = "Pathfinder Savior")
	public static NpcStringId PATHFINDER_SAVIOR;
	
	@ClientString(id = 1800200, message = "Pathfinder Supporter")
	public static NpcStringId PATHFINDER_SUPPORTER;
	
	@ClientString(id = 1800201, message = "Some Kanabions who haven't fully adjusted yet to their new physical form are known to exhibit symptoms of an extremely weakened body structure sometimes. If you attack them at that moment, you will have great results.")
	public static NpcStringId SOME_KANABIONS_WHO_HAVEN_T_FULLY_ADJUSTED_YET_TO_THEIR_NEW_PHYSICAL_FORM_ARE_KNOWN_TO_EXHIBIT_SYMPTOMS_OF_AN_EXTREMELY_WEAKENED_BODY_STRUCTURE_SOMETIMES_IF_YOU_ATTACK_THEM_AT_THAT_MOMENT_YOU_WILL_HAVE_GREAT_RESULTS;
	
	@ClientString(id = 1800202, message = "Have you ever heard of $s1? They say it's a genuine $s2!")
	public static NpcStringId HAVE_YOU_EVER_HEARD_OF_S1_THEY_SAY_IT_S_A_GENUINE_S2;
	
	@ClientString(id = 1800203, message = "There are 5 minutes remaining to register for Kratei's cube match.")
	public static NpcStringId THERE_ARE_5_MINUTES_REMAINING_TO_REGISTER_FOR_KRATEI_S_CUBE_MATCH;
	
	@ClientString(id = 1800204, message = "There are 3 minutes remaining to register for Kratei's cube match.")
	public static NpcStringId THERE_ARE_3_MINUTES_REMAINING_TO_REGISTER_FOR_KRATEI_S_CUBE_MATCH;
	
	@ClientString(id = 1800205, message = "There are 1 minutes remaining to register for Kratei's cube match.")
	public static NpcStringId THERE_ARE_1_MINUTES_REMAINING_TO_REGISTER_FOR_KRATEI_S_CUBE_MATCH;
	
	@ClientString(id = 1800206, message = "The match will begin in $s1 minute(s).")
	public static NpcStringId THE_MATCH_WILL_BEGIN_IN_S1_MINUTE_S;
	
	@ClientString(id = 1800207, message = "The match will begin shortly.")
	public static NpcStringId THE_MATCH_WILL_BEGIN_SHORTLY;
	
	@ClientString(id = 1800208, message = "You have $s1 minute(s) to register for the match.")
	public static NpcStringId YOU_HAVE_S1_MINUTE_S_TO_REGISTER_FOR_THE_MATCH;
	
	@ClientString(id = 1800209, message = "Ohh...oh...oh...")
	public static NpcStringId OHH_OH_OH;
	
	@ClientString(id = 1800210, message = "Fire")
	public static NpcStringId FIRE;
	
	@ClientString(id = 1800211, message = "Water")
	public static NpcStringId WATER;
	
	@ClientString(id = 1800212, message = "Wind")
	public static NpcStringId WIND;
	
	@ClientString(id = 1800213, message = "Earth")
	public static NpcStringId EARTH;
	
	@ClientString(id = 1800214, message = "...It's $s1...")
	public static NpcStringId IT_S_S1;
	
	@ClientString(id = 1800215, message = "...$s1 is strong...")
	public static NpcStringId S1_IS_STRONG;
	
	@ClientString(id = 1800216, message = "...It's always $s1...")
	public static NpcStringId IT_S_ALWAYS_S1;
	
	@ClientString(id = 1800217, message = "...$s1 won't do...")
	public static NpcStringId S1_WON_T_DO;
	
	@ClientString(id = 1800218, message = "You will be cursed for seeking the treasure!")
	public static NpcStringId YOU_WILL_BE_CURSED_FOR_SEEKING_THE_TREASURE;
	
	@ClientString(id = 1800219, message = "The airship has been summoned. It will automatically depart in 5 minutes.")
	public static NpcStringId THE_AIRSHIP_HAS_BEEN_SUMMONED_IT_WILL_AUTOMATICALLY_DEPART_IN_5_MINUTES;
	
	@ClientString(id = 1800220, message = "The regularly scheduled airship has arrived. It will depart for the Aden continent in 1 minute.")
	public static NpcStringId THE_REGULARLY_SCHEDULED_AIRSHIP_HAS_ARRIVED_IT_WILL_DEPART_FOR_THE_ADEN_CONTINENT_IN_1_MINUTE;
	
	@ClientString(id = 1800221, message = "The regularly scheduled airship that flies to the Aden continent has departed.")
	public static NpcStringId THE_REGULARLY_SCHEDULED_AIRSHIP_THAT_FLIES_TO_THE_ADEN_CONTINENT_HAS_DEPARTED;
	
	@ClientString(id = 1800222, message = "The regularly scheduled airship has arrived. It will depart for the Gracia continent in 1 minute.")
	public static NpcStringId THE_REGULARLY_SCHEDULED_AIRSHIP_HAS_ARRIVED_IT_WILL_DEPART_FOR_THE_GRACIA_CONTINENT_IN_1_MINUTE;
	
	@ClientString(id = 1800223, message = "The regularly scheduled airship that flies to the Gracia continent has departed.")
	public static NpcStringId THE_REGULARLY_SCHEDULED_AIRSHIP_THAT_FLIES_TO_THE_GRACIA_CONTINENT_HAS_DEPARTED;
	
	@ClientString(id = 1800224, message = "Another airship has been summoned to the wharf. Please try again later.")
	public static NpcStringId ANOTHER_AIRSHIP_HAS_BEEN_SUMMONED_TO_THE_WHARF_PLEASE_TRY_AGAIN_LATER;
	
	@ClientString(id = 1800225, message = "Huh? The sky looks funny. What's that?")
	public static NpcStringId HUH_THE_SKY_LOOKS_FUNNY_WHAT_S_THAT;
	
	@ClientString(id = 1800226, message = "A powerful subordinate is being held by the Barrier Orb! This reaction means...!")
	public static NpcStringId A_POWERFUL_SUBORDINATE_IS_BEING_HELD_BY_THE_BARRIER_ORB_THIS_REACTION_MEANS;
	
	@ClientString(id = 1800227, message = "Be careful...! Something's coming...!")
	public static NpcStringId BE_CAREFUL_SOMETHING_S_COMING;
	
	@ClientString(id = 1800228, message = "You must first found a clan or belong to one.")
	public static NpcStringId YOU_MUST_FIRST_FOUND_A_CLAN_OR_BELONG_TO_ONE;
	
	@ClientString(id = 1800229, message = "There is no party currently challenging Ekimus. n If no party enters within $s1 seconds, the attack on the Heart of Infinity will fail...")
	public static NpcStringId THERE_IS_NO_PARTY_CURRENTLY_CHALLENGING_EKIMUS_N_IF_NO_PARTY_ENTERS_WITHIN_S1_SECONDS_THE_ATTACK_ON_THE_HEART_OF_INFINITY_WILL_FAIL;
	
	@ClientString(id = 1800230, message = "Ekimus has gained strength from a tumor...")
	public static NpcStringId EKIMUS_HAS_GAINED_STRENGTH_FROM_A_TUMOR;
	
	@ClientString(id = 1800231, message = "Ekimus has been weakened by losing strength from a tumor!")
	public static NpcStringId EKIMUS_HAS_BEEN_WEAKENED_BY_LOSING_STRENGTH_FROM_A_TUMOR;
	
	@ClientString(id = 1800232, message = "The Soul Coffin has awakened Ekimus. If $s1 more Soul Coffin(s) are created, the defense of the Heart of Infinity will fail...")
	public static NpcStringId THE_SOUL_COFFIN_HAS_AWAKENED_EKIMUS_IF_S1_MORE_SOUL_COFFIN_S_ARE_CREATED_THE_DEFENSE_OF_THE_HEART_OF_INFINITY_WILL_FAIL;
	
	@ClientString(id = 1800233, message = "C'mon, c'mon! Show your face, you little rats! Let me see what the doomed weaklings are scheming!")
	public static NpcStringId C_MON_C_MON_SHOW_YOUR_FACE_YOU_LITTLE_RATS_LET_ME_SEE_WHAT_THE_DOOMED_WEAKLINGS_ARE_SCHEMING;
	
	@ClientString(id = 1800234, message = "Impressive.... Hahaha it's so much fun, but I need to chill a little while.  Argekunte, clear the way!")
	public static NpcStringId IMPRESSIVE_HAHAHA_IT_S_SO_MUCH_FUN_BUT_I_NEED_TO_CHILL_A_LITTLE_WHILE_ARGEKUNTE_CLEAR_THE_WAY;
	
	@ClientString(id = 1800235, message = "Kyahaha! Since the tumor has been resurrected, I no longer need to waste my time on you!")
	public static NpcStringId KYAHAHA_SINCE_THE_TUMOR_HAS_BEEN_RESURRECTED_I_NO_LONGER_NEED_TO_WASTE_MY_TIME_ON_YOU;
	
	@ClientString(id = 1800236, message = "Keu... I will leave for now... But don't think this is over... The Seed of Infinity can never die...")
	public static NpcStringId KEU_I_WILL_LEAVE_FOR_NOW_BUT_DON_T_THINK_THIS_IS_OVER_THE_SEED_OF_INFINITY_CAN_NEVER_DIE;
	
	@ClientString(id = 1800237, message = "Kahahaha! That guy's nothing! He can't even kill without my permission! See here! Ultimate forgotten magic! Deathless Guardian!")
	public static NpcStringId KAHAHAHA_THAT_GUY_S_NOTHING_HE_CAN_T_EVEN_KILL_WITHOUT_MY_PERMISSION_SEE_HERE_ULTIMATE_FORGOTTEN_MAGIC_DEATHLESS_GUARDIAN;
	
	@ClientString(id = 1800238, message = "I curse the day that I became your slave in order to escape death, Cohemenes! I swear that I shall see you die with my own eyes!")
	public static NpcStringId I_CURSE_THE_DAY_THAT_I_BECAME_YOUR_SLAVE_IN_ORDER_TO_ESCAPE_DEATH_COHEMENES_I_SWEAR_THAT_I_SHALL_SEE_YOU_DIE_WITH_MY_OWN_EYES;
	
	@ClientString(id = 1800239, message = "My enemy is dying, and my blood is boiling! What cruel curse is this!")
	public static NpcStringId MY_ENEMY_IS_DYING_AND_MY_BLOOD_IS_BOILING_WHAT_CRUEL_CURSE_IS_THIS;
	
	@ClientString(id = 1800240, message = "Hall of Suffering")
	public static NpcStringId HALL_OF_SUFFERING;
	
	@ClientString(id = 1800241, message = "Hall of Erosion")
	public static NpcStringId HALL_OF_EROSION;
	
	@ClientString(id = 1800242, message = "Heart of Infinity")
	public static NpcStringId HEART_OF_INFINITY;
	
	@ClientString(id = 1800243, message = "Attack")
	public static NpcStringId ATTACK;
	
	@ClientString(id = 1800244, message = "Defend")
	public static NpcStringId DEFEND;
	
	@ClientString(id = 1800245, message = "Congratulations! You have succeeded at $s1 $s2! The instance will shortly expire.")
	public static NpcStringId CONGRATULATIONS_YOU_HAVE_SUCCEEDED_AT_S1_S2_THE_INSTANCE_WILL_SHORTLY_EXPIRE;
	
	@ClientString(id = 1800246, message = "You have failed at $s1 $s2... The instance will shortly expire.")
	public static NpcStringId YOU_HAVE_FAILED_AT_S1_S2_THE_INSTANCE_WILL_SHORTLY_EXPIRE;
	
	@ClientString(id = 1800247, message = "$s1's party has moved to a different location through the crack in the tumor!")
	public static NpcStringId S1_S_PARTY_HAS_MOVED_TO_A_DIFFERENT_LOCATION_THROUGH_THE_CRACK_IN_THE_TUMOR;
	
	@ClientString(id = 1800248, message = "$s1's party has entered the Chamber of Ekimus through the crack in the tumor!")
	public static NpcStringId S1_S_PARTY_HAS_ENTERED_THE_CHAMBER_OF_EKIMUS_THROUGH_THE_CRACK_IN_THE_TUMOR;
	
	@ClientString(id = 1800249, message = "Ekimus has sensed abnormal activity. nThe advancing party is forcefully expelled!")
	public static NpcStringId EKIMUS_HAS_SENSED_ABNORMAL_ACTIVITY_NTHE_ADVANCING_PARTY_IS_FORCEFULLY_EXPELLED;
	
	@ClientString(id = 1800250, message = "There aren't enough items. In order to summon the airship, you need 5 Energy Star Stones.")
	public static NpcStringId THERE_AREN_T_ENOUGH_ITEMS_IN_ORDER_TO_SUMMON_THE_AIRSHIP_YOU_NEED_5_ENERGY_STAR_STONES;
	
	@ClientString(id = 1800251, message = "The Soul Devourers who are greedy to eat the Seeds of Life that remain alive until the end have awakened...!")
	public static NpcStringId THE_SOUL_DEVOURERS_WHO_ARE_GREEDY_TO_EAT_THE_SEEDS_OF_LIFE_THAT_REMAIN_ALIVE_UNTIL_THE_END_HAVE_AWAKENED;
	
	@ClientString(id = 1800252, message = "The first Feral Hound of the Netherworld has awakened!")
	public static NpcStringId THE_FIRST_FERAL_HOUND_OF_THE_NETHERWORLD_HAS_AWAKENED;
	
	@ClientString(id = 1800253, message = "The second Feral Hound of the Netherworld has awakened!")
	public static NpcStringId THE_SECOND_FERAL_HOUND_OF_THE_NETHERWORLD_HAS_AWAKENED;
	
	@ClientString(id = 1800254, message = "Clinging on won't help you! Ultimate forgotten magic, Blade Turn!")
	public static NpcStringId CLINGING_ON_WON_T_HELP_YOU_ULTIMATE_FORGOTTEN_MAGIC_BLADE_TURN;
	
	@ClientString(id = 1800255, message = "Even special sauce can't help you! Ultimate forgotten magic, Force Shield!")
	public static NpcStringId EVEN_SPECIAL_SAUCE_CAN_T_HELP_YOU_ULTIMATE_FORGOTTEN_MAGIC_FORCE_SHIELD;
	
	@ClientString(id = 1800256, message = "You little doomed maggots! Even if you keep swarming, the power of Immortality will only grow stronger!")
	public static NpcStringId YOU_LITTLE_DOOMED_MAGGOTS_EVEN_IF_YOU_KEEP_SWARMING_THE_POWER_OF_IMMORTALITY_WILL_ONLY_GROW_STRONGER;
	
	@ClientString(id = 1800257, message = "The Airship Summon License has been awarded. Your clan can now summon an airship.")
	public static NpcStringId THE_AIRSHIP_SUMMON_LICENSE_HAS_BEEN_AWARDED_YOUR_CLAN_CAN_NOW_SUMMON_AN_AIRSHIP;
	
	@ClientString(id = 1800258, message = "The Gracia treasure chest has appeared!")
	public static NpcStringId THE_GRACIA_TREASURE_CHEST_HAS_APPEARED;
	
	@ClientString(id = 1800259, message = "The Gracia treasure chest will soon disappear!")
	public static NpcStringId THE_GRACIA_TREASURE_CHEST_WILL_SOON_DISAPPEAR;
	
	@ClientString(id = 1800260, message = "You have been cursed by the tumor and have incurred $s1 damage.")
	public static NpcStringId YOU_HAVE_BEEN_CURSED_BY_THE_TUMOR_AND_HAVE_INCURRED_S1_DAMAGE;
	
	@ClientString(id = 1800261, message = "I shall accept your challenge, $s1! Come and die in the arms of immortality!")
	public static NpcStringId I_SHALL_ACCEPT_YOUR_CHALLENGE_S1_COME_AND_DIE_IN_THE_ARMS_OF_IMMORTALITY;
	
	@ClientString(id = 1800262, message = "You will participate in $s1 $s2 shortly. Be prepared for anything.")
	public static NpcStringId YOU_WILL_PARTICIPATE_IN_S1_S2_SHORTLY_BE_PREPARED_FOR_ANYTHING;
	
	@ClientString(id = 1800263, message = "You can hear the undead of Ekimus rushing toward you. $s1 $s2, it has now begun!")
	public static NpcStringId YOU_CAN_HEAR_THE_UNDEAD_OF_EKIMUS_RUSHING_TOWARD_YOU_S1_S2_IT_HAS_NOW_BEGUN;
	
	@ClientString(id = 1800264, message = "You can feel the surging energy of death from the tumor.")
	public static NpcStringId YOU_CAN_FEEL_THE_SURGING_ENERGY_OF_DEATH_FROM_THE_TUMOR;
	
	@ClientString(id = 1800265, message = "The area near the tumor is full of ominous energy.")
	public static NpcStringId THE_AREA_NEAR_THE_TUMOR_IS_FULL_OF_OMINOUS_ENERGY;
	
	@ClientString(id = 1800266, message = "You tried to drop us. How stupid!")
	public static NpcStringId YOU_TRIED_TO_DROP_US_HOW_STUPID;
	
	@ClientString(id = 1800267, message = "We are blood brethren. I can't fall so easily here and leave my brother behind.")
	public static NpcStringId WE_ARE_BLOOD_BRETHREN_I_CAN_T_FALL_SO_EASILY_HERE_AND_LEAVE_MY_BROTHER_BEHIND;
	
	@ClientString(id = 1800268, message = "You were always what I aspired to be. Do you think I would fall so easily here when I have a brother like that?")
	public static NpcStringId YOU_WERE_ALWAYS_WHAT_I_ASPIRED_TO_BE_DO_YOU_THINK_I_WOULD_FALL_SO_EASILY_HERE_WHEN_I_HAVE_A_BROTHER_LIKE_THAT;
	
	@ClientString(id = 1800269, message = "With all connections to the tumor severed, Ekimus has lost its power to control the Feral Hound!")
	public static NpcStringId WITH_ALL_CONNECTIONS_TO_THE_TUMOR_SEVERED_EKIMUS_HAS_LOST_ITS_POWER_TO_CONTROL_THE_FERAL_HOUND;
	
	@ClientString(id = 1800270, message = "With the connection to the tumor restored, Ekimus has regained control over the Feral Hound...")
	public static NpcStringId WITH_THE_CONNECTION_TO_THE_TUMOR_RESTORED_EKIMUS_HAS_REGAINED_CONTROL_OVER_THE_FERAL_HOUND;
	
	@ClientString(id = 1800271, message = "Woooong!")
	public static NpcStringId WOOOONG;
	
	@ClientString(id = 1800272, message = "Woong... Woong... Woo...")
	public static NpcStringId WOONG_WOONG_WOO;
	
	@ClientString(id = 1800273, message = "The enemies have attacked. Everyone come out and fight!!!! ... Urgh~!")
	public static NpcStringId THE_ENEMIES_HAVE_ATTACKED_EVERYONE_COME_OUT_AND_FIGHT_URGH;
	
	@ClientString(id = 1800274, message = "The tumor inside $s1 has been destroyed! nIn order to draw out the cowardly Cohemenes, you must destroy all the tumors!")
	public static NpcStringId THE_TUMOR_INSIDE_S1_HAS_BEEN_DESTROYED_NIN_ORDER_TO_DRAW_OUT_THE_COWARDLY_COHEMENES_YOU_MUST_DESTROY_ALL_THE_TUMORS;
	
	@ClientString(id = 1800275, message = "The tumor inside $s1 has completely revived. nThe restrengthened Cohemenes has fled deeper inside the seed...")
	public static NpcStringId THE_TUMOR_INSIDE_S1_HAS_COMPLETELY_REVIVED_NTHE_RESTRENGTHENED_COHEMENES_HAS_FLED_DEEPER_INSIDE_THE_SEED;
	
	@ClientString(id = 1800276, message = "The awarded Airship Summon License has been received.")
	public static NpcStringId THE_AWARDED_AIRSHIP_SUMMON_LICENSE_HAS_BEEN_RECEIVED;
	
	@ClientString(id = 1800277, message = "You do not currently have an Airship Summon License. You can earn your Airship Summon License through Engineer Lekon.")
	public static NpcStringId YOU_DO_NOT_CURRENTLY_HAVE_AN_AIRSHIP_SUMMON_LICENSE_YOU_CAN_EARN_YOUR_AIRSHIP_SUMMON_LICENSE_THROUGH_ENGINEER_LEKON;
	
	@ClientString(id = 1800278, message = "The Airship Summon License has already been awarded.")
	public static NpcStringId THE_AIRSHIP_SUMMON_LICENSE_HAS_ALREADY_BEEN_AWARDED;
	
	@ClientString(id = 1800279, message = "If you have items, please give them to me.")
	public static NpcStringId IF_YOU_HAVE_ITEMS_PLEASE_GIVE_THEM_TO_ME;
	
	@ClientString(id = 1800280, message = "My stomach is empty.")
	public static NpcStringId MY_STOMACH_IS_EMPTY;
	
	@ClientString(id = 1800281, message = "I'm hungry, I'm hungry!")
	public static NpcStringId I_M_HUNGRY_I_M_HUNGRY;
	
	@ClientString(id = 1800282, message = "I'm still not full...")
	public static NpcStringId I_M_STILL_NOT_FULL;
	
	@ClientString(id = 1800283, message = "I'm still hungry~")
	public static NpcStringId I_M_STILL_HUNGRY;
	
	@ClientString(id = 1800284, message = "I feel a little woozy...")
	public static NpcStringId I_FEEL_A_LITTLE_WOOZY;
	
	@ClientString(id = 1800285, message = "Give me something to eat.")
	public static NpcStringId GIVE_ME_SOMETHING_TO_EAT;
	
	@ClientString(id = 1800286, message = "Now it's time to eat~")
	public static NpcStringId NOW_IT_S_TIME_TO_EAT;
	
	@ClientString(id = 1800287, message = "I also need dessert.")
	public static NpcStringId I_ALSO_NEED_DESSERT;
	
	@ClientString(id = 1800288, message = "I'm still hungry.")
	public static NpcStringId I_M_STILL_HUNGRY2;
	
	@ClientString(id = 1800289, message = "I'm full now, I don't want to eat anymore.")
	public static NpcStringId I_M_FULL_NOW_I_DON_T_WANT_TO_EAT_ANYMORE;
	
	@ClientString(id = 1800290, message = "I haven't eaten anything, I'm so weak~")
	public static NpcStringId I_HAVEN_T_EATEN_ANYTHING_I_M_SO_WEAK;
	
	@ClientString(id = 1800291, message = "Nom nom!")
	public static NpcStringId NOM_NOM;
	
	@ClientString(id = 1800292, message = "You've sustained $s1 damage as Tumor's shell started melting after touching the sacred seal on the shield!")
	public static NpcStringId YOU_VE_SUSTAINED_S1_DAMAGE_AS_TUMOR_S_SHELL_STARTED_MELTING_AFTER_TOUCHING_THE_SACRED_SEAL_ON_THE_SHIELD;
	
	@ClientString(id = 1800293, message = "You've sustained $s1 damage as Soul Coffin's shell started melting after touching the sacred seal on the shield!")
	public static NpcStringId YOU_VE_SUSTAINED_S1_DAMAGE_AS_SOUL_COFFIN_S_SHELL_STARTED_MELTING_AFTER_TOUCHING_THE_SACRED_SEAL_ON_THE_SHIELD;
	
	@ClientString(id = 1800294, message = "Raid rewards are given to the attacker ($s1).")
	public static NpcStringId RAID_REWARDS_ARE_GIVEN_TO_THE_ATTACKER_S1;
	
	@ClientString(id = 1800295, message = "Obelisk has collapsed. Don't let the enemies jump around wildly anymore!!!!")
	public static NpcStringId OBELISK_HAS_COLLAPSED_DON_T_LET_THE_ENEMIES_JUMP_AROUND_WILDLY_ANYMORE;
	
	@ClientString(id = 1800296, message = "Enemies are trying to destroy the fortress. Everyone defend the fortress!!!!")
	public static NpcStringId ENEMIES_ARE_TRYING_TO_DESTROY_THE_FORTRESS_EVERYONE_DEFEND_THE_FORTRESS;
	
	@ClientString(id = 1800297, message = "Come out, warriors. Protect Seed of Destruction.")
	public static NpcStringId COME_OUT_WARRIORS_PROTECT_SEED_OF_DESTRUCTION;
	
	@ClientString(id = 1800298, message = "The undead of Ekimus is attacking Seed of Life. Defending Hall of Erosion will fail even if one Seed of Life is destroyed...")
	public static NpcStringId THE_UNDEAD_OF_EKIMUS_IS_ATTACKING_SEED_OF_LIFE_DEFENDING_HALL_OF_EROSION_WILL_FAIL_EVEN_IF_ONE_SEED_OF_LIFE_IS_DESTROYED;
	
	@ClientString(id = 1800299, message = "All the tumors inside $s1 have been destroyed! Driven into a corner, Cohemenes appears close by!")
	public static NpcStringId ALL_THE_TUMORS_INSIDE_S1_HAVE_BEEN_DESTROYED_DRIVEN_INTO_A_CORNER_COHEMENES_APPEARS_CLOSE_BY;
	
	@ClientString(id = 1800300, message = "The tumor inside $s1 has been destroyed! nThe nearby Undead that were attacking Seed of Life start losing their energy and run away!")
	public static NpcStringId THE_TUMOR_INSIDE_S1_HAS_BEEN_DESTROYED_NTHE_NEARBY_UNDEAD_THAT_WERE_ATTACKING_SEED_OF_LIFE_START_LOSING_THEIR_ENERGY_AND_RUN_AWAY;
	
	@ClientString(id = 1800301, message = "The tumor inside $s1 has completely revived. nRecovered nearby Undead are swarming toward Seed of Life...")
	public static NpcStringId THE_TUMOR_INSIDE_S1_HAS_COMPLETELY_REVIVED_NRECOVERED_NEARBY_UNDEAD_ARE_SWARMING_TOWARD_SEED_OF_LIFE;
	
	@ClientString(id = 1800302, message = "The tumor inside $s1 that has provided energy n to Ekimus is destroyed!")
	public static NpcStringId THE_TUMOR_INSIDE_S1_THAT_HAS_PROVIDED_ENERGY_N_TO_EKIMUS_IS_DESTROYED;
	
	@ClientString(id = 1800303, message = "The tumor inside $s1 has been completely resurrected n and started to energize Ekimus again...")
	public static NpcStringId THE_TUMOR_INSIDE_S1_HAS_BEEN_COMPLETELY_RESURRECTED_N_AND_STARTED_TO_ENERGIZE_EKIMUS_AGAIN;
	
	@ClientString(id = 1800304, message = "The tumor inside $s1 has been destroyed! nThe speed that Ekimus calls out his prey has slowed down!")
	public static NpcStringId THE_TUMOR_INSIDE_S1_HAS_BEEN_DESTROYED_NTHE_SPEED_THAT_EKIMUS_CALLS_OUT_HIS_PREY_HAS_SLOWED_DOWN;
	
	@ClientString(id = 1800305, message = "The tumor inside $s1 has completely revived. nEkimus started to regain his energy and is desperately looking for his prey...")
	public static NpcStringId THE_TUMOR_INSIDE_S1_HAS_COMPLETELY_REVIVED_NEKIMUS_STARTED_TO_REGAIN_HIS_ENERGY_AND_IS_DESPERATELY_LOOKING_FOR_HIS_PREY;
	
	@ClientString(id = 1800306, message = "Bring more, more souls...!")
	public static NpcStringId BRING_MORE_MORE_SOULS;
	
	@ClientString(id = 1800307, message = "The Hall of Erosion attack will fail unless you make a quick attack against the tumor!")
	public static NpcStringId THE_HALL_OF_EROSION_ATTACK_WILL_FAIL_UNLESS_YOU_MAKE_A_QUICK_ATTACK_AGAINST_THE_TUMOR;
	
	@ClientString(id = 1800308, message = "As the tumor was not threatened, Cohemenes completely ran away to deep inside the Seed...")
	public static NpcStringId AS_THE_TUMOR_WAS_NOT_THREATENED_COHEMENES_COMPLETELY_RAN_AWAY_TO_DEEP_INSIDE_THE_SEED;
	
	@ClientString(id = 1800309, message = "Your goal will be obstructed or be under a restraint.")
	public static NpcStringId YOUR_GOAL_WILL_BE_OBSTRUCTED_OR_BE_UNDER_A_RESTRAINT;
	
	@ClientString(id = 1800310, message = "You may face an unforeseen problem on your course toward the goal.")
	public static NpcStringId YOU_MAY_FACE_AN_UNFORESEEN_PROBLEM_ON_YOUR_COURSE_TOWARD_THE_GOAL;
	
	@ClientString(id = 1800311, message = "You may feel nervous and anxious because of unfavorable situations.")
	public static NpcStringId YOU_MAY_FEEL_NERVOUS_AND_ANXIOUS_BECAUSE_OF_UNFAVORABLE_SITUATIONS;
	
	@ClientString(id = 1800312, message = "Be warned when the situation is difficult because you may lose your judgment and make an irrational mistake.")
	public static NpcStringId BE_WARNED_WHEN_THE_SITUATION_IS_DIFFICULT_BECAUSE_YOU_MAY_LOSE_YOUR_JUDGMENT_AND_MAKE_AN_IRRATIONAL_MISTAKE;
	
	@ClientString(id = 1800313, message = "You may meet a trustworthy person or a good opportunity.")
	public static NpcStringId YOU_MAY_MEET_A_TRUSTWORTHY_PERSON_OR_A_GOOD_OPPORTUNITY;
	
	@ClientString(id = 1800314, message = "Your downward life starts taking an upturn.")
	public static NpcStringId YOUR_DOWNWARD_LIFE_STARTS_TAKING_AN_UPTURN;
	
	@ClientString(id = 1800315, message = "You will attract attention from people with your popularity.")
	public static NpcStringId YOU_WILL_ATTRACT_ATTENTION_FROM_PEOPLE_WITH_YOUR_POPULARITY;
	
	@ClientString(id = 1800316, message = "Your star of fortune says there'll be fish snapping at your bait.")
	public static NpcStringId YOUR_STAR_OF_FORTUNE_SAYS_THERE_LL_BE_FISH_SNAPPING_AT_YOUR_BAIT;
	
	@ClientString(id = 1800317, message = "There may be a conflict born of your dogmatic procedures.")
	public static NpcStringId THERE_MAY_BE_A_CONFLICT_BORN_OF_YOUR_DOGMATIC_PROCEDURES;
	
	@ClientString(id = 1800318, message = "Your wisdom and creativity may lead you to success with your goal.")
	public static NpcStringId YOUR_WISDOM_AND_CREATIVITY_MAY_LEAD_YOU_TO_SUCCESS_WITH_YOUR_GOAL;
	
	@ClientString(id = 1800319, message = "You may accomplish your goals if you diligently pursue them without giving up.")
	public static NpcStringId YOU_MAY_ACCOMPLISH_YOUR_GOALS_IF_YOU_DILIGENTLY_PURSUE_THEM_WITHOUT_GIVING_UP;
	
	@ClientString(id = 1800320, message = "You may get help if you go through the front door without seeking tricks or maneuvers.")
	public static NpcStringId YOU_MAY_GET_HELP_IF_YOU_GO_THROUGH_THE_FRONT_DOOR_WITHOUT_SEEKING_TRICKS_OR_MANEUVERS;
	
	@ClientString(id = 1800321, message = "A good result is on the way if you set a goal and bravely proceed toward it.")
	public static NpcStringId A_GOOD_RESULT_IS_ON_THE_WAY_IF_YOU_SET_A_GOAL_AND_BRAVELY_PROCEED_TOWARD_IT;
	
	@ClientString(id = 1800322, message = "Everything will be smoothly managed no matter how difficult.")
	public static NpcStringId EVERYTHING_WILL_BE_SMOOTHLY_MANAGED_NO_MATTER_HOW_DIFFICULT;
	
	@ClientString(id = 1800323, message = "Be firm and carefully scrutinize circumstances even when things are difficult.")
	public static NpcStringId BE_FIRM_AND_CAREFULLY_SCRUTINIZE_CIRCUMSTANCES_EVEN_WHEN_THINGS_ARE_DIFFICULT;
	
	@ClientString(id = 1800324, message = "Always think over to find neglected problems you haven't taken care of yet.")
	public static NpcStringId ALWAYS_THINK_OVER_TO_FIND_NEGLECTED_PROBLEMS_YOU_HAVEN_T_TAKEN_CARE_OF_YET;
	
	@ClientString(id = 1800325, message = "Financial fortune will greet your poor life.")
	public static NpcStringId FINANCIAL_FORTUNE_WILL_GREET_YOUR_POOR_LIFE;
	
	@ClientString(id = 1800326, message = "You may acquire wealth and fame after unlucky circumstances.")
	public static NpcStringId YOU_MAY_ACQUIRE_WEALTH_AND_FAME_AFTER_UNLUCKY_CIRCUMSTANCES;
	
	@ClientString(id = 1800327, message = "The difficult situations will turn to hope with unforeseen help.")
	public static NpcStringId THE_DIFFICULT_SITUATIONS_WILL_TURN_TO_HOPE_WITH_UNFORESEEN_HELP;
	
	@ClientString(id = 1800328, message = "A great task will result in success.")
	public static NpcStringId A_GREAT_TASK_WILL_RESULT_IN_SUCCESS;
	
	@ClientString(id = 1800329, message = "You may encounter a precious person who will lift your loneliness and discord.")
	public static NpcStringId YOU_MAY_ENCOUNTER_A_PRECIOUS_PERSON_WHO_WILL_LIFT_YOUR_LONELINESS_AND_DISCORD;
	
	@ClientString(id = 1800330, message = "People around you will encourage your every task in the future.")
	public static NpcStringId PEOPLE_AROUND_YOU_WILL_ENCOURAGE_YOUR_EVERY_TASK_IN_THE_FUTURE;
	
	@ClientString(id = 1800331, message = "Everything will be smoothly managed.")
	public static NpcStringId EVERYTHING_WILL_BE_SMOOTHLY_MANAGED;
	
	@ClientString(id = 1800332, message = "You will meet a person who can cherish your values if you maintain good ties with people.")
	public static NpcStringId YOU_WILL_MEET_A_PERSON_WHO_CAN_CHERISH_YOUR_VALUES_IF_YOU_MAINTAIN_GOOD_TIES_WITH_PEOPLE;
	
	@ClientString(id = 1800333, message = "Maintain cooperative attitude since you will meet someone to be of help.")
	public static NpcStringId MAINTAIN_COOPERATIVE_ATTITUDE_SINCE_YOU_WILL_MEET_SOMEONE_TO_BE_OF_HELP;
	
	@ClientString(id = 1800334, message = "Keep your moderation and ego in check even in successful phases of your life.")
	public static NpcStringId KEEP_YOUR_MODERATION_AND_EGO_IN_CHECK_EVEN_IN_SUCCESSFUL_PHASES_OF_YOUR_LIFE;
	
	@ClientString(id = 1800335, message = "When it comes to work, lifestyle and relationships you'll be better off to go by the text rather than tricks.")
	public static NpcStringId WHEN_IT_COMES_TO_WORK_LIFESTYLE_AND_RELATIONSHIPS_YOU_LL_BE_BETTER_OFF_TO_GO_BY_THE_TEXT_RATHER_THAN_TRICKS;
	
	@ClientString(id = 1800336, message = "Your task will receive substantial support since the surroundings will fully develop.")
	public static NpcStringId YOUR_TASK_WILL_RECEIVE_SUBSTANTIAL_SUPPORT_SINCE_THE_SURROUNDINGS_WILL_FULLY_DEVELOP;
	
	@ClientString(id = 1800337, message = "Your star of fortune indicate a success with mental and material assistance.")
	public static NpcStringId YOUR_STAR_OF_FORTUNE_INDICATE_A_SUCCESS_WITH_MENTAL_AND_MATERIAL_ASSISTANCE;
	
	@ClientString(id = 1800338, message = "You will enjoy popularity with your creative talents and clever acts.")
	public static NpcStringId YOU_WILL_ENJOY_POPULARITY_WITH_YOUR_CREATIVE_TALENTS_AND_CLEVER_ACTS;
	
	@ClientString(id = 1800339, message = "People will line up to be of assistance to you.")
	public static NpcStringId PEOPLE_WILL_LINE_UP_TO_BE_OF_ASSISTANCE_TO_YOU;
	
	@ClientString(id = 1800340, message = "You may meet someone to share your journey.")
	public static NpcStringId YOU_MAY_MEET_SOMEONE_TO_SHARE_YOUR_JOURNEY;
	
	@ClientString(id = 1800341, message = "You may achieve connections in many fields.")
	public static NpcStringId YOU_MAY_ACHIEVE_CONNECTIONS_IN_MANY_FIELDS;
	
	@ClientString(id = 1800342, message = "An attitude that continually studies and explores is needed, and always be sincere.")
	public static NpcStringId AN_ATTITUDE_THAT_CONTINUALLY_STUDIES_AND_EXPLORES_IS_NEEDED_AND_ALWAYS_BE_SINCERE;
	
	@ClientString(id = 1800343, message = "It's an image of a butterfly on a flower in warm spring air.")
	public static NpcStringId IT_S_AN_IMAGE_OF_A_BUTTERFLY_ON_A_FLOWER_IN_WARM_SPRING_AIR;
	
	@ClientString(id = 1800344, message = "Your goals will move smoothly with peace and happiness in your life.")
	public static NpcStringId YOUR_GOALS_WILL_MOVE_SMOOTHLY_WITH_PEACE_AND_HAPPINESS_IN_YOUR_LIFE;
	
	@ClientString(id = 1800345, message = "Love may sprout its leaves when you treat those around you with care.")
	public static NpcStringId LOVE_MAY_SPROUT_ITS_LEAVES_WHEN_YOU_TREAT_THOSE_AROUND_YOU_WITH_CARE;
	
	@ClientString(id = 1800346, message = "You may climb into a higher position with others' trust if you faithfully carry out your duties.")
	public static NpcStringId YOU_MAY_CLIMB_INTO_A_HIGHER_POSITION_WITH_OTHERS_TRUST_IF_YOU_FAITHFULLY_CARRY_OUT_YOUR_DUTIES;
	
	@ClientString(id = 1800347, message = "Everything can fall apart if you greedily aim by pure luck.")
	public static NpcStringId EVERYTHING_CAN_FALL_APART_IF_YOU_GREEDILY_AIM_BY_PURE_LUCK;
	
	@ClientString(id = 1800348, message = "Do not underestimate the importance of meeting people.")
	public static NpcStringId DO_NOT_UNDERESTIMATE_THE_IMPORTANCE_OF_MEETING_PEOPLE;
	
	@ClientString(id = 1800349, message = "An arrow will coalesce into the bow.")
	public static NpcStringId AN_ARROW_WILL_COALESCE_INTO_THE_BOW;
	
	@ClientString(id = 1800350, message = "A bony limb of a tree may bear its fruit.")
	public static NpcStringId A_BONY_LIMB_OF_A_TREE_MAY_BEAR_ITS_FRUIT;
	
	@ClientString(id = 1800351, message = "You will be rewarded for your efforts and accomplishments.")
	public static NpcStringId YOU_WILL_BE_REWARDED_FOR_YOUR_EFFORTS_AND_ACCOMPLISHMENTS;
	
	@ClientString(id = 1800352, message = "No matter where it lies, your faithful drive leads you to success.")
	public static NpcStringId NO_MATTER_WHERE_IT_LIES_YOUR_FAITHFUL_DRIVE_LEADS_YOU_TO_SUCCESS;
	
	@ClientString(id = 1800353, message = "People will be attracted to your loyalties.")
	public static NpcStringId PEOPLE_WILL_BE_ATTRACTED_TO_YOUR_LOYALTIES;
	
	@ClientString(id = 1800354, message = "You may trust yourself rather than others' talks.")
	public static NpcStringId YOU_MAY_TRUST_YOURSELF_RATHER_THAN_OTHERS_TALKS;
	
	@ClientString(id = 1800355, message = "Creative thinking away from the old viewpoint may help you.")
	public static NpcStringId CREATIVE_THINKING_AWAY_FROM_THE_OLD_VIEWPOINT_MAY_HELP_YOU;
	
	@ClientString(id = 1800356, message = "Patience without being impetuous of the results will only bear a positive outcome.")
	public static NpcStringId PATIENCE_WITHOUT_BEING_IMPETUOUS_OF_THE_RESULTS_WILL_ONLY_BEAR_A_POSITIVE_OUTCOME;
	
	@ClientString(id = 1800357, message = "The dead will come alive.")
	public static NpcStringId THE_DEAD_WILL_COME_ALIVE;
	
	@ClientString(id = 1800358, message = "There will be a shocking incident.")
	public static NpcStringId THERE_WILL_BE_A_SHOCKING_INCIDENT;
	
	@ClientString(id = 1800359, message = "You will enjoy a huge success after unforeseen luck comes before you.")
	public static NpcStringId YOU_WILL_ENJOY_A_HUGE_SUCCESS_AFTER_UNFORESEEN_LUCK_COMES_BEFORE_YOU;
	
	@ClientString(id = 1800360, message = "Do not give up since there may be a miraculous rescue from the course of despair.")
	public static NpcStringId DO_NOT_GIVE_UP_SINCE_THERE_MAY_BE_A_MIRACULOUS_RESCUE_FROM_THE_COURSE_OF_DESPAIR;
	
	@ClientString(id = 1800361, message = "An attitude to try one's best to pursue the goal is needed.")
	public static NpcStringId AN_ATTITUDE_TO_TRY_ONE_S_BEST_TO_PURSUE_THE_GOAL_IS_NEEDED;
	
	@ClientString(id = 1800362, message = "You may get a shot in the arm in your life after meeting a good person.")
	public static NpcStringId YOU_MAY_GET_A_SHOT_IN_THE_ARM_IN_YOUR_LIFE_AFTER_MEETING_A_GOOD_PERSON;
	
	@ClientString(id = 1800363, message = "You may get a big help in the course of your life.")
	public static NpcStringId YOU_MAY_GET_A_BIG_HELP_IN_THE_COURSE_OF_YOUR_LIFE;
	
	@ClientString(id = 1800364, message = "A rare opportunity will come to you so you may prosper.")
	public static NpcStringId A_RARE_OPPORTUNITY_WILL_COME_TO_YOU_SO_YOU_MAY_PROSPER;
	
	@ClientString(id = 1800365, message = "A hungry falcon will have meat.")
	public static NpcStringId A_HUNGRY_FALCON_WILL_HAVE_MEAT;
	
	@ClientString(id = 1800366, message = "A household in need will acquire a fortune and meat.")
	public static NpcStringId A_HOUSEHOLD_IN_NEED_WILL_ACQUIRE_A_FORTUNE_AND_MEAT;
	
	@ClientString(id = 1800367, message = "A hard situation will come to its end with materialistic and mental help from others.")
	public static NpcStringId A_HARD_SITUATION_WILL_COME_TO_ITS_END_WITH_MATERIALISTIC_AND_MENTAL_HELP_FROM_OTHERS;
	
	@ClientString(id = 1800368, message = "If you set a firm goal without surrender, there will be a person who can offer help and care.")
	public static NpcStringId IF_YOU_SET_A_FIRM_GOAL_WITHOUT_SURRENDER_THERE_WILL_BE_A_PERSON_WHO_CAN_OFFER_HELP_AND_CARE;
	
	@ClientString(id = 1800369, message = "You'll gain others' trust when you maintain a sincere and honest attitude.")
	public static NpcStringId YOU_LL_GAIN_OTHERS_TRUST_WHEN_YOU_MAINTAIN_A_SINCERE_AND_HONEST_ATTITUDE;
	
	@ClientString(id = 1800370, message = "Be independent at all times.")
	public static NpcStringId BE_INDEPENDENT_AT_ALL_TIMES;
	
	@ClientString(id = 1800371, message = "What good is a wagon with no wheels?")
	public static NpcStringId WHAT_GOOD_IS_A_WAGON_WITH_NO_WHEELS;
	
	@ClientString(id = 1800372, message = "You've set a goal but there may be obstacles in reality.")
	public static NpcStringId YOU_VE_SET_A_GOAL_BUT_THERE_MAY_BE_OBSTACLES_IN_REALITY;
	
	@ClientString(id = 1800373, message = "You're running toward the goal but there won't be as many outcomes as you thought.")
	public static NpcStringId YOU_RE_RUNNING_TOWARD_THE_GOAL_BUT_THERE_WON_T_BE_AS_MANY_OUTCOMES_AS_YOU_THOUGHT;
	
	@ClientString(id = 1800374, message = "There are many things to consider after encountering hindrances.")
	public static NpcStringId THERE_ARE_MANY_THINGS_TO_CONSIDER_AFTER_ENCOUNTERING_HINDRANCES;
	
	@ClientString(id = 1800375, message = "A reckless move may bring a failure.")
	public static NpcStringId A_RECKLESS_MOVE_MAY_BRING_A_FAILURE;
	
	@ClientString(id = 1800376, message = "You may lose people's trust if you lack prudence at all times.")
	public static NpcStringId YOU_MAY_LOSE_PEOPLE_S_TRUST_IF_YOU_LACK_PRUDENCE_AT_ALL_TIMES;
	
	@ClientString(id = 1800377, message = "You may need to reflect on yourself with deliberation and wait for an opportunity.")
	public static NpcStringId YOU_MAY_NEED_TO_REFLECT_ON_YOURSELF_WITH_DELIBERATION_AND_WAIT_FOR_AN_OPPORTUNITY;
	
	@ClientString(id = 1800378, message = "A poor scholar receives a stipend.")
	public static NpcStringId A_POOR_SCHOLAR_RECEIVES_A_STIPEND;
	
	@ClientString(id = 1800379, message = "A scholar gets a pass toward fame and fortune.")
	public static NpcStringId A_SCHOLAR_GETS_A_PASS_TOWARD_FAME_AND_FORTUNE;
	
	@ClientString(id = 1800380, message = "Your ambition and dream will come true.")
	public static NpcStringId YOUR_AMBITION_AND_DREAM_WILL_COME_TRUE;
	
	@ClientString(id = 1800381, message = "Complicated problems around you may start being solved one after another.")
	public static NpcStringId COMPLICATED_PROBLEMS_AROUND_YOU_MAY_START_BEING_SOLVED_ONE_AFTER_ANOTHER;
	
	@ClientString(id = 1800382, message = "You will have a good result if you diligently pursue one goal without being dragged from your past.")
	public static NpcStringId YOU_WILL_HAVE_A_GOOD_RESULT_IF_YOU_DILIGENTLY_PURSUE_ONE_GOAL_WITHOUT_BEING_DRAGGED_FROM_YOUR_PAST;
	
	@ClientString(id = 1800383, message = "You may need to rid yourself of old and worn habits.")
	public static NpcStringId YOU_MAY_NEED_TO_RID_YOURSELF_OF_OLD_AND_WORN_HABITS;
	
	@ClientString(id = 1800384, message = "Be responsible with your tasks but do not hesitate to ask for colleagues' help.")
	public static NpcStringId BE_RESPONSIBLE_WITH_YOUR_TASKS_BUT_DO_NOT_HESITATE_TO_ASK_FOR_COLLEAGUES_HELP;
	
	@ClientString(id = 1800385, message = "Fish transforms into a dragon.")
	public static NpcStringId FISH_TRANSFORMS_INTO_A_DRAGON;
	
	@ClientString(id = 1800386, message = "Your dream may come true, and fame and fortune will come to you.")
	public static NpcStringId YOUR_DREAM_MAY_COME_TRUE_AND_FAME_AND_FORTUNE_WILL_COME_TO_YOU;
	
	@ClientString(id = 1800387, message = "What you've planed will be accomplished.")
	public static NpcStringId WHAT_YOU_VE_PLANED_WILL_BE_ACCOMPLISHED;
	
	@ClientString(id = 1800388, message = "You may acquire money or a new opportunity from a place you wouldn't have thought of.")
	public static NpcStringId YOU_MAY_ACQUIRE_MONEY_OR_A_NEW_OPPORTUNITY_FROM_A_PLACE_YOU_WOULDN_T_HAVE_THOUGHT_OF;
	
	@ClientString(id = 1800389, message = "There will be many offers to you, you may think them over carefully.")
	public static NpcStringId THERE_WILL_BE_MANY_OFFERS_TO_YOU_YOU_MAY_THINK_THEM_OVER_CAREFULLY;
	
	@ClientString(id = 1800390, message = "It may be a good idea not to become involved in others' business.")
	public static NpcStringId IT_MAY_BE_A_GOOD_IDEA_NOT_TO_BECOME_INVOLVED_IN_OTHERS_BUSINESS;
	
	@ClientString(id = 1800391, message = "Everything will go smoothly but be aware of danger from being careless and remiss.")
	public static NpcStringId EVERYTHING_WILL_GO_SMOOTHLY_BUT_BE_AWARE_OF_DANGER_FROM_BEING_CARELESS_AND_REMISS;
	
	@ClientString(id = 1800392, message = "If you sincerely care for someone you love, a big reward will return to you.")
	public static NpcStringId IF_YOU_SINCERELY_CARE_FOR_SOMEONE_YOU_LOVE_A_BIG_REWARD_WILL_RETURN_TO_YOU;
	
	@ClientString(id = 1800393, message = "A remedy is on its way for a serious illness.")
	public static NpcStringId A_REMEDY_IS_ON_ITS_WAY_FOR_A_SERIOUS_ILLNESS;
	
	@ClientString(id = 1800394, message = "You may acquire a precious medicine to recover after suffering a disease of a serious nature.")
	public static NpcStringId YOU_MAY_ACQUIRE_A_PRECIOUS_MEDICINE_TO_RECOVER_AFTER_SUFFERING_A_DISEASE_OF_A_SERIOUS_NATURE;
	
	@ClientString(id = 1800395, message = "You may realize your dream by meeting a man of distinction at a difficult time.")
	public static NpcStringId YOU_MAY_REALIZE_YOUR_DREAM_BY_MEETING_A_MAN_OF_DISTINCTION_AT_A_DIFFICULT_TIME;
	
	@ClientString(id = 1800396, message = "You may suffer one or two hardships on your journey.")
	public static NpcStringId YOU_MAY_SUFFER_ONE_OR_TWO_HARDSHIPS_ON_YOUR_JOURNEY;
	
	@ClientString(id = 1800397, message = "If you keep smiling without despair, people will come to trust you and offer help.")
	public static NpcStringId IF_YOU_KEEP_SMILING_WITHOUT_DESPAIR_PEOPLE_WILL_COME_TO_TRUST_YOU_AND_OFFER_HELP;
	
	@ClientString(id = 1800398, message = "Seek stability rather than dynamics in your life.")
	public static NpcStringId SEEK_STABILITY_RATHER_THAN_DYNAMICS_IN_YOUR_LIFE;
	
	@ClientString(id = 1800399, message = "It's a good idea to be careful and secure at all times.")
	public static NpcStringId IT_S_A_GOOD_IDEA_TO_BE_CAREFUL_AND_SECURE_AT_ALL_TIMES;
	
	@ClientString(id = 1800400, message = "You can't perform the job with bound hands.")
	public static NpcStringId YOU_CAN_T_PERFORM_THE_JOB_WITH_BOUND_HANDS;
	
	@ClientString(id = 1800401, message = "You may lose your drive and feel lost.")
	public static NpcStringId YOU_MAY_LOSE_YOUR_DRIVE_AND_FEEL_LOST;
	
	@ClientString(id = 1800402, message = "You may be unable to concentrate with so many problems occurring.")
	public static NpcStringId YOU_MAY_BE_UNABLE_TO_CONCENTRATE_WITH_SO_MANY_PROBLEMS_OCCURRING;
	
	@ClientString(id = 1800403, message = "Your achievement unfairly may go somewhere else.")
	public static NpcStringId YOUR_ACHIEVEMENT_UNFAIRLY_MAY_GO_SOMEWHERE_ELSE;
	
	@ClientString(id = 1800404, message = "Do not start a task that's not clear to you.")
	public static NpcStringId DO_NOT_START_A_TASK_THAT_S_NOT_CLEAR_TO_YOU;
	
	@ClientString(id = 1800405, message = "You will need to be prepared for all events.")
	public static NpcStringId YOU_WILL_NEED_TO_BE_PREPARED_FOR_ALL_EVENTS;
	
	@ClientString(id = 1800406, message = "Someone will acknowledge you if you relentlessly keep trying and do not give up when facing hardships.")
	public static NpcStringId SOMEONE_WILL_ACKNOWLEDGE_YOU_IF_YOU_RELENTLESSLY_KEEP_TRYING_AND_DO_NOT_GIVE_UP_WHEN_FACING_HARDSHIPS;
	
	@ClientString(id = 1800407, message = "You may perfect yourself like a dragon's horn decorates the dragon.")
	public static NpcStringId YOU_MAY_PERFECT_YOURSELF_LIKE_A_DRAGON_S_HORN_DECORATES_THE_DRAGON;
	
	@ClientString(id = 1800408, message = "Your true value starts to shine.")
	public static NpcStringId YOUR_TRUE_VALUE_STARTS_TO_SHINE;
	
	@ClientString(id = 1800409, message = "Your steady pursuit of new information and staying ahead of others will raise your value.")
	public static NpcStringId YOUR_STEADY_PURSUIT_OF_NEW_INFORMATION_AND_STAYING_AHEAD_OF_OTHERS_WILL_RAISE_YOUR_VALUE;
	
	@ClientString(id = 1800410, message = "Maintaining confidence with work or relationships may bring good results.")
	public static NpcStringId MAINTAINING_CONFIDENCE_WITH_WORK_OR_RELATIONSHIPS_MAY_BRING_GOOD_RESULTS;
	
	@ClientString(id = 1800411, message = "Learn to work with others since overconfidence will bear wrath.")
	public static NpcStringId LEARN_TO_WORK_WITH_OTHERS_SINCE_OVERCONFIDENCE_WILL_BEAR_WRATH;
	
	@ClientString(id = 1800412, message = "The dragon now acquires an eagle's wings.")
	public static NpcStringId THE_DRAGON_NOW_ACQUIRES_AN_EAGLE_S_WINGS;
	
	@ClientString(id = 1800413, message = "As the dragon flies high in the sky, your goals and dreams may come true.")
	public static NpcStringId AS_THE_DRAGON_FLIES_HIGH_IN_THE_SKY_YOUR_GOALS_AND_DREAMS_MAY_COME_TRUE;
	
	@ClientString(id = 1800414, message = "Luck enters into your work, hobby, family and love.")
	public static NpcStringId LUCK_ENTERS_INTO_YOUR_WORK_HOBBY_FAMILY_AND_LOVE;
	
	@ClientString(id = 1800415, message = "Whatever you do, it will accompany winning.")
	public static NpcStringId WHATEVER_YOU_DO_IT_WILL_ACCOMPANY_WINNING;
	
	@ClientString(id = 1800416, message = "It's as good as it gets with unforeseen fortune rolling your way.")
	public static NpcStringId IT_S_AS_GOOD_AS_IT_GETS_WITH_UNFORESEEN_FORTUNE_ROLLING_YOUR_WAY;
	
	@ClientString(id = 1800417, message = "A greedy act with no prudence will bring a surprise at the end.")
	public static NpcStringId A_GREEDY_ACT_WITH_NO_PRUDENCE_WILL_BRING_A_SURPRISE_AT_THE_END;
	
	@ClientString(id = 1800418, message = "Think carefully and act with caution at all times.")
	public static NpcStringId THINK_CAREFULLY_AND_ACT_WITH_CAUTION_AT_ALL_TIMES;
	
	@ClientString(id = 1800419, message = "If a tree doesn't have its roots, there will be no fruit.")
	public static NpcStringId IF_A_TREE_DOESN_T_HAVE_ITS_ROOTS_THERE_WILL_BE_NO_FRUIT;
	
	@ClientString(id = 1800420, message = "Hard work doesn't bear fruit.")
	public static NpcStringId HARD_WORK_DOESN_T_BEAR_FRUIT;
	
	@ClientString(id = 1800421, message = "Financial difficulties may bring an ordeal.")
	public static NpcStringId FINANCIAL_DIFFICULTIES_MAY_BRING_AN_ORDEAL;
	
	@ClientString(id = 1800422, message = "What used to be well managed may stumble one after another.")
	public static NpcStringId WHAT_USED_TO_BE_WELL_MANAGED_MAY_STUMBLE_ONE_AFTER_ANOTHER;
	
	@ClientString(id = 1800423, message = "A feeling of frustration may follow disappointment.")
	public static NpcStringId A_FEELING_OF_FRUSTRATION_MAY_FOLLOW_DISAPPOINTMENT;
	
	@ClientString(id = 1800424, message = "Be cautioned as unharnessed behavior at difficult times can ruin relationships.")
	public static NpcStringId BE_CAUTIONED_AS_UNHARNESSED_BEHAVIOR_AT_DIFFICULT_TIMES_CAN_RUIN_RELATIONSHIPS;
	
	@ClientString(id = 1800425, message = "Curtail greed and be grateful for small returns as modesty is needed.")
	public static NpcStringId CURTAIL_GREED_AND_BE_GRATEFUL_FOR_SMALL_RETURNS_AS_MODESTY_IS_NEEDED;
	
	@ClientString(id = 1800426, message = "The person that came under your wings will leave.")
	public static NpcStringId THE_PERSON_THAT_CAME_UNDER_YOUR_WINGS_WILL_LEAVE;
	
	@ClientString(id = 1800427, message = "Your work and relationship with colleagues will be well managed if you maintain your devotion.")
	public static NpcStringId YOUR_WORK_AND_RELATIONSHIP_WITH_COLLEAGUES_WILL_BE_WELL_MANAGED_IF_YOU_MAINTAIN_YOUR_DEVOTION;
	
	@ClientString(id = 1800428, message = "Calculating your profit in relationships without displaying any courteous manners will bring malicious gossip and ruin your value.")
	public static NpcStringId CALCULATING_YOUR_PROFIT_IN_RELATIONSHIPS_WITHOUT_DISPLAYING_ANY_COURTEOUS_MANNERS_WILL_BRING_MALICIOUS_GOSSIP_AND_RUIN_YOUR_VALUE;
	
	@ClientString(id = 1800429, message = "Consider other's situations and treat them sincerely at all times.")
	public static NpcStringId CONSIDER_OTHER_S_SITUATIONS_AND_TREAT_THEM_SINCERELY_AT_ALL_TIMES;
	
	@ClientString(id = 1800430, message = "Do not loosen up with your precautions.")
	public static NpcStringId DO_NOT_LOOSEN_UP_WITH_YOUR_PRECAUTIONS;
	
	@ClientString(id = 1800431, message = "Reflect other's opinions as a mistake always lies ahead of an arbitrary decision.")
	public static NpcStringId REFLECT_OTHER_S_OPINIONS_AS_A_MISTAKE_ALWAYS_LIES_AHEAD_OF_AN_ARBITRARY_DECISION;
	
	@ClientString(id = 1800432, message = "A blind man goes right through the door.")
	public static NpcStringId A_BLIND_MAN_GOES_RIGHT_THROUGH_THE_DOOR;
	
	@ClientString(id = 1800433, message = "A heart falls into hopelessness as things are in disarray.")
	public static NpcStringId A_HEART_FALLS_INTO_HOPELESSNESS_AS_THINGS_ARE_IN_DISARRAY;
	
	@ClientString(id = 1800434, message = "Hopelessness may fill your heart as your work falls into a maze.")
	public static NpcStringId HOPELESSNESS_MAY_FILL_YOUR_HEART_AS_YOUR_WORK_FALLS_INTO_A_MAZE;
	
	@ClientString(id = 1800435, message = "Difficulties lie ahead of an unforeseen problem even with your hard work.")
	public static NpcStringId DIFFICULTIES_LIE_AHEAD_OF_AN_UNFORESEEN_PROBLEM_EVEN_WITH_YOUR_HARD_WORK;
	
	@ClientString(id = 1800436, message = "There may be more occasions you will want to ask favors from others as you lose confidence in you.")
	public static NpcStringId THERE_MAY_BE_MORE_OCCASIONS_YOU_WILL_WANT_TO_ASK_FAVORS_FROM_OTHERS_AS_YOU_LOSE_CONFIDENCE_IN_YOU;
	
	@ClientString(id = 1800437, message = "Be brave and ambitious as no bird can fly into the sky by staying in their nest.")
	public static NpcStringId BE_BRAVE_AND_AMBITIOUS_AS_NO_BIRD_CAN_FLY_INTO_THE_SKY_BY_STAYING_IN_THEIR_NEST;
	
	@ClientString(id = 1800438, message = "It's a good idea not to start an unclear task and always look for someone you can trust and rely upon.")
	public static NpcStringId IT_S_A_GOOD_IDEA_NOT_TO_START_AN_UNCLEAR_TASK_AND_ALWAYS_LOOK_FOR_SOMEONE_YOU_CAN_TRUST_AND_RELY_UPON;
	
	@ClientString(id = 1800439, message = "Hunting won't be successful as the falcon lacks its claws.")
	public static NpcStringId HUNTING_WON_T_BE_SUCCESSFUL_AS_THE_FALCON_LACKS_ITS_CLAWS;
	
	@ClientString(id = 1800440, message = "A prepared plan won't move smoothly.")
	public static NpcStringId A_PREPARED_PLAN_WON_T_MOVE_SMOOTHLY;
	
	@ClientString(id = 1800441, message = "An easy task may fail if one is consumed by greed and overconfidence.")
	public static NpcStringId AN_EASY_TASK_MAY_FAIL_IF_ONE_IS_CONSUMED_BY_GREED_AND_OVERCONFIDENCE;
	
	@ClientString(id = 1800442, message = "Impatience may lie ahead as the situation is unfavorable.")
	public static NpcStringId IMPATIENCE_MAY_LIE_AHEAD_AS_THE_SITUATION_IS_UNFAVORABLE;
	
	@ClientString(id = 1800443, message = "Thoughtful foresight is needed before a disaster may fall upon you.")
	public static NpcStringId THOUGHTFUL_FORESIGHT_IS_NEEDED_BEFORE_A_DISASTER_MAY_FALL_UPON_YOU;
	
	@ClientString(id = 1800444, message = "Refrain from dictatorial acts as caring for others around you with dignity is much needed.")
	public static NpcStringId REFRAIN_FROM_DICTATORIAL_ACTS_AS_CARING_FOR_OTHERS_AROUND_YOU_WITH_DIGNITY_IS_MUCH_NEEDED;
	
	@ClientString(id = 1800445, message = "The stars are scattered. Not a good sign.")
	public static NpcStringId THE_STARS_ARE_SCATTERED_NOT_A_GOOD_SIGN;
	
	@ClientString(id = 1800446, message = "You may fall into a vexing situation as bad circumstances will arise.")
	public static NpcStringId YOU_MAY_FALL_INTO_A_VEXING_SITUATION_AS_BAD_CIRCUMSTANCES_WILL_ARISE;
	
	@ClientString(id = 1800447, message = "Relationships with people may be contrary to your expectations.")
	public static NpcStringId RELATIONSHIPS_WITH_PEOPLE_MAY_BE_CONTRARY_TO_YOUR_EXPECTATIONS;
	
	@ClientString(id = 1800448, message = "Do not seek a quick fix as the problem needs a fundamental resolution.")
	public static NpcStringId DO_NOT_SEEK_A_QUICK_FIX_AS_THE_PROBLEM_NEEDS_A_FUNDAMENTAL_RESOLUTION;
	
	@ClientString(id = 1800449, message = "Seek peace in your heart as vulgar display of your emotions may harm you.")
	public static NpcStringId SEEK_PEACE_IN_YOUR_HEART_AS_VULGAR_DISPLAY_OF_YOUR_EMOTIONS_MAY_HARM_YOU;
	
	@ClientString(id = 1800450, message = "Information for success may come from the conversations with people around you.")
	public static NpcStringId INFORMATION_FOR_SUCCESS_MAY_COME_FROM_THE_CONVERSATIONS_WITH_PEOPLE_AROUND_YOU;
	
	@ClientString(id = 1800451, message = "Be confident and act reliantly at all times.")
	public static NpcStringId BE_CONFIDENT_AND_ACT_RELIANTLY_AT_ALL_TIMES;
	
	@ClientString(id = 1800452, message = "A child gets a treasure.")
	public static NpcStringId A_CHILD_GETS_A_TREASURE;
	
	@ClientString(id = 1800453, message = "Good fortune and opportunity may lie ahead as if one's born in a golden spoon.")
	public static NpcStringId GOOD_FORTUNE_AND_OPPORTUNITY_MAY_LIE_AHEAD_AS_IF_ONE_S_BORN_IN_A_GOLDEN_SPOON;
	
	@ClientString(id = 1800454, message = "Your life flows as if it's on a silk surface and unexpected fortune and success may come to you.")
	public static NpcStringId YOUR_LIFE_FLOWS_AS_IF_IT_S_ON_A_SILK_SURFACE_AND_UNEXPECTED_FORTUNE_AND_SUCCESS_MAY_COME_TO_YOU;
	
	@ClientString(id = 1800455, message = "Temporary luck may come to you with no effort.")
	public static NpcStringId TEMPORARY_LUCK_MAY_COME_TO_YOU_WITH_NO_EFFORT;
	
	@ClientString(id = 1800456, message = "Plan ahead with patience but execute with swiftness.")
	public static NpcStringId PLAN_AHEAD_WITH_PATIENCE_BUT_EXECUTE_WITH_SWIFTNESS;
	
	@ClientString(id = 1800457, message = "The abilities to amend, foresee and analyze may raise your value.")
	public static NpcStringId THE_ABILITIES_TO_AMEND_FORESEE_AND_ANALYZE_MAY_RAISE_YOUR_VALUE;
	
	@ClientString(id = 1800458, message = "Bigger mistakes will be on the road if you fail to correct a small mistake.")
	public static NpcStringId BIGGER_MISTAKES_WILL_BE_ON_THE_ROAD_IF_YOU_FAIL_TO_CORRECT_A_SMALL_MISTAKE;
	
	@ClientString(id = 1800459, message = "Don't be evasive to accept new findings or experiences.")
	public static NpcStringId DON_T_BE_EVASIVE_TO_ACCEPT_NEW_FINDINGS_OR_EXPERIENCES;
	
	@ClientString(id = 1800460, message = "Don't be irritated as the situations don't move as planned.")
	public static NpcStringId DON_T_BE_IRRITATED_AS_THE_SITUATIONS_DON_T_MOVE_AS_PLANNED;
	
	@ClientString(id = 1800461, message = "Be warned as you may be overwhelmed by surroundings if you lack a clear opinion.")
	public static NpcStringId BE_WARNED_AS_YOU_MAY_BE_OVERWHELMED_BY_SURROUNDINGS_IF_YOU_LACK_A_CLEAR_OPINION;
	
	@ClientString(id = 1800462, message = "You may live an affluent life even without possessions.")
	public static NpcStringId YOU_MAY_LIVE_AN_AFFLUENT_LIFE_EVEN_WITHOUT_POSSESSIONS;
	
	@ClientString(id = 1800463, message = "You will gain popularity as you help people with money you earnestly earned.")
	public static NpcStringId YOU_WILL_GAIN_POPULARITY_AS_YOU_HELP_PEOPLE_WITH_MONEY_YOU_EARNESTLY_EARNED;
	
	@ClientString(id = 1800464, message = "Your heart and body may be in health.")
	public static NpcStringId YOUR_HEART_AND_BODY_MAY_BE_IN_HEALTH;
	
	@ClientString(id = 1800465, message = "Be warned as you may be dragged to an unwanted direction if not cautious.")
	public static NpcStringId BE_WARNED_AS_YOU_MAY_BE_DRAGGED_TO_AN_UNWANTED_DIRECTION_IF_NOT_CAUTIOUS;
	
	@ClientString(id = 1800466, message = "You may meet many new people but it will be difficult to find a perfect person who wins your heart.")
	public static NpcStringId YOU_MAY_MEET_MANY_NEW_PEOPLE_BUT_IT_WILL_BE_DIFFICULT_TO_FIND_A_PERFECT_PERSON_WHO_WINS_YOUR_HEART;
	
	@ClientString(id = 1800467, message = "There may be an occasion where you are consoled by people.")
	public static NpcStringId THERE_MAY_BE_AN_OCCASION_WHERE_YOU_ARE_CONSOLED_BY_PEOPLE;
	
	@ClientString(id = 1800468, message = "It may not be a good time for a change even if there's tedium in daily life.")
	public static NpcStringId IT_MAY_NOT_BE_A_GOOD_TIME_FOR_A_CHANGE_EVEN_IF_THERE_S_TEDIUM_IN_DAILY_LIFE;
	
	@ClientString(id = 1800469, message = "The money you spend for yourself may act as an investment and bring you a return.")
	public static NpcStringId THE_MONEY_YOU_SPEND_FOR_YOURSELF_MAY_ACT_AS_AN_INVESTMENT_AND_BRING_YOU_A_RETURN;
	
	@ClientString(id = 1800470, message = "The money you spend for others will be wasted so be cautious.")
	public static NpcStringId THE_MONEY_YOU_SPEND_FOR_OTHERS_WILL_BE_WASTED_SO_BE_CAUTIOUS;
	
	@ClientString(id = 1800471, message = "Be warned so as not to have unnecessary expenses.")
	public static NpcStringId BE_WARNED_SO_AS_NOT_TO_HAVE_UNNECESSARY_EXPENSES;
	
	@ClientString(id = 1800472, message = "Your star indicated such good luck, participate in bonus giveaways or events.")
	public static NpcStringId YOUR_STAR_INDICATED_SUCH_GOOD_LUCK_PARTICIPATE_IN_BONUS_GIVEAWAYS_OR_EVENTS;
	
	@ClientString(id = 1800473, message = "You may grab unexpected luck.")
	public static NpcStringId YOU_MAY_GRAB_UNEXPECTED_LUCK;
	
	@ClientString(id = 1800474, message = "The person in your heart may naturally come to you.")
	public static NpcStringId THE_PERSON_IN_YOUR_HEART_MAY_NATURALLY_COME_TO_YOU;
	
	@ClientString(id = 1800475, message = "There will be a good result if you keep your own pace regardless of others' judgment.")
	public static NpcStringId THERE_WILL_BE_A_GOOD_RESULT_IF_YOU_KEEP_YOUR_OWN_PACE_REGARDLESS_OF_OTHERS_JUDGMENT;
	
	@ClientString(id = 1800476, message = "Be warned as unexpected luck may be wasted with your reckless comments.")
	public static NpcStringId BE_WARNED_AS_UNEXPECTED_LUCK_MAY_BE_WASTED_WITH_YOUR_RECKLESS_COMMENTS;
	
	@ClientString(id = 1800477, message = "Overconfidence will convince you to carry a task above your reach and there will be consequences.")
	public static NpcStringId OVERCONFIDENCE_WILL_CONVINCE_YOU_TO_CARRY_A_TASK_ABOVE_YOUR_REACH_AND_THERE_WILL_BE_CONSEQUENCES;
	
	@ClientString(id = 1800478, message = "Momentarily delay an important decision.")
	public static NpcStringId MOMENTARILY_DELAY_AN_IMPORTANT_DECISION;
	
	@ClientString(id = 1800479, message = "Trouble spots lie ahead when talking to superiors or people close to you.")
	public static NpcStringId TROUBLE_SPOTS_LIE_AHEAD_WHEN_TALKING_TO_SUPERIORS_OR_PEOPLE_CLOSE_TO_YOU;
	
	@ClientString(id = 1800480, message = "Be warned as your words can hurt others or others' words can hurt you.")
	public static NpcStringId BE_WARNED_AS_YOUR_WORDS_CAN_HURT_OTHERS_OR_OTHERS_WORDS_CAN_HURT_YOU;
	
	@ClientString(id = 1800481, message = "Make a loud boast and you may have to pay to cover unnecessary expenses.")
	public static NpcStringId MAKE_A_LOUD_BOAST_AND_YOU_MAY_HAVE_TO_PAY_TO_COVER_UNNECESSARY_EXPENSES;
	
	@ClientString(id = 1800482, message = "Skillful evasion is needed when dealing with people who pick fights as a disaster may arise from it.")
	public static NpcStringId SKILLFUL_EVASION_IS_NEEDED_WHEN_DEALING_WITH_PEOPLE_WHO_PICK_FIGHTS_AS_A_DISASTER_MAY_ARISE_FROM_IT;
	
	@ClientString(id = 1800483, message = "Keep a low profile as too strong an opinion will attract adverse reactions.")
	public static NpcStringId KEEP_A_LOW_PROFILE_AS_TOO_STRONG_AN_OPINION_WILL_ATTRACT_ADVERSE_REACTIONS;
	
	@ClientString(id = 1800484, message = "Do not unnecessarily provoke misunderstanding as you may be involved in malicious gossip.")
	public static NpcStringId DO_NOT_UNNECESSARILY_PROVOKE_MISUNDERSTANDING_AS_YOU_MAY_BE_INVOLVED_IN_MALICIOUS_GOSSIP;
	
	@ClientString(id = 1800485, message = "Check your belongings as you may lose what you possess.")
	public static NpcStringId CHECK_YOUR_BELONGINGS_AS_YOU_MAY_LOSE_WHAT_YOU_POSSESS;
	
	@ClientString(id = 1800486, message = "Be flexible enough to play up to others.")
	public static NpcStringId BE_FLEXIBLE_ENOUGH_TO_PLAY_UP_TO_OTHERS;
	
	@ClientString(id = 1800487, message = "Pay special attention when meeting or talking to people as relationships may go amiss.")
	public static NpcStringId PAY_SPECIAL_ATTENTION_WHEN_MEETING_OR_TALKING_TO_PEOPLE_AS_RELATIONSHIPS_MAY_GO_AMISS;
	
	@ClientString(id = 1800488, message = "When the important moment arrives, decide upon what you truly want without measuring others' judgment.")
	public static NpcStringId WHEN_THE_IMPORTANT_MOMENT_ARRIVES_DECIDE_UPON_WHAT_YOU_TRULY_WANT_WITHOUT_MEASURING_OTHERS_JUDGMENT;
	
	@ClientString(id = 1800489, message = "Luck will always follow you if you travel and read many books.")
	public static NpcStringId LUCK_WILL_ALWAYS_FOLLOW_YOU_IF_YOU_TRAVEL_AND_READ_MANY_BOOKS;
	
	@ClientString(id = 1800490, message = "Head to a place that needs your advice as good ideas and wisdom will flourish.")
	public static NpcStringId HEAD_TO_A_PLACE_THAT_NEEDS_YOUR_ADVICE_AS_GOOD_IDEAS_AND_WISDOM_WILL_FLOURISH;
	
	@ClientString(id = 1800491, message = "Someone's life may change upon your advice.")
	public static NpcStringId SOMEONE_S_LIFE_MAY_CHANGE_UPON_YOUR_ADVICE;
	
	@ClientString(id = 1800492, message = "It's a proper time to plan for the future rather than a short term plan.")
	public static NpcStringId IT_S_A_PROPER_TIME_TO_PLAN_FOR_THE_FUTURE_RATHER_THAN_A_SHORT_TERM_PLAN;
	
	@ClientString(id = 1800493, message = "Many thoughtful plans at present time will be of great help in the future.")
	public static NpcStringId MANY_THOUGHTFUL_PLANS_AT_PRESENT_TIME_WILL_BE_OF_GREAT_HELP_IN_THE_FUTURE;
	
	@ClientString(id = 1800494, message = "Patience may be needed as a big quarrel arises between you and a person close to you.")
	public static NpcStringId PATIENCE_MAY_BE_NEEDED_AS_A_BIG_QUARREL_ARISES_BETWEEN_YOU_AND_A_PERSON_CLOSE_TO_YOU;
	
	@ClientString(id = 1800495, message = "Do not ask for financial help when the time is difficult. Your pride will be hurt without gaining any money.")
	public static NpcStringId DO_NOT_ASK_FOR_FINANCIAL_HELP_WHEN_THE_TIME_IS_DIFFICULT_YOUR_PRIDE_WILL_BE_HURT_WITHOUT_GAINING_ANY_MONEY;
	
	@ClientString(id = 1800496, message = "Connection with a special person starts with a mere incident.")
	public static NpcStringId CONNECTION_WITH_A_SPECIAL_PERSON_STARTS_WITH_A_MERE_INCIDENT;
	
	@ClientString(id = 1800497, message = "Stubbornness regardless of the matter will only bear danger.")
	public static NpcStringId STUBBORNNESS_REGARDLESS_OF_THE_MATTER_WILL_ONLY_BEAR_DANGER;
	
	@ClientString(id = 1800498, message = "Keep good manners and value taciturnity as light-heartedness may bring misfortune.")
	public static NpcStringId KEEP_GOOD_MANNERS_AND_VALUE_TACITURNITY_AS_LIGHT_HEARTEDNESS_MAY_BRING_MISFORTUNE;
	
	@ClientString(id = 1800499, message = "You may meet the opposite sex.")
	public static NpcStringId YOU_MAY_MEET_THE_OPPOSITE_SEX;
	
	@ClientString(id = 1800500, message = "Greed by wanting to take wealth may bring unfortunate disaster.")
	public static NpcStringId GREED_BY_WANTING_TO_TAKE_WEALTH_MAY_BRING_UNFORTUNATE_DISASTER;
	
	@ClientString(id = 1800501, message = "Loss is ahead, refrain from investing. Try to save the money in your pockets.")
	public static NpcStringId LOSS_IS_AHEAD_REFRAIN_FROM_INVESTING_TRY_TO_SAVE_THE_MONEY_IN_YOUR_POCKETS;
	
	@ClientString(id = 1800502, message = "Your wealth luck is dim, avoid any offers.")
	public static NpcStringId YOUR_WEALTH_LUCK_IS_DIM_AVOID_ANY_OFFERS;
	
	@ClientString(id = 1800503, message = "A bigger challenge may be when delaying today's work.")
	public static NpcStringId A_BIGGER_CHALLENGE_MAY_BE_WHEN_DELAYING_TODAY_S_WORK;
	
	@ClientString(id = 1800504, message = "There will be difficulty, but a good result may be ahead when facing it responsibly.")
	public static NpcStringId THERE_WILL_BE_DIFFICULTY_BUT_A_GOOD_RESULT_MAY_BE_AHEAD_WHEN_FACING_IT_RESPONSIBLY;
	
	@ClientString(id = 1800505, message = "Even with some difficulties, expand the range of your scope where you are in charge. It will return to you as help.")
	public static NpcStringId EVEN_WITH_SOME_DIFFICULTIES_EXPAND_THE_RANGE_OF_YOUR_SCOPE_WHERE_YOU_ARE_IN_CHARGE_IT_WILL_RETURN_TO_YOU_AS_HELP;
	
	@ClientString(id = 1800506, message = "Focus on maintaining organized surroundings to help reduce your losses.")
	public static NpcStringId FOCUS_ON_MAINTAINING_ORGANIZED_SURROUNDINGS_TO_HELP_REDUCE_YOUR_LOSSES;
	
	@ClientString(id = 1800507, message = "Luck lies ahead when waiting for people rather than following them.")
	public static NpcStringId LUCK_LIES_AHEAD_WHEN_WAITING_FOR_PEOPLE_RATHER_THAN_FOLLOWING_THEM;
	
	@ClientString(id = 1800508, message = "Do not offer your hand first even when things are hasty. The relationship may fall apart.")
	public static NpcStringId DO_NOT_OFFER_YOUR_HAND_FIRST_EVEN_WHEN_THINGS_ARE_HASTY_THE_RELATIONSHIP_MAY_FALL_APART;
	
	@ClientString(id = 1800509, message = "Your wealth luck is rising, there will be some good result.")
	public static NpcStringId YOUR_WEALTH_LUCK_IS_RISING_THERE_WILL_BE_SOME_GOOD_RESULT;
	
	@ClientString(id = 1800510, message = "You may fall in danger each time when acting upon improvisation.")
	public static NpcStringId YOU_MAY_FALL_IN_DANGER_EACH_TIME_WHEN_ACTING_UPON_IMPROVISATION;
	
	@ClientString(id = 1800511, message = "Be warned as a childishly act before elders may ruin everything.")
	public static NpcStringId BE_WARNED_AS_A_CHILDISHLY_ACT_BEFORE_ELDERS_MAY_RUIN_EVERYTHING;
	
	@ClientString(id = 1800512, message = "Things will move effortlessly but luck will vanish with your audacity.")
	public static NpcStringId THINGS_WILL_MOVE_EFFORTLESSLY_BUT_LUCK_WILL_VANISH_WITH_YOUR_AUDACITY;
	
	@ClientString(id = 1800513, message = "Luck may be continued only when humility is maintained after success.")
	public static NpcStringId LUCK_MAY_BE_CONTINUED_ONLY_WHEN_HUMILITY_IS_MAINTAINED_AFTER_SUCCESS;
	
	@ClientString(id = 1800514, message = "A new person may appear to create a love triangle.")
	public static NpcStringId A_NEW_PERSON_MAY_APPEAR_TO_CREATE_A_LOVE_TRIANGLE;
	
	@ClientString(id = 1800515, message = "Look for someone with a similar style. It will open up for the good.")
	public static NpcStringId LOOK_FOR_SOMEONE_WITH_A_SIMILAR_STYLE_IT_WILL_OPEN_UP_FOR_THE_GOOD;
	
	@ClientString(id = 1800516, message = "An offer may soon be made to collaborate a task but delaying it will be a good idea.")
	public static NpcStringId AN_OFFER_MAY_SOON_BE_MADE_TO_COLLABORATE_A_TASK_BUT_DELAYING_IT_WILL_BE_A_GOOD_IDEA;
	
	@ClientString(id = 1800517, message = "Partnership is out of luck, avoid someone who rushes you to start a collaboration.")
	public static NpcStringId PARTNERSHIP_IS_OUT_OF_LUCK_AVOID_SOMEONE_WHO_RUSHES_YOU_TO_START_A_COLLABORATION;
	
	@ClientString(id = 1800518, message = "Focus on networking with like-minded people. They may join you for a big mission in the future.")
	public static NpcStringId FOCUS_ON_NETWORKING_WITH_LIKE_MINDED_PEOPLE_THEY_MAY_JOIN_YOU_FOR_A_BIG_MISSION_IN_THE_FUTURE;
	
	@ClientString(id = 1800519, message = "Be warned when someone says you are innocent as that's not a compliment.")
	public static NpcStringId BE_WARNED_WHEN_SOMEONE_SAYS_YOU_ARE_INNOCENT_AS_THAT_S_NOT_A_COMPLIMENT;
	
	@ClientString(id = 1800520, message = "You may be scammed. Be cautious as there may be a big loss by underestimating others.")
	public static NpcStringId YOU_MAY_BE_SCAMMED_BE_CAUTIOUS_AS_THERE_MAY_BE_A_BIG_LOSS_BY_UNDERESTIMATING_OTHERS;
	
	@ClientString(id = 1800521, message = "Luck at decision-making is dim, avoid subjective conclusions and rely on universal common-sense.")
	public static NpcStringId LUCK_AT_DECISION_MAKING_IS_DIM_AVOID_SUBJECTIVE_CONCLUSIONS_AND_RELY_ON_UNIVERSAL_COMMON_SENSE;
	
	@ClientString(id = 1800522, message = "Your weakness may invite hardships, cautiously take a strong position as needed.")
	public static NpcStringId YOUR_WEAKNESS_MAY_INVITE_HARDSHIPS_CAUTIOUSLY_TAKE_A_STRONG_POSITION_AS_NEEDED;
	
	@ClientString(id = 1800523, message = "Be wary of someone who talks and entertains too much. The person may bring you misfortune.")
	public static NpcStringId BE_WARY_OF_SOMEONE_WHO_TALKS_AND_ENTERTAINS_TOO_MUCH_THE_PERSON_MAY_BRING_YOU_MISFORTUNE;
	
	@ClientString(id = 1800524, message = "You may enjoy a beginner's luck.")
	public static NpcStringId YOU_MAY_ENJOY_A_BEGINNER_S_LUCK;
	
	@ClientString(id = 1800525, message = "Your wealth luck is strong but you should know when to withdraw.")
	public static NpcStringId YOUR_WEALTH_LUCK_IS_STRONG_BUT_YOU_SHOULD_KNOW_WHEN_TO_WITHDRAW;
	
	@ClientString(id = 1800526, message = "Already acquired wealth can be lost by greed.")
	public static NpcStringId ALREADY_ACQUIRED_WEALTH_CAN_BE_LOST_BY_GREED;
	
	@ClientString(id = 1800527, message = "Even if you can complete it by yourself, it's a good idea to have someone help you.")
	public static NpcStringId EVEN_IF_YOU_CAN_COMPLETE_IT_BY_YOURSELF_IT_S_A_GOOD_IDEA_TO_HAVE_SOMEONE_HELP_YOU;
	
	@ClientString(id = 1800528, message = "Make harmony with people the priority. Stubbornness may bring hardships.")
	public static NpcStringId MAKE_HARMONY_WITH_PEOPLE_THE_PRIORITY_STUBBORNNESS_MAY_BRING_HARDSHIPS;
	
	@ClientString(id = 1800529, message = "There may be a chance when you can see a new aspect of a close friend.")
	public static NpcStringId THERE_MAY_BE_A_CHANCE_WHEN_YOU_CAN_SEE_A_NEW_ASPECT_OF_A_CLOSE_FRIEND;
	
	@ClientString(id = 1800530, message = "Try to be close to someone different from you without any stereotypical judgment.")
	public static NpcStringId TRY_TO_BE_CLOSE_TO_SOMEONE_DIFFERENT_FROM_YOU_WITHOUT_ANY_STEREOTYPICAL_JUDGMENT;
	
	@ClientString(id = 1800531, message = "Good luck in becoming a leader with many followers. However, it'll only be after hard work.")
	public static NpcStringId GOOD_LUCK_IN_BECOMING_A_LEADER_WITH_MANY_FOLLOWERS_HOWEVER_IT_LL_ONLY_BE_AFTER_HARD_WORK;
	
	@ClientString(id = 1800532, message = "Your wealth luck is rising, expenditures will be followed by substantial income as you are able to sustain.")
	public static NpcStringId YOUR_WEALTH_LUCK_IS_RISING_EXPENDITURES_WILL_BE_FOLLOWED_BY_SUBSTANTIAL_INCOME_AS_YOU_ARE_ABLE_TO_SUSTAIN;
	
	@ClientString(id = 1800533, message = "Be cautious as your wealth luck can be either very good or very bad.")
	public static NpcStringId BE_CAUTIOUS_AS_YOUR_WEALTH_LUCK_CAN_BE_EITHER_VERY_GOOD_OR_VERY_BAD;
	
	@ClientString(id = 1800534, message = "Be warned as a small argument can distance you from a close friend.")
	public static NpcStringId BE_WARNED_AS_A_SMALL_ARGUMENT_CAN_DISTANCE_YOU_FROM_A_CLOSE_FRIEND;
	
	@ClientString(id = 1800535, message = "There is luck in love with a new person.")
	public static NpcStringId THERE_IS_LUCK_IN_LOVE_WITH_A_NEW_PERSON;
	
	@ClientString(id = 1800536, message = "A bigger fortune will be followed by your good deed.")
	public static NpcStringId A_BIGGER_FORTUNE_WILL_BE_FOLLOWED_BY_YOUR_GOOD_DEED;
	
	@ClientString(id = 1800537, message = "There may be a relationship breaking, try to eliminate misunderstandings.")
	public static NpcStringId THERE_MAY_BE_A_RELATIONSHIP_BREAKING_TRY_TO_ELIMINATE_MISUNDERSTANDINGS;
	
	@ClientString(id = 1800538, message = "Be cautious not to be emotionally moved even if it's convincing.")
	public static NpcStringId BE_CAUTIOUS_NOT_TO_BE_EMOTIONALLY_MOVED_EVEN_IF_IT_S_CONVINCING;
	
	@ClientString(id = 1800539, message = "Smiling will bring good luck.")
	public static NpcStringId SMILING_WILL_BRING_GOOD_LUCK;
	
	@ClientString(id = 1800540, message = "It's a good idea to let go of a small loss.")
	public static NpcStringId IT_S_A_GOOD_IDEA_TO_LET_GO_OF_A_SMALL_LOSS;
	
	@ClientString(id = 1800541, message = "Conveying your own truth may be difficult and easy misunderstandings will follow.")
	public static NpcStringId CONVEYING_YOUR_OWN_TRUTH_MAY_BE_DIFFICULT_AND_EASY_MISUNDERSTANDINGS_WILL_FOLLOW;
	
	@ClientString(id = 1800542, message = "There is good luck in a place with many people.")
	public static NpcStringId THERE_IS_GOOD_LUCK_IN_A_PLACE_WITH_MANY_PEOPLE;
	
	@ClientString(id = 1800543, message = "Try to avoid directness if you can.")
	public static NpcStringId TRY_TO_AVOID_DIRECTNESS_IF_YOU_CAN;
	
	@ClientString(id = 1800544, message = "Value substance opposed to the sake honor and look beyond what's in front of you.")
	public static NpcStringId VALUE_SUBSTANCE_OPPOSED_TO_THE_SAKE_HONOR_AND_LOOK_BEYOND_WHAT_S_IN_FRONT_OF_YOU;
	
	@ClientString(id = 1800545, message = "Expanding a relationship with humor may be a good idea.")
	public static NpcStringId EXPANDING_A_RELATIONSHIP_WITH_HUMOR_MAY_BE_A_GOOD_IDEA;
	
	@ClientString(id = 1800546, message = "An enjoyable event may be ahead if you accept a simple bet.")
	public static NpcStringId AN_ENJOYABLE_EVENT_MAY_BE_AHEAD_IF_YOU_ACCEPT_A_SIMPLE_BET;
	
	@ClientString(id = 1800547, message = "Being level-headed not focusing on emotions may help with relationships.")
	public static NpcStringId BEING_LEVEL_HEADED_NOT_FOCUSING_ON_EMOTIONS_MAY_HELP_WITH_RELATIONSHIPS;
	
	@ClientString(id = 1800548, message = "It's a good idea to take care of matters in sequential order without measuring their importance.")
	public static NpcStringId IT_S_A_GOOD_IDEA_TO_TAKE_CARE_OF_MATTERS_IN_SEQUENTIAL_ORDER_WITHOUT_MEASURING_THEIR_IMPORTANCE;
	
	@ClientString(id = 1800549, message = "A determined act after prepared research will attract people.")
	public static NpcStringId A_DETERMINED_ACT_AFTER_PREPARED_RESEARCH_WILL_ATTRACT_PEOPLE;
	
	@ClientString(id = 1800550, message = "A little humor may bring complete attention to you.")
	public static NpcStringId A_LITTLE_HUMOR_MAY_BRING_COMPLETE_ATTENTION_TO_YOU;
	
	@ClientString(id = 1800551, message = "It may not be a good time for an important decision, be wary of temptations and avoid monetary dealings.")
	public static NpcStringId IT_MAY_NOT_BE_A_GOOD_TIME_FOR_AN_IMPORTANT_DECISION_BE_WARY_OF_TEMPTATIONS_AND_AVOID_MONETARY_DEALINGS;
	
	@ClientString(id = 1800552, message = "Pay special attention to advice from a close friend.")
	public static NpcStringId PAY_SPECIAL_ATTENTION_TO_ADVICE_FROM_A_CLOSE_FRIEND;
	
	@ClientString(id = 1800553, message = "There may be moderate solutions to every problem when they're viewed from a 3rd party's point of view.")
	public static NpcStringId THERE_MAY_BE_MODERATE_SOLUTIONS_TO_EVERY_PROBLEM_WHEN_THEY_RE_VIEWED_FROM_A_3RD_PARTY_S_POINT_OF_VIEW;
	
	@ClientString(id = 1800554, message = "Dealings with close friends only bring frustration and headache, politely decline and mention another chance.")
	public static NpcStringId DEALINGS_WITH_CLOSE_FRIENDS_ONLY_BRING_FRUSTRATION_AND_HEADACHE_POLITELY_DECLINE_AND_MENTION_ANOTHER_CHANCE;
	
	@ClientString(id = 1800555, message = "There may be a problem at completion if the basic matters are not considered from the beginning.")
	public static NpcStringId THERE_MAY_BE_A_PROBLEM_AT_COMPLETION_IF_THE_BASIC_MATTERS_ARE_NOT_CONSIDERED_FROM_THE_BEGINNING;
	
	@ClientString(id = 1800556, message = "Distinguishing business from a private matter is needed to succeed.")
	public static NpcStringId DISTINGUISHING_BUSINESS_FROM_A_PRIVATE_MATTER_IS_NEEDED_TO_SUCCEED;
	
	@ClientString(id = 1800557, message = "A change in rules may be helpful when problems are persistent.")
	public static NpcStringId A_CHANGE_IN_RULES_MAY_BE_HELPFUL_WHEN_PROBLEMS_ARE_PERSISTENT;
	
	@ClientString(id = 1800558, message = "Preparing for an unforeseen situation will be difficult when small matters are ignored.")
	public static NpcStringId PREPARING_FOR_AN_UNFORESEEN_SITUATION_WILL_BE_DIFFICULT_WHEN_SMALL_MATTERS_ARE_IGNORED;
	
	@ClientString(id = 1800559, message = "Refrain from getting involved in others' business, try to be loose as a goose.")
	public static NpcStringId REFRAIN_FROM_GETTING_INVOLVED_IN_OTHERS_BUSINESS_TRY_TO_BE_LOOSE_AS_A_GOOSE;
	
	@ClientString(id = 1800560, message = "Being neutral is a good way to go, but clarity may be helpful contrary to your hesitance.")
	public static NpcStringId BEING_NEUTRAL_IS_A_GOOD_WAY_TO_GO_BUT_CLARITY_MAY_BE_HELPFUL_CONTRARY_TO_YOUR_HESITANCE;
	
	@ClientString(id = 1800561, message = "Be cautious of your own actions, the past may bring misunderstandings.")
	public static NpcStringId BE_CAUTIOUS_OF_YOUR_OWN_ACTIONS_THE_PAST_MAY_BRING_MISUNDERSTANDINGS;
	
	@ClientString(id = 1800562, message = "Pay attention to time management, emotions may waste your time.")
	public static NpcStringId PAY_ATTENTION_TO_TIME_MANAGEMENT_EMOTIONS_MAY_WASTE_YOUR_TIME;
	
	@ClientString(id = 1800563, message = "Heroism will be rewarded, but be careful not to display arrogance or lack of sincerity.")
	public static NpcStringId HEROISM_WILL_BE_REWARDED_BUT_BE_CAREFUL_NOT_TO_DISPLAY_ARROGANCE_OR_LACK_OF_SINCERITY;
	
	@ClientString(id = 1800564, message = "If you want to maintain relationship connections, offer reconciliation to those who had misunderstandings with you.")
	public static NpcStringId IF_YOU_WANT_TO_MAINTAIN_RELATIONSHIP_CONNECTIONS_OFFER_RECONCILIATION_TO_THOSE_WHO_HAD_MISUNDERSTANDINGS_WITH_YOU;
	
	@ClientString(id = 1800565, message = "Step forward to solve others' problems when they are unable.")
	public static NpcStringId STEP_FORWARD_TO_SOLVE_OTHERS_PROBLEMS_WHEN_THEY_ARE_UNABLE;
	
	@ClientString(id = 1800566, message = "There may be a little loss, but think of it as an investment for yourself.")
	public static NpcStringId THERE_MAY_BE_A_LITTLE_LOSS_BUT_THINK_OF_IT_AS_AN_INVESTMENT_FOR_YOURSELF;
	
	@ClientString(id = 1800567, message = "Avarice bears a bigger greed, being satisfied with moderation is needed.")
	public static NpcStringId AVARICE_BEARS_A_BIGGER_GREED_BEING_SATISFIED_WITH_MODERATION_IS_NEEDED;
	
	@ClientString(id = 1800568, message = "A rational analysis is needed as unplanned actions may bring criticism.")
	public static NpcStringId A_RATIONAL_ANALYSIS_IS_NEEDED_AS_UNPLANNED_ACTIONS_MAY_BRING_CRITICISM;
	
	@ClientString(id = 1800569, message = "Reflect upon your shortcomings before criticizing others.")
	public static NpcStringId REFLECT_UPON_YOUR_SHORTCOMINGS_BEFORE_CRITICIZING_OTHERS;
	
	@ClientString(id = 1800570, message = "Follow-up care is always needed after an emergency evasion.")
	public static NpcStringId FOLLOW_UP_CARE_IS_ALWAYS_NEEDED_AFTER_AN_EMERGENCY_EVASION;
	
	@ClientString(id = 1800571, message = "You may look for a new challenge but vast knowledge is required.")
	public static NpcStringId YOU_MAY_LOOK_FOR_A_NEW_CHALLENGE_BUT_VAST_KNOWLEDGE_IS_REQUIRED;
	
	@ClientString(id = 1800572, message = "When one puts aside their ego any misunderstanding will be solved.")
	public static NpcStringId WHEN_ONE_PUTS_ASIDE_THEIR_EGO_ANY_MISUNDERSTANDING_WILL_BE_SOLVED;
	
	@ClientString(id = 1800573, message = "Listen to the advice that's given to you with a humble attitude.")
	public static NpcStringId LISTEN_TO_THE_ADVICE_THAT_S_GIVEN_TO_YOU_WITH_A_HUMBLE_ATTITUDE;
	
	@ClientString(id = 1800574, message = "Equilibrium is achieved when one understands a downshift is evident after the rise.")
	public static NpcStringId EQUILIBRIUM_IS_ACHIEVED_WHEN_ONE_UNDERSTANDS_A_DOWNSHIFT_IS_EVIDENT_AFTER_THE_RISE;
	
	@ClientString(id = 1800575, message = "What you sow is what you reap, faithfully follow the plan.")
	public static NpcStringId WHAT_YOU_SOW_IS_WHAT_YOU_REAP_FAITHFULLY_FOLLOW_THE_PLAN;
	
	@ClientString(id = 1800576, message = "Meticulous preparation is needed as spontaneous actions only bear mental and monetary losses.")
	public static NpcStringId METICULOUS_PREPARATION_IS_NEEDED_AS_SPONTANEOUS_ACTIONS_ONLY_BEAR_MENTAL_AND_MONETARY_LOSSES;
	
	@ClientString(id = 1800577, message = "The right time to bear fruit is delayed while the farmer ponders opinions.")
	public static NpcStringId THE_RIGHT_TIME_TO_BEAR_FRUIT_IS_DELAYED_WHILE_THE_FARMER_PONDERS_OPINIONS;
	
	@ClientString(id = 1800578, message = "Help each other among close friends.")
	public static NpcStringId HELP_EACH_OTHER_AMONG_CLOSE_FRIENDS;
	
	@ClientString(id = 1800579, message = "Obsessing over a small profit will place people apart.")
	public static NpcStringId OBSESSING_OVER_A_SMALL_PROFIT_WILL_PLACE_PEOPLE_APART;
	
	@ClientString(id = 1800580, message = "Don't cling to the result of a gamble.")
	public static NpcStringId DON_T_CLING_TO_THE_RESULT_OF_A_GAMBLE;
	
	@ClientString(id = 1800581, message = "Small troubles and arguments are ahead, face them with a mature attitude.")
	public static NpcStringId SMALL_TROUBLES_AND_ARGUMENTS_ARE_AHEAD_FACE_THEM_WITH_A_MATURE_ATTITUDE;
	
	@ClientString(id = 1800582, message = "Neglecting a promise may put you in distress.")
	public static NpcStringId NEGLECTING_A_PROMISE_MAY_PUT_YOU_IN_DISTRESS;
	
	@ClientString(id = 1800583, message = "Delay any dealings as you may easily omit addressing what's important to you.")
	public static NpcStringId DELAY_ANY_DEALINGS_AS_YOU_MAY_EASILY_OMIT_ADDRESSING_WHAT_S_IMPORTANT_TO_YOU;
	
	@ClientString(id = 1800584, message = "A comparison to others may be helpful.")
	public static NpcStringId A_COMPARISON_TO_OTHERS_MAY_BE_HELPFUL;
	
	@ClientString(id = 1800585, message = "What you've endured will return as a benefit.")
	public static NpcStringId WHAT_YOU_VE_ENDURED_WILL_RETURN_AS_A_BENEFIT;
	
	@ClientString(id = 1800586, message = "Try to be courteous to the opposite sex and follow a virtuous path.")
	public static NpcStringId TRY_TO_BE_COURTEOUS_TO_THE_OPPOSITE_SEX_AND_FOLLOW_A_VIRTUOUS_PATH;
	
	@ClientString(id = 1800587, message = "Joy may come from small things.")
	public static NpcStringId JOY_MAY_COME_FROM_SMALL_THINGS;
	
	@ClientString(id = 1800588, message = "Be confident in your actions as good luck shadows the result.")
	public static NpcStringId BE_CONFIDENT_IN_YOUR_ACTIONS_AS_GOOD_LUCK_SHADOWS_THE_RESULT;
	
	@ClientString(id = 1800589, message = "Be confident without hesitation when your honesty is above reproach in dealings.")
	public static NpcStringId BE_CONFIDENT_WITHOUT_HESITATION_WHEN_YOUR_HONESTY_IS_ABOVE_REPROACH_IN_DEALINGS;
	
	@ClientString(id = 1800590, message = "A matter related to a close friend can isolate you, keep staying on the right path.")
	public static NpcStringId A_MATTER_RELATED_TO_A_CLOSE_FRIEND_CAN_ISOLATE_YOU_KEEP_STAYING_ON_THE_RIGHT_PATH;
	
	@ClientString(id = 1800591, message = "Too much focus on the result may bring continuous misfortune.")
	public static NpcStringId TOO_MUCH_FOCUS_ON_THE_RESULT_MAY_BRING_CONTINUOUS_MISFORTUNE;
	
	@ClientString(id = 1800592, message = "Be tenacious until the finish as halfway abandonment causes a troubled ending.")
	public static NpcStringId BE_TENACIOUS_UNTIL_THE_FINISH_AS_HALFWAY_ABANDONMENT_CAUSES_A_TROUBLED_ENDING;
	
	@ClientString(id = 1800593, message = "There will be no advantage in a group deal.")
	public static NpcStringId THERE_WILL_BE_NO_ADVANTAGE_IN_A_GROUP_DEAL;
	
	@ClientString(id = 1800594, message = "Refrain from stepping-up but take a moment to ponder to be flexible with situations.")
	public static NpcStringId REFRAIN_FROM_STEPPING_UP_BUT_TAKE_A_MOMENT_TO_PONDER_TO_BE_FLEXIBLE_WITH_SITUATIONS;
	
	@ClientString(id = 1800595, message = "There will be a small opportunity when information is best utilized.")
	public static NpcStringId THERE_WILL_BE_A_SMALL_OPPORTUNITY_WHEN_INFORMATION_IS_BEST_UTILIZED;
	
	@ClientString(id = 1800596, message = "Belongings are at loose ends, keep track of the things you value.")
	public static NpcStringId BELONGINGS_ARE_AT_LOOSE_ENDS_KEEP_TRACK_OF_THE_THINGS_YOU_VALUE;
	
	@ClientString(id = 1800597, message = "What you sow is what you reap, try your best.")
	public static NpcStringId WHAT_YOU_SOW_IS_WHAT_YOU_REAP_TRY_YOUR_BEST;
	
	@ClientString(id = 1800598, message = "With the beginner's attitude, shortcomings can be easily mended.")
	public static NpcStringId WITH_THE_BEGINNER_S_ATTITUDE_SHORTCOMINGS_CAN_BE_EASILY_MENDED;
	
	@ClientString(id = 1800599, message = "When facing difficulties, seek a totally different direction.")
	public static NpcStringId WHEN_FACING_DIFFICULTIES_SEEK_A_TOTALLY_DIFFERENT_DIRECTION;
	
	@ClientString(id = 1800600, message = "Lifetime savings can disappear with one-time greed.")
	public static NpcStringId LIFETIME_SAVINGS_CAN_DISAPPEAR_WITH_ONE_TIME_GREED;
	
	@ClientString(id = 1800601, message = "With your heart avoid extremes and peace will stay.")
	public static NpcStringId WITH_YOUR_HEART_AVOID_EXTREMES_AND_PEACE_WILL_STAY;
	
	@ClientString(id = 1800602, message = "Be cautious as instant recklessness may bring malicious gossip.")
	public static NpcStringId BE_CAUTIOUS_AS_INSTANT_RECKLESSNESS_MAY_BRING_MALICIOUS_GOSSIP;
	
	@ClientString(id = 1800603, message = "Be tenacious to the end because a strong luck with winning is ahead.")
	public static NpcStringId BE_TENACIOUS_TO_THE_END_BECAUSE_A_STRONG_LUCK_WITH_WINNING_IS_AHEAD;
	
	@ClientString(id = 1800604, message = "Be kind to and care for those close to you, they may help in the future.")
	public static NpcStringId BE_KIND_TO_AND_CARE_FOR_THOSE_CLOSE_TO_YOU_THEY_MAY_HELP_IN_THE_FUTURE;
	
	@ClientString(id = 1800605, message = "Positivity may bring good results.")
	public static NpcStringId POSITIVITY_MAY_BRING_GOOD_RESULTS;
	
	@ClientString(id = 1800606, message = "Be gracious to cover a close friend's fault.")
	public static NpcStringId BE_GRACIOUS_TO_COVER_A_CLOSE_FRIEND_S_FAULT;
	
	@ClientString(id = 1800607, message = "Be prepared for an expected cost.")
	public static NpcStringId BE_PREPARED_FOR_AN_EXPECTED_COST;
	
	@ClientString(id = 1800608, message = "Be considerate to others and avoid focusing only on winning or a wound will be left untreated.")
	public static NpcStringId BE_CONSIDERATE_TO_OTHERS_AND_AVOID_FOCUSING_ONLY_ON_WINNING_OR_A_WOUND_WILL_BE_LEFT_UNTREATED;
	
	@ClientString(id = 1800609, message = "An accessory or decoration may bring a good luck.")
	public static NpcStringId AN_ACCESSORY_OR_DECORATION_MAY_BRING_A_GOOD_LUCK;
	
	@ClientString(id = 1800610, message = "Only reflection and humility may bring success.")
	public static NpcStringId ONLY_REFLECTION_AND_HUMILITY_MAY_BRING_SUCCESS;
	
	@ClientString(id = 1800611, message = "A small misunderstanding may cause quarrels.")
	public static NpcStringId A_SMALL_MISUNDERSTANDING_MAY_CAUSE_QUARRELS;
	
	@ClientString(id = 1800612, message = "Avoid advancing beyond your ability and focus on the flowing stream.")
	public static NpcStringId AVOID_ADVANCING_BEYOND_YOUR_ABILITY_AND_FOCUS_ON_THE_FLOWING_STREAM;
	
	@ClientString(id = 1800613, message = "Considering others with a good heart before self-interest will bring a triumph.")
	public static NpcStringId CONSIDERING_OTHERS_WITH_A_GOOD_HEART_BEFORE_SELF_INTEREST_WILL_BRING_A_TRIUMPH;
	
	@ClientString(id = 1800614, message = "Visiting a place you've never been before may bring luck.")
	public static NpcStringId VISITING_A_PLACE_YOU_VE_NEVER_BEEN_BEFORE_MAY_BRING_LUCK;
	
	@ClientString(id = 1800615, message = "A good thing may happen in a place with a few people.")
	public static NpcStringId A_GOOD_THING_MAY_HAPPEN_IN_A_PLACE_WITH_A_FEW_PEOPLE;
	
	@ClientString(id = 1800616, message = "Being high-strung can cause loss of trust from others because it can be viewed as light-hearted, act sincerely but yet do not lack humor.")
	public static NpcStringId BEING_HIGH_STRUNG_CAN_CAUSE_LOSS_OF_TRUST_FROM_OTHERS_BECAUSE_IT_CAN_BE_VIEWED_AS_LIGHT_HEARTED_ACT_SINCERELY_BUT_YET_DO_NOT_LACK_HUMOR;
	
	@ClientString(id = 1800617, message = "Perfection at the finish can cover faulty work in the process.")
	public static NpcStringId PERFECTION_AT_THE_FINISH_CAN_COVER_FAULTY_WORK_IN_THE_PROCESS;
	
	@ClientString(id = 1800618, message = "Abstain from laziness, much work brings many gains and satisfactory rewards.")
	public static NpcStringId ABSTAIN_FROM_LAZINESS_MUCH_WORK_BRINGS_MANY_GAINS_AND_SATISFACTORY_REWARDS;
	
	@ClientString(id = 1800619, message = "Staying busy rather than being stationary will help.")
	public static NpcStringId STAYING_BUSY_RATHER_THAN_BEING_STATIONARY_WILL_HELP;
	
	@ClientString(id = 1800620, message = "Handling the work by yourself may lead you into temptation.")
	public static NpcStringId HANDLING_THE_WORK_BY_YOURSELF_MAY_LEAD_YOU_INTO_TEMPTATION;
	
	@ClientString(id = 1800621, message = "Pay attention to any small advice without being indifferent.")
	public static NpcStringId PAY_ATTENTION_TO_ANY_SMALL_ADVICE_WITHOUT_BEING_INDIFFERENT;
	
	@ClientString(id = 1800622, message = "Small things make up big things so even value trivial matters.")
	public static NpcStringId SMALL_THINGS_MAKE_UP_BIG_THINGS_SO_EVEN_VALUE_TRIVIAL_MATTERS;
	
	@ClientString(id = 1800623, message = "Action toward the result rather than waiting for the right circumstances may lead you to a fast success.")
	public static NpcStringId ACTION_TOWARD_THE_RESULT_RATHER_THAN_WAITING_FOR_THE_RIGHT_CIRCUMSTANCES_MAY_LEAD_YOU_TO_A_FAST_SUCCESS;
	
	@ClientString(id = 1800624, message = "Don't try to save small expenditures, it will lead to future returns.")
	public static NpcStringId DON_T_TRY_TO_SAVE_SMALL_EXPENDITURES_IT_WILL_LEAD_TO_FUTURE_RETURNS;
	
	@ClientString(id = 1800625, message = "Be cautious to control emotions as temptations are nearby.")
	public static NpcStringId BE_CAUTIOUS_TO_CONTROL_EMOTIONS_AS_TEMPTATIONS_ARE_NEARBY;
	
	@ClientString(id = 1800626, message = "Be warned as neglecting a matter because it's small can cause you trouble.")
	public static NpcStringId BE_WARNED_AS_NEGLECTING_A_MATTER_BECAUSE_IT_S_SMALL_CAN_CAUSE_YOU_TROUBLE;
	
	@ClientString(id = 1800627, message = "Spend when needed rather than trying to unconditionally save.")
	public static NpcStringId SPEND_WHEN_NEEDED_RATHER_THAN_TRYING_TO_UNCONDITIONALLY_SAVE;
	
	@ClientString(id = 1800628, message = "Prejudice will take you to a small gain with a big loss.")
	public static NpcStringId PREJUDICE_WILL_TAKE_YOU_TO_A_SMALL_GAIN_WITH_A_BIG_LOSS;
	
	@ClientString(id = 1800629, message = "Sweet food may bring good luck.")
	public static NpcStringId SWEET_FOOD_MAY_BRING_GOOD_LUCK;
	
	@ClientString(id = 1800630, message = "You may be paid for what you're owed or for your past loss.")
	public static NpcStringId YOU_MAY_BE_PAID_FOR_WHAT_YOU_RE_OWED_OR_FOR_YOUR_PAST_LOSS;
	
	@ClientString(id = 1800631, message = "There may be conflict in basic matters.")
	public static NpcStringId THERE_MAY_BE_CONFLICT_IN_BASIC_MATTERS;
	
	@ClientString(id = 1800632, message = "Be observant to close friends' small behaviors while refraining from excessive kindness.")
	public static NpcStringId BE_OBSERVANT_TO_CLOSE_FRIENDS_SMALL_BEHAVIORS_WHILE_REFRAINING_FROM_EXCESSIVE_KINDNESS;
	
	@ClientString(id = 1800633, message = "Do not show your distress nor lose your smile.")
	public static NpcStringId DO_NOT_SHOW_YOUR_DISTRESS_NOR_LOSE_YOUR_SMILE;
	
	@ClientString(id = 1800634, message = "Showing change may be of help.")
	public static NpcStringId SHOWING_CHANGE_MAY_BE_OF_HELP;
	
	@ClientString(id = 1800635, message = "The intended result may be on your way if the time is perfectly managed.")
	public static NpcStringId THE_INTENDED_RESULT_MAY_BE_ON_YOUR_WAY_IF_THE_TIME_IS_PERFECTLY_MANAGED;
	
	@ClientString(id = 1800636, message = "Hardships may arise if flexibility is not well played.")
	public static NpcStringId HARDSHIPS_MAY_ARISE_IF_FLEXIBILITY_IS_NOT_WELL_PLAYED;
	
	@ClientString(id = 1800637, message = "Keep cool headed because carelessness or inattentiveness may cause misfortune.")
	public static NpcStringId KEEP_COOL_HEADED_BECAUSE_CARELESSNESS_OR_INATTENTIVENESS_MAY_CAUSE_MISFORTUNE;
	
	@ClientString(id = 1800638, message = "Be cautious as you may get hurt after last night's sinister dream.")
	public static NpcStringId BE_CAUTIOUS_AS_YOU_MAY_GET_HURT_AFTER_LAST_NIGHT_S_SINISTER_DREAM;
	
	@ClientString(id = 1800639, message = "A strong wealth luck is ahead but be careful with emotions that may bring losses.")
	public static NpcStringId A_STRONG_WEALTH_LUCK_IS_AHEAD_BUT_BE_CAREFUL_WITH_EMOTIONS_THAT_MAY_BRING_LOSSES;
	
	@ClientString(id = 1800640, message = "Proceed as you wish when it's pertinent to the person you like.")
	public static NpcStringId PROCEED_AS_YOU_WISH_WHEN_IT_S_PERTINENT_TO_THE_PERSON_YOU_LIKE;
	
	@ClientString(id = 1800641, message = "You may deepen the relationship with the opposite sex through conversation.")
	public static NpcStringId YOU_MAY_DEEPEN_THE_RELATIONSHIP_WITH_THE_OPPOSITE_SEX_THROUGH_CONVERSATION;
	
	@ClientString(id = 1800642, message = "Investment into solid material may bring profit.")
	public static NpcStringId INVESTMENT_INTO_SOLID_MATERIAL_MAY_BRING_PROFIT;
	
	@ClientString(id = 1800643, message = "Investment into what you enjoy may be of help.")
	public static NpcStringId INVESTMENT_INTO_WHAT_YOU_ENJOY_MAY_BE_OF_HELP;
	
	@ClientString(id = 1800644, message = "Being busy may help catching up with many changes.")
	public static NpcStringId BEING_BUSY_MAY_HELP_CATCHING_UP_WITH_MANY_CHANGES;
	
	@ClientString(id = 1800645, message = "Choose substance over honor.")
	public static NpcStringId CHOOSE_SUBSTANCE_OVER_HONOR;
	
	@ClientString(id = 1800646, message = "Remember to decline any financial dealings because a good deed may return as resentment.")
	public static NpcStringId REMEMBER_TO_DECLINE_ANY_FINANCIAL_DEALINGS_BECAUSE_A_GOOD_DEED_MAY_RETURN_AS_RESENTMENT;
	
	@ClientString(id = 1800647, message = "Be careful not to make a mistake with a new person.")
	public static NpcStringId BE_CAREFUL_NOT_TO_MAKE_A_MISTAKE_WITH_A_NEW_PERSON;
	
	@ClientString(id = 1800648, message = "Do not be obsessive over a dragged out project since it won't get any better with more time.")
	public static NpcStringId DO_NOT_BE_OBSESSIVE_OVER_A_DRAGGED_OUT_PROJECT_SINCE_IT_WON_T_GET_ANY_BETTER_WITH_MORE_TIME;
	
	@ClientString(id = 1800649, message = "Do not yield what's rightfully yours or tolerate losses.")
	public static NpcStringId DO_NOT_YIELD_WHAT_S_RIGHTFULLY_YOURS_OR_TOLERATE_LOSSES;
	
	@ClientString(id = 1800650, message = "There's luck in relationships so become interested in the opposite sex.")
	public static NpcStringId THERE_S_LUCK_IN_RELATIONSHIPS_SO_BECOME_INTERESTED_IN_THE_OPPOSITE_SEX;
	
	@ClientString(id = 1800651, message = "Seeking others' help rather than trying by yourself may result in two birds with one stone.")
	public static NpcStringId SEEKING_OTHERS_HELP_RATHER_THAN_TRYING_BY_YOURSELF_MAY_RESULT_IN_TWO_BIRDS_WITH_ONE_STONE;
	
	@ClientString(id = 1800652, message = "Persuading the other may result in your gain.")
	public static NpcStringId PERSUADING_THE_OTHER_MAY_RESULT_IN_YOUR_GAIN;
	
	@ClientString(id = 1800653, message = "A good opportunity may come when keeping patience without excessiveness.")
	public static NpcStringId A_GOOD_OPPORTUNITY_MAY_COME_WHEN_KEEPING_PATIENCE_WITHOUT_EXCESSIVENESS;
	
	@ClientString(id = 1800654, message = "Someone of the opposite sex will bring you fortune.")
	public static NpcStringId SOMEONE_OF_THE_OPPOSITE_SEX_WILL_BRING_YOU_FORTUNE;
	
	@ClientString(id = 1800655, message = "Doing favor for other people may bring fortune in the future.")
	public static NpcStringId DOING_FAVOR_FOR_OTHER_PEOPLE_MAY_BRING_FORTUNE_IN_THE_FUTURE;
	
	@ClientString(id = 1800656, message = "Luck may stay near if a smile is kept during difficult times.")
	public static NpcStringId LUCK_MAY_STAY_NEAR_IF_A_SMILE_IS_KEPT_DURING_DIFFICULT_TIMES;
	
	@ClientString(id = 1800657, message = "You may reveal your true self like iron is molten into an strong sword.")
	public static NpcStringId YOU_MAY_REVEAL_YOUR_TRUE_SELF_LIKE_IRON_IS_MOLTEN_INTO_AN_STRONG_SWORD;
	
	@ClientString(id = 1800658, message = "Your value will shine as your potential is finally realized.")
	public static NpcStringId YOUR_VALUE_WILL_SHINE_AS_YOUR_POTENTIAL_IS_FINALLY_REALIZED;
	
	@ClientString(id = 1800659, message = "Tenacious efforts in solving a difficult mission or hardship may bring good results as well as realizing your hidden potential.")
	public static NpcStringId TENACIOUS_EFFORTS_IN_SOLVING_A_DIFFICULT_MISSION_OR_HARDSHIP_MAY_BRING_GOOD_RESULTS_AS_WELL_AS_REALIZING_YOUR_HIDDEN_POTENTIAL;
	
	@ClientString(id = 1800660, message = "People will appreciate your positivity and joyful entertaining.")
	public static NpcStringId PEOPLE_WILL_APPRECIATE_YOUR_POSITIVITY_AND_JOYFUL_ENTERTAINING;
	
	@ClientString(id = 1800661, message = "Things will move smoothly with your full wisdom and abilities.")
	public static NpcStringId THINGS_WILL_MOVE_SMOOTHLY_WITH_YOUR_FULL_WISDOM_AND_ABILITIES;
	
	@ClientString(id = 1800662, message = "You may meet a sage who can help you find the right path.")
	public static NpcStringId YOU_MAY_MEET_A_SAGE_WHO_CAN_HELP_YOU_FIND_THE_RIGHT_PATH;
	
	@ClientString(id = 1800663, message = "Keen instinct and foresight will shine their values.")
	public static NpcStringId KEEN_INSTINCT_AND_FORESIGHT_WILL_SHINE_THEIR_VALUES;
	
	@ClientString(id = 1800664, message = "You may bring good luck to those around you.")
	public static NpcStringId YOU_MAY_BRING_GOOD_LUCK_TO_THOSE_AROUND_YOU;
	
	@ClientString(id = 1800665, message = "Your goal may be realized when emotional details are well defined.")
	public static NpcStringId YOUR_GOAL_MAY_BE_REALIZED_WHEN_EMOTIONAL_DETAILS_ARE_WELL_DEFINED;
	
	@ClientString(id = 1800666, message = "You may enjoy affluence after meeting a precious person.")
	public static NpcStringId YOU_MAY_ENJOY_AFFLUENCE_AFTER_MEETING_A_PRECIOUS_PERSON;
	
	@ClientString(id = 1800667, message = "You may meet the opposite sex who has materialistic attractions.")
	public static NpcStringId YOU_MAY_MEET_THE_OPPOSITE_SEX_WHO_HAS_MATERIALISTIC_ATTRACTIONS;
	
	@ClientString(id = 1800668, message = "A big success will follow all possible efforts in competition.")
	public static NpcStringId A_BIG_SUCCESS_WILL_FOLLOW_ALL_POSSIBLE_EFFORTS_IN_COMPETITION;
	
	@ClientString(id = 1800669, message = "A consequence from past actions will be on display.")
	public static NpcStringId A_CONSEQUENCE_FROM_PAST_ACTIONS_WILL_BE_ON_DISPLAY;
	
	@ClientString(id = 1800670, message = "Whatever happened to you and the other person will replay, but this time, the opposite will be the result.")
	public static NpcStringId WHATEVER_HAPPENED_TO_YOU_AND_THE_OTHER_PERSON_WILL_REPLAY_BUT_THIS_TIME_THE_OPPOSITE_WILL_BE_THE_RESULT;
	
	@ClientString(id = 1800671, message = "You may need to sacrifice for a higher cause.")
	public static NpcStringId YOU_MAY_NEED_TO_SACRIFICE_FOR_A_HIGHER_CAUSE;
	
	@ClientString(id = 1800672, message = "You may lose an item but will gain honor.")
	public static NpcStringId YOU_MAY_LOSE_AN_ITEM_BUT_WILL_GAIN_HONOR;
	
	@ClientString(id = 1800673, message = "A new trial or start may be successful as luck shadows changes.")
	public static NpcStringId A_NEW_TRIAL_OR_START_MAY_BE_SUCCESSFUL_AS_LUCK_SHADOWS_CHANGES;
	
	@ClientString(id = 1800674, message = "Be sophisticated without showing your true emotions as tricks and materialistic temptations lie ahead.")
	public static NpcStringId BE_SOPHISTICATED_WITHOUT_SHOWING_YOUR_TRUE_EMOTIONS_AS_TRICKS_AND_MATERIALISTIC_TEMPTATIONS_LIE_AHEAD;
	
	@ClientString(id = 1800675, message = "Do not attempt a dangerous adventure.")
	public static NpcStringId DO_NOT_ATTEMPT_A_DANGEROUS_ADVENTURE;
	
	@ClientString(id = 1800676, message = "Do not be afraid of change. A risk will be another opportunity.")
	public static NpcStringId DO_NOT_BE_AFRAID_OF_CHANGE_A_RISK_WILL_BE_ANOTHER_OPPORTUNITY;
	
	@ClientString(id = 1800677, message = "Be confident and act tenaciously at all times. You may be able to accomplish to perfection during somewhat unstable situations.")
	public static NpcStringId BE_CONFIDENT_AND_ACT_TENACIOUSLY_AT_ALL_TIMES_YOU_MAY_BE_ABLE_TO_ACCOMPLISH_TO_PERFECTION_DURING_SOMEWHAT_UNSTABLE_SITUATIONS;
	
	@ClientString(id = 1800678, message = "You may expect a bright and hopeful future.")
	public static NpcStringId YOU_MAY_EXPECT_A_BRIGHT_AND_HOPEFUL_FUTURE;
	
	@ClientString(id = 1800679, message = "A rest will promise a bigger development.")
	public static NpcStringId A_REST_WILL_PROMISE_A_BIGGER_DEVELOPMENT;
	
	@ClientString(id = 1800680, message = "A positive mindset is a must!")
	public static NpcStringId A_POSITIVE_MINDSET_IS_A_MUST;
	
	@ClientString(id = 1800681, message = "Positive thinking and energetic actions will take you to the center of the glorious stage.")
	public static NpcStringId POSITIVE_THINKING_AND_ENERGETIC_ACTIONS_WILL_TAKE_YOU_TO_THE_CENTER_OF_THE_GLORIOUS_STAGE;
	
	@ClientString(id = 1800682, message = "Your self confidence and intuition may solve the difficulties.")
	public static NpcStringId YOUR_SELF_CONFIDENCE_AND_INTUITION_MAY_SOLVE_THE_DIFFICULTIES;
	
	@ClientString(id = 1800683, message = "Everything is brilliant and joyful, share it with others. A bigger fortune will follow.")
	public static NpcStringId EVERYTHING_IS_BRILLIANT_AND_JOYFUL_SHARE_IT_WITH_OTHERS_A_BIGGER_FORTUNE_WILL_FOLLOW;
	
	@ClientString(id = 1800684, message = "A fair assessment and reward for past actions lie ahead.")
	public static NpcStringId A_FAIR_ASSESSMENT_AND_REWARD_FOR_PAST_ACTIONS_LIE_AHEAD;
	
	@ClientString(id = 1800685, message = "Pay accurately the old liability or debt, if applicable. A new joy lies ahead.")
	public static NpcStringId PAY_ACCURATELY_THE_OLD_LIABILITY_OR_DEBT_IF_APPLICABLE_A_NEW_JOY_LIES_AHEAD;
	
	@ClientString(id = 1800686, message = "An excessive humility can harm you back.")
	public static NpcStringId AN_EXCESSIVE_HUMILITY_CAN_HARM_YOU_BACK;
	
	@ClientString(id = 1800687, message = "A reward for the past work will come through.")
	public static NpcStringId A_REWARD_FOR_THE_PAST_WORK_WILL_COME_THROUGH;
	
	@ClientString(id = 1800688, message = "Your past fruitless effort will finally be rewarded with something unexpected.")
	public static NpcStringId YOUR_PAST_FRUITLESS_EFFORT_WILL_FINALLY_BE_REWARDED_WITH_SOMETHING_UNEXPECTED;
	
	@ClientString(id = 1800689, message = "There's strong luck in a revival, abandon the old and create the new.")
	public static NpcStringId THERE_S_STRONG_LUCK_IN_A_REVIVAL_ABANDON_THE_OLD_AND_CREATE_THE_NEW;
	
	@ClientString(id = 1800690, message = "You may gain materialistic or mental aid from close friends.")
	public static NpcStringId YOU_MAY_GAIN_MATERIALISTIC_OR_MENTAL_AID_FROM_CLOSE_FRIENDS;
	
	@ClientString(id = 1800691, message = "A good beginning is awaiting you.")
	public static NpcStringId A_GOOD_BEGINNING_IS_AWAITING_YOU;
	
	@ClientString(id = 1800692, message = "You may meet the person you've longed to see.")
	public static NpcStringId YOU_MAY_MEET_THE_PERSON_YOU_VE_LONGED_TO_SEE;
	
	@ClientString(id = 1800693, message = "You may sustain a loss due to your kindness.")
	public static NpcStringId YOU_MAY_SUSTAIN_A_LOSS_DUE_TO_YOUR_KINDNESS;
	
	@ClientString(id = 1800694, message = "Treat everyone well, for a stranger of today may save you another day.")
	public static NpcStringId TREAT_EVERYONE_WELL_FOR_A_STRANGER_OF_TODAY_MAY_SAVE_YOU_ANOTHER_DAY;
	
	@ClientString(id = 1800695, message = "Spread the word! Heroes young and old are gathering to march to the Seed of Destruction, and take down Tiat once and for all!")
	public static NpcStringId SPREAD_THE_WORD_HEROES_YOUNG_AND_OLD_ARE_GATHERING_TO_MARCH_TO_THE_SEED_OF_DESTRUCTION_AND_TAKE_DOWN_TIAT_ONCE_AND_FOR_ALL;
	
	@ClientString(id = 1800696, message = "Spread the word! The flag of the Keucereus Alliance flies proudly over the Seed of Destruction!")
	public static NpcStringId SPREAD_THE_WORD_THE_FLAG_OF_THE_KEUCEREUS_ALLIANCE_FLIES_PROUDLY_OVER_THE_SEED_OF_DESTRUCTION;
	
	@ClientString(id = 1800697, message = "Spread the word! Tiat's Dragontroop are marching to reclaim the Seed of Destruction! All able-bodied warriors, unite!")
	public static NpcStringId SPREAD_THE_WORD_TIAT_S_DRAGONTROOP_ARE_MARCHING_TO_RECLAIM_THE_SEED_OF_DESTRUCTION_ALL_ABLE_BODIED_WARRIORS_UNITE;
	
	@ClientString(id = 1800698, message = "Spread the word! Brave warriors have stormed the Hall of Suffering, and are marching onto the Hall of Erosion in the Seed of Infinity!")
	public static NpcStringId SPREAD_THE_WORD_BRAVE_WARRIORS_HAVE_STORMED_THE_HALL_OF_SUFFERING_AND_ARE_MARCHING_ONTO_THE_HALL_OF_EROSION_IN_THE_SEED_OF_INFINITY;
	
	@ClientString(id = 1800699, message = "Spread the word! Allies have penetrated the Heart of Infinity, and are locked in battle with Ekimus himself while eradicating the remaining Undead!")
	public static NpcStringId SPREAD_THE_WORD_ALLIES_HAVE_PENETRATED_THE_HEART_OF_INFINITY_AND_ARE_LOCKED_IN_BATTLE_WITH_EKIMUS_HIMSELF_WHILE_ERADICATING_THE_REMAINING_UNDEAD;
	
	@ClientString(id = 1800700, message = "Spread the word! The flag of the Keucereus Alliance flies proudly over the Seed of Infinity!")
	public static NpcStringId SPREAD_THE_WORD_THE_FLAG_OF_THE_KEUCEREUS_ALLIANCE_FLIES_PROUDLY_OVER_THE_SEED_OF_INFINITY;
	
	@ClientString(id = 1800702, message = "Spread the word! The Undead are back, and pouring out into the Hall of Suffering and the Hall of Erosion!")
	public static NpcStringId SPREAD_THE_WORD_THE_UNDEAD_ARE_BACK_AND_POURING_OUT_INTO_THE_HALL_OF_SUFFERING_AND_THE_HALL_OF_EROSION;
	
	@ClientString(id = 1800703, message = "Spread the word! The returning Undead are about to bring Ekimus back. Send reinforcements to the Heart of Infinity and the Hall of Suffering!")
	public static NpcStringId SPREAD_THE_WORD_THE_RETURNING_UNDEAD_ARE_ABOUT_TO_BRING_EKIMUS_BACK_SEND_REINFORCEMENTS_TO_THE_HEART_OF_INFINITY_AND_THE_HALL_OF_SUFFERING;
	
	@ClientString(id = 1800704, message = "Stabbing three times!")
	public static NpcStringId STABBING_THREE_TIMES;
	
	@ClientString(id = 1800705, message = "Poor creatures, feel the power of darkness!")
	public static NpcStringId POOR_CREATURES_FEEL_THE_POWER_OF_DARKNESS;
	
	@ClientString(id = 1800706, message = "Whoaaaaaa!!!!")
	public static NpcStringId WHOAAAAAA2;
	
	@ClientString(id = 1800707, message = "You'll regret challenging me!!!!")
	public static NpcStringId YOU_LL_REGRET_CHALLENGING_ME;
	
	@ClientString(id = 1800708, message = "It's currently occupied by the enemy and our troops are attacking.")
	public static NpcStringId IT_S_CURRENTLY_OCCUPIED_BY_THE_ENEMY_AND_OUR_TROOPS_ARE_ATTACKING;
	
	@ClientString(id = 1800709, message = "It's under occupation by our forces, and I heard that Kucereus' clan is organizing the remnants.")
	public static NpcStringId IT_S_UNDER_OCCUPATION_BY_OUR_FORCES_AND_I_HEARD_THAT_KUCEREUS_CLAN_IS_ORGANIZING_THE_REMNANTS;
	
	@ClientString(id = 1800710, message = "Although we currently have control of it, the enemy is pushing back with a powerful attack.")
	public static NpcStringId ALTHOUGH_WE_CURRENTLY_HAVE_CONTROL_OF_IT_THE_ENEMY_IS_PUSHING_BACK_WITH_A_POWERFUL_ATTACK;
	
	@ClientString(id = 1800711, message = "It's under the enemy's occupation, and the military forces of adventurers and clan members are unleashing an onslaught upon the Hall of Suffering and the Hall of Erosion.")
	public static NpcStringId IT_S_UNDER_THE_ENEMY_S_OCCUPATION_AND_THE_MILITARY_FORCES_OF_ADVENTURERS_AND_CLAN_MEMBERS_ARE_UNLEASHING_AN_ONSLAUGHT_UPON_THE_HALL_OF_SUFFERING_AND_THE_HALL_OF_EROSION;
	
	@ClientString(id = 1800712, message = "It's under enemy occupation, but the situation is currently favorable, and an infiltration route to the Heart has been secured. All that is left is the final battle with Ekimus and the clean-up of his followers hiding in the Hall of Suffering!")
	public static NpcStringId IT_S_UNDER_ENEMY_OCCUPATION_BUT_THE_SITUATION_IS_CURRENTLY_FAVORABLE_AND_AN_INFILTRATION_ROUTE_TO_THE_HEART_HAS_BEEN_SECURED_ALL_THAT_IS_LEFT_IS_THE_FINAL_BATTLE_WITH_EKIMUS_AND_THE_CLEAN_UP_OF_HIS_FOLLOWERS_HIDING_IN_THE_HALL_OF_SUFFERING;
	
	@ClientString(id = 1800713, message = "Our forces have occupied it and are currently investigating the depths.")
	public static NpcStringId OUR_FORCES_HAVE_OCCUPIED_IT_AND_ARE_CURRENTLY_INVESTIGATING_THE_DEPTHS;
	
	@ClientString(id = 1800714, message = "It's under occupation by our forces, but the enemy has resurrected and is attacking toward the Hall of Suffering and the Hall of Erosion.")
	public static NpcStringId IT_S_UNDER_OCCUPATION_BY_OUR_FORCES_BUT_THE_ENEMY_HAS_RESURRECTED_AND_IS_ATTACKING_TOWARD_THE_HALL_OF_SUFFERING_AND_THE_HALL_OF_EROSION;
	
	@ClientString(id = 1800715, message = "It's under occupation by our forces, but the enemy has already overtaken the Hall of Erosion and is driving out our forces from the Hall of Suffering toward the Heart. It seems that Ekimus will revive shortly.")
	public static NpcStringId IT_S_UNDER_OCCUPATION_BY_OUR_FORCES_BUT_THE_ENEMY_HAS_ALREADY_OVERTAKEN_THE_HALL_OF_EROSION_AND_IS_DRIVING_OUT_OUR_FORCES_FROM_THE_HALL_OF_SUFFERING_TOWARD_THE_HEART_IT_SEEMS_THAT_EKIMUS_WILL_REVIVE_SHORTLY;
	
	@ClientString(id = 1800717, message = "Tiat's followers are coming to retake the Seed of Destruction! Get ready to stop the enemies!")
	public static NpcStringId TIAT_S_FOLLOWERS_ARE_COMING_TO_RETAKE_THE_SEED_OF_DESTRUCTION_GET_READY_TO_STOP_THE_ENEMIES;
	
	@ClientString(id = 1800718, message = "It's hurting... I'm in pain... What can I do for the pain...")
	public static NpcStringId IT_S_HURTING_I_M_IN_PAIN_WHAT_CAN_I_DO_FOR_THE_PAIN;
	
	@ClientString(id = 1800719, message = "No... When I lose that one... I'll be in more pain...")
	public static NpcStringId NO_WHEN_I_LOSE_THAT_ONE_I_LL_BE_IN_MORE_PAIN;
	
	@ClientString(id = 1800720, message = "Hahahah!!! I captured Santa!! There will be no gifts this year!!!")
	public static NpcStringId HAHAHAH_I_CAPTURED_SANTA_THERE_WILL_BE_NO_GIFTS_THIS_YEAR;
	
	@ClientString(id = 1800721, message = "Now! Why don't you take up the challenge?")
	public static NpcStringId NOW_WHY_DON_T_YOU_TAKE_UP_THE_CHALLENGE;
	
	@ClientString(id = 1800722, message = "Come on, I'll take all of you on!")
	public static NpcStringId COME_ON_I_LL_TAKE_ALL_OF_YOU_ON;
	
	@ClientString(id = 1800723, message = "How about it? I think I won?")
	public static NpcStringId HOW_ABOUT_IT_I_THINK_I_WON;
	
	@ClientString(id = 1800724, message = "Now!! Those of you who lost, go away!")
	public static NpcStringId NOW_THOSE_OF_YOU_WHO_LOST_GO_AWAY;
	
	@ClientString(id = 1800725, message = "What a bunch of losers.")
	public static NpcStringId WHAT_A_BUNCH_OF_LOSERS;
	
	@ClientString(id = 1800726, message = "I guess you came to rescue Santa. But you picked the wrong person.")
	public static NpcStringId I_GUESS_YOU_CAME_TO_RESCUE_SANTA_BUT_YOU_PICKED_THE_WRONG_PERSON;
	
	@ClientString(id = 1800727, message = "Ah, okay...")
	public static NpcStringId AH_OKAY;
	
	@ClientString(id = 1800728, message = "Agh!! I wasn't going to do that!")
	public static NpcStringId AGH_I_WASN_T_GOING_TO_DO_THAT;
	
	@ClientString(id = 1800729, message = "You're cursed!! Oh.. What?")
	public static NpcStringId YOU_RE_CURSED_OH_WHAT;
	
	@ClientString(id = 1800730, message = "Have you done nothing but rock-paper-scissors??")
	public static NpcStringId HAVE_YOU_DONE_NOTHING_BUT_ROCK_PAPER_SCISSORS;
	
	@ClientString(id = 1800731, message = "Stop it, no more... I did it because I was too lonely...")
	public static NpcStringId STOP_IT_NO_MORE_I_DID_IT_BECAUSE_I_WAS_TOO_LONELY;
	
	@ClientString(id = 1800732, message = "I have to release Santa... How infuriating!!!")
	public static NpcStringId I_HAVE_TO_RELEASE_SANTA_HOW_INFURIATING;
	
	@ClientString(id = 1800733, message = "I hate happy Happy Holidays!!!")
	public static NpcStringId I_HATE_HAPPY_HAPPY_HOLIDAYS;
	
	@ClientString(id = 1800734, message = "Oh. I'm bored.")
	public static NpcStringId OH_I_M_BORED;
	
	@ClientString(id = 1800735, message = "Shall I go to take a look if Santa is still there? Hehe")
	public static NpcStringId SHALL_I_GO_TO_TAKE_A_LOOK_IF_SANTA_IS_STILL_THERE_HEHE;
	
	@ClientString(id = 1800736, message = "Oh ho ho.... Happy Holidays!!")
	public static NpcStringId OH_HO_HO_HAPPY_HOLIDAYS;
	
	@ClientString(id = 1800737, message = "Santa could give nice presents only if he's released from the Turkey...")
	public static NpcStringId SANTA_COULD_GIVE_NICE_PRESENTS_ONLY_IF_HE_S_RELEASED_FROM_THE_TURKEY;
	
	@ClientString(id = 1800738, message = "Oh ho ho... Oh ho ho... Thank you. Ladies and gentlemen!  I will repay you for sure.")
	public static NpcStringId OH_HO_HO_OH_HO_HO_THANK_YOU_LADIES_AND_GENTLEMEN_I_WILL_REPAY_YOU_FOR_SURE;
	
	@ClientString(id = 1800739, message = "Happy Holidays~ You're doing a good job.")
	public static NpcStringId HAPPY_HOLIDAYS_YOU_RE_DOING_A_GOOD_JOB;
	
	@ClientString(id = 1800740, message = "Happy Holidays~ Thank you for rescuing me from that wretched Turkey.")
	public static NpcStringId HAPPY_HOLIDAYS_THANK_YOU_FOR_RESCUING_ME_FROM_THAT_WRETCHED_TURKEY;
	
	@ClientString(id = 1800741, message = "$s1 . I have prepared a gift for you.")
	public static NpcStringId S1_I_HAVE_PREPARED_A_GIFT_FOR_YOU;
	
	@ClientString(id = 1800742, message = "I have a gift for $s1.")
	public static NpcStringId I_HAVE_A_GIFT_FOR_S1;
	
	@ClientString(id = 1800743, message = "Take a look at the inventory. I hope you like the gift I gave you.")
	public static NpcStringId TAKE_A_LOOK_AT_THE_INVENTORY_I_HOPE_YOU_LIKE_THE_GIFT_I_GAVE_YOU;
	
	@ClientString(id = 1800744, message = "Take a look at the inventory. Perhaps there might be a big present~")
	public static NpcStringId TAKE_A_LOOK_AT_THE_INVENTORY_PERHAPS_THERE_MIGHT_BE_A_BIG_PRESENT;
	
	@ClientString(id = 1800745, message = "I'm tired of dealing with you. I'm leaving.")
	public static NpcStringId I_M_TIRED_OF_DEALING_WITH_YOU_I_M_LEAVING;
	
	@ClientString(id = 1800746, message = "When are you going to stop? I slowly started to be tired of it.")
	public static NpcStringId WHEN_ARE_YOU_GOING_TO_STOP_I_SLOWLY_STARTED_TO_BE_TIRED_OF_IT;
	
	@ClientString(id = 1800747, message = "Message from Santa: Many blessings to $s1, who saved me~")
	public static NpcStringId MESSAGE_FROM_SANTA_MANY_BLESSINGS_TO_S1_WHO_SAVED_ME;
	
	@ClientString(id = 1800748, message = "I am already dead. You cannot kill me again...")
	public static NpcStringId I_AM_ALREADY_DEAD_YOU_CANNOT_KILL_ME_AGAIN;
	
	@ClientString(id = 1800749, message = "Oh followers of the Dragon of Darkness, fight by my side!")
	public static NpcStringId OH_FOLLOWERS_OF_THE_DRAGON_OF_DARKNESS_FIGHT_BY_MY_SIDE;
	
	@ClientString(id = 1800750, message = "The Dragon Race... are invading... Prepare... for battle...")
	public static NpcStringId THE_DRAGON_RACE_ARE_INVADING_PREPARE_FOR_BATTLE;
	
	@ClientString(id = 1800751, message = "$s1 rescued Santa of $s2 territory from the turkey.")
	public static NpcStringId S1_RESCUED_SANTA_OF_S2_TERRITORY_FROM_THE_TURKEY;
	
	@ClientString(id = 1800752, message = "Santa Rescue Success!")
	public static NpcStringId SANTA_RESCUE_SUCCESS;
	
	@ClientString(id = 1800753, message = "$s1 received +$s2 $s3 through the weapon coupon.")
	public static NpcStringId S1_RECEIVED_S2_S3_THROUGH_THE_WEAPON_COUPON;
	
	@ClientString(id = 1800754, message = "Don't go prattling on!")
	public static NpcStringId DON_T_GO_PRATTLING_ON;
	
	@ClientString(id = 1800755, message = "You lowlifes with not even an ounce of pride! You're not worthy of opposing me!")
	public static NpcStringId YOU_LOWLIFES_WITH_NOT_EVEN_AN_OUNCE_OF_PRIDE_YOU_RE_NOT_WORTHY_OF_OPPOSING_ME;
	
	@ClientString(id = 1800756, message = "Roar~! No~ Oink oink~! See~! I'm a pig~! Oink oink~!!")
	public static NpcStringId ROAR_NO_OINK_OINK_SEE_I_M_A_PIG_OINK_OINK;
	
	@ClientString(id = 1800757, message = "Who am I~ Where am I... Oink oink!")
	public static NpcStringId WHO_AM_I_WHERE_AM_I_OINK_OINK;
	
	@ClientString(id = 1800758, message = "I just followed my friend here for fun~ Oink oink!")
	public static NpcStringId I_JUST_FOLLOWED_MY_FRIEND_HERE_FOR_FUN_OINK_OINK;
	
	@ClientString(id = 1800759, message = "Wow! That's what I call a Cure-All!")
	public static NpcStringId WOW_THAT_S_WHAT_I_CALL_A_CURE_ALL;
	
	@ClientString(id = 1800760, message = "I'm starving. Should I go chew some grass?")
	public static NpcStringId I_M_STARVING_SHOULD_I_GO_CHEW_SOME_GRASS;
	
	@ClientString(id = 1800761, message = "Thank you, thank you!")
	public static NpcStringId THANK_YOU_THANK_YOU;
	
	@ClientString(id = 1800762, message = "Whoa! Feels like my energy is back~!")
	public static NpcStringId WHOA_FEELS_LIKE_MY_ENERGY_IS_BACK;
	
	@ClientString(id = 1800763, message = "I feel so light! This is it! I know this feeling!")
	public static NpcStringId I_FEEL_SO_LIGHT_THIS_IS_IT_I_KNOW_THIS_FEELING;
	
	@ClientString(id = 1800764, message = "Wow~ My fatigue is completely gone!")
	public static NpcStringId WOW_MY_FATIGUE_IS_COMPLETELY_GONE;
	
	@ClientString(id = 1800765, message = "Hey~ that ominous feeling has disappeared!")
	public static NpcStringId HEY_THAT_OMINOUS_FEELING_HAS_DISAPPEARED;
	
	@ClientString(id = 1800766, message = "My body feels as light as a feather~")
	public static NpcStringId MY_BODY_FEELS_AS_LIGHT_AS_A_FEATHER;
	
	@ClientString(id = 1800767, message = "What's this? Food?")
	public static NpcStringId WHAT_S_THIS_FOOD;
	
	@ClientString(id = 1800768, message = "My energy is overflowing!!! I don't need any Energy Potion~")
	public static NpcStringId MY_ENERGY_IS_OVERFLOWING_I_DON_T_NEED_ANY_ENERGY_POTION;
	
	@ClientString(id = 1800769, message = "What's the matter? That's an amateur move!")
	public static NpcStringId WHAT_S_THE_MATTER_THAT_S_AN_AMATEUR_MOVE;
	
	@ClientString(id = 1800770, message = "Fortune Timer: Reward increases 2 times if completed within 10 seconds!")
	public static NpcStringId FORTUNE_TIMER_REWARD_INCREASES_2_TIMES_IF_COMPLETED_WITHIN_10_SECONDS;
	
	@ClientString(id = 1800771, message = "Fortune Timer: Reward increases 2 times if completed within 40 seconds!")
	public static NpcStringId FORTUNE_TIMER_REWARD_INCREASES_2_TIMES_IF_COMPLETED_WITHIN_40_SECONDS;
	
	@ClientString(id = 1800772, message = "40 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING;
	
	@ClientString(id = 1800773, message = "39 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING2;
	
	@ClientString(id = 1800774, message = "38 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING3;
	
	@ClientString(id = 1800775, message = "37 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING4;
	
	@ClientString(id = 1800776, message = "36 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING5;
	
	@ClientString(id = 1800777, message = "35 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING6;
	
	@ClientString(id = 1800778, message = "34 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING7;
	
	@ClientString(id = 1800779, message = "33 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING8;
	
	@ClientString(id = 1800780, message = "32 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING9;
	
	@ClientString(id = 1800781, message = "31 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING10;
	
	@ClientString(id = 1800782, message = "30 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING11;
	
	@ClientString(id = 1800783, message = "29 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING12;
	
	@ClientString(id = 1800784, message = "28 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING13;
	
	@ClientString(id = 1800785, message = "27 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING14;
	
	@ClientString(id = 1800786, message = "26 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING15;
	
	@ClientString(id = 1800787, message = "25 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING16;
	
	@ClientString(id = 1800788, message = "24 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING17;
	
	@ClientString(id = 1800789, message = "23 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING18;
	
	@ClientString(id = 1800790, message = "22 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING19;
	
	@ClientString(id = 1800791, message = "21 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING20;
	
	@ClientString(id = 1800792, message = "20 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING21;
	
	@ClientString(id = 1800793, message = "19 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING22;
	
	@ClientString(id = 1800794, message = "18 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING23;
	
	@ClientString(id = 1800795, message = "17 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING24;
	
	@ClientString(id = 1800796, message = "16 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING25;
	
	@ClientString(id = 1800797, message = "15 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING26;
	
	@ClientString(id = 1800798, message = "14 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING27;
	
	@ClientString(id = 1800799, message = "13 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING28;
	
	@ClientString(id = 1800800, message = "12 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING29;
	
	@ClientString(id = 1800801, message = "11 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING30;
	
	@ClientString(id = 1800802, message = "10 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING31;
	
	@ClientString(id = 1800803, message = "9 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING32;
	
	@ClientString(id = 1800804, message = "8 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING33;
	
	@ClientString(id = 1800805, message = "7 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING34;
	
	@ClientString(id = 1800806, message = "6 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING35;
	
	@ClientString(id = 1800807, message = "5 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING36;
	
	@ClientString(id = 1800808, message = "4 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING37;
	
	@ClientString(id = 1800809, message = "3 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING38;
	
	@ClientString(id = 1800810, message = "2 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING39;
	
	@ClientString(id = 1800811, message = "1 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING40;
	
	@ClientString(id = 1800812, message = "Time up!")
	public static NpcStringId TIME_UP;
	
	@ClientString(id = 1800813, message = "Mission failed!")
	public static NpcStringId MISSION_FAILED;
	
	@ClientString(id = 1800814, message = "Mission success!")
	public static NpcStringId MISSION_SUCCESS;
	
	@ClientString(id = 1800815, message = "Hey! I already have an owner!")
	public static NpcStringId HEY_I_ALREADY_HAVE_AN_OWNER;
	
	@ClientString(id = 1800816, message = "Hey~. Are you planning on eating me?! Use a Cupid's Energy Potion already!")
	public static NpcStringId HEY_ARE_YOU_PLANNING_ON_EATING_ME_USE_A_CUPID_S_ENERGY_POTION_ALREADY;
	
	@ClientString(id = 1800817, message = "I'll pass on an amateur's meridian massage. Use a Cupid's Energy Potion already!")
	public static NpcStringId I_LL_PASS_ON_AN_AMATEUR_S_MERIDIAN_MASSAGE_USE_A_CUPID_S_ENERGY_POTION_ALREADY;
	
	@ClientString(id = 1800818, message = "I already feel more energetic. Thanks, $s1 -")
	public static NpcStringId I_ALREADY_FEEL_MORE_ENERGETIC_THANKS_S1;
	
	@ClientString(id = 1800819, message = "How refreshing! You wouldn't happen to be a master masseuse, $s1, would you?")
	public static NpcStringId HOW_REFRESHING_YOU_WOULDN_T_HAPPEN_TO_BE_A_MASTER_MASSEUSE_S1_WOULD_YOU;
	
	@ClientString(id = 1800820, message = "Incredible. From now on, I'll compare all massages to this one with $s1!")
	public static NpcStringId INCREDIBLE_FROM_NOW_ON_I_LL_COMPARE_ALL_MASSAGES_TO_THIS_ONE_WITH_S1;
	
	@ClientString(id = 1800821, message = "Isn't it tough doing it all on your own! Next time, try making a party with some comrades~")
	public static NpcStringId ISN_T_IT_TOUGH_DOING_IT_ALL_ON_YOUR_OWN_NEXT_TIME_TRY_MAKING_A_PARTY_WITH_SOME_COMRADES;
	
	@ClientString(id = 1800822, message = "Sorry, but I'll leave my friend in your care as well~ Thanks.")
	public static NpcStringId SORRY_BUT_I_LL_LEAVE_MY_FRIEND_IN_YOUR_CARE_AS_WELL_THANKS;
	
	@ClientString(id = 1800823, message = "Sniff sniff~ Do you smell the scent of a fresh-baked baguette?")
	public static NpcStringId SNIFF_SNIFF_DO_YOU_SMELL_THE_SCENT_OF_A_FRESH_BAKED_BAGUETTE;
	
	@ClientString(id = 1800824, message = "Who am I~? Let me know if you wanna buy my bread~~")
	public static NpcStringId WHO_AM_I_LET_ME_KNOW_IF_YOU_WANNA_BUY_MY_BREAD;
	
	@ClientString(id = 1800825, message = "I just want to make your weapons stronger~ Abra Kadabra~!")
	public static NpcStringId I_JUST_WANT_TO_MAKE_YOUR_WEAPONS_STRONGER_ABRA_KADABRA;
	
	@ClientString(id = 1800826, message = "What? You don't like it? What's the matter with you~ Like an amateur!")
	public static NpcStringId WHAT_YOU_DON_T_LIKE_IT_WHAT_S_THE_MATTER_WITH_YOU_LIKE_AN_AMATEUR;
	
	@ClientString(id = 1800827, message = "Hey~ Did you tell a lie on April Fool's Day? Don't talk to me if you didn't~!")
	public static NpcStringId HEY_DID_YOU_TELL_A_LIE_ON_APRIL_FOOL_S_DAY_DON_T_TALK_TO_ME_IF_YOU_DIDN_T;
	
	@ClientString(id = 1800828, message = "Grunt... What's... wrong with me...")
	public static NpcStringId GRUNT_WHAT_S_WRONG_WITH_ME;
	
	@ClientString(id = 1800829, message = "...Grunt... Oh...")
	public static NpcStringId GRUNT_OH;
	
	@ClientString(id = 1800830, message = "The grave robber warrior has been filled with dark energy and is attacking you!")
	public static NpcStringId THE_GRAVE_ROBBER_WARRIOR_HAS_BEEN_FILLED_WITH_DARK_ENERGY_AND_IS_ATTACKING_YOU;
	
	@ClientString(id = 1800831, message = "The altar guardian is scrutinizing you!! n You will repay your debt with death.")
	public static NpcStringId THE_ALTAR_GUARDIAN_IS_SCRUTINIZING_YOU_N_YOU_WILL_REPAY_YOUR_DEBT_WITH_DEATH;
	
	@ClientString(id = 1800832, message = "Wait... Wait! Stop! Save me, and I'll give you 10,000,000 Adena!!")
	public static NpcStringId WAIT_WAIT_STOP_SAVE_ME_AND_I_LL_GIVE_YOU_10_000_000_ADENA;
	
	@ClientString(id = 1800833, message = "I... don't want to fight...")
	public static NpcStringId I_DON_T_WANT_TO_FIGHT;
	
	@ClientString(id = 1800834, message = "Is this really necessary...?")
	public static NpcStringId IS_THIS_REALLY_NECESSARY;
	
	@ClientString(id = 1800835, message = "Th... Thanks... I could have become good friends with you...")
	public static NpcStringId TH_THANKS_I_COULD_HAVE_BECOME_GOOD_FRIENDS_WITH_YOU;
	
	@ClientString(id = 1800836, message = "I'll give you 10,000,000 Adena, like I promised! I might be an orc who keeps my promises!")
	public static NpcStringId I_LL_GIVE_YOU_10_000_000_ADENA_LIKE_I_PROMISED_I_MIGHT_BE_AN_ORC_WHO_KEEPS_MY_PROMISES;
	
	@ClientString(id = 1800837, message = "Thanks, but that thing about 10,000,000 Adena was a lie! See ya!!")
	public static NpcStringId THANKS_BUT_THAT_THING_ABOUT_10_000_000_ADENA_WAS_A_LIE_SEE_YA;
	
	@ClientString(id = 1800838, message = "You're pretty dumb to believe me!")
	public static NpcStringId YOU_RE_PRETTY_DUMB_TO_BELIEVE_ME;
	
	@ClientString(id = 1800839, message = "Ugh... A curse upon you...!")
	public static NpcStringId UGH_A_CURSE_UPON_YOU;
	
	@ClientString(id = 1800840, message = "I really... didn't want... to fight...")
	public static NpcStringId I_REALLY_DIDN_T_WANT_TO_FIGHT;
	
	@ClientString(id = 1800841, message = "All 4 of Kasha's Eyes have appeared!")
	public static NpcStringId ALL_4_OF_KASHA_S_EYES_HAVE_APPEARED;
	
	@ClientString(id = 1800842, message = "Kasha's Eye emanates a foreboding curse.")
	public static NpcStringId KASHA_S_EYE_EMANATES_A_FOREBODING_CURSE;
	
	@ClientString(id = 1800843, message = "Defeat Kasha's Eye to put a stop to the foreboding curse!")
	public static NpcStringId DEFEAT_KASHA_S_EYE_TO_PUT_A_STOP_TO_THE_FOREBODING_CURSE;
	
	@ClientString(id = 1800844, message = "I protect the altar! You can't escape the altar!")
	public static NpcStringId I_PROTECT_THE_ALTAR_YOU_CAN_T_ESCAPE_THE_ALTAR;
	
	@ClientString(id = 1800845, message = "$s1! That stranger must be defeated. Here is the ultimate help!")
	public static NpcStringId S1_THAT_STRANGER_MUST_BE_DEFEATED_HERE_IS_THE_ULTIMATE_HELP;
	
	@ClientString(id = 1800846, message = "Look here!! $s1. Don't fall too far behind.")
	public static NpcStringId LOOK_HERE_S1_DON_T_FALL_TOO_FAR_BEHIND;
	
	@ClientString(id = 1800847, message = "Well done. $s1. Your help is much appreciated.")
	public static NpcStringId WELL_DONE_S1_YOUR_HELP_IS_MUCH_APPRECIATED;
	
	@ClientString(id = 1800848, message = "Who has awakened us from our slumber?")
	public static NpcStringId WHO_HAS_AWAKENED_US_FROM_OUR_SLUMBER;
	
	@ClientString(id = 1800849, message = "All will pay a severe price to me and these here.")
	public static NpcStringId ALL_WILL_PAY_A_SEVERE_PRICE_TO_ME_AND_THESE_HERE;
	
	@ClientString(id = 1800850, message = "Shyeed's cry is steadily dying down.")
	public static NpcStringId SHYEED_S_CRY_IS_STEADILY_DYING_DOWN;
	
	@ClientString(id = 1800851, message = "Alert! Alert! Damage detection recognized. Countermeasures enabled.")
	public static NpcStringId ALERT_ALERT_DAMAGE_DETECTION_RECOGNIZED_COUNTERMEASURES_ENABLED;
	
	@ClientString(id = 1800852, message = "Target recognition achieved. Attack sequence commencing.")
	public static NpcStringId TARGET_RECOGNITION_ACHIEVED_ATTACK_SEQUENCE_COMMENCING;
	
	@ClientString(id = 1800853, message = "Target. Threat. Level. Launching. Strongest. Countermeasure.")
	public static NpcStringId TARGET_THREAT_LEVEL_LAUNCHING_STRONGEST_COUNTERMEASURE;
	
	@ClientString(id = 1800854, message = "The purification field is being attacked. Guardian Spirits! Protect the Magic Force!!")
	public static NpcStringId THE_PURIFICATION_FIELD_IS_BEING_ATTACKED_GUARDIAN_SPIRITS_PROTECT_THE_MAGIC_FORCE;
	
	@ClientString(id = 1800855, message = "Protect the Braziers of Purity at all costs!!")
	public static NpcStringId PROTECT_THE_BRAZIERS_OF_PURITY_AT_ALL_COSTS;
	
	@ClientString(id = 1800856, message = "Defend our domain even at risk of your own life!")
	public static NpcStringId DEFEND_OUR_DOMAIN_EVEN_AT_RISK_OF_YOUR_OWN_LIFE;
	
	@ClientString(id = 1800857, message = "Peunglui muglanep!")
	public static NpcStringId PEUNGLUI_MUGLANEP;
	
	@ClientString(id = 1800858, message = "Naia waganagel peutagun!")
	public static NpcStringId NAIA_WAGANAGEL_PEUTAGUN;
	
	@ClientString(id = 1800859, message = "Drive device partial destruction impulse result")
	public static NpcStringId DRIVE_DEVICE_PARTIAL_DESTRUCTION_IMPULSE_RESULT;
	
	@ClientString(id = 1800860, message = "Even the Magic Force  binds you, you  will never be forgiven...")
	public static NpcStringId EVEN_THE_MAGIC_FORCE_BINDS_YOU_YOU_WILL_NEVER_BE_FORGIVEN;
	
	@ClientString(id = 1800861, message = "Oh giants, an intruder has been discovered.")
	public static NpcStringId OH_GIANTS_AN_INTRUDER_HAS_BEEN_DISCOVERED;
	
	@ClientString(id = 1800862, message = "All is vanity! but this cannot be the end!")
	public static NpcStringId ALL_IS_VANITY_BUT_THIS_CANNOT_BE_THE_END;
	
	@ClientString(id = 1800863, message = "Those who are in front of my eyes! will be destroyed!")
	public static NpcStringId THOSE_WHO_ARE_IN_FRONT_OF_MY_EYES_WILL_BE_DESTROYED;
	
	@ClientString(id = 1800864, message = "I am tired! Do not wake me up again!")
	public static NpcStringId I_AM_TIRED_DO_NOT_WAKE_ME_UP_AGAIN;
	
	@ClientString(id = 1800865, message = "Intruder detected")
	public static NpcStringId INTRUDER_DETECTED;
	
	@ClientString(id = 1800866, message = "The candles can lead you to Zaken. Destroy him")
	public static NpcStringId THE_CANDLES_CAN_LEAD_YOU_TO_ZAKEN_DESTROY_HIM;
	
	@ClientString(id = 1800867, message = "Who dares awaken the mighty Zaken?")
	public static NpcStringId WHO_DARES_AWAKEN_THE_MIGHTY_ZAKEN;
	
	@ClientString(id = 1800868, message = "Ye not be finding me below the drink!")
	public static NpcStringId YE_NOT_BE_FINDING_ME_BELOW_THE_DRINK;
	
	@ClientString(id = 1800869, message = "Ye must be three sheets to the wind if yer lookin for me there.")
	public static NpcStringId YE_MUST_BE_THREE_SHEETS_TO_THE_WIND_IF_YER_LOOKIN_FOR_ME_THERE;
	
	@ClientString(id = 1800870, message = "Ye not be finding me in the Crows Nest.")
	public static NpcStringId YE_NOT_BE_FINDING_ME_IN_THE_CROWS_NEST;
	
	@ClientString(id = 1800871, message = "Sorry but this is all I have.. Give me a break!")
	public static NpcStringId SORRY_BUT_THIS_IS_ALL_I_HAVE_GIVE_ME_A_BREAK;
	
	@ClientString(id = 1800872, message = "Peunglui muglanep Naia waganagel peutagun!")
	public static NpcStringId PEUNGLUI_MUGLANEP_NAIA_WAGANAGEL_PEUTAGUN;
	
	@ClientString(id = 1800873, message = "Drive device entire destruction moving suspension")
	public static NpcStringId DRIVE_DEVICE_ENTIRE_DESTRUCTION_MOVING_SUSPENSION;
	
	@ClientString(id = 1800874, message = "Ah ah... From the Magic Force,  no more... I will be freed")
	public static NpcStringId AH_AH_FROM_THE_MAGIC_FORCE_NO_MORE_I_WILL_BE_FREED;
	
	@ClientString(id = 1800875, message = "You guys are detected!")
	public static NpcStringId YOU_GUYS_ARE_DETECTED;
	
	@ClientString(id = 1800876, message = "What kind of creatures are you!")
	public static NpcStringId WHAT_KIND_OF_CREATURES_ARE_YOU;
	
	@ClientString(id = 1800877, message = "$s2 of level $s1 is acquired")
	public static NpcStringId S2_OF_LEVEL_S1_IS_ACQUIRED;
	
	@ClientString(id = 1800878, message = "Life Stone from the Beginning acquired")
	public static NpcStringId LIFE_STONE_FROM_THE_BEGINNING_ACQUIRED;
	
	@ClientString(id = 1800879, message = "When inventory weight/number are more than 80%, the Life Stone from the Beginning cannot be acquired.")
	public static NpcStringId WHEN_INVENTORY_WEIGHT_NUMBER_ARE_MORE_THAN_80_THE_LIFE_STONE_FROM_THE_BEGINNING_CANNOT_BE_ACQUIRED;
	
	@ClientString(id = 1800880, message = "You are under my thumb!!")
	public static NpcStringId YOU_ARE_UNDER_MY_THUMB;
	
	@ClientString(id = 1800881, message = "20 minutes are added to the  remaining time in the Instant Zone.")
	public static NpcStringId MINUTES_ARE_ADDED_TO_THE_REMAINING_TIME_IN_THE_INSTANT_ZONE;
	
	@ClientString(id = 1800882, message = "Hurry hurry")
	public static NpcStringId HURRY_HURRY;
	
	@ClientString(id = 1800883, message = "I am not that type of person who stays in one place for a long time")
	public static NpcStringId I_AM_NOT_THAT_TYPE_OF_PERSON_WHO_STAYS_IN_ONE_PLACE_FOR_A_LONG_TIME;
	
	@ClientString(id = 1800884, message = "It's hard for me to keep standing like this")
	public static NpcStringId IT_S_HARD_FOR_ME_TO_KEEP_STANDING_LIKE_THIS;
	
	@ClientString(id = 1800885, message = "Why don't I go that way this time")
	public static NpcStringId WHY_DON_T_I_GO_THAT_WAY_THIS_TIME;
	
	@ClientString(id = 1800886, message = "Welcome!")
	public static NpcStringId WELCOME;
	
	@ClientString(id = 1800887, message = "Is that it? Is that the extent of your abilities? Put in a little more effort!!")
	public static NpcStringId IS_THAT_IT_IS_THAT_THE_EXTENT_OF_YOUR_ABILITIES_PUT_IN_A_LITTLE_MORE_EFFORT;
	
	@ClientString(id = 1800888, message = "Your abilities are pitiful... You are far from a worthy opponent.")
	public static NpcStringId YOUR_ABILITIES_ARE_PITIFUL_YOU_ARE_FAR_FROM_A_WORTHY_OPPONENT;
	
	@ClientString(id = 1800889, message = "Even after death you order me to wander around looking for the scapegoats!")
	public static NpcStringId EVEN_AFTER_DEATH_YOU_ORDER_ME_TO_WANDER_AROUND_LOOKING_FOR_THE_SCAPEGOATS;
	
	@ClientString(id = 1800890, message = "Here goes the Heatstroke! If you can withstand the hot heatstroke up to the 3rd stage, the Sultriness will come to you.")
	public static NpcStringId HERE_GOES_THE_HEATSTROKE_IF_YOU_CAN_WITHSTAND_THE_HOT_HEATSTROKE_UP_TO_THE_3RD_STAGE_THE_SULTRINESS_WILL_COME_TO_YOU;
	
	@ClientString(id = 1800891, message = "Just you wait! Humidity is a blistering fireball which can easily withstand plenty of Cool Air Cannon attacks!")
	public static NpcStringId JUST_YOU_WAIT_HUMIDITY_IS_A_BLISTERING_FIREBALL_WHICH_CAN_EASILY_WITHSTAND_PLENTY_OF_COOL_AIR_CANNON_ATTACKS;
	
	@ClientString(id = 1800892, message = "In order to defeat Humidity, you must obtain the Heatstroke Prevention Effect from Doctor Ice and fire more than 10 rounds of the Cool Air Cannon on it!")
	public static NpcStringId IN_ORDER_TO_DEFEAT_HUMIDITY_YOU_MUST_OBTAIN_THE_HEATSTROKE_PREVENTION_EFFECT_FROM_DOCTOR_ICE_AND_FIRE_MORE_THAN_10_ROUNDS_OF_THE_COOL_AIR_CANNON_ON_IT;
	
	@ClientString(id = 1800893, message = "You are here, $s1! I'll teach you a lesson! Bring it on!")
	public static NpcStringId YOU_ARE_HERE_S1_I_LL_TEACH_YOU_A_LESSON_BRING_IT_ON;
	
	@ClientString(id = 1800894, message = "That's cold! Isn't it one of those Cool Packs? I hate anything that's cold!")
	public static NpcStringId THAT_S_COLD_ISN_T_IT_ONE_OF_THOSE_COOL_PACKS_I_HATE_ANYTHING_THAT_S_COLD;
	
	@ClientString(id = 1800895, message = "Huh! You've missed! Is that all you have?")
	public static NpcStringId HUH_YOU_VE_MISSED_IS_THAT_ALL_YOU_HAVE;
	
	@ClientString(id = 1800896, message = "I will give you precious things that I have stolen. So stop bothering me!")
	public static NpcStringId I_WILL_GIVE_YOU_PRECIOUS_THINGS_THAT_I_HAVE_STOLEN_SO_STOP_BOTHERING_ME;
	
	@ClientString(id = 1800897, message = "I was going to give you a jackpot item. You don't have enough inventory room. See you next time.")
	public static NpcStringId I_WAS_GOING_TO_GIVE_YOU_A_JACKPOT_ITEM_YOU_DON_T_HAVE_ENOUGH_INVENTORY_ROOM_SEE_YOU_NEXT_TIME;
	
	@ClientString(id = 1800898, message = "$s1 defeated the Sultriness and acquired item S84")
	public static NpcStringId S1_DEFEATED_THE_SULTRINESS_AND_ACQUIRED_ITEM_S84;
	
	@ClientString(id = 1800899, message = "$s1 defeated the Sultriness and acquired item S80")
	public static NpcStringId S1_DEFEATED_THE_SULTRINESS_AND_ACQUIRED_ITEM_S80;
	
	@ClientString(id = 1800900, message = "I am not here for you! Your Cool Pack attack does not work against me!")
	public static NpcStringId I_AM_NOT_HERE_FOR_YOU_YOUR_COOL_PACK_ATTACK_DOES_NOT_WORK_AGAINST_ME;
	
	@ClientString(id = 1800901, message = "Uh oh? Where are you hiding? There is nobody who matches my skills? Well, I guess I'd better get going....")
	public static NpcStringId UH_OH_WHERE_ARE_YOU_HIDING_THERE_IS_NOBODY_WHO_MATCHES_MY_SKILLS_WELL_I_GUESS_I_D_BETTER_GET_GOING;
	
	@ClientString(id = 1800902, message = "Why are you not responding? You don't even have any Cool Packs! You can't fight me!")
	public static NpcStringId WHY_ARE_YOU_NOT_RESPONDING_YOU_DON_T_EVEN_HAVE_ANY_COOL_PACKS_YOU_CAN_T_FIGHT_ME;
	
	@ClientString(id = 1800903, message = "Oh~! Where I be~? Who call me~?")
	public static NpcStringId OH_WHERE_I_BE_WHO_CALL_ME;
	
	@ClientString(id = 1800904, message = "Tada~! It's a watermelon~!")
	public static NpcStringId TADA_IT_S_A_WATERMELON;
	
	@ClientString(id = 1800905, message = ")_(... Did ya call me...?")
	public static NpcStringId DID_YA_CALL_ME;
	
	@ClientString(id = 1800906, message = "Enter the watermelon~! It's gonna grow and grow from now on!")
	public static NpcStringId ENTER_THE_WATERMELON_IT_S_GONNA_GROW_AND_GROW_FROM_NOW_ON;
	
	@ClientString(id = 1800907, message = "Oh, ouch~! Did I see you before~?")
	public static NpcStringId OH_OUCH_DID_I_SEE_YOU_BEFORE;
	
	@ClientString(id = 1800908, message = "A new season! Summer is all about the watermelon~!")
	public static NpcStringId A_NEW_SEASON_SUMMER_IS_ALL_ABOUT_THE_WATERMELON;
	
	@ClientString(id = 1800909, message = "Did ya call~? Ho! Thought you'd get something~?^^")
	public static NpcStringId DID_YA_CALL_HO_THOUGHT_YOU_D_GET_SOMETHING;
	
	@ClientString(id = 1800910, message = "Do you want to see my beautiful self~?")
	public static NpcStringId DO_YOU_WANT_TO_SEE_MY_BEAUTIFUL_SELF;
	
	@ClientString(id = 1800911, message = "Hohoho! Let's do it together~!")
	public static NpcStringId HOHOHO_LET_S_DO_IT_TOGETHER;
	
	@ClientString(id = 1800912, message = "It's a giant watermelon if you raise it right~ And a watermelon slice if you mess up~!")
	public static NpcStringId IT_S_A_GIANT_WATERMELON_IF_YOU_RAISE_IT_RIGHT_AND_A_WATERMELON_SLICE_IF_YOU_MESS_UP;
	
	@ClientString(id = 1800913, message = "Tada~ Transformation~ complete~!")
	public static NpcStringId TADA_TRANSFORMATION_COMPLETE;
	
	@ClientString(id = 1800914, message = "Am I a rain watermelon? Or a defective watermelon?")
	public static NpcStringId AM_I_A_RAIN_WATERMELON_OR_A_DEFECTIVE_WATERMELON;
	
	@ClientString(id = 1800915, message = "Now! I've gotten big~! Everyone, come at me!")
	public static NpcStringId NOW_I_VE_GOTTEN_BIG_EVERYONE_COME_AT_ME;
	
	@ClientString(id = 1800916, message = "Get bigger~ Get stronger~! Tell me your wish~~!")
	public static NpcStringId GET_BIGGER_GET_STRONGER_TELL_ME_YOUR_WISH;
	
	@ClientString(id = 1800917, message = "A watermelon slice's wish! But I'm bigger already?")
	public static NpcStringId A_WATERMELON_SLICE_S_WISH_BUT_I_M_BIGGER_ALREADY;
	
	@ClientString(id = 1800918, message = "A large watermelon's wish! Well, try to break me!")
	public static NpcStringId A_LARGE_WATERMELON_S_WISH_WELL_TRY_TO_BREAK_ME;
	
	@ClientString(id = 1800919, message = "I'm done growing~! I'm running away now~^^")
	public static NpcStringId I_M_DONE_GROWING_I_M_RUNNING_AWAY_NOW;
	
	@ClientString(id = 1800920, message = "If you let me go, I'll give you ten million Adena!")
	public static NpcStringId IF_YOU_LET_ME_GO_I_LL_GIVE_YOU_TEN_MILLION_ADENA;
	
	@ClientString(id = 1800921, message = "Freedom~! What do you think I have inside?")
	public static NpcStringId FREEDOM_WHAT_DO_YOU_THINK_I_HAVE_INSIDE;
	
	@ClientString(id = 1800922, message = "OK~ OK~ Good job. You know what to do next, right?")
	public static NpcStringId OK_OK_GOOD_JOB_YOU_KNOW_WHAT_TO_DO_NEXT_RIGHT;
	
	@ClientString(id = 1800923, message = "Look here! Do it right! You spilled! This precious...")
	public static NpcStringId LOOK_HERE_DO_IT_RIGHT_YOU_SPILLED_THIS_PRECIOUS;
	
	@ClientString(id = 1800924, message = "Ah~ Refreshing! Spray a little more~!")
	public static NpcStringId AH_REFRESHING_SPRAY_A_LITTLE_MORE;
	
	@ClientString(id = 1800925, message = "Gulp gulp~ Great! But isn't there more?")
	public static NpcStringId GULP_GULP_GREAT_BUT_ISN_T_THERE_MORE;
	
	@ClientString(id = 1800926, message = "Can't you even aim right? Have you even been to the army?")
	public static NpcStringId CAN_T_YOU_EVEN_AIM_RIGHT_HAVE_YOU_EVEN_BEEN_TO_THE_ARMY;
	
	@ClientString(id = 1800927, message = "Did you mix this with water~? Why's it taste like this~!")
	public static NpcStringId DID_YOU_MIX_THIS_WITH_WATER_WHY_S_IT_TASTE_LIKE_THIS;
	
	@ClientString(id = 1800928, message = "Oh! Good! Do a little more. Yeah~")
	public static NpcStringId OH_GOOD_DO_A_LITTLE_MORE_YEAH;
	
	@ClientString(id = 1800929, message = "Hoho! It's not there! Over here~! Am I so small that you can even spray me right?")
	public static NpcStringId HOHO_IT_S_NOT_THERE_OVER_HERE_AM_I_SO_SMALL_THAT_YOU_CAN_EVEN_SPRAY_ME_RIGHT;
	
	@ClientString(id = 1800930, message = "Yuck! What is this~! Are you sure this is nectar~?")
	public static NpcStringId YUCK_WHAT_IS_THIS_ARE_YOU_SURE_THIS_IS_NECTAR;
	
	@ClientString(id = 1800931, message = "Do your best~! I become a big watermelon after just five bottles~!")
	public static NpcStringId DO_YOUR_BEST_I_BECOME_A_BIG_WATERMELON_AFTER_JUST_FIVE_BOTTLES;
	
	@ClientString(id = 1800932, message = "Of course~! Watermelon is the best nectar! Hahaha~!")
	public static NpcStringId OF_COURSE_WATERMELON_IS_THE_BEST_NECTAR_HAHAHA;
	
	@ClientString(id = 1800933, message = "Ahh! Good~ )_( Slap slap me~")
	public static NpcStringId AHH_GOOD_SLAP_SLAP_ME;
	
	@ClientString(id = 1800934, message = "Oww! You're just beating me now~ Give me nectar~")
	public static NpcStringId OWW_YOU_RE_JUST_BEATING_ME_NOW_GIVE_ME_NECTAR;
	
	@ClientString(id = 1800935, message = "Look~!! It's gonna break~!")
	public static NpcStringId LOOK_IT_S_GONNA_BREAK;
	
	@ClientString(id = 1800936, message = "Now! Are you trying to eat without doing the work? Fine~ Do what you want~ I'll hate you if you don't give me any nectar!")
	public static NpcStringId NOW_ARE_YOU_TRYING_TO_EAT_WITHOUT_DOING_THE_WORK_FINE_DO_WHAT_YOU_WANT_I_LL_HATE_YOU_IF_YOU_DON_T_GIVE_ME_ANY_NECTAR;
	
	@ClientString(id = 1800937, message = "Hit me more! Hit me more!")
	public static NpcStringId HIT_ME_MORE_HIT_ME_MORE;
	
	@ClientString(id = 1800938, message = "I'm gonna wither like this~ Damn it~!")
	public static NpcStringId I_M_GONNA_WITHER_LIKE_THIS_DAMN_IT;
	
	@ClientString(id = 1800939, message = "Hey you~ If I die like this, there'll be no item either~ Are you really so stingy with the nectar?")
	public static NpcStringId HEY_YOU_IF_I_DIE_LIKE_THIS_THERE_LL_BE_NO_ITEM_EITHER_ARE_YOU_REALLY_SO_STINGY_WITH_THE_NECTAR;
	
	@ClientString(id = 1800940, message = "It's just a little more!! Good luck~!")
	public static NpcStringId IT_S_JUST_A_LITTLE_MORE_GOOD_LUCK;
	
	@ClientString(id = 1800941, message = "Save me~ I'm about to die without tasting nectar even once~")
	public static NpcStringId SAVE_ME_I_M_ABOUT_TO_DIE_WITHOUT_TASTING_NECTAR_EVEN_ONCE;
	
	@ClientString(id = 1800942, message = "If I die like this, I'll just be a watermelon slice~!")
	public static NpcStringId IF_I_DIE_LIKE_THIS_I_LL_JUST_BE_A_WATERMELON_SLICE;
	
	@ClientString(id = 1800943, message = "I'm getting stronger~? I think I'll be able to run away in 30 seconds~ Hoho~")
	public static NpcStringId I_M_GETTING_STRONGER_I_THINK_I_LL_BE_ABLE_TO_RUN_AWAY_IN_30_SECONDS_HOHO;
	
	@ClientString(id = 1800944, message = "It's goodbye after 20 seconds~!")
	public static NpcStringId IT_S_GOODBYE_AFTER_20_SECONDS;
	
	@ClientString(id = 1800945, message = "Yeah, 10 seconds left~! 9... 8... 7...!")
	public static NpcStringId YEAH_10_SECONDS_LEFT_9_8_7;
	
	@ClientString(id = 1800946, message = "I'm leaving in 2 minutes if you don't give me any nectar~!")
	public static NpcStringId I_M_LEAVING_IN_2_MINUTES_IF_YOU_DON_T_GIVE_ME_ANY_NECTAR;
	
	@ClientString(id = 1800947, message = "I'm leaving in 1 minutes if you don't give me any nectar~!")
	public static NpcStringId I_M_LEAVING_IN_1_MINUTES_IF_YOU_DON_T_GIVE_ME_ANY_NECTAR;
	
	@ClientString(id = 1800948, message = "I'm leaving now~! Then, goodbye~!")
	public static NpcStringId I_M_LEAVING_NOW_THEN_GOODBYE;
	
	@ClientString(id = 1800949, message = "Sorry, but this large watermelon is disappearing here~!")
	public static NpcStringId SORRY_BUT_THIS_LARGE_WATERMELON_IS_DISAPPEARING_HERE;
	
	@ClientString(id = 1800950, message = "Too late~! Have a good time~!")
	public static NpcStringId TOO_LATE_HAVE_A_GOOD_TIME;
	
	@ClientString(id = 1800951, message = "Ding ding~ That's the bell. Put away your weapons and try for next time~!")
	public static NpcStringId DING_DING_THAT_S_THE_BELL_PUT_AWAY_YOUR_WEAPONS_AND_TRY_FOR_NEXT_TIME;
	
	@ClientString(id = 1800952, message = "Too bad~! You raised it up, too! -_-")
	public static NpcStringId TOO_BAD_YOU_RAISED_IT_UP_TOO;
	
	@ClientString(id = 1800953, message = "Oh~ What a nice sound~")
	public static NpcStringId OH_WHAT_A_NICE_SOUND;
	
	@ClientString(id = 1800954, message = "The instrument is nice, but there's no song. Shall I sing for you?")
	public static NpcStringId THE_INSTRUMENT_IS_NICE_BUT_THERE_S_NO_SONG_SHALL_I_SING_FOR_YOU;
	
	@ClientString(id = 1800955, message = "What beautiful music!")
	public static NpcStringId WHAT_BEAUTIFUL_MUSIC;
	
	@ClientString(id = 1800956, message = "I feel good~ Play some more!")
	public static NpcStringId I_FEEL_GOOD_PLAY_SOME_MORE;
	
	@ClientString(id = 1800957, message = "My heart is being captured by the sound of Crono!")
	public static NpcStringId MY_HEART_IS_BEING_CAPTURED_BY_THE_SOUND_OF_CRONO;
	
	@ClientString(id = 1800958, message = "Get the notes right~! Hey old man~ That was wrong!")
	public static NpcStringId GET_THE_NOTES_RIGHT_HEY_OLD_MAN_THAT_WAS_WRONG;
	
	@ClientString(id = 1800959, message = "I like it~!")
	public static NpcStringId I_LIKE_IT;
	
	@ClientString(id = 1800960, message = "Ooh~~ My body wants to open!")
	public static NpcStringId OOH_MY_BODY_WANTS_TO_OPEN;
	
	@ClientString(id = 1800961, message = "Oh~! This chord! My heart is being torn! Play a little more!")
	public static NpcStringId OH_THIS_CHORD_MY_HEART_IS_BEING_TORN_PLAY_A_LITTLE_MORE;
	
	@ClientString(id = 1800962, message = "It's this~ This! I wanted this sound! Why don't you try becoming a singer?")
	public static NpcStringId IT_S_THIS_THIS_I_WANTED_THIS_SOUND_WHY_DON_T_YOU_TRY_BECOMING_A_SINGER;
	
	@ClientString(id = 1800963, message = "You can try a hundred times on this~ You won't get anything good~!")
	public static NpcStringId YOU_CAN_TRY_A_HUNDRED_TIMES_ON_THIS_YOU_WON_T_GET_ANYTHING_GOOD;
	
	@ClientString(id = 1800964, message = "It hurts~! Play just the instrument~!")
	public static NpcStringId IT_HURTS_PLAY_JUST_THE_INSTRUMENT;
	
	@ClientString(id = 1800965, message = "Only good music can open my body!")
	public static NpcStringId ONLY_GOOD_MUSIC_CAN_OPEN_MY_BODY;
	
	@ClientString(id = 1800966, message = "Not this, but you know, that~ What you got as a Chronicle souvenir. Play with that!")
	public static NpcStringId NOT_THIS_BUT_YOU_KNOW_THAT_WHAT_YOU_GOT_AS_A_CHRONICLE_SOUVENIR_PLAY_WITH_THAT;
	
	@ClientString(id = 1800967, message = "Why~ You have no music? Boring... I'm leaving now~")
	public static NpcStringId WHY_YOU_HAVE_NO_MUSIC_BORING_I_M_LEAVING_NOW;
	
	@ClientString(id = 1800968, message = "Not those sharp things~! Use the ones that make nice sounds!")
	public static NpcStringId NOT_THOSE_SHARP_THINGS_USE_THE_ONES_THAT_MAKE_NICE_SOUNDS;
	
	@ClientString(id = 1800969, message = "Large watermelons only open with music~ Just striking with a weapon won't work~!")
	public static NpcStringId LARGE_WATERMELONS_ONLY_OPEN_WITH_MUSIC_JUST_STRIKING_WITH_A_WEAPON_WON_T_WORK;
	
	@ClientString(id = 1800970, message = "Strike with music~! Not with something like this. You need music~!")
	public static NpcStringId STRIKE_WITH_MUSIC_NOT_WITH_SOMETHING_LIKE_THIS_YOU_NEED_MUSIC;
	
	@ClientString(id = 1800971, message = "You're pretty amazing~! But it's all for nothing~~!")
	public static NpcStringId YOU_RE_PRETTY_AMAZING_BUT_IT_S_ALL_FOR_NOTHING;
	
	@ClientString(id = 1800972, message = "Use that on monsters, OK? I want Crono~!")
	public static NpcStringId USE_THAT_ON_MONSTERS_OK_I_WANT_CRONO;
	
	@ClientString(id = 1800973, message = "Everyone~ The watermelon is breaking!!!")
	public static NpcStringId EVERYONE_THE_WATERMELON_IS_BREAKING;
	
	@ClientString(id = 1800974, message = "It's like a watermelon slice~!")
	public static NpcStringId IT_S_LIKE_A_WATERMELON_SLICE;
	
	@ClientString(id = 1800975, message = "A goblin is coming out(?)!")
	public static NpcStringId A_GOBLIN_IS_COMING_OUT;
	
	@ClientString(id = 1800976, message = "Large watermelon~! Make a wish!!")
	public static NpcStringId LARGE_WATERMELON_MAKE_A_WISH;
	
	@ClientString(id = 1800977, message = "Don't tell anyone about my death~")
	public static NpcStringId DON_T_TELL_ANYONE_ABOUT_MY_DEATH;
	
	@ClientString(id = 1800978, message = "Ugh~ The red juice is flowing out!")
	public static NpcStringId UGH_THE_RED_JUICE_IS_FLOWING_OUT;
	
	@ClientString(id = 1800979, message = "This is all~")
	public static NpcStringId THIS_IS_ALL;
	
	@ClientString(id = 1800980, message = "Kyaahh!!! I'm mad...!")
	public static NpcStringId KYAAHH_I_M_MAD;
	
	@ClientString(id = 1800981, message = "Everyone~ This watermelon broke open!! The item is falling out!")
	public static NpcStringId EVERYONE_THIS_WATERMELON_BROKE_OPEN_THE_ITEM_IS_FALLING_OUT;
	
	@ClientString(id = 1800982, message = "Oh! It burst! The contents are spilling out~")
	public static NpcStringId OH_IT_BURST_THE_CONTENTS_ARE_SPILLING_OUT;
	
	@ClientString(id = 1800983, message = "Hohoho~ Play better!")
	public static NpcStringId HOHOHO_PLAY_BETTER;
	
	@ClientString(id = 1800984, message = "Oh~!! You're very talented, huh~?")
	public static NpcStringId OH_YOU_RE_VERY_TALENTED_HUH;
	
	@ClientString(id = 1800985, message = "Play some more! More!  More!  More!")
	public static NpcStringId PLAY_SOME_MORE_MORE_MORE_MORE;
	
	@ClientString(id = 1800986, message = "I eat hits and grow!!")
	public static NpcStringId I_EAT_HITS_AND_GROW;
	
	@ClientString(id = 1800987, message = "Buck up~ There isn't much time~")
	public static NpcStringId BUCK_UP_THERE_ISN_T_MUCH_TIME;
	
	@ClientString(id = 1800988, message = "Do you think I'll burst with just that~?")
	public static NpcStringId DO_YOU_THINK_I_LL_BURST_WITH_JUST_THAT;
	
	@ClientString(id = 1800989, message = "What a nice attack~ You might be able to kill a passing fly~")
	public static NpcStringId WHAT_A_NICE_ATTACK_YOU_MIGHT_BE_ABLE_TO_KILL_A_PASSING_FLY;
	
	@ClientString(id = 1800990, message = "Right there! A little to the right~! Ah~ Refreshing.")
	public static NpcStringId RIGHT_THERE_A_LITTLE_TO_THE_RIGHT_AH_REFRESHING;
	
	@ClientString(id = 1800991, message = "You call that hitting? Bring some more talented friends!")
	public static NpcStringId YOU_CALL_THAT_HITTING_BRING_SOME_MORE_TALENTED_FRIENDS;
	
	@ClientString(id = 1800992, message = "Don't think! Just hit! We're hitting!")
	public static NpcStringId DON_T_THINK_JUST_HIT_WE_RE_HITTING;
	
	@ClientString(id = 1800993, message = "I need nectar~ Gourd nectar!")
	public static NpcStringId I_NEED_NECTAR_GOURD_NECTAR;
	
	@ClientString(id = 1800994, message = "I can only grow by drinking nectar~")
	public static NpcStringId I_CAN_ONLY_GROW_BY_DRINKING_NECTAR;
	
	@ClientString(id = 1800995, message = "Grow me quick! If you're good, it's a large watermelon. If you're bad, it a watermelon slice~!")
	public static NpcStringId GROW_ME_QUICK_IF_YOU_RE_GOOD_IT_S_A_LARGE_WATERMELON_IF_YOU_RE_BAD_IT_A_WATERMELON_SLICE;
	
	@ClientString(id = 1800996, message = "Gimme nectar~ I'm hungry~")
	public static NpcStringId GIMME_NECTAR_I_M_HUNGRY;
	
	@ClientString(id = 1800997, message = "Hurry and bring me nectar... Not a necktie~... (sorry)")
	public static NpcStringId HURRY_AND_BRING_ME_NECTAR_NOT_A_NECKTIE_SORRY;
	
	@ClientString(id = 1800998, message = "Bring me nectar. Then, I'll drink and grow!")
	public static NpcStringId BRING_ME_NECTAR_THEN_I_LL_DRINK_AND_GROW;
	
	@ClientString(id = 1800999, message = "You wanna eat a tiny watermelon like me? Try giving me some nectar. I'll get huge~!")
	public static NpcStringId YOU_WANNA_EAT_A_TINY_WATERMELON_LIKE_ME_TRY_GIVING_ME_SOME_NECTAR_I_LL_GET_HUGE;
	
	@ClientString(id = 1801000, message = "Hehehe... Grow me well and you'll get a reward. Grow me bad and who knows what'll happen~")
	public static NpcStringId HEHEHE_GROW_ME_WELL_AND_YOU_LL_GET_A_REWARD_GROW_ME_BAD_AND_WHO_KNOWS_WHAT_LL_HAPPEN;
	
	@ClientString(id = 1801001, message = "You want a large watermelon? I'd like to be a watermelon slice~")
	public static NpcStringId YOU_WANT_A_LARGE_WATERMELON_I_D_LIKE_TO_BE_A_WATERMELON_SLICE;
	
	@ClientString(id = 1801002, message = "Trust me and bring me some nectar!! I'll become a large watermelon for you~!")
	public static NpcStringId TRUST_ME_AND_BRING_ME_SOME_NECTAR_I_LL_BECOME_A_LARGE_WATERMELON_FOR_YOU;
	
	@ClientString(id = 1801003, message = "I see. Beleth has recovered all of its magic power. What remains here is just its trace.")
	public static NpcStringId I_SEE_BELETH_HAS_RECOVERED_ALL_OF_ITS_MAGIC_POWER_WHAT_REMAINS_HERE_IS_JUST_ITS_TRACE;
	
	@ClientString(id = 1801004, message = "Command Channel Leader $s1, Beleth' Ring has been acquired.")
	public static NpcStringId COMMAND_CHANNEL_LEADER_S1_BELETH_RING_HAS_BEEN_ACQUIRED;
	
	@ClientString(id = 1801005, message = "You summoned me, so you must be confident, huh? Here I come! Jack game!")
	public static NpcStringId YOU_SUMMONED_ME_SO_YOU_MUST_BE_CONFIDENT_HUH_HERE_I_COME_JACK_GAME;
	
	@ClientString(id = 1801006, message = "Hello. Let's have a good Jack game.")
	public static NpcStringId HELLO_LET_S_HAVE_A_GOOD_JACK_GAME;
	
	@ClientString(id = 1801007, message = "I'm starting! Now, show me the card you want!")
	public static NpcStringId I_M_STARTING_NOW_SHOW_ME_THE_CARD_YOU_WANT;
	
	@ClientString(id = 1801008, message = "We'll start now! Show me the card you want!")
	public static NpcStringId WE_LL_START_NOW_SHOW_ME_THE_CARD_YOU_WANT;
	
	@ClientString(id = 1801009, message = "I'm showing the Rotten Pumpkin Card!")
	public static NpcStringId I_M_SHOWING_THE_ROTTEN_PUMPKIN_CARD;
	
	@ClientString(id = 1801010, message = "I'll be showing the Rotten Pumpkin Card!")
	public static NpcStringId I_LL_BE_SHOWING_THE_ROTTEN_PUMPKIN_CARD;
	
	@ClientString(id = 1801011, message = "I'm showing the Jack Pumpkin Card!")
	public static NpcStringId I_M_SHOWING_THE_JACK_PUMPKIN_CARD;
	
	@ClientString(id = 1801012, message = "I'll be showing the Jack Pumpkin Card!")
	public static NpcStringId I_LL_BE_SHOWING_THE_JACK_PUMPKIN_CARD;
	
	@ClientString(id = 1801013, message = "That's my precious Fantastic Chocolate Banana Ultra Favor Candy!!! I'm definitely winning the next round!!")
	public static NpcStringId THAT_S_MY_PRECIOUS_FANTASTIC_CHOCOLATE_BANANA_ULTRA_FAVOR_CANDY_I_M_DEFINITELY_WINNING_THE_NEXT_ROUND;
	
	@ClientString(id = 1801014, message = "It's my precious candy, but... I'll happily give it to you~!")
	public static NpcStringId IT_S_MY_PRECIOUS_CANDY_BUT_I_LL_HAPPILY_GIVE_IT_TO_YOU;
	
	@ClientString(id = 1801015, message = "The candy fell. I'll give you my toy chest instead.")
	public static NpcStringId THE_CANDY_FELL_I_LL_GIVE_YOU_MY_TOY_CHEST_INSTEAD;
	
	@ClientString(id = 1801016, message = "Since the candy fell, I will give you my toy chest instead.")
	public static NpcStringId SINCE_THE_CANDY_FELL_I_WILL_GIVE_YOU_MY_TOY_CHEST_INSTEAD;
	
	@ClientString(id = 1801017, message = "You're not peeking at my card, are you? This time, I'll wager a special scroll.")
	public static NpcStringId YOU_RE_NOT_PEEKING_AT_MY_CARD_ARE_YOU_THIS_TIME_I_LL_WAGER_A_SPECIAL_SCROLL;
	
	@ClientString(id = 1801018, message = "We're getting serious now. If you win again, I'll give you a special scroll.")
	public static NpcStringId WE_RE_GETTING_SERIOUS_NOW_IF_YOU_WIN_AGAIN_I_LL_GIVE_YOU_A_SPECIAL_SCROLL;
	
	@ClientString(id = 1801019, message = "You could probably enter the underworld pro league!")
	public static NpcStringId YOU_COULD_PROBABLY_ENTER_THE_UNDERWORLD_PRO_LEAGUE;
	
	@ClientString(id = 1801020, message = "Even pros can't do this much. You're amazing.")
	public static NpcStringId EVEN_PROS_CAN_T_DO_THIS_MUCH_YOU_RE_AMAZING;
	
	@ClientString(id = 1801021, message = "Who's the monster here?! This time, I'll bet my precious Transformation Stick.")
	public static NpcStringId WHO_S_THE_MONSTER_HERE_THIS_TIME_I_LL_BET_MY_PRECIOUS_TRANSFORMATION_STICK;
	
	@ClientString(id = 1801022, message = "I lost again. I won't lose this time. I'm betting my Transformation Stick.")
	public static NpcStringId I_LOST_AGAIN_I_WON_T_LOSE_THIS_TIME_I_M_BETTING_MY_TRANSFORMATION_STICK;
	
	@ClientString(id = 1801023, message = "Lost again! Hmph. Next time, I'll bet an incredible gift! Wait for it if you want!")
	public static NpcStringId LOST_AGAIN_HMPH_NEXT_TIME_I_LL_BET_AN_INCREDIBLE_GIFT_WAIT_FOR_IT_IF_YOU_WANT;
	
	@ClientString(id = 1801024, message = "You're too good. Next time, I'll give you an incredible gift! Please wait for you.")
	public static NpcStringId YOU_RE_TOO_GOOD_NEXT_TIME_I_LL_GIVE_YOU_AN_INCREDIBLE_GIFT_PLEASE_WAIT_FOR_YOU;
	
	@ClientString(id = 1801025, message = "My pride can't handle you winning anymore!")
	public static NpcStringId MY_PRIDE_CAN_T_HANDLE_YOU_WINNING_ANYMORE;
	
	@ClientString(id = 1801026, message = "I would be embarrassing to lose again here...")
	public static NpcStringId I_WOULD_BE_EMBARRASSING_TO_LOSE_AGAIN_HERE;
	
	@ClientString(id = 1801027, message = "What's your name? I'm gonna remember you!")
	public static NpcStringId WHAT_S_YOUR_NAME_I_M_GONNA_REMEMBER_YOU;
	
	@ClientString(id = 1801028, message = "People from the above ground world are really good at games.")
	public static NpcStringId PEOPLE_FROM_THE_ABOVE_GROUND_WORLD_ARE_REALLY_GOOD_AT_GAMES;
	
	@ClientString(id = 1801029, message = "You've played a lot in the underworld, haven't you?!")
	public static NpcStringId YOU_VE_PLAYED_A_LOT_IN_THE_UNDERWORLD_HAVEN_T_YOU;
	
	@ClientString(id = 1801030, message = "I've never met someone so good before.")
	public static NpcStringId I_VE_NEVER_MET_SOMEONE_SO_GOOD_BEFORE;
	
	@ClientString(id = 1801031, message = "13 wins in a row. You're pretty lucky today, huh?")
	public static NpcStringId WINS_IN_A_ROW_YOU_RE_PRETTY_LUCKY_TODAY_HUH;
	
	@ClientString(id = 1801032, message = "I never thought I would see 13 wins in a row.")
	public static NpcStringId I_NEVER_THOUGHT_I_WOULD_SEE_13_WINS_IN_A_ROW;
	
	@ClientString(id = 1801033, message = "This is the best record I've seen! Next time I'll give you my treasure -- the Golden Jack O'Lantern!")
	public static NpcStringId THIS_IS_THE_BEST_RECORD_I_VE_SEEN_NEXT_TIME_I_LL_GIVE_YOU_MY_TREASURE_THE_GOLDEN_JACK_O_LANTERN;
	
	@ClientString(id = 1801034, message = "Even pros can't do 14 wins...! Next time I'll give you my treasure -- the Golden Jack O'Lantern!")
	public static NpcStringId EVEN_PROS_CAN_T_DO_14_WINS_NEXT_TIME_I_LL_GIVE_YOU_MY_TREASURE_THE_GOLDEN_JACK_O_LANTERN;
	
	@ClientString(id = 1801035, message = "I can't do this anymore! You win! I acknowledge you as the best I've ever met in all my 583 years!")
	public static NpcStringId I_CAN_T_DO_THIS_ANYMORE_YOU_WIN_I_ACKNOWLEDGE_YOU_AS_THE_BEST_I_VE_EVER_MET_IN_ALL_MY_583_YEARS;
	
	@ClientString(id = 1801036, message = "Playing any more is meaningless. You were my greatest opponent.")
	public static NpcStringId PLAYING_ANY_MORE_IS_MEANINGLESS_YOU_WERE_MY_GREATEST_OPPONENT;
	
	@ClientString(id = 1801037, message = "I won this round...! It was fun.")
	public static NpcStringId I_WON_THIS_ROUND_IT_WAS_FUN;
	
	@ClientString(id = 1801038, message = "I won this round. It was enjoyable.")
	public static NpcStringId I_WON_THIS_ROUND_IT_WAS_ENJOYABLE;
	
	@ClientString(id = 1801039, message = "Above world people are so fun...! Then, see you later!")
	public static NpcStringId ABOVE_WORLD_PEOPLE_ARE_SO_FUN_THEN_SEE_YOU_LATER;
	
	@ClientString(id = 1801040, message = "Call me again next time. I want to play again with you.")
	public static NpcStringId CALL_ME_AGAIN_NEXT_TIME_I_WANT_TO_PLAY_AGAIN_WITH_YOU;
	
	@ClientString(id = 1801041, message = "You wanna play some more? I'm out of presents, but I'll give you candy!")
	public static NpcStringId YOU_WANNA_PLAY_SOME_MORE_I_M_OUT_OF_PRESENTS_BUT_I_LL_GIVE_YOU_CANDY;
	
	@ClientString(id = 1801042, message = "Will you play some more? I don't have any more presents, but I will give you candy if you win.")
	public static NpcStringId WILL_YOU_PLAY_SOME_MORE_I_DON_T_HAVE_ANY_MORE_PRESENTS_BUT_I_WILL_GIVE_YOU_CANDY_IF_YOU_WIN;
	
	@ClientString(id = 1801043, message = "You're the best. Out of all the Jack's game players I've ever met... I give up!")
	public static NpcStringId YOU_RE_THE_BEST_OUT_OF_ALL_THE_JACK_S_GAME_PLAYERS_I_VE_EVER_MET_I_GIVE_UP;
	
	@ClientString(id = 1801044, message = "Wowww. Awesome. Really. I have never met someone as good as you before. Now... I can't play anymore.")
	public static NpcStringId WOWWW_AWESOME_REALLY_I_HAVE_NEVER_MET_SOMEONE_AS_GOOD_AS_YOU_BEFORE_NOW_I_CAN_T_PLAY_ANYMORE;
	
	@ClientString(id = 1801045, message = "$s1 has won $s2 Jack's games in a row.")
	public static NpcStringId S1_HAS_WON_S2_JACK_S_GAMES_IN_A_ROW;
	
	@ClientString(id = 1801046, message = "Congratulations! $s1 has won $s2 Jack's games in a row.")
	public static NpcStringId CONGRATULATIONS_S1_HAS_WON_S2_JACK_S_GAMES_IN_A_ROW;
	
	@ClientString(id = 1801047, message = "Congratulations on getting 1st place in Jack's game!")
	public static NpcStringId CONGRATULATIONS_ON_GETTING_1ST_PLACE_IN_JACK_S_GAME;
	
	@ClientString(id = 1801048, message = "Hello~! I'm Belldandy. Congratulations on winning 1st place in Jack's game... If you go and find my sibling Skooldie in the village, you'll get an amazing gift! Let's play Jack's game again!")
	public static NpcStringId HELLO_I_M_BELLDANDY_CONGRATULATIONS_ON_WINNING_1ST_PLACE_IN_JACK_S_GAME_IF_YOU_GO_AND_FIND_MY_SIBLING_SKOOLDIE_IN_THE_VILLAGE_YOU_LL_GET_AN_AMAZING_GIFT_LET_S_PLAY_JACK_S_GAME_AGAIN;
	
	@ClientString(id = 1801049, message = "Hmm. You're playing Jack's game for the first time, huh? You couldn't even take out your card at the right time~! My goodness...")
	public static NpcStringId HMM_YOU_RE_PLAYING_JACK_S_GAME_FOR_THE_FIRST_TIME_HUH_YOU_COULDN_T_EVEN_TAKE_OUT_YOUR_CARD_AT_THE_RIGHT_TIME_MY_GOODNESS;
	
	@ClientString(id = 1801050, message = "Oh. You're not very familiar with Jack's game, right? You didn't take out your card at the right time...")
	public static NpcStringId OH_YOU_RE_NOT_VERY_FAMILIAR_WITH_JACK_S_GAME_RIGHT_YOU_DIDN_T_TAKE_OUT_YOUR_CARD_AT_THE_RIGHT_TIME;
	
	@ClientString(id = 1801051, message = "You have to use the card skill on the mask before the gauge above my head disappears.")
	public static NpcStringId YOU_HAVE_TO_USE_THE_CARD_SKILL_ON_THE_MASK_BEFORE_THE_GAUGE_ABOVE_MY_HEAD_DISAPPEARS;
	
	@ClientString(id = 1801052, message = "You must use the card skill on the mask before the gauge above my head disappears.")
	public static NpcStringId YOU_MUST_USE_THE_CARD_SKILL_ON_THE_MASK_BEFORE_THE_GAUGE_ABOVE_MY_HEAD_DISAPPEARS;
	
	@ClientString(id = 1801053, message = "If you show the same card as me, you win. If they're different, I win. Understand? Now, let's go!")
	public static NpcStringId IF_YOU_SHOW_THE_SAME_CARD_AS_ME_YOU_WIN_IF_THEY_RE_DIFFERENT_I_WIN_UNDERSTAND_NOW_LET_S_GO;
	
	@ClientString(id = 1801054, message = "You will win if you show the same card as me. It's my victory if the cards are different. Well, let's start again~")
	public static NpcStringId YOU_WILL_WIN_IF_YOU_SHOW_THE_SAME_CARD_AS_ME_IT_S_MY_VICTORY_IF_THE_CARDS_ARE_DIFFERENT_WELL_LET_S_START_AGAIN;
	
	@ClientString(id = 1801055, message = "Ack! You didn't show a card? You have to use the card skill before the gauge disappears. Hmph. Then, I'm going.")
	public static NpcStringId ACK_YOU_DIDN_T_SHOW_A_CARD_YOU_HAVE_TO_USE_THE_CARD_SKILL_BEFORE_THE_GAUGE_DISAPPEARS_HMPH_THEN_I_M_GOING;
	
	@ClientString(id = 1801056, message = "Ahh. You didn't show a card. You must use the card skill at the right time. It's unfortunate. Then, I will go now~")
	public static NpcStringId AHH_YOU_DIDN_T_SHOW_A_CARD_YOU_MUST_USE_THE_CARD_SKILL_AT_THE_RIGHT_TIME_IT_S_UNFORTUNATE_THEN_I_WILL_GO_NOW;
	
	@ClientString(id = 1801057, message = "Let's learn about the Jack's game together~! You can play with me 3 times.")
	public static NpcStringId LET_S_LEARN_ABOUT_THE_JACK_S_GAME_TOGETHER_YOU_CAN_PLAY_WITH_ME_3_TIMES;
	
	@ClientString(id = 1801058, message = "Let's start! Show the card you want! The card skill is attached to the mask.")
	public static NpcStringId LET_S_START_SHOW_THE_CARD_YOU_WANT_THE_CARD_SKILL_IS_ATTACHED_TO_THE_MASK;
	
	@ClientString(id = 1801059, message = "You showed the same card as me, so you win.")
	public static NpcStringId YOU_SHOWED_THE_SAME_CARD_AS_ME_SO_YOU_WIN;
	
	@ClientString(id = 1801060, message = "You showed a different card from me, so you lose.")
	public static NpcStringId YOU_SHOWED_A_DIFFERENT_CARD_FROM_ME_SO_YOU_LOSE;
	
	@ClientString(id = 1801061, message = "That was practice, so there's no candy even if you win~")
	public static NpcStringId THAT_WAS_PRACTICE_SO_THERE_S_NO_CANDY_EVEN_IF_YOU_WIN;
	
	@ClientString(id = 1801062, message = "It's unfortunate. Let's practice one more time.")
	public static NpcStringId IT_S_UNFORTUNATE_LET_S_PRACTICE_ONE_MORE_TIME;
	
	@ClientString(id = 1801063, message = "You gotta show the card at the right time. Use the card skill you want before the gauge above my head disappears!")
	public static NpcStringId YOU_GOTTA_SHOW_THE_CARD_AT_THE_RIGHT_TIME_USE_THE_CARD_SKILL_YOU_WANT_BEFORE_THE_GAUGE_ABOVE_MY_HEAD_DISAPPEARS;
	
	@ClientString(id = 1801064, message = "The card skills are attached to the Jack O'Lantern mask, right? That's what you use.")
	public static NpcStringId THE_CARD_SKILLS_ARE_ATTACHED_TO_THE_JACK_O_LANTERN_MASK_RIGHT_THAT_S_WHAT_YOU_USE;
	
	@ClientString(id = 1801065, message = "You win if you show the same card as me, and I win if the cards are different. OK, let's go~")
	public static NpcStringId YOU_WIN_IF_YOU_SHOW_THE_SAME_CARD_AS_ME_AND_I_WIN_IF_THE_CARDS_ARE_DIFFERENT_OK_LET_S_GO;
	
	@ClientString(id = 1801066, message = "You didn't show a card again? We'll try again later. I'm gonna go now~")
	public static NpcStringId YOU_DIDN_T_SHOW_A_CARD_AGAIN_WE_LL_TRY_AGAIN_LATER_I_M_GONNA_GO_NOW;
	
	@ClientString(id = 1801067, message = "Now, do you understand a little about Jack's game? The real game's with Uldie and Belldandy. Well, see you later!")
	public static NpcStringId NOW_DO_YOU_UNDERSTAND_A_LITTLE_ABOUT_JACK_S_GAME_THE_REAL_GAME_S_WITH_ULDIE_AND_BELLDANDY_WELL_SEE_YOU_LATER;
	
	@ClientString(id = 1801068, message = "Hahahaha!")
	public static NpcStringId HAHAHAHA;
	
	@ClientString(id = 1801069, message = "Where are you looking?")
	public static NpcStringId WHERE_ARE_YOU_LOOKING;
	
	@ClientString(id = 1801070, message = "I'm right here.")
	public static NpcStringId I_M_RIGHT_HERE;
	
	@ClientString(id = 1801071, message = "Annoying concentration attacks are disrupting Valakas' concentration!nIf it continues, you may get a great opportunity!")
	public static NpcStringId ANNOYING_CONCENTRATION_ATTACKS_ARE_DISRUPTING_VALAKAS_CONCENTRATION_NIF_IT_CONTINUES_YOU_MAY_GET_A_GREAT_OPPORTUNITY;
	
	@ClientString(id = 1801072, message = "Some warrior's blow has left a huge gash between the great scales of Valakas!nValakas' P. Def. is greatly decreased!")
	public static NpcStringId SOME_WARRIOR_S_BLOW_HAS_LEFT_A_HUGE_GASH_BETWEEN_THE_GREAT_SCALES_OF_VALAKAS_NVALAKAS_P_DEF_IS_GREATLY_DECREASED;
	
	@ClientString(id = 1801073, message = "Annoying concentration attacks overwhelmed Valakas, making it forget its rage and become distracted!")
	public static NpcStringId ANNOYING_CONCENTRATION_ATTACKS_OVERWHELMED_VALAKAS_MAKING_IT_FORGET_ITS_RAGE_AND_BECOME_DISTRACTED;
	
	@ClientString(id = 1801074, message = "Long-range concentration attacks have enraged Valakas!nIf you continue, it may become a dangerous situation!")
	public static NpcStringId LONG_RANGE_CONCENTRATION_ATTACKS_HAVE_ENRAGED_VALAKAS_NIF_YOU_CONTINUE_IT_MAY_BECOME_A_DANGEROUS_SITUATION;
	
	@ClientString(id = 1801075, message = "Because the cowardly counterattacks continued, Valakas' fury has reached its maximum!nValakas' P. Atk. is greatly increased!")
	public static NpcStringId BECAUSE_THE_COWARDLY_COUNTERATTACKS_CONTINUED_VALAKAS_FURY_HAS_REACHED_ITS_MAXIMUM_NVALAKAS_P_ATK_IS_GREATLY_INCREASED;
	
	@ClientString(id = 1801076, message = "Valakas has been enraged by the long-range concentration attacks and is coming toward its target with even greater zeal!")
	public static NpcStringId VALAKAS_HAS_BEEN_ENRAGED_BY_THE_LONG_RANGE_CONCENTRATION_ATTACKS_AND_IS_COMING_TOWARD_ITS_TARGET_WITH_EVEN_GREATER_ZEAL;
	
	@ClientString(id = 1801077, message = "Listen, oh Tantas! I have returned! The Prophet Yugoros of the Black Abyss is with me, so do not be afraid!")
	public static NpcStringId LISTEN_OH_TANTAS_I_HAVE_RETURNED_THE_PROPHET_YUGOROS_OF_THE_BLACK_ABYSS_IS_WITH_ME_SO_DO_NOT_BE_AFRAID;
	
	@ClientString(id = 1801078, message = "Welcome, $s1! Let us see if you have brought a worthy offering for the Black Abyss!")
	public static NpcStringId WELCOME_S1_LET_US_SEE_IF_YOU_HAVE_BROUGHT_A_WORTHY_OFFERING_FOR_THE_BLACK_ABYSS;
	
	@ClientString(id = 1801079, message = "What a formidable foe! But I have the Abyss Weed given to me by the Black Abyss! Let me see...")
	public static NpcStringId WHAT_A_FORMIDABLE_FOE_BUT_I_HAVE_THE_ABYSS_WEED_GIVEN_TO_ME_BY_THE_BLACK_ABYSS_LET_ME_SEE;
	
	@ClientString(id = 1801080, message = "Muhahaha! Ah, this burning sensation in my mouth! The power of the Black Abyss is reviving me!")
	public static NpcStringId MUHAHAHA_AH_THIS_BURNING_SENSATION_IN_MY_MOUTH_THE_POWER_OF_THE_BLACK_ABYSS_IS_REVIVING_ME;
	
	@ClientString(id = 1801081, message = "No! How dare you stop me from using the Abyss Weed... Do you know what you have done?!")
	public static NpcStringId NO_HOW_DARE_YOU_STOP_ME_FROM_USING_THE_ABYSS_WEED_DO_YOU_KNOW_WHAT_YOU_HAVE_DONE;
	
	@ClientString(id = 1801082, message = "A limp creature like this is unworthy to be an offering... You have no right to sacrifice to the Black Abyss!")
	public static NpcStringId A_LIMP_CREATURE_LIKE_THIS_IS_UNWORTHY_TO_BE_AN_OFFERING_YOU_HAVE_NO_RIGHT_TO_SACRIFICE_TO_THE_BLACK_ABYSS;
	
	@ClientString(id = 1801083, message = "Listen, oh Tantas! The Black Abyss is famished! Find some fresh offerings!")
	public static NpcStringId LISTEN_OH_TANTAS_THE_BLACK_ABYSS_IS_FAMISHED_FIND_SOME_FRESH_OFFERINGS;
	
	@ClientString(id = 1801084, message = "Ah... How could I lose... Oh, Black Abyss, receive me...")
	public static NpcStringId AH_HOW_COULD_I_LOSE_OH_BLACK_ABYSS_RECEIVE_ME;
	
	@ClientString(id = 1801085, message = "Alarm system destroyed. Intruder excluded.")
	public static NpcStringId ALARM_SYSTEM_DESTROYED_INTRUDER_EXCLUDED;
	
	@ClientString(id = 1801086, message = "Begin stage 1")
	public static NpcStringId BEGIN_STAGE_12;
	
	@ClientString(id = 1801087, message = "Begin stage 2")
	public static NpcStringId BEGIN_STAGE_22;
	
	@ClientString(id = 1801088, message = "Begin stage 3")
	public static NpcStringId BEGIN_STAGE_32;
	
	@ClientString(id = 1801089, message = "Begin stage 4")
	public static NpcStringId BEGIN_STAGE_4;
	
	@ClientString(id = 1801090, message = "Time remaining until next battle")
	public static NpcStringId TIME_REMAINING_UNTIL_NEXT_BATTLE;
	
	@ClientString(id = 1801091, message = "The beast ate the feed, but it isn't showing a response, perhaps because it's already full.")
	public static NpcStringId THE_BEAST_ATE_THE_FEED_BUT_IT_ISN_T_SHOWING_A_RESPONSE_PERHAPS_BECAUSE_IT_S_ALREADY_FULL;
	
	@ClientString(id = 1801092, message = "The beast spit out the feed instead of eating it.")
	public static NpcStringId THE_BEAST_SPIT_OUT_THE_FEED_INSTEAD_OF_EATING_IT;
	
	@ClientString(id = 1801093, message = "The beast spit out the feed and is instead attacking you!")
	public static NpcStringId THE_BEAST_SPIT_OUT_THE_FEED_AND_IS_INSTEAD_ATTACKING_YOU;
	
	@ClientString(id = 1801094, message = "$s1 is leaving you because you don't have enough Golden Spices.")
	public static NpcStringId S1_IS_LEAVING_YOU_BECAUSE_YOU_DON_T_HAVE_ENOUGH_GOLDEN_SPICES;
	
	@ClientString(id = 1801095, message = "$s1 is leaving you because you don't have enough Crystal Spices.")
	public static NpcStringId S1_IS_LEAVING_YOU_BECAUSE_YOU_DON_T_HAVE_ENOUGH_CRYSTAL_SPICES;
	
	@ClientString(id = 1801096, message = "$s1. May the protection of the gods be upon you!")
	public static NpcStringId S1_MAY_THE_PROTECTION_OF_THE_GODS_BE_UPON_YOU;
	
	@ClientString(id = 1801097, message = "Freya has started to move.")
	public static NpcStringId FREYA_HAS_STARTED_TO_MOVE;
	
	@ClientString(id = 1801098, message = "How could I... fall... in a place like this...")
	public static NpcStringId HOW_COULD_I_FALL_IN_A_PLACE_LIKE_THIS;
	
	@ClientString(id = 1801099, message = "I can finally take a breather. By the way, who are you? Hmm... I think I know who sent you.")
	public static NpcStringId I_CAN_FINALLY_TAKE_A_BREATHER_BY_THE_WAY_WHO_ARE_YOU_HMM_I_THINK_I_KNOW_WHO_SENT_YOU;
	
	@ClientString(id = 1801100, message = "$s1 of Balance")
	public static NpcStringId S1_OF_BALANCE;
	
	@ClientString(id = 1801101, message = "Swift $s1")
	public static NpcStringId SWIFT_S1;
	
	@ClientString(id = 1801102, message = "$s1 of Blessing")
	public static NpcStringId S1_OF_BLESSING;
	
	@ClientString(id = 1801103, message = "Sharp $s1")
	public static NpcStringId SHARP_S1;
	
	@ClientString(id = 1801104, message = "Useful $s1")
	public static NpcStringId USEFUL_S1;
	
	@ClientString(id = 1801105, message = "Reckless $s1")
	public static NpcStringId RECKLESS_S1;
	
	@ClientString(id = 1801106, message = "Alpen Kookaburra")
	public static NpcStringId ALPEN_KOOKABURRA;
	
	@ClientString(id = 1801107, message = "Alpen Cougar")
	public static NpcStringId ALPEN_COUGAR;
	
	@ClientString(id = 1801108, message = "Alpen Buffalo")
	public static NpcStringId ALPEN_BUFFALO;
	
	@ClientString(id = 1801109, message = "Alpen Grendel")
	public static NpcStringId ALPEN_GRENDEL;
	
	@ClientString(id = 1801110, message = "Battle end limit time")
	public static NpcStringId BATTLE_END_LIMIT_TIME;
	
	@ClientString(id = 1801111, message = "Strong magic power can be felt from somewhere!!")
	public static NpcStringId STRONG_MAGIC_POWER_CAN_BE_FELT_FROM_SOMEWHERE;
	
	@ClientString(id = 1801112, message = "How dare you attack my recruits!!")
	public static NpcStringId HOW_DARE_YOU_ATTACK_MY_RECRUITS;
	
	@ClientString(id = 1801113, message = "Who is disrupting the order?!")
	public static NpcStringId WHO_IS_DISRUPTING_THE_ORDER;
	
	@ClientString(id = 1801114, message = "The drillmaster is dead!")
	public static NpcStringId THE_DRILLMASTER_IS_DEAD;
	
	@ClientString(id = 1801115, message = "Line up the ranks!!")
	public static NpcStringId LINE_UP_THE_RANKS;
	
	@ClientString(id = 1801116, message = "I brought the food.")
	public static NpcStringId I_BROUGHT_THE_FOOD;
	
	@ClientString(id = 1801117, message = "Come and eat.")
	public static NpcStringId COME_AND_EAT;
	
	@ClientString(id = 1801118, message = "Looks delicious.")
	public static NpcStringId LOOKS_DELICIOUS;
	
	@ClientString(id = 1801119, message = "Let's go eat.")
	public static NpcStringId LET_S_GO_EAT;
	
	@ClientString(id = 1801120, message = "Archer. Give your breath for the intruder!")
	public static NpcStringId ARCHER_GIVE_YOUR_BREATH_FOR_THE_INTRUDER;
	
	@ClientString(id = 1801121, message = "My knights. Show your loyalty!!")
	public static NpcStringId MY_KNIGHTS_SHOW_YOUR_LOYALTY;
	
	@ClientString(id = 1801122, message = "I can take it no longer!!!")
	public static NpcStringId I_CAN_TAKE_IT_NO_LONGER;
	
	@ClientString(id = 1801123, message = "Archer. Heed my call!")
	public static NpcStringId ARCHER_HEED_MY_CALL;
	
	@ClientString(id = 1801124, message = "The space feels like its gradually starting to shake.")
	public static NpcStringId THE_SPACE_FEELS_LIKE_ITS_GRADUALLY_STARTING_TO_SHAKE;
	
	@ClientString(id = 1801125, message = "I can no longer stand by.")
	public static NpcStringId I_CAN_NO_LONGER_STAND_BY;
	
	@ClientString(id = 1801126, message = "Taklacan is gathering its strength and launching an attack.")
	public static NpcStringId TAKLACAN_IS_GATHERING_ITS_STRENGTH_AND_LAUNCHING_AN_ATTACK;
	
	@ClientString(id = 1801127, message = "Taklacan received Kokracon and became weaker.")
	public static NpcStringId TAKLACAN_RECEIVED_KOKRACON_AND_BECAME_WEAKER;
	
	@ClientString(id = 1801128, message = "Kokracon's power can be felt nearby.")
	public static NpcStringId KOKRACON_S_POWER_CAN_BE_FELT_NEARBY;
	
	@ClientString(id = 1801129, message = "Taklacan is preparing to hide itself.")
	public static NpcStringId TAKLACAN_IS_PREPARING_TO_HIDE_ITSELF;
	
	@ClientString(id = 1801130, message = "Taklacan disappears into the space of futility with a roar.")
	public static NpcStringId TAKLACAN_DISAPPEARS_INTO_THE_SPACE_OF_FUTILITY_WITH_A_ROAR;
	
	@ClientString(id = 1801131, message = "Torumba's body is starting to move.")
	public static NpcStringId TORUMBA_S_BODY_IS_STARTING_TO_MOVE;
	
	@ClientString(id = 1801132, message = "A response can be felt from Torumba's tough skin.")
	public static NpcStringId A_RESPONSE_CAN_BE_FELT_FROM_TORUMBA_S_TOUGH_SKIN;
	
	@ClientString(id = 1801133, message = "A mark remains on Torumba's tough skin.")
	public static NpcStringId A_MARK_REMAINS_ON_TORUMBA_S_TOUGH_SKIN;
	
	@ClientString(id = 1801134, message = "The light is beginning to weaken in Torumba's eyes.")
	public static NpcStringId THE_LIGHT_IS_BEGINNING_TO_WEAKEN_IN_TORUMBA_S_EYES;
	
	@ClientString(id = 1801135, message = "Torumba's left leg was damaged.")
	public static NpcStringId TORUMBA_S_LEFT_LEG_WAS_DAMAGED;
	
	@ClientString(id = 1801136, message = "Torumba's right leg was damaged.")
	public static NpcStringId TORUMBA_S_RIGHT_LEG_WAS_DAMAGED;
	
	@ClientString(id = 1801137, message = "Torumba's left arm was damaged.")
	public static NpcStringId TORUMBA_S_LEFT_ARM_WAS_DAMAGED;
	
	@ClientString(id = 1801138, message = "Torumba's right arm was damaged.")
	public static NpcStringId TORUMBA_S_RIGHT_ARM_WAS_DAMAGED;
	
	@ClientString(id = 1801139, message = "A deep wound appeared on Torumba's tough skin.")
	public static NpcStringId A_DEEP_WOUND_APPEARED_ON_TORUMBA_S_TOUGH_SKIN;
	
	@ClientString(id = 1801140, message = "The light is slowly fading from Torumba's eyes.")
	public static NpcStringId THE_LIGHT_IS_SLOWLY_FADING_FROM_TORUMBA_S_EYES;
	
	@ClientString(id = 1801141, message = "Torumba is preparing to hide its body.")
	public static NpcStringId TORUMBA_IS_PREPARING_TO_HIDE_ITS_BODY;
	
	@ClientString(id = 1801142, message = "Torumba disappeared into his space.")
	public static NpcStringId TORUMBA_DISAPPEARED_INTO_HIS_SPACE;
	
	@ClientString(id = 1801143, message = "Torumba is preparing to hide itself in the twisted space.")
	public static NpcStringId TORUMBA_IS_PREPARING_TO_HIDE_ITSELF_IN_THE_TWISTED_SPACE;
	
	@ClientString(id = 1801144, message = "Dopagen has started to move.")
	public static NpcStringId DOPAGEN_HAS_STARTED_TO_MOVE;
	
	@ClientString(id = 1801145, message = "Leptilikon's energy feels like it's being condensed.")
	public static NpcStringId LEPTILIKON_S_ENERGY_FEELS_LIKE_IT_S_BEING_CONDENSED;
	
	@ClientString(id = 1801146, message = "Dopagen is preparing to hide itself with a strange scent.")
	public static NpcStringId DOPAGEN_IS_PREPARING_TO_HIDE_ITSELF_WITH_A_STRANGE_SCENT;
	
	@ClientString(id = 1801147, message = "Dopagen is preparing to hide itself.")
	public static NpcStringId DOPAGEN_IS_PREPARING_TO_HIDE_ITSELF;
	
	@ClientString(id = 1801148, message = "Dopagen is disappearing in between the twisted space.")
	public static NpcStringId DOPAGEN_IS_DISAPPEARING_IN_BETWEEN_THE_TWISTED_SPACE;
	
	@ClientString(id = 1801149, message = "Maguen appearance!!!")
	public static NpcStringId MAGUEN_APPEARANCE;
	
	@ClientString(id = 1801150, message = "Enough Maguen Plasma - Bistakon have gathered.")
	public static NpcStringId ENOUGH_MAGUEN_PLASMA_BISTAKON_HAVE_GATHERED;
	
	@ClientString(id = 1801151, message = "Enough Maguen Plasma - Kokracon have gathered.")
	public static NpcStringId ENOUGH_MAGUEN_PLASMA_KOKRACON_HAVE_GATHERED;
	
	@ClientString(id = 1801152, message = "Enough Maguen Plasma - Leptilikon have gathered.")
	public static NpcStringId ENOUGH_MAGUEN_PLASMA_LEPTILIKON_HAVE_GATHERED;
	
	@ClientString(id = 1801153, message = "The plasmas have filled the aeroscope and are harmonized.")
	public static NpcStringId THE_PLASMAS_HAVE_FILLED_THE_AEROSCOPE_AND_ARE_HARMONIZED;
	
	@ClientString(id = 1801154, message = "The plasmas have filled the aeroscope but they are ramming into each other, exploding, and dying.")
	public static NpcStringId THE_PLASMAS_HAVE_FILLED_THE_AEROSCOPE_BUT_THEY_ARE_RAMMING_INTO_EACH_OTHER_EXPLODING_AND_DYING;
	
	@ClientString(id = 1801155, message = "Amazing. $s1 took 100 of these soul stone fragments. What a complete swindler.")
	public static NpcStringId AMAZING_S1_TOOK_100_OF_THESE_SOUL_STONE_FRAGMENTS_WHAT_A_COMPLETE_SWINDLER;
	
	@ClientString(id = 1801156, message = "Hmm? Hey, did you give $s1 something? But it was just 1. Haha.")
	public static NpcStringId HMM_HEY_DID_YOU_GIVE_S1_SOMETHING_BUT_IT_WAS_JUST_1_HAHA;
	
	@ClientString(id = 1801157, message = "$s1 pulled one with $s2 digits. Lucky~ Not bad~")
	public static NpcStringId S1_PULLED_ONE_WITH_S2_DIGITS_LUCKY_NOT_BAD;
	
	@ClientString(id = 1801158, message = "It's better than losing it all, right? Or does this feel worse?")
	public static NpcStringId IT_S_BETTER_THAN_LOSING_IT_ALL_RIGHT_OR_DOES_THIS_FEEL_WORSE;
	
	@ClientString(id = 1801159, message = "Ahem~!  $s1 has no luck at all. Try praying.")
	public static NpcStringId AHEM_S1_HAS_NO_LUCK_AT_ALL_TRY_PRAYING;
	
	@ClientString(id = 1801160, message = "Ah... It's over. What kind of guy is that? Damn... Fine, you $s1, take it and get outta here.")
	public static NpcStringId AH_IT_S_OVER_WHAT_KIND_OF_GUY_IS_THAT_DAMN_FINE_YOU_S1_TAKE_IT_AND_GET_OUTTA_HERE;
	
	@ClientString(id = 1801161, message = "A big piece is made up of little pieces. So here's a little piece~")
	public static NpcStringId A_BIG_PIECE_IS_MADE_UP_OF_LITTLE_PIECES_SO_HERE_S_A_LITTLE_PIECE;
	
	@ClientString(id = 1801162, message = "You don't feel bad, right? Are you sad? But don't cry~")
	public static NpcStringId YOU_DON_T_FEEL_BAD_RIGHT_ARE_YOU_SAD_BUT_DON_T_CRY;
	
	@ClientString(id = 1801163, message = "OK~ Who's next? It all depends on your fate and luck, right? At least come and take a look.")
	public static NpcStringId OK_WHO_S_NEXT_IT_ALL_DEPENDS_ON_YOUR_FATE_AND_LUCK_RIGHT_AT_LEAST_COME_AND_TAKE_A_LOOK;
	
	@ClientString(id = 1801164, message = "No one else? Don't worry~ I don't bite. Haha~!")
	public static NpcStringId NO_ONE_ELSE_DON_T_WORRY_I_DON_T_BITE_HAHA;
	
	@ClientString(id = 1801165, message = "There was someone who won 10,000 from me. A warrior shouldn't just be good at fighting, right? You've gotta be good in everything.")
	public static NpcStringId THERE_WAS_SOMEONE_WHO_WON_10_000_FROM_ME_A_WARRIOR_SHOULDN_T_JUST_BE_GOOD_AT_FIGHTING_RIGHT_YOU_VE_GOTTA_BE_GOOD_IN_EVERYTHING;
	
	@ClientString(id = 1801166, message = "OK~ Master of luck? That's you? Haha~! Well, anyone can come after all.")
	public static NpcStringId OK_MASTER_OF_LUCK_THAT_S_YOU_HAHA_WELL_ANYONE_CAN_COME_AFTER_ALL;
	
	@ClientString(id = 1801167, message = "Shedding blood is a given on the battleground. At least it's safe here.")
	public static NpcStringId SHEDDING_BLOOD_IS_A_GIVEN_ON_THE_BATTLEGROUND_AT_LEAST_IT_S_SAFE_HERE;
	
	@ClientString(id = 1801168, message = "Gasp!")
	public static NpcStringId GASP2;
	
	@ClientString(id = 1801169, message = "Is it still long off?")
	public static NpcStringId IS_IT_STILL_LONG_OFF;
	
	@ClientString(id = 1801170, message = "Is Ermian well? Even I can't believe that I survived in a place like this...")
	public static NpcStringId IS_ERMIAN_WELL_EVEN_I_CAN_T_BELIEVE_THAT_I_SURVIVED_IN_A_PLACE_LIKE_THIS;
	
	@ClientString(id = 1801171, message = "I don't know how long it's been since I parted company with you... Time doesn't seem to move... It just feels too long...")
	public static NpcStringId I_DON_T_KNOW_HOW_LONG_IT_S_BEEN_SINCE_I_PARTED_COMPANY_WITH_YOU_TIME_DOESN_T_SEEM_TO_MOVE_IT_JUST_FEELS_TOO_LONG;
	
	@ClientString(id = 1801172, message = "Sorry to say this, but... The place you struck me before now hurts greatly...")
	public static NpcStringId SORRY_TO_SAY_THIS_BUT_THE_PLACE_YOU_STRUCK_ME_BEFORE_NOW_HURTS_GREATLY;
	
	@ClientString(id = 1801173, message = "Ugh... I'm sorry... It looks like this is it for me... I wanted to live and see my family...")
	public static NpcStringId UGH_I_M_SORRY_IT_LOOKS_LIKE_THIS_IS_IT_FOR_ME_I_WANTED_TO_LIVE_AND_SEE_MY_FAMILY;
	
	@ClientString(id = 1801174, message = "Where are you? I can't see anything.")
	public static NpcStringId WHERE_ARE_YOU_I_CAN_T_SEE_ANYTHING;
	
	@ClientString(id = 1801175, message = "Where are you, really? I can't follow you like this.")
	public static NpcStringId WHERE_ARE_YOU_REALLY_I_CAN_T_FOLLOW_YOU_LIKE_THIS;
	
	@ClientString(id = 1801176, message = "I'm sorry... This is it for me.")
	public static NpcStringId I_M_SORRY_THIS_IS_IT_FOR_ME;
	
	@ClientString(id = 1801177, message = "Sob~ To see Ermian again... Can I go to my family now?")
	public static NpcStringId SOB_TO_SEE_ERMIAN_AGAIN_CAN_I_GO_TO_MY_FAMILY_NOW;
	
	@ClientString(id = 1801178, message = "What's this feeling? Oh oh~! Feels like my energy is back~!")
	public static NpcStringId WHAT_S_THIS_FEELING_OH_OH_FEELS_LIKE_MY_ENERGY_IS_BACK;
	
	@ClientString(id = 1801179, message = "I feel so light! This is it! I know this feeling!")
	public static NpcStringId I_FEEL_SO_LIGHT_THIS_IS_IT_I_KNOW_THIS_FEELING2;
	
	@ClientString(id = 1801180, message = "Wow~ My fatigue is completely gone!")
	public static NpcStringId WOW_MY_FATIGUE_IS_COMPLETELY_GONE2;
	
	@ClientString(id = 1801181, message = "Hey~ that ominous feeling has disappeared!")
	public static NpcStringId HEY_THAT_OMINOUS_FEELING_HAS_DISAPPEARED2;
	
	@ClientString(id = 1801182, message = "My body feels as light as a feather~")
	public static NpcStringId MY_BODY_FEELS_AS_LIGHT_AS_A_FEATHER2;
	
	@ClientString(id = 1801183, message = "My energy was recovered so quickly. Thanks, $s1 -")
	public static NpcStringId MY_ENERGY_WAS_RECOVERED_SO_QUICKLY_THANKS_S1;
	
	@ClientString(id = 1801184, message = "How refreshing! You wouldn't happen to be a master masseuse, $s1, would you?")
	public static NpcStringId HOW_REFRESHING_YOU_WOULDN_T_HAPPEN_TO_BE_A_MASTER_MASSEUSE_S1_WOULD_YOU2;
	
	@ClientString(id = 1801185, message = "Incredible. From now on, I'll compare all massages to this one with $s1!")
	public static NpcStringId INCREDIBLE_FROM_NOW_ON_I_LL_COMPARE_ALL_MASSAGES_TO_THIS_ONE_WITH_S12;
	
	@ClientString(id = 1801186, message = "That's what I call a Cure-All!")
	public static NpcStringId THAT_S_WHAT_I_CALL_A_CURE_ALL;
	
	@ClientString(id = 1801187, message = "I'm starving! I'll go find some meat.")
	public static NpcStringId I_M_STARVING_I_LL_GO_FIND_SOME_MEAT;
	
	@ClientString(id = 1801188, message = "Thank you, thank you!")
	public static NpcStringId THANK_YOU_THANK_YOU2;
	
	@ClientString(id = 1801189, message = "Magic power so strong that it could make you lose your mind can be felt from somewhere!!")
	public static NpcStringId MAGIC_POWER_SO_STRONG_THAT_IT_COULD_MAKE_YOU_LOSE_YOUR_MIND_CAN_BE_FELT_FROM_SOMEWHERE;
	
	@ClientString(id = 1801190, message = "Even though you bring something called a gift among your humans, it would just be problematic for me...")
	public static NpcStringId EVEN_THOUGH_YOU_BRING_SOMETHING_CALLED_A_GIFT_AMONG_YOUR_HUMANS_IT_WOULD_JUST_BE_PROBLEMATIC_FOR_ME;
	
	@ClientString(id = 1801191, message = "I just don't know what expression I should have it appeared on me. Are human's emotions like this feeling?")
	public static NpcStringId I_JUST_DON_T_KNOW_WHAT_EXPRESSION_I_SHOULD_HAVE_IT_APPEARED_ON_ME_ARE_HUMAN_S_EMOTIONS_LIKE_THIS_FEELING;
	
	@ClientString(id = 1801192, message = "The feeling of thanks is just too much distant memory for me...")
	public static NpcStringId THE_FEELING_OF_THANKS_IS_JUST_TOO_MUCH_DISTANT_MEMORY_FOR_ME;
	
	@ClientString(id = 1801193, message = "But I kind of miss it... Like I had felt this feeling before...")
	public static NpcStringId BUT_I_KIND_OF_MISS_IT_LIKE_I_HAD_FELT_THIS_FEELING_BEFORE;
	
	@ClientString(id = 1801194, message = "I am Ice Queen Freya... This feeling and emotion are nothing but a part of Maliss'a memories.")
	public static NpcStringId I_AM_ICE_QUEEN_FREYA_THIS_FEELING_AND_EMOTION_ARE_NOTHING_BUT_A_PART_OF_MALISS_A_MEMORIES;
	
	@ClientString(id = 1801195, message = "Dear $s1... Think of this as my appreciation for the gift. Take this with you. There's nothing strange about it. It's just a bit of my capriciousness...")
	public static NpcStringId DEAR_S1_THINK_OF_THIS_AS_MY_APPRECIATION_FOR_THE_GIFT_TAKE_THIS_WITH_YOU_THERE_S_NOTHING_STRANGE_ABOUT_IT_IT_S_JUST_A_BIT_OF_MY_CAPRICIOUSNESS;
	
	@ClientString(id = 1801196, message = "The kindness of somebody is not a bad feeling... Dear $s1, I will take this gift out of respect your kindness.")
	public static NpcStringId THE_KINDNESS_OF_SOMEBODY_IS_NOT_A_BAD_FEELING_DEAR_S1_I_WILL_TAKE_THIS_GIFT_OUT_OF_RESPECT_YOUR_KINDNESS;
	
	@ClientString(id = 1801197, message = "$s1 $s2 $s3 minute(s) remaining!")
	public static NpcStringId S1_S2_S3_MINUTE_S_REMAINING;
	
	@ClientString(id = 1801198, message = "Successful Destruction of Stronghold $s1")
	public static NpcStringId SUCCESSFUL_DESTRUCTION_OF_STRONGHOLD_S1;
	
	@ClientString(id = 1801199, message = "Successful Destruction of Stronghold: Entry Accessed")
	public static NpcStringId SUCCESSFUL_DESTRUCTION_OF_STRONGHOLD_ENTRY_ACCESSED;
	
	@ClientString(id = 1801200, message = "The portal to the next room is now open.")
	public static NpcStringId THE_PORTAL_TO_THE_NEXT_ROOM_IS_NOW_OPEN;
	
	@ClientString(id = 1801201, message = "Golem entered the required zone.")
	public static NpcStringId GOLEM_ENTERED_THE_REQUIRED_ZONE;
	
	@ClientString(id = 1801202, message = "Golem location successful: Entry Accessed")
	public static NpcStringId GOLEM_LOCATION_SUCCESSFUL_ENTRY_ACCESSED;
	
	@ClientString(id = 1801203, message = "Bard's Joy")
	public static NpcStringId BARD_S_JOY;
	
	@ClientString(id = 1801204, message = "The Bard will enter soon. Please prepare the performance.")
	public static NpcStringId THE_BARD_WILL_ENTER_SOON_PLEASE_PREPARE_THE_PERFORMANCE;
	
	@ClientString(id = 1801205, message = "Perform my actions!")
	public static NpcStringId PERFORM_MY_ACTIONS;
	
	@ClientString(id = 1801206, message = "Perform the opposite of my actions!")
	public static NpcStringId PERFORM_THE_OPPOSITE_OF_MY_ACTIONS;
	
	@ClientString(id = 1801207, message = "Party members that I name must follow my actions!")
	public static NpcStringId PARTY_MEMBERS_THAT_I_NAME_MUST_FOLLOW_MY_ACTIONS;
	
	@ClientString(id = 1801208, message = "Please perform your actions to the Small Bard I name!")
	public static NpcStringId PLEASE_PERFORM_YOUR_ACTIONS_TO_THE_SMALL_BARD_I_NAME;
	
	@ClientString(id = 1801209, message = "Show (Dance) and (Applaud) to the Glad Small Bard!")
	public static NpcStringId SHOW_DANCE_AND_APPLAUD_TO_THE_GLAD_SMALL_BARD;
	
	@ClientString(id = 1801210, message = "Show (Sorrow) and (No) to the Sad Small Bard!")
	public static NpcStringId SHOW_SORROW_AND_NO_TO_THE_SAD_SMALL_BARD;
	
	@ClientString(id = 1801211, message = "Show (Laugh) and (Victory) action to the Mad Small Bard!")
	public static NpcStringId SHOW_LAUGH_AND_VICTORY_ACTION_TO_THE_MAD_SMALL_BARD;
	
	@ClientString(id = 1801212, message = "The Bard shows (Greeting) action.")
	public static NpcStringId THE_BARD_SHOWS_GREETING_ACTION;
	
	@ClientString(id = 1801213, message = "The Bard shows (Victory) action.")
	public static NpcStringId THE_BARD_SHOWS_VICTORY_ACTION;
	
	@ClientString(id = 1801214, message = "The Bard shows (Dance) action.")
	public static NpcStringId THE_BARD_SHOWS_DANCE_ACTION;
	
	@ClientString(id = 1801215, message = "The Bard shows (Applaud) action.")
	public static NpcStringId THE_BARD_SHOWS_APPLAUD_ACTION;
	
	@ClientString(id = 1801216, message = "The Bard shows (Yes) action.")
	public static NpcStringId THE_BARD_SHOWS_YES_ACTION;
	
	@ClientString(id = 1801217, message = "The Bard shows (No) action.")
	public static NpcStringId THE_BARD_SHOWS_NO_ACTION;
	
	@ClientString(id = 1801218, message = "The Bard shows (Laugh) action.")
	public static NpcStringId THE_BARD_SHOWS_LAUGH_ACTION;
	
	@ClientString(id = 1801219, message = "The Bard shows (Sorrow) action.")
	public static NpcStringId THE_BARD_SHOWS_SORROW_ACTION;
	
	@ClientString(id = 1801220, message = "This is fun! You are amazing!")
	public static NpcStringId THIS_IS_FUN_YOU_ARE_AMAZING;
	
	@ClientString(id = 1801221, message = "Disappointing! Please follow my actions!")
	public static NpcStringId DISAPPOINTING_PLEASE_FOLLOW_MY_ACTIONS;
	
	@ClientString(id = 1801222, message = "That was a wonderful performance! I had a great time!")
	public static NpcStringId THAT_WAS_A_WONDERFUL_PERFORMANCE_I_HAD_A_GREAT_TIME;
	
	@ClientString(id = 1801223, message = "I hope to have another wonderful time with all of you again!")
	public static NpcStringId I_HOPE_TO_HAVE_ANOTHER_WONDERFUL_TIME_WITH_ALL_OF_YOU_AGAIN;
	
	@ClientString(id = 1801224, message = "Then, I will leave now! Have a good weekend!")
	public static NpcStringId THEN_I_WILL_LEAVE_NOW_HAVE_A_GOOD_WEEKEND;
	
	@ClientString(id = 1801225, message = "Times up! Looks like we have to try again later!")
	public static NpcStringId TIMES_UP_LOOKS_LIKE_WE_HAVE_TO_TRY_AGAIN_LATER;
	
	@ClientString(id = 1801226, message = "Show $s1's party group (Greeting) action!")
	public static NpcStringId SHOW_S1_S_PARTY_GROUP_GREETING_ACTION;
	
	@ClientString(id = 1801227, message = "Show $s1's party group (victory) action!")
	public static NpcStringId SHOW_S1_S_PARTY_GROUP_VICTORY_ACTION;
	
	@ClientString(id = 1801228, message = "Show $s1's party group (dance) action!")
	public static NpcStringId SHOW_S1_S_PARTY_GROUP_DANCE_ACTION;
	
	@ClientString(id = 1801229, message = "Show $s1's party group (clap) action!")
	public static NpcStringId SHOW_S1_S_PARTY_GROUP_CLAP_ACTION;
	
	@ClientString(id = 1801230, message = "Show $s1's party group (yes) action!")
	public static NpcStringId SHOW_S1_S_PARTY_GROUP_YES_ACTION;
	
	@ClientString(id = 1801231, message = "Show $s1's party group (no) action!")
	public static NpcStringId SHOW_S1_S_PARTY_GROUP_NO_ACTION;
	
	@ClientString(id = 1801232, message = "Show $s1's party group (laugh) action!")
	public static NpcStringId SHOW_S1_S_PARTY_GROUP_LAUGH_ACTION;
	
	@ClientString(id = 1801233, message = "Show $s1's party group (sorrow) action!")
	public static NpcStringId SHOW_S1_S_PARTY_GROUP_SORROW_ACTION;
	
	@ClientString(id = 1801234, message = "Someone is off the stage. Please bring him/her back.")
	public static NpcStringId SOMEONE_IS_OFF_THE_STAGE_PLEASE_BRING_HIM_HER_BACK;
	
	@ClientString(id = 1801235, message = "Hahaha... You dare to disrupt me... I will be your nightmare from which you can never awaken!")
	public static NpcStringId HAHAHA_YOU_DARE_TO_DISRUPT_ME_I_WILL_BE_YOUR_NIGHTMARE_FROM_WHICH_YOU_CAN_NEVER_AWAKEN;
	
	@ClientString(id = 1801236, message = "You dare attack me? I will fill your nightmares with blood!")
	public static NpcStringId YOU_DARE_ATTACK_ME_I_WILL_FILL_YOUR_NIGHTMARES_WITH_BLOOD;
	
	@ClientString(id = 1801237, message = "I cannot let you stop the wraith of Shilen!")
	public static NpcStringId I_CANNOT_LET_YOU_STOP_THE_WRAITH_OF_SHILEN;
	
	@ClientString(id = 1801238, message = "Ah…")
	public static NpcStringId AH2;
	
	@ClientString(id = 1801239, message = "Halt! Your nightmares will fill you with dread!")
	public static NpcStringId HALT_YOUR_NIGHTMARES_WILL_FILL_YOU_WITH_DREAD;
	
	@ClientString(id = 1801240, message = "You won't get away!")
	public static NpcStringId YOU_WON_T_GET_AWAY;
	
	@ClientString(id = 1801241, message = "How... All that power... Removed...")
	public static NpcStringId HOW_ALL_THAT_POWER_REMOVED;
	
	@ClientString(id = 1801242, message = "Shilen... I have failed...")
	public static NpcStringId SHILEN_I_HAVE_FAILED;
	
	@ClientString(id = 1801243, message = "To think that I could fail… impossible.")
	public static NpcStringId TO_THINK_THAT_I_COULD_FAIL_IMPOSSIBLE;
	
	@ClientString(id = 1801244, message = "Who dares to bother us?")
	public static NpcStringId WHO_DARES_TO_BOTHER_US;
	
	@ClientString(id = 1801245, message = "How foolish. The price of attacking me is death!")
	public static NpcStringId HOW_FOOLISH_THE_PRICE_OF_ATTACKING_ME_IS_DEATH;
	
	@ClientString(id = 1801246, message = "My sword will take your life!")
	public static NpcStringId MY_SWORD_WILL_TAKE_YOUR_LIFE;
	
	@ClientString(id = 1801247, message = "Yaaah...")
	public static NpcStringId YAAAH;
	
	@ClientString(id = 1801248, message = "Prepare! I shall grant you death!")
	public static NpcStringId PREPARE_I_SHALL_GRANT_YOU_DEATH;
	
	@ClientString(id = 1801249, message = "Oh, Shilen... Give me strength...")
	public static NpcStringId OH_SHILEN_GIVE_ME_STRENGTH;
	
	@ClientString(id = 1801250, message = "I would defeated...")
	public static NpcStringId I_WOULD_DEFEATED;
	
	@ClientString(id = 1801251, message = "Don't think this is the end!")
	public static NpcStringId DON_T_THINK_THIS_IS_THE_END;
	
	@ClientString(id = 1801252, message = "No... I lost all the gathered power of light to this... this...")
	public static NpcStringId NO_I_LOST_ALL_THE_GATHERED_POWER_OF_LIGHT_TO_THIS_THIS;
	
	@ClientString(id = 1801253, message = "Are you the one to shatter the peace?")
	public static NpcStringId ARE_YOU_THE_ONE_TO_SHATTER_THE_PEACE;
	
	@ClientString(id = 1801254, message = "Our mission is to resurrect the goddess. Do not interfere.")
	public static NpcStringId OUR_MISSION_IS_TO_RESURRECT_THE_GODDESS_DO_NOT_INTERFERE;
	
	@ClientString(id = 1801255, message = "I will let you sleep in darkness. Forever...")
	public static NpcStringId I_WILL_LET_YOU_SLEEP_IN_DARKNESS_FOREVER;
	
	@ClientString(id = 1801256, message = "Hyaaaaaah....")
	public static NpcStringId HYAAAAAAH;
	
	@ClientString(id = 1801257, message = "Feel the true terror of darkness!")
	public static NpcStringId FEEL_THE_TRUE_TERROR_OF_DARKNESS;
	
	@ClientString(id = 1801258, message = "Oh, creatures of the goddess! Lend me your strength...")
	public static NpcStringId OH_CREATURES_OF_THE_GODDESS_LEND_ME_YOUR_STRENGTH;
	
	@ClientString(id = 1801259, message = "No! N... no! No!")
	public static NpcStringId NO_N_NO_NO;
	
	@ClientString(id = 1801260, message = "I will always watch you from the darkness...")
	public static NpcStringId I_WILL_ALWAYS_WATCH_YOU_FROM_THE_DARKNESS;
	
	@ClientString(id = 1801261, message = "I mustn't lose the strength...")
	public static NpcStringId I_MUSTN_T_LOSE_THE_STRENGTH;
	
	@ClientString(id = 1801262, message = "Hehehe, I'm glad you came. I was bored!")
	public static NpcStringId HEHEHE_I_M_GLAD_YOU_CAME_I_WAS_BORED;
	
	@ClientString(id = 1801263, message = "Hehehe, I'm glad you came. I was hungry!")
	public static NpcStringId HEHEHE_I_M_GLAD_YOU_CAME_I_WAS_HUNGRY;
	
	@ClientString(id = 1801264, message = "Hehehe, shall we play?")
	public static NpcStringId HEHEHE_SHALL_WE_PLAY;
	
	@ClientString(id = 1801265, message = "Kyaaah...")
	public static NpcStringId KYAAAH;
	
	@ClientString(id = 1801266, message = "Small fry! I will show you true madness!‎ Hahaha!!")
	public static NpcStringId SMALL_FRY_I_WILL_SHOW_YOU_TRUE_MADNESS_HAHAHA;
	
	@ClientString(id = 1801267, message = "Hehehe! Prepare! My madness will swallow you up!")
	public static NpcStringId HEHEHE_PREPARE_MY_MADNESS_WILL_SWALLOW_YOU_UP;
	
	@ClientString(id = 1801268, message = "Huh? What happened...? I... I lost?")
	public static NpcStringId HUH_WHAT_HAPPENED_I_I_LOST;
	
	@ClientString(id = 1801269, message = "Huhuhu... Huhuhu... Huhahaha!")
	public static NpcStringId HUHUHU_HUHUHU_HUHAHAHA;
	
	@ClientString(id = 1801270, message = "Ack! No! My body... It's disappearing...")
	public static NpcStringId ACK_NO_MY_BODY_IT_S_DISAPPEARING;
	
	@ClientString(id = 1801271, message = "Die...")
	public static NpcStringId DIE2;
	
	@ClientString(id = 1801272, message = "Do not interfere...")
	public static NpcStringId DO_NOT_INTERFERE;
	
	@ClientString(id = 1801273, message = "For the goddess...")
	public static NpcStringId FOR_THE_GODDESS;
	
	@ClientString(id = 1801274, message = "Ooooh...")
	public static NpcStringId OOOOH;
	
	@ClientString(id = 1801275, message = "You will die.")
	public static NpcStringId YOU_WILL_DIE;
	
	@ClientString(id = 1801276, message = "You will be destroyed.")
	public static NpcStringId YOU_WILL_BE_DESTROYED;
	
	@ClientString(id = 1801277, message = "Is this the end...?")
	public static NpcStringId IS_THIS_THE_END2;
	
	@ClientString(id = 1801278, message = "Oh, goddess...")
	public static NpcStringId OH_GODDESS;
	
	@ClientString(id = 1801279, message = "No! I didn't stay silent all this time, just to disappear now like this!")
	public static NpcStringId NO_I_DIDN_T_STAY_SILENT_ALL_THIS_TIME_JUST_TO_DISAPPEAR_NOW_LIKE_THIS;
	
	@ClientString(id = 1801280, message = "No! I don't want to die!")
	public static NpcStringId NO_I_DON_T_WANT_TO_DIE;
	
	@ClientString(id = 1801281, message = "Yaaaahhh")
	public static NpcStringId YAAAAHHH;
	
	@ClientString(id = 1801282, message = "This is crazy! I have to get out of here!")
	public static NpcStringId THIS_IS_CRAZY_I_HAVE_TO_GET_OUT_OF_HERE;
	
	@ClientString(id = 1801283, message = "I will protect my family. C'mon!")
	public static NpcStringId I_WILL_PROTECT_MY_FAMILY_C_MON;
	
	@ClientString(id = 1801284, message = "I won't forgive you for attacking my child!")
	public static NpcStringId I_WON_T_FORGIVE_YOU_FOR_ATTACKING_MY_CHILD;
	
	@ClientString(id = 1801285, message = "Don't bother my wife!")
	public static NpcStringId DON_T_BOTHER_MY_WIFE;
	
	@ClientString(id = 1801286, message = "Don't bother my kids!")
	public static NpcStringId DON_T_BOTHER_MY_KIDS;
	
	@ClientString(id = 1801287, message = "My husband is the pillar of our family!")
	public static NpcStringId MY_HUSBAND_IS_THE_PILLAR_OF_OUR_FAMILY;
	
	@ClientString(id = 1801288, message = "My child is my treasure. How dare you hit him!")
	public static NpcStringId MY_CHILD_IS_MY_TREASURE_HOW_DARE_YOU_HIT_HIM;
	
	@ClientString(id = 1801289, message = "Save me~")
	public static NpcStringId SAVE_ME;
	
	@ClientString(id = 1801290, message = "Mom, Dad... Help~")
	public static NpcStringId MOM_DAD_HELP;
	
	@ClientString(id = 1801291, message = "Waah. Stop hitting~")
	public static NpcStringId WAAH_STOP_HITTING;
	
	@ClientString(id = 1801292, message = "No! Stop!")
	public static NpcStringId NO_STOP;
	
	@ClientString(id = 1801293, message = "I've never seen this before...")
	public static NpcStringId I_VE_NEVER_SEEN_THIS_BEFORE;
	
	@ClientString(id = 1801294, message = "I had a feeling that something like this would happen. But no one listened to me.")
	public static NpcStringId I_HAD_A_FEELING_THAT_SOMETHING_LIKE_THIS_WOULD_HAPPEN_BUT_NO_ONE_LISTENED_TO_ME;
	
	@ClientString(id = 1801295, message = "Stop!")
	public static NpcStringId STOP;
	
	@ClientString(id = 1801296, message = "How dare you enter here!")
	public static NpcStringId HOW_DARE_YOU_ENTER_HERE;
	
	@ClientString(id = 1801297, message = "Do not think that you will avoid bloodshed by leaving here on your own free will. Your life is already forfeit!")
	public static NpcStringId DO_NOT_THINK_THAT_YOU_WILL_AVOID_BLOODSHED_BY_LEAVING_HERE_ON_YOUR_OWN_FREE_WILL_YOUR_LIFE_IS_ALREADY_FORFEIT;
	
	@ClientString(id = 1801300, message = "You filthy monsters. I will take you on.")
	public static NpcStringId YOU_FILTHY_MONSTERS_I_WILL_TAKE_YOU_ON;
	
	@ClientString(id = 1801301, message = "Stop right there. I will be your opponent.")
	public static NpcStringId STOP_RIGHT_THERE_I_WILL_BE_YOUR_OPPONENT;
	
	@ClientString(id = 1801302, message = "I will show you the justice of Aden.")
	public static NpcStringId I_WILL_SHOW_YOU_THE_JUSTICE_OF_ADEN;
	
	@ClientString(id = 1801303, message = "It's not over.")
	public static NpcStringId IT_S_NOT_OVER;
	
	@ClientString(id = 1801304, message = "Stop there.")
	public static NpcStringId STOP_THERE;
	
	@ClientString(id = 1801305, message = "Waaaaaaaahhhhhh")
	public static NpcStringId WAAAAAAAAHHHHHH;
	
	@ClientString(id = 1801306, message = "Die!")
	public static NpcStringId DIE3;
	
	@ClientString(id = 1801307, message = "Turn to ashes.")
	public static NpcStringId TURN_TO_ASHES;
	
	@ClientString(id = 1801308, message = "Multiple goals, shooting complete")
	public static NpcStringId MULTIPLE_GOALS_SHOOTING_COMPLETE;
	
	@ClientString(id = 1801309, message = "Goal discovered nearby. Begin shooting.")
	public static NpcStringId GOAL_DISCOVERED_NEARBY_BEGIN_SHOOTING;
	
	@ClientString(id = 1801310, message = "Aim for goal. Shooting complete.")
	public static NpcStringId AIM_FOR_GOAL_SHOOTING_COMPLETE;
	
	@ClientString(id = 1801311, message = "Withdraw.")
	public static NpcStringId WITHDRAW;
	
	@ClientString(id = 1801312, message = "Disappear.")
	public static NpcStringId DISAPPEAR;
	
	@ClientString(id = 1801313, message = "Spirits, show yourselves.")
	public static NpcStringId SPIRITS_SHOW_YOURSELVES;
	
	@ClientString(id = 1801314, message = "Power of light. Protect us.")
	public static NpcStringId POWER_OF_LIGHT_PROTECT_US;
	
	@ClientString(id = 1801315, message = "Come forth, Tree of Life!")
	public static NpcStringId COME_FORTH_TREE_OF_LIFE;
	
	@ClientString(id = 1801316, message = "MEN focus! Charging complete!")
	public static NpcStringId MEN_FOCUS_CHARGING_COMPLETE;
	
	@ClientString(id = 1801317, message = "Save me! I can't move!")
	public static NpcStringId SAVE_ME_I_CAN_T_MOVE;
	
	@ClientString(id = 1801318, message = "For us to live, you have to treat me until this altar comes to a stop.")
	public static NpcStringId FOR_US_TO_LIVE_YOU_HAVE_TO_TREAT_ME_UNTIL_THIS_ALTAR_COMES_TO_A_STOP;
	
	@ClientString(id = 1801319, message = "Ack! No...")
	public static NpcStringId ACK_NO;
	
	@ClientString(id = 1801320, message = "I am safe, thanks to you. I will begin supporting as soon as preparations are complete.")
	public static NpcStringId I_AM_SAFE_THANKS_TO_YOU_I_WILL_BEGIN_SUPPORTING_AS_SOON_AS_PREPARATIONS_ARE_COMPLETE;
	
	@ClientString(id = 1801321, message = "Vanguard of Aden! We have returned.")
	public static NpcStringId VANGUARD_OF_ADEN_WE_HAVE_RETURNED;
	
	@ClientString(id = 1801322, message = "Defeat all the monsters!")
	public static NpcStringId DEFEAT_ALL_THE_MONSTERS;
	
	@ClientString(id = 1801323, message = "You can no longer live here. Have a taste of the dimensional poison.")
	public static NpcStringId YOU_CAN_NO_LONGER_LIVE_HERE_HAVE_A_TASTE_OF_THE_DIMENSIONAL_POISON;
	
	@ClientString(id = 1801324, message = "ﾠNot bad for a bunch of humans. I'm leaving!")
	public static NpcStringId NOT_BAD_FOR_A_BUNCH_OF_HUMANS_I_M_LEAVING;
	
	@ClientString(id = 1801325, message = "The ritual is complete. Now, I will show you hell.")
	public static NpcStringId THE_RITUAL_IS_COMPLETE_NOW_I_WILL_SHOW_YOU_HELL;
	
	@ClientString(id = 1801326, message = "You puny insects don't know your place. You cannot stop me.")
	public static NpcStringId YOU_PUNY_INSECTS_DON_T_KNOW_YOUR_PLACE_YOU_CANNOT_STOP_ME;
	
	@ClientString(id = 1801327, message = "No... How could this be... I can't go back to Nihil like this...")
	public static NpcStringId NO_HOW_COULD_THIS_BE_I_CAN_T_GO_BACK_TO_NIHIL_LIKE_THIS;
	
	@ClientString(id = 1801328, message = "Kyeeeek. Kyaaaah.")
	public static NpcStringId KYEEEEK_KYAAAAH;
	
	@ClientString(id = 1801329, message = "Preparation")
	public static NpcStringId PREPARATION2;
	
	@ClientString(id = 1801330, message = "$s1 seconds to go!")
	public static NpcStringId S1_SECONDS_TO_GO;
	
	@ClientString(id = 1801331, message = "Start!")
	public static NpcStringId START;
	
	@ClientString(id = 1801332, message = "Victory?! Victory?! Please come back after more honing.")
	public static NpcStringId VICTORY_VICTORY_PLEASE_COME_BACK_AFTER_MORE_HONING;
	
	@ClientString(id = 1801333, message = "Huh? That tickles... Is that the best you can do?")
	public static NpcStringId HUH_THAT_TICKLES_IS_THAT_THE_BEST_YOU_CAN_DO;
	
	@ClientString(id = 1801334, message = "Huh? That kinda hurts. I don't like it~!")
	public static NpcStringId HUH_THAT_KINDA_HURTS_I_DON_T_LIKE_IT;
	
	@ClientString(id = 1801335, message = "Gasp... How could I fall in just $s1 seconds...?!")
	public static NpcStringId GASP_HOW_COULD_I_FALL_IN_JUST_S1_SECONDS;
	
	@ClientString(id = 1801336, message = "My strength explodes with my sword dance!!!")
	public static NpcStringId MY_STRENGTH_EXPLODES_WITH_MY_SWORD_DANCE;
	
	@ClientString(id = 1801337, message = "My strength is multiplying!! Hahaha!!!")
	public static NpcStringId MY_STRENGTH_IS_MULTIPLYING_HAHAHA;
	
	@ClientString(id = 1801338, message = "Forget about getting out of here alive...!")
	public static NpcStringId FORGET_ABOUT_GETTING_OUT_OF_HERE_ALIVE;
	
	@ClientString(id = 1801339, message = "You hit me? Go ahead, try again!")
	public static NpcStringId YOU_HIT_ME_GO_AHEAD_TRY_AGAIN;
	
	@ClientString(id = 1801340, message = "The Guillotine is full of power. It's shooting out.")
	public static NpcStringId THE_GUILLOTINE_IS_FULL_OF_POWER_IT_S_SHOOTING_OUT;
	
	@ClientString(id = 1801341, message = "Now... Feel my invincibility...")
	public static NpcStringId NOW_FEEL_MY_INVINCIBILITY;
	
	@ClientString(id = 1801342, message = "I have decided to settle in this fortress. So get out of here!")
	public static NpcStringId I_HAVE_DECIDED_TO_SETTLE_IN_THIS_FORTRESS_SO_GET_OUT_OF_HERE;
	
	@ClientString(id = 1801343, message = "Look around... Do you think you can defeat all of them?")
	public static NpcStringId LOOK_AROUND_DO_YOU_THINK_YOU_CAN_DEFEAT_ALL_OF_THEM;
	
	@ClientString(id = 1801344, message = "Even if the fires of hell surround me...!")
	public static NpcStringId EVEN_IF_THE_FIRES_OF_HELL_SURROUND_ME;
	
	@ClientString(id = 1801345, message = "I will feed your corpses to the Mandragoras.")
	public static NpcStringId I_WILL_FEED_YOUR_CORPSES_TO_THE_MANDRAGORAS;
	
	@ClientString(id = 1801346, message = "Stop your breathing! You have the stench of the living.")
	public static NpcStringId STOP_YOUR_BREATHING_YOU_HAVE_THE_STENCH_OF_THE_LIVING;
	
	@ClientString(id = 1801347, message = "Guillotine! These humans dare to attack you.")
	public static NpcStringId GUILLOTINE_THESE_HUMANS_DARE_TO_ATTACK_YOU;
	
	@ClientString(id = 1801348, message = "I curse those whom I have killed!")
	public static NpcStringId I_CURSE_THOSE_WHOM_I_HAVE_KILLED;
	
	@ClientString(id = 1801349, message = "Kyahahaha, stop making me laugh! That tickles!")
	public static NpcStringId KYAHAHAHA_STOP_MAKING_ME_LAUGH_THAT_TICKLES;
	
	@ClientString(id = 1801350, message = "Stop! That's enough.")
	public static NpcStringId STOP_THAT_S_ENOUGH;
	
	@ClientString(id = 1801351, message = "I can't move...")
	public static NpcStringId I_CAN_T_MOVE;
	
	@ClientString(id = 1801352, message = "Ooooh...")
	public static NpcStringId OOOOH2;
	
	@ClientString(id = 1801353, message = "Argh!!")
	public static NpcStringId ARGH;
	
	@ClientString(id = 1801354, message = "This can't be...!")
	public static NpcStringId THIS_CAN_T_BE;
	
	@ClientString(id = 1801355, message = "Curses!")
	public static NpcStringId CURSES;
	
	@ClientString(id = 1801356, message = "So, there are those who dare to challenge this sharp blade...")
	public static NpcStringId SO_THERE_ARE_THOSE_WHO_DARE_TO_CHALLENGE_THIS_SHARP_BLADE;
	
	@ClientString(id = 1801357, message = "Ergh... I will bring you to the brink of death!")
	public static NpcStringId ERGH_I_WILL_BRING_YOU_TO_THE_BRINK_OF_DEATH;
	
	@ClientString(id = 1801358, message = "Now, you won't feel pain or suffering anymore...!")
	public static NpcStringId NOW_YOU_WON_T_FEEL_PAIN_OR_SUFFERING_ANYMORE;
	
	@ClientString(id = 1801359, message = "My whole body was born away, and I drowned in suffering...")
	public static NpcStringId MY_WHOLE_BODY_WAS_BORN_AWAY_AND_I_DROWNED_IN_SUFFERING;
	
	@ClientString(id = 1801360, message = "Now, it is your turn to taste my suffering!")
	public static NpcStringId NOW_IT_IS_YOUR_TURN_TO_TASTE_MY_SUFFERING;
	
	@ClientString(id = 1801361, message = "Kekeke... I have become accustomed to this pain.")
	public static NpcStringId KEKEKE_I_HAVE_BECOME_ACCUSTOMED_TO_THIS_PAIN;
	
	@ClientString(id = 1801362, message = "Those who hear my screams will understand my pain...")
	public static NpcStringId THOSE_WHO_HEAR_MY_SCREAMS_WILL_UNDERSTAND_MY_PAIN;
	
	@ClientString(id = 1801363, message = "Do not come any closer! The nightmare of death is coming!")
	public static NpcStringId DO_NOT_COME_ANY_CLOSER_THE_NIGHTMARE_OF_DEATH_IS_COMING;
	
	@ClientString(id = 1801364, message = "Depart from me! Stop causing me pain...")
	public static NpcStringId DEPART_FROM_ME_STOP_CAUSING_ME_PAIN;
	
	@ClientString(id = 1801365, message = "Each day is painful. This body...")
	public static NpcStringId EACH_DAY_IS_PAINFUL_THIS_BODY;
	
	@ClientString(id = 1801366, message = "I was falsely accused... And I lost my eyes to torture...")
	public static NpcStringId I_WAS_FALSELY_ACCUSED_AND_I_LOST_MY_EYES_TO_TORTURE;
	
	@ClientString(id = 1801367, message = "My broken body can no longer hold a sword...")
	public static NpcStringId MY_BROKEN_BODY_CAN_NO_LONGER_HOLD_A_SWORD;
	
	@ClientString(id = 1801368, message = "I survive on these things that surround my body.")
	public static NpcStringId I_SURVIVE_ON_THESE_THINGS_THAT_SURROUND_MY_BODY;
	
	@ClientString(id = 1801369, message = "Why?! If I had known the reason, I wouldn't have died with such regret.")
	public static NpcStringId WHY_IF_I_HAD_KNOWN_THE_REASON_I_WOULDN_T_HAVE_DIED_WITH_SUCH_REGRET;
	
	@ClientString(id = 1801370, message = "I was a talented knighthood. But look at me now...")
	public static NpcStringId I_WAS_A_TALENTED_KNIGHTHOOD_BUT_LOOK_AT_ME_NOW;
	
	@ClientString(id = 1801371, message = "I must live in this body. It is unfair...")
	public static NpcStringId I_MUST_LIVE_IN_THIS_BODY_IT_IS_UNFAIR;
	
	@ClientString(id = 1801372, message = "I was torn apart without a reason. I despised everything.")
	public static NpcStringId I_WAS_TORN_APART_WITHOUT_A_REASON_I_DESPISED_EVERYTHING;
	
	@ClientString(id = 1801373, message = "I will never forgive them! Never!")
	public static NpcStringId I_WILL_NEVER_FORGIVE_THEM_NEVER;
	
	@ClientString(id = 1801374, message = "I want to rest for eternity... Who bothers me now?")
	public static NpcStringId I_WANT_TO_REST_FOR_ETERNITY_WHO_BOTHERS_ME_NOW;
	
	@ClientString(id = 1801375, message = "You should be careful. You might die!")
	public static NpcStringId YOU_SHOULD_BE_CAREFUL_YOU_MIGHT_DIE;
	
	@ClientString(id = 1801376, message = "Hohoho... Do I look pitiful? That makes my anger rise.")
	public static NpcStringId HOHOHO_DO_I_LOOK_PITIFUL_THAT_MAKES_MY_ANGER_RISE;
	
	@ClientString(id = 1801377, message = "Cry! Cry out!")
	public static NpcStringId CRY_CRY_OUT;
	
	@ClientString(id = 1801378, message = "See my appearance and suffer...")
	public static NpcStringId SEE_MY_APPEARANCE_AND_SUFFER;
	
	@ClientString(id = 1801379, message = "Sever!!!")
	public static NpcStringId SEVER;
	
	@ClientString(id = 1801380, message = "I will punish everyone who turned me into this!")
	public static NpcStringId I_WILL_PUNISH_EVERYONE_WHO_TURNED_ME_INTO_THIS;
	
	@ClientString(id = 1801381, message = "Stop tormenting me.")
	public static NpcStringId STOP_TORMENTING_ME;
	
	@ClientString(id = 1801382, message = "Ah ah ah uh uh...!")
	public static NpcStringId AH_AH_AH_UH_UH;
	
	@ClientString(id = 1801383, message = "Keuk!")
	public static NpcStringId KEUK;
	
	@ClientString(id = 1801384, message = "The suffering ends...!")
	public static NpcStringId THE_SUFFERING_ENDS;
	
	@ClientString(id = 1801385, message = "I shall curse you from the grave.")
	public static NpcStringId I_SHALL_CURSE_YOU_FROM_THE_GRAVE;
	
	@ClientString(id = 1801386, message = "Kyaaaaaaaahhhhh!")
	public static NpcStringId KYAAAAAAAAHHHHH;
	
	@ClientString(id = 1801387, message = "Sh! Sh! Shhhhhhh! Sssss....")
	public static NpcStringId SH_SH_SHHHHHHH_SSSSS;
	
	@ClientString(id = 1801388, message = "K.. k.. kkkkk....!")
	public static NpcStringId K_K_KKKKK;
	
	@ClientString(id = 1801389, message = "Hakum! Daigan-dalaolsum!")
	public static NpcStringId HAKUM_DAIGAN_DALAOLSUM;
	
	@ClientString(id = 1801390, message = "Kkk... Kyaahh!")
	public static NpcStringId KKK_KYAAHH;
	
	@ClientString(id = 1801391, message = "Bombomsara-bombomupsalla!")
	public static NpcStringId BOMBOMSARA_BOMBOMUPSALLA;
	
	@ClientString(id = 1801392, message = "Sarasara bujarasarasara!!")
	public static NpcStringId SARASARA_BUJARASARASARA;
	
	@ClientString(id = 1801393, message = "Bukijika bukaja")
	public static NpcStringId BUKIJIKA_BUKAJA;
	
	@ClientString(id = 1801394, message = "Peeuuuukkkk!")
	public static NpcStringId PEEUUUUKKKK;
	
	@ClientString(id = 1801395, message = "Ack!")
	public static NpcStringId ACK;
	
	@ClientString(id = 1801396, message = "Taaak tak!")
	public static NpcStringId TAAAK_TAK;
	
	@ClientString(id = 1801397, message = "Oooo Oooo...")
	public static NpcStringId OOOO_OOOO;
	
	@ClientString(id = 1801398, message = "Hyak!")
	public static NpcStringId HYAK;
	
	@ClientString(id = 1801399, message = "Heuooo...")
	public static NpcStringId HEUOOO;
	
	@ClientString(id = 1801400, message = "Kh ah ah ah!")
	public static NpcStringId KH_AH_AH_AH;
	
	@ClientString(id = 1801401, message = "Keeuuuuhhhh!!")
	public static NpcStringId KEEUUUUHHHH;
	
	@ClientString(id = 1801402, message = "Keu... ooo...")
	public static NpcStringId KEU_OOO;
	
	@ClientString(id = 1801403, message = "Kaaahhh!")
	public static NpcStringId KAAAHHH;
	
	@ClientString(id = 1801404, message = "Krrrr...!!")
	public static NpcStringId KRRRR;
	
	@ClientString(id = 1801405, message = "Kuuuuu")
	public static NpcStringId KUUUUU;
	
	@ClientString(id = 1801406, message = "Urggghh....!")
	public static NpcStringId URGGGHH;
	
	@ClientString(id = 1801407, message = "If you wish to entrust your body to the guillotine, then come on!")
	public static NpcStringId IF_YOU_WISH_TO_ENTRUST_YOUR_BODY_TO_THE_GUILLOTINE_THEN_COME_ON;
	
	@ClientString(id = 1801408, message = "Do you want the puppet experience?")
	public static NpcStringId DO_YOU_WANT_THE_PUPPET_EXPERIENCE;
	
	@ClientString(id = 1801409, message = "Hazkal, I will accept the orders of the Guillotine and remove these purities.")
	public static NpcStringId HAZKAL_I_WILL_ACCEPT_THE_ORDERS_OF_THE_GUILLOTINE_AND_REMOVE_THESE_PURITIES;
	
	@ClientString(id = 1801410, message = "I will offer them as a sacrifice to the guillotine.")
	public static NpcStringId I_WILL_OFFER_THEM_AS_A_SACRIFICE_TO_THE_GUILLOTINE;
	
	@ClientString(id = 1801411, message = "Those who enter the fortress without permission have nothing to say!")
	public static NpcStringId THOSE_WHO_ENTER_THE_FORTRESS_WITHOUT_PERMISSION_HAVE_NOTHING_TO_SAY;
	
	@ClientString(id = 1801412, message = "I can hear a scream.")
	public static NpcStringId I_CAN_HEAR_A_SCREAM;
	
	@ClientString(id = 1801413, message = "What a strange weapon. It's all shiny and cute.")
	public static NpcStringId WHAT_A_STRANGE_WEAPON_IT_S_ALL_SHINY_AND_CUTE;
	
	@ClientString(id = 1801414, message = "I will destroy you one by one.")
	public static NpcStringId I_WILL_DESTROY_YOU_ONE_BY_ONE;
	
	@ClientString(id = 1801415, message = "Submit to me, the faithful right-hand man.")
	public static NpcStringId SUBMIT_TO_ME_THE_FAITHFUL_RIGHT_HAND_MAN;
	
	@ClientString(id = 1801416, message = "Lose hope! Despair! Then, die!")
	public static NpcStringId LOSE_HOPE_DESPAIR_THEN_DIE;
	
	@ClientString(id = 1801417, message = "Yes, it's this feeling! The feeling of striking you down!")
	public static NpcStringId YES_IT_S_THIS_FEELING_THE_FEELING_OF_STRIKING_YOU_DOWN;
	
	@ClientString(id = 1801418, message = "You shall wither and die beneath the great guillotine fortress!")
	public static NpcStringId YOU_SHALL_WITHER_AND_DIE_BENEATH_THE_GREAT_GUILLOTINE_FORTRESS;
	
	@ClientString(id = 1801419, message = "Your blood shall fill this fortress.")
	public static NpcStringId YOUR_BLOOD_SHALL_FILL_THIS_FORTRESS;
	
	@ClientString(id = 1801420, message = "Kahahaha kyahahaha!")
	public static NpcStringId KAHAHAHA_KYAHAHAHA;
	
	@ClientString(id = 1801421, message = "That tickles!")
	public static NpcStringId THAT_TICKLES;
	
	@ClientString(id = 1801422, message = "Oh guillotine! Great guillotine! See them!!")
	public static NpcStringId OH_GUILLOTINE_GREAT_GUILLOTINE_SEE_THEM;
	
	@ClientString(id = 1801423, message = "I'm warning you! Stop right here.")
	public static NpcStringId I_M_WARNING_YOU_STOP_RIGHT_HERE;
	
	@ClientString(id = 1801424, message = "I am eternal! I am Hazkal, the closets being to the end of all things!")
	public static NpcStringId I_AM_ETERNAL_I_AM_HAZKAL_THE_CLOSETS_BEING_TO_THE_END_OF_ALL_THINGS;
	
	@ClientString(id = 1801425, message = "I don't want to die! I don't want to die for the second time!")
	public static NpcStringId I_DON_T_WANT_TO_DIE_I_DON_T_WANT_TO_DIE_FOR_THE_SECOND_TIME;
	
	@ClientString(id = 1801426, message = "Save me...")
	public static NpcStringId SAVE_ME2;
	
	@ClientString(id = 1801427, message = "I still have a lot of souls to eat!")
	public static NpcStringId I_STILL_HAVE_A_LOT_OF_SOULS_TO_EAT;
	
	@ClientString(id = 1801428, message = "Gasp!")
	public static NpcStringId GASP3;
	
	@ClientString(id = 1801429, message = "Urgh!")
	public static NpcStringId URGH;
	
	@ClientString(id = 1801430, message = "Oh guillotine, invaders have entered.")
	public static NpcStringId OH_GUILLOTINE_INVADERS_HAVE_ENTERED;
	
	@ClientString(id = 1801431, message = "The darkness will swallow you up.")
	public static NpcStringId THE_DARKNESS_WILL_SWALLOW_YOU_UP;
	
	@ClientString(id = 1801432, message = "Binding, and the esthetics of tossing out")
	public static NpcStringId BINDING_AND_THE_ESTHETICS_OF_TOSSING_OUT;
	
	@ClientString(id = 1801433, message = "Blocking, and the power of force")
	public static NpcStringId BLOCKING_AND_THE_POWER_OF_FORCE;
	
	@ClientString(id = 1801434, message = "Gasp")
	public static NpcStringId GASP4;
	
	@ClientString(id = 1801435, message = "Keuk!")
	public static NpcStringId KEUK2;
	
	@ClientString(id = 1801436, message = "Urgh...!")
	public static NpcStringId URGH2;
	
	@ClientString(id = 1801437, message = "Ooh...")
	public static NpcStringId OOH2;
	
	@ClientString(id = 1801438, message = "You! The one hiding there!")
	public static NpcStringId YOU_THE_ONE_HIDING_THERE;
	
	@ClientString(id = 1801439, message = "My bowstring never rests.")
	public static NpcStringId MY_BOWSTRING_NEVER_RESTS;
	
	@ClientString(id = 1801440, message = "I protect the guillotine fortress. Challenge me.")
	public static NpcStringId I_PROTECT_THE_GUILLOTINE_FORTRESS_CHALLENGE_ME;
	
	@ClientString(id = 1801441, message = "I will pierce you without end!")
	public static NpcStringId I_WILL_PIERCE_YOU_WITHOUT_END;
	
	@ClientString(id = 1801442, message = "Fight fairly!")
	public static NpcStringId FIGHT_FAIRLY;
	
	@ClientString(id = 1801443, message = "I shall have to report to Houpon.")
	public static NpcStringId I_SHALL_HAVE_TO_REPORT_TO_HOUPON;
	
	@ClientString(id = 1801444, message = "I shall pierce your heart.")
	public static NpcStringId I_SHALL_PIERCE_YOUR_HEART;
	
	@ClientString(id = 1801445, message = "I will give you rest with the arrow of despair.")
	public static NpcStringId I_WILL_GIVE_YOU_REST_WITH_THE_ARROW_OF_DESPAIR;
	
	@ClientString(id = 1801446, message = "Do not take lightly the archers of the guillotine fortress!")
	public static NpcStringId DO_NOT_TAKE_LIGHTLY_THE_ARCHERS_OF_THE_GUILLOTINE_FORTRESS;
	
	@ClientString(id = 1801447, message = "You have to give me time to set up! I'm losing my mind.")
	public static NpcStringId YOU_HAVE_TO_GIVE_ME_TIME_TO_SET_UP_I_M_LOSING_MY_MIND;
	
	@ClientString(id = 1801448, message = "Focused attack!")
	public static NpcStringId FOCUSED_ATTACK;
	
	@ClientString(id = 1801449, message = "Ack!")
	public static NpcStringId ACK2;
	
	@ClientString(id = 1801450, message = "Ooo... ooo...")
	public static NpcStringId OOO_OOO;
	
	@ClientString(id = 1801451, message = "I surrender! I surrender!")
	public static NpcStringId I_SURRENDER_I_SURRENDER;
	
	@ClientString(id = 1801452, message = "Houpon! I must call Houpon.")
	public static NpcStringId HOUPON_I_MUST_CALL_HOUPON;
	
	@ClientString(id = 1801453, message = "Houpon! Please avenge me.")
	public static NpcStringId HOUPON_PLEASE_AVENGE_ME;
	
	@ClientString(id = 1801454, message = "I must report to Houpon.")
	public static NpcStringId I_MUST_REPORT_TO_HOUPON;
	
	@ClientString(id = 1801455, message = "Suffering... Lost... Forgetting...")
	public static NpcStringId SUFFERING_LOST_FORGETTING;
	
	@ClientString(id = 1801456, message = "Eternal life... Absorb... Resurrection...")
	public static NpcStringId ETERNAL_LIFE_ABSORB_RESURRECTION;
	
	@ClientString(id = 1801457, message = "Prophecy...")
	public static NpcStringId PROPHECY;
	
	@ClientString(id = 1801458, message = "Slaughter... Contempt...")
	public static NpcStringId SLAUGHTER_CONTEMPT;
	
	@ClientString(id = 1801459, message = "Future... Choice...")
	public static NpcStringId FUTURE_CHOICE;
	
	@ClientString(id = 1801460, message = "Death...")
	public static NpcStringId DEATH;
	
	@ClientString(id = 1801461, message = "Beheading...")
	public static NpcStringId BEHEADING;
	
	@ClientString(id = 1801462, message = "It's no fun if you die in one blow, right?")
	public static NpcStringId IT_S_NO_FUN_IF_YOU_DIE_IN_ONE_BLOW_RIGHT;
	
	@ClientString(id = 1801463, message = "My sharp blade is thirsty for your blood...")
	public static NpcStringId MY_SHARP_BLADE_IS_THIRSTY_FOR_YOUR_BLOOD;
	
	@ClientString(id = 1801464, message = "Do not interfere. Are you using violence inside the fortress?!")
	public static NpcStringId DO_NOT_INTERFERE_ARE_YOU_USING_VIOLENCE_INSIDE_THE_FORTRESS;
	
	@ClientString(id = 1801465, message = "There are mountains of corpses of those who attacked me without taking me seriously...")
	public static NpcStringId THERE_ARE_MOUNTAINS_OF_CORPSES_OF_THOSE_WHO_ATTACKED_ME_WITHOUT_TAKING_ME_SERIOUSLY;
	
	@ClientString(id = 1801466, message = "I will slash your flesh with this rune sword!")
	public static NpcStringId I_WILL_SLASH_YOUR_FLESH_WITH_THIS_RUNE_SWORD;
	
	@ClientString(id = 1801467, message = "Huhuhu... What curious people you are.")
	public static NpcStringId HUHUHU_WHAT_CURIOUS_PEOPLE_YOU_ARE;
	
	@ClientString(id = 1801468, message = "Slowly... And with suspense!")
	public static NpcStringId SLOWLY_AND_WITH_SUSPENSE;
	
	@ClientString(id = 1801469, message = "You're laughing? Go ahead! Then, you will die.")
	public static NpcStringId YOU_RE_LAUGHING_GO_AHEAD_THEN_YOU_WILL_DIE;
	
	@ClientString(id = 1801470, message = "Taste the sharpness of my blade and die.")
	public static NpcStringId TASTE_THE_SHARPNESS_OF_MY_BLADE_AND_DIE;
	
	@ClientString(id = 1801471, message = "Stab and twist. Simple, right?")
	public static NpcStringId STAB_AND_TWIST_SIMPLE_RIGHT;
	
	@ClientString(id = 1801472, message = "My sword will cut through your armor like paper.")
	public static NpcStringId MY_SWORD_WILL_CUT_THROUGH_YOUR_ARMOR_LIKE_PAPER;
	
	@ClientString(id = 1801473, message = "Ha!")
	public static NpcStringId HA;
	
	@ClientString(id = 1801474, message = "What a pathetic end to carelessness...")
	public static NpcStringId WHAT_A_PATHETIC_END_TO_CARELESSNESS;
	
	@ClientString(id = 1801475, message = "I cannot stand any longer.")
	public static NpcStringId I_CANNOT_STAND_ANY_LONGER;
	
	@ClientString(id = 1801476, message = "A bit more... Clench the teeth! Kyah!")
	public static NpcStringId A_BIT_MORE_CLENCH_THE_TEETH_KYAH;
	
	@ClientString(id = 1801477, message = "Now, don't die too easily.")
	public static NpcStringId NOW_DON_T_DIE_TOO_EASILY;
	
	@ClientString(id = 1801478, message = "I am the guillotine. If you remember me from the past, that will be your carelessness.")
	public static NpcStringId I_AM_THE_GUILLOTINE_IF_YOU_REMEMBER_ME_FROM_THE_PAST_THAT_WILL_BE_YOUR_CARELESSNESS;
	
	@ClientString(id = 1801479, message = "See this nightmarish truth. How would you like to see your comrades die?")
	public static NpcStringId SEE_THIS_NIGHTMARISH_TRUTH_HOW_WOULD_YOU_LIKE_TO_SEE_YOUR_COMRADES_DIE;
	
	@ClientString(id = 1801480, message = "Your group attacks so easily. I am alone.")
	public static NpcStringId YOUR_GROUP_ATTACKS_SO_EASILY_I_AM_ALONE;
	
	@ClientString(id = 1801481, message = "Swing and strike! You are just a lump of flesh.")
	public static NpcStringId SWING_AND_STRIKE_YOU_ARE_JUST_A_LUMP_OF_FLESH;
	
	@ClientString(id = 1801482, message = "Puhahahahaha! This is fun!")
	public static NpcStringId PUHAHAHAHAHA_THIS_IS_FUN;
	
	@ClientString(id = 1801483, message = "You killed many of my subordinates.")
	public static NpcStringId YOU_KILLED_MANY_OF_MY_SUBORDINATES;
	
	@ClientString(id = 1801484, message = "You can stand it, right? Good! That's good! Hahaha!")
	public static NpcStringId YOU_CAN_STAND_IT_RIGHT_GOOD_THAT_S_GOOD_HAHAHA;
	
	@ClientString(id = 1801485, message = "I'm getting more and more excited!")
	public static NpcStringId I_M_GETTING_MORE_AND_MORE_EXCITED;
	
	@ClientString(id = 1801486, message = "Insects! Insignificant bugs to be killed!")
	public static NpcStringId INSECTS_INSIGNIFICANT_BUGS_TO_BE_KILLED;
	
	@ClientString(id = 1801487, message = "I'm the boss of this place! How dare you act with such impudence!!!")
	public static NpcStringId I_M_THE_BOSS_OF_THIS_PLACE_HOW_DARE_YOU_ACT_WITH_SUCH_IMPUDENCE;
	
	@ClientString(id = 1801488, message = "I'm feeling power! Must be Katarsis on the battlefield! Hahaha!")
	public static NpcStringId I_M_FEELING_POWER_MUST_BE_KATARSIS_ON_THE_BATTLEFIELD_HAHAHA;
	
	@ClientString(id = 1801489, message = "I hate all of you who are alive!")
	public static NpcStringId I_HATE_ALL_OF_YOU_WHO_ARE_ALIVE;
	
	@ClientString(id = 1801490, message = "Undead? Don't label me so easily!")
	public static NpcStringId UNDEAD_DON_T_LABEL_ME_SO_EASILY;
	
	@ClientString(id = 1801491, message = "Kuhahahahaha! C'mon! Try some more!!")
	public static NpcStringId KUHAHAHAHAHA_C_MON_TRY_SOME_MORE;
	
	@ClientString(id = 1801492, message = "I will tear your soul at the tip of my spear.")
	public static NpcStringId I_WILL_TEAR_YOUR_SOUL_AT_THE_TIP_OF_MY_SPEAR;
	
	@ClientString(id = 1801493, message = "Protect feelings? When you die, die with certainty!")
	public static NpcStringId PROTECT_FEELINGS_WHEN_YOU_DIE_DIE_WITH_CERTAINTY;
	
	@ClientString(id = 1801494, message = "This fortress is not your playground! How dare you!")
	public static NpcStringId THIS_FORTRESS_IS_NOT_YOUR_PLAYGROUND_HOW_DARE_YOU;
	
	@ClientString(id = 1801495, message = "Sadness surrounds me.")
	public static NpcStringId SADNESS_SURROUNDS_ME;
	
	@ClientString(id = 1801496, message = "Do you know how it feels to die on those you love?")
	public static NpcStringId DO_YOU_KNOW_HOW_IT_FEELS_TO_DIE_ON_THOSE_YOU_LOVE;
	
	@ClientString(id = 1801497, message = "I will bring peace back to this place.")
	public static NpcStringId I_WILL_BRING_PEACE_BACK_TO_THIS_PLACE;
	
	@ClientString(id = 1801498, message = "The guillotine of death is hiding your identity.")
	public static NpcStringId THE_GUILLOTINE_OF_DEATH_IS_HIDING_YOUR_IDENTITY;
	
	@ClientString(id = 1801499, message = "The guillotine always makes me cower.")
	public static NpcStringId THE_GUILLOTINE_ALWAYS_MAKES_ME_COWER;
	
	@ClientString(id = 1801500, message = "Sometimes, I cannot accept my death.")
	public static NpcStringId SOMETIMES_I_CANNOT_ACCEPT_MY_DEATH;
	
	@ClientString(id = 1801501, message = "I died without mercy in this place.")
	public static NpcStringId I_DIED_WITHOUT_MERCY_IN_THIS_PLACE;
	
	@ClientString(id = 1801502, message = "My brutal curse keeps me trapped in this place.")
	public static NpcStringId MY_BRUTAL_CURSE_KEEPS_ME_TRAPPED_IN_THIS_PLACE;
	
	@ClientString(id = 1801503, message = "I cannot rely on anyone anymore.")
	public static NpcStringId I_CANNOT_RELY_ON_ANYONE_ANYMORE;
	
	@ClientString(id = 1801504, message = "Hurry and escape from this place...")
	public static NpcStringId HURRY_AND_ESCAPE_FROM_THIS_PLACE;
	
	@ClientString(id = 1801505, message = "I love you. Please don't forget me.")
	public static NpcStringId I_LOVE_YOU_PLEASE_DON_T_FORGET_ME;
	
	@ClientString(id = 1801506, message = "I really didn't want to die.")
	public static NpcStringId I_REALLY_DIDN_T_WANT_TO_DIE;
	
	@ClientString(id = 1801507, message = "You! Can you see me?")
	public static NpcStringId YOU_CAN_YOU_SEE_ME;
	
	@ClientString(id = 1801508, message = "This place is all filled with suffocating darkness!")
	public static NpcStringId THIS_PLACE_IS_ALL_FILLED_WITH_SUFFOCATING_DARKNESS;
	
	@ClientString(id = 1801509, message = "My body was beaten to a pulp.")
	public static NpcStringId MY_BODY_WAS_BEATEN_TO_A_PULP;
	
	@ClientString(id = 1801510, message = "I will never forget the face of my torturer...!")
	public static NpcStringId I_WILL_NEVER_FORGET_THE_FACE_OF_MY_TORTURER;
	
	@ClientString(id = 1801511, message = "My entire body was destroyed by fear.")
	public static NpcStringId MY_ENTIRE_BODY_WAS_DESTROYED_BY_FEAR;
	
	@ClientString(id = 1801512, message = "No! Go away! Don't let anyone come toward me...!")
	public static NpcStringId NO_GO_AWAY_DON_T_LET_ANYONE_COME_TOWARD_ME;
	
	@ClientString(id = 1801513, message = "I left my family behind and was dragged to this place.")
	public static NpcStringId I_LEFT_MY_FAMILY_BEHIND_AND_WAS_DRAGGED_TO_THIS_PLACE;
	
	@ClientString(id = 1801514, message = "If I let go of my grudge, will I be able to escape from this place?")
	public static NpcStringId IF_I_LET_GO_OF_MY_GRUDGE_WILL_I_BE_ABLE_TO_ESCAPE_FROM_THIS_PLACE;
	
	@ClientString(id = 1801515, message = "I want to feel the ground under my feet. But I no longer have any weight.")
	public static NpcStringId I_WANT_TO_FEEL_THE_GROUND_UNDER_MY_FEET_BUT_I_NO_LONGER_HAVE_ANY_WEIGHT;
	
	@ClientString(id = 1801516, message = "Hot! Too hot!")
	public static NpcStringId HOT_TOO_HOT;
	
	@ClientString(id = 1801517, message = "The air here feels like it's burning!")
	public static NpcStringId THE_AIR_HERE_FEELS_LIKE_IT_S_BURNING;
	
	@ClientString(id = 1801518, message = "Get away! Get that fire away from me!")
	public static NpcStringId GET_AWAY_GET_THAT_FIRE_AWAY_FROM_ME;
	
	@ClientString(id = 1801519, message = "Ack! AAAHHHH!!")
	public static NpcStringId ACK_AAAHHHH;
	
	@ClientString(id = 1801520, message = "I'm always covered in sweat. So pitiful...")
	public static NpcStringId I_M_ALWAYS_COVERED_IN_SWEAT_SO_PITIFUL;
	
	@ClientString(id = 1801521, message = "I don't know how many times I pleaded for my life.")
	public static NpcStringId I_DON_T_KNOW_HOW_MANY_TIMES_I_PLEADED_FOR_MY_LIFE;
	
	@ClientString(id = 1801522, message = "Please destroy the guillotine. That is my wish!")
	public static NpcStringId PLEASE_DESTROY_THE_GUILLOTINE_THAT_IS_MY_WISH;
	
	@ClientString(id = 1801523, message = "Crackle crackle... That sound drives me crazy.")
	public static NpcStringId CRACKLE_CRACKLE_THAT_SOUND_DRIVES_ME_CRAZY;
	
	@ClientString(id = 1801524, message = "I remember the eyes of the monsters that looked upon me...")
	public static NpcStringId I_REMEMBER_THE_EYES_OF_THE_MONSTERS_THAT_LOOKED_UPON_ME;
	
	@ClientString(id = 1801525, message = "Please... just kill me...")
	public static NpcStringId PLEASE_JUST_KILL_ME;
	
	@ClientString(id = 1801526, message = "I hate red. The color of fire... and blood...")
	public static NpcStringId I_HATE_RED_THE_COLOR_OF_FIRE_AND_BLOOD;
	
	@ClientString(id = 1801527, message = "Kkk... Did you see that mysterious image at night?")
	public static NpcStringId KKK_DID_YOU_SEE_THAT_MYSTERIOUS_IMAGE_AT_NIGHT;
	
	@ClientString(id = 1801528, message = "I want to see the image that appears in the darkness.")
	public static NpcStringId I_WANT_TO_SEE_THE_IMAGE_THAT_APPEARS_IN_THE_DARKNESS;
	
	@ClientString(id = 1801529, message = "Darkness and annihilation. Don't those words go well together?")
	public static NpcStringId DARKNESS_AND_ANNIHILATION_DON_T_THOSE_WORDS_GO_WELL_TOGETHER;
	
	@ClientString(id = 1801530, message = "Things you cannot see during the day appear at night.")
	public static NpcStringId THINGS_YOU_CANNOT_SEE_DURING_THE_DAY_APPEAR_AT_NIGHT;
	
	@ClientString(id = 1801531, message = "Don't walk around carelessly. You might die in an instant!")
	public static NpcStringId DON_T_WALK_AROUND_CARELESSLY_YOU_MIGHT_DIE_IN_AN_INSTANT;
	
	@ClientString(id = 1801532, message = "Rare life forms appear here from time to time.")
	public static NpcStringId RARE_LIFE_FORMS_APPEAR_HERE_FROM_TIME_TO_TIME;
	
	@ClientString(id = 1801533, message = "Have you ever held the Proof of Survival in your hand?")
	public static NpcStringId HAVE_YOU_EVER_HELD_THE_PROOF_OF_SURVIVAL_IN_YOUR_HAND;
	
	@ClientString(id = 1801534, message = "They say that the monster that carries around treasure holds it.")
	public static NpcStringId THEY_SAY_THAT_THE_MONSTER_THAT_CARRIES_AROUND_TREASURE_HOLDS_IT;
	
	@ClientString(id = 1801535, message = "Isn't it illogical to search for money in a barren land?")
	public static NpcStringId ISN_T_IT_ILLOGICAL_TO_SEARCH_FOR_MONEY_IN_A_BARREN_LAND;
	
	@ClientString(id = 1801536, message = "Let go of your greed. There is a greater reward waiting for you.")
	public static NpcStringId LET_GO_OF_YOUR_GREED_THERE_IS_A_GREATER_REWARD_WAITING_FOR_YOU;
	
	@ClientString(id = 1801537, message = "Let go of the things you desire, so that good fortune finds you...")
	public static NpcStringId LET_GO_OF_THE_THINGS_YOU_DESIRE_SO_THAT_GOOD_FORTUNE_FINDS_YOU;
	
	@ClientString(id = 1801538, message = "Treasure the time that you have, even if it's only 10 minutes.")
	public static NpcStringId TREASURE_THE_TIME_THAT_YOU_HAVE_EVEN_IF_IT_S_ONLY_10_MINUTES;
	
	@ClientString(id = 1801539, message = "People can obtain a great fortune in a short amount of time.")
	public static NpcStringId PEOPLE_CAN_OBTAIN_A_GREAT_FORTUNE_IN_A_SHORT_AMOUNT_OF_TIME;
	
	@ClientString(id = 1801540, message = "Don't worry about me. I will speak the truth until my dying day.")
	public static NpcStringId DON_T_WORRY_ABOUT_ME_I_WILL_SPEAK_THE_TRUTH_UNTIL_MY_DYING_DAY;
	
	@ClientString(id = 1801541, message = "The mandragora plants had many varieties.")
	public static NpcStringId THE_MANDRAGORA_PLANTS_HAD_MANY_VARIETIES;
	
	@ClientString(id = 1801542, message = "Skull Dissect... That monster doesn't dare show itself.")
	public static NpcStringId SKULL_DISSECT_THAT_MONSTER_DOESN_T_DARE_SHOW_ITSELF;
	
	@ClientString(id = 1801543, message = "Do not fear their shields. It is just an illusion.")
	public static NpcStringId DO_NOT_FEAR_THEIR_SHIELDS_IT_IS_JUST_AN_ILLUSION;
	
	@ClientString(id = 1801544, message = "Do not obsess over the small things. Look with a wide view...")
	public static NpcStringId DO_NOT_OBSESS_OVER_THE_SMALL_THINGS_LOOK_WITH_A_WIDE_VIEW;
	
	@ClientString(id = 1801545, message = "Their shields are strong. But they are also careless!")
	public static NpcStringId THEIR_SHIELDS_ARE_STRONG_BUT_THEY_ARE_ALSO_CARELESS;
	
	@ClientString(id = 1801546, message = "Hun.. hungry...")
	public static NpcStringId HUN_HUNGRY2;
	
	@ClientString(id = 1801547, message = "Aha~")
	public static NpcStringId AHA;
	
	@ClientString(id = 1801548, message = "If it's a lot, I'll eat more.")
	public static NpcStringId IF_IT_S_A_LOT_I_LL_EAT_MORE;
	
	@ClientString(id = 1801549, message = "Wooowww!")
	public static NpcStringId WOOOWWW;
	
	@ClientString(id = 1801550, message = "No way! Stop! Ack!")
	public static NpcStringId NO_WAY_STOP_ACK;
	
	@ClientString(id = 1801551, message = "You lot...!!")
	public static NpcStringId YOU_LOT;
	
	@ClientString(id = 1801552, message = "I'm getting hungry.")
	public static NpcStringId I_M_GETTING_HUNGRY;
	
	@ClientString(id = 1801553, message = "Your kids are really cute.")
	public static NpcStringId YOUR_KIDS_ARE_REALLY_CUTE;
	
	@ClientString(id = 1801554, message = "Have a good and safe day.")
	public static NpcStringId HAVE_A_GOOD_AND_SAFE_DAY;
	
	@ClientString(id = 1801555, message = "Who's there?")
	public static NpcStringId WHO_S_THERE;
	
	@ClientString(id = 1801556, message = "Where do you think you are?")
	public static NpcStringId WHERE_DO_YOU_THINK_YOU_ARE;
	
	@ClientString(id = 1801557, message = "Hey, stop!")
	public static NpcStringId HEY_STOP;
	
	@ClientString(id = 1801558, message = "You killed our baby!")
	public static NpcStringId YOU_KILLED_OUR_BABY;
	
	@ClientString(id = 1801559, message = "Kyah! I won't forgive you!")
	public static NpcStringId KYAH_I_WON_T_FORGIVE_YOU;
	
	@ClientString(id = 1801560, message = "Even if I die, I can't forgive you. We'll die together!")
	public static NpcStringId EVEN_IF_I_DIE_I_CAN_T_FORGIVE_YOU_WE_LL_DIE_TOGETHER;
	
	@ClientString(id = 1801561, message = "Everyone, come here.")
	public static NpcStringId EVERYONE_COME_HERE;
	
	@ClientString(id = 1801562, message = "Babies, you mustn't stray from your mom.")
	public static NpcStringId BABIES_YOU_MUSTN_T_STRAY_FROM_YOUR_MOM;
	
	@ClientString(id = 1801563, message = "What should we have for dinner?")
	public static NpcStringId WHAT_SHOULD_WE_HAVE_FOR_DINNER;
	
	@ClientString(id = 1801564, message = "Mom, I'm hungry.")
	public static NpcStringId MOM_I_M_HUNGRY;
	
	@ClientString(id = 1801565, message = "Mom and Dad, let's play.")
	public static NpcStringId MOM_AND_DAD_LET_S_PLAY;
	
	@ClientString(id = 1801566, message = "Yap yap yap.")
	public static NpcStringId YAP_YAP_YAP;
	
	@ClientString(id = 1801567, message = "Stop fooling around and get to work!")
	public static NpcStringId STOP_FOOLING_AROUND_AND_GET_TO_WORK;
	
	@ClientString(id = 1801568, message = "Stop goofing off. Want a time-out? Or a spanking?")
	public static NpcStringId STOP_GOOFING_OFF_WANT_A_TIME_OUT_OR_A_SPANKING;
	
	@ClientString(id = 1801569, message = "It's too late. Don't you know that it should have ended 30 minutes ago?")
	public static NpcStringId IT_S_TOO_LATE_DON_T_YOU_KNOW_THAT_IT_SHOULD_HAVE_ENDED_30_MINUTES_AGO;
	
	@ClientString(id = 1801570, message = "My machine is big and beautiful~")
	public static NpcStringId MY_MACHINE_IS_BIG_AND_BEAUTIFUL;
	
	@ClientString(id = 1801571, message = "My machine is awesome~")
	public static NpcStringId MY_MACHINE_IS_AWESOME;
	
	@ClientString(id = 1801572, message = "Ooh yah~")
	public static NpcStringId OOH_YAH;
	
	@ClientString(id = 1801573, message = "Hey, wipe that correctly.")
	public static NpcStringId HEY_WIPE_THAT_CORRECTLY;
	
	@ClientString(id = 1801574, message = "Come here and give me a massage. Nanim is still working and is probably tired.")
	public static NpcStringId COME_HERE_AND_GIVE_ME_A_MASSAGE_NANIM_IS_STILL_WORKING_AND_IS_PROBABLY_TIRED;
	
	@ClientString(id = 1801575, message = "A caterpillar should eat pine needles, right? Kekeke.")
	public static NpcStringId A_CATERPILLAR_SHOULD_EAT_PINE_NEEDLES_RIGHT_KEKEKE;
	
	@ClientString(id = 1801576, message = "5 second(s) to go!")
	public static NpcStringId SECOND_S_TO_GO;
	
	@ClientString(id = 1801577, message = "4 second(s) to go!")
	public static NpcStringId SECOND_S_TO_GO2;
	
	@ClientString(id = 1801578, message = "3 second(s) to go!")
	public static NpcStringId SECOND_S_TO_GO3;
	
	@ClientString(id = 1801579, message = "2 second(s) to go!")
	public static NpcStringId SECOND_S_TO_GO4;
	
	@ClientString(id = 1801580, message = "1 second(s) to go!")
	public static NpcStringId SECOND_S_TO_GO5;
	
	@ClientString(id = 1801581, message = "Let's rest for awhile.")
	public static NpcStringId LET_S_REST_FOR_AWHILE;
	
	@ClientString(id = 1801582, message = "The Chaos Shield is shattered and the true form is revealed!")
	public static NpcStringId THE_CHAOS_SHIELD_IS_SHATTERED_AND_THE_TRUE_FORM_IS_REVEALED;
	
	@ClientString(id = 1801583, message = "Don't come this way!")
	public static NpcStringId DON_T_COME_THIS_WAY;
	
	@ClientString(id = 1801584, message = "This rat-like world!")
	public static NpcStringId THIS_RAT_LIKE_WORLD;
	
	@ClientString(id = 1801585, message = "What a big rat. Today will be a feast.")
	public static NpcStringId WHAT_A_BIG_RAT_TODAY_WILL_BE_A_FEAST;
	
	@ClientString(id = 1801586, message = "It sure is nice to be free from having to go to the bathroom.")
	public static NpcStringId IT_SURE_IS_NICE_TO_BE_FREE_FROM_HAVING_TO_GO_TO_THE_BATHROOM;
	
	@ClientString(id = 1801587, message = "Hoo~ Haa~ Hoo~ Haa~")
	public static NpcStringId HOO_HAA_HOO_HAA;
	
	@ClientString(id = 1801588, message = "Why did I come here? This lot!")
	public static NpcStringId WHY_DID_I_COME_HERE_THIS_LOT;
	
	@ClientString(id = 1801589, message = "This is all Tauti's fault!")
	public static NpcStringId THIS_IS_ALL_TAUTI_S_FAULT;
	
	@ClientString(id = 1801590, message = "What a worthwhile day!.")
	public static NpcStringId WHAT_A_WORTHWHILE_DAY;
	
	@ClientString(id = 1801591, message = "Today was terrible. I only caught 3 rats.")
	public static NpcStringId TODAY_WAS_TERRIBLE_I_ONLY_CAUGHT_3_RATS;
	
	@ClientString(id = 1801592, message = "I'm hope to be an Engineer someday.")
	public static NpcStringId I_M_HOPE_TO_BE_AN_ENGINEER_SOMEDAY;
	
	@ClientString(id = 1801593, message = "Today's dinner is a huge rat roast.")
	public static NpcStringId TODAY_S_DINNER_IS_A_HUGE_RAT_ROAST;
	
	@ClientString(id = 1801594, message = "Get it right, or you're done! There's plenty Zofans that want this work.")
	public static NpcStringId GET_IT_RIGHT_OR_YOU_RE_DONE_THERE_S_PLENTY_ZOFANS_THAT_WANT_THIS_WORK;
	
	@ClientString(id = 1801595, message = "Upward mobility? Don't make me laugh.")
	public static NpcStringId UPWARD_MOBILITY_DON_T_MAKE_ME_LAUGH;
	
	@ClientString(id = 1801596, message = "Well done. That rat body is your reward. Hahaha.")
	public static NpcStringId WELL_DONE_THAT_RAT_BODY_IS_YOUR_REWARD_HAHAHA;
	
	@ClientString(id = 1801597, message = "If you don't do better today, that's it for you!")
	public static NpcStringId IF_YOU_DON_T_DO_BETTER_TODAY_THAT_S_IT_FOR_YOU;
	
	@ClientString(id = 1801598, message = "What a nuisance you worthless creatures are!")
	public static NpcStringId WHAT_A_NUISANCE_YOU_WORTHLESS_CREATURES_ARE;
	
	@ClientString(id = 1801599, message = "Everyone die!")
	public static NpcStringId EVERYONE_DIE;
	
	@ClientString(id = 1801600, message = "For Tauti!")
	public static NpcStringId FOR_TAUTI;
	
	@ClientString(id = 1801601, message = "Fools. I will make this a habitat for the rats. They shall feed on your body!")
	public static NpcStringId FOOLS_I_WILL_MAKE_THIS_A_HABITAT_FOR_THE_RATS_THEY_SHALL_FEED_ON_YOUR_BODY;
	
	@ClientString(id = 1801602, message = "How dare you insects attack me?")
	public static NpcStringId HOW_DARE_YOU_INSECTS_ATTACK_ME;
	
	@ClientString(id = 1801603, message = "You rat-like creatures!")
	public static NpcStringId YOU_RAT_LIKE_CREATURES;
	
	@ClientString(id = 1801604, message = "Today is a feasting day for my weapon -- on your Petras!")
	public static NpcStringId TODAY_IS_A_FEASTING_DAY_FOR_MY_WEAPON_ON_YOUR_PETRAS;
	
	@ClientString(id = 1801605, message = "Hahahaha hahahaha puny insects!")
	public static NpcStringId HAHAHAHA_HAHAHAHA_PUNY_INSECTS;
	
	@ClientString(id = 1801606, message = "Everyone die!")
	public static NpcStringId EVERYONE_DIE2;
	
	@ClientString(id = 1801607, message = "For Tauti!")
	public static NpcStringId FOR_TAUTI2;
	
	@ClientString(id = 1801608, message = "I will punish you in the name of Tauti! The crime is stealing. The punishment is death!")
	public static NpcStringId I_WILL_PUNISH_YOU_IN_THE_NAME_OF_TAUTI_THE_CRIME_IS_STEALING_THE_PUNISHMENT_IS_DEATH;
	
	@ClientString(id = 1801609, message = "Fight for the sake of our future!")
	public static NpcStringId FIGHT_FOR_THE_SAKE_OF_OUR_FUTURE;
	
	@ClientString(id = 1801610, message = "Even rats struggle when you step on them!")
	public static NpcStringId EVEN_RATS_STRUGGLE_WHEN_YOU_STEP_ON_THEM;
	
	@ClientString(id = 1801611, message = "For our friends and family!")
	public static NpcStringId FOR_OUR_FRIENDS_AND_FAMILY;
	
	@ClientString(id = 1801612, message = "You worthless Kundas! My weapon may not be the bst, but it's still enough to cut your heads off!")
	public static NpcStringId YOU_WORTHLESS_KUNDAS_MY_WEAPON_MAY_NOT_BE_THE_BST_BUT_IT_S_STILL_ENOUGH_TO_CUT_YOUR_HEADS_OFF;
	
	@ClientString(id = 1801613, message = "Give me freedom or give me death!")
	public static NpcStringId GIVE_ME_FREEDOM_OR_GIVE_ME_DEATH;
	
	@ClientString(id = 1801614, message = "Us! Today! Here! We shall write new history by defeating Tauti! For freedom and happiness!")
	public static NpcStringId US_TODAY_HERE_WE_SHALL_WRITE_NEW_HISTORY_BY_DEFEATING_TAUTI_FOR_FREEDOM_AND_HAPPINESS;
	
	@ClientString(id = 1801615, message = "We are not your pets or cattle!")
	public static NpcStringId WE_ARE_NOT_YOUR_PETS_OR_CATTLE;
	
	@ClientString(id = 1801616, message = "You will die!     And I will live.")
	public static NpcStringId YOU_WILL_DIE_AND_I_WILL_LIVE;
	
	@ClientString(id = 1801617, message = "We cannot forgive Tauti for feeding on us anymore!")
	public static NpcStringId WE_CANNOT_FORGIVE_TAUTI_FOR_FEEDING_ON_US_ANYMORE;
	
	@ClientString(id = 1801618, message = "Fight for the sake of our future!")
	public static NpcStringId FIGHT_FOR_THE_SAKE_OF_OUR_FUTURE2;
	
	@ClientString(id = 1801619, message = "If we all fall here, our plan will certainly fail. Please protect my friends.")
	public static NpcStringId IF_WE_ALL_FALL_HERE_OUR_PLAN_WILL_CERTAINLY_FAIL_PLEASE_PROTECT_MY_FRIENDS;
	
	@ClientString(id = 1801620, message = "Kill that $s1 over there first. He is too ugly.")
	public static NpcStringId KILL_THAT_S1_OVER_THERE_FIRST_HE_IS_TOO_UGLY;
	
	@ClientString(id = 1801621, message = "Flood in, my followers. I grant you permission.")
	public static NpcStringId FLOOD_IN_MY_FOLLOWERS_I_GRANT_YOU_PERMISSION;
	
	@ClientString(id = 1801622, message = "Do you need my help. Here it is.")
	public static NpcStringId DO_YOU_NEED_MY_HELP_HERE_IT_IS;
	
	@ClientString(id = 1801623, message = "The plan has failed. Everyone, retreat!")
	public static NpcStringId THE_PLAN_HAS_FAILED_EVERYONE_RETREAT;
	
	@ClientString(id = 1801624, message = "Warning! Warning! Overheating has begun!")
	public static NpcStringId WARNING_WARNING_OVERHEATING_HAS_BEGUN;
	
	@ClientString(id = 1801625, message = "Warning! Warning! Coolant is leaking!")
	public static NpcStringId WARNING_WARNING_COOLANT_IS_LEAKING;
	
	@ClientString(id = 1801626, message = "Current mode is reverting to self recovery mode!")
	public static NpcStringId CURRENT_MODE_IS_REVERTING_TO_SELF_RECOVERY_MODE;
	
	@ClientString(id = 1801627, message = "Current mode is reverting to self defense mode!")
	public static NpcStringId CURRENT_MODE_IS_REVERTING_TO_SELF_DEFENSE_MODE;
	
	@ClientString(id = 1801628, message = "If you hit me, I might break! So watch it~")
	public static NpcStringId IF_YOU_HIT_ME_I_MIGHT_BREAK_SO_WATCH_IT;
	
	@ClientString(id = 1801629, message = "Raise the egg temperature~")
	public static NpcStringId RAISE_THE_EGG_TEMPERATURE;
	
	@ClientString(id = 1801630, message = "Too hot!!")
	public static NpcStringId TOO_HOT;
	
	@ClientString(id = 1801631, message = "If you make it too hot, it can break~ So watch out~")
	public static NpcStringId IF_YOU_MAKE_IT_TOO_HOT_IT_CAN_BREAK_SO_WATCH_OUT;
	
	@ClientString(id = 1801632, message = "Do you know how stuffy it is in this egg? Get me out~")
	public static NpcStringId DO_YOU_KNOW_HOW_STUFFY_IT_IS_IN_THIS_EGG_GET_ME_OUT;
	
	@ClientString(id = 1801633, message = "If you incubate me, I will give you 10,000,000 Adena.")
	public static NpcStringId IF_YOU_INCUBATE_ME_I_WILL_GIVE_YOU_10_000_000_ADENA;
	
	@ClientString(id = 1801634, message = "Oh! I've been incubated... Incubated!!")
	public static NpcStringId OH_I_VE_BEEN_INCUBATED_INCUBATED;
	
	@ClientString(id = 1801635, message = "The world is so big and beautiful. I love this beautiful world.")
	public static NpcStringId THE_WORLD_IS_SO_BIG_AND_BEAUTIFUL_I_LOVE_THIS_BEAUTIFUL_WORLD;
	
	@ClientString(id = 1801636, message = "I will make myself pretty for you!")
	public static NpcStringId I_WILL_MAKE_MYSELF_PRETTY_FOR_YOU;
	
	@ClientString(id = 1801637, message = "Mommy, mommy, mommy! Huh? Daddy...?")
	public static NpcStringId MOMMY_MOMMY_MOMMY_HUH_DADDY;
	
	@ClientString(id = 1801638, message = "Boing boing! Boing boing!")
	public static NpcStringId BOING_BOING_BOING_BOING;
	
	@ClientString(id = 1801639, message = "Aren't I cute?!")
	public static NpcStringId AREN_T_I_CUTE;
	
	@ClientString(id = 1801640, message = "Incubating is 1 percent luck and 99 percent effort~")
	public static NpcStringId INCUBATING_IS_1_PERCENT_LUCK_AND_99_PERCENT_EFFORT;
	
	@ClientString(id = 1801641, message = "Director! I want to be incubated.")
	public static NpcStringId DIRECTOR_I_WANT_TO_BE_INCUBATED;
	
	@ClientString(id = 1801642, message = "Thank you. You raised it a lot.")
	public static NpcStringId THANK_YOU_YOU_RAISED_IT_A_LOT;
	
	@ClientString(id = 1801643, message = "If you incubate properly, there won't be any bloodshed.")
	public static NpcStringId IF_YOU_INCUBATE_PROPERLY_THERE_WON_T_BE_ANY_BLOODSHED;
	
	@ClientString(id = 1801644, message = "You are the clan member who produced $s1, this week's winner. You have tremendous abilities and luck. I give you my master's blessing...!")
	public static NpcStringId YOU_ARE_THE_CLAN_MEMBER_WHO_PRODUCED_S1_THIS_WEEK_S_WINNER_YOU_HAVE_TREMENDOUS_ABILITIES_AND_LUCK_I_GIVE_YOU_MY_MASTER_S_BLESSING;
	
	@ClientString(id = 1801645, message = "The clan that seized the most honor this week is $s1.")
	public static NpcStringId THE_CLAN_THAT_SEIZED_THE_MOST_HONOR_THIS_WEEK_IS_S1;
	
	@ClientString(id = 1801646, message = "Is there no clan that can raise a true ultimate warrior...?")
	public static NpcStringId IS_THERE_NO_CLAN_THAT_CAN_RAISE_A_TRUE_ULTIMATE_WARRIOR;
	
	@ClientString(id = 1801647, message = "$s1, the Proof of Survival was acquired.")
	public static NpcStringId S1_THE_PROOF_OF_SURVIVAL_WAS_ACQUIRED;
	
	@ClientString(id = 1801648, message = "The Proof of Survival is disappearing.")
	public static NpcStringId THE_PROOF_OF_SURVIVAL_IS_DISAPPEARING;
	
	@ClientString(id = 1801649, message = "Zahak is infusing its Petra to Tauti.")
	public static NpcStringId ZAHAK_IS_INFUSING_ITS_PETRA_TO_TAUTI;
	
	@ClientString(id = 1801650, message = "Lord Tauti, receive my Petra and be strengthened. Then, defeat these feeble wretches!")
	public static NpcStringId LORD_TAUTI_RECEIVE_MY_PETRA_AND_BE_STRENGTHENED_THEN_DEFEAT_THESE_FEEBLE_WRETCHES;
	
	@ClientString(id = 1801651, message = "10 second(s) before the end!")
	public static NpcStringId SECOND_S_BEFORE_THE_END;
	
	@ClientString(id = 1801652, message = "5 second(s) before the end!")
	public static NpcStringId SECOND_S_BEFORE_THE_END2;
	
	@ClientString(id = 1801653, message = "4 second(s) before the end!")
	public static NpcStringId SECOND_S_BEFORE_THE_END3;
	
	@ClientString(id = 1801654, message = "3 second(s) before the end!")
	public static NpcStringId SECOND_S_BEFORE_THE_END4;
	
	@ClientString(id = 1801655, message = "2 second(s) before the end!")
	public static NpcStringId SECOND_S_BEFORE_THE_END5;
	
	@ClientString(id = 1801656, message = "1 second(s) before the end!")
	public static NpcStringId SECOND_S_BEFORE_THE_END6;
	
	@ClientString(id = 1801657, message = "If you don't want to be obliterated, don't provoke me...!!!")
	public static NpcStringId IF_YOU_DON_T_WANT_TO_BE_OBLITERATED_DON_T_PROVOKE_ME;
	
	@ClientString(id = 1801658, message = "Don't you insects know how to fear a lion?!")
	public static NpcStringId DON_T_YOU_INSECTS_KNOW_HOW_TO_FEAR_A_LION;
	
	@ClientString(id = 1801659, message = "If you hear a scream?! Then, life has ended! Hahaha!")
	public static NpcStringId IF_YOU_HEAR_A_SCREAM_THEN_LIFE_HAS_ENDED_HAHAHA;
	
	@ClientString(id = 1801660, message = "How do you like the view? You are that much closer to the edge!")
	public static NpcStringId HOW_DO_YOU_LIKE_THE_VIEW_YOU_ARE_THAT_MUCH_CLOSER_TO_THE_EDGE;
	
	@ClientString(id = 1801661, message = "Do you know why I am called the Guillotine...? It's because I will slice you like a sharp blade!")
	public static NpcStringId DO_YOU_KNOW_WHY_I_AM_CALLED_THE_GUILLOTINE_IT_S_BECAUSE_I_WILL_SLICE_YOU_LIKE_A_SHARP_BLADE;
	
	@ClientString(id = 1801662, message = "You will die!")
	public static NpcStringId YOU_WILL_DIE2;
	
	@ClientString(id = 1801663, message = "I will fill those who remember me with nightmares and spite...!")
	public static NpcStringId I_WILL_FILL_THOSE_WHO_REMEMBER_ME_WITH_NIGHTMARES_AND_SPITE;
	
	@ClientString(id = 1801664, message = "If you want to play some more, you have to match my beat, right?")
	public static NpcStringId IF_YOU_WANT_TO_PLAY_SOME_MORE_YOU_HAVE_TO_MATCH_MY_BEAT_RIGHT;
	
	@ClientString(id = 1801665, message = "Now, this may hurt a bit... I've finished my warm-up... Kukuku...")
	public static NpcStringId NOW_THIS_MAY_HURT_A_BIT_I_VE_FINISHED_MY_WARM_UP_KUKUKU;
	
	@ClientString(id = 1801666, message = "Come on!! Enjoy it!! Have fun!! To the afterlife!! Kuhaha!!")
	public static NpcStringId COME_ON_ENJOY_IT_HAVE_FUN_TO_THE_AFTERLIFE_KUHAHA;
	
	@ClientString(id = 1801667, message = "It's time for an untimely demise...!")
	public static NpcStringId IT_S_TIME_FOR_AN_UNTIMELY_DEMISE;
	
	@ClientString(id = 1801668, message = "Hmm~~ It still doesn't feel just right...!")
	public static NpcStringId HMM_IT_STILL_DOESN_T_FEEL_JUST_RIGHT;
	
	@ClientString(id = 1801669, message = "Hey, you! Kuhaha! Does it hurt? Kuhahahaha!")
	public static NpcStringId HEY_YOU_KUHAHA_DOES_IT_HURT_KUHAHAHAHA;
	
	@ClientString(id = 1801670, message = "Now~ Are you coming or going? Cute little things...")
	public static NpcStringId NOW_ARE_YOU_COMING_OR_GOING_CUTE_LITTLE_THINGS;
	
	@ClientString(id = 1801671, message = "Amuse me more. Oh, that's right! Kukuku...")
	public static NpcStringId AMUSE_ME_MORE_OH_THAT_S_RIGHT_KUKUKU;
	
	@ClientString(id = 1801672, message = "Are you hitting? Tell me when you're done. I'm going to take a nap! Kuhaha.")
	public static NpcStringId ARE_YOU_HITTING_TELL_ME_WHEN_YOU_RE_DONE_I_M_GOING_TO_TAKE_A_NAP_KUHAHA;
	
	@ClientString(id = 1801673, message = "Bong! Bong! Bong! That feels refreshing.")
	public static NpcStringId BONG_BONG_BONG_THAT_FEELS_REFRESHING;
	
	@ClientString(id = 1801674, message = "How stubborn you are, even after all those cuts...")
	public static NpcStringId HOW_STUBBORN_YOU_ARE_EVEN_AFTER_ALL_THOSE_CUTS;
	
	@ClientString(id = 1801675, message = "How about now? Now?!")
	public static NpcStringId HOW_ABOUT_NOW_NOW;
	
	@ClientString(id = 1801676, message = "It feels alright looking down on you from here.")
	public static NpcStringId IT_FEELS_ALRIGHT_LOOKING_DOWN_ON_YOU_FROM_HERE;
	
	@ClientString(id = 1801677, message = "Blood! Blood is dirtying my castle!")
	public static NpcStringId BLOOD_BLOOD_IS_DIRTYING_MY_CASTLE;
	
	@ClientString(id = 1801678, message = "Hyah...! Yaaah!!")
	public static NpcStringId HYAH_YAAAH;
	
	@ClientString(id = 1801679, message = "That was enjoyable...! I don't want any more noise in my fortress!")
	public static NpcStringId THAT_WAS_ENJOYABLE_I_DON_T_WANT_ANY_MORE_NOISE_IN_MY_FORTRESS;
	
	@ClientString(id = 1801680, message = "Use something new! Huh? Something I haven't seen before! Something that hurts! Kuhaha.")
	public static NpcStringId USE_SOMETHING_NEW_HUH_SOMETHING_I_HAVEN_T_SEEN_BEFORE_SOMETHING_THAT_HURTS_KUHAHA;
	
	@ClientString(id = 1801681, message = "I lost... I acknowledge it. My defeat...")
	public static NpcStringId I_LOST_I_ACKNOWLEDGE_IT_MY_DEFEAT;
	
	@ClientString(id = 1801682, message = "My blood that has spilled in this fortress shall raise me up again...! Kuuk...")
	public static NpcStringId MY_BLOOD_THAT_HAS_SPILLED_IN_THIS_FORTRESS_SHALL_RAISE_ME_UP_AGAIN_KUUK;
	
	@ClientString(id = 1801683, message = "I accept defeat...!")
	public static NpcStringId I_ACCEPT_DEFEAT;
	
	@ClientString(id = 1801684, message = "I acknowledge your tenacity...")
	public static NpcStringId I_ACKNOWLEDGE_YOUR_TENACITY;
	
	@ClientString(id = 1801685, message = "I've lost...!")
	public static NpcStringId I_VE_LOST;
	
	@ClientString(id = 1801686, message = "Kuku... Let me introduce you to my servants. Come forth, Calibus!!")
	public static NpcStringId KUKU_LET_ME_INTRODUCE_YOU_TO_MY_SERVANTS_COME_FORTH_CALIBUS;
	
	@ClientString(id = 1801687, message = "Come here, Tempest! You worthless creature!")
	public static NpcStringId COME_HERE_TEMPEST_YOU_WORTHLESS_CREATURE;
	
	@ClientString(id = 1801688, message = "Kukukuku...!! You're the healer, right? Kuhahaha!!!")
	public static NpcStringId KUKUKUKU_YOU_RE_THE_HEALER_RIGHT_KUHAHAHA;
	
	@ClientString(id = 1801689, message = "Calibus! This wretch is the healer! You shall only attack this one!! Kuhahaha!!!")
	public static NpcStringId CALIBUS_THIS_WRETCH_IS_THE_HEALER_YOU_SHALL_ONLY_ATTACK_THIS_ONE_KUHAHAHA;
	
	@ClientString(id = 1801690, message = "I hear and obey, oh, Guillotine!!")
	public static NpcStringId I_HEAR_AND_OBEY_OH_GUILLOTINE;
	
	@ClientString(id = 1801691, message = "Sir! I will destroy the healer as you command!!")
	public static NpcStringId SIR_I_WILL_DESTROY_THE_HEALER_AS_YOU_COMMAND;
	
	@ClientString(id = 1801692, message = "Keuk...! Acheron... Hurry...")
	public static NpcStringId KEUK_ACHERON_HURRY;
	
	@ClientString(id = 1801693, message = "Hmm... Calibus, that fool...")
	public static NpcStringId HMM_CALIBUS_THAT_FOOL;
	
	@ClientString(id = 1801694, message = "Only I am the true right arm of the great Guillotine!!")
	public static NpcStringId ONLY_I_AM_THE_TRUE_RIGHT_ARM_OF_THE_GREAT_GUILLOTINE;
	
	@ClientString(id = 1801695, message = "Will you... Remember my faithfulness?!")
	public static NpcStringId WILL_YOU_REMEMBER_MY_FAITHFULNESS;
	
	@ClientString(id = 1801696, message = "Calibus... They were useless even in the past...")
	public static NpcStringId CALIBUS_THEY_WERE_USELESS_EVEN_IN_THE_PAST;
	
	@ClientString(id = 1801697, message = "Urgh! Argh..!!")
	public static NpcStringId URGH_ARGH;
	
	@ClientString(id = 1801698, message = "Uuurrghh!")
	public static NpcStringId UUURRGHH;
	
	@ClientString(id = 1801699, message = "Ooohh!!")
	public static NpcStringId OOOHH;
	
	@ClientString(id = 1801700, message = "Oooh...")
	public static NpcStringId OOOH2;
	
	@ClientString(id = 1801701, message = "Keuk! Gulp...")
	public static NpcStringId KEUK_GULP;
	
	@ClientString(id = 1801702, message = "Keuu... Keuu...")
	public static NpcStringId KEUU_KEUU;
	
	@ClientString(id = 1801703, message = "Keep up! Or you will die!")
	public static NpcStringId KEEP_UP_OR_YOU_WILL_DIE;
	
	@ClientString(id = 1801704, message = "Now! I am tired of cleaning up the trash...!")
	public static NpcStringId NOW_I_AM_TIRED_OF_CLEANING_UP_THE_TRASH;
	
	@ClientString(id = 1801705, message = "Impure fools are making a ruckus in the fortress...!")
	public static NpcStringId IMPURE_FOOLS_ARE_MAKING_A_RUCKUS_IN_THE_FORTRESS;
	
	@ClientString(id = 1801706, message = "I will keep your fingers as my trophies...!")
	public static NpcStringId I_WILL_KEEP_YOUR_FINGERS_AS_MY_TROPHIES;
	
	@ClientString(id = 1801707, message = "This powerful forearm will give you suffering!")
	public static NpcStringId THIS_POWERFUL_FOREARM_WILL_GIVE_YOU_SUFFERING;
	
	@ClientString(id = 1801708, message = "Hyah! Hyah! Another chilled night...")
	public static NpcStringId HYAH_HYAH_ANOTHER_CHILLED_NIGHT;
	
	@ClientString(id = 1801709, message = "Become! There are enough of us here...!")
	public static NpcStringId BECOME_THERE_ARE_ENOUGH_OF_US_HERE;
	
	@ClientString(id = 1801710, message = "My heart finds rest at night...!")
	public static NpcStringId MY_HEART_FINDS_REST_AT_NIGHT;
	
	@ClientString(id = 1801711, message = "Don't come any closer.")
	public static NpcStringId DON_T_COME_ANY_CLOSER;
	
	@ClientString(id = 1801712, message = "Fools deserve the ultimate punishment.")
	public static NpcStringId FOOLS_DESERVE_THE_ULTIMATE_PUNISHMENT;
	
	@ClientString(id = 1801713, message = "Why are you here when you should be asleep and dreaming...?")
	public static NpcStringId WHY_ARE_YOU_HERE_WHEN_YOU_SHOULD_BE_ASLEEP_AND_DREAMING;
	
	@ClientString(id = 1801714, message = "A hectic battlefield... Lives are disappearing even now...!")
	public static NpcStringId A_HECTIC_BATTLEFIELD_LIVES_ARE_DISAPPEARING_EVEN_NOW;
	
	@ClientString(id = 1801715, message = "My eyes burn brighter in the darkness...!")
	public static NpcStringId MY_EYES_BURN_BRIGHTER_IN_THE_DARKNESS;
	
	@ClientString(id = 1801716, message = "If you try to harm me, I won't stand for it...!")
	public static NpcStringId IF_YOU_TRY_TO_HARM_ME_I_WON_T_STAND_FOR_IT;
	
	@ClientString(id = 1801717, message = "Lives don't last long in the fortress...")
	public static NpcStringId LIVES_DON_T_LAST_LONG_IN_THE_FORTRESS;
	
	@ClientString(id = 1801718, message = "I will grant you the ultimate punishment...!")
	public static NpcStringId I_WILL_GRANT_YOU_THE_ULTIMATE_PUNISHMENT;
	
	@ClientString(id = 1801719, message = "Kyaaak!!")
	public static NpcStringId KYAAAK2;
	
	@ClientString(id = 1801720, message = "Ah ah ah uh uh...!")
	public static NpcStringId AH_AH_AH_UH_UH2;
	
	@ClientString(id = 1801721, message = "Huh? Again? Why??")
	public static NpcStringId HUH_AGAIN_WHY;
	
	@ClientString(id = 1801722, message = "Bang~ Bang bang~ You know... Why bang~ Huh?")
	public static NpcStringId BANG_BANG_BANG_YOU_KNOW_WHY_BANG_HUH;
	
	@ClientString(id = 1801723, message = "Are you here to harass us again? Huh?")
	public static NpcStringId ARE_YOU_HERE_TO_HARASS_US_AGAIN_HUH;
	
	@ClientString(id = 1801724, message = "Hello... Huhuhuhu...")
	public static NpcStringId HELLO_HUHUHUHU;
	
	@ClientString(id = 1801725, message = "You know, so do it.")
	public static NpcStringId YOU_KNOW_SO_DO_IT;
	
	@ClientString(id = 1801726, message = "So... Kuhuhuhuhuhu!!!")
	public static NpcStringId SO_KUHUHUHUHUHU;
	
	@ClientString(id = 1801727, message = "So, I'm saying...")
	public static NpcStringId SO_I_M_SAYING;
	
	@ClientString(id = 1801728, message = "What???? No way!")
	public static NpcStringId WHAT_NO_WAY;
	
	@ClientString(id = 1801729, message = "But still, we have no choice...")
	public static NpcStringId BUT_STILL_WE_HAVE_NO_CHOICE;
	
	@ClientString(id = 1801730, message = "I'm famous around here...!")
	public static NpcStringId I_M_FAMOUS_AROUND_HERE;
	
	@ClientString(id = 1801731, message = "Why? Seriously, I'm not crazy! I'm fine!!")
	public static NpcStringId WHY_SERIOUSLY_I_M_NOT_CRAZY_I_M_FINE;
	
	@ClientString(id = 1801732, message = "Come! Don't come! Go! No, come! Acckkk!")
	public static NpcStringId COME_DON_T_COME_GO_NO_COME_ACCKKK;
	
	@ClientString(id = 1801733, message = "Hit me! No, go away! It hurts! Kyahh!")
	public static NpcStringId HIT_ME_NO_GO_AWAY_IT_HURTS_KYAHH;
	
	@ClientString(id = 1801734, message = "Patrolling in progress. Nothing to report!")
	public static NpcStringId PATROLLING_IN_PROGRESS_NOTHING_TO_REPORT;
	
	@ClientString(id = 1801735, message = "Watching in progress. Nothing to report!")
	public static NpcStringId WATCHING_IN_PROGRESS_NOTHING_TO_REPORT;
	
	@ClientString(id = 1801736, message = "The enemy! I will stab them!")
	public static NpcStringId THE_ENEMY_I_WILL_STAB_THEM;
	
	@ClientString(id = 1801737, message = "Intruders in the fortress!")
	public static NpcStringId INTRUDERS_IN_THE_FORTRESS;
	
	@ClientString(id = 1801738, message = "Instant attack when discovered!")
	public static NpcStringId INSTANT_ATTACK_WHEN_DISCOVERED;
	
	@ClientString(id = 1801739, message = "Who are you?!")
	public static NpcStringId WHO_ARE_YOU;
	
	@ClientString(id = 1801740, message = "How dare you come here!!")
	public static NpcStringId HOW_DARE_YOU_COME_HERE;
	
	@ClientString(id = 1801741, message = "Hmm... My body is too heavy to play now...")
	public static NpcStringId HMM_MY_BODY_IS_TOO_HEAVY_TO_PLAY_NOW;
	
	@ClientString(id = 1801742, message = "I am Guillotine. I didn't become the boss around here by accident...!")
	public static NpcStringId I_AM_GUILLOTINE_I_DIDN_T_BECOME_THE_BOSS_AROUND_HERE_BY_ACCIDENT;
	
	@ClientString(id = 1801743, message = "Kekeke... I'm so happy that you can play awhile...")
	public static NpcStringId KEKEKE_I_M_SO_HAPPY_THAT_YOU_CAN_PLAY_AWHILE;
	
	@ClientString(id = 1801744, message = "You fools... It was fun until now...")
	public static NpcStringId YOU_FOOLS_IT_WAS_FUN_UNTIL_NOW;
	
	@ClientString(id = 1801745, message = "It's time for an enjoyable execution.")
	public static NpcStringId IT_S_TIME_FOR_AN_ENJOYABLE_EXECUTION;
	
	@ClientString(id = 1801746, message = "Our powers are on opposite poles!")
	public static NpcStringId OUR_POWERS_ARE_ON_OPPOSITE_POLES;
	
	@ClientString(id = 1801747, message = "Kuhahahahaha!!!")
	public static NpcStringId KUHAHAHAHAHA;
	
	@ClientString(id = 1801748, message = "Hyah!")
	public static NpcStringId HYAH;
	
	@ClientString(id = 1801749, message = "I will stamp you out!")
	public static NpcStringId I_WILL_STAMP_YOU_OUT;
	
	@ClientString(id = 1801750, message = "I will hurl all of you away!")
	public static NpcStringId I_WILL_HURL_ALL_OF_YOU_AWAY;
	
	@ClientString(id = 1801751, message = "This time, I will hurl away you rat-like creatures who are attacking from afar!")
	public static NpcStringId THIS_TIME_I_WILL_HURL_AWAY_YOU_RAT_LIKE_CREATURES_WHO_ARE_ATTACKING_FROM_AFAR;
	
	@ClientString(id = 1801752, message = "The monster dropped a Proof of Survival.")
	public static NpcStringId THE_MONSTER_DROPPED_A_PROOF_OF_SURVIVAL;
	
	@ClientString(id = 1801753, message = "$s1, shield breakthrough with a powerful half-kill attack!!")
	public static NpcStringId S1_SHIELD_BREAKTHROUGH_WITH_A_POWERFUL_HALF_KILL_ATTACK;
	
	@ClientString(id = 1801754, message = "Incubate me~ Please?")
	public static NpcStringId INCUBATE_ME_PLEASE;
	
	@ClientString(id = 1801755, message = "You and me~ I can't now~")
	public static NpcStringId YOU_AND_ME_I_CAN_T_NOW;
	
	@ClientString(id = 1801756, message = "When Daddy goes to work, bless~ When Mommy hugs me, bless~")
	public static NpcStringId WHEN_DADDY_GOES_TO_WORK_BLESS_WHEN_MOMMY_HUGS_ME_BLESS;
	
	@ClientString(id = 1801757, message = "Oh, too hot.")
	public static NpcStringId OH_TOO_HOT;
	
	@ClientString(id = 1801758, message = "Hot, hot. Too hot.")
	public static NpcStringId HOT_HOT_TOO_HOT;
	
	@ClientString(id = 1801759, message = "I'm a lonely Draco~ A cute baby dinosaur~")
	public static NpcStringId I_M_A_LONELY_DRACO_A_CUTE_BABY_DINOSAUR;
	
	@ClientString(id = 1801760, message = "I want out right now~")
	public static NpcStringId I_WANT_OUT_RIGHT_NOW;
	
	@ClientString(id = 1801761, message = "No~! You can't incubate yet~")
	public static NpcStringId NO_YOU_CAN_T_INCUBATE_YET;
	
	@ClientString(id = 1801762, message = "I love you, I love you, I love you...")
	public static NpcStringId I_LOVE_YOU_I_LOVE_YOU_I_LOVE_YOU;
	
	@ClientString(id = 1801763, message = "Dinosaurs are the future.")
	public static NpcStringId DINOSAURS_ARE_THE_FUTURE;
	
	@ClientString(id = 1801764, message = "The whole world awaits Draco~")
	public static NpcStringId THE_WHOLE_WORLD_AWAITS_DRACO;
	
	@ClientString(id = 1801765, message = "Incubation prep! One, two! One, two!")
	public static NpcStringId INCUBATION_PREP_ONE_TWO_ONE_TWO;
	
	@ClientString(id = 1801766, message = "I'm not an egg. I'm Draco.")
	public static NpcStringId I_M_NOT_AN_EGG_I_M_DRACO;
	
	@ClientString(id = 1801767, message = "My incubation score will be made public in 60 seconds.")
	public static NpcStringId MY_INCUBATION_SCORE_WILL_BE_MADE_PUBLIC_IN_60_SECONDS;
	
	@ClientString(id = 1801768, message = "Dra! Dra-dra!")
	public static NpcStringId DRA_DRA_DRA;
	
	@ClientString(id = 1801769, message = "You're already incubated.")
	public static NpcStringId YOU_RE_ALREADY_INCUBATED;
	
	@ClientString(id = 1801770, message = "Remember every time you see the egg!")
	public static NpcStringId REMEMBER_EVERY_TIME_YOU_SEE_THE_EGG;
	
	@ClientString(id = 1801771, message = "I'm Draco... I never give up...")
	public static NpcStringId I_M_DRACO_I_NEVER_GIVE_UP;
	
	@ClientString(id = 1801772, message = "Incubate me sometime~")
	public static NpcStringId INCUBATE_ME_SOMETIME;
	
	@ClientString(id = 1801773, message = "Chaos Shield breakthrough!!")
	public static NpcStringId CHAOS_SHIELD_BREAKTHROUGH;
	
	@ClientString(id = 1801774, message = "$s1's Proof of Survival disappears and Scaldisect the Furious appears!")
	public static NpcStringId S1_S_PROOF_OF_SURVIVAL_DISAPPEARS_AND_SCALDISECT_THE_FURIOUS_APPEARS;
	
	@ClientString(id = 1801775, message = "Don't make excuses. You'll get hit.")
	public static NpcStringId DON_T_MAKE_EXCUSES_YOU_LL_GET_HIT;
	
	@ClientString(id = 1801776, message = "I shouldn't have left the egg~")
	public static NpcStringId I_SHOULDN_T_HAVE_LEFT_THE_EGG;
	
	@ClientString(id = 1801777, message = "You did a good job with raising the temperature. Thanks~~")
	public static NpcStringId YOU_DID_A_GOOD_JOB_WITH_RAISING_THE_TEMPERATURE_THANKS;
	
	@ClientString(id = 1801778, message = "It's not hard to incubate me~")
	public static NpcStringId IT_S_NOT_HARD_TO_INCUBATE_ME;
	
	@ClientString(id = 1801779, message = "Hey, you! Help me out a little with this incubation.")
	public static NpcStringId HEY_YOU_HELP_ME_OUT_A_LITTLE_WITH_THIS_INCUBATION;
	
	@ClientString(id = 1801780, message = "* Draco's Birth Event in Progress *")
	public static NpcStringId DRACO_S_BIRTH_EVENT_IN_PROGRESS;
	
	@ClientString(id = 1801781, message = "* Reach for Good Luck *")
	public static NpcStringId REACH_FOR_GOOD_LUCK;
	
	@ClientString(id = 1801782, message = "Chaos Shield Weakened")
	public static NpcStringId CHAOS_SHIELD_WEAKENED;
	
	@ClientString(id = 1801783, message = "You rat-like creatures! Taste my attack!")
	public static NpcStringId YOU_RAT_LIKE_CREATURES_TASTE_MY_ATTACK;
	
	@ClientString(id = 1801784, message = "Do you think you are safe outside? Feel my strength!")
	public static NpcStringId DO_YOU_THINK_YOU_ARE_SAFE_OUTSIDE_FEEL_MY_STRENGTH;
	
	@ClientString(id = 1801785, message = "Thank you for saving me.")
	public static NpcStringId THANK_YOU_FOR_SAVING_ME2;
	
	@ClientString(id = 1801786, message = "Stop the Cannon Golem from Firing")
	public static NpcStringId STOP_THE_CANNON_GOLEM_FROM_FIRING;
	
	@ClientString(id = 1801787, message = "Extract Metucella")
	public static NpcStringId EXTRACT_METUCELLA;
	
	@ClientString(id = 1801788, message = "Ha! Not even a trace, hmm?")
	public static NpcStringId HA_NOT_EVEN_A_TRACE_HMM;
	
	@ClientString(id = 1801789, message = "The Relics of the Giants have great exploration value!")
	public static NpcStringId THE_RELICS_OF_THE_GIANTS_HAVE_GREAT_EXPLORATION_VALUE;
	
	@ClientString(id = 1801790, message = "Chaos! How dare he turn this place into this!")
	public static NpcStringId CHAOS_HOW_DARE_HE_TURN_THIS_PLACE_INTO_THIS;
	
	@ClientString(id = 1801791, message = "I just want to finish up the repairs and get out of this dangerous place.")
	public static NpcStringId I_JUST_WANT_TO_FINISH_UP_THE_REPAIRS_AND_GET_OUT_OF_THIS_DANGEROUS_PLACE;
	
	@ClientString(id = 1801792, message = "Attack the Golems and Spiculas! They're the enemy's bioweapons!")
	public static NpcStringId ATTACK_THE_GOLEMS_AND_SPICULAS_THEY_RE_THE_ENEMY_S_BIOWEAPONS;
	
	@ClientString(id = 1801793, message = "Warning. Intruders. Warning. Intruders.")
	public static NpcStringId WARNING_INTRUDERS_WARNING_INTRUDERS;
	
	@ClientString(id = 1801794, message = "Reinforcements requested. Reinforcements requested.")
	public static NpcStringId REINFORCEMENTS_REQUESTED_REINFORCEMENTS_REQUESTED;
	
	@ClientString(id = 1801795, message = "I shall grow stronger with the power of darkness!")
	public static NpcStringId I_SHALL_GROW_STRONGER_WITH_THE_POWER_OF_DARKNESS;
	
	@ClientString(id = 1801796, message = "Stop! Stop! It hurts!")
	public static NpcStringId STOP_STOP_IT_HURTS;
	
	@ClientString(id = 1801797, message = "Cocoon destroyer! Cocoon destroyer!")
	public static NpcStringId COCOON_DESTROYER_COCOON_DESTROYER;
	
	@ClientString(id = 1801798, message = "Ahh, the cocoon is well-roasted!")
	public static NpcStringId AHH_THE_COCOON_IS_WELL_ROASTED;
	
	@ClientString(id = 1801799, message = "Help me now, you miserable slaves!")
	public static NpcStringId HELP_ME_NOW_YOU_MISERABLE_SLAVES;
	
	@ClientString(id = 1801800, message = "Come out, fairy of darkness!")
	public static NpcStringId COME_OUT_FAIRY_OF_DARKNESS;
	
	@ClientString(id = 1801801, message = "Request Progress")
	public static NpcStringId REQUEST_PROGRESS;
	
	@ClientString(id = 1801802, message = "Request Progress")
	public static NpcStringId REQUEST_PROGRESS2;
	
	@ClientString(id = 1801803, message = "Request Progress")
	public static NpcStringId REQUEST_PROGRESS3;
	
	@ClientString(id = 1801804, message = "Let the game begin!")
	public static NpcStringId LET_THE_GAME_BEGIN;
	
	@ClientString(id = 1801805, message = "A Mysterious Dark Knight is here.")
	public static NpcStringId A_MYSTERIOUS_DARK_KNIGHT_IS_HERE;
	
	@ClientString(id = 1801806, message = "Talk to the Mysterious Dark Knight.")
	public static NpcStringId TALK_TO_THE_MYSTERIOUS_DARK_KNIGHT;
	
	@ClientString(id = 1801807, message = "Wow, $s1's damage per second is $s2.")
	public static NpcStringId WOW_S1_S_DAMAGE_PER_SECOND_IS_S2;
	
	@ClientString(id = 1801808, message = "Wow, $s1 and friends' damage per second is $s2.")
	public static NpcStringId WOW_S1_AND_FRIENDS_DAMAGE_PER_SECOND_IS_S2;
	
	@ClientString(id = 1801809, message = "Congratulations! You broke the record!")
	public static NpcStringId CONGRATULATIONS_YOU_BROKE_THE_RECORD;
	
	@ClientString(id = 1801810, message = "Oh no! You have lost the lead on highest damage!")
	public static NpcStringId OH_NO_YOU_HAVE_LOST_THE_LEAD_ON_HIGHEST_DAMAGE;
	
	@ClientString(id = 1801811, message = "Infected")
	public static NpcStringId INFECTED;
	
	@ClientString(id = 1801812, message = "I will follow you, Master.")
	public static NpcStringId I_WILL_FOLLOW_YOU_MASTER;
	
	@ClientString(id = 1801813, message = "Fire Cannon")
	public static NpcStringId FIRE_CANNON;
	
	@ClientString(id = 1801814, message = "Fire Cannon (Preparing)")
	public static NpcStringId FIRE_CANNON_PREPARING;
	
	@ClientString(id = 1802000, message = "Defensive")
	public static NpcStringId DEFENSIVE;
	
	@ClientString(id = 1802001, message = "Aggressive")
	public static NpcStringId AGGRESSIVE;
	
	@ClientString(id = 1802002, message = "Subordinate Summoner")
	public static NpcStringId SUBORDINATE_SUMMONER;
	
	@ClientString(id = 1802003, message = "Conquerable")
	public static NpcStringId CONQUERABLE;
	
	@ClientString(id = 1802004, message = "Conquered")
	public static NpcStringId CONQUERED;
	
	@ClientString(id = 1802005, message = "The poor slaves are free at last! Good friends, may luck be with you.")
	public static NpcStringId THE_POOR_SLAVES_ARE_FREE_AT_LAST_GOOD_FRIENDS_MAY_LUCK_BE_WITH_YOU;
	
	@ClientString(id = 1802006, message = "The curse must be working on Giselle as well! Hurry, set her free.")
	public static NpcStringId THE_CURSE_MUST_BE_WORKING_ON_GISELLE_AS_WELL_HURRY_SET_HER_FREE;
	
	@ClientString(id = 1802007, message = "Please, grant freedom to these wretched souls.")
	public static NpcStringId PLEASE_GRANT_FREEDOM_TO_THESE_WRETCHED_SOULS;
	
	@ClientString(id = 1802008, message = "Today isn't a lucky day. Stop here for a moment.")
	public static NpcStringId TODAY_ISN_T_A_LUCKY_DAY_STOP_HERE_FOR_A_MOMENT;
	
	@ClientString(id = 1802009, message = "Shame. You are awaited by many, it seems.")
	public static NpcStringId SHAME_YOU_ARE_AWAITED_BY_MANY_IT_SEEMS;
	
	@ClientString(id = 1802010, message = "You unfortunate souls…I can sense your hearts growing heavier as you head her way.")
	public static NpcStringId YOU_UNFORTUNATE_SOULS_I_CAN_SENSE_YOUR_HEARTS_GROWING_HEAVIER_AS_YOU_HEAD_HER_WAY;
	
	@ClientString(id = 1802011, message = "The Vampire Adepts outside the Outer Castle Gates have all fallen.")
	public static NpcStringId THE_VAMPIRE_ADEPTS_OUTSIDE_THE_OUTER_CASTLE_GATES_HAVE_ALL_FALLEN;
	
	@ClientString(id = 1802012, message = "Giselle von Hellmann's HP has decreased.")
	public static NpcStringId GISELLE_VON_HELLMANN_S_HP_HAS_DECREASED;
	
	@ClientString(id = 1802013, message = "The HP of monsters outside the Outer Castle Gates have decreased.")
	public static NpcStringId THE_HP_OF_MONSTERS_OUTSIDE_THE_OUTER_CASTLE_GATES_HAVE_DECREASED;
	
	@ClientString(id = 1802014, message = "Barbanas is casting Hold.")
	public static NpcStringId BARBANAS_IS_CASTING_HOLD;
	
	@ClientString(id = 1802015, message = "More monsters have spawned outside the Outer Castle Gates.")
	public static NpcStringId MORE_MONSTERS_HAVE_SPAWNED_OUTSIDE_THE_OUTER_CASTLE_GATES;
	
	@ClientString(id = 1802016, message = "Barbanas is casting Slow.")
	public static NpcStringId BARBANAS_IS_CASTING_SLOW;
	
	@ClientString(id = 1802017, message = "Speak with Darin.")
	public static NpcStringId SPEAK_WITH_DARIN;
	
	@ClientString(id = 1802018, message = "Speak with Roxxy.")
	public static NpcStringId SPEAK_WITH_ROXXY;
	
	@ClientString(id = 1802019, message = "Go outside the temple.")
	public static NpcStringId GO_OUTSIDE_THE_TEMPLE;
	
	@ClientString(id = 1802020, message = "Detecting a sandstorm!")
	public static NpcStringId DETECTING_A_SANDSTORM;
	
	@ClientString(id = 1802021, message = "Repair Robots annihilated! Failed to fulfill request!")
	public static NpcStringId REPAIR_ROBOTS_ANNIHILATED_FAILED_TO_FULFILL_REQUEST;
	
	@ClientString(id = 1802022, message = "Repair Robot(s) eliminated! Protect the Repair Robots!")
	public static NpcStringId REPAIR_ROBOT_S_ELIMINATED_PROTECT_THE_REPAIR_ROBOTS;
	
	@ClientString(id = 1802023, message = "The sandstorm is attacking the Stone Gathering Robots.")
	public static NpcStringId THE_SANDSTORM_IS_ATTACKING_THE_STONE_GATHERING_ROBOTS;
	
	@ClientString(id = 1802024, message = "Focus on repairing the Stone Gathering Robot(s)!")
	public static NpcStringId FOCUS_ON_REPAIRING_THE_STONE_GATHERING_ROBOT_S;
	
	@ClientString(id = 1802025, message = "The Destroyer Robots are attacking the Stone Gathering Robots!")
	public static NpcStringId THE_DESTROYER_ROBOTS_ARE_ATTACKING_THE_STONE_GATHERING_ROBOTS;
	
	@ClientString(id = 1802026, message = "Use the Multi Hammer Skill to repair the Stone Gathering Robot(s)!")
	public static NpcStringId USE_THE_MULTI_HAMMER_SKILL_TO_REPAIR_THE_STONE_GATHERING_ROBOT_S;
	
	@ClientString(id = 1802027, message = "Use the Multi Hammer Skill to attack the Destroyer Robot(s)!")
	public static NpcStringId USE_THE_MULTI_HAMMER_SKILL_TO_ATTACK_THE_DESTROYER_ROBOT_S;
	
	@ClientString(id = 1802028, message = "Broken! Please repair!")
	public static NpcStringId BROKEN_PLEASE_REPAIR;
	
	@ClientString(id = 1802029, message = "Handcrafting Progress does not increase when your HP drops below a certain level!")
	public static NpcStringId HANDCRAFTING_PROGRESS_DOES_NOT_INCREASE_WHEN_YOUR_HP_DROPS_BELOW_A_CERTAIN_LEVEL;
	
	@ClientString(id = 1802030, message = "You can only repair by using the Multi Hammer item skill.")
	public static NpcStringId YOU_CAN_ONLY_REPAIR_BY_USING_THE_MULTI_HAMMER_ITEM_SKILL;
	
	@ClientString(id = 1802031, message = "A Repair Robot will appear shortly.")
	public static NpcStringId A_REPAIR_ROBOT_WILL_APPEAR_SHORTLY;
	
	@ClientString(id = 1802032, message = "Destroy the Repair Robot attacking the Broken Toad Cannon!")
	public static NpcStringId DESTROY_THE_REPAIR_ROBOT_ATTACKING_THE_BROKEN_TOAD_CANNON;
	
	@ClientString(id = 1802033, message = "Repair the Repair Robot repairing the Broken Toad Cannon!")
	public static NpcStringId REPAIR_THE_REPAIR_ROBOT_REPAIRING_THE_BROKEN_TOAD_CANNON;
	
	@ClientString(id = 1802034, message = "You have failed.")
	public static NpcStringId YOU_HAVE_FAILED;
	
	@ClientString(id = 1802035, message = "The Dark Raider is attacking the Stone Gathering Robot(s)!")
	public static NpcStringId THE_DARK_RAIDER_IS_ATTACKING_THE_STONE_GATHERING_ROBOT_S;
	
	@ClientString(id = 1802036, message = "Use the Multi Hammer Skill to repair the Stone Gathering Robot(s)!")
	public static NpcStringId USE_THE_MULTI_HAMMER_SKILL_TO_REPAIR_THE_STONE_GATHERING_ROBOT_S2;
	
	@ClientString(id = 1802037, message = "Use the Multi Hammer Skill to attack the Dark Raider!")
	public static NpcStringId USE_THE_MULTI_HAMMER_SKILL_TO_ATTACK_THE_DARK_RAIDER;
	
	@ClientString(id = 1802038, message = "The Repair Robot breaks down when its HP drops to below a certain level.")
	public static NpcStringId THE_REPAIR_ROBOT_BREAKS_DOWN_WHEN_ITS_HP_DROPS_TO_BELOW_A_CERTAIN_LEVEL;
	
	@ClientString(id = 1802039, message = "Repair the Repair Robot continuously!")
	public static NpcStringId REPAIR_THE_REPAIR_ROBOT_CONTINUOUSLY;
	
	@ClientString(id = 1802040, message = "Alteration Progress does not increase when the Repair Robot is broken.")
	public static NpcStringId ALTERATION_PROGRESS_DOES_NOT_INCREASE_WHEN_THE_REPAIR_ROBOT_IS_BROKEN;
	
	@ClientString(id = 1802041, message = "Alteration Progress does not increase when the Broken Toad Cannon's HP drops below a certain level.")
	public static NpcStringId ALTERATION_PROGRESS_DOES_NOT_INCREASE_WHEN_THE_BROKEN_TOAD_CANNON_S_HP_DROPS_BELOW_A_CERTAIN_LEVEL;
	
	@ClientString(id = 1802042, message = "(Clan Hall Siege Alert)nA clan has risen up to challenge Nurka!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_RISEN_UP_TO_CHALLENGE_NURKA;
	
	@ClientString(id = 1802043, message = "(Clan Hall Siege Alert)nA clan has decreased Nurka's HP to 70%!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_DECREASED_NURKA_S_HP_TO_70;
	
	@ClientString(id = 1802044, message = "(Clan Hall Siege Alert)nA clan has decreased Nurka's HP to 40%!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_DECREASED_NURKA_S_HP_TO_40;
	
	@ClientString(id = 1802045, message = "(Clan Hall Siege Alert)nA clan has decreased Nurka's HP to 10%!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_DECREASED_NURKA_S_HP_TO_10;
	
	@ClientString(id = 1802046, message = "(Clan Hall Siege Alert)nA clan has defeated Nurka for the first time!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_DEFEATED_NURKA_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802047, message = "(Clan Hall Siege Alert)nA clan has risen up to challenge the von Hellmann House!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_RISEN_UP_TO_CHALLENGE_THE_VON_HELLMANN_HOUSE;
	
	@ClientString(id = 1802048, message = "(Clan Hall Siege Alert)nA clan has defeated 1 mid-level boss in the von Hellmann House!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_DEFEATED_1_MID_LEVEL_BOSS_IN_THE_VON_HELLMANN_HOUSE;
	
	@ClientString(id = 1802049, message = "(Clan Hall Siege Alert)nA clan has defeated 2 mid-level bosses in the von Hellmann House!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_DEFEATED_2_MID_LEVEL_BOSSES_IN_THE_VON_HELLMANN_HOUSE;
	
	@ClientString(id = 1802050, message = "(Clan Hall Siege Alert)nA clan has defeated Lidia von Hellmann for the first time!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_DEFEATED_LIDIA_VON_HELLMANN_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802051, message = "(Clan Hall Siege Alert)nA clan has risen up to challenge the Devastated Castle!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_RISEN_UP_TO_CHALLENGE_THE_DEVASTATED_CASTLE;
	
	@ClientString(id = 1802052, message = "(Clan Hall Siege Alert)nA clan has conquered 1 Stronghold for the first time!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_CONQUERED_1_STRONGHOLD_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802053, message = "(Clan Hall Siege Alert)nA clan has conquered 2 Strongholds for the first time!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_CONQUERED_2_STRONGHOLDS_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802054, message = "(Clan Hall Siege Alert)nA clan has conquered all 3 Strongholds for the first time!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_CONQUERED_ALL_3_STRONGHOLDS_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802055, message = "(Clan Hall Siege Alert)nA clan has defeated Gustav for the first time!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_DEFEATED_GUSTAV_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802056, message = "You…our house is already fallen to wretchedness, and you dare add insult to injury!")
	public static NpcStringId YOU_OUR_HOUSE_IS_ALREADY_FALLEN_TO_WRETCHEDNESS_AND_YOU_DARE_ADD_INSULT_TO_INJURY;
	
	@ClientString(id = 1802057, message = "You don't deserve such luck…")
	public static NpcStringId YOU_DON_T_DESERVE_SUCH_LUCK;
	
	@ClientString(id = 1802058, message = "You don't deserve to even take such form!")
	public static NpcStringId YOU_DON_T_DESERVE_TO_EVEN_TAKE_SUCH_FORM;
	
	@ClientString(id = 1802059, message = "Alfred! How can you be so calm? After being murdered so brutally!")
	public static NpcStringId ALFRED_HOW_CAN_YOU_BE_SO_CALM_AFTER_BEING_MURDERED_SO_BRUTALLY;
	
	@ClientString(id = 1802060, message = "Giselle! Show yourself once more.")
	public static NpcStringId GISELLE_SHOW_YOURSELF_ONCE_MORE;
	
	@ClientString(id = 1802061, message = "Characters near Sophie von Hellmann increase in Spd.")
	public static NpcStringId CHARACTERS_NEAR_SOPHIE_VON_HELLMANN_INCREASE_IN_SPD;
	
	@ClientString(id = 1802062, message = "Valuable items are appearing in the Fortress of the Dead.")
	public static NpcStringId VALUABLE_ITEMS_ARE_APPEARING_IN_THE_FORTRESS_OF_THE_DEAD;
	
	@ClientString(id = 1802063, message = "$s1's damage on Alfred von Hellmann is greatly increased.")
	public static NpcStringId S1_S_DAMAGE_ON_ALFRED_VON_HELLMANN_IS_GREATLY_INCREASED;
	
	@ClientString(id = 1802064, message = "Characters near Sophie von Hellmann are mass mutated.")
	public static NpcStringId CHARACTERS_NEAR_SOPHIE_VON_HELLMANN_ARE_MASS_MUTATED;
	
	@ClientString(id = 1802065, message = "Alfred von Hellmann attacks with his minions.")
	public static NpcStringId ALFRED_VON_HELLMANN_ATTACKS_WITH_HIS_MINIONS;
	
	@ClientString(id = 1802066, message = "Giselle von Hellmann resurrects.")
	public static NpcStringId GISELLE_VON_HELLMANN_RESURRECTS;
	
	@ClientString(id = 1802067, message = "Energetic Battle Supporter")
	public static NpcStringId ENERGETIC_BATTLE_SUPPORTER;
	
	@ClientString(id = 1802068, message = "Something good may happen if you fully restore Logan's HP.")
	public static NpcStringId SOMETHING_GOOD_MAY_HAPPEN_IF_YOU_FULLY_RESTORE_LOGAN_S_HP;
	
	@ClientString(id = 1802069, message = "So much energy! It is a little fearsome to be honest, but I will lend you a hand as payback for what I went through!")
	public static NpcStringId SO_MUCH_ENERGY_IT_IS_A_LITTLE_FEARSOME_TO_BE_HONEST_BUT_I_WILL_LEND_YOU_A_HAND_AS_PAYBACK_FOR_WHAT_I_WENT_THROUGH;
	
	@ClientString(id = 1802070, message = "You gave me the strength to escape. I will give you the strength to rise again!")
	public static NpcStringId YOU_GAVE_ME_THE_STRENGTH_TO_ESCAPE_I_WILL_GIVE_YOU_THE_STRENGTH_TO_RISE_AGAIN;
	
	@ClientString(id = 1802071, message = "This…this is my last gift to you before I make my escape. Lidia!")
	public static NpcStringId THIS_THIS_IS_MY_LAST_GIFT_TO_YOU_BEFORE_I_MAKE_MY_ESCAPE_LIDIA;
	
	@ClientString(id = 1802072, message = "Oh…mistake. Sorry about that.")
	public static NpcStringId OH_MISTAKE_SORRY_ABOUT_THAT;
	
	@ClientString(id = 1802073, message = "You cold-blooded monsters! Curse be upon you all! You will not live to escape this place!")
	public static NpcStringId YOU_COLD_BLOODED_MONSTERS_CURSE_BE_UPON_YOU_ALL_YOU_WILL_NOT_LIVE_TO_ESCAPE_THIS_PLACE;
	
	@ClientString(id = 1802074, message = "Logan joins the battle.")
	public static NpcStringId LOGAN_JOINS_THE_BATTLE;
	
	@ClientString(id = 1802075, message = "Characters near Logan increase greatly in HP Recovery Amount.")
	public static NpcStringId CHARACTERS_NEAR_LOGAN_INCREASE_GREATLY_IN_HP_RECOVERY_AMOUNT;
	
	@ClientString(id = 1802076, message = "Lidia von Hellmann is dragged out alone.")
	public static NpcStringId LIDIA_VON_HELLMANN_IS_DRAGGED_OUT_ALONE;
	
	@ClientString(id = 1802077, message = "Lidia von Hellmann's Atk. Spd. is increased.")
	public static NpcStringId LIDIA_VON_HELLMANN_S_ATK_SPD_IS_INCREASED;
	
	@ClientString(id = 1802078, message = "Von Hellmann House's spirits appear in Lidia's room.")
	public static NpcStringId VON_HELLMANN_HOUSE_S_SPIRITS_APPEAR_IN_LIDIA_S_ROOM;
	
	@ClientString(id = 1802079, message = "Vampires in Lidia's room become invincible.")
	public static NpcStringId VAMPIRES_IN_LIDIA_S_ROOM_BECOME_INVINCIBLE;
	
	@ClientString(id = 1802080, message = "Demolished Throne Stronghold will become Conquerable after $s1 minute(s).")
	public static NpcStringId DEMOLISHED_THRONE_STRONGHOLD_WILL_BECOME_CONQUERABLE_AFTER_S1_MINUTE_S;
	
	@ClientString(id = 1802081, message = "I will make my leave, Master…")
	public static NpcStringId I_WILL_MAKE_MY_LEAVE_MASTER;
	
	@ClientString(id = 1802082, message = "You can only hit me with a Multi Hammer!")
	public static NpcStringId YOU_CAN_ONLY_HIT_ME_WITH_A_MULTI_HAMMER;
	
	@ClientString(id = 1802083, message = "Hit me with a Multi Hammer to repair me!")
	public static NpcStringId HIT_ME_WITH_A_MULTI_HAMMER_TO_REPAIR_ME;
	
	@ClientString(id = 1802084, message = "Repair successful! Alteration Progress increases!")
	public static NpcStringId REPAIR_SUCCESSFUL_ALTERATION_PROGRESS_INCREASES;
	
	@ClientString(id = 1802085, message = "You must hit 17 times within 10 seconds!")
	public static NpcStringId YOU_MUST_HIT_17_TIMES_WITHIN_10_SECONDS;
	
	@ClientString(id = 1802086, message = "You must hit 27 times within 10 seconds!")
	public static NpcStringId YOU_MUST_HIT_27_TIMES_WITHIN_10_SECONDS;
	
	@ClientString(id = 1802087, message = "10 seconds have passed. Disappearing!")
	public static NpcStringId SECONDS_HAVE_PASSED_DISAPPEARING;
	
	@ClientString(id = 1802088, message = "Repair Progress")
	public static NpcStringId REPAIR_PROGRESS;
	
	@ClientString(id = 1802089, message = "Broken Guardian Robot has appeared. Hit it 17 times within 10 seconds to repair it!")
	public static NpcStringId BROKEN_GUARDIAN_ROBOT_HAS_APPEARED_HIT_IT_17_TIMES_WITHIN_10_SECONDS_TO_REPAIR_IT;
	
	@ClientString(id = 1802090, message = "Broken Guardian Robot has appeared. Hit it 27 times within 10 seconds to repair it!")
	public static NpcStringId BROKEN_GUARDIAN_ROBOT_HAS_APPEARED_HIT_IT_27_TIMES_WITHIN_10_SECONDS_TO_REPAIR_IT;
	
	@ClientString(id = 1802091, message = "Silvermane White Tiger has appeared. Hit it a certain number of times with the Multi Hammer to make it go away!")
	public static NpcStringId SILVERMANE_WHITE_TIGER_HAS_APPEARED_HIT_IT_A_CERTAIN_NUMBER_OF_TIMES_WITH_THE_MULTI_HAMMER_TO_MAKE_IT_GO_AWAY;
	
	@ClientString(id = 1802092, message = "Protect the Buffalo from the Silvermane White Tiger!")
	public static NpcStringId PROTECT_THE_BUFFALO_FROM_THE_SILVERMANE_WHITE_TIGER;
	
	@ClientString(id = 1802093, message = "The Buffalo must be alive in order for the Handcrafting Progress to increase!")
	public static NpcStringId THE_BUFFALO_MUST_BE_ALIVE_IN_ORDER_FOR_THE_HANDCRAFTING_PROGRESS_TO_INCREASE;
	
	@ClientString(id = 1802094, message = "The Request Manager will disappear in 9 minutes.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_9_MINUTES;
	
	@ClientString(id = 1802095, message = "The Request Manager will disappear in 8 minutes.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_8_MINUTES;
	
	@ClientString(id = 1802096, message = "The Request Manager will disappear in 7 minutes.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_7_MINUTES;
	
	@ClientString(id = 1802097, message = "The Request Manager will disappear in 6 minutes.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_6_MINUTES;
	
	@ClientString(id = 1802098, message = "The Request Manager will disappear in 5 minutes.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_5_MINUTES;
	
	@ClientString(id = 1802099, message = "The Request Manager will disappear in 4 minutes.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_4_MINUTES;
	
	@ClientString(id = 1802100, message = "The Request Manager will disappear in 3 minutes.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_3_MINUTES;
	
	@ClientString(id = 1802101, message = "The Request Manager will disappear in 2 minutes.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_2_MINUTES;
	
	@ClientString(id = 1802102, message = "The Request Manager will disappear in 1 minute.")
	public static NpcStringId THE_REQUEST_MANAGER_WILL_DISAPPEAR_IN_1_MINUTE;
	
	@ClientString(id = 1802103, message = "Dismantling is $s1% complete!")
	public static NpcStringId DISMANTLING_IS_S1_COMPLETE;
	
	@ClientString(id = 1802104, message = "The Life Plunderer's true form is revealed.")
	public static NpcStringId THE_LIFE_PLUNDERER_S_TRUE_FORM_IS_REVEALED;
	
	@ClientString(id = 1802105, message = "The Life Plunderer is feeling threatened.")
	public static NpcStringId THE_LIFE_PLUNDERER_IS_FEELING_THREATENED;
	
	@ClientString(id = 1802106, message = "Burning Blood's effect is felt.")
	public static NpcStringId BURNING_BLOOD_S_EFFECT_IS_FELT;
	
	@ClientString(id = 1802107, message = "The Life Plunderer has disappeared.")
	public static NpcStringId THE_LIFE_PLUNDERER_HAS_DISAPPEARED;
	
	@ClientString(id = 1802108, message = "Click on 'Summon Clan Member' to be teleported with the clan member to the Request Zone.")
	public static NpcStringId CLICK_ON_SUMMON_CLAN_MEMBER_TO_BE_TELEPORTED_WITH_THE_CLAN_MEMBER_TO_THE_REQUEST_ZONE;
	
	@ClientString(id = 1802109, message = "The Request Zone disappears in 10 minutes. Receive a Multi Hammer and begin the Request.")
	public static NpcStringId THE_REQUEST_ZONE_DISAPPEARS_IN_10_MINUTES_RECEIVE_A_MULTI_HAMMER_AND_BEGIN_THE_REQUEST;
	
	@ClientString(id = 1802110, message = "Request begins in 5 seconds. Speak with the Clan Request Assistant to receive a Multi Hammer.")
	public static NpcStringId REQUEST_BEGINS_IN_5_SECONDS_SPEAK_WITH_THE_CLAN_REQUEST_ASSISTANT_TO_RECEIVE_A_MULTI_HAMMER;
	
	@ClientString(id = 1802111, message = "5 seconds before Request starts! Equip the Multi Hammer!")
	public static NpcStringId SECONDS_BEFORE_REQUEST_STARTS_EQUIP_THE_MULTI_HAMMER;
	
	@ClientString(id = 1802112, message = "4 seconds before Request starts! Equip the Multi Hammer!")
	public static NpcStringId SECONDS_BEFORE_REQUEST_STARTS_EQUIP_THE_MULTI_HAMMER2;
	
	@ClientString(id = 1802113, message = "3 seconds before Request starts! Equip the Multi Hammer!")
	public static NpcStringId SECONDS_BEFORE_REQUEST_STARTS_EQUIP_THE_MULTI_HAMMER3;
	
	@ClientString(id = 1802114, message = "2 seconds before Request starts! Equip the Multi Hammer!")
	public static NpcStringId SECONDS_BEFORE_REQUEST_STARTS_EQUIP_THE_MULTI_HAMMER4;
	
	@ClientString(id = 1802115, message = "1 seconds before Request starts! Equip the Multi Hammer!")
	public static NpcStringId SECONDS_BEFORE_REQUEST_STARTS_EQUIP_THE_MULTI_HAMMER5;
	
	@ClientString(id = 1802116, message = "Begin Request!")
	public static NpcStringId BEGIN_REQUEST;
	
	@ClientString(id = 1802117, message = "Request successfully fulfilled! Claim your reward from the Clan Request Assistant.")
	public static NpcStringId REQUEST_SUCCESSFULLY_FULFILLED_CLAIM_YOUR_REWARD_FROM_THE_CLAN_REQUEST_ASSISTANT;
	
	@ClientString(id = 1802118, message = "Failed to fulfill request! Claim your reward from the Clan Request Assistant.")
	public static NpcStringId FAILED_TO_FULFILL_REQUEST_CLAIM_YOUR_REWARD_FROM_THE_CLAN_REQUEST_ASSISTANT;
	
	@ClientString(id = 1802119, message = "You need the Buffalo for leather! Protect the Buffalo from the Silvermane White Tiger!")
	public static NpcStringId YOU_NEED_THE_BUFFALO_FOR_LEATHER_PROTECT_THE_BUFFALO_FROM_THE_SILVERMANE_WHITE_TIGER;
	
	@ClientString(id = 1802120, message = "Your Request Progress increases with the number of surviving Buffalo.")
	public static NpcStringId YOUR_REQUEST_PROGRESS_INCREASES_WITH_THE_NUMBER_OF_SURVIVING_BUFFALO;
	
	@ClientString(id = 1802121, message = "The Buffalo are being attacked! Use the Multi Hammer to defeat the Silvermane White Tiger.")
	public static NpcStringId THE_BUFFALO_ARE_BEING_ATTACKED_USE_THE_MULTI_HAMMER_TO_DEFEAT_THE_SILVERMANE_WHITE_TIGER;
	
	@ClientString(id = 1802122, message = "Dismantle the Broken Guardian Robot to recycle its parts.")
	public static NpcStringId DISMANTLE_THE_BROKEN_GUARDIAN_ROBOT_TO_RECYCLE_ITS_PARTS;
	
	@ClientString(id = 1802123, message = "Your Request Progress increases with the speed with which you dismantle the broken robot.")
	public static NpcStringId YOUR_REQUEST_PROGRESS_INCREASES_WITH_THE_SPEED_WITH_WHICH_YOU_DISMANTLE_THE_BROKEN_ROBOT;
	
	@ClientString(id = 1802124, message = "Stock the broken robots! Hit them with the Multi Hammer to dismantle them, and they will explode within 10 seconds of stocking.")
	public static NpcStringId STOCK_THE_BROKEN_ROBOTS_HIT_THEM_WITH_THE_MULTI_HAMMER_TO_DISMANTLE_THEM_AND_THEY_WILL_EXPLODE_WITHIN_10_SECONDS_OF_STOCKING;
	
	@ClientString(id = 1802125, message = "Wake up as many Milk Cows as possible for fast milk production.")
	public static NpcStringId WAKE_UP_AS_MANY_MILK_COWS_AS_POSSIBLE_FOR_FAST_MILK_PRODUCTION;
	
	@ClientString(id = 1802126, message = "You must continuously wake up the Milk Cows while the chefs interrupt.")
	public static NpcStringId YOU_MUST_CONTINUOUSLY_WAKE_UP_THE_MILK_COWS_WHILE_THE_CHEFS_INTERRUPT;
	
	@ClientString(id = 1802127, message = "You must keep your Request Progress at + % by waking up as many cows as possible within the given time.")
	public static NpcStringId YOU_MUST_KEEP_YOUR_REQUEST_PROGRESS_AT_BY_WAKING_UP_AS_MANY_COWS_AS_POSSIBLE_WITHIN_THE_GIVEN_TIME;
	
	@ClientString(id = 1802128, message = "Click on 'Summon Clan Member' to bring the clan member to your location.")
	public static NpcStringId CLICK_ON_SUMMON_CLAN_MEMBER_TO_BRING_THE_CLAN_MEMBER_TO_YOUR_LOCATION;
	
	@ClientString(id = 1802129, message = "Click on 'Start Quest' to begin Request.")
	public static NpcStringId CLICK_ON_START_QUEST_TO_BEGIN_REQUEST;
	
	@ClientString(id = 1802130, message = "You must make a party with your clan members to begin.")
	public static NpcStringId YOU_MUST_MAKE_A_PARTY_WITH_YOUR_CLAN_MEMBERS_TO_BEGIN;
	
	@ClientString(id = 1802131, message = "Request starts in 5 seconds.")
	public static NpcStringId REQUEST_STARTS_IN_5_SECONDS;
	
	@ClientString(id = 1802132, message = "5 Seconds before Quest Starts")
	public static NpcStringId SECONDS_BEFORE_QUEST_STARTS;
	
	@ClientString(id = 1802133, message = "4 Seconds before Quest Starts")
	public static NpcStringId SECONDS_BEFORE_QUEST_STARTS2;
	
	@ClientString(id = 1802134, message = "3 Seconds before Quest Starts")
	public static NpcStringId SECONDS_BEFORE_QUEST_STARTS3;
	
	@ClientString(id = 1802135, message = "2 Seconds before Quest Starts")
	public static NpcStringId SECONDS_BEFORE_QUEST_STARTS4;
	
	@ClientString(id = 1802136, message = "1 Seconds before Quest Starts")
	public static NpcStringId SECONDS_BEFORE_QUEST_STARTS5;
	
	@ClientString(id = 1802137, message = "Begin Quest!")
	public static NpcStringId BEGIN_QUEST;
	
	@ClientString(id = 1802138, message = "Quest Successful!")
	public static NpcStringId QUEST_SUCCESSFUL;
	
	@ClientString(id = 1802139, message = "Quest Failed!")
	public static NpcStringId QUEST_FAILED;
	
	@ClientString(id = 1802140, message = "Your clan has acquired 100 points to its Clan Reputation. Clan Reputation is given from Clan Level 5 or above.")
	public static NpcStringId YOUR_CLAN_HAS_ACQUIRED_100_POINTS_TO_ITS_CLAN_REPUTATION_CLAN_REPUTATION_IS_GIVEN_FROM_CLAN_LEVEL_5_OR_ABOVE;
	
	@ClientString(id = 1802141, message = "You can obtain the Multi Hammer through the Clan Request Assistant.")
	public static NpcStringId YOU_CAN_OBTAIN_THE_MULTI_HAMMER_THROUGH_THE_CLAN_REQUEST_ASSISTANT;
	
	@ClientString(id = 1802142, message = "Zzz… Zzz…")
	public static NpcStringId ZZZ_ZZZ;
	
	@ClientString(id = 1802143, message = "A white tiger! A white tiger!")
	public static NpcStringId A_WHITE_TIGER_A_WHITE_TIGER;
	
	@ClientString(id = 1802144, message = "Hmm… Do you think you can break through Lord Nurka's formation? Have fun! Hahaha!")
	public static NpcStringId HMM_DO_YOU_THINK_YOU_CAN_BREAK_THROUGH_LORD_NURKA_S_FORMATION_HAVE_FUN_HAHAHA;
	
	@ClientString(id = 1802145, message = "Man, just that time when everyone's resting. Consider yourself lucky!")
	public static NpcStringId MAN_JUST_THAT_TIME_WHEN_EVERYONE_S_RESTING_CONSIDER_YOURSELF_LUCKY;
	
	@ClientString(id = 1802146, message = "Haha, oh man you guys are out of luck. Think you can hold out against Lord Nurka's Royal Guards?")
	public static NpcStringId HAHA_OH_MAN_YOU_GUYS_ARE_OUT_OF_LUCK_THINK_YOU_CAN_HOLD_OUT_AGAINST_LORD_NURKA_S_ROYAL_GUARDS;
	
	@ClientString(id = 1802147, message = "Ouch, ouch! Help!")
	public static NpcStringId OUCH_OUCH_HELP;
	
	@ClientString(id = 1802148, message = "Please get rid of the White Tiger!")
	public static NpcStringId PLEASE_GET_RID_OF_THE_WHITE_TIGER;
	
	@ClientString(id = 1802149, message = "Please protect us!")
	public static NpcStringId PLEASE_PROTECT_US;
	
	@ClientString(id = 1802150, message = "Moo~ I'm dying. Help this poor little buffalo!")
	public static NpcStringId MOO_I_M_DYING_HELP_THIS_POOR_LITTLE_BUFFALO;
	
	@ClientString(id = 1802151, message = "You can only Summon Clan Member once.")
	public static NpcStringId YOU_CAN_ONLY_SUMMON_CLAN_MEMBER_ONCE;
	
	@ClientString(id = 1802152, message = "You can only Begin Request once.")
	public static NpcStringId YOU_CAN_ONLY_BEGIN_REQUEST_ONCE;
	
	@ClientString(id = 1802153, message = "You cannot Begin Request in this area.")
	public static NpcStringId YOU_CANNOT_BEGIN_REQUEST_IN_THIS_AREA;
	
	@ClientString(id = 1802154, message = "You cannot Summon Clan Member in this area.")
	public static NpcStringId YOU_CANNOT_SUMMON_CLAN_MEMBER_IN_THIS_AREA;
	
	@ClientString(id = 1802155, message = "The Movable Stronghold will be Conquerable once it reaches the opposite wall of the castle.")
	public static NpcStringId THE_MOVABLE_STRONGHOLD_WILL_BE_CONQUERABLE_ONCE_IT_REACHES_THE_OPPOSITE_WALL_OF_THE_CASTLE;
	
	@ClientString(id = 1802156, message = "The Movable Stronghold has crossed halfway.")
	public static NpcStringId THE_MOVABLE_STRONGHOLD_HAS_CROSSED_HALFWAY;
	
	@ClientString(id = 1802157, message = "The Movable Stronghold has reached the opposite wall of the castle and become Conquerable.")
	public static NpcStringId THE_MOVABLE_STRONGHOLD_HAS_REACHED_THE_OPPOSITE_WALL_OF_THE_CASTLE_AND_BECOME_CONQUERABLE;
	
	@ClientString(id = 1802158, message = "If the Attack Stronghold's HP is fully restored, it will become Conquerable.")
	public static NpcStringId IF_THE_ATTACK_STRONGHOLD_S_HP_IS_FULLY_RESTORED_IT_WILL_BECOME_CONQUERABLE;
	
	@ClientString(id = 1802159, message = "The Attack Stronghold's HP has been halfway restored.")
	public static NpcStringId THE_ATTACK_STRONGHOLD_S_HP_HAS_BEEN_HALFWAY_RESTORED;
	
	@ClientString(id = 1802160, message = "The Attack Stronghold's HP has been fully restored, and become Conquerable.")
	public static NpcStringId THE_ATTACK_STRONGHOLD_S_HP_HAS_BEEN_FULLY_RESTORED_AND_BECOME_CONQUERABLE;
	
	@ClientString(id = 1802161, message = "Our castle's strength remains strong on the Stronghold. You won't be able to lay a hand on me!")
	public static NpcStringId OUR_CASTLE_S_STRENGTH_REMAINS_STRONG_ON_THE_STRONGHOLD_YOU_WON_T_BE_ABLE_TO_LAY_A_HAND_ON_ME;
	
	@ClientString(id = 1802162, message = "You dare attack me without even conquering the Movable Stronghold? You've got guts, I'll give you that.")
	public static NpcStringId YOU_DARE_ATTACK_ME_WITHOUT_EVEN_CONQUERING_THE_MOVABLE_STRONGHOLD_YOU_VE_GOT_GUTS_I_LL_GIVE_YOU_THAT;
	
	@ClientString(id = 1802163, message = "Our castle's strength remains on the Stronghold, and I will protect it as I should!")
	public static NpcStringId OUR_CASTLE_S_STRENGTH_REMAINS_ON_THE_STRONGHOLD_AND_I_WILL_PROTECT_IT_AS_I_SHOULD;
	
	@ClientString(id = 1802164, message = "(Clan Hall Siege Alert)nA clan has opened the Outer Castle Gates to the Fortress of the Dead!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_OPENED_THE_OUTER_CASTLE_GATES_TO_THE_FORTRESS_OF_THE_DEAD;
	
	@ClientString(id = 1802165, message = "(Clan Hall Siege Alert)nA clan has opened the Inner Castle Gates to the Fortress of the Dead!")
	public static NpcStringId CLAN_HALL_SIEGE_ALERT_NA_CLAN_HAS_OPENED_THE_INNER_CASTLE_GATES_TO_THE_FORTRESS_OF_THE_DEAD;
	
	@ClientString(id = 1802166, message = "Your clan has opened the Outer Castle Gates to the Fortress of the Dead for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_OPENED_THE_OUTER_CASTLE_GATES_TO_THE_FORTRESS_OF_THE_DEAD_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802167, message = "Your clan has defeated 1 mid-level boss in the von Hellmann House for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_DEFEATED_1_MID_LEVEL_BOSS_IN_THE_VON_HELLMANN_HOUSE_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802168, message = "Your clan has defeated 2 mid-level bosses in the von Hellmann House for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_DEFEATED_2_MID_LEVEL_BOSSES_IN_THE_VON_HELLMANN_HOUSE_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802169, message = "Your clan has opened the Inner Castle Gates to the Fortress of the Dead for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_OPENED_THE_INNER_CASTLE_GATES_TO_THE_FORTRESS_OF_THE_DEAD_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802170, message = "Your clan has defeated Lidia von Hellmann for the first time! Congratulations!")
	public static NpcStringId YOUR_CLAN_HAS_DEFEATED_LIDIA_VON_HELLMANN_FOR_THE_FIRST_TIME_CONGRATULATIONS;
	
	@ClientString(id = 1802171, message = "Your clan has conquered 1 Stronghold for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_CONQUERED_1_STRONGHOLD_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802172, message = "Your clan has conquered 2 Strongholds for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_CONQUERED_2_STRONGHOLDS_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802173, message = "Your clan has conquered all 3 Strongholds for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_CONQUERED_ALL_3_STRONGHOLDS_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802174, message = "Your clan has defeated Gustav for the first time! Congratulations!")
	public static NpcStringId YOUR_CLAN_HAS_DEFEATED_GUSTAV_FOR_THE_FIRST_TIME_CONGRATULATIONS;
	
	@ClientString(id = 1802175, message = "Your clan has decreased Nurka's HP to 70% for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_DECREASED_NURKA_S_HP_TO_70_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802176, message = "Your clan has decreased Nurka's HP to 40% for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_DECREASED_NURKA_S_HP_TO_40_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802177, message = "Your clan has decreased Nurka's HP to 10% for the first time!")
	public static NpcStringId YOUR_CLAN_HAS_DECREASED_NURKA_S_HP_TO_10_FOR_THE_FIRST_TIME;
	
	@ClientString(id = 1802178, message = "Your clan has defeated Nurka for the first time! Congratulations!")
	public static NpcStringId YOUR_CLAN_HAS_DEFEATED_NURKA_FOR_THE_FIRST_TIME_CONGRATULATIONS;
	
	@ClientString(id = 1802179, message = "Malfunction. Malfunction.")
	public static NpcStringId MALFUNCTION_MALFUNCTION;
	
	@ClientString(id = 1802180, message = "Request failed. Please try again.")
	public static NpcStringId REQUEST_FAILED_PLEASE_TRY_AGAIN;
	
	@ClientString(id = 1802181, message = "???")
	public static NpcStringId EMPTY4;
	
	@ClientString(id = 1802182, message = "Receive your Multi Hammer(s)!")
	public static NpcStringId RECEIVE_YOUR_MULTI_HAMMER_S;
	
	@ClientString(id = 1802183, message = "You can receive your reward after completing the Request.")
	public static NpcStringId YOU_CAN_RECEIVE_YOUR_REWARD_AFTER_COMPLETING_THE_REQUEST;
	
	@ClientString(id = 1802184, message = "Only those who summoned the Clan Request Manager may use this menu.")
	public static NpcStringId ONLY_THOSE_WHO_SUMMONED_THE_CLAN_REQUEST_MANAGER_MAY_USE_THIS_MENU;
	
	@ClientString(id = 1802185, message = "Charlotte appears while looking around fretfully.")
	public static NpcStringId CHARLOTTE_APPEARS_WHILE_LOOKING_AROUND_FRETFULLY;
	
	@ClientString(id = 1802186, message = "Whew…just watching from the shadows scared me out of my wits!")
	public static NpcStringId WHEW_JUST_WATCHING_FROM_THE_SHADOWS_SCARED_ME_OUT_OF_MY_WITS;
	
	@ClientString(id = 1802187, message = "News! News! Boss monsters are showing up at the Elven Village!")
	public static NpcStringId NEWS_NEWS_BOSS_MONSTERS_ARE_SHOWING_UP_AT_THE_ELVEN_VILLAGE;
	
	@ClientString(id = 1802188, message = "Lyn Draco appears due to $s1.")
	public static NpcStringId LYN_DRACO_APPEARS_DUE_TO_S1;
	
	@ClientString(id = 1802189, message = "I'll be back…")
	public static NpcStringId I_LL_BE_BACK2;
	
	@ClientString(id = 1802190, message = "Who called me? I'm gonna tell Mom!")
	public static NpcStringId WHO_CALLED_ME_I_M_GONNA_TELL_MOM;
	
	@ClientString(id = 1802191, message = "When I grow up, I'm gonna on a bloodstorm!")
	public static NpcStringId WHEN_I_GROW_UP_I_M_GONNA_ON_A_BLOODSTORM;
	
	@ClientString(id = 1802192, message = "You let me leave like this, and you'll regret it later!")
	public static NpcStringId YOU_LET_ME_LEAVE_LIKE_THIS_AND_YOU_LL_REGRET_IT_LATER;
	
	@ClientString(id = 1802193, message = "How do you like the color of my scales? I bet you never saw the likes of them!")
	public static NpcStringId HOW_DO_YOU_LIKE_THE_COLOR_OF_MY_SCALES_I_BET_YOU_NEVER_SAW_THE_LIKES_OF_THEM;
	
	@ClientString(id = 1802194, message = "Lyn Draco appears due to $s1's party.")
	public static NpcStringId LYN_DRACO_APPEARS_DUE_TO_S1_S_PARTY;
	
	@ClientString(id = 1802195, message = "You won't lay a finger on His Majesty Travis!")
	public static NpcStringId YOU_WON_T_LAY_A_FINGER_ON_HIS_MAJESTY_TRAVIS;
	
	@ClientString(id = 1802196, message = "What is this sickening aura…?")
	public static NpcStringId WHAT_IS_THIS_SICKENING_AURA;
	
	@ClientString(id = 1802197, message = "His Majesty Travis has charged me with the duty to protect. And protect I shall!")
	public static NpcStringId HIS_MAJESTY_TRAVIS_HAS_CHARGED_ME_WITH_THE_DUTY_TO_PROTECT_AND_PROTECT_I_SHALL;
	
	@ClientString(id = 1802198, message = "I hear His Majesty's voice…I hear it every waking moment…")
	public static NpcStringId I_HEAR_HIS_MAJESTY_S_VOICE_I_HEAR_IT_EVERY_WAKING_MOMENT;
	
	@ClientString(id = 1802199, message = "Hah! My body is hale yet!")
	public static NpcStringId HAH_MY_BODY_IS_HALE_YET;
	
	@ClientString(id = 1802200, message = "Behold…my power!")
	public static NpcStringId BEHOLD_MY_POWER;
	
	@ClientString(id = 1802201, message = "A long slumber has come to an end…")
	public static NpcStringId A_LONG_SLUMBER_HAS_COME_TO_AN_END;
	
	@ClientString(id = 1802202, message = "Long have I waited for His Majesty's call…")
	public static NpcStringId LONG_HAVE_I_WAITED_FOR_HIS_MAJESTY_S_CALL;
	
	@ClientString(id = 1802203, message = "I will destroy you, just as I have destroyed the enemy forces!")
	public static NpcStringId I_WILL_DESTROY_YOU_JUST_AS_I_HAVE_DESTROYED_THE_ENEMY_FORCES;
	
	@ClientString(id = 1802204, message = "You…!")
	public static NpcStringId YOU;
	
	@ClientString(id = 1802205, message = "What...what is this power…?")
	public static NpcStringId WHAT_WHAT_IS_THIS_POWER;
	
	@ClientString(id = 1802206, message = "My strength has not yet fully returned…")
	public static NpcStringId MY_STRENGTH_HAS_NOT_YET_FULLY_RETURNED;
	
	@ClientString(id = 1802207, message = "I won't beg for my life. Do what you will!")
	public static NpcStringId I_WON_T_BEG_FOR_MY_LIFE_DO_WHAT_YOU_WILL;
	
	@ClientString(id = 1802208, message = "I will remember you…")
	public static NpcStringId I_WILL_REMEMBER_YOU;
	
	@ClientString(id = 1802209, message = "How dare you try to harm me with your evil schemes!")
	public static NpcStringId HOW_DARE_YOU_TRY_TO_HARM_ME_WITH_YOUR_EVIL_SCHEMES;
	
	@ClientString(id = 1802210, message = "Feel for yourself…the strength of my will!")
	public static NpcStringId FEEL_FOR_YOURSELF_THE_STRENGTH_OF_MY_WILL;
	
	@ClientString(id = 1802211, message = "I don't have much time left…")
	public static NpcStringId I_DON_T_HAVE_MUCH_TIME_LEFT;
	
	@ClientString(id = 1802212, message = "Such deadly aura… haven't felt the likes of this in years!")
	public static NpcStringId SUCH_DEADLY_AURA_HAVEN_T_FELT_THE_LIKES_OF_THIS_IN_YEARS;
	
	@ClientString(id = 1802213, message = "Are you trying to make me die a second death?")
	public static NpcStringId ARE_YOU_TRYING_TO_MAKE_ME_DIE_A_SECOND_DEATH;
	
	@ClientString(id = 1802214, message = "His Majesty Travis has ordered immediate execution!")
	public static NpcStringId HIS_MAJESTY_TRAVIS_HAS_ORDERED_IMMEDIATE_EXECUTION;
	
	@ClientString(id = 1802215, message = "Agh! My eye! My eye!")
	public static NpcStringId AGH_MY_EYE_MY_EYE;
	
	@ClientString(id = 1802216, message = "Do not defile this place with your presence!")
	public static NpcStringId DO_NOT_DEFILE_THIS_PLACE_WITH_YOUR_PRESENCE;
	
	@ClientString(id = 1802217, message = "We were the ones who cleansed this place of evil pests!")
	public static NpcStringId WE_WERE_THE_ONES_WHO_CLEANSED_THIS_PLACE_OF_EVIL_PESTS;
	
	@ClientString(id = 1802218, message = "What would you know of loyalty?!")
	public static NpcStringId WHAT_WOULD_YOU_KNOW_OF_LOYALTY;
	
	@ClientString(id = 1802219, message = "A mysterious power has taken residence in me…")
	public static NpcStringId A_MYSTERIOUS_POWER_HAS_TAKEN_RESIDENCE_IN_ME;
	
	@ClientString(id = 1802220, message = "My ears…they keep ringing!")
	public static NpcStringId MY_EARS_THEY_KEEP_RINGING;
	
	@ClientString(id = 1802221, message = "What is this! So disorienting!")
	public static NpcStringId WHAT_IS_THIS_SO_DISORIENTING;
	
	@ClientString(id = 1802222, message = "So you killed my comrades, did you?!")
	public static NpcStringId SO_YOU_KILLED_MY_COMRADES_DID_YOU;
	
	@ClientString(id = 1802223, message = "I will avenge my fallen comrades…")
	public static NpcStringId I_WILL_AVENGE_MY_FALLEN_COMRADES;
	
	@ClientString(id = 1802224, message = "Let me be blind…let me be deaf… but I will not stop!!")
	public static NpcStringId LET_ME_BE_BLIND_LET_ME_BE_DEAF_BUT_I_WILL_NOT_STOP;
	
	@ClientString(id = 1802225, message = "Remember our history…our glorious history!")
	public static NpcStringId REMEMBER_OUR_HISTORY_OUR_GLORIOUS_HISTORY;
	
	@ClientString(id = 1802226, message = "I cannot…I will not…back down!")
	public static NpcStringId I_CANNOT_I_WILL_NOT_BACK_DOWN;
	
	@ClientString(id = 1802227, message = "A mountain of corpses…")
	public static NpcStringId A_MOUNTAIN_OF_CORPSES;
	
	@ClientString(id = 1802228, message = "My slumbering muscles begin to wake…")
	public static NpcStringId MY_SLUMBERING_MUSCLES_BEGIN_TO_WAKE;
	
	@ClientString(id = 1802229, message = "No use trying your petty tricks!")
	public static NpcStringId NO_USE_TRYING_YOUR_PETTY_TRICKS;
	
	@ClientString(id = 1802230, message = "I'm ten steps ahead of you. Don't bother trying.")
	public static NpcStringId I_M_TEN_STEPS_AHEAD_OF_YOU_DON_T_BOTHER_TRYING;
	
	@ClientString(id = 1802231, message = "We haven't even begun yet.")
	public static NpcStringId WE_HAVEN_T_EVEN_BEGUN_YET;
	
	@ClientString(id = 1802232, message = "Are you watching? Watch… and feel what true fear is!")
	public static NpcStringId ARE_YOU_WATCHING_WATCH_AND_FEEL_WHAT_TRUE_FEAR_IS;
	
	@ClientString(id = 1802233, message = "Who are you? You weren't here before..")
	public static NpcStringId WHO_ARE_YOU_YOU_WEREN_T_HERE_BEFORE;
	
	@ClientString(id = 1802234, message = "Look upon this great grave, and tell me you see nothing!")
	public static NpcStringId LOOK_UPON_THIS_GREAT_GRAVE_AND_TELL_ME_YOU_SEE_NOTHING;
	
	@ClientString(id = 1802235, message = "The dead shall pass judgment on the living!")
	public static NpcStringId THE_DEAD_SHALL_PASS_JUDGMENT_ON_THE_LIVING;
	
	@ClientString(id = 1802236, message = "Heh, interesting trick you use there.")
	public static NpcStringId HEH_INTERESTING_TRICK_YOU_USE_THERE;
	
	@ClientString(id = 1802237, message = "Now, your turn!")
	public static NpcStringId NOW_YOUR_TURN;
	
	@ClientString(id = 1802238, message = "Today's your judgment day!")
	public static NpcStringId TODAY_S_YOUR_JUDGMENT_DAY;
	
	@ClientString(id = 1802239, message = "Fool…")
	public static NpcStringId FOOL;
	
	@ClientString(id = 1802240, message = "What foolishness.")
	public static NpcStringId WHAT_FOOLISHNESS;
	
	@ClientString(id = 1802241, message = "I will end that sorry life.")
	public static NpcStringId I_WILL_END_THAT_SORRY_LIFE;
	
	@ClientString(id = 1802242, message = "Well, what do you know… you're gutsier than you look.")
	public static NpcStringId WELL_WHAT_DO_YOU_KNOW_YOU_RE_GUTSIER_THAN_YOU_LOOK;
	
	@ClientString(id = 1802243, message = "So this is the end…")
	public static NpcStringId SO_THIS_IS_THE_END;
	
	@ClientString(id = 1802244, message = "May Aden live on forever!")
	public static NpcStringId MAY_ADEN_LIVE_ON_FOREVER;
	
	@ClientString(id = 1802245, message = "Glory to Aden!")
	public static NpcStringId GLORY_TO_ADEN;
	
	@ClientString(id = 1802246, message = "Hail His Majesty Travis!")
	public static NpcStringId HAIL_HIS_MAJESTY_TRAVIS;
	
	@ClientString(id = 1802247, message = "How dare you?!")
	public static NpcStringId HOW_DARE_YOU2;
	
	@ClientString(id = 1802248, message = "Do you have any idea who I am?!")
	public static NpcStringId DO_YOU_HAVE_ANY_IDEA_WHO_I_AM;
	
	@ClientString(id = 1802249, message = "Cough…wheeze…")
	public static NpcStringId COUGH_WHEEZE;
	
	@ClientString(id = 1802250, message = "My body… my body is not yet well…")
	public static NpcStringId MY_BODY_MY_BODY_IS_NOT_YET_WELL;
	
	@ClientString(id = 1802251, message = "Be gone, I say!")
	public static NpcStringId BE_GONE_I_SAY;
	
	@ClientString(id = 1802252, message = "Imbecile…")
	public static NpcStringId IMBECILE;
	
	@ClientString(id = 1802253, message = "Shame on you!")
	public static NpcStringId SHAME_ON_YOU;
	
	@ClientString(id = 1802254, message = "Are you not ashamed before the gods?!")
	public static NpcStringId ARE_YOU_NOT_ASHAMED_BEFORE_THE_GODS;
	
	@ClientString(id = 1802255, message = "Shilen…Shilen. Such is the name imprinted in my memory…")
	public static NpcStringId SHILEN_SHILEN_SUCH_IS_THE_NAME_IMPRINTED_IN_MY_MEMORY;
	
	@ClientString(id = 1802256, message = "I am Travis! I shall not bow down to you!")
	public static NpcStringId I_AM_TRAVIS_I_SHALL_NOT_BOW_DOWN_TO_YOU;
	
	@ClientString(id = 1802257, message = "Not enough time…not enough time at all.")
	public static NpcStringId NOT_ENOUGH_TIME_NOT_ENOUGH_TIME_AT_ALL;
	
	@ClientString(id = 1802258, message = "No…argh…")
	public static NpcStringId NO_ARGH;
	
	@ClientString(id = 1802259, message = "How…how dare you do such a foul thing!")
	public static NpcStringId HOW_HOW_DARE_YOU_DO_SUCH_A_FOUL_THING;
	
	@ClientString(id = 1802260, message = "You will pay… upon my word you will pay!!")
	public static NpcStringId YOU_WILL_PAY_UPON_MY_WORD_YOU_WILL_PAY;
	
	@ClientString(id = 1802261, message = "A powerful wind begins to blow.")
	public static NpcStringId A_POWERFUL_WIND_BEGINS_TO_BLOW;
	
	@ClientString(id = 1802262, message = "Aghh! It's Lindvior! Lindvior's up in the sky!")
	public static NpcStringId AGHH_IT_S_LINDVIOR_LINDVIOR_S_UP_IN_THE_SKY;
	
	@ClientString(id = 1802263, message = "A dragon has begun attacking from north of the Town of Giran!")
	public static NpcStringId A_DRAGON_HAS_BEGUN_ATTACKING_FROM_NORTH_OF_THE_TOWN_OF_GIRAN;
	
	@ClientString(id = 1802264, message = "A dragon has begun attacking from north of Talking Island Village!")
	public static NpcStringId A_DRAGON_HAS_BEGUN_ATTACKING_FROM_NORTH_OF_TALKING_ISLAND_VILLAGE;
	
	@ClientString(id = 1802265, message = "A dragon has begun attacking from south of the Town of Aden!")
	public static NpcStringId A_DRAGON_HAS_BEGUN_ATTACKING_FROM_SOUTH_OF_THE_TOWN_OF_ADEN;
	
	@ClientString(id = 1802266, message = "Please…stop the dragon…")
	public static NpcStringId PLEASE_STOP_THE_DRAGON;
	
	@ClientString(id = 1802267, message = "Thank you! I'll be going to protect the village now that you're here!")
	public static NpcStringId THANK_YOU_I_LL_BE_GOING_TO_PROTECT_THE_VILLAGE_NOW_THAT_YOU_RE_HERE;
	
	@ClientString(id = 1802268, message = "It's good to have you here.")
	public static NpcStringId IT_S_GOOD_TO_HAVE_YOU_HERE;
	
	@ClientString(id = 1802269, message = "Well, how is this?!")
	public static NpcStringId WELL_HOW_IS_THIS;
	
	@ClientString(id = 1802270, message = "We did it! All right, I'll be on my way to protect the village!")
	public static NpcStringId WE_DID_IT_ALL_RIGHT_I_LL_BE_ON_MY_WAY_TO_PROTECT_THE_VILLAGE;
	
	@ClientString(id = 1802271, message = "We did it! We…defeated…")
	public static NpcStringId WE_DID_IT_WE_DEFEATED;
	
	@ClientString(id = 1802272, message = "Nornil Error. Nornil Error.")
	public static NpcStringId NORNIL_ERROR_NORNIL_ERROR;
	
	@ClientString(id = 1802273, message = "$s1 has obtained bonus XP for defeating a rare A-grade soldier.")
	public static NpcStringId S1_HAS_OBTAINED_BONUS_XP_FOR_DEFEATING_A_RARE_A_GRADE_SOLDIER;
	
	@ClientString(id = 1802274, message = "Nornil's Will Manifest. Nornil's Will Manifest.")
	public static NpcStringId NORNIL_S_WILL_MANIFEST_NORNIL_S_WILL_MANIFEST;
	
	@ClientString(id = 1802275, message = "Virus Intrusion. Virus Intrusion.")
	public static NpcStringId VIRUS_INTRUSION_VIRUS_INTRUSION;
	
	@ClientString(id = 1802276, message = "Malicious Code Entered. Malicious Code Entered.")
	public static NpcStringId MALICIOUS_CODE_ENTERED_MALICIOUS_CODE_ENTERED;
	
	@ClientString(id = 1802277, message = "$s1 has summoned Elite Soldiers through the Clone Generator.")
	public static NpcStringId S1_HAS_SUMMONED_ELITE_SOLDIERS_THROUGH_THE_CLONE_GENERATOR;
	
	@ClientString(id = 1802278, message = "Golems United. Power Increased. Golems United. Power Increased.")
	public static NpcStringId GOLEMS_UNITED_POWER_INCREASED_GOLEMS_UNITED_POWER_INCREASED;
	
	@ClientString(id = 1802279, message = "Some HP is restored.")
	public static NpcStringId SOME_HP_IS_RESTORED;
	
	@ClientString(id = 1802280, message = "HP is restored.")
	public static NpcStringId HP_IS_RESTORED;
	
	@ClientString(id = 1802281, message = "HP is greatly restored.")
	public static NpcStringId HP_IS_GREATLY_RESTORED;
	
	@ClientString(id = 1802282, message = "Some MP is restored.")
	public static NpcStringId SOME_MP_IS_RESTORED;
	
	@ClientString(id = 1802283, message = "MP is restored.")
	public static NpcStringId MP_IS_RESTORED;
	
	@ClientString(id = 1802284, message = "MP is greatly restored.")
	public static NpcStringId MP_IS_GREATLY_RESTORED;
	
	@ClientString(id = 1802285, message = "P. Atk. increases for some time.")
	public static NpcStringId P_ATK_INCREASES_FOR_SOME_TIME;
	
	@ClientString(id = 1802286, message = "M. Atk. increases for some time.")
	public static NpcStringId M_ATK_INCREASES_FOR_SOME_TIME;
	
	@ClientString(id = 1802287, message = "Atk. Spd. increases for some time.")
	public static NpcStringId ATK_SPD_INCREASES_FOR_SOME_TIME;
	
	@ClientString(id = 1802288, message = "Casting Spd. increases for some time.")
	public static NpcStringId CASTING_SPD_INCREASES_FOR_SOME_TIME;
	
	@ClientString(id = 1802289, message = "Critical Rate increases for some time.")
	public static NpcStringId CRITICAL_RATE_INCREASES_FOR_SOME_TIME;
	
	@ClientString(id = 1802290, message = "Speed increases for some time.")
	public static NpcStringId SPEED_INCREASES_FOR_SOME_TIME;
	
	@ClientString(id = 1802291, message = "Has a chance of absorbing damage inflicted on enemy as HP for some time.")
	public static NpcStringId HAS_A_CHANCE_OF_ABSORBING_DAMAGE_INFLICTED_ON_ENEMY_AS_HP_FOR_SOME_TIME;
	
	@ClientString(id = 1802292, message = "Your head is enlarged.")
	public static NpcStringId YOUR_HEAD_IS_ENLARGED;
	
	@ClientString(id = 1802293, message = "Critical Damage increases for some time.")
	public static NpcStringId CRITICAL_DAMAGE_INCREASES_FOR_SOME_TIME;
	
	@ClientString(id = 1802294, message = "Vitality is replenished with XP gain.")
	public static NpcStringId VITALITY_IS_REPLENISHED_WITH_XP_GAIN;
	
	@ClientString(id = 1802295, message = "Bow/Crossbow range increases for some time.")
	public static NpcStringId BOW_CROSSBOW_RANGE_INCREASES_FOR_SOME_TIME;
	
	@ClientString(id = 1802296, message = "P. Atk., Atk. Spd., and Critical Rate increase for some time.")
	public static NpcStringId P_ATK_ATK_SPD_AND_CRITICAL_RATE_INCREASE_FOR_SOME_TIME;
	
	@ClientString(id = 1802297, message = "HP and MP are greatly restored.")
	public static NpcStringId HP_AND_MP_ARE_GREATLY_RESTORED;
	
	@ClientString(id = 1802298, message = "The Golem Generator has appeared.")
	public static NpcStringId THE_GOLEM_GENERATOR_HAS_APPEARED;
	
	@ClientString(id = 1802299, message = "I'm the only merchant you'll find in this dangerous place!")
	public static NpcStringId I_M_THE_ONLY_MERCHANT_YOU_LL_FIND_IN_THIS_DANGEROUS_PLACE;
	
	@ClientString(id = 1802300, message = "Find the prison key carried by the Nerva Orcs, and get me out of here! I'll make it worth your while, I promise.")
	public static NpcStringId FIND_THE_PRISON_KEY_CARRIED_BY_THE_NERVA_ORCS_AND_GET_ME_OUT_OF_HERE_I_LL_MAKE_IT_WORTH_YOUR_WHILE_I_PROMISE;
	
	@ClientString(id = 1802301, message = "What… what's happening to my body? Argh..!")
	public static NpcStringId WHAT_WHAT_S_HAPPENING_TO_MY_BODY_ARGH;
	
	@ClientString(id = 1802302, message = "Help me, my comrades! Come out of your hiding and help me!")
	public static NpcStringId HELP_ME_MY_COMRADES_COME_OUT_OF_YOUR_HIDING_AND_HELP_ME;
	
	@ClientString(id = 1802303, message = "The Nerva's energy increases!")
	public static NpcStringId THE_NERVA_S_ENERGY_INCREASES;
	
	@ClientString(id = 1802304, message = "You have obtained Nerva's Temporary Prison Key.")
	public static NpcStringId YOU_HAVE_OBTAINED_NERVA_S_TEMPORARY_PRISON_KEY;
	
	@ClientString(id = 1802305, message = "You thought I'd be a pushover, didn't you?!")
	public static NpcStringId YOU_THOUGHT_I_D_BE_A_PUSHOVER_DIDN_T_YOU;
	
	@ClientString(id = 1802306, message = "HP is fully restored.")
	public static NpcStringId HP_IS_FULLY_RESTORED;
	
	@ClientString(id = 1802307, message = "You can only use the Vengeful Spirit once after the blood is all collected.")
	public static NpcStringId YOU_CAN_ONLY_USE_THE_VENGEFUL_SPIRIT_ONCE_AFTER_THE_BLOOD_IS_ALL_COLLECTED;
	
	@ClientString(id = 1802308, message = "Nearby monsters will gather at one point shortly.")
	public static NpcStringId NEARBY_MONSTERS_WILL_GATHER_AT_ONE_POINT_SHORTLY;
	
	@ClientString(id = 1802309, message = "More monsters will spawn shortly.")
	public static NpcStringId MORE_MONSTERS_WILL_SPAWN_SHORTLY;
	
	@ClientString(id = 1802310, message = "Weakened monsters will spawn shortly.")
	public static NpcStringId WEAKENED_MONSTERS_WILL_SPAWN_SHORTLY;
	
	@ClientString(id = 1802311, message = "Krogel has died. Parts of him are spawning into monsters.")
	public static NpcStringId KROGEL_HAS_DIED_PARTS_OF_HIM_ARE_SPAWNING_INTO_MONSTERS;
	
	@ClientString(id = 1802312, message = "You hear the voice of Etis van Etina.")
	public static NpcStringId YOU_HEAR_THE_VOICE_OF_ETIS_VAN_ETINA;
	
	@ClientString(id = 1802313, message = "HP is halfway restored.")
	public static NpcStringId HP_IS_HALFWAY_RESTORED;
	
	@ClientString(id = 1802314, message = "You must have looked hard for me to no avail. Hah!")
	public static NpcStringId YOU_MUST_HAVE_LOOKED_HARD_FOR_ME_TO_NO_AVAIL_HAH;
	
	@ClientString(id = 1802315, message = "Come now, exalted warriors of Raider's Crossroads!")
	public static NpcStringId COME_NOW_EXALTED_WARRIORS_OF_RAIDER_S_CROSSROADS;
	
	@ClientString(id = 1802316, message = "I am the commander in chief of this place!")
	public static NpcStringId I_AM_THE_COMMANDER_IN_CHIEF_OF_THIS_PLACE;
	
	@ClientString(id = 1802317, message = "Fearsome Watchman")
	public static NpcStringId FEARSOME_WATCHMAN;
	
	@ClientString(id = 1802318, message = "Monitor the Damage")
	public static NpcStringId MONITOR_THE_DAMAGE;
	
	@ClientString(id = 1802319, message = "Demonic System Activated")
	public static NpcStringId DEMONIC_SYSTEM_ACTIVATED;
	
	@ClientString(id = 1802320, message = "The Raider's Crossroads is a dangerous place. Please be careful.")
	public static NpcStringId THE_RAIDER_S_CROSSROADS_IS_A_DANGEROUS_PLACE_PLEASE_BE_CAREFUL;
	
	@ClientString(id = 1802321, message = "How is Treasure Dealer Kaysen doing?")
	public static NpcStringId HOW_IS_TREASURE_DEALER_KAYSEN_DOING;
	
	@ClientString(id = 1802322, message = "Danger Increasing. Danger Increasing.")
	public static NpcStringId DANGER_INCREASING_DANGER_INCREASING;
	
	@ClientString(id = 1802323, message = "So who's gonna take me on?!")
	public static NpcStringId SO_WHO_S_GONNA_TAKE_ME_ON;
	
	@ClientString(id = 1802324, message = "Ye who covet the power of Chaos…come to me.")
	public static NpcStringId YE_WHO_COVET_THE_POWER_OF_CHAOS_COME_TO_ME;
	
	@ClientString(id = 1802325, message = "Speak with the Agent of Chaos if you desire power.")
	public static NpcStringId SPEAK_WITH_THE_AGENT_OF_CHAOS_IF_YOU_DESIRE_POWER;
	
	@ClientString(id = 1802326, message = "Let's get rid of the Nerva Orcs, before they grow any stronger!")
	public static NpcStringId LET_S_GET_RID_OF_THE_NERVA_ORCS_BEFORE_THEY_GROW_ANY_STRONGER;
	
	@ClientString(id = 1802327, message = "Watchman")
	public static NpcStringId WATCHMAN;
	
	@ClientString(id = 1802328, message = "You took down the Nerva Orcs and got their Temporary Prison Key!")
	public static NpcStringId YOU_TOOK_DOWN_THE_NERVA_ORCS_AND_GOT_THEIR_TEMPORARY_PRISON_KEY;
	
	@ClientString(id = 1802330, message = "Incarnation")
	public static NpcStringId INCARNATION;
	
	@ClientString(id = 1802331, message = "Let me know when you're all ready.")
	public static NpcStringId LET_ME_KNOW_WHEN_YOU_RE_ALL_READY;
	
	@ClientString(id = 1802332, message = "All right! Let's move out!")
	public static NpcStringId ALL_RIGHT_LET_S_MOVE_OUT;
	
	@ClientString(id = 1802333, message = "Come on! Onto the next place!")
	public static NpcStringId COME_ON_ONTO_THE_NEXT_PLACE;
	
	@ClientString(id = 1802334, message = "Device destroyed! Let's go, onto the next!")
	public static NpcStringId DEVICE_DESTROYED_LET_S_GO_ONTO_THE_NEXT;
	
	@ClientString(id = 1802335, message = "Hey, you're not all bad. Let me know when you're ready.")
	public static NpcStringId HEY_YOU_RE_NOT_ALL_BAD_LET_ME_KNOW_WHEN_YOU_RE_READY;
	
	@ClientString(id = 1802336, message = "Ready? Let me know.")
	public static NpcStringId READY_LET_ME_KNOW;
	
	@ClientString(id = 1802337, message = "Something ominous in there… I hope you're really ready for this. Let me know.")
	public static NpcStringId SOMETHING_OMINOUS_IN_THERE_I_HOPE_YOU_RE_REALLY_READY_FOR_THIS_LET_ME_KNOW;
	
	@ClientString(id = 1802338, message = "Good work! You're really something!")
	public static NpcStringId GOOD_WORK_YOU_RE_REALLY_SOMETHING;
	
	@ClientString(id = 1802339, message = "I can't…let it end like this…")
	public static NpcStringId I_CAN_T_LET_IT_END_LIKE_THIS;
	
	@ClientString(id = 1802340, message = "Whew, that's more than I thought I could handle! I need to take cover for a bit,.")
	public static NpcStringId WHEW_THAT_S_MORE_THAN_I_THOUGHT_I_COULD_HANDLE_I_NEED_TO_TAKE_COVER_FOR_A_BIT;
	
	@ClientString(id = 1802341, message = "Dance time! Show me what ya got!")
	public static NpcStringId DANCE_TIME_SHOW_ME_WHAT_YA_GOT;
	
	@ClientString(id = 1802342, message = "Equip the 9th Anniversary Instrument. You can get it from Angel Cat right here.")
	public static NpcStringId EQUIP_THE_9TH_ANNIVERSARY_INSTRUMENT_YOU_CAN_GET_IT_FROM_ANGEL_CAT_RIGHT_HERE;
	
	@ClientString(id = 1802343, message = "Hide. Hide.")
	public static NpcStringId HIDE_HIDE;
	
	@ClientString(id = 1802344, message = "Aak, there goes the Hide!")
	public static NpcStringId AAK_THERE_GOES_THE_HIDE;
	
	@ClientString(id = 1802345, message = "Nooo…avenge me!")
	public static NpcStringId NOOO_AVENGE_ME;
	
	@ClientString(id = 1802346, message = "I come to help a comrade in need!")
	public static NpcStringId I_COME_TO_HELP_A_COMRADE_IN_NEED;
	
	@ClientString(id = 1802347, message = "You have obtained extra XP.")
	public static NpcStringId YOU_HAVE_OBTAINED_EXTRA_XP;
	
	@ClientString(id = 1802348, message = "I've never seen so many scholars and wizards in my life.")
	public static NpcStringId I_VE_NEVER_SEEN_SO_MANY_SCHOLARS_AND_WIZARDS_IN_MY_LIFE;
	
	@ClientString(id = 1802349, message = "It's not everyday you get to see such a sight, huh?")
	public static NpcStringId IT_S_NOT_EVERYDAY_YOU_GET_TO_SEE_SUCH_A_SIGHT_HUH;
	
	@ClientString(id = 1802350, message = "It just goes to show how important and difficult it is to activate the Seal Device!")
	public static NpcStringId IT_JUST_GOES_TO_SHOW_HOW_IMPORTANT_AND_DIFFICULT_IT_IS_TO_ACTIVATE_THE_SEAL_DEVICE;
	
	@ClientString(id = 1802351, message = "This has been too taxing on us all.")
	public static NpcStringId THIS_HAS_BEEN_TOO_TAXING_ON_US_ALL;
	
	@ClientString(id = 1802352, message = "We need a new soul that can maintain the seal.")
	public static NpcStringId WE_NEED_A_NEW_SOUL_THAT_CAN_MAINTAIN_THE_SEAL;
	
	@ClientString(id = 1802353, message = "It sure seems sturdy, but would this really be able to stop the sacrifices? Hm.")
	public static NpcStringId IT_SURE_SEEMS_STURDY_BUT_WOULD_THIS_REALLY_BE_ABLE_TO_STOP_THE_SACRIFICES_HM;
	
	@ClientString(id = 1802354, message = "We did make this Generator at Lady Jenna's suggestion, but…I'm still nervous.")
	public static NpcStringId WE_DID_MAKE_THIS_GENERATOR_AT_LADY_JENNA_S_SUGGESTION_BUT_I_M_STILL_NERVOUS;
	
	@ClientString(id = 1802355, message = "Rumors have it that Lindvior is headed this way.")
	public static NpcStringId RUMORS_HAVE_IT_THAT_LINDVIOR_IS_HEADED_THIS_WAY;
	
	@ClientString(id = 1802356, message = "Do you think he can be stopped?")
	public static NpcStringId DO_YOU_THINK_HE_CAN_BE_STOPPED;
	
	@ClientString(id = 1802357, message = "For now, we have no choice but to rely on these cannons placed around the Generators.")
	public static NpcStringId FOR_NOW_WE_HAVE_NO_CHOICE_BUT_TO_RELY_ON_THESE_CANNONS_PLACED_AROUND_THE_GENERATORS;
	
	@ClientString(id = 1802358, message = "May the gods watch over us.")
	public static NpcStringId MAY_THE_GODS_WATCH_OVER_US;
	
	@ClientString(id = 1802359, message = "The Generator is connected to the cannon!")
	public static NpcStringId THE_GENERATOR_IS_CONNECTED_TO_THE_CANNON;
	
	@ClientString(id = 1802360, message = "Hold onto the Generator to activate the Charge skill!")
	public static NpcStringId HOLD_ONTO_THE_GENERATOR_TO_ACTIVATE_THE_CHARGE_SKILL;
	
	@ClientString(id = 1802361, message = "You must charge the cannon to shoot down Lindvior!")
	public static NpcStringId YOU_MUST_CHARGE_THE_CANNON_TO_SHOOT_DOWN_LINDVIOR;
	
	@ClientString(id = 1802362, message = "Protect the Generator!")
	public static NpcStringId PROTECT_THE_GENERATOR;
	
	@ClientString(id = 1802363, message = "The Generator must not fall!")
	public static NpcStringId THE_GENERATOR_MUST_NOT_FALL;
	
	@ClientString(id = 1802364, message = "We will hold off Lindvior's minions!")
	public static NpcStringId WE_WILL_HOLD_OFF_LINDVIOR_S_MINIONS;
	
	@ClientString(id = 1802365, message = "Activate the Generator! Hurry!")
	public static NpcStringId ACTIVATE_THE_GENERATOR_HURRY;
	
	@ClientString(id = 1802366, message = "All 4 Generators must be activated.")
	public static NpcStringId ALL_4_GENERATORS_MUST_BE_ACTIVATED;
	
	@ClientString(id = 1802367, message = "My comrades and I will protect this place!")
	public static NpcStringId MY_COMRADES_AND_I_WILL_PROTECT_THIS_PLACE;
	
	@ClientString(id = 1802368, message = "Well, that's my cue. I'm off.")
	public static NpcStringId WELL_THAT_S_MY_CUE_I_M_OFF;
	
	@ClientString(id = 1802369, message = "Come, Kampf! Protect me!")
	public static NpcStringId COME_KAMPF_PROTECT_ME;
	
	@ClientString(id = 1802370, message = "Come, Belos! Protect me!")
	public static NpcStringId COME_BELOS_PROTECT_ME;
	
	@ClientString(id = 1802371, message = "Rawrghh!!")
	public static NpcStringId RAWRGHH;
	
	@ClientString(id = 1802372, message = "You dare interfere with Embryo? Surely you wish for death!")
	public static NpcStringId YOU_DARE_INTERFERE_WITH_EMBRYO_SURELY_YOU_WISH_FOR_DEATH;
	
	@ClientString(id = 1802373, message = "10 minutes until Antharas goes berserk!")
	public static NpcStringId MINUTES_UNTIL_ANTHARAS_GOES_BERSERK;
	
	@ClientString(id = 1802374, message = "Antharas is on a rampage. The terrains are twisting and turning.")
	public static NpcStringId ANTHARAS_IS_ON_A_RAMPAGE_THE_TERRAINS_ARE_TWISTING_AND_TURNING;
	
	@ClientString(id = 1802375, message = "10 minutes until Valakas goes berserk!")
	public static NpcStringId MINUTES_UNTIL_VALAKAS_GOES_BERSERK;
	
	@ClientString(id = 1802376, message = "Valakas is on a rampage. The land begins to burn.")
	public static NpcStringId VALAKAS_IS_ON_A_RAMPAGE_THE_LAND_BEGINS_TO_BURN;
	
	@ClientString(id = 1802377, message = "You can receive the Octavis Crystal from Lydia.")
	public static NpcStringId YOU_CAN_RECEIVE_THE_OCTAVIS_CRYSTAL_FROM_LYDIA;
	
	@ClientString(id = 1802378, message = "We just located Lindvior. Those who are willing to fight can do so at any time now.")
	public static NpcStringId WE_JUST_LOCATED_LINDVIOR_THOSE_WHO_ARE_WILLING_TO_FIGHT_CAN_DO_SO_AT_ANY_TIME_NOW;
	
	@ClientString(id = 1802379, message = "You need to use a skill just right on the Generator to obtain a scale. Talk to Jenna about it.")
	public static NpcStringId YOU_NEED_TO_USE_A_SKILL_JUST_RIGHT_ON_THE_GENERATOR_TO_OBTAIN_A_SCALE_TALK_TO_JENNA_ABOUT_IT;
	
	@ClientString(id = 1802380, message = "Write your wish for Lineage II in 2013 onto the Wish Tree!")
	public static NpcStringId WRITE_YOUR_WISH_FOR_LINEAGE_II_IN_2013_ONTO_THE_WISH_TREE;
	
	@ClientString(id = 1802381, message = "Do you want to change into a Santa Outfit? Write a wish on the Wish Tree!")
	public static NpcStringId DO_YOU_WANT_TO_CHANGE_INTO_A_SANTA_OUTFIT_WRITE_A_WISH_ON_THE_WISH_TREE;
	
	@ClientString(id = 1802382, message = "Happy Holidays!")
	public static NpcStringId HAPPY_HOLIDAYS;
	
	@ClientString(id = 1802383, message = "$s1 is giving out gifts. Everyone gather around!")
	public static NpcStringId S1_IS_GIVING_OUT_GIFTS_EVERYONE_GATHER_AROUND;
	
	@ClientString(id = 1802384, message = "You've already received a gift from me! I can only give one per person, you know.")
	public static NpcStringId YOU_VE_ALREADY_RECEIVED_A_GIFT_FROM_ME_I_CAN_ONLY_GIVE_ONE_PER_PERSON_YOU_KNOW;
	
	@ClientString(id = 1802385, message = "Here, a special gift for $s1. Take it, go on.")
	public static NpcStringId HERE_A_SPECIAL_GIFT_FOR_S1_TAKE_IT_GO_ON;
	
	@ClientString(id = 1802386, message = "Santa is all about Gangnam Style! Ho ho ho.")
	public static NpcStringId SANTA_IS_ALL_ABOUT_GANGNAM_STYLE_HO_HO_HO;
	
	@ClientString(id = 1802387, message = "Ho ho ho! Happy Holidays!")
	public static NpcStringId HO_HO_HO_HAPPY_HOLIDAYS;
	
	@ClientString(id = 1802388, message = "Hey there, kid! Try the Wintermelon Event, will you?")
	public static NpcStringId HEY_THERE_KID_TRY_THE_WINTERMELON_EVENT_WILL_YOU;
	
	@ClientString(id = 1802389, message = "How far did you look? Come on, try breaking a Wintermelon. You'll get a lot of rewards!")
	public static NpcStringId HOW_FAR_DID_YOU_LOOK_COME_ON_TRY_BREAKING_A_WINTERMELON_YOU_LL_GET_A_LOT_OF_REWARDS;
	
	@ClientString(id = 1802390, message = "Ow, my back. Isn't there a doctor around here?")
	public static NpcStringId OW_MY_BACK_ISN_T_THERE_A_DOCTOR_AROUND_HERE;
	
	@ClientString(id = 1802391, message = "I hope everyone has had a great year! Ho ho ho!")
	public static NpcStringId I_HOPE_EVERYONE_HAS_HAD_A_GREAT_YEAR_HO_HO_HO;
	
	@ClientString(id = 1802392, message = "Hope your new year is a happy one, everyone! Ho ho ho!")
	public static NpcStringId HOPE_YOUR_NEW_YEAR_IS_A_HAPPY_ONE_EVERYONE_HO_HO_HO;
	
	@ClientString(id = 1802393, message = "Is there no Mrs. Claus around here? I hate being lonely on Christmas…")
	public static NpcStringId IS_THERE_NO_MRS_CLAUS_AROUND_HERE_I_HATE_BEING_LONELY_ON_CHRISTMAS;
	
	@ClientString(id = 1802394, message = "You cannot teleport from the GM Consultation Service.")
	public static NpcStringId YOU_CANNOT_TELEPORT_FROM_THE_GM_CONSULTATION_SERVICE;
	
	@ClientString(id = 1802395, message = "Oooh!")
	public static NpcStringId OOOH3;
	
	@ClientString(id = 1802396, message = "Don't bother me... The foremen are watching...")
	public static NpcStringId DON_T_BOTHER_ME_THE_FOREMEN_ARE_WATCHING;
	
	@ClientString(id = 1802397, message = "Kukuku... Do I look like a slave?")
	public static NpcStringId KUKUKU_DO_I_LOOK_LIKE_A_SLAVE;
	
	@ClientString(id = 1802398, message = "Not bad. But it won't be so easy this time. Look carefully.")
	public static NpcStringId NOT_BAD_BUT_IT_WON_T_BE_SO_EASY_THIS_TIME_LOOK_CAREFULLY;
	
	@ClientString(id = 1802399, message = "It won't be easy competing with me!")
	public static NpcStringId IT_WON_T_BE_EASY_COMPETING_WITH_ME;
	
	@ClientString(id = 1802400, message = "Collect more power and continue with the experiments! Hiss...")
	public static NpcStringId COLLECT_MORE_POWER_AND_CONTINUE_WITH_THE_EXPERIMENTS_HISS;
	
	@ClientString(id = 1802401, message = "The power of the magic field has reached maximum capacity.")
	public static NpcStringId THE_POWER_OF_THE_MAGIC_FIELD_HAS_REACHED_MAXIMUM_CAPACITY;
	
	@ClientString(id = 1802402, message = "We were thankfully saved!")
	public static NpcStringId WE_WERE_THANKFULLY_SAVED;
	
	@ClientString(id = 1802403, message = "Beleth's magic has finally broken!")
	public static NpcStringId BELETH_S_MAGIC_HAS_FINALLY_BROKEN;
	
	@ClientString(id = 1802404, message = "Thank you. How can I repay this debt?!")
	public static NpcStringId THANK_YOU_HOW_CAN_I_REPAY_THIS_DEBT;
	
	@ClientString(id = 1802405, message = "Thank you for freeing me before I was turned into a shell of a monster!")
	public static NpcStringId THANK_YOU_FOR_FREEING_ME_BEFORE_I_WAS_TURNED_INTO_A_SHELL_OF_A_MONSTER;
	
	@ClientString(id = 1802406, message = "I returned to my original form!")
	public static NpcStringId I_RETURNED_TO_MY_ORIGINAL_FORM;
	
	@ClientString(id = 1802407, message = "I'm alive! I didn't become a monster!")
	public static NpcStringId I_M_ALIVE_I_DIDN_T_BECOME_A_MONSTER;
	
	@ClientString(id = 1802408, message = "The magic circle is operating normally.")
	public static NpcStringId THE_MAGIC_CIRCLE_IS_OPERATING_NORMALLY;
	
	@ClientString(id = 1802409, message = "The magic circle has become abnormal.")
	public static NpcStringId THE_MAGIC_CIRCLE_HAS_BECOME_ABNORMAL;
	
	@ClientString(id = 1802410, message = "Kefensis' Hallucination appeared.")
	public static NpcStringId KEFENSIS_HALLUCINATION_APPEARED;
	
	@ClientString(id = 1802411, message = "The Oasis Mirage appeared.")
	public static NpcStringId THE_OASIS_MIRAGE_APPEARED;
	
	@ClientString(id = 1802412, message = "The Oasis Mirage appeared.")
	public static NpcStringId THE_OASIS_MIRAGE_APPEARED2;
	
	@ClientString(id = 1802413, message = "A powerful and dangerous energy can be felt.")
	public static NpcStringId A_POWERFUL_AND_DANGEROUS_ENERGY_CAN_BE_FELT;
	
	@ClientString(id = 1802414, message = "$s1$s2 Kefensis' power can be felt.")
	public static NpcStringId S1_S2_KEFENSIS_POWER_CAN_BE_FELT;
	
	@ClientString(id = 1802415, message = "$s1, the Doom Shrieker will explode in 15 seconds.")
	public static NpcStringId S1_THE_DOOM_SHRIEKER_WILL_EXPLODE_IN_15_SECONDS;
	
	@ClientString(id = 1802416, message = "Abnormal Magic Circle")
	public static NpcStringId ABNORMAL_MAGIC_CIRCLE;
	
	@ClientString(id = 1802417, message = "Please return the experiment subjects to their original form!")
	public static NpcStringId PLEASE_RETURN_THE_EXPERIMENT_SUBJECTS_TO_THEIR_ORIGINAL_FORM;
	
	@ClientString(id = 1802418, message = "You may fall into danger with a sudden attack, so be careful!")
	public static NpcStringId YOU_MAY_FALL_INTO_DANGER_WITH_A_SUDDEN_ATTACK_SO_BE_CAREFUL;
	
	@ClientString(id = 1802419, message = "$s1, thank you for saving me. I wish you good fortune...")
	public static NpcStringId S1_THANK_YOU_FOR_SAVING_ME_I_WISH_YOU_GOOD_FORTUNE;
	
	@ClientString(id = 1802420, message = "Gasp... gasp...")
	public static NpcStringId GASP_GASP;
	
	@ClientString(id = 1802421, message = "My body... what happened?")
	public static NpcStringId MY_BODY_WHAT_HAPPENED;
	
	@ClientString(id = 1802422, message = "Please, take my power. It is not much, but…")
	public static NpcStringId PLEASE_TAKE_MY_POWER_IT_IS_NOT_MUCH_BUT;
	
	@ClientString(id = 1802423, message = "$s1! I will not forget this, you nasty little grunt!")
	public static NpcStringId S1_I_WILL_NOT_FORGET_THIS_YOU_NASTY_LITTLE_GRUNT;
	
	@ClientString(id = 1802425, message = "Too bad for you~~")
	public static NpcStringId TOO_BAD_FOR_YOU;
	
	@ClientString(id = 1802426, message = "Run to the Magic Circle!")
	public static NpcStringId RUN_TO_THE_MAGIC_CIRCLE;
	
	@ClientString(id = 1802427, message = "My incarnations! Appear and come to my aid!")
	public static NpcStringId MY_INCARNATIONS_APPEAR_AND_COME_TO_MY_AID;
	
	@ClientString(id = 1802428, message = "I don't feel any evil from them. I will take of them so please go and attack Beleth's incarnation!")
	public static NpcStringId I_DON_T_FEEL_ANY_EVIL_FROM_THEM_I_WILL_TAKE_OF_THEM_SO_PLEASE_GO_AND_ATTACK_BELETH_S_INCARNATION;
	
	@ClientString(id = 1802429, message = "I will go and gather reinforcements. In the meantime, please defeat the demons of Hellbound.")
	public static NpcStringId I_WILL_GO_AND_GATHER_REINFORCEMENTS_IN_THE_MEANTIME_PLEASE_DEFEAT_THE_DEMONS_OF_HELLBOUND;
	
	@ClientString(id = 1802430, message = "You have defeated the forces of evil while I was gathering reinforcements. I wish to give you a reward, so please come here.")
	public static NpcStringId YOU_HAVE_DEFEATED_THE_FORCES_OF_EVIL_WHILE_I_WAS_GATHERING_REINFORCEMENTS_I_WISH_TO_GIVE_YOU_A_REWARD_SO_PLEASE_COME_HERE;
	
	@ClientString(id = 1802431, message = "Kyaaah. Do you wish to see the true power of Beleth?!")
	public static NpcStringId KYAAAH_DO_YOU_WISH_TO_SEE_THE_TRUE_POWER_OF_BELETH;
	
	@ClientString(id = 1802432, message = "Kyaaah. Do you wish to see the true power of Darion, the Protector of the Steel Citadel?!")
	public static NpcStringId KYAAAH_DO_YOU_WISH_TO_SEE_THE_TRUE_POWER_OF_DARION_THE_PROTECTOR_OF_THE_STEEL_CITADEL;
	
	@ClientString(id = 1802433, message = "I will withdraw for now, but next time, it won't be so easy!")
	public static NpcStringId I_WILL_WITHDRAW_FOR_NOW_BUT_NEXT_TIME_IT_WON_T_BE_SO_EASY;
	
	@ClientString(id = 1802434, message = "I will show you the power of Beleth, the creator of Hellbound!")
	public static NpcStringId I_WILL_SHOW_YOU_THE_POWER_OF_BELETH_THE_CREATOR_OF_HELLBOUND;
	
	@ClientString(id = 1802435, message = "Come forth, servants of the Steel Citadel and show the power of the Protector Darion!")
	public static NpcStringId COME_FORTH_SERVANTS_OF_THE_STEEL_CITADEL_AND_SHOW_THE_POWER_OF_THE_PROTECTOR_DARION;
	
	@ClientString(id = 1802436, message = "There will soon be a powerful explosion.")
	public static NpcStringId THERE_WILL_SOON_BE_A_POWERFUL_EXPLOSION;
	
	@ClientString(id = 1802437, message = "I will show you the solidarity of Amos! Let all the Amos nearby gather their energy!")
	public static NpcStringId I_WILL_SHOW_YOU_THE_SOLIDARITY_OF_AMOS_LET_ALL_THE_AMOS_NEARBY_GATHER_THEIR_ENERGY;
	
	@ClientString(id = 1802438, message = "Oh! Over-hit!")
	public static NpcStringId OH_OVER_HIT;
	
	@ClientString(id = 1802439, message = "Tyrr... Argh! The disorder incarnation appears!")
	public static NpcStringId TYRR_ARGH_THE_DISORDER_INCARNATION_APPEARS;
	
	@ClientString(id = 1802440, message = "Magic Field Region 1 - stage 1 in progress")
	public static NpcStringId MAGIC_FIELD_REGION_1_STAGE_1_IN_PROGRESS;
	
	@ClientString(id = 1802441, message = "Magic Field Region 1 - stage 1 in progress")
	public static NpcStringId MAGIC_FIELD_REGION_1_STAGE_1_IN_PROGRESS2;
	
	@ClientString(id = 1802442, message = "Magic Field Region 2 - stage 1 in progress")
	public static NpcStringId MAGIC_FIELD_REGION_2_STAGE_1_IN_PROGRESS;
	
	@ClientString(id = 1802443, message = "Magic Field Region 2 - stage 2 in progress")
	public static NpcStringId MAGIC_FIELD_REGION_2_STAGE_2_IN_PROGRESS;
	
	@ClientString(id = 1802444, message = "Lavi's Boss")
	public static NpcStringId LAVI_S_BOSS;
	
	@ClientString(id = 1802445, message = "Lavisys's Boss")
	public static NpcStringId LAVISYS_S_BOSS;
	
	@ClientString(id = 1802446, message = "The next evolved monster reacts to $s1's power.")
	public static NpcStringId THE_NEXT_EVOLVED_MONSTER_REACTS_TO_S1_S_POWER;
	
	@ClientString(id = 1802447, message = "Foolish ones! Depart at once!")
	public static NpcStringId FOOLISH_ONES_DEPART_AT_ONCE;
	
	@ClientString(id = 1802448, message = "The highest evolved stage 4 monster appears.")
	public static NpcStringId THE_HIGHEST_EVOLVED_STAGE_4_MONSTER_APPEARS;
	
	@ClientString(id = 1802449, message = "The device activator was received from $s1.")
	public static NpcStringId THE_DEVICE_ACTIVATOR_WAS_RECEIVED_FROM_S1;
	
	@ClientString(id = 1802450, message = "One of the party members received the device activator.")
	public static NpcStringId ONE_OF_THE_PARTY_MEMBERS_RECEIVED_THE_DEVICE_ACTIVATOR;
	
	@ClientString(id = 1802451, message = "Leona Blackbird gave Beleth's Ring as a gift to $s1.")
	public static NpcStringId LEONA_BLACKBIRD_GAVE_BELETH_S_RING_AS_A_GIFT_TO_S1;
	
	@ClientString(id = 1802452, message = "You have acquired SP x 2.")
	public static NpcStringId YOU_HAVE_ACQUIRED_SP_X_2;
	
	@ClientString(id = 1802453, message = "You have acquired SP x 4.")
	public static NpcStringId YOU_HAVE_ACQUIRED_SP_X_4;
	
	@ClientString(id = 1802454, message = "You have acquired SP x 8.")
	public static NpcStringId YOU_HAVE_ACQUIRED_SP_X_8;
	
	@ClientString(id = 1802455, message = "You have acquired SP x 16.")
	public static NpcStringId YOU_HAVE_ACQUIRED_SP_X_16;
	
	@ClientString(id = 1802456, message = "$s1 acquired 32 times the skill points as a reward.")
	public static NpcStringId S1_ACQUIRED_32_TIMES_THE_SKILL_POINTS_AS_A_REWARD;
	
	@ClientString(id = 1802457, message = "Captured Caravan")
	public static NpcStringId CAPTURED_CARAVAN;
	
	@ClientString(id = 1802458, message = "Contemptible fools!")
	public static NpcStringId CONTEMPTIBLE_FOOLS;
	
	@ClientString(id = 1802459, message = "I will leave this place once day breaks. If you want a reward, hurry up and get it. Be careful not to lose the research rewards in the void!")
	public static NpcStringId I_WILL_LEAVE_THIS_PLACE_ONCE_DAY_BREAKS_IF_YOU_WANT_A_REWARD_HURRY_UP_AND_GET_IT_BE_CAREFUL_NOT_TO_LOSE_THE_RESEARCH_REWARDS_IN_THE_VOID;
	
	@ClientString(id = 1802460, message = "I'm not what I used to be.")
	public static NpcStringId I_M_NOT_WHAT_I_USED_TO_BE;
	
	@ClientString(id = 1802461, message = "You can use the jump board to get straight to the 2nd floor.")
	public static NpcStringId YOU_CAN_USE_THE_JUMP_BOARD_TO_GET_STRAIGHT_TO_THE_2ND_FLOOR;
	
	@ClientString(id = 1802462, message = "You can be teleported to each Seed if you volunteer. Why not try?")
	public static NpcStringId YOU_CAN_BE_TELEPORTED_TO_EACH_SEED_IF_YOU_VOLUNTEER_WHY_NOT_TRY;
	
	@ClientString(id = 1802463, message = "$s1 has summoned Goldeen.")
	public static NpcStringId S1_HAS_SUMMONED_GOLDEEN;
	
	@ClientString(id = 1802464, message = "I'll be going now.")
	public static NpcStringId I_LL_BE_GOING_NOW;
	
	@ClientString(id = 1802465, message = "You think your puny skills will work on me? Here's some petrification for every attempt!")
	public static NpcStringId YOU_THINK_YOUR_PUNY_SKILLS_WILL_WORK_ON_ME_HERE_S_SOME_PETRIFICATION_FOR_EVERY_ATTEMPT;
	
	@ClientString(id = 1802466, message = "Drat! We'll have to retreat!")
	public static NpcStringId DRAT_WE_LL_HAVE_TO_RETREAT;
	
	@ClientString(id = 1802467, message = "Stop the monsters that are trying to release the power of the tumor!")
	public static NpcStringId STOP_THE_MONSTERS_THAT_ARE_TRYING_TO_RELEASE_THE_POWER_OF_THE_TUMOR;
	
	@ClientString(id = 1802468, message = "Ekimus' minions have appeared in nearby chambers! Stop them!")
	public static NpcStringId EKIMUS_MINIONS_HAVE_APPEARED_IN_NEARBY_CHAMBERS_STOP_THEM;
	
	@ClientString(id = 1802469, message = "Tumors have appeared in nearby corridors. Stop them before Ekimus receives more power!")
	public static NpcStringId TUMORS_HAVE_APPEARED_IN_NEARBY_CORRIDORS_STOP_THEM_BEFORE_EKIMUS_RECEIVES_MORE_POWER;
	
	@ClientString(id = 1802470, message = "Stop the enemies approaching from the north to contact Ekimus!")
	public static NpcStringId STOP_THE_ENEMIES_APPROACHING_FROM_THE_NORTH_TO_CONTACT_EKIMUS;
	
	@ClientString(id = 1802471, message = "Stop the enemies approaching from the north and south to contact Ekimus!")
	public static NpcStringId STOP_THE_ENEMIES_APPROACHING_FROM_THE_NORTH_AND_SOUTH_TO_CONTACT_EKIMUS;
	
	@ClientString(id = 1802472, message = "All minions have been eliminated.")
	public static NpcStringId ALL_MINIONS_HAVE_BEEN_ELIMINATED;
	
	@ClientString(id = 1802473, message = "Ekimus prepares the 1st powerful spell with the magic sacrificed by the 1 remaining minion!")
	public static NpcStringId EKIMUS_PREPARES_THE_1ST_POWERFUL_SPELL_WITH_THE_MAGIC_SACRIFICED_BY_THE_1_REMAINING_MINION;
	
	@ClientString(id = 1802474, message = "Ekimus prepares the 2nd powerful spell with the magic sacrificed by the 2 remaining minions!")
	public static NpcStringId EKIMUS_PREPARES_THE_2ND_POWERFUL_SPELL_WITH_THE_MAGIC_SACRIFICED_BY_THE_2_REMAINING_MINIONS;
	
	@ClientString(id = 1802475, message = "Ekimus prepares the 3rd powerful spell with the magic sacrificed by the 3 remaining minions!")
	public static NpcStringId EKIMUS_PREPARES_THE_3RD_POWERFUL_SPELL_WITH_THE_MAGIC_SACRIFICED_BY_THE_3_REMAINING_MINIONS;
	
	@ClientString(id = 1802476, message = "Ekimus prepares the 4th powerful spell with the magic sacrificed by the 4 remaining minions!")
	public static NpcStringId EKIMUS_PREPARES_THE_4TH_POWERFUL_SPELL_WITH_THE_MAGIC_SACRIFICED_BY_THE_4_REMAINING_MINIONS;
	
	@ClientString(id = 1802477, message = "Ekimus prepares the 5th powerful spell with the magic sacrificed by the 5 remaining minions!")
	public static NpcStringId EKIMUS_PREPARES_THE_5TH_POWERFUL_SPELL_WITH_THE_MAGIC_SACRIFICED_BY_THE_5_REMAINING_MINIONS;
	
	@ClientString(id = 1802478, message = "Ekimus prepares the 6th powerful spell with the magic sacrificed by the 6 remaining minions!")
	public static NpcStringId EKIMUS_PREPARES_THE_6TH_POWERFUL_SPELL_WITH_THE_MAGIC_SACRIFICED_BY_THE_6_REMAINING_MINIONS;
	
	@ClientString(id = 1802479, message = "Mayday! Mayday! We need backup! We need backup!")
	public static NpcStringId MAYDAY_MAYDAY_WE_NEED_BACKUP_WE_NEED_BACKUP;
	
	@ClientString(id = 1802480, message = "Whoa, this is actualy kinda scary… I'll need to retreat after a few token blows!")
	public static NpcStringId WHOA_THIS_IS_ACTUALY_KINDA_SCARY_I_LL_NEED_TO_RETREAT_AFTER_A_FEW_TOKEN_BLOWS;
	
	@ClientString(id = 1802481, message = "You have gained 5 extra minutes for defeating the Hiding Tracker.")
	public static NpcStringId YOU_HAVE_GAINED_5_EXTRA_MINUTES_FOR_DEFEATING_THE_HIDING_TRACKER;
	
	@ClientString(id = 1802482, message = "The Yehan Brothers have emerged from the innermost hall.")
	public static NpcStringId THE_YEHAN_BROTHERS_HAVE_EMERGED_FROM_THE_INNERMOST_HALL;
	
	@ClientString(id = 1802483, message = "I'll be leaving soon. If you're going to trade fish, you'd better do it now.")
	public static NpcStringId I_LL_BE_LEAVING_SOON_IF_YOU_RE_GOING_TO_TRADE_FISH_YOU_D_BETTER_DO_IT_NOW;
	
	@ClientString(id = 1802484, message = "Protect 3 Suppressed Tumors.")
	public static NpcStringId PROTECT_3_SUPPRESSED_TUMORS;
	
	@ClientString(id = 1802485, message = "The surviving Evil Tumor is strengthening Ekimus' power!")
	public static NpcStringId THE_SURVIVING_EVIL_TUMOR_IS_STRENGTHENING_EKIMUS_POWER;
	
	@ClientString(id = 1802486, message = "10 minutes until the Raid Boss goes berserk.")
	public static NpcStringId MINUTES_UNTIL_THE_RAID_BOSS_GOES_BERSERK;
	
	@ClientString(id = 1802487, message = "So who caught me? No one? Left already, huh? Well, I'm outta here!")
	public static NpcStringId SO_WHO_CAUGHT_ME_NO_ONE_LEFT_ALREADY_HUH_WELL_I_M_OUTTA_HERE;
	
	@ClientString(id = 1802488, message = "You cannot teleport while in a transformed state.")
	public static NpcStringId YOU_CANNOT_TELEPORT_WHILE_IN_A_TRANSFORMED_STATE;
	
	@ClientString(id = 1802489, message = "Raid Herald Linea has appeared.")
	public static NpcStringId RAID_HERALD_LINEA_HAS_APPEARED;
	
	@ClientString(id = 1802490, message = "Frintezza is playing my victory song!")
	public static NpcStringId FRINTEZZA_IS_PLAYING_MY_VICTORY_SONG;
	
	@ClientString(id = 1802491, message = "Well, it's been nice knowing you. Shall we have the last dance?")
	public static NpcStringId WELL_IT_S_BEEN_NICE_KNOWING_YOU_SHALL_WE_HAVE_THE_LAST_DANCE;
	
	@ClientString(id = 1802492, message = "Back away! I will use Tauti's Cyclone.")
	public static NpcStringId BACK_AWAY_I_WILL_USE_TAUTI_S_CYCLONE;
	
	@ClientString(id = 1802493, message = "Magic and arrows, hm? Well, take a dose of Tauti's Typhoon!")
	public static NpcStringId MAGIC_AND_ARROWS_HM_WELL_TAKE_A_DOSE_OF_TAUTI_S_TYPHOON;
	
	@ClientString(id = 1802494, message = "Talk to Victory for rewards.")
	public static NpcStringId TALK_TO_VICTORY_FOR_REWARDS;
	
	@ClientString(id = 1802495, message = "Talk to Defeat for rewards.")
	public static NpcStringId TALK_TO_DEFEAT_FOR_REWARDS;
	
	@ClientString(id = 1802496, message = "The Repository is attacked! Fight! Fight!")
	public static NpcStringId THE_REPOSITORY_IS_ATTACKED_FIGHT_FIGHT;
	
	@ClientString(id = 1802497, message = "Argh! Who is…hiding there…?")
	public static NpcStringId ARGH_WHO_IS_HIDING_THERE;
	
	@ClientString(id = 1802498, message = "A smart Giant, huh? Well, hand it over! The Kartia's Seed is ours!")
	public static NpcStringId A_SMART_GIANT_HUH_WELL_HAND_IT_OVER_THE_KARTIA_S_SEED_IS_OURS;
	
	@ClientString(id = 1802499, message = "Kartia's Seed! Got it!")
	public static NpcStringId KARTIA_S_SEED_GOT_IT;
	
	@ClientString(id = 1802500, message = "You worthless Giant…curse you for eternity!")
	public static NpcStringId YOU_WORTHLESS_GIANT_CURSE_YOU_FOR_ETERNITY;
	
	@ClientString(id = 1802501, message = "Thank you…thank you for helping!")
	public static NpcStringId THANK_YOU_THANK_YOU_FOR_HELPING;
	
	@ClientString(id = 1802502, message = "Thank you. You are kind.")
	public static NpcStringId THANK_YOU_YOU_ARE_KIND;
	
	@ClientString(id = 1802503, message = "You are the keeper of the wind. I pledge my fealty to you!")
	public static NpcStringId YOU_ARE_THE_KEEPER_OF_THE_WIND_I_PLEDGE_MY_FEALTY_TO_YOU;
	
	@ClientString(id = 1802504, message = "Arghh!!")
	public static NpcStringId ARGHH;
	
	@ClientString(id = 1802505, message = "Did you all crawl out of the same hole? I will crush you all!")
	public static NpcStringId DID_YOU_ALL_CRAWL_OUT_OF_THE_SAME_HOLE_I_WILL_CRUSH_YOU_ALL;
	
	@ClientString(id = 1802506, message = "Check on Telesha.")
	public static NpcStringId CHECK_ON_TELESHA;
	
	@ClientString(id = 1802507, message = "Talk to the Mysterious Wizard.")
	public static NpcStringId TALK_TO_THE_MYSTERIOUS_WIZARD2;
	
	@ClientString(id = 1802508, message = "Talk to Kain Van Halter.")
	public static NpcStringId TALK_TO_KAIN_VAN_HALTER;
	
	@ClientString(id = 1802509, message = "Time to move onto the next place.")
	public static NpcStringId TIME_TO_MOVE_ONTO_THE_NEXT_PLACE;
	
	@ClientString(id = 1802510, message = "Talk to Queen Navari.")
	public static NpcStringId TALK_TO_QUEEN_NAVARI;
	
	@ClientString(id = 1802511, message = "Attack the Training Dummy.")
	public static NpcStringId ATTACK_THE_TRAINING_DUMMY;
	
	@ClientString(id = 1802512, message = "Queen Navari has sent a letter.nClick the question-mark icon to read.")
	public static NpcStringId QUEEN_NAVARI_HAS_SENT_A_LETTER_NCLICK_THE_QUESTION_MARK_ICON_TO_READ2;
	
	@ClientString(id = 1802513, message = "Talk to the Apprentice Adventurer's Guide.")
	public static NpcStringId TALK_TO_THE_APPRENTICE_ADVENTURER_S_GUIDE;
	
	@ClientString(id = 1802514, message = "Attack the monster!")
	public static NpcStringId ATTACK_THE_MONSTER;
	
	@ClientString(id = 1802515, message = "Talk to Magister Ayanthe.")
	public static NpcStringId TALK_TO_MAGISTER_AYANTHE;
	
	@ClientString(id = 1802516, message = "Fight using Skills!")
	public static NpcStringId FIGHT_USING_SKILLS;
	
	@ClientString(id = 1802517, message = "Talk to Master Katalin.")
	public static NpcStringId TALK_TO_MASTER_KATALIN;
	
	@ClientString(id = 1802518, message = "The southern part of Dragon Valley is much more dangerous than the north! Be careful.")
	public static NpcStringId THE_SOUTHERN_PART_OF_DRAGON_VALLEY_IS_MUCH_MORE_DANGEROUS_THAN_THE_NORTH_BE_CAREFUL;
	
	@ClientString(id = 1802519, message = "This place swarms with Dragons by day, and Undead by night.")
	public static NpcStringId THIS_PLACE_SWARMS_WITH_DRAGONS_BY_DAY_AND_UNDEAD_BY_NIGHT;
	
	@ClientString(id = 1802520, message = "Monsters appear here and there, confused by the Whirling Vortex.")
	public static NpcStringId MONSTERS_APPEAR_HERE_AND_THERE_CONFUSED_BY_THE_WHIRLING_VORTEX;
	
	@ClientString(id = 1802521, message = "Hey, that hurt! You just wait here, and I'll be back as a stronger dragon!")
	public static NpcStringId HEY_THAT_HURT_YOU_JUST_WAIT_HERE_AND_I_LL_BE_BACK_AS_A_STRONGER_DRAGON;
	
	@ClientString(id = 1802522, message = "I'm going to transform with the power of the vortex! You just watch!")
	public static NpcStringId I_M_GOING_TO_TRANSFORM_WITH_THE_POWER_OF_THE_VORTEX_YOU_JUST_WATCH;
	
	@ClientString(id = 1802523, message = "Read the Mysterious Letter in your inventory.")
	public static NpcStringId READ_THE_MYSTERIOUS_LETTER_IN_YOUR_INVENTORY;
	
	@ClientString(id = 1802524, message = "You can now access the Subclass! Talk to Raina again.")
	public static NpcStringId YOU_CAN_NOW_ACCESS_THE_SUBCLASS_TALK_TO_RAINA_AGAIN;
	
	@ClientString(id = 1802525, message = "You have reached Anghel Waterfall! Go inside the cave.")
	public static NpcStringId YOU_HAVE_REACHED_ANGHEL_WATERFALL_GO_INSIDE_THE_CAVE;
	
	@ClientString(id = 1802526, message = "Use the Fondest Heart in your inventory.")
	public static NpcStringId USE_THE_FONDEST_HEART_IN_YOUR_INVENTORY;
	
	@ClientString(id = 1802527, message = "Use the Fiercest Flame in your inventory.")
	public static NpcStringId USE_THE_FIERCEST_FLAME_IN_YOUR_INVENTORY;
	
	@ClientString(id = 1802528, message = "Use the Brightest Light in your inventory.")
	public static NpcStringId USE_THE_BRIGHTEST_LIGHT_IN_YOUR_INVENTORY;
	
	@ClientString(id = 1802529, message = "Use the Purest Soul in your inventory.")
	public static NpcStringId USE_THE_PUREST_SOUL_IN_YOUR_INVENTORY;
	
	@ClientString(id = 1802530, message = "Use the Clearest Water in your inventory.")
	public static NpcStringId USE_THE_CLEAREST_WATER_IN_YOUR_INVENTORY;
	
	@ClientString(id = 1802531, message = "You can now access the Dual Class! Talk to Raina in Talking Island Village for more details.")
	public static NpcStringId YOU_CAN_NOW_ACCESS_THE_DUAL_CLASS_TALK_TO_RAINA_IN_TALKING_ISLAND_VILLAGE_FOR_MORE_DETAILS;
	
	@ClientString(id = 1802532, message = "You will kneel to me!")
	public static NpcStringId YOU_WILL_KNEEL_TO_ME;
	
	@ClientString(id = 1802533, message = "The dead are calling…and I answer!")
	public static NpcStringId THE_DEAD_ARE_CALLING_AND_I_ANSWER;
	
	@ClientString(id = 1802534, message = "Kill, all you cursed soldiers! Kill everything!")
	public static NpcStringId KILL_ALL_YOU_CURSED_SOLDIERS_KILL_EVERYTHING;
	
	@ClientString(id = 1802535, message = "My wind barrier! How? Only the Ertheia can - wait - unless you are..?!")
	public static NpcStringId MY_WIND_BARRIER_HOW_ONLY_THE_ERTHEIA_CAN_WAIT_UNLESS_YOU_ARE;
	
	@ClientString(id = 1802536, message = "The Crusher is activated!")
	public static NpcStringId THE_CRUSHER_IS_ACTIVATED;
	
	@ClientString(id = 1802537, message = "$s1 object(s) destroyed!")
	public static NpcStringId S1_OBJECT_S_DESTROYED;
	
	@ClientString(id = 1802538, message = "The device ran out of magic.")
	public static NpcStringId THE_DEVICE_RAN_OUT_OF_MAGIC;
	
	@ClientString(id = 1802539, message = "The device ran out of magic. Try looking for another!")
	public static NpcStringId THE_DEVICE_RAN_OUT_OF_MAGIC_TRY_LOOKING_FOR_ANOTHER;
	
	@ClientString(id = 1802540, message = "Kill them! Don't let them get away with the fragment!")
	public static NpcStringId KILL_THEM_DON_T_LET_THEM_GET_AWAY_WITH_THE_FRAGMENT;
	
	@ClientString(id = 1802541, message = "To Queen Navari of Faeron!")
	public static NpcStringId TO_QUEEN_NAVARI_OF_FAERON;
	
	@ClientString(id = 1802542, message = "Cursed Ertheia! I will kill you all!")
	public static NpcStringId CURSED_ERTHEIA_I_WILL_KILL_YOU_ALL;
	
	@ClientString(id = 1802543, message = "You can gather more Intact Stakato Talons.")
	public static NpcStringId YOU_CAN_GATHER_MORE_INTACT_STAKATO_TALONS;
	
	@ClientString(id = 1802544, message = "You can gather more Monster Blood.")
	public static NpcStringId YOU_CAN_GATHER_MORE_MONSTER_BLOOD;
	
	@ClientString(id = 1802545, message = "You can gather more Powerful Dark Malice.")
	public static NpcStringId YOU_CAN_GATHER_MORE_POWERFUL_DARK_MALICE;
	
	@ClientString(id = 1802546, message = "You can gather more Varka's Badge: General.")
	public static NpcStringId YOU_CAN_GATHER_MORE_VARKA_S_BADGE_GENERAL;
	
	@ClientString(id = 1802547, message = "You can gather more Ketra's Badge: General.")
	public static NpcStringId YOU_CAN_GATHER_MORE_KETRA_S_BADGE_GENERAL;
	
	@ClientString(id = 1802548, message = "You can gather more Ragna Orc's Greater Amulets.")
	public static NpcStringId YOU_CAN_GATHER_MORE_RAGNA_ORC_S_GREATER_AMULETS;
	
	@ClientString(id = 1802549, message = "You can gather more Elite Mahum ID Tags.")
	public static NpcStringId YOU_CAN_GATHER_MORE_ELITE_MAHUM_ID_TAGS;
	
	@ClientString(id = 1802550, message = "You can gather more Precious Blood Stones.")
	public static NpcStringId YOU_CAN_GATHER_MORE_PRECIOUS_BLOOD_STONES;
	
	@ClientString(id = 1802551, message = "You can gather more Quality Mucrokian Hide.")
	public static NpcStringId YOU_CAN_GATHER_MORE_QUALITY_MUCROKIAN_HIDE;
	
	@ClientString(id = 1802552, message = "Alligator Island is a good hunting zone for Lv. 40 or above.")
	public static NpcStringId ALLIGATOR_ISLAND_IS_A_GOOD_HUNTING_ZONE_FOR_LV_40_OR_ABOVE;
	
	@ClientString(id = 1802553, message = "Outlaw Forest is a good hunting zone for Lv. 46 or above.")
	public static NpcStringId OUTLAW_FOREST_IS_A_GOOD_HUNTING_ZONE_FOR_LV_46_OR_ABOVE;
	
	@ClientString(id = 1802554, message = "Sea of Spores is a good hunting zone for Lv. 52 or above.")
	public static NpcStringId SEA_OF_SPORES_IS_A_GOOD_HUNTING_ZONE_FOR_LV_52_OR_ABOVE;
	
	@ClientString(id = 1802555, message = "Forsaken Plains ia a good hunting zone for Lv. 58 or above.")
	public static NpcStringId FORSAKEN_PLAINS_IA_A_GOOD_HUNTING_ZONE_FOR_LV_58_OR_ABOVE;
	
	@ClientString(id = 1802556, message = "Fields of Massacre is a good hunting zone for Lv. 61 or above.")
	public static NpcStringId FIELDS_OF_MASSACRE_IS_A_GOOD_HUNTING_ZONE_FOR_LV_61_OR_ABOVE;
	
	@ClientString(id = 1802557, message = "Swamp of Screams ia a good hunting zone for Lv. 65 or above.")
	public static NpcStringId SWAMP_OF_SCREAMS_IA_A_GOOD_HUNTING_ZONE_FOR_LV_65_OR_ABOVE;
	
	@ClientString(id = 1802558, message = "Forest of the Dead is a good hunting zone for Lv. 65 or above.")
	public static NpcStringId FOREST_OF_THE_DEAD_IS_A_GOOD_HUNTING_ZONE_FOR_LV_65_OR_ABOVE;
	
	@ClientString(id = 1802559, message = "Wall of Argos is a good hunting zone for Lv. 70 or above.")
	public static NpcStringId WALL_OF_ARGOS_IS_A_GOOD_HUNTING_ZONE_FOR_LV_70_OR_ABOVE;
	
	@ClientString(id = 1802560, message = "Varka Silenos Barracks is a good hunting zone for Lv. 76 or above.")
	public static NpcStringId VARKA_SILENOS_BARRACKS_IS_A_GOOD_HUNTING_ZONE_FOR_LV_76_OR_ABOVE;
	
	@ClientString(id = 1802561, message = "Ketra Orc Outpost is a good hunting zone for Lv. 76 or above.")
	public static NpcStringId KETRA_ORC_OUTPOST_IS_A_GOOD_HUNTING_ZONE_FOR_LV_76_OR_ABOVE;
	
	@ClientString(id = 1802562, message = "Den of Evil is a good hunting zone for Lv. 81 or above.")
	public static NpcStringId DEN_OF_EVIL_IS_A_GOOD_HUNTING_ZONE_FOR_LV_81_OR_ABOVE;
	
	@ClientString(id = 1802563, message = "Sel Mahum Training Grounds is a good hunting zone for Lv. 81 or above.")
	public static NpcStringId SEL_MAHUM_TRAINING_GROUNDS_IS_A_GOOD_HUNTING_ZONE_FOR_LV_81_OR_ABOVE;
	
	@ClientString(id = 1802564, message = "Plains of the Lizardmen is a good hunting zone for Lv. 81 or above.")
	public static NpcStringId PLAINS_OF_THE_LIZARDMEN_IS_A_GOOD_HUNTING_ZONE_FOR_LV_81_OR_ABOVE;
	
	@ClientString(id = 1802565, message = "Field of Silence and Field or Whispers are good hunting zones for Lv. 81 or above.")
	public static NpcStringId FIELD_OF_SILENCE_AND_FIELD_OR_WHISPERS_ARE_GOOD_HUNTING_ZONES_FOR_LV_81_OR_ABOVE;
	
	@ClientString(id = 1802566, message = "Grow stronger here until you receive the next letter from Kekropus at Lv. 46!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_KEKROPUS_AT_LV_46;
	
	@ClientString(id = 1802567, message = "Grow stronger here until you receive the next letter from Kekropus at Lv. 52!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_KEKROPUS_AT_LV_52;
	
	@ClientString(id = 1802568, message = "Grow stronger here until you receive the next letter from Kekropus at Lv. 58!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_KEKROPUS_AT_LV_58;
	
	@ClientString(id = 1802569, message = "Grow stronger here until you receive the next letter from Kekropus at Lv. 61!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_KEKROPUS_AT_LV_61;
	
	@ClientString(id = 1802570, message = "Grow stronger here until you receive the next letter from Kekropus at Lv. 65!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_KEKROPUS_AT_LV_65;
	
	@ClientString(id = 1802571, message = "Grow stronger here until you receive the next letter from Kekropus at Lv. 70!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_KEKROPUS_AT_LV_70;
	
	@ClientString(id = 1802572, message = "Grow stronger here until you receive the next letter from Kekropus at Lv. 76!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_KEKROPUS_AT_LV_76;
	
	@ClientString(id = 1802573, message = "Grow stronger here until you receive the next letter from Kekropus at Lv. 81!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_KEKROPUS_AT_LV_81;
	
	@ClientString(id = 1802574, message = "You have finished all of Kekropus' Letters! Grow stronger here until you receive letters from a minstrel at Lv. 85.")
	public static NpcStringId YOU_HAVE_FINISHED_ALL_OF_KEKROPUS_LETTERS_GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_LETTERS_FROM_A_MINSTREL_AT_LV_85;
	
	@ClientString(id = 1802575, message = "Dimensional Warp Lv. $s1")
	public static NpcStringId DIMENSIONAL_WARP_LV_S1;
	
	@ClientString(id = 1802576, message = "I thought I had warned you all.")
	public static NpcStringId I_THOUGHT_I_HAD_WARNED_YOU_ALL;
	
	@ClientString(id = 1802577, message = "No more talk! Only judgment!")
	public static NpcStringId NO_MORE_TALK_ONLY_JUDGMENT;
	
	@ClientString(id = 1802578, message = "Ultimate Defense!")
	public static NpcStringId ULTIMATE_DEFENSE;
	
	@ClientString(id = 1802579, message = "You will regret getting this far!")
	public static NpcStringId YOU_WILL_REGRET_GETTING_THIS_FAR;
	
	@ClientString(id = 1802580, message = "Crush them all! We must destroy the barrier!")
	public static NpcStringId CRUSH_THEM_ALL_WE_MUST_DESTROY_THE_BARRIER;
	
	@ClientString(id = 1802581, message = "Congratulations! You are now a Noblesse.")
	public static NpcStringId CONGRATULATIONS_YOU_ARE_NOW_A_NOBLESSE;
	
	@ClientString(id = 1802582, message = "Congratulations! You can now access a subclass.")
	public static NpcStringId CONGRATULATIONS_YOU_CAN_NOW_ACCESS_A_SUBCLASS;
	
	@ClientString(id = 1802583, message = "Congratulations! You can now access a dual class.")
	public static NpcStringId CONGRATULATIONS_YOU_CAN_NOW_ACCESS_A_DUAL_CLASS;
	
	@ClientString(id = 1802584, message = "Click on the Summoning Stone to talk to Lanya the water spirit!")
	public static NpcStringId CLICK_ON_THE_SUMMONING_STONE_TO_TALK_TO_LANYA_THE_WATER_SPIRIT;
	
	@ClientString(id = 1802585, message = "You can access the subclass through Raina in Talking Island Village.")
	public static NpcStringId YOU_CAN_ACCESS_THE_SUBCLASS_THROUGH_RAINA_IN_TALKING_ISLAND_VILLAGE;
	
	@ClientString(id = 1802586, message = "You can access the dual class through Raina in Talking Island Village.")
	public static NpcStringId YOU_CAN_ACCESS_THE_DUAL_CLASS_THROUGH_RAINA_IN_TALKING_ISLAND_VILLAGE;
	
	@ClientString(id = 1802587, message = "Try using the teleport scroll Levian gave you.")
	public static NpcStringId TRY_USING_THE_TELEPORT_SCROLL_LEVIAN_GAVE_YOU;
	
	@ClientString(id = 1802588, message = "Go where you feel the vengeful spirit's presence.")
	public static NpcStringId GO_WHERE_YOU_FEEL_THE_VENGEFUL_SPIRIT_S_PRESENCE;
	
	@ClientString(id = 1802589, message = "Talk to the Ghost of von Hellmann.")
	public static NpcStringId TALK_TO_THE_GHOST_OF_VON_HELLMANN;
	
	@ClientString(id = 1802590, message = "Try using the teleport scroll Levian gave you to go to Orc Barracks.")
	public static NpcStringId TRY_USING_THE_TELEPORT_SCROLL_LEVIAN_GAVE_YOU_TO_GO_TO_ORC_BARRACKS;
	
	@ClientString(id = 1802591, message = "Try using the teleport scroll Sylvain gave you to go to Cruma Tower.")
	public static NpcStringId TRY_USING_THE_TELEPORT_SCROLL_SYLVAIN_GAVE_YOU_TO_GO_TO_CRUMA_TOWER;
	
	@ClientString(id = 1802592, message = "Try using the teleport scroll Sylvain gave you to go to Cruma Tower.")
	public static NpcStringId TRY_USING_THE_TELEPORT_SCROLL_SYLVAIN_GAVE_YOU_TO_GO_TO_CRUMA_TOWER2;
	
	@ClientString(id = 1802593, message = "The surrounding energy has dissipated.")
	public static NpcStringId THE_SURROUNDING_ENERGY_HAS_DISSIPATED;
	
	@ClientString(id = 1802594, message = "Do you know who it is that you face? It is best that you run now.")
	public static NpcStringId DO_YOU_KNOW_WHO_IT_IS_THAT_YOU_FACE_IT_IS_BEST_THAT_YOU_RUN_NOW;
	
	@ClientString(id = 1802595, message = "I commend your tenacity in coming this far. But now, it ends.")
	public static NpcStringId I_COMMEND_YOUR_TENACITY_IN_COMING_THIS_FAR_BUT_NOW_IT_ENDS;
	
	@ClientString(id = 1802596, message = "Do you see this sword? The light that screams with the life of those it has killed?")
	public static NpcStringId DO_YOU_SEE_THIS_SWORD_THE_LIGHT_THAT_SCREAMS_WITH_THE_LIFE_OF_THOSE_IT_HAS_KILLED;
	
	@ClientString(id = 1802597, message = "Look into my eyes, and see what countless others have seen before their death.")
	public static NpcStringId LOOK_INTO_MY_EYES_AND_SEE_WHAT_COUNTLESS_OTHERS_HAVE_SEEN_BEFORE_THEIR_DEATH;
	
	@ClientString(id = 1802598, message = "I will show you what true power is!")
	public static NpcStringId I_WILL_SHOW_YOU_WHAT_TRUE_POWER_IS;
	
	@ClientString(id = 1802599, message = "You will need to surpass me eventually. But don't forget! Knowing how to flee is an important part of battle.")
	public static NpcStringId YOU_WILL_NEED_TO_SURPASS_ME_EVENTUALLY_BUT_DON_T_FORGET_KNOWING_HOW_TO_FLEE_IS_AN_IMPORTANT_PART_OF_BATTLE;
	
	@ClientString(id = 1802600, message = "You go no further.")
	public static NpcStringId YOU_GO_NO_FURTHER;
	
	@ClientString(id = 1802601, message = "You have a blessed scroll of resurrection, yes? Then run!")
	public static NpcStringId YOU_HAVE_A_BLESSED_SCROLL_OF_RESURRECTION_YES_THEN_RUN;
	
	@ClientString(id = 1802602, message = "Try using the teleport scroll Nerupa gave yout o go to the Valley of Saints.")
	public static NpcStringId TRY_USING_THE_TELEPORT_SCROLL_NERUPA_GAVE_YOUT_O_GO_TO_THE_VALLEY_OF_SAINTS;
	
	@ClientString(id = 1802603, message = "Try using the teleport scroll Innocentin gave you to go to the Forge of the Gods.")
	public static NpcStringId TRY_USING_THE_TELEPORT_SCROLL_INNOCENTIN_GAVE_YOU_TO_GO_TO_THE_FORGE_OF_THE_GODS;
	
	@ClientString(id = 1802604, message = "Try using the teleport scroll Vulcan gave you to go to Ivory Tower.")
	public static NpcStringId TRY_USING_THE_TELEPORT_SCROLL_VULCAN_GAVE_YOU_TO_GO_TO_IVORY_TOWER;
	
	@ClientString(id = 1802605, message = "Go to the Hot Springs")
	public static NpcStringId GO_TO_THE_HOT_SPRINGS;
	
	@ClientString(id = 1802606, message = "Fill the Empty Bottle with Water from the Hot Springs")
	public static NpcStringId FILL_THE_EMPTY_BOTTLE_WITH_WATER_FROM_THE_HOT_SPRINGS;
	
	@ClientString(id = 1802607, message = "Try using the teleport scroll Eva's Avatar gave you to go to the Hot Springs.")
	public static NpcStringId TRY_USING_THE_TELEPORT_SCROLL_EVA_S_AVATAR_GAVE_YOU_TO_GO_TO_THE_HOT_SPRINGS;
	
	@ClientString(id = 1802608, message = "A powerful monster has come to face you!")
	public static NpcStringId A_POWERFUL_MONSTER_HAS_COME_TO_FACE_YOU;
	
	@ClientString(id = 1802609, message = "$s1 seconds have been added to the instanced zone duration.")
	public static NpcStringId S1_SECONDS_HAVE_BEEN_ADDED_TO_THE_INSTANCED_ZONE_DURATION;
	
	@ClientString(id = 1802610, message = "Master Katalin has sent a letter.nClick the question-mark icon to read.")
	public static NpcStringId MASTER_KATALIN_HAS_SENT_A_LETTER_NCLICK_THE_QUESTION_MARK_ICON_TO_READ;
	
	@ClientString(id = 1802611, message = "Magister Ayanthe has sent a letter.nClick the question-mark icon to read.")
	public static NpcStringId MAGISTER_AYANTHE_HAS_SENT_A_LETTER_NCLICK_THE_QUESTION_MARK_ICON_TO_READ;
	
	@ClientString(id = 1802612, message = "Such monsters in a place like this…! Unbelievable!")
	public static NpcStringId SUCH_MONSTERS_IN_A_PLACE_LIKE_THIS_UNBELIEVABLE;
	
	@ClientString(id = 1802613, message = "Be on your toes!")
	public static NpcStringId BE_ON_YOUR_TOES;
	
	@ClientString(id = 1802614, message = "Leave this place to Kain.nGo to the next room.")
	public static NpcStringId LEAVE_THIS_PLACE_TO_KAIN_NGO_TO_THE_NEXT_ROOM;
	
	@ClientString(id = 1802615, message = "Try talking to Vorbos by the well.nYou can receive Queen Navari's next letter at Lv. 40!")
	public static NpcStringId TRY_TALKING_TO_VORBOS_BY_THE_WELL_NYOU_CAN_RECEIVE_QUEEN_NAVARI_S_NEXT_LETTER_AT_LV_40;
	
	@ClientString(id = 1802616, message = "Queen Navari calls.")
	public static NpcStringId QUEEN_NAVARI_CALLS;
	
	@ClientString(id = 1802617, message = "Ricky is not here.nTry searching another Kiku's Cave.")
	public static NpcStringId RICKY_IS_NOT_HERE_NTRY_SEARCHING_ANOTHER_KIKU_S_CAVE;
	
	@ClientString(id = 1802618, message = "Talk to Katalin to leave the training grounds.")
	public static NpcStringId TALK_TO_KATALIN_TO_LEAVE_THE_TRAINING_GROUNDS;
	
	@ClientString(id = 1802619, message = "Talk to Ayanthe to leave the training grounds.")
	public static NpcStringId TALK_TO_AYANTHE_TO_LEAVE_THE_TRAINING_GROUNDS;
	
	@ClientString(id = 1802620, message = "Check your equipment in your inventory.")
	public static NpcStringId CHECK_YOUR_EQUIPMENT_IN_YOUR_INVENTORY;
	
	@ClientString(id = 1802621, message = "I am loyal to you, master of the winds, and loyal I shall remain, if my very soul betrays me!")
	public static NpcStringId I_AM_LOYAL_TO_YOU_MASTER_OF_THE_WINDS_AND_LOYAL_I_SHALL_REMAIN_IF_MY_VERY_SOUL_BETRAYS_ME;
	
	@ClientString(id = 1802622, message = "n")
	public static NpcStringId N;
	
	@ClientString(id = 1802623, message = "Return to Raymond of the Town of Gludio.")
	public static NpcStringId RETURN_TO_RAYMOND_OF_THE_TOWN_OF_GLUDIO;
	
	@ClientString(id = 1802624, message = "Dimensional Imp!")
	public static NpcStringId DIMENSIONAL_IMP;
	
	@ClientString(id = 1802625, message = "Unworldly Imp!")
	public static NpcStringId UNWORLDLY_IMP;
	
	@ClientString(id = 1802626, message = "Abyssal Imp!")
	public static NpcStringId ABYSSAL_IMP;
	
	@ClientString(id = 1802627, message = "I'm outta here as soon as I find some treasure.")
	public static NpcStringId I_M_OUTTA_HERE_AS_SOON_AS_I_FIND_SOME_TREASURE;
	
	@ClientString(id = 1802628, message = "You didn't forget your escape scrolls, did you? Because I'm about to use this key to use my ultimate skill!")
	public static NpcStringId YOU_DIDN_T_FORGET_YOUR_ESCAPE_SCROLLS_DID_YOU_BECAUSE_I_M_ABOUT_TO_USE_THIS_KEY_TO_USE_MY_ULTIMATE_SKILL;
	
	@ClientString(id = 1802629, message = "My ultimate skill is…Lv. 36 Retreat!")
	public static NpcStringId MY_ULTIMATE_SKILL_IS_LV_36_RETREAT;
	
	@ClientString(id = 1802630, message = "I'm not helping you cuz I feel sorry or anything!")
	public static NpcStringId I_M_NOT_HELPING_YOU_CUZ_I_FEEL_SORRY_OR_ANYTHING;
	
	@ClientString(id = 1802631, message = "You should be honored that I'm helping you.")
	public static NpcStringId YOU_SHOULD_BE_HONORED_THAT_I_M_HELPING_YOU;
	
	@ClientString(id = 1802632, message = "I'm gonna tell Daddy if you keep hitting me!")
	public static NpcStringId I_M_GONNA_TELL_DADDY_IF_YOU_KEEP_HITTING_ME;
	
	@ClientString(id = 1802633, message = "Do you know who my daddy is? He's Archmage Venir! Ha!")
	public static NpcStringId DO_YOU_KNOW_WHO_MY_DADDY_IS_HE_S_ARCHMAGE_VENIR_HA;
	
	@ClientString(id = 1802634, message = "You can't die here! I didn't learn Resurrect yet!")
	public static NpcStringId YOU_CAN_T_DIE_HERE_I_DIDN_T_LEARN_RESURRECT_YET;
	
	@ClientString(id = 1802635, message = "Do you think I'll grow taller if I eat lots and lots?")
	public static NpcStringId DO_YOU_THINK_I_LL_GROW_TALLER_IF_I_EAT_LOTS_AND_LOTS;
	
	@ClientString(id = 1802636, message = "That guy Kain has a smarmy face.")
	public static NpcStringId THAT_GUY_KAIN_HAS_A_SMARMY_FACE;
	
	@ClientString(id = 1802637, message = "Giselle was such a sweet child.")
	public static NpcStringId GISELLE_WAS_SUCH_A_SWEET_CHILD;
	
	@ClientString(id = 1802638, message = "Don't let your guard down.")
	public static NpcStringId DON_T_LET_YOUR_GUARD_DOWN;
	
	@ClientString(id = 1802639, message = "That's a bratty kid if I've ever seen one. You two close?")
	public static NpcStringId THAT_S_A_BRATTY_KID_IF_I_VE_EVER_SEEN_ONE_YOU_TWO_CLOSE;
	
	@ClientString(id = 1802640, message = "Is that all? What a joke.")
	public static NpcStringId IS_THAT_ALL_WHAT_A_JOKE;
	
	@ClientString(id = 1802641, message = "Do you really think the Grail is here?")
	public static NpcStringId DO_YOU_REALLY_THINK_THE_GRAIL_IS_HERE;
	
	@ClientString(id = 1802642, message = "That's the monster that attacked Faeron. You're outmatched here. Go ahead; I'll catch up.")
	public static NpcStringId THAT_S_THE_MONSTER_THAT_ATTACKED_FAERON_YOU_RE_OUTMATCHED_HERE_GO_AHEAD_I_LL_CATCH_UP;
	
	@ClientString(id = 1802643, message = "Follow Ricky!")
	public static NpcStringId FOLLOW_RICKY;
	
	@ClientString(id = 1802644, message = "Ricky has found Leira.")
	public static NpcStringId RICKY_HAS_FOUND_LEIRA;
	
	@ClientString(id = 1802645, message = "Talk to Dolkin and leave the Karaphon Habitat.")
	public static NpcStringId TALK_TO_DOLKIN_AND_LEAVE_THE_KARAPHON_HABITAT;
	
	@ClientString(id = 1802646, message = "Leave this to me. Go!")
	public static NpcStringId LEAVE_THIS_TO_ME_GO;
	
	@ClientString(id = 1802647, message = "Go now! Kain can handle this.")
	public static NpcStringId GO_NOW_KAIN_CAN_HANDLE_THIS;
	
	@ClientString(id = 1802648, message = "Don't underestimate me!")
	public static NpcStringId DON_T_UNDERESTIMATE_ME;
	
	@ClientString(id = 1802649, message = "Everything will end soon!")
	public static NpcStringId EVERYTHING_WILL_END_SOON;
	
	@ClientString(id = 1802650, message = "Enjoy the present, because it won't last!")
	public static NpcStringId ENJOY_THE_PRESENT_BECAUSE_IT_WON_T_LAST;
	
	@ClientString(id = 1802651, message = "I'll show you what I'm made of!")
	public static NpcStringId I_LL_SHOW_YOU_WHAT_I_M_MADE_OF;
	
	@ClientString(id = 1802652, message = "Time for judgment, fools!")
	public static NpcStringId TIME_FOR_JUDGMENT_FOOLS;
	
	@ClientString(id = 1802653, message = "Run if you wish to live.")
	public static NpcStringId RUN_IF_YOU_WISH_TO_LIVE;
	
	@ClientString(id = 1802654, message = "A dragon appears, confused by the dimensional gale!")
	public static NpcStringId A_DRAGON_APPEARS_CONFUSED_BY_THE_DIMENSIONAL_GALE;
	
	@ClientString(id = 1802655, message = "An eerie red force has branded you with $s1.")
	public static NpcStringId AN_EERIE_RED_FORCE_HAS_BRANDED_YOU_WITH_S1;
	
	@ClientString(id = 1802656, message = "You can go to Underground Lv. 3 using the elevator in the back.")
	public static NpcStringId YOU_CAN_GO_TO_UNDERGROUND_LV_3_USING_THE_ELEVATOR_IN_THE_BACK;
	
	@ClientString(id = 1802657, message = "You cannot Liberate while your servitor is present.")
	public static NpcStringId YOU_CANNOT_LIBERATE_WHILE_YOUR_SERVITOR_IS_PRESENT;
	
	@ClientString(id = 1802658, message = "You cannot Liberate while transformed.")
	public static NpcStringId YOU_CANNOT_LIBERATE_WHILE_TRANSFORMED;
	
	@ClientString(id = 1802659, message = "Thank you. Deliver this Mark of Gratitude to Leo.")
	public static NpcStringId THANK_YOU_DELIVER_THIS_MARK_OF_GRATITUDE_TO_LEO;
	
	@ClientString(id = 1802660, message = "Hey, nice kick!")
	public static NpcStringId HEY_NICE_KICK;
	
	@ClientString(id = 1802661, message = "But aren't you being a little hard?")
	public static NpcStringId BUT_AREN_T_YOU_BEING_A_LITTLE_HARD;
	
	@ClientString(id = 1802662, message = "I remember being hit with something like this when I went hunting for Frintezza back in the days.")
	public static NpcStringId I_REMEMBER_BEING_HIT_WITH_SOMETHING_LIKE_THIS_WHEN_I_WENT_HUNTING_FOR_FRINTEZZA_BACK_IN_THE_DAYS;
	
	@ClientString(id = 1802663, message = "Tug-o-war? You can call me champi-ah!!")
	public static NpcStringId TUG_O_WAR_YOU_CAN_CALL_ME_CHAMPI_AH;
	
	@ClientString(id = 1802664, message = "I believe I can flyyy!")
	public static NpcStringId I_BELIEVE_I_CAN_FLYYY;
	
	@ClientString(id = 1802665, message = "Is that a love poke? Heh.")
	public static NpcStringId IS_THAT_A_LOVE_POKE_HEH;
	
	@ClientString(id = 1802666, message = "The war is not yet over.")
	public static NpcStringId THE_WAR_IS_NOT_YET_OVER;
	
	@ClientString(id = 1802667, message = "We will not turn back!")
	public static NpcStringId WE_WILL_NOT_TURN_BACK;
	
	@ClientString(id = 1802668, message = "This choice cannot be reversed.")
	public static NpcStringId THIS_CHOICE_CANNOT_BE_REVERSED;
	
	@ClientString(id = 1802669, message = "Lada has sent a letter.nClick the question-mark icon to read.")
	public static NpcStringId LADA_HAS_SENT_A_LETTER_NCLICK_THE_QUESTION_MARK_ICON_TO_READ;
	
	@ClientString(id = 1802670, message = "Grow stronger here until you receive the next letter from Queen Navari at Lv. 30!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_QUEEN_NAVARI_AT_LV_30;
	
	@ClientString(id = 1802671, message = "Grow stronger here until you receive the next letter from Queen Navari at Lv. 46!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_QUEEN_NAVARI_AT_LV_46;
	
	@ClientString(id = 1802672, message = "Grow stronger here until you receive the next letter from Queen Navari at Lv. 52!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_QUEEN_NAVARI_AT_LV_52;
	
	@ClientString(id = 1802673, message = "Grow stronger here until you receive the next letter from Queen Navari at Lv. 58!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_QUEEN_NAVARI_AT_LV_58;
	
	@ClientString(id = 1802674, message = "Grow stronger here until you receive the next letter from Queen Navari at Lv. 61!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_QUEEN_NAVARI_AT_LV_61;
	
	@ClientString(id = 1802675, message = "Grow stronger here until you receive the next letter from Queen Navari at Lv. 65!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_QUEEN_NAVARI_AT_LV_65;
	
	@ClientString(id = 1802676, message = "Grow stronger here until you receive the next letter from Queen Navari at Lv. 70!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_QUEEN_NAVARI_AT_LV_70;
	
	@ClientString(id = 1802677, message = "Grow stronger here until you receive the next letter from Queen Navari at Lv. 76!")
	public static NpcStringId GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_THE_NEXT_LETTER_FROM_QUEEN_NAVARI_AT_LV_76;
	
	@ClientString(id = 1802678, message = "You have finished all of Queen Navari's Letters! Grow stronger here until you receive letters from a minstrel at Lv. 85.")
	public static NpcStringId YOU_HAVE_FINISHED_ALL_OF_QUEEN_NAVARI_S_LETTERS_GROW_STRONGER_HERE_UNTIL_YOU_RECEIVE_LETTERS_FROM_A_MINSTREL_AT_LV_85;
	
	@ClientString(id = 1802679, message = "So good, neigh!")
	public static NpcStringId SO_GOOD_NEIGH;
	
	@ClientString(id = 1802680, message = "Thank you! What a delicious carrot.")
	public static NpcStringId THANK_YOU_WHAT_A_DELICIOUS_CARROT;
	
	@ClientString(id = 1802681, message = "So good…so good.")
	public static NpcStringId SO_GOOD_SO_GOOD;
	
	@ClientString(id = 1802682, message = "This is a true carrot! A true carrot!")
	public static NpcStringId THIS_IS_A_TRUE_CARROT_A_TRUE_CARROT;
	
	@ClientString(id = 1802683, message = "Just like home.")
	public static NpcStringId JUST_LIKE_HOME;
	
	@ClientString(id = 1802684, message = "You just don't understand.")
	public static NpcStringId YOU_JUST_DON_T_UNDERSTAND;
	
	@ClientString(id = 1802685, message = "A carrot is much better than sticks. But some horses do like sticks better…")
	public static NpcStringId A_CARROT_IS_MUCH_BETTER_THAN_STICKS_BUT_SOME_HORSES_DO_LIKE_STICKS_BETTER;
	
	@ClientString(id = 1802686, message = "Carrot! Carroticious! Carrogant!")
	public static NpcStringId CARROT_CARROTICIOUS_CARROGANT;
	
	@ClientString(id = 1802687, message = "So full. Carrots are great, though.")
	public static NpcStringId SO_FULL_CARROTS_ARE_GREAT_THOUGH;
	
	@ClientString(id = 1802688, message = "A full tummy is the key to happiness! That's my original quote.")
	public static NpcStringId A_FULL_TUMMY_IS_THE_KEY_TO_HAPPINESS_THAT_S_MY_ORIGINAL_QUOTE;
	
	@ClientString(id = 1802689, message = "Hey, wanna take this? A little gift from me.")
	public static NpcStringId HEY_WANNA_TAKE_THIS_A_LITTLE_GIFT_FROM_ME;
	
	@ClientString(id = 1802690, message = "I'm a little horsie, a beautiful little horsie.")
	public static NpcStringId I_M_A_LITTLE_HORSIE_A_BEAUTIFUL_LITTLE_HORSIE;
	
	@ClientString(id = 1802691, message = "Ever seen a cute little pegasus? White and all? It's my brother!")
	public static NpcStringId EVER_SEEN_A_CUTE_LITTLE_PEGASUS_WHITE_AND_ALL_IT_S_MY_BROTHER;
	
	@ClientString(id = 1802692, message = "I plan on going on a diet. Some day.")
	public static NpcStringId I_PLAN_ON_GOING_ON_A_DIET_SOME_DAY;
	
	@ClientString(id = 1802693, message = "I am still hungry!")
	public static NpcStringId I_AM_STILL_HUNGRY;
	
	@ClientString(id = 1802694, message = "I will take this carrot, please.")
	public static NpcStringId I_WILL_TAKE_THIS_CARROT_PLEASE;
	
	@ClientString(id = 1802695, message = "Come on, when is my carrot coming? I'm going to faint.")
	public static NpcStringId COME_ON_WHEN_IS_MY_CARROT_COMING_I_M_GOING_TO_FAINT;
	
	@ClientString(id = 1802696, message = "But I only eat carrots!")
	public static NpcStringId BUT_I_ONLY_EAT_CARROTS;
	
	@ClientString(id = 1802697, message = "You're not hiding a carrot behind your back, are you?")
	public static NpcStringId YOU_RE_NOT_HIDING_A_CARROT_BEHIND_YOUR_BACK_ARE_YOU;
	
	@ClientString(id = 1802698, message = "A carrot first! I will not say another word until I have a carrot.")
	public static NpcStringId A_CARROT_FIRST_I_WILL_NOT_SAY_ANOTHER_WORD_UNTIL_I_HAVE_A_CARROT;
	
	@ClientString(id = 1802699, message = "The carrot is the most perfect vegetable in existence. This is a fact.")
	public static NpcStringId THE_CARROT_IS_THE_MOST_PERFECT_VEGETABLE_IN_EXISTENCE_THIS_IS_A_FACT;
	
	@ClientString(id = 1802700, message = "Hey, hey, is that a carrot?")
	public static NpcStringId HEY_HEY_IS_THAT_A_CARROT;
	
	@ClientString(id = 1802701, message = "This pouch is for you!")
	public static NpcStringId THIS_POUCH_IS_FOR_YOU;
	
	@ClientString(id = 1802702, message = "My little gift to you. Ho ho!")
	public static NpcStringId MY_LITTLE_GIFT_TO_YOU_HO_HO;
	
	@ClientString(id = 1802703, message = "Happy New Year! Heh heh!")
	public static NpcStringId HAPPY_NEW_YEAR_HEH_HEH;
	
	@ClientString(id = 1802704, message = "The instanced zone will close soon!")
	public static NpcStringId THE_INSTANCED_ZONE_WILL_CLOSE_SOON;
	
	@ClientString(id = 1802705, message = "Nooo…not enough Vitality yet!")
	public static NpcStringId NOOO_NOT_ENOUGH_VITALITY_YET;
	
	@ClientString(id = 1802706, message = "Ow, ow! It hurts…")
	public static NpcStringId OW_OW_IT_HURTS;
	
	@ClientString(id = 1802707, message = "It tizzes and bungles! Tingles and…ow!")
	public static NpcStringId IT_TIZZES_AND_BUNGLES_TINGLES_AND_OW;
	
	@ClientString(id = 1802708, message = "Ergh, stop, stop…")
	public static NpcStringId ERGH_STOP_STOP;
	
	@ClientString(id = 1802709, message = "You think you can stop me from getting ow!")
	public static NpcStringId YOU_THINK_YOU_CAN_STOP_ME_FROM_GETTING_OW;
	
	@ClientString(id = 1802710, message = "Just wait a minute! I'm almost done getting all the Vitality!")
	public static NpcStringId JUST_WAIT_A_MINUTE_I_M_ALMOST_DONE_GETTING_ALL_THE_VITALITY;
	
	@ClientString(id = 1802711, message = "Hey, stop the violence and I'll give you the Vitality I got!")
	public static NpcStringId HEY_STOP_THE_VIOLENCE_AND_I_LL_GIVE_YOU_THE_VITALITY_I_GOT;
	
	@ClientString(id = 1802712, message = "Argh, you got me with your laser gun - or not! Haha, sucker!")
	public static NpcStringId ARGH_YOU_GOT_ME_WITH_YOUR_LASER_GUN_OR_NOT_HAHA_SUCKER;
	
	@ClientString(id = 1802713, message = "STABILIZING BLAZING SWAMP")
	public static NpcStringId STABILIZING_BLAZING_SWAMP;
	
	@ClientString(id = 1802714, message = "Defense Reduction")
	public static NpcStringId DEFENSE_REDUCTION;
	
	@ClientString(id = 1802715, message = "Ifrit's Wrath")
	public static NpcStringId IFRIT_S_WRATH;
	
	@ClientString(id = 1802716, message = "The monsters' defense has been reduced.")
	public static NpcStringId THE_MONSTERS_DEFENSE_HAS_BEEN_REDUCED;
	
	@ClientString(id = 1802717, message = "A strong monster has appeared.")
	public static NpcStringId A_STRONG_MONSTER_HAS_APPEARED;
	
	@ClientString(id = 1802718, message = "$s1, Blazing Seal!")
	public static NpcStringId S1_BLAZING_SEAL;
	
	@ClientString(id = 1802719, message = "$s1, dangerous figure recognized / Type - High")
	public static NpcStringId S1_DANGEROUS_FIGURE_RECOGNIZED_TYPE_HIGH;
	
	@ClientString(id = 1802720, message = "$s1, dangerous figure recognized / Type - Normal")
	public static NpcStringId S1_DANGEROUS_FIGURE_RECOGNIZED_TYPE_NORMAL;
	
	@ClientString(id = 1802721, message = "$s1, dangerous figure recognized / Type - Low")
	public static NpcStringId S1_DANGEROUS_FIGURE_RECOGNIZED_TYPE_LOW;
	
	@ClientString(id = 1802722, message = "We are the strongest! How dare you challenge us!")
	public static NpcStringId WE_ARE_THE_STRONGEST_HOW_DARE_YOU_CHALLENGE_US;
	
	@ClientString(id = 1802723, message = "It won't be easy to pull us apart!")
	public static NpcStringId IT_WON_T_BE_EASY_TO_PULL_US_APART;
	
	@ClientString(id = 1802724, message = "You measly creatures are so bothersome!")
	public static NpcStringId YOU_MEASLY_CREATURES_ARE_SO_BOTHERSOME;
	
	@ClientString(id = 1802725, message = "We are two in one! We won't lose to you!")
	public static NpcStringId WE_ARE_TWO_IN_ONE_WE_WON_T_LOSE_TO_YOU;
	
	@ClientString(id = 1802726, message = "How did this happen? Please avenge me!")
	public static NpcStringId HOW_DID_THIS_HAPPEN_PLEASE_AVENGE_ME;
	
	@ClientString(id = 1802727, message = "Ugh, I couldn't avenge my other half!")
	public static NpcStringId UGH_I_COULDN_T_AVENGE_MY_OTHER_HALF;
	
	@ClientString(id = 1802728, message = "We will never be separated!")
	public static NpcStringId WE_WILL_NEVER_BE_SEPARATED;
	
	@ClientString(id = 1802729, message = "We have the power to do anything, you lowly creatures!")
	public static NpcStringId WE_HAVE_THE_POWER_TO_DO_ANYTHING_YOU_LOWLY_CREATURES;
	
	@ClientString(id = 1802730, message = "Those weak attacks can't even scratch my armor!")
	public static NpcStringId THOSE_WEAK_ATTACKS_CAN_T_EVEN_SCRATCH_MY_ARMOR;
	
	@ClientString(id = 1802731, message = "I would never lose to some weaklings who can't even invest this much!")
	public static NpcStringId I_WOULD_NEVER_LOSE_TO_SOME_WEAKLINGS_WHO_CAN_T_EVEN_INVEST_THIS_MUCH;
	
	@ClientString(id = 1802732, message = "My dear other half, please avenge me.")
	public static NpcStringId MY_DEAR_OTHER_HALF_PLEASE_AVENGE_ME;
	
	@ClientString(id = 1802733, message = "I lost? Do I have to invest even more?")
	public static NpcStringId I_LOST_DO_I_HAVE_TO_INVEST_EVEN_MORE;
	
	@ClientString(id = 1802734, message = "You dare challenge me? How foolish!")
	public static NpcStringId YOU_DARE_CHALLENGE_ME_HOW_FOOLISH;
	
	@ClientString(id = 1802735, message = "You dare challenge me? You know no fear, indeed.")
	public static NpcStringId YOU_DARE_CHALLENGE_ME_YOU_KNOW_NO_FEAR_INDEED;
	
	@ClientString(id = 1802736, message = "Your arrogance will lead to even more pain.")
	public static NpcStringId YOUR_ARROGANCE_WILL_LEAD_TO_EVEN_MORE_PAIN;
	
	@ClientString(id = 1802737, message = "How foolish. Not knowing is a sin!")
	public static NpcStringId HOW_FOOLISH_NOT_KNOWING_IS_A_SIN;
	
	@ClientString(id = 1802738, message = "How did I lose to these worthless creatures?")
	public static NpcStringId HOW_DID_I_LOSE_TO_THESE_WORTHLESS_CREATURES;
	
	@ClientString(id = 1802739, message = "This pains me. I couldn’t even get my revenge!")
	public static NpcStringId THIS_PAINS_ME_I_COULDN_T_EVEN_GET_MY_REVENGE;
	
	@ClientString(id = 1802740, message = "I will show you what true strength is.")
	public static NpcStringId I_WILL_SHOW_YOU_WHAT_TRUE_STRENGTH_IS;
	
	@ClientString(id = 1802741, message = "Is this it?")
	public static NpcStringId IS_THIS_IT;
	
	@ClientString(id = 1802742, message = "Ha, ha, ha! Be afraid!")
	public static NpcStringId HA_HA_HA_BE_AFRAID;
	
	@ClientString(id = 1802743, message = "Ugh! I can't believe I was defeated like this.")
	public static NpcStringId UGH_I_CAN_T_BELIEVE_I_WAS_DEFEATED_LIKE_THIS;
	
	@ClientString(id = 1802744, message = "Wealth makes me stronger!")
	public static NpcStringId WEALTH_MAKES_ME_STRONGER;
	
	@ClientString(id = 1802745, message = "It feels like an ant is trying to bite me.")
	public static NpcStringId IT_FEELS_LIKE_AN_ANT_IS_TRYING_TO_BITE_ME;
	
	@ClientString(id = 1802746, message = "It doesn't hurt! Not at all!")
	public static NpcStringId IT_DOESN_T_HURT_NOT_AT_ALL;
	
	@ClientString(id = 1802747, message = "I won't just disappear like this!")
	public static NpcStringId I_WON_T_JUST_DISAPPEAR_LIKE_THIS;
	
	@ClientString(id = 1802748, message = "You dare challenge me? Are you ready for this?")
	public static NpcStringId YOU_DARE_CHALLENGE_ME_ARE_YOU_READY_FOR_THIS;
	
	@ClientString(id = 1802749, message = "Strength alone will not help you against me.")
	public static NpcStringId STRENGTH_ALONE_WILL_NOT_HELP_YOU_AGAINST_ME;
	
	@ClientString(id = 1802750, message = "With that kind of brain, you won't be able to surpass me!")
	public static NpcStringId WITH_THAT_KIND_OF_BRAIN_YOU_WON_T_BE_ABLE_TO_SURPASS_ME;
	
	@ClientString(id = 1802751, message = "How did this happen!")
	public static NpcStringId HOW_DID_THIS_HAPPEN;
	
	@ClientString(id = 1802752, message = "Learn what it is to fear!")
	public static NpcStringId LEARN_WHAT_IT_IS_TO_FEAR;
	
	@ClientString(id = 1802753, message = "Ha, ha, ha! That won't be enough to defeat me!")
	public static NpcStringId HA_HA_HA_THAT_WON_T_BE_ENOUGH_TO_DEFEAT_ME;
	
	@ClientString(id = 1802754, message = "Bow down to my power!")
	public static NpcStringId BOW_DOWN_TO_MY_POWER;
	
	@ClientString(id = 1802755, message = "I lost to this? This can't be happening!")
	public static NpcStringId I_LOST_TO_THIS_THIS_CAN_T_BE_HAPPENING;
	
	@ClientString(id = 1802756, message = "You see how my wallet expanded tenfold?")
	public static NpcStringId YOU_SEE_HOW_MY_WALLET_EXPANDED_TENFOLD;
	
	@ClientString(id = 1802757, message = "How much did your weapon cost?")
	public static NpcStringId HOW_MUCH_DID_YOUR_WEAPON_COST;
	
	@ClientString(id = 1802758, message = "Your equipment looks so cheap.")
	public static NpcStringId YOUR_EQUIPMENT_LOOKS_SO_CHEAP;
	
	@ClientString(id = 1802759, message = "Don't even think for a second that I'll let this go easily!")
	public static NpcStringId DON_T_EVEN_THINK_FOR_A_SECOND_THAT_I_LL_LET_THIS_GO_EASILY;
	
	@ClientString(id = 1802760, message = "No matter how much you use your brains, you won't be able to find me!")
	public static NpcStringId NO_MATTER_HOW_MUCH_YOU_USE_YOUR_BRAINS_YOU_WON_T_BE_ABLE_TO_FIND_ME;
	
	@ClientString(id = 1802761, message = "You will never be at my level with those brains!")
	public static NpcStringId YOU_WILL_NEVER_BE_AT_MY_LEVEL_WITH_THOSE_BRAINS;
	
	@ClientString(id = 1802762, message = "This is our difference!")
	public static NpcStringId THIS_IS_OUR_DIFFERENCE;
	
	@ClientString(id = 1802763, message = "You think this will be enough to defeat me? Not a chance!")
	public static NpcStringId YOU_THINK_THIS_WILL_BE_ENOUGH_TO_DEFEAT_ME_NOT_A_CHANCE;
	
	@ClientString(id = 1810363, message = "You are too far from the corpse.")
	public static NpcStringId YOU_ARE_TOO_FAR_FROM_THE_CORPSE;
	
	@ClientString(id = 1810364, message = "Grudge of Ye Sagira victims have been relieved with your tears.")
	public static NpcStringId GRUDGE_OF_YE_SAGIRA_VICTIMS_HAVE_BEEN_RELIEVED_WITH_YOUR_TEARS;
	
	@ClientString(id = 1811000, message = "Fighter")
	public static NpcStringId FIGHTER;
	
	@ClientString(id = 1811001, message = "Warrior")
	public static NpcStringId WARRIOR;
	
	@ClientString(id = 1811002, message = "Gladiator")
	public static NpcStringId GLADIATOR;
	
	@ClientString(id = 1811003, message = "Warlord")
	public static NpcStringId WARLORD;
	
	@ClientString(id = 1811004, message = "Knight")
	public static NpcStringId KNIGHT;
	
	@ClientString(id = 1811005, message = "Paladin")
	public static NpcStringId PALADIN;
	
	@ClientString(id = 1811006, message = "Dark Avenger")
	public static NpcStringId DARK_AVENGER;
	
	@ClientString(id = 1811007, message = "Rogue")
	public static NpcStringId ROGUE;
	
	@ClientString(id = 1811008, message = "Treasure Hunter")
	public static NpcStringId TREASURE_HUNTER;
	
	@ClientString(id = 1811009, message = "Hawkeye")
	public static NpcStringId HAWKEYE;
	
	@ClientString(id = 1811010, message = "Mage")
	public static NpcStringId MAGE;
	
	@ClientString(id = 1811011, message = "Wizard")
	public static NpcStringId WIZARD;
	
	@ClientString(id = 1811012, message = "Sorcerer")
	public static NpcStringId SORCERER;
	
	@ClientString(id = 1811013, message = "Necromancer")
	public static NpcStringId NECROMANCER;
	
	@ClientString(id = 1811014, message = "Warlock")
	public static NpcStringId WARLOCK;
	
	@ClientString(id = 1811015, message = "Cleric")
	public static NpcStringId CLERIC;
	
	@ClientString(id = 1811016, message = "Bishop")
	public static NpcStringId BISHOP;
	
	@ClientString(id = 1811017, message = "Prophet")
	public static NpcStringId PROPHET;
	
	@ClientString(id = 1811018, message = "Elven Fighter")
	public static NpcStringId ELVEN_FIGHTER;
	
	@ClientString(id = 1811019, message = "Elven Knight")
	public static NpcStringId ELVEN_KNIGHT;
	
	@ClientString(id = 1811020, message = "Temple Knight")
	public static NpcStringId TEMPLE_KNIGHT;
	
	@ClientString(id = 1811021, message = "Swordsinger")
	public static NpcStringId SWORDSINGER;
	
	@ClientString(id = 1811022, message = "Elven Scout")
	public static NpcStringId ELVEN_SCOUT;
	
	@ClientString(id = 1811023, message = "Plains Walker")
	public static NpcStringId PLAINS_WALKER;
	
	@ClientString(id = 1811024, message = "Silver Ranger")
	public static NpcStringId SILVER_RANGER;
	
	@ClientString(id = 1811025, message = "Elven Mage")
	public static NpcStringId ELVEN_MAGE;
	
	@ClientString(id = 1811026, message = "Elven Wizard")
	public static NpcStringId ELVEN_WIZARD;
	
	@ClientString(id = 1811027, message = "Spell Singer")
	public static NpcStringId SPELL_SINGER;
	
	@ClientString(id = 1811028, message = "Elemental Summoner")
	public static NpcStringId ELEMENTAL_SUMMONER;
	
	@ClientString(id = 1811029, message = "Oracle")
	public static NpcStringId ORACLE;
	
	@ClientString(id = 1811030, message = "Elder")
	public static NpcStringId ELDER;
	
	@ClientString(id = 1811031, message = "Dark Fighter")
	public static NpcStringId DARK_FIGHTER;
	
	@ClientString(id = 1811032, message = "Palace Knight")
	public static NpcStringId PALACE_KNIGHT;
	
	@ClientString(id = 1811033, message = "Shillien Knight")
	public static NpcStringId SHILLIEN_KNIGHT;
	
	@ClientString(id = 1811034, message = "Blade Dancer")
	public static NpcStringId BLADE_DANCER;
	
	@ClientString(id = 1811035, message = "Assassin")
	public static NpcStringId ASSASSIN;
	
	@ClientString(id = 1811036, message = "Abyss Walker")
	public static NpcStringId ABYSS_WALKER;
	
	@ClientString(id = 1811037, message = "Phantom Ranger")
	public static NpcStringId PHANTOM_RANGER;
	
	@ClientString(id = 1811038, message = "Dark Mage")
	public static NpcStringId DARK_MAGE;
	
	@ClientString(id = 1811039, message = "Dark Wizard")
	public static NpcStringId DARK_WIZARD;
	
	@ClientString(id = 1811040, message = "Spellhowler")
	public static NpcStringId SPELLHOWLER;
	
	@ClientString(id = 1811041, message = "Phantom Summoner")
	public static NpcStringId PHANTOM_SUMMONER;
	
	@ClientString(id = 1811042, message = "Shillien Oracle")
	public static NpcStringId SHILLIEN_ORACLE;
	
	@ClientString(id = 1811043, message = "Shillien Elder")
	public static NpcStringId SHILLIEN_ELDER;
	
	@ClientString(id = 1811044, message = "Orc Fighter")
	public static NpcStringId ORC_FIGHTER;
	
	@ClientString(id = 1811045, message = "Orc Raider")
	public static NpcStringId ORC_RAIDER;
	
	@ClientString(id = 1811046, message = "Destroyer")
	public static NpcStringId DESTROYER;
	
	@ClientString(id = 1811047, message = "Orc Monk")
	public static NpcStringId ORC_MONK;
	
	@ClientString(id = 1811048, message = "Tyrant")
	public static NpcStringId TYRANT;
	
	@ClientString(id = 1811049, message = "Orc Mage")
	public static NpcStringId ORC_MAGE;
	
	@ClientString(id = 1811050, message = "Orc Shaman")
	public static NpcStringId ORC_SHAMAN;
	
	@ClientString(id = 1811051, message = "Overlord")
	public static NpcStringId OVERLORD;
	
	@ClientString(id = 1811052, message = "Warcryer")
	public static NpcStringId WARCRYER;
	
	@ClientString(id = 1811053, message = "Dwarven Fighter")
	public static NpcStringId DWARVEN_FIGHTER;
	
	@ClientString(id = 1811054, message = "Scavenger")
	public static NpcStringId SCAVENGER;
	
	@ClientString(id = 1811055, message = "Bounty Hunter")
	public static NpcStringId BOUNTY_HUNTER;
	
	@ClientString(id = 1811056, message = "Artisan")
	public static NpcStringId ARTISAN;
	
	@ClientString(id = 1811057, message = "Warsmith")
	public static NpcStringId WARSMITH;
	
	@ClientString(id = 1811088, message = "Duelist")
	public static NpcStringId DUELIST;
	
	@ClientString(id = 1811089, message = "Dreadnought")
	public static NpcStringId DREADNOUGHT;
	
	@ClientString(id = 1811090, message = "Phoenix Knight")
	public static NpcStringId PHOENIX_KNIGHT;
	
	@ClientString(id = 1811091, message = "Hell Knight")
	public static NpcStringId HELL_KNIGHT;
	
	@ClientString(id = 1811092, message = "Sagittarius")
	public static NpcStringId SAGITTARIUS;
	
	@ClientString(id = 1811093, message = "Adventurer")
	public static NpcStringId ADVENTURER;
	
	@ClientString(id = 1811094, message = "Archmage")
	public static NpcStringId ARCHMAGE;
	
	@ClientString(id = 1811095, message = "Soultaker")
	public static NpcStringId SOULTAKER;
	
	@ClientString(id = 1811096, message = "Arcana Lord")
	public static NpcStringId ARCANA_LORD;
	
	@ClientString(id = 1811097, message = "Cardinal")
	public static NpcStringId CARDINAL;
	
	@ClientString(id = 1811098, message = "Hierophant")
	public static NpcStringId HIEROPHANT;
	
	@ClientString(id = 1811099, message = "Eva's Templar")
	public static NpcStringId EVA_S_TEMPLAR;
	
	@ClientString(id = 1811100, message = "Sword Muse")
	public static NpcStringId SWORD_MUSE;
	
	@ClientString(id = 1811101, message = "Wind Rider")
	public static NpcStringId WIND_RIDER;
	
	@ClientString(id = 1811102, message = "Moonlight Sentinel")
	public static NpcStringId MOONLIGHT_SENTINEL;
	
	@ClientString(id = 1811103, message = "Mystic Muse")
	public static NpcStringId MYSTIC_MUSE;
	
	@ClientString(id = 1811104, message = "Elemental Master")
	public static NpcStringId ELEMENTAL_MASTER;
	
	@ClientString(id = 1811105, message = "Eva's Saint")
	public static NpcStringId EVA_S_SAINT;
	
	@ClientString(id = 1811106, message = "Shillien Templar")
	public static NpcStringId SHILLIEN_TEMPLAR;
	
	@ClientString(id = 1811107, message = "Spectral Dancer")
	public static NpcStringId SPECTRAL_DANCER;
	
	@ClientString(id = 1811108, message = "Ghost Hunter")
	public static NpcStringId GHOST_HUNTER;
	
	@ClientString(id = 1811109, message = "Ghost Sentinel")
	public static NpcStringId GHOST_SENTINEL;
	
	@ClientString(id = 1811110, message = "Storm Screamer")
	public static NpcStringId STORM_SCREAMER;
	
	@ClientString(id = 1811111, message = "Spectral Master")
	public static NpcStringId SPECTRAL_MASTER;
	
	@ClientString(id = 1811112, message = "Shillien Saint")
	public static NpcStringId SHILLIEN_SAINT;
	
	@ClientString(id = 1811113, message = "Titan")
	public static NpcStringId TITAN;
	
	@ClientString(id = 1811114, message = "Grand Khavatari")
	public static NpcStringId GRAND_KHAVATARI;
	
	@ClientString(id = 1811115, message = "Dominator")
	public static NpcStringId DOMINATOR;
	
	@ClientString(id = 1811116, message = "Doomcryer")
	public static NpcStringId DOOMCRYER;
	
	@ClientString(id = 1811117, message = "Fortune Seeker")
	public static NpcStringId FORTUNE_SEEKER;
	
	@ClientString(id = 1811118, message = "Maestro")
	public static NpcStringId MAESTRO;
	
	@ClientString(id = 1811123, message = "Kamael Soldier")
	public static NpcStringId KAMAEL_SOLDIER;
	
	@ClientString(id = 1811124, message = "Kamael Soldier")
	public static NpcStringId KAMAEL_SOLDIER2;
	
	@ClientString(id = 1811125, message = "Trooper")
	public static NpcStringId TROOPER;
	
	@ClientString(id = 1811126, message = "Warder")
	public static NpcStringId WARDER;
	
	@ClientString(id = 1811127, message = "Berserker")
	public static NpcStringId BERSERKER;
	
	@ClientString(id = 1811128, message = "Soul Breaker")
	public static NpcStringId SOUL_BREAKER;
	
	@ClientString(id = 1811129, message = "Soul Breaker")
	public static NpcStringId SOUL_BREAKER2;
	
	@ClientString(id = 1811130, message = "Arbalester")
	public static NpcStringId ARBALESTER;
	
	@ClientString(id = 1811131, message = "Doombringer")
	public static NpcStringId DOOMBRINGER;
	
	@ClientString(id = 1811132, message = "Soul Hound")
	public static NpcStringId SOUL_HOUND;
	
	@ClientString(id = 1811133, message = "Soul Hound")
	public static NpcStringId SOUL_HOUND2;
	
	@ClientString(id = 1811134, message = "Trickster")
	public static NpcStringId TRICKSTER;
	
	@ClientString(id = 1811135, message = "Inspector")
	public static NpcStringId INSPECTOR;
	
	@ClientString(id = 1811136, message = "Judicator")
	public static NpcStringId JUDICATOR;
	
	@ClientString(id = 1811137, message = "Who's there? If you disturb the temper of the great Land Dragon Antharas, I will never forgive you!")
	public static NpcStringId WHO_S_THERE_IF_YOU_DISTURB_THE_TEMPER_OF_THE_GREAT_LAND_DRAGON_ANTHARAS_I_WILL_NEVER_FORGIVE_YOU;
	
	@ClientString(id = 1811138, message = "Istina's soul stone starts powerfully illuminating in red.")
	public static NpcStringId ISTINA_S_SOUL_STONE_STARTS_POWERFULLY_ILLUMINATING_IN_RED;
	
	@ClientString(id = 1811139, message = "Istina's soul stone starts powerfully illuminating in blue.")
	public static NpcStringId ISTINA_S_SOUL_STONE_STARTS_POWERFULLY_ILLUMINATING_IN_BLUE;
	
	@ClientString(id = 1811140, message = "Istina's soul stone starts powerfully illuminating in green.")
	public static NpcStringId ISTINA_S_SOUL_STONE_STARTS_POWERFULLY_ILLUMINATING_IN_GREEN;
	
	@ClientString(id = 1811141, message = "Istina gets furious and recklessly crazy.")
	public static NpcStringId ISTINA_GETS_FURIOUS_AND_RECKLESSLY_CRAZY;
	
	@ClientString(id = 1811142, message = "There is still lots of time left. Do not stop here.")
	public static NpcStringId THERE_IS_STILL_LOTS_OF_TIME_LEFT_DO_NOT_STOP_HERE;
	
	@ClientString(id = 1811143, message = "Put in some more effort. Your effort can be a much needed help for us.")
	public static NpcStringId PUT_IN_SOME_MORE_EFFORT_YOUR_EFFORT_CAN_BE_A_MUCH_NEEDED_HELP_FOR_US;
	
	@ClientString(id = 1811144, message = "Istina calls her creatures with tremendous anger.")
	public static NpcStringId ISTINA_CALLS_HER_CREATURES_WITH_TREMENDOUS_ANGER;
	
	@ClientString(id = 1811145, message = "Maguen stole $s1 pieces of bio-energy residue.")
	public static NpcStringId MAGUEN_STOLE_S1_PIECES_OF_BIO_ENERGY_RESIDUE;
	
	@ClientString(id = 1811146, message = "Soldier Tie received $s1 pieces of bio-energy residue.")
	public static NpcStringId SOLDIER_TIE_RECEIVED_S1_PIECES_OF_BIO_ENERGY_RESIDUE;
	
	@ClientString(id = 1811147, message = "Maguen gets surprised and gives $s1 pieces of bio-energy residue.")
	public static NpcStringId MAGUEN_GETS_SURPRISED_AND_GIVES_S1_PIECES_OF_BIO_ENERGY_RESIDUE;
	
	@ClientString(id = 1811148, message = "Istina spreads the reflecting protective sheet.")
	public static NpcStringId ISTINA_SPREADS_THE_REFLECTING_PROTECTIVE_SHEET;
	
	@ClientString(id = 1811149, message = "You need to find Escape Device. Re-entry is not allowed once you've left the instant zone.")
	public static NpcStringId YOU_NEED_TO_FIND_ESCAPE_DEVICE_RE_ENTRY_IS_NOT_ALLOWED_ONCE_YOU_VE_LEFT_THE_INSTANT_ZONE;
	
	@ClientString(id = 1811150, message = "The location of the escape device is moved.")
	public static NpcStringId THE_LOCATION_OF_THE_ESCAPE_DEVICE_IS_MOVED;
	
	@ClientString(id = 1811151, message = "When the time bomb is activated, a door opens somewhere.")
	public static NpcStringId WHEN_THE_TIME_BOMB_IS_ACTIVATED_A_DOOR_OPENS_SOMEWHERE;
	
	@ClientString(id = 1811152, message = "The door is open, somebody needs to stay to watch the time bomb.")
	public static NpcStringId THE_DOOR_IS_OPEN_SOMEBODY_NEEDS_TO_STAY_TO_WATCH_THE_TIME_BOMB;
	
	@ClientString(id = 1811153, message = "Spezion's status will only change when exposed to light.")
	public static NpcStringId SPEZION_S_STATUS_WILL_ONLY_CHANGE_WHEN_EXPOSED_TO_LIGHT;
	
	@ClientString(id = 1811154, message = "The door is open.")
	public static NpcStringId THE_DOOR_IS_OPEN;
	
	@ClientString(id = 1811155, message = "After $s1 seconds, the charging magic Ballistas starts.")
	public static NpcStringId AFTER_S1_SECONDS_THE_CHARGING_MAGIC_BALLISTAS_STARTS;
	
	@ClientString(id = 1811156, message = "Powerful acidic energy is erupting from Istina's body.")
	public static NpcStringId POWERFUL_ACIDIC_ENERGY_IS_ERUPTING_FROM_ISTINA_S_BODY;
	
	@ClientString(id = 1811157, message = "Soldier Tie absorbed reproductive energy from your body and converted $s1 pieces of bio-energy.")
	public static NpcStringId SOLDIER_TIE_ABSORBED_REPRODUCTIVE_ENERGY_FROM_YOUR_BODY_AND_CONVERTED_S1_PIECES_OF_BIO_ENERGY;
	
	@ClientString(id = 1811158, message = "Pfft! This one is a miss. I wasted too much strength. Whoa~!")
	public static NpcStringId PFFT_THIS_ONE_IS_A_MISS_I_WASTED_TOO_MUCH_STRENGTH_WHOA;
	
	@ClientString(id = 1811159, message = "It's not very easy to gather this much residue, but you can probably gather more.")
	public static NpcStringId IT_S_NOT_VERY_EASY_TO_GATHER_THIS_MUCH_RESIDUE_BUT_YOU_CAN_PROBABLY_GATHER_MORE;
	
	@ClientString(id = 1811160, message = "I'm sorry, but we can't accept residue with questionable sources. Let me destroy this.")
	public static NpcStringId I_M_SORRY_BUT_WE_CAN_T_ACCEPT_RESIDUE_WITH_QUESTIONABLE_SOURCES_LET_ME_DESTROY_THIS;
	
	@ClientString(id = 1811161, message = "I must find gardener and beg for forgiveness...")
	public static NpcStringId I_MUST_FIND_GARDENER_AND_BEG_FOR_FORGIVENESS;
	
	@ClientString(id = 1811162, message = "Ah... this is the Windmill Hill")
	public static NpcStringId AH_THIS_IS_THE_WINDMILL_HILL;
	
	@ClientString(id = 1811163, message = "We're collecting Seeker of Aden!")
	public static NpcStringId WE_RE_COLLECTING_SEEKER_OF_ADEN;
	
	@ClientString(id = 1811164, message = "This is Ruins of Agony, where Kanilov is...")
	public static NpcStringId THIS_IS_RUINS_OF_AGONY_WHERE_KANILOV_IS;
	
	@ClientString(id = 1811165, message = "What shall we do?")
	public static NpcStringId WHAT_SHALL_WE_DO;
	
	@ClientString(id = 1811166, message = "This is the Ruins of Agony, where Poslof is...")
	public static NpcStringId THIS_IS_THE_RUINS_OF_AGONY_WHERE_POSLOF_IS;
	
	@ClientString(id = 1811167, message = "It's dangerous, so be careful...")
	public static NpcStringId IT_S_DANGEROUS_SO_BE_CAREFUL;
	
	@ClientString(id = 1811168, message = "Attack!")
	public static NpcStringId ATTACK2;
	
	@ClientString(id = 1811169, message = "Follow me!")
	public static NpcStringId FOLLOW_ME3;
	
	@ClientString(id = 1811170, message = "Thank you...!")
	public static NpcStringId THANK_YOU;
	
	@ClientString(id = 1811171, message = "Release the residents in pain and eliminate the totem...!!")
	public static NpcStringId RELEASE_THE_RESIDENTS_IN_PAIN_AND_ELIMINATE_THE_TOTEM;
	
	@ClientString(id = 1811172, message = "Start charging mana ballista!")
	public static NpcStringId START_CHARGING_MANA_BALLISTA;
	
	@ClientString(id = 1811173, message = "Taste the future of your class! And learn!")
	public static NpcStringId TASTE_THE_FUTURE_OF_YOUR_CLASS_AND_LEARN;
	
	@ClientString(id = 1811174, message = "I will show you my real power later. This is it for today.")
	public static NpcStringId I_WILL_SHOW_YOU_MY_REAL_POWER_LATER_THIS_IS_IT_FOR_TODAY;
	
	@ClientString(id = 1811175, message = "Berserker of Istina has been disabled.")
	public static NpcStringId BERSERKER_OF_ISTINA_HAS_BEEN_DISABLED;
	
	@ClientString(id = 1811176, message = "$s1, locked away in the prison.")
	public static NpcStringId S1_LOCKED_AWAY_IN_THE_PRISON;
	
	@ClientString(id = 1811177, message = "Flame area")
	public static NpcStringId FLAME_AREA;
	
	@ClientString(id = 1811178, message = "Istina shoots powerful acid into the air.")
	public static NpcStringId ISTINA_SHOOTS_POWERFUL_ACID_INTO_THE_AIR;
	
	@ClientString(id = 1811179, message = "Received Regeneration Energy!")
	public static NpcStringId RECEIVED_REGENERATION_ENERGY;
	
	@ClientString(id = 1811180, message = "Lesser Giant's Essence")
	public static NpcStringId LESSER_GIANT_S_ESSENCE;
	
	@ClientString(id = 1811181, message = "Let's get this over with!")
	public static NpcStringId LET_S_GET_THIS_OVER_WITH;
	
	@ClientString(id = 1811182, message = "Taste my strength!")
	public static NpcStringId TASTE_MY_STRENGTH;
	
	@ClientString(id = 1811183, message = "Pretty good!")
	public static NpcStringId PRETTY_GOOD2;
	
	@ClientString(id = 1811184, message = "This is the real beginning!")
	public static NpcStringId THIS_IS_THE_REAL_BEGINNING;
	
	@ClientString(id = 1811185, message = "It's surprising that you last until now!")
	public static NpcStringId IT_S_SURPRISING_THAT_YOU_LAST_UNTIL_NOW;
	
	@ClientString(id = 1811186, message = "I will not lose!")
	public static NpcStringId I_WILL_NOT_LOSE;
	
	@ClientString(id = 1811187, message = "Istina's Mark shines above the head.")
	public static NpcStringId ISTINA_S_MARK_SHINES_ABOVE_THE_HEAD;
	
	@ClientString(id = 1811188, message = "The scouts should be alive...")
	public static NpcStringId THE_SCOUTS_SHOULD_BE_ALIVE;
	
	@ClientString(id = 1811189, message = "Don't get your hopes up.")
	public static NpcStringId DON_T_GET_YOUR_HOPES_UP;
	
	@ClientString(id = 1811190, message = "Don't say that!")
	public static NpcStringId DON_T_SAY_THAT;
	
	@ClientString(id = 1811191, message = "So many people have fallen here... Special things don't happen.")
	public static NpcStringId SO_MANY_PEOPLE_HAVE_FALLEN_HERE_SPECIAL_THINGS_DON_T_HAPPEN;
	
	@ClientString(id = 1811192, message = "Ahh... there's no more help to deploy...")
	public static NpcStringId AHH_THERE_S_NO_MORE_HELP_TO_DEPLOY;
	
	@ClientString(id = 1811193, message = "We're looking for someone to help the Antharas Raid!")
	public static NpcStringId WE_RE_LOOKING_FOR_SOMEONE_TO_HELP_THE_ANTHARAS_RAID;
	
	@ClientString(id = 1811194, message = "Behind you! The enemy is ambushing you!")
	public static NpcStringId BEHIND_YOU_THE_ENEMY_IS_AMBUSHING_YOU;
	
	@ClientString(id = 1811195, message = "Kill the guy messing with the Electric Device!")
	public static NpcStringId KILL_THE_GUY_MESSING_WITH_THE_ELECTRIC_DEVICE;
	
	@ClientString(id = 1811196, message = "Focus on attacking the guy in the room!")
	public static NpcStringId FOCUS_ON_ATTACKING_THE_GUY_IN_THE_ROOM;
	
	@ClientString(id = 1811197, message = "If Terain dies, the mission will fail.")
	public static NpcStringId IF_TERAIN_DIES_THE_MISSION_WILL_FAIL;
	
	@ClientString(id = 1811198, message = "Heavens will know no greater wrath!")
	public static NpcStringId HEAVENS_WILL_KNOW_NO_GREATER_WRATH;
	
	@ClientString(id = 1811199, message = "Mark of Belis can be acquired from enemies.nUse them in the Belis Verification System")
	public static NpcStringId MARK_OF_BELIS_CAN_BE_ACQUIRED_FROM_ENEMIES_NUSE_THEM_IN_THE_BELIS_VERIFICATION_SYSTEM;
	
	@ClientString(id = 1811200, message = "Electronic device has been destroyed.")
	public static NpcStringId ELECTRONIC_DEVICE_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 1811201, message = "When the combat gets difficult, I will help.")
	public static NpcStringId WHEN_THE_COMBAT_GETS_DIFFICULT_I_WILL_HELP;
	
	@ClientString(id = 1811202, message = "$s1. The key does not match, so we're in trouble!")
	public static NpcStringId S1_THE_KEY_DOES_NOT_MATCH_SO_WE_RE_IN_TROUBLE;
	
	@ClientString(id = 1811203, message = "When the cursed aura of totem comes near...!")
	public static NpcStringId WHEN_THE_CURSED_AURA_OF_TOTEM_COMES_NEAR;
	
	@ClientString(id = 1811204, message = "If I can't become a gardener... I will stay a traitor!")
	public static NpcStringId IF_I_CAN_T_BECOME_A_GARDENER_I_WILL_STAY_A_TRAITOR;
	
	@ClientString(id = 1811205, message = "This is the beginning to get into the deepest parts of Seed of Annihilation.")
	public static NpcStringId THIS_IS_THE_BEGINNING_TO_GET_INTO_THE_DEEPEST_PARTS_OF_SEED_OF_ANNIHILATION;
	
	@ClientString(id = 1811206, message = "Our allies went all the way to Istina's Habitat and challenging him.")
	public static NpcStringId OUR_ALLIES_WENT_ALL_THE_WAY_TO_ISTINA_S_HABITAT_AND_CHALLENGING_HIM;
	
	@ClientString(id = 1811207, message = "We're charging to weaken him. Showing them what we're made of.")
	public static NpcStringId WE_RE_CHARGING_TO_WEAKEN_HIM_SHOWING_THEM_WHAT_WE_RE_MADE_OF;
	
	@ClientString(id = 1811208, message = "Allies are cornered. We've been stalled.")
	public static NpcStringId ALLIES_ARE_CORNERED_WE_VE_BEEN_STALLED;
	
	@ClientString(id = 1811209, message = "Messenger, tell the allies of the alliance! Go and begin charging at Seed of Annihilation!")
	public static NpcStringId MESSENGER_TELL_THE_ALLIES_OF_THE_ALLIANCE_GO_AND_BEGIN_CHARGING_AT_SEED_OF_ANNIHILATION;
	
	@ClientString(id = 1811210, message = "Messenger, tell the allies of the alliance! Get ready to challenge Istina!")
	public static NpcStringId MESSENGER_TELL_THE_ALLIES_OF_THE_ALLIANCE_GET_READY_TO_CHALLENGE_ISTINA;
	
	@ClientString(id = 1811211, message = "Messenger, tell the allies of the alliance! I command you to charge Istina's Habitat!")
	public static NpcStringId MESSENGER_TELL_THE_ALLIES_OF_THE_ALLIANCE_I_COMMAND_YOU_TO_CHARGE_ISTINA_S_HABITAT;
	
	@ClientString(id = 1811212, message = "Messenger, tell the allies of the alliance! We're not doing well here in Seed of Annihilation, but supply must be provided!")
	public static NpcStringId MESSENGER_TELL_THE_ALLIES_OF_THE_ALLIANCE_WE_RE_NOT_DOING_WELL_HERE_IN_SEED_OF_ANNIHILATION_BUT_SUPPLY_MUST_BE_PROVIDED;
	
	@ClientString(id = 1811213, message = "I, Hermuncus, give my power to those who fight for me.")
	public static NpcStringId I_HERMUNCUS_GIVE_MY_POWER_TO_THOSE_WHO_FIGHT_FOR_ME;
	
	@ClientString(id = 1811214, message = "Though small, this power will help you greatly.")
	public static NpcStringId THOUGH_SMALL_THIS_POWER_WILL_HELP_YOU_GREATLY;
	
	@ClientString(id = 1811215, message = "Release me and you have the power of Giants.")
	public static NpcStringId RELEASE_ME_AND_YOU_HAVE_THE_POWER_OF_GIANTS;
	
	@ClientString(id = 1811216, message = "Free the Giant from his imprisonment and awaken your true power.")
	public static NpcStringId FREE_THE_GIANT_FROM_HIS_IMPRISONMENT_AND_AWAKEN_YOUR_TRUE_POWER;
	
	@ClientString(id = 1811217, message = "Don't come back here!!")
	public static NpcStringId DON_T_COME_BACK_HERE;
	
	@ClientString(id = 1811218, message = "Dimensional Rifter summoned you.")
	public static NpcStringId DIMENSIONAL_RIFTER_SUMMONED_YOU;
	
	@ClientString(id = 1811219, message = "When you move outside of the area, Teredor gets ready to summon subordinates.")
	public static NpcStringId WHEN_YOU_MOVE_OUTSIDE_OF_THE_AREA_TEREDOR_GETS_READY_TO_SUMMON_SUBORDINATES;
	
	@ClientString(id = 1811220, message = "Teredor summons subordinate because you moved out of Teredor's area")
	public static NpcStringId TEREDOR_SUMMONS_SUBORDINATE_BECAUSE_YOU_MOVED_OUT_OF_TEREDOR_S_AREA;
	
	@ClientString(id = 1811221, message = "Lovely Plagueworms, contaminate the swamp even more!")
	public static NpcStringId LOVELY_PLAGUEWORMS_CONTAMINATE_THE_SWAMP_EVEN_MORE;
	
	@ClientString(id = 1811222, message = "I must go help some more.")
	public static NpcStringId I_MUST_GO_HELP_SOME_MORE;
	
	@ClientString(id = 1811223, message = "No! The Seal Controls have been exposed. Guards protect the Seal Controls!")
	public static NpcStringId NO_THE_SEAL_CONTROLS_HAVE_BEEN_EXPOSED_GUARDS_PROTECT_THE_SEAL_CONTROLS;
	
	@ClientString(id = 1811224, message = "$s1 succeeded in hunting rare Altar of Evil Spirit Offering Box to acquire bonus XP")
	public static NpcStringId S1_SUCCEEDED_IN_HUNTING_RARE_ALTAR_OF_EVIL_SPIRIT_OFFERING_BOX_TO_ACQUIRE_BONUS_XP;
	
	@ClientString(id = 1811225, message = "Pa'agrio Lord Kakai is calling for you.")
	public static NpcStringId PA_AGRIO_LORD_KAKAI_IS_CALLING_FOR_YOU;
	
	@ClientString(id = 1811226, message = "Disable device will go out of control in 1 minute")
	public static NpcStringId DISABLE_DEVICE_WILL_GO_OUT_OF_CONTROL_IN_1_MINUTE;
	
	@ClientString(id = 1811227, message = "50 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING41;
	
	@ClientString(id = 1811228, message = "40 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING42;
	
	@ClientString(id = 1811229, message = "30 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING43;
	
	@ClientString(id = 1811230, message = "20 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING44;
	
	@ClientString(id = 1811231, message = "10 seconds are remaining.")
	public static NpcStringId SECONDS_ARE_REMAINING45;
	
	@ClientString(id = 1811232, message = "5 Seconds")
	public static NpcStringId SECONDS;
	
	@ClientString(id = 1811233, message = "4 Seconds")
	public static NpcStringId SECONDS2;
	
	@ClientString(id = 1811234, message = "3 Seconds")
	public static NpcStringId SECONDS3;
	
	@ClientString(id = 1811235, message = "2 Seconds")
	public static NpcStringId SECONDS4;
	
	@ClientString(id = 1811236, message = "1 Second")
	public static NpcStringId SECOND;
	
	@ClientString(id = 1811237, message = "You may not teleport in middle of a siege.")
	public static NpcStringId YOU_MAY_NOT_TELEPORT_IN_MIDDLE_OF_A_SIEGE;
	
	@ClientString(id = 1811238, message = "You may not teleport in middle of a territory war.")
	public static NpcStringId YOU_MAY_NOT_TELEPORT_IN_MIDDLE_OF_A_TERRITORY_WAR;
	
	@ClientString(id = 1811239, message = "You cannot teleport in party status.")
	public static NpcStringId YOU_CANNOT_TELEPORT_IN_PARTY_STATUS;
	
	@ClientString(id = 1811240, message = "You may not teleport while using instance zone.")
	public static NpcStringId YOU_MAY_NOT_TELEPORT_WHILE_USING_INSTANCE_ZONE;
	
	@ClientString(id = 1811241, message = "You cannot teleport in combat.")
	public static NpcStringId YOU_CANNOT_TELEPORT_IN_COMBAT;
	
	@ClientString(id = 1811242, message = "Can't manage soulshots.")
	public static NpcStringId CAN_T_MANAGE_SOULSHOTS;
	
	@ClientString(id = 1811243, message = "Talking Island Village is really beautiful.")
	public static NpcStringId TALKING_ISLAND_VILLAGE_IS_REALLY_BEAUTIFUL;
	
	@ClientString(id = 1811244, message = "I haven't felt this good in ages.")
	public static NpcStringId I_HAVEN_T_FELT_THIS_GOOD_IN_AGES;
	
	@ClientString(id = 1811245, message = "Alchemy is a science and an art.")
	public static NpcStringId ALCHEMY_IS_A_SCIENCE_AND_AN_ART;
	
	@ClientString(id = 1811246, message = "Tell me about moving to Aden Continent.")
	public static NpcStringId TELL_ME_ABOUT_MOVING_TO_ADEN_CONTINENT;
	
	@ClientString(id = 1811247, message = "Hmm… is this still a decent weapon?")
	public static NpcStringId HMM_IS_THIS_STILL_A_DECENT_WEAPON;
	
	@ClientString(id = 1811248, message = "Spiritshots are mixed!")
	public static NpcStringId SPIRITSHOTS_ARE_MIXED;
	
	@ClientString(id = 1811249, message = "When will I organize this all.")
	public static NpcStringId WHEN_WILL_I_ORGANIZE_THIS_ALL;
	
	@ClientString(id = 1811250, message = "All races came together to rebuild Talking Island Village.")
	public static NpcStringId ALL_RACES_CAME_TOGETHER_TO_REBUILD_TALKING_ISLAND_VILLAGE;
	
	@ClientString(id = 1811251, message = "It's a miracle that the Talking Island has restored.")
	public static NpcStringId IT_S_A_MIRACLE_THAT_THE_TALKING_ISLAND_HAS_RESTORED;
	
	@ClientString(id = 1811252, message = "Every race built a piece of this Village.")
	public static NpcStringId EVERY_RACE_BUILT_A_PIECE_OF_THIS_VILLAGE;
	
	@ClientString(id = 1811253, message = "Weee!")
	public static NpcStringId WEEE;
	
	@ClientString(id = 1811254, message = "Boys are so annoying.")
	public static NpcStringId BOYS_ARE_SO_ANNOYING;
	
	@ClientString(id = 1811255, message = "If you ignore the Training Grounds, you'll regret it.")
	public static NpcStringId IF_YOU_IGNORE_THE_TRAINING_GROUNDS_YOU_LL_REGRET_IT;
	
	@ClientString(id = 1811256, message = "You'll earn tons of items using the Training Grounds.")
	public static NpcStringId YOU_LL_EARN_TONS_OF_ITEMS_USING_THE_TRAINING_GROUNDS;
	
	@ClientString(id = 1811257, message = "Kids these days...")
	public static NpcStringId KIDS_THESE_DAYS;
	
	@ClientString(id = 1811258, message = "When I was young...")
	public static NpcStringId WHEN_I_WAS_YOUNG;
	
	@ClientString(id = 1811259, message = "Giran Shuttle does not come anymore. It's all in the past.")
	public static NpcStringId GIRAN_SHUTTLE_DOES_NOT_COME_ANYMORE_IT_S_ALL_IN_THE_PAST;
	
	@ClientString(id = 1811260, message = "Lady. You must go in.")
	public static NpcStringId LADY_YOU_MUST_GO_IN;
	
	@ClientString(id = 1811261, message = "Impressive looking statues.")
	public static NpcStringId IMPRESSIVE_LOOKING_STATUES;
	
	@ClientString(id = 1811262, message = "It's here!!!")
	public static NpcStringId IT_S_HERE;
	
	@ClientString(id = 1811263, message = "Ha!!")
	public static NpcStringId HA2;
	
	@ClientString(id = 1811264, message = "I will guide you. Follow me.")
	public static NpcStringId I_WILL_GUIDE_YOU_FOLLOW_ME;
	
	@ClientString(id = 1811265, message = "Hey kid! Hurry up and follow me.")
	public static NpcStringId HEY_KID_HURRY_UP_AND_FOLLOW_ME;
	
	@ClientString(id = 1811266, message = "You must be the one Kakai talked about.")
	public static NpcStringId YOU_MUST_BE_THE_ONE_KAKAI_TALKED_ABOUT;
	
	@ClientString(id = 1811267, message = "You should ride Kukuri to go to Ye Sagira.")
	public static NpcStringId YOU_SHOULD_RIDE_KUKURI_TO_GO_TO_YE_SAGIRA;
	
	@ClientString(id = 1811268, message = "Talk to that apprentice and get on Kukuri.")
	public static NpcStringId TALK_TO_THAT_APPRENTICE_AND_GET_ON_KUKURI;
	
	@ClientString(id = 1811269, message = "Looks good. Follow me.")
	public static NpcStringId LOOKS_GOOD_FOLLOW_ME;
	
	@ClientString(id = 1811270, message = "Open your map when you arrive at Ye Sagira.")
	public static NpcStringId OPEN_YOUR_MAP_WHEN_YOU_ARRIVE_AT_YE_SAGIRA;
	
	@ClientString(id = 1811271, message = "It's hard to tell where you are at without a map in Ye Sagira.")
	public static NpcStringId IT_S_HARD_TO_TELL_WHERE_YOU_ARE_AT_WITHOUT_A_MAP_IN_YE_SAGIRA;
	
	@ClientString(id = 1811272, message = "When you're done, find the apprentice in the entrance of the forest.")
	public static NpcStringId WHEN_YOU_RE_DONE_FIND_THE_APPRENTICE_IN_THE_ENTRANCE_OF_THE_FOREST;
	
	@ClientString(id = 1811273, message = "This is it for me.")
	public static NpcStringId THIS_IS_IT_FOR_ME;
	
	@ClientString(id = 1811274, message = "I will understand that as please kill you fast. Get a taste of my subordinate's powers...!!")
	public static NpcStringId I_WILL_UNDERSTAND_THAT_AS_PLEASE_KILL_YOU_FAST_GET_A_TASTE_OF_MY_SUBORDINATE_S_POWERS;
	
	@ClientString(id = 1811275, message = "All this for me...so dishonest!")
	public static NpcStringId ALL_THIS_FOR_ME_SO_DISHONEST;
	
	@ClientString(id = 1811276, message = "Are you alone? Good attitude, but the results are obvious...")
	public static NpcStringId ARE_YOU_ALONE_GOOD_ATTITUDE_BUT_THE_RESULTS_ARE_OBVIOUS;
	
	@ClientString(id = 1811277, message = "My subordinates are as stubborn as you guys..")
	public static NpcStringId MY_SUBORDINATES_ARE_AS_STUBBORN_AS_YOU_GUYS;
	
	@ClientString(id = 1811278, message = "Why are you annoying me! My Subordinates will fight back..!")
	public static NpcStringId WHY_ARE_YOU_ANNOYING_ME_MY_SUBORDINATES_WILL_FIGHT_BACK;
	
	@ClientString(id = 1811279, message = "The spell will complete soon. My illusions will make you go in pain. Get ready..!")
	public static NpcStringId THE_SPELL_WILL_COMPLETE_SOON_MY_ILLUSIONS_WILL_MAKE_YOU_GO_IN_PAIN_GET_READY;
	
	@ClientString(id = 1811280, message = "I will put you to sleep with my subordinates. In your tomb!")
	public static NpcStringId I_WILL_PUT_YOU_TO_SLEEP_WITH_MY_SUBORDINATES_IN_YOUR_TOMB;
	
	@ClientString(id = 1811281, message = "Aggressive greetings. Try hard.")
	public static NpcStringId AGGRESSIVE_GREETINGS_TRY_HARD;
	
	@ClientString(id = 1811282, message = "Attack all of a sudden... try me.")
	public static NpcStringId ATTACK_ALL_OF_A_SUDDEN_TRY_ME;
	
	@ClientString(id = 1811283, message = "Shilen...")
	public static NpcStringId SHILEN;
	
	@ClientString(id = 1811284, message = "Too dangerous. Find others to help you.")
	public static NpcStringId TOO_DANGEROUS_FIND_OTHERS_TO_HELP_YOU;
	
	@ClientString(id = 1811285, message = "Does the Goddess see what she has done?")
	public static NpcStringId DOES_THE_GODDESS_SEE_WHAT_SHE_HAS_DONE;
	
	@ClientString(id = 1811286, message = "Why have the heroes abandoned us?")
	public static NpcStringId WHY_HAVE_THE_HEROES_ABANDONED_US;
	
	@ClientString(id = 1811287, message = "Something like that comes out of the ruins...")
	public static NpcStringId SOMETHING_LIKE_THAT_COMES_OUT_OF_THE_RUINS;
	
	@ClientString(id = 1811288, message = "Stop them from approaching the town!!")
	public static NpcStringId STOP_THEM_FROM_APPROACHING_THE_TOWN;
	
	@ClientString(id = 1811289, message = "The Seven Signs ended…")
	public static NpcStringId THE_SEVEN_SIGNS_ENDED;
	
	@ClientString(id = 1811290, message = "We need to find her weakness...")
	public static NpcStringId WE_NEED_TO_FIND_HER_WEAKNESS;
	
	@ClientString(id = 1811291, message = "Is it better to end destiny or start destiny…")
	public static NpcStringId IS_IT_BETTER_TO_END_DESTINY_OR_START_DESTINY;
	
	@ClientString(id = 1811292, message = "Look at all the newbies. Ha ha ha.")
	public static NpcStringId LOOK_AT_ALL_THE_NEWBIES_HA_HA_HA;
	
	@ClientString(id = 1811293, message = "Tis a scratch…")
	public static NpcStringId TIS_A_SCRATCH;
	
	@ClientString(id = 1811294, message = "And now your journey begins.")
	public static NpcStringId AND_NOW_YOUR_JOURNEY_BEGINS;
	
	@ClientString(id = 1811295, message = "Ow~! Since I'm done, I'm gone~ ow~!")
	public static NpcStringId OW_SINCE_I_M_DONE_I_M_GONE_OW;
	
	@ClientString(id = 1811296, message = "Istina, I will come to you.")
	public static NpcStringId ISTINA_I_WILL_COME_TO_YOU;
	
	@ClientString(id = 1811297, message = "You can't ride a Kukuri now.")
	public static NpcStringId YOU_CAN_T_RIDE_A_KUKURI_NOW;
	
	@ClientString(id = 1811298, message = "$s1 has demonstrated respect.")
	public static NpcStringId S1_HAS_DEMONSTRATED_RESPECT;
	
	@ClientString(id = 1811299, message = "$s1, your empathy is appreciated.")
	public static NpcStringId S1_YOUR_EMPATHY_IS_APPRECIATED;
	
	@ClientString(id = 1811300, message = "Thank you for attending this mourning service. It is alright to show your sorrow…")
	public static NpcStringId THANK_YOU_FOR_ATTENDING_THIS_MOURNING_SERVICE_IT_IS_ALRIGHT_TO_SHOW_YOUR_SORROW;
	
	@ClientString(id = 1811301, message = "The name is Janson... Bow your head and show some respect for the dead.")
	public static NpcStringId THE_NAME_IS_JANSON_BOW_YOUR_HEAD_AND_SHOW_SOME_RESPECT_FOR_THE_DEAD;
	
	@ClientString(id = 1811302, message = "Remaining Time")
	public static NpcStringId REMAINING_TIME;
	
	@ClientString(id = 1811303, message = "The beach where relics of Giants had fallen has now become clean.")
	public static NpcStringId THE_BEACH_WHERE_RELICS_OF_GIANTS_HAD_FALLEN_HAS_NOW_BECOME_CLEAN;
	
	@ClientString(id = 1811304, message = "Occasionally rare jewels of Giants are discovered in the beach.")
	public static NpcStringId OCCASIONALLY_RARE_JEWELS_OF_GIANTS_ARE_DISCOVERED_IN_THE_BEACH;
	
	@ClientString(id = 1811305, message = "There is a day where you can see even the Aden continent if the weather is good.")
	public static NpcStringId THERE_IS_A_DAY_WHERE_YOU_CAN_SEE_EVEN_THE_ADEN_CONTINENT_IF_THE_WEATHER_IS_GOOD;
	
	@ClientString(id = 1811306, message = "If I'm here, it feels like time has stopped.")
	public static NpcStringId IF_I_M_HERE_IT_FEELS_LIKE_TIME_HAS_STOPPED;
	
	@ClientString(id = 1811307, message = "Perhaps even the village becomes dangerous?")
	public static NpcStringId PERHAPS_EVEN_THE_VILLAGE_BECOMES_DANGEROUS;
	
	@ClientString(id = 1811308, message = "Speak with me about traveling around Aden.")
	public static NpcStringId SPEAK_WITH_ME_ABOUT_TRAVELING_AROUND_ADEN;
	
	@ClientString(id = 1811309, message = "Hey guys! Don't go too far~")
	public static NpcStringId HEY_GUYS_DON_T_GO_TOO_FAR;
	
	@ClientString(id = 1811310, message = "Rumors say that these heroes have set unbelievable records.")
	public static NpcStringId RUMORS_SAY_THAT_THESE_HEROES_HAVE_SET_UNBELIEVABLE_RECORDS;
	
	@ClientString(id = 1811311, message = "According to this, records are broken every month.")
	public static NpcStringId ACCORDING_TO_THIS_RECORDS_ARE_BROKEN_EVERY_MONTH;
	
	@ClientString(id = 1811312, message = "Wanna try a Kukuri race?")
	public static NpcStringId WANNA_TRY_A_KUKURI_RACE;
	
	@ClientString(id = 1811313, message = "The land of Aden is in need of materials to rebuild from Shilen's destruction.")
	public static NpcStringId THE_LAND_OF_ADEN_IS_IN_NEED_OF_MATERIALS_TO_REBUILD_FROM_SHILEN_S_DESTRUCTION;
	
	@ClientString(id = 1811314, message = "Please donate any unused materials you have to help rebuild Aden.")
	public static NpcStringId PLEASE_DONATE_ANY_UNUSED_MATERIALS_YOU_HAVE_TO_HELP_REBUILD_ADEN;
	
	@ClientString(id = 1811315, message = "You'll receive a gift for any applicable donation.")
	public static NpcStringId YOU_LL_RECEIVE_A_GIFT_FOR_ANY_APPLICABLE_DONATION;
	
	@ClientString(id = 1811316, message = "Istina is hiding itself in the twisted space.")
	public static NpcStringId ISTINA_IS_HIDING_ITSELF_IN_THE_TWISTED_SPACE;
	
	@ClientString(id = 1811317, message = "Maguen quickly hides after stealing the bio-energy residue.")
	public static NpcStringId MAGUEN_QUICKLY_HIDES_AFTER_STEALING_THE_BIO_ENERGY_RESIDUE;
	
	@ClientString(id = 1811318, message = "You cannot teleport while you are dead.")
	public static NpcStringId YOU_CANNOT_TELEPORT_WHILE_YOU_ARE_DEAD;
	
	@ClientString(id = 1811319, message = "Nonsense!!")
	public static NpcStringId NONSENSE;
	
	@ClientString(id = 1811320, message = "You poor and pitiful creature!")
	public static NpcStringId YOU_POOR_AND_PITIFUL_CREATURE;
	
	@ClientString(id = 1811321, message = "The Daemon Angma Mastermind has appeared in the Dark Elf Village.")
	public static NpcStringId THE_DAEMON_ANGMA_MASTERMIND_HAS_APPEARED_IN_THE_DARK_ELF_VILLAGE;
	
	@ClientString(id = 1811322, message = "Our leader is around this village.")
	public static NpcStringId OUR_LEADER_IS_AROUND_THIS_VILLAGE;
	
	@ClientString(id = 1811323, message = "Would you care to die?")
	public static NpcStringId WOULD_YOU_CARE_TO_DIE;
	
	@ClientString(id = 1811324, message = "You want a piece of me!")
	public static NpcStringId YOU_WANT_A_PIECE_OF_ME;
	
	@ClientString(id = 1811325, message = "That's what I thought, all talk!")
	public static NpcStringId THAT_S_WHAT_I_THOUGHT_ALL_TALK;
	
	@ClientString(id = 1811326, message = "Thank you! And now you die!")
	public static NpcStringId THANK_YOU_AND_NOW_YOU_DIE;
	
	@ClientString(id = 1811327, message = "It's time to turn that frown upside down… it's party town!")
	public static NpcStringId IT_S_TIME_TO_TURN_THAT_FROWN_UPSIDE_DOWN_IT_S_PARTY_TOWN;
	
	@ClientString(id = 1811328, message = "Enjoy the festivities!")
	public static NpcStringId ENJOY_THE_FESTIVITIES;
	
	@ClientString(id = 1811329, message = "Come on, let's dance!")
	public static NpcStringId COME_ON_LET_S_DANCE;
	
	@ClientString(id = 1811330, message = "Time to get down and get funky!")
	public static NpcStringId TIME_TO_GET_DOWN_AND_GET_FUNKY;
	
	@ClientString(id = 1811331, message = "Hahaha, I've got the best steps.")
	public static NpcStringId HAHAHA_I_VE_GOT_THE_BEST_STEPS;
	
	@ClientString(id = 1811332, message = "May I have this dance?")
	public static NpcStringId MAY_I_HAVE_THIS_DANCE;
	
	@ClientString(id = 1811333, message = "Which Echo Crystals do you have? I have them all, Feast, Love, and Celebration.")
	public static NpcStringId WHICH_ECHO_CRYSTALS_DO_YOU_HAVE_I_HAVE_THEM_ALL_FEAST_LOVE_AND_CELEBRATION;
	
	@ClientString(id = 1811334, message = "Are you ready to bust a groove?")
	public static NpcStringId ARE_YOU_READY_TO_BUST_A_GROOVE;
	
	@ClientString(id = 1811335, message = "Wow, $s1 has serious moves!")
	public static NpcStringId WOW_S1_HAS_SERIOUS_MOVES;
	
	@ClientString(id = 1811336, message = "Oh, I'm so embarrassed...")
	public static NpcStringId OH_I_M_SO_EMBARRASSED;
	
	@ClientString(id = 1811337, message = "$s1 seems a little shy.")
	public static NpcStringId S1_SEEMS_A_LITTLE_SHY;
	
	@ClientString(id = 1811338, message = "Wahaha~ How fun!")
	public static NpcStringId WAHAHA_HOW_FUN;
	
	@ClientString(id = 1811339, message = "Put your hands up!")
	public static NpcStringId PUT_YOUR_HANDS_UP;
	
	@ClientString(id = 1811340, message = "You obtained a poisonous Tama... Thank you, $s1.")
	public static NpcStringId YOU_OBTAINED_A_POISONOUS_TAMA_THANK_YOU_S1;
	
	@ClientString(id = 1811341, message = "I am looking for the poisonous Tama that only grows in the Cruma Marshlands, the Swamp of Screams, and the Bloody Swampland.")
	public static NpcStringId I_AM_LOOKING_FOR_THE_POISONOUS_TAMA_THAT_ONLY_GROWS_IN_THE_CRUMA_MARSHLANDS_THE_SWAMP_OF_SCREAMS_AND_THE_BLOODY_SWAMPLAND;
	
	@ClientString(id = 1811342, message = "Have you obtained the plant? I will send you to the Cardia Adventurer.")
	public static NpcStringId HAVE_YOU_OBTAINED_THE_PLANT_I_WILL_SEND_YOU_TO_THE_CARDIA_ADVENTURER;
	
	@ClientString(id = 1811343, message = "The red tama is more effective~! You can see the adventurer's location on the map.")
	public static NpcStringId THE_RED_TAMA_IS_MORE_EFFECTIVE_YOU_CAN_SEE_THE_ADVENTURER_S_LOCATION_ON_THE_MAP;
	
	@ClientString(id = 1811344, message = "Use the Fearsome Firecracker on the Bat Colony.")
	public static NpcStringId USE_THE_FEARSOME_FIRECRACKER_ON_THE_BAT_COLONY;
	
	@ClientString(id = 1811345, message = "Wow, thanks!")
	public static NpcStringId WOW_THANKS;
	
	@ClientString(id = 1811346, message = "If you excuse me, I need to find a hiding spot.")
	public static NpcStringId IF_YOU_EXCUSE_ME_I_NEED_TO_FIND_A_HIDING_SPOT;
	
	@ClientString(id = 1811347, message = "Replenish ballista magic power")
	public static NpcStringId REPLENISH_BALLISTA_MAGIC_POWER;
	
	@ClientString(id = 1811740, message = "Apherus Garden Door was opened.")
	public static NpcStringId APHERUS_GARDEN_DOOR_WAS_OPENED;
	
	@ClientString(id = 1811750, message = "Intruder! I will let you know the strength of my power!")
	public static NpcStringId INTRUDER_I_WILL_LET_YOU_KNOW_THE_STRENGTH_OF_MY_POWER;
	
	@ClientString(id = 1811760, message = "Apherus is out of energy. No longer moving.")
	public static NpcStringId APHERUS_IS_OUT_OF_ENERGY_NO_LONGER_MOVING;
	
	@ClientString(id = 1811770, message = "I've brought a guard as a payback!")
	public static NpcStringId I_VE_BROUGHT_A_GUARD_AS_A_PAYBACK;
	
	@ClientString(id = 1811780, message = "How dare you! Attack!")
	public static NpcStringId HOW_DARE_YOU_ATTACK;
	
	@ClientString(id = 1811790, message = "Moved to Apherus dimension.")
	public static NpcStringId MOVED_TO_APHERUS_DIMENSION;
	
	@ClientString(id = 1900000, message = "Kehahaha!!! I captured Santa!! There will be no gifts this year!!!")
	public static NpcStringId KEHAHAHA_I_CAPTURED_SANTA_THERE_WILL_BE_NO_GIFTS_THIS_YEAR;
	
	@ClientString(id = 1900001, message = "Now! Why don't you take the challenge?")
	public static NpcStringId NOW_WHY_DON_T_YOU_TAKE_THE_CHALLENGE;
	
	@ClientString(id = 1900002, message = "Come on, I'll take all of you on!")
	public static NpcStringId COME_ON_I_LL_TAKE_ALL_OF_YOU_ON2;
	
	@ClientString(id = 1900003, message = "Well? I win, right?")
	public static NpcStringId WELL_I_WIN_RIGHT;
	
	@ClientString(id = 1900004, message = "Now!! All of you losers, get out of here!")
	public static NpcStringId NOW_ALL_OF_YOU_LOSERS_GET_OUT_OF_HERE;
	
	@ClientString(id = 1900005, message = "What a bunch of losers.")
	public static NpcStringId WHAT_A_BUNCH_OF_LOSERS2;
	
	@ClientString(id = 1900006, message = "I guess you came to rescue Santa. But you picked the wrong opponent.")
	public static NpcStringId I_GUESS_YOU_CAME_TO_RESCUE_SANTA_BUT_YOU_PICKED_THE_WRONG_OPPONENT;
	
	@ClientString(id = 1900007, message = "Oh, not bad...")
	public static NpcStringId OH_NOT_BAD;
	
	@ClientString(id = 1900008, message = "Agh!! That's not what I meant to do!")
	public static NpcStringId AGH_THAT_S_NOT_WHAT_I_MEANT_TO_DO;
	
	@ClientString(id = 1900009, message = "Curse you!! Huh... What...?")
	public static NpcStringId CURSE_YOU_HUH_WHAT;
	
	@ClientString(id = 1900010, message = "Have you done nothing but rock-paper-scissors?!")
	public static NpcStringId HAVE_YOU_DONE_NOTHING_BUT_ROCK_PAPER_SCISSORS2;
	
	@ClientString(id = 1900011, message = "Stop it, no more... I did it because I was too lonely...")
	public static NpcStringId STOP_IT_NO_MORE_I_DID_IT_BECAUSE_I_WAS_TOO_LONELY2;
	
	@ClientString(id = 1900012, message = "I have to release Santa... How infuriating!!!")
	public static NpcStringId I_HAVE_TO_RELEASE_SANTA_HOW_INFURIATING2;
	
	@ClientString(id = 1900013, message = "I hate happy Happy Holidays!!!")
	public static NpcStringId I_HATE_HAPPY_HAPPY_HOLIDAYS2;
	
	@ClientString(id = 1900014, message = "Oh. I'm bored.")
	public static NpcStringId OH_I_M_BORED2;
	
	@ClientString(id = 1900015, message = "Shall I go to see if Santa is still there? Hehe~")
	public static NpcStringId SHALL_I_GO_TO_SEE_IF_SANTA_IS_STILL_THERE_HEHE;
	
	@ClientString(id = 1900016, message = "Oh ho ho.... Happy Holidays!!")
	public static NpcStringId OH_HO_HO_HAPPY_HOLIDAYS2;
	
	@ClientString(id = 1900017, message = "Santa can give nice presents only if he's released from the Turkey...")
	public static NpcStringId SANTA_CAN_GIVE_NICE_PRESENTS_ONLY_IF_HE_S_RELEASED_FROM_THE_TURKEY;
	
	@ClientString(id = 1900018, message = "Oh ho ho... Oh ho ho... Thank you. Everyone!  I will repay you for sure.")
	public static NpcStringId OH_HO_HO_OH_HO_HO_THANK_YOU_EVERYONE_I_WILL_REPAY_YOU_FOR_SURE;
	
	@ClientString(id = 1900019, message = "Happy Holidays~ Well done.")
	public static NpcStringId HAPPY_HOLIDAYS_WELL_DONE;
	
	@ClientString(id = 1900020, message = "Happy Holidays~ Thank you for rescuing me from that wretched Turkey.")
	public static NpcStringId HAPPY_HOLIDAYS_THANK_YOU_FOR_RESCUING_ME_FROM_THAT_WRETCHED_TURKEY2;
	
	@ClientString(id = 1900021, message = "%s. I have prepared a gift for you.")
	public static NpcStringId S_I_HAVE_PREPARED_A_GIFT_FOR_YOU;
	
	@ClientString(id = 1900022, message = "I have a gift for %s.")
	public static NpcStringId I_HAVE_A_GIFT_FOR_S;
	
	@ClientString(id = 1900023, message = "Take a look at the inventory. I hope you like the gift I gave you.")
	public static NpcStringId TAKE_A_LOOK_AT_THE_INVENTORY_I_HOPE_YOU_LIKE_THE_GIFT_I_GAVE_YOU2;
	
	@ClientString(id = 1900024, message = "Take a look at the inventory. Perhaps there will be a big present~")
	public static NpcStringId TAKE_A_LOOK_AT_THE_INVENTORY_PERHAPS_THERE_WILL_BE_A_BIG_PRESENT;
	
	@ClientString(id = 1900025, message = "I'm tired of dealing with you. I'm leaving.")
	public static NpcStringId I_M_TIRED_OF_DEALING_WITH_YOU_I_M_LEAVING2;
	
	@ClientString(id = 1900026, message = "When are you going to stop? I'm slowly getting tired of this.")
	public static NpcStringId WHEN_ARE_YOU_GOING_TO_STOP_I_M_SLOWLY_GETTING_TIRED_OF_THIS;
	
	@ClientString(id = 1900027, message = "Message from Santa: Many blessings to %s, who saved me~")
	public static NpcStringId MESSAGE_FROM_SANTA_MANY_BLESSINGS_TO_S_WHO_SAVED_ME;
	
	@ClientString(id = 1900028, message = "How dare you awaken me. Feel the pain of the flames.")
	public static NpcStringId HOW_DARE_YOU_AWAKEN_ME_FEEL_THE_PAIN_OF_THE_FLAMES;
	
	@ClientString(id = 1900029, message = "Who dares oppose the majesty of fire...?")
	public static NpcStringId WHO_DARES_OPPOSE_THE_MAJESTY_OF_FIRE;
	
	@ClientString(id = 1900030, message = "Oh... Ouch! No, not there! Not my bead!")
	public static NpcStringId OH_OUCH_NO_NOT_THERE_NOT_MY_BEAD;
	
	@ClientString(id = 1900031, message = "Co... Cold! That's cold! Ack! Ack!")
	public static NpcStringId CO_COLD_THAT_S_COLD_ACK_ACK;
	
	@ClientString(id = 1900032, message = "Please, %s... Don't hit me... Please.")
	public static NpcStringId PLEASE_S_DON_T_HIT_ME_PLEASE;
	
	@ClientString(id = 1900033, message = "Kuaaannggg! Shake in fear!")
	public static NpcStringId KUAAANNGGG_SHAKE_IN_FEAR;
	
	@ClientString(id = 1900034, message = "If you attack me right now, you're really going to get it!!!")
	public static NpcStringId IF_YOU_ATTACK_ME_RIGHT_NOW_YOU_RE_REALLY_GOING_TO_GET_IT;
	
	@ClientString(id = 1900035, message = "Just you wait! I'm going to show you my killer technique.")
	public static NpcStringId JUST_YOU_WAIT_I_M_GOING_TO_SHOW_YOU_MY_KILLER_TECHNIQUE;
	
	@ClientString(id = 1900036, message = "You don't dare attack me!")
	public static NpcStringId YOU_DON_T_DARE_ATTACK_ME;
	
	@ClientString(id = 1900037, message = "It's different from the Fire Spirit. It's not the Fire Spirit! Feel my wrath!")
	public static NpcStringId IT_S_DIFFERENT_FROM_THE_FIRE_SPIRIT_IT_S_NOT_THE_FIRE_SPIRIT_FEEL_MY_WRATH;
	
	@ClientString(id = 1900038, message = "Cold... This place...  Is this where I die...?")
	public static NpcStringId COLD_THIS_PLACE_IS_THIS_WHERE_I_DIE;
	
	@ClientString(id = 1900039, message = "My body is cooling. Oh, Gran Kain... Forgive me...")
	public static NpcStringId MY_BODY_IS_COOLING_OH_GRAN_KAIN_FORGIVE_ME;
	
	@ClientString(id = 1900040, message = "Idiot! I only incur damage from bare-handed attacks!")
	public static NpcStringId IDIOT_I_ONLY_INCUR_DAMAGE_FROM_BARE_HANDED_ATTACKS;
	
	@ClientString(id = 1900041, message = "You summoned me, so you must be confident, huh? Here I come! Jack game!")
	public static NpcStringId YOU_SUMMONED_ME_SO_YOU_MUST_BE_CONFIDENT_HUH_HERE_I_COME_JACK_GAME2;
	
	@ClientString(id = 1900042, message = "Hello. Let's have a good Jack game.")
	public static NpcStringId HELLO_LET_S_HAVE_A_GOOD_JACK_GAME2;
	
	@ClientString(id = 1900043, message = "I'm starting! Now, show me the card you want!")
	public static NpcStringId I_M_STARTING_NOW_SHOW_ME_THE_CARD_YOU_WANT2;
	
	@ClientString(id = 1900044, message = "We'll start now! Show me the card you want!")
	public static NpcStringId WE_LL_START_NOW_SHOW_ME_THE_CARD_YOU_WANT2;
	
	@ClientString(id = 1900045, message = "I'm showing the Rotten Pumpkin Card!")
	public static NpcStringId I_M_SHOWING_THE_ROTTEN_PUMPKIN_CARD2;
	
	@ClientString(id = 1900046, message = "I'll be showing the Rotten Pumpkin Card!")
	public static NpcStringId I_LL_BE_SHOWING_THE_ROTTEN_PUMPKIN_CARD2;
	
	@ClientString(id = 1900047, message = "I'm showing the Jack Pumpkin Card!")
	public static NpcStringId I_M_SHOWING_THE_JACK_PUMPKIN_CARD2;
	
	@ClientString(id = 1900048, message = "I'll be showing the Jack Pumpkin Card!")
	public static NpcStringId I_LL_BE_SHOWING_THE_JACK_PUMPKIN_CARD2;
	
	@ClientString(id = 1900049, message = "That's my precious Fantastic Chocolate Banana Ultra Favor Candy!!! I'm definitely winning the next round!!")
	public static NpcStringId THAT_S_MY_PRECIOUS_FANTASTIC_CHOCOLATE_BANANA_ULTRA_FAVOR_CANDY_I_M_DEFINITELY_WINNING_THE_NEXT_ROUND2;
	
	@ClientString(id = 1900050, message = "It's my precious candy, but... I'll happily give it to you~!")
	public static NpcStringId IT_S_MY_PRECIOUS_CANDY_BUT_I_LL_HAPPILY_GIVE_IT_TO_YOU2;
	
	@ClientString(id = 1900051, message = "I'm out of candy. I'll give you my toy chest instead.")
	public static NpcStringId I_M_OUT_OF_CANDY_I_LL_GIVE_YOU_MY_TOY_CHEST_INSTEAD;
	
	@ClientString(id = 1900052, message = "Since I'm out of candy, I will give you my toy chest instead.")
	public static NpcStringId SINCE_I_M_OUT_OF_CANDY_I_WILL_GIVE_YOU_MY_TOY_CHEST_INSTEAD;
	
	@ClientString(id = 1900053, message = "You're not peeking at my card, are you? This time, I'll wager a special scroll.")
	public static NpcStringId YOU_RE_NOT_PEEKING_AT_MY_CARD_ARE_YOU_THIS_TIME_I_LL_WAGER_A_SPECIAL_SCROLL2;
	
	@ClientString(id = 1900054, message = "We're getting serious now. If you win again, I'll give you a special scroll.")
	public static NpcStringId WE_RE_GETTING_SERIOUS_NOW_IF_YOU_WIN_AGAIN_I_LL_GIVE_YOU_A_SPECIAL_SCROLL2;
	
	@ClientString(id = 1900055, message = "You could probably enter the underworld pro league!")
	public static NpcStringId YOU_COULD_PROBABLY_ENTER_THE_UNDERWORLD_PRO_LEAGUE2;
	
	@ClientString(id = 1900056, message = "Even pros can't do this much. You're amazing.")
	public static NpcStringId EVEN_PROS_CAN_T_DO_THIS_MUCH_YOU_RE_AMAZING2;
	
	@ClientString(id = 1900057, message = "Who's the monster here?! This time, I'll bet my precious Transformation Stick.")
	public static NpcStringId WHO_S_THE_MONSTER_HERE_THIS_TIME_I_LL_BET_MY_PRECIOUS_TRANSFORMATION_STICK2;
	
	@ClientString(id = 1900058, message = "I lost again. I won't lose this time. I'm betting my Transformation Stick.")
	public static NpcStringId I_LOST_AGAIN_I_WON_T_LOSE_THIS_TIME_I_M_BETTING_MY_TRANSFORMATION_STICK2;
	
	@ClientString(id = 1900059, message = "Lost again! Hmph. Next time, I'll bet an incredible gift! Wait for it if you want!")
	public static NpcStringId LOST_AGAIN_HMPH_NEXT_TIME_I_LL_BET_AN_INCREDIBLE_GIFT_WAIT_FOR_IT_IF_YOU_WANT2;
	
	@ClientString(id = 1900060, message = "You're too good. Next time, I'll give you an incredible gift! Please wait for it.")
	public static NpcStringId YOU_RE_TOO_GOOD_NEXT_TIME_I_LL_GIVE_YOU_AN_INCREDIBLE_GIFT_PLEASE_WAIT_FOR_IT;
	
	@ClientString(id = 1900061, message = "My pride can't handle you winning anymore!")
	public static NpcStringId MY_PRIDE_CAN_T_HANDLE_YOU_WINNING_ANYMORE2;
	
	@ClientString(id = 1900062, message = "I would be embarrassed to lose again here...")
	public static NpcStringId I_WOULD_BE_EMBARRASSED_TO_LOSE_AGAIN_HERE;
	
	@ClientString(id = 1900063, message = "What's your name? I'm gonna remember you!")
	public static NpcStringId WHAT_S_YOUR_NAME_I_M_GONNA_REMEMBER_YOU2;
	
	@ClientString(id = 1900064, message = "People from the above ground world are really good at games.")
	public static NpcStringId PEOPLE_FROM_THE_ABOVE_GROUND_WORLD_ARE_REALLY_GOOD_AT_GAMES2;
	
	@ClientString(id = 1900065, message = "You've played a lot in the underworld, haven't you?!")
	public static NpcStringId YOU_VE_PLAYED_A_LOT_IN_THE_UNDERWORLD_HAVEN_T_YOU2;
	
	@ClientString(id = 1900066, message = "I've never met someone so good before.")
	public static NpcStringId I_VE_NEVER_MET_SOMEONE_SO_GOOD_BEFORE2;
	
	@ClientString(id = 1900067, message = "13 wins in a row. You're pretty lucky today, huh?")
	public static NpcStringId WINS_IN_A_ROW_YOU_RE_PRETTY_LUCKY_TODAY_HUH2;
	
	@ClientString(id = 1900068, message = "I never thought I would see 13 wins in a row.")
	public static NpcStringId I_NEVER_THOUGHT_I_WOULD_SEE_13_WINS_IN_A_ROW2;
	
	@ClientString(id = 1900069, message = "This is the highest record in my life! Next time, I'll give you my treasure -- the Golden Jack O'Lantern!")
	public static NpcStringId THIS_IS_THE_HIGHEST_RECORD_IN_MY_LIFE_NEXT_TIME_I_LL_GIVE_YOU_MY_TREASURE_THE_GOLDEN_JACK_O_LANTERN;
	
	@ClientString(id = 1900070, message = "Even pros can't do 14 wins in a row...! Next time, I'll give you my treasure, the Golden Jack O'Lantern")
	public static NpcStringId EVEN_PROS_CAN_T_DO_14_WINS_IN_A_ROW_NEXT_TIME_I_LL_GIVE_YOU_MY_TREASURE_THE_GOLDEN_JACK_O_LANTERN;
	
	@ClientString(id = 1900071, message = "I can't do this anymore! You win! In all my 583 years, you're the best that I've seen!")
	public static NpcStringId I_CAN_T_DO_THIS_ANYMORE_YOU_WIN_IN_ALL_MY_583_YEARS_YOU_RE_THE_BEST_THAT_I_VE_SEEN;
	
	@ClientString(id = 1900072, message = "Playing any more is meaningless. You were my greatest opponent.")
	public static NpcStringId PLAYING_ANY_MORE_IS_MEANINGLESS_YOU_WERE_MY_GREATEST_OPPONENT2;
	
	@ClientString(id = 1900073, message = "I won this round...! It was fun.")
	public static NpcStringId I_WON_THIS_ROUND_IT_WAS_FUN2;
	
	@ClientString(id = 1900074, message = "I won this round. It was enjoyable.")
	public static NpcStringId I_WON_THIS_ROUND_IT_WAS_ENJOYABLE2;
	
	@ClientString(id = 1900075, message = "Above world people are so fun...! Then, see you later!")
	public static NpcStringId ABOVE_WORLD_PEOPLE_ARE_SO_FUN_THEN_SEE_YOU_LATER2;
	
	@ClientString(id = 1900076, message = "Call me again next time. I want to play again with you.")
	public static NpcStringId CALL_ME_AGAIN_NEXT_TIME_I_WANT_TO_PLAY_AGAIN_WITH_YOU2;
	
	@ClientString(id = 1900077, message = "You wanna play some more? I'm out of presents, but I'll give you candy!")
	public static NpcStringId YOU_WANNA_PLAY_SOME_MORE_I_M_OUT_OF_PRESENTS_BUT_I_LL_GIVE_YOU_CANDY2;
	
	@ClientString(id = 1900078, message = "Will you play some more? I don't have any more presents, but I will give you candy if you win.")
	public static NpcStringId WILL_YOU_PLAY_SOME_MORE_I_DON_T_HAVE_ANY_MORE_PRESENTS_BUT_I_WILL_GIVE_YOU_CANDY_IF_YOU_WIN2;
	
	@ClientString(id = 1900079, message = "You're the best. Out of all the Jack's game players I've ever met... I give up!")
	public static NpcStringId YOU_RE_THE_BEST_OUT_OF_ALL_THE_JACK_S_GAME_PLAYERS_I_VE_EVER_MET_I_GIVE_UP2;
	
	@ClientString(id = 1900080, message = "Wowww. Awesome. Really. I have never met someone as good as you before. Now... I can't play anymore.")
	public static NpcStringId WOWWW_AWESOME_REALLY_I_HAVE_NEVER_MET_SOMEONE_AS_GOOD_AS_YOU_BEFORE_NOW_I_CAN_T_PLAY_ANYMORE2;
	
	@ClientString(id = 1900081, message = "%s has won %s Jack's games in a row.")
	public static NpcStringId S_HAS_WON_S_JACK_S_GAMES_IN_A_ROW;
	
	@ClientString(id = 1900082, message = "Congratulations! %s has won %s Jack's games in a row.")
	public static NpcStringId CONGRATULATIONS_S_HAS_WON_S_JACK_S_GAMES_IN_A_ROW;
	
	@ClientString(id = 1900083, message = "Congratulations on getting 1st place in Jack's game!")
	public static NpcStringId CONGRATULATIONS_ON_GETTING_1ST_PLACE_IN_JACK_S_GAME2;
	
	@ClientString(id = 1900084, message = "Hello~! I'm Belldandy. Congratulations on getting 1st place in Jack's game. If you go and find my sibling Skooldie in the village, you'll get an amazing gift! Let's play Jack's game again!")
	public static NpcStringId HELLO_I_M_BELLDANDY_CONGRATULATIONS_ON_GETTING_1ST_PLACE_IN_JACK_S_GAME_IF_YOU_GO_AND_FIND_MY_SIBLING_SKOOLDIE_IN_THE_VILLAGE_YOU_LL_GET_AN_AMAZING_GIFT_LET_S_PLAY_JACK_S_GAME_AGAIN;
	
	@ClientString(id = 1900085, message = "Hmm. You're playing Jack's game for the first time, huh? You couldn't even take out your card at the right time~! My goodness...")
	public static NpcStringId HMM_YOU_RE_PLAYING_JACK_S_GAME_FOR_THE_FIRST_TIME_HUH_YOU_COULDN_T_EVEN_TAKE_OUT_YOUR_CARD_AT_THE_RIGHT_TIME_MY_GOODNESS2;
	
	@ClientString(id = 1900086, message = "Oh. You're not very familiar with Jack's game, right? You didn't take out your card at the right time...")
	public static NpcStringId OH_YOU_RE_NOT_VERY_FAMILIAR_WITH_JACK_S_GAME_RIGHT_YOU_DIDN_T_TAKE_OUT_YOUR_CARD_AT_THE_RIGHT_TIME2;
	
	@ClientString(id = 1900087, message = "You have to use the card skill on the mask before the gauge above my head disappears.")
	public static NpcStringId YOU_HAVE_TO_USE_THE_CARD_SKILL_ON_THE_MASK_BEFORE_THE_GAUGE_ABOVE_MY_HEAD_DISAPPEARS2;
	
	@ClientString(id = 1900088, message = "You must use the card skill on the mask before the gauge above my head disappears.")
	public static NpcStringId YOU_MUST_USE_THE_CARD_SKILL_ON_THE_MASK_BEFORE_THE_GAUGE_ABOVE_MY_HEAD_DISAPPEARS2;
	
	@ClientString(id = 1900089, message = "If you show the same card as me, you win. If they're different, I win. Understand? Now, let's go!")
	public static NpcStringId IF_YOU_SHOW_THE_SAME_CARD_AS_ME_YOU_WIN_IF_THEY_RE_DIFFERENT_I_WIN_UNDERSTAND_NOW_LET_S_GO2;
	
	@ClientString(id = 1900090, message = "You will win if you show the same card as me. It's my victory if the cards are different. Well, let's start again~")
	public static NpcStringId YOU_WILL_WIN_IF_YOU_SHOW_THE_SAME_CARD_AS_ME_IT_S_MY_VICTORY_IF_THE_CARDS_ARE_DIFFERENT_WELL_LET_S_START_AGAIN2;
	
	@ClientString(id = 1900091, message = "Ack! You didn't show a card? You have to use the card skill before the gauge disappears. Hmph. Then, I'm going.")
	public static NpcStringId ACK_YOU_DIDN_T_SHOW_A_CARD_YOU_HAVE_TO_USE_THE_CARD_SKILL_BEFORE_THE_GAUGE_DISAPPEARS_HMPH_THEN_I_M_GOING2;
	
	@ClientString(id = 1900092, message = "Ahh. You didn't show a card. You must use the card skill at the right time. It's unfortunate. Then, I will go now~")
	public static NpcStringId AHH_YOU_DIDN_T_SHOW_A_CARD_YOU_MUST_USE_THE_CARD_SKILL_AT_THE_RIGHT_TIME_IT_S_UNFORTUNATE_THEN_I_WILL_GO_NOW2;
	
	@ClientString(id = 1900093, message = "Let's learn about the Jack's game together~! You can play with me 3 times.")
	public static NpcStringId LET_S_LEARN_ABOUT_THE_JACK_S_GAME_TOGETHER_YOU_CAN_PLAY_WITH_ME_3_TIMES2;
	
	@ClientString(id = 1900094, message = "Let's start! Show the card you want! The card skill is attached to the mask.")
	public static NpcStringId LET_S_START_SHOW_THE_CARD_YOU_WANT_THE_CARD_SKILL_IS_ATTACHED_TO_THE_MASK2;
	
	@ClientString(id = 1900095, message = "You showed the same card as me, so you win.")
	public static NpcStringId YOU_SHOWED_THE_SAME_CARD_AS_ME_SO_YOU_WIN2;
	
	@ClientString(id = 1900096, message = "You showed a different card from me, so you lose.")
	public static NpcStringId YOU_SHOWED_A_DIFFERENT_CARD_FROM_ME_SO_YOU_LOSE2;
	
	@ClientString(id = 1900097, message = "That was practice, so there's no candy even if you win~")
	public static NpcStringId THAT_WAS_PRACTICE_SO_THERE_S_NO_CANDY_EVEN_IF_YOU_WIN2;
	
	@ClientString(id = 1900098, message = "It's unfortunate. Let's practice one more time.")
	public static NpcStringId IT_S_UNFORTUNATE_LET_S_PRACTICE_ONE_MORE_TIME2;
	
	@ClientString(id = 1900099, message = "You gotta show the card at the right time. Use the card skill you want before the gauge above my head disappears!")
	public static NpcStringId YOU_GOTTA_SHOW_THE_CARD_AT_THE_RIGHT_TIME_USE_THE_CARD_SKILL_YOU_WANT_BEFORE_THE_GAUGE_ABOVE_MY_HEAD_DISAPPEARS2;
	
	@ClientString(id = 1900100, message = "The card skills are attached to the Jack O'Lantern mask, right? That's what you use.")
	public static NpcStringId THE_CARD_SKILLS_ARE_ATTACHED_TO_THE_JACK_O_LANTERN_MASK_RIGHT_THAT_S_WHAT_YOU_USE2;
	
	@ClientString(id = 1900101, message = "You win if you show the same card as me, and I win if the cards are different. OK, let's go~")
	public static NpcStringId YOU_WIN_IF_YOU_SHOW_THE_SAME_CARD_AS_ME_AND_I_WIN_IF_THE_CARDS_ARE_DIFFERENT_OK_LET_S_GO2;
	
	@ClientString(id = 1900102, message = "You didn't show a card again? We'll try again later. I'm gonna go now~")
	public static NpcStringId YOU_DIDN_T_SHOW_A_CARD_AGAIN_WE_LL_TRY_AGAIN_LATER_I_M_GONNA_GO_NOW2;
	
	@ClientString(id = 1900103, message = "Now, do you understand a little about Jack's game? The real game's with Uldie and Belldandy. Well, see you later!")
	public static NpcStringId NOW_DO_YOU_UNDERSTAND_A_LITTLE_ABOUT_JACK_S_GAME_THE_REAL_GAME_S_WITH_ULDIE_AND_BELLDANDY_WELL_SEE_YOU_LATER2;
	
	@ClientString(id = 1900104, message = "Yawn~ Ahh... Hello~! Nice weather we're having.")
	public static NpcStringId YAWN_AHH_HELLO_NICE_WEATHER_WE_RE_HAVING;
	
	@ClientString(id = 1900105, message = "Ah, I'm hungry. Do you have any baby food? That's what I need to eat to get bigger.")
	public static NpcStringId AH_I_M_HUNGRY_DO_YOU_HAVE_ANY_BABY_FOOD_THAT_S_WHAT_I_NEED_TO_EAT_TO_GET_BIGGER;
	
	@ClientString(id = 1900106, message = "I gotta grow up fast. I want to pull Santa's sled, too.")
	public static NpcStringId I_GOTTA_GROW_UP_FAST_I_WANT_TO_PULL_SANTA_S_SLED_TOO;
	
	@ClientString(id = 1900107, message = "Yummy! I think I can grow up now!")
	public static NpcStringId YUMMY_I_THINK_I_CAN_GROW_UP_NOW;
	
	@ClientString(id = 1900108, message = "Thanks to you, I grew up into a Boy Rudolph!")
	public static NpcStringId THANKS_TO_YOU_I_GREW_UP_INTO_A_BOY_RUDOLPH;
	
	@ClientString(id = 1900109, message = "It's great weather for running around.")
	public static NpcStringId IT_S_GREAT_WEATHER_FOR_RUNNING_AROUND;
	
	@ClientString(id = 1900110, message = "What'll I be when I grow up? I wonder.")
	public static NpcStringId WHAT_LL_I_BE_WHEN_I_GROW_UP_I_WONDER;
	
	@ClientString(id = 1900111, message = "If you take good care of me, I'll never forget it!")
	public static NpcStringId IF_YOU_TAKE_GOOD_CARE_OF_ME_I_LL_NEVER_FORGET_IT;
	
	@ClientString(id = 1900112, message = "Please pet me lovingly. You can use the 'Hand of Warmth' skill under the action tab.")
	public static NpcStringId PLEASE_PET_ME_LOVINGLY_YOU_CAN_USE_THE_HAND_OF_WARMTH_SKILL_UNDER_THE_ACTION_TAB;
	
	@ClientString(id = 1900113, message = "I feel great! Thank you!")
	public static NpcStringId I_FEEL_GREAT_THANK_YOU;
	
	@ClientString(id = 1900114, message = "Woo. What a good feeling~ I gotta grow more and more!")
	public static NpcStringId WOO_WHAT_A_GOOD_FEELING_I_GOTTA_GROW_MORE_AND_MORE;
	
	@ClientString(id = 1900115, message = "Oh, yummy! If I keep eating this, I think I can grow up all the way.")
	public static NpcStringId OH_YUMMY_IF_I_KEEP_EATING_THIS_I_THINK_I_CAN_GROW_UP_ALL_THE_WAY;
	
	@ClientString(id = 1900116, message = "Yum yum. Delicious. Give me more of this!")
	public static NpcStringId YUM_YUM_DELICIOUS_GIVE_ME_MORE_OF_THIS;
	
	@ClientString(id = 1900117, message = "Wow. This taste. There's a whole new world in my mouth!")
	public static NpcStringId WOW_THIS_TASTE_THERE_S_A_WHOLE_NEW_WORLD_IN_MY_MOUTH;
	
	@ClientString(id = 1900118, message = "Yeah. It's so delicious. This is that star food!")
	public static NpcStringId YEAH_IT_S_SO_DELICIOUS_THIS_IS_THAT_STAR_FOOD;
	
	@ClientString(id = 1900119, message = "Pay some attention to me! Hmph.")
	public static NpcStringId PAY_SOME_ATTENTION_TO_ME_HMPH;
	
	@ClientString(id = 1900120, message = "Thank you. I was able to grow up into an adult. Here is my gift.")
	public static NpcStringId THANK_YOU_I_WAS_ABLE_TO_GROW_UP_INTO_AN_ADULT_HERE_IS_MY_GIFT;
	
	@ClientString(id = 1900121, message = "Thank you. %s. Now, I can pull the sled.")
	public static NpcStringId THANK_YOU_S_NOW_I_CAN_PULL_THE_SLED;
	
	@ClientString(id = 1900122, message = "%s. Thank you for taking care of me all this time. I enjoyed it very much.")
	public static NpcStringId S_THANK_YOU_FOR_TAKING_CARE_OF_ME_ALL_THIS_TIME_I_ENJOYED_IT_VERY_MUCH;
	
	@ClientString(id = 1900123, message = "%s. It won't be long now until it becomes time to pull the sled. It's too bad.")
	public static NpcStringId S_IT_WON_T_BE_LONG_NOW_UNTIL_IT_BECOMES_TIME_TO_PULL_THE_SLED_IT_S_TOO_BAD;
	
	@ClientString(id = 1900124, message = "I must return to Santa now. Thank you for everything!")
	public static NpcStringId I_MUST_RETURN_TO_SANTA_NOW_THANK_YOU_FOR_EVERYTHING;
	
	@ClientString(id = 1900125, message = "Hello. I'm a girl Rudolph. I was able to find my true self thanks to you.")
	public static NpcStringId HELLO_I_M_A_GIRL_RUDOLPH_I_WAS_ABLE_TO_FIND_MY_TRUE_SELF_THANKS_TO_YOU;
	
	@ClientString(id = 1900126, message = "This is my gift of thanks. Thank you for taking care of me~")
	public static NpcStringId THIS_IS_MY_GIFT_OF_THANKS_THANK_YOU_FOR_TAKING_CARE_OF_ME;
	
	@ClientString(id = 1900127, message = "%s. I was always grateful.")
	public static NpcStringId S_I_WAS_ALWAYS_GRATEFUL;
	
	@ClientString(id = 1900128, message = "I'm a little sad. It's time to leave now.")
	public static NpcStringId I_M_A_LITTLE_SAD_IT_S_TIME_TO_LEAVE_NOW;
	
	@ClientString(id = 1900129, message = "%s. The time has come for me to return to my home.")
	public static NpcStringId S_THE_TIME_HAS_COME_FOR_ME_TO_RETURN_TO_MY_HOME;
	
	@ClientString(id = 1900130, message = "%s. Thank you.")
	public static NpcStringId S_THANK_YOU;
	
	@ClientString(id = 1900131, message = "Hahaha!!! I captured Santa!! Huh? Where is this? Who are you?")
	public static NpcStringId HAHAHA_I_CAPTURED_SANTA_HUH_WHERE_IS_THIS_WHO_ARE_YOU;
	
	@ClientString(id = 1900132, message = "...I lost at Rock Paper Scissors and was taken captive... I might as well take out my anger on you! Huh~ What?")
	public static NpcStringId I_LOST_AT_ROCK_PAPER_SCISSORS_AND_WAS_TAKEN_CAPTIVE_I_MIGHT_AS_WELL_TAKE_OUT_MY_ANGER_ON_YOU_HUH_WHAT;
	
	@ClientString(id = 1900133, message = "Nothing's working... I'm leaving! I'll definitely capture Santa again! Just you wait!")
	public static NpcStringId NOTHING_S_WORKING_I_M_LEAVING_I_LL_DEFINITELY_CAPTURE_SANTA_AGAIN_JUST_YOU_WAIT;
	
	@ClientString(id = 1900134, message = "I must raise Rudolph quickly. This year's Christmas gifts have to be delivered...")
	public static NpcStringId I_MUST_RAISE_RUDOLPH_QUICKLY_THIS_YEAR_S_CHRISTMAS_GIFTS_HAVE_TO_BE_DELIVERED;
	
	@ClientString(id = 1900135, message = "Happy Holidays~ Thanks to your efforts in raising Rudolph, the gift delivery was a success.")
	public static NpcStringId HAPPY_HOLIDAYS_THANKS_TO_YOUR_EFFORTS_IN_RAISING_RUDOLPH_THE_GIFT_DELIVERY_WAS_A_SUCCESS;
	
	@ClientString(id = 1900136, message = "In 10 minutes, it will be 1 hour since you started raising me.")
	public static NpcStringId IN_10_MINUTES_IT_WILL_BE_1_HOUR_SINCE_YOU_STARTED_RAISING_ME;
	
	@ClientString(id = 1900137, message = "After 5 minutes, if my Full Feeling and Affection Level reach 99%, I can grow bigger.")
	public static NpcStringId AFTER_5_MINUTES_IF_MY_FULL_FEELING_AND_AFFECTION_LEVEL_REACH_99_I_CAN_GROW_BIGGER;
	
	@ClientString(id = 1900138, message = "The resupply time of %s hour(s) %s minute(s) %s second(s) remain for the Gift of Energy.")
	public static NpcStringId THE_RESUPPLY_TIME_OF_S_HOUR_S_S_MINUTE_S_S_SECOND_S_REMAIN_FOR_THE_GIFT_OF_ENERGY;
	
	@ClientString(id = 1900139, message = "Want to test your luck? Give me Adena!")
	public static NpcStringId WANT_TO_TEST_YOUR_LUCK_GIVE_ME_ADENA;
	
	@ClientString(id = 1900140, message = "If you give me Adena 5 times, I will test your luck!")
	public static NpcStringId IF_YOU_GIVE_ME_ADENA_5_TIMES_I_WILL_TEST_YOUR_LUCK;
	
	@ClientString(id = 1900141, message = "Yummy. I feel good today!")
	public static NpcStringId YUMMY_I_FEEL_GOOD_TODAY;
	
	@ClientString(id = 1900142, message = "Yamyam. This doesn't feel very good.")
	public static NpcStringId YAMYAM_THIS_DOESN_T_FEEL_VERY_GOOD;
	
	@ClientString(id = 1900143, message = "I'm full~ I'll tell your fortune!")
	public static NpcStringId I_M_FULL_I_LL_TELL_YOUR_FORTUNE;
	
	@ClientString(id = 1900144, message = "Thank you. See you later~")
	public static NpcStringId THANK_YOU_SEE_YOU_LATER;
	
	@ClientString(id = 1900145, message = "Today is the best day of your life! Congratulations! Only good things will come today.")
	public static NpcStringId TODAY_IS_THE_BEST_DAY_OF_YOUR_LIFE_CONGRATULATIONS_ONLY_GOOD_THINGS_WILL_COME_TODAY;
	
	@ClientString(id = 1900146, message = "You're very lucky today! You should enchant on days like this!")
	public static NpcStringId YOU_RE_VERY_LUCKY_TODAY_YOU_SHOULD_ENCHANT_ON_DAYS_LIKE_THIS;
	
	@ClientString(id = 1900147, message = "Ha.. Sorry, but today is not looking very good. Watch out for monsters!")
	public static NpcStringId HA_SORRY_BUT_TODAY_IS_NOT_LOOKING_VERY_GOOD_WATCH_OUT_FOR_MONSTERS;
	
	@ClientString(id = 1900148, message = "Death! Death is near! No one can do anything about it! Muhaha... Ahhahahaha!")
	public static NpcStringId DEATH_DEATH_IS_NEAR_NO_ONE_CAN_DO_ANYTHING_ABOUT_IT_MUHAHA_AHHAHAHAHA;
	
	@ClientString(id = 1900149, message = "Everyone.. Turn into flesh! Kekekeke...")
	public static NpcStringId EVERYONE_TURN_INTO_FLESH_KEKEKEKE;
	
	@ClientString(id = 1900150, message = "Black color ate.. all... of red blood! Understand?!")
	public static NpcStringId BLACK_COLOR_ATE_ALL_OF_RED_BLOOD_UNDERSTAND;
	
	@ClientString(id = 1900151, message = "All gonna die! Smashed like meat.. Will get sucked into the darkness!")
	public static NpcStringId ALL_GONNA_DIE_SMASHED_LIKE_MEAT_WILL_GET_SUCKED_INTO_THE_DARKNESS;
	
	@ClientString(id = 1900152, message = "I don't wanna die... sniff.. I... don't want to die like that..")
	public static NpcStringId I_DON_T_WANNA_DIE_SNIFF_I_DON_T_WANT_TO_DIE_LIKE_THAT;
	
	@ClientString(id = 1900153, message = "Don't...come.. Go away! Ahh...ahhh!")
	public static NpcStringId DON_T_COME_GO_AWAY_AHH_AHHH;
	
	@ClientString(id = 1900154, message = "Give you life to the Goddess of Death! Kahahaha!")
	public static NpcStringId GIVE_YOU_LIFE_TO_THE_GODDESS_OF_DEATH_KAHAHAHA;
	
	@ClientString(id = 1900155, message = "It's not yet complete! Death! Is not yet complete!")
	public static NpcStringId IT_S_NOT_YET_COMPLETE_DEATH_IS_NOT_YET_COMPLETE;
	
	@ClientString(id = 1900156, message = "No.. I.. can't die like this. Go...go away!")
	public static NpcStringId NO_I_CAN_T_DIE_LIKE_THIS_GO_GO_AWAY;
	
	@ClientString(id = 1900157, message = "I saw.. blood.. torn flesh... getting...sucked into... the ground...")
	public static NpcStringId I_SAW_BLOOD_TORN_FLESH_GETTING_SUCKED_INTO_THE_GROUND;
	
	@ClientString(id = 1900158, message = "Ughh.... blood.. drips... bone.... crushed...")
	public static NpcStringId UGHH_BLOOD_DRIPS_BONE_CRUSHED;
	
	@ClientString(id = 1900159, message = "Voice.. was good... as if you're possessed... No! I'm already possessed!")
	public static NpcStringId VOICE_WAS_GOOD_AS_IF_YOU_RE_POSSESSED_NO_I_M_ALREADY_POSSESSED;
	
	@ClientString(id = 1900160, message = "I heard the sound. Sacrifice your blood now. You'll resurrect later! Kehehehe!")
	public static NpcStringId I_HEARD_THE_SOUND_SACRIFICE_YOUR_BLOOD_NOW_YOU_LL_RESURRECT_LATER_KEHEHEHE;
	
	@ClientString(id = 1900161, message = "Don't die! But, you will. I will! You too! Hehehe!")
	public static NpcStringId DON_T_DIE_BUT_YOU_WILL_I_WILL_YOU_TOO_HEHEHE;
	
	@ClientString(id = 1900162, message = "I saw... the end of darkness... I had to give up.")
	public static NpcStringId I_SAW_THE_END_OF_DARKNESS_I_HAD_TO_GIVE_UP;
	
	@ClientString(id = 1900163, message = "Flesh rips...bone crushes. Haha...hehehe...it's red. All!")
	public static NpcStringId FLESH_RIPS_BONE_CRUSHES_HAHA_HEHEHE_IT_S_RED_ALL;
	
	@ClientString(id = 1900164, message = "I do not...want to die. I don't want to...die miserably like that!")
	public static NpcStringId I_DO_NOT_WANT_TO_DIE_I_DON_T_WANT_TO_DIE_MISERABLY_LIKE_THAT;
	
	@ClientString(id = 1900165, message = "Go away! I do not want to see anyone! No... Don't leave me! I'm scared!")
	public static NpcStringId GO_AWAY_I_DO_NOT_WANT_TO_SEE_ANYONE_NO_DON_T_LEAVE_ME_I_M_SCARED;
	
	@ClientString(id = 1900166, message = "I didn't want to see. Hehe.. But, I saw.. I saw! Hehehehe!")
	public static NpcStringId I_DIDN_T_WANT_TO_SEE_HEHE_BUT_I_SAW_I_SAW_HEHEHEHE;
	
	@ClientString(id = 1900167, message = "I heard the sound. Sound of the hell!")
	public static NpcStringId I_HEARD_THE_SOUND_SOUND_OF_THE_HELL;
	
	@ClientString(id = 1900168, message = "Let me.. live!! Please! Help me! That black ground is trying to swallow me!")
	public static NpcStringId LET_ME_LIVE_PLEASE_HELP_ME_THAT_BLACK_GROUND_IS_TRYING_TO_SWALLOW_ME;
	
	@ClientString(id = 1900169, message = "Hundreds and thousands of monsters turned into flesh! Kehehehe!")
	public static NpcStringId HUNDREDS_AND_THOUSANDS_OF_MONSTERS_TURNED_INTO_FLESH_KEHEHEHE;
	
	@ClientString(id = 1900170, message = "I was swallowed after smashed. By darkness! Into the darkness!")
	public static NpcStringId I_WAS_SWALLOWED_AFTER_SMASHED_BY_DARKNESS_INTO_THE_DARKNESS;
	
	@ClientString(id = 1900171, message = "I don't like it! Don't touch me! Please.. I beg you... Sniff... Sniff...")
	public static NpcStringId I_DON_T_LIKE_IT_DON_T_TOUCH_ME_PLEASE_I_BEG_YOU_SNIFF_SNIFF;
	
	@ClientString(id = 1900172, message = "Sweet voice. Hehe.. You die if you get possessed by that sound. Like them! Hehe...hahaha!")
	public static NpcStringId SWEET_VOICE_HEHE_YOU_DIE_IF_YOU_GET_POSSESSED_BY_THAT_SOUND_LIKE_THEM_HEHE_HAHAHA;
	
	@ClientString(id = 1900173, message = "They will resurrect. She will... They will... Heh...")
	public static NpcStringId THEY_WILL_RESURRECT_SHE_WILL_THEY_WILL_HEH;
	
	@ClientString(id = 1900174, message = "Dead.. or alive... I don't want to see anything..")
	public static NpcStringId DEAD_OR_ALIVE_I_DON_T_WANT_TO_SEE_ANYTHING;
	
	@ClientString(id = 1900175, message = "Death. No one can do anything! Deep! Deep death!")
	public static NpcStringId DEATH_NO_ONE_CAN_DO_ANYTHING_DEEP_DEEP_DEATH;
	
	@ClientString(id = 1900176, message = "Even if you try, it's useless. Die with me. Keheheh!")
	public static NpcStringId EVEN_IF_YOU_TRY_IT_S_USELESS_DIE_WITH_ME_KEHEHEH;
	
	@ClientString(id = 1900177, message = "Dark voice.. Can you hear? I.. can hear.. But... I don't want to..")
	public static NpcStringId DARK_VOICE_CAN_YOU_HEAR_I_CAN_HEAR_BUT_I_DON_T_WANT_TO;
	
	@ClientString(id = 1900178, message = "I can still hear the sound.. of flesh and bones grinding. Hehehehe!")
	public static NpcStringId I_CAN_STILL_HEAR_THE_SOUND_OF_FLESH_AND_BONES_GRINDING_HEHEHEHE;
	
	@ClientString(id = 1900179, message = "Eek! Run away! If you want to live... If you want to stay alive! Leave here! Right now!")
	public static NpcStringId EEK_RUN_AWAY_IF_YOU_WANT_TO_LIVE_IF_YOU_WANT_TO_STAY_ALIVE_LEAVE_HERE_RIGHT_NOW;
	
	@ClientString(id = 1900180, message = "You don't want to die? But, you'll die soon. Keheheh!")
	public static NpcStringId YOU_DON_T_WANT_TO_DIE_BUT_YOU_LL_DIE_SOON_KEHEHEH;
	
	@ClientString(id = 1900181, message = "There's no chance! No chance at all! You! And I will! Become red flesh! Keheheh!")
	public static NpcStringId THERE_S_NO_CHANCE_NO_CHANCE_AT_ALL_YOU_AND_I_WILL_BECOME_RED_FLESH_KEHEHEH;
	
	@ClientString(id = 1900182, message = "Die! Before she swallows you! Die!")
	public static NpcStringId DIE_BEFORE_SHE_SWALLOWS_YOU_DIE;
	
	@ClientString(id = 1900183, message = "Don't wanna die. But, I can't live either. Sniff... Sniff...")
	public static NpcStringId DON_T_WANNA_DIE_BUT_I_CAN_T_LIVE_EITHER_SNIFF_SNIFF;
	
	@ClientString(id = 1900184, message = "Eek! Don't kill me! I don't wanna die. I don't wanna become a patty of meat.. No.. No!")
	public static NpcStringId EEK_DON_T_KILL_ME_I_DON_T_WANNA_DIE_I_DON_T_WANNA_BECOME_A_PATTY_OF_MEAT_NO_NO;
	
	@ClientString(id = 1900185, message = "I can hear.. the bone crushing sound..")
	public static NpcStringId I_CAN_HEAR_THE_BONE_CRUSHING_SOUND;
	
	@ClientString(id = 1900186, message = "I saw the sea of blood. That day..I.. Ahhhh!")
	public static NpcStringId I_SAW_THE_SEA_OF_BLOOD_THAT_DAY_I_AHHHH;
	
	@ClientString(id = 1900187, message = "I couldn't run.. I saw... Everything! I was already dead then. Kehe...hahaha..")
	public static NpcStringId I_COULDN_T_RUN_I_SAW_EVERYTHING_I_WAS_ALREADY_DEAD_THEN_KEHE_HAHAHA;
	
	@ClientString(id = 1900188, message = "I'm gonna get caught! Then you'll die! Ahh! God have forsaken us!")
	public static NpcStringId I_M_GONNA_GET_CAUGHT_THEN_YOU_LL_DIE_AHH_GOD_HAVE_FORSAKEN_US;
	
	@ClientString(id = 1900189, message = "I want to live! I don't want to be swallowed to death!")
	public static NpcStringId I_WANT_TO_LIVE_I_DON_T_WANT_TO_BE_SWALLOWED_TO_DEATH;
	
	@ClientString(id = 1900190, message = "Nothing comes out neither from inside or outside.")
	public static NpcStringId NOTHING_COMES_OUT_NEITHER_FROM_INSIDE_OR_OUTSIDE;
	
	@ClientString(id = 1900191, message = "As it didn't exist")
	public static NpcStringId AS_IT_DIDN_T_EXIST;
	
	@ClientString(id = 1900192, message = "It left nothing behind..")
	public static NpcStringId IT_LEFT_NOTHING_BEHIND;
	
	@ClientString(id = 1900193, message = "Should we report it to the kingdom?")
	public static NpcStringId SHOULD_WE_REPORT_IT_TO_THE_KINGDOM;
	
	@ClientString(id = 1900194, message = "I'm in a pickle. We can't go back. Let's look further.")
	public static NpcStringId I_M_IN_A_PICKLE_WE_CAN_T_GO_BACK_LET_S_LOOK_FURTHER;
	
	@ClientString(id = 1900195, message = "We'll begin internal purification process.")
	public static NpcStringId WE_LL_BEGIN_INTERNAL_PURIFICATION_PROCESS;
	
	@ClientString(id = 1900196, message = "Please.")
	public static NpcStringId PLEASE;
	
	@ClientString(id = 1900197, message = "Nothing can be found.")
	public static NpcStringId NOTHING_CAN_BE_FOUND;
	
	@ClientString(id = 1900198, message = "As if it didn't exist from the beginning")
	public static NpcStringId AS_IF_IT_DIDN_T_EXIST_FROM_THE_BEGINNING;
	
	@ClientString(id = 1900199, message = "Hm..")
	public static NpcStringId HM;
	
	@ClientString(id = 1900200, message = "I heard the witness went crazy after this incident.")
	public static NpcStringId I_HEARD_THE_WITNESS_WENT_CRAZY_AFTER_THIS_INCIDENT;
	
	@ClientString(id = 1900201, message = "I think we must begin purifying this bad aura before we do anything.")
	public static NpcStringId I_THINK_WE_MUST_BEGIN_PURIFYING_THIS_BAD_AURA_BEFORE_WE_DO_ANYTHING;
	
	@ClientString(id = 1900202, message = "I will report to Kingdom.")
	public static NpcStringId I_WILL_REPORT_TO_KINGDOM;
	
	@ClientString(id = 1900203, message = "Yes. You can do that.")
	public static NpcStringId YES_YOU_CAN_DO_THAT;
	
	@ClientString(id = 1900204, message = "From what I heard from the witness, I heard the monsters were completely ripped apart and absorbed into the ground.")
	public static NpcStringId FROM_WHAT_I_HEARD_FROM_THE_WITNESS_I_HEARD_THE_MONSTERS_WERE_COMPLETELY_RIPPED_APART_AND_ABSORBED_INTO_THE_GROUND;
	
	@ClientString(id = 1900205, message = "He's a crazy man, so it's not so credible...")
	public static NpcStringId HE_S_A_CRAZY_MAN_SO_IT_S_NOT_SO_CREDIBLE;
	
	@ClientString(id = 1900206, message = "Sigh.. It doesn't make sense that this brutal scene has nothing left behind.")
	public static NpcStringId SIGH_IT_DOESN_T_MAKE_SENSE_THAT_THIS_BRUTAL_SCENE_HAS_NOTHING_LEFT_BEHIND;
	
	@ClientString(id = 1900207, message = "What should we do?")
	public static NpcStringId WHAT_SHOULD_WE_DO;
	
	@ClientString(id = 1900208, message = "Let's report it to the kingdom first.")
	public static NpcStringId LET_S_REPORT_IT_TO_THE_KINGDOM_FIRST;
	
	@ClientString(id = 1900209, message = "Okay.")
	public static NpcStringId OKAY;
	
	@ClientString(id = 1900210, message = "A crazy man said all the monsters were shredded and absorbed into the ground.")
	public static NpcStringId A_CRAZY_MAN_SAID_ALL_THE_MONSTERS_WERE_SHREDDED_AND_ABSORBED_INTO_THE_GROUND;
	
	@ClientString(id = 1900211, message = "What do you think?")
	public static NpcStringId WHAT_DO_YOU_THINK;
	
	@ClientString(id = 1900212, message = "Unbelievable words from unbelievable man.")
	public static NpcStringId UNBELIEVABLE_WORDS_FROM_UNBELIEVABLE_MAN;
	
	@ClientString(id = 1900213, message = "I think so too. But, there's no practical way to explain why the monsters have disappeared..")
	public static NpcStringId I_THINK_SO_TOO_BUT_THERE_S_NO_PRACTICAL_WAY_TO_EXPLAIN_WHY_THE_MONSTERS_HAVE_DISAPPEARED;
	
	@ClientString(id = 1900214, message = "Sigh.. We'll begin internal purification process.")
	public static NpcStringId SIGH_WE_LL_BEGIN_INTERNAL_PURIFICATION_PROCESS;
	
	@ClientString(id = 1900215, message = "Yes. Then I will report back to the Kingdom.")
	public static NpcStringId YES_THEN_I_WILL_REPORT_BACK_TO_THE_KINGDOM;
	
	@ClientString(id = 1900216, message = "I can't believe it.")
	public static NpcStringId I_CAN_T_BELIEVE_IT;
	
	@ClientString(id = 1900217, message = "Monsters all disappeared at once...")
	public static NpcStringId MONSTERS_ALL_DISAPPEARED_AT_ONCE;
	
	@ClientString(id = 1900218, message = "I never felt this kind of deep darkness before.")
	public static NpcStringId I_NEVER_FELT_THIS_KIND_OF_DEEP_DARKNESS_BEFORE;
	
	@ClientString(id = 1900219, message = "I wonder if we can purify this.")
	public static NpcStringId I_WONDER_IF_WE_CAN_PURIFY_THIS;
	
	@ClientString(id = 1900220, message = "This may be difficult, but please.")
	public static NpcStringId THIS_MAY_BE_DIFFICULT_BUT_PLEASE;
	
	@ClientString(id = 1900221, message = "Stop outsiders from approaching.")
	public static NpcStringId STOP_OUTSIDERS_FROM_APPROACHING;
	
	@ClientString(id = 1900222, message = "Okay.")
	public static NpcStringId OKAY2;
	
	@ClientString(id = 1900223, message = "Sigh..")
	public static NpcStringId SIGH;
	
	@ClientString(id = 1900224, message = "Other places can't seem to find anything either.")
	public static NpcStringId OTHER_PLACES_CAN_T_SEEM_TO_FIND_ANYTHING_EITHER;
	
	@ClientString(id = 1900225, message = "If it's even possible for all monsters to disappear from all catacombs and necropolis...")
	public static NpcStringId IF_IT_S_EVEN_POSSIBLE_FOR_ALL_MONSTERS_TO_DISAPPEAR_FROM_ALL_CATACOMBS_AND_NECROPOLIS;
	
	@ClientString(id = 1900226, message = "The witness went crazy. He's not being very helpful.")
	public static NpcStringId THE_WITNESS_WENT_CRAZY_HE_S_NOT_BEING_VERY_HELPFUL;
	
	@ClientString(id = 1900227, message = "I think we must eliminate this dark aura first.")
	public static NpcStringId I_THINK_WE_MUST_ELIMINATE_THIS_DARK_AURA_FIRST;
	
	@ClientString(id = 1900228, message = "I think you should investigate afterwards.")
	public static NpcStringId I_THINK_YOU_SHOULD_INVESTIGATE_AFTERWARDS;
	
	@ClientString(id = 1900229, message = "Please.")
	public static NpcStringId PLEASE2;
	
	@ClientString(id = 1900230, message = "Do you think it's possible for all monsters to disappear at once?")
	public static NpcStringId DO_YOU_THINK_IT_S_POSSIBLE_FOR_ALL_MONSTERS_TO_DISAPPEAR_AT_ONCE;
	
	@ClientString(id = 1900231, message = "I.. don't believe that it's possible.")
	public static NpcStringId I_DON_T_BELIEVE_THAT_IT_S_POSSIBLE;
	
	@ClientString(id = 1900232, message = "I don't think it's possible...")
	public static NpcStringId I_DON_T_THINK_IT_S_POSSIBLE;
	
	@ClientString(id = 1900233, message = "The witness went crazy. His words are meaningless.")
	public static NpcStringId THE_WITNESS_WENT_CRAZY_HIS_WORDS_ARE_MEANINGLESS;
	
	@ClientString(id = 1900234, message = "Sigh.. It's tiring trying to guess.")
	public static NpcStringId SIGH_IT_S_TIRING_TRYING_TO_GUESS;
	
	@ClientString(id = 1900235, message = "First.. Please take care of the purification process.")
	public static NpcStringId FIRST_PLEASE_TAKE_CARE_OF_THE_PURIFICATION_PROCESS;
	
	@ClientString(id = 1900236, message = "Then I will report back to the Kingdom.")
	public static NpcStringId THEN_I_WILL_REPORT_BACK_TO_THE_KINGDOM;
	
	@ClientString(id = 1900237, message = "Okay.")
	public static NpcStringId OKAY3;
	
	@ClientString(id = 1900238, message = "I'm worried.")
	public static NpcStringId I_M_WORRIED;
	
	@ClientString(id = 1900239, message = "Other places can't seem to find anything either.")
	public static NpcStringId OTHER_PLACES_CAN_T_SEEM_TO_FIND_ANYTHING_EITHER2;
	
	@ClientString(id = 1900240, message = "Do you think it's possible for all monsters to disappear from all catacombs and necropolis?")
	public static NpcStringId DO_YOU_THINK_IT_S_POSSIBLE_FOR_ALL_MONSTERS_TO_DISAPPEAR_FROM_ALL_CATACOMBS_AND_NECROPOLIS;
	
	@ClientString(id = 1900241, message = "The witness went crazy. He's not being very helpful.")
	public static NpcStringId THE_WITNESS_WENT_CRAZY_HE_S_NOT_BEING_VERY_HELPFUL2;
	
	@ClientString(id = 1900242, message = "I think we must eliminate this dark aura first.")
	public static NpcStringId I_THINK_WE_MUST_ELIMINATE_THIS_DARK_AURA_FIRST2;
	
	@ClientString(id = 1900243, message = "Please.")
	public static NpcStringId PLEASE3;
	
	@ClientString(id = 1900244, message = "Ahh.. What should we do with this situation...")
	public static NpcStringId AHH_WHAT_SHOULD_WE_DO_WITH_THIS_SITUATION;
	
	@ClientString(id = 1900245, message = "Calm down.")
	public static NpcStringId CALM_DOWN;
	
	@ClientString(id = 1900246, message = "Does it even make sense that all monsters from Catacomb and Necropolis disappeared?")
	public static NpcStringId DOES_IT_EVEN_MAKE_SENSE_THAT_ALL_MONSTERS_FROM_CATACOMB_AND_NECROPOLIS_DISAPPEARED;
	
	@ClientString(id = 1900247, message = "This has never happened in the past.")
	public static NpcStringId THIS_HAS_NEVER_HAPPENED_IN_THE_PAST;
	
	@ClientString(id = 1900248, message = "First.. Calm down...")
	public static NpcStringId FIRST_CALM_DOWN;
	
	@ClientString(id = 1900249, message = "Shilen...Shilen may be resurrecting...")
	public static NpcStringId SHILEN_SHILEN_MAY_BE_RESURRECTING;
	
	@ClientString(id = 1900250, message = "Sigh.. First... I will start with purification process.")
	public static NpcStringId SIGH_FIRST_I_WILL_START_WITH_PURIFICATION_PROCESS;
	
	@ClientString(id = 1900251, message = "When you're calm, please report to the kingdom.")
	public static NpcStringId WHEN_YOU_RE_CALM_PLEASE_REPORT_TO_THE_KINGDOM;
	
	@ClientString(id = 1900252, message = "What do you think?")
	public static NpcStringId WHAT_DO_YOU_THINK2;
	
	@ClientString(id = 1900253, message = "This doesn't seem like an ordinary situation.")
	public static NpcStringId THIS_DOESN_T_SEEM_LIKE_AN_ORDINARY_SITUATION;
	
	@ClientString(id = 1900254, message = "If that crazy man's words are true... Monsters... May have been... absorbed by Shilen.")
	public static NpcStringId IF_THAT_CRAZY_MAN_S_WORDS_ARE_TRUE_MONSTERS_MAY_HAVE_BEEN_ABSORBED_BY_SHILEN;
	
	@ClientString(id = 1900255, message = "Shilen.. I don't even know how to report for this.")
	public static NpcStringId SHILEN_I_DON_T_EVEN_KNOW_HOW_TO_REPORT_FOR_THIS;
	
	@ClientString(id = 1900256, message = "I will purify this place first.")
	public static NpcStringId I_WILL_PURIFY_THIS_PLACE_FIRST;
	
	@ClientString(id = 1900257, message = "Please.")
	public static NpcStringId PLEASE4;
	
	@ClientString(id = 1900258, message = "What he said is out of ordinary.")
	public static NpcStringId WHAT_HE_SAID_IS_OUT_OF_ORDINARY;
	
	@ClientString(id = 1900259, message = "The monsters were shredded into pieces and absorbed into the ground...")
	public static NpcStringId THE_MONSTERS_WERE_SHREDDED_INTO_PIECES_AND_ABSORBED_INTO_THE_GROUND;
	
	@ClientString(id = 1900260, message = "This is not something any person can do.")
	public static NpcStringId THIS_IS_NOT_SOMETHING_ANY_PERSON_CAN_DO;
	
	@ClientString(id = 1900261, message = "Then... I see... Shilen sounds like a viable theory.")
	public static NpcStringId THEN_I_SEE_SHILEN_SOUNDS_LIKE_A_VIABLE_THEORY;
	
	@ClientString(id = 1900262, message = "It may not be the truth.")
	public static NpcStringId IT_MAY_NOT_BE_THE_TRUTH;
	
	@ClientString(id = 1900263, message = "You may be correct.")
	public static NpcStringId YOU_MAY_BE_CORRECT;
	
	@ClientString(id = 1900264, message = "We should report to the Kingdom first.")
	public static NpcStringId WE_SHOULD_REPORT_TO_THE_KINGDOM_FIRST;
	
	@ClientString(id = 1900265, message = "Okay.")
	public static NpcStringId OKAY4;
	
	@ClientString(id = 1900266, message = "Ha.. I have no idea.")
	public static NpcStringId HA_I_HAVE_NO_IDEA;
	
	@ClientString(id = 1900267, message = "I can't find any trace. Any trace of the monsters!")
	public static NpcStringId I_CAN_T_FIND_ANY_TRACE_ANY_TRACE_OF_THE_MONSTERS;
	
	@ClientString(id = 1900268, message = "Calm down.")
	public static NpcStringId CALM_DOWN2;
	
	@ClientString(id = 1900269, message = "No race can do such thing.")
	public static NpcStringId NO_RACE_CAN_DO_SUCH_THING;
	
	@ClientString(id = 1900270, message = "If anyone could... he would be 'God.'")
	public static NpcStringId IF_ANYONE_COULD_HE_WOULD_BE_GOD;
	
	@ClientString(id = 1900271, message = "First.. We'll purify this evil aura.")
	public static NpcStringId FIRST_WE_LL_PURIFY_THIS_EVIL_AURA;
	
	@ClientString(id = 1900272, message = "Please.")
	public static NpcStringId PLEASE5;
	
	@ClientString(id = 1900273, message = "We'll go into investigation when the purification is complete.")
	public static NpcStringId WE_LL_GO_INTO_INVESTIGATION_WHEN_THE_PURIFICATION_IS_COMPLETE;
	
	@ClientString(id = 1900274, message = "Okay.")
	public static NpcStringId OKAY5;
	
	@ClientString(id = 1900275, message = "What do you think about what that crazy man said?")
	public static NpcStringId WHAT_DO_YOU_THINK_ABOUT_WHAT_THAT_CRAZY_MAN_SAID;
	
	@ClientString(id = 1900276, message = "It sounds like crazy talk from a crazy man?")
	public static NpcStringId IT_SOUNDS_LIKE_CRAZY_TALK_FROM_A_CRAZY_MAN;
	
	@ClientString(id = 1900277, message = "When there's no proof... Don't you think it sounds pretty credible?")
	public static NpcStringId WHEN_THERE_S_NO_PROOF_DON_T_YOU_THINK_IT_SOUNDS_PRETTY_CREDIBLE;
	
	@ClientString(id = 1900278, message = "You're putting too much weight into it.")
	public static NpcStringId YOU_RE_PUTTING_TOO_MUCH_WEIGHT_INTO_IT;
	
	@ClientString(id = 1900279, message = "It's unrealistic that all monsters would disappear at once.")
	public static NpcStringId IT_S_UNREALISTIC_THAT_ALL_MONSTERS_WOULD_DISAPPEAR_AT_ONCE;
	
	@ClientString(id = 1900280, message = "Even if you believe that... there's really not much we can do.")
	public static NpcStringId EVEN_IF_YOU_BELIEVE_THAT_THERE_S_REALLY_NOT_MUCH_WE_CAN_DO;
	
	@ClientString(id = 1900281, message = "Please purify inside of the building.")
	public static NpcStringId PLEASE_PURIFY_INSIDE_OF_THE_BUILDING;
	
	@ClientString(id = 1900282, message = "Okay.")
	public static NpcStringId OKAY6;
	
	@ClientString(id = 1900283, message = "It's a strange thing.")
	public static NpcStringId IT_S_A_STRANGE_THING;
	
	@ClientString(id = 1900284, message = "Right. It's definitely not a human being can do.")
	public static NpcStringId RIGHT_IT_S_DEFINITELY_NOT_A_HUMAN_BEING_CAN_DO;
	
	@ClientString(id = 1900285, message = "Monsters all disappeared at once...")
	public static NpcStringId MONSTERS_ALL_DISAPPEARED_AT_ONCE2;
	
	@ClientString(id = 1900286, message = "Do you think God did this?")
	public static NpcStringId DO_YOU_THINK_GOD_DID_THIS;
	
	@ClientString(id = 1900287, message = "Someone who can do this... is probably... Goddess of Death...")
	public static NpcStringId SOMEONE_WHO_CAN_DO_THIS_IS_PROBABLY_GODDESS_OF_DEATH;
	
	@ClientString(id = 1900288, message = "That's the worst guess.")
	public static NpcStringId THAT_S_THE_WORST_GUESS;
	
	@ClientString(id = 1900289, message = "I think it's best that we do not tell the Kingdom yet.")
	public static NpcStringId I_THINK_IT_S_BEST_THAT_WE_DO_NOT_TELL_THE_KINGDOM_YET;
	
	@ClientString(id = 1900290, message = "We must find something that can be a clue...")
	public static NpcStringId WE_MUST_FIND_SOMETHING_THAT_CAN_BE_A_CLUE;
	
	@ClientString(id = 1910172, message = "Show respect to the heroes who defeated the evil dragon and protected this Aden world!")
	public static NpcStringId SHOW_RESPECT_TO_THE_HEROES_WHO_DEFEATED_THE_EVIL_DRAGON_AND_PROTECTED_THIS_ADEN_WORLD;
	
	@ClientString(id = 1910173, message = "Shout to celebrate the victory of the heroes!")
	public static NpcStringId SHOUT_TO_CELEBRATE_THE_VICTORY_OF_THE_HEROES;
	
	@ClientString(id = 1910174, message = "Praise the achievement of the heroes and receive Tersi's blessing!")
	public static NpcStringId PRAISE_THE_ACHIEVEMENT_OF_THE_HEROES_AND_RECEIVE_TERSI_S_BLESSING;
	
	@ClientString(id = 1911111, message = "You will soon become the sacrifice for us, those full of deceit and sin whom you despise!")
	public static NpcStringId YOU_WILL_SOON_BECOME_THE_SACRIFICE_FOR_US_THOSE_FULL_OF_DECEIT_AND_SIN_WHOM_YOU_DESPISE;
	
	@ClientString(id = 1911112, message = "My brethren who are stronger than me will punish you. You will soon be covered in your own blood and crying in anguish!")
	public static NpcStringId MY_BRETHREN_WHO_ARE_STRONGER_THAN_ME_WILL_PUNISH_YOU_YOU_WILL_SOON_BE_COVERED_IN_YOUR_OWN_BLOOD_AND_CRYING_IN_ANGUISH;
	
	@ClientString(id = 1911113, message = "How could I lose against these worthless creatures...?")
	public static NpcStringId HOW_COULD_I_LOSE_AGAINST_THESE_WORTHLESS_CREATURES;
	
	@ClientString(id = 1911114, message = "Foolish creatures... the flames of hell are drawing closer.")
	public static NpcStringId FOOLISH_CREATURES_THE_FLAMES_OF_HELL_ARE_DRAWING_CLOSER;
	
	@ClientString(id = 1911115, message = "No matter how you struggle this place will soon be covered with your blood.")
	public static NpcStringId NO_MATTER_HOW_YOU_STRUGGLE_THIS_PLACE_WILL_SOON_BE_COVERED_WITH_YOUR_BLOOD;
	
	@ClientString(id = 1911116, message = "Those who set foot in this place shall not leave alive.")
	public static NpcStringId THOSE_WHO_SET_FOOT_IN_THIS_PLACE_SHALL_NOT_LEAVE_ALIVE;
	
	@ClientString(id = 1911117, message = "Worthless creatures, I will grant you eternal sleep in fire and brimstone.")
	public static NpcStringId WORTHLESS_CREATURES_I_WILL_GRANT_YOU_ETERNAL_SLEEP_IN_FIRE_AND_BRIMSTONE;
	
	@ClientString(id = 1911118, message = "If you wish to see hell, I will grant you your wish.")
	public static NpcStringId IF_YOU_WISH_TO_SEE_HELL_I_WILL_GRANT_YOU_YOUR_WISH;
	
	@ClientString(id = 1911119, message = "Elapsed Time :")
	public static NpcStringId ELAPSED_TIME;
	
	@ClientString(id = 1911120, message = "Time Remaining :")
	public static NpcStringId TIME_REMAINING;
	
	@ClientString(id = 2000000, message = "Is this the surface world?")
	public static NpcStringId IS_THIS_THE_SURFACE_WORLD;
	
	@ClientString(id = 2000001, message = "I can feel a dangerous energy! I must find young mistress and young master as soon as possible...")
	public static NpcStringId I_CAN_FEEL_A_DANGEROUS_ENERGY_I_MUST_FIND_YOUNG_MISTRESS_AND_YOUNG_MASTER_AS_SOON_AS_POSSIBLE;
	
	@ClientString(id = 2000002, message = "Urgh! Above world people are stronger than I thought~ I'll need to leave a trace to let young master and young mistress know that I was here.")
	public static NpcStringId URGH_ABOVE_WORLD_PEOPLE_ARE_STRONGER_THAN_I_THOUGHT_I_LL_NEED_TO_LEAVE_A_TRACE_TO_LET_YOUNG_MASTER_AND_YOUNG_MISTRESS_KNOW_THAT_I_WAS_HERE;
	
	@ClientString(id = 2000003, message = "I'm not strong enough! But I won't give up. I will go to the underworld to recharge my energy, then I will come back.")
	public static NpcStringId I_M_NOT_STRONG_ENOUGH_BUT_I_WON_T_GIVE_UP_I_WILL_GO_TO_THE_UNDERWORLD_TO_RECHARGE_MY_ENERGY_THEN_I_WILL_COME_BACK;
	
	@ClientString(id = 2000004, message = "Miss Uldie! Where are you? It's time to return to the underworld.")
	public static NpcStringId MISS_ULDIE_WHERE_ARE_YOU_IT_S_TIME_TO_RETURN_TO_THE_UNDERWORLD;
	
	@ClientString(id = 2000005, message = "Miss Belldandy! The master is searching for you.")
	public static NpcStringId MISS_BELLDANDY_THE_MASTER_IS_SEARCHING_FOR_YOU;
	
	@ClientString(id = 2000006, message = "Young master Skooldie! It's time to go home. Where are you?")
	public static NpcStringId YOUNG_MASTER_SKOOLDIE_IT_S_TIME_TO_GO_HOME_WHERE_ARE_YOU;
	
	@ClientString(id = 2000007, message = "You rude above world people! Stop being so discourteous!")
	public static NpcStringId YOU_RUDE_ABOVE_WORLD_PEOPLE_STOP_BEING_SO_DISCOURTEOUS;
	
	@ClientString(id = 2000008, message = "You above world people! Feel the amazing power of this great Norhand!")
	public static NpcStringId YOU_ABOVE_WORLD_PEOPLE_FEEL_THE_AMAZING_POWER_OF_THIS_GREAT_NORHAND;
	
	@ClientString(id = 2000009, message = "You above world people! It is time for you to go back the way you came.")
	public static NpcStringId YOU_ABOVE_WORLD_PEOPLE_IT_IS_TIME_FOR_YOU_TO_GO_BACK_THE_WAY_YOU_CAME;
	
	@ClientString(id = 2000010, message = "You received stage 1 Holiday Blessing.")
	public static NpcStringId YOU_RECEIVED_STAGE_1_HOLIDAY_BLESSING;
	
	@ClientString(id = 2000011, message = "You received stage 2 Holiday Blessing.")
	public static NpcStringId YOU_RECEIVED_STAGE_2_HOLIDAY_BLESSING;
	
	@ClientString(id = 2000012, message = "You received stage 3 Holiday Blessing.")
	public static NpcStringId YOU_RECEIVED_STAGE_3_HOLIDAY_BLESSING;
	
	@ClientString(id = 2000013, message = "You received stage 4 Holiday Blessing.")
	public static NpcStringId YOU_RECEIVED_STAGE_4_HOLIDAY_BLESSING;
	
	@ClientString(id = 2000014, message = "You received stage 5 Holiday Blessing.")
	public static NpcStringId YOU_RECEIVED_STAGE_5_HOLIDAY_BLESSING;
	
	@ClientString(id = 2000015, message = "Hey, who are you? Get off! This tree decoration is mine!!")
	public static NpcStringId HEY_WHO_ARE_YOU_GET_OFF_THIS_TREE_DECORATION_IS_MINE;
	
	@ClientString(id = 2000016, message = "Give yourself in to your instincts, new ruler! Seek true power!")
	public static NpcStringId GIVE_YOURSELF_IN_TO_YOUR_INSTINCTS_NEW_RULER_SEEK_TRUE_POWER;
	
	@ClientString(id = 2000017, message = "The Mysterious Power of Kartia's Labyrinth shall give you great power.")
	public static NpcStringId THE_MYSTERIOUS_POWER_OF_KARTIA_S_LABYRINTH_SHALL_GIVE_YOU_GREAT_POWER;
	
	@ClientString(id = 2000018, message = "Keep the Crystal of Protection safe from enemies!")
	public static NpcStringId KEEP_THE_CRYSTAL_OF_PROTECTION_SAFE_FROM_ENEMIES;
	
	@ClientString(id = 2000019, message = "Claim the Source of Kartia's Power, and taste the sweetness of true strength!")
	public static NpcStringId CLAIM_THE_SOURCE_OF_KARTIA_S_POWER_AND_TASTE_THE_SWEETNESS_OF_TRUE_STRENGTH;
	
	@ClientString(id = 2000020, message = "You cannot show your condolences yet. Please try again later.")
	public static NpcStringId YOU_CANNOT_SHOW_YOUR_CONDOLENCES_YET_PLEASE_TRY_AGAIN_LATER;
	
	@ClientString(id = 2000021, message = "You are too far from Uthanka's Tombstone to show your condolences.")
	public static NpcStringId YOU_ARE_TOO_FAR_FROM_UTHANKA_S_TOMBSTONE_TO_SHOW_YOUR_CONDOLENCES;
	
	@ClientString(id = 2000022, message = "You have mourned for the Pirate Captain Uthanka.")
	public static NpcStringId YOU_HAVE_MOURNED_FOR_THE_PIRATE_CAPTAIN_UTHANKA;
	
	@ClientString(id = 2000023, message = "Hey, let's duke it out, you and me! Haven't worked up a sweat for a while!")
	public static NpcStringId HEY_LET_S_DUKE_IT_OUT_YOU_AND_ME_HAVEN_T_WORKED_UP_A_SWEAT_FOR_A_WHILE;
	
	@ClientString(id = 2000024, message = "Ouch! That hurts! Stop it stop it stop it!")
	public static NpcStringId OUCH_THAT_HURTS_STOP_IT_STOP_IT_STOP_IT;
	
	@ClientString(id = 2000025, message = "How could you do that to me! I'm the Great Captain Uthanka, back after being gone for, like, ever!")
	public static NpcStringId HOW_COULD_YOU_DO_THAT_TO_ME_I_M_THE_GREAT_CAPTAIN_UTHANKA_BACK_AFTER_BEING_GONE_FOR_LIKE_EVER;
	
	@ClientString(id = 2000026, message = "You have received the Mysterious Power of Kartia's Labyrinth.")
	public static NpcStringId YOU_HAVE_RECEIVED_THE_MYSTERIOUS_POWER_OF_KARTIA_S_LABYRINTH;
	
	@ClientString(id = 2000027, message = "You have received the Strong Mysterious Power of Kartia's Labyrinth.")
	public static NpcStringId YOU_HAVE_RECEIVED_THE_STRONG_MYSTERIOUS_POWER_OF_KARTIA_S_LABYRINTH;
	
	@ClientString(id = 2000028, message = "I rest my hopes on your feats.")
	public static NpcStringId I_REST_MY_HOPES_ON_YOUR_FEATS;
	
	@ClientString(id = 2000029, message = "I have seen your true powers. Please allow me to follow you.")
	public static NpcStringId I_HAVE_SEEN_YOUR_TRUE_POWERS_PLEASE_ALLOW_ME_TO_FOLLOW_YOU;
	
	@ClientString(id = 2000030, message = "I offer you all that I have, Master.")
	public static NpcStringId I_OFFER_YOU_ALL_THAT_I_HAVE_MASTER;
	
	@ClientString(id = 2000031, message = "It seems your fate ends here.")
	public static NpcStringId IT_SEEMS_YOUR_FATE_ENDS_HERE;
	
	@ClientString(id = 2000032, message = "You're an interesting one… I can feel the power of the giants from you, and you're not even a giant!")
	public static NpcStringId YOU_RE_AN_INTERESTING_ONE_I_CAN_FEEL_THE_POWER_OF_THE_GIANTS_FROM_YOU_AND_YOU_RE_NOT_EVEN_A_GIANT;
	
	@ClientString(id = 2000033, message = "Display your dominance over either Kajin or Beryn,#and the other one will submit to you.")
	public static NpcStringId DISPLAY_YOUR_DOMINANCE_OVER_EITHER_KAJIN_OR_BERYN_AND_THE_OTHER_ONE_WILL_SUBMIT_TO_YOU;
	
	@ClientString(id = 2000034, message = "The Ancient Kartia Records hold the beginning and the end of Ancient Kartia's Labyrinth,#and everything in between.")
	public static NpcStringId THE_ANCIENT_KARTIA_RECORDS_HOLD_THE_BEGINNING_AND_THE_END_OF_ANCIENT_KARTIA_S_LABYRINTH_AND_EVERYTHING_IN_BETWEEN;
	
	@ClientString(id = 2000035, message = "Wait, don't tell me you don't recognize the Great Pirate Captain Uthanka?!")
	public static NpcStringId WAIT_DON_T_TELL_ME_YOU_DON_T_RECOGNIZE_THE_GREAT_PIRATE_CAPTAIN_UTHANKA;
	
	@ClientString(id = 2000036, message = "Well, for starters, the Great Pirate Captain Uthanka - that's me - was the ruler of the very Talking Island you're standing on!")
	public static NpcStringId WELL_FOR_STARTERS_THE_GREAT_PIRATE_CAPTAIN_UTHANKA_THAT_S_ME_WAS_THE_RULER_OF_THE_VERY_TALKING_ISLAND_YOU_RE_STANDING_ON;
	
	@ClientString(id = 2000037, message = "Back in those days, nobodies like you wouldn't have stood a chance against the likes of me.")
	public static NpcStringId BACK_IN_THOSE_DAYS_NOBODIES_LIKE_YOU_WOULDN_T_HAVE_STOOD_A_CHANCE_AGAINST_THE_LIKES_OF_ME;
	
	@ClientString(id = 2000038, message = "Well, all that came to an end when Ye Sagira crashed onto the Talking Island because of Shilen's resurrection, so it's all done and past, I guess…")
	public static NpcStringId WELL_ALL_THAT_CAME_TO_AN_END_WHEN_YE_SAGIRA_CRASHED_ONTO_THE_TALKING_ISLAND_BECAUSE_OF_SHILEN_S_RESURRECTION_SO_IT_S_ALL_DONE_AND_PAST_I_GUESS;
	
	@ClientString(id = 2000039, message = "If it hadn't been for that, this island would have stayed under my great rule.")
	public static NpcStringId IF_IT_HADN_T_BEEN_FOR_THAT_THIS_ISLAND_WOULD_HAVE_STAYED_UNDER_MY_GREAT_RULE;
	
	@ClientString(id = 2000040, message = "Ah, the good old days…")
	public static NpcStringId AH_THE_GOOD_OLD_DAYS;
	
	@ClientString(id = 2000041, message = "What, still here? You were actually listening?")
	public static NpcStringId WHAT_STILL_HERE_YOU_WERE_ACTUALLY_LISTENING;
	
	@ClientString(id = 2000042, message = "Ahem, eh, well, thanks…")
	public static NpcStringId AHEM_EH_WELL_THANKS;
	
	@ClientString(id = 2000043, message = "Well, since you stayed to hear my story, I can't let you leave empty-handed now, can I?")
	public static NpcStringId WELL_SINCE_YOU_STAYED_TO_HEAR_MY_STORY_I_CAN_T_LET_YOU_LEAVE_EMPTY_HANDED_NOW_CAN_I;
	
	@ClientString(id = 2000044, message = "I'll give you a special gift. It's not for everyone, mind you! It's the most valuable thing in the history of Aden!")
	public static NpcStringId I_LL_GIVE_YOU_A_SPECIAL_GIFT_IT_S_NOT_FOR_EVERYONE_MIND_YOU_IT_S_THE_MOST_VALUABLE_THING_IN_THE_HISTORY_OF_ADEN;
	
	@ClientString(id = 2000045, message = "It's in your pouch, so use it well, you hear?")
	public static NpcStringId IT_S_IN_YOUR_POUCH_SO_USE_IT_WELL_YOU_HEAR;
	
	@ClientString(id = 2000046, message = "Man, I'm tired… I forgot how much talking can wear you out!")
	public static NpcStringId MAN_I_M_TIRED_I_FORGOT_HOW_MUCH_TALKING_CAN_WEAR_YOU_OUT;
	
	@ClientString(id = 2000047, message = "Only one ruler left! Let's finish this up!")
	public static NpcStringId ONLY_ONE_RULER_LEFT_LET_S_FINISH_THIS_UP;
	
	@ClientString(id = 2000048, message = "Everyone watch out for ice shards when debuffing.")
	public static NpcStringId EVERYONE_WATCH_OUT_FOR_ICE_SHARDS_WHEN_DEBUFFING;
	
	@ClientString(id = 2000049, message = "So sleepy… better finish this fast so I can go sleep!")
	public static NpcStringId SO_SLEEPY_BETTER_FINISH_THIS_FAST_SO_I_CAN_GO_SLEEP;
	
	@ClientString(id = 2000050, message = "Now, don't make yourself look stupid. Finish it clean.")
	public static NpcStringId NOW_DON_T_MAKE_YOURSELF_LOOK_STUPID_FINISH_IT_CLEAN;
	
	@ClientString(id = 2000051, message = "Watch out for lags, and start! Go, go, go!")
	public static NpcStringId WATCH_OUT_FOR_LAGS_AND_START_GO_GO_GO;
	
	@ClientString(id = 2000052, message = "Man, that hurts like hell! Heal, give me a heal!")
	public static NpcStringId MAN_THAT_HURTS_LIKE_HELL_HEAL_GIVE_ME_A_HEAL;
	
	@ClientString(id = 2000053, message = "The buffs are out! Give me buffs!")
	public static NpcStringId THE_BUFFS_ARE_OUT_GIVE_ME_BUFFS;
	
	@ClientString(id = 2000054, message = "Pattern 2 begun! Don't let the boss cast buffs!")
	public static NpcStringId PATTERN_2_BEGUN_DON_T_LET_THE_BOSS_CAST_BUFFS;
	
	@ClientString(id = 2000055, message = "This is the 3rd and final pattern. Focus your hits on the Crystal of Protection.")
	public static NpcStringId THIS_IS_THE_3RD_AND_FINAL_PATTERN_FOCUS_YOUR_HITS_ON_THE_CRYSTAL_OF_PROTECTION;
	
	@ClientString(id = 2000056, message = "Hit the Crystal, hit it, hit it!")
	public static NpcStringId HIT_THE_CRYSTAL_HIT_IT_HIT_IT;
	
	@ClientString(id = 2000057, message = "Heal! Heal! Heal me, come on!")
	public static NpcStringId HEAL_HEAL_HEAL_ME_COME_ON;
	
	@ClientString(id = 2000058, message = "Why am I the only one getting hit? Not fair!")
	public static NpcStringId WHY_AM_I_THE_ONLY_ONE_GETTING_HIT_NOT_FAIR;
	
	@ClientString(id = 2000059, message = "What, a heal? Man, we just can't catch a break today!")
	public static NpcStringId WHAT_A_HEAL_MAN_WE_JUST_CAN_T_CATCH_A_BREAK_TODAY;
	
	@ClientString(id = 2000060, message = "Come on, let's hurry up and finish it off so we can go eat.")
	public static NpcStringId COME_ON_LET_S_HURRY_UP_AND_FINISH_IT_OFF_SO_WE_CAN_GO_EAT;
	
	@ClientString(id = 2000061, message = "Why is the AI so good? Do they want us to beat this thing or not?")
	public static NpcStringId WHY_IS_THE_AI_SO_GOOD_DO_THEY_WANT_US_TO_BEAT_THIS_THING_OR_NOT;
	
	@ClientString(id = 2000062, message = "We will be faithful servants to our new ruler.")
	public static NpcStringId WE_WILL_BE_FAITHFUL_SERVANTS_TO_OUR_NEW_RULER;
	
	@ClientString(id = 2000063, message = "We humbly offer you this Chest of Offerings.")
	public static NpcStringId WE_HUMBLY_OFFER_YOU_THIS_CHEST_OF_OFFERINGS;
	
	@ClientString(id = 2000064, message = "Shilen's shout: I will take that power off your hands now.")
	public static NpcStringId SHILEN_S_SHOUT_I_WILL_TAKE_THAT_POWER_OFF_YOUR_HANDS_NOW;
	
	@ClientString(id = 2000065, message = "Punitive creatures, die!")
	public static NpcStringId PUNITIVE_CREATURES_DIE;
	
	@ClientString(id = 2000066, message = "I'm the new ruler here!")
	public static NpcStringId I_M_THE_NEW_RULER_HERE;
	
	@ClientString(id = 2000067, message = "Only death awaits those who refuse to serve me.")
	public static NpcStringId ONLY_DEATH_AWAITS_THOSE_WHO_REFUSE_TO_SERVE_ME;
	
	@ClientString(id = 2000068, message = "Will Jermann be restored to what he used to be?")
	public static NpcStringId WILL_JERMANN_BE_RESTORED_TO_WHAT_HE_USED_TO_BE;
	
	@ClientString(id = 2000069, message = "We must accept that only the mighty can survive! We must obtain power so that we can have our revenge on Shilen!")
	public static NpcStringId WE_MUST_ACCEPT_THAT_ONLY_THE_MIGHTY_CAN_SURVIVE_WE_MUST_OBTAIN_POWER_SO_THAT_WE_CAN_HAVE_OUR_REVENGE_ON_SHILEN;
	
	@ClientString(id = 2000070, message = "Seek the stone statue of Jermann the Great Leader for assistance, and you will be guided to the next step.")
	public static NpcStringId SEEK_THE_STONE_STATUE_OF_JERMANN_THE_GREAT_LEADER_FOR_ASSISTANCE_AND_YOU_WILL_BE_GUIDED_TO_THE_NEXT_STEP;
	
	@ClientString(id = 2000071, message = "You will need to prepare skills that match your new power.")
	public static NpcStringId YOU_WILL_NEED_TO_PREPARE_SKILLS_THAT_MATCH_YOUR_NEW_POWER;
	
	@ClientString(id = 2000072, message = "Well, I'll continue my story next time we meet. See ya.")
	public static NpcStringId WELL_I_LL_CONTINUE_MY_STORY_NEXT_TIME_WE_MEET_SEE_YA;
	
	@ClientString(id = 2000073, message = "Loyal Captain")
	public static NpcStringId LOYAL_CAPTAIN;
	
	@ClientString(id = 2000074, message = "I think this is as far as I can go in helping you, $s1. I will bow out now.")
	public static NpcStringId I_THINK_THIS_IS_AS_FAR_AS_I_CAN_GO_IN_HELPING_YOU_S1_I_WILL_BOW_OUT_NOW;
	
	@ClientString(id = 2000075, message = "I want a slushie…oh, if only I could have a slushie…hey, if you bring me 10 of each ingredient, I'll make you a Summer Slushie!")
	public static NpcStringId I_WANT_A_SLUSHIE_OH_IF_ONLY_I_COULD_HAVE_A_SLUSHIE_HEY_IF_YOU_BRING_ME_10_OF_EACH_INGREDIENT_I_LL_MAKE_YOU_A_SUMMER_SLUSHIE;
	
	@ClientString(id = 2000076, message = "I'll give you a Boss Accessory Pack if you bring me 15 Summer Slushies!")
	public static NpcStringId I_LL_GIVE_YOU_A_BOSS_ACCESSORY_PACK_IF_YOU_BRING_ME_15_SUMMER_SLUSHIES;
	
	@ClientString(id = 2000077, message = "Well, howdy-do! Isn't my swimsuit the cutest thing ever? It's time for a tan, yes it is!")
	public static NpcStringId WELL_HOWDY_DO_ISN_T_MY_SWIMSUIT_THE_CUTEST_THING_EVER_IT_S_TIME_FOR_A_TAN_YES_IT_IS;
	
	@ClientString(id = 2000078, message = "Thanks! I'll be counting on you next time too!")
	public static NpcStringId THANKS_I_LL_BE_COUNTING_ON_YOU_NEXT_TIME_TOO;
	
	@ClientString(id = 2000079, message = "Oh come on… you have to bring me the right materials!")
	public static NpcStringId OH_COME_ON_YOU_HAVE_TO_BRING_ME_THE_RIGHT_MATERIALS;
	
	@ClientString(id = 2000080, message = "Aghh! Zombies! Zombies!")
	public static NpcStringId AGHH_ZOMBIES_ZOMBIES;
	
	@ClientString(id = 2000081, message = "Ugh…please, someone get rid of these zombies…")
	public static NpcStringId UGH_PLEASE_SOMEONE_GET_RID_OF_THESE_ZOMBIES;
	
	@ClientString(id = 2000082, message = "I've been bitten by a zombie…someone help me…")
	public static NpcStringId I_VE_BEEN_BITTEN_BY_A_ZOMBIE_SOMEONE_HELP_ME;
	
	@ClientString(id = 2000083, message = "Is there no hero that can save us?")
	public static NpcStringId IS_THERE_NO_HERO_THAT_CAN_SAVE_US;
	
	@ClientString(id = 2000084, message = "Einhasad, dear goddess…please save us…")
	public static NpcStringId EINHASAD_DEAR_GODDESS_PLEASE_SAVE_US;
	
	@ClientString(id = 2000085, message = "We have to protect the Crystal of Protection if we want to stop these blasted zombies.")
	public static NpcStringId WE_HAVE_TO_PROTECT_THE_CRYSTAL_OF_PROTECTION_IF_WE_WANT_TO_STOP_THESE_BLASTED_ZOMBIES;
	
	@ClientString(id = 2000086, message = "Filthy creatures of Shilen… the town will be ruined if we don't stop the zombies.")
	public static NpcStringId FILTHY_CREATURES_OF_SHILEN_THE_TOWN_WILL_BE_RUINED_IF_WE_DON_T_STOP_THE_ZOMBIES;
	
	@ClientString(id = 2000087, message = "I can't partake in the battle because I have to defend the Crystal of Protection. Please, someone stop these zombies.")
	public static NpcStringId I_CAN_T_PARTAKE_IN_THE_BATTLE_BECAUSE_I_HAVE_TO_DEFEND_THE_CRYSTAL_OF_PROTECTION_PLEASE_SOMEONE_STOP_THESE_ZOMBIES;
	
	@ClientString(id = 2000088, message = "Zombies are rising at the outskirts of the Town of Gludio, and coming this way.")
	public static NpcStringId ZOMBIES_ARE_RISING_AT_THE_OUTSKIRTS_OF_THE_TOWN_OF_GLUDIO_AND_COMING_THIS_WAY;
	
	@ClientString(id = 2000089, message = "Zombies are rising at the outskirts of the Town of Aden, and coming this way.")
	public static NpcStringId ZOMBIES_ARE_RISING_AT_THE_OUTSKIRTS_OF_THE_TOWN_OF_ADEN_AND_COMING_THIS_WAY;
	
	@ClientString(id = 2000090, message = "Zombies are rising at the outskirts of the Town of Schuttgart, and coming this way.")
	public static NpcStringId ZOMBIES_ARE_RISING_AT_THE_OUTSKIRTS_OF_THE_TOWN_OF_SCHUTTGART_AND_COMING_THIS_WAY;
	
	@ClientString(id = 2000091, message = "Zombies are rising at the outskirts of Talking Island Village, and coming this way.")
	public static NpcStringId ZOMBIES_ARE_RISING_AT_THE_OUTSKIRTS_OF_TALKING_ISLAND_VILLAGE_AND_COMING_THIS_WAY;
	
	@ClientString(id = 2000092, message = "The age of chaos and darkness has come. I shall guide you to darkness.")
	public static NpcStringId THE_AGE_OF_CHAOS_AND_DARKNESS_HAS_COME_I_SHALL_GUIDE_YOU_TO_DARKNESS;
	
	@ClientString(id = 2000093, message = "I will make you invincible with the power of this plague.")
	public static NpcStringId I_WILL_MAKE_YOU_INVINCIBLE_WITH_THE_POWER_OF_THIS_PLAGUE;
	
	@ClientString(id = 2000094, message = "Humans…so complacent and weak after years of peace. How about I let you spread the fear of pestilence unto them?")
	public static NpcStringId HUMANS_SO_COMPLACENT_AND_WEAK_AFTER_YEARS_OF_PEACE_HOW_ABOUT_I_LET_YOU_SPREAD_THE_FEAR_OF_PESTILENCE_UNTO_THEM;
	
	@ClientString(id = 2000095, message = "For the prosperity of my people, I will use my power on Jermann and Jermann only.  I cannot overpower my own people.")
	public static NpcStringId FOR_THE_PROSPERITY_OF_MY_PEOPLE_I_WILL_USE_MY_POWER_ON_JERMANN_AND_JERMANN_ONLY_I_CANNOT_OVERPOWER_MY_OWN_PEOPLE;
	
	@ClientString(id = 2000096, message = "Are you ready to face your enemies?")
	public static NpcStringId ARE_YOU_READY_TO_FACE_YOUR_ENEMIES;
	
	@ClientString(id = 2000097, message = "You sense the presence of Pirate Captain Uthanka at the outskirts of Talking Island Village.")
	public static NpcStringId YOU_SENSE_THE_PRESENCE_OF_PIRATE_CAPTAIN_UTHANKA_AT_THE_OUTSKIRTS_OF_TALKING_ISLAND_VILLAGE;
	
	@ClientString(id = 2000098, message = "Dwarven Rescue")
	public static NpcStringId DWARVEN_RESCUE;
	
	@ClientString(id = 2000099, message = "Support the Troops")
	public static NpcStringId SUPPORT_THE_TROOPS;
	
	@ClientString(id = 2000100, message = "Eliminate the Grotesque Giant Bug")
	public static NpcStringId ELIMINATE_THE_GROTESQUE_GIANT_BUG;
	
	@ClientString(id = 2000101, message = "Analyzing the Purple Crevice")
	public static NpcStringId ANALYZING_THE_PURPLE_CREVICE;
	
	@ClientString(id = 2000102, message = "It would be rude to turn down a personal invitation, wouldn't it?")
	public static NpcStringId IT_WOULD_BE_RUDE_TO_TURN_DOWN_A_PERSONAL_INVITATION_WOULDN_T_IT;
	
	@ClientString(id = 2000103, message = "Ah, someone who recognizes me for the great soul that I am! Haha. Of course I'll come! I have a moral obligation to honor you with my presence.")
	public static NpcStringId AH_SOMEONE_WHO_RECOGNIZES_ME_FOR_THE_GREAT_SOUL_THAT_I_AM_HAHA_OF_COURSE_I_LL_COME_I_HAVE_A_MORAL_OBLIGATION_TO_HONOR_YOU_WITH_MY_PRESENCE;
	
	@ClientString(id = 2000104, message = "Oh, that's right! I should go and see how my money's being spent!")
	public static NpcStringId OH_THAT_S_RIGHT_I_SHOULD_GO_AND_SEE_HOW_MY_MONEY_S_BEING_SPENT;
	
	@ClientString(id = 2000105, message = "Um, I can't right now. Sorry. Maybe someone else?")
	public static NpcStringId UM_I_CAN_T_RIGHT_NOW_SORRY_MAYBE_SOMEONE_ELSE;
	
	@ClientString(id = 2000106, message = "Oh, you can have this instead. It's a Contender's Mark.")
	public static NpcStringId OH_YOU_CAN_HAVE_THIS_INSTEAD_IT_S_A_CONTENDER_S_MARK;
	
	@ClientString(id = 2000107, message = "A festival at Fantasy Isle? Sounds fun! I'll go over and check it out right now.")
	public static NpcStringId A_FESTIVAL_AT_FANTASY_ISLE_SOUNDS_FUN_I_LL_GO_OVER_AND_CHECK_IT_OUT_RIGHT_NOW;
	
	@ClientString(id = 2000108, message = "Fantasy Isle has a festival, you say? I'd be happy to go!")
	public static NpcStringId FANTASY_ISLE_HAS_A_FESTIVAL_YOU_SAY_I_D_BE_HAPPY_TO_GO;
	
	@ClientString(id = 2000109, message = "Does it have a lot of attractions? No, don't tell me - I'll go see for myself!")
	public static NpcStringId DOES_IT_HAVE_A_LOT_OF_ATTRACTIONS_NO_DON_T_TELL_ME_I_LL_GO_SEE_FOR_MYSELF;
	
	@ClientString(id = 2000110, message = "Ooh, sorry, but I've just been. It was a lot of fun!")
	public static NpcStringId OOH_SORRY_BUT_I_VE_JUST_BEEN_IT_WAS_A_LOT_OF_FUN;
	
	@ClientString(id = 2000111, message = "The new monster duels were my favorite!")
	public static NpcStringId THE_NEW_MONSTER_DUELS_WERE_MY_FAVORITE;
	
	@ClientString(id = 2000112, message = "Haha, another fruitful day!")
	public static NpcStringId HAHA_ANOTHER_FRUITFUL_DAY;
	
	@ClientString(id = 2000113, message = "Whoa! What the heck?")
	public static NpcStringId WHOA_WHAT_THE_HECK;
	
	@ClientString(id = 2000114, message = "What are you supposed to be? Get out of my way!")
	public static NpcStringId WHAT_ARE_YOU_SUPPOSED_TO_BE_GET_OUT_OF_MY_WAY;
	
	@ClientString(id = 2000115, message = "Congratulations! You have won the Monster Duel with 3 wins and obtained the bonus item!")
	public static NpcStringId CONGRATULATIONS_YOU_HAVE_WON_THE_MONSTER_DUEL_WITH_3_WINS_AND_OBTAINED_THE_BONUS_ITEM;
	
	@ClientString(id = 2000116, message = "Calculating results for the final round. Please wait a moment.")
	public static NpcStringId CALCULATING_RESULTS_FOR_THE_FINAL_ROUND_PLEASE_WAIT_A_MOMENT;
	
	@ClientString(id = 2000117, message = "You have lost with $s1 wins, $s2 ties, and $s3 losses.")
	public static NpcStringId YOU_HAVE_LOST_WITH_S1_WINS_S2_TIES_AND_S3_LOSSES;
	
	@ClientString(id = 2000118, message = "You have tied with $s1 wins, $s2 ties, and $s3 losses.")
	public static NpcStringId YOU_HAVE_TIED_WITH_S1_WINS_S2_TIES_AND_S3_LOSSES;
	
	@ClientString(id = 2000119, message = "Do you have any idea how much that outfit costs? Watch your step, will ya?!")
	public static NpcStringId DO_YOU_HAVE_ANY_IDEA_HOW_MUCH_THAT_OUTFIT_COSTS_WATCH_YOUR_STEP_WILL_YA;
	
	@ClientString(id = 2000120, message = "Move, move, move! Back straight! Shoulders wide!")
	public static NpcStringId MOVE_MOVE_MOVE_BACK_STRAIGHT_SHOULDERS_WIDE;
	
	@ClientString(id = 2000121, message = "Remarkable! It's even better than my expectations. Such extraordinary fashion!")
	public static NpcStringId REMARKABLE_IT_S_EVEN_BETTER_THAN_MY_EXPECTATIONS_SUCH_EXTRAORDINARY_FASHION;
	
	@ClientString(id = 2000122, message = "Adena can't buy my works of art! I'd rather give them free!")
	public static NpcStringId ADENA_CAN_T_BUY_MY_WORKS_OF_ART_I_D_RATHER_GIVE_THEM_FREE;
	
	@ClientString(id = 2000123, message = "You know what kind of guests are coming to the show today, don't you? Get your acts together!")
	public static NpcStringId YOU_KNOW_WHAT_KIND_OF_GUESTS_ARE_COMING_TO_THE_SHOW_TODAY_DON_T_YOU_GET_YOUR_ACTS_TOGETHER;
	
	@ClientString(id = 2000124, message = "I…I wanna go home…")
	public static NpcStringId I_I_WANNA_GO_HOME;
	
	@ClientString(id = 2000125, message = "Show start! Come on, snap snap!")
	public static NpcStringId SHOW_START_COME_ON_SNAP_SNAP;
	
	@ClientString(id = 2000126, message = "My beauties! You were fabulous! Now back to practice!")
	public static NpcStringId MY_BEAUTIES_YOU_WERE_FABULOUS_NOW_BACK_TO_PRACTICE;
	
	@ClientString(id = 2000127, message = "The fashion show will begin shortly. Please take a seat!")
	public static NpcStringId THE_FASHION_SHOW_WILL_BEGIN_SHORTLY_PLEASE_TAKE_A_SEAT;
	
	@ClientString(id = 2000128, message = "The Military Look is a multi-dimensional hit! It will catch on in Aden in no time for sure.")
	public static NpcStringId THE_MILITARY_LOOK_IS_A_MULTI_DIMENSIONAL_HIT_IT_WILL_CATCH_ON_IN_ADEN_IN_NO_TIME_FOR_SURE;
	
	@ClientString(id = 2000129, message = "Futuristic, isn't it? The Metal Suit looks stunning with an Emerald Weapon!")
	public static NpcStringId FUTURISTIC_ISN_T_IT_THE_METAL_SUIT_LOOKS_STUNNING_WITH_AN_EMERALD_WEAPON;
	
	@ClientString(id = 2000130, message = "Can't you just feel the chill air of secrecy? Be a ninja with this outfit, and walk in shadow!")
	public static NpcStringId CAN_T_YOU_JUST_FEEL_THE_CHILL_AIR_OF_SECRECY_BE_A_NINJA_WITH_THIS_OUTFIT_AND_WALK_IN_SHADOW;
	
	@ClientString(id = 2000131, message = "The Traditional Taiwanese Costume is for real warriors. It's becoming even on a hulking orc!")
	public static NpcStringId THE_TRADITIONAL_TAIWANESE_COSTUME_IS_FOR_REAL_WARRIORS_IT_S_BECOMING_EVEN_ON_A_HULKING_ORC;
	
	@ClientString(id = 2000132, message = "A round of applause for these fantastic designs, ladies and gentlemen!")
	public static NpcStringId A_ROUND_OF_APPLAUSE_FOR_THESE_FANTASTIC_DESIGNS_LADIES_AND_GENTLEMEN;
	
	@ClientString(id = 2000133, message = "This closes tonight's fashion show, ladies and gentlemen! Come again tomorrow!")
	public static NpcStringId THIS_CLOSES_TONIGHT_S_FASHION_SHOW_LADIES_AND_GENTLEMEN_COME_AGAIN_TOMORROW;
	
	@ClientString(id = 2000134, message = "Um…like this?")
	public static NpcStringId UM_LIKE_THIS;
	
	@ClientString(id = 2000135, message = "Ok, let me try that again.")
	public static NpcStringId OK_LET_ME_TRY_THAT_AGAIN;
	
	@ClientString(id = 2000136, message = "I think I'm getting the hang of it…")
	public static NpcStringId I_THINK_I_M_GETTING_THE_HANG_OF_IT;
	
	@ClientString(id = 2000137, message = "Drat, not again.")
	public static NpcStringId DRAT_NOT_AGAIN;
	
	@ClientString(id = 2000138, message = "I'm getting better! I feel it!")
	public static NpcStringId I_M_GETTING_BETTER_I_FEEL_IT;
	
	@ClientString(id = 2000139, message = "Really, would a bonus pay hurt that much?")
	public static NpcStringId REALLY_WOULD_A_BONUS_PAY_HURT_THAT_MUCH;
	
	@ClientString(id = 2000140, message = "Here, take this with you. I heard you could do monster duels on Fantasy Isle.")
	public static NpcStringId HERE_TAKE_THIS_WITH_YOU_I_HEARD_YOU_COULD_DO_MONSTER_DUELS_ON_FANTASY_ISLE;
	
	@ClientString(id = 2000141, message = "Here, I'll give you a Contender's Mark I have left over. Why don't you go give it a try?")
	public static NpcStringId HERE_I_LL_GIVE_YOU_A_CONTENDER_S_MARK_I_HAVE_LEFT_OVER_WHY_DON_T_YOU_GO_GIVE_IT_A_TRY;
	
	@ClientString(id = 2000142, message = "Round $s1 will begin.")
	public static NpcStringId ROUND_S1_WILL_BEGIN;
	
	@ClientString(id = 2000143, message = "You have won Round $s1. (You currently have $s2 wins, $s3 ties, and $s4 losses.)")
	public static NpcStringId YOU_HAVE_WON_ROUND_S1_YOU_CURRENTLY_HAVE_S2_WINS_S3_TIES_AND_S4_LOSSES;
	
	@ClientString(id = 2000144, message = "You have tied Round $s1. (You currently have $s2 wins, $s3 ties, and $s4 losses.)")
	public static NpcStringId YOU_HAVE_TIED_ROUND_S1_YOU_CURRENTLY_HAVE_S2_WINS_S3_TIES_AND_S4_LOSSES;
	
	@ClientString(id = 2000145, message = "You have lost Round $s1. (You currently have $s2 wins, $s3 ties, and $s4 losses.)")
	public static NpcStringId YOU_HAVE_LOST_ROUND_S1_YOU_CURRENTLY_HAVE_S2_WINS_S3_TIES_AND_S4_LOSSES;
	
	@ClientString(id = 2000146, message = "You have won the Monster Duel with 3 wins, but failed to obtain the bonus item.")
	public static NpcStringId YOU_HAVE_WON_THE_MONSTER_DUEL_WITH_3_WINS_BUT_FAILED_TO_OBTAIN_THE_BONUS_ITEM;
	
	@ClientString(id = 2000147, message = "A cute maid costume. Sorry, but only for ladies.")
	public static NpcStringId A_CUTE_MAID_COSTUME_SORRY_BUT_ONLY_FOR_LADIES;
	
	@ClientString(id = 2000148, message = "Another performance complete! All right, back to practice! Let's go!")
	public static NpcStringId ANOTHER_PERFORMANCE_COMPLETE_ALL_RIGHT_BACK_TO_PRACTICE_LET_S_GO;
	
	@ClientString(id = 2000149, message = "Ta-da! The great Uthanka is back!")
	public static NpcStringId TA_DA_THE_GREAT_UTHANKA_IS_BACK;
	
	@ClientString(id = 2000150, message = "The great Uthanka will take Talking Island for himself this time, like, for real, thank you much!")
	public static NpcStringId THE_GREAT_UTHANKA_WILL_TAKE_TALKING_ISLAND_FOR_HIMSELF_THIS_TIME_LIKE_FOR_REAL_THANK_YOU_MUCH;
	
	@ClientString(id = 2000151, message = "Don't say I didn't warn you when you tremble at my might!")
	public static NpcStringId DON_T_SAY_I_DIDN_T_WARN_YOU_WHEN_YOU_TREMBLE_AT_MY_MIGHT;
	
	@ClientString(id = 2000152, message = "Uthanka - power - up!")
	public static NpcStringId UTHANKA_POWER_UP;
	
	@ClientString(id = 2000153, message = "I'm not what I used to be.")
	public static NpcStringId I_M_NOT_WHAT_I_USED_TO_BE2;
	
	@ClientString(id = 2000154, message = "Take out the trash, my minions.")
	public static NpcStringId TAKE_OUT_THE_TRASH_MY_MINIONS;
	
	@ClientString(id = 2000155, message = "I look west.")
	public static NpcStringId I_LOOK_WEST;
	
	@ClientString(id = 2000156, message = "Eve will bring you great fortune.")
	public static NpcStringId EVE_WILL_BRING_YOU_GREAT_FORTUNE;
	
	@ClientString(id = 2000157, message = "You will one day ask me for guidance in your path.")
	public static NpcStringId YOU_WILL_ONE_DAY_ASK_ME_FOR_GUIDANCE_IN_YOUR_PATH;
	
	@ClientString(id = 2000158, message = "U- wah- wahhh!")
	public static NpcStringId U_WAH_WAHHH;
	
	@ClientString(id = 2000159, message = "Don't forget to brush your teeth, kiddos!")
	public static NpcStringId DON_T_FORGET_TO_BRUSH_YOUR_TEETH_KIDDOS;
	
	@ClientString(id = 2000160, message = "No one gets to Master Uthanka without getting through me first!")
	public static NpcStringId NO_ONE_GETS_TO_MASTER_UTHANKA_WITHOUT_GETTING_THROUGH_ME_FIRST;
	
	@ClientString(id = 2000161, message = "I'm more than enough for the likes of you.")
	public static NpcStringId I_M_MORE_THAN_ENOUGH_FOR_THE_LIKES_OF_YOU;
	
	@ClientString(id = 2000162, message = "Get away from Master Uthanka, now!")
	public static NpcStringId GET_AWAY_FROM_MASTER_UTHANKA_NOW;
	
	@ClientString(id = 2000163, message = "Mas…ter… Uthan…ka…")
	public static NpcStringId MAS_TER_UTHAN_KA;
	
	@ClientString(id = 2000164, message = "$s1 is inflicted with the Blood Seal. Only $s1 can face Bloody Katura for the next 30 seconds.")
	public static NpcStringId S1_IS_INFLICTED_WITH_THE_BLOOD_SEAL_ONLY_S1_CAN_FACE_BLOODY_KATURA_FOR_THE_NEXT_30_SECONDS;
	
	@ClientString(id = 2000165, message = "Katura Queen is summoning minions…")
	public static NpcStringId KATURA_QUEEN_IS_SUMMONING_MINIONS;
	
	@ClientString(id = 2000166, message = "Katura Queen is in defensive mode. Now is your chance to inflict additional damage!")
	public static NpcStringId KATURA_QUEEN_IS_IN_DEFENSIVE_MODE_NOW_IS_YOUR_CHANCE_TO_INFLICT_ADDITIONAL_DAMAGE;
	
	@ClientString(id = 2000167, message = "Katura Queen has restored her HP by absorbing her minions.")
	public static NpcStringId KATURA_QUEEN_HAS_RESTORED_HER_HP_BY_ABSORBING_HER_MINIONS;
	
	@ClientString(id = 2000168, message = "Katura Queen is weakened by the death of her minions!")
	public static NpcStringId KATURA_QUEEN_IS_WEAKENED_BY_THE_DEATH_OF_HER_MINIONS;
	
	@ClientString(id = 2000169, message = "You have defeated Katura Queen and won the battle of Infernal Kamaloka.")
	public static NpcStringId YOU_HAVE_DEFEATED_KATURA_QUEEN_AND_WON_THE_BATTLE_OF_INFERNAL_KAMALOKA;
	
	@ClientString(id = 2000170, message = "$s1 is inflicted with the Poison Seed, which will explode in 10 seconds.")
	public static NpcStringId S1_IS_INFLICTED_WITH_THE_POISON_SEED_WHICH_WILL_EXPLODE_IN_10_SECONDS;
	
	@ClientString(id = 2000171, message = "Lunatic Lord is in defensive mode. Now is your chance to inflict additional damage!")
	public static NpcStringId LUNATIC_LORD_IS_IN_DEFENSIVE_MODE_NOW_IS_YOUR_CHANCE_TO_INFLICT_ADDITIONAL_DAMAGE;
	
	@ClientString(id = 2000172, message = "Lunatic Lord is summoning minions…")
	public static NpcStringId LUNATIC_LORD_IS_SUMMONING_MINIONS;
	
	@ClientString(id = 2000173, message = "Lunatic Lord is weakened by the simultaneous death of his minions.")
	public static NpcStringId LUNATIC_LORD_IS_WEAKENED_BY_THE_SIMULTANEOUS_DEATH_OF_HIS_MINIONS;
	
	@ClientString(id = 2000174, message = "Time is up, and the enemies have withdrawn. The battle is over.")
	public static NpcStringId TIME_IS_UP_AND_THE_ENEMIES_HAVE_WITHDRAWN_THE_BATTLE_IS_OVER;
	
	@ClientString(id = 2000175, message = "You have defeated Lunatic Lord and won the battle of Lunatic Kamaloka.")
	public static NpcStringId YOU_HAVE_DEFEATED_LUNATIC_LORD_AND_WON_THE_BATTLE_OF_LUNATIC_KAMALOKA;
	
	@ClientString(id = 2000176, message = "Those hit with the Poison Blast are suffering from Dimensional Poisoning.")
	public static NpcStringId THOSE_HIT_WITH_THE_POISON_BLAST_ARE_SUFFERING_FROM_DIMENSIONAL_POISONING;
	
	@ClientString(id = 2000177, message = "Lunatic Lord is enraged by the death of his minion!")
	public static NpcStringId LUNATIC_LORD_IS_ENRAGED_BY_THE_DEATH_OF_HIS_MINION;
	
	@ClientString(id = 2000178, message = "The death of a minion has given Lunatic Lord additional defenses!")
	public static NpcStringId THE_DEATH_OF_A_MINION_HAS_GIVEN_LUNATIC_LORD_ADDITIONAL_DEFENSES;
	
	@ClientString(id = 2000179, message = "$s1 has become a target for the Blood Seal due to failing to kill Bloody Katura within 30 seconds.")
	public static NpcStringId S1_HAS_BECOME_A_TARGET_FOR_THE_BLOOD_SEAL_DUE_TO_FAILING_TO_KILL_BLOODY_KATURA_WITHIN_30_SECONDS;
	
	@ClientString(id = 2000180, message = "Bloody Katura is vulnerable for 5 seconds!")
	public static NpcStringId BLOODY_KATURA_IS_VULNERABLE_FOR_5_SECONDS;
	
	@ClientString(id = 2000181, message = "Bloody Katura's death has lifted the Blood Seal.")
	public static NpcStringId BLOODY_KATURA_S_DEATH_HAS_LIFTED_THE_BLOOD_SEAL;
	
	@ClientString(id = 2000182, message = "A Kamaloka Chest has appeared. You can loot after battle!")
	public static NpcStringId A_KAMALOKA_CHEST_HAS_APPEARED_YOU_CAN_LOOT_AFTER_BATTLE;
	
	@ClientString(id = 2000183, message = "$s1 has used 1 Kamaloka Secret Chest Key.")
	public static NpcStringId S1_HAS_USED_1_KAMALOKA_SECRET_CHEST_KEY;
	
	@ClientString(id = 2000184, message = "Kamaloka Secret Chest is open!")
	public static NpcStringId KAMALOKA_SECRET_CHEST_IS_OPEN;
	
	@ClientString(id = 2000185, message = "$s1 has used 1 Kamaloka Chest Key.")
	public static NpcStringId S1_HAS_USED_1_KAMALOKA_CHEST_KEY;
	
	@ClientString(id = 2000186, message = "Kamaloka Chest is open!")
	public static NpcStringId KAMALOKA_CHEST_IS_OPEN;
	
	@ClientString(id = 2000187, message = "Victory! You scored $s1 points.")
	public static NpcStringId VICTORY_YOU_SCORED_S1_POINTS;
	
	@ClientString(id = 2000188, message = "The enemy has retreated, ending the battle. You scored $s1 points.")
	public static NpcStringId THE_ENEMY_HAS_RETREATED_ENDING_THE_BATTLE_YOU_SCORED_S1_POINTS;
	
	@ClientString(id = 2000189, message = "The altar's protection has eroded, leaving it vulnerable for the next 5 minutes.")
	public static NpcStringId THE_ALTAR_S_PROTECTION_HAS_ERODED_LEAVING_IT_VULNERABLE_FOR_THE_NEXT_5_MINUTES;
	
	@ClientString(id = 2000190, message = "The Fire Altar has been destroyed. You will receive the Fire Altar's Blessing for the next 5 minutes.")
	public static NpcStringId THE_FIRE_ALTAR_HAS_BEEN_DESTROYED_YOU_WILL_RECEIVE_THE_FIRE_ALTAR_S_BLESSING_FOR_THE_NEXT_5_MINUTES;
	
	@ClientString(id = 2000191, message = "The Water Altar has been destroyed. You will receive the Water Altar's Blessing for the next 5 minutes.")
	public static NpcStringId THE_WATER_ALTAR_HAS_BEEN_DESTROYED_YOU_WILL_RECEIVE_THE_WATER_ALTAR_S_BLESSING_FOR_THE_NEXT_5_MINUTES;
	
	@ClientString(id = 2000192, message = "The Earth Altar has been destroyed. You will receive the Earth Altar's Blessing for the next 5 minutes.")
	public static NpcStringId THE_EARTH_ALTAR_HAS_BEEN_DESTROYED_YOU_WILL_RECEIVE_THE_EARTH_ALTAR_S_BLESSING_FOR_THE_NEXT_5_MINUTES;
	
	@ClientString(id = 2000193, message = "The Dark Altar has been destroyed. You will receive the Dark Altar's Blessing for the next 5 minutes.")
	public static NpcStringId THE_DARK_ALTAR_HAS_BEEN_DESTROYED_YOU_WILL_RECEIVE_THE_DARK_ALTAR_S_BLESSING_FOR_THE_NEXT_5_MINUTES;
	
	@ClientString(id = 2000194, message = "You have failed to destroy the altar, rendering it invincible again under Katura Queen's defenses.")
	public static NpcStringId YOU_HAVE_FAILED_TO_DESTROY_THE_ALTAR_RENDERING_IT_INVINCIBLE_AGAIN_UNDER_KATURA_QUEEN_S_DEFENSES;
	
	@ClientString(id = 2000195, message = "Katura Queen's minions are being summoned at the altar!")
	public static NpcStringId KATURA_QUEEN_S_MINIONS_ARE_BEING_SUMMONED_AT_THE_ALTAR;
	
	@ClientString(id = 2000196, message = "You have defeated Bloody Katura.")
	public static NpcStringId YOU_HAVE_DEFEATED_BLOODY_KATURA;
	
	@ClientString(id = 2000197, message = "You have failed to defeat Bloody Katura before it was absorbed by Katura Queen for HP.")
	public static NpcStringId YOU_HAVE_FAILED_TO_DEFEAT_BLOODY_KATURA_BEFORE_IT_WAS_ABSORBED_BY_KATURA_QUEEN_FOR_HP;
	
	@ClientString(id = 2000198, message = "$s1 has won the main event for players under Lv. $s2, and earned $s3 points!")
	public static NpcStringId S1_HAS_WON_THE_MAIN_EVENT_FOR_PLAYERS_UNDER_LV_S2_AND_EARNED_S3_POINTS;
	
	@ClientString(id = 2000199, message = "$s1 has earned $s2 points in the main event for unlimited levels.")
	public static NpcStringId S1_HAS_EARNED_S2_POINTS_IN_THE_MAIN_EVENT_FOR_UNLIMITED_LEVELS;
	
	@ClientString(id = 2000200, message = "Normal or Superior Medal")
	public static NpcStringId NORMAL_OR_SUPERIOR_MEDAL;
	
	@ClientString(id = 2000201, message = "Normal or Superior Medal")
	public static NpcStringId NORMAL_OR_SUPERIOR_MEDAL2;
	
	@ClientString(id = 2000202, message = "Please choose a monster, or one will be automatically designated in $s1 seconds!")
	public static NpcStringId PLEASE_CHOOSE_A_MONSTER_OR_ONE_WILL_BE_AUTOMATICALLY_DESIGNATED_IN_S1_SECONDS;
	
	@ClientString(id = 2000203, message = "A monster has been automatically designated.")
	public static NpcStringId A_MONSTER_HAS_BEEN_AUTOMATICALLY_DESIGNATED;
	
	@ClientString(id = 2000204, message = "Congratulations! You are the victor with $s1 wins and $s3 losses!")
	public static NpcStringId CONGRATULATIONS_YOU_ARE_THE_VICTOR_WITH_S1_WINS_AND_S3_LOSSES;
	
	@ClientString(id = 2010001, message = "Welcome to Ancient City Arcan!")
	public static NpcStringId WELCOME_TO_ANCIENT_CITY_ARCAN;
	
	@ClientString(id = 2010002, message = "Nice to meet you")
	public static NpcStringId NICE_TO_MEET_YOU;
	
	@ClientString(id = 2010003, message = "Hello~")
	public static NpcStringId HELLO;
	
	@ClientString(id = 2010004, message = "Have a good day~")
	public static NpcStringId HAVE_A_GOOD_DAY;
	
	@ClientString(id = 2010005, message = "Hello, good to see you!")
	public static NpcStringId HELLO_GOOD_TO_SEE_YOU;
	
	@ClientString(id = 2010006, message = "Please pet me.")
	public static NpcStringId PLEASE_PET_ME;
	
	@ClientString(id = 2010007, message = "Welcome to Ancient City Arcan.")
	public static NpcStringId WELCOME_TO_ANCIENT_CITY_ARCAN2;
	
	@ClientString(id = 2010008, message = "I am the most handsome cat.")
	public static NpcStringId I_AM_THE_MOST_HANDSOME_CAT;
	
	@ClientString(id = 2010009, message = "Today must be a lucky day.")
	public static NpcStringId TODAY_MUST_BE_A_LUCKY_DAY;
	
	@ClientString(id = 2010010, message = "It smells good.")
	public static NpcStringId IT_SMELLS_GOOD;
	
	@ClientString(id = 2010011, message = "Things are strange so I'm cautious.")
	public static NpcStringId THINGS_ARE_STRANGE_SO_I_M_CAUTIOUS;
	
	@ClientString(id = 2010012, message = "You seem nice.")
	public static NpcStringId YOU_SEEM_NICE;
	
	@ClientString(id = 2010013, message = "For Great General Kimerian!")
	public static NpcStringId FOR_GREAT_GENERAL_KIMERIAN;
	
	@ClientString(id = 2010014, message = "Glory to Kimerian!")
	public static NpcStringId GLORY_TO_KIMERIAN;
	
	@ClientString(id = 2010015, message = "Eternity to Kimerian!")
	public static NpcStringId ETERNITY_TO_KIMERIAN;
	
	@ClientString(id = 2010016, message = "Devote Myself for the Glory of Kimerian!")
	public static NpcStringId DEVOTE_MYSELF_FOR_THE_GLORY_OF_KIMERIAN;
	
	@ClientString(id = 2010017, message = "For Kimerian!")
	public static NpcStringId FOR_KIMERIAN;
	
	@ClientString(id = 2010018, message = "Chrrk! Kimerian!")
	public static NpcStringId CHRRK_KIMERIAN;
	
	@ClientString(id = 2010019, message = "For the Glory of Kimerian!")
	public static NpcStringId FOR_THE_GLORY_OF_KIMERIAN;
	
	@ClientString(id = 2010020, message = "Foreigners OUT!")
	public static NpcStringId FOREIGNERS_OUT;
	
	@ClientString(id = 2010021, message = "Stop them!")
	public static NpcStringId STOP_THEM;
	
	@ClientString(id = 2010022, message = "For Great General Kimerian!")
	public static NpcStringId FOR_GREAT_GENERAL_KIMERIAN2;
	
	@ClientString(id = 2010023, message = "Failure is not acceptable...")
	public static NpcStringId FAILURE_IS_NOT_ACCEPTABLE;
	
	@ClientString(id = 2010024, message = "Eternity to Kimerian!")
	public static NpcStringId ETERNITY_TO_KIMERIAN2;
	
	@ClientString(id = 2010025, message = "No regrets until death")
	public static NpcStringId NO_REGRETS_UNTIL_DEATH;
	
	@ClientString(id = 2010026, message = "A faint breeze makes me sensitive")
	public static NpcStringId A_FAINT_BREEZE_MAKES_ME_SENSITIVE;
	
	@ClientString(id = 2010027, message = "I'm ashamed that I can't confront my enemies")
	public static NpcStringId I_M_ASHAMED_THAT_I_CAN_T_CONFRONT_MY_ENEMIES;
	
	@ClientString(id = 2010028, message = "Defeat all of the resistance fighters!")
	public static NpcStringId DEFEAT_ALL_OF_THE_RESISTANCE_FIGHTERS;
	
	@ClientString(id = 2010029, message = "Hurray Kimerian!")
	public static NpcStringId HURRAY_KIMERIAN;
	
	@ClientString(id = 2010030, message = "Great Leader Kimerian!")
	public static NpcStringId GREAT_LEADER_KIMERIAN;
	
	@ClientString(id = 2010031, message = "Go Advance!")
	public static NpcStringId GO_ADVANCE;
	
	@ClientString(id = 2010032, message = "All Attack!")
	public static NpcStringId ALL_ATTACK;
	
	@ClientString(id = 2010033, message = "Forward!")
	public static NpcStringId FORWARD;
	
	@ClientString(id = 2010034, message = "For the Freedom!")
	public static NpcStringId FOR_THE_FREEDOM;
	
	@ClientString(id = 2010035, message = "Ahh...")
	public static NpcStringId AHH;
	
	@ClientString(id = 2010036, message = "We can't stop here...")
	public static NpcStringId WE_CAN_T_STOP_HERE;
	
	@ClientString(id = 2010037, message = "Please don't make my death fruitless...")
	public static NpcStringId PLEASE_DON_T_MAKE_MY_DEATH_FRUITLESS;
	
	@ClientString(id = 2010038, message = "Now! Before it's too late, we need to track Kimerian to catch him!")
	public static NpcStringId NOW_BEFORE_IT_S_TOO_LATE_WE_NEED_TO_TRACK_KIMERIAN_TO_CATCH_HIM;
	
	@ClientString(id = 2010039, message = "I don't think this is the right way.")
	public static NpcStringId I_DON_T_THINK_THIS_IS_THE_RIGHT_WAY;
	
	@ClientString(id = 2010040, message = "We lost tracking Kimerian. Let's turn to another direction.")
	public static NpcStringId WE_LOST_TRACKING_KIMERIAN_LET_S_TURN_TO_ANOTHER_DIRECTION;
	
	@ClientString(id = 2010041, message = "Tracking goes to another direction.")
	public static NpcStringId TRACKING_GOES_TO_ANOTHER_DIRECTION;
	
	@ClientString(id = 2010042, message = "This seems to be a wrong direction.")
	public static NpcStringId THIS_SEEMS_TO_BE_A_WRONG_DIRECTION;
	
	@ClientString(id = 2010043, message = "Let's go upward deep inside the forest.")
	public static NpcStringId LET_S_GO_UPWARD_DEEP_INSIDE_THE_FOREST;
	
	@ClientString(id = 2010044, message = "Tracking goes to the waterfall.")
	public static NpcStringId TRACKING_GOES_TO_THE_WATERFALL;
	
	@ClientString(id = 2010045, message = "Let's continue on this road.")
	public static NpcStringId LET_S_CONTINUE_ON_THIS_ROAD;
	
	@ClientString(id = 2010046, message = "Let's keep going upward a bit more.")
	public static NpcStringId LET_S_KEEP_GOING_UPWARD_A_BIT_MORE;
	
	@ClientString(id = 2010047, message = "There Kimerian!")
	public static NpcStringId THERE_KIMERIAN;
	
	@ClientString(id = 2010048, message = "Found Kimerian!")
	public static NpcStringId FOUND_KIMERIAN;
	
	@ClientString(id = 2010049, message = "Rotten Kimerian! This is the fury of Fairies!")
	public static NpcStringId ROTTEN_KIMERIAN_THIS_IS_THE_FURY_OF_FAIRIES;
	
	@ClientString(id = 2010050, message = "Kimerian is hiding right there!")
	public static NpcStringId KIMERIAN_IS_HIDING_RIGHT_THERE;
	
	@ClientString(id = 2010051, message = "We found Kimerian hiding in the cave right there!")
	public static NpcStringId WE_FOUND_KIMERIAN_HIDING_IN_THE_CAVE_RIGHT_THERE;
	
	@ClientString(id = 2010052, message = "Followers of Kimerian!")
	public static NpcStringId FOLLOWERS_OF_KIMERIAN;
	
	@ClientString(id = 2010053, message = "For the Freedom of Fairies!")
	public static NpcStringId FOR_THE_FREEDOM_OF_FAIRIES;
	
	@ClientString(id = 2010054, message = "Ready!")
	public static NpcStringId READY;
	
	@ClientString(id = 2010055, message = "Go! Wipe out Kimerian!")
	public static NpcStringId GO_WIPE_OUT_KIMERIAN;
	
	@ClientString(id = 2010056, message = "It smells in here.")
	public static NpcStringId IT_SMELLS_IN_HERE;
	
	@ClientString(id = 2010057, message = "Let's go in.")
	public static NpcStringId LET_S_GO_IN;
	
	@ClientString(id = 2010058, message = "Let's go in.")
	public static NpcStringId LET_S_GO_IN2;
	
	@ClientString(id = 2010059, message = "My heart is pounding.")
	public static NpcStringId MY_HEART_IS_POUNDING;
	
	@ClientString(id = 2010060, message = "Until the end! Kimerian!")
	public static NpcStringId UNTIL_THE_END_KIMERIAN;
	
	@ClientString(id = 2010061, message = "Kimerian! This is my revenge for the Fairy victims!")
	public static NpcStringId KIMERIAN_THIS_IS_MY_REVENGE_FOR_THE_FAIRY_VICTIMS;
	
	@ClientString(id = 2010062, message = "Kimerian! We are here to punish you!")
	public static NpcStringId KIMERIAN_WE_ARE_HERE_TO_PUNISH_YOU;
	
	@ClientString(id = 2010063, message = "This is my revenge for my comrades who were transformed and have disappeared!")
	public static NpcStringId THIS_IS_MY_REVENGE_FOR_MY_COMRADES_WHO_WERE_TRANSFORMED_AND_HAVE_DISAPPEARED;
	
	@ClientString(id = 2010064, message = "How ridiculous! You think you can find me?")
	public static NpcStringId HOW_RIDICULOUS_YOU_THINK_YOU_CAN_FIND_ME;
	
	@ClientString(id = 2010065, message = "Then try. Ha ha ha.")
	public static NpcStringId THEN_TRY_HA_HA_HA;
	
	@ClientString(id = 2010066, message = "You're still trying?")
	public static NpcStringId YOU_RE_STILL_TRYING;
	
	@ClientString(id = 2010067, message = "Ha ha ha ha...")
	public static NpcStringId HA_HA_HA_HA;
	
	@ClientString(id = 2010068, message = "Arrogant asses...! I won't forget this...!")
	public static NpcStringId ARROGANT_ASSES_I_WON_T_FORGET_THIS;
	
	@ClientString(id = 2010069, message = "You did it! Thank you!")
	public static NpcStringId YOU_DID_IT_THANK_YOU;
	
	@ClientString(id = 2010070, message = "Unfortunately, they ran away.")
	public static NpcStringId UNFORTUNATELY_THEY_RAN_AWAY;
	
	@ClientString(id = 2010071, message = "I'm going to die! Use glimmer on me!")
	public static NpcStringId I_M_GOING_TO_DIE_USE_GLIMMER_ON_ME;
	
	@ClientString(id = 2010072, message = "Use glimmer on me!")
	public static NpcStringId USE_GLIMMER_ON_ME;
	
	@ClientString(id = 2010073, message = "You are a resistance fighter! Please take me with you!")
	public static NpcStringId YOU_ARE_A_RESISTANCE_FIGHTER_PLEASE_TAKE_ME_WITH_YOU;
	
	@ClientString(id = 2010074, message = "We can find Sword of Revolution!")
	public static NpcStringId WE_CAN_FIND_SWORD_OF_REVOLUTION;
	
	@ClientString(id = 2010075, message = "I hope this is helpful.")
	public static NpcStringId I_HOPE_THIS_IS_HELPFUL;
	
	@ClientString(id = 2010076, message = "We defeated Kimerian!")
	public static NpcStringId WE_DEFEATED_KIMERIAN;
	
	@ClientString(id = 2010077, message = "We did it!")
	public static NpcStringId WE_DID_IT;
	
	@ClientString(id = 2010078, message = "Hurray!")
	public static NpcStringId HURRAY;
	
	@ClientString(id = 2010079, message = "You did it well!")
	public static NpcStringId YOU_DID_IT_WELL;
	
	@ClientString(id = 2010080, message = "We defeated a tyrant!")
	public static NpcStringId WE_DEFEATED_A_TYRANT;
	
	@ClientString(id = 2010081, message = "Wooow!")
	public static NpcStringId WOOOW;
	
	@ClientString(id = 2010082, message = "Lalala~ Even though the Mother Tree is ill and the town is plagued...")
	public static NpcStringId LALALA_EVEN_THOUGH_THE_MOTHER_TREE_IS_ILL_AND_THE_TOWN_IS_PLAGUED;
	
	@ClientString(id = 2010083, message = "I am so thankful that Rizraell is staying with me~")
	public static NpcStringId I_AM_SO_THANKFUL_THAT_RIZRAELL_IS_STAYING_WITH_ME;
	
	@ClientString(id = 2010084, message = "Dark energy is creeping in and Shilen is about to resurrect...")
	public static NpcStringId DARK_ENERGY_IS_CREEPING_IN_AND_SHILEN_IS_ABOUT_TO_RESURRECT;
	
	@ClientString(id = 2010085, message = "Let's give warm healings to each other~")
	public static NpcStringId LET_S_GIVE_WARM_HEALINGS_TO_EACH_OTHER;
	
	@ClientString(id = 2010086, message = "La lalala~")
	public static NpcStringId LA_LALALA;
	
	@ClientString(id = 2010087, message = "Resurrect and open your eyes in the dark...")
	public static NpcStringId RESURRECT_AND_OPEN_YOUR_EYES_IN_THE_DARK;
	
	@ClientString(id = 2010088, message = "Then create more offerings of Shilen!")
	public static NpcStringId THEN_CREATE_MORE_OFFERINGS_OF_SHILEN;
	
	@ClientString(id = 2010089, message = "Kimerian ran away. Let's return to the village and talk.")
	public static NpcStringId KIMERIAN_RAN_AWAY_LET_S_RETURN_TO_THE_VILLAGE_AND_TALK;
	
	@ClientString(id = 2010090, message = "You did a lot of work! Let's go back to the village and have a congratulatory drink!")
	public static NpcStringId YOU_DID_A_LOT_OF_WORK_LET_S_GO_BACK_TO_THE_VILLAGE_AND_HAVE_A_CONGRATULATORY_DRINK;
	
	@ClientString(id = 2010091, message = "They are rookie rebellions.")
	public static NpcStringId THEY_ARE_ROOKIE_REBELLIONS;
	
	@ClientString(id = 2010092, message = "Resistance underlings!")
	public static NpcStringId RESISTANCE_UNDERLINGS;
	
	@ClientString(id = 2010093, message = "Treason is punishable by death.")
	public static NpcStringId TREASON_IS_PUNISHABLE_BY_DEATH;
	
	@ClientString(id = 2010094, message = "Who do you think you are to try my authority!")
	public static NpcStringId WHO_DO_YOU_THINK_YOU_ARE_TO_TRY_MY_AUTHORITY;
	
	@ClientString(id = 2010095, message = "Defeat! Kimerian!")
	public static NpcStringId DEFEAT_KIMERIAN;
	
	@ClientString(id = 2010096, message = "For the Freedom of Fairies!")
	public static NpcStringId FOR_THE_FREEDOM_OF_FAIRIES2;
	
	@ClientString(id = 2010097, message = "Phantom image!")
	public static NpcStringId PHANTOM_IMAGE;
	
	@ClientString(id = 2010098, message = "Umm. You're still alive?")
	public static NpcStringId UMM_YOU_RE_STILL_ALIVE;
	
	@ClientString(id = 2010099, message = "I won't let you get away any more!")
	public static NpcStringId I_WON_T_LET_YOU_GET_AWAY_ANY_MORE;
	
	@ClientString(id = 2010100, message = "How dare you came to me! This is where you'll be buried.")
	public static NpcStringId HOW_DARE_YOU_CAME_TO_ME_THIS_IS_WHERE_YOU_LL_BE_BURIED;
	
	@ClientString(id = 2010101, message = "But you came this far, I should at least play with you a little.")
	public static NpcStringId BUT_YOU_CAME_THIS_FAR_I_SHOULD_AT_LEAST_PLAY_WITH_YOU_A_LITTLE;
	
	@ClientString(id = 2010102, message = "Thanks!")
	public static NpcStringId THANKS;
	
	@ClientString(id = 2010103, message = "Now I feel much better.")
	public static NpcStringId NOW_I_FEEL_MUCH_BETTER;
	
	@ClientString(id = 2010104, message = "Energy is soaring!")
	public static NpcStringId ENERGY_IS_SOARING;
	
	@ClientString(id = 2010105, message = "I feel better!")
	public static NpcStringId I_FEEL_BETTER;
	
	@ClientString(id = 2010106, message = "Thank you for saving me!")
	public static NpcStringId THANK_YOU_FOR_SAVING_ME3;
	
	@ClientString(id = 2010107, message = "You did it well. Good bye now.")
	public static NpcStringId YOU_DID_IT_WELL_GOOD_BYE_NOW;
	
	@ClientString(id = 2010108, message = "Protect the spirits of light!")
	public static NpcStringId PROTECT_THE_SPIRITS_OF_LIGHT;
	
	@ClientString(id = 2010109, message = "For peace!")
	public static NpcStringId FOR_PEACE;
	
	@ClientString(id = 2010110, message = "Dear darkness! Dear goddess!")
	public static NpcStringId DEAR_DARKNESS_DEAR_GODDESS;
	
	@ClientString(id = 2010111, message = "Dear the goddess of destruction... The light and their creatures fear you...")
	public static NpcStringId DEAR_THE_GODDESS_OF_DESTRUCTION_THE_LIGHT_AND_THEIR_CREATURES_FEAR_YOU;
	
	@ClientString(id = 2010112, message = "Tainted Dimension")
	public static NpcStringId TAINTED_DIMENSION;
	
	@ClientString(id = 2010113, message = "Advance Signal Scouts")
	public static NpcStringId ADVANCE_SIGNAL_SCOUTS;
	
	@ClientString(id = 2010114, message = "$s1! Please give me an advance order!")
	public static NpcStringId S1_PLEASE_GIVE_ME_AN_ADVANCE_ORDER;
	
	@ClientString(id = 2010115, message = "Huh hu hu. Did you think you caught me? Can you really find me?")
	public static NpcStringId HUH_HU_HU_DID_YOU_THINK_YOU_CAUGHT_ME_CAN_YOU_REALLY_FIND_ME;
	
	@ClientString(id = 2010116, message = "Thank you for fighting well until the end. I will request that you be additionally rewarded.")
	public static NpcStringId THANK_YOU_FOR_FIGHTING_WELL_UNTIL_THE_END_I_WILL_REQUEST_THAT_YOU_BE_ADDITIONALLY_REWARDED;
	
	@ClientString(id = 2518001, message = "Sacrifice has been killed! (Sacrifice left: $s1)")
	public static NpcStringId SACRIFICE_HAS_BEEN_KILLED_SACRIFICE_LEFT_S1;
	
	@ClientString(id = 2518002, message = "All have been sacrificed. You have failed.")
	public static NpcStringId ALL_HAVE_BEEN_SACRIFICED_YOU_HAVE_FAILED;
	
	@ClientString(id = 2518003, message = "Sacrifice left: $s1")
	public static NpcStringId SACRIFICE_LEFT_S1;
	
	@ClientString(id = 2518004, message = "You must stop the altar before everything is sacrificed.")
	public static NpcStringId YOU_MUST_STOP_THE_ALTAR_BEFORE_EVERYTHING_IS_SACRIFICED;
	
	@ClientString(id = 2518005, message = "Altar has stopped.")
	public static NpcStringId ALTAR_HAS_STOPPED;
	
	@ClientString(id = 2518006, message = "Altar of Shilen is starting! Must focus fire the altar!")
	public static NpcStringId ALTAR_OF_SHILEN_IS_STARTING_MUST_FOCUS_FIRE_THE_ALTAR;
	
	@ClientString(id = 2518007, message = "Focus fire the altar to stop blessing of Shilen!")
	public static NpcStringId FOCUS_FIRE_THE_ALTAR_TO_STOP_BLESSING_OF_SHILEN;
	
	@ClientString(id = 2518008, message = "Start the altar")
	public static NpcStringId START_THE_ALTAR;
	
	@ClientString(id = 2518009, message = "Sacrifice has been killed!")
	public static NpcStringId SACRIFICE_HAS_BEEN_KILLED;
	
	@ClientString(id = 2518010, message = "Evil aura comes out of Altar of Shilen.")
	public static NpcStringId EVIL_AURA_COMES_OUT_OF_ALTAR_OF_SHILEN;
	
	@ClientString(id = 2518011, message = "Altar of Shilen has been destroyed! You've won.")
	public static NpcStringId ALTAR_OF_SHILEN_HAS_BEEN_DESTROYED_YOU_VE_WON;
	
	@ClientString(id = 2621101, message = "Escape device location change")
	public static NpcStringId ESCAPE_DEVICE_LOCATION_CHANGE;
	
	@ClientString(id = 8888001, message = "Intruder. The gate will soon be closed.")
	public static NpcStringId INTRUDER_THE_GATE_WILL_SOON_BE_CLOSED;
	
	@ClientString(id = 8888002, message = "Who dare to interrupt our rest...")
	public static NpcStringId WHO_DARE_TO_INTERRUPT_OUR_REST;
	
	@ClientString(id = 8888003, message = "Those who came here looking for cursed ones... welcome.")
	public static NpcStringId THOSE_WHO_CAME_HERE_LOOKING_FOR_CURSED_ONES_WELCOME;
	
	@ClientString(id = 8888004, message = "Bloodsucking creatures! Absorb the light and fill it into darkness.")
	public static NpcStringId BLOODSUCKING_CREATURES_ABSORB_THE_LIGHT_AND_FILL_IT_INTO_DARKNESS;
	
	@ClientString(id = 8888005, message = "Let's see how much you can endure...")
	public static NpcStringId LET_S_SEE_HOW_MUCH_YOU_CAN_ENDURE;
	
	@ClientString(id = 8888006, message = "Don't you fear death?")
	public static NpcStringId DON_T_YOU_FEAR_DEATH;
	
	@ClientString(id = 8888007, message = "This is only the start.")
	public static NpcStringId THIS_IS_ONLY_THE_START;
	
	@ClientString(id = 8888008, message = "Darkness! Swallow everything away.")
	public static NpcStringId DARKNESS_SWALLOW_EVERYTHING_AWAY;
	
	@ClientString(id = 8888009, message = "Amazing... but this is the end. Full force advance!")
	public static NpcStringId AMAZING_BUT_THIS_IS_THE_END_FULL_FORCE_ADVANCE;
	
	@ClientString(id = 8888010, message = "I now have to go and handle it.")
	public static NpcStringId I_NOW_HAVE_TO_GO_AND_HANDLE_IT;
	
	@ClientString(id = 8888011, message = "For the eternal rest of the forgotten heroes!")
	public static NpcStringId FOR_THE_ETERNAL_REST_OF_THE_FORGOTTEN_HEROES;
	
	@ClientString(id = 8888012, message = "Their possession can be broken by breaking the sphere of light.")
	public static NpcStringId THEIR_POSSESSION_CAN_BE_BROKEN_BY_BREAKING_THE_SPHERE_OF_LIGHT;
	
	@ClientString(id = 8888013, message = "We need a little more...")
	public static NpcStringId WE_NEED_A_LITTLE_MORE;
	
	@ClientString(id = 8888014, message = "I will summon more spheres of light with the last of my strength.")
	public static NpcStringId I_WILL_SUMMON_MORE_SPHERES_OF_LIGHT_WITH_THE_LAST_OF_MY_STRENGTH;
	
	@ClientString(id = 8888015, message = "A little more... Please try a little more.")
	public static NpcStringId A_LITTLE_MORE_PLEASE_TRY_A_LITTLE_MORE;
	
	@ClientString(id = 8888016, message = "Bloodsucking creatures! Wake the soldiers now!")
	public static NpcStringId BLOODSUCKING_CREATURES_WAKE_THE_SOLDIERS_NOW;
	
	@ClientString(id = 8888017, message = "Stage 1")
	public static NpcStringId STAGE_1;
	
	@ClientString(id = 8888018, message = "Stage 2")
	public static NpcStringId STAGE_2;
	
	@ClientString(id = 8888019, message = "Stage 3")
	public static NpcStringId STAGE_3;
	
	@ClientString(id = 8888020, message = "Stage 4")
	public static NpcStringId STAGE_4;
	
	@ClientString(id = 8888021, message = "Stage 5")
	public static NpcStringId STAGE_5;
	
	@ClientString(id = 8888022, message = "Stage 6")
	public static NpcStringId STAGE_6;
	
	@ClientString(id = 8888023, message = "Stage 7")
	public static NpcStringId STAGE_7;
	
	@ClientString(id = 8888024, message = "Stage 8")
	public static NpcStringId STAGE_8;
	
	@ClientString(id = 8888025, message = "Final Stage")
	public static NpcStringId FINAL_STAGE;
	
	@ClientString(id = 8888026, message = "Bonus Stage")
	public static NpcStringId BONUS_STAGE;
	
	@ClientString(id = 8888027, message = "Weeping Yui appears.")
	public static NpcStringId WEEPING_YUI_APPEARS;
	
	@ClientString(id = 8888028, message = "Mukshu the Coward and Blind Hornafi appears.")
	public static NpcStringId MUKSHU_THE_COWARD_AND_BLIND_HORNAFI_APPEARS;
	
	@ClientString(id = 8888029, message = "Enraged Master Kinen appears.")
	public static NpcStringId ENRAGED_MASTER_KINEN_APPEARS;
	
	@ClientString(id = 8888030, message = "Sir Lesyinda of the Black Shadow appears.")
	public static NpcStringId SIR_LESYINDA_OF_THE_BLACK_SHADOW_APPEARS;
	
	@ClientString(id = 8888031, message = "Magical Warrior Konyar appears.")
	public static NpcStringId MAGICAL_WARRIOR_KONYAR_APPEARS;
	
	@ClientString(id = 8888032, message = "Yoentumak the Waiter appears.")
	public static NpcStringId YOENTUMAK_THE_WAITER_APPEARS;
	
	@ClientString(id = 8888033, message = "Ron appears.")
	public static NpcStringId RON_APPEARS;
	
	@ClientString(id = 8888100, message = "Listen! The Realm of Light will kneel in front of Goddess Shilen.")
	public static NpcStringId LISTEN_THE_REALM_OF_LIGHT_WILL_KNEEL_IN_FRONT_OF_GODDESS_SHILEN;
	
	@ClientString(id = 8888101, message = "Sacrifice of blood for resurrection and destruction!")
	public static NpcStringId SACRIFICE_OF_BLOOD_FOR_RESURRECTION_AND_DESTRUCTION;
	
	@ClientString(id = 8888102, message = "Great Goddess of Destruction")
	public static NpcStringId GREAT_GODDESS_OF_DESTRUCTION;
	
	@ClientString(id = 8888103, message = "Drink the sacrifice of blood that we have")
	public static NpcStringId DRINK_THE_SACRIFICE_OF_BLOOD_THAT_WE_HAVE;
	
	@ClientString(id = 8888104, message = "and bring down the hammer of justice!")
	public static NpcStringId AND_BRING_DOWN_THE_HAMMER_OF_JUSTICE;
	
	@ClientString(id = 8888105, message = "For the destruction and resurrection!")
	public static NpcStringId FOR_THE_DESTRUCTION_AND_RESURRECTION;
	
	@ClientString(id = 8888106, message = "The increased grasp of dark energy causes the ground to shake.")
	public static NpcStringId THE_INCREASED_GRASP_OF_DARK_ENERGY_CAUSES_THE_GROUND_TO_SHAKE;
	
	@ClientString(id = 8888107, message = "There's bright power emanating from the center of the town.")
	public static NpcStringId THERE_S_BRIGHT_POWER_EMANATING_FROM_THE_CENTER_OF_THE_TOWN;
	
	@ClientString(id = 8888108, message = "Dark power seeps out from the middle of the town.")
	public static NpcStringId DARK_POWER_SEEPS_OUT_FROM_THE_MIDDLE_OF_THE_TOWN;
	
	@ClientString(id = 8888109, message = "Please, let us go!")
	public static NpcStringId PLEASE_LET_US_GO;
	
	@ClientString(id = 8888110, message = "Let us live!!")
	public static NpcStringId LET_US_LIVE;
	
	@ClientString(id = 8888111, message = "I don't wanna die!")
	public static NpcStringId I_DON_T_WANNA_DIE;
	
	@ClientString(id = 8888112, message = "Mom...")
	public static NpcStringId MOM;
	
	@ClientString(id = 8888113, message = "Oh Einhasad...")
	public static NpcStringId OH_EINHASAD;
	
	@ClientString(id = 8888114, message = "Cough! Cough! My body is rotting...")
	public static NpcStringId COUGH_COUGH_MY_BODY_IS_ROTTING;
	
	@ClientString(id = 8888115, message = "Dirty Shilen's dogs!")
	public static NpcStringId DIRTY_SHILEN_S_DOGS;
	
	@ClientString(id = 8888116, message = "Ahhk!")
	public static NpcStringId AHHK;
	
	@ClientString(id = 8888117, message = "Ughh...!")
	public static NpcStringId UGHH;
	
	@ClientString(id = 8888118, message = "You feel yourself being dragged to another location.")
	public static NpcStringId YOU_FEEL_YOURSELF_BEING_DRAGGED_TO_ANOTHER_LOCATION;
	
	@ClientString(id = 8888119, message = "On your way to the spirit world of light, you suddenly feel a dark aura surrounding you.")
	public static NpcStringId ON_YOUR_WAY_TO_THE_SPIRIT_WORLD_OF_LIGHT_YOU_SUDDENLY_FEEL_A_DARK_AURA_SURROUNDING_YOU;
	
	@ClientString(id = 8888120, message = "$s1...You ended up giving rotting dark to them! Darkness will never disappear. I will be back!")
	public static NpcStringId S1_YOU_ENDED_UP_GIVING_ROTTING_DARK_TO_THEM_DARKNESS_WILL_NEVER_DISAPPEAR_I_WILL_BE_BACK;
	
	@ClientString(id = 8888121, message = "$s1! Cannot give them decaying darkness!")
	public static NpcStringId S1_CANNOT_GIVE_THEM_DECAYING_DARKNESS;
	
	@ClientString(id = 8888122, message = "$s1! This brings peace to the town! Thank you!")
	public static NpcStringId S1_THIS_BRINGS_PEACE_TO_THE_TOWN_THANK_YOU;
	
	@ClientString(id = 8888123, message = "$s1, May the darkness bless you...heheheh")
	public static NpcStringId S1_MAY_THE_DARKNESS_BLESS_YOU_HEHEHEH;
	
	@ClientString(id = 8888124, message = "$s1 successfully enchanted to + 18! Let's all congratulate him!")
	public static NpcStringId S1_SUCCESSFULLY_ENCHANTED_TO_18_LET_S_ALL_CONGRATULATE_HIM;
	
	@ClientString(id = 10004431, message = "It teleports the guard members of the Aden Imperial Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_ADEN_IMPERIAL_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004432, message = "It teleports the guard members of the Gludio Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_GLUDIO_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004433, message = "It teleports the guard members of the Dion Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_DION_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004434, message = "It teleports the guard members of the Giran Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_GIRAN_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004435, message = "It teleports the guard members of the Dion Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_DION_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE2;
	
	@ClientString(id = 10004436, message = "It teleports the guard members of the Aden Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_ADEN_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004437, message = "It teleports the guard members of the Innadril Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_INNADRIL_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004438, message = "It teleports the guard members of the Goddard Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_GODDARD_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004439, message = "It teleports the guard members of the Rune Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_RUNE_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004440, message = "It teleports the guard members of the Schuttgart Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_SCHUTTGART_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10004441, message = "It teleports the guard members of the Elmore Imperial Castle to the inside of the castle.")
	public static NpcStringId IT_TELEPORTS_THE_GUARD_MEMBERS_OF_THE_ELMORE_IMPERIAL_CASTLE_TO_THE_INSIDE_OF_THE_CASTLE;
	
	@ClientString(id = 10105681, message = "Wall of Argos (Altar of White Allosce)")
	public static NpcStringId WALL_OF_ARGOS_ALTAR_OF_WHITE_ALLOSCE;
	
	@ClientString(id = 10303001, message = "$s1! May the blessing of light be with you!")
	public static NpcStringId S1_MAY_THE_BLESSING_OF_LIGHT_BE_WITH_YOU;
	
	@ClientString(id = 10303002, message = "$s1! You will regret not giving me the trace of decaying darkness!")
	public static NpcStringId S1_YOU_WILL_REGRET_NOT_GIVING_ME_THE_TRACE_OF_DECAYING_DARKNESS;
	
	@ClientString(id = 10303003, message = "$s1. May the blessing of darkness be with you!")
	public static NpcStringId S1_MAY_THE_BLESSING_OF_DARKNESS_BE_WITH_YOU;
	
	@ClientString(id = 10303004, message = "$s1. You fool! You will regret not giving me the trace of decaying darkness!")
	public static NpcStringId S1_YOU_FOOL_YOU_WILL_REGRET_NOT_GIVING_ME_THE_TRACE_OF_DECAYING_DARKNESS;
	
	@ClientString(id = 10307000, message = "You are too far from the corpse to show your condolences.")
	public static NpcStringId YOU_ARE_TOO_FAR_FROM_THE_CORPSE_TO_SHOW_YOUR_CONDOLENCES;
	
	@ClientString(id = 10307001, message = "$s1! Even if you make me fall, it's all useless!")
	public static NpcStringId S1_EVEN_IF_YOU_MAKE_ME_FALL_IT_S_ALL_USELESS;
	
	@ClientString(id = 10307002, message = "I will come back alive with rotting aura!")
	public static NpcStringId I_WILL_COME_BACK_ALIVE_WITH_ROTTING_AURA;
	
	@ClientString(id = 10307003, message = "Darkness that engulfed me one day made me this way!")
	public static NpcStringId DARKNESS_THAT_ENGULFED_ME_ONE_DAY_MADE_ME_THIS_WAY;
	
	@ClientString(id = 10307004, message = "$s1, be careful. Sad... my fairies!")
	public static NpcStringId S1_BE_CAREFUL_SAD_MY_FAIRIES;
	
	@ClientString(id = 10307005, message = "Given to $s1")
	public static NpcStringId GIVEN_TO_S1;
	
	@ClientString(id = 10307006, message = "Traitor Crystalline Golem")
	public static NpcStringId TRAITOR_CRYSTALLINE_GOLEM;
	
	@ClientString(id = 10307007, message = "Crystalline Golem")
	public static NpcStringId CRYSTALLINE_GOLEM;
	
	@ClientString(id = 10307008, message = "Scout")
	public static NpcStringId SCOUT;
	
	@ClientString(id = 10307009, message = "No.")
	public static NpcStringId NO;
	
	@ClientString(id = 10307010, message = "It's a secret.")
	public static NpcStringId IT_S_A_SECRET;
	
	@ClientString(id = 10307011, message = "I can't tell you.")
	public static NpcStringId I_CAN_T_TELL_YOU;
	
	@ClientString(id = 10307012, message = "No. No.")
	public static NpcStringId NO_NO;
	
	@ClientString(id = 10307013, message = "Yes. Yes. Yes. Yes.")
	public static NpcStringId YES_YES_YES_YES;
	
	@ClientString(id = 10307014, message = "It hurts. Stop.")
	public static NpcStringId IT_HURTS_STOP;
	
	@ClientString(id = 10338001, message = "The one who resonates to darkness...")
	public static NpcStringId THE_ONE_WHO_RESONATES_TO_DARKNESS;
	
	@ClientString(id = 10338002, message = "Find Hermuncus.")
	public static NpcStringId FIND_HERMUNCUS;
	
	@ClientString(id = 10338003, message = "Find Hermuncus.")
	public static NpcStringId FIND_HERMUNCUS2;
	
	@ClientString(id = 10338004, message = "Find Hermuncus.")
	public static NpcStringId FIND_HERMUNCUS3;
	
	@ClientString(id = 10338005, message = "Break the seal.")
	public static NpcStringId BREAK_THE_SEAL;
	
	@ClientString(id = 10338006, message = "Free me from this binding of light!")
	public static NpcStringId FREE_ME_FROM_THIS_BINDING_OF_LIGHT;
	
	@ClientString(id = 10338007, message = "Destroy the Ghost of Harnak… this corrupted creature.")
	public static NpcStringId DESTROY_THE_GHOST_OF_HARNAK_THIS_CORRUPTED_CREATURE;
	
	@ClientString(id = 10338008, message = "Free me... And I promise you the power of Giants!")
	public static NpcStringId FREE_ME_AND_I_PROMISE_YOU_THE_POWER_OF_GIANTS;
	
	@ClientString(id = 10338009, message = "Fight for me...ﾠ The true leader of all the races!")
	public static NpcStringId FIGHT_FOR_ME_THE_TRUE_LEADER_OF_ALL_THE_RACES;
	
	@ClientString(id = 10338010, message = "You may use Scroll of Afterlife from Hermuncus to Awaken.")
	public static NpcStringId YOU_MAY_USE_SCROLL_OF_AFTERLIFE_FROM_HERMUNCUS_TO_AWAKEN;
	
	@ClientString(id = 10338011, message = "An intruder… interesting.")
	public static NpcStringId AN_INTRUDER_INTERESTING;
	
	@ClientString(id = 10338012, message = "Prove your worth…")
	public static NpcStringId PROVE_YOUR_WORTH;
	
	@ClientString(id = 10338013, message = "Only those strong enough shall proceed.")
	public static NpcStringId ONLY_THOSE_STRONG_ENOUGH_SHALL_PROCEED;
	
	@ClientString(id = 10338014, message = "Are you against the will of light?")
	public static NpcStringId ARE_YOU_AGAINST_THE_WILL_OF_LIGHT;
	
	@ClientString(id = 10338015, message = "Come! Attack me if you dare!")
	public static NpcStringId COME_ATTACK_ME_IF_YOU_DARE;
	
	@ClientString(id = 10338016, message = "Are you planning to betray the gods and follow a Giant?")
	public static NpcStringId ARE_YOU_PLANNING_TO_BETRAY_THE_GODS_AND_FOLLOW_A_GIANT;
	
	@ClientString(id = 10338017, message = "It's the end for you traitor!")
	public static NpcStringId IT_S_THE_END_FOR_YOU_TRAITOR;
	
	@ClientString(id = 10338018, message = "Haha…")
	public static NpcStringId HAHA;
	
	@ClientString(id = 10338019, message = "I want to hear you cry.")
	public static NpcStringId I_WANT_TO_HEAR_YOU_CRY;
	
	@ClientString(id = 10338020, message = "Mortal!")
	public static NpcStringId MORTAL;
	
	@ClientString(id = 10338021, message = "Lets see what you are made of!")
	public static NpcStringId LETS_SEE_WHAT_YOU_ARE_MADE_OF;
	
	@ClientString(id = 10338022, message = "You will not free Hermuncus.ﾠ")
	public static NpcStringId YOU_WILL_NOT_FREE_HERMUNCUS;
	
	@ClientString(id = 10338023, message = "You'll have to kill us first!")
	public static NpcStringId YOU_LL_HAVE_TO_KILL_US_FIRST;
	
	@ClientString(id = 10338024, message = "Trying to free Hermuncus…")
	public static NpcStringId TRYING_TO_FREE_HERMUNCUS;
	
	@ClientString(id = 10338025, message = "Repent and your death will be quick!")
	public static NpcStringId REPENT_AND_YOUR_DEATH_WILL_BE_QUICK;
	
	@ClientString(id = 10338026, message = "You will never break the seal!ﾠ")
	public static NpcStringId YOU_WILL_NEVER_BREAK_THE_SEAL;
	
	@ClientString(id = 10338027, message = "Die traitor!")
	public static NpcStringId DIE_TRAITOR;
	
	@ClientString(id = 10338028, message = "Only the light may pass.ﾠ")
	public static NpcStringId ONLY_THE_LIGHT_MAY_PASS;
	
	@ClientString(id = 10338029, message = "You are not light. You may not pass.")
	public static NpcStringId YOU_ARE_NOT_LIGHT_YOU_MAY_NOT_PASS;
	
	@ClientString(id = 10338030, message = "Prove your worth…")
	public static NpcStringId PROVE_YOUR_WORTH2;
	
	@ClientString(id = 10338031, message = "Are you strong or weak... Of the Light or darkness…")
	public static NpcStringId ARE_YOU_STRONG_OR_WEAK_OF_THE_LIGHT_OR_DARKNESS;
	
	@ClientString(id = 10338032, message = "Only those of light may pass. Others must prove their strength.")
	public static NpcStringId ONLY_THOSE_OF_LIGHT_MAY_PASS_OTHERS_MUST_PROVE_THEIR_STRENGTH;
	
	@ClientString(id = 10338033, message = "Warrior. I've finally got out of seal of light thanks to you.")
	public static NpcStringId WARRIOR_I_VE_FINALLY_GOT_OUT_OF_SEAL_OF_LIGHT_THANKS_TO_YOU;
	
	@ClientString(id = 10338034, message = "Is strength to defend the world what you wanted?")
	public static NpcStringId IS_STRENGTH_TO_DEFEND_THE_WORLD_WHAT_YOU_WANTED;
	
	@ClientString(id = 10338035, message = "I will awaken you...")
	public static NpcStringId I_WILL_AWAKEN_YOU;
	
	@ClientString(id = 10338036, message = "Receive this power form the ancient Giant.ﾠ")
	public static NpcStringId RECEIVE_THIS_POWER_FORM_THE_ANCIENT_GIANT;
	
	@ClientString(id = 10338037, message = "Use this new power when the time is right.")
	public static NpcStringId USE_THIS_NEW_POWER_WHEN_THE_TIME_IS_RIGHT;
	
	@ClientString(id = 10363001, message = "Resurrected Soul")
	public static NpcStringId RESURRECTED_SOUL;
	
	@ClientString(id = 11021701, message = "We saw people use Sayune other than the Giant's Minions.")
	public static NpcStringId WE_SAW_PEOPLE_USE_SAYUNE_OTHER_THAN_THE_GIANT_S_MINIONS;
	
	@ClientString(id = 11021702, message = "It's because they have become Awaken… the power of the Giant's.")
	public static NpcStringId IT_S_BECAUSE_THEY_HAVE_BECOME_AWAKEN_THE_POWER_OF_THE_GIANT_S;
	
	@ClientString(id = 11021703, message = "Duh, you can't use Sayune without powers of Giants in the first place.")
	public static NpcStringId DUH_YOU_CAN_T_USE_SAYUNE_WITHOUT_POWERS_OF_GIANTS_IN_THE_FIRST_PLACE;
	
	@ClientString(id = 11021704, message = "I think that's why many people come to visit Celphine.")
	public static NpcStringId I_THINK_THAT_S_WHY_MANY_PEOPLE_COME_TO_VISIT_CELPHINE;
	
	@ClientString(id = 11021705, message = "After Ye Sagira fell there were many sacrifices made.")
	public static NpcStringId AFTER_YE_SAGIRA_FELL_THERE_WERE_MANY_SACRIFICES_MADE;
	
	@ClientString(id = 11021706, message = "Didn't that release the seal on Hermuncus?")
	public static NpcStringId DIDN_T_THAT_RELEASE_THE_SEAL_ON_HERMUNCUS;
	
	@ClientString(id = 11021707, message = "That's the rumor, but nothing to confirm it.")
	public static NpcStringId THAT_S_THE_RUMOR_BUT_NOTHING_TO_CONFIRM_IT;
	
	@ClientString(id = 11021708, message = "It's got to be true, we need to find it.")
	public static NpcStringId IT_S_GOT_TO_BE_TRUE_WE_NEED_TO_FIND_IT;
	
	@ClientString(id = 11022201, message = "Accessories have been added to your Inventory")
	public static NpcStringId ACCESSORIES_HAVE_BEEN_ADDED_TO_YOUR_INVENTORY;
	
	@ClientString(id = 11022202, message = "Armor has been added to your Inventory.")
	public static NpcStringId ARMOR_HAS_BEEN_ADDED_TO_YOUR_INVENTORY;
	
	@ClientString(id = 11170000, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Original Class(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ORIGINAL_CLASS_BUTTON;
	
	@ClientString(id = 11170002, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Gladiator / Duelist(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_GLADIATOR_DUELIST_BUTTON;
	
	@ClientString(id = 11170003, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Warlord / Dreadnought(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_WARLORD_DREADNOUGHT_BUTTON;
	
	@ClientString(id = 11170005, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Paladin / Phoenix Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PALADIN_PHOENIX_KNIGHT_BUTTON;
	
	@ClientString(id = 11170006, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Dark Avenger / Hell Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_DARK_AVENGER_HELL_KNIGHT_BUTTON;
	
	@ClientString(id = 11170008, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Treasure Hunter / Adventurer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_TREASURE_HUNTER_ADVENTURER_BUTTON;
	
	@ClientString(id = 11170009, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Hawkeye / Sagittarius(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_HAWKEYE_SAGITTARIUS_BUTTON;
	
	@ClientString(id = 11170012, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Sorcerer / Archmage(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SORCERER_ARCHMAGE_BUTTON;
	
	@ClientString(id = 11170013, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Necromancer / Soultaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_NECROMANCER_SOULTAKER_BUTTON;
	
	@ClientString(id = 11170014, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Warlock / Arcana Lord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_WARLOCK_ARCANA_LORD_BUTTON;
	
	@ClientString(id = 11170016, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Bishop / Cardinal(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_BISHOP_CARDINAL_BUTTON;
	
	@ClientString(id = 11170017, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Prophet / Hierophant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PROPHET_HIEROPHANT_BUTTON;
	
	@ClientString(id = 11170020, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Temple Knight / Eva's Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_TEMPLE_KNIGHT_EVA_S_TEMPLAR_BUTTON;
	
	@ClientString(id = 11170021, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Swordsinger / Sword Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SWORDSINGER_SWORD_MUSE_BUTTON;
	
	@ClientString(id = 11170023, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Plains Walker / Wind Rider(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PLAINS_WALKER_WIND_RIDER_BUTTON;
	
	@ClientString(id = 11170024, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2') Silver Ranger / Moonlight Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SILVER_RANGER_MOONLIGHT_SENTINEL_BUTTON;
	
	@ClientString(id = 11170027, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Spellsinger / Mystic Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SPELLSINGER_MYSTIC_MUSE_BUTTON;
	
	@ClientString(id = 11170028, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Elemental Summoner / Elemental Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ELEMENTAL_SUMMONER_ELEMENTAL_MASTER_BUTTON;
	
	@ClientString(id = 11170030, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Elder / Eva's Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ELDER_EVA_S_SAINT_BUTTON;
	
	@ClientString(id = 11170033, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Shillien Knight / Shillien Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SHILLIEN_KNIGHT_SHILLIEN_TEMPLAR_BUTTON;
	
	@ClientString(id = 11170034, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Blade Dancer / Spectral Dancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_BLADE_DANCER_SPECTRAL_DANCER_BUTTON;
	
	@ClientString(id = 11170036, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Abyss Walker / Ghost Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ABYSS_WALKER_GHOST_HUNTER_BUTTON;
	
	@ClientString(id = 11170037, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2') Phantom Ranger / Ghost Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PHANTOM_RANGER_GHOST_SENTINEL_BUTTON;
	
	@ClientString(id = 11170040, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Spellhowler / Storm Screamer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SPELLHOWLER_STORM_SCREAMER_BUTTON;
	
	@ClientString(id = 11170041, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Phantom Summoner / Spectral Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PHANTOM_SUMMONER_SPECTRAL_MASTER_BUTTON;
	
	@ClientString(id = 11170043, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Shillien Elder / Shillien Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SHILLIEN_ELDER_SHILLIEN_SAINT_BUTTON;
	
	@ClientString(id = 11170046, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Destroyer / Titan(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_DESTROYER_TITAN_BUTTON;
	
	@ClientString(id = 11170048, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Tyrant / Grand Khavatari(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_TYRANT_GRAND_KHAVATARI_BUTTON;
	
	@ClientString(id = 11170051, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Overlord / Dominator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_OVERLORD_DOMINATOR_BUTTON;
	
	@ClientString(id = 11170052, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Warcryer / Doomcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_WARCRYER_DOOMCRYER_BUTTON;
	
	@ClientString(id = 11170055, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Bounty Hunter / Fortune Seeker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_BOUNTY_HUNTER_FORTUNE_SEEKER_BUTTON;
	
	@ClientString(id = 11170057, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Warsmith / Maestro(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_WARSMITH_MAESTRO_BUTTON;
	
	@ClientString(id = 11170088, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Gladiator / Duelist(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_GLADIATOR_DUELIST_BUTTON2;
	
	@ClientString(id = 11170089, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Warlord / Dreadnought(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_WARLORD_DREADNOUGHT_BUTTON2;
	
	@ClientString(id = 11170090, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Paladin / Phoenix Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PALADIN_PHOENIX_KNIGHT_BUTTON2;
	
	@ClientString(id = 11170091, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Dark Avenger / Hell Knight(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_DARK_AVENGER_HELL_KNIGHT_BUTTON2;
	
	@ClientString(id = 11170092, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Hawkeye / Sagittarius(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_HAWKEYE_SAGITTARIUS_BUTTON2;
	
	@ClientString(id = 11170093, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Treasure Hunter / Adventurer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_TREASURE_HUNTER_ADVENTURER_BUTTON2;
	
	@ClientString(id = 11170094, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Sorcerer / Archmage(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SORCERER_ARCHMAGE_BUTTON2;
	
	@ClientString(id = 11170095, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Necromancer / Soultaker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_NECROMANCER_SOULTAKER_BUTTON2;
	
	@ClientString(id = 11170096, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Warlock / Arcana Lord(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_WARLOCK_ARCANA_LORD_BUTTON2;
	
	@ClientString(id = 11170097, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Bishop / Cardinal(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_BISHOP_CARDINAL_BUTTON2;
	
	@ClientString(id = 11170098, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Prophet / Hierophant(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PROPHET_HIEROPHANT_BUTTON2;
	
	@ClientString(id = 11170099, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Temple Knight / Eva's Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_TEMPLE_KNIGHT_EVA_S_TEMPLAR_BUTTON2;
	
	@ClientString(id = 11170100, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Swordsinger / Sword Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SWORDSINGER_SWORD_MUSE_BUTTON2;
	
	@ClientString(id = 11170101, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Plains Walker / Wind Rider(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PLAINS_WALKER_WIND_RIDER_BUTTON2;
	
	@ClientString(id = 11170102, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Silver Ranger / Moonlight Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SILVER_RANGER_MOONLIGHT_SENTINEL_BUTTON2;
	
	@ClientString(id = 11170103, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Spellsinger / Mystic Muse(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SPELLSINGER_MYSTIC_MUSE_BUTTON2;
	
	@ClientString(id = 11170104, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Elemental Summoner / Elemental Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ELEMENTAL_SUMMONER_ELEMENTAL_MASTER_BUTTON2;
	
	@ClientString(id = 11170105, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Elder / Eva's Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ELDER_EVA_S_SAINT_BUTTON2;
	
	@ClientString(id = 11170106, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Shillien Knight / Shillien Templar(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SHILLIEN_KNIGHT_SHILLIEN_TEMPLAR_BUTTON2;
	
	@ClientString(id = 11170107, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Blade Dancer / Spectral Dancer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_BLADE_DANCER_SPECTRAL_DANCER_BUTTON2;
	
	@ClientString(id = 11170108, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Abyss Walker / Ghost Hunter(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ABYSS_WALKER_GHOST_HUNTER_BUTTON2;
	
	@ClientString(id = 11170109, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Phantom Ranger / Ghost Sentinel(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PHANTOM_RANGER_GHOST_SENTINEL_BUTTON2;
	
	@ClientString(id = 11170110, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Spellhowler / Storm Screamer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SPELLHOWLER_STORM_SCREAMER_BUTTON2;
	
	@ClientString(id = 11170111, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Phantom Summoner / Spectral Master(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_PHANTOM_SUMMONER_SPECTRAL_MASTER_BUTTON2;
	
	@ClientString(id = 11170112, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Shillien Elder / Shillien Saint(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SHILLIEN_ELDER_SHILLIEN_SAINT_BUTTON2;
	
	@ClientString(id = 11170113, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Destroyer / Titan(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_DESTROYER_TITAN_BUTTON2;
	
	@ClientString(id = 11170114, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Tyrant / Grand Khavatari(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_TYRANT_GRAND_KHAVATARI_BUTTON2;
	
	@ClientString(id = 11170115, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Overlord / Dominator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_OVERLORD_DOMINATOR_BUTTON2;
	
	@ClientString(id = 11170116, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Warcryer / Doomcryer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_WARCRYER_DOOMCRYER_BUTTON2;
	
	@ClientString(id = 11170117, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Bounty Hunter / Fortune Seeker(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_BOUNTY_HUNTER_FORTUNE_SEEKER_BUTTON2;
	
	@ClientString(id = 11170118, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Warsmith / Maestro(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_WARSMITH_MAESTRO_BUTTON2;
	
	@ClientString(id = 11170127, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Berserker / Doombringer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_BERSERKER_DOOMBRINGER_BUTTON;
	
	@ClientString(id = 11170128, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Soul Breaker (Male) / Soulhound (Male)(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SOUL_BREAKER_MALE_SOULHOUND_MALE_BUTTON;
	
	@ClientString(id = 11170129, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Soul Breaker (Female) / Soulhound (Female)(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SOUL_BREAKER_FEMALE_SOULHOUND_FEMALE_BUTTON;
	
	@ClientString(id = 11170130, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Arbalester / Trickster(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ARBALESTER_TRICKSTER_BUTTON;
	
	@ClientString(id = 11170131, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Berserker / DoomBringer(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_BERSERKER_DOOMBRINGER_BUTTON2;
	
	@ClientString(id = 11170132, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Soul Breaker (Male) / Soul Hound(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SOUL_BREAKER_MALE_SOUL_HOUND_BUTTON;
	
	@ClientString(id = 11170133, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Soul Breaker (Female) / Soul Hound(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_SOUL_BREAKER_FEMALE_SOUL_HOUND_BUTTON;
	
	@ClientString(id = 11170134, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Arbalester / Trickster(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_ARBALESTER_TRICKSTER_BUTTON2;
	
	@ClientString(id = 11170135, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Inspector / Judicator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_INSPECTOR_JUDICATOR_BUTTON;
	
	@ClientString(id = 11170136, message = "(Button ALIGN=LEFT ICON='NORMAL' action='bypass -h menu_select?ask=$s1andreply=$s2')Inspector / Judicator(/button)")
	public static NpcStringId BUTTON_ALIGN_LEFT_ICON_NORMAL_ACTION_BYPASS_H_MENU_SELECT_ASK_S1ANDREPLY_S2_INSPECTOR_JUDICATOR_BUTTON2;
	
	@ClientString(id = 14211701, message = "You must activate the 4 Generators.")
	public static NpcStringId YOU_MUST_ACTIVATE_THE_4_GENERATORS;
	
	@ClientString(id = 14211702, message = "Protect the Generator!")
	public static NpcStringId PROTECT_THE_GENERATOR2;
	
	@ClientString(id = 14211703, message = "$s1 has charged the cannon!")
	public static NpcStringId S1_HAS_CHARGED_THE_CANNON;
	
	@ClientString(id = 14211704, message = "Charge the cannon using the Generator.")
	public static NpcStringId CHARGE_THE_CANNON_USING_THE_GENERATOR;
	
	@ClientString(id = 14211705, message = "A fearsome power emanates from Lindvior!")
	public static NpcStringId A_FEARSOME_POWER_EMANATES_FROM_LINDVIOR;
	
	@ClientString(id = 14211706, message = "A gigantic whirlwind has appeared!")
	public static NpcStringId A_GIGANTIC_WHIRLWIND_HAS_APPEARED;
	
	@ClientString(id = 14211707, message = "$s1 minutes left until Lindvior gathers his full energy!")
	public static NpcStringId S1_MINUTES_LEFT_UNTIL_LINDVIOR_GATHERS_HIS_FULL_ENERGY;
	
	@ClientString(id = 14211708, message = "Lindvior has fallen from the sky!")
	public static NpcStringId LINDVIOR_HAS_FALLEN_FROM_THE_SKY;
	
	@ClientString(id = 14211709, message = "Lindvior has landed!")
	public static NpcStringId LINDVIOR_HAS_LANDED;
	
	@ClientString(id = 14211710, message = "Target the Generator and use a Chain Skill!")
	public static NpcStringId TARGET_THE_GENERATOR_AND_USE_A_CHAIN_SKILL;
	
	@ClientString(id = 14211711, message = "The Generator has been destroyed!")
	public static NpcStringId THE_GENERATOR_HAS_BEEN_DESTROYED;
	
	@ClientString(id = 14211712, message = "Tracker")
	public static NpcStringId TRACKER;
	
	@ClientString(id = 14211713, message = "Aggressive")
	public static NpcStringId AGGRESSIVE2;
	
	@ClientString(id = 14211714, message = "Meddler")
	public static NpcStringId MEDDLER;
	
	@ClientString(id = 14211715, message = "Honorable warriors have driven off Lindvior, the evil wind dragon!")
	public static NpcStringId HONORABLE_WARRIORS_HAVE_DRIVEN_OFF_LINDVIOR_THE_EVIL_WIND_DRAGON;
	
	@ClientString(id = 14211716, message = "Beleth's Appearance")
	public static NpcStringId BELETH_S_APPEARANCE;
	
	@ClientString(id = 14211717, message = "Darion's Appearance")
	public static NpcStringId DARION_S_APPEARANCE;
	
	@ClientString(id = 16211701, message = "Charging")
	public static NpcStringId CHARGING;
	
	@ClientString(id = 17171749, message = "Congratulations! You will now graduate from the Clan Academy and leave your current clan. You can now join a clan as a regular member without being subject to any penalties.")
	public static NpcStringId CONGRATULATIONS_YOU_WILL_NOW_GRADUATE_FROM_THE_CLAN_ACADEMY_AND_LEAVE_YOUR_CURRENT_CLAN_YOU_CAN_NOW_JOIN_A_CLAN_AS_A_REGULAR_MEMBER_WITHOUT_BEING_SUBJECT_TO_ANY_PENALTIES;
	
	@ClientString(id = 17171750, message = "Congratulations! After graduating the clan academy, you quit the clan.")
	public static NpcStringId CONGRATULATIONS_AFTER_GRADUATING_THE_CLAN_ACADEMY_YOU_QUIT_THE_CLAN;
	
	@ClientString(id = 17171751, message = "The graduate may join the clan without any penalty.")
	public static NpcStringId THE_GRADUATE_MAY_JOIN_THE_CLAN_WITHOUT_ANY_PENALTY;
	
	@ClientString(id = 17173500, message = "Activate the automatic use of soulshot for an effective battle.")
	public static NpcStringId ACTIVATE_THE_AUTOMATIC_USE_OF_SOULSHOT_FOR_AN_EFFECTIVE_BATTLE;
	
	@ClientString(id = 17173501, message = "The color of monster's name indicates the difference from your level.")
	public static NpcStringId THE_COLOR_OF_MONSTER_S_NAME_INDICATES_THE_DIFFERENCE_FROM_YOUR_LEVEL;
	
	@ClientString(id = 17173502, message = "In order to use the skill, utilize the shortcut by placing the skill icon on the shortcut.")
	public static NpcStringId IN_ORDER_TO_USE_THE_SKILL_UTILIZE_THE_SHORTCUT_BY_PLACING_THE_SKILL_ICON_ON_THE_SHORTCUT;
	
	@ClientString(id = 17173503, message = "Others and etc., it will be useful on A.")
	public static NpcStringId OTHERS_AND_ETC_IT_WILL_BE_USEFUL_ON_A;
	
	@ClientString(id = 17173504, message = "Others and etc., it will be useful on B.")
	public static NpcStringId OTHERS_AND_ETC_IT_WILL_BE_USEFUL_ON_B;
	
	@ClientString(id = 17178300, message = "Shoot fire at the imbecile!")
	public static NpcStringId SHOOT_FIRE_AT_THE_IMBECILE;
	
	@ClientString(id = 17178301, message = "You're even dishonest. Power of earth will drop from the sky!")
	public static NpcStringId YOU_RE_EVEN_DISHONEST_POWER_OF_EARTH_WILL_DROP_FROM_THE_SKY;
	
	@ClientString(id = 17178302, message = "Wrath of the ground will fall from the sky on $s1!")
	public static NpcStringId WRATH_OF_THE_GROUND_WILL_FALL_FROM_THE_SKY_ON_S1;
	
	@ClientString(id = 17178303, message = "Behemoth and Tarrasque, Rise with the powers of the ground and help me.")
	public static NpcStringId BEHEMOTH_AND_TARRASQUE_RISE_WITH_THE_POWERS_OF_THE_GROUND_AND_HELP_ME;
	
	@ClientString(id = 17178304, message = "Not enough... I will have to go myself.")
	public static NpcStringId NOT_ENOUGH_I_WILL_HAVE_TO_GO_MYSELF;
	
	@ClientString(id = 17178305, message = "Children. With noble your sacrifice, give them pain!")
	public static NpcStringId CHILDREN_WITH_NOBLE_YOUR_SACRIFICE_GIVE_THEM_PAIN;
	
	@ClientString(id = 17178306, message = "It's stronger than expected! I didn't think I'd be hurt this much...")
	public static NpcStringId IT_S_STRONGER_THAN_EXPECTED_I_DIDN_T_THINK_I_D_BE_HURT_THIS_MUCH;
	
	@ClientString(id = 17178307, message = "Children. Heal me with your noble sacrifice.")
	public static NpcStringId CHILDREN_HEAL_ME_WITH_YOUR_NOBLE_SACRIFICE;
	
	@ClientString(id = 17178308, message = "Your sacrifices will become a new rescue...")
	public static NpcStringId YOUR_SACRIFICES_WILL_BECOME_A_NEW_RESCUE;
	
	@ClientString(id = 17178309, message = "Children. Give everything you've got!")
	public static NpcStringId CHILDREN_GIVE_EVERYTHING_YOU_VE_GOT;
	
	@ClientString(id = 17178310, message = "Lowly beings! Can you handle my wrath!")
	public static NpcStringId LOWLY_BEINGS_CAN_YOU_HANDLE_MY_WRATH;
	
	@ClientString(id = 17178311, message = "This is the end of your desperate measures!")
	public static NpcStringId THIS_IS_THE_END_OF_YOUR_DESPERATE_MEASURES;
	
	@ClientString(id = 17178312, message = "Is this the end...")
	public static NpcStringId IS_THIS_THE_END3;
	
	@ClientString(id = 17178313, message = "Can't hold any longer.")
	public static NpcStringId CAN_T_HOLD_ANY_LONGER;
	
	@ClientString(id = 17178314, message = "Don't you think we have a chance if we attack some more?")
	public static NpcStringId DON_T_YOU_THINK_WE_HAVE_A_CHANCE_IF_WE_ATTACK_SOME_MORE;
	
	@ClientString(id = 17178315, message = "I want to go back!!")
	public static NpcStringId I_WANT_TO_GO_BACK;
	
	@ClientString(id = 17178316, message = "Is this Antharas...")
	public static NpcStringId IS_THIS_ANTHARAS;
	
	@ClientString(id = 17178317, message = "We have no more chance. We must go back!")
	public static NpcStringId WE_HAVE_NO_MORE_CHANCE_WE_MUST_GO_BACK;
	
	@ClientString(id = 17178318, message = "I think we hurt him good. We can defeat him!")
	public static NpcStringId I_THINK_WE_HURT_HIM_GOOD_WE_CAN_DEFEAT_HIM;
	
	@ClientString(id = 17178319, message = "You want more losses?!")
	public static NpcStringId YOU_WANT_MORE_LOSSES;
	
	@ClientString(id = 17178320, message = "Watch your words!")
	public static NpcStringId WATCH_YOUR_WORDS;
	
	@ClientString(id = 17178321, message = "Everyone listen!")
	public static NpcStringId EVERYONE_LISTEN;
	
	@ClientString(id = 17178322, message = "This is their limit!")
	public static NpcStringId THIS_IS_THEIR_LIMIT;
	
	@ClientString(id = 17178323, message = "Do your best for those who died for us!!")
	public static NpcStringId DO_YOUR_BEST_FOR_THOSE_WHO_DIED_FOR_US;
	
	@ClientString(id = 17178324, message = "Charge!!!!")
	public static NpcStringId CHARGE2;
	
	@ClientString(id = 17178325, message = "Damn it! Will we end up dead here...")
	public static NpcStringId DAMN_IT_WILL_WE_END_UP_DEAD_HERE;
	
	@ClientString(id = 17178326, message = "Whoaaaaaa!!!!")
	public static NpcStringId WHOAAAAAA3;
	
	@ClientString(id = 17178327, message = "How stubborn... Squirming 'til the last minute")
	public static NpcStringId HOW_STUBBORN_SQUIRMING_TIL_THE_LAST_MINUTE;
	
	@ClientString(id = 17178328, message = "I can't die like this! I will get backup from the Kingdom!")
	public static NpcStringId I_CAN_T_DIE_LIKE_THIS_I_WILL_GET_BACKUP_FROM_THE_KINGDOM;
	
	@ClientString(id = 17178329, message = "Mercenaries! I think they're here to support us!")
	public static NpcStringId MERCENARIES_I_THINK_THEY_RE_HERE_TO_SUPPORT_US;
	
	@ClientString(id = 17178330, message = "Mercenaries!! Everyone charge! We can't let them win it for us!")
	public static NpcStringId MERCENARIES_EVERYONE_CHARGE_WE_CAN_T_LET_THEM_WIN_IT_FOR_US;
	
	@ClientString(id = 17178331, message = "Be happy that I'm backing off today.")
	public static NpcStringId BE_HAPPY_THAT_I_M_BACKING_OFF_TODAY;
	
	@ClientString(id = 17178332, message = "Imbeciles...you'll disappear on the day of destruction...")
	public static NpcStringId IMBECILES_YOU_LL_DISAPPEAR_ON_THE_DAY_OF_DESTRUCTION;
	
	@ClientString(id = 17178333, message = "Ah.. Did the backup get wiped out... Looks like we're late.")
	public static NpcStringId AH_DID_THE_BACKUP_GET_WIPED_OUT_LOOKS_LIKE_WE_RE_LATE;
	
	@ClientString(id = 17178334, message = "You guys are the mercenaries.")
	public static NpcStringId YOU_GUYS_ARE_THE_MERCENARIES;
	
	@ClientString(id = 17178335, message = "He's quiet again. Thanks.")
	public static NpcStringId HE_S_QUIET_AGAIN_THANKS;
	
	@ClientString(id = 17178336, message = "This.. We brought this to support the backup, but we could give these to you.")
	public static NpcStringId THIS_WE_BROUGHT_THIS_TO_SUPPORT_THE_BACKUP_BUT_WE_COULD_GIVE_THESE_TO_YOU;
	
	@ClientString(id = 17178337, message = "Courageous ones who supported Antharas force, come and take the Kingdom's reward.")
	public static NpcStringId COURAGEOUS_ONES_WHO_SUPPORTED_ANTHARAS_FORCE_COME_AND_TAKE_THE_KINGDOM_S_REWARD;
	
	@ClientString(id = 17178338, message = "Are there those who didn't receive the rewards yet? Come and get it from me.")
	public static NpcStringId ARE_THERE_THOSE_WHO_DIDN_T_RECEIVE_THE_REWARDS_YET_COME_AND_GET_IT_FROM_ME;
	
	@ClientString(id = 17178339, message = "Catch up to King, he's waiting.")
	public static NpcStringId CATCH_UP_TO_KING_HE_S_WAITING;
	
	@ClientString(id = 17178340, message = "Ruff!! Ruff! Rrrrrr!!!")
	public static NpcStringId RUFF_RUFF_RRRRRR;
	
	@ClientString(id = 17178341, message = "You must move to Exploration Area 5 in order to continue.")
	public static NpcStringId YOU_MUST_MOVE_TO_EXPLORATION_AREA_5_IN_ORDER_TO_CONTINUE;
	
	@ClientString(id = 17178342, message = "King has returned to Def. Return to Def and start again.")
	public static NpcStringId KING_HAS_RETURNED_TO_DEF_RETURN_TO_DEF_AND_START_AGAIN;
	
	@ClientString(id = 17178343, message = "Can't help you now.")
	public static NpcStringId CAN_T_HELP_YOU_NOW;
	
	@ClientString(id = 17178344, message = "I should rest.")
	public static NpcStringId I_SHOULD_REST;
	
	@ClientString(id = 17178345, message = "Try riding a Kukuri.")
	public static NpcStringId TRY_RIDING_A_KUKURI;
	
	@ClientString(id = 17178346, message = "Scared!! Let's run!")
	public static NpcStringId SCARED_LET_S_RUN;
	
	@ClientString(id = 17178347, message = "Can't die in a place like this!!")
	public static NpcStringId CAN_T_DIE_IN_A_PLACE_LIKE_THIS;
	
	@ClientString(id = 17178348, message = "Ahh...Save me!!")
	public static NpcStringId AHH_SAVE_ME;
	
	@ClientString(id = 17178349, message = "Urgh...")
	public static NpcStringId URGH3;
	
	@ClientString(id = 17178350, message = "Ahhk...")
	public static NpcStringId AHHK2;
	
	@ClientString(id = 17178351, message = "Darn it!! Dying here like this!!!!")
	public static NpcStringId DARN_IT_DYING_HERE_LIKE_THIS;
	
	@ClientString(id = 17178352, message = "Is this as far as I go...")
	public static NpcStringId IS_THIS_AS_FAR_AS_I_GO;
	
	@ClientString(id = 17178353, message = "I'm fighting. Talk to me later!")
	public static NpcStringId I_M_FIGHTING_TALK_TO_ME_LATER;
	
	@ClientString(id = 44440000, message = "Uh ah ah ah ah...")
	public static NpcStringId UH_AH_AH_AH_AH;
	
	@ClientString(id = 44440001, message = "Ah uh ah uh ah...")
	public static NpcStringId AH_UH_AH_UH_AH;
	
	@ClientString(id = 44440002, message = "Krrrr...")
	public static NpcStringId KRRRR2;
	
	@ClientString(id = 44440003, message = "Ah ah ah uh uh...")
	public static NpcStringId AH_AH_AH_UH_UH3;
	
	@ClientString(id = 44440004, message = "Give me my youth back!")
	public static NpcStringId GIVE_ME_MY_YOUTH_BACK;
	
	@ClientString(id = 44440005, message = "Don't you want to sleep forever with me?")
	public static NpcStringId DON_T_YOU_WANT_TO_SLEEP_FOREVER_WITH_ME;
	
	@ClientString(id = 44440006, message = "My heart aches with loneliness...")
	public static NpcStringId MY_HEART_ACHES_WITH_LONELINESS;
	
	@ClientString(id = 44440007, message = "I'm su.. suffering...")
	public static NpcStringId I_M_SU_SUFFERING;
	
	@ClientString(id = 44440008, message = "No more….")
	public static NpcStringId NO_MORE;
	
	@ClientString(id = 44440009, message = "Attracted to $s1")
	public static NpcStringId ATTRACTED_TO_S1;
	
	@ClientString(id = 44440010, message = "$s1, you don't know how long I've waited for you...")
	public static NpcStringId S1_YOU_DON_T_KNOW_HOW_LONG_I_VE_WAITED_FOR_YOU;
	
	@ClientString(id = 44440011, message = "Quiet!")
	public static NpcStringId QUIET;
	
	@ClientString(id = 44440012, message = "Ahh! Let's run!")
	public static NpcStringId AHH_LET_S_RUN;
	
	@ClientString(id = 44440013, message = "Hornapi! Attack!")
	public static NpcStringId HORNAPI_ATTACK;
	
	@ClientString(id = 44440014, message = "Please get rid of them!")
	public static NpcStringId PLEASE_GET_RID_OF_THEM;
	
	@ClientString(id = 44440015, message = "Hornapi!")
	public static NpcStringId HORNAPI;
	
	@ClientString(id = 44440016, message = "Mukshu!")
	public static NpcStringId MUKSHU;
	
	@ClientString(id = 44440017, message = "Don't make me angry!")
	public static NpcStringId DON_T_MAKE_ME_ANGRY;
	
	@ClientString(id = 44440018, message = "It hurts! I'm angry!")
	public static NpcStringId IT_HURTS_I_M_ANGRY;
	
	@ClientString(id = 44440019, message = "I lost...")
	public static NpcStringId I_LOST;
	
	@ClientString(id = 44440020, message = "Give me power!")
	public static NpcStringId GIVE_ME_POWER;
	
	@ClientString(id = 44440021, message = "Rotting pain!")
	public static NpcStringId ROTTING_PAIN;
	
	@ClientString(id = 44440022, message = "Witness my true power.")
	public static NpcStringId WITNESS_MY_TRUE_POWER2;
	
	@ClientString(id = 44440023, message = "The fire in me is fierce!")
	public static NpcStringId THE_FIRE_IN_ME_IS_FIERCE;
	
	@ClientString(id = 44440024, message = "Kh ah ah ah!")
	public static NpcStringId KH_AH_AH_AH2;
	
	@ClientString(id = 44440025, message = "Kh oh oh oh!")
	public static NpcStringId KH_OH_OH_OH;
	
	@ClientString(id = 44440026, message = "I smell fresh raw meat!")
	public static NpcStringId I_SMELL_FRESH_RAW_MEAT;
	
	@ClientString(id = 44440027, message = "I'm entering the darkness...")
	public static NpcStringId I_M_ENTERING_THE_DARKNESS;
	
	@ClientString(id = 44440028, message = "Deeper into the dark, deeper...")
	public static NpcStringId DEEPER_INTO_THE_DARK_DEEPER;
	
	@ClientString(id = 44440029, message = "Experience fear and powerlessness!")
	public static NpcStringId EXPERIENCE_FEAR_AND_POWERLESSNESS;
	
	@ClientString(id = 44440030, message = "Kh ah! Burning light!")
	public static NpcStringId KH_AH_BURNING_LIGHT;
	
	@ClientString(id = 44440031, message = "I'll show you what true fear feels like!")
	public static NpcStringId I_LL_SHOW_YOU_WHAT_TRUE_FEAR_FEELS_LIKE;
	
	@ClientString(id = 44440032, message = "Darkness never goes away...")
	public static NpcStringId DARKNESS_NEVER_GOES_AWAY;
	
	@ClientString(id = 44440033, message = "Light is painful!")
	public static NpcStringId LIGHT_IS_PAINFUL;
	
	@ClientString(id = 44440034, message = "I'll rot with your corpses! Hahaha!")
	public static NpcStringId I_LL_ROT_WITH_YOUR_CORPSES_HAHAHA;
	
	@ClientString(id = 44440035, message = "I should inhale some rotten poisonous air.")
	public static NpcStringId I_SHOULD_INHALE_SOME_ROTTEN_POISONOUS_AIR;
	
	@ClientString(id = 44440036, message = "Absorb bright stars to fill the black darkness with...")
	public static NpcStringId ABSORB_BRIGHT_STARS_TO_FILL_THE_BLACK_DARKNESS_WITH;
	
	@ClientString(id = 44440037, message = "Kh ha ah ah!")
	public static NpcStringId KH_HA_AH_AH;
	
	@ClientString(id = 44440038, message = "Khh ahh!")
	public static NpcStringId KHH_AHH;
	
	@ClientString(id = 44440039, message = "I hear something moving! Who are you!")
	public static NpcStringId I_HEAR_SOMETHING_MOVING_WHO_ARE_YOU;
	
	@ClientString(id = 44440040, message = "Krrrr...")
	public static NpcStringId KRRRR3;
	
	@ClientString(id = 44440041, message = "Hornapi! Use the invincible iron!")
	public static NpcStringId HORNAPI_USE_THE_INVINCIBLE_IRON;
	
	@ClientString(id = 44440042, message = "Yes, Mukshu! Iron!")
	public static NpcStringId YES_MUKSHU_IRON;
	
	@ClientString(id = 44440043, message = "Image")
	public static NpcStringId IMAGE;
	
	@ClientString(id = 44440044, message = "Flame of darkness...!")
	public static NpcStringId FLAME_OF_DARKNESS;
	
	@ClientString(id = 901900139, message = "Lukpie~! I'm Lukpie, the spirit that loves Adena~")
	public static NpcStringId LUKPIE_I_M_LUKPIE_THE_SPIRIT_THAT_LOVES_ADENA;
	
	@ClientString(id = 901900140, message = "Lukpie~! I want to eat Adena... Give me Adena!")
	public static NpcStringId LUKPIE_I_WANT_TO_EAT_ADENA_GIVE_ME_ADENA;
	
	@ClientString(id = 901900141, message = "Lukpie~! If I eat too much Adena, my wings disappear…")
	public static NpcStringId LUKPIE_IF_I_EAT_TOO_MUCH_ADENA_MY_WINGS_DISAPPEAR;
	
	@ClientString(id = 901900142, message = "Yummy. Thanks! Lukpie~!")
	public static NpcStringId YUMMY_THANKS_LUKPIE;
	
	@ClientString(id = 901900143, message = "Grrrr... Yuck…")
	public static NpcStringId GRRRR_YUCK;
	
	@ClientString(id = 901900144, message = "Lukpie~! Adena is so good! My body is getting bigger!")
	public static NpcStringId LUKPIE_ADENA_IS_SO_GOOD_MY_BODY_IS_GETTING_BIGGER;
	
	@ClientString(id = 901900145, message = "Lukpie~! You won't give me any more Adena? Oh... The body is too heavy!")
	public static NpcStringId LUKPIE_YOU_WON_T_GIVE_ME_ANY_MORE_ADENA_OH_THE_BODY_IS_TOO_HEAVY;
	
	@ClientString(id = 901900146, message = "Lukpie~! I'm full~ It was delicious! Oh... The body is too heavy!")
	public static NpcStringId LUKPIE_I_M_FULL_IT_WAS_DELICIOUS_OH_THE_BODY_IS_TOO_HEAVY;
	
	@ClientString(id = 901900147, message = "Lukpie~! Too little... It must be at least %s or more!")
	public static NpcStringId LUKPIE_TOO_LITTLE_IT_MUST_BE_AT_LEAST_S_OR_MORE;
	
	@ClientString(id = 901900148, message = "Oh! Wings are gone!... Are you going to hit me? If you hit me, I will vomit what I've eaten!")
	public static NpcStringId OH_WINGS_ARE_GONE_ARE_YOU_GOING_TO_HIT_ME_IF_YOU_HIT_ME_I_WILL_VOMIT_WHAT_I_VE_EATEN;
	
	@ClientString(id = 901900149, message = "Oh! My wing...ehk! Are you going to hit me? Scary Scary! Bad things will happen when you hit me!")
	public static NpcStringId OH_MY_WING_EHK_ARE_YOU_GOING_TO_HIT_ME_SCARY_SCARY_BAD_THINGS_WILL_HAPPEN_WHEN_YOU_HIT_ME;
	
	@ClientString(id = 901900150, message = "The evil Land Dragon Antharas has been defeated by brave heroes!!!")
	public static NpcStringId THE_EVIL_LAND_DRAGON_ANTHARAS_HAS_BEEN_DEFEATED_BY_BRAVE_HEROES;
	
	@ClientString(id = 901900151, message = "The evil Fire Dragon Valakas has been defeated!")
	public static NpcStringId THE_EVIL_FIRE_DRAGON_VALAKAS_HAS_BEEN_DEFEATED;
	
	@ClientString(id = 901900152, message = "To serve him now means you will be able to escape a worse situation.")
	public static NpcStringId TO_SERVE_HIM_NOW_MEANS_YOU_WILL_BE_ABLE_TO_ESCAPE_A_WORSE_SITUATION;
	
	@ClientString(id = 901900153, message = "Oh goddess of destruction, forgive us...")
	public static NpcStringId OH_GODDESS_OF_DESTRUCTION_FORGIVE_US;
	
	@ClientString(id = 901900154, message = "When the sky turns blood red and the earth begins to crumble… from the darkness… she will return.")
	public static NpcStringId WHEN_THE_SKY_TURNS_BLOOD_RED_AND_THE_EARTH_BEGINS_TO_CRUMBLE_FROM_THE_DARKNESS_SHE_WILL_RETURN;
	
	@ClientString(id = 901900155, message = "Energy of earth is gathering near  Antharas' legs.")
	public static NpcStringId ENERGY_OF_EARTH_IS_GATHERING_NEAR_ANTHARAS_LEGS;
	
	@ClientString(id = 901900156, message = "Antharas starts to absorb the energy of earth.")
	public static NpcStringId ANTHARAS_STARTS_TO_ABSORB_THE_ENERGY_OF_EARTH;
	
	@ClientString(id = 901900157, message = "Antharas raises its thick tail.")
	public static NpcStringId ANTHARAS_RAISES_ITS_THICK_TAIL;
	
	@ClientString(id = 901900158, message = "You are overcome by the strength of Antharas.")
	public static NpcStringId YOU_ARE_OVERCOME_BY_THE_STRENGTH_OF_ANTHARAS;
	
	@ClientString(id = 901900159, message = "Antharas' eyes are filled with rage.")
	public static NpcStringId ANTHARAS_EYES_ARE_FILLED_WITH_RAGE;
	
	@ClientString(id = 901900160, message = "$s1, I can feel their presence from you.")
	public static NpcStringId S1_I_CAN_FEEL_THEIR_PRESENCE_FROM_YOU;
	
	@ClientString(id = 901900161, message = "$s1, brethren, come to my side and follow me.")
	public static NpcStringId S1_BRETHREN_COME_TO_MY_SIDE_AND_FOLLOW_ME;
	
	@ClientString(id = 901900162, message = "Antharas roars.")
	public static NpcStringId ANTHARAS_ROARS;
	
	@ClientString(id = 901900163, message = "Flame energy is being directed towards Valakas.")
	public static NpcStringId FLAME_ENERGY_IS_BEING_DIRECTED_TOWARDS_VALAKAS;
	
	@ClientString(id = 901900164, message = "You are overcome by the strength of Valakas.")
	public static NpcStringId YOU_ARE_OVERCOME_BY_THE_STRENGTH_OF_VALAKAS;
	
	@ClientString(id = 901900165, message = "Valakas' tail flails dangerously.")
	public static NpcStringId VALAKAS_TAIL_FLAILS_DANGEROUSLY;
	
	@ClientString(id = 901900166, message = "Valakas raises its tail.")
	public static NpcStringId VALAKAS_RAISES_ITS_TAIL;
	
	@ClientString(id = 901900167, message = "Valakas starts to absorb the flame energy.")
	public static NpcStringId VALAKAS_STARTS_TO_ABSORB_THE_FLAME_ENERGY;
	
	@ClientString(id = 901900168, message = "Valakas looks to its left.")
	public static NpcStringId VALAKAS_LOOKS_TO_ITS_LEFT;
	
	@ClientString(id = 901900169, message = "Valakas looks to its right.")
	public static NpcStringId VALAKAS_LOOKS_TO_ITS_RIGHT;
	
	@ClientString(id = 901900170, message = "By my authority, I command you, creature, turn to dust.")
	public static NpcStringId BY_MY_AUTHORITY_I_COMMAND_YOU_CREATURE_TURN_TO_DUST;
	
	@ClientString(id = 901900171, message = "By my wrath, I command you, creature, lose your mind.")
	public static NpcStringId BY_MY_WRATH_I_COMMAND_YOU_CREATURE_LOSE_YOUR_MIND;
	
	@ClientString(id = 901900172, message = "Show respect to the heroes who defeated the evil dragon and protected this Aden world!")
	public static NpcStringId SHOW_RESPECT_TO_THE_HEROES_WHO_DEFEATED_THE_EVIL_DRAGON_AND_PROTECTED_THIS_ADEN_WORLD2;
	
	@ClientString(id = 901900173, message = "Shout to celebrate the victory of the heroes!")
	public static NpcStringId SHOUT_TO_CELEBRATE_THE_VICTORY_OF_THE_HEROES2;
	
	@ClientString(id = 901900174, message = "Praise the achievement of the heroes and receive Tersi's blessing!")
	public static NpcStringId PRAISE_THE_ACHIEVEMENT_OF_THE_HEROES_AND_RECEIVE_TERSI_S_BLESSING2;
	
	@ClientString(id = 901900175, message = "Ugh… I think this is it for me…")
	public static NpcStringId UGH_I_THINK_THIS_IS_IT_FOR_ME;
	
	@ClientString(id = 901900176, message = "Valakas forcefully recalls the summoner or the summoned pet.")
	public static NpcStringId VALAKAS_FORCEFULLY_RECALLS_THE_SUMMONER_OR_THE_SUMMONED_PET;
	
	@ClientString(id = 901900177, message = "Plague, new nightmare (Lv. 80-85)")
	public static NpcStringId PLAGUE_NEW_NIGHTMARE_LV_80_85;
	
	static
	{
		buildFastLookupTable();
	}
	
	private static final void buildFastLookupTable()
	{
		final Field[] fields = NpcStringId.class.getDeclaredFields();
		
		for (final Field field : fields)
		{
			final int mod = field.getModifiers();
			if (Modifier.isStatic(mod) && Modifier.isPublic(mod) && field.getType().equals(NpcStringId.class) && field.isAnnotationPresent(ClientString.class))
			{
				try
				{
					final ClientString annotation = field.getAnnotationsByType(ClientString.class)[0];
					final NpcStringId nsId = new NpcStringId(annotation.id());
					nsId.setName(field.getName());
					nsId.setParamCount(parseMessageParameters(field.getName()));
					field.set(null, nsId);
					VALUES.put(nsId.getId(), nsId);
				}
				catch (final Exception e)
				{
					_log.log(Level.WARNING, "NpcStringId: Failed field access for '" + field.getName() + "'", e);
				}
			}
		}
	}
	
	private static final int parseMessageParameters(final String name)
	{
		int paramCount = 0;
		char c1, c2;
		for (int i = 0; i < (name.length() - 1); i++)
		{
			c1 = name.charAt(i);
			if ((c1 == 'C') || (c1 == 'S'))
			{
				c2 = name.charAt(i + 1);
				if (Character.isDigit(c2))
				{
					paramCount = Math.max(paramCount, Character.getNumericValue(c2));
					i++;
				}
			}
		}
		return paramCount;
	}
	
	public static final NpcStringId getNpcStringId(final int id)
	{
		return getNpcStringIdOrDefault(id, new NpcStringId(id));
	}
	
	public static final NpcStringId getNpcStringIdOrDefault(final int id, NpcStringId defaultValue)
	{
		final NpcStringId nsi = getNpcStringIdInternal(id);
		return nsi == null ? defaultValue : nsi;
	}
	
	private static final NpcStringId getNpcStringIdInternal(final int id)
	{
		return VALUES.get(id);
	}
	
	public static final NpcStringId getNpcStringId(final String name)
	{
		try
		{
			return (NpcStringId) NpcStringId.class.getField(name).get(null);
		}
		catch (final Exception e)
		{
			return null;
		}
	}
	
	public static final void reloadLocalisations()
	{
		for (final NpcStringId nsId : VALUES.values())
		{
			if (nsId != null)
			{
				nsId.removeAllLocalisations();
			}
		}
		
		if (!Config.L2JMOD_MULTILANG_NS_ENABLE)
		{
			_log.log(Level.INFO, "NpcStringId: MultiLanguage disabled.");
			return;
		}
		
		final List<String> languages = Config.L2JMOD_MULTILANG_NS_ALLOWED;
		final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		factory.setIgnoringComments(true);
		
		File file;
		Node node;
		Document doc;
		NamedNodeMap nnmb;
		NpcStringId nsId;
		String text;
		for (final String lang : languages)
		{
			file = new File(Config.DATAPACK_ROOT, "/data/lang/" + lang + "/ns/NpcStringLocalisation.xml");
			if (!file.isFile())
			{
				continue;
			}
			
			_log.log(Level.INFO, "NpcStringId: Loading localisation for '" + lang + "'");
			
			try
			{
				doc = factory.newDocumentBuilder().parse(file);
				for (Node na = doc.getFirstChild(); na != null; na = na.getNextSibling())
				{
					if ("list".equals(na.getNodeName()))
					{
						for (Node nb = na.getFirstChild(); nb != null; nb = nb.getNextSibling())
						{
							if ("ns".equals(nb.getNodeName()))
							{
								nnmb = nb.getAttributes();
								node = nnmb.getNamedItem("id");
								if (node != null)
								{
									nsId = getNpcStringId(Integer.parseInt(node.getNodeValue()));
									if (nsId == null)
									{
										_log.log(Level.WARNING, "NpcStringId: Unknown NSID '" + node.getNodeValue() + "', lang '" + lang + "'.");
										continue;
									}
								}
								else
								{
									node = nnmb.getNamedItem("name");
									nsId = getNpcStringId(node.getNodeValue());
									if (nsId == null)
									{
										_log.log(Level.WARNING, "NpcStringId: Unknown NSID '" + node.getNodeValue() + "', lang '" + lang + "'.");
										continue;
									}
								}
								
								node = nnmb.getNamedItem("text");
								if (node == null)
								{
									_log.log(Level.WARNING, "NpcStringId: No text defined for NSID '" + nsId + "', lang '" + lang + "'.");
									continue;
								}
								
								text = node.getNodeValue();
								if (text.isEmpty() || (text.length() > 255))
								{
									_log.log(Level.WARNING, "NpcStringId: Invalid text defined for NSID '" + nsId + "' (to long or empty), lang '" + lang + "'.");
									continue;
								}
								
								nsId.attachLocalizedText(lang, text);
							}
						}
					}
				}
			}
			catch (final Exception e)
			{
				_log.log(Level.SEVERE, "NpcStringId: Failed loading '" + file + "'", e);
			}
		}
	}
	
	private final int _id;
	private String _name;
	private byte _params;
	private NSLocalisation[] _localisations;
	private ExShowScreenMessage _staticScreenMessage;
	
	protected NpcStringId(final int id)
	{
		_id = id;
		_localisations = EMPTY_NSL_ARRAY;
	}
	
	public final int getId()
	{
		return _id;
	}
	
	private final void setName(final String name)
	{
		_name = name;
	}
	
	public final String getName()
	{
		return _name;
	}
	
	public final int getParamCount()
	{
		return _params;
	}
	
	/**
	 * You better don`t touch this!
	 * @param params
	 */
	public final void setParamCount(final int params)
	{
		if (params < 0)
		{
			throw new IllegalArgumentException("Invalid negative param count: " + params);
		}
		
		if (params > 10)
		{
			throw new IllegalArgumentException("Maximum param count exceeded: " + params);
		}
		
		if (params != 0)
		{
			_staticScreenMessage = null;
		}
		
		_params = (byte) params;
	}
	
	public final NSLocalisation getLocalisation(final String lang)
	{
		NSLocalisation nsl;
		for (int i = _localisations.length; i-- > 0;)
		{
			nsl = _localisations[i];
			if (nsl.getLanguage().hashCode() == lang.hashCode())
			{
				return nsl;
			}
		}
		return null;
	}
	
	public final void attachLocalizedText(final String lang, final String text)
	{
		final int length = _localisations.length;
		final NSLocalisation[] localisations = Arrays.copyOf(_localisations, length + 1);
		localisations[length] = new NSLocalisation(lang, text);
		_localisations = localisations;
	}
	
	public final void removeAllLocalisations()
	{
		_localisations = EMPTY_NSL_ARRAY;
	}
	
	public final ExShowScreenMessage getStaticScreenMessage()
	{
		return _staticScreenMessage;
	}
	
	public final void setStaticSystemMessage(final ExShowScreenMessage ns)
	{
		_staticScreenMessage = ns;
	}
	
	@Override
	public final String toString()
	{
		return "NS[" + getId() + ":" + getName() + "]";
	}
	
	public static final class NSLocalisation
	{
		private final String _lang;
		private final Builder _builder;
		
		public NSLocalisation(final String lang, final String text)
		{
			_lang = lang;
			_builder = Builder.newBuilder(text);
		}
		
		public final String getLanguage()
		{
			return _lang;
		}
		
		public final String getLocalisation(final Object... params)
		{
			return _builder.toString(params);
		}
	}
}
